/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <adr.h>
#include <mtpaths.h>
#include <cattypes.h>
#include <mte.h>
#include <catgloba.h>
#include <quicksor.h>
#include <confvars.h>
#include <varname1.h>
#include <protokol.h>
#include <groupsel.h>
#include <listdl.h>
#include <maustaus.h>

#include <mtappl.h>
#include <mttextfi.h>
#include <mtutils.h>

#include <s_endian.h>
#include <routinec.h>
#include <routines.h>
#include <file_io.h>
#include <dir_io.h>
#include <strconv.h>
#include <catutil.h>


LOCAL List data_names[2];

LOCAL _BOOL nameDialIsOpen;
LOCAL _WORD globalLength;

#define TAB '\t'

LOCAL CONST _WORD titles[ADR_ANZ_MODES] = {
	ADR_TIT_0,
	ADR_TIT_1,
	ADR_TIT_2,
	ADR_TIT_3,
	ADR_TIT_4,
	ADR_TIT_5,
	ADR_TIT_6,
	ADR_TIT_7,
	ADR_TIT_8,		/*DYJ 5.34d */
	ADR_TIT_9		/*DYJ 5.35c */
};

typedef struct _adrEntryType {
	_UWORD len;
	_UWORD number;
	_BOOL selected;
	_UBYTE gName[1];
} adrEntryType;

typedef struct {
	_WORD exitBut;
	_BOOL isChange;
	specialButHdler specials[2];
	OBJECT *adr;
	_VOID *v;
	_WORD nor_alias;
} adrInfo;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*--- Listen einlesen ---*/

/* Ein Listenelement an die angegebene Liste anhaengen;     *
 * 'spezielle' Stringlisten fuer Gruppen- und Absendernamen */
LOCAL _BOOL AppendToList(List *list, CONST _UBYTE *str)
{
	adrEntryType *new;
	_UWORD l;
	
	l = (_UWORD)strLen(str);
	new = MALLOC(sizeof(adrEntryType) + l, "adr_AppendToList");
	if (new == NULL)
		return FALSE;
	new->len = l;
	new->number = (_UWORD)Lists_NoOfEntries(list) + 1; /* naechste Nummer vergeben */
	new->selected = FALSE;
	strCpy(new->gName, str);
	if (!Lists_AppendEntry(list, new))
	{
		SFREE(new);
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ClearList(List *list, _BOOL killCarrier)
{
	Lists_DeleteSimpleList(list, mem_free, killCarrier);
}

/*** ---------------------------------------------------------------------- ***/

typedef enum { nothingWrong, onlyFileNotFound, severeFault } whatsWrong;

/* Eine der Namenlisten einlesen */
LOCAL whatsWrong ReadList(List *list, CONST _UBYTE *fname)
{
	TEXTFILENO in;
	_BOOL err;
	_UBYTE scrap[1023];

	if (mtTextfiles_OpenTextfile(fname, READ, 0, &in) != GERR_OK)
		return onlyFileNotFound;
	err = FALSE;
	while (!err && !mtTextfiles_EndofText(in))
	{
		mtTextfiles_ReadLine(in, scrap, sizeof(scrap));
		mtTextfiles_ReadLn(in);
		if (scrap[0] != '\0')
			err = !AppendToList(list, scrap); /* Fehler, falls es nicht geklappt hat */
	}
	mtTextfiles_CloseTextfile(&in);
	
	if (err)
		return severeFault;
	return nothingWrong;
}

/*** ------------------------------------------------------------------------ ***/
/*	LOCAL whatsWrong ReadListSp(List *list)								*/
/*	Namenliste aus den Aliaseintrgen in der CATUSER.inf der 4.14 erstellen		*/
/*																				*/
/*	Parameter:	list: das Ziel													*/
/*	Ergebnis:	Fehlercode														*/		
/*	27.4.00 von Dimitri Junker	DYJ 4.15										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.21b */
LOCAL whatsWrong ReadListSp(List *list)
{
	_BOOL err,warnung=FALSE;
	_UBYTE scrap[1023];
	_UBYTE varname[50];
	_WORD x;

	err = FALSE;
	for (x = 1; x <= 10; x++)
	{
		make_varname(varname, sizeof(varname), cOwnNameAlias, x);
		ConfVars_GetConfDefStr(varname, scrap, sizeof(scrap), "");
		if(*scrap)
		{
			err = !AppendToList(list, scrap); /* Fehler, falls es nicht geklappt hat */
			ConfVars_DeleteConfigVar(varname);/* wird nicht mehr bentigt	*/
			warnung=TRUE;
		}
	}
	adr_SaveNames(ADR_ALIAS);
	if(warnung)
		mtAlerts_AlertRSC(1, AL_ALIAS414);

	if (err)
		return severeFault;
	return nothingWrong;
}

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID E_Alias_Config(_VOID)											*/
/*	Aliasadressen eingeben														*/
/*																				*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	27.4.00 von Dimitri Junker	DYJ 4.15										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID E_Alias_Config(_VOID)
{
	_UBYTE name[256]="";
	adr_SelectName(ADR_ALIAS,name, sizeof(name));
}


/********************************************************************/
/*																	*/
/*	LOCAL _VOID getAdrInfPath(_WORD nor_alias,CONST _UBYTE *pfad)	*/
/*	bestimmt den Pfad der Adressdatei. Diese ist normalerweise im	*/
/*	Data-Ordner, kann aber per catuser.inf auch irgendwo anders sein*/
/*	Parameter:	nor_alias:	welche der Adressdateien				*/
/*				pfad:		da wird der Pfad hinkopiert. Es mu		*/
/*							mindestens PATH_MAX lang sein!			*/
/*	Rckgabe:	geschieht ber pfad									*/
/*																	*/
/*	DYJ 4.16 														*/
/********************************************************************/
LOCAL _VOID getAdrInfPath(_WORD nor_alias,_UBYTE *pfad)
{
	if(nor_alias==ADR_ALIAS)
	{
		if(!ConfVars_GetConfigString(cAdrAliasPath,pfad,PATH_MAX))
		{
			strCpy(pfad, DataPath);
			F_Path_Append(pfad, catAdrAliInf);
		}
	}
	else
	{
		if(!ConfVars_GetConfigString(cAdrPath,pfad,PATH_MAX))
		{
			strCpy(pfad, DataPath);
			F_Path_Append(pfad, catAdrInf);
		}
	}
}
	

/*** ---------------------------------------------------------------------- ***/

/* Neuen Absender uebernehmen, meckert selber, falls es nicht klappt */
LOCAL _VOID adr_AppendName(_WORD nor_alias,CONST _UBYTE *name)
{
	_UBYTE file[PATH_MAX];
	if (AppendToList(&data_names[nor_alias], name))
	{
	getAdrInfPath(nor_alias,file);
	GroupSelect_NameToFile(name, file, NULL);
/* Jetzt eventuell noch sortieren */
		adr_SortList(nor_alias,FUNK_NULL);
	}
}

/*** ---------------------------------------------------------------------- ***/
GLOBAL _BOOL adr_ReadNames(_WORD nor_alias)
{
	_WORD ret;
	_UBYTE file[PATH_MAX];

	/* Erstmal beide Listen loeschen, falls schon was drin ist */
	if (data_names[nor_alias].root == NULL)
	{
		if (!Lists_CreateList(&data_names[nor_alias], "data: list of names"))
			return FALSE;
	}
	ClearList(&data_names[nor_alias], FALSE);

	getAdrInfPath(nor_alias,file);
	ret= ReadList(&data_names[nor_alias], file);
	if(ret==onlyFileNotFound && nor_alias==ADR_ALIAS) /* nach einem Wechsel 4.14->4.15*/
		ret= ReadListSp(&data_names[nor_alias]);
	return ret!= severeFault;

}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID adr_ClearList(_WORD nor_alias)
{
	ClearList(&data_names[nor_alias], TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Sichert die Liste der Namen komplett
 */
GLOBAL _VOID adr_SaveNames(_WORD nor_alias)
{
	_UBYTE file[PATH_MAX];
	TEXTFILENO out;
	_UWORD num, i;
	adrEntryType *entry;

	getAdrInfPath(nor_alias,file);
	if (mtTextfiles_OpenTextfile(file, WRITE, 0, &out) == GERR_OK)
	{
		num = (_UWORD)Lists_NoOfEntries(&data_names[nor_alias]);
		for (i = 1; i <= num; i++)
		{
			Lists_ResetList(&data_names[nor_alias]);
			while ((entry = Lists_NextEntry(&data_names[nor_alias])) != NULL && entry->number != i)
				;
			if (entry != NULL)
			{
				mtTextfiles_WriteLine(out, entry->gName);
				mtTextfiles_WriteLn(out);
			}
		}
		mtTextfiles_CloseTextfile(&out);
		Protokoll_SendPathUpdate(DataPath);
	}
}


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL adrComp(CONST _VOID *a1, CONST _VOID *a2, _VOID *private)
{
	CONST adrEntryType *lp1;
	CONST adrEntryType *lp2;
	
	UNUSED(private);
	lp1 = *((CONST adrEntryType **)a1);
	lp2 = *((CONST adrEntryType **)a2);
	return strSortCmp(lp1->gName, lp2->gName) < 0;
}

/*** ---------------------------------------------------------------------- ***/

/* sortiert die Namensliste */
GLOBAL _VOID adr_SortList(_WORD nor_alias,_VOID (*vorher)(_VOID))
{
	_BOOL sortit;
	
	ConfVars_GetConfDefBool(cSortNames, &sortit, TRUE);
	if (sortit)
	{
		if (vorher != FUNK_NULL)
			vorher();
		GroupSelect_SortAnyList(&data_names[nor_alias], adrComp);
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID getInfo(CONST _UBYTE *s1, _UBYTE *s2, size_t strsize)
{
	CONST _UBYTE *p;
	size_t len;

	p = strChr(s1, TAB);
	if (p != NULL)
	{
		p++;
		len = strLen(p) + 1;
	} else
	{
		p = " ";
		len = 2;
	}
	if (len > strsize)
		len = strsize;
	strMcpy(s2, len, p);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID getName(CONST _UBYTE *s1, _UBYTE *s2, size_t strsize)
{
	CONST _UBYTE *p;
	size_t len;

	p = strChr(s1, TAB);
	if (p != NULL)
		len = p - s1 + 1;
	else
		len = strLen(s1) + 1;
	if (len > strsize)
		len = strsize;
	strMcpy(s2, len, s1);
}


/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID nameInfoToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)	*/
/*	Kopiert Name und Infostring zusammen in einen Zielstring					*/
/*	Abgeleitet von nameToStr													*/
/*	Parameter:	entry:	Nummer der zu lesenden Adresse							*/
/*				env:	unbenutzt												*/
/*				str und strsize: Zielstring und seine Lnge						*/
/*	Ergebnis:	keines															*/		
/*	20.3.00 von Dimitri Junker		DYJ 4.14									*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _VOID nameInfoToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)
{
	adrEntryType *gr;

	UNUSED(env);
	gr = entry;
	if (gr != NULL)
		strnCpy(str,gr->gName,strsize);
	else
		*str = '\0';
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID infoToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)*/
/*	Kopiert den Infostring in einen Zielstring									*/
/*	Abgeleitet von nameToStr													*/
/*	Parameter:	entry:	Nummer der zu lesenden Adresse							*/
/*				env:	unbenutzt												*/
/*				str und strsize: Zielstring und seine Lnge						*/
/*	Ergebnis:	keines															*/		
/*	20.3.00 von Dimitri Junker		DYJ 4.14									*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _VOID infoToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)
{
	adrEntryType *gr;

	UNUSED(env);
	gr = entry;
	if (gr != NULL)
		getInfo(gr->gName, str, strsize);
	else
		*str = '\0';
}
/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID nameToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)
{
	adrEntryType *gr;

	UNUSED(env);
	gr = entry;
	if (gr != NULL)
		getName(gr->gName, str, strsize);
	else
		*str = '\0';
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL selNameEntry(_VOID *entry, _VOID *env, _WORD line)
{
	adrEntryType *e;
	group_line str;
	GRECT r;
	OBJECT *adr;

	e = entry;
	if (e != NULL)
	{
		adr = ((adrInfo *)env)->adr;
		e->selected = !e->selected;
		if (e->selected && line >= 0)
		{
			getName(e->gName, str, sizeof(str));
			SetObjcString(adr, NEWADR, str);
			WinDials_WinDialDraw(adr, NEWADR, 0, &r, FALSE);
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findNameSel(_VOID *e, _VOID *info)
{
	adrEntryType *entry;

	UNUSED(info);
	entry = e;
	return entry->selected;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findName(_VOID *e, _VOID *info)
{
	adrEntryType *entry;
	group_line name;

	entry = e;
	getName(entry->gName, name, sizeof(name));
	return strCmp(name, info) == 0;
}

/*** ---------------------------------------------------------------------- ***/

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

typedef struct {
	_UWORD selectedEntry;
	group_line name;
	group_line info;
	group_line newName;
	group_line newInfo;
	_BOOL inList;
	_WORD exitBut;
	_WORD nor_alias;
} nameInfo;

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL nameIsSelected(_VOID *entry, _VOID *env)
{
	adrEntryType *ent;

	UNUSED(env);
	ent = entry;
	return ent->selected;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isEnabled(_VOID *entry, _VOID *env)
{
	UNUSED(entry);
	UNUSED(env);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *nextEntry(_VOID *list)
{
	return Lists_NextEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *prevEntry(_VOID *list)
{
	return Lists_PrevEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID resetList(_VOID *list)
{
	Lists_ResetList(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findNum(_VOID *e, _VOID *i)
{
	adrEntryType *entry;
	_UWORD *info;
	
	entry = e;
	info = i;
	return entry->number == *info;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID countEntries(_VOID *l, _LONG *ll, _WORD *ww)
{
	*ww = globalLength;
	*ll = Lists_NoOfEntries(l);
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
LOCAL _BOOL nameButton(OBJECT *tree, _VOID *env, _WORD button, _WORD mx, _WORD my, _UWORD kstate, _WORD clicks)
{
	nameInfo *nInfo;
	GRECT r;

	UNUSED(mx);
	UNUSED(my);
	UNUSED(kstate);
	UNUSED(clicks);
	nInfo = env;
	if(button==-1)
		return FALSE;
	button &= 0x7fff;
	DelObjState(tree, button, SELECTED);
	if (button == NBOK)
	{
		ObjcString(tree, NBNAME1, nInfo->newName, sizeof(nInfo->newName));
		if (*nInfo->newName == '\0')
		{
			MTE_infoRSC(AL_EMPTY_ADDR);
			WinDials_WinDialDraw(tree, button, MAX_DEPTH, &r, FALSE);
			return FALSE;
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID nameGetSetValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exit)
{
	nameInfo *nInfo;
	_UWORD num;
	_WORD but;
	adrEntryType *entry;
	
	nInfo = env;
	but = exit & 0x7fff; /* Doppelclick entfernen */
	if (set)
	{
		/* Nach Namen suchen */
		strCpy(nInfo->newName, nInfo->name);
		strCpy(nInfo->newInfo, nInfo->info);
		Lists_ResetList(&data_names[nInfo->nor_alias]);
		if ((nInfo->inList = Lists_ScanEntries(&data_names[nInfo->nor_alias], Lists_forward, findName, nInfo->name)) != FALSE)
		{
			entry = Lists_CurrentEntry(&data_names[nInfo->nor_alias]);
			nInfo->selectedEntry = entry->number;
			if (nInfo->info[0] == '\0')
				getInfo(entry->gName, nInfo->newInfo, sizeof(nInfo->newInfo));
		}
		SetObjcString(tree, NBNAME1, nInfo->newName);
		SetObjcString(tree, NBINFO1, nInfo->newInfo);
	} else
	{
		nInfo->exitBut = but;
		if (but == NBOK)
		{
			ObjcString(tree, NBINFO1, nInfo->newInfo, sizeof(nInfo->newInfo));

			/* den Namen haben wir schon ausgelesen */
			globalLength = Max(globalLength, (_WORD)strLen(nInfo->newName) + 1);
			globalLength = Max(globalLength, (_WORD)strLen(nInfo->newInfo) + 3);
			strBcat(nInfo->newName, "\t");
			strBcat(nInfo->newName, nInfo->newInfo);
			if (nInfo->inList)
			{
				/* Eintrag aendern */
				num = nInfo->selectedEntry;
				Lists_ResetList(&data_names[nInfo->nor_alias]);
				if (Lists_ScanEntries(&data_names[nInfo->nor_alias], Lists_forward, findNum, &num))
				{
					entry = MALLOC(sizeof(adrEntryType) + strLen(nInfo->newName), "nameGetSetValues");
					if (entry != NULL)
					{
						entry->len = (_UWORD)strLen(nInfo->newName);
						entry->number = num;
						entry->selected = TRUE;
						strCpy(entry->gName, nInfo->newName);
						/* Neuen Eintrag anlegen */
						Lists_RemoveEntry(&data_names[nInfo->nor_alias], mem_free);
						if (Lists_InsertEntry(&data_names[nInfo->nor_alias], entry))
						{
							/* Liste speichern */
							adr_SaveNames(nInfo->nor_alias);
						} else
						{
							SFREE(entry); /* Tja, schlecht, der alte ist weg */
						}
					}
				}
			} else
			{
				adr_AppendName(nInfo->nor_alias,nInfo->newName);
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Fuehrt den Dialog fuer einen neuen Namen durch
 */
GLOBAL _BOOL adr_NewNameEntry(_WORD nor_alias,CONST _UBYTE *name, CONST _UBYTE *info)
{
	OBJECT *tree;
	nameInfo *nInfo;
	_UBYTE *title;
	_BOOL changed;
	
	/* Baumadresse holen */
	tree = TreeAddr[NAMEBOX];
	nInfo = NEW(nameInfo, "adr_NewNameEntry");
	if (nInfo == NULL)
		return FALSE;
	/* Nach Namen suchen */
	strBcpy(nInfo->name, name);
	strBcpy(nInfo->info, info);
	nInfo->exitBut = NBQUIT;
	nInfo->nor_alias=nor_alias;
	Lists_ResetList(&data_names[nor_alias]);
	if ((nInfo->inList = Lists_ScanEntries(&data_names[nor_alias], Lists_forward, findName, nInfo->name)) != FALSE)
	{
		title = MausTauschrsc_WrkString(ADR_TITLE_CHANGE);
	} else
	{
		title = MausTauschrsc_WrkString(ADR_TITLE_NEW);
	}
	SetObjcStringAdr(tree, NBTITLE, title, strLen(title) + 1);

	if (WinDials_OpenWinDial(tree, TRUE,
		nameGetSetValues,
		nameGetSetValues,
		nameButton,
		WinDials_defDraw,
		NULL,
		nInfo))
	{
		WinDials_WinDialHandleEvents();
	}
	changed = nInfo->exitBut == NBOK;
	OFREE(nInfo);
	return changed;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID newEntry(_WORD obj, _VOID *env, LISTINFO inf, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID newEntry(_WORD obj, _VOID *env, LISTINFO inf, _BOOL *draw, _BOOL *exit)
*/
{
	group_line name;
	group_line info;
	adrEntryType *entry;
	GRECT r;
	OBJECT *adr;

	UNUSED(inf);
	adr = ((adrInfo *)env)->adr;

	/* selektieren Eintrag finden */
	Lists_ResetList(&data_names[((adrInfo *)env)->nor_alias]);
	if (Lists_ScanEntries(&data_names[((adrInfo *)env)->nor_alias], Lists_forward, findNameSel, NULL))
	{
		entry = Lists_CurrentEntry(&data_names[((adrInfo *)env)->nor_alias]);
		getName(entry->gName, name, sizeof(name));
		getInfo(entry->gName, info, sizeof(info));
	} else
	{
		ObjcString(adr, NEWADR, name, sizeof(name));
		*info = '\0';
	}
	adr_NewNameEntry(((adrInfo *)env)->nor_alias,name, info);
	DelObjState(adr, obj, SELECTED);
	WinDials_WinDialDraw(adr, obj, 0, &r, FALSE);
	*draw = TRUE;
	*exit = FALSE;
	*lstSel=-1;	/*DYJ 5.30f */
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
/*DYJ 5.30f Start*/
LOCAL _VOID delEntry(_WORD obj, _VOID *env, LISTINFO inf, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID delEntry(_WORD obj, _VOID *env, LISTINFO inf, _BOOL *draw, _BOOL *exit)
*/
{
	adrEntryType *entry;
	_UWORD num;
	GRECT r;
	OBJECT *adr;

	UNUSED(inf);
	adr = ((adrInfo *)env)->adr;
	
	*draw = FALSE;
	/* selektieren Eintrag finden */
	Lists_ResetList(&data_names[((adrInfo *)env)->nor_alias]);
	if (Lists_ScanEntries(&data_names[((adrInfo *)env)->nor_alias], Lists_forward, findNameSel, NULL))
	{
		if (mtAlerts_AlertSichLRSC(1,2, AL_DEL_NAME) == 1)
		{
			entry = Lists_CurrentEntry(&data_names[((adrInfo *)env)->nor_alias]);
			num = entry->number;
			Lists_RemoveEntry(&data_names[((adrInfo *)env)->nor_alias], mem_free);
			Lists_ResetList(&data_names[((adrInfo *)env)->nor_alias]);
			*lstSel=-1;	/*DYJ 5.30f */
			while ((entry = Lists_NextEntry(&data_names[((adrInfo *)env)->nor_alias])) != NULL)
			{
				if (entry->number > num)
					--entry->number;
			}
			/* Liste jetzt noch speichern */
			adr_SaveNames(((adrInfo *)env)->nor_alias);
			*draw = TRUE;
		}
	}
	DelObjState(adr, obj, SELECTED);
	WinDials_WinDialDraw(adr, obj, 0, &r, FALSE);
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID updateNameButton(_VOID *entry, _VOID *env)
{
	_BOOL oneSel;
	GRECT r;
	adrInfo *info;
	OBJECT *adr;
	
	UNUSED(entry);
	info = env;
	adr = info->adr;
	Lists_ResetList(&data_names[info->nor_alias]);
	oneSel = Lists_ScanEntries(&data_names[info->nor_alias], Lists_forward, findNameSel, NULL);
	if (oneSel == GetObjState(adr, DELUSER, DISABLED))
	{
		EnableObjState(adr, DELUSER, DISABLED, !oneSel);
		WinDials_WinDialDraw(adr, DELUSER, 0, &r, FALSE);
	}
	if (oneSel && !info->isChange)
	{
		SetObjcString(adr, ADR_NEW_USER, MausTauschrsc_WrkString(ADR_CHANGE));
		WinDials_WinDialDraw(adr, ADR_NEW_USER, 0, &r, FALSE);
		info->isChange = TRUE;
	} else if (!oneSel && info->isChange)
	{
		SetObjcString(adr, ADR_NEW_USER, MausTauschrsc_WrkString(ADR_NEW));
		WinDials_WinDialDraw(adr, ADR_NEW_USER, 0, &r, FALSE);
		info->isChange = FALSE;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID drawNameEntry(_VOID *entry, _VOID *env, _WORD x, _WORD y, _WORD offset, CONST GRECT *clip)
{
	adrEntryType *e;
	group_line str;
	OBJECT *tree = TreeAddr[LISTTXT];

	UNUSED(offset);
	e = entry;
	tree[ROOT].ob_x = x;
	tree[ROOT].ob_y = y;
	tree[ROOT].ob_width = globalLength;
	if (e != NULL)
	{
		/* Jetzt hier den String fuer den Gruppennamen zusammenbauen */
		getName(e->gName, str, sizeof(str));
		SetObjcStringAdr(tree, LISTENTRY, str, sizeof(str));
		EnableObjState(tree, ROOT, SELECTED, nameIsSelected(entry, env));
		EnableObjState(tree, ROOT, DISABLED, !isEnabled(entry, env));
		Objc_draw(tree, ROOT, MAX_DEPTH, clip);
		tree[ROOT].ob_y += CharHeight;
		strCpy(str, "  ");
		getInfo(e->gName, str + 2, sizeof(str) - 2);
	} else
	{
		SetObjcStringAdr(tree, LISTENTRY, "", 1);
		EnableObjState(tree, ROOT, SELECTED, FALSE);
		EnableObjState(tree, ROOT, DISABLED, FALSE);
		Objc_draw(tree, ROOT, MAX_DEPTH, clip);
		tree[ROOT].ob_y += CharHeight;
	}
	Objc_draw(tree, ROOT, MAX_DEPTH, clip);
	SetObjcStringAdr(tree, LISTENTRY, "", 1);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID setAndGetNameValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exitBut)
{
	adrInfo *adr;

	UNUSED(tree);
	if (set)
	{
	} else
	{
		adr = env;
		adr->exitBut = exitBut;
	}
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */	/*DYJ 5.23i */
GLOBAL _BOOL adr_SelectName(_WORD nor_alias,_UBYTE *Name, size_t namesize)
{
	_WORD exit;
	adrEntryType *selAdr;
	_WORD maxLength;
	GRECT r;
	group_line str;
	ldHandler dialHandler;
	group_line name;
	adrInfo *env;
	_WORD intval,xFlags;
	_BOOL readOnly=TRUE;
	_BOOL mayBeEmpty=FALSE;	
	
	if (nameDialIsOpen)
		return FALSE;
	env = NEW(adrInfo, "adr_SelectName");
	if (env == NULL)
		return FALSE;
	env->adr = TreeAddr[ADR];
	MausTauschrsc_AdjustDialogSize(env->adr, ADR);

	/* Strings setzen */
	SetObjcString(env->adr, ADRTITEL, MausTauschrsc_WrkString(titles[nor_alias]));
	switch(nor_alias)
	{
/*DYJ 5.35c Start*/
		case ADR_NORMAL_EMPTY:
		case ADR_STDTO:
		case ADR_MLIST:
			mayBeEmpty=TRUE;
/*DYJ  Ende; alt:	
		case ADR_NORMAL_EMPTY:
			mayBeEmpty=TRUE;
		case ADR_MLIST:		/*DYJ 5.34d */
*/
			nor_alias=ADR_NORMAL;
		case ADR_NORMAL:
		case ADR_ALIAS:
			readOnly=FALSE;
			break;
		case ADR_N_REPLY:
		case ADR_N_SENDER:
		case ADR_N_FROM:
			readOnly=TRUE;
			nor_alias=ADR_NORMAL;
			mayBeEmpty=TRUE;
			break;
		case ADR_A_REPLY:
		case ADR_A_SENDER:
		case ADR_A_FROM:
			readOnly=TRUE;
			nor_alias=ADR_ALIAS;
			mayBeEmpty=TRUE;
			break;
	}
	EnableObjFlags(env->adr, ADR_NEW_USER, HIDETREE, readOnly);
	EnableObjFlags(env->adr, DELUSER, HIDETREE, readOnly);
		

	env->nor_alias=nor_alias;

	SetObjcString(env->adr, NEWADR, Name);
	SetObjcString(env->adr, ADR_NEW_USER, MausTauschrsc_WrkString(ADR_NEW));
	env->isChange = FALSE;

	/* Maximale Breite eines Listeneintrages herausfinden */
	maxLength = 0;
	Lists_ResetList(&data_names[nor_alias]);
	while ((selAdr = Lists_NextEntry(&data_names[nor_alias])) != NULL)
	{
		selAdr->selected = FALSE;
		getName(selAdr->gName, str, sizeof(str));
		maxLength = Max(maxLength, (_WORD)strLen(str) + 1);
		getInfo(selAdr->gName, str, sizeof(str));
		maxLength = Max(maxLength, (_WORD)strLen(str) + 3);
	}

	/* Jetzt das Object zusammenbauen */
	CalcArea(env->adr, ADRBOX, &r);
	maxLength = Max(maxLength * CharWidth, r.g_w);

	globalLength = maxLength;

	/* specials bestimmen */
	env->specials[0].objc = ADR_NEW_USER;
	env->specials[0].proc = newEntry;
	env->specials[1].objc = DELUSER;
	env->specials[1].proc = delEntry;

	nameDialIsOpen = TRUE;
	/* WindowDialog vorbereiten */
	ListDl_BuildLdHandler(&data_names[nor_alias],
		resetList,
		nextEntry,
		prevEntry,
		countEntries,
		isEnabled,
		selNameEntry,
		nameToStr,
		nameIsSelected,
		drawNameEntry,
		FUNK_NULL,
		0, 2 * CharHeight,
		8, globalLength,
		&dialHandler);
		ConfVars_GetConfDefInt(cLocateAdrMode, &intval, 4);
		xFlags=ldTabEqNxt|ldMatchAny;
		switch(intval)
		{
			case 0:		/*sucht am Anfang des Namensfeldes (Originalmodus)*/
				xFlags=0;
				break;
			case 1:		/* sucht irgendwo im Namensfeld*/
				break;
			case 2:	/* sucht am Anfang des Infofeldes*/
				xFlags=ldTabEqNxt;
			case 3:	/* sucht irgendwo im Infofeld*/
				ListDl_BuildLdHandlerX(infoToStr,&dialHandler);
				break;
			case 4:	/* sucht irgendwo im Namensfeld und Infofeld*/
				ListDl_BuildLdHandlerX(nameInfoToStr,&dialHandler);
		}
	env->exitBut = ADR_CANCEL;
	/* ListWinDialog oeffnen */
	if (ListDl_WinListDial(env->adr,
		ldSelect | ldArrows | ldDoubleExit | ldAutolocate | xFlags | ldModal,
		&dialHandler,
		ADRBOX,
		ADRBACK,
		ADROK,
		ADR_CANCEL,
		env,
		ADROK,
		NEWADR,
		updateNameButton,
		env->specials,
		2,
		ListDl_dummyCheckExit,
		setAndGetNameValues,
		setAndGetNameValues,
		&env->v))
	{
		exit = env->exitBut;
		nameDialIsOpen = FALSE;
		ObjcString(env->adr, NEWADR, name, sizeof(name));
		if (exit == ADROK && name[0] == '\0' && !mayBeEmpty)
		{
			OFREE(env);
			return FALSE;
		}
		if (exit != ADR_CANCEL)
		{
			strMcpy(Name, namesize, name);
			OFREE(env);
			return TRUE;
		}
	}
	nameDialIsOpen = FALSE;
	OFREE(env);
	return FALSE;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL adr_exist(_WORD nor_alias, _UBYTE *name)						*/
/*	Prft ob der Name in der angegebenen Adressenliste ist						*/
/*	Parameter:	or_alias:	Welche der 2 Listen									*/
/*				name:		der gesuchte Name, kann auch Liste sein				*/
/*	Ergebnis:	gibt Pointer auf die erste gefundene Adresse zurck oder NULL	*/
/*	27.4.00 von Dimitri Junker	DYJ 4.15										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL CONST _UBYTE *adr_exist(_WORD nor_alias, _UBYTE *name)
{
	group_line aliasName;
	CONST _UBYTE *ret=NULL;
	adrEntryType *selAdr;
	if(nor_alias==ADR_TRUE)
		return "";
	
	if(nor_alias==ADR_EIGENE)
	{
		if ((ret=strCaseWord(name, userAddr))!=NULL)
			return ret;
		nor_alias=ADR_ALIAS;
	}
	Lists_ResetList(&data_names[nor_alias]);
	while (ret==NULL && (selAdr = Lists_NextEntry(&data_names[nor_alias])) != NULL)
	{
		getName(selAdr->gName, aliasName, sizeof(aliasName));
		if(*aliasName)
			ret=strCaseWord(name, aliasName);
	}
	return ret;
}
		
/*** ------------------------------------------------------------------------ ***/
/*	_BOOL adr_getRealNameOfAlias(_UBYTE *realName,size_t len,_UBYTE *adresse)	*/
/*	Ermittelt den Realname zu einer Aliasadresse								*/
/*	Parameter:	realName,len: Ziel und Lnge									*/
/*				adresse:		Die Adresse										*/
/*	Ergebnis:	Erfolgreich?													*/		
/*	11.1.02 von Dimitri Junker	DYJ 4.34j										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL adr_getRealNameOfAlias(_UBYTE *realName,size_t len,_UBYTE *adresse)
{
	group_line aliasName;
	_BOOL ret=FALSE,hasRN;
	adrEntryType *selAdr;
	
	ConfVars_GetConfDefBool(cAliasInfoIsRName, &hasRN,FALSE);
	if(hasRN)
	{
		Lists_ResetList(&data_names[ADR_ALIAS]);
		while (!ret && (selAdr = Lists_NextEntry(&data_names[ADR_ALIAS])) != NULL)
		{
			getName(selAdr->gName, aliasName, sizeof(aliasName));
			if(*aliasName)
				ret=(strCaseWord(adresse, aliasName)!=NULL);
		}
		if(ret)
			getInfo(selAdr->gName, realName, len);
	}
	return ret;
}
