/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <alias.h>
#include <confvars.h>
#include <varnamel.h>
#include <lists.h>
#include <wildcard.h>
#include <listdl.h>
#include <routinec.h>
#include <vdiutil.h>
#include <portvdi.h>
#include <cathelp.h>
#include <mtappl.h>
#include <mtutils.h>
#include <routines.h>

#include <maustaus.h>

typedef struct {
	_UBYTE *entry;
	_BOOL selected;
} aliasEntry;

typedef struct {
	_WORD exitBut;
	specialButHdler specials[6];
	_VOID *lastSel;
	_WORD globalLength;
	List aliasList;
	List *aliasGroups;
} aliasInfo;

LOCAL OBJECT *aliasBox;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID *nextEntry(_VOID *list)
{
	return Lists_NextEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *prevEntry(_VOID *list)
{
	return Lists_PrevEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID resetList(_VOID *list)
{
	Lists_ResetList(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID countEntries(_VOID *l, _LONG *ll, _WORD *ww)
{
	List *lp;
	aliasInfo *info;
	
	lp = l;
	info = lp->user;
	*ww = info->globalLength * CharWidth;
	*ll = Lists_NoOfEntries(lp);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isEnabled(_VOID *entry, _VOID *env)
{
	UNUSED(entry);
	UNUSED(env);
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL selGrEntry(_VOID *entry, _VOID *env, _WORD line)
{
	aliasEntry *gr;

	UNUSED(env);
	UNUSED(line);
	gr = entry;
	if (gr == NULL)
		return FALSE;
	gr->selected = !gr->selected;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID groupToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)
{
	aliasEntry *gr;
	aliasInfo *info;
	
	info = env;
	gr = entry;
	if (gr == NULL)
	{
		*str = '\0';
	} else
	{
		strMcpy(str, strsize, gr->entry);
		info->globalLength = Max(info->globalLength, (_WORD)strLen(str) + 2);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isSelected(_VOID *entry, _VOID *inf)
{
	aliasEntry *gr = entry;

	UNUSED(inf);
	return gr->selected;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID disposeEntry(_VOID *e)
{
	aliasEntry *entry;
	
	entry = e;
	if (entry != NULL)
	{
		if (entry->entry != NULL)
			SFREE(entry->entry);
		OFREE(entry);
	}
}
	
/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID updateButton(_VOID *entry, _VOID *env)
{
	_BOOL oneSel;
	GRECT r;
	aliasInfo *info;
	
	UNUSED(entry);
	info = env;
	Lists_ResetList(&info->aliasList);
	oneSel = Lists_ScanEntries(&info->aliasList, Lists_forward, isSelected, NULL);
	if ((oneSel && GetObjState(aliasBox, ALIAS_DELETE, DISABLED)) ||
		(!oneSel && !GetObjState(aliasBox, ALIAS_DELETE, DISABLED)))
	{
		EnableObjState(aliasBox, ALIAS_DELETE, DISABLED, !oneSel);
		WinDials_WinDialDraw(aliasBox, ALIAS_DELETE, 0, &r, FALSE);
		EnableObjState(aliasBox, ALIAS_CHANGE, DISABLED, !oneSel);
		WinDials_WinDialDraw(aliasBox, ALIAS_CHANGE, 0, &r, FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID drawGroupEntry(_VOID *entry, _VOID *env, _WORD x, _WORD y, _WORD offset, CONST GRECT *clip)
{
	aliasEntry *e;
	_UBYTE str[256];
	aliasInfo *info;
	GRECT fill;
	_WORD pxy[4];
	_BOOL isSel;
	_UWORD eff;
	_WORD fillcol;
	
	UNUSED(offset);
	info = env;
	e = entry;

	fill.g_x = x;
	fill.g_y = y;
	fill.g_w = fill.g_x + info->globalLength * CharWidth - 1;
	fill.g_h = fill.g_y + CharHeight - 1;
	pxy[0] = clip->g_x;
	pxy[1] = clip->g_y;
	pxy[2] = clip->g_x + clip->g_w - 1;
	pxy[3] = clip->g_y + clip->g_h - 1;

	vs_clip(PrivateWS, TRUE, pxy);
	vswr_mode(PrivateWS, MD_REPLACE);

	fillcol = WHITE;
	isSel = entry != NULL && isSelected(entry, env);
	if (isSel)
		fillcol = BLACK;
	MouseOff();
	vsf_color(PrivateWS, fillcol);
	vr_recfl(PrivateWS, (_WORD *)&fill);
	
	if (e != NULL)
	{
		/* Jetzt hier den String fuer den Gruppennamen zusammenbauen */
		groupToStr(e, env, str, sizeof(str));
		eff = TXT_NORMAL;
		if (!isEnabled(entry, env))
			eff |= TXT_LIGHT;
		if (isSel)
			vswr_mode(PrivateWS, MD_XOR);
		vst_effects(PrivateWS, eff);
		v_gtext(PrivateWS, x, y, str);
		vst_effects(PrivateWS, 0);
		vswr_mode(PrivateWS, MD_REPLACE);
	}
	vs_clip(PrivateWS, FALSE, pxy);
	MouseOn();
}

/*------------------------------------------------------------------------*/

LOCAL _VOID killAliasList(aliasInfo *info)
{
	Lists_DeleteSimpleList(&info->aliasList, disposeEntry, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID Alias_SaveList(List *aliasGroups)
{
	_WORD i;
	_UBYTE varName[128];
	_UBYTE *pattern;
	
	i = 0;
	for (;;)
	{
		make_varname(varName, sizeof(varName), cShowAliasName, i);
		if (!ConfVars_DeleteConfigVar(varName))
			break;
		i++;
	}
	i = 0;
	Lists_ResetList(aliasGroups);
	while ((pattern = Lists_NextEntry(aliasGroups)) != NULL)
	{
		make_varname(varName, sizeof(varName), cShowAliasName, i);
		if (!ConfVars_SetConfigString(varName, pattern))
			break;
		i++;
	}
}

/*------------------------------------------------------------------------*/

LOCAL _VOID setAndGetValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exitBut)
{
	aliasInfo *info;
	_BOOL aliasNames;
	_BOOL succ;
	aliasEntry *group;
	
	UNUSED(tree);
	info = env;
	if (set)
	{
		EnableObjState(aliasBox, ALIAS_DELETE, DISABLED, TRUE);
		EnableObjState(aliasBox, ALIAS_CHANGE, DISABLED, TRUE);
		ConfVars_GetConfDefBool(cShowAliasNames, &aliasNames, FALSE);
		EnableObjState(aliasBox, ALIAS_USE, SELECTED, aliasNames);
	} else
	{
		info->exitBut = exitBut;

		if (exitBut == ALIAS_OK)
		{
			aliasNames = GetObjState(aliasBox, ALIAS_USE, SELECTED);
			ConfVars_SetConfigBool(cShowAliasNames, aliasNames);
			/* Veraenderte Liste speichern */
			/* Erstmal die alte loeschen */
			Alias_DeleteList(FALSE);
			Lists_ResetList(info->aliasGroups);
			Lists_ResetList(&info->aliasList);
			succ = TRUE;
			while (succ && (group = Lists_NextEntry(&info->aliasList)) != NULL)
			{
				if (!Lists_AppendEntry(info->aliasGroups, group->entry))
					succ = FALSE;
				else
					group->entry = NULL;
			}
			if (succ)
				Alias_SaveList(info->aliasGroups);
		}
		/* Interne Liste wieder freigeben */
		killAliasList(info);
		OFREE(info);
	}
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID helpButton(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
{
	GRECT r;
	UNUSED(lstSel);
/*DYJ  Ende; alt:	
LOCAL _VOID helpButton(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
{
	GRECT r;
*/

	UNUSED(info);
	UNUSED(env);
	CatHelp_DoHelp(CatHelp_alias);
	DelObjState(aliasBox, obj, SELECTED);
	WinDials_WinDialDraw(aliasBox, obj, 1, &r, FALSE);
	*draw = FALSE;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL ChangeEntry(_UBYTE *entry, size_t len)
{
	OBJECT *dialog;

	dialog = TreeAddr[ALIAS_BX];
	SetObjcString(dialog, ALIAS_NAME, entry);
	if (VDIUtil_FormHandle(dialog) == ALIAS_NOK)
	{
		ObjcString(dialog, ALIAS_NAME, entry, len);
		return TRUE;
	}
	return FALSE;
}

/*------------------------------------------------------------------------*/

/*DYJ 5.30f Start*/
LOCAL _VOID changeGroup(_WORD obj, _VOID *env, LISTINFO listinfo, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:
LOCAL _VOID changeGroup(_WORD obj, _VOID *env, LISTINFO listinfo, _BOOL *draw, _BOOL *exit)
*/
{
	_UBYTE cPath[256];
	_BOOL oneSel;
	_BOOL ok;
	aliasEntry *gr;
	aliasEntry *new;
	_UBYTE *path;
	aliasInfo *info;
	GRECT r;
	
	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(listinfo);
	
	info = env;
	*draw = FALSE;
	Lists_ResetList(&info->aliasList);
	oneSel = Lists_ScanEntries(&info->aliasList, Lists_forward, isSelected, NULL);
	if (oneSel)
	{
		gr = Lists_CurrentEntry(&info->aliasList);
		strBcpy(cPath, gr->entry);
	} else
	{
		*cPath = '\0';
	}
	if (ChangeEntry(cPath, sizeof(cPath)))
	{
		/* Neuen Eintrag erzeugen */
		path = STRDUP(cPath, "changeGroup: path");
		if (path != NULL)
		{
			new = NEW(aliasEntry, "changeGroup: entry");
			if (new != NULL)
			{
				new->entry = path;
				info->globalLength = Max(info->globalLength, (_WORD)strLen(cPath) + 2);
				new->selected = FALSE;
				/* Jetzt nach Objekt unterscheiden */
				if (obj == ALIAS_NEW)
				{
					/* Neuen Eintrag anlegen */
					ok = Lists_AppendEntry(&info->aliasList, new);
				} else
				{
					/* vorhandenen Eintrag suchen und ersetzen */
					Lists_ResetList(&info->aliasList);
					Lists_ScanEntries(&info->aliasList, Lists_forward, isSelected, NULL);
					Lists_RemoveEntry(&info->aliasList, disposeEntry);
					ok = Lists_InsertEntry(&info->aliasList, new);
				}
				if (!ok)
					disposeEntry(new);
				*draw = TRUE;
			} else
			{
				SFREE(path);
			}
		}
	}
	DelObjState(aliasBox, obj, SELECTED);
	WinDials_WinDialDraw(aliasBox, obj, 1, &r, FALSE);
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID cutEntry(_WORD obj, _VOID *env, LISTINFO listinfo, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID cutEntry(_WORD obj, _VOID *env, LISTINFO listinfo, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;
	_BOOL found;
	aliasInfo *info;
	
	UNUSED(lstSel);	/*DYJ 5.30f */
	info = env;
	/* Selektierten Eintrag in der Liste finden und aus der Liste entfernen */
	Lists_ResetList(&info->aliasList);
	if ((found = Lists_ScanEntries(&info->aliasList, Lists_forward, isSelected, NULL)) != FALSE)
	{
		Lists_RemoveEntry(&info->aliasList, disposeEntry);
		ListMan_ListScroll2Selection(listinfo, TRUE);
	}
	DelObjState(aliasBox, obj, SELECTED);
	WinDials_WinDialDraw(aliasBox, obj, 1, &r, FALSE);
	*draw = found;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Alias_Config(_VOID)
{
	aliasEntry *group;
	_WORD maxLength;
	GRECT r;
	_UBYTE *entry;
	_BOOL succ;
	ldHandler aliasHandler;
	aliasInfo *info;
	
	info = NEW(aliasInfo, "Alias_Config");
	if (info == NULL)
		return;
	
	aliasBox = TreeAddr[ALIAS_DIALOG];
	
	/* specials bestimmen */
	info->specials[0].objc = ALIAS_DELETE;
	info->specials[0].proc = cutEntry;
	info->specials[1].objc = ALIAS_NEW;
	info->specials[1].proc = changeGroup;
	info->specials[2].objc = ALIAS_CHANGE;
	info->specials[2].proc = changeGroup;
	info->specials[3].objc = ALIAS_HELP;
	info->specials[3].proc = helpButton;
	
	/* Jetzt die Liste aufbauen */
	if (!Lists_CreateList(&info->aliasList, "Alias_Config: aliasList"))
	{
		OFREE(info);
		return;
	}
	info->aliasList.user = info;
	info->aliasGroups = Alias_GetList();
	
	succ = TRUE;
	Lists_ResetList(info->aliasGroups);
	while (succ && (entry = Lists_NextEntry(info->aliasGroups)) != NULL)
	{
		group = NEW(aliasEntry, "Alias_Config: entry");
		if (group == NULL)
		{
			succ = FALSE;
		} else
		{
			group->entry = STRDUP(entry, "Alias_Config: pattern");
			if (group->entry == NULL)
			{
				OFREE(group);
				succ = FALSE;
			} else
			{
				group->selected = FALSE;
				if (!Lists_AppendEntry(&info->aliasList, group))
				{
					SFREE(group->entry);
					OFREE(group);
					succ = FALSE;
				}
			}
		}
	}
	
	if (!succ)
	{
		killAliasList(info);
		OFREE(info);
		return;
	}
	
	/* Maximale Breite eines Listeneintrages herausfinden */
	maxLength = 0;
	Lists_ResetList(&info->aliasList);
	while ((group = Lists_NextEntry(&info->aliasList)) != NULL)
	{
		if ((_WORD)strLen(group->entry) > maxLength)
			maxLength = (_WORD)strLen(group->entry);
	}
	++maxLength;   /* damit es besser aussieht :-) */
	
	/* Jetzt das Object zusammenbauen */
	CalcArea(aliasBox, ALIAS_BOX, &r);
	maxLength = Max(maxLength, r.g_w / CharWidth);
	
	info->globalLength = maxLength;
	
	/* Handler bauen */
	ListDl_BuildLdHandler(&info->aliasList,
		resetList,
		nextEntry,
		prevEntry,
		countEntries,
		isEnabled,
		selGrEntry,
		groupToStr,
		isSelected,
		drawGroupEntry,
		FUNK_NULL,
		0, CharHeight, 
		8, maxLength * CharWidth,
		&aliasHandler);
	
	/* ListDialog durchfuehren */
	if (ListDl_WinListDial(aliasBox, ldSelect | ldArrows | ldDoubleExit | ldModal,
		&aliasHandler,
		ALIAS_BOX,
		ALIAS_BACK,
		ALIAS_OK,
		ALIAS_CANCEL,
		info,
		ALIAS_CHANGE,
		-1,
		updateButton,
		info->specials,
		3,
		ListDl_dummyCheckExit,
		setAndGetValues,
		setAndGetValues,
		&info->lastSel))
	{
	} else
	{
		/* Interne Liste wieder freigeben */
		killAliasList(info);
		OFREE(info);
	}
}

