/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 * 24.2.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.21j'
 *	Beim Backup der Outfiles wurde statt der gepackten Outfiles der Packer gesichert ;-)
 */


#include <arcstart.h>
#include <mtpaths.h>
#include <userinfo.h>
#include <catgloba.h>
#include <catlog.h>
#include <confvars.h>
#include <varnamel.h>
#include <execv.h>
#include <arcass.h>

#include <mtappl.h>
#include <mtdir.h>
#include <tos.h>
#include <mintutil.h>

#include <routinec.h>
#include <file_io.h>
#include <routines.h>
#include <CATFILES.h>

LOCAL _BOOL outputDone;


/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ClearScreen(_VOID)
{
	/* geaendert von Frank Roeske (Frank_Roeske@LU2.maus.de)                  
	 */
	/* es wird zuerst ueberprueft, ob evtl. die Variable NoCLS in der catuser.inf
	 * gesetzt wurde, und somit der ClearScreen abschaltbar gemacht.        
	 */
	_BOOL var;
	
	ConfVars_GetConfDefBool(cNoCLS, &var, FALSE);
	if (!var)
	{
		Cconout(0x1b);
		Cconout('E');
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID outProc(_UBYTE ch)
{
	outputDone = TRUE;
	CatLog_Write(ch);
}

/*** ---------------------------------------------------------------------- ***/

/* Wenn noRedirect angegeben ist, dann wird die Ausgabe des Programmes
 * nie umgelenkt.
 */
GLOBAL _WORD ARCStarter_ExecProgram(CONST _UBYTE *name, CONST _UBYTE *cmd, _UWORD flags)
{
	_WORD pid;
	FILENO pipe;
	_LONG size;
	_UBYTE arcName[PATH_MAX];
	_UBYTE currentPath[PATH_MAX];
	_UBYTE path[PATH_MAX];
	_WORD res;
	
	outputDone = FALSE;
	mtDir_SplitPath(name, path, arcName, currentPath);
	F_Path_Get(currentPath);
	F_Path_Set(path);
	if (IsMiNT() && !(flags & ARCSTART_NOREDIRECT))
	{
		/* Packer starten und Ausgabe in Pipe umlenken und Pipe leerlesen */
		pid = ExecPipe(name, cmd, NULL, &pipe);
		if (pid > 0)
		{
			do
			{
				size = ReadPipe(pid, pipe, arcName, sizeof(arcName) - 1, &res);
				if (size > 0 && (flags & ARCSTART_TOLOG))
				{
					if (size < sizeof(arcName))
						arcName[size] = '\0';
					outputDone = TRUE;
					CatLog_WriteString(arcName);
				}
			} while (size >= 0);
			res = 0;
		} else
		{
			if (pid == 0)
				res = -1;
			else
				res = pid;
			return res;
		}
	} else
	{
		if (UserBLK.RedirectPacker && !(flags & ARCSTART_NOREDIRECT))
		{
			ARCAss_SetTrap();
			if ((flags & ARCSTART_TOLOG) && magIx && magIxVer >= 0x200)
				ARCAss_SetOutputProc(outProc);
		} else
		{
			ClearScreen();
		}
		if (cmd[0] > 126)   /* Laenge der Kommandozeile */
		{
			_UBYTE buf[256]; /* mehr geht momentan schon aus anderen Gruenden nicht */
			
			strMcpy(buf + 1, sizeof(buf) - 1, cmd + 1);
			buf[0] = (_UBYTE)strLen(buf + 1);
			res = ExecV_Pexec(0, name, buf, NULL,FALSE);
		} else
		{
			res = (_WORD)Pexec(0, name, cmd, NULL);
		}
		if (UserBLK.RedirectPacker && !(flags & ARCSTART_NOREDIRECT))
		{
			ARCAss_RestoreTrap();
		} else
		{
			if (flags & ARCSTART_REDRAW)
				FullScreenRedraw();
		}
	}
	if ((flags & ARCSTART_TOLOG) && outputDone)
		CatLog_WriteLn();
	F_Path_Set(currentPath);
	return res;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.20zd */	/*DYJ 5.21j */
GLOBAL _WORD ARCStarter_DeArc(_BOOL toLog)
{
	_UBYTE cmd[128];
	_UBYTE arcName[PATH_MAX];
	_UWORD flags;
	
	if (ConfVars_GetConfigString(cOutfBckPath, arcName, sizeof(arcName)))
		CatFiles_BackupFile(arcName,ImportPath,archiveName);
	strMcpy(cmd + 1, sizeof(cmd) - 1, dearcStr);
	cmd[0] = (_UBYTE)strLen(cmd + 1);
	strBcpy(arcName, ARCPath);
	F_Path_Append(arcName, ARCName);
	flags = ARCSTART_REDRAW;
	if (toLog)
		flags |= ARCSTART_TOLOG;
	return ARCStarter_ExecProgram(arcName, cmd, flags);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD ARCStarter_Archive(_VOID)
{
	_UBYTE cmd[128];
	_UBYTE arcName[PATH_MAX];
	
	strMcpy(cmd + 1, sizeof(cmd) - 1, enarcStr);
	cmd[0] = (_UBYTE)strLen(cmd + 1);
	strBcpy(arcName, ARCPath);
	F_Path_Append(arcName, ARCName);
	return ARCStarter_ExecProgram(arcName, cmd, ARCSTART_REDRAW);
}
