/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <catgscr.h>
#include <catmain.h>
#include <gscript.h>
#include <strconv.h>
#include <ro_mem.h>
#include <wdwmanag.h>
#include <grin.h>
#include <grintool.h>
#include <editor.h>
#include <catgloba.h>
#include <routinec.h>
#include <routines.h>
#include <msglist.h>
#include <catbutto.h>
#include <mtdir.h>
#include <mtedit2.h>
#include <adr.h>
#include <groupsel.h>
#include <listhelp.h>
#include <confvars.h>
#include <varnamel.h>
#include <wiedervo.h>
#include <userinfo.h>
#include <msginfil.h>
/*
 * So meldet sich CAT per Default bei AppGetLongName beim GEMScript-Interpreter
 */
/*DYJ 5.38e Start*/
#define GS_APPLONGNAME "CAT 5"
/*DYJ  Ende; alt:	
#define GS_APPLONGNAME "CAT 4"
*/


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _UBYTE *gs_erg(CONST _UBYTE *res)
{
	_UBYTE *erg;
	size_t len;
	
	/*
	 * 1 Byte mehr als eigentlich noetig, weil das Ergebnis
	 * mit 2 Null-Byte abgeschlossen werden muss
	 */
	len = strLen(res);
	erg = SYSMALLOC(len + 2, MEM_NOERR | MEM_SHARABLE, "gs_erg");
	if (erg != NULL)
	{
		strCpy(erg, res);
		erg[len + 1] = '\0';
	}
	return erg;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *gs_ok(_VOID)
{
	return gs_erg("1");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *gs_error(_VOID)
{
	return gs_erg("0");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *gs_res(_BOOL res)
{
	return res ? gs_ok() : gs_error();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL HWND nameToWdw(CONST _UBYTE *arg)
{
	HWND wdw;
	_BOOL valid;
	
	wdw = StrToInt(arg, NULL, &valid);
	if (!valid)
		return -1;
	return wdw;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *wdwToName(HWND wdw)
{
	_UBYTE str[20];

	IntToStr(wdw, 0, str, sizeof(str));
	return gs_erg(str);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsNewMessage(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	if (argc >= 3 && argv[1][0] != '\0' && argv[2][0] != '\0')
		*erg = gs_res(grinTools_NewMessage(argv[1], argv[2]));
	else
		*erg = gs_error();
}

/*** ---------------------------------------------------------------------- ***/
LOCAL _VOID gsSave(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	UNUSED(argc);
	UNUSED(argv);
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	if (wdw >= 0 && Editor_IsEditTop(wdw))
		*erg = gs_res(Editor_EditSave(wdw, NULL, NULL, FALSE,TRUE));
	else
		*erg = gs_error();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSaveAs(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	UNUSED(argc);
	UNUSED(argv);
	if (argc > 2)
		wdw = nameToWdw(argv[2]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0 && argc > 1 && argv[1] != NULL && argv[1][0] != '\0')
	{
		strBcpy(SavePath, argv[1]);
		F_Path_Dirname(SavePath);
		strBcpy(SaveName, F_Path_Basename(argv[1]));
		if (msgList_listWindowTop(wdw))
		{
			res = msgList_listSaveAs(wdw, FALSE);
		} else if (Editor_IsEditTop(wdw))
		{
			res = Editor_EditSave(wdw, SavePath, SaveName, TRUE,TRUE);
		}
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsClose(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	if (argc <= 1)
	{
		wdw = WdwManager_GetOwnTopWindow();
		if (wdw < 0)
			wdw = CatButtons_WinHandle;			/* ist nicht in der Window-Queue! */
		WdwManager_CloseWindow(wdw, TRUE);
	} else
	{
		while (--argc)
		{
			wdw = nameToWdw(*++argv);
			WdwManager_CloseWindow(wdw, FALSE);
		}
	}
	*erg = gs_ok();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsGetFront(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	UNUSED(argc);
	UNUSED(argv);
	wdw = WdwManager_GetOwnTopWindow();
	if (wdw < 0)
		wdw = CatButtons_WinHandle;			/* ist nicht in der Window-Queue! */
	*erg = wdwToName(wdw);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsToFront(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	if (argc > 1)
	{
		wdw = nameToWdw(argv[1]);
	} else
	{
		wdw = WdwManager_GetOwnTopWindow();
		if (wdw < 0)
			wdw = CatButtons_WinHandle;		/* ist nicht in der Window-Queue! */
	}
	if (wdw >= 0)
	{
		WdwManager_TopWindow(wdw);
		*erg = gs_ok();
	} else
	{
		*erg = gs_error();
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsCopy(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0)
	{
		res = copyWdw(wdw, FALSE);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsCut(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0)
	{
		res = cutWdw(wdw, FALSE, TRUE);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsPaste(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	if (wdw >= 0)
	{
		Editor_EditPaste(wdw);
		*erg = gs_ok();
	} else
	{
		*erg = gs_error();
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsDelete(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0)
	{
		res = cutWdw(wdw, FALSE, FALSE);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsPrint(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	if (wdw >= 0)
	{
		printFile(wdw);
	}
	*erg = gs_ok();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSelectAll(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0)
	{
		res = selectAll(wdw);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsDeselectAll(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0)
	{
		res = deSelectAll(wdw);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsCheckApp(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_UBYTE path[PATH_MAX];
	_UBYTE name[PATH_MAX];
	_WORD chPid;
	
	if (argc > 1 && argv[1] != NULL && argv[1][0] != '\0')
	{
		strBcpy(path, argv[1]);
		F_Path_Dirname(path);
		strBcpy(name, F_Path_Basename(argv[1]));
		StartPrg(path, name, (CONST _UBYTE *)"", TRUE, TRUE, &chPid);
	}
	*erg = gs_ok();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsOpen(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_UBYTE suff[PATH_MAX];
	
	if (argc > 1 && argv[1] != NULL && argv[1][0] != '\0')
	{
		mtDir_SplitPath(argv[1], openPath, openSel, suff);
		MTEdit2_OpenEditWindow(openPath, openSel, FALSE, FALSE);
	} else
	{
		handleFSelMessage(FALSE, WdwManager_GetOwnTopWindow());
	}
	*erg = gs_ok();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsUndo(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	if (wdw >= 0)
	{
		if (Editor_IsEditTop(wdw))
			Editor_DoUndo(wdw);
	}
	*erg = gs_ok();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsReadInf(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_UBYTE suff[PATH_MAX];
	_BOOL res;
	
	if (argc > 1 && argv[1] != NULL && argv[1][0] != '\0')
	{
		/* Ist wohl ein Dateiname, also INF laden */
		mtDir_SplitPath(argv[1], infPath, infName, suff);
		res = ReadInfoFile(FALSE, FALSE);
	} else
	{
		res = ReadInfoFile(TRUE, FALSE);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSelectName(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_UBYTE name[256];
	
	if (argc > 1)
		strBcpy(name, argv[1]);
	else
		*name = '\0';
	if (!adr_SelectName(ADR_NORMAL,name, sizeof(name)))
		*name = '\0';
	*erg = gs_erg(name);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSelectGroup(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	group_line gruppe;
	CAT_GROUP_NUMBER ca;
	
	if (argc > 1)
	{
		strBcpy(gruppe, argv[1]);
		ListHelp_SetLastGroup(gruppe);
	} else
	{
		*gruppe = '\0';
	}
	if (!ListHelp_SelectGroup(gruppe, sizeof(gruppe), &ca, TRUE, FALSE, FALSE, gsmWrite) ||
		GroupSelect_isPrivate(ca))
		*gruppe = '\0';
	*erg = gs_erg(gruppe);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSelectSubject(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	group_line Wegen;
	
	if (argc > 1)
		strBcpy(Wegen, argv[1]);
	else
		*Wegen = '\0';
	if (!grinTools_BetreffHolen(-1, Wegen, sizeof(Wegen), "Stichwort f\201r neue Mitteilung", "", "Wegen:", BH_LEER))
		*Wegen = '\0';
	*erg = gs_erg(Wegen);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsWhoami(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	if (argc > 1)
	{
		strBcpy(userAddr, argv[1]);
		ConfVars_SetConfigString(cMausId, userAddr);
	}
	*erg = gs_erg(userAddr);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsFrom(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActFromSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsFrom");
		if (*erg != NULL)
		{
			grin_ActualFrom(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsRealname(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActNameSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsRealname");
		if (*erg != NULL)
		{
			grin_ActualName(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsTo(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActReceiverSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsTo");
		if (*erg != NULL)
		{
			grin_ActualReceiver(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsMausID(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActIdSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsMausID");
		if (*erg != NULL)
		{
			grin_ActualID(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsMsgID(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActMIdSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsMsgID");
		if (*erg != NULL)
		{
			grin_ActualMId(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsMausRef(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActRefIdSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsMausRef");
		if (*erg != NULL)
		{
			grin_ActualRefId(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsRef(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActRIdSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsRef");
		if (*erg != NULL)
		{
			grin_ActualRId(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSubject(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActSubjectSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsSubject");
		if (*erg != NULL)
		{
			grin_ActualSubject(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsBox(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActBoxSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsBox");
		if (*erg != NULL)
		{
			grin_ActualBox(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsGate(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActGateSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsGate");
		if (*erg != NULL)
		{
			grin_ActualGate(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsDistribution(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	tDistribution dist;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = 1;
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsDistribution");
		if (*erg != NULL)
		{
			grin_ActualDist(wdw, &dist);
			(*erg)[0] = "KLMN"[dist];
			(*erg)[1] = '\0';
			(*erg)[2] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsMime(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActMimeSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsMime");
		if (*erg != NULL)
		{
			grin_ActualMime(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsStatus(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActStatusSize(wdw);
		*erg = SYSMALLOC(size + 1, MEM_NOERR | MEM_SHARABLE, "gsStatus");
		if (*erg != NULL)
		{
			grin_ActualStatus(wdw, *erg, size);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsFollowup(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActFollowupSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsFollowup");
		if (*erg != NULL)
		{
			grin_ActualFollowup(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsReplyto(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActReplyToSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsReplyTo");
		if (*erg != NULL)
		{
			grin_ActualReplyto(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsMsgText(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActTextSize(wdw,FALSE);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsMsgText");
		if (*erg != NULL)
		{
			grin_ActualText(wdw, *erg, size,FALSE);
			(*erg)[size] = '\0';
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID gsMsgFullText(_WORD argc, _UBYTE *argv[], _UBYTE **erg)			*/
/*	Wie die vorige Funktion, gibt aber den vollen Text zurck, also incl.		*/
/*	Header, ob dieser gerade angezeigt wird oder nicht.							*/
/*	Parameter:	argc:	Anzahl Argumente										*/
/*				argv[]:	Die Argumente											*/
/*				erg:	Dahin soll das Ergebnis									*/
/*	Ergebnis:	keines															*/		
/*	20.10.00 von Dimitri Junker	DYJ 4.18zi										*/
/*** ------------------------------------------------------------------------ ***/

LOCAL _VOID gsMsgFullText(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActTextSize(wdw,TRUE);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsMsgFullText");
		if (*erg != NULL)
		{
			grin_ActualText(wdw, *erg, size,TRUE);
			(*erg)[size] = '\0';
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}
/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsMsgDate(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	F_TIME time;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = 14;
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsMsgDate");
		if (*erg != NULL)
		{
			grin_ActualDate(wdw, &time);
			ConvertDate_DateToText(&time, "YYYYMMDD", *erg, 9);
			ConvertDate_TimeToText(&time, "HHMMSS", *erg + 8, 7);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsStatusDate(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	F_TIME time;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = 14;
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsStatusDate");
		if (*erg != NULL)
		{
			grin_ActualStatusDate(wdw, &time);
			if (T_IsZero(&time))
			{
				strCpy(*erg, "00000000000000");
			} else
			{
				ConvertDate_DateToText(&time, "YYYYMMDD", *erg, 9);
				ConvertDate_TimeToText(&time, "HHMMSS", *erg + 8, 7);
			}
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsGroup(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActGroupSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gs_erg");
		if (*erg != NULL)
		{
			grin_ActualGroup(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsOrgText(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActOrgTextSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsOrgText");
		if (*erg != NULL)
		{
			grin_ActualOrgText(wdw, *erg, size);
			(*erg)[size] = '\0';
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsIsOwn(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = 1;
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsIsOwn");
		if (*erg != NULL)
		{
			(*erg)[0] = grin_ActualIsOwn(wdw)==MSG_FREMD ? '0' : '1';
			(*erg)[size] = '\0';
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSender(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	HWND wdw;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		grin_GetGrinTopWdw(&wdw);
	if (grin_grinWindowTop(wdw))
	{
		size = grin_ActSenderSize(wdw);
		*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gsSender");
		if (*erg != NULL)
		{
			grin_ActualSender(wdw, *erg, size + 1);
			(*erg)[size + 1] = '\0';
			return;
		}
	}
	*erg = gs_erg("");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsGetResubList(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_ULONG size;
	
	UNUSED(argc);
	UNUSED(argv);
	size = WiederVorlage_ResubListSize();
	*erg = SYSMALLOC(size + 2, MEM_NOERR | MEM_SHARABLE, "gs_erg");
	if (*erg != NULL)
		WiederVorlage_GetResubList(*erg);
}

/*** ---------------------------------------------------------------------- ***/
LOCAL MSGNR getIdNumber(OneGroupHandle ptr, CONST _UBYTE *id)
{
	return data_NumberOfID(ptr, id);

}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsOpenMessage(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	CAT_GROUP_NUMBER gruppe;
	LOCAL HWND openWindow;
	
	if (argc > 2)
	{
		if (GroupSelect_GroupNumber(argv[1], &gruppe))
		{
			batchProcess = TRUE;
			if (openWindow > 0)
			{
				if (!grin_grinSwitchToMess(openWindow, gruppe, getIdNumber, argv[2]))
					openWindow = grin_grinOpenWithProc(gruppe, getIdNumber, argv[2], grin_grinNextMess, NULL, mOther);
			} else
			{
				openWindow = grin_grinOpenWithProc(gruppe, getIdNumber, argv[2], grin_grinNextMess, NULL, mOther);
			}
			batchProcess = FALSE;
			if (openWindow > 0)
			{
				*erg = gs_ok();
				return;
			}
		}
	}
	*erg = gs_error();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsCreateInfile(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	UNUSED(argc);
	UNUSED(argv);
	batchProcess = TRUE;
	*erg = gs_res(MsgInfile_MakeInFile());
	batchProcess = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsStartTerminal(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	UNUSED(argc);
	UNUSED(argv);
	batchProcess = TRUE;
	*erg = gs_res(!UserBLK.TerminalAsOverlay && StartTerminal(TRUE));
	batchProcess = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20ze */
LOCAL _VOID gsDecodeOutfile(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	UNUSED(argc);
	UNUSED(argv);
	batchProcess = TRUE;
	*erg = gs_res(DecodeOutfile(TRUE));
	batchProcess = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsGetGroupList(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	List list;
	group *gr;
	size_t len;
	_UBYTE *dst;
	
	UNUSED(argc);
	UNUSED(argv);
	*erg = NULL;
	if (!GroupSelect_GetReadList(&list, TRUE, FALSE, FALSE))
	{
		return;
	}
	len = 0;
	Lists_ResetList(&list);
	while ((gr = Lists_NextEntry(&list)) != NULL)
	{
		len += strLen(gr->name) + 1;
	}
	if (len != 0)
	{
		len++;
		*erg = SYSMALLOC(len, MEM_NOERR | MEM_SHARABLE, "gsGetGroupList");
		if (*erg != NULL)
		{
			dst = *erg;
			Lists_ResetList(&list);
			while ((gr = Lists_NextEntry(&list)) != NULL)
			{
				strCpy(dst, gr->name);
				dst += strLen(dst) + 1;
			}
			*dst = '\0';
		}
	}
	GroupSelect_DeleteList(&list);
}

/*** ---------------------------------------------------------------------- ***/

typedef struct _gs_command {
	CONST _UBYTE *name;
	_VOID (*func)(_WORD argc, _UBYTE *argv[], _UBYTE **erg);
} GS_COMMAND;

LOCAL _VOID gsGetAllCommands(_WORD argc, _UBYTE *argv[], _UBYTE **erg);
LOCAL _VOID gsCheckCommand(_WORD argc, _UBYTE *argv[], _UBYTE **erg);

LOCAL GS_COMMAND gs_command[] = {
	{ "CheckCommand", gsCheckCommand },
	{ "Close", gsClose },
	{ "Copy", gsCopy },
	{ "Cut", gsCut },
	{ "Delete", gsDelete },
	{ "DeselectAll", gsDeselectAll },
	{ "GetAllCommands", gsGetAllCommands },
	{ "GetFront", gsGetFront },
	{ "New", gsNewMessage },
	{ "Open", gsOpen },
	{ "Paste", gsPaste },
	{ "Print", gsPrint },
	{ "Save", gsSave },
	{ "SaveAs", gsSaveAs },
	{ "SelectAll", gsSelectAll },
	{ "ToFront", gsToFront },
	{ "Undo", gsUndo },
	{ "CheckApp", gsCheckApp },

	{ "ReadInf", gsReadInf },
	{ "SelectName", gsSelectName },
	{ "SelectGroup", gsSelectGroup },
	{ "SelectSubject", gsSelectSubject },
	{ "WhoAmI", gsWhoami },
	{ "GetResubList", gsGetResubList },
	{ "OpenMessage", gsOpenMessage },
	{ "CreateInfile", gsCreateInfile },
	{ "StartTerminal", gsStartTerminal },
	{ "DecodeOutfile", gsDecodeOutfile },
	{ "GetGroupList", gsGetGroupList },
	
	{ "From", gsFrom },
	{ "Realname", gsRealname },
	{ "To", gsTo },
	{ "MausID", gsMausID },
	{ "MsgID", gsMsgID },
	{ "MausRef", gsMausRef },
	{ "References", gsRef },
	{ "Subject", gsSubject },
	{ "Organization", gsBox },
	{ "Gate", gsGate },
	{ "Distribution", gsDistribution },
	{ "Mime", gsMime },
	{ "Status", gsStatus },
	{ "Followup", gsFollowup },
	{ "Replyto", gsReplyto },
	{ "MsgText", gsMsgText },
	{ "MsgDate", gsMsgDate },
	{ "StatusDate", gsStatusDate },
	{ "Group", gsGroup },
	{ "OrgText", gsOrgText },
	{ "IsOwn", gsIsOwn },
	{ "Sender", gsSender },
	{ "MsgFullText", gsMsgFullText },
	
	{ NULL, FUNK_NULL }
};

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsGetAllCommands(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_WORD i;
	size_t len;
	_UBYTE *p;
	
	UNUSED(argc);
	UNUSED(argv);
	len = 0;
	for (i = 0; gs_command[i].name != NULL; i++)
		len += strLen(gs_command[i].name) + 1;
	len += 14 + 1; /* AppGetLongName */
	len += 4 + 1;  /* Quit */
	
	*erg = SYSMALLOC(len + 1, MEM_NOERR | MEM_SHARABLE, "gsGetAllCommands");
	if (*erg != NULL)
	{
		p = *erg;
		for (i = 0; gs_command[i].name != NULL; i++)
		{
			strCpy(p, gs_command[i].name);
			p += strLen(gs_command[i].name) + 1;
		}
		strCpy(p, "AppGetLongName");
		p += 14 + 1;
		strCpy(p, "Quit");
		p += 4 + 1;
		*p = '\0';
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsCheckCommand(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_WORD i;
	
	if (argc > 1)
	{
		for (i = 0; gs_command[i].name != NULL; i++)
			if (strCaseCmp(argv[1], gs_command[i].name) == 0)
			{
				*erg = gs_ok();
				return;
			}
		if (strCaseCmp(argv[1], "Quit") == 0 ||
			strCaseCmp(argv[1], "AppGetLongName") == 0)
		{
			*erg = gs_ok();
			return;
		}
	}
	*erg = gs_error();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CATGScript_Init(_VOID)
{
	_WORD i;
	
	GScript_Init(GS_APPLONGNAME);
	for (i = 0; gs_command[i].name != NULL; i++)
		GEMScript_Command(gs_command[i].name, gs_command[i].func);
}
