/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 * 3.4.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.24f'
 *	Die Hilfetexte aktualisiert und entspr. Anpassungen in CAT
 *
 * 18.8.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.29f'
 *	Die Verwendung von cat.hlp berarbeitet, da wurden teils falsche
 *	Codewrter verwendet. Auerdem cat.hlp selber etwas erweitert.
 *
 * 20.8.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.29j'
 *	cat.hlp existiert jetzt auch in einer englischen Version.
 *	bersetzt wurde es hauptschlich von Ekkehard Flessa mit
 *	kleineren Teilen von mir (DYJ). Genau so wie Cat cat_de.rsc
 *	oder cat_en.rsc verwendet whlt es jetzt auch zwischen
 *	cat_de.hlp und cat_en.hlp aus. Fr die bersetzung des HYP
 *	werden noch Freiwillige gesucht.
 *
 */


#include <cathelp.h>
#include <listdl.h>
#include <cattypes.h>
#include <maustaus.h>
#include <mte.h>
#include <catgloba.h>
#include <datasys.h>
#include <catfiles.h>
#include <varname3.h>
#include <varnamel.h>
#include <confvars.h>
#include <protokol.h>

#include <mtappl.h>
#include <mtdials.h>
#include <mttextfi.h>
#include <mtutils.h>

#include <s_endian.h>
#include <file_io.h>
#include <ro_mem.h>
#include <routinec.h>
#include <routines.h>
#include <mtrsc.h>


#define cHelpCode "#(&)"

#define HELP_TABLE_SIZE (CatHelp_anzahl - CatHelp_general)
LOCAL CONST _UBYTE *CONST cHelpTable[HELP_TABLE_SIZE] = {
	"#(&)CAT",
	"#(&)CATINFO",
	"#(&)EDITOR",
	"#(&)ANZEIGE",
	"#(&)MESSAGES",
	"#(&)STICHWORT",
	"#(&)B\216UME",
	"#(&)FORMATE",
	"#(&)KONFIG",
	"#(&)OPTIONS",
	"#(&)TAUSCH",
	"#(&)LISTCONF",
	"#(&)SEARCHDIAL",
	"#(&)SMALLSEARCH",
	"#(&)FILTER",
	"#(&)GROUPLIST",
	"#(&)GROUPREAD",
	"#(&)GROUPWRITE",
	"#(&)GROUPUSER",
	"#(&)INFOLOAD",
	"#(&)INFOCHANGE",
	"#(&)MAXIMALSIZE",
	"#(&)SELECTGROUP",
	"#(&)REGISTER",
	"#(&)WIEDERVORLAGE",
	"#(&)KOMMENTARE",
	"#(&)TEXTSEARCH",
	"#(&)TEXTREPLACE",
	"#(&)CHANGEMSG",
	"#(&)ITKHELP",
	"#(&)MACROHELP",
	"#(&)CONFIGINFILE",
	"#(&)CONFIGEDITOR",
	"#(&)IOFILTER",
	"#(&)DIALOGSIZE",
	"#(&)HOTKEYS",
	"#(&)SOUNDS",
	"#(&)HILFSPRG",
	"#(&)ALIAS",
	"#(&)FOLLOWUP"
};
/* :language: */
LOCAL CONST _UBYTE *CONST cGuideHelpTable[HELP_TABLE_SIZE] = {
	"CAT - E-Mails auf Ataris",
	"Freeware",
	"Der Editor",
	"Die Nachrichtenanzeige",
	"Kommentar \224ffnen...",
	"Die Stichwortliste",
	"Die Baumanzeige",
	"Formate...",
	"Konfiguration...",
	"Allgemein...",
	"Tausch...",
	"Stichwortliste...",
	"Der gro\236e (kompliziertere) Suchdialog",
	"Der kleine (einfachere) Suchdialog",
	"Block filtern...",
	"Gruppen...",
	"Gruppen lesen",
	"Neue \224ffentliche...",
	"User an-/abstellen",
	"Mausinfo laden...",
	"Infodateien...",
	"Maximalgr\224\236e \201berschritten!",
	"Gruppen ausw\204hlen...",
	"Freeware",
	"Wiedervorlage...",
	"Kommentare auf eigene Msg...",
	"Textsuche...",
	"Text ersetzen",
	"Nachricht ndern...",
	"(Maus)Tausch-Kommandos...",
	"ITK-Makros",
	"Infile konfigurieren...",
	"Editor...",
	"I/O-Filter...",
	"Dialogboxen...",
	"Tastenkrzel...",
	"Kl\204nge...",
	"Hilfsprogramme...",
	"Gruppen-Alias-Namen...",
	"FollowUp setzen",
	"CbC Vor-Vorwahl...",
	"PM-Filter...",
	"Die Exportfunktion"
};

#define cHelpMagic          0x43415448L /* 'CATH' */
#define cHelpVersion        0x0100
#define cHelpVersionMagic   0x020A

#define NO_IDX ((_ULONG)-1)

typedef struct {
	_ULONG helpSize;
	_ULONG helpOffset[HELP_TABLE_SIZE];
} tHelpIdx;

typedef struct {
	List l;
	OBJECT *tree;
	_WORD maxWidth;
	_VOID *lastElem;
} tHelpInfo;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL buildIdx(CONST _UBYTE *fname, _ULONG size, TEXTFILENO f, tHelpIdx *helpIdx)
{
	_UBYTE skipString[128];
	tHelp i;
	_BOOL found;
	FileHeaderType header;
	FILENO fi;
	_UBYTE path[PATH_MAX];
	
	helpIdx->helpSize = size;
	for (i = 0; i < HELP_TABLE_SIZE; i++)
		helpIdx->helpOffset[i] = NO_IDX;
	/* Jetzt alle Zeilen nach dem Key durchsuchen */
	for (;;)
	{
		/* Zeilen bis zum Anfang des Helptextes ueberspringen */
		found = FALSE;
		mtTextfiles_ReadLine(f, skipString, sizeof(skipString));
		if (mtTextfiles_EndofText(f))
			break;
		if (strnCmp(skipString, cHelpCode, sizeof(cHelpCode)-1) == 0)
		{
			i = 0;
			while (!found)
			{
				if (strStr(skipString, cHelpTable[i]) != NULL)
				{
					/* Gefunden. In der naechsten Zeile geht es los */
					mtTextfiles_ReadLn(f);
					/* Offset merken */
					helpIdx->helpOffset[i] = mtTextfiles_Textpos(f);
					found = TRUE;
				} else
				{
					++i;
					if (i >= HELP_TABLE_SIZE)
						break;
				}
			}
		}
		if (!found)
			mtTextfiles_ReadLn(f);
		if (mtTextfiles_EndofText(f))
			break;
	}
	mtTextfiles_Reset(f);
	/* Jetzt den Hilfeindex schreiben */
	fi = CatFiles_CreateFile(fname);
	if (!FILENO_OK(fi))
		return FALSE;
	/* Fileheader aufbauen */
	header.CatMagic = h2nl(cHelpMagic);
	header.Version = h2ns(cHelpVersion);
	header.VersionMagic = h2ns(cHelpVersionMagic);
	CatFiles_WriteMuch(dbHeaderLength, fi, &header);
	IfNotBigEndian
	{
		helpIdx->helpSize = h2nl(helpIdx->helpSize);
		for (i = 0; i < HELP_TABLE_SIZE; i++)
			helpIdx->helpOffset[i] = h2nl(helpIdx->helpOffset[i]);
	}
	CatFiles_WriteMuch(sizeof(tHelpIdx), fi, helpIdx);
	IfNotBigEndian
	{
		helpIdx->helpSize = n2hl(helpIdx->helpSize);
		for (i = 0; i < HELP_TABLE_SIZE; i++)
			helpIdx->helpOffset[i] = n2hl(helpIdx->helpOffset[i]);
	}
	CatFiles_CloseFile(fi);
	strBcpy(path, fname);
	F_Path_Dirname(path);
	Protokoll_SendPathUpdate(path);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/*  Lies Hilfe-Datei ein.
 */
/*DYJ 5.21i */	/*DYJ 5.29j */
LOCAL _BOOL readHelp(List *text, tHelp which)
{
	_UBYTE helpName[PATH_MAX];
	_UBYTE skipString[128];
	_UBYTE helpKey[128];
	_UBYTE *str;
	TEXTFILENO f;
	FileHeaderType header;
	FILENO fi;
	_ULONG helpSize;
	tHelpIdx helpIdx;
	_UBYTE path[PATH_MAX];
	_BOOL succ;
	_BOOL exists;
	F_ERROR error;
	
	_UBYTE hlpFname[PATH_MAX];
	CatGlobal_ApplPath(helpName);

	strCpy(hlpFname, CAT_BASE);

	Rsc_EnvLang(helpName,strChr(hlpFname,'\0'));
	strCat(hlpFname,HLP_EXT);
	F_Path_Append(helpName, hlpFname);


	MouseBusy();
	if (mtTextfiles_OpenTextfile(helpName, READ, 0, &f) != GERR_OK)
	{
		F_Path_Get_Program(helpName, hlpFname);
		if (mtTextfiles_OpenTextfile(helpName, READ, 0, &f) != GERR_OK)
		{
			MouseArrow();
			sprintf(skipString, Rsc_WrkString(MTE_noFileOpen), hlpFname);
			MTE_info(skipString);
			return FALSE;
		}
	}

	/* Groesse der Hilfedatei feststellen */
	helpSize = CatFiles_FileSize(helpName, &exists);
	/* Jetzt nachsehen, ob wir auch eine Indexdatei haben. Wenn nein,
	 * dann wird die jetzt aufgebaut
	 */
	strCpy(path, helpName);
	F_Path_Dirname(path);
	F_Path_Append(path, CAT_HELPIDX);
	fi = CatFiles_OpenFile(path, CatFiles_readFile);
	if (!FILENO_OK(fi))
	{
		/* Kein Index vorhanden, also aufbauen */
		succ = buildIdx(path, helpSize, f, &helpIdx);
	} else
	{
		/* Erstmal Header lesen und pruefen */
		if (CatFiles_ReadDbHeader(fi, &header) != NoError ||
			header.CatMagic != cHelpMagic ||
			header.Version  != cHelpVersion ||
			header.VersionMagic != cHelpVersionMagic)
		{
			/* Index ist ungueltig, neu aufbauen */
			CatFiles_CloseFile(fi);
			succ = buildIdx(path, helpSize, f, &helpIdx);
		} else
		{
			error = CatFiles_ReadMuch(sizeof(helpIdx), fi, &helpIdx);
			if (error != NoError)
			{
				F_ErrorAlert(error);
				CatFiles_CloseFile(fi);
				succ = buildIdx(path, helpSize, f, &helpIdx);
			} else
			{
				if (helpIdx.helpSize != helpSize)
				{
					/* Index veraltet, neu aufbauen */
					CatFiles_CloseFile(fi);
					succ = buildIdx(path, helpSize, f, &helpIdx);
				} else
				{
					CatFiles_CloseFile(fi);
					succ = TRUE;
				}
			}
		}
	}
	/* Jetzt haben wir einen gueltigen Index, nun muessen wir nur noch den
	 * Schluessel suchen
	 */
	if (succ && helpIdx.helpOffset[which] < helpIdx.helpSize)
	{
		/* Auf richtige Position setzen */
		mtTextfiles_SetTextpos(f, helpIdx.helpOffset[which]);
	} else
	{
		/* Schade, Schluessel kann nicht stimmen, also doch einfach suchen */

		/* Schluesselwort fuer Suche bestimmen */
		strBcpy(helpKey, cHelpTable[which]);
		/* Jetzt alle Zeilen nach dem Key durchsuchen */
		for (;;)
		{
			/* Zeilen bis zum Anfang des Helptextes ueberspringen */
			mtTextfiles_ReadLine(f, skipString, sizeof(skipString));
			if (mtTextfiles_EndofText(f))
				break;
			if (strStr(skipString, helpKey) != NULL)
			{
				/* Gefunden. In der naechsten Zeile geht es los */
				mtTextfiles_ReadLn(f);
				break;
			}
			mtTextfiles_ReadLn(f);
			if (mtTextfiles_EndofText(f))
				break;
		}
	}

	if (!Lists_CreateList(text, "readHelp: list"))
	{
		MouseArrow();
		return FALSE;
	}
	for (;;)
	{
		mtTextfiles_ReadLine(f, skipString, sizeof(skipString));
		if (strStr(skipString, cHelpCode) != NULL)
		{
			/* Gefunden. Hilfetext ist nun zu Ende */
			break;
		}
		str = STRDUP(skipString, "readHelp: str");
		if (str == NULL)
			break;
		if (!Lists_AppendEntry(text, str))
		{
			SFREE(str);
			break;
		}
		if (mtTextfiles_EndofText(f))
			break;
		mtTextfiles_ReadLn(f);
		if (mtTextfiles_EndofText(f))
			break;
	}

	mtTextfiles_CloseTextfile(&f);
	MouseArrow();
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID lineToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)
{
	_UBYTE *inf;

	UNUSED(env);
	inf = entry;
	if (inf == NULL)
		*str = '\0';
	else
		strMcpy(str, strsize, inf);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL selEntry(_VOID *entry, _VOID *env, _WORD line)
{
	UNUSED(entry);
	UNUSED(env);
	UNUSED(line);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isSelected(_VOID *entry, _VOID *env)
{
	UNUSED(entry);
	UNUSED(env);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isEnabled(_VOID *entry, _VOID *env)
{
	UNUSED(entry);
	UNUSED(env);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *nextEntry(_VOID *list)
{
	return Lists_NextEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *prevEntry(_VOID *list)
{
	return Lists_PrevEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID resetList(_VOID *list)
{
	Lists_ResetList(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID countEntries(_VOID *l, _LONG *ll, _WORD *ww)
{
	*ww = 0;
	*ll = Lists_NoOfEntries(l);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID drawHelpEntry(_VOID *entry, _VOID *env, _WORD x, _WORD y, _WORD offset, CONST GRECT *clip)
{
	OBJECT *tree;
	tHelpInfo *l;

	UNUSED(offset);
	l = env;
	tree = l->tree;
	tree[ROOT].ob_x = x;
	tree[ROOT].ob_y = y;
	if (entry != NULL)
	{
		/* Jetzt hier den String fuer den Gruppennamen zusammenbauen */
		SetObjcStringAdr(tree, LISTENTRY, entry, l->maxWidth + 1);
	} else
	{
		SetObjcStringAdr(tree, LISTENTRY, "", l->maxWidth + 1);
	}
	Objc_draw(tree, ROOT, MAX_DEPTH, clip);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID setAndGetHelpValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exitBut)
{
	tHelpInfo *l;

	UNUSED(tree);
	UNUSED(exitBut);
	if (!set)
	{
		l = env;
		Lists_DeleteSimpleList(&l->l, mem_free, TRUE);
		OFREE(l);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID internalDoHelp(tHelp topic, _BOOL doReadHelp, List *externalHelp)
{
	tHelpInfo *l;
	OBJECT *helpBox;
	_WORD maxLength;
	GRECT r;
	ldHandler dialHandler;
	OBJECT *tree;

	l = NEW(tHelpInfo, "internalDoHelp");
	if (l == NULL)
		return;

	helpBox = TreeAddr[HELPBOX];

	if (doReadHelp && !readHelp(&l->l, topic))
	{
		OFREE(l);
		return;
	}
	MausTauschrsc_AdjustDialogSize(helpBox, HELPBOX);
	
	if (!doReadHelp)
		l->l = *externalHelp;

	/* Jetzt das Object zusammenbauen */
	CalcArea(helpBox, HHELPBOX, &r);
	maxLength = r.g_w / CharWidth;

	/* Objectadresse holen */
	tree = TreeAddr[LISTTXT];
	l->tree = tree;
	l->maxWidth = maxLength;
	tree[LISTENTRY].ob_width = maxLength * CharWidth;
	DelObjState(tree, ROOT, SELECTED | DISABLED);

	/* WindowDialog vorbereiten */
	ListDl_BuildLdHandler(&l->l,
		resetList,
		nextEntry,
		prevEntry,
		countEntries,
		isEnabled,
		selEntry,
		lineToStr,
		isSelected,
		drawHelpEntry,
		FUNK_NULL,
		0, CharHeight,
		0, 0,
		&dialHandler);

	/* ListWinDialog oeffnen */
	if (ListDl_WinListDial(helpBox,
		ldArrows | ldModal,
		&dialHandler,
		HHELPBOX,
		HHELPBACK,
		HOK,
		-1,
		l,
		-1,
		-1,
		ListDl_dummyInLoop,
		NULL,
		0,
		ListDl_dummyCheckExit,
		setAndGetHelpValues,
		setAndGetHelpValues,
		&l->lastElem))
	{
	} else
	{
		setAndGetHelpValues(helpBox, l, FALSE, -1);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID doHelp(tHelp topic, _BOOL externalHelp)
{
	_UBYTE helpText[PATH_MAX];
	_WORD avModus;

	if (externalHelp ||
		(ConfVars_GetConfigBool(cExternHelp, &externalHelp) && externalHelp))
	{
		ConfVars_GetConfDefStr(cCatHelpText, helpText, sizeof(helpText), CAT_HYPNAME);
		/* SendExtendedHelp liefert FALSE zurueck, falls das Hilfesystem
		 * ST-Guide nicht gefunden wurde
		 */
		ConfVars_GetConfDefInt(cAvModus, &avModus, 2);
		externalHelp = Protokoll_SendExtendedHelp(cGuideHelpTable[topic], helpText,avModus);
	}
	/* Wenn der externe Aufruf nicht geklappt hat, dann wird hier
	 * noch die interne Hilfe aufgerufen
	 */
	if (!externalHelp)
		internalDoHelp(topic, TRUE, NULL);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatHelp_DoHelp(tHelp topic)
{
	doHelp(topic, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatHelp_DoHelpExtern(tHelp topic)
{
	doHelp(topic, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatHelp_DoInternalHelp(List *helpList)
{
	internalDoHelp(CatHelp_general, FALSE, helpList);
}
