/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <catlog.h>
#include <catvers.h>
#include <mttextfi.h>
#include <cattypes.h>
#include <convertd.h>
#include <mtpaths.h>

#include <ro_mem.h>
#include <routinec.h>
#include <ro_time.h>
#include <file_io.h>


LOCAL TEXTFILENO log;
LOCAL _BOOL is_open;

GLOBAL extern _UBYTE LOG_TITLE[];
GLOBAL extern _UBYTE LOG_END[];
GLOBAL extern _UBYTE LOG_FNAME[];
GLOBAL extern _UBYTE LOG_FHDRNAME[];

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_WriteCard(_ULONG c)
{
	if (is_open)
		mtTextfiles_WriteLongCard(log, c, 0);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_WriteInt(_LONG i)
{
	if (is_open)
		mtTextfiles_WriteLongInt(log, i, 0);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_Write(_UBYTE c)
{
	if (is_open)
		mtTextfiles_WriteChar(log, c);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_WriteString(CONST _UBYTE *str)
{
	if (is_open)
		mtTextfiles_WriteLine(log, str);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_WriteKonjString(CONST _UBYTE *str, CONST _UBYTE *single, CONST _UBYTE *plural, _ULONG count)
{
	CatLog_WriteString(str);
	CatLog_WriteString(count == 1 ? single : plural);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_WriteLn(_VOID)
{
	if (is_open)
		mtTextfiles_WriteLn(log);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_WriteLine(CONST _UBYTE *str)
{
	CatLog_WriteString(str);
	CatLog_WriteLn();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_WriteTearLine(_VOID)
{
	CatLog_WriteLine("--------------------------------------------------------------------------------");
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_WriteStringNTime(CONST _UBYTE *str)
{
	CatLog_putTime();
	CatLog_WriteString(str);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_WriteLineNTime(CONST _UBYTE *str)
{
	CatLog_WriteStringNTime(str);
	CatLog_WriteLn();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_putDate(_VOID)
{
	_UBYTE str[20];

	T_Strlftime(str, sizeof(str), "%d.%m.%Y", T_Systime());
	CatLog_WriteString(str);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_putTime(_VOID)
{
	_UBYTE str[20];

	T_Strlftime(str, sizeof(str), "%H:%M:%S ", T_Systime());
	CatLog_WriteString(str);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL CatLog_OpenLog(_BOOL isHdr)
{
	_UBYTE scrap[PATH_MAX];
	
	strBcpy(scrap, DataPath);
	if(isHdr)
		F_Path_Append(scrap, LOG_FHDRNAME);
	else
		F_Path_Append(scrap, LOG_FNAME);
	/* Buffergroessen veraendert */
	is_open = mtTextfiles_OpenTextfile(scrap, WRITE, 0, &log) == GERR_OK;
	CatLog_putDate();
	CatLog_WriteString(", ");
	CatLog_putTime();
	CatLog_WriteString(LOG_TITLE);
	CatLog_WriteString(CatVersion);
	CatLog_WriteLine(" ge\224ffnet");
	CatLog_WriteTearLine();
	return is_open;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatLog_CloseLog(_VOID)
{
	if (is_open)
	{
		CatLog_WriteTearLine();
		CatLog_WriteLineNTime(LOG_END);
		mtTextfiles_CloseTextfile(&log);
		is_open = FALSE;
	}
}
