/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <catmenu.h>
#include <confvars.h>
#include <varname2.h>
#include <maustaus.h>

#include <quicksor.h>
#include <keycode.h>
#include <menu_key.h>
#include <windials.h>

#include <mtappl.h>
#include <mtdials.h>
#include <mtutils.h>
#include <mtpopups.h>

#include <portvdi.h>
#include <ro_mem.h>
#include <debug.h>
#include <routines.h>


typedef struct {
	_WORD numTitles;
	_WORD numEntries;
	_UWORD allowedMask;
	_BOOL upper;
	OBJECT *menu;
	_WORD currentEntry;
	GRECT dummyGr;
	_WORD dummyMessage[8];
	_VOID (*doHelp)(_VOID);
	MENU_KEY_ENTRY *defaultMenuTable;
	MENU_KEY_ENTRY menuTable[1];
} MENU_EDIT;

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Menu_Savekeys(OBJECT *tree, MENU_KEY_ENTRY *menuTable, _WORD numEntries)
{
	_WORD i;
	_UBYTE varName[256];
	
	UNUSED(tree);
	for (i = 0; i < numEntries; i++)
	{
		make_varname(varName, sizeof(varName), cMenuEntry, menuTable[i].configNum);
		if (menuTable[i].key == menuTable[i].defaultKey)
			ConfVars_DeleteConfigVar(varName);
		else
			ConfVars_SetConfigInt(varName, menuTable[i].key);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Menu_Loadkeys(OBJECT *menu, MENU_KEY_ENTRY *menuTable, _WORD numEntries)
{
	_WORD i;
	_UBYTE varName[256];
	
	for (i = 0; i < numEntries; i++)
	{
		make_varname(varName, sizeof(varName), cMenuEntry, menuTable[i].configNum);
		ConfVars_GetConfDefInt(varName, &menuTable[i].key, menuTable[i].defaultKey);
		menuTable[i].title = MenuKey_FindTitle(menu, menuTable[i].entry);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Menu_Setkeys(OBJECT *menu, MENU_KEY_ENTRY *menuTable, _WORD numEntries, _BOOL settext)
{
	_WORD i;
	_UBYTE str[80];
	_UBYTE *entry;
	_WORD key;
	_WORD offset;
	
	for (i = 0; i < numEntries; i++)
	{
		entry = menu[menuTable[i].entry].ob_spec.free_string;
		MenuKey_KeyToStr(menuTable[i].key, str, sizeof(str));
		if (MenuKey_FindKeycode(entry, &key, &offset))
		{
			MemSet(entry + offset, ' ', strLen(entry) - offset);
		}
		strCpy(entry + strLen(entry) - strLen(str), str);
		if (settext)
			menu_text(menu, menuTable[i].entry, entry);
	}
	MenuKey_Init(menu);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL cmpEntry(CONST _VOID *a1, CONST _VOID *a2, _VOID *env)
{
	CONST MENU_KEY_ENTRY *e1;
	CONST MENU_KEY_ENTRY *e2;
	
	UNUSED(env);
	e1 = a1;
	e2 = a2;
	return e1->entry < e2->entry;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID menuSetValues(OBJECT *tree, MENU_EDIT *menu, _BOOL redraw)
{
	GRECT r;
	_UBYTE shortcut[10];
	MENU_KEY_ENTRY *entry;
	
	entry = &menu->menuTable[menu->currentEntry];
	SetObjcString(tree, MENU_TITLE1, menu->menu[entry->title].ob_spec.free_string);
	SetObjcString(tree, MENU_ENTRY1, entry->text);
	MenuKey_KeyToStr(entry->key, shortcut, sizeof(shortcut));
	SetObjcString(tree, MENU_KEY, shortcut);
	if (redraw)
	{
		WinDials_WinDialDraw(tree, MENU_TITLE1, MAX_DEPTH, &r, FALSE);
		WinDials_WinDialDraw(tree, MENU_ENTRY1, MAX_DEPTH, &r, FALSE);
		WinDials_WinDialDraw(tree, MENU_KEY, MAX_DEPTH, &r, FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID menuSelectTitle(MENU_EDIT *menu, OBJECT *tree)
{
	OBJECT *popup;
	_WORD i, idx, lasttitle;
	_WORD x, y;
	_WORD select;
	_WORD titleNum;
	
	popup = MALLOC((menu->numTitles + 1) * sizeof(OBJECT), "menuSelectTitle: popup");
	if (popup == NULL)
		return;
	popup[ROOT].ob_next = NIL;
	popup[ROOT].ob_head = 1;
	popup[ROOT].ob_tail = menu->numTitles;
	popup[ROOT].ob_flags = NONE;
	popup[ROOT].ob_state = SHADOWED;
	popup[ROOT].ob_type = G_BOX;
	popup[ROOT].ob_spec.index = OBSPEC_MAKE('\0', -1, BLACK, BLACK, FALSE, IP_HOLLOW, WHITE);
	popup[ROOT].ob_x = 0;
	popup[ROOT].ob_y = 0;
	popup[ROOT].ob_width = tree[MENU_TITLE1].ob_width;
	popup[ROOT].ob_height = menu->numTitles * CharHeight;
	idx = 1;
	lasttitle = -1;
	titleNum = 0;
	for (i = 0; i < menu->numEntries; i++)
	{
		if (menu->menuTable[i].title != lasttitle)
		{
			lasttitle = menu->menuTable[i].title;
			popup[idx].ob_next = idx + 1;
			popup[idx].ob_head = NIL;
			popup[idx].ob_tail = NIL;
			popup[idx].ob_flags = SELECTABLE;
			popup[idx].ob_state = NONE;
			popup[idx].ob_type = G_STRING;
			popup[idx].ob_spec.free_string = menu->menu[lasttitle].ob_spec.free_string;
			popup[idx].ob_x = 0;
			popup[idx].ob_y = (idx - 1) * CharHeight;
			popup[idx].ob_width = popup[ROOT].ob_width;
			popup[idx].ob_height = CharHeight;
			idx++;
			if (i < menu->currentEntry)
				++titleNum;
		}
	}
	ASSERT(idx == (menu->numTitles + 1));
	popup[idx - 1].ob_next = ROOT;
	popup[idx - 1].ob_flags |= LASTOB;
	objc_offset(tree, MENU_TITLE1, &x, &y);
	select = mtPopups_TreePopup(popup, x, y, titleNum) - 1;
	if (select >= 0)
	{
		lasttitle = -1;
		for (i = 0; i < menu->numEntries; i++)
		{
			if (menu->menuTable[i].title != lasttitle)
			{
				if (select == 0)
					break;
				lasttitle = menu->menuTable[i].title;
				--select;
			}
		}
		if (i < menu->numEntries)
		{
			menu->currentEntry = i;
			menuSetValues(tree, menu, TRUE);
		}
	}
	FREE(popup, (menu->numTitles + 1) * sizeof(OBJECT));
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID menuSelectEntry(MENU_EDIT *menu, OBJECT *tree)
{
	OBJECT *popup;
	_WORD i, idx, title;
	_WORD x, y;
	_WORD select;
	_WORD numEntries;
	_WORD firstIndex;
	
	i = menu->currentEntry;
	title = menu->menuTable[i].title;
	while (i > 0 && menu->menuTable[i - 1].title == title)
		--i;
	firstIndex = i;
	numEntries = 0;
	for (i = firstIndex; i < menu->numEntries && menu->menuTable[i].title == title; i++)
		++numEntries;
	popup = MALLOC((numEntries + 1) * sizeof(OBJECT), "menuSelectEntry: popup");
	if (popup == NULL)
		return;
	popup[ROOT].ob_next = NIL;
	popup[ROOT].ob_head = 1;
	popup[ROOT].ob_tail = numEntries;
	popup[ROOT].ob_flags = NONE;
	popup[ROOT].ob_state = SHADOWED;
	popup[ROOT].ob_type = G_BOX;
	popup[ROOT].ob_spec.index = OBSPEC_MAKE('\0', -1, BLACK, BLACK, FALSE, IP_HOLLOW, WHITE);
	popup[ROOT].ob_x = 0;
	popup[ROOT].ob_y = 0;
	popup[ROOT].ob_width = tree[MENU_ENTRY1].ob_width;
	popup[ROOT].ob_height = numEntries * CharHeight;
	idx = 1;
	for (i = firstIndex; i < menu->numEntries && menu->menuTable[i].title == title; i++)
	{
		popup[idx].ob_next = idx + 1;
		popup[idx].ob_head = NIL;
		popup[idx].ob_tail = NIL;
		popup[idx].ob_flags = SELECTABLE;
		popup[idx].ob_state = NONE;
		popup[idx].ob_type = G_STRING;
		popup[idx].ob_spec.free_string = menu->menuTable[i].text;
		popup[idx].ob_x = 0;
		popup[idx].ob_y = (idx - 1) * CharHeight;
		popup[idx].ob_width = popup[ROOT].ob_width;
		popup[idx].ob_height = CharHeight;
		idx++;
	}
	ASSERT(idx == (numEntries + 1));
	popup[idx - 1].ob_next = ROOT;
	popup[idx - 1].ob_flags |= LASTOB;
	objc_offset(tree, MENU_ENTRY1, &x, &y);
	select = mtPopups_TreePopup(popup, x, y, menu->currentEntry - firstIndex) - 1;
	if (select >= 0)
	{
		menu->currentEntry = firstIndex + select;
		menuSetValues(tree, menu, TRUE);
	}
	FREE(popup, (numEntries + 1) * sizeof(OBJECT));
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL menuButton(OBJECT *tree, _VOID *env, _WORD button, _WORD mx, _WORD my, _UWORD kstate, _WORD clicks)
{
	GRECT r;
	MENU_EDIT *menu;
	_WORD i;
	_WORD title;
	
	UNUSED(mx);
	UNUSED(my);
	UNUSED(kstate);
	UNUSED(clicks);
	
	if(button==-1)
		return FALSE;
	button &= 0x7fff;
	DelObjState(tree, button, SELECTED);
	menu = env;
	switch (button)
	{
	case MENU_OK:
	case MENU_CANCEL:
		return TRUE;
	case MENU_HELP:
		if (menu->doHelp != FUNK_NULL)
			menu->doHelp();
		break;
	case MENU_DEFAULT:
		for (i = 0; i < menu->numEntries; i++)
			menu->menuTable[i].key = menu->menuTable[i].defaultKey;
		menuSetValues(tree, menu, TRUE);
		break;
	case MENU_LEFT:
		if (menu->currentEntry == 0)
			menu->currentEntry = menu->numEntries - 1;
		else
			--menu->currentEntry;
		menuSetValues(tree, menu, TRUE);
		break;
	case MENU_RIGHT:
		++menu->currentEntry;
		if (menu->currentEntry >= menu->numEntries)
			menu->currentEntry = 0;
		menuSetValues(tree, menu, TRUE);
		break;
	case MENU_ENTRY1:
		menuSelectEntry(menu, tree);
		break;
	case MENU_ENTRY2:
		i = menu->currentEntry;
		title = menu->menuTable[i].title;
		++i;
		if (i >= menu->numEntries || menu->menuTable[i].title != title)
		{
			i = 0;
			while (i < menu->numEntries && menu->menuTable[i].title != title)
				++i;
		}
		menu->currentEntry = i;
		menuSetValues(tree, menu, TRUE);
		break;
	case MENU_TITLE1:
		menuSelectTitle(menu, tree);
		break;
	case MENU_TITLE2:
		i = menu->currentEntry;
		title = menu->menuTable[i].title;
		while (i < menu->numEntries && menu->menuTable[i].title == title)
			++i;
		if (i >= menu->numEntries)
			i = 0;
		menu->currentEntry = i;
		menuSetValues(tree, menu, TRUE);
		break;
	case MENU_KEY:
		break;
	}
	WinDials_WinDialDraw(tree, button, MAX_DEPTH, &r, FALSE);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
LOCAL _UWORD menuKey(OBJECT *tree, _WORD mx, _WORD my, _UBYTE scan, _UWORD kbshift, _UWORD event, _WORD edObj, _VOID *special)
{
	_UWORD keycode;
	_UWORD mask;
	MENU_EDIT *menu;
	_UBYTE shortcut[20];
	_WORD i;
	
	UNUSED(mx);
	UNUSED(my);
	UNUSED(edObj);
	if (event & MU_KEYBD)
	{
		mask = 0;
		if (kbshift & (K_LSHIFT | K_RSHIFT))
			mask |= KEY_SHIFT;
		if (kbshift & K_CTRL)
			mask |= KEY_CTRL;
		if (kbshift & K_ALT)
			mask |= KEY_ALT;
		menu = special;
		if ((mask & ~menu->allowedMask) == 0)
		{
			if (scan == 0x39)
			{
				keycode = 0;
				menu->menuTable[menu->currentEntry].key = keycode;
				menuSetValues(tree, menu, TRUE);
			} else
			{
				keycode = MenuKey_TranslateKey(scan << 8, kbshift);
				if (keycode != 0)
				{
					MenuKey_KeyToStr(keycode, shortcut, sizeof(shortcut));
					if (shortcut[0] != '\0')
					{
						for (i = 0; i < menu->numEntries; i++)
							if (menu->menuTable[i].key == (_WORD)keycode && i != menu->currentEntry)
							{
								_UBYTE alt[512];
								
								sprintf(alt, MausTauschrsc_WrkString(AL_DUPLICATE_KEY),
									menu->menu[menu->menuTable[i].title].ob_spec.free_string,
									menu->menuTable[i].text);
								mtAlerts_Alert(1, alt);
								return 1;
							}
						menu->menuTable[menu->currentEntry].key = keycode;
						menuSetValues(tree, menu, TRUE);
					} else
					{
						return 0;
					}
				}
			}
			return 1;
		}
	}
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID menuGetSetValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exit)
{
	MENU_EDIT *menu;
	_WORD i;
	
	menu = env;
	if (set)
	{
		menuSetValues(tree, menu, FALSE);
		mtDials_InstallHandler(tree, menuKey, CallByEvent, 0, 0, &menu->dummyGr, menu->dummyMessage, menu);
		mtDials_SetUserkey(tree, MENU_KEY, 0x39, 0, FALSE, TRUE);		/* Space */
		mtDials_SetUserkey(tree, MENU_LEFT, 0x4b, 0, TRUE, TRUE);		/* Cursor Left */
		mtDials_SetUserkey(tree, MENU_RIGHT, 0x4d, 0, TRUE, TRUE);		/* Cursor Right */
	} else
	{
		if (exit == MENU_OK)
		{
			MemCpy(menu->defaultMenuTable, menu->menuTable, menu->numEntries * sizeof(MENU_KEY_ENTRY));
			Menu_Savekeys(menu->menu, menu->menuTable, menu->numEntries);
			Menu_Setkeys(menu->menu, menu->menuTable, menu->numEntries, TRUE);
		}
		for (i = 0; i < menu->numEntries; i++)
			SFREE(menu->menuTable[i].text);
		OFREE(menu);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Menu_Editkeys(OBJECT *menuTree, MENU_KEY_ENTRY *defaultMenuTable, _WORD numEntries, _VOID (*doHelp)(_VOID))
{
	MENU_EDIT *menu;
	_WORD i;
	_WORD lastTitle;
	_UBYTE str[80];
	_WORD key, offset;
	
	menu = MALLOC(sizeof(MENU_EDIT) + numEntries * sizeof(MENU_KEY_ENTRY), "MausTauschrsc_ShortCuts");
	if (menu == NULL)
		return;
	menu->defaultMenuTable = defaultMenuTable;
	MemCpy(menu->menuTable, defaultMenuTable, numEntries * sizeof(MENU_KEY_ENTRY));
	QuickSort_sortIt(0, numEntries - 1, menu->menuTable, cmpEntry, sizeof(MENU_KEY_ENTRY), QuickSort_noBreak, NULL);
	menu->numTitles = 0;
	lastTitle = -1;
	for (i = 0; i < numEntries; i++)
	{
		if (menu->menuTable[i].title != lastTitle)
		{
			menu->numTitles++;
			lastTitle = menu->menuTable[i].title;
		}
		ObjcString(menuTree, menu->menuTable[i].entry, str, sizeof(str));
		if (MenuKey_FindKeycode(str, &key, &offset))
			str[offset] = '\0';
		menu->menuTable[i].text = STRDUP(str, "MausTauschrsc_Shortcuts: text");
		if (menu->menuTable[i].text == NULL)
		{
			while (--i >= 0)
				SFREE(menu->menuTable[i].text);
			OFREE(menu);
			return;
		}
	}
	menu->numEntries = numEntries;
	menu->currentEntry = 0;
	menu->allowedMask = KEY_CTRL | KEY_ALT;
	menu->upper = TRUE;
	menu->menu = menuTree;
	menu->doHelp = doHelp;
	if (!WinDials_OpenWinDial(TreeAddr[SHORTCUTS], FALSE,
		menuGetSetValues,
		menuGetSetValues,
		menuButton,
		WinDials_defDraw,
		NULL,
		menu))
	{
		for (i = 0; i < menu->numEntries; i++)
			SFREE(menu->menuTable[i].text);
		OFREE(menu);
	}
}
