/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 * 20.10.00 Dimitri Junker (DYJ)
 *	Ich habe je einen GemScript und CatProtokoll Befehl hinzugefgt:
 *	MsgFullText bzw CAT_FTEXT die jeweils den Messagetext incl Header
 *	zurckgeben.
 */


/*==============================================================*
 * Modul:               Protokollmodul fuer Cat                 *
 * Autor:               Johannes Goettker-Schnetmann            *
 * erstellt am:         Maerz 1991                              *
 * letzte Aenderung am: 01.12.1992                              *
 * Version:             1.0                                     *
 * Interne Version:     V#0002                                  *
 *==============================================================*

 Dieses Modul macht Cat zu einem Xacc-Hauptprogramm der Stufe 0. Ausserdem
 werden vom AV/VA-Protokoll die sinnvollen Msgs unterstuetzt.

 Weiterhin ist hier das Protokoll fuer den WatchDog verwirklicht:
 Ein WatchDog-Acc wird erkannt an seinem Namen "WATCHDOG", der beim AV- und
 XAcc-Protokoll erkannt wird. Sobald er erkannt wurde, bekommt er eine Msg
 [HelloWatchDog, CatGreetings, Version, (CatVersionString)]. Er kann dann
 [HelloWatchDog, enable/disableWatchDog] verschicken und bekommt jeweils
 eine [HelloWatchDog, CatAffirm, on/off] als Bestaetigung. Ausserdem bekommt
 er eine Msg CatTerminate am Programmende von Cat. Ist der WatchDog
 waehrend des Einfuegens von Msgs angeschaltet, dann wird er jeweils mit
 einem [HelloWatchDog, CatAsk, ptr] gefragt, ob diese Msg passieren darf.
 Er kann mit einem [HelloWatchDog, CatAccept/CatForget/CatFiltered] antworten.
 Im ersten Fall wird die Msg uebernommen, im zweiten Fall wird sie verworfen und
 im dritten Fall wird bei ihr das "gefiltert"-Flag gesetzt.

 Dieses WatchDog-Protokoll macht unter MultiTOS mit Memoryprotection vorraussichtlich
 Probleme, da der WatchDog nach der bisherigen Implementation auf Speicher von
 Cat zugreift! Wahrscheinlich muessen wir uns dafuer noch etwas einfallen lassen.
 (shared-memory..)

 Die Erlaeuterung des erweiterten Protokolls steht weiter unten. "HelloWatchDog"
 hat jetzt die allgemeinere Bedeutung "CatMsg".
 Insbesondere die Erkennung des WatchDogs soll in einer der naechsten Versionen
 nach Moeglichkeit auf die Methode des erweiterten Protokolls umgestellt werden.

 *----------------------------------------------------------------------------
 * Datum    Vers. Autor  Aenderung (Arbeitsbericht)
 *----------------------------------------------------------------------------
 * 01.02.92 0002  JGS    Protokollerweiterung fuer Steueraccs
 * 24.05.93 0003  DS     Erweiterung um MultiTasking-Unterstuetzung
 *                       CAT XACC-Versionsnummer geaendert auf 20H
 *----------------------------------------------------------------------------
 */


#include <protokol.h>
#include <catproto.h>
#include <catvers.h>
#include <xacc.h>
#include <vaproto.h>
#include <catgscr.h>
#include <gscript.h>
#include <olga.h>
#include <watchdog.h>
#include <wdwmanag.h>
#include <cattypes.h>
#include <data.h>
#include <grin.h>
#include <catgloba.h>
#include <msgwindo.h>

#ifndef __MINTBIND_H__
#include <mintbind.h>
#endif
#include <convertd.h>
#include <ro_mem.h>
#include <file_io.h>
#include <mtcomman.h>
#include <catmain.h>

/*
 * Name unter dem sich CAT ueber das XACC-Protokoll zu erkennen gibt
 */
#define appName    "CAT     \0" "XDSC\0" "1MausTausch-Programm\0" "2DC\0" "XFontAck\0" "\0"

/* Sub-Opcodes fuer Cat2FiltMsg */

#define FilterStandard    0        /* Default */

/* Sub-Opcodes fuer ExtCatMsg */

#define CatReadMsgInfo  1   /* MSGINFO.DAT neu einlesen */


/* Sub-Opcodes fuer neues CAT-Protokoll */
/* Doku: Siehe NEWPROTO.TXT            */
enum { 	CAT_ACK,		CAT_ERROR,		CAT_LOGIN,	CAT_LOGOFF,		CAT_SENDER,
		CAT_RNAME,		CAT_RECEIVER,	CAT_MAUS_ID,CAT_MSG_ID,		CAT_MAUS_REF,
		CAT_REF,		CAT_TOPIC,		CAT_BOX,	CAT_GATE,		CAT_DIST,
		CAT_MIME,		CAT_STATUS,		CAT_FOLLOWUP,CAT_REPLYTO,	CAT_TEXT,
		CAT_MSG_DATE,	CAT_STATUS_DATE,CAT_GROUP,	CAT_ORG_TEXT,	CAT_IS_OWN,
		CAT_UNIQUE_ID,	CAT_REAL_SENDER,CAT_WDW_NAME,
		CAT_FTEXT,

		CAT_NPROT_ANZ}NEWPROTO;

#define CAT_NEW_MSG       100
#define CAT_TERMINATE     101


/* --------------------------------------------------------------------------
Also dann:
mess[0] ist bei allen Msgs 0x8000, d.h. 8000 hexadezimal.
mess[1] ist ja einfach die Applikation-ID des Absenders;
mess[2] ist wie ueblich die Ueberlaenge der Nachricht, also die Gesamtlaenge
der Nachricht - 16. CB == CatBaum als Arbeitsname.

mess[3],mess[4]
CatGreetings
 6              CB->Cat Anmeldung bei Cat (evtl. kommt in mess[5] noch eine
                Unterscheidung, ob es sich um einen WatchDog handelt, also
                dort am besten eine Null eintragen). Fuer etwas anderes als
                einen Watchdog darf in mess[5] keine 0 stehen.
 6      version Cat-CB  Anmeldung angekommen, Protokoll-Versionsnummer zum
                ueberpruefen als Anlage. (evtl. mess[5,6,7] Cat-Versionsnummer
                als String? - erstmal nicht!)

WdwManager
 9      art     ACC->Cat, Anforderung der offenen Fenster, mess[5+6]
                enthalten eine Adresse fuer den Buffer, in den CAT
                die Information hineinschreibt. mess[7] enthaelt die
                Maximalanzahl an Fenstern, die in den Buffer passen
                (pro Eintrag ein Integer, also 2 Byte)
                "art" enthaelt die Art der Fenster, die geliefert
                werden sollen: 0: MsgFenster, 1: Editoren, 2:
                Stichwortlisten
 9      art     Cat->ACC, offene Fenster, mess[5+6] enthaelt den Pointer
                auf den Buffer, der vorher uebergeben wurde. In dem Buffer
                steht die Liste der Fensterhandles, wenn das vom AES aus
                klappt. mess[7] enthaelt die Anzahl der Fenster, die von
                CAT geliefert werden.

Die folgenden Msgs werden aber zunaechst einmal nur von MsgFenstern
unterstuetzt.

CatWinInfo
10      whdl    ACC->Cat, Informationen anfordern. mess[5+6] enhaelt eine
                Adresse, die sagt, wo die Informationen sollen , Format der
                Information wie "winInfo" (noch nicht festgelegt). mess[7]
                enthaelt die Laenge des Buffers in mess[5+6].
10      whdl    Cat->ACC, Information wurde geschrieben, Bufferadresse
                wieder in mess[5+6].
CatWhatsIn
11      whdl    CB->Cat, Welche Informationen werden im angegebenen Fenster
                dargestellt?
11      whdl    Cat->CB mess[5] Gruppe, mess[6] interne MsgNummer
CatPosition
12      whdl    CB->Cat, Informationen ueber MsgPositionen. mess[7] gibt an,
                um welche Art es sich handelt: 0 Neue, 1 letzte Position, 2
                ab Datum, 3 ab Nummer. In den letzten beiden Faellen wird in
                mess[5+6] noch eine Adresse auf einen String uebergeben, der
                das Datum bzw. die ID enthaelt. Er ist nullterminiert.
12      whdl    Cat->CB, mess[5] interne Msgenummer; 0xffff heisst: Nicht
                gefunden
CatNewMess
13      whdl    CB->Cat, neuen Inhalt setzen: mess[5] Gruppe mess[6] interne
                MsgNummer
13      whdl    Cat->CB mess[5] == 0 -> hat geklappt, mess[5] != 0 -> geht
                nich..

Hier nun meine erste Idee zu der Sache mit den Fensterwechseln. Dabei gibt
es jetzt ja ein paar Probleme. Ich muss dann ja den Speicher des Accs
beschreiben, das sollte ich aber nur nach Aufforderung machen. Wenn der User
jetzt sehr schnell durch die Msgs durchrennt, dann bekommt das Acc nicht
jeden Wechsel mit, da es ja immer die neue Informationen anfordern muss. Aber
den Prozesswechsel kann man dann ja vielleicht doch irgendwie forcieren. Dann
weiss das Acc zwar, dass es hier einen neuen Inhalt gibt und bekommt auch mit,
wenn der User weitergeblaettert hat, ohne dass die Information ueber die neue
Fensterposition schon angekommen ist, es kommt aber nicht zu Verstopfungen.
Aber vielleicht ist dieses Verhalten ja auch erwuenscht, denn wenn er dann
anhaelt, wird die neue Position ja dargestellt.

CatInform
14              CB->Cat, Acc in die Fensterwechsel/-oeffnen-Informationsliste
                ein-/austragen. mess[5] == 0 austragen, mess[5] == 1 eintragen,
                mess[6] ist das Window-Handle des ACC-Fensters
14              Cat->CB Acc ein-/ausgetragen, mess[5] wie oben.
CatWinDirty
15      whdl    Cat->CB dieses Fenster hat einen neuen Inhalt bzw. wurde
                geoeffnet. mess[5] == 0: neuer Inhalt; 1: getopped; 2: neu
                geoeffnet; 3: geschlossen


---- ab hier Erweitungsplanung fuer spaeter, erstmal ist hier Ende der Erweiterung -----
Es wird etwas problematisch, wenn das Acc in die MsgBase schreiben will.
Zunaechst ist das in Cat nur dann ohne groessere Probleme moeglich, wenn auch
die entsprechende Gruppe geoeffnet ist. Daher brauche ich das whdl, um damit
das Gruppenhandle zu bekommen. Mir faellt auch gerade auf, dass wir noch keine
Moeglichkeit vorgesehen haben, an MsgInfos von Msgs zu kommen, die nicht
gerade in einem Fenster dargestellt werden. Das muss ja wohl auch noch rein,
wenn Du damit Baeume markieren willst. Also
16      whdl    CB->Cat; Infos zu einer anderen Msg aus dieser Gruppe
                anfordern. mess[5+6] Pointer wie in Msg Nummer 10, mess[6]
                interne Nummer. Leider koennen wir dann nicht unterscheiden,
                welche Infos angefordert werden sollen. Hmm..
Die Gruppe wird hier also ueber das whdl festgelegt, was natuerlich nicht so
schoen ist. Wie gesagt, wahrscheinlich muessen wir uns beim Schreiben noch
etwas neues ausdenken.
Anwort ist dann eine Msg vom Typ 10
17      whdl    CB->Cat; Flags setzen. mess[5] interne Nummer; mess[6] neue
                gesetzte Flags; mess[7] neu geloeschte Flags.
17      whdl    Cat->CB; Flags setzen; mess[5] == 0 alles ok; mess[5] != 0 ==
                etwas ist schiefgelaufen.

-----------------------------------------------------------------------------*/


#define maxAcc 80       /* Wegen MTOS von 6 auf 80 erweitert. Zwar kann
                         * man unter MTOS auch mehr als 80 ACCs und Applikationen
                         * haben, aber das waere ziemlich krank */

LOCAL _UWORD protoCnt;
LOCAL _WORD protoAcc[maxAcc];	/* Noch ein Array fuer Apps,
                                 * diesmal fuer die, die am CAT-Proto
                                 * teilnehmen
                                 */

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _VOID Catproto_SendNewMsgInWdw(_VOID)
{
	struct {
		_WORD messId;
		_WORD apId;
		_WORD over;
		_WORD catType;
		_ULONG length;
		_VOID *buf;
	} message;
	_UWORD i;

	message.messId = CatProtoMsg;
	message.apId = gl_apid;
	message.over = 0;
	message.catType = CAT_NEW_MSG;
	message.length = 0;
	message.buf = NULL;
	Wind_Update(END_UPDATE); /* Um es auch mal abzuschicken.. */
	for (i = 0; i < protoCnt; i++)
		appl_write(protoAcc[i], 16, (_WORD *)&message);
	Wind_Update(BEG_UPDATE);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

typedef struct {
	_WORD messId;
	_WORD apId;
	_WORD over;
	_WORD catType;
	_ULONG length;
	_VOID *buf;
} mType;

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ReplyError(CONST mType *mess)
{
	mType msg;

	msg.messId = mess->messId;
	msg.apId = gl_apid;
	msg.over = 0;
	msg.catType = CAT_ERROR;
	msg.length = ((_ULONG)mess->catType) << 16;
	msg.buf = NULL;
	appl_write(mess->apId, 16, (_WORD *)&msg);
	/* Um es auch mal abzuschicken.. */
	Wind_Update(END_UPDATE);
	Wind_Update(BEG_UPDATE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ReplyOk(CONST mType *mess)
{
	mType msg;

	msg.messId = mess->messId;
	msg.apId = gl_apid;
	msg.over = 0;
	msg.catType = CAT_ACK;
	msg.length = 0;
	msg.buf = NULL;
	appl_write(mess->apId, 16, (_WORD *)&msg);
	/* Um es auch mal abzuschicken.. */
	Wind_Update(END_UPDATE);
	Wind_Update(BEG_UPDATE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID Reply(CONST mType *mess)
{
	mType msg;

	msg.messId = mess->messId;
	msg.apId = gl_apid;
	msg.over = 0;
	msg.catType = mess->catType;
	msg.length = mess->length;
	msg.buf = mess->buf;
	appl_write(mess->apId, 16, (_WORD *)&msg);
	/* Um es auch mal abzuschicken.. */
	Wind_Update(END_UPDATE);
	Wind_Update(BEG_UPDATE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID RemoveAcc(_WORD apId)
{
	_UWORD i;

	for (i = 0; i < protoCnt && protoAcc[i] != apId; i++)
		;
	if (i < protoCnt)
	{
		--protoCnt;
		for (; i < protoCnt; i++)
			protoAcc[i] = protoAcc[i + 1];
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Prueft, ob schon ein Prozess mit der ID registriert ist */
LOCAL _BOOL checkId(_WORD id)
{
	_UWORD i;

	for (i = 0; i < protoCnt; i++)
		if (protoAcc[i] == id)
			return FALSE;
	return id != gl_apid;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL AddAcc(_WORD apId)
{
	if (protoCnt < maxAcc && checkId(apId))
	{
		protoAcc[protoCnt] = apId;
		protoCnt++;
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID HandleCatProtoMsg(CONST _VOID *mess)
{
	CONST mType *mPtr;
	HWND wdw;
	struct {
		_UWORD date;
		_UWORD time;
	} *dateRecPtr;
	_UBYTE *strPtr;
	_ULONG *lcPtr;
	_WORD *intPtr;
	_ULONG lCard;
	_BOOL sizeReq;
	tDistribution dist;
	F_TIME time;
	mPtr = mess;
	if (mPtr->catType >= CAT_NPROT_ANZ && mPtr->catType != CAT_NEW_MSG && mPtr->catType != CAT_TERMINATE)
	{
		ReplyError(mPtr);
		return;
	}
	switch (mPtr->catType)
	{
	case CAT_ACK:
		break;

	case CAT_ERROR:
		break;

	case CAT_LOGIN:
		/* ACC anmelden */
		if (AddAcc(mPtr->apId))
			ReplyOk(mPtr);
		else
			ReplyError(mPtr);
		break;

	case CAT_LOGOFF:
		RemoveAcc(mPtr->apId);
		ReplyOk(mess);
		break;

	case CAT_NEW_MSG:
		break;

	case CAT_TERMINATE:
		break;

	default:
		/* Window von grin erfragen */
		grin_GetGrinTopWdw(&wdw);
		if (wdw < 0)
		{
			ReplyError(mPtr);
			return;
		}
		/* Andere Nachricht */
		lcPtr = &lCard;
		if (mPtr->length == 0)
		{
			sizeReq = TRUE;
		} else
		{
			sizeReq = FALSE;
		}
		/* Groessenabfrage */
		switch (mPtr->catType)
		{
		case CAT_SENDER:
			*lcPtr = grin_ActFromSize(wdw);
			break;
		case CAT_RNAME:
			*lcPtr = grin_ActNameSize(wdw);
			break;
		case CAT_RECEIVER:
			*lcPtr = grin_ActReceiverSize(wdw);
			break;
		case CAT_MAUS_ID:
			*lcPtr = grin_ActIdSize(wdw);
			break;
		case CAT_MSG_ID:
			*lcPtr = grin_ActMIdSize(wdw);
			break;
		case CAT_MAUS_REF:
			*lcPtr = grin_ActRefIdSize(wdw);
			break;
		case CAT_REF:
			*lcPtr = grin_ActRIdSize(wdw);
			break;
		case CAT_TOPIC:
			*lcPtr = grin_ActSubjectSize(wdw);
			break;
		case CAT_BOX:
			*lcPtr = grin_ActBoxSize(wdw);
			break;
		case CAT_GATE:
			*lcPtr = grin_ActGateSize(wdw);
			break;
		case CAT_DIST:
			*lcPtr = grin_ActDistSize(wdw);
			break;
		case CAT_MIME:
			*lcPtr = grin_ActMimeSize(wdw);
			break;
		case CAT_STATUS:
			*lcPtr = grin_ActStatusSize(wdw);
			break;
		case CAT_FOLLOWUP:
			*lcPtr = grin_ActFollowupSize(wdw);
			break;
		case CAT_REPLYTO:
			*lcPtr = grin_ActReplyToSize(wdw);
			break;
		case CAT_TEXT:
		case CAT_FTEXT:
			*lcPtr = grin_ActTextSize(wdw,mPtr->catType==CAT_FTEXT);
				break;
		case CAT_MSG_DATE:
			*lcPtr = grin_ActDateSize(wdw);
			break;
		case CAT_STATUS_DATE:
			*lcPtr = grin_ActStatusDateSize(wdw);
			break;
		case CAT_GROUP:
			*lcPtr = grin_ActGroupSize(wdw);
			break;
		case CAT_ORG_TEXT:
			*lcPtr = grin_ActOrgTextSize(wdw);
			break;
		case CAT_IS_OWN:
			*lcPtr = grin_ActIsOwnSize(wdw);
			break;
		case CAT_UNIQUE_ID:
			*lcPtr = grin_ActInternalIdSize(wdw);
			break;
		case CAT_REAL_SENDER:
			*lcPtr = grin_ActSenderSize(wdw);
			break;
		case CAT_WDW_NAME:
			*lcPtr = grin_ActWdwNameSize(wdw);
			break;
		}
		if (sizeReq)
		{
			mType mess;
			
			mess = *mPtr;
			mess.length = *lcPtr;
			/* Antwort schicken */
			Reply(&mess);
		} else
		{
			/* Datenabfrage */
			if (lCard > mPtr->length || mPtr->buf == NULL)
			{
				ReplyError(mPtr);
				return;
			}
			strPtr = mPtr->buf;
			dateRecPtr = mPtr->buf;
			switch (mPtr->catType)
			{
			case CAT_SENDER:
				grin_ActualFrom(wdw, strPtr, mPtr->length);
				break;
			case CAT_RNAME:
				grin_ActualName(wdw, strPtr, mPtr->length);
				break;
			case CAT_RECEIVER:
				grin_ActualReceiver(wdw, strPtr, mPtr->length);
				break;
			case CAT_MAUS_ID:
				grin_ActualID(wdw, strPtr, mPtr->length);
				break;
			case CAT_MSG_ID:
				grin_ActualMId(wdw, strPtr, mPtr->length);
				break;
			case CAT_MAUS_REF:
				grin_ActualRefId(wdw, strPtr, mPtr->length);
				break;
			case CAT_REF:
				grin_ActualRId(wdw, strPtr, mPtr->length);
				break;
			case CAT_TOPIC:
				grin_ActualSubject (wdw, strPtr, mPtr->length);
				break;
			case CAT_BOX:
				grin_ActualBox(wdw, strPtr, mPtr->length);
				break;
			case CAT_GATE:
				grin_ActualGate(wdw, strPtr, mPtr->length);
				break;
			case CAT_DIST:
				grin_ActualDist(wdw, &dist);
				strPtr[0] = "KLMN"[dist];
				strPtr[1] = '\0';
				break;
			case CAT_MIME:
				grin_ActualMime(wdw, strPtr, mPtr->length);
				break;
			case CAT_STATUS:
				grin_ActualStatus(wdw, strPtr, mPtr->length);
				break;
			case CAT_FOLLOWUP:
				grin_ActualFollowup(wdw, strPtr, mPtr->length);
				break;
			case CAT_REPLYTO:
				grin_ActualReplyto(wdw, strPtr, mPtr->length);
				break;
			case CAT_TEXT:
			case CAT_FTEXT:
				grin_ActualText(wdw, strPtr, mPtr->length,mPtr->catType==CAT_FTEXT);
				break;
			case CAT_MSG_DATE:
				grin_ActualDate(wdw, &time);
				T_SysToDostime(T_Mktime(&time), &dateRecPtr->date, &dateRecPtr->time);
				break;
			case CAT_STATUS_DATE:
				grin_ActualStatusDate(wdw, &time);
				T_SysToDostime(T_Mktime(&time), &dateRecPtr->date, &dateRecPtr->time);
				break;
			case CAT_GROUP:
				grin_ActualGroup(wdw, strPtr, mPtr->length);
				break;
			case CAT_ORG_TEXT:
				grin_ActualOrgText(wdw, strPtr, mPtr->length);
				break;
			case CAT_IS_OWN:
				intPtr = mPtr->buf;
				if (grin_ActualIsOwn(wdw)!=MSG_FREMD)
					*intPtr = 1;
				else
					*intPtr = 0;
				break;
			case CAT_UNIQUE_ID:
				lcPtr = mPtr->buf;
				grin_ActualInternalId(wdw, lcPtr);
				break;
			case CAT_REAL_SENDER:
				grin_ActualSender(wdw, strPtr, mPtr->length);
				break;
			case CAT_WDW_NAME:
				grin_ActualWdwName(wdw, strPtr, mPtr->length);
				break;
			}
			Reply(mPtr);
		}
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID HandleCatMsg(CONST _VOID *m)
{
	CONST struct {
		_WORD messId;
		_WORD apId;
		_WORD over;
		_WORD catType;
		union {
			_WORD data[4];
			struct {
				_WORD d0;
				_VOID *adr;
				_WORD d1;
			} a;
		} d;
	} *mess;
#if GRIN_WINDOW_PROTO
	_VOID *a;
	_WORD wAnz;
#endif

	mess = m;
	switch (mess->catType)
	{
	case CatGreetings:
		/* "Hallo Acc, hier bin ich und das ist meine Versionsnummer!" */
		Protokoll_SendMessInt(CatMsg, CatGreetings, CatProtoVersion, 0, 0, 0, mess->apId);
		break;

#if GRIN_WINDOW_PROTO
	case WdwManager: /* Typ in data[0] */
		if (mess->d.data[0] == 0)
		{
			a = mess->d.a.adr; /* Dort sollen sie hin. */
			grin_getAllWinds(a, mess->d.data[3], &wAnz); /* Fenster besorgen */
			Protokoll_SendAdr(CatMsg, WdwManager, 0, a, wAnz, mess->apId);
		} else
		{
			/* Koennen wir noch nicht */
			Protokoll_SendAdr(CatMsg, WdwManager, mess->d.data[0], a, 0, mess->apId);
		}
		break;

	case CatWinInfo:
		a = mess->d.a.adr; /* Dort sollen sie hin. */
		if (grin_getInfo(mess->d.data[0], a, mess->d.data[3]))
			Protokoll_SendAdr(CatMsg, CatWinInfo, mess->d.data[0], a, mess->d.data[3], mess->apId);
		else
			Protokoll_SendAdr(CatMsg, CatWinInfo, mess->d.data[0], NULL, 0, mess->apId);
		break;

	case CatWhatsIn:
		{
			struct {
				GROUPNR gr;
				MSGNR nr;
				_ULONG am;
			} grInfo, *b;
			b = mess->d.a.adr;
			if (grin_getBasicInfo(mess->d.data[0], &grInfo.gr, &grInfo.nr, &grInfo.am))
			{
				*b = grInfo;
				Protokoll_SendAdr(CatMsg, CatWinInfo, mess->d.data[0], b, 0, mess->apId);
			} else
			{
				/* Protokoll_SendMessInt(CatMsg, CatWhatsIn, mess->data[0], INTEGER(gr), INTEGER(nr), INTEGER(am), mess->apId); */
				/* sind eigentlich CARDINAL-Werte, aber die Typkonversion aendert ja nichts.. */
				Protokoll_SendAdr(CatMsg, CatWinInfo, mess->d.data[0], NULL, 0, mess->apId);
				/* Protokoll_SendMessInt(CatMsg, CatWhatsIn, mess->d.data[0], 0, 0, 0, mess->apId); */
			}
		}
		break;

	case CatPosition:
		a = mess->d.a.adr;
		Protokoll_SendMessInt(CatMsg, CatPosition, mess->d.a.d0,
			grin_getPos(mess->d.a.d0, mess->d.a.d1, a),
			0, 0, mess->apId);
		break;

	case CatNewMess:
		if (grin_switchMess(mess->d.data[0], mess->d.data[2], mess->d.data[1]))
			wAnz = 0;
		else
			wAnz = 1;
		Protokoll_SendMessInt(CatMsg, CatNewMess, wAnz, 0, 0, 0, mess->apId);
		break;

	case CatInform:
		break;
	/* CatWinDirty kommt hier nicht an.. :-) */
#endif /* GRIN_WINDOW_PROTO */
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Protokoll_Send2Filter(_WORD to, _UWORD length, _VOID *what)
{
	_WORD message[8];

	if (!Protokoll_Init())
		return FALSE;
	if (length > memDescrSize)
		return FALSE;
	MemCpy(memPtr, what, length);
	message[0] = Cat2FiltMsg;
	message[1] = gl_apid;
	message[2] = 0;
	message[3] = length;
	*((_UBYTE **)(&message[4])) = memPtr;
	message[6] = 0;
	message[7] = 0;
	appl_write(to, 16, message);
	Wind_Update(END_UPDATE); /* Um es auch mal abzuschicken.. */
	Wind_Update(BEG_UPDATE);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL Catproto_HandleMessage(CONST _WORD *message)
{
	if ((_UWORD)message[0] == CatMsg)              /* CatMsg */
	{
		HandleCatMsg(message);
	} else if ((_UWORD)message[0] == ExtCatMsg)    /* CatMsg */
	{
		switch (message[3])
		{
		case CatReadMsgInfo:
			MsgWindow_msgUpdateInfo();
			break;
		default:
			return FALSE;
		}
	} else if ((_UWORD)message[0] == CatProtoMsg)  /* neues CAT-Protokoll */
	{
		/* wird komplett hier behandelt: */
		HandleCatProtoMsg(message);
	} else
	{
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Nachricht an die Protokoll-Partner schicken, dass Cat beendet wurde */
GLOBAL _VOID Protokoll_SendTerminate(_BOOL terminate)
{
	struct {
		_UWORD messId;
		_WORD apId;
		_WORD over;
		_UWORD catType;
		_ULONG length;
		_VOID *buf;
	} mess;
	_UWORD i;
	
	WatchDog_SendTerminate();

	/* Und unter MultiTOS auch noch ACC_EXIT verschicken an alle
	 * anderen Applikationen, die sich gemeldet haben
	 */
	Xacc_SendTerminate(terminate);
	/* und auch an die VA-Partner */
	VaProto_SendTerminate(terminate);
	
	if (terminate)
	{
		mess.messId  = CatProtoMsg;
		mess.apId    = gl_apid;
		mess.over    = 0;
		mess.catType = CAT_TERMINATE;
		mess.length  = 0;
		mess.buf     = NULL;
		Wind_Update(END_UPDATE);
		for (i = 0; i < protoCnt; i++)
			appl_write(protoAcc[i], 16, (_WORD *)&mess);
		Wind_Update(BEG_UPDATE);
		GScript_Exit();
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Protokoll_ProtoInit(_VOID)
{
	Protokoll_Handle_Message(Catproto_HandleMessage);
	WatchDog_Init();
	Xacc_Init(appName, sizeof(appName));
	VaProto_Init(appName, sizeof(appName));
	CATGScript_Init();
	Olga_Init(OL_CLIENT, OLGA_Comm);
}
