/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <catutil.h>
#include <cattypes.h>
#include <catfiles.h>
#include <catgloba.h>
#include <editor.h>
#include <shellcal.h>
#include <convertd.h>
#include <confvars.h>
#include <varn_ede.h>
#include <varn_edi.h>
#include <msgwindo.h>
#include <grin.h>
#include <msglist.h>
#include <treelist.h>
#include <arcstart.h>
#include <mtpaths.h>
#include <userinfo.h>

#include <mttextfi.h>
#include <strconv.h>
#include <magicstr.h>
#include <mtappl.h>

#include <ro_mem.h>
#include <file_io.h>
#include <routine.h>
#include <tos.h>
#include <mtalerts.h>
#include <varname5.h>
#include <varnamel.h>
#include <routinec.h>
#include <routines.h>


#define C_EXEC_SELF 0

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _VOID CatUtil_makeTmpName(_UBYTE *path, _UBYTE *name)
{
	_UBYTE currentPath[PATH_MAX];
	_UBYTE tname[PATH_MAX];
	_ULONG mtime;
	
	/* Aktuellen Pfad merken */
	F_Path_Get(currentPath);
	/* TmpDir holen */
	if (!CatFiles_GetTmpDir(path))
	{
		/* tmpDir ist aktuelles Verzeichnis! */
		strMcpy(path, PATH_MAX, currentPath);
	}
	/* Jetzt Pfad richtig abschliessen mit Backslash */
	F_Path_Append(path, "");
	/* Tmp-Pfad setzen */
	F_Path_Set(path);
	/* Dateinamen basteln aus Inhalt des 200 Hz Timers */
	do
	{
		mtime = Random();
		strCpy(tname, "ct");
		LNumToStr(mtime, 36, 6, '0', tname + 2, sizeof(tname) - 2);
		strBcat(tname, ".asc");
		/* Jetzt testen, ob Datei schon vorhanden */
		strMcpy(name, PATH_MAX, tname);
	} while (CatFiles_Exists(name));
	/* Aktuellen Pfad wiederherstellen */
	F_Path_Set(currentPath);
}

/*** ---------------------------------------------------------------------- ***/

/* In dieser Prozedur werden hauptsaechlich Strings ersetzt
 * Das eigentliche zusammenbauen der Kommandozeile wird von einer
 * Prozedur weiter oben in der Hierarchie erledigt
 * Folgendes wird ersetzt: $B fuer Eingabeblock durch ersten Namen (in),
 * $O fuer Ausgabe des Kommandos durch zweiten Namen
 */
GLOBAL _VOID CatUtil_buildCommand(HWND wdw, CONST _UBYTE *com, CONST _UBYTE *path, CONST _UBYTE *in, CONST _UBYTE *out, _UBYTE *cmd, size_t cmdsize)
{
	_UBYTE tname[PATH_MAX];
	_UBYTE *p;
	
	strBcpy(tname, path);
	F_Path_Append(tname, in);
	strMcpy(cmd, cmdsize, com);
	CatGlobal_ReplaceStr(cmd, cmdsize, "$B", tname);
	strBcpy(tname, path);
	F_Path_Append(tname, out);
	CatGlobal_ReplaceStr(cmd, cmdsize, "$O", tname);
	if (Messages_MsgEditorTop(wdw))
	{
		Messages_MsgEditGetInfos(wdw, itReceiver, tname, sizeof(tname));
		p = strChr(tname, '@');
		if (p != NULL)
		{
			*p = '\0';
			if (p != tname && p[-1] == ' ')
				p[-1] = '\0';
		}
		CatGlobal_ReplaceStr(cmd, cmdsize, "$U", tname);
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#if C_EXEC_SELF

LOCAL _VOID getSystemPaths(List *pathList)
{
	_UBYTE *systemPath;
	_UBYTE *last, *p;
	_UBYTE path[PATH_MAX];
	size_t len;
	_UBYTE *new;
	
	systemPath = getenv("PATH");
	if (systemPath != NULL && *systemPath != '\0')
	{
		last = systemPath;
		if (*last == ',' || *last == ';')
			last++;
		do
		{
			p = strChr(last, ',');
			if (p == NULL)
				p = strChr(last, ';');
			if (p != NULL)
			{
				len = p - last + 1;
				if (len > sizeof(path))
					len = sizeof(path);
				strMcpy(path, len, last);
				/* Jetzt neuen Eintrag fuer die Liste erzeugen */
				new = STRDUP(path, "getSystemPaths");
				if (new != NULL)
				{
					/* Neuen Eintrag anlegen */
					Lists_AppendEntry(pathList, new);
				}
			}
			last = p;
		} while (last != NULL);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL SearchExec(_UBYTE *name)
{
	List paths;
	_BOOL found;
	_UBYTE newName[PATH_MAX];
	
	Lists_CreateList(&paths, "SearchExec");
	getSystemPaths(&paths);
	Paths_SearchFile(name, &paths, Paths_fromStart, &found, newName);
	/* Liste wieder loeschen */
	Lists_DeleteSimpleList(&paths, mem_free, TRUE);
	if (found)
		strMcpy(name, PATH_MAX, newName);
	return found;
}

#endif /* C_EXEC_SELF */
/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatUtil_PipeEditBlock(HWND wdw, CONST _UBYTE *command)
{
	_UBYTE path[PATH_MAX];
	_UBYTE tname[PATH_MAX];
	_UBYTE inName[PATH_MAX];
	_UBYTE outName[PATH_MAX];
	_ULONG len;
	_UBYTE cmdLine[256];
	_BOOL done;
	_BOOL exists;
	
	done = FALSE;
	if (*command == '\0')
		return;
	
	/* Vorgehensweise: 2*tmpFileName erstellen (in und out)
	 * Kommando mit Redirection an shell absetzen
	 * kommando <in >out
	 */
	CatUtil_makeTmpName(path, inName);
	/* Block aus Editor schreiben */
	Editor_EditSave(wdw, path, inName, FALSE,TRUE);
	/* zweiten tmpnamen basteln */
	CatUtil_makeTmpName(path, outName);
	/* Shellkommando basteln */
	CatUtil_buildCommand(wdw, command, path, inName, outName, cmdLine, sizeof(cmdLine));
#if C_EXEC_SELF
	/* Jetzt sehen, ob MagiX 5.04 oder groesser */
	if (magIx && magIxVer >= 0x504)
	{
		/* Testen, ob mit IO-Redirection. Wenn nicht, starten wir es
		 * direkt selber
		 */
		if (strChr(cmdLine, '<') == NULL &&
			strChr(cmdLine, '>') == NULL &&
			strChr(cmdLine, '|') == NULL)
		{
			_UBYTE *p;
			_WORD res;
			_UBYTE cmdName[PATH_MAX];
			_UBYTE cmdL[256];
			
			/* Keine IO-Redirection, keine Pipes, also selber ausfuehren */
			p = strChr(cmdLine, ' ');
			if (p != NULL)
			{
				/* split command line */
				strMcpy(cmdName, p - cmdLine + 1, p);
				strMcpy(cmdL, sizeof(cmdL) -1, p + 1);
				strleft(cmdL + 1);
				strdelRightSpace(cmdL + 1);
				cmdL[0] = (_UBYTE)strLen(cmdL + 1);
				if (SearchExec(cmdName))
				{
					res = ARCStarter_ExecProgram(cmdName, cmdL, 0);
					done = res == 0;
				}
			}
		}
	}
	done = TRUE;
#endif
	if (!done)
	{
		/* Shellkommando aufrufen */
		ShellCall_CallShell(cmdLine, TRUE);
	}
	/* Testen, ob Ausgabe existiert und laenger als 0 Bytes ist */
	strBcpy(tname, path);
	F_Path_Append(tname, outName);
	len = CatFiles_FileSize(tname, &exists);
	if (exists && len > 0)
	{
		/* Ausgabe in Editor einlesen */
		Editor_EditPasteFile(wdw, TRUE, path, outName);
	}
	/* temporaere Files wieder loeschen */
	CatFiles_DeleteFile(path, inName);
	F_File_Delete(tname);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CatUtil_GetPosPath(_UBYTE *path)
{
	_BOOL bool;
	
	/* Jetzt HOME Pfad holen */
	ConfVars_GetConfDefBool(cUseHomePath, &bool, FALSE);
	if (!bool || !CatFiles_GetHomeDir(path))
	{
		/* Jetzt temporaeren Pfad holen */
		ConfVars_GetConfDefBool(cUseTmpPath, &bool, FALSE);
		if (!bool || !CatFiles_GetTmpDir(path))
			CatGlobal_ApplPath(path);
	}
	/* ggf. Backslash anhaengen */
	F_Path_Append(path, "");
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID insert_app_name(_UBYTE *title, size_t strsize)
{
	_BOOL setappname;
	
	ConfVars_GetConfDefBool(cApplicationNameInWindow, &setappname, TRUE);
	if (multiTask && setappname)
		Insert("[" CAT_PROGNAME "] ", title, strsize, 0);
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID insert_box_name(_UBYTE *title, size_t strsize)					*/
/*	Fgt den Boxnamen an den Fenstertitel an falls dies lt. cBoxNameInWindow 	*/
/*	gewnscht ist.																*/
/*	Parameter:	title:	da soll es rein											*/
/*				strsize:und soviel Platz ist da									*/
/*	Ergebnis:	keines															*/		
/*	30.8.00 von Dimitri Junker	DYJ 4.18										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID insert_box_name(_UBYTE *title, size_t strsize)
{
	_BOOL setboxname;
	_UBYTE username[255],*boxname;
	
	ConfVars_GetConfDefBool(cBoxNameInWindow, &setboxname, TRUE);
	if (setboxname)
	{
		ConfVars_GetConfDefStr(cMausId, username, sizeof(username)-1, "");
		boxname=strChr(username,'@');
		if(boxname)
		{
			if(boxname[1]==' ')
				boxname++;
			boxname++;
			strCat(boxname,"/");
			Insert(boxname, title, strsize, 0);
		}
	}
}
/*DYJ  Ende */

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL CatUtil_WriteSavePos(_BOOL force, _BOOL delVars)
{
	_UBYTE ptail[PATH_MAX];
	_BOOL apName;

	_UBYTE username[255];
	_BOOL markBck,boxId;
	_BOOL scrollbar;

	ConfVars_GetConfDefBool(cMsgMarkBackground, &markBck, TRUE);
	ConfVars_GetConfDefBool(cBoxNameInWindow, &boxId, TRUE);
	ConfVars_GetConfDefStr(cMausId, username, sizeof(username)-3, "");
	ConfVars_GetConfDefBool(cApplicationNameInWindow, &apName, TRUE);
	ConfVars_GetConfDefBool(cMsgNoScrollbar, &scrollbar, FALSE);
	ConfVars_ConfigConfigModule(TRUE);
	ConfVars_SetConfigBool(cMsgMarkBackground, markBck);
	ConfVars_SetConfigBool(cBoxNameInWindow, boxId);
	ConfVars_SetConfigString(cMausId, username);
	ConfVars_SetConfigBool(cApplicationNameInWindow, apName);
	ConfVars_SetConfigBool(cMsgNoScrollbar, scrollbar);

	MsgWindow_msgSavePos();
	ConfVars_ConfigConfigModule(FALSE);
	if (!MsgWindow_msgCloseAll(force))
		return FALSE;
	ConfVars_ConfigConfigModule(TRUE);
	grin_grinSavePos();
	msgList_listSavePos();
	Editor_EditSavePos();
	ConfVars_SetConfigString(cLastCatInf, lastInfName);
	ConfVars_ConfigConfigModule(FALSE);
	if (!Editor_CloseAllEditors(FALSE))
		return FALSE;
	grin_grinCloseAll();
	msgList_listCloseAll();
	treeList_treeCloseAll();
	msgList_listFreeAll();
	/* Jetzt temporaeren Pfad holen */
	CatUtil_GetPosPath(ptail);
	F_Path_Append(ptail, tmpInf);
	ConfVars_ConfigConfigModule(TRUE);
	isTmp = TRUE;
	ConfVars_WriteConfigFile(ptail, UserInformation_writeInfo);
/*DYJ 5.30j Start*/
	strCpy(ptail,DataPath);
	F_Path_Append(ptail, tmpInf);
	ConfVars_WriteConfigFile(ptail, UserInformation_writeInfo);
/*DYJ  Ende; */
	if (delVars)
	{
		/* Variablen wieder loeschen */
		ConfVars_DeleteAllVars();
	}
	/* Modul wieder zurueckkonfigurieren */
	ConfVars_ConfigConfigModule(FALSE);
	return TRUE;
}

GLOBAL _WORD mtAlerts_AlertSichL(_WORD unsave, _WORD save, CONST _UBYTE *str)
{
	_WORD sichL,def=save,ret;
	ConfVars_GetConfDefInt(cSicherheitsLevel, &sichL, 2);
	switch(sichL)
	{
		case 0:			/*keine Abfrage	*/
			ret=unsave;
			break;
		case 1:			/*unsichere Abfrage	*/
			def=unsave;
		case 2:			/*Sichere Abfrage	*/
			ret=mtAlerts_Alert(def, str);
			break;
	}
	return ret;
}

