/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <clip.h>
#ifndef __MINTBIND_H__
#include <mintbind.h>
#endif
#include <catfiles.h>
#include <mtdir.h>
#include <mtalerts.h>
#include <catgloba.h>
#include <file_io.h>
#include <dir_io.h>
#include <ro_mem.h>
#include <protokol.h>
#include <wildcard.h>

#include <routines.h>
#include <routinec.h>
#include <debug.h>
#include <mtrsc.h>
#include <catlib.h>


/*==============================================================*
 * Modul:               Clipboard-Modul, nach Atari-Doku        *
 * Autor:               Johannes Goettker-Schnetmann            *
 * erstellt am:         10.02.1992                              *
 * letzte Aenderung am: 09.03.1992                              *
 * Version:             1.1                                     *
 * Interne Version:     V#0003                                  *
 *==============================================================*

 *----------------------------------------------------------------------------
 * Datum    Vers. Autor  Aenderung (Arbeitsbericht)
 *----------------------------------------------------------------------------
 * 10.02.92 0001  JGS    Erstellen
 * 08.03.92 0002  JGS    Environment beruecksichtigen
 * 09.03.92 0003  JGS    ScrapClear/ScrapDirty
 * 07.09.92 0004  DS     Bei GetScrap wird ein \CLIPBRD\ genommen, falls auf einem
 *                       Laufwerk vorhanden. Erst danach wird beim User angefragt.
 * 08.12.92 0005  DS     Bei folderExists wird auch Ruecksicht auf gelockte Laufwerke
 *                       genommen, d.h. bei ACCDN wird nicht nachgefragt
 * 11.09.94 0006  DS     Wenn wir etwas ins Clipboard schreiben, wird ein Broadcast
 *                       mit der Message SC_CHANGED gemacht.
 *----------------------------------------------------------------------------
 */

#define rawClip    "x:\\CLIPBRD"

LOCAL _BOOL scrapDirty;

#define ENV_Clipbrd     "CLIPBRD"     /* Clipboard-Verzeichnis */

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID ClearPath(CONST _UBYTE *path, CONST _UBYTE *mask, CONST _UBYTE *lock)
{
	Directory *d;
	DirectoryEntry *e;
	_UBYTE file[PATH_MAX];
	_UBYTE maskbuf[PATH_MAX];
	_UBYTE lockbuf[PATH_MAX];

	if ((d = D_Opendir(path)) == NULL)
	{
		F_ErrorAlert(-34);/*!ERROR*/
	} else
	{
		strBcpy(maskbuf, mask);
		strLwr(maskbuf);
		if (lock != NULL)
		{
			strCpy(lockbuf, lock);
			strLwr(lockbuf);
		} else
		{
			*lockbuf = '\0';
		}
		while ((e = D_Readdir(d)) != NULL)
		{
			strBcpy(file, e->d_name);
			strLwr(file);
			if (WildCards_MatchName(file, maskbuf, TRUE) &&
				!WildCards_MatchName(file, lockbuf, TRUE))
			{
				strBcpy(file, path);
				F_Path_Append(file, e->d_name);
				if (F_File_Delete(file) != GERR_OK)
					F_ErrorAlert(-36);/*!ERROR*/
			}
		}
		D_Closedir(d);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* leert das Clipboard falls Pfad leer, sonst den uebergebenen Pfad */
GLOBAL _VOID Clip_ScrapClear(CONST _UBYTE *f, CONST _UBYTE *not)
{
	_UBYTE path[PATH_MAX];

	strBcpy(path, f);
	/* Falls kein Pfad uebergeben wurde, dann mal anfragen */
	if (path[0] == '\0' && !Clip_GetScrap(path))
		return;
	ClearPath(path, SCRAP_MASK, not);
	scrapDirty = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL folderExist(_UWORD l, CONST _UBYTE *pfad)
{
	F_STAT st;
	_UBYTE name[PATH_MAX];
	
	strBcpy(name, pfad);
	if (l > 0 && name[l-1] == '\\')
		name[--l] = '\0';
	if (l == 0)
		return FALSE;
	if (F_Getinfo(name, &st, TRUE) == GERR_OK && st.st_is_directory)
	{
		return TRUE;
	}
	SPEC_DEBUG_OUT((EO_TRACE, "directory '%s' doesn`t exist", name));
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert den aktuellen Clipboard-Ordner */
/*DYJ 5.21b */
GLOBAL _BOOL Clip_GetScrap(_UBYTE *f)
{
	_UBYTE scrap[PATH_MAX];
	_UBYTE name[PATH_MAX];
	_BOOL setNew;
	_UBYTE *env;
	_UWORD z;

	setNew = FALSE; /* vorlaeufig nicht neu setzen */
	f[0]   = '\0';    /* Erstmal entwerten */

	/* if scrp_read() returns an error code of zero, then the
	  application should act as though nothing is available
	  on the clipboard */
	if (scrp_read(scrap) == 0)
	{
		SPEC_DEBUG_OUT((EO_DEBUG, "scrp_read failed"));
		return FALSE;
	}
	
	/* Jetzt noch im Environment nachsehen, falls bisher noch nichts gefunden */
	if (scrap[0] == '\0')
	{
		SPEC_DEBUG_OUT((EO_DEBUG, "clipboard path empty"));
		if ((env = getenv(ENV_Clipbrd)) != NULL)
			strBcpy(scrap, env);
	}

	if (scrap[0] != '\0')
	{
		z = (_UWORD)strLen(scrap);

		setNew = scrap[z-1] != '\\';
		/* In all diesen Faellen muss das Clipboard neu gesetzt werden */

		if (!folderExist(z, scrap))
		{
			scrap[0] = '\0'; /* Dann User fragen  */
		}
	}

	if (scrap[0] == '\0')	/* User fragen */
	{
		_ULONG drives;
		_WORD drv;

		/* Zunaechst mal bei den existierenden Laufwerken nach dem Rechten sehen..*/
		drives = Dsetdrv(Dgetdrv());
		SPEC_DEBUG_OUT((EO_DEBUG, "no clipboard found; checking drives $%08lx", drives));
		if (drives & 0x04)
		{
			drv = 2;	/* erstmal auf C:\ */
		} else if (drives & 0x01)
		{
			drv = 0;	/* dann auf A:\ */
		} else
		{
			/* und jetzt mal suchen, ob sonst vielleicht noch eins zu finden ist */
			z = 0;
			while (!(drives & 1) && z < 32)
			{
				z++;
				drives >>= 1;
			}
			drv = z;
			if (z >= 32)
			{
				/* Da kann man nichts machen.. */
				mtAlerts_Alert(1, Rsc_WrkString(AL_NO_DRIVES));
				drv = 0;
			}
		}

		strCpy(scrap, rawClip);
		scrap[0] = 'A' + drv;		/* passendes Laufwerk aussuchen */
		/* mal nachsehen, ob's dort ein CLIPBRD gibt */

		if (!folderExist((_UWORD)strLen(rawClip), scrap))
		{
			if (mtAlerts_Alert(1, Rsc_WrkString(AL_SEARCH_CLIP)) == 1)
			{
				/* nur das Laufwerk vorschlagen */

				scrap[3] = '\0';
				*name = '\0';
				if (!mtDir_FselGet(scrap, name, ALL_FILE_MASK, Rsc_WrkString(FS_SEARCH_CLIP), TRUE))	/* Mal beim User fragen */
				{
					return FALSE;
				}

				if (name[0] != '\0')	/* Es wurde ein Dateiname eingegeben */
				{
					if (mtAlerts_Alert(1, Rsc_WrkString(AL_CREATE_CLIP)) == 1)
					{
						/* Versuchen, diesen anzulegen */
						F_Path_Append(scrap, name);
						if (F_Path_Create(scrap) != GERR_OK)
						{
							SPEC_DEBUG_OUT((EO_DEBUG, "failed to create directory %s", scrap));
							mtAlerts_Alert(1, Rsc_WrkString(AL_CREATE_DIR));
							return FALSE;
						}
					} else
					{
						return FALSE;
					}
				}
				setNew = TRUE; /* Jetzt in jedem Fall setzen */
			} else
			{
				return FALSE;
			}
		} else
		{
			/* folder existiert, neu setzen darauf! */
			setNew = TRUE;
		}
	}
	if (setNew)
	{
		F_Path_Append(scrap, "");
		SPEC_DEBUG_OUT((EO_DEBUG, "setting new clipboard path %s", scrap));
		scrp_write(scrap); /* und mal alles anmelden */
	}
	strMcpy(f, PATH_MAX, scrap); /* .. und am Ende das Resultat zuweisen */
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Ist etwas im Clipboard? */
/* d.h. sieht entweder im uebergebenen Pfad oder im Clipboard nach */
/* ob ein SCRAP.TXT vorhanden ist */
GLOBAL _BOOL Clip_ScrapDirty(_VOID)
{
	_UBYTE path[PATH_MAX];

	/* Falls kein Pfad uebergeben wurde, dann mal anfragen */
	if (!Clip_GetScrap(path))
	{
		scrapDirty = FALSE;
	} else
	{
		F_Path_Append(path, SCRAP_TXT);
		scrapDirty = F_File_Exists(path);
	}
	return scrapDirty;
}

/*** ---------------------------------------------------------------------- ***/

/* Wir haben selber etwas in's Clipboard geschrieben */
GLOBAL _VOID Clip_ScrapWritten(_UWORD type, CONST _UBYTE *defExt)
{
	_WORD msg[8];

	scrapDirty = TRUE;
	/* Nachricht basteln */
	msg[0] = SC_CHANGED;
	msg[1] = gl_apid;
	msg[2] = 0;
	msg[3] = type;
	MemCpy(&msg[4], defExt, 4);
	msg[6] = 0;
	msg[7] = 0;
	Protokoll_Broadcast(msg);
}

/*** ---------------------------------------------------------------------- ***/

/* Sieht intern in einer Variablen nach. Im Zweifelsfall eher
 * ScrapDirty benutzen
 */
GLOBAL _BOOL Clip_SomethingInScrap(_VOID)
{
	return scrapDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Clip_ScrapIsDirty(_VOID)
{
	scrapDirty = TRUE;
}
