/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <convertd.h>
#include <strconv.h>
#include <magicstr.h>
#include <ro_time.h>
#include <routines.h>
#include <routinec.h>
#include <ro_mem.h>

#include <tos.h>
#include <mtrsc.h>
#include <catlib.h>

LOCAL _LONG mDays[12] = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };

#define StdDateMask "DD.MM.YYYY"
#define StdTimeMask "HH:MM:SS"

#define L_USER     (-1)


/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID GetCurrentDate(Date *d)
{
	F_TIME t;

	T_Localtime(T_Systime(), &t);
	d->day = t.tm_mday + 1;
	d->month = t.tm_mon + 1;
	d->year = t.tm_year;
}

/*** ---------------------------------------------------------------------- ***/

/* Berechnet die Anzahl der Schalttage, die bis zum Datum d seit
 * dem 1.1.1980 vergangen sind.
 */
LOCAL _LONG schaltTage(CONST Date *d)
{
	_WORD sd;

	/* Erst mal einfach durch 4 teilen und 1 dazu */
	sd = (d->year - 1980) / 4 + 1;
	/* Hier wird dann noch die Korrektur fuer das aktuelle Jahr gemacht */
	if ((d->month < 3) && (((d->year % 4) == 0) &&
		(((d->year % 100) != 0) || ((d->year % 400) == 0))))
	{
		--sd;
	}
	/* Ab dem Jahr 2100 wird die Funktion nicht mehr ganz stimmen, dann
	 * muesste noch ein Tag abgezogen werden. Dazu hatte ich aber keine
	 * Lust :-)
	 */
	return sd;
}

/*** ---------------------------------------------------------------------- ***/

/* Berechnet aus Datum die Anzahl der Tage seit dem 1.1.1980.
 * Muesste auch mit anderen Jahren funktionieren, Hauptsache, das Jahr
 * ist ein Schaltjahr. Fuer den Fall einfach die 1980 ueberall austauschen.
 */
LOCAL _LONG Date2Days(CONST Date *d)
{
	return (_LONG)(d->year - 1980L) * 365L + schaltTage(d) + mDays[d->month-1] + d->day - 1L;
}

/*** ---------------------------------------------------------------------- ***/

/* Hier wird aus einer Anzahl Tagen wieder ein Datum gemacht.
 * Das ist etwas untrivialer, als es zuerst scheint, hat mich am meisten
 * Zeit gekostet und ist auch fuerchterlich chaotisch, funktioniert aber.
 * Und beim nachtraeglichen Dokumentieren habe ich mich auch gewundert,
 * warum das ganze funktioniert (siehe Trick), aber es geht wirklich
 */
LOCAL _VOID CalcDate(_LONG days, Date *d)
{
	_LONG jahrGrob, monGrob, dayGrob, nDays, rDays;

	/* Erst mal Jahr und Monat grob bestimmen */
	jahrGrob = days / 365;
	rDays = days % 365 - (jahrGrob % 4);
	/* Das folgende ist eine Division durch 30.6, aber ich wollte die REAL-
	 * Konvertierung umgehen. 30.6 ist die Durchschnittszahl von Tagen/Monat.
	 */
	monGrob = (5 * rDays) / 153;
	do
	{
		rDays = days - (jahrGrob * 365) - (jahrGrob % 4);
		/* Tage berechnen, gehen von 1..31 */
		dayGrob = rDays - mDays[(_WORD)monGrob];
		if (dayGrob <= 0)
			--monGrob;
		/* Bei mehr als 29 Tagen erst mal einen Monat mehr nehmen, wird
		 * weiter unten wieder korrigiert
		 */
		if (dayGrob > 29)
		{
			++monGrob;
			dayGrob -= 29;
			/* Wenn > 11, dann haben wir schon naechstes Jahr */
			if (monGrob > 11)
			{
				++jahrGrob;
				monGrob -= 12;
			}
		}
		/* Wenn < 0, dann stimmt wohl das Jahr auch nicht, nehmen wir mal
		 * eines weniger
		 */
		if (monGrob < 0)
		{
			--jahrGrob;
			monGrob += 12;
		}
	} while (monGrob < 0 || dayGrob <= 0);
	/* So, jetzt haben wir ein ziemlich ungenaues Datum (vielleicht stimmt's
	 * ja auch!)
	 */
	d->day = (_WORD)dayGrob;
	d->month = (_WORD)monGrob + 1;
	d->year = (_WORD)jahrGrob + 1980;
	/* Jetzt kommt der grosse Trick! */
	for (;;)
	{
		/* Ich hatte es auch einmal probeweise in einen riesigen
		 * booleschen Ausdruck gepackt, aber diese Loesung ist von der
		 * Codegroesse her besser.
		 * Die Laufzeit ist sowieso unkritisch, schnell genug ist es immer.
		 */
		if (d->day > 30 && (d->month == 2 || d->month == 4 || d->month == 6 ||
			d->month == 9 || d->month == 11))
		{
			/* Mehr als 30 Tage und Monat passt halt nicht */
			++d->month;
			d->day -= 30;
		} else if (d->day >= 29 && d->month == 2)
		{
			/* 29 Tage im Februar? Mal sehen, haben wir ein Schaltjahr? */
			if (!(((d->year % 4) == 0) &&
				 (((d->year % 100) != 0) || ((d->year % 400) == 0))))
			{
				/* Nein, kein Schaltjahr, also geht auch 29 Tage im Februar nicht! */
				d->day -= 28;
				++d->month;
			} else
			{
				if (d->day > 29)
				{
					d->day += 29;
					++d->month;
				}
			}
		}
		/* Da isser: Ich rechne das errechnete Datum nochmal zurueck!
		 */
		nDays = Date2Days(d);
		/* Ist gleich, stimmt wohl */
		if (nDays == days)
		{
			break;
		} else if (nDays > days)
		{
			/* Stimmt nicht, nach unten korrigieren.
			 * Dabei natuerlich Monats- und Jahreswechsel beruecksichtigen
			 */
			nDays -= days;
			if (d->day <= nDays)
			{
				--d->month;
				if (d->month <= 0)
				{
					d->month += 12;
					--d->year;
				}
			} else
			{
				d->day -= nDays;
			}
		} else if (days > nDays)
		{
			/* Stimmt nicht, nach oben korrigieren.
			 * Dabei natuerlich Monats- und Jahreswechsel beruecksichtigen
			 */
			d->day += days - nDays;
			if (d->day > 31)
			{
				d->day -= 31;
				++d->month;
				if (d->month > 12)
				{
					d->month -= 12;
					++d->year;
				}
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Wandelt Datum in GEMDOS-Datum um
 */
GLOBAL _UWORD ConvertDate_PackDate(CONST Date *date)
{
	_WORD year;
	_UWORD d;

	year = date->year;
	if (year > 1980)
		year -= 1980;
	d = (((year << 4) | date->month) << 5) | date->day;
	return d;
}

/*** ---------------------------------------------------------------------- ***/

/* Wandelt GEMDOS-Datum in Datum um
 */
GLOBAL Date ConvertDate_UnpackDate(_UWORD date)
{
	Date d;

	d.day = date & 0x1f;
	date >>= 5;
	d.month = date & 0x0f;
	date >>= 4;
	d.year = (date & 0x7f) + 1980;
	return d;
}

/*** ---------------------------------------------------------------------- ***/

/* Wandelt Zeit in GEMDOS-Zeit um
 */
GLOBAL _UWORD ConvertDate_PackTime(CONST Time *time)
{
	_UWORD t;

	t = (((time->hour << 6) + time->minute) << 5) + (time->second >> 1);
	return t;
}

/*** ---------------------------------------------------------------------- ***/

/* Wandelt GEMDOS-Zeit in Zeit um
 */
GLOBAL Time ConvertDate_UnpackTime(_UWORD time)
{
	Time ti;

	ti.second = (time & 0x1f) << 1;
	time >>= 5;
	ti.minute = time & 0x3f;
	ti.hour = (time >> 6) & 0x1f;
	return ti;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UWORD ConvertDate_NumInDate(_ULONG num)
{
	Date d;
	
	CalcDate(num, &d);
	return ConvertDate_PackDate(&d);
}
/* war weiter unten auskomentiert	*/
LOCAL _BOOL LeapYear(_WORD year)
{
	return ((year % 4) == 0) && ((year % 100 != 0) || (year % 400 == 0));
}

/*DYJ 5.00a */
GLOBAL CATDATE ConvertDate_UTC(CATDATE cd,_UBYTE *zeitX)
{
	F_TIME t;
	_WORD val,mTage[]={31,28,31,30,31,30,31,31,30,31,30,31};
	ConvertDate_CatDate2Datim(cd,zeitX,&t);
	if(t.tm_tZone!= TZ_LOCALTIME)
	{
		t.tm_min-=t.tm_tZone;
		val=t.tm_min;
		if(val<0 || val>59)
		{
			t.tm_min%=60;
			val-=t.tm_min;
			t.tm_hour+=val/60;
			val=t.tm_hour;
			if(val<0 || val>23)
			{
				t.tm_hour%=24;
				val-=t.tm_hour;
				t.tm_mday+=val/24;
				if(LeapYear(t.tm_year))
					mTage[1]++;
				if(t.tm_mday<1)
				{
					t.tm_mon--;
					if(t.tm_mon<0)			/*Achtung tm_mon(0-11) und tm_mday(0-30)	*/
					{
						t.tm_mon=11;
						t.tm_year--;
					}
					t.tm_mday+=mTage[t.tm_mon];
				}
				else if(t.tm_mday>=mTage[t.tm_mon])
				{
					t.tm_mday-=mTage[t.tm_mon];
					t.tm_mon++;
					if(t.tm_mon==12)
					{
						t.tm_mon=0;
						t.tm_year++;
					}
				}
			}
		}
		cd=(t.tm_year-1990)*100000000l+(t.tm_mon+1)*1000000l
			+(t.tm_mday+1)*10000l+t.tm_hour*100+t.tm_min;
	}
	return cd;
}
/*** ---------------------------------------------------------------------- ***/

/* Wandelt ein CAT-Datum in ein Datum und eine Uhrzeit um
 */
/*DYJ 5.00a */
GLOBAL _VOID ConvertDate_CatDate2Datim(CATDATE cd,_UBYTE *zeitX, F_TIME *t)
{
	if (cd == 0)
	{
		T_SetZero(t);
	} else
	{
		t->tm_sec = 0;
		t->tm_min = (_WORD)(cd % 100);
		cd /= 100;
		t->tm_hour = (_WORD)(cd % 100);
		cd /= 100;
		t->tm_mday = (_WORD)(cd % 100) - 1;
		cd /= 100;
		t->tm_mon = (_WORD)(cd % 100) - 1;
		cd /= 100;
		t->tm_year = (_WORD)(cd % 100) + 1990;
		t->tm_tZone= TZ_LOCALTIME;
		if(zeitX && *zeitX)
		{
			if(sscanf(zeitX,"%d %d %d",&(t->tm_year),&(t->tm_sec),&(t->tm_tZone))==2)
				t->tm_tZone= TZ_LOCALTIME;
		}
		/*
		 * Wochentag und Tag des Jahres berechnen lassen
		 */
		T_Localtime(T_Mktime(t), t);
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID ConvertDate_GdDate2Datim(_UWORD date,_UWORD time, F_TIME *t)			*/
/* 	Wandelt ein Datum und Zeit im Gemdos Format in F_TIME-Frmat					*/
/*	Parameter:	date:	Datum im Gemdos-Format									*/
/*				time:	und die Zeit im GEMDOS-Format							*/
/*				t:		fr das Ergebnis										*/
/*	Ergebnis:	keines															*/		
/*	23.11.01 von Dimitri Junker	DYJ 4.30zf										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID ConvertDate_GdDate2Datim(_UWORD date,_UWORD time, F_TIME *t)
{
	t->tm_year = ((date >> 9) & 0x7f) + 1980;
	t->tm_mon  = ((date >> 5) & 0x0f) - 1;
	t->tm_mday = (date		 & 0x1f) - 1;

	t->tm_hour = (time >> 11) & 0x1f;
	t->tm_min  = (time >> 5) & 0x3f;
	t->tm_sec  = (time & 0x1f) * 2;

	t->tm_yday = 0;
	t->tm_wday = 0;

	/*
	 * Wochentag und Tag des Jahres berechnen lassen
	 */
	T_Localtime(T_Mktime(t), t);
}

/*** ---------------------------------------------------------------------- ***/

/* Eingabe: String mit relativ beliebigem Format.
 * Ausgabe: Datum im Maustausch-Format, Uhrzeit wird auf 0 Uhr gesetzt
 */
GLOBAL _VOID ConvertDate_StrToMTDate(CONST _UBYTE *str, _UBYTE *res, size_t strsize)
{
	Date d;
	_UWORD cD;
	
	/* Umwandeln in Datum */
	ConvertDate_TextToDate(str, &d);
	cD = ConvertDate_PackDate(&d);
	ConvertDate_MakeFreeDate(cD, 0, res, strsize);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _ULONG ConvertDate_TextDateToNum(CONST _UBYTE *date)
{
	Date d;

	/* Umwandeln in Datum */
	if (!ConvertDate_TextToDate(date, &d))
		return 0;
	return Date2Days(&d);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _ULONG ConvertDate_DateInNum(_UWORD date)
{
	Date d;
	
	d = ConvertDate_UnpackDate(date);
	return Date2Days(&d);
}

/*** ---------------------------------------------------------------------- ***/

/* Wandelt ein Datum in eine Zahl (Tage seit 1.1.1980) um
 */
GLOBAL _ULONG ConvertDate_DateToJulian(CONST Date *d)
{
	return Date2Days(d);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID GoodFriday(_WORD Year, _WORD *day, _WORD *month)
{
	_WORD Golden;          /* Year in Mentonic cycle */
	_WORD Easter;          /* Easter Sunday */
	_WORD GrCor;           /* Gregorian correction */
	_WORD ClCor;           /* Clavian correction */
	_WORD Epact;           /* age of calendar moon at start of Year */
	_WORD Century;         /* Century */
	_WORD Extra;           /* when Sunday occurs in March */
	_WORD Emonth;          /* Month of Easter Sunday (March/April) */

	Golden = (Year / 19) * (-19);
	Golden = Golden + Year+1;
	
	if (Year > 1582)
	{
		Century = (Year / 100) + 1;
		
		GrCor = ((Century * 3) / (-4)) + 12;
		
		ClCor = ((Century - 18) / (-25)) + ((Century - 16) / 3);
		
		Extra = (Year * 5 / 4) + GrCor - 10;
		
		Epact = (Golden * 11) + 20 + ClCor + GrCor;
		Epact = Epact + (Epact / 30) * (-30);
		
		if (Epact <= 0)
			Epact = Epact + 30;
		if ((Epact == 25 && Golden > 11) || Epact == 24)
			++Epact;
	} else
	{
		Extra = (Year * 5) / 4;
		Epact = (Golden * 11) - 4;
		Epact = Epact + (Epact / 30) * (-30);
		++Epact;
	}
	
	Easter = -Epact + 44;
	if (Easter < 21)
		Easter = Easter + 30;
	
	Extra = Extra + Easter;
	Extra = Extra + (Extra / 7) * (-7);
	
	Easter = Easter - Extra + 7;
	Emonth = 3;
	
	if (Easter > 31)
	{
		Easter = Easter - 31;
		++Emonth;
	}
	/* Easter/Emonth now contain date of Easter sunday */
	
	/* calculate good friday's date */
	Easter = Easter - 2;
	if (Easter < 0)
	{
		Easter = Easter + 31;
		++Emonth;
	}
	
	*day = Easter;
	*month = Emonth;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert den aktuellen Wochentag zurueck
 */
GLOBAL tWeekday ConvertDate_WeekDay(CONST Date *d)
{
	_LONG jul;

	jul = Date2Days(d);
	return (tWeekday)((jul + 2) % 7);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _LONG BuBtag(_WORD year)
{
	Date d;
	tWeekday weekday;

	/* Wochentag berechnen */
	d.year = year;
	d.day = 16;
	d.month = 11;
	weekday = ConvertDate_WeekDay(&d);
	
	d.year = year;
	d.month = 11;
	d.day = 26 - weekday;
	if (d.day > 22)
		d.day -= 7;
	
	return Date2Days(&d);
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert TRUE, wenn das Datum am Wochenende ist
 */
GLOBAL _BOOL ConvertDate_WeekEnd(CONST Date *d)
{
	switch (ConvertDate_WeekDay(d))
	{
		case Saturday: return TRUE;
		case Sunday: return TRUE;
		default: break;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt zurueck, ob das Datum ein deutscher Feiertag ist
 */
GLOBAL _BOOL ConvertDate_Holiday(CONST Date *d)
{
	typedef struct {
		_WORD day;
		_WORD month;
	} daymonth;
	LOCAL daymonth holidays[] = {
		{  1,  1 },  /* Neujahr */
		{  1,  5 },  /* 1. Mai */
		{  3, 10 },  /* Tag der dt. Einheit */
/* Kein Bundeseinheitlicher Feiertag */
		{ 24, 12 },  /* Heiligabend */
		{ 25, 12 },  /* 1. Weihnachtstag */
		{ 26, 12 },  /* 2. Weihnachtstag */
		{ 31, 12 },  /* Sylvester */
	};

	_WORD dy, mn, year;
	_WORD i;
	_LONG jul;
	_LONG good_friday;
	_LONG eastermonday;
	_LONG eastersunday;
	Date date;
	
	year = d->year;

	if (year <= 90)	
		year += 2000;
	else if (year <= 99)
		year += 1900;

	/* wann ist Ostern? */
	GoodFriday(year, &dy, &mn);
	date.day = dy;
	date.month = mn;
	date.year = year;
	good_friday = Date2Days(&date);
	eastersunday = good_friday + 2;
	eastermonday = good_friday + 3;
	
	for (i = 0; i < 8; i++)
	{
		if (d->day == holidays[i].day && d->month == holidays[i].month)
			return TRUE;
	}
	
	jul = Date2Days(d);
	
	if (jul == good_friday ||
		jul == eastermonday ||
		jul == eastersunday ||
		jul == eastersunday + 39 ||      /* Christi Himmelfahrt */
		jul == eastersunday + 50)      /* Pfingstmontag */
	{
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert das Datum vom Buss- und Bettag
 */
GLOBAL _VOID ConvertDate_BussBet(_WORD year, Date *d)
{
	_LONG bub;
	
	bub = BuBtag(year);
	CalcDate(bub, d);
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert das Datum von Karfreitag
 */
GLOBAL _VOID ConvertDate_KarFreitag(_WORD year, Date *d)
{
	_WORD dd, dm;
	
	GoodFriday(year, &dd, &dm);
	d->day = dd;
	d->month = dm;
	d->year = year;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert das Datum von Karfreitag als Zahl
 * Damit kann man einige andere variable Feiertage berechnen:
 *
 *  Ostersonntag == Karfreitag + 2
 *  Ostermontag  == Karfreitag + 3
 *  Christi Himmelfahrt  == Karfreitag + 41
 *  Pfingstmontag  == Karfreitag + 52
 */
GLOBAL _ULONG ConvertDate_KarFreitagNumber(_WORD year)
{
	Date d;

	ConvertDate_KarFreitag(year, &d);
	return Date2Days(&d);
}

/*** ---------------------------------------------------------------------- ***/

#define langHigh 2  /* hoechster Sprachindex f. 'mon' */

#define Repl(src, n, l, dst) \
	MemCpy(dst, src + n, l)

LOCAL _VOID set_time(_UWORD value, _BOOL zeroes, size_t *i, size_t *j, _UBYTE *d)
{
	_UBYTE ins[80];
	
	NumToStr(value, 10, 2, '0', ins, sizeof(ins));
	if (zeroes || ins[0] != '0')
	{
		d[*j] = ins[0];
		++(*j);
	}
	d[*j] = ins[1];
	++(*j);
	*i += 2;
}

/* %% Komplett neue PROCEDURE TimeToText. Wertet jetzt 'mask' komplett
 * aus. Jeder Wert wird nur einmal in das Ergebnis gesetzt.
 * Es wird kein Test auf ungueltige Werte in 'date' vorgenommen, sollten
 * aber auch nicht drin sein.
 */
GLOBAL _VOID ConvertDate_TimeToText(CONST F_TIME *date, CONST _UBYTE *mask, _UBYTE *d, size_t strsize)
{
	_BOOL english;
	size_t l;
	size_t i, j;
	_UBYTE ch;
	_BOOL up, hSet, mSet, sSet, eSet;
	CONST _UBYTE *p;
	
#define copy() d[j++] = mask[i++]

	if (strsize == 0)
		return;
	--strsize;

	if (mask == NULL || *mask == '\0')
		mask = StdTimeMask;

	l = strLen(mask);
	if (l > strsize)
	{
		/* stringoverflow */
		l = strsize;
	}
	
	/* Flags initialisieren */
	english = FALSE;
	hSet = FALSE;
	mSet = FALSE;
	sSet = FALSE;
	eSet = FALSE;

	/* Test auf englische Notierung: */
	p = mask;
	for (;;)
	{
		p = strChr(p, '#');
		if (p == NULL)
			break;
		++p;
		if (ucase(*p) == 'E')
		{
			english = TRUE;
			break;
		}
	}
	
	/* Maske scannen */
	j = 0;
	i = 0;
	while (i < l)
	{
		ch = mask[i];
		if (ch == '#' && ucase(mask[i+1]) == 'E' && !eSet)
		{
			up = mask[i+1] == 'E';
			if (date->tm_hour > 12)
				d[j] = 'p';
			else
				d[j] = 'a';
			if (up)
				d[j] = ucase(d[j]);
			++j;
			d[j] = up ? 'M' : 'm';
			++j;
			i += 2;
			eSet = TRUE;
		} else if (mask[i+1] == ch)
		{
			up = ucase(ch) == ch;
			if (ucase(ch) == 'H' && !hSet)
			{
				if (english && date->tm_hour > 12)
					set_time(date->tm_hour - 12, up, &i, &j, d);
				else
					set_time(date->tm_hour, up, &i, &j, d);
				hSet = TRUE;
			} else if (ucase(ch) == 'M' && !mSet)
			{
				set_time(date->tm_min, up, &i, &j, d);
				mSet = TRUE;
			} else if (ucase(ch) == 'S' && !sSet)
			{
				set_time(date->tm_sec, up, &i, &j, d);
				sSet = TRUE;
			} else
			{
				copy();
			}
		} else
		{
			copy();
		}
	}
	d[j] = '\0';
#undef copy
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
/* lang: -1: User-defined, 1: Deutsch, 2:Frz., 0:Engl */
LOCAL CONST _UBYTE *mon(_WORD lang, _UWORD month)
{
	LOCAL CONST _UBYTE ger[12][10] = {
		"Januar",
		"Februar",
		"M\204rz",
		"April",
		"Mai",
		"Juni",
		"Juli",
		"August",
		"September",
		"Oktober",
		"November",
		"Dezember",
	};
	LOCAL CONST _UBYTE frz[12][10] = {
		"Janvier",
		"Fvrier",
		"Mars",
		"Avril",
		"Mai",
		"Juin",
		"Juillet",
		"Aot",
		"Septembre",
		"Octobre",
		"Novembre",
		"Dcembre",
	};
	LOCAL CONST _UBYTE eng[12][10] = {
		"January",
		"February",
		"March",
		"April",
		"May",
		"June",
		"July",
		"August",
		"September",
		"October",
		"November",
		"December",
	};
	
	month--;
	switch (lang)
	{
		case L_ENGLISH: return eng[month];
		case L_FRENCH: return frz[month];
	}
	return ger[month];
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL set_date(_UBYTE ch, _UWORD v, _UBYTE *d)
{
	size_t l, n;
	_UBYTE *p;
	_UBYTE s2[80];
	_UBYTE c1, c2;
	_UWORD t;

	if ((p = strChr(d, ch)) != NULL)
	{
		n = 1;
		while (p[n] == ch)
			n++;
		if (p[n] == '#' && d[n+1] == 'E')
		{
			t = v % 10;
			if (t > 0 && t < 4 && (v < 10 || v > 20))
			{
				if (t == 1)
				{
					c1 = 's';
					c2 = 't';
				} else if (t == 2)
				{
					c1 = 'n';
					c2 = 'd';
				} else if (t == 3)
				{
					c1 = 'r';
					c2 = 'd';
				} else
				{
					c1 ='t';
					c2 ='h';
				}
			} else
			{
				c1 = 't';
				c2 = 'h';
			}
			p[n] = c1;
			p[n+1] = c2;
		}
		CardToStr(v, 0, s2, sizeof(s2));
		l = strLen(s2);
		if (l >= n)
		{
			/* Feld wird vollst. gefuellt; die Zahl rechtsbuendig kopieren: */
			MemCpy(p, s2 + l - n, n);
		} else
		{
			if (ch >= 'a')
			{
				/* Feld ist zu gross; kuerzen: */
				strCpy(p, p + n - l);
				MemCpy(p, s2, l);
			} else
			{
				/* Feld ist zu gross; mit Nullen fuellen: */
				while (l != n)
				{
					*p++ = '0';
					--n;
				}
				MemCpy(p, s2, l);
			}
		}
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID monset(_UBYTE ch, _WORD lang, _UWORD month, _UBYTE *d)
{
	size_t l, n;
	_UBYTE *p;
	CONST _UBYTE *s2;

	if ((p = strChr(d, ch)) != NULL)
	{
		n = 1;
		while (p[n] == ch)
			n++;
		s2 = mon(lang, month);
		l = strLen(s2);
		if (l >= n)
		{
			/* Feld wird vollst. gefuellt; den String linksbuendig kopieren: */
			l = n;
		} else
		{
			/* Feld ist zu gross; kuerzen: */
			strCpy(p, p + n - l);
		}
		MemCpy(p, s2, l);
	}
}

/*** ---------------------------------------------------------------------- ***/
	
GLOBAL _VOID ConvertDate_DateToText(CONST F_TIME *s, CONST _UBYTE *mask, _UBYTE *d, size_t strsize)
{
	size_t l;
	
	if (strsize == 0)
		return;

	if (mask == NULL || *mask == '\0')
		mask = StdDateMask;
	
	l = strLen(mask) + 1;
	if (l > strsize)
	{
		/* stringoverflow */
		l = strsize;
	}
	strMcpy(d, l, mask);
	if (!set_date('D', s->tm_mday + 1, d))
		set_date('d', s->tm_mday + 1, d);
	if (!set_date('M', s->tm_mon + 1, d))
		set_date('m', s->tm_mon + 1, d);
	if (!set_date('Y', s->tm_year, d))
		set_date('y', s->tm_year, d);
	monset('U', L_USER, s->tm_mon + 1, d);
	monset('G', L_GERMAN, s->tm_mon + 1, d);
	monset('F', L_FRENCH, s->tm_mon + 1, d);
	monset('E', L_ENGLISH, s->tm_mon + 1, d);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL CONST _UBYTE *daystr(_WORD lang, _WORD wday)
{
	LOCAL CONST _UBYTE ger[7][11] = {
		"Sonntag",
		"Montag",
		"Dienstag",
		"Mittwoch",
		"Donnerstag",
		"Freitag",
		"Samstag",
	};
	LOCAL CONST _UBYTE frz[7][10] = {
		"Dimanche",
		"Lundi",
		"Mardi",
		"Mercredi",
		"Jeudi",
		"Vrendredi",
		"Samedi",
	};
	LOCAL CONST _UBYTE eng[7][10] = {
		"Sunday",
		"Monday",
		"Tuesday",
		"Wednesday",
		"Thursday",
		"Friday",
		"Saturday",
	};
	
	switch (lang)
	{
		case L_ENGLISH: return eng[wday];
		case L_FRENCH: return frz[wday];
	}
	return ger[wday];
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL dayset(_UBYTE ch, _WORD lang, _WORD wday, _UBYTE *d)
{
	size_t l, n;
	_UBYTE *p;
	CONST _UBYTE *s2;

	if ((p = strChr(d, ch)) != NULL)
	{
		n = 1;
		while (p[n] == ch)
			n++;
		s2 = daystr(lang, wday);
		l = strLen(s2);
		if (l >= n)
		{
			/* Feld wird vollst. gefuellt; den String linksbuendig kopieren: */
			l = n;
		} else
		{
			/* Feld ist zu gross; kuerzen: */
			strCpy(p, p + n - l);
		}
		MemCpy(p, s2, l);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ConvertDate_DayToText(CONST F_TIME *date, CONST _UBYTE *mask, _UBYTE *dest, size_t strsize)
{
	size_t l;
	F_TIME t;
	
	if (strsize == 0)
		return;
	if (mask == NULL || *mask == '\0')
		mask = "GGGGGGGGGGG";
	l = strLen(mask) + 1;
	if (l > strsize)
	{
		/* stringoverflow */
		l = strsize;
	}
	strMcpy(dest, l, mask);
	strUpr(dest);
	t = *date;
	T_Localtime(T_Mktime(&t), &t);
	if (!dayset('G', L_GERMAN, t.tm_wday, dest))
		if (!dayset('F', L_FRENCH, t.tm_wday, dest))
			dayset('E', L_ENGLISH, t.tm_wday, dest);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* FuzzyTime */

typedef struct {
	CONST _UBYTE *lang_min;
	CONST _UBYTE *lang_halb;
	_WORD offset;
} MINUTES;

LOCAL CONST MINUTES minutenDeutsch[12] = {
		{ "", "", 0 },
		{ "F\201nf nach", "", 0 },
		{ "Zehn nach", "", 0 },
		{ "Viertel nach", "", 0 },
		{ "Zwanzig nach", "", 0 },
		{ "F\201nf vor", "halb ", 1 },
		{ "", "Halb ", 1 },
		{ "F\201nf nach", "halb ", 1 },
		{ "Zwanzig vor", "", 1 },
		{ "Viertel vor", "", 1 },
		{ "Zehn vor", "", 1 },
		{ "F\201nf vor", "", 1 }
};
LOCAL CONST MINUTES minutenEnglish[12] = {
		{ "", "", 0 },
		{ "five past", "", 0 },
		{ "ten past", "", 0 },
		{ "a quarter", "past ", 0 },
		{ "twenty past", "", 0 },
		{ "twenty-five", "past ", 0 },
		{ "half past", "", 0 },
		{ "twenty-five", "to ", 1 },
		{ "twenty to", "", 1 },
		{ "a quarter", "to ", 1 },
		{ "ten to", "", 1 },
		{ "five to", "", 1 }
};

LOCAL CONST _UBYTE *CONST stundenDeutsch[12] = {
	"Zw\224lf", "Eins", "Zwei", "Drei", "Vier", "F\201nf",
	"Sechs", "Sieben", "Acht", "Neun", "Zehn", "Elf"
};
LOCAL CONST _UBYTE *CONST stundenEnglish[12] = {
	"twelve", "one", "two", "three", "four", "five",
	"six", "seven", "eight", "nine", "ten", "eleven"
};

/*** ---------------------------------------------------------------------- ***/

/*************************************************************************
**
*A  fuzzytime ( ftime ) . . . . . . . . . . . . . . . . .  FuzzyTime lesen
*/

GLOBAL _VOID ConvertDate_FuzzyTime(CONST F_TIME *time, CONST _UBYTE *mask, _UBYTE *ftime, size_t strsize)
{
	_UBYTE tmp[80];
	CONST MINUTES *minuten;
	CONST _UBYTE *CONST *stunden;
	CONST _UBYTE *eins;
	CONST _UBYTE *uhr;
	_WORD std, min, lang;
	
	if (mask == NULL)
		mask = "G";
	switch (ucase(*mask))
	{
		case 'U': lang = L_USER; break;
		case 'E': lang = L_ENGLISH; break;
		case 'F': lang = L_FRENCH; break;
		default:
		case 'G': lang = L_GERMAN; break;
	}
	switch (lang)
	{
	default:
	case L_GERMAN:
		minuten = minutenDeutsch;
		stunden = stundenDeutsch;
		eins = "Ein";
		uhr = " Uhr";
		break;
	case L_ENGLISH:
		minuten = minutenEnglish;
		stunden = stundenEnglish;
		eins = "one";
		uhr = " O'Clock";
		break;
	}
	switch (time->tm_min)
	{
		case 2:  min = 1; break;
		case 28: min = 5; break;
		case 32: min = 7; break;
		case 58: min = 11; break;
		default: min = ((time->tm_min + 2) % 60) / 5; break;
	}
	std = time->tm_hour;
	if (time->tm_min > 57)
		std += 1;
	else
		std += minuten[min].offset;
	std = std % 12;
	
	if (min == 0)
	{
		if (std == 1)
			strBcpy(tmp, eins);
		else
			strBcpy(tmp, stunden[std]);
		strBcat(tmp, uhr);
		if (lang == L_GERMAN)
			tmp[0] = ucase(tmp[0]);
	} else
	{
		strBcpy(tmp, minuten[min].lang_halb);
		strBcat(tmp, stunden[std]);
	}
	strMcpy(ftime, strsize, minuten[min].lang_min);
	if (lang == L_GERMAN)
	{
		switch (time->tm_min)
		{
		case 27: case 28:
		case 57: case 58:
			strMcpy(ftime, strsize, "Kurz vor");
			break;
		case 2: case 3:
		case 32: case 33:
			strMcpy(ftime, strsize, "Kurz nach");
			break;
		}
	}
	if (*ftime != '\0')
		strMcat(ftime, strsize, " ");
	strMcat(ftime, strsize, tmp);
}


/*** ---------------------------------------------------------------------- ***/

/* Die Routine liefert fuer alle Daten, die innerhalb der letzten Woche liegen,
 * entweder den Wochentag oder heute/gestern/vorgestern zurueck.
 * Zusaetzlich wird auch noch festgestellt, ob es morgens, mittags, nachmittags,
 * abends oder nachts ist.
 */
/*DYJ 5.21b */
GLOBAL _VOID ConvertDate_FuzzyDate(CONST F_TIME *dt, CONST _UBYTE *mask, _UBYTE *str, size_t strsize)
{
	SYSTIME dateLong;
	F_TIME date;
	_WORD dateDiff;
	SYSTIME today;
	_WORD wd;
	_BOOL appendS;
	
	UNUSED(mask);
	appendS = FALSE;
	/* Wochentag bestimmen */
	date = *dt;
	dateLong = T_Mktime(&date);
	T_Localtime(dateLong, &date);
	wd = date.tm_wday;
	today = T_Systime();
	dateLong /= SECS_PER_DAY;
	today /= SECS_PER_DAY;
	if (dateLong > today)
	{
		/* In der Zukunft, das mag die Routine nicht so */
		strMcpy(str, strsize, Rsc_WrkString(FUZY_FUT));
	} else
	{
		dateDiff = (_WORD)(today - dateLong);
		switch (dateDiff)
		{
		case 0:
			strMcpy(str, strsize, Rsc_WrkString(FUZY_TOD));
			break;
		case 1:
			strMcpy(str, strsize, Rsc_WrkString(FUZY_YEST));
			break;
		case 2:
			strMcpy(str, strsize, Rsc_WrkString(FUZY_BEFY));
			break;
		default:
			if (dateDiff <= 7)
			{
				strMcpy(str, strsize, Rsc_WrkString(FUZY_TOD));
				strMcat(str, strsize, daystr(Rsc_lang(), wd));
			} else
			{
				strMcpy(str, strsize, Rsc_WrkString(FUZY_DARK));
				appendS = TRUE;
			}
			break;
		}
	}
	/* Nun noch den Tageszusatz betimmen */
	if (date.tm_hour >= 7 && date.tm_hour < 12)
	{
		/* morgens */
		strMcat(str, strsize, Rsc_WrkString(FUZ_MORN));
	} else if (date.tm_hour >= 12 && date.tm_hour < 14)
	{
		/* mittags */
		strMcat(str, strsize, Rsc_WrkString(FUZ_NOON));
	} else if (date.tm_hour >= 14 && date.tm_hour < 18)
	{
		/* nachmittags */
		strMcat(str, strsize, Rsc_WrkString(FUZ_ANOO));
	} else if (date.tm_hour >= 18 && date.tm_hour < 23)
	{
		/* abends */
		strMcat(str, strsize, Rsc_WrkString(FUZ_EVE));
	} else if (date.tm_hour >= 23 || date.tm_hour < 7)
	{
		/* nachts */
		strMcat(str, strsize, Rsc_WrkString(FUZ_NIG));
	}
	if (appendS)
		strMcat(str, strsize, "s");
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Erzeugt die Datumszeile */
GLOBAL _VOID ConvertDate_MakeFreeDate(_UWORD cDt, _UWORD cTi, _UBYTE *str, size_t strsize)
{
	_UBYTE tmp[12];
	F_TIME t;
	
	T_Localtime(T_DosToSystime(cDt, cTi), &t);
	ConvertDate_DateToText(&t, "YYYYMMDD", str, strsize);
	ConvertDate_TimeToText(&t, "HHMMSS", tmp, sizeof(tmp));
	strMcat(str, strsize, tmp);
}
/*** ------------------------------------------------------------------------ ***/
/*	_WORD startWeekOfMonth(_WORD monat,_WORD jahr,_WORD woche)					*/
/*	Gibt den Start der n.Woche eines Monats zurck.								*/
/*	Parameter:	monat:	Monat													*/
/*				jahr:	wegen Schaltjahr										*/
/*				woche:	1-4 oder 5 fr letzte Woche								*/
/*	Ergebnis:	1. Tag der Woche												*/		
/*	23.11.01 von Dimitri Junker	DYJ 4.30zf										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _WORD startWeekOfMonth(_WORD monat,_WORD jahr,_WORD woche)
{
	_WORD mTage[]={0,31,28,31,30,31,30,31,31,30,31,30,31};
	_WORD ret;
	if(woche==5)
	{
		if(LeapYear(jahr))
			mTage[1]++;
		ret=mTage[monat]-6;
	}
	else
		ret=7*woche-6;
	return ret;
	
}
/* Doku zur Environmentvariable TZ aus date.1
     TZ        Angaben ber die verwendete Zeitzone (wird nur zur 
               Ermittlung des Zeitzonennamens bentigt und hat nur einen 
               Teil der von POSIX.1 definierten Funktionalitt). Bei-
               spiel:

               MEZ-1MSZ,M3.5.0,M10.5.0

               MEZ       Standardname der Zeitzone
               -1        Abstand zur UCT (Greenwich Meantime)
               MSZ       Name der Zeitzone bei Sommerzeit
               M3.5.0    Sommerzeit beginnt am 0. Tag (Sonntag), der 
                         letzten Woche von Monat 3.
               M10.5.0   Sommerzeit endet am 0. Tag (Sonntag) der letz-
                         ten Woche von Monat 10.
DYJ: ich habe es erweitert -> der Abstand zur UTC kann auch die Form h.m haben,
	falls es keine ganzen Stunden sind
*/
/*** ------------------------------------------------------------------------ ***/
/*	void set_LocalTZ(F_TIME *t)													*/
/*	Setzt in der bergebenen Zeitstruktur die Zeitzohne auf die zur jeweiligen	*/
/*	Zeit lokal gltigen Zeitzone												*/
/*	Parameter:	t:	die Zeitstruktur											*/
/*	Ergebnis:	nur innerhalb von t												*/		
/*	23.11.01 von Dimitri Junker	DYJ 4.30zf										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.20w */
LOCAL void set_LocalTZ(F_TIME *t)
{
	static _WORD dMin=60,stMon=3,stWo=5,stTag=0,endMo=10,endWo=5,endTag=0,first=1;
	_UBYTE *env;
	_WORD tag;
	if(first)
	{
		first=0;
		if ((env = getenv("TZ")) != NULL)
		{
			env+=strcSpn(env,"+-0123456789");	/*Zeitzone: Stundendiff*/
			dMin=atoi(env)*(-60);
			env+=strcSpn(env,".,");
			if(*env=='.')		/*Zeitzone: Minutendiff*/
			{
				if(dMin<0)
					dMin-=atoi(env+1);
				else
					dMin+=atoi(env+1);
				env=strChr(env,',');
			}
			if(env && *env)
			{
				env+=2;
				sscanf(env,"%d.%d.%d,M%d%d%d",&stMon,&stWo,&stTag,&endMo,&endWo,&endTag);
			}
			else
				stMon=0;
		}
	}
			
	t->tm_tZone=dMin;
	if(stMon)
	{
		if((t->tm_mon+1)==stMon)		/* Im Startmonat der Sommerzeit?*/
		{
			tag=t->tm_mday-t->tm_wday+stTag;
			if(tag>t->tm_mday)
				tag-=7;
			if(tag>=startWeekOfMonth(stMon,t->tm_year,stWo))
				t->tm_tZone+=60;
		}
		else if((t->tm_mon+1)==endMo)	/* Im Endmonat der Sommerzeit?*/
		{
			tag=t->tm_mday-t->tm_wday+endTag;
			if(tag<t->tm_mday)
				tag+=7;
			if(tag<startWeekOfMonth(endMo,t->tm_year,endWo))
				t->tm_tZone+=60;
		}
		else if(stMon<endMo)	/* Nordhalbkugel	*/
		{
			if((t->tm_mon+1)>stMon && (t->tm_mon+1)<endMo)
				t->tm_tZone+=60;
		}
		else									/* Sdhalbkugel	*/
		{
			if((t->tm_mon+1)>stMon || (t->tm_mon+1)<endMo)
				t->tm_tZone+=60;
		}
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID ConvertDate_MakeInDate(_UWORD cDt, _UWORD cTi, _UBYTE *str, size_t strsize)	*/
/*	Erzeugt einen Datumsstring im Format fr NNTP-Header						*/
/*	Parameter:	cDt:	Datum													*/
/*				cTi:	Zeit													*/
/*				str:	Da hinein soll es										*/
/*				strsize:Lnge von str											*/
/*	Ergebnis:	keines															*/		
/*	25.11.01 von Dimitri Junker	DYJ 4.30zf										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID ConvertDate_MakeInDate(_UWORD cDt, _UWORD cTi, _UBYTE *str, size_t strsize)
{
	F_TIME t;
	
	ConvertDate_GdDate2Datim(cDt,cTi,&t);
	set_LocalTZ(&t);
	T_Strftime(str,strsize,"%0a, %d %0b %Y %H:%M:%S %z",&t);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ConvertDate_MakeDate(_UBYTE *str, size_t strsize)
{
	ConvertDate_MakeFreeDate(Tgetdate(), Tgettime(), str, strsize);
}

/*** ---------------------------------------------------------------------- ***/

/* skip: nicht-numerische Zeichen im String s werden uebersprungen.
 * p gibt dann die Position des naechsten numerischen Zeichens im String an
 */
#define is_digit(c) ((c) >= '0' && (c) <= '9')

LOCAL size_t skip(CONST _UBYTE *s, size_t p)
{
	while (s[p] != '\0' && !is_digit(s[p]))
		p++;
	return p;
}

/*** ---------------------------------------------------------------------- ***/

/* get: liest aus dem String s eine Zahl aus und ueberprueft, ob diese
 *      im uebergebenen Wertebereich liegt. Wenn ja, wird valid auf TRUE
 *      gesetzt. Wenn nein, ist valid FALSE und als Ergebnis wird die
 *      untere Grenze 'lo' zurueckgegeben.
 */
LOCAL _WORD get(CONST _UBYTE *s, _WORD lo, _WORD hi, size_t *pos, _BOOL *valid)
{
	_WORD v;

	v = StrToInt(s, pos, valid);
	*pos = skip(s, *pos);
	if (v < lo || v > hi)
		*valid = FALSE;
	if (!*valid)
		v = lo;
	return v;
}

/*** ---------------------------------------------------------------------- ***/

/* getd: Holt mittels get den Tag des Datums aus dem String */
LOCAL _VOID getd(CONST _UBYTE *s, size_t *pos, _BOOL *valid, Date *d)
{
	if (*valid)
		d->day = get(s, 1, 31, pos, valid);
}

/*** ---------------------------------------------------------------------- ***/

/* getm: Holt mittels get den Monat des Datums aus dem String */
LOCAL _VOID getm(CONST _UBYTE *s, size_t *pos, _BOOL *valid, Date *d)
{
	if (*valid)
		d->month = get(s, 1, 12, pos, valid);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gety(CONST _UBYTE *s, size_t *pos, _BOOL *valid, Date *d)
{
	if (*valid)
	{
		d->year = get(s, -9999, 9999, pos, valid);
		if (*valid)
		{
			if (d->year <= 90)		
				d->year += 2000;
			else if (d->year <= 99)
				d->year += 1900;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gethour(CONST _UBYTE *s, size_t *pos, _BOOL *valid, Time *t)
{
	if (*valid)
		t->hour = get(s, 0, 23, pos, valid);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID getmin(CONST _UBYTE *s, size_t *pos, _BOOL *valid, Time *t)
{
	if (*valid)
		t->minute = get(s, 0, 59, pos, valid);
}

/*** ---------------------------------------------------------------------- ***/

/* Die eigentliche Hauptfunktion: Sie wandelt sowohl ins Datum und
 * ueberprueft auch, ob das Datum gueltig ist. Die wird auch von den
 * Funktionen DateOK und StrToDate aufgerufen.
 */
GLOBAL _BOOL ConvertDate_TextToDate(CONST _UBYTE *s, Date *d)
{
	size_t p;
	_WORD i;
	Date today;
	_LONG jul;
	_BOOL valid;

	/* init */
	p = 0;
	valid = TRUE;
	GetCurrentDate(&today);

	/* convert: Erst mal sehen, ob es nur eine Integerzahl ist */
	while (*s == ' ')
		s++;
	i = StrToInt(s, &p, &valid);
	if ((i <= 0 && valid) ||
		(i > 0 && valid && s[0] == '+'))
	{
		/* Ist negativ,        */
		/* Datum zurueckrechnen */
		jul = Date2Days(&today);
		jul = jul + (_LONG)(i);   /* da i negativ ist, + */
		/* es wird bis maximal 1.1.1980 zurueckgerechnet,
		 * weiter geht nicht, fuehrt zu Loop!
		 */
		if (jul < 0)
			return FALSE;
		CalcDate(jul, d);
	} else
	{
		/* Ist keine Integerzahl oder positiv, also nehmen wir ein normales
		 * Datum an.
		 */
		p = 0;
		getd(s, &p, &valid, d);
		/* Nur wenn Tag nicht gueltig, schlaegt alles fehl!!*/
		if (valid)
		{
			getm(s, &p, &valid, d);
			if (valid)
			{
				gety(s, &p, &valid, d);
				if (!valid)
					d->year = today.year;
			} else
			{
				d->year = today.year;
				d->month = today.month;
			}
			valid = TRUE;
		}
	}

	/* Numerisch gueltig, jetzt logische Pruefung */
	if (valid)
	{
		/* Ich hatte es auch einmal probeweise in einen riesigen
		 * booleschen Ausdruck gepackt, aber diese Loesung ist von der
		 * Codegroesse her besser.
		 * Die Laufzeit ist sowieso unkritisch, schnell genug ist es immer.
		 */
		if ((d->day > 30) && (d->month == 2 || d->month == 4 || d->month == 6 ||
			d->month == 9 || d->month == 11))
		{
			/* Mehr als 30 Tage und Monat passt halt nicht */
			valid = FALSE;
		} else if (d->day > 29 && d->month == 2)
		{
			/* Mehr als 29 Tage im Februar geht nun wirklich nicht */
			valid = FALSE;
		} else if (d->day == 29 && d->month == 2)
		{
			/* 29 Tage im Februar? Mal sehen, haben wir ein Schaltjahr? */
			if (!(((d->year % 4) == 0) &&
				(((d->year % 100) != 0) || ((d->year % 400) == 0))))
			{
				/* Nein, kein Schaltjahr, also geht auch 29 Tage im Februar nicht! */
				valid = FALSE;
			}
		}
	}
	/* Jetzt noch pruefen, ob Jahr < 1990 */
	if (d->year < 1990)
	{
		valid = FALSE;
	}
	/* Das wars! */
	return valid;
}

/*** ---------------------------------------------------------------------- ***/

/* DateOk prueft nach, ob ein Datum in einem String ein gueltiges Datum ist.
 * Wenn nicht, wird FALSE zurueckgegeben, andernfalls ...
 */
GLOBAL _BOOL ConvertDate_DateOk(CONST _UBYTE *str)
{
	Date d;

	/* TextToDate macht das schon :-) */
	return ConvertDate_TextToDate(str, &d);
}

/*** ---------------------------------------------------------------------- ***/

/* Eingabe: String mit relativ beliebigem Format.
 * Ausgabe: LONGCARD mit folgenden Format:
 *          jjmmddhhmm, Jahr minus 1990.
 *          0000  immer, da Zeit nicht eingegeben wird.
 * Anmerkung: Diese Funktion gibt nur bis 2032 das korrekte Datum
 *            zurueck, da danach ein Ueberlauf des 32-Bit Wertes
 *            auftritt. Spaetestens dann ist diese Funktion bzw.
 *            das Datumsformat zu ueberarbeiten :-)
 */
GLOBAL CATDATE ConvertDate_StrToDate(CONST _UBYTE *str)
{
	Date d;

	/* Umwandeln in Datum */
	ConvertDate_TextToDate(str, &d);
	/* Vom Jahr 1990 abziehen (bloeder Kommentar, nicht) */
	d.year -= 1990;
	/* Frueher als 1990 geht nicht! */
	if (d.year < 0)
		d.year = 0;
	/* und jetzt noch die Riesenzahl draus machen
	 * Hier koennte man noch Zeit und Datum als Random-Zahlen einstreuen :-)
	 */
	return (((((_LONG)d.year * 100L) + d.month) * 100L) + d.day) * 10000L;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL CATDATE ConvertDate_CurrentDate(_VOID)
{
	Date d;

	GetCurrentDate(&d);
	d.year -= 1990;
	/* Frueher als 1990 geht nicht! */
	if (d.year < 0)
		d.year = 0;
	return (((((_LONG)d.year * 100L) + d.month) * 100L) + d.day) * 10000L;
}

/*** ---------------------------------------------------------------------- ***/

/* Prueft einen String im MausTausch-Format und konvertiert ihn in ein Datum (MH) */
GLOBAL _BOOL ConvertDate_MTTextToDate(CONST _UBYTE *s, Date *d)
{
	/* MT-Format: YYYYMMDDHHMM */
	_UBYTE temp[5];
	size_t pos;
	_BOOL valid;
	
	if (strLen(s) != 12)
		return FALSE;
	
	valid = TRUE;
	pos = 0;
	strMcpy(temp, 5, s);
	gety(temp, &pos, &valid, d);
	if (valid)
	{
		pos = 0;
		strMcpy(temp, 3, s + 4);
		getm(temp, &pos, &valid, d);
		if (valid)
		{
			pos = 0;
			strMcpy(temp, 3, s + 6);
			getd(temp, &pos, &valid, d);
		}
	}
	return valid;
}

/*** ---------------------------------------------------------------------- ***/

/* Prueft einen String im MausTausch-Format und konvertiert ihn in eine Zeit (MH) */
GLOBAL _BOOL ConvertDate_MTTextToTime(CONST _UBYTE *s, Time *t)
{
	/* MT-Format: YYYYMMDDHHMM */
	_UBYTE temp[3];
	size_t pos;
	_BOOL valid;
	
	if (strLen(s) != 12)
		return FALSE;
	
	valid = TRUE;
	pos = 0;
	strMcpy(temp, 3, s + 8);
	gethour(temp, &pos, &valid, t);
	if (valid)
	{
		pos = 0;
		strMcpy(temp, 3, s + 10);
		getmin(temp, &pos, &valid, t);
		t->second = 0;
	}
	return valid;
}

/*** ---------------------------------------------------------------------- ***/

#ifdef MAIN

/* Nur fuer Test: */

int main(void)
{
	_UBYTE str[80];
	Date d;
	
	GetCurrentDate(&d);
	printf("aktuelles Datum: %d.%d.%d\n", d.day, d.month, d.year);
	for (;;)
	{
		printf("Bitte geben Sie ein Datum ein: ");
		fflush(stdout);
		gets(str);
		if (str[0] == '\0')
			break;
		ConvertDate_TextToDate(str, &d);
		printf("\n");;
		printf("Tag  : %d\n", d.day);
		printf("Monat: %d\n", d.month);
		printf("Jahr : %d\n", d.year);
		printf("\n");;
		printf("CAT-Format : >%10lu<\n", ConvertDate_StrToDate(str));
		printf("Feiertag: %s\n", ConvertDate_Holiday(&d) ? "Ja!" : "Nein");
		printf("Wochentag: %s\n", daystr(L_GERMAN, ConvertDate_WeekDay(&d)));
		ConvertDate_KarFreitag(d.year, &d);
		printf("Karfreitag ist dieses Jahr: %d.%d.%d\n", d.day, d.month, d.year);
		ConvertDate_BussBet(d.year, &d);
		printf("Bu\236- und Bettag ist dieses Jahr: %d.%d.%d\n", d.day, d.month, d.year);
	}
	return 0;
}
#endif
