/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


/* 29.03.98 Markus Hoffmann @ DU (MH)
 * - Es wird an den Parser herausgereicht, ob die gerade angehaengte Mitteilung ein
 *   Kommentar auf eigene Msg ist.
 *
 * 15.03.00 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 4.13d'
 *	CAT weigerte sich eine schreibgeschtzte Database zu lesen, hauptschlich
 *	wegen des Lockfiletests. Schlgt dieser Test fehl prfe ich jetzt
 *	ob das Dir schreibgeschtzt ist,...
 * 1.4.00 bei der vorigen nderung gab es einen Fehler der erst in der 4.14 
 *	behoben wurde, dieser Teil ist al 4.13d2 gekennzeichnet
 *
 * 23.03.00 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 4.14b'
 *	Teilweise wurden Flags an Nachrichten vererbt, obwohl sie garkeine Komentare auf
 *	die andere Message waren. Die vorgenommene nderung geht auf eine Beschreibung
 *	von Dirk Steins zurck
 * 15.04.00 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 4.14n'
 *	Trotz 'gefilterte ignorieren' wurden direkt nach einem automatischen Gruppenwechsel
 *	eine gefilterte gezeigt. Dies ist jetzt behoben.
 */

#define __CAT_GROUP_IMPLEMENTATION__

#include <data.h>
#include <catfiles.h>
#include <mtpaths.h>
#include <zcalccrc.h>
#include <cattypes.h>
#include <catlog.h>
#include <mte.h>
#include <hashing2.h>
#include <searchhe.h>
#include <find2.h>
#include <catgloba.h>
#include <quicksor.h>
#include <confvars.h>
#include <varname1.h>
#include <varname3.h>
#include <groupsel.h>
#include <convertd.h>
#include <wildcard.h>
#include <protokol.h>
#include <assfuncs.h>
#include <vdiutil.h>
#include <groupcom.h>
#include <userinfo.h>

#include <mtappl.h>
#include <mtalerts.h>
#include <mttextfi.h>
#include <mtutils.h>

#include <s_endian.h>
#include <routine.h>
#include <file_io.h>
#include <dir_io.h>
#include <strconv.h>
#include <debug.h>

#ifndef __TOS_H__
#include <tos.h>
#endif


#define LF 0x0a


#define editSecureBytes 15 /* Wieviele Bytes sollen fuer den Editor beim Text zur Sicherheit */
                           /* zusaetzlich angefordert werden? */

#define fillBytePersonal 0xCC /* Fuellbyte fuer das Alignen der PersonalInfo */


#define rawDate        "xx.xx.xx xx:xx"

#define tabWild   "*" tabTag
#define parWild   "*" parTag


#define minDatBuffer (32768UL)	/* Erst ab 32k puffern */
#define maxDatBuffer (65536UL)	/* bis zu 64k puffern */

struct _oneGroupHandle {
	GROUPNR group;
	MSGNR anz;				/* Anzahl der Nachrichten in dieser Gruppe */
	FILENO tabHandle;
	FILENO parHandle;
	FILENO datHandle;		/* handles der entsprechenden Dateien */
	pBlock *parBuff;		/* Wenn != NULL, dann gepuffert */
	CRC16 *tabBuff;			/* dto. */
	_UBYTE *datBuff;
	MSGNR minPar;			/* erster Parameterblock, der im Speicher steht */
	MSGNR ParAnz;			/* maximale Anzahl, die noch im Speicher gehalten werden kann */
	MSGNR ParBuffAnz;		/* Anzahl der gepufferten Parameterbloecke */
	MSGNR minTab;			/* erste crc, die im Speicher steht */
	MSGNR TabAnz;			/* maximale Anzahl, die im Speicher gehalten werden kann */
	MSGNR TabBuffAnz;		/* Anzahl der gepufferten crcs */
	_ULONG DatAnz;			/* Anzahl der belegbaren Bytes im DAT-AppendBuffer */
	_ULONG DatBuffAnz;		/* Anzahl belegte Bytes im DAT-AppendBuffer */
	_ULONG DatBuffStart;		/* Dateiposition, an der der DAT-Buffer beginnt */
	_ULONG DatSize;			/* Groesse der .DAT zu Testzwecken */
	
	/* Hashing */
	hashHandle hash;

	/* Noch nicht beachtet: */
	_BOOL writeThrough;		/* trotz Pufferung Schreiboperationen durchlassen */
	_BOOL readOnly;
};


/* Die Nummer 0 ist ab sofort die fuer die persoenlichen Msgs, d.h. die Nummerierung auf
 * der Platte stimmt nicht mehr ganz, bleibt aber zunaechst beim alten und muss also
 * fuer den Zugriff umgerechnet werden
 */

LOCAL grPosType grPos;
LOCAL _BOOL grPosRead;	/* Wurde die Tabelle schon gelesen? */
LOCAL _UBYTE emptyString[11];

LOCAL _BOOL dangerousDupeMode;
LOCAL _BOOL usenetChaining;
LOCAL _BOOL isInSearch;	/* gerade laeuft eine Suche */

#define IDError   "ID-Fehler"

LOCAL _BOOL SearchNCountNew(OneGroupHandle handle, MSGNR start, MSGNR *pos, MSGNR *count, _BOOL searchOnly);


LOCAL _BOOL unreadOk(OneGroupHandle handle);

#define BLOCK_CRC(ptr) \
	ZCalcCrc_CalcCrcArray((CONST _UBYTE *)(ptr) + sizeof(CRC16), sizeof(*(ptr)) - sizeof(CRC16))


#define AL_CORE_DUMPED           "[1][%P:|Fehlerhafter Msg-Header,|Debuginfo gespeichert!][[OK|[Translate]"
#define AL_TRANSLATION           "[1][%P:|Bogon warning.|Incoming enemy bogon deflected.][Confirm]"
#define AL_NO_GRPOS              "[3][%P:|Dateifehler in GRUPPEN.POS][[Abbruch]"
#define AL_BAD_MSG               "[3][%P:|Diese Nachricht ist defekt!][[Abbruch]"
#define AL_OLD_VERSION           "[3][%P:|Alte CAT-Datenbank!][[Abbruch]"
#define AL_POSTABLE_NOT_WRITTEN  "[1][%P:|Positionstabelle nicht geschrieben!][ [OK ]"
#define AL_POSSIBLE_OLD_VERSION  "[2][%P:|Versionsinfo fehlt,|m\224glicherweise alte|CAT-Datenbank?][[Weiter|:[Abbruch]"


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL isOldHeader(CONST FileHeaderType *head)
{
	return head->CatMagic != dbCatMagic ||
		   head->Version != dbVersion ||
		   head->VersionMagic != dbVersionMagic;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*------ Prozeduren fuer dynamisches GRUPPEN.POS -------------------*/

LOCAL _BOOL CreatePosArray(GROUPNR used)
{
	GROUPNR i;

	grPos.usedGroups = used;
	grPos.posGroups = grPos.usedGroups;
	if (grPos.posGroups < maxGroup - 50)
		grPos.posGroups += 50;
	else
		grPos.posGroups = maxGroup;
	/* Jetzt Speicher allozieren */
	grPos.pos = MALLOC(grPos.posGroups * sizeof(onePos), "CreatePosArray");
	if (grPos.pos == NULL)
	{
		grPos.usedGroups = 0;
		grPos.posGroups = 0;
		return FALSE;
	}
	for (i = 0; i < grPos.posGroups; i++)
	{
		grPos.pos[i].aktuellePos = dataSys_empty;
		grPos.pos[i].neuePos = dataSys_empty;
		grPos.pos[i].letztePos = dataSys_empty;
		grPos.pos[i].unreadPos = dataSys_empty;
		grPos.pos[i].unreadCount = dataSys_empty;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID FreePosArray(_VOID)
{
	if (grPos.pos != NULL)
	{
		SFREE(grPos.pos);
		grPos.pos = NULL;
		grPos.usedGroups = 0;
		grPos.posGroups = 0;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL MSGNR GetOnePos(GROUPNR group, posType subIdx)
{
	if (group < grPos.usedGroups)
	{
		switch (subIdx)
		{
			case aktuellePos: return grPos.pos[group].aktuellePos;
			case neuePos: return grPos.pos[group].neuePos;
			case letztePos: return grPos.pos[group].letztePos;
			case unreadPos: return grPos.pos[group].unreadPos;
			case unreadCount: return grPos.pos[group].unreadCount;
		}
	}
	return dataSys_empty;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID SetOnePos(GROUPNR group, posType subIdx, MSGNR value)
{
	GROUPNR newSize;
	onePos *newArray;
	GROUPNR i;
	
	if (!grPosRead)
		if (!data_ReadPos())
			return;
	if (group >= grPos.posGroups)
	{
		/* Realloc, Array vergroessern */
		if (group < maxGroup - 20)			/* !!! hier stand newSize < ... */
			newSize = group + 20;
		else
			newSize = maxGroup;
		newArray = MALLOC(newSize * sizeof(onePos), "SetOnePos");
		if (newArray == NULL)
			return;
		MemSetZero(newArray, newSize * sizeof(onePos));		/* ??? wirklich noetig */
		for (i = grPos.posGroups; i < newSize; i++)		/* !!! hier stand TO newsize */
		{
			newArray[i].aktuellePos = dataSys_empty;
			newArray[i].neuePos = dataSys_empty;
			newArray[i].letztePos = dataSys_empty;
			newArray[i].unreadPos = dataSys_empty;
			newArray[i].unreadCount = dataSys_empty;
		}
		MemCpy(newArray, grPos.pos, grPos.posGroups * sizeof(onePos));
		SFREE(grPos.pos);
		grPos.posGroups = newSize;
		grPos.pos = newArray;
	}
	if (group >= grPos.usedGroups)
		grPos.usedGroups = group + 1;
	switch (subIdx)
	{
		case aktuellePos: grPos.pos[group].aktuellePos = value; break;
		case neuePos: grPos.pos[group].neuePos = value; break;
		case letztePos: grPos.pos[group].letztePos = value; break;
		case unreadPos: grPos.pos[group].unreadPos = value; break;
		case unreadCount: grPos.pos[group].unreadCount = value; break;
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*--- Datumsprozeduren ---*/

/* Maus-Datumsstring in ein Cat-Datum umwandeln */
/* Format Jahr|Jahr|Jahr|Jahr|Monat|Monat|Tag|Tag|Stunde|Stunde|Minute|Minute */
/* Jahr = Jahr - 1990                                              */
/* Format Jahr|Jahr|Monat|Monat|Tag|Tag|Stunde|Stunde|Minute|Minute */
GLOBAL CATDATE data_mDateStr2Long(CONST _UBYTE *str, _UWORD start)
{
	_UWORD year;
	_ULONG dt2;
	_UBYTE tmp[12];

	year =         str[3+start] - '0';
	year = year + (str[2+start] - '0') * 10;
	year = year + (str[1+start] - '0') * 100;
	year = year + (str[  start] - '0') * 1000;
	if (year >= 1990)
		year -= 1990;
	else
		year = 0;
	/* Neue Routine */
	strMcpy(tmp, 9, str + start + 4);
	dt2 = StrToLCard(tmp, NULL, NULL);
	dt2 = dt2 + (_ULONG)year * 100000000L;
	if (dt2 < 1010000L)
	{
		/* Es muss mindestens der 1.1.90 sein. */
		dt2 = 1010000L;
	}
	return dt2;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID data_Long2DateStr(CATDATE date, _UBYTE *str, size_t strsize)
{
	F_TIME t;
	_UBYTE datestr[20];
	
	ConvertDate_CatDate2Datim(date, &t);
	T_Strftime(datestr, sizeof(datestr), "%1a, %d.%m.%y %H:%M", &t);
	strcpy(datestr + 2, datestr + 3);
	strMcpy(str, strsize, datestr);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Initialisieren, Fehlermeldung wird selber ausgegeben */
GLOBAL _BOOL data_InitDataBase(_VOID)
{
	_BOOL err;

	err = data_ReadPos();
	
	ConfVars_GetConfDefBool(cDangerousDupeMode, &dangerousDupeMode, TRUE);
	ConfVars_GetConfDefBool(cUsenetChaining, &usenetChaining, FALSE);
	
	return err;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID data_ResetDataBase(_VOID)
{
}

/*** ---------------------------------------------------------------------- ***/

/* Datenbank schliessen, abmelden am Ende */

GLOBAL _VOID data_CloseBase(_VOID)
{
	FreePosArray();
}

/*** ---------------------------------------------------------------------- ***/

/* setzt die Positionen so, dass nach dem Einfuegen etwas huebsches rauskommt :-) */
GLOBAL _VOID data_PrepareToWrite(_VOID)
{
	GROUPNR group;

	for (group = 0; group < grPos.posGroups; group++)
	{
		/* hier ohne SetOnePos, da keine Ueberschreitung der Grenze auftritt */
		grPos.pos[group].neuePos = dataSys_empty;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Aus der Gruppennummer den Namensanfang (ohne Extension) erstellen */
GLOBAL _VOID data_Number2Name(GROUPNR nr, _UBYTE *name)
{
	if (nr == dataSys_private)
	{
		strcpy(name, rawPrivatename);
	} else
	{
		/* Fuer den Zugriff auf die Platte ist die Gruppennummer um eins zu verringern */
		--nr;
		strcpy(name, rawGroupname);
		if (nr >= 100)
		{
			name[5] = (nr / 100) + '0';
			nr %= 100;
		}
		name[6] = (nr / 10) + '0';
		name[7] = (nr % 10) + '0';
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _ULONG FileLength(FILENO file)
{
	return Fseek(0L, file, SEEK_END);
}

/*** ---------------------------------------------------------------------- ***/

/* erste Ungelesene suchen und durchzaehlen */
GLOBAL _VOID data_forceUnreadRefresh(OneGroupHandle handle)
{
	MSGNR pos;
	MSGNR count;
	
	pos = GetOnePos(handle->group, unreadPos);
	SearchNCountNew(handle, 0, &pos, &count, FALSE);
	SetOnePos(handle->group, unreadPos, pos);
	SetOnePos(handle->group, unreadCount, count);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Falls etwas schiefgeht, wenn alle Dateien offen sind */
LOCAL _VOID open_Abort(OneGroupHandle *new, F_ERROR error)
{
	F_ErrorAlert(error);
	CatFiles_CloseFile((*new)->tabHandle);
	CatFiles_CloseFile((*new)->parHandle);
	CatFiles_CloseFile((*new)->datHandle);
	if ((*new)->tabBuff != NULL)
		SFREE((*new)->tabBuff);
	if ((*new)->parBuff != NULL)
		SFREE((*new)->parBuff);
	OFREE(*new);
	*new = NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID open_writeLog(CONST _UBYTE *open, _WORD error)
{
	_UBYTE errMsg[256];

	CatLog_WriteLine("- Fehler beim \224ffnen von Dateien -");
	CatLog_WriteString("Kein Zugriff auf ");
	CatLog_WriteString(open);
	CatLog_WriteString(" -> Gemdos-Fehler #");
	CatLog_WriteInt(error);
	CatLog_WriteLn();
	F_GetErrorMsg(error, errMsg, sizeof(errMsg));
	CatLog_WriteString("Fehlermeldung: ");
	CatLog_WriteString(errMsg);
	CatLog_WriteLn();
	CatLog_WriteLine("-Ende der Durchsage -");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID switch_parbuff(pBlock *par, MSGNR anz)
{
	IfNotBigEndian
	{
		while (anz != 0)
		{
			par->crc = n2hs(par->crc);
			par->Datum = n2hl(par->Datum);
			par->items = n2hs(par->items);
			par->bits = n2hs(par->bits);
			par->hLength = n2hs(par->hLength);
			par->idLength = n2hs(par->idLength);
			par->Length = n2hs(par->Length);
			par->Start = n2hl(par->Start);
			par->upMess = n2hs(par->upMess);
			par->downMess = n2hs(par->downMess);
			par->rightMess = n2hs(par->rightMess);
			par->leftMess = n2hs(par->leftMess);
			par->KomCount = n2hs(par->KomCount);
			par++;
			anz--;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID switch_dupe(dupeInfoType *dupe)
{
	IfNotBigEndian
	{
		dupe->Datum = n2hl(dupe->Datum);
		dupe->items = n2hs(dupe->items);
		dupe->bits = n2hs(dupe->bits);
		dupe->hLength = n2hs(dupe->hLength);
		dupe->idLength = n2hs(dupe->idLength);
		dupe->Length = n2hs(dupe->Length);
		dupe->Start = n2hl(dupe->Start);
		dupe->setTerminator = n2hs(dupe->setTerminator);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID switch_tabbuff(CRC16 *tab, MSGNR anz)
{
	IfNotBigEndian
	{
		while (anz != 0)
		{
			*tab = h2ns(*tab);
			tab++;
			anz--;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* FALSE, wenn nicht geklappt */
/* add gibt an, fuer wieviele neue Nachrichten Platz benoetigt wird */
/*DYJ 4.13d */
GLOBAL _BOOL data_OpenOneGroup(GROUPNR group, MSGNR add, _BOOL wannaWrite, OneGroupHandle *pnew)
{
	_UBYTE name[PATH_MAX];
	_UBYTE open[PATH_MAX];
	_UBYTE fname[PATH_MAX];
	_ULONG avail, alloc, len;
	_ULONG tabSize, parSize;
	FileHeaderType head;
	OneGroupHandle new;
	F_ERROR error;
	openMode oM;
	if(MTPaths_DatabaseRonly())
		oM=CatFiles_readFile;
	else
		oM=CatFiles_readWrite;
	
	*pnew = new = NEW(oneGroupHandle, "data_OpenOneGroup: groupHandle");
	if (new == NULL)
		return FALSE;
	MemSetZeroStruct(*new);
	
	new->group = group;
	data_Number2Name(group, name);
	
	/* Dat-Datei oeffnen */
	strcat(strcpy(open, name), datTag);
	strcpy(fname, DataPath);
	F_Path_Append(fname, open);
	new->datHandle = CatFiles_OpenFile(fname, oM);
	if (!FILENO_OK(new->datHandle))
	{
		sprintf(name, MTE_noFileAccess, open);
		MTE_info(name);
		open_writeLog(open, TO_HANDLE(new->datHandle));
		OFREE(new);
		*pnew = NULL;
		return FALSE;
	}
	new->DatSize = FileLength(new->datHandle);
	CatFiles_Seek(0, new->datHandle, CatFiles_start);
	
	/* Tab-Datei oeffnen */
	strcat(strcpy(open, name), tabTag);
	strcpy(fname, DataPath);
	F_Path_Append(fname, open);
	new->tabHandle = CatFiles_OpenFile(fname, oM);
	if (!FILENO_OK(new->tabHandle))
	{
		sprintf(name, MTE_noFileAccess, open);
		MTE_info(name);
		open_writeLog(open, TO_HANDLE(new->tabHandle));
		CatFiles_CloseFile(new->datHandle);
		OFREE(new);
		*pnew = NULL;
		return FALSE;
	}
	
	/* Par-Datei oeffnen */
	strcat(strcpy(open, name), parTag);
	strcpy(fname, DataPath);
	F_Path_Append(fname, open);
	new->parHandle = CatFiles_OpenFile(fname, oM);
	if (!FILENO_OK(new->parHandle))
	{
		sprintf(name, MTE_noFileAccess, open);
		MTE_info(name);
		open_writeLog(open, TO_HANDLE(new->parHandle));
		CatFiles_CloseFile(new->tabHandle);
		CatFiles_CloseFile(new->datHandle);
		OFREE(new);
		*pnew = NULL;
		return FALSE;
	}
	
	len = FileLength(new->parHandle);
	if (len == 0)	/* Neue Datei, erstmal den Header schreiben */
	{
		CatFiles_Seek(0, new->parHandle, CatFiles_start);
		head.CatMagic = h2nl(dbCatMagic);
		head.Version = h2ns(dbVersion);
		head.VersionMagic = h2ns(dbVersionMagic);
		error = CatFiles_WriteMuch(dbHeaderLength, new->parHandle, &head);
	} else
	{
		CatFiles_Seek(0, new->parHandle, CatFiles_start);
		error = CatFiles_ReadDbHeader(new->parHandle, &head);
		if (error != NoError || isOldHeader(&head))
		{
			MTE_info(AL_OLD_VERSION);
			CatFiles_CloseFile(new->parHandle);
			CatFiles_CloseFile(new->tabHandle);
			CatFiles_CloseFile(new->datHandle);
			OFREE(new);
			*pnew = NULL;
			return FALSE;
		}
		len -= dbHeaderLength;
	}
	new->anz = (MSGNR)(len / sizeof(pBlock));
	/* Anzahl an Nachrichten in der Gruppe */
	if (error < NoError)
	{
		/* Dann ist beim Lesen des Kopfes oder beim Schreiben etwas schiefgelaufen */
		F_ErrorAlert(error);
		open_writeLog(open, error);
		CatFiles_CloseFile(new->parHandle);
		CatFiles_CloseFile(new->tabHandle);
		CatFiles_CloseFile(new->datHandle);
		OFREE(new);
		*pnew = NULL;
		return FALSE;
	}
	
	SetOnePos(group, letztePos, new->anz - 1);

	if (new->anz > dataSys_notSaved - add)
		new->TabAnz	= dataSys_notSaved;
	else
		new->TabAnz = new->anz + add;
	
	/* Allozierung nur, wenn genug Speicher frei ist */
	
	/* Wieviel ist frei */
	avail = (_ULONG)Malloc(-1L);		/* TODO */
	
	/* Berechnen, was freibleiben soll */
	tabSize = new->TabAnz * sizeof(*(new->tabBuff));
	if (new->TabAnz > 0 && avail > (minDatBuffer + tabSize))
		new->tabBuff = SYSMALLOC(tabSize, MEM_NOERR, "data_OpenOneGroup: tabBuff");
	else
		new->tabBuff = NULL;
	
	avail = (_ULONG)Malloc(-1L);		/* TODO */
	
	/* Berechnen, was freibleiben soll */
	parSize = new->TabAnz * sizeof(pBlock);
	if (new->TabAnz > 0 && avail > (minDatBuffer + parSize))
		new->parBuff = SYSMALLOC(parSize, MEM_NOERR, "dataOpenOneGroup: parBuff");
	else
		new->parBuff = NULL;
	
	new->ParAnz = new->TabAnz; /* Das ist erstmal dasselbe */
	
	new->minTab = 0;
	new->minPar = 0; /* More sophisticatet buffering in the future */
	
	if (new->tabBuff != NULL && new->anz > 0)	/* und fuellen.. */
	{
		CatFiles_Seek(0, new->tabHandle, CatFiles_start);
		error = CatFiles_ReadMuch(new->anz * sizeof(*new->tabBuff), new->tabHandle, new->tabBuff);
		new->TabBuffAnz = new->anz;
		if (error < NoError)
		{
			open_Abort(pnew, error);
			open_writeLog("die crc-Tabelle", error);
			return FALSE;
		}
		switch_tabbuff(new->tabBuff, new->anz);
	} else
	{
		new->TabBuffAnz   = 0; /* In beiden ist erstmal nichts drin */
	}
	
	if (new->parBuff != NULL && new->anz > 0)	/* und fuellen.. */
	{
		CatFiles_Seek(dbHeaderLength, new->parHandle, CatFiles_start);
		error = CatFiles_ReadMuch(new->anz * sizeof(pBlock), new->parHandle, new->parBuff);
		new->ParBuffAnz = new->anz;
		if (error < NoError)
		{
			open_Abort(pnew, error);
			open_writeLog("die Parameterdatei", error);
			return FALSE;
		}
		switch_parbuff(new->parBuff, new->ParBuffAnz);
	} else
	{
		new->ParBuffAnz   = 0; /* In beiden ist erstmal nichts drin */
	}
	
	if (wannaWrite)
	{
		if (new->tabBuff != NULL)
			new->hash = Hashing2_ToHash(new->tabBuff, new->TabBuffAnz, add);
		else
			new->hash = Hashing2_getEmptyHash();
		
		alloc = maxDatBuffer;
		new->datBuff = SYSMALLOC(alloc, MEM_NOERR, "data_OpenOneGroup: datBuff");
		if (new->datBuff == NULL)
		{
			alloc = minDatBuffer;
			new->datBuff = SYSMALLOC(alloc, MEM_NOERR, "data_OpenOneGroup: datBuff");
			if (new->datBuff == NULL)
				alloc = 0;
		}
		new->DatAnz = alloc;
		new->DatBuffStart = FileLength(new->datHandle);
		new->DatBuffAnz = 0;
		
		if (GetOnePos(group, neuePos) == dataSys_empty)
			SetOnePos(group, neuePos, new->anz);
		/* Das passiert also nur einmal pro Einfuegen */
		
	} else
	{
		new->hash = Hashing2_getEmptyHash();
		new->datBuff = NULL;
	}
	
	/* Noetigenfalls einmal die ungelesenen Msgs durchzaehlen und die erste suchen */
	if (new->parBuff != NULL)
	{
		/* Wenn die PAR gebuffert ist, dann geht das so schnell, das koennen wir
		 * immer machen!
		 */
		data_forceUnreadRefresh(new);
	} else if (!unreadOk(new))
	{
		data_forceUnreadRefresh(new);
	}
	
	CatLog_WriteStringNTime("Hashtabelle ");
	if (Hashing2_emptyHash(new->hash))
		CatLog_WriteString("nicht ");
	CatLog_WriteString("angelegt ");
	
	CatLog_WriteString("PAR ");
	if (new->parBuff == NULL)
		CatLog_WriteString("nicht ");
	CatLog_WriteString("gepuffert ");
	
	CatLog_WriteString("TAB ");
	if (new->tabBuff == NULL)
		CatLog_WriteString("nicht ");
	CatLog_WriteString("gepuffert ");
	
	CatLog_WriteString("DAT ");
	if (new->datBuff == NULL)
		CatLog_WriteString("nicht ");
	CatLog_WriteLine("gepuffert ");
	
	new->writeThrough = TRUE;
	new->readOnly = FALSE;
	SetOnePos(group, letztePos, new->anz - 1);
	
	/* und evtl. noch ein paar Checks/Fehlerkorrekturen, z.B. letzte Msg */
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Gruppe, die zum Lesen geoeffnet war schliessen */
GLOBAL _VOID data_CloseOneGroup(OneGroupHandle *handle)
{
	_BOOL abort;
	
	data_CloseOneWriteGroup(handle, 0, TRUE, &abort);
	data_WritePos();
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID close_writeLog(_BOOL abort, CONST _UBYTE *open, F_ERROR error)
{
	_UBYTE errMsg[256];

	if (abort)
	{
		CatLog_WriteLine("- Fehler beim Schlie\236en von Dateien -");
		CatLog_WriteString("Kein Schreibzugriff auf ");
		CatLog_WriteString(open);
		CatLog_WriteString(" -> Gemdos-Fehler #");
		CatLog_WriteInt(error);
		CatLog_WriteLn();
		F_GetErrorMsg(error, errMsg, sizeof(errMsg));
		CatLog_WriteString("Fehlermeldung: ");
		CatLog_WriteString(errMsg);
		CatLog_WriteLn();
		CatLog_WriteLine("-Ende der Durchsage -");
		if (!batchProcess)
			F_ErrorAlert(error);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Gruppe, die zum schreiben geoeffnet war schliessen */
/* newMsg -> Anzahl der neuen (d.h. nicht gefilterten oder ignorierten) Msgs */
/* write  -> Puffer wirklich zurueckschreiben? Normalerweise nicht bei Abbruch */
/* abort  -> etwas ist beim Schliessen schiefgelaufen */

/*DYJ 4.13d */
GLOBAL _VOID data_CloseOneWriteGroup(OneGroupHandle *phandle, MSGNR newMsg, _BOOL write, _BOOL *abort)
{
	OneGroupHandle handle;
	F_ERROR error;
	if(MTPaths_DatabaseRonly())
		write=FALSE;
	
	*abort = FALSE;
	if ((handle = *phandle) == NULL)
		return;
	SetOnePos(handle->group, letztePos, handle->anz - 1);
	if (GetOnePos(handle->group, unreadCount) == dataSys_empty ||
		GetOnePos(handle->group, unreadPos) == dataSys_empty ||
		handle->parBuff != NULL)
	{
		data_forceUnreadRefresh(handle);
	} else
	{
		/* Hier auch ohne SetOnePos, da auch hier keine Arrayueberschreitung
		 * auftreten kann. Wenn es eine gab, dann wurde die schon vorher behoben
		 */
		grPos.pos[handle->group].unreadCount += newMsg;
	}
	
	/* Puffer freigeben.. umgekehrte Reihenfolge wie oben */
	if (handle->datBuff != NULL)
	{
		if (write)
		{
			CatFiles_Seek(0, handle->datHandle, CatFiles_end);
			error = CatFiles_WriteMuch(handle->DatBuffAnz, handle->datHandle, handle->datBuff);
			*abort = error < NoError;
			close_writeLog(*abort, "die Haupt-Datendatei", error);
		}
		SFREE(handle->datBuff);
	}
	
	if (handle->parBuff != NULL)
	{
		if (!*abort && write)
		{
			CatFiles_Seek(handle->minPar * sizeof(pBlock) + dbHeaderLength, handle->parHandle, CatFiles_start);
			switch_parbuff(handle->parBuff, handle->ParBuffAnz);
			error = CatFiles_WriteMuch(handle->ParBuffAnz * sizeof(pBlock), handle->parHandle, handle->parBuff);
			*abort = error < NoError;
			close_writeLog(*abort, "die Parameterdatei", error);
		}
		SFREE(handle->parBuff);
	}
	
	if (handle->tabBuff != NULL)
	{
		if (!*abort && write)
		{
			CatFiles_Seek(handle->minTab * sizeof(*handle->tabBuff), handle->tabHandle, CatFiles_start);
			switch_tabbuff(handle->tabBuff, handle->TabBuffAnz);
			error = CatFiles_WriteMuch(handle->TabBuffAnz * sizeof(*handle->tabBuff), handle->tabHandle, handle->tabBuff);
			*abort = error < NoError;
			close_writeLog(*abort, "die Crc-Tabelle", error);
		}
		SFREE(handle->tabBuff);
	}
	
	/* Dateien schliessen */
	CatFiles_CloseFile(handle->tabHandle);
	CatFiles_CloseFile(handle->parHandle);
	CatFiles_CloseFile(handle->datHandle);
	
	Hashing2_ReleaseHash(handle->hash);
	
	OFREE(handle);
	*phandle = NULL;
}

/*** ---------------------------------------------------------------------- ***/

/* Datum aus dem Logfile uebernehmen */

GLOBAL _VOID data_SetLogDate(_UBYTE *date, size_t strsize)
{
	*date = '\0';
	UNUSED(strsize);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL exists(CONST _UBYTE *path, CONST _UBYTE *name)
{
	_UBYTE file[PATH_MAX];
	
	strBcpy(file, path);
	F_Path_Append(file, name);
	return CatFiles_Exists(file);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID switch_one_pos(onePos *p)
{
	p->aktuellePos = h2ns(p->aktuellePos);
	p->neuePos = h2ns(p->neuePos);
	p->letztePos = h2ns(p->letztePos);
	p->unreadPos = h2ns(p->unreadPos);
	p->unreadCount = h2ns(p->unreadCount);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID switch_posarray(_VOID)
{
	MSGNR i, count;
	
	IfNotBigEndian
	{
		count = grPos.usedGroups;
		switch_one_pos(&grPos.save);
		for (i = 0; i < count; i++)
			switch_one_pos(&grPos.pos[i]);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL data_ReadPos(_VOID)
{
	FILENO handle;
	_UWORD z;
	_BOOL ready;
	MSGNR count;
	_ULONG size;
	_UBYTE fname[PATH_MAX];
	F_ERROR error;
	
	/* Erstmal alten Kram deallozieren */
	FreePosArray();
	
	strBcpy(fname, DataPath);
	F_Path_Append(fname, catGroupPos);
	handle = CatFiles_OpenFile(fname, CatFiles_readFile);
	if (FILENO_OK(handle))
		ready = CatFiles_ReadDbHeader(handle, &grPos.head) == NoError;
	else
		ready = FALSE;

	if (ready)
	{
		/* Jetzt pruefen, welchen Header wir haben */
		if (grPos.head.CatMagic != dbCatMagic ||
			grPos.head.Version != dbVersion ||
			(grPos.head.VersionMagic != dbVersionMagic && grPos.head.VersionMagic != grPosVersionMagic))
		{
			MTE_info(AL_OLD_VERSION);
			CatFiles_CloseFile(handle);
			return FALSE;
		}
		if (grPos.head.VersionMagic != grPosVersionMagic)
		{
			/* Altes Gruppen.POS, konvertieren */
			/* wieviele Gruppen haben wir denn? */
			count = grPos.usedGroups = GroupSelect_CountCatGroups();
			if (!CreatePosArray(count))
			{
				CatFiles_CloseFile(handle);
				return FALSE;
			}
			/* Und jetzt einlesen */
			error = CatFiles_ReadMuch(count * sizeof(onePos), handle, grPos.pos);
			ready = error == NoError;
			/* Savebereich fuer Datum noch loeschen */
			MemSetZeroStruct(grPos.save);
		} else
		{
			/* Jetzt mal sehen, wieviele Gruppen darin gespeichert sind */
			/* Groesse feststellen */
			CatFiles_Seek(0, handle, CatFiles_end);
			CatFiles_FilePos(handle, &size);
			CatFiles_Seek(0, handle, CatFiles_start);
			count = (MSGNR)((size - dbHeaderLength) / sizeof(onePos));
			--count;  /* Wegen save-Info */
			CatFiles_Seek(dbHeaderLength, handle, CatFiles_start);
			error = CatFiles_ReadMuch(sizeof(onePos), handle, &grPos.save);
			ready = error == NoError;
			/* Jetzt Speicher allozieren */
			if (!CreatePosArray(count))
			{
				CatFiles_CloseFile(handle);
				return FALSE;
			}
			error = CatFiles_ReadMuch(count * sizeof(onePos), handle, grPos.pos);
			ready = ready && error == NoError;
		}
		CatFiles_CloseFile(handle);
		if (!ready)
			MTE_info(AL_NO_GRPOS);
		switch_posarray();
	} else
	{
		ready = TRUE;
		/* Mal sehen, ob der Typ Dateien hat.. */
		if (exists(DataPath, rawGroupname parTag) ||
			exists(DataPath, rawGroupname datTag) ||
			exists(DataPath, rawGroupname tabTag) ||
			exists(DataPath, rawPrivatename parTag) ||
			exists(DataPath, rawPrivatename datTag) ||
			exists(DataPath, rawPrivatename tabTag))
		{
			/* ja, er hat eine, dann werden wir ihn mal interviewen */
			if (mtAlerts_Alert(2, AL_POSSIBLE_OLD_VERSION) != 1)
				return FALSE;
		}
		/* Jetzt Default anlegen */
		count = GroupSelect_CountCatGroups();
		if (!CreatePosArray(count))
			return FALSE;
		for (z = 0; z < grPos.posGroups; z++)
		{
			grPos.pos[z].aktuellePos = dataSys_empty;
			grPos.pos[z].neuePos = dataSys_empty;
			grPos.pos[z].letztePos = dataSys_empty;
			grPos.pos[z].unreadPos = dataSys_empty;
			grPos.pos[z].unreadCount = dataSys_empty;
		}
		grPos.head.CatMagic = dbCatMagic;
		grPos.head.Version = dbVersion;
		grPos.head.VersionMagic = grPosVersionMagic;
		MemSetZeroStruct(grPos.save);
	}
	grPosRead = ready; /* globale Variable fuer WritePos */
	return grPosRead;
}

/*** ---------------------------------------------------------------------- ***/

/* Positionstabelle einlesen und schreiben */
/*DYJ 4.13d */
GLOBAL _BOOL data_WritePos(_VOID)
{
	FILENO handle;
	_UBYTE path[PATH_MAX];
	F_ERROR error;
	
	if (grPosRead)
	{
		strBcpy(path, DataPath);
		F_Path_Append(path, catGroupPos);
		handle = CatFiles_CreateFile(path);
		error = NoError;
		if (FILENO_OK(handle))
		{
			grPos.head.CatMagic = h2nl(dbCatMagic);
			grPos.head.Version = h2ns(dbVersion);
			grPos.head.VersionMagic = h2ns(grPosVersionMagic);
			/* Header schreiben */
			error = CatFiles_WriteMuch(dbHeaderLength, handle, &grPos.head);
			if (error == NoError)
			{
				switch_posarray();
				/* Save-Bereich schreiben */
				error = CatFiles_WriteMuch(sizeof(onePos), handle, &grPos.save);
				/* Gruppenbereich schreiben */
				if (error == NoError)
					error = CatFiles_WriteMuch(grPos.usedGroups * sizeof(onePos), handle, grPos.pos);
				switch_posarray();
			}
			CatFiles_CloseFile(handle);
		}
		if ((!FILENO_OK(handle) &&(handle!=-36 || !MTPaths_DatabaseRonly())) || error < NoError)
		{
			MTE_info(AL_POSTABLE_NOT_WRITTEN);
			return FALSE;
		}
	}
	return TRUE; /* wurde garnicht gelesen! */
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Holt die Pruefarea fuer das Datum aus dem GRUPPEN.POS
 */
GLOBAL _VOID data_GetCheckArea(onePos *area)
{
	*area = grPos.save;
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die Pruefarea fuer das Datum in dem GRUPPEN.POS
 */
GLOBAL _VOID data_SetCheckArea(CONST onePos *area)
{
	grPos.save = *area;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* erste Msg aus dem letzten Outfile in dieser Gruppe */
GLOBAL MSGNR data_FirstNewMsg(GROUPNR group)
{
	return GetOnePos(group, neuePos);
}

/*** ---------------------------------------------------------------------- ***/

/* letzte Msg der Gruppe */
GLOBAL MSGNR data_LastMsgOfGroup(GROUPNR group)
{
	return GetOnePos(group, letztePos);
}

/*** ---------------------------------------------------------------------- ***/

/* letzte vom User gelesene Msg dieser Gruppe */
GLOBAL MSGNR data_lastReadMsgOfGroup(GROUPNR group)
{
	return GetOnePos(group, aktuellePos);
}

/*** ---------------------------------------------------------------------- ***/

/* Anzahl der ungelesenen Msgs in der Gruppe */
GLOBAL MSGNR data_unreadMsgCount(GROUPNR group)
{
	return GetOnePos(group, unreadCount);
}

/*** ---------------------------------------------------------------------- ***/

/* Position der ersten ungelesenen Msg in der Gruppe */
GLOBAL MSGNR data_unreadMsgPos(GROUPNR group)
{
	return GetOnePos(group, unreadPos);
}

/*** ---------------------------------------------------------------------- ***/

/* neue letzte vom User gelesene Msg setzen */
GLOBAL _VOID data_SetLastReadMsg(GROUPNR group, MSGNR nr)
{
	SetOnePos(group, aktuellePos, nr);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#define is_par_buffered(nr) \
	(handle->parBuff != NULL && \
	 (nr) >= handle->minPar && \
	 (((nr) - handle->minPar) < handle->ParBuffAnz))

/* Fuer evtl. Pufferung, die ungewoehnliche Deklaration, kein unnoetiges umkopieren */
GLOBAL F_ERROR data_ReadBlock(OneGroupHandle handle, MSGNR nr, pBlockPtr *ptr)
{
	F_ERROR error;
	
	if (nr < handle->anz)	/* echt kleiner! Siehe z.B. eine drin und Nummer 0 lesen */
	{
		if (is_par_buffered(nr))
		{
			*ptr = &handle->parBuff[nr - handle->minPar]; /* Haehae, guter Einfall, nicht? */
			return NoError;
		}
		CatFiles_Seek(nr * sizeof(pBlock) + dbHeaderLength, handle->parHandle, CatFiles_start);
		error = CatFiles_ReadMuch(sizeof(pBlock), handle->parHandle, *ptr);
		if (error < NoError)
			return error;
		switch_parbuff(*ptr, 1);
		return NoError;
	}
	return NotFound;
}

/*** ---------------------------------------------------------------------- ***/

/* Block mit crc-Check einlesen */
GLOBAL F_ERROR data_ReadBlockCrc(OneGroupHandle handle, MSGNR nr, pBlockPtr *ptr)
{
	F_ERROR err;
	CRC16 compareCrc;

	err = data_ReadBlock(handle, nr, ptr);
	if (err == NoError)
	{
		compareCrc = BLOCK_CRC(*ptr);
		if (compareCrc != (*ptr)->crc)
			return crcError;
	}
	return err;
}

/*** ---------------------------------------------------------------------- ***/

/* Fehlerbehandlung..!!! */
/* Hier auch einen Pointer uebergeben, bei Pufferung einfach nichts machen, wenn */
/* der Zeiger schon richtig steht. Die anderen Prozeduren arbeiten im Normalfall */
/* mit einem Block, dessen Zeiger sie vielleicht schon von ReadBlock bekommen haben */
GLOBAL F_ERROR data_WriteBlock(OneGroupHandle handle, MSGNR nr, pBlock *ptr)
{
	F_ERROR error;
	
#if 0
	/* Debug: Fehler provozieren */
	if (stopSearch())
	{
		return data_fileError;
	}
#endif
	/* writeThrough? */
	if (nr < handle->anz)	/* echt kleiner! Siehe z.B. eine drin und Nummer 0 lesen */
	{
		if (is_par_buffered(nr))
		{
			if (ptr != &handle->parBuff[nr - handle->minPar])
				handle->parBuff[nr - handle->minPar] = *ptr; /* Dann noch zuweisen */
		} else
		{
			CatFiles_Seek(nr * sizeof(pBlock) + dbHeaderLength, handle->parHandle, CatFiles_start);
			switch_parbuff(ptr, 1);
			error = CatFiles_WriteMuch(sizeof(pBlock), handle->parHandle, ptr);
			switch_parbuff(ptr, 1);
			if (error < NoError)
				return error;
		}
	} else
	{
		/* Das sollte naemlich nicht auftreten.. */
		/* HALT; */
		return -64; /*!ERROR*/
	}
	return NoError;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL F_ERROR data_WriteBlockCrc(OneGroupHandle handle, MSGNR nr, pBlock *ptr)
{
	ptr->crc = BLOCK_CRC(ptr);
	return data_WriteBlock(handle, nr, ptr);
}

/*** ---------------------------------------------------------------------- ***/

/* Fehlerbehandlung!! */
/* Einen Block an PAR anhaengen */
LOCAL F_ERROR AppendBlock(OneGroupHandle handle, pBlock *block)
{
	F_ERROR error;
	
	if (handle->parBuff != NULL &&
		handle->anz < (handle->minPar + handle->ParAnz) &&
		handle->ParBuffAnz < handle->ParAnz)
	{
		handle->parBuff[handle->anz - handle->minPar] = *block;
		++handle->ParBuffAnz;
	} else
	{
		CatFiles_Seek(0, handle->parHandle, CatFiles_end);
		switch_parbuff(block, 1);
		error = CatFiles_WriteMuch(sizeof(pBlock), handle->parHandle, block);
		switch_parbuff(block, 1);
		if (error < NoError)
			return error;
	}
	++handle->anz; /* Jetzt haben wir einen mehr */
	return NoError;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#define is_tab_buffered(nr) \
	(handle->tabBuff != NULL && \
	 (nr) >= handle->minTab && \
	 (((nr) - handle->minTab) < handle->TabBuffAnz))

GLOBAL F_ERROR data_GetCrc(OneGroupHandle handle, MSGNR nr, CRC16 *crc)
{
	CRC16 z;
	F_ERROR error;
	
	if (nr < handle->anz)		/* echt kleiner! Siehe z.B. eine drin und Nummer 0 lesen */
	{
		if (is_tab_buffered(nr))
		{
			z = handle->tabBuff[nr - handle->minTab];
		} else
		{
			CatFiles_Seek(nr * sizeof(CRC16), handle->tabHandle, CatFiles_start);
			error = CatFiles_ReadMuch(sizeof(CRC16), handle->tabHandle, &z);
			if (error < NoError)
				return error;
			z = n2hs(z);
		}
	} else
	{
		/* Sollte nicht auftreten */
		/* HALT; */
		return -64; /*!ERROR*/
	}
	*crc = z;
	return NoError;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL F_ERROR data_PutCrc(OneGroupHandle handle, MSGNR nr, CRC16 crc)
{
	F_ERROR error;
	
	if (nr < handle->anz)		/* echt kleiner! Siehe z.B. eine drin und Nummer 0 lesen */
	{
		if (is_tab_buffered(nr))
		{
			handle->tabBuff[nr - handle->minTab] = crc;
		} else
		{
			CatFiles_Seek(nr * sizeof(CRC16), handle->tabHandle, CatFiles_start);
			crc = h2ns(crc);
			error = CatFiles_WriteMuch(sizeof(CRC16), handle->tabHandle, &crc);
			if (error < NoError)
				return error;
		}
	} else
	{
		/* Sollte nicht auftreten */
		/* HALT; */
		return -64; /*!ERROR*/
	}
	return NoError;
}

/*** ---------------------------------------------------------------------- ***/

/* Eine Crc an TAB anhaengen */
LOCAL F_ERROR AppendCrc(OneGroupHandle handle, CRC16 crc)
{
	F_ERROR error;
	
	if (handle->tabBuff != NULL &&
		handle->anz < handle->minTab + handle->TabAnz &&
		handle->TabBuffAnz < handle->TabAnz)
	{
		handle->tabBuff[handle->anz - handle->minTab] = crc;
		++handle->TabBuffAnz;
		Hashing2_AddCrc(handle->hash, crc);
	} else
	{
		CatFiles_Seek(0, handle->tabHandle, CatFiles_end);
		crc = h2ns(crc);
		error = CatFiles_WriteMuch(sizeof(CRC16), handle->tabHandle, &crc);
		if (error < NoError)
			return error;
	}
	return NoError;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#define is_dat_buffered(start, len) \
	(handle->datBuff != NULL && \
	 start >= handle->DatBuffStart && \
	 (start + len) <= (handle->DatBuffStart + handle->DatBuffAnz))
	
LOCAL F_ERROR ReadFromDat(OneGroupHandle handle, _ULONG start, _ULONG len, _VOID *strAdr)
{
	F_ERROR error;
	
	if (is_dat_buffered(start, len))
	{
		MemCpy(strAdr, &handle->datBuff[start - handle->DatBuffStart], len);
	} else
	{
		if (start <= handle->DatSize)
		{
			CatFiles_Seek(start, handle->datHandle, CatFiles_start);
			error = CatFiles_ReadMuch(len, handle->datHandle, strAdr);
			if (error < NoError)
				return error;
		} else
		{
			return -64; /*!ERROR*/
		}
	}
	return NoError;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL F_ERROR WriteToDat(OneGroupHandle handle, _ULONG start, _ULONG len, _VOID *strAdr)
{
	CatFiles_Seek(start, handle->datHandle, CatFiles_start);
	return CatFiles_WriteMuch(len, handle->datHandle, strAdr);
}

/*** ---------------------------------------------------------------------- ***/

/* Daten an die DAT anhaengen */
LOCAL F_ERROR AppendToDat(OneGroupHandle handle, _VOID *adr, _ULONG howMuch, _ULONG *startOffs)
{
	_ULONG start;
	F_ERROR error;
	
	if (handle->datBuff != NULL)
	{
		error = NoError;
		if (handle->DatBuffAnz + howMuch > handle->DatAnz)	/* Kein Platz mehr */
		{
			CatFiles_Seek(0, handle->datHandle, CatFiles_end);
			error = CatFiles_WriteMuch(handle->DatBuffAnz, handle->datHandle, handle->datBuff);
			handle->DatSize += handle->DatBuffAnz;
			/* Erstmal den Puffer leeren */
			handle->DatBuffAnz = 0;
			handle->DatBuffStart = Fseek(0L, handle->datHandle, SEEK_END);
		}
		if (howMuch > handle->DatAnz)
		{
			/* Dann ist in jedem Fall auch die obige IF-Bedingung erfuellt gewesen! */
			start = handle->DatBuffStart;
			error = CatFiles_WriteMuch(howMuch, handle->datHandle, adr);
			handle->DatSize += howMuch;
		} else
		{
			start = handle->DatBuffStart + handle->DatBuffAnz;
			MemCpy(&handle->datBuff[handle->DatBuffAnz], adr, howMuch);
			handle->DatBuffAnz += howMuch;
		}
	} else
	{
		start = Fseek(0L, handle->datHandle, SEEK_END);
		error = CatFiles_WriteMuch(howMuch, handle->datHandle, adr);
		handle->DatSize += howMuch;
	}
	if (startOffs != NULL)
		*startOffs = start;
	return error;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL F_ERROR ReadID(OneGroupHandle handle, _ULONG IDStart, size_t count, _UBYTE *id)
{
	return ReadFromDat(handle, IDStart, count, id);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL F_ERROR ReadOtherID(OneGroupHandle handle, MSGNR nr, _UBYTE *id, size_t len, _BOOL *isOldDupe)
{
	pBlock s, *blockPtr;
	F_ERROR error;
	
	blockPtr = &s;
	if ((error = data_ReadBlock(handle, nr, &blockPtr)) == NoError)
	{
		if (blockPtr->idLength < len)
			len = blockPtr->idLength;
		error = ReadID(handle, blockPtr->Start, len, id);
		*isOldDupe = dangerousDupeMode && (blockPtr->bits & bOldDupe);
	} else
	{
		strMcpy(id, len, IDError);
		*isOldDupe = FALSE;
	}
	return error;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL ReadOtherRId(OneGroupHandle handle, MSGNR nr, _UBYTE *id, size_t len)
{
	pBlock s, *blockPtr;
	_UBYTE *InfoStrings;
	data_offset *pos;
	_UWORD z;
	_UBYTE *ptr;
	size_t hLen;
	
	blockPtr  = &s;
	if (data_ReadBlock(handle, nr, &blockPtr) != NoError)
		return FALSE;
	if (!(blockPtr->items & itemMId))
		return FALSE;
	InfoStrings = MALLOC(blockPtr->hLength, "ReadOtherRId");
	if (InfoStrings == NULL)
		return FALSE;
	if (ReadFromDat(handle, blockPtr->Start, blockPtr->hLength, InfoStrings) != NoError)
		return FALSE;
	hLen = blockPtr->idLength;
	if (hLen & 1)
		hLen++;
	pos = (data_offset *)(InfoStrings + hLen);
	z = 2;
	/* Reply-ID ist da in diesem Fall! */
	if (blockPtr->items & itemVon)
		z++;
	if (blockPtr->items & itemAn)
		z++;
	if (blockPtr->items & itemMId)
		z++;
	ptr = InfoStrings + n2hs(pos[z - 1]);
	strMcpy(id, len, ptr);
	SFREE(InfoStrings);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Datenbank nach der ersten ungelesenen durchsuchen und die ungelesenen durchzaehlen */
LOCAL _BOOL SearchNCountNew(OneGroupHandle handle, MSGNR start, MSGNR *pos, MSGNR *count, _BOOL searchOnly)
{
	pBlock s, *blockPtr;
	_BOOL found;

	found = FALSE;
	*count = 0;
	blockPtr = &s;
	while (start <= handle->anz && data_ReadBlock(handle, start, &blockPtr) == NoError)
	{
		if (!(blockPtr->bits & bGelesen) &&		
			!((blockPtr->bits & bFiltered) && UserBLK.ignoreFiltered))	/*DYJ 4.14n*/
		{
			if (!found)
			{
				*pos   = start; /* Hier ist die erste ungelesene.. */
				if (searchOnly)
					return TRUE; /* alles klar, wir haben einen gefunden.. */
				found = TRUE;
			}
			++(*count);
		}
		++start;
		blockPtr = &s;
	}
	return found;
}

/*** ---------------------------------------------------------------------- ***/

/* "erste ungelesene" ueberpruefen, ob sie wirklich ungelesen ist */

LOCAL _BOOL unreadOk(OneGroupHandle handle)
{
	pBlock s, *blockPtr;
	
	if (GetOnePos(handle->group, unreadPos) == dataSys_empty ||
		GetOnePos(handle->group, unreadPos) >= handle->anz ||
		GetOnePos(handle->group, unreadCount) == dataSys_empty ||
		GetOnePos(handle->group, unreadCount) >= handle->anz)
	{
		return FALSE;
	}
	blockPtr = &s;
	return data_ReadBlock(handle, GetOnePos(handle->group, unreadPos), &blockPtr) == NoError &&
		!(blockPtr->bits & bGelesen);
}


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*--- bestimmte Sachen suchen ---*/


LOCAL _BOOL IDOkay(OneGroupHandle handle, CONST _UBYTE *searchID, MSGNR nr)
{
	_UBYTE scrapID[1024];
	_BOOL isOldDupe;
	F_ERROR error;
	
	if ((error = ReadOtherID(handle, nr, scrapID, sizeof(scrapID), &isOldDupe)) != NoError)
		return error;
	if (CmpId(scrapID, searchID) && !isOldDupe)
		return NoError;
	/* Erlaeuterung: Die letzten beiden werden nur verwendet, wenn man im */
	/* DangerousDupeMode ist. Ansonsten ist isOldDupe _immer_ FALSE, es  */
	/* passiert also nix */
	return NotFound;
}

/*** ---------------------------------------------------------------------- ***/

/* Die Nummer der Nachricht mit der uebergebenen ID herausfinden */
LOCAL _BOOL SearchID(
	OneGroupHandle handle,
	CONST _UBYTE *searchID,
	MSGNR start,
	_BOOL rev,
	MSGNR *nr)
{
	CRC16 compareCrc;
	CRC16 crc;
	F_ERROR error;
	
	compareCrc = ZCalcCrc_CalcIdCrc(searchID);
	if (Hashing2_emptyHash(handle->hash))
	{
		if (rev)
		{
			if (handle->anz == 0)
				return FALSE;
			*nr = handle->anz;
			while (*nr != 0)
			{
				--(*nr);
				if (data_GetCrc(handle, *nr, &crc) != NoError)
					break;
				if (crc == compareCrc)
				{
					error = IDOkay(handle, searchID, *nr);
					if (error == NoError)
						return TRUE;
					if (error != NotFound)
						break;
				}
			}
		} else
		{
			*nr = start;
			while (*nr < handle->anz)
			{
				if (data_GetCrc(handle, *nr, &crc) != NoError)
					break;
				if (crc == compareCrc)
				{
					error = IDOkay(handle, searchID, *nr);
					if (error == NoError)
						return TRUE;
					if (error != NotFound)
						break;
				}
				++(*nr);
			}
		}
		*nr = dataSys_empty;
	} else
	{
		*nr = Hashing2_GetFirst(handle->hash, compareCrc);
		while (*nr != dataSys_empty)
		{
			error = IDOkay(handle, searchID, *nr);
			if (error == NoError)
				return TRUE;
			if (error != NotFound)
				break;
			*nr = Hashing2_GetNext(handle->hash);
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Fehlerrueckgabe der naechsten 2 Positionsfunktionen: NoError/NotFound */

/* MsgNummer zu einer ID feststellen */
GLOBAL MSGNR data_NumberOfID(OneGroupHandle handle, CONST _UBYTE *ID)
{
	MSGNR nr;
	
	if (ID == NULL || *ID == '\0')
	{
		return dataSys_empty;
	} else if (SearchID(handle, ID, 0, FALSE, &nr))	/* Neueste mit dieser ID finden.. */
	{
		return nr;
	} else
	{
		return dataSys_empty;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL MSGNR data_NumberOfDate(OneGroupHandle handle, CATDATE date)
{
	pBlock s, *blockPtr;
	MSGNR nr;
	_BOOL ok;

	nr = 0;
	do
	{
		blockPtr = &s;
		ok = data_ReadBlock(handle, nr, &blockPtr) == NoError;
		++nr;
	} while (ok && nr < handle->anz && blockPtr->Datum < date);
	return nr - 1;
	/* d.h. evtl. wird hier genau der defekte Block zurueckgegeben */
	/* crc-data_error notfalls selber behandeln! */
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*--- Hilfprozeduren zum lesen ---*/

/* Fuellt mess mit dem, was man aus dem ParameterBlock bekommt,
 * Eigentlich sind keine Fehler moeglich.. :-)
 */
LOCAL F_ERROR iReadHeader(OneGroupHandle handle, MSGNR nr, pBlock *blockPtr, MessageType *mess)
{
	CRC16 compareCrc;
	
	compareCrc = BLOCK_CRC(blockPtr);
	if (compareCrc != blockPtr->crc)
	{
		MTE_info(AL_BAD_MSG);
		return crcError;
	}
	MemSetZeroStruct(*mess);
	mess->MailNr            = nr;
	mess->MailAnz           = handle->anz;
	mess->MailID            = emptyString;      /* noch nicht bekannt */
	mess->KommentierteID[0] = '\0';             /* noch nicht bekannt */
	mess->fromOther         = FALSE;            /* auch unbekannt */
	mess->Betreff           = emptyString;      /* Kommt auch aus der *.dat */
	mess->Absender          = emptyString;      /* noch nicht bekannt */
	mess->Gruppe            = handle->group;
	data_Long2DateStr(blockPtr->Datum, mess->Datum, sizeof(mess->Datum));
	mess->EigeneNachricht   = FALSE;
	mess->StatusDatum[0]    = '\0';
	
	mess->left            = blockPtr->leftMess;
	mess->right           = blockPtr->rightMess;
	mess->up              = blockPtr->upMess;
	mess->down            = blockPtr->downMess;
	mess->KommentarAnzahl = blockPtr->KomCount;
	mess->tauschDate      = blockPtr->Datum;
	
	mess->infoLen         = blockPtr->hLength;
	mess->textLen         = blockPtr->Length;
	mess->StatusBits      = blockPtr->bits;
	mess->Status          = 'N';
	mess->statusDate      = 0;
	mess->Text            = NULL;
	mess->InfoStrings     = NULL;
	mess->distribution    = distNone;
	return NoError;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID lauf2Ptr(pBlock *blockPtr, ITEMBITS lauf, data_offset **pos, MessageType *mess)
{
	_UBYTE *ptr;

	if (blockPtr->items & lauf)
	{
		ptr = mess->InfoStrings + n2hs(**pos);
		++(*pos);
	} else
	{
		ptr = emptyString;
	}
	switch (lauf)
	{
		case itemVon: mess->Absender = ptr; break;
		case itemAn: mess->Empfaenger = ptr; break;
		case itemMId: mess->mid = ptr; break;
		case itemRId: mess->rid = ptr; break;
		case itemBox: mess->box = ptr; break;
		case itemName: mess->name = ptr; break;
		case itemGate: mess->gate = ptr; break;
		case itemMime: mess->mime = ptr; break;
		case itemFollowup: mess->followupTo = ptr; break;
		case itemReplyTo: mess->replyTo = ptr; break;
		case itemSender: mess->sender = ptr; break;
		case itemRefNr: strBcpy(mess->KommentierteID, ptr); break;
		case itemDistribution:
			if (ptr != NULL)
			{
				switch (ptr[0])
				{
					case 'N': mess->distribution = distNet; break;
					case 'M': mess->distribution = distMausNet; break;
					case 'L': mess->distribution = distLokal; break;
					default: mess->distribution  = distNone; break;
				}
			} else
			{
				mess->distribution = distNone;
			}
			break;
		default:
			ErrorOut(EO_ERROR, "iReadRest: unknown item");
			break;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Restliche Daten lesen, nur falls bisher alles gut gelesen wurde,
 * d.h. data_error == NoError
 */
LOCAL F_ERROR iReadRest(OneGroupHandle handle, pBlock *blockPtr, _BOOL readText, MessageType *mess)
{
	data_offset *pos;
	data_offset *z;
	ITEMBITS lauf;
	pInfoType *p;
	_UBYTE *cp;
	_UWORD cz;
	size_t hLen;
	_BOOL isOldDupe;
	F_ERROR error;
	
	/*
	 * ein Byte mehr allozieren, falls wir die PrivateBytes alignen muessen
	 */
	mess->InfoStrings = MALLOC(blockPtr->hLength + 1, "iReadRest: InfoStrings");
	if (mess->InfoStrings != NULL)
	{
		error = ReadFromDat(handle, blockPtr->Start, blockPtr->hLength, mess->InfoStrings);
		if (error != NoError)
			return error;
		hLen = blockPtr->idLength;
		if (hLen & 1)
			++hLen;
		pos = (data_offset *)(mess->InfoStrings + hLen);
		mess->MailID  = mess->InfoStrings;
		mess->Betreff = mess->InfoStrings + n2hs(pos[1]);
		z = pos + 2;
		for (lauf = itemVon; lauf <= itemSender; lauf <<= 1)
			lauf2Ptr(blockPtr, lauf, &z, mess);
		if (blockPtr->items & itemPrivateBytes)
		{
			hLen = blockPtr->hLength - sizeof(pInfoType);
			p = (pInfoType *)(mess->InfoStrings + hLen);
			if (hLen & 1)
			{
				/* Speziell fuer Datenbanken auf 68000ern, die nicht aligned sind */
				/* Davon gibt's nur eine, die von meinem Bruder! */
				cp = (_UBYTE *)p;
				for (cz = (_UWORD)sizeof(pInfoType); cz > 0; --cz)
				{
					cp[cz] = cp[cz - 1];
				}
				p = (pInfoType *)(mess->InfoStrings + hLen + 1);
			}
			p->LeseDatum = n2hl(p->LeseDatum);
			data_Long2DateStr(p->LeseDatum, mess->StatusDatum, sizeof(mess->StatusDatum));
			mess->EigeneNachricht = p->locked != '\0';
			mess->Status          = p->Status;
			mess->statusDate      = p->LeseDatum;
		}
	} else
	{
		return -39; /*!ERROR*/
	}
	/*-- Ist ja jetzt im Header und wird normalerweise von dort geladen! --*/
	if (blockPtr->upMess != dataSys_empty &&
		blockPtr->upMess != dataSys_notSaved &&
		blockPtr->upMess < handle->anz && !(blockPtr->items & itemRefNr))
	{
		error = ReadOtherID(handle, blockPtr->upMess, mess->KommentierteID, sizeof(mess->KommentierteID), &isOldDupe);
		/* Nur in diesem Fall von der kommentierten Msg besorgen! */
		if (error == NoError)
		{
			mess->fromOther = TRUE;
		} else
		{
			mess->fromOther = FALSE;
			mess->KommentierteID[0] = '\0';
		}
		error = NoError;
	}
	/*---------*/
	if (readText)
	{
		mess->Text = MALLOC(blockPtr->Length + editSecureBytes, "iReadRest: text");
		if (mess->Text != NULL)
		{
			error = ReadFromDat(handle, blockPtr->Start + blockPtr->hLength, blockPtr->Length, mess->Text);
		} else
		{
			SFREE(mess->InfoStrings);
			mess->InfoStrings = NULL;
			error = -39; /*!ERROR*/
		}
	}
	return error;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*--- Leseprozeduren ---*/

/* Nur die Flags lesen, fuer "Gelesene ignorieren" */
GLOBAL F_ERROR data_ReadState(OneGroupHandle handle, MSGNR nr, MSGBITS *flags)
{
	pBlock s, *blockPtr;
	F_ERROR error;
	
	blockPtr = &s;
	if ((error = data_ReadBlockCrc(handle, nr, &blockPtr)) == NoError)
		*flags = blockPtr->bits;
	else
		*flags = 0;
	return error;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL F_ERROR data_ReadPersState(OneGroupHandle handle, MSGNR nr, _UBYTE *state, _BOOL *own, MSGBITS *bits)
{
	pBlock s, *blockPtr;
	pInfoType pers;
	F_ERROR error;
	
	blockPtr = &s;
	if ((error = data_ReadBlockCrc(handle, nr, &blockPtr)) == NoError)
	{
		*own   = FALSE;
		*state = '?';
		if (handle->group == dataSys_private)
		{
			CatFiles_Seek(blockPtr->Start + blockPtr->hLength - sizeof(pInfoType), handle->datHandle, CatFiles_start);
			error = CatFiles_ReadMuch(sizeof(pInfoType), handle->datHandle, &pers);
			if (error == NoError)
			{
				*own   = pers.locked != '\0';
				*state = pers.Status;
			}
		}
		*bits  = blockPtr->bits;
		
		if ((*bits & bOldComToOwnMessage) && !handle->readOnly)
		{
			*bits |= bComToOwnMessage;
			*bits &= ~bOldComToOwnMessage;
			blockPtr->bits = *bits;
			error = data_WriteBlockCrc(handle, nr, blockPtr);
		}
	} else
	{
		*own   = FALSE;
		*state = '?';
		*bits  = 0;
	}
	return error;
}

/*** ---------------------------------------------------------------------- ***/

/* Nummer der Nachricht in der Kommentarverkettung nach rechts erfragen */
GLOBAL _VOID data_ReadRightNumber(OneGroupHandle handle, MSGNR nr, MSGNR *right)
{
	pBlock s, *blockPtr;

	blockPtr = &s;
	if (data_ReadBlockCrc(handle, nr, &blockPtr) == NoError)
		*right = blockPtr->rightMess;
	else
		*right = dataSys_empty;
}

/*** ---------------------------------------------------------------------- ***/

/* Testet, ob diese Msg noch eine Antwort hat, fuer grin.grinResetState */
GLOBAL _BOOL data_HasAnswer(OneGroupHandle handle, MSGNR nr, MSGBITS *bits)
{
	pBlock s, *blockPtr;

	blockPtr = &s;
	if (data_ReadBlockCrc(handle, nr, &blockPtr) == NoError)
	{
		*bits = blockPtr->bits;
		return blockPtr->downMess != dataSys_empty;
	}
	*bits = 0;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* den Text zur msg in den Puffer lesen, muss natuerlich gross genug sein! */
GLOBAL F_ERROR data_ReadText(OneGroupHandle handle, MSGNR nr, _VOID *adr, size_t bufsize)
{
	pBlock s, *blockPtr;
	F_ERROR error;
	
	blockPtr = &s;
	if ((error = data_ReadBlockCrc(handle, nr, &blockPtr)) == NoError)
	{
		if (blockPtr->Length < bufsize)
			bufsize = blockPtr->Length;
		error = ReadFromDat(handle, blockPtr->Start + blockPtr->hLength, bufsize, adr);
	}
	return error;
}

/*** ---------------------------------------------------------------------- ***/

/* Eine Msg aus der Datenbank lesen */
LOCAL F_ERROR iReadMessage(OneGroupHandle handle, MSGNR nr, _BOOL text, MessageType *mess)
{
	pBlock s, *blockPtr;
	F_ERROR error;
	
	blockPtr = &s;
	error = data_ReadBlock(handle, nr, &blockPtr);
	if (error == NoError)
	{
		error = iReadHeader(handle, nr, blockPtr, mess);
		if (error == NoError)
			error = iReadRest(handle, blockPtr, text, mess);
	}
	return error;
}

/*** ---------------------------------------------------------------------- ***/

/* Moegliche Fehler: NoError, notFound, IOError */

/* Eine Msg aus der Datenbank lesen */
GLOBAL F_ERROR data_ReadMessage(OneGroupHandle handle, MSGNR nr, MessageType *mess)
{
	return iReadMessage(handle, nr, TRUE, mess);
}

/*** ---------------------------------------------------------------------- ***/

/* Nur einen Teil der Msg lesen, evtl. weitere Funktionen */
GLOBAL F_ERROR data_ReadHeader(OneGroupHandle handle, MSGNR nr, MessageType *mess)
{
	return iReadMessage(handle, nr, FALSE, mess);
}

/*** ---------------------------------------------------------------------- ***/

/* Nur einen Teil der Msg lesen, evtl. weitere Funktionen */
GLOBAL F_ERROR data_ReadSmallHeader(OneGroupHandle handle, MSGNR nr, MessageType *mess)
{
	pBlock s, *blockPtr;
	F_ERROR error;
	
	blockPtr = &s;
	error = data_ReadBlock(handle, nr, &blockPtr);
	if (error == NoError)
		error = iReadHeader(handle, nr, blockPtr, mess);
	return error;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*--- Veraenderungen waehrend des normalen Lesens ---*/

/* group   : Gruppe der Msg */
/* nr      : Nummer der Msg */
/* newread : Wurde sie neu gelesen oder ungelesen? */
/* Wird von SetBits automatisch aufgerufen */
LOCAL _VOID SomethingRead(OneGroupHandle handle, MSGNR nr, _BOOL newread)
{
	MSGNR start;
	MSGNR posi;
	MSGNR count;
	
	if (newread)
	{
		count = GetOnePos(handle->group, unreadCount);
		if (count > 0)	/* alles klar.. */
		{
			SetOnePos(handle->group, unreadCount, count - 1);
			start = nr;
			posi = GetOnePos(handle->group, unreadPos);
			if (posi < start)
				start = posi;
			/* Naechste ungelesene Msg suchen: */
			SearchNCountNew(handle, start, &posi, &count, TRUE);
			SetOnePos(handle->group, unreadPos, posi);
		} else	/* irgendwas stimmt nicht, mal neu zaehlen & suchen */
		{
			data_forceUnreadRefresh(handle);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Statusbits setzen */
GLOBAL F_ERROR data_SetBits(OneGroupHandle handle, MSGNR nr, MSGBITS bits)
{
	pBlock s, *blockPtr;
	_BOOL newread;
	_BOOL unread;
	F_ERROR error;
	
	blockPtr = &s;
	error = data_ReadBlock(handle, nr, &blockPtr);
	if (error == NoError)
	{
		newread = !(blockPtr->bits & bGelesen) && (bits & bGelesen);
		unread = (blockPtr->bits & bGelesen) && !(bits & bGelesen);
		blockPtr->bits = bits;
		
		if (!handle->readOnly)
		{
			if (bits & bOldComToOwnMessage)
			{
				bits |= bComToOwnMessage;
				bits &= ~bOldComToOwnMessage;
				blockPtr->bits = bits;
				if ((error = data_WriteBlockCrc(handle, nr, blockPtr)) != NoError)
					return error;
			}
			
			error = data_WriteBlockCrc(handle, nr, blockPtr); /* erst aendern, ist wichtig fuer die Suche gleich */
		}
		if (newread)
			SomethingRead(handle, nr, TRUE);
		if (unread)
			data_forceUnreadRefresh(handle);
	}
	return error;
}

/*** ---------------------------------------------------------------------- ***/

/* StatusChar setzen */
LOCAL F_ERROR SetState(OneGroupHandle handle, MSGNR nr, CATDATE date, _UBYTE State)
{
	pBlock s, *blockPtr;
	pInfoType p;
	F_ERROR error;
	
	blockPtr = &s;
	if ((error = data_ReadBlock(handle, nr, &blockPtr)) != NoError)
		return error;
	if (blockPtr->items & itemPrivateBytes)
	{
		if ((error = ReadFromDat(handle, blockPtr->Start + blockPtr->hLength - sizeof(pInfoType), sizeof(pInfoType), &p)) != NoError)
			return error;
		if (p.Status != State || n2hl(p.LeseDatum) != date)
		{
			p.Status    = State;
			p.LeseDatum = h2nl(date);
			if (!handle->readOnly)
				if ((error = WriteToDat(handle, blockPtr->Start + blockPtr->hLength - sizeof(pInfoType), sizeof(pInfoType), &p)) != NoError)
					return error;
		}
	}
	return NoError;
}

/*** ---------------------------------------------------------------------- ***/

/* StatusChar setzen */
GLOBAL F_ERROR data_ChangeState(OneGroupHandle handle, MSGNR nr, _UBYTE NewState)
{
	return SetState(handle, nr, ConvertDate_CurrentDate(), NewState);
	/* Was bei Statuswechsel von Z -> G oder Z -> B oder x -> Z ? */
}

/******************************************************************************/
/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/
/******************************************************************************/

typedef data_offset *cardPtrType;

/* Hier auch die Anpassung fuer Rueckwaertssuche */
#define isTheEnd(start) \
	(((start >= search->s.handle->anz) && !(search->s.what & reverse)) || \
	 ((start == (MSGNR)-1) && (search->s.what & reverse)))

#define nextMess(start) \
	if (search->s.what & reverse) --start; else ++start

#define SetBits(set, clear, messBits) \
	if ((search->s.what & changeBits) && !(search->s.what & ask)) \
	{ \
		messBits |= set; \
		messBits &= ~clear; \
	}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL BitsOk(MSGBITS set, MSGBITS clear, MSGBITS messBits)
{
	MSGBITS mask;
	
	mask = set | clear;
	messBits = messBits & ~(messBits & ~mask);
	return messBits == set;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL stopSearch(_VOID)
{
	_UBYTE ch, scan;
	_UWORD kstate;
	
	if (VDIUtil_isKey(&kstate, &scan, &ch))
		return ch == 0x1b;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Testet, ob der String an der gewuenschten Stelle steht */
LOCAL _BOOL CheckWhere(cardPtrType cardPtr, data_len idLength, data_len hLength, MSGBITS bits, _WORD cond, data_offset start, data_offset len)
{
	/* Wegen der &%$%$%$%-68000er muss hier eine Sonderbehandlung her, */
	/* um ungerade Adressen zu umgehen */
	typedef union {
		data_offset card;
		_UBYTE byte[2];
	} TrickType;
	_WORD c;
	_UBYTE *TrickPointer;
	TrickType t[5];
	
	/* Keine Laengenueberpruefung, da ein 0C am Ende jedes Stringes steht */
	/* Also kann kein gefundener String ueber ein Feldende hinausgehen! */

	TrickPointer = (_UBYTE *)cardPtr;
	for (c = 0; c < 5; c++)
	{
		t[c].byte[0] = TrickPointer[2 * c];
		t[c].byte[1] = TrickPointer[2 * c + 1];
	}
	/* evtl. wird hier auf nicht vorhandene Cardinals zugegriffen, das ist aber   */
	/* nicht weiter tragisch, da die Werte in diesen Faellen auch nicht beachtet   */
	/* werden! */

	
	switch (cond)
	{
	case sUEBERALL:
		return (start < idLength) || (start >= idLength + (t[0].card + 1) * 2);
		/* Also in der ID gefunden oder aber hinter der CARDINAL-Tabelle */
	
	case sTEXT:
		return start >= hLength;
	
	case sBETREFF:
		return (start >= t[1].card) &&
			((t[0].card >= 2 && start < t[2].card) ||
			 (t[0].card <  2 && start < hLength));

	case sTEXTnBETREFF:
		return (start >= t[1].card &&
			((t[0].card >= 2 && start < t[2].card) ||
			 (t[0].card <  2 && start < hLength))) ||
			 start >= hLength;
			 
	case sABSENDER:
		return (bits & itemVon) && (start >= t[2].card) &&
			((t[0].card >= 3 && start < t[3].card) ||
			 (t[0].card <  3 && start < hLength));

	case sEMPFAENGER:
		if (bits & itemVon)
			c = 3;
		else
			c = 2;
		/* Position der Startangabe berechnen */
		return (bits & itemAn) && (start >= t[c].card) &&
			((t[0].card >= c+1 && start < t[c+1].card) ||
			 (t[0].card <  c+1 && start < hLength));
		
	case sID:
		return (start < idLength) && (len <= idLength);
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID MakeNewCardPtr(cardPtrType *ptr)
{
	union {
		_UBYTE b[2];
		data_offset c;
	} count;
	_UBYTE *bPtr;
	cardPtrType newPtr;
	
	bPtr = (_UBYTE *)(*ptr);
	count.b[0] = bPtr[0];
	count.b[1] = bPtr[1];
	newPtr = MALLOC((count.c + 1) * sizeof(*newPtr), "MakeNewCardPtr");
	/* !!! Fehlerbehandlung */
	MemCpy(newPtr, *ptr, (count.c + 1) * sizeof(*newPtr));
	*ptr = newPtr;
}

/*** ---------------------------------------------------------------------- ***/

/* Sucht einen Text mit Ueberpruefung der Fundstelle auf Korrektheit */
LOCAL _BOOL FindWithBitsWhere(
	_UBYTE *Text,		/* Anfangsadresse des zu durchsuchenden Textes */
	_LONG TextLen, 		/* Gesamtlaenge des Textes */
	tPattern *p,		/* Zeiger auf vorcompiliertes Suchmuster */
	_WORD wo,
	CONST _UBYTE *str,
	cardPtrType cardPtr,
	pBlock *blockPtr)
{
	_BOOL found, textfound;
	_UBYTE *save, *helpPtr;
	_UBYTE *matchPos;
	_WORD matchLen;
	
	helpPtr = Text;
	do
	{
		textfound = Find2_Find(Text, TextLen, p, 0);
		Find2_GetMatch(p, 0, &matchPos, &matchLen);
		found     = textfound && CheckWhere(cardPtr, blockPtr->idLength, blockPtr->hLength, blockPtr->items, wo,
		                      (data_offset)(matchPos - helpPtr), (data_len)strlen(str));
		save      = Text;
		Text      = matchPos + 1;
		TextLen   = Text - save + 1;
	} while (textfound && !found);
	return found;
}

/*** ---------------------------------------------------------------------- ***/

typedef struct {
	SearchInfo s;
	tPattern *pPatt;
	tPattern *pPatt2;
	tPattern *pPatt3;
	tPattern *pPatt4;
} iSearchInfo;

LOCAL _BOOL FindWithBits(iSearchInfo *search)
{
	_BOOL found;
	_UBYTE *helpPtr;
	cardPtrType cardPtr;
	_ULONG calcLen;
	_ULONG datStart;
	pBlock s, *blockPtr;
	size_t hLen;
	F_ERROR error;
	
	do
	{
		/* Ist eine zweite REPEAT-Schleife vielleicht besser? */
		found    = FALSE; /* gonna be pessimistic today */
		blockPtr = &s;
		error = data_ReadBlock/*Crc*/(search->s.handle, search->s.start, &blockPtr);
		/* Naja, auch mal anpassen, falls wirklich ein uebler Fehler auftritt */
		
		/* Erstmal nach einer Message mit den passenden Bits suchen */
		while (error == NoError &&    /* Suchen solange kein Dateifehler..    */
			!isTheEnd(search->s.start) &&      /* .. und noch kein Ende erreicht..     */
			(found = BitsOk(search->s.setBits, search->s.clearBits, blockPtr->bits)) == FALSE)   /* .. sowie die Bits nicht stimmen.   */
		{
			/* erstmal in den Bits suchen, dass geht viel schneller */
			nextMess(search->s.start);
			if (!found && !isTheEnd(search->s.start))
			{
				blockPtr = &s;
				error = data_ReadBlock/*Crc*/(search->s.handle, search->s.start, &blockPtr);
			}
      	}
		if (found && (/*(search->s.what & inSubject) || */ (search->s.what & inText)))
		{
			/* -- Nachricht laden und durchsuchen */
			/* -- Laenge des benoetigten Blocks berechnen */
			calcLen  = blockPtr->Length + blockPtr->hLength;
			datStart = blockPtr->Start;
			/* -- Anfordern.. */
			helpPtr = MALLOC(calcLen, "FindWithBits");
			if (helpPtr != NULL)
			{
				CatGlobal_busyMouse();
				/*-- laden.. */
				error = ReadFromDat(search->s.handle, datStart, calcLen, helpPtr);
				/* -- und durchsuchen */
				
				/* found = Find2_Find(helpPtr, calcLen, pPatt, 0); */
				hLen = blockPtr->idLength;
				if (hLen & 1)
					++hLen;
				cardPtr = (cardPtrType)(helpPtr + hLen);
				MakeNewCardPtr(&cardPtr);
				/* Zeiger auf die Anfangsoffsets der einzelnen Strings in der Datenbank */
				
				found = FindWithBitsWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt, search->s.wo1, search->s.str, cardPtr, blockPtr);
				
				/* Jetzt noch logische Verknuepfungen machen */
				if (search->pPatt2 != NULL)
				{
					/* der erste leere String bricht ab */
					if (search->s.verkn1 == vUND)
						found &= FindWithBitsWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt2, search->s.wo2, search->s.str2, cardPtr, blockPtr);
					else
						found |= FindWithBitsWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt2, search->s.wo2, search->s.str2, cardPtr, blockPtr);
					
					if (search->pPatt3 != NULL)
					{
						/* der erste leere String bricht ab */
						if (search->s.verkn2 == vUND)
							found &= FindWithBitsWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt3, search->s.wo3, search->s.str3, cardPtr, blockPtr);
						else
							found |= FindWithBitsWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt3, search->s.wo3, search->s.str3, cardPtr, blockPtr);
					
						if (search->pPatt4 != NULL)
						{
							/* der erste leere String bricht ab */
							if (search->s.verkn3 == vUND)
								found &= FindWithBitsWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt4, search->s.wo4, search->s.str4, cardPtr, blockPtr);
							else
								found |= FindWithBitsWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt4, search->s.wo4, search->s.str4, cardPtr, blockPtr);
						}
					}
    	        }

				/*
				found = BoyerMoore_Pos(0,
					helpPtr,
					LONGINT(calcLen),
					str,
					LENGTH(str),
					gross,
					Tabelle) <= LONGINT(calcLen);
				*/
				SFREE(helpPtr);
				SFREE(cardPtr);
			} else
			{
				return FALSE;
			}
		}
		if (found)
		{
			/* Hier werden jetzt die Flags gesetzt! */
			SetBits(search->s.toSetBits, search->s.toClearBits, blockPtr->bits);
			error = data_WriteBlockCrc(search->s.handle, search->s.start, blockPtr);
			if (search->s.what & dontShow)
			{
				found = FALSE; /* hihi */
				nextMess(search->s.start); /* aktuelle Msg ueberblaettern */
			}
		} else	 /* Hier kommen wir an, wenn die Bits richtig sind, aber der Text nicht! */
		{
			nextMess(search->s.start);
			/* Oben erfolgt auch eine Erhoehung, sogar die wichtigere!        */
			/* Hier unten wird die Nachricht ueberblaettert, die zwar richtige */
			/* Bits hat, bei der aber der Text falsch ist!                   */
		}
		*(search->s.break_) = stopSearch();
	} while (!*(search->s.break_) && !found && error == NoError && !isTheEnd(search->s.start));
	return found && !*(search->s.break_);
}

/*** ---------------------------------------------------------------------- ***/

/* Sucht einen Text mit Ueberpruefung der Fundstelle auf Korrektheit */
LOCAL _BOOL FindWithStringWhere(
	_UBYTE *Text,		/* Anfangsadresse des zu durchsuchenden Textes */
	_LONG TextLen, 		/* Gesamtlaenge des Textes */
	tPattern *p,		/* Zeiger auf vorcompiliertes Suchmuster */
	_WORD wo,
	CONST _UBYTE *str,
	cardPtrType cardPtr,
	pBlock *blockPtr)
{
	_BOOL found, textfound;
	_UBYTE *save, *helpPtr;
	_UBYTE *matchPos;
	_WORD matchLen;
	
	helpPtr = Text;
	do
	{
		textfound = Find2_Find(Text, TextLen, p, 0);
		Find2_GetMatch(p, 0, &matchPos, &matchLen);
		
		found     = textfound && CheckWhere(cardPtr, blockPtr->idLength, blockPtr->hLength, blockPtr->items, wo,
		                      (data_offset)(matchPos - helpPtr), (data_len)strlen(str));
		save      = Text;
		Text      = matchPos + 1;
		TextLen   = Text - save + 1;
	} while (textfound && !found);
	return found;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL FindStringOnly(iSearchInfo *search)
{
#define buffSize (64L*1024L)
	_BOOL found;
	_UBYTE *helpPtr; /* BoyerMoore.longPtr; */
	cardPtrType cardPtr;
	_UBYTE *freePtr; /* BoyerMoore.longPtr; */
	_ULONG amount; /* Wieviel wurde gelesen */
	_ULONG startPos; /* Startposition in der DAT */
	_ULONG dLen; /* Dateilaenge der DAT */
	size_t hLen;
	pBlock s, *blockPtr;
	F_ERROR error;
	
	freePtr = MALLOC(buffSize, "FindStringOnly");
	if (freePtr != NULL)
	{
		/* found = FALSE; */
		dLen = FileLength(search->s.handle->datHandle);
		blockPtr = &s;
		error = data_ReadBlock/*Crc*/(search->s.handle, search->s.start, &blockPtr);
		do		/* Suchen bis Abbruch oder gefunden oder am Ende */
		{
			/* einlesen */
			if (error != NoError)	 /* Wenn wir's nicht lesen koennen, keine Chance.. */
			{
				FREE(freePtr, buffSize);
				return FALSE;
			}
			if (search->s.what & reverse)
			{
				if (blockPtr->Start + blockPtr->hLength + blockPtr->Length > buffSize)
					startPos = blockPtr->Start + blockPtr->hLength + blockPtr->Length - buffSize;
				else
					startPos = 0;
			} else
			{
				startPos = blockPtr->Start;
			}
			amount = LMin(buffSize, dLen - startPos);
			error = ReadFromDat(search->s.handle, startPos, amount, freePtr);

/* Wir haben hier eine experimentelle Suchfunktion, die noch ein paar     */
/* Probleme hat. z.B. muss beim rueckwaerts-Suchen BackPos verwendet werden  */
/* und es gibt Probleme, wenn das Wort gerade auf Puffergrenzen liegt.    */
/* Das kann man dadurch abfangen, dass man bestimmt, zu welcher Msg        */
/* das/der Pufferende/-anfang gehoert und entsprechend dies immer komplett */
/* einliest. Mache ich einmal bei Gelegenheit, viel bringt die Routine    */
/* bei meine Maxtor sowieso nicht. Aber fuer langsame Platten und ohne     */
/* Pufferung waere sie doch bei seltenen Worten deutlich schneller.        */
/* Dann kann man auch die Puffergroesse variabel machen, vielleicht anhand  */
/* anhand einer Bewertungsfunktion, die aus der Laenge und den Buchstaben  */
/* berechnet, wie wahrscheinlich es in einer Msg vorkommen kann. Je       */
/* unwahrscheinlicher, desto groesser sollte man den Puffer waehlen.         */

#if 0
			if (search->s.handle->parBuff != NULL)
			{
				_ULONG where;
				MSGNR sLeft;
				MSGNR sRight;
				_BOOL notAgain;

				/* Wenn die par gepuffert ist, dann einfach den Block durchsuchen */
				notAgain = FALSE;
				do
				{
					found = Find2_Find(freePtr, amount, search->pPatt, 0);
					/*
					where = BoyerMoore.Pos(0,
						freePtr,
						LONGINT(amount),
						str,
						LENGTH(str),
						gross,
						Tabelle);
						found = where <= amount;
					*/
					if (!found)
					{
						if (!(startPos == 0 && (search->s.what & reverse)) &&
							!(!(search->s.what & reverse) && (startPos + buffSize > dLen)))
						{
							if (search->s.what & reverse)
							{
								if (startPos > buffSize)
									startPos -= buffSize;
								else
									startPos = 0;
							} else
							{
								if (startPos + buffSize < dLen)
									startPos += buffSize;
								else
									startPos = startPos + buffSize - dLen;
							}
							amount = LMin(buffSize, dLen - startPos);
							CatGlobal_busyMouse();
							error = ReadFromDat(search->s.handle, startPos, amount, freePtr);
						} else
						{
							notAgain = TRUE;
						}
					}
				} while (!found && !notAgain && error == NoError);
				/* Ok, im aktuellen Block steht jetzt moeglicherweise das Wort: */
				if (found)
				{
					/* Jetzt binaer die Msg finden, fuer die Start <= startPos+where < Start+hLength+Length */
					if (blockPtr->Start + blockPtr->hLength + blockPtr->Length < startPos + where)
					{
						/* Gesuchte Stelle ist weiter vorne! */
						sLeft = search->s.start;
						sRight = search->s.handle->anz - 1; /* In leeren Gruppen kann man sowieso nicht suchen */
					} else if (blockPtr->Start > startPos + where)
					{
						/* Gesuchte Stelle ist weiter hinten! */
						sLeft = 0;
						sRight = search->s.start;
					}
					do
					{
						if (blockPtr->Start + blockPtr->hLength + blockPtr->Length < startPos + where)
						{
							/* Gesuchte Stelle ist weiter vorne! */
							sLeft = search->s.start;
							search->s.start = (sLeft + sRight) / 2;
							if (search->s.start == sLeft)
								++search->s.start;
						} else if (blockPtr->Start > startPos + where)
						{
							/* Gesuchte Stelle ist weiter hinten! */
							sRight = search->s.start;
							search->s.start  = (sLeft + sRight) / 2;
							if (search->s.start == sRight)
								--search->s.start;
						}
						blockPtr = &s;
						error = data_ReadBlock/*Crc*/(search->s.handle, search->s.start, &blockPtr);
					} while ((blockPtr->Start > startPos + where) ||
						(startPos + where > blockPtr->Start + blockPtr->hLength +  blockPtr->Length));
						/* Anschliessend muss in "start" die Nummer dieser Msg stehen! */
				}
				FREE(freePtr, buffSize);
				return found;
			} else
#endif /* Ende der experimentellen Suchfunktion */
			{
				
				/* "Alte" Suchfunktion, jetzt auch recht schnell & sicher */
				do /* Msgs im Puffer einzeln durchsuchen */
				{
					if (error != NoError)
					{
						/* Wenn wir's nicht lesen koennen, keine Chance.. */
						/* hier, da am Ende der Schleife ein neuer Block gelesen wird */
						FREE(freePtr, buffSize);
						return FALSE;
					}
					helpPtr = freePtr + blockPtr->Start - startPos;
					hLen = blockPtr->idLength;
					if (hLen & 1)
						hLen++;
					cardPtr = (cardPtrType)(helpPtr + hLen);
					/* Jetzt die Bytes mit den Laengen (Offsets) einfach kopieren, so
					 * dass kein Buserror auftritt
					 */
					MakeNewCardPtr(&cardPtr);
					/* Zeiger auf die Anfangsoffsets der einzelnen Strings in der Datenbank */
					
					found = FindWithStringWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt, search->s.wo1, search->s.str, cardPtr, blockPtr);
					
					/* Jetzt noch logische Verknuepfungen machen */
					if (search->pPatt2 != NULL)
					{
						/* der erste leere String bricht ab */
						if (search->s.verkn1 == vUND)
							found &= FindWithStringWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt2, search->s.wo2, search->s.str2, cardPtr, blockPtr);
						else
							found |= FindWithStringWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt2, search->s.wo2, search->s.str2, cardPtr, blockPtr);
						
						if (search->pPatt3 != NULL)
						{
							/* der erste leere String bricht ab */
							if (search->s.verkn2 == vUND)
								found &= FindWithStringWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt3, search->s.wo3, search->s.str3, cardPtr, blockPtr);
							else
								found |= FindWithStringWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt3, search->s.wo3, search->s.str3, cardPtr, blockPtr);
							
							if (search->pPatt4 != NULL)
							{
								/* der erste leere String bricht ab */
								if (search->s.verkn3 == vUND)
									found &= FindWithStringWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt4, search->s.wo4, search->s.str4, cardPtr, blockPtr);
								else
									found |= FindWithStringWhere(helpPtr, blockPtr->Length + blockPtr->hLength, search->pPatt4, search->s.wo4, search->s.str4, cardPtr, blockPtr);
							}
						}
					}
					
					/* Speicher fuer CardArray wieder freigeben */
					SFREE(cardPtr);
					
					/*
					found = BoyerMoore.Pos(0,
						helpPtr,
						LONGINT(_LONG(blockPtr->Length)+_LONG(blockPtr->hLength)),
						str,
						LENGTH(str),
						gross,
						Tabelle) <= LONGINT(_LONG(blockPtr->Length)+_LONG(blockPtr->hLength));
					*/
					if (found)
					{
						/* Hier werden jetzt die Flags gesetzt! */
						SetBits(search->s.toSetBits, search->s.toClearBits, blockPtr->bits);
						error = data_WriteBlockCrc(search->s.handle, search->s.start, blockPtr);
						if (search->s.what & dontShow)
						{
							found = FALSE; /* hihi */
							/* nextMess(search->s.start); */
						}
					}
					/* Die Sache mit nextMess kann man nicht vereinfachen, da start wichtig ist: */
					/* Returnwert fuer die Nummer der gefundenen Nachricht                        */
					if (!found)
					{
						nextMess(search->s.start);
						if (!isTheEnd(search->s.start))
						{
							blockPtr = &s;
							if ((search->s.start % 100) == 0)
								CatGlobal_busyMouse();
							error = data_ReadBlock/*Crc*/(search->s.handle, search->s.start, &blockPtr);
						}
					}
				} while (!found && !isTheEnd(search->s.start) &&
					(((blockPtr->Start + blockPtr->Length + blockPtr->hLength) <= startPos+amount) || (search->s.what & reverse)) &&
					 ((blockPtr->Start >= startPos) || !(search->s.what & reverse)));
				/* Wenn beim vorwaerts-Suchen das Ende der Msg nicht mehr drin liegt, oder
				 * beim rueckwaerts suchen der Anfang nicht mehr ist diese Schleife zuende
				 */
			}

			*(search->s.break_) = stopSearch();
		} while (!*(search->s.break_) && !found && !isTheEnd(search->s.start)); /* oben naechsten Puffer einlesen */
		FREE(freePtr, bufSize);
	} else
	{
		return FALSE;
	}
	/* Phewww! */
	return found && !*(search->s.break_);
}
	
/*** ---------------------------------------------------------------------- ***/

/*
TYPE SearchType == (inBits, inSubject, inText, changeBits, dontShow, reverse);
*/
/* Die ersten drei geben je an, wo gesucht werden soll. changeBits sagt, */
/* dass die Bits der gefundenen Msg gemaess der Setzmaske gesetzt werden    */
/* sollen. Bei dontShow werden alle Msgs bis zum Ende behandlet. Mit     */
/* reverse kann man die Suchrichtung auf rueckwaerts stellen               */

/* Sucht eine Nachricht nach den angegebenen Werten, falls sie gefunden wird */
/* dann ist found TRUE und es wird die Nummer dieser Nachricht zurueckgegeben */
/* str wird evtl. wegen gross-klein-Unterscheidung geaendert, deswegen         */
/* wo1..wo4, verkn1..verkn3 wie in SearchHelp.. */

GLOBAL _BOOL data_ComplexSearch(SearchInfo *search, MSGNR *nr)
{
	iSearchInfo s;
	_BOOL found;
	
	if (isInSearch)
		return FALSE;
	if (search->start >= search->handle->anz)
		return FALSE;
	/* Damit wollen wir doch garnicht erst anfangen.. tststs */
	isInSearch = TRUE;
	s.s = *search;
	s.pPatt = NULL;
	s.pPatt2 = NULL;
	s.pPatt3 = NULL;
	s.pPatt4 = NULL;
	if (search->what & inText /* || (search->what & inSubjectt) */)
	{
		/* Tabelle fuer spaetere Suche initialisieren */
		/*    BoyerMoore.InitTable(Tabelle, str, LENGTH(str), gross, FALSE); */
		s.pPatt = Find2_Compile(search->str, strlen(search->str), '?', '\001', '*', LF, search->gross, FALSE);
		if (search->str2 != NULL && search->str2[0] != '\0')
			s.pPatt2 = Find2_Compile(search->str2, strlen(search->str2), '?', '\001', '*', LF, search->gross, FALSE);
		if (search->str3 != NULL && search->str3[0] != '\0')
			s.pPatt3 = Find2_Compile(search->str3, strlen(search->str3), '?', '\001', '*', LF, search->gross, FALSE);
		if (search->str4 != NULL && search->str4[0] != '\0')
			s.pPatt4 = Find2_Compile(search->str4, strlen(search->str4), '?', '\001', '*', LF, search->gross, FALSE);
	}
	StoreMouse();
	CatGlobal_busyMouse();
	if (multiTask)
	{
		/* Bildschirm freigeben und alle Menus sperren
		 */
		Wind_Update(END_UPDATE);
	}
	if (search->what & inBits)
		found = FindWithBits(&s);
	else
		found = FindStringOnly(&s);
	if (multiTask)
	{
		/* WindUpdate wieder setzen und Menu wieder anschalten */
		Wind_Update(BEG_UPDATE);
	}
	RestoreMouse();
	*nr = s.s.start;
	Find2_Dispose(&s.pPatt);
	Find2_Dispose(&s.pPatt2);
	Find2_Dispose(&s.pPatt3);
	Find2_Dispose(&s.pPatt4);
	isInSearch = FALSE;
	return found;
}


/******************************************************************************/
/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/
/******************************************************************************/

/*--- Schreibprozeduren ---*/

LOCAL _VOID insert_Abort(_UWORD which, F_ERROR error, MSGNR nr, GROUPNR group)
{
	_UBYTE scrap[256];

	CatLog_WriteLine("- data.c meldet sich wg. Dateifehler -");
	CatLog_WriteString("Dateifehler beim Einf\201gen in die Gruppe ");
	GroupSelect_GroupName(group, scrap, sizeof(scrap));
	CatLog_WriteString(scrap);
	CatLog_Write('.');
	CatLog_WriteLn();
	CatLog_WriteString(" -> Gemdos-Fehler #");
	CatLog_WriteInt(error);
	CatLog_WriteLn();
	CatLog_WriteString("Und zwar in der ");
	switch (which)
	{
	case 0:
		CatLog_WriteLine("Crc-Tabelle.");
		break;
	case 1:
		CatLog_WriteString("Parameterdatei bei MsgNummer ");
		CatLog_WriteCard(nr);
		CatLog_Write('.');
		CatLog_WriteLn();
		break;
	case 2:
		CatLog_WriteLine("Daten-Hauptdatei.");
		break;
	}
	F_GetErrorMsg(error, scrap, sizeof(scrap));
	CatLog_WriteString("Fehlermeldung: ");
	CatLog_WriteString(scrap);
	CatLog_WriteLn();
	if (error == crcError)
	{
		CatLog_WriteString("Grund: Die Crc der Msg ist falsch.");
		CatLog_WriteLn();
	}
	CatLog_WriteLine("Daher wurde das Einf\201gen abgebrochen.");
	CatLog_WriteLine("- Ende der Durchsage -");
	if (!batchProcess)
		F_ErrorAlert(error);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL SearchWithHash(OneGroupHandle handle, CONST _UBYTE *refNr, MSGNR *origin, _BOOL *eindeutig, _BOOL *searchAgain)
{
	CRC16 compareCrc;

	if (Hashing2_emptyHash(handle->hash))
		return FALSE;
	/* Keine Hashtabelle, SearchAgain sowieso TRUE, d.h. nochmal suchen */
	compareCrc = ZCalcCrc_CalcIdCrc(refNr);
	*origin = Hashing2_GetFirst(handle->hash, compareCrc);
	/* Erste Msg mit gesuchter Crc oder dataSys_empty */
	*eindeutig = Hashing2_GetNext(handle->hash) == dataSys_empty;
	/* Falls es keinen weiteren gibt, ist es eindeutig */
	*searchAgain = !*eindeutig;
	/* nicht eindeutig => nochmal suchen */
	/* Sonst haben wir einen Kandidaten, dann nicht nochmal suchen */
	return *eindeutig && (*origin != dataSys_empty); /* gefunden und eindeutig */
}

/*** ---------------------------------------------------------------------- ***/

/* Sucht eine Usenet-ID */
LOCAL _BOOL SearchUsenet(OneGroupHandle handle, CONST _UBYTE *search, MSGNR *origin, MSGNR start)
{
	group_line id;
	MSGNR nr;
	
	nr = start;
	while (nr != (_UWORD)-1)
	{
		if (ReadOtherRId(handle, nr, id, sizeof(id)))
		{
			if (CmpId(id, search))
			{
				*origin = nr;
				return TRUE;
			}
		}
		--nr;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL F_ERROR InsertKom(
	OneGroupHandle handle,
	MSGNR MessageNummer,
	_BOOL usenetOnly,
	MSGBITS *addFlags,	/* Flags, die wg. Vererben-Flag gesetzt werden sollen */
	MSGNR *upMess,
	MSGNR *leftMess,
	CONST _UBYTE *refNr,
	_BOOL *abort)
{
	MSGNR origin, Nr, last, next;
	pBlock s, *blockPtr;
	F_ERROR err;
	_BOOL eindeutig;
	_BOOL searchAgain;
	_UBYTE scrapID[1024];
	_BOOL filteredAreRead;
	size_t idLen;
	
	*addFlags = 0; /* erstmal nix */
	/* Erstmal initialisieren, dann kann man ohne Probleme returnen */
	*upMess   = dataSys_notSaved;
	*leftMess = dataSys_empty;
	eindeutig = FALSE;
	/*  abort = FALSE; */
	/* Ist die crc eindeutig? Dann beim Herstellen der Verkettung die ID untersuchen */
	searchAgain = TRUE;
	
	if ((usenetOnly && SearchUsenet(handle, refNr, &origin, MessageNummer)) ||
		(!usenetOnly && (SearchWithHash(handle, refNr, &origin, &eindeutig, &searchAgain) ||
		 (searchAgain && SearchID(handle, refNr, 0, TRUE, &origin)))))
		        /* Zur neuesten verketten ^^^^ */
	{
		/* Erlaeuterung: Der erste Term ist falsch, wenn entweder keine Hashtabelle da ist */
		/* oder aber dort die gesuchte Crc nicht gefunden wurde oder sie nicht eindeutig  */
		/* ist. Falls sie nicht eindeutig ist, oder keine Hashtabelle da ist, soll nach   */
		/* alten Methode gesucht werden.                                                  */
		blockPtr = &s;
		err = data_ReadBlockCrc(handle, origin, &blockPtr); /* Hier Fehlerbehandlung!! */
		if (err != NoError)
		{
			insert_Abort(1, err, origin, handle->group);
			*abort = TRUE;
			return err;
		}
		
		if (blockPtr->bits & bVererben)
		{
			/* hier wird ueberprueft, ob die Muttermsg das Vererben-Flag hat und wenn ja, */
			/* dann werden alle Flags an die Tochtermsg weitergegeben                   */
			*addFlags = blockPtr->bits & ~(bGelesen | bKommentieren | bAntworten | bOwnMessage | bComToOwnMessage);
			if (*addFlags & bFiltered)
			{
				ConfVars_GetConfDefBool(cFilteredAreRead, &filteredAreRead, FALSE);
				if (filteredAreRead)
					*addFlags |= bGelesen;
			}
		}
		
		if (blockPtr->bits & bOwnMessage)
		{
			/* Kommentare auf eigene Nachrichten erhalten ein spezielles Flag */
			*addFlags |= bComToOwnMessage;
			/* Jetzt noch merken, dass es in dieser Gruppe Kommentare auf
			 * eigene Nachrichten gab
			 */
		}
		
		if (eindeutig)
		{
			/* Falls die Crc eindeutig war, muss noch die ID selber ueberprueft werden */
			idLen = blockPtr->idLength;
			if (idLen > sizeof(scrapID)) /* Hmm, kann ja wohl eigentlich nicht sein */
			{
				idLen = sizeof(scrapID) - 1;
				scrapID[idLen] = '\0';
			} 
			err = ReadID(handle, blockPtr->Start, idLen, scrapID);
			if (err != NoError)
			{
				insert_Abort(2, err, origin, handle->group);
				*abort = TRUE;
				return err;
			}
			
			/* IF ~MagicStrings.Equal(scrapID, refNr) THEN */
			if (!CmpId(scrapID, refNr))
			{
				*addFlags=0;	/*DYJ 4.14b */
				return NoError;
			}
		}
		
		*upMess = origin;
		Nr     = origin; /* Fuer's Schreiben unten */
		if (blockPtr->downMess == dataSys_empty || blockPtr->downMess <= origin)
		{
			/* Im zweiten Fall waer's ein Verkettungsfehler -> anmeckern */
			blockPtr->downMess = MessageNummer;
			blockPtr->KomCount = 1;
			*leftMess           = dataSys_empty;
		} else	/* blockPtr->downMess != dataSys_empty */
		{
			/* Jetzt beim ersten Kommentar, nach rechts bis zum Ende gehen */
			
			last = origin; /* Ursprungmsg, bzw. kommentierte Msg */
			next = blockPtr->downMess;
			++blockPtr->KomCount; /* Jetzt haben wir einen Kommentar mehr */
			err = data_WriteBlockCrc(handle, origin, blockPtr);
			if (err != NoError)
			{
				insert_Abort(1, err, origin, handle->group);
				*abort = TRUE;
				return err;
			}
			
			/* ..und jetzt suchen, bis wir am Ende des Kommentarbaumes sind */
			while (last < next && next < handle->anz && next != dataSys_empty)
			{
				blockPtr = &s;
				err = data_ReadBlockCrc(handle, next, &blockPtr); /* Fehlerbehandlung..! */
				if (err != NoError)
				{
					insert_Abort(1, err, next, handle->group);
					*abort = TRUE;
					return err;
				}
				last = next;
				next = blockPtr->rightMess;
			}
			/* Verkettungsfehler anmeckern */
			Nr = last;
			blockPtr->rightMess = MessageNummer;
			*leftMess            = Nr;
		}
		err = data_WriteBlockCrc(handle, Nr, blockPtr);
		if (err != NoError)
		{
			insert_Abort(1, err, Nr, handle->group);
			*abort = TRUE;
			return err;
		}
	}
	return NoError;
}

/*** ---------------------------------------------------------------------- ***/

/* Testet die Nachricht einmal durch, damit der parser schon einen Teil der */
/* Fehlerbehandlung sowie die WatchDog-Behandlung machen kann               */
GLOBAL WhatsThat data_PreCheck(PtrRecord *mess)
{
	stringSet mask;
	
	mask = ~mess->whatsThere;
	/* Persoenliche Nachricht? */
	if ((mask & (bVon | bAn | bWegen | bEZeit | bBSZeit | bText)) == 0 &&
		!(mess->whatsThere & bGruppe))
	{
		return msg_personal;
	} else
	/* Eigene, persoenliche Nachricht? */
	if ((mask & (bAn | bWegen | bEZeit)) == 0 &&
		!(mess->whatsThere & bGruppe) &&
		((mess->whatsThere & bText) || (mess->whatsThere & bTextDatei)))
	{
		return msg_own;
	} else
	/* Normale Gruppen-Nachricht? */
	if ((mask & (bWegen | bEZeit | bGruppe | bText)) == 0
		/* && !(mess->whatsThere & bBSZeit) wg. "RAUS" */)
	{
		return msg_normal;
	} else
	/* Statusmeldung? */
	if ((mask & (bId | bBSZeit | bEZeit)) == 0)
	{
		return msg_status;
	}
	return msg_garbage;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isDupe(OneGroupHandle handle, PtrRecord *mess, CATDATE date, WhatsThat type)
{
	MSGNR nr;
	pBlock s, *blockPtr;
	MessageType oldMess;
	
	if (!SearchID(handle, mess->pId, 0, TRUE, &nr))
		return FALSE;
	blockPtr = &s;
	if (data_ReadBlockCrc(handle, nr, &blockPtr) != NoError)
		return FALSE; /* Gibt's einen Dateifehler ist's auch kein Dupe :-) */
	if (blockPtr->Datum == date	&& blockPtr->Length == mess->TextMax)
		return TRUE; /* doch'n Dupe */
	if (type == msg_own && blockPtr->Datum == date)
	{
		/* Da ist die Laenge noch nicht gesetzt, daher Dupe! */
		return TRUE;
	} else if (type == msg_own)
	{
		/* Mal sehen, ob wir eine MId haben */
		if ((mess->whatsThere & bMId) && (blockPtr->items & itemMId))
		{
			/* MId ist da, alte MId lesen */
			if (data_ReadHeader(handle, nr, &oldMess) != NoError)
				return FALSE;
			if (mess->pMId != NULL && oldMess.mid != NULL)
			{
				if (CmpId(mess->pMId, oldMess.mid))
				{
					SFREE(oldMess.InfoStrings);
					return TRUE;
				}
			}
			SFREE(oldMess.InfoStrings);
		}
	}
	blockPtr->bits |= bOldDupe;
	data_WriteBlockCrc(handle, nr, blockPtr);
	return FALSE; /* Kein Dupe, aber gleiche ID */
}

/*** ---------------------------------------------------------------------- ***/

/* Text aus Textdatei in die Datenbank packen */
LOCAL F_ERROR Transfer(OneGroupHandle handle, mess_len *len, TEXTFILENO txt)
{
	_UBYTE scrap[4096];
	_UBYTE scrap2[4096+1];
	mess_len l;
	_WORD TabSize;
	F_ERROR error;
	
	ConfVars_GetConfDefInt(cTabSize, &TabSize, 4);
	*len = 0;
	do
	{
		mtTextfiles_ReadLine(txt, scrap, sizeof(scrap));
		mtTextfiles_ReadLn(txt);
		l = (mess_len)Str_ConvertTabs(scrap, scrap2, sizeof(scrap2) - 1, TabSize);
		scrap2[l++] = LF;
		if ((error = AppendToDat(handle, scrap2, l, NULL)) != NoError)
			return error;
		*len += l;
	} while (!mtTextfiles_EndofText(txt));
	return NoError;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *Str2Ptr(PtrRecord *mess, stringTypes str)
{
	if (str == bWegen)
		return mess->pWegen;
	if (str == bVon)
		return mess->pVon;
	if (str == bAn)
		return mess->pAn;
	if (str == bMId)
		return mess->pMId;
	if (str == bRId)
		return mess->pRId;
	if (str == bBox)
		return mess->pBox;
	if (str == bName)
		return mess->pName;
	if (str == bRefNr)
		return mess->pRefNr;
	if (str == bDistribution)
		return mess->pDistribution;
	if (str == bGate)
		return mess->pGate;
	if (str == bMime)
		return mess->pMime;
	if (str == bFollowupTo)
		return mess->pFollowupTo;
	if (str == bReplyTo)
		return mess->pReplyTo;
	if (str == bSender)
		return mess->pSender;
	ErrorOut(EO_ERROR, "Str2Ptr: unknown string type");
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL ITEMBITS item(stringTypes str)
{
	if (str == bVon)
		return itemVon;
	if (str == bAn)
		return itemAn;
	if (str == bMId)
		return itemMId;
	if (str == bRId)
		return itemRId;
	if (str == bBox)
		return itemBox;
	if (str == bName)
		return itemName;
	if (str == bRefNr)
		return itemRefNr;
	if (str == bDistribution)
		return itemDistribution;
	if (str == bGate)
		return itemGate;
	if (str == bMime)
		return itemMime;
	if (str == bFollowupTo)
		return itemFollowup;
	if (str == bReplyTo)
		return itemReplyTo;
	if (str == bSender)
		return itemSender;
	ErrorOut(EO_ERROR, "item: unknown string type");
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isMouse(CONST _UBYTE *ptr)
{
	return strcmp(ptr, ".maus.de") == 0;
}

/*** ---------------------------------------------------------------------- ***/

/* Eine Message bearbeiten/ abort -> sofort abbrechen, schwerer Fehler */
/* Returnwert zeigt an, ob die Nachricht geschrieben wurde */
/* Es koennen natuerlich nur die Typen own, personal, status und normal geschrieben */
/* werden. Bei Missbrauch: Garbage in possible crash out! */

GLOBAL _BOOL data_AppendMessage(
	OneGroupHandle handle,
	WhatsThat type,
	MSGBITS bitWishes,	/* Bits, die beim Einfuegen schon gesetzt werden sollen; bFiltered bisher */
	PtrRecord *mess,
	_BOOL *usenetChain,
	_BOOL *usenetOk,
	_BOOL *isOneDupe,
	_BOOL *isCommentToOwn,	/* Weil hier festgestellt wird, ob die Mail ein C2O ist (MH) */
	_BOOL *abort)
{
	pBlock block;
	_UWORD z;
	MSGNR nr;
	CRC16 CRC;
	pInfoType pInfo;
	dupeInfoType *pDupe;		/* Fuer redundante Daten in der *.DAT */
	data_len len[33], l2[33];	/* Darein kommt der Laengen-Header */
	stringTypes lauf;
	_UBYTE *str;			/* Allzweckpointer.. */
	_UBYTE saveChar;
	_UBYTE saveCh2;
	_BOOL inserted;			/* Flag, ob Nachricht mit RId eingefuegt wurde,
							 * ansonsten wird noch Wildwest versucht */
	_UBYTE *charPos;		/* beides fuer die Kommentarverkettung per RId */
	_UBYTE *ptRId;			/* ebenfalls fuer RId-Verkettung */
	MSGBITS addBits;		/* Zusaetzlich zu setzende Bits, die wg. VererbenFlag von der */
							/* Muttermsg uebernommen werden sollen                        */
	F_ERROR error;
	
	*usenetChain = FALSE;
	*usenetOk = FALSE;
	*isOneDupe = FALSE;
	*isCommentToOwn = FALSE;	/* (MH) */
	/* default: Keine Usenetverkettung, hat also auch nicht geklappt */
	*abort = FALSE; /* Es geht nichts schief.. :-) */
	/* bId sollte immer gesetzt sein */
	
	if (type == msg_status)		/**  Statusmeldung **/
	{
		if (SearchID(handle, mess->pId, 0, TRUE, &nr))
			if (SetState(handle, nr, data_mDateStr2Long(mess->pBSZeit, 1), mess->pBSZeit[0]) != NoError)
			{
				*abort = TRUE;
				return FALSE;
			}
	} else if (type == msg_personal || type == msg_normal || type == msg_own)
	{
		/* Die crc-Pruefsumme ueber den Block wird weiter unten berechnet */
		if (mess->whatsThere & bEZeit)
		{
			block.Datum = data_mDateStr2Long(mess->pEZeit, 0);
		} else
		{
			block.Datum = ConvertDate_CurrentDate();
			/*
			GetActualDate(Datum);
			*/
		}
		/* Das Datum wird fuer den Dupecheck jetzt schon benoetigt. */
		
		if (isDupe(handle, mess, block.Datum, type))
		{
			if (type != msg_own)
			{
				CatLog_WriteStringNTime(mess->pId);
				CatLog_WriteLine(" ist ein Dupe und wird nicht eingef\201gt");
			}
			*isOneDupe = TRUE;
			return FALSE;
		}
		if (handle->anz >= dataSys_notSaved)
		{
			CatLog_WriteStringNTime(mess->pId);
			CatLog_WriteLineNTime(" wird nicht eingef\201gt, weil Gruppe voll ist");
			return FALSE;
		}


		/* Die crc-Pruefsumme ueber den Block wird weiter unten berechnet */
		block.bits = bitWishes;
		/* Erst einmal sicherheitshalber zuweisen */
		block.upMess  = dataSys_empty;
		block.leftMess = dataSys_empty;
		addBits = 0;
		if (mess->whatsThere & bRefNr)		/* Jetzt evtl. Kommentarverkettung */
		{
			InsertKom(handle, handle->anz, FALSE, &addBits, &block.upMess, &block.leftMess, mess->pRefNr, abort);
			if (*abort)
				return FALSE;
		} else if (mess->whatsThere & bRId)
		{
			/* Zuerst die Position vom @ finden */
			inserted = FALSE;
			charPos = strchr(mess->pRId, '@');
			if (charPos != NULL)
			{
				charPos = strchr(charPos, '.');
				if (charPos != NULL && isMouse(charPos))
				{
					saveChar = *charPos;
					*charPos = '\0'; /* das xxx.maus.de abschneiden */
					/* Neue IDs (kann ich von ausgehen!), jetzt vorne Teil vor . suchen */
					ptRId = strchr(mess->pRId, '.');
					if (ptRId != NULL)
					{
						/* . vorne gefunden, wir setzen den Zeiger dahinter! */
						saveCh2 = *++ptRId;
						*ptRId = ucase(*ptRId);
					} else
					{
						ptRId = mess->pRId;
						saveCh2 = '\0';
					}
					/* Und jetzt Kommentar einfuegen */
					InsertKom(handle, handle->anz, FALSE, &addBits, &block.upMess, &block.leftMess, ptRId, abort);
					inserted = TRUE;
					*charPos = saveChar; /* ..und wieder herstellen.. */
					if (saveCh2 != '\0')
						*ptRId = saveCh2;
				} else if (usenetChaining)		/* soll die Usenetverkettung hergestellt werden? */
				{
					*usenetChain = TRUE;
					InsertKom(handle, handle->anz, TRUE, &addBits, &block.upMess, &block.leftMess, mess->pRId, abort);
					/* Verkettung mit Usenet-ID herstellen, langsam.. */
					*usenetOk = block.upMess != dataSys_notSaved;
					inserted = *usenetOk;
				}
			}
			/* Neu fuer Wildwestverkettungen bei Mails mit RId */
			if (!inserted && (mess->whatsThere & bText) &&
				GroupComment_PrepareID(mess->pText, &str, &saveChar, &charPos))
			{
				/* Verkettung anhand der ersten Msgzeile, nur fuer den Notfall, deswegen */
				/* hier am Ende, mit niedrigster Prioritaet.                             */
				InsertKom(handle, handle->anz, FALSE, &addBits, &block.upMess, &block.leftMess, str, abort);
				*charPos = saveChar; /* wurde in diesem Fall in PrepareID geaendert. */
			}
		} else if ((mess->whatsThere & bText) &&
			GroupComment_PrepareID(mess->pText, &str, &saveChar, &charPos))
		{
			/* Verkettung anhand der ersten Msgzeile, nur fuer den Notfall, deswegen */
			/* hier am Ende, mit niedrigster Prioritaet.                             */
			/* Schneller als Usenet-Verkettung, aber Usenetter haben diese Zeile nicht! */
			InsertKom(handle, handle->anz, FALSE, &addBits, &block.upMess, &block.leftMess, str, abort);
			*charPos = saveChar; /* wurde in diesem Fall in PrepareID geaendert. */
		}
		block.bits |= addBits; /* Vererben-Flag */
		block.downMess = dataSys_empty;
		block.rightMess = dataSys_empty;
		block.KomCount = 0;
		
		/* Flag nun setzen (MH) */
		if (block.bits & bComToOwnMessage)	
		{
			*isCommentToOwn = TRUE;
		}

		/* Hier wird der Header in der *.DAT erstellt */
		block.idLength = (data_len)strlen(mess->pId) + 1;
		block.hLength = block.idLength;
		if (block.hLength & 1)
			++block.hLength;
		/* Damit man auf die Tabelle auch auf 68000 zugreifen kann */
		/* Zunaechst einmal zur Sicherheit, das kann man spaeter ja  */
		/* auch ohne Codeaenderung wieder weglassen                 */
		if ((error = AppendToDat(handle, mess->pId, block.hLength, &block.Start)) != NoError)
		{
			*abort = TRUE;
			insert_Abort(2, error, handle->anz, handle->group);
			return FALSE;
		}

		block.items = 0;
		/* Hier kommt jetzt die Tabelle der Laengen hin */
		l2[1] = block.hLength;
		len[0] = 0; /* Anzahl der Strings, Wegen ist da! */
		for (lauf = bWegen; lauf <= bSender; lauf <<= 1)
		{
			/* bWegen, bVon, bAn, bMId, bRId, bBox, bName, bRefNr, bDistribution, bGate, bMime, bFollowupTo, bReplyTo, bSender */
			if (mess->whatsThere & lauf)
			{
				data_len l;
				
				++len[0];          /* Wieder einen String gefunden */
				str = Str2Ptr(mess, lauf); /* Pointer abrufen */
				l = (data_len)strlen(str) + 1;  /* Laenge inklusive Nullbyte bestimmen */
				block.hLength += l;      /* der Header ist laenger geworden */
				len[len[0]] = l;     /* Stringlaenge merken */
				if (lauf != bWegen)
				{
					block.items |= item(lauf); /* und auch im Parameterblock merken */
				}
			}
		}
		/* DEBUG: DIE LAENGEN SIND KORREKT */
		/* Laenge der Tabelle auf die Startposition addieren: */
		l2[1] += (data_len)((len[0] + 1) * sizeof(data_offset)); /* hLength steht schon drin */
		block.hLength += (data_len)((len[0] + 1) * sizeof(data_offset));
		l2[0] = len[0];
		for (z = 2; z <= len[0]; z++)
			l2[z] = len[z-1] + l2[z-1]; /* aus Laengen Startpositionen machen */
		IfNotBigEndian
		{
			for (z = 0; z <= len[0]; z++)
				l2[z] = h2ns(l2[z]);
		}
		/* Tabelle mit den StringLaengen */
		if ((error = AppendToDat(handle, l2, (len[0] + 1 ) * sizeof(data_offset), NULL)) != NoError)
		{
			*abort = TRUE;
			insert_Abort(2, error, handle->anz, handle->group);
			return FALSE;
		}


		/* Evtl. noch ueber einen Puffer laufen lassen?   */
		/* Da aber evtl. die Ausgabe schon gepuffert ist */
		/* macht das auch nicht mehr viel aus            */
		z = 1; /* Position in len */
		for (lauf = bWegen; lauf <= bSender; lauf <<= 1)
		{
			/* bWegen, bVon, bAn, bMId, bRId, bBox, bName, bRefNr, bDistribution, bGate, bMime, bFollowupTo, bReplyTo, bSender */
			if (mess->whatsThere & lauf)
			{
				if ((error = AppendToDat(handle, Str2Ptr(mess, lauf), len[z], NULL)) != NoError)
				{
					*abort = TRUE;
					insert_Abort(2, error, handle->anz, handle->group);
					return FALSE;
				}
				++z;
			}
		}
			
		/* Hier muessen notfalls die unbekannten Zeilen verarbeitet werden */
		
		/* ..und hier kommt noch bei privaten die private-Bytes, logisch, nech? Arararahhh */
		
 		if (handle->group == dataSys_private)
 		{
			if (type == msg_personal)
			{
				pInfo.LeseDatum = h2nl(data_mDateStr2Long(mess->pBSZeit, 1));
				pInfo.Status = mess->pBSZeit[0];
				if (bitWishes & bOwnMessage)
				{
					/* bitWishes &= ~bOwnMessage; */
					block.bits &= ~bOwnMessage;
					pInfo.locked = 1;
				} else
				{
					pInfo.locked = 0; /* Da eigene Msgs auch hierher kommen, muss hier nochwas neues her */
				}
			} else if (type == msg_own)
			{
				pInfo.LeseDatum = h2nl(ConvertDate_CurrentDate());
				pInfo.Status = 'N';
				pInfo.locked = 1;
				if (mess->whatsThere & bBSZeit)
				{
					pInfo.LeseDatum = h2nl(data_mDateStr2Long(mess->pBSZeit, 1));
					pInfo.Status = mess->pBSZeit[0];
				}
			}
			if (block.hLength & 1)	/* alignen */
			{
				++block.hLength;
				saveChar = fillBytePersonal;
				if ((error = AppendToDat(handle, &saveChar, 1, NULL)) != NoError)
				{
					*abort = TRUE;
					insert_Abort(2, error, handle->anz, handle->group);
					return FALSE;
				}
			}
			if ((error = AppendToDat(handle, &pInfo, sizeof(pInfoType), NULL)) != NoError)
			{
				*abort = TRUE;
				insert_Abort(2, error, handle->anz, handle->group);
				return FALSE;
			}
			block.items |= itemPrivateBytes;
			block.hLength += (data_len)sizeof(pInfoType);
		} else if (mess->whatsThere & bBSZeit)
		{
			/* Status-Zeile bei OMs bei Laberfilter RAUS */
			if (mess->pBSZeit[0] == 'G')
			{
				block.bits |= bGelesen;
			} else if (mess->pBSZeit[0] == 'F')
			{
				block.bits |= bFiltered;
			}
		}
		/* ..und schon haben wir uns einen schoenen Header gebastelt */

		if (mess->whatsThere & bTextDatei)
		{
			/* Dann steht der Text in einer Datei */
			if ((error = Transfer(handle, &block.Length, mess->txt)) != NoError)
			{
				*abort = TRUE;
				insert_Abort(2, error, handle->anz, handle->group);
				return FALSE;
			}
		} else if (mess->whatsThere & bText)
		{
             /* Sonst wurde er per Pointer uebergeben */
			block.Length = mess->TextMax; /* Auf Nullbyte am Ende achten! */
			if ((error = AppendToDat(handle, mess->pText, block.Length, NULL)) != NoError)
			{
				*abort = TRUE;
				insert_Abort(2, error, handle->anz, handle->group);
				return FALSE;
			}
		} else
		{
			/* Uahh! Wo ist der Text geblieben? */
			block.Length = 0;
			SPEC_DEBUG_OUT((EO_ERROR, "no message text, msgNr %u, group %u", handle->anz, handle->group));
		}
		
		CRC = ZCalcCrc_CalcIdCrc(mess->pId);
		if ((error = AppendCrc(handle, CRC)) != NoError)
		{
			*abort = TRUE;
			insert_Abort(0, error, handle->anz, handle->group);
			return FALSE;
		}
		
		/* Achtung, Reihenfolge mit AppendBlock wg. anz-Erhoehung wichtig */
		
		block.crc = BLOCK_CRC(&block);
		/* Neuerdings wird SICHERHEIT grossgeschrieben :-) */
		
		/* .. und an die halbfertige *.PAR anhaengen */
		if ((error = AppendBlock(handle, &block)) != NoError)
		{
			*abort = TRUE;
			insert_Abort(1, error, handle->anz, handle->group);
			return FALSE;
		}
		
		/* Diese beiden Schreiboperationen muessen evtl. noch umgedreht werden! */
		pDupe = (dupeInfoType *)(&block.Datum);
		pDupe->setTerminator = Terminator;
		switch_dupe(pDupe);
		if ((error = AppendToDat(handle, pDupe, sizeof(dupeInfoType), NULL)) != NoError)
		{
			*abort = TRUE;
			insert_Abort(2, error, handle->anz - 1, handle->group);
			return FALSE;
		}
	}
	
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Fehlermeldung zu dieser Nachricht in catlog.txt ausgeben */
GLOBAL _VOID data_dumpMess(CONST PtrRecord *mess)
{
	_UBYTE tmp[256];
	size_t copyLen;

	CatLog_WriteLineNTime("Fehler im Outfile:");
	/*
	CatLog_WriteLine("- data.c: Melde mich wg. dingenskirchen -");
	*/
	CatLog_WriteLine("illegales OUTFILE-Format");
	CatLog_WriteLine("folgende Zeilen wurden gefunden:");
	if (mess->whatsThere & bId           ) { CatLog_WriteString("Id    : ");      CatLog_WriteLine(mess->pId); }
	if (mess->whatsThere & bRefNr        ) { CatLog_WriteString("RefNr : ");      CatLog_WriteLine(mess->pRefNr); }
	if (mess->whatsThere & bVon          ) { CatLog_WriteString("Von   : ");      CatLog_WriteLine(mess->pVon); }
	if (mess->whatsThere & bAn           ) { CatLog_WriteString("An    : ");      CatLog_WriteLine(mess->pAn); }
	if (mess->whatsThere & bWegen        ) { CatLog_WriteString("Wegen : ");      CatLog_WriteLine(mess->pWegen); }
	if (mess->whatsThere & bEZeit        ) { CatLog_WriteString("EZeit : ");      CatLog_WriteLine(mess->pEZeit); }
	if (mess->whatsThere & bGruppe       ) { CatLog_WriteString("Gruppe: ");      CatLog_WriteLine(mess->pGruppe); }
	if (mess->whatsThere & bBSZeit       ) { CatLog_WriteString("BSZeit: ");      CatLog_WriteLine(mess->pBSZeit); }
	if (mess->whatsThere & bMId          ) { CatLog_WriteString("MId   : ");      CatLog_WriteLine(mess->pMId); }
	if (mess->whatsThere & bRId          ) { CatLog_WriteString("RId   : ");      CatLog_WriteLine(mess->pRId); }
	if (mess->whatsThere & bBox          ) { CatLog_WriteString("Box   : ");      CatLog_WriteLine(mess->pBox); }
	if (mess->whatsThere & bName         ) { CatLog_WriteString("Name  : ");      CatLog_WriteLine(mess->pName); }
	if (mess->whatsThere & bGate         ) { CatLog_WriteString("Gate  : ");      CatLog_WriteLine(mess->pGate); }
	if (mess->whatsThere & bMime         ) { CatLog_WriteString("MIME  : ");      CatLog_WriteLine(mess->pMime); }
	if (mess->whatsThere & bFollowupTo   ) { CatLog_WriteString("Followup-To: "); CatLog_WriteLine(mess->pFollowupTo); }
	if (mess->whatsThere & bReplyTo      ) { CatLog_WriteString("Reply-To: ");    CatLog_WriteLine(mess->pReplyTo); }
	if (mess->whatsThere & bSender       ) { CatLog_WriteString("Sender  : ");    CatLog_WriteLine(mess->pSender); }
	if (mess->whatsThere & bDistribution ) { CatLog_WriteString("Distr.: ");      CatLog_WriteLine(mess->pDistribution); }
	if (mess->whatsThere & bText)
	{
		CatLog_WriteLine("Text gefunden:");
		copyLen = mess->TextMax + 1;
		if (copyLen > sizeof(tmp))
			copyLen = sizeof(tmp);
		strMcpy(tmp, copyLen, mess->pText);
		CatLog_WriteLine(tmp);
	}
	if (mess->whatsThere & bTextDatei) { CatLog_WriteLine("Text in Datei \201bergeben"); }
	CatLog_WriteLine("- data.c: Ende der Durchsage -------------");
	/*
	MTE_warnAlert(mtAlerts_Alert(1, AL_CORE_DUMPED) != 1, AL_TRANSLATION, NULL, NULL);
	*/
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Versucht den fuer den naechsten Einfuegevorgang noetigen Platz zu schaetzen */
GLOBAL _ULONG data_EstimateNecessaryMemory(MSGNR additional)
{
	_ULONG maxTab;
	_ULONG maxPar;
	_ULONG maxAmount;
	Directory *dir;
	DirectoryEntry *entry;
	_UBYTE name[PATH_MAX];

	/* !!! name-matching konnte	hier nicht funktionieren */
	maxPar = 0;
	maxTab = 0;
	dir = D_Opendir(DataPath);
	if (dir != NULL)
	{
		while ((entry = D_Readdir(dir)) != NULL)
		{
			strBcpy(name, entry->d_name);
			strLwr(name);
			if (WildCards_NameMatching(name, tabWild))
			{
				if (entry->d_st.st_size > maxPar)
					maxPar = entry->d_st.st_size;
			} else if (WildCards_NameMatching(name, tabWild))
			{
				if (entry->d_st.st_size > maxTab)
					maxTab = entry->d_st.st_size;
			}
		}
		D_Closedir(dir);
		maxAmount = maxTab + maxPar;
	} else
	{
		maxAmount = 32000; /* min size */
	}
	return maxAmount + minDatBuffer + sizeof(pBlock) * additional + sizeof(CRC16) * additional;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* --- Baum durchlaufen --- */

typedef struct {
	OneGroupHandle handle;
	pBlock s;
	MSGNR max;
	treeProc proc;
	_UWORD maxDepth;
	_VOID *private;
} walkInfo;

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL doWalk(walkInfo *info, MSGNR idx)
{
	pBlock *pb;
	
	do
	{
		pb = &info->s;
		if (data_ReadBlockCrc(info->handle, idx, &pb) != NoError)
			return FALSE;
		if (!info->proc(info->handle, idx, pb, info->private))
			return FALSE;
		idx = pb->rightMess;
		if (pb->downMess <= info->max)
		{
			if (!doWalk(info, pb->downMess))
				return FALSE;
		}
	} while (idx <= info->max);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Allgemeine Prozedur, um einen Baum zu durchlaufen */
GLOBAL _VOID data_WalkTree(OneGroupHandle handle, MSGNR mess, _BOOL fromTop, treeProc proc, _VOID *private)
{
	walkInfo info;
	MSGNR idx;
	pBlock *pb;
	F_ERROR err;

	idx = mess;
	info.max = handle->anz;
	if (info.max == 0)
		return;
	--info.max;
	info.handle = handle;
	info.proc = proc;
	info.private = private;
	if (fromTop)
	{
		/* Erste Nachricht im Baum suchen */
		do
		{
			pb = &info.s;
			err = data_ReadBlockCrc(handle, idx, &pb);
			if (err == NoError)
			{
				if (pb->upMess <= info.max)
					idx = pb->upMess;
			}
		} while (err == NoError && pb->upMess <= info.max);
	}
	if (!fromTop)
	{
		/* nur "runter" loeschen: aktuelle Msg behandeln und dann nur noch Tochtermsgs */
		pb = &info.s;
		err = data_ReadBlockCrc(handle, idx, &pb);
		if (err == NoError)
		{
			if (proc(handle, idx, pb, private) &&
				pb->downMess <= info.max)
			{
				doWalk(&info, pb->downMess);
			}
		}
	} else
	{
		doWalk(&info, idx);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL doWalk2(walkInfo *info, MSGNR idx, _UWORD depth)
{
	pBlock *pb;
	
	do
	{
		pb = &info->s;
		if (data_ReadBlockCrc(info->handle, idx, &pb) != NoError)
			return FALSE;
		if (!info->proc(info->handle, idx, pb, info->private))
			return FALSE;
		idx = pb->rightMess;
		if (pb->downMess < dataSys_notSaved && depth < info->maxDepth)
		{
			if (!doWalk2(info, pb->downMess, depth + 1))
				return FALSE;
		}
	} while (idx <= info->max);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Etwas weniger allgemeine Prozedur, um einen Baum zu durchlaufen.
 * Es wird maximal um maxUp Ebenen nach oben gegangen, und von dort aus
 * maximal maxDepth Ebenen nach unten
 */
GLOBAL _VOID data_WalkTree2(OneGroupHandle handle, MSGNR mess, _WORD maxUp, _WORD maxDepth, treeProc proc, _VOID *private)
{
	walkInfo info;
	MSGNR idx;
	pBlock *pb;
	F_ERROR err;
	_WORD upCount;
	
	idx = mess;
	info.max = handle->anz;
	if (info.max == 0)
		return;
	--info.max;
	info.handle = handle;
	info.proc = proc;
	info.maxDepth = maxDepth;
	info.private = private;
	/* Nach oben gehen, maximal maxUp mal */
	upCount = 0;
	pb = &info.s;
	err = data_ReadBlock(handle, idx, &pb);
	if (err == NoError)
		if (pb->downMess >= dataSys_notSaved)
		{
			/* In dem Fall korrigieren wir maxUp */
			maxUp = maxDepth - 1;
		}
	do
	{
		pb = &info.s;
		err = data_ReadBlock(handle, idx, &pb);
		if (err == NoError)
		{
			if (pb->upMess < dataSys_notSaved)
				idx = pb->upMess;
		}
		++upCount;
	} while (pb->upMess < dataSys_notSaved && upCount < maxUp);
	/* nur "runter" loeschen: aktuelle Msg behandeln und dann nur noch Tochtermsgs */
	pb = &info.s;
	err = data_ReadBlockCrc(handle, idx, &pb);
	if (err == NoError)
	{
		if (proc(handle, idx, pb, private) &&
			pb->downMess != dataSys_empty &&
			pb->downMess != dataSys_notSaved)
		{
			doWalk2(&info, pb->downMess, 1);
		}
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* --- Prozeduren fuer die Flagsbehandlung eines Baumes --- */

typedef struct {
	updateProc update;
	flagchangeProc fc;
	CONST changeInfo *changeInfo;
} treeDeleteInfo;

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL helpTreeDelete(OneGroupHandle handle, MSGNR idx, pBlock *pb, _VOID *private)
{
	treeDeleteInfo *info;
	MSGBITS newflags;
	
	info = private;
	newflags = info->fc(pb->bits, info->changeInfo);
	if (newflags != pb->bits)
	{
		pb->bits = newflags;
		if (data_WriteBlockCrc(handle, idx, pb) != NoError)
			return FALSE;
		info->update(idx, pb->bits);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Flags eines Baumes bearbeiten */
GLOBAL _VOID data_TreeFlags(OneGroupHandle handle, MSGNR mess, _BOOL fromTop, updateProc updt, flagchangeProc flagchange, CONST changeInfo *changeInfo)
{
	treeDeleteInfo info;
	
	info.update = updt;
	info.fc = flagchange;
	info.changeInfo = changeInfo;
	data_WalkTree(handle, mess, fromTop, helpTreeDelete, &info);
}

