/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


/*
 * 2.04 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.34a'
 *	Cat bis V5.33 gab defaultmig falsche Kommandos an In2Cat,
 * 	PN statt PA, und hnlich mit PNV,ON und ONV. Ab 5.34 gibt
 *	Cat die richtigen Befehle, und In2Cat versteht vorsichtshalber
 * 	beide.
 */ 

#define __ITK_IMPLEMENTATION__

#include <doitk.h>
#include <itkmacro.h>
#include <catgloba.h>
#include <cattypes.h>
#include <catfiles.h>
#include <infofile.h>
#include <mte.h>
#include <maustaus.h>
#include <listhelp.h>
#include <grin.h>
#include <wdwmanag.h>
#include <convertd.h>
#include <mtpaths.h>
#include <catutil.h>
#include <confvars.h>
#include <varname4.h>
#include <userinfo.h>
#include <cathelp.h>
#include <quicksor.h>
#include <protokol.h>
#include <adr.h>
#include <groupsel.h>
#include <vdiutil.h>
#include <mtpopups.h>

#include <mttextfi.h>
#include <mtappl.h>
#include <mtutils.h>
#include <mtdials.h>
#include <magicstr.h>
#include <portvdi.h>
#include <routines.h>

#include <strconv.h>

#include <set.h>
#include <ro_mem.h>
#include <routinec.h>
#include <file_io.h>
#include <mtrsc.h>


#define SORT_CMD_GROUPS  0
#define SORT_CMD_ALPHA   1

#define T_NORMAL      0
#define T_HELP        1
#define T_PREVIEW     2
#define T_OK          3
#define T_CANCEL      4
#define T_SKIP_NEXT   5
#define T_SKIP_PREV   6
#define T_GROUP       7
#define T_CHIEF_GROUP 8
#define T_PT_GROUP    9
#define T_USER        10
#define T_POPUP       11
#define T_CIRCLE      12


typedef struct _tIntCmdType
{
	_WORD intIdent;
	_WORD intVersion;
} tIntCmdType;

typedef struct _tCommandGroupName
{
	_UBYTE cmdGroupId[8];				/* Gruppe des Kommandos, Sortierkriterium */
	_UBYTE cmdGroupTitle[256];			/* Bezeichnung */
} tCommandGroupName;


#define Box            ROOT
#define Mover          1
#define Header         2				/* Ueberschrift */

LOCAL tIntCmdType internalCmds[] =
{
	{ 36, 0 }							/* G+,G-: Gruppen bestellen/abstellen */
};
#define maxInternalCmds (sizeof(internalCmds) / sizeof(internalCmds[0]))

LOCAL tCommandGroupName groupNames[] =	/* (MH) */
{
	{ "A",     "Administrative Userdaten"  },
	{ "B",     "Boxkonfiguration"          },
	{ "F",     "FE-Features und Technik"   },
	{ "G",     "Gruppendaten"              },
 	{ "Makro", "Makros"                    },
	{ "P",     "Programmteil"              },
	{ "T",     "Tausch-Optionen"           },
	{ "U",     "Userdaten"                 },
	{ "?",     "Unbekannte Kommandogruppe" }
};
#define MaxGroupNames (sizeof(groupNames) / sizeof(groupNames[0]))

LOCAL _UBYTE userDefault[256];
LOCAL _UBYTE groupDefault[256];
LOCAL _UBYTE ptgrpDefault[256];
LOCAL _UBYTE chgrpDefault[256];

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----------------------------------------------------------------------
 * Zusatzfunktionen fuer die Listen
 *----------------------------------------------------------------------*/

LOCAL _VOID delEnum(_VOID *entry)
{
	SFREE(entry);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID delParam(_VOID *entry)
{
	OneParam *pParam;

	pParam = entry;
	if (pParam->hasEnums)
		Lists_DeleteSimpleList(&pParam->paramEnum, delEnum, TRUE);
	if (pParam->paramHelp != NULL)
		SFREE(pParam->paramHelp);
	SFREE(entry);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID delCmd(_VOID *entry)
{
	OneCommand *pCmd;

	pCmd = entry;
	if (pCmd->hasParms)
		Lists_DeleteSimpleList(&pCmd->cmdParms, delParam, TRUE);
	if (pCmd->macro)
	{
		Lists_DeleteSimpleList(pCmd->itkMacro, mem_free, TRUE);
		OFREE(pCmd->itkMacro);
	}
	if (pCmd->cmdHelp != NULL)
		SFREE(pCmd->cmdHelp);
	SFREE(entry);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID GetEnumValue(_WORD idx, CONST _UBYTE *str, _UBYTE *target, size_t strsize)
{
	_WORD count;
	CONST _UBYTE *lastPos;
	CONST _UBYTE *pos;
	_UBYTE val[256];
	size_t len;
	
	count = 0;
	pos = str;
	lastPos = str;
	while (pos != NULL)
	{
		pos = strChr(lastPos, ':');
		if (count == idx && pos != NULL)
		{
			len = pos - lastPos + 1;
			if (len > sizeof(val))
				len = sizeof(val);
			strMcpy(val, len, lastPos);
			strMcat(target, strsize, val);
			return;
		}
		if (pos == NULL && count == idx)
		{
			/* letzter Parameter oder nur ein Wert */
			len = &str[strLen(str)] - lastPos + 1;
			if (len > sizeof(val))
				len = sizeof(val);
			strMcpy(val, len, lastPos);
			strMcat(target, strsize, val);
			return;
		}
		lastPos = pos + 1;
		++count;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL getCmdName(List *cmdList, List *macroList)
{
	OneCommand *pCmd;
	OneMacro *pMacro;
	List tempList;
	_BOOL found;
	
	/* Makroliste abklappern */
	tempList = *cmdList;	
	Lists_ResetList(macroList);
	while ((pMacro = Lists_NextEntry(macroList)) != NULL)
	{
		/* Kommandoliste nach Kommandonummer und -version durchsuchen */
		found = FALSE;
		Lists_ResetList(&tempList);
		while ((pCmd = Lists_NextEntry(&tempList)) != NULL)
		{
			if (pCmd->cmdIdent == pMacro->command &&
				pCmd->cmdVersion == pMacro->version)
			{
				pMacro->cmdName = pCmd->cmdName;
				pMacro->pCommand = pCmd;
				found = TRUE;
				break;
			}
		}
		if (!found)
			return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *getSelected(List *list)
{
	OneCommand *pCmd;
	
	Lists_ResetList(list);
	while ((pCmd = Lists_NextEntry(list)) != NULL)
	{
		if (pCmd->selected)
			return pCmd;
	}
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID DoITK_InitMacroList(List *cmdList)
{
	OneCommand *pCmd;
	List *list;
	
	list = cmdList;
	Lists_ResetList(list);
	while ((pCmd = Lists_NextEntry(list)) != NULL)
	{
		if (pCmd->macro)
		{
			/* Kommandonamen zuweisen. Falls das Kommando nicht gefunden
			   wurde, dann Makro loeschen */
			if (!getCmdName(list, pCmd->itkMacro))
			{
				Lists_RemoveEntry(list, delCmd);
			} else
			{
				/* Makroliste neu durchnumerieren */
				Macro_RenumberSteps(pCmd->itkMacro);
			}
		}
	}
}	

/*----------------------------------------------------------------------
 * Interne Stringfunktionen
 *----------------------------------------------------------------------*/

LOCAL _VOID StripShortcut(CONST _UBYTE *paramName, _UBYTE *pName, size_t strsize)
{
	_UBYTE *p;

	strMcpy(pName, strsize, paramName);
	p = strChr(pName, '[');
	if (p != NULL)
		strCpy(p, p + 1);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CalcWidth(ITK *itk)
{
	OneCommand *pCmd;
	GRECT r;
	_WORD extent[8];
	
	/* Maximale Breite eines Listeneintrages herausfinden */
	itk->globalLength = 0;
	itk->groupNameLength = 0;
	Lists_ResetList(itk->itkCmds);
	while ((pCmd = Lists_NextEntry(itk->itkCmds)) != NULL)
	{
		if (pCmd->entryVisible)
		{
			vqt_extent(PrivateWS, pCmd->cmdName, extent);
			itk->globalLength = Max(extent[2] - extent[0], itk->globalLength);
			vqt_extent(PrivateWS, pCmd->cmdGroup, extent);
			itk->groupNameLength = Max(extent[2] - extent[0], itk->groupNameLength);
		}
	}
	itk->globalLength += (2 + 1) * CharWidth + itk->groupNameLength;
	CalcArea(itk->CmdBox, CCMDBOX, &r);
	CalcArea(itk->CmdBox, CCMDBOX, &r);
	itk->globalLength = Max(itk->globalLength, r.g_w);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----------------------------------------------------------------------
 * Hilfe zu einem Kommando zusammenbauen
 *----------------------------------------------------------------------*/
#define helpLineLength 65

LOCAL _BOOL AddToHelp(CONST _UBYTE *str, List *helpList)
{
	_UBYTE *helpLine;
	size_t fromPos;
	size_t toPos;
	size_t len;
	size_t alloc;
	
	if (str != NULL && str[0] != '\0')
	{
		fromPos = 0;
		toPos = len = strLen(str);
		while (fromPos < toPos)
		{
			if (toPos - fromPos >= helpLineLength)
			{
				/* Umbruch an Wortgrenze durchfuehren */
				toPos = fromPos + helpLineLength - 1;
				if (toPos > len)
					toPos = len - 1;
				while (str[toPos] != ' ' && toPos > fromPos)
					--toPos;
				if (toPos == fromPos)
				{
					toPos = fromPos + helpLineLength - 1;
					if (toPos > len)
						toPos = len;
				}
			}
			alloc = toPos - fromPos + 1;
			helpLine = MALLOC(alloc, "AddToHelp");
			if (helpLine == NULL)
			{
				Lists_DeleteSimpleList(helpList, delEnum, TRUE);
				return FALSE;
			}
			MemCpy(helpLine, str + fromPos, alloc - 1);
			helpLine[alloc-1] = '\0';
			if (!Lists_AppendEntry(helpList, helpLine))
			{
				SFREE(helpLine);
				Lists_DeleteSimpleList(helpList, delEnum, TRUE);
				return FALSE;
			}
			fromPos = toPos + 1;
			toPos = len;
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL AddParmHelp(OneParam *pParm, List *helpList)
{
	_UBYTE pName[256];

	if (pParm->paramName[0] != '\0')
	{
		StripShortcut(pParm->paramName, pName, sizeof(pName));
		Insert("Parameter: ", pName, sizeof(pName), 0);
		if (!AddToHelp(pName, helpList))
			return FALSE;
		if (!AddToHelp(" ", helpList))
			return FALSE;
	}
	if (pParm->paramHelp != NULL)
	{
		if (!AddToHelp(pParm->paramHelp, helpList))
			return FALSE;
		if (!AddToHelp(" ", helpList))
			return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL BuildHelp(OneCommand *pCmd, List *helpList)
{
	OneParam *pParm;

	if (!Lists_CreateList(helpList, "BuildHelp"))
		return FALSE;
	if (pCmd->cmdHelp != NULL)
	{
		if (!AddToHelp(pCmd->cmdName, helpList))
			return FALSE;
		if (!AddToHelp(" ", helpList))
			return FALSE;
		if (!AddToHelp(pCmd->cmdHelp, helpList))
			return FALSE;
		if (!AddToHelp(" ", helpList))
			return FALSE;
	}
	if (pCmd->hasParms)
	{
		Lists_ResetList(&pCmd->cmdParms);
		while ((pParm = Lists_NextEntry(&pCmd->cmdParms)) != NULL)
			if (!AddParmHelp(pParm, helpList))
				return FALSE;
	}
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----------------------------------------------------------------------
 * ITK Dialogboxen zusammenbauen
 *----------------------------------------------------------------------*/
/*DYJ 5.21b */
LOCAL _BOOL SetObjc(
	ITK *itk, _WORD parent, _WORD typ,
	_WORD x, _WORD y, _WORD w, _WORD h,
	_UWORD f, _UWORD s,
	_VOID *spec)
{
	if (itk->currObIdx >= cMaxObjs)
	{
		MTE_infoRSC(AL_TOOMANY_OBJECTS);
		return FALSE;
	}
	itk->Tree[itk->currObIdx].ob_next = NIL;
	itk->Tree[itk->currObIdx].ob_head = NIL;
	itk->Tree[itk->currObIdx].ob_tail = NIL;
	itk->Tree[itk->currObIdx].ob_type = typ;
	itk->Tree[itk->currObIdx].ob_flags = f;
	itk->Tree[itk->currObIdx].ob_state = s;
	itk->Tree[itk->currObIdx].ob_spec.free_string = spec;
	itk->Tree[itk->currObIdx].ob_x = x;
	itk->Tree[itk->currObIdx].ob_y = y;
	itk->Tree[itk->currObIdx].ob_width = w;
	itk->Tree[itk->currObIdx].ob_height = h;
	itk->type[itk->currObIdx] = T_NORMAL;
	objc_add(itk->Tree, parent, itk->currObIdx);
	itk->Tree[Box].ob_width = Max(x + w, itk->Tree[Box].ob_width);
	itk->Tree[Box].ob_height = Max(y + h, itk->Tree[Box].ob_height);
	++itk->currObIdx;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID PrepareTree(ITK *itk, _UBYTE *title)
{
	itk->Tree[Box].ob_next = NIL;
	itk->Tree[Box].ob_head = NIL;
	itk->Tree[Box].ob_tail = NIL;
	itk->Tree[Box].ob_type = G_BOX;
	itk->Tree[Box].ob_flags = FL3DBAK;
	itk->Tree[Box].ob_state = OUTLINED;
	OBSPEC_SET_OBSPEC(itk->Tree[Box].ob_spec, '\0', 2, BLUE, BLACK, FALSE, IP_HOLLOW, WHITE);
	itk->Tree[Box].ob_x = 0;
	itk->Tree[Box].ob_y = 0;
	itk->Tree[Box].ob_width = 0;
	itk->Tree[Box].ob_height = 0;
	itk->currObIdx = 1;
	itk->tedIdx = 0;
	
	/* Flugecke */
	SetObjc(itk, Box, (MoveBox << 8) | G_IBOX, 0, 0, 16, 16, NONE, OUTLINED, NULL);
	OBSPEC_SET_OBSPEC(itk->Tree[itk->currObIdx - 1].ob_spec, '\0', 1, BLUE, BLACK, FALSE, IP_HOLLOW, WHITE);
	
	/* Titelzeile hinzufuegen */
	SetObjc(itk, Box, (SpecText << 8) | G_STRING, 2 * CharWidth, CharHeight, (_WORD)strLen(title) * CharWidth, 0, NONE, SHADOWED, title);
	itk->lastyPos = (itk->currObIdx - 1) * CharHeight;
	
	setclr(itk->usedKeys);
	setincl(itk->usedKeys, 'A');		/* fuer "Abbruch" */
	setincl(itk->usedKeys, 'O');		/* fuer "OK" */
	setincl(itk->usedKeys, 'P');		/* fuer "Preview" */
	if (itk->runMacro)
	{
		setincl(itk->usedKeys, 'V');	/* fuer "Vorwaerts" */
		setincl(itk->usedKeys, 'Z');	/* fuer "Zurueck" */
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CalcCoords(ITK *itk, CONST _UBYTE *text, _WORD *x, _WORD *y, _WORD *w, _WORD *h)
{
	*x = CharWidth * 2;
	*y = itk->lastyPos + CharHeight;
	*w = CharWidth * ((_WORD)strLen(text) + 4);
	*h = CharHeight;
	itk->lastyPos = *y;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CalcCoords2(ITK *itk, CONST _UBYTE *name, _WORD maxLen, _WORD *x, _WORD *y, _WORD *w, _WORD *h)
{
	itk->lastyPos -= CharHeight;
	CalcCoords(itk, name, x, y, w, h);
	*x = *x + *w - 2 * CharWidth;
	if (maxLen > 50)
		maxLen = 50;
	if (maxLen == 0)
		maxLen = 50;
	*w = CharWidth * (maxLen + 1);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CalcEdButCoords(_WORD maxLen, _WORD *x, _WORD *y, _WORD *w, _WORD *h)
{
	UNUSED(y);
	UNUSED(h);
	*x = *x + *w + 2 * CharWidth;
	if (maxLen > 50)
		maxLen = 50;
	if (maxLen == 0)
		maxLen = 50;
	*w = CharWidth * (maxLen + 4);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID AddShortcut(ITK *itk, _UBYTE *name, size_t namesize)
{
	size_t i, l;
	_UBYTE ch;
	_UBYTE *p;
	
	if ((p = strChr(name, '[')) != NULL && p[1] != '\0')
	{
		ch = ucase(p[1]);
		setincl(itk->usedKeys, ch);
	} else
	{
		l = strLen(name);
		i = 0;
		ch = ucase(name[i]);
		while (i < l &&
			   (setin(itk->usedKeys, ch) || ch < 'A' || ch > 'Z'))
		{
			++i;
			ch = ucase(name[i]);
		}
		if (i < l)
		{
			/* freien Shortcut gefunden */
			setincl(itk->usedKeys, ch);
			Insert("[", name, namesize, i);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD AddCheckButton(ITK *itk, _UBYTE *name, size_t namesize, _BOOL threeState)
{
	_UWORD obType;						/* Vereinigt normalen und erweiterten Objekttyp */
	_WORD x, y, w, h;

	if (threeState)
		obType = (ThreeState << 8) | G_BUTTON;
	else
		obType = (SpecButton << 8) | G_BUTTON;
	CalcCoords(itk, name, &x, &y, &w, &h);
	AddShortcut(itk, name, namesize);
	if (!SetObjc(itk, Box, obType, x, y, w, h, SELECTABLE, NORMAL, name))
		return -1;
	return itk->currObIdx - 1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD AddRadioButton(ITK *itk, _WORD parent, _UBYTE *name, size_t namesize)
{
	_UWORD obType;						/* Vereinigt normalen und erweiterten Objekttyp */
	_WORD x, y, w, h;

	obType = (SpecButton << 8) | G_BUTTON;
	CalcCoords(itk, name, &x, &y, &w, &h);
	AddShortcut(itk, name, namesize);
	if (!SetObjc(itk, parent, obType, x, y, w, h, SELECTABLE | RBUTTON, NORMAL, name))
		return -1;
	return itk->currObIdx - 1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD AddText(ITK *itk, _WORD parent, _UBYTE *name)
{
	_UWORD obType;						/* Vereinigt normalen und erweiterten Objekttyp */
	_WORD x, y, w, h;

	obType = G_STRING;
	CalcCoords(itk, name, &x, &y, &w, &h);
	if (!SetObjc(itk, parent, obType, x, y, w, h, NONE, NORMAL, name))
		return -1;
	return itk->currObIdx - 1;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
LOCAL _BOOL PrepareEdit(ITK *itk, _BOOL isNum, _WORD maxLength, CONST _UBYTE *deflt, _UWORD *obType)
{
	_UBYTE validChar;
	_WORD i;

	if (itk->tedIdx >= cMaxTeds)
	{
		MTE_infoRSC(AL_TOOMANY_TEDS);
		return FALSE;
	}
	if (maxLength == 0 && !isNum)
		maxLength = 255;
	else if (maxLength == 0 && isNum)
		maxLength = 10;
	if (maxLength > 50 && !isNum)
		*obType = (LongEdit << 8) | G_FTEXT;
	else
		*obType = G_FTEXT;
	itk->Teds[itk->tedIdx].te_ptext = MALLOC(maxLength + 1, "PrepareEdit: te_ptext");
	if (itk->Teds[itk->tedIdx].te_ptext == NULL)
		return FALSE;
	itk->Teds[itk->tedIdx].te_pvalid = MALLOC(maxLength + 1, "PrepareEdit: te_pvalid");
	if (itk->Teds[itk->tedIdx].te_pvalid == NULL)
	{
		SFREE(itk->Teds[itk->tedIdx].te_ptext);
		itk->Teds[itk->tedIdx].te_ptext = NULL;
		return FALSE;
	}
	itk->Teds[itk->tedIdx].te_ptmplt = MALLOC(maxLength + 1, "PrepareEdit: te_ptmplt");
	if (itk->Teds[itk->tedIdx].te_ptmplt == NULL)
	{
		SFREE(itk->Teds[itk->tedIdx].te_pvalid);
		itk->Teds[itk->tedIdx].te_pvalid = NULL;
		SFREE(itk->Teds[itk->tedIdx].te_ptext);
		itk->Teds[itk->tedIdx].te_ptext = NULL;
		return FALSE;
	}
	itk->Teds[itk->tedIdx].te_font = IBM;
	itk->Teds[itk->tedIdx].te_fontid = 0;
	itk->Teds[itk->tedIdx].te_just = TE_LEFT;
	itk->Teds[itk->tedIdx].te_color = COLSPEC_MAKE(BLACK, BLACK, TRUE, IP_HOLLOW, WHITE);
	itk->Teds[itk->tedIdx].te_fontsize = 0;
	itk->Teds[itk->tedIdx].te_thickness = -1;
	itk->Teds[itk->tedIdx].te_txtlen = maxLength + 1;
	itk->Teds[itk->tedIdx].te_tmplen = maxLength + 1;
	strMcpy(itk->Teds[itk->tedIdx].te_ptext, maxLength + 1, deflt);
	if (isNum)
		validChar = '9';
	else
		validChar = 'X';
	for (i = 0; i < maxLength; i++)
	{
		itk->Teds[itk->tedIdx].te_pvalid[i] = validChar;
		itk->Teds[itk->tedIdx].te_ptmplt[i] = '_';
	}
	itk->Teds[itk->tedIdx].te_pvalid[maxLength] = '\0';
	itk->Teds[itk->tedIdx].te_ptmplt[maxLength] = '\0';
	/* Jetzt testen, ob wir unter MagiC laufen und die MagiC
	 * Scrollfelder nutzen
	 */
	if (IsMagiCScroll &&
		maxLength > 50 && !isNum)
	{
		/* Lange Editfelder, umwandeln in MagiC-Scroller */
		itk->XTeds[itk->tedIdx].xte_ptmplt = itk->Teds[itk->tedIdx].te_ptmplt;
		itk->XTeds[itk->tedIdx].xte_pvalid = itk->Teds[itk->tedIdx].te_pvalid;
		itk->XTeds[itk->tedIdx].xte_vislen = 50;	/* Mehr nehmen wir nicht! */
		itk->XTeds[itk->tedIdx].xte_scroll = 0;
		itk->Teds[itk->tedIdx].te_ptmplt = NULL;
		itk->Teds[itk->tedIdx].te_pvalid = (_VOID *)&itk->XTeds[itk->tedIdx];
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD AddEditField(ITK *itk, _UBYTE *name, _BOOL isNum, _WORD maxLength, CONST _UBYTE *deflt)
{
	_UWORD obType;
	_WORD x, y, w, h;

	if (!PrepareEdit(itk, isNum, maxLength, deflt, &obType))
		return -1;
	if (AddText(itk, Box, name) < 0)
		return -1;

	CalcCoords2(itk, name, maxLength, &x, &y, &w, &h);

	if (!SetObjc(itk, Box, obType, x, y, w, h, EDITABLE|FL3DBAK,NORMAL, &itk->Teds[itk->tedIdx]))
		return -1;
	++itk->tedIdx;
	return itk->currObIdx - 1;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
LOCAL _BOOL AddHelpBut(ITK *itk, _WORD x, _WORD y, _WORD w, _WORD h)
{
	if (itk->tedIdx >= cMaxTeds)
	{
		MTE_infoRSC(AL_TOOMANY_TEDS);
		return FALSE;
	}
	itk->Teds[itk->tedIdx].te_ptext = STRDUP(MausTauschrsc_WrkString(HelpBut), "AddHelpBut");
	if (itk->Teds[itk->tedIdx].te_ptext == NULL)
		return FALSE;
	itk->Teds[itk->tedIdx].te_pvalid = NULL;
	itk->Teds[itk->tedIdx].te_ptmplt = NULL;
	itk->Teds[itk->tedIdx].te_font = SMALL;
	itk->Teds[itk->tedIdx].te_fontid = 0;
	itk->Teds[itk->tedIdx].te_just = TE_CNTR;
	itk->Teds[itk->tedIdx].te_color = COLSPEC_MAKE(BLACK, BLACK, TRUE, IP_HOLLOW, WHITE);
	itk->Teds[itk->tedIdx].te_fontsize = 0;
	itk->Teds[itk->tedIdx].te_thickness = -1;
	itk->Teds[itk->tedIdx].te_txtlen = (_WORD)(strLen(MausTauschrsc_WrkString(HelpBut)) + 1);
	itk->Teds[itk->tedIdx].te_tmplen = 0;
	if (!SetObjc(itk, Box, (HelpButton << 8) | G_BOXTEXT, x, y, w, h, FL3DBAK|SELECTABLE|EXIT|HelpButtonFlag, OUTLINED|SHADOWED, &itk->Teds[itk->tedIdx]))
		return FALSE;
	++itk->tedIdx;
	itk->type[itk->currObIdx - 1] = T_HELP;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Hilfe-, Preview-, OK-, Abbruch-Button sowie bei Makroausfuehrung
 * die Skipbuttons hinzufuegen. Es werden zwei Reihen hinzugefuegt, 
 * in der oberen der Hilfe-, der Vorwaerts- und der OK-Button, jeweils
 * darunter der Preview-, Zurueck- und Abbruch-Button. 
 * Das sieht dann so aus:
 *
 * |  +------------+  +-------------+                  +-------------+  |
 * |  |   Help     |  |  Vorwaerts  |                  |     OK      |  |
 * |  +------------+  +-------------+                  +-------------+  |
 * |                                                                    |
 * |  +------------+  +-------------+                  +-------------+  |
 * |  |  Preview   |  |   Zurueck   |                  |   Abbruch   |  |
 * |  +------------+  +-------------+                  +-------------+  |
 * +--------------------------------------------------------------------+
 *
 */
/*DYJ 5.21b */
LOCAL _BOOL AddExitButs(ITK *itk)
{
	_UWORD obType;
	_WORD x, y, w, h;
	_WORD factor;
	OneMacro *pMacro, *nMacro;
	_WORD disabled;

	/* Breite um fuenf Zeichen vergroessern */
	itk->Tree[Box].ob_width += 5 * CharWidth;
	obType = (SpecButton << 8) | G_BUTTON;

	/* Position berechnen */
	h = CharHeight;
	w = CharWidth * (_WORD)(strLen(MausTauschrsc_WrkString(CancelBut)) + 2);
	factor = itk->runMacro ? 4 : 2;
	itk->Tree[Box].ob_width = Max(itk->Tree[Box].ob_width, factor * w + 6 * CharWidth);
	x = 2 * CharWidth;
	y = itk->lastyPos + 2 * CharHeight;

	/* Hilfebutton */
	if (!AddHelpBut(itk, x, y, 48,h))
		return FALSE;

	/* Vorwaerts-Button */
	if (itk->runMacro)
	{
		pMacro = Lists_CurrentEntry(&itk->macroListe);
		nMacro = Lists_NextEntry(&itk->macroListe);		/* Zeiger eins vor */
		if ((pMacro->aktStep < pMacro->totalSteps) && (nMacro != NULL && nMacro->opened))
			disabled = NORMAL;
		else
			disabled = DISABLED;
		x += w + 3 * CharWidth;
		if (!SetObjc(itk, Box, obType, x, y, w, h, SELECTABLE|EXIT, disabled, MausTauschrsc_WrkString(SkipNextBut)))
			return FALSE;
		itk->type[itk->currObIdx - 1] = T_SKIP_NEXT;
		pMacro = Lists_PrevEntry(&itk->macroListe);		/* Zeiger wieder zurueck */
	}
	
	/* OK-Button */
	x = itk->Tree[Box].ob_width - 2 * CharWidth - w;
	if (!SetObjc(itk, Box, obType, x, y, w, h, SELECTABLE|DEFAULT|EXIT, NORMAL, MausTauschrsc_WrkString(OkBut)))
		return FALSE;
	itk->type[itk->currObIdx - 1] = T_OK;

	/* Previewbutton */
	x = 2 * CharWidth;
	y = itk->lastyPos + 4 * CharHeight;
	if (!SetObjc(itk, Box, obType, x, y, w, h, SELECTABLE|EXIT, NORMAL, MausTauschrsc_WrkString(PreviewBut)))
		return FALSE;
	itk->type[itk->currObIdx - 1] = T_PREVIEW;

	/* Zurueck-Button */
	if (itk->runMacro)
	{
		pMacro = Lists_CurrentEntry(&itk->macroListe);
		disabled = pMacro->aktStep > 1 ? NORMAL : DISABLED;
		x += w + 3 * CharWidth;
		if (!SetObjc(itk, Box, obType, x, y, w, h, SELECTABLE|EXIT, disabled, MausTauschrsc_WrkString(SkipPrevBut)))
			return FALSE;
		itk->type[itk->currObIdx - 1] = T_SKIP_PREV;
	}

	/* Abbruch-Button */
	x = itk->Tree[Box].ob_width - 2 * CharWidth - w;
	if (!SetObjc(itk, Box, (UndoButton << 8) | G_BUTTON, x, y, w, h, SELECTABLE|EXIT|LASTOB, NORMAL, MausTauschrsc_WrkString(CancelBut)))
		return FALSE;
	itk->type[itk->currObIdx - 1] = T_CANCEL;
	itk->Tree[Box].ob_height = itk->lastyPos + 6 * CharHeight;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD AddGroupSelect(ITK *itk, _UBYTE *name, size_t namesize, _BOOL chiefGroup, _BOOL ptGroup, _WORD maxLength, CONST _UBYTE *deflt)
{
	_UWORD obType;
	_UWORD type;
	_WORD x, y, w, h;

	/* Button mit Bit fuer Groupselect gesetzt */
	obType = (SpecButton << 8) | G_BUTTON;
	if (chiefGroup)
		type = T_CHIEF_GROUP;
	else if (ptGroup)
		type = T_PT_GROUP;
	else
		type = T_GROUP;
	CalcCoords(itk, name, &x, &y, &w, &h);
	AddShortcut(itk, name, namesize);
	if (itk->currObIdx > Header + 1)
	{
		y += CharHeight;
		itk->lastyPos += CharHeight;
	}
	if (!SetObjc(itk, Box, obType, x, y, w, h, SELECTABLE|EXIT, NORMAL, name))
		return -1;
	itk->type[itk->currObIdx - 1] = type;

	/* Editfeld hinzufuegen */
	if (!PrepareEdit(itk, FALSE, maxLength, deflt, &obType))
		return -1;
	CalcEdButCoords(maxLength, &x, &y, &w, &h);
	if (!SetObjc(itk, Box, obType, x, y, w, h, EDITABLE|FL3DBAK, NORMAL, &itk->Teds[itk->tedIdx]))
		return -1;
	++itk->tedIdx;
	itk->lastyPos += CharHeight;
	return itk->currObIdx - 1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD AddUserSelect(ITK *itk, _UBYTE *name, size_t namesize, _WORD maxLength, CONST _UBYTE *deflt)
{
	_UWORD obType;
	_WORD x, y, w, h;

	/* Button mit Bit fuer Groupselect gesetzt */
	obType = (SpecButton << 8) | G_BUTTON;
	CalcCoords(itk, name, &x, &y, &w, &h);
	AddShortcut(itk, name, namesize);
	if (itk->currObIdx > Header + 1)
	{
		y += CharHeight;
		itk->lastyPos += CharHeight;
	}
	if (!SetObjc(itk, Box, obType, x, y, w, h, SELECTABLE|EXIT, NORMAL, name))
		return -1;
	itk->type[itk->currObIdx - 1] = T_USER;

	/* Editfeld hinzufuegen */
#ifndef ITK_STANDALONE
	if (deflt[0] == '\0')
	{
		HWND botWdw;
		_UBYTE defName[256];
		
		defName[0] = '\0';
		/* Aktuellen User abfragen */
		if (grin_grinWindowTop(ownTopWindow))
		{
			grin_ActualFrom(ownTopWindow, defName, sizeof(defName));
		} else if ((WdwManager_GetSecondWdw(&botWdw) && grin_grinWindowTop(botWdw)) ||
			grin_grinOnlyOneWind(&botWdw))
		{
			grin_ActualFrom(botWdw, defName, sizeof(defName));
		}
		if (!PrepareEdit(itk, FALSE, maxLength, defName, &obType))
			return -1;
	} else
#endif
	{
		if (!PrepareEdit(itk, FALSE, maxLength, deflt, &obType))
			return -1;
	}

	CalcEdButCoords(maxLength, &x, &y, &w, &h);
	if (!SetObjc(itk, Box, obType, x, y, w, h, EDITABLE|FL3DBAK, NORMAL, &itk->Teds[itk->tedIdx]))
		return -1;
	++itk->tedIdx;
	itk->lastyPos += CharHeight;
	return itk->currObIdx - 1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID SetRadioSelect(OBJECT *tree, _WORD button, _WORD select)
{
	_WORD i, count;
	
	count = tree[button + 2].ob_tail - tree[button + 2].ob_head + 1;
	for (i = 0; i < count; i++)
		EnableObjState(tree, button + 3 + i, SELECTED, i == select);
	if (select >= 0)
	{
		VDIUtil_SetXState(tree, button, select);
		VDIUtil_SetXStringAdr(tree, button, &tree[button + 2], select);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL AddRadioEnums(ITK *itk, OneParam *pParm)
{
	OneEnum *pEnum;
	_WORD count;
	_WORD parent;
	_WORD x, y, w, h;
	_UBYTE value[256];
	_WORD select;
	_WORD i;
#define minPopupCount 3

	count = (_WORD)Lists_NoOfEntries(&pParm->paramEnum);
	h = 0;
	if (count >= minPopupCount)
	{
		_WORD box;
		
		/*
		 * Box fuer Popup und Circle-Button erzeugen
		 */
		if (itk->currObIdx > Header + 1)
			itk->lastyPos += CharHeight / 2;
		if (!SetObjc(itk, Box, G_BOX, 0, 0, 0, CharHeight, NONE, SHADOWED, NULL))
			return FALSE;
		box = itk->currObIdx - 1;
		OBSPEC_SET_OBSPEC(itk->Tree[box].ob_spec, '\0', -1, BLACK, BLACK, FALSE, IP_HOLLOW, WHITE);
		h = itk->Tree[Box].ob_height;
		/*
		 * Objekt fuer Popup erzeugen
		 */
		if (!SetObjc(itk, box, (CircleButton << 8) | G_BUTTON, 0, 0, 0, CharHeight, TOUCHEXIT|FL3DBAK, NORMAL, NULL))
			return FALSE;
		itk->type[itk->currObIdx - 1] = T_POPUP;
		/*
		 * Circle-Button erzeugen
		 */
		if (!SetObjc(itk, box, (CircleButton << 8) | G_BOXCHAR, 0, 0, 2 * CharWidth, CharHeight, TOUCHEXIT, NORMAL, NULL))
			return FALSE;
		OBSPEC_SET_OBSPEC(itk->Tree[itk->currObIdx - 1].ob_spec, '\001', -1, BLACK, BLACK, FALSE, IP_HOLLOW, WHITE);
		itk->type[itk->currObIdx - 1] = T_CIRCLE;
		/*
		 * Parent-Objekt fuer die Eintraege erzeugen
		 */
		if (!SetObjc(itk, Box, G_BOX, 0, 0, 0, 0, FL3DBAK, SHADOWED, NULL))
			return FALSE;
		OBSPEC_SET_OBSPEC(itk->Tree[itk->currObIdx - 1].ob_spec, '\0', -1, BLACK, BLACK, FALSE, IP_HOLLOW, WHITE);
		parent = itk->currObIdx - 1;
	} else if (count >= 2)
	{
		/*
		 * Box-Objekt fuer die Radio-Buttons erzeugen,
		 * falls mehrere solcher Parameter in dem Kommando sind
		 */
		if (!SetObjc(itk, Box, G_IBOX, 0, 0, 0, 0, FL3DBAK, NORMAL, NULL))
			return FALSE;
		OBSPEC_SET_OBSPEC(itk->Tree[itk->currObIdx - 1].ob_spec, '\0', 0, BLACK, BLACK, FALSE, IP_HOLLOW, WHITE);
		parent = itk->currObIdx - 1;
	} else
	{
		parent = Box;
	}
	Lists_ResetList(&pParm->paramEnum);
	select = -1;
	while ((pEnum = Lists_NextEntry(&pParm->paramEnum)) != NULL)
	{
		if (count == 1)
			pEnum->obLow = AddCheckButton(itk, pEnum->enumName, sizeof(pEnum->enumName), FALSE);
		else if (count >= minPopupCount)
			pEnum->obLow = AddText(itk, parent, pEnum->enumName);
		else if (count >= 2)
			pEnum->obLow = AddRadioButton(itk, parent, pEnum->enumName, sizeof(pEnum->enumName));
		else
			return FALSE;
		if (pEnum->obLow < 0)
			return FALSE;
		SetObjFlags(itk->Tree, pEnum->obLow, SELECTABLE);
		*value = '\0';
		GetEnumValue(0, pEnum->enumValue, value, sizeof(value));
		EnableObjState(itk->Tree, pEnum->obLow, SELECTED, strCmp(pParm->paramValue, value) == 0);
		if (GetObjState(itk->Tree, pEnum->obLow, SELECTED))
			select = pEnum->obLow - parent - 1;
	}
	if (count >= 2)
	{
		/*
		 * Jetzt noch Groesse und Position der Box,
		 * und Position der Radiobuttons in der Box anpassen
		 */
		Lists_ResetList(&pParm->paramEnum);
		pEnum = Lists_NextEntry(&pParm->paramEnum);
		objc_offset(itk->Tree, pEnum->obLow, &x, &y);
		itk->Tree[parent].ob_x = x;
		itk->Tree[parent].ob_y = y;
		w = 0;
		while (pEnum != NULL)
		{
			itk->Tree[pEnum->obLow].ob_x -= x;
			itk->Tree[pEnum->obLow].ob_y -= y;
			w = Max(w, itk->Tree[pEnum->obLow].ob_x + itk->Tree[pEnum->obLow].ob_width);
			pEnum = Lists_NextEntry(&pParm->paramEnum);
		}
		itk->Tree[parent].ob_width = w;
		itk->Tree[parent].ob_height = count * CharHeight;
		
		Lists_ResetList(&pParm->paramEnum);
		pEnum = Lists_NextEntry(&pParm->paramEnum);
		while (pEnum != NULL)
		{
			itk->Tree[pEnum->obLow].ob_width = w;
			pEnum = Lists_NextEntry(&pParm->paramEnum);
		}
		if (count >= minPopupCount)
		{
			/*
			 * Box fuer Popup positionieren
			 */
			itk->Tree[parent-3].ob_x = itk->Tree[parent].ob_x;
			itk->Tree[parent-3].ob_y = itk->Tree[parent].ob_y;
			itk->Tree[parent-3].ob_width = itk->Tree[parent].ob_width + itk->Tree[parent-1].ob_width;
			/*
			 * Popup positionieren
			 */
			itk->Tree[parent-2].ob_width = itk->Tree[parent].ob_width;
			/*
			 * Circle-Button positionieren
			 */
			itk->Tree[parent-1].ob_x = itk->Tree[parent-2].ob_x + itk->Tree[parent-2].ob_width;
			/*
			 * Parent-Objekt fuer Eintraege verstecken
			 */
			objc_delete(itk->Tree, parent);
			for (i = 0; i < count; i++)
				itk->Tree[parent + 1 + i].ob_next = i + 2;
			itk->Tree[parent + count].ob_next = ROOT;
			itk->Tree[parent].ob_next = NIL;
			itk->Tree[parent].ob_head = 1;
			itk->Tree[parent].ob_tail = count;
			/*
			 * Start-Wert vorbelegen
			 */
			if (select < 0)
				select = 0;
			SetRadioSelect(itk->Tree, parent - 2, select);

			itk->lastyPos = y + CharHeight + CharHeight / 2;
			itk->Tree[Box].ob_height = h;
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD CheckEnumValue(CONST _UBYTE *str)
{
	_WORD count;
	CONST _UBYTE *pos;

	count = 0;
	pos = str;
	while (pos != NULL)
	{
		pos = strChr(pos, ':');
		if (pos != NULL)
			++pos;
		++count;
	}
	return count;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL AddCheckEnums(ITK *itk, List *paramEnums)
{
	OneEnum *pEnum;
	_WORD count;
	_WORD value;
	_UBYTE str[256];
	
	Lists_ResetList(paramEnums);
	while ((pEnum = Lists_NextEntry(paramEnums)) != NULL)
	{
		count = CheckEnumValue(pEnum->enumValue);
		if (count < 3)
		{
			pEnum->obLow = AddCheckButton(itk, pEnum->enumName, sizeof(pEnum->enumName), FALSE);
			if (pEnum->obLow < 0)
				return FALSE;
			*str = '\0';
			GetEnumValue(0, pEnum->enumValue, str, sizeof(str));
			EnableObjState(itk->Tree, pEnum->obLow, SELECTED, strCmp(str, pEnum->enumCurValue) == 0);
		} else if (count == 3)
		{
			pEnum->obLow = AddCheckButton(itk, pEnum->enumName, sizeof(pEnum->enumName), TRUE);
			if (pEnum->obLow < 0)
				return FALSE;
			*str = '\0';
			GetEnumValue(0, pEnum->enumValue, str, sizeof(str));
			if (strCmp(pEnum->enumCurValue, str) == 0)
			{
				value = SETNEW;
			} else
			{
				*str = '\0';
				GetEnumValue(1, pEnum->enumValue, str, sizeof(str));
				if (strCmp(pEnum->enumCurValue, str) == 0)
					value = CLEAR;
				else
					value = NOCHANGE;
			}
			SetThreeState(itk->Tree, pEnum->obLow, value);
		} else
		{
			/* Mehr als 3 Zustnde sind momentan nicht behandelbar */
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
LOCAL _BOOL AddParamObjcs(ITK *itk, OneParam *pParam)
{
	CONST _UBYTE *str;
	OneMacro *pMacro;
	
	pMacro = Lists_CurrentEntry(&itk->macroListe);
	switch (pParam->paramType)
	{
	case tpConst:
		break;
	case tpEnum:
		if (pParam->hasEnums)
			if (!AddRadioEnums(itk, pParam))
				return FALSE;
		break;
	case tpEnumSet:
		if (pParam->hasEnums)
			if (!AddCheckEnums(itk, &pParam->paramEnum))
				return FALSE;
		break;
	case tpMTDate:
	case tpDate:
		pParam->obLow = AddEditField(itk, pParam->paramName, FALSE, 12, pParam->paramDflt);
		if (pParam->obLow < 0)
			return FALSE;
		SetObjcString(itk->Tree, pParam->obLow, pParam->paramValue);
		break;
	case tpString:
	case tpPassword:
	case tpNewPassword:
		pParam->obLow = AddEditField(itk, pParam->paramName, FALSE, (_WORD)(pParam->paramMin), pParam->paramDflt);
		if (pParam->obLow < 0)
			return FALSE;
		SetObjcString(itk->Tree, pParam->obLow, pParam->paramValue);
		break;
	case tpInt:
	case tpRange:
		pParam->obLow = AddEditField(itk, pParam->paramName, TRUE, 10, pParam->paramDflt);
		if (pParam->obLow < 0)
			return FALSE;
		SetObjcString(itk->Tree, pParam->obLow, pParam->paramValue);
		break;
	case tpGroup:
	case tpPtGroup:
	case tpChiefGroup:
		/* Button und dahinter Editfeld */
		pParam->obLow = AddGroupSelect(itk, pParam->paramName, sizeof(pParam->paramName), pParam->paramType == tpChiefGroup, pParam->paramType == tpPtGroup, (_WORD)(pParam->paramMin), pParam->paramDflt);
		if (pParam->obLow < 0)
			return FALSE;
		if (itk->configModus)
		{
			str = MausTauschrsc_WrkString(DefaultStr);
		} else
		{
			if (pParam->paramType == tpGroup)
				str = (itk->runMacro && pMacro->resSetup[0] == '\0') ? groupDefault : pParam->paramValue;
			else if (pParam->paramType == tpPtGroup)
				str = (itk->runMacro && pMacro->resSetup[0] == '\0') ? ptgrpDefault : pParam->paramValue;
			else
				str = (itk->runMacro && pMacro->resSetup[0] == '\0') ? chgrpDefault : pParam->paramValue;
		}
		SetObjcString(itk->Tree, pParam->obLow, str);
		break;
	case tpUsername:
		/* Button und dahinter Editfeld */
		pParam->obLow = AddUserSelect(itk, pParam->paramName, sizeof(pParam->paramName), (_WORD)(pParam->paramMin), pParam->paramDflt);
		if (pParam->obLow < 0)
			return FALSE;
		if (itk->configModus)										/* "Default-Eintrag" */
			str = MausTauschrsc_WrkString(DefaultStr);
		else if (itk->runMacro && pMacro->resSetup[0] == '\0')		/* Default-Vorbelegung */
			str = userDefault;
		else if (itk->runMacro && pMacro->resSetup[0] != '\0')		/* erfolgte Makro-Eingabe */
			str = pParam->paramValue;
		else
			str = ObjcStringAdr(itk->Tree, pParam->obLow);			/* urspruenglicher Inhalt */
		SetObjcString(itk->Tree, pParam->obLow, str);
		break;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL BuildTree(OneCommand *pCmd)
{
	ITK *itk;
	OneParam *pParam;
	OneMacro *pMacro;
	_UBYTE str2[64];
	
	itk = pCmd->itk;
	strBcpy(itk->title, pCmd->cmdName);
	if (itk->runMacro)
	{
		/* Stepanzeige fuer Makro anhaengen */
		pMacro = Lists_CurrentEntry(&itk->macroListe);
		if (pMacro != NULL)
		{
			sprintf(str2, " (%d/%d)", pMacro->aktStep, pMacro->totalSteps);
			strBcat(itk->title, str2);
		}
	}
	PrepareTree(itk, itk->title);
	if (pCmd->hasParms)
	{
		Lists_ResetList(&pCmd->cmdParms);
		while ((pParam = Lists_NextEntry(&pCmd->cmdParms)) != NULL)
		{
	/*		MemSetZeroStruct(pParam->paramValue); */
			if (!AddParamObjcs(itk, pParam))
				return FALSE;
		}
	}
	if (!AddExitButs(itk))
		return FALSE;
	/* Mover noch verschieben */
	itk->Tree[Mover].ob_x = itk->Tree[Box].ob_width - itk->Tree[Mover].ob_width;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Allozierten Speicher fr Editfelder wieder freigeben */
LOCAL _VOID RemoveTree(ITK *itk)
{
	_WORD i;

	for (i = 0; i < itk->tedIdx; i++)
	{
		if (itk->Teds[i].te_ptmplt == NULL)
		{
			/* MagiC - Editfeld */
			if (itk->XTeds[i].xte_pvalid != NULL)
			{
				SFREE(itk->XTeds[i].xte_pvalid);
				itk->XTeds[i].xte_pvalid = NULL;
			}
			if (itk->XTeds[i].xte_ptmplt != NULL)
			{
				SFREE(itk->XTeds[i].xte_ptmplt);
				itk->XTeds[i].xte_ptmplt = NULL;
			}
		} else
		{
			if (itk->Teds[i].te_pvalid != NULL)
			{
				SFREE(itk->Teds[i].te_pvalid);
				itk->Teds[i].te_pvalid = NULL;
			}
			SFREE(itk->Teds[i].te_ptmplt);
			itk->Teds[i].te_ptmplt = NULL;
		}
		if (itk->Teds[i].te_ptext != NULL)
		{
			SFREE(itk->Teds[i].te_ptext);
			itk->Teds[i].te_ptext = NULL;
		}
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----------------------------------------------------------------------
 * ITK Werte aus Dialogbox auslesen
 *----------------------------------------------------------------------*/

LOCAL _BOOL CheckRadioEnums(ITK *itk, List *paramEnums, _UBYTE *str, size_t strsize)
{
	OneEnum *pEnum;
	_WORD count;

	count = (_WORD)Lists_NoOfEntries(paramEnums);
	Lists_ResetList(paramEnums);
	while ((pEnum = Lists_NextEntry(paramEnums)) != NULL)
	{
		if (GetObjState(itk->Tree, pEnum->obLow, SELECTED))
		{
			/* Ersten Wert aus Aufzaehlung nehmen! */
			GetEnumValue(0, pEnum->enumValue, str, strsize);
			return TRUE;
		} else if (count == 1)
		{
			/* Zweiten Wert aus Aufzaehlung nehmen! */
			GetEnumValue(1, pEnum->enumValue, str, strsize);
			return TRUE;
		}
	}
	/* Hier kommen wir nur hier, wenn keiner selektiert ist */
	return count == 1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL CheckCheckEnums(ITK *itk, List *paramEnum, _UBYTE *str, size_t strsize, _BOOL getDefault)
{
	OneEnum *pEnum;
	_WORD count;
	_BOOL oneSet;
	_WORD value;
	_UBYTE val[256];

	Lists_ResetList(paramEnum);
	oneSet = FALSE;
	while ((pEnum = Lists_NextEntry(paramEnum)) != NULL)
	{
		count = CheckEnumValue(pEnum->enumValue);
		*val = '\0';
		if (getDefault)
		{
			if (count < 3)
				GetEnumValue(1, pEnum->enumValue, val, sizeof(val));
			else
				GetEnumValue(2, pEnum->enumValue, val, sizeof(val));
			oneSet = TRUE;
		} else
		{
			if (count < 3)
			{
				if (GetObjState(itk->Tree, pEnum->obLow, SELECTED))
				{
					GetEnumValue(0, pEnum->enumValue, val, sizeof(val));
					oneSet = TRUE;
				} else
				{
					GetEnumValue(1, pEnum->enumValue, val, sizeof(val));
				}
			} else if (count == 3)
			{
				value = GetThreeState(itk->Tree, pEnum->obLow);
				if (value == CLEAR)
				{
					/* zweiten Wert aus Enum nehmen */
					GetEnumValue(1, pEnum->enumValue, val, sizeof(val));
					oneSet = TRUE;
				} else if (value == SETNEW)
				{
					/* ersten Wert aus Enum nehmen */
					GetEnumValue(0, pEnum->enumValue, val, sizeof(val));
					oneSet = TRUE;
				} else
				{
					/* dritten und letzten Wert aus Enum nehmen */
					GetEnumValue(2, pEnum->enumValue, val, sizeof(val));
				}
			} else
			{
				/* Nicht behandelbar momentan */
			}
		}
		strBcpy(pEnum->enumCurValue, val);
		strMcat(str, strsize, val);
	}
	return oneSet;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
LOCAL _BOOL GetParam(ITK *itk, OneParam *pParm, _UBYTE *str, size_t strsize, _BOOL getDefault)
{
	_BOOL bRes;
	_BOOL normalDate;
	OneMacro *pMacro;
	Date dt;
	Time ti;	/* fuer Pruefzwecke */
	_UBYTE temp[256];
	_LONG lIntVal;
	_UBYTE pName[128];

	switch (pParm->paramType)
	{
	case tpConst:						/* Konstanten Teil uebernehmen */
		strMcat(str, strsize, pParm->paramDflt);
		break;

	case tpEnum:
		if (pParm->hasEnums)
		{
			if (getDefault)
			{
				if (pParm->hasDefault)
				{
					strBcpy(pParm->paramValue, pParm->paramDflt);
				} else
				{
					OneEnum *pEnum;
					
					Lists_ResetList(&pParm->paramEnum);
					pEnum = Lists_NextEntry(&pParm->paramEnum);
					pParm->paramValue[0] = '\0';
					GetEnumValue(0, pEnum->enumValue, pParm->paramValue, sizeof(pParm->paramValue));
				}
			} else
			{
				pParm->paramValue[0] = '\0';
				bRes = CheckRadioEnums(itk, &pParm->paramEnum, pParm->paramValue, sizeof(pParm->paramValue));
				if (!bRes && !pParm->optional)
				{
					/* Fehlermeldung */
					StripShortcut(pParm->paramName, pName, sizeof(pName));
					sprintf(temp, MausTauschrsc_WrkString(eEnumCheck), pName);
					MTE_info(temp);
					return FALSE;
				}
			}
			strMcat(str, strsize, pParm->paramValue);
		}
		break;

	case tpEnumSet:
		if (pParm->hasEnums)
		{
			bRes = CheckCheckEnums(itk, &pParm->paramEnum, str, strsize, getDefault);
			if (!bRes && !pParm->optional)
			{
				/* Fehlermeldung */
				StripShortcut(pParm->paramName, pName, sizeof(pName));
				sprintf(temp, MausTauschrsc_WrkString(eEnumCheck), pName);
				MTE_info(temp);
				return FALSE;
			}
		}
		break;
	
	case tpMTDate:
	case tpDate:						/* Editfeld auslesen */
		if (getDefault)
		{
			strBcpy(pParm->paramValue, pParm->paramDflt);
		} else
		{
			ObjcString(itk->Tree, pParm->obLow, pParm->paramValue, sizeof(pParm->paramValue));
			if (pParm->paramValue[0] == '\0')
			{
				if (!pParm->optional)
				{
					/* Fehlermeldung */
					StripShortcut(pParm->paramName, pName, sizeof(pName));
					sprintf(temp, MausTauschrsc_WrkString(eEmptyParm), pName);
					MTE_info(temp);
					return FALSE;
				}
			} 
			/* Datum pruefen (Standard- und MausTausch-Format) */
			else if ((normalDate = ConvertDate_DateOk(pParm->paramValue)) == FALSE &&
				     (!ConvertDate_MTTextToDate(pParm->paramValue, &dt) ||
					  !ConvertDate_MTTextToTime(pParm->paramValue, &ti)))
			{
				/* Fehlermeldung */
				StripShortcut(pParm->paramName, pName, sizeof(pName));
				sprintf(temp, MausTauschrsc_WrkString(eDateAlt), pName);
				MTE_info(temp);
				return FALSE;
			} else
			{
				/* Datum konvertieren */
				if (pParm->paramType == tpMTDate)
				{
					if (normalDate)
						ConvertDate_StrToMTDate(pParm->paramValue, temp, sizeof(temp));
					else
						strBcpy(temp, pParm->paramValue);
				} else
				{
					F_TIME t;
					
					ConvertDate_TextToDate(pParm->paramValue, &dt);
					T_Localtime(T_DosToSystime(ConvertDate_PackDate(&dt), (_UWORD)(SECS_PER_DAY / 2)), &t);
					ConvertDate_DateToText(&t, "DD.MM.YYYY", temp, sizeof(temp));
				}
				strBcpy(pParm->paramValue, temp);
			}
		}
		strMcat(str, strsize, pParm->paramValue);
		break;
	
	case tpUsername:
	case tpGroup:
	case tpPtGroup:
	case tpChiefGroup:
	case tpString:
	case tpPassword:
	case tpNewPassword:
		if (getDefault)
		{
			strBcpy(pParm->paramValue, pParm->paramDflt);
		} else
		{
			ObjcString(itk->Tree, pParm->obLow, pParm->paramValue, sizeof(pParm->paramValue));
			strleft(pParm->paramValue);
			strdelRightSpace(pParm->paramValue);
			if (pParm->paramValue[0] == '\0')
			{
				if (!pParm->optional)
				{
					/* Fehlermeldung */
					StripShortcut(pParm->paramName, pName, sizeof(pName));
					sprintf(temp, MausTauschrsc_WrkString(eEmptyParm), pName);
					MTE_info(temp);
					return FALSE;
				}
			}
			/* Defaulteintraege uebernehmen */
			pMacro = Lists_CurrentEntry(&itk->macroListe);
			if (pMacro != NULL && pMacro->resSetup[0] == '\0')
			{
				switch (pParm->paramType)
				{
				case tpUsername:
					strBcpy(userDefault,  pParm->paramValue);
					break;
				case tpGroup:
					strBcpy(groupDefault, pParm->paramValue);
					break;
				case tpChiefGroup:	
					strBcpy(chgrpDefault, pParm->paramValue);
					break;
				case tpPtGroup:
					strBcpy(ptgrpDefault, pParm->paramValue);
					break;
				case tpConst:
				case tpEnum:
				case tpEnumSet:
				case tpMTDate:
				case tpDate:
				case tpString:
				case tpPassword:
				case tpNewPassword:
				case tpInt:
				case tpRange:
					break;
				}
			}
		}
		strMcat(str, strsize, pParm->paramValue);
		break;
	
	case tpInt:
	case tpRange:
		if (getDefault)
		{
			strBcpy(pParm->paramValue, pParm->paramDflt);
		} else
		{
			ObjcString(itk->Tree, pParm->obLow, pParm->paramValue, sizeof(pParm->paramValue));
			strdelSpace(pParm->paramValue);
			if (pParm->paramValue[0] == '+')
				strCpy(pParm->paramValue, pParm->paramValue + 1);
			if (pParm->paramValue[0] == '\0')
			{
				if (!pParm->optional)
				{
					/* Fehlermeldung */
					StripShortcut(pParm->paramName, pName, sizeof(pName));
					sprintf(temp, MausTauschrsc_WrkString(eEmptyParm), pName);
					MTE_info(temp);
					return FALSE;
				}
			} else
			{
				lIntVal = StrToLInt(pParm->paramValue, NULL, NULL);
				if (pParm->paramType == tpRange)
				{
					if (lIntVal < pParm->paramMin || lIntVal > pParm->paramMax)
					{
						/* Fehlermeldung */
						StripShortcut(pParm->paramName, pName, sizeof(pName));
						sprintf(temp, MausTauschrsc_WrkString(eRangeAlt), pName, pParm->paramMin, pParm->paramMax);
						MTE_info(temp);
						return FALSE;
					}
				}
				if (lIntVal >= 0)
				{
					pParm->paramValue[0] = '+';
					IntToStr(lIntVal, 0, pParm->paramValue + 1, sizeof(pParm->paramValue) - 1);
				} else
				{
					IntToStr(lIntVal, 0, pParm->paramValue, sizeof(pParm->paramValue));
				}
			}
		}
		strMcat(str, strsize, pParm->paramValue);
		break;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21i */
LOCAL _VOID WriteCmdToFile(CONST _UBYTE *str)
{
	_UBYTE fName[PATH_MAX];
	TEXTFILENO fHdl;
	_UBYTE scrap[256];
	
	if (str[0] == '\0')
		return;
	strBcpy(fName, MessagePath);
	F_Path_Append(fName, userCmdFile);
	if (mtTextfiles_OpenTextfile(fName, APPEND, 512, &fHdl) == GERR_OK ||
		mtTextfiles_OpenTextfile(fName, WRITE, 512, &fHdl) == GERR_OK)
	{
		mtTextfiles_WriteChar(fHdl, ':');
		mtTextfiles_WriteLine(fHdl, str);
		mtTextfiles_WriteLn(fHdl);
		mtTextfiles_CloseTextfile(&fHdl);
		Protokoll_SendPathUpdate(MessagePath);
	} else
	{
		sprintf(scrap, Rsc_WrkString(MTE_noFileWrite), fName);
		MTE_info(scrap);
	}
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21i */
LOCAL _BOOL SaveITKMacro(ITK *itk)
{
	OneCommand *pCmd;
	OneMacro *pMacro;
	_UBYTE fName[PATH_MAX];
	TEXTFILENO fHdl;
	_UBYTE scrap[260];
	_UWORD macroCount;
	_BOOL isOpen;
	
	isOpen = FALSE;
	macroCount = 1;
	Lists_ResetList(itk->itkCmds);
	while ((pCmd = Lists_NextEntry(itk->itkCmds)) != NULL)
	{
		if (pCmd->macro)
		{
			if (!isOpen)
			{
				strBcpy(fName, DataPath);
				F_Path_Append(fName, itkMacroName);
				if (mtTextfiles_OpenTextfile(fName, WRITE, 512, &fHdl) != GERR_OK)
				{
					sprintf(scrap, Rsc_WrkString(MTE_noFileWrite), fName);
					MTE_info(scrap);
					return FALSE;
				}
				isOpen = TRUE;
			}
			sprintf(scrap, "#-%d", macroCount++);
			mtTextfiles_WriteLine(fHdl, scrap);
			mtTextfiles_WriteLn(fHdl);
			mtTextfiles_WriteLine(fHdl, "GMakro");
			mtTextfiles_WriteLn(fHdl);
			sprintf(scrap, "N%s", pCmd->cmdName);
			mtTextfiles_WriteLine(fHdl, scrap);
			mtTextfiles_WriteLn(fHdl);
			
			Lists_ResetList(pCmd->itkMacro);
			while ((pMacro = Lists_NextEntry(pCmd->itkMacro)) != NULL)
			{
				sprintf(scrap, "s%d,%d,%d", pMacro->aktStep, pMacro->command, pMacro->version);
				mtTextfiles_WriteLine(fHdl, scrap);
				mtTextfiles_WriteLn(fHdl);
				if (pMacro->defSetup[0] != '\0')
				{
					sprintf(scrap, "d%s", pMacro->defSetup);
					mtTextfiles_WriteLine(fHdl, scrap);
					mtTextfiles_WriteLn(fHdl);
				}
			}
		}
	}
	if (isOpen)
	{
		mtTextfiles_CloseTextfile(&fHdl);
	}
	else
	{
		/* kein Makro da, also CATMACRO.INF loeschen */
		CatFiles_DeleteFile(DataPath, itkMacroName);
	}
	Protokoll_SendPathUpdate(DataPath);
	return TRUE;
}					

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID setMaxIdent(_VOID)
{
	_UBYTE varName[256];
	_WORD i, maxIdent, newMax;
	_WORD version;

	ConfVars_GetConfigInt(cMTCmdMaxIdent, &maxIdent);
	newMax = -1;
	for (i = 0; i <= maxIdent; i++)
	{
		make_varname(varName, sizeof(varName), cMTCmdVersion, i);
		if (ConfVars_GetConfigInt(varName, &version))
			newMax = i;
	}
	if (newMax < 0)
		ConfVars_DeleteConfigVar(cMTCmdMaxIdent);
	else
		ConfVars_SetConfigInt(cMTCmdMaxIdent, newMax);
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Uebersetzt die Einstellungen aus dem Dialog 'Infile konfigurieren'
 * in die entsprechenden MT-Kommandos. Diese Funktion wird nur
 * uebergangsweise benoetigt, um die alten Einstellungen
 * uebernehmen zu koennen
 */
LOCAL _VOID transformCommands(List *itkCmds, _BOOL deleteOld)
{
	_BOOL bool;
	_UBYTE varName[256];
	OneCommand *pCmd;
	OneParam *pParam;
	_WORD maxIdent;
	_BOOL isnew;
	ConfVars_GetConfDefBool(cItkTransformed, &isnew, FALSE);
	if(isnew)
		return;
	if(deleteOld)
		ConfVars_SetConfigBool(cItkTransformed, TRUE);
	
	Lists_ResetList(itkCmds);
	ConfVars_GetConfDefInt(cMTCmdMaxIdent, &maxIdent, -1);
	while ((pCmd = Lists_NextEntry(itkCmds)) != NULL)
	{
		if (!pCmd->title)
		{
			pParam = NULL;
			if (pCmd->hasParms)
			{
				Lists_ResetList(&pCmd->cmdParms);
				pParam = Lists_NextEntry(&pCmd->cmdParms);
			}
			switch (pCmd->cmdIdent)
			{
			case 1:		/* Persoenliche Mitteilungen anfordern */
				ConfVars_GetConfDefBool(cRequestPmails, &bool, TRUE);
				if (deleteOld)
					ConfVars_DeleteConfigVar(cRequestPmails);
				if (pParam != NULL && !pParam->hasDefault)
				{
					strBcpy(pParam->paramDflt, "PN");
					pParam->hasDefault = TRUE;
				}
				make_varname(varName, sizeof(varName), cMTCmd, pCmd->cmdIdent);
				ConfVars_SetConfigString(varName, bool ? "PN" : "");
				make_varname(varName, sizeof(varName), cMTCmdVersion, pCmd->cmdIdent);
				ConfVars_SetConfigInt(varName, pCmd->cmdVersion);
				if (maxIdent < pCmd->cmdIdent)
					maxIdent = pCmd->cmdIdent;
				break;
			case 2:		/* Eigene persoenliche Mitteilungen zurueckschicken */
				ConfVars_GetConfDefBool(cRequestBackOwnPmails, &bool, TRUE);
				if (deleteOld)
					ConfVars_DeleteConfigVar(cRequestBackOwnPmails);
				if (pParam != NULL && !pParam->hasDefault)
				{
					strBcpy(pParam->paramDflt, "");
					pParam->hasDefault = TRUE;
				}
				make_varname(varName, sizeof(varName), cMTCmd, pCmd->cmdIdent);
				ConfVars_SetConfigString(varName, bool ? "PE" : "");
				make_varname(varName, sizeof(varName), cMTCmdVersion, pCmd->cmdIdent);
				ConfVars_SetConfigInt(varName, pCmd->cmdVersion);
				if (maxIdent < pCmd->cmdIdent)
					maxIdent = pCmd->cmdIdent;
				break;
			case 3:		/* PM-Status anfordern */
				ConfVars_GetConfDefBool(cSendPSN, &bool, TRUE);
				if (deleteOld)
					ConfVars_DeleteConfigVar(cSendPSN);
				if (pParam != NULL && !pParam->hasDefault)
				{
					strBcpy(pParam->paramDflt, "PSN");
					pParam->hasDefault = TRUE;
				}
				make_varname(varName, sizeof(varName), cMTCmd, pCmd->cmdIdent);
				ConfVars_SetConfigString(varName, bool ? "PSN" : "PSA");
				make_varname(varName, sizeof(varName), cMTCmdVersion, pCmd->cmdIdent);
				ConfVars_SetConfigInt(varName, pCmd->cmdVersion);
				if (maxIdent < pCmd->cmdIdent)
					maxIdent = pCmd->cmdIdent;
				break;
			case 4:		/* Oeffentliche Mitteilungen anfordern */
				ConfVars_GetConfDefBool(cRequestGrpMsgs, &bool, TRUE);
				if (deleteOld)
					ConfVars_DeleteConfigVar(cRequestGrpMsgs);
				if (pParam != NULL && !pParam->hasDefault)
				{
					strBcpy(pParam->paramDflt, "ON");
					pParam->hasDefault = TRUE;
				}
				make_varname(varName, sizeof(varName), cMTCmd, pCmd->cmdIdent);
				ConfVars_SetConfigString(varName, bool ? "ON" : "");
				make_varname(varName, sizeof(varName), cMTCmdVersion, pCmd->cmdIdent);
				ConfVars_SetConfigInt(varName, pCmd->cmdVersion);
				if (maxIdent < pCmd->cmdIdent)
					maxIdent = pCmd->cmdIdent;
				break;
			case 5:		/* Eigene oeffentliche Mitteilungen zuruecksenden */
				if (pParam != NULL && !pParam->hasDefault)
				{
					strBcpy(pParam->paramDflt, "");
					pParam->hasDefault = TRUE;
				}
				break;
			case 6:		/* Gruppenumbenennungsinformationen anfordern */
				if (pParam != NULL && (pParam = Lists_NextEntry(&pCmd->cmdParms)) != NULL && !pParam->hasDefault)
				{
					strBcpy(pParam->paramDflt, "");
					pParam->hasDefault = TRUE;
				}
				break;
			case 7:		/* Erweiterte Features einschalten */
				ConfVars_GetConfDefBool("GenerateExtendedInfile", &bool, FALSE);
				/* Wird immer geloescht */
				ConfVars_DeleteConfigVar("GenerateExtendedInfile");
				if (pParam != NULL && !pParam->hasDefault)
				{
					strBcpy(pParam->paramDflt, "");
					pParam->hasDefault = TRUE;
				}
				make_varname(varName, sizeof(varName), cMTCmd, pCmd->cmdIdent);
				ConfVars_SetConfigString(varName, bool ? "TS" : "");
				make_varname(varName, sizeof(varName), cMTCmdVersion, pCmd->cmdIdent);
				ConfVars_SetConfigInt(varName, pCmd->cmdVersion);
				if (maxIdent < pCmd->cmdIdent)
					maxIdent = pCmd->cmdIdent;
				break;
			case 8:		/* Maximalgroesse des OUTFILEs einstellen */
				ConfVars_GetConfDefBool(cMemoryLineInInfile, &bool, TRUE);
				if (deleteOld)
				{
					ConfVars_DeleteConfigVar(cMemoryLineInInfile);
					if (bool)
					{
						make_varname(varName, sizeof(varName), cMTCmd, pCmd->cmdIdent);
						ConfVars_SetConfigString(varName, "M0");
						make_varname(varName, sizeof(varName), cMTCmdVersion, pCmd->cmdIdent);
						ConfVars_SetConfigInt(varName, pCmd->cmdVersion);
						if (maxIdent < pCmd->cmdIdent)
							maxIdent = pCmd->cmdIdent;
					}
				}
				if (pParam != NULL && (pParam = Lists_NextEntry(&pCmd->cmdParms)) != NULL)
				{
					if (!pParam->hasDefault)
					{
						strBcpy(pParam->paramDflt, "0");
						pParam->hasDefault = TRUE;
					}
					pParam->optional = TRUE;
					pParam->paramMin = 0;		/* 0 == automatisch bestimmen */
				}
				break;
			}
		}
	}
	ConfVars_SetConfigInt(cMTCmdMaxIdent, maxIdent);
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Gegenstueck zu TransformCommands: uebersetzt Einstellungen
 * aus dem Dialog in die internen Einstellungen.
 */

LOCAL _VOID checkCommand(OneCommand *pCmd)
{
	_UBYTE varName[256];
	OneParam *pParam;
	
	if (!pCmd->title)
	{
		pParam = NULL;
		if (pCmd->hasParms)
		{
			Lists_ResetList(&pCmd->cmdParms);
			pParam = Lists_NextEntry(&pCmd->cmdParms);
		}
		switch (pCmd->cmdIdent)
		{
		case 1:		/* Persoenliche Mitteilungen anfordern */
			break;
		case 2:		/* Eigene persoenliche Mitteilungen zurueckschicken */
			break;
		case 3:		/* PM-Status anfordern */
			break;
		case 4:		/* Oeffentliche Mitteilungen anfordern */
			break;
		case 5:		/* Eigene oeffentliche Mitteilungen zuruecksenden */
			/* keine interne Einstellung dafuer */
			break;
		case 6:		/* Gruppenumbenennungsinformationen anfordern */
			break;
		case 7:		/* Erweiterte Features einschalten */
			break;
		case 8:		/* Maximalgroesse des OUTFILEs einstellen */
			if (pParam != NULL && (pParam = Lists_NextEntry(&pCmd->cmdParms)) != NULL && pParam->paramValue[0] != '\0')
			{
			} else
			{
				/*
				 * Sonst Kommando loeschen (ist sonst ungueltig),
				 * und auch keine automatische Ermittlung
				 */
				make_varname(varName, sizeof(varName), cMTCmd, pCmd->cmdIdent);
				ConfVars_DeleteConfigVar(varName);
				make_varname(varName, sizeof(varName), cMTCmdVersion, pCmd->cmdIdent);
				ConfVars_DeleteConfigVar(varName);
			}
			break;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL ParseITKCommand(CONST _UBYTE *command, OneCommand *pCmd)
{
	OneParam *pParm, *next;
	size_t pos, nextPos;
	size_t len;
	OneEnum *pEnum;
	_WORD count;
	_UBYTE str[256];
	CONST _UBYTE *cp;
	_BOOL valid;
	_BOOL emptyFound;
	
	if (!pCmd->hasParms)
		return FALSE;
	if (command[0] == '\0')
	{
		Lists_ResetList(&pCmd->cmdParms);
		while ((pParm = Lists_NextEntry(&pCmd->cmdParms)) != NULL)
		{
			pParm->paramValue[0] = '\0'; 
		}
		return TRUE;
	}
	Lists_ResetList(&pCmd->cmdParms);
	pos = 0;
	pParm = Lists_NextEntry(&pCmd->cmdParms);
	while (pParm != NULL)
	{
		next = Lists_NextEntry(&pCmd->cmdParms);
		switch (pParm->paramType)
		{
		case tpConst:
			len = strLen(pParm->paramDflt);
			if (strnCmp(&command[pos], pParm->paramDflt, len) != 0)
				return FALSE;
			pos += len;
			break;
		
		case tpEnum:
			pParm->paramValue[0] = '\0';
			valid = FALSE;
			len = 0;
			if (pParm->hasEnums)
			{
				count = (_WORD)Lists_NoOfEntries(&pParm->paramEnum);
				Lists_ResetList(&pParm->paramEnum);
				emptyFound = TRUE;
				while ((pEnum = Lists_NextEntry(&pParm->paramEnum)) != NULL)
				{
					*str = '\0';
					/* Ersten Wert aus Aufzaehlung nehmen! */
					GetEnumValue(0, pEnum->enumValue, str, sizeof(str));
					len = strLen(str);
					if (len != 0 && strnCmp(&command[pos], str, len) == 0)
					{
						valid = TRUE;
						break;
					}
					if (len == 0)
						emptyFound = TRUE;
					if (count == 1)
					{
						*str = '\0';
						/* Zweiten Wert aus Aufzaehlung nehmen! */
						GetEnumValue(1, pEnum->enumValue, str, sizeof(str));
						len = strLen(str);
						if (len == 0 || strnCmp(&command[pos], str, len) == 0)
						{
							valid = TRUE;
							break;
						}
						len = 0;
						valid = TRUE;
						break;
					}
				}
				if (!valid && (emptyFound || pParm->optional))
				{
					valid = TRUE;
					len = 0;
				}
			}
			if (!valid)
				return FALSE;
			nextPos = pos + len;
			if (len >= sizeof(pParm->paramValue))
				len = sizeof(pParm->paramValue) - 1;
			strMcpy(pParm->paramValue, len + 1, &command[pos]);
			pos = nextPos;
			break;
	
		case tpEnumSet:
			if (pParm->hasEnums)
			{
				Lists_ResetList(&pParm->paramEnum);
				while ((pEnum = Lists_NextEntry(&pParm->paramEnum)) != NULL)
				{
					pEnum->enumCurValue[0] = '\0';
					valid = FALSE;
					len = 0;
					count = CheckEnumValue(pEnum->enumValue);
					if (count > 3)
					{
						/* Nicht behandelbar momentan */
						return FALSE;
					}
					*str = '\0';
					GetEnumValue(0, pEnum->enumValue, str, sizeof(str));
					len = strLen(str);
					if (len != 0 && strnCmp(&command[pos], str, len) == 0)
					{
						valid = TRUE;
						goto write;
					}
					*str = '\0';
					GetEnumValue(1, pEnum->enumValue, str, sizeof(str));
					len = strLen(str);
					if (len != 0 && strnCmp(&command[pos], str, len) == 0)
					{
						valid = TRUE;
						goto write;
					}
					if (count == 3)
					{
						*str = '\0';
						GetEnumValue(2, pEnum->enumValue, str, sizeof(str));
						len = strLen(str);
						if (len == 0 || strnCmp(&command[pos], str, len) == 0)
						{
							valid = TRUE;
						}
					}
					if (valid)
					{
write:					nextPos = pos + len;
						if (len >= sizeof(pEnum->enumCurValue))
							len = sizeof(pEnum->enumCurValue) - 1;
						strMcpy(pEnum->enumCurValue, len + 1, &command[pos]);
						pos = nextPos;
					}
				}
			}
			break;

		case tpString:
		case tpMTDate:
		case tpDate:
		case tpPassword:
		case tpNewPassword:
		case tpUsername:
		case tpGroup:
		case tpPtGroup:
		case tpChiefGroup:
			if (next != NULL)
			{
				if (next->paramType != tpConst)
					return FALSE;
				cp = strStr(&command[pos], next->paramDflt);
				if (cp == NULL)
					return FALSE;
				len = cp - &command[pos];
			} else
			{
				len = strLen(&command[pos]);
			}
			nextPos = pos + len;
			if (len >= sizeof(pParm->paramValue))
				len = sizeof(pParm->paramValue) - 1;
			strMcpy(pParm->paramValue, len + 1, &command[pos]);
			pos = nextPos;
			break;
		
		case tpInt:
		case tpRange:
			nextPos = pos;
			if (command[pos] == '+')
				nextPos++;
			(_VOID)StrToLInt(command, &nextPos, &valid);
			if (!valid && !pParm->optional)
				return FALSE;
			len = nextPos - pos;
			if (len >= sizeof(pParm->paramValue))
				len = sizeof(pParm->paramValue) - 1;
			strMcpy(pParm->paramValue, len + 1, &command[pos]);
			pos = nextPos;
			break;
		}
		pParm = next;
	}
	if (command[pos] != '\0')
		return FALSE;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
LOCAL _BOOL GetValues(OneCommand *pCmd, _BOOL doWrite, _BOOL preview, _BOOL getDefault, _BOOL setDefault)
{
	OneParam *pParm;
	OneMacro *pMacro;
	_UBYTE str[1024];
	_UBYTE tmp[1024];
	_WORD select;
	_WORD maxIdent;
	ITK *itk;
	MACRO *macro;
	
	/* Parameter und Objekte auslesen und pruefen */
	*str = '\0';
	itk = pCmd->itk;
	macro = itk->macro;
	if (pCmd->hasParms)
	{
		Lists_ResetList(&pCmd->cmdParms);
		while ((pParm = Lists_NextEntry(&pCmd->cmdParms)) != NULL)
		{
			if (!GetParam(itk, pParm, str, sizeof(str), getDefault))
				return FALSE;
		}
		if (preview)
		{
			sprintf(tmp, MausTauschrsc_WrkString(ePreview), str);
			MTE_info(tmp);
		}
		if (doWrite)
		{
			if (pCmd->cmdType == tcConfig)
			{
				if (itk->runMacro)
					select = 3;
				else
				{
					select = 1;
					if (!UserBLK.AutoSaveOptions)
						select = 2;
					/* Abfragen, ob es gespeichert oder nur einmal geschrieben werden soll */
					sprintf(tmp, MausTauschrsc_WrkString(eSaveCmd), pCmd->cmdName);
					select = mtAlerts_Alert(select, tmp);
				}
				if (select == 1 || select == 2)
				{
					/* Sichern in CATUSER.INF */
					make_varname(tmp, sizeof(tmp), cMTCmd, pCmd->cmdIdent);
					ConfVars_SetConfigString(tmp, str);
					make_varname(tmp, sizeof(tmp), cMTCmdVersion, pCmd->cmdIdent);
					ConfVars_SetConfigInt(tmp, pCmd->cmdVersion);
					ConfVars_GetConfDefInt(cMTCmdMaxIdent, &maxIdent, -1);
					pCmd->isInInf = TRUE;
					
					if (maxIdent < pCmd->cmdIdent)
					{
						ConfVars_SetConfigInt(cMTCmdMaxIdent, pCmd->cmdIdent);
					}
					checkCommand(pCmd);
					setMaxIdent();
					if (select == 2)
						UserInformation_WriteUserBlock();
				} else if (select == 3)
				{
					if (itk->runMacro)
					{
						pMacro = Lists_CurrentEntry(&itk->macroListe);
						strMcpy(pMacro->resSetup, sizeof(pMacro->resSetup), str);	
					} else
					{
						WriteCmdToFile(str);
					}
				}
			} else
			{
				if (itk->runMacro)
				{
					pMacro = Lists_CurrentEntry(&itk->macroListe);
					strMcpy(pMacro->resSetup, sizeof(pMacro->resSetup), str);	
				} else
				{
					WriteCmdToFile(str);
				}
			}
		}
		if (setDefault)
		{
			Lists_ResetList(macro->MacroList);
			while ((pMacro = Lists_NextEntry(macro->MacroList)) != NULL)
			{
				if (pMacro->selected)
				{
					strMcpy(pMacro->defSetup, sizeof(pMacro->defSetup), str);
					break;
				}
			}
		}
	}
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----------------------------------------------------------------------
 * Parser fuer das ITK
 *----------------------------------------------------------------------*/
/*DYJ 5.21b */
LOCAL _BOOL ParseDataType(CONST _UBYTE *typeStr, OneCommand *pCmd, OneParam *param, _BOOL warn)
{
	size_t pos;

	pos = 1;
	switch (typeStr[0])
	{
	case 'A':
		param->paramType = tpEnum;
		break;
	case 'M':
		param->paramType = tpEnumSet;
		break;
	case 'D':
		param->paramType = tpMTDate;
		break;
	case 'd':
		param->paramType = tpDate;
		break;
	case 'S':
		param->paramType = tpString;
		/* Maxlaenge auslesen */
		param->paramMin = StrToLInt(typeStr, &pos, NULL);
		break;
	case 'P':
		param->paramType = tpPassword;
		/* Maxlaenge auslesen */
		param->paramMin = StrToLInt(typeStr, &pos, NULL);
		break;
	case 'p':
		param->paramType = tpNewPassword;
		/* Maxlaenge auslesen */
		param->paramMin = StrToLInt(typeStr, &pos, NULL);
		break;
	case 'U':
		param->paramType = tpUsername;
		break;
	case 'I':
		param->paramType = tpInt;
		if (strChr(typeStr, ',') != NULL)
		{
			/* Direkt aufsplitten und Min und Max rauslesen */
			param->paramMin = StrToLInt(typeStr, &pos, NULL);
			++pos;					/* Komma ueberspringen */
			param->paramMax = StrToLInt(typeStr, &pos, NULL);
			if (param->paramMin <= param->paramMax)
				param->paramType = tpRange;
		}
		break;
	case 'G':
		if (typeStr[1] == 'P')
			param->paramType = tpPtGroup;
		else	
			param->paramType = tpGroup;
		break;
	case 'g':
		param->paramType = tpChiefGroup;
		break;
	case 'T':
		param->paramType = tpPtGroup;
		break;
	default:
		if (warn)
			MTE_numAlert(pCmd->cmdIdent, MausTauschrsc_WrkString(eDataType));
		return FALSE;
	}
	param->optional = typeStr[pos] == 'o';
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL WriteParam(OneParam *cParam, OneCommand *cCmd, _BOOL *paramDataDa)
{
	OneParam *pParam;

	pParam = NEW(OneParam, "WriteParam");
	if (pParam == NULL)
		return FALSE;
	/* Inhalt kopieren */
	*pParam = *cParam;
	/* An Liste anhaengen */
	if (!Lists_AppendEntry(&cCmd->cmdParms, pParam))
	{
		OFREE(pParam);
		return FALSE;
	}
	/* Daten zuruecksetzen */
	*paramDataDa = FALSE;
	cParam->paramName[0] = '\0';
	cParam->paramDflt[0] = '\0';
	cParam->paramMin = 0;
	cParam->paramMax = 0;
	cParam->optional = FALSE;
	cParam->paramHelp = NULL;
	cParam->hasEnums = FALSE;
	cParam->hasDefault = FALSE;
	/* Und Returncode von AppendEntry zurueckgeben */
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL WriteEnumData(OneParam *param, OneEnum *oneenum, _BOOL *enumDataDa)
{
	OneEnum *pEnum;

	pEnum = NEW(OneEnum, "WriteEnuemData");
	if (pEnum == NULL)
		return FALSE;
	/* Inhalt kopieren */
	*pEnum = *oneenum;

	/* An Liste anhaengen */
	if (!Lists_AppendEntry(&param->paramEnum, pEnum))
	{
		OFREE(pEnum);
		return FALSE;
	}
	oneenum->enumName[0] = '\0';
	oneenum->enumValue[0] = '\0';

	*enumDataDa = FALSE;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL WriteMacroData(OneCommand *cCmd, _UBYTE *str, _BOOL *macroInit)
{
	OneMacro *pMacro;
	List *newList;
	size_t pos;
	
	if (*macroInit == FALSE)
	{
		newList = NEW(List, "NewList");
		if (newList == NULL || !Lists_CreateList(newList, "CreateNewList"))
		{
			OFREE(newList);
			return FALSE;
		}
		cCmd->itkMacro = newList;
		*macroInit = TRUE;
	}
	pMacro = NEW(OneMacro, "pMacro");
	if (pMacro == NULL || !Lists_AppendEntry(cCmd->itkMacro, pMacro))
	{
		Lists_DeleteSimpleList(cCmd->itkMacro, mem_free, TRUE);
		return FALSE;
	}
	Lists_NextEntry(cCmd->itkMacro); /* current setzen */
	MemSetZeroStruct(*pMacro);
	pos = 0;
	pMacro->aktStep = StrToInt(str, &pos, NULL); 
	++pos;	
	pMacro->command = StrToInt(str, &pos, NULL);
	++pos;	
	pMacro->version = StrToInt(str, &pos, NULL);
	return TRUE;
}
	
/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL IsInternalCmd(OneCommand *cCmd)
{
	_WORD i;

	for (i = 0; i < maxInternalCmds; i++)
	{
		if (cCmd->cmdIdent == internalCmds[i].intIdent &&
			cCmd->cmdVersion == internalCmds[i].intVersion)
		{
			return TRUE;
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL WriteCmd(List *itkCmds, OneCommand *cmd, _BOOL *cmdDataDa)
{
	OneCommand *pCmd;
	_BOOL result;
	
	if (IsInternalCmd(cmd))
	{
		/* Speicher freigeben */
		if (cmd->hasParms)
			Lists_DeleteSimpleList(&cmd->cmdParms, delParam, TRUE);
		if (cmd->cmdHelp != NULL)
		{
			SFREE(cmd->cmdHelp);
			cmd->cmdHelp = NULL;
		}
		result = TRUE;
	} else
	{
		pCmd = NEW(OneCommand, "WriteCmd");
		if (pCmd == NULL)
			return FALSE;
		/* Inhalt kopieren */
		*pCmd = *cmd;
		pCmd->selected = FALSE;
		pCmd->title = FALSE;
		pCmd->macro = pCmd->cmdIdent < 0 ? TRUE : FALSE;
		pCmd->groupVisible = TRUE;
		pCmd->entryVisible = TRUE;
		/* An Liste anhaengen */
		result = Lists_AppendEntry(itkCmds, pCmd);
		if (!result)
			OFREE(pCmd);
	}
	cmd->cmdName[0] = '\0';
	cmd->cmdGroup[0] = '\0';
	cmd->cmdVersion = 0;
	cmd->hasParms = FALSE;
	cmd->cmdHelp = NULL;

	*cmdDataDa = FALSE;
	return result;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL compITKGroup(CONST _VOID *p1, CONST _VOID *p2, _VOID *private)
{
	CONST OneCommand *CONST *pA1;
	CONST OneCommand *CONST *pA2;
	CONST OneCommand *itk1;
	CONST OneCommand *itk2;
	_WORD res;

	UNUSED(private);	
	pA1 = p1;
	pA2 = p2;
	itk1 = *pA1;
	itk2 = *pA2;
	if ((res = strCmp(itk1->cmdGroup, itk2->cmdGroup)) == 0)
		return strSortCmp(itk1->cmdName, itk2->cmdName) < 0;
	return res < 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL compITKName(CONST _VOID *p1, CONST _VOID *p2, _VOID *private)
{
	CONST OneCommand *CONST *pA1;
	CONST OneCommand *CONST *pA2;
	CONST OneCommand *itk1;
	CONST OneCommand *itk2;
	_WORD res;

	UNUSED(private);	
	pA1 = p1;
	pA2 = p2;
	itk1 = *pA1;
	itk2 = *pA2;
	if ((res = strSortCmp(itk1->cmdName, itk2->cmdName)) == 0)
		return strCmp(itk1->cmdGroup, itk2->cmdGroup) < 0;
	return res < 0;
}

/*** ---------------------------------------------------------------------- ***/

/* sortiert die Kommandoliste */
GLOBAL _VOID DoITK_SortITK(ITK *itk)
{
	_ULONG i;
	_UBYTE lastGroup[8];
	_UBYTE var_name[32];
	_BOOL found;
	_BOOL visible;
	OneCommand *pCmd;
	OneCommand *pTitle;
	
	/* Die Liste aufraeumen ... */
	Lists_ResetList(itk->itkCmds);
	while ((pCmd = Lists_NextEntry(itk->itkCmds)) != NULL)
	{
		if (pCmd->title)
		{
			Lists_RemoveEntry(itk->itkCmds, delCmd);
		} else
		{
			pCmd->groupVisible = TRUE;	/* Alles sichtbar machen */
			pCmd->entryVisible = TRUE;
		}
	}
	
	/* ... sortieren ... */
	switch (itk->sort)
	{
	case SORT_CMD_GROUPS:
		GroupSelect_SortAnyList(itk->itkCmds, compITKGroup);
		/* ... und Ueberschriften wieder einfuegen */
		Lists_ResetList(itk->itkCmds);
		lastGroup[0] = '\0';
		visible = TRUE;
		while ((pCmd = Lists_NextEntry(itk->itkCmds)) != NULL)
		{
			/* Gruppenwechsel? */
			if (strCmp(pCmd->cmdGroup, lastGroup) != 0)	
			{
				strBcpy(lastGroup, pCmd->cmdGroup);
				
				/* Variable holen */
				strBcpy(var_name, cMTGroupVisible);
				strBcat(var_name, ".");
				strBcat(var_name, pCmd->cmdGroup);
				ConfVars_GetConfDefBool(var_name, &visible, TRUE);

				/* Titeleintrag erzeugen */
				pTitle = NEW(OneCommand, "WriteTitle");
				if (pTitle == NULL)
					return;	
				MemSetZeroStruct(*pTitle);
				pTitle->title = TRUE;
				pTitle->groupVisible = visible;		/* Je nach Einstellung */
				pTitle->entryVisible = TRUE;		/* Auf jeden Fall sichtbar */
				strBcpy(pTitle->cmdGroup, pCmd->cmdGroup);
				
				/* Titelueberschrift suchen und kopieren */
				found = FALSE;
				for (i = 0; i < MaxGroupNames - 1; i++)
				{
					if (strCmp(groupNames[i].cmdGroupId, pCmd->cmdGroup) == 0)
					{
						strBcpy(pTitle->cmdName, groupNames[i].cmdGroupTitle);
						found = TRUE;
					}
				} 
				/* Durchgelaufen, nix gefunden, also Defaulttitel */
				if (!found)
				{
					strBcpy(pTitle->cmdName, groupNames[MaxGroupNames - 1].cmdGroupTitle);
					strBcat(pTitle->cmdName, " '");
					strBcat(pTitle->cmdName, pCmd->cmdGroup);
					strBcat(pTitle->cmdName, "'");
				}	
				pCmd = Lists_PrevEntry(itk->itkCmds);	/* davor eintragen */
				Lists_InsertEntry(itk->itkCmds, pTitle);
				pCmd = Lists_NextEntry(itk->itkCmds);
			}	
			
			pCmd->groupVisible = FALSE;		/* Wird eh nicht ausgewertet ... */
			pCmd->entryVisible = visible;	/* Je nach Einstellung */
		}	
		break;
	
	case SORT_CMD_ALPHA:
		GroupSelect_SortAnyList(itk->itkCmds, compITKName);
		break;
	}
	CalcWidth(itk);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL ParseITK(TEXTFILENO file, List *itkCmds, ITK *itk, _BOOL warn)
{
	_UBYTE *newPtr;
	OneParam cParam;
	OneEnum cEnum;
	OneCommand cCmd;
	OneMacro *pMacro;

	_BOOL cmdDataDa;
	_BOOL enumDataDa;
	_BOOL paramDataDa;
	_BOOL macroInit;

	_UBYTE scrap[256];
	_UBYTE cLineType;

	paramDataDa = FALSE;
	enumDataDa = FALSE;
	cmdDataDa = FALSE;
	macroInit = FALSE;

	MemSetZeroStruct(cParam);
	MemSetZeroStruct(cCmd);
	MemSetZeroStruct(cEnum);
	cCmd.itk = itk;
	cCmd.cmdHelp = NULL;
	cCmd.hasParms = FALSE;
	cCmd.cmdName[0] = '\0';
	cCmd.cmdGroup[0] = '\0';
	cCmd.cmdVersion = 0;
	cParam.paramName[0] = '\0';
	cParam.paramHelp = NULL;
	cParam.hasEnums = FALSE;
	cParam.hasDefault = FALSE;
	Lists_InitList(&cParam.paramEnum);
	
#define ParseError() \
	{ \
		if (cCmd.hasParms) \
			Lists_DeleteSimpleList(&cCmd.cmdParms, delParam, TRUE); \
		if (cCmd.macro) \
		{ \
			Lists_DeleteSimpleList(cCmd.itkMacro, mem_free, TRUE); \
			OFREE(cCmd.itkMacro); \
		} \
		if (cCmd.cmdHelp != NULL) \
			SFREE(cCmd.cmdHelp); \
		if (cParam.hasEnums) \
			Lists_DeleteSimpleList(&cParam.paramEnum, delEnum, TRUE); \
		if (cParam.paramHelp != NULL) \
			SFREE(cParam.paramHelp); \
		return FALSE; \
	}
	
	while (!mtTextfiles_EndofText(file))
	{
		mtTextfiles_ReadLine(file, scrap, sizeof(scrap));
		mtTextfiles_ReadLn(file);
		/* Erstes Zeichen extrahieren und loeschen */
		cLineType = scrap[0];
		strCpy(scrap, scrap + 1);
		switch (cLineType)
		{
		case '#':							/* ID des Kommandos */
			if (enumDataDa)
			{
				if (!WriteEnumData(&cParam, &cEnum, &enumDataDa))
					ParseError();
			}
			if (paramDataDa)
			{
				if (!WriteParam(&cParam, &cCmd, &paramDataDa))
					ParseError();
			}
			if (cmdDataDa)
			{
				if (!WriteCmd(itkCmds, &cCmd, &cmdDataDa))
					ParseError();
			}
			cCmd.cmdIdent = StrToInt(scrap, NULL, NULL);
			macroInit = FALSE;
			cmdDataDa = TRUE;
			break;
		
		case 'N':							/* Name des Kommandos */
			strBcpy(cCmd.cmdName, scrap);
			break;

		case 'V':							/* Version des Kommandos */
			cCmd.cmdVersion = StrToInt(scrap, NULL, NULL);
			break;

		case 'G':							/* Gruppe des Kommandos */
			strBcpy(cCmd.cmdGroup, scrap);
			break;

		case 'H':							/* Hilfetext */
			if (cCmd.cmdHelp == NULL)
			{
				cCmd.cmdHelp = STRDUP(scrap, "ParseITK: cmdHelp1");
				if (cCmd.cmdHelp == NULL)
					ParseError();
			} else
			{
				newPtr = MALLOC(strLen(cCmd.cmdHelp) + strLen(scrap) + 2, "ParseITK: cmdHelp2");
				if (newPtr == NULL)
					ParseError();
				/* String zusammensetzen */
				strCpy(newPtr, cCmd.cmdHelp);
				strCat(newPtr, " ");
				strCat(newPtr, scrap);
				SFREE(cCmd.cmdHelp);
				cCmd.cmdHelp = newPtr;
			}
			break;
		
		case 'T':							/* Kommandotyp */
			switch (scrap[0])
			{
			case 'K':
				cCmd.cmdType = tcConfig;
				break;
			case 'E':
				cCmd.cmdType = tcSingle;
				break;
			case 'B':
				cCmd.cmdType = tcMultiple;
				break;
			default:
				cCmd.cmdType = tcSingle;
				break;
			}
			break;

		case 'C':							/* Konstanter Syntax-Teil, neuer Parameter */
			if (enumDataDa)
			{
				if (!WriteEnumData(&cParam, &cEnum, &enumDataDa))
					ParseError();
			}
			if (paramDataDa)
			{
				if (!WriteParam(&cParam, &cCmd, &paramDataDa))
					ParseError();
			} else
			{
				if (!Lists_CreateList(&cCmd.cmdParms, "ParseITK: cmdParms"))
					ParseError();
				cCmd.hasParms = TRUE;
			}
			strBcpy(cParam.paramDflt, scrap);
			cParam.paramType = tpConst;
			paramDataDa = TRUE;
			break;

		case 'F':							/* Variabler Parameter, Datentyp */
			if (enumDataDa)
			{
				if (!WriteEnumData(&cParam, &cEnum, &enumDataDa))
					ParseError();
			}
			if (paramDataDa)
			{
				if (!WriteParam(&cParam, &cCmd, &paramDataDa))
					ParseError();
			} else
			{
				if (!Lists_CreateList(&cCmd.cmdParms, "ParseITK: cmdParms"))
					ParseError();
				cCmd.hasParms = TRUE;
			}
			if (!ParseDataType(scrap, &cCmd, &cParam, warn))
				ParseError();
			paramDataDa = TRUE;
			break;

			/* Einzelne Teile eines Parameters */
		case 'K':							/* Name des Parameters */
			strBcpy(cParam.paramName, scrap);
			break;

		case 'D':							/* Defaultwert des Parameters */
			strBcpy(cParam.paramDflt, scrap);
			cParam.hasDefault = TRUE;
			break;

		case 'L':							/* Hilfe zu einem Parameter */
			if (cParam.paramHelp == NULL)
			{
				cParam.paramHelp = STRDUP(scrap, "ParseITK: paramHelp1");
				if (cParam.paramHelp == NULL)
					ParseError();
			} else
			{
				newPtr = MALLOC(strLen(cParam.paramHelp) + strLen(scrap) + 2, "ParseITK: ParamHelp2");
				if (newPtr == NULL)
					ParseError();
				/* String zusammensetzen */
				strCpy(newPtr, cParam.paramHelp);
				strCat(newPtr, " ");
				strCat(newPtr, scrap);
				SFREE(cParam.paramHelp);
				cParam.paramHelp = newPtr;
			}
			break;
		
		case 'A':							/* Aufzaehlungsteil des Parameters */
			if (enumDataDa)
			{
				if (!WriteEnumData(&cParam, &cEnum, &enumDataDa))
					ParseError();
			} else
			{
				if (!Lists_CreateList(&cParam.paramEnum, "ParseITK: paramEnum"))
					ParseError();
				cParam.hasEnums = TRUE;
			}
			strBcpy(cEnum.enumValue, scrap);
			enumDataDa = TRUE;
			break;
			
		case 'B':							/* Kurzbeschreibung eines Enumwertes */
			strBcpy(cEnum.enumName, scrap);
			break;
			
		case 's':							/* Makroschritt */
			if (cmdDataDa)
			{
				if (!WriteMacroData(&cCmd, scrap, &macroInit))
					ParseError();
			}
			break;
			
		case 'd':							/* Defaulteinstellung des Makros */
			if (macroInit)
			{
				pMacro = Lists_CurrentEntry(cCmd.itkMacro);
				strBcpy(pMacro->defSetup, scrap);
			}	
			break;				
		}
	}
	if (enumDataDa)
	{
		if (!WriteEnumData(&cParam, &cEnum, &enumDataDa))
			ParseError();
	}
	if (paramDataDa)
	{
		if (!WriteParam(&cParam, &cCmd, &paramDataDa))
			ParseError();
	}
	if (cmdDataDa)
	{
		if (!WriteCmd(itkCmds, &cCmd, &cmdDataDa))
			ParseError();
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Kommandoliste wieder freigeben
 */
GLOBAL _VOID DoITK_FreeCmdList(List *itkCmds)
{
	/* Liste wieder entfernen */
	Lists_DeleteSimpleList(itkCmds, delCmd, TRUE);
	SFREE(itkCmds);
}

/*** ---------------------------------------------------------------------- ***/

/* Kommandoliste aus ITK aufbauen
 */
/*DYJ 5.21b */	/*DYJ 5.21i */
LOCAL _VOID *iInitCmdList(ITK *itk, _BOOL warn)
{
	TEXTFILENO itkFile, macroFile;
	_UBYTE fname[PATH_MAX];
	List *itkCmds;
	_UBYTE scrap[256];
	
	if (!Infofiles_GetInfoFilename(ITK_SHORT_NAME, fname))
	{
		Infofiles_SaveInfoInf();
		if (warn)
			MTE_infoRSC(noITKAlert);
		return NULL;
	}
	itkCmds = NULL;
	if (mtTextfiles_OpenTextfile(fname, READ, 0, &itkFile) == GERR_OK)
	{
		itkCmds = NEW(List, "InitCmdList");
		if (itkCmds != NULL)
		{
			if (Lists_CreateList(itkCmds, "InitCmdList"))
			{
				if (!ParseITK(itkFile, itkCmds, itk, warn))
				{
					/* Bisher allozierten Kram wieder freigeben */
					DoITK_FreeCmdList(itkCmds);
					itkCmds = NULL;
				} else
				{
					/* Versuchen, CATMACRO.DAT zu oeffnen */
					strBcpy(fname, DataPath);
					F_Path_Append(fname, itkMacroName);
					if (mtTextfiles_OpenTextfile(fname, READ, 0, &macroFile) == GERR_OK)
					{
						if (ParseITK(macroFile, itkCmds, itk, FALSE))
						{
							DoITK_InitMacroList(itkCmds);
						}
						mtTextfiles_CloseTextfile(&macroFile);
					}
				}
			} else
			{
				OFREE(itkCmds);
				itkCmds = NULL;
			}
		}
		mtTextfiles_CloseTextfile(&itkFile);
	} else
	{
		if (warn)
		{
			sprintf(scrap, Rsc_WrkString(MTE_noFileOpen), fname);
			MTE_info(scrap);
		}
	}
	return itkCmds;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL DoITK_TransformCommands(_BOOL deleteOld)
{
	List *itkCmds;
	_WORD maxIdent;
	
	itkCmds = iInitCmdList(NULL, FALSE);
	if (itkCmds == NULL)
	{
		/*
		 * moeglicherweise keine ITK; sollte eigentlich nur noch
		 * nach einer Neuinstallation, vor dem ersten Tausch
		 * auftreten koennen; Default-Einstellungen setzen
		 */


		if(UserBLK.isInternet)
		{
			ConfVars_SetConfigString(cMTCmd "1", "PA");
			ConfVars_SetConfigString(cMTCmd "3", "");
			ConfVars_SetConfigString(cMTCmd "4", "OA");	/*DYJ 5.34a */
			ConfVars_SetConfigString(cMTCmd "6", "");
		}
		else
		{
			ConfVars_SetConfigString(cMTCmd "1", "PN");
			ConfVars_SetConfigString(cMTCmd "3", "PSN");
			ConfVars_SetConfigString(cMTCmd "4", "ON");	/*DYJ 5.34a */
			ConfVars_SetConfigString(cMTCmd "6", "REN");
		}
		
		ConfVars_SetConfigInt(cMTCmdVersion "1", 0);
		ConfVars_SetConfigString(cMTCmd "2", "");
		ConfVars_SetConfigInt(cMTCmdVersion "2", 0);
		ConfVars_SetConfigInt(cMTCmdVersion "3", 0);
/*DYJ 5.34a Start*/
/*DYJ  Ende; alt:	
		ConfVars_SetConfigString(cMTCmd "4", "ON");
*/
		ConfVars_SetConfigInt(cMTCmdVersion "4", 0);
		ConfVars_SetConfigString(cMTCmd "5", "");
		ConfVars_SetConfigInt(cMTCmdVersion "5", 0);
		ConfVars_SetConfigInt(cMTCmdVersion "6", 0);
		ConfVars_SetConfigString(cMTCmd "7", "");
		ConfVars_SetConfigInt(cMTCmdVersion "7", 0);
		ConfVars_SetConfigString(cMTCmd "8", "M0");
		ConfVars_SetConfigInt(cMTCmdVersion "8", 0);
		
		ConfVars_GetConfDefInt(cMTCmdMaxIdent, &maxIdent, -1);
		if (maxIdent < 8)
			maxIdent = 8;
		ConfVars_SetConfigInt(cMTCmdMaxIdent, maxIdent);
		return FALSE;
	}
	transformCommands(itkCmds, deleteOld);
	DoITK_FreeCmdList(itkCmds);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID checkInf(List *itkCmds)
{
	_UBYTE varName[128];
	_WORD version;
	OneCommand *pCmd;
	
	Lists_ResetList(itkCmds);
	while ((pCmd = Lists_NextEntry(itkCmds)) != NULL)
	{
		if (!pCmd->title)
		{
			make_varname(varName, sizeof(varName), cMTCmdVersion, pCmd->cmdIdent);
			pCmd->isInInf = ConfVars_GetConfigInt(varName, &version) /* && version == pCmd->cmdVersion */;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL List *DoITK_InitCmdList(_VOID)
{
	List *itkCmds;
	
	itkCmds = iInitCmdList(NULL, FALSE);
	if (itkCmds != NULL)
	{
		transformCommands(itkCmds, FALSE);
		checkInf(itkCmds);
	}
	return itkCmds;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Userinterface */

LOCAL _VOID lineToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)
{
	OneCommand *pCmd;

	UNUSED(env);
	pCmd = entry;
	if (pCmd == NULL)
		*str = '\0';
	else
		strMcpy(str, strsize, pCmd->cmdName);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID dragAndDrop(_VOID *entry, _VOID *env, _WORD x, _WORD y, HWND window)
{
	MACRO *macro;
	ITK *itk;
	OneCommand *pCmd;
	OneMacro *pMacro;

	UNUSED(x);
	UNUSED(y);
	itk = env;
	macro = itk->macro;
	pCmd = entry;
	
	if (window == macro->MacroWindow)
	{
		/* Neuen Eintrag anlegen */
		pMacro = NEW(OneMacro, "DDMacro");
		if (pMacro == NULL)
			return;
		MemSetZeroStruct(*pMacro);
		pMacro->command  = pCmd->cmdIdent;
		pMacro->version  = pCmd->cmdVersion;
		pMacro->cmdName  = pCmd->cmdName;

		if (!Lists_AppendEntry(macro->MacroList, pMacro))
		{
			OFREE(pMacro);
			return;
		}
		macro->listChanged = TRUE;
		Macro_RedrawList(macro);
		Macro_UpdateButtons(NULL, macro);
		Macro_RenumberSteps(macro->MacroList);
		itk->changed = TRUE;
	}	
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL selEntry(_VOID *entry, _VOID *env, _WORD line)
{
	OneCommand *pCmd;
	ITK *itk;

	UNUSED(line);
	pCmd = entry;
	itk = env;
	if (pCmd != NULL)
	{
		pCmd->selected = !pCmd->selected;
		if (itk->configModus)
		{
			if (!pCmd->title && !pCmd->macro)
				ListMan_SetDragDropProc(itk->CmdBox, dragAndDrop);
			else
				ListMan_SetDragDropProc(itk->CmdBox, FUNK_NULL);
		}		
	}		
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isSelected(_VOID *entry, _VOID *env)
{
	OneCommand *pCmd;

	UNUSED(env);
	pCmd = entry;
	if (pCmd == NULL)
		return FALSE;
	return pCmd->selected;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isEnabled(_VOID *entry, _VOID *env)
{
	UNUSED(entry);
	UNUSED(env);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *nextEntry(_VOID *list)
{
	OneCommand *pCmd;
	List *lp;
	
	lp = list;	
	while ((pCmd = Lists_NextEntry(lp)) != NULL)
	{
		if (pCmd->entryVisible)
			return pCmd;
	}
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *prevEntry(_VOID *list)
{
	OneCommand *pCmd;
	List *lp;
	
	lp = list;	
	while ((pCmd = Lists_PrevEntry(lp)) != NULL)
	{
		if (pCmd->entryVisible)
			return pCmd;
	}
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID resetList(_VOID *list)
{
	Lists_ResetList(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID countEntries(_VOID *l, _LONG *ll, _WORD *ww)
{
	List *lp;
	ITK *itk;
	
	lp = l;
	itk = lp->user;
	if (itk->listChanged)
	{
		CalcWidth(itk);
		itk->listChanged = FALSE;
	}
	*ww = itk->globalLength;
	*ll = 0;
	/* Nur sichtbare Eintraege zaehlen (MH) */
	Lists_ResetList(lp);
	while (nextEntry(lp) != NULL)
	{
		++(*ll);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID drawCmdEntry(_VOID *entry, _VOID *env, _WORD x, _WORD y, _WORD offset, CONST GRECT *clip)
{
	OneCommand *pCmd;
	ITK *itk;
	_UWORD eff;
	_WORD fillcol;
	GRECT fill;
	_WORD pxy[4];
	_BOOL isSel;
	
	UNUSED(offset);
	itk = env;
	fill.g_x = x;
	fill.g_y = y;
	fill.g_w = fill.g_x + itk->globalLength - 1;
	fill.g_h = fill.g_y + CharHeight - 1;
	pxy[0] = clip->g_x;
	pxy[1] = clip->g_y;
	pxy[2] = clip->g_x + clip->g_w - 1;
	pxy[3] = clip->g_y + clip->g_h - 1;
	
	vs_clip(PrivateWS, TRUE, pxy);
	vswr_mode(PrivateWS, MD_REPLACE);

	fillcol = WHITE;
	isSel = isSelected(entry, env);
	if (isSel)
		fillcol = BLACK;
	MouseOff();
	vsf_color(PrivateWS, fillcol);
	vr_recfl(PrivateWS, (_WORD *)&fill);
	
	if (entry != NULL)
	{
		pCmd = entry;
		eff = TXT_NORMAL;
		if (!isEnabled(entry, env))
			eff |= TXT_LIGHT;
		if (pCmd->title)
			eff |= TXT_THICKENED;
		if (isSel)
			vswr_mode(PrivateWS, MD_XOR);
		/* Jetzt hier den String fuer den Gruppennamen zusammenbauen */
		vst_effects(PrivateWS, eff);

		if (pCmd->isInInf)
			v_gtext(PrivateWS, x, y, "\010");	/*TODO*/
		else if (pCmd->macro && itk->sort == SORT_CMD_ALPHA)
			v_gtext(PrivateWS, x, y, "M");		/* Makro */
		if (!pCmd->title)
			x += 2 * CharWidth;
		v_gtext(PrivateWS, x, y, pCmd->cmdName);
		vst_effects(PrivateWS, 0);
		vswr_mode(PrivateWS, MD_REPLACE);
	}
	vs_clip(PrivateWS, FALSE, pxy);
	MouseOn();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID checkAndDraw(OBJECT *tree, _WORD obj, _BOOL cond)
{
	GRECT r;
	
	if (GetObjState(tree, obj, DISABLED) == cond)
	{
		EnableObjState(tree, obj, DISABLED, !cond);
		WinDials_WinDialDraw(tree, obj, 0, &r, FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID DoITK_UpdateButtons(_VOID *entry, _VOID *env)
{
	_BOOL oneSel;
	ITK *itk;
	OneCommand *pCmd;

	UNUSED(entry);
	itk = env;
	Lists_ResetList(itk->itkCmds);
	oneSel = Lists_ScanEntries(itk->itkCmds, Lists_forward, isSelected, NULL);
	pCmd = Lists_CurrentEntry(itk->itkCmds);
	if (oneSel)
	{
		if (itk->configModus)
		{
			if (pCmd->title)
			{
				checkAndDraw(itk->CmdBox, CMDOPEN,   TRUE);
				checkAndDraw(itk->CmdBox, CMDDELETE, FALSE);
				checkAndDraw(itk->CmdBox, CMDMAKRO,  FALSE);
			} else
			{
				checkAndDraw(itk->CmdBox, CMDOPEN,   FALSE);
				checkAndDraw(itk->CmdBox, CMDDELETE, FALSE);
				checkAndDraw(itk->CmdBox, CMDMAKRO,  FALSE);
			}
		} else
		{
			if (pCmd->title)
			{
				checkAndDraw(itk->CmdBox, CMDOPEN,   TRUE);
				checkAndDraw(itk->CmdBox, CMDDELETE, FALSE);
				checkAndDraw(itk->CmdBox, CMDMAKRO,  FALSE);
			}
			else if (pCmd->macro)
			{
				checkAndDraw(itk->CmdBox, CMDOPEN,   TRUE);
				checkAndDraw(itk->CmdBox, CMDDELETE, TRUE);
				checkAndDraw(itk->CmdBox, CMDMAKRO,  TRUE);
			}
			else	/* Eintrag */
			{
				checkAndDraw(itk->CmdBox, CMDOPEN,   TRUE);
				checkAndDraw(itk->CmdBox, CMDDELETE, pCmd->isInInf);
				checkAndDraw(itk->CmdBox, CMDMAKRO,  FALSE);
			}
		}
	} else
	{
		checkAndDraw(itk->CmdBox, CMDOPEN,   FALSE);
		checkAndDraw(itk->CmdBox, CMDDELETE, FALSE);
		checkAndDraw(itk->CmdBox, CMDMAKRO,  !itk->configModus);
	}	
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID DoITK_RedrawList(ITK *itk)
{
	WinDials_WinDialDraw(itk->CmdBox, CCMDBACK, MAX_DEPTH, NULL, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL itkButton(OBJECT *tree, _VOID *env, _WORD button, _WORD mx, _WORD my, _UWORD kstate, _WORD clicks)
{
	_UBYTE str[256];
	GRECT r;
	OneCommand *pCmd;
	OneMacro *pMacro;
	List help;
	CAT_GROUP_NUMBER gruppe;
	ITK *itk;
	_WORD x, y;
	_WORD select;
	
	UNUSED(mx);
	UNUSED(my);
	UNUSED(kstate);
	UNUSED(clicks);
	if(button==-1)
		return FALSE;
	button &= 0x7fff;
	DelObjState(tree, button, SELECTED);
	/* Jetzt mal nachsehen, was das fuer ein Button ist.
	 * Wenn es OK, Abbruch oder ein Skip-Button ist, dann gehen wir raus.
	 */
	pCmd = env;
	itk = pCmd->itk;
	itk->macroExitBut = button;
	pMacro = Lists_CurrentEntry(&itk->macroListe);
	str[0] = '\0';
	switch (itk->type[button])
	{
	case T_HELP:
		if (BuildHelp(pCmd, &help))
			CatHelp_DoInternalHelp(&help);
		WinDials_WinDialDraw(tree, button, 1, &r, FALSE);
		break;
	case T_OK:
	case T_SKIP_NEXT:
	case T_SKIP_PREV:
		/* Valid-Check der einzelnen Elemente */
		if (GetValues(pCmd, FALSE, FALSE, FALSE, FALSE))
			return TRUE;
		WinDials_WinDialDraw(tree, button, 1, &r, FALSE);
		break;
	case T_PREVIEW:
		/* Valid-Check der einzelnen Elemente */
		GetValues(pCmd, FALSE, TRUE, FALSE, FALSE);
		WinDials_WinDialDraw(tree, button, 1, &r, FALSE);
		break;
	case T_CANCEL:
		return TRUE;
	case T_USER:
		/* Userselect, Default vorbelegen */
		if (pMacro != NULL && pMacro->resSetup[0] != '\0')
			ObjcString(tree, button + 1, str, sizeof(str));
		else
			strMcpy(str, sizeof(str), userDefault);
		/* Neue Userauswahl (MH) */
		ListHelp_SetLastGroup(str);
		if (ListHelp_SelectGroup(str, sizeof(str), &gruppe, FALSE, FALSE, FALSE, gsmUsername) ||
			adr_SelectName(ADR_NORMAL,str, sizeof(str)))
		{
			SetObjcString(tree, button + 1, str);
			WinDials_WinDialDraw(tree, button + 1, 1, &r, FALSE);			/* (MH) */
			WinDials_WinEditChanged(tree, button + 1);				
		}
		WinDials_WinDialDraw(tree, button, 1, &r, FALSE);
		break;
	case T_CHIEF_GROUP:
		/* Gruppenselect nur fuer Chef, Default vorbelegen */
		if (pMacro != NULL && pMacro->resSetup[0] != '\0')
			ObjcString(tree, button + 1, str, sizeof(str));
		else
			strMcpy(str, sizeof(str), chgrpDefault);
		ListHelp_SetLastGroup(str);
		if (ListHelp_SelectGroup(str, sizeof(str), &gruppe, FALSE, FALSE, FALSE, gsmChief))
		{
			SetObjcString(tree, button + 1, str);
			WinDials_WinDialDraw(tree, button + 1, 1, &r, FALSE);			/* (MH) */
			WinDials_WinEditChanged(tree, button + 1);
		}
		WinDials_WinDialDraw(tree, button, 1, &r, FALSE);
		break;
	case T_PT_GROUP:
		/* PT-Gruppenselect, Default vorbelegen */
		if (pMacro != NULL && pMacro->resSetup[0] != '\0')
			ObjcString(tree, button + 1, str, sizeof(str));
		else
			strMcpy(str, sizeof(str), ptgrpDefault);
		ListHelp_SetLastGroup(str);
		if (ListHelp_SelectGroup(str, sizeof(str), &gruppe, FALSE, FALSE, FALSE, gsmProgteil))
		{
			SetObjcString(tree, button + 1, str);
			WinDials_WinDialDraw(tree, button + 1, 1, &r, FALSE);			/* (MH) */
			WinDials_WinEditChanged(tree, button + 1);
		}
		WinDials_WinDialDraw(tree, button, 1, &r, FALSE);
		break;
	case T_GROUP:
		/* Gruppenselect, Default vorbelegen */
		if (pMacro != NULL && pMacro->resSetup[0] != '\0')
			ObjcString(tree, button + 1, str, sizeof(str));
		else
			strMcpy(str, sizeof(str), groupDefault);
		ListHelp_SetLastGroup(str);
		if (ListHelp_SelectGroup(str, sizeof(str), &gruppe, FALSE, FALSE, FALSE, gsmOther))
		{
			SetObjcString(tree, button + 1, str);
			WinDials_WinDialDraw(tree, button + 1, 1, &r, FALSE);			/* (MH) */
			WinDials_WinEditChanged(tree, button + 1);
		}
		WinDials_WinDialDraw(tree, button, 1, &r, FALSE);
		break;
	case T_POPUP:
		objc_offset(tree, button, &x, &y);
		SetRadioSelect(tree, button, -1);
		select = mtPopups_TreePopup(&tree[button + 2], x, y, VDIUtil_GetXState(tree, button)) - 1;
		if (select < 0)
			select = VDIUtil_GetXState(tree, button);
		SetRadioSelect(tree, button, select);
		WinDials_WinDialDraw(tree, button, 1, &r, FALSE);
		break;
	case T_CIRCLE:
		select = VDIUtil_GetXState(tree, button - 1);
		select = (select + 1) % (tree[button+1].ob_tail - tree[button+1].ob_head + 1);
		SetRadioSelect(tree, button - 1, select);
		WinDials_WinDialDraw(tree, button - 1, 1, &r, FALSE);
		break;
	default:
		return FALSE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID itkGetSetValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exit)
{
	OneCommand *pCmd;
	ITK *itk;
	GRECT r;
	
	UNUSED(tree);
	if (!set)
	{
		pCmd = env;
		itk = pCmd->itk;
		if (itk->type[exit] == T_OK ||
		    itk->type[exit] == T_SKIP_NEXT ||
		    itk->type[exit] == T_SKIP_PREV)
		{
			GetValues(pCmd, !itk->configModus, FALSE, FALSE, itk->configModus);
			WinDials_WinDialDraw(itk->CmdBox, CCMDBACK, MAX_DEPTH, &r, FALSE);
		}
		itk->inCommand = FALSE;
		RemoveTree(itk);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL DoITK_HandleCommand(OneCommand *pCmd, _VOID *tMacro)
{
	ITK *itk;
	OneMacro *pMacro;
	_UBYTE str[1024];
	_UBYTE varName[256];
	
	itk = pCmd->itk;
	pMacro = tMacro;
	if (pMacro != NULL)
	{
		if (itk->runMacro)
		{
			/* schon Werte eingetragen? */
			if (pMacro->resSetup[0] != '\0')	
				ParseITKCommand(pMacro->resSetup, pCmd); /* ja, diese uebernehmen */			
			else
				ParseITKCommand(pMacro->defSetup, pCmd); /* nein, Default uebernehmen */			
		}
		if (itk->configModus)
		{
			ParseITKCommand(pMacro->defSetup, pCmd);			
		}		
	} else 
	{
		make_varname(varName, sizeof(varName), cMTCmd, pCmd->cmdIdent);
		if (ConfVars_GetConfigString(varName, str, sizeof(str)))
			ParseITKCommand(str, pCmd);
		else
			GetValues(pCmd, FALSE, FALSE, TRUE, FALSE);
	}
	if (!BuildTree(pCmd))
		return FALSE;
	if (!WinDials_OpenWinDial(itk->Tree, TRUE,
		itkGetSetValues,
		itkGetSetValues,
		itkButton,
		WinDials_defDraw,
		NULL,
		pCmd))
	{
		RemoveTree(pCmd->itk);
		return FALSE;
	}
	itk->inCommand = TRUE;
	if (pMacro != NULL)
		pMacro->opened = TRUE;
	/* Leider ein modaler Dialog, da nicht reentrant */
	WinDials_WinDialHandleEvents();
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID itkDeleteCmd(OneCommand *pCmd, ITK *itk)
{
	_UBYTE varName[256];
	
	UNUSED(itk);
	make_varname(varName, sizeof(varName), cMTCmd, pCmd->cmdIdent);
	ConfVars_DeleteConfigVar(varName);
	make_varname(varName, sizeof(varName), cMTCmdVersion, pCmd->cmdIdent);
	ConfVars_DeleteConfigVar(varName);
	pCmd->isInInf = FALSE;
	setMaxIdent();
	GetValues(pCmd, FALSE, FALSE, TRUE, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/* Versteckt/zeigt alle Gruppen (MH) */
LOCAL _VOID showHideAllGroups(ITK *itk, _BOOL visible)
{
	OneCommand *pCmd;
	_UBYTE var_name[256];
	
	Lists_ResetList(itk->itkCmds);
	while ((pCmd = Lists_NextEntry(itk->itkCmds)) != NULL)
	{
		if (pCmd->title)
		{
			pCmd->groupVisible = visible;
			pCmd->entryVisible = TRUE;
			
			strBcpy(var_name, cMTGroupVisible);
			strBcat(var_name, ".");
			strBcat(var_name, pCmd->cmdGroup);
			ConfVars_SetConfigBool(var_name, visible);
		} else
		{
			pCmd->groupVisible = FALSE;
			pCmd->entryVisible = visible;
		}
	}
	CalcWidth(itk);
}

/*** ---------------------------------------------------------------------- ***/

/* Versteckt/zeigt eine Gruppe (MH) */
LOCAL _VOID showHideOneGroup(ITK *itk, _UBYTE *group, _BOOL visible)
{
	OneCommand *pCmd;
	_UBYTE var_name[256];
	
	Lists_ResetList(itk->itkCmds);
	while ((pCmd = Lists_NextEntry(itk->itkCmds)) != NULL)
	{
		if (strCmp(group, pCmd->cmdGroup) == 0)
		{
			if (pCmd->title)
			{
				pCmd->groupVisible = visible;
				pCmd->entryVisible = TRUE;
				
				strBcpy(var_name, cMTGroupVisible);
				strBcat(var_name, ".");
				strBcat(var_name, pCmd->cmdGroup);
				ConfVars_SetConfigBool(var_name, visible);
			} else
			{
				pCmd->groupVisible = FALSE;
				pCmd->entryVisible = visible;
			}
		}
	}
	CalcWidth(itk);
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
LOCAL _BOOL itkCheckExit(OBJECT *tree, _VOID *env, _WORD button, _VOID *lastSel, _UWORD kstate)
{
	GRECT r;
	ITK *itk;
	MACRO *macro;
	_WORD select;
	
	UNUSED(tree);
	UNUSED(env);
	UNUSED(lastSel);
	UNUSED(kstate);
	itk = env;
	macro = itk->macro;
	button &= 0x7fff;
	DelObjState(tree, button, SELECTED);
	switch (button)
	{
	case CCANCEL:
		if (itk->changed)
		{
			select = mtAlerts_AlertRSC(1, eChangedMacro);
			if (select == 1)
			{
				SaveITKMacro(itk);
			} else if (select != 2)
			{
				break;
			}
		}
		if (itk->configModus)
		{
			WdwManager_CloseWindow(macro->MacroWindow, FALSE);
		} 	
		return TRUE;
	case COK:
		if (itk->changed)
		{
			SaveITKMacro(itk);		
		}
		if (itk->configModus)
			WdwManager_CloseWindow(macro->MacroWindow, FALSE);				
		return TRUE;
	case CHELP:
		CatHelp_DoHelp(CatHelp_itkhelp);
		break;
	}
	WinDials_WinDialDraw(tree, button, 0, &r, FALSE);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID setAndGetCmdValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exitBut)
{
	ITK *itk;
	
	UNUSED(tree);
	if (set)
	{
	} else
	{
		itk = env;
		if (exitBut == COK && itk->lastCmd != NULL)
		{
		}
		if (itk->inCommand)
			WinDials_CloseWinDial(itk->Tree);
		DoITK_FreeCmdList(itk->itkCmds);
		mtDials_ResetUserkeys(tree);
		OFREE(itk);
	}
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID doItkHelp(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID doItkHelp(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;
	ITK *itk;
	
	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(info);
	itk = env;
	CatHelp_DoHelp(CatHelp_itkhelp);
	EnableObjState(itk->CmdBox, obj, SELECTED, FALSE);
	WinDials_WinDialDraw(itk->CmdBox, obj, 1, &r, FALSE);
	*draw = FALSE;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
/*DYJ 5.30f Start*/
LOCAL _VOID doItkDelete(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID doItkDelete(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;
	ITK *itk;
	OneCommand *lastSel, *pCmd;
	_WORD select;
	
	UNUSED(info);
	*lstSel=-1;	/*DYJ 5.30f */
	*draw = FALSE;
	itk = env;
	lastSel = getSelected(itk->itkCmds);
	if (lastSel->macro)
	{
		select = mtAlerts_AlertRSC(2, eDeleteMacro);
		if (select == 1)
		{
			Lists_RemoveEntry(itk->itkCmds, delCmd);
			DoITK_SortITK(itk);
			pCmd = Lists_CurrentEntry(itk->itkCmds);
			DoITK_UpdateButtons(pCmd, itk);
			itk->changed = TRUE;
			*draw = TRUE;
		}
	} else
	{
		itkDeleteCmd(lastSel, env);
		*draw = TRUE;
	}
	DelObjState(itk->CmdBox, obj, SELECTED);
	WinDials_WinDialDraw(itk->CmdBox, obj, 1, &r, FALSE);
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID doMacro(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID doMacro(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;
	ITK *itk;
	MACRO *macro;
	HWND wdw;
	OneCommand *pCmd;
	_BOOL oneSel;
	
	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(info);
	itk = env;
	pCmd = getSelected(itk->itkCmds);
	/* Makro aendern oder neu anlegen? */
	Lists_ResetList(itk->itkCmds);
	oneSel = Lists_ScanEntries(itk->itkCmds, Lists_forward, isSelected, NULL);
	if (oneSel && !pCmd->macro)	/* Kein Makro, dann neu anlegen */
		pCmd = NULL;
	/* Zeigeraustausch fuer Kommunikation zwischen den Fenstern */
	if (Macro_ConfigMacro(pCmd, itk, &itk->macro))
	{	
		macro = itk->macro;
		wdw = WinDials_IsWinDial(macro->MacroBox);
		itk->configModus = (wdw > -1) ? TRUE : FALSE;
		Macro_UpdateButtons(NULL, itk->macro);
	}
	DelObjState(itk->CmdBox, obj, SELECTED);
	WinDials_WinDialDraw(itk->CmdBox, obj, 1, &r, FALSE);
	*draw = FALSE;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID openCmd(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID openCmd(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;
	ITK *itk;
	OneCommand *pCmd;
	OneMacro *pMacro;
	_WORD dummy;
	_UWORD udummy;
	_UWORD kstate;
	_BOOL done;
	
	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(info);
	itk = env;
	pCmd = getSelected(itk->itkCmds);
	*draw = FALSE;
	*exit = FALSE;
	
	if (pCmd != NULL)
	{
		graf_mkstate(&dummy, &dummy, &udummy, &kstate);
		/* Gruppen einklappen/ausfahren (MH) */
		if (pCmd->title)
		{
			if (CatGlobal_WithShift(kstate))	
			{
				/* Alle Gruppen zeigen/verstecken */
				showHideAllGroups(itk, !pCmd->groupVisible);
				*draw = TRUE;	
			} else
			{
				/* Eine Gruppe zeigen/verstecken */
				showHideOneGroup(itk, pCmd->cmdGroup, !pCmd->groupVisible);
				*draw = TRUE;
			}
		} else if (pCmd->macro)
		{
			/* Variablen initialisieren ... */
			userDefault[0]  = '\0';
			groupDefault[0] = '\0';
			chgrpDefault[0] = '\0';
			ptgrpDefault[0] = '\0';
			itk->macroListe = *pCmd->itkMacro;
			Lists_ResetList(&itk->macroListe);
			while ((pMacro = Lists_NextEntry(&itk->macroListe)) != NULL)
			{
				pMacro->resSetup[0] = '\0';
				pMacro->opened = FALSE;
			}
			/* ... und Liste abarbeiten */
			Lists_ResetList(&itk->macroListe);
			Lists_NextEntry(&itk->macroListe);
			itk->runMacro = TRUE;
			done = FALSE;
			while (!done)
			{
				pMacro = Lists_CurrentEntry(&itk->macroListe);
				pCmd = pMacro->pCommand;
				if (!DoITK_HandleCommand(pCmd, pMacro))
				{
					/* Fehler, Ende */
					done = TRUE;
				} else
				{
					switch (itk->type[itk->macroExitBut])
					{
					case T_CANCEL:
						/* Raus aus der Schleife */
						done = TRUE;
						break;
					case T_SKIP_NEXT:
						/* Naechstes Kommando */
						Lists_NextEntry(&itk->macroListe);
						break;
					case T_SKIP_PREV:
						/* Vorheriges Kommando */
						Lists_PrevEntry(&itk->macroListe);
						break;
					case T_OK:
						if (pMacro->aktStep == pMacro->totalSteps)
						{
							/* Kommandos schreiben */
							Lists_ResetList(&itk->macroListe);
							while ((pMacro = Lists_NextEntry(&itk->macroListe)) != NULL)
							{
								WriteCmdToFile(pMacro->resSetup);
							}
							done = TRUE;
						} else
						{
							/* Naechstes Kommando */
							Lists_NextEntry(&itk->macroListe);
						}
						break;
					}
				}
			}
			itk->runMacro = FALSE;
		} else 		
		{
			if (!itk->configModus)
			{
				DoITK_HandleCommand(pCmd, NULL);
			}
		}
	}
	DelObjState(itk->CmdBox, obj, SELECTED);
	WinDials_WinDialDraw(itk->CmdBox, obj, 1, &r, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID switchSort(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID switchSort(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;
	ITK *itk;
	OBJECT *popup;
	OBJECT *tree;
	_WORD x, y;
	_WORD select;
	_WORD but;
	
	UNUSED(info);
	
	*lstSel=-1;	/*DYJ 5.30f */
	itk = env;
	popup = TreeAddr[CMDSORT];
	tree = itk->CmdBox;
	obj &= 0x7fff;
	switch (obj)
	{
	case CMDSORT0:
	case CMDSORT1:
		but = CMDSORT1;
		objc_offset(tree, but, &x, &y);
		select = VDIUtil_GetXState(tree, but);
		select = mtPopups_TreePopup(popup, x, y, select) - 1;
		if (select >= 0)
		{
			VDIUtil_SetXState(tree, but, select);
			VDIUtil_SetXString(tree, but, popup, select);
			WinDials_WinDialDraw(tree, but, 0, &r, FALSE);
		}
		break;
	case CMDSORT2:
		select = VDIUtil_GetXState(tree, CMDSORT1);
		select = (select + 1) % 2;
		VDIUtil_SetXState(tree, CMDSORT1, select);
		VDIUtil_SetXString(tree, CMDSORT1, popup, select);
		WinDials_WinDialDraw(tree, CMDSORT1, 0, &r, FALSE);
		break;
	}
	itk->sort = VDIUtil_GetXState(tree, CMDSORT1);
	ConfVars_SetConfigInt(cMTSortType, itk->sort);
	DoITK_SortITK(itk);
	
	*draw = TRUE;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Kommando aus ITK auswaehlen und aufrufen
 * Ruft selber InitCmdList und FreeCmdList
 * auf
 */
GLOBAL _BOOL DoITK_SelectCmd(_VOID)
{
	ldHandler dialHandler;
	List *itkCmds;
	ITK *itk;
	OBJECT *popup;
	
	itk = NEW(ITK, "SelectCmd");
	if (itk == NULL)
		return FALSE;
	MemSetZeroStruct(*itk);
	itkCmds = iInitCmdList(itk, TRUE);
	if (itkCmds == NULL)
	{
		OFREE(itk);
		return FALSE;
	}
	transformCommands(itkCmds, TRUE);
	checkInf(itkCmds);
	
	itk->itkCmds = itkCmds;
	itkCmds->user = itk;

	/* Objektadresse holen */
	itk->CmdBox = TreeAddr[CMDBOX];
	itk->specials[0].objc = CHELP;
	itk->specials[0].proc = doItkHelp;
	itk->specials[1].objc = CMDSORT0;
	itk->specials[1].proc = switchSort;
	itk->specials[2].objc = CMDSORT1;
	itk->specials[2].proc = switchSort;
	itk->specials[3].objc = CMDSORT2;
	itk->specials[3].proc = switchSort;
	itk->specials[4].objc = CMDDELETE;
	itk->specials[4].proc = doItkDelete;
	itk->specials[5].objc = CMDMAKRO;
	itk->specials[5].proc = doMacro;
	itk->specials[6].objc = CMDOPEN;
	itk->specials[6].proc = openCmd;

	/* Jetzt das Object zusammenbauen */
	MausTauschrsc_AdjustDialogSize(itk->CmdBox, CMDBOX);

	/* Jetzt noch ITK sortieren */
	ConfVars_GetConfDefInt(cMTSortType, &itk->sort, SORT_CMD_GROUPS);
	DoITK_SortITK(itk);
	
	popup = TreeAddr[CMDSORT];
	VDIUtil_SetXState(itk->CmdBox, CMDSORT1, itk->sort);
	VDIUtil_SetXString(itk->CmdBox, CMDSORT1, popup, itk->sort);
	EnableObjState(itk->CmdBox, CMDOPEN, DISABLED, TRUE);
	EnableObjState(itk->CmdBox, CMDDELETE, DISABLED, TRUE);
	EnableObjState(itk->CmdBox, CMDMAKRO, DISABLED, FALSE);

	/* WindowDialog vorbereiten */
	ListDl_BuildLdHandler(itkCmds,
		resetList,
		nextEntry,
		prevEntry,
		countEntries,
		isEnabled,
		selEntry,
		lineToStr,
		isSelected,
		drawCmdEntry,
		FUNK_NULL,
		0, CharHeight,
		CharWidth, itk->globalLength,
		&dialHandler);

	/* ListWinDialog oeffnen */
	if (!ListDl_WinListDial(itk->CmdBox,
		ldArrows | ldSelect | ldDoubleExit,
		&dialHandler,
		CCMDBOX,
		CCMDBACK,
		COK,		/* okBut */
		CCANCEL,	/* cancelBut */
		itk,		/* env */
		CMDOPEN,	/* doubleClickButton */
		-1,			/* autoEdit */
		DoITK_UpdateButtons,
		itk->specials,
		7,
		itkCheckExit,
		setAndGetCmdValues,
		setAndGetCmdValues,
		&itk->lastCmd))
	{
		DoITK_FreeCmdList(itkCmds);
		OFREE(itk);
		return FALSE;
	}
	
	mtDials_SetUserkey(itk->CmdBox, CMDDELETE, 0x53, 0, FALSE, TRUE);
	
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL GetVersion(_VOID *elem, _VOID *info)
{
	_WORD *pInt;
	OneCommand *pCmd;

	pInt = info;
	pCmd = elem;
	return pCmd != NULL && pCmd->cmdIdent == *pInt;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert die aktuelle Version eines ITK-Kommandos
 * InitCmdList muss vorher aufgerufen werden,
 * und wenn die Daten nicht mehr benoetigt werden, dann FreeCmdList
 */
GLOBAL _BOOL DoITK_GetCmdVersion(List *itkCmds, _WORD cmd, OneCommand **pCmd, _WORD *version)
{
	OneCommand *tCmd;

	if (Lists_ScanEntries(itkCmds, Lists_forward, GetVersion, &cmd))
	{
		tCmd = Lists_CurrentEntry(itkCmds);
		*version = tCmd->cmdVersion;
		*pCmd = tCmd;
		return TRUE;
	}
	return FALSE;
}
