/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <groupcom.h>
#include <routines.h>

/*==============================================================*
 * Modul:               gruppenuebergreifende Kommentare        *
 * Autor:               Johannes Goettker-Schnetmann            *
 * erstellt am:         28.12.1991                              *
 * letzte Aenderung am: 30.01.1993                              *
 * Version:             0.1                                     *
 * Interne Version:     V#0002                                  *
 *==============================================================*
  
  Das Problem, aus einem String eine ID und eine Gruppe herauszuholen besteht sowohl
  in data.i als auch in grin_i. In ersterem Modul schon waehrend des Einfuegens,
  dann wird versucht anhand einer solchen Zeile eine Kommentarverkettung herzustellen
  und in grin beim Sprung ueber eine gruppenuebergreifende Kommentarverkettung.

 *----------------------------------------------------------------------------
 * Datum    Vers. Autor  Aenderung (Arbeitsbericht)                            
 *----------------------------------------------------------------------------
 * 28.12.91 0001  JGS    Erste Version
 * 30.01.93 0002  JGS    Gruppennamen werden "geupcased"
 *----------------------------------------------------------------------------
 */

#define KOMMENTAR "KOMMENTAR ZU "
#define DEMENTI   "DEMENTI ZU "

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Sieht nach, ob mit dem uebergebenen String eine Kommentarverkettung hergestellt */
/* werden kann. Falls ja, steht der Beginn der ID in <begin>                      */
LOCAL _BOOL IDPos(CONST _UBYTE *str, size_t *begin)
{
	size_t pos;
	_BOOL ok;
	
	pos = 0;
	ok = TRUE;
	while (KOMMENTAR[pos] != '\0' && ok)
	{
		if (ucase(str[pos]) != (_UBYTE)KOMMENTAR[pos])
			ok = FALSE;
		++pos;
	}
	if (ok)
	{
		*begin = pos;
		return TRUE;
	}
	pos = 0;
	ok = TRUE;
	while (DEMENTI[pos] != '\0' && ok)
	{
		if (ucase(str[pos]) != (_UBYTE)DEMENTI[pos])
			ok = FALSE;
		++pos;
	}
	if (ok)
	{
		*begin = pos;
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Hier jetzt noch das Ende suchen.. (Buchstabe ab/unter ' ' == 32C) */

LOCAL _VOID stampEnd(_UBYTE *str, _UBYTE *saveChar, _UBYTE **savePos, _BOOL *atSign, _UBYTE termChar)
{
	*atSign = FALSE;
	while (*str != '\0' && *str > termChar)
	{
		if (*str == '@')
			*atSign = TRUE;
		++str;
	}
	*saveChar = *str;
	*savePos = str;
	*str = '\0';
}

/*** ---------------------------------------------------------------------- ***/

/* Aus dem in <str> stehenden String eine ID herausholen oder auch nicht.. :-) */
/* In <saveChar> steht anschliessend ein Buchstabe, der bei <savePos> im        */
/* erfolgreichen Fall entnommen wurde */
GLOBAL _BOOL GroupComment_PrepareID(_UBYTE *pText, _UBYTE **id, _UBYTE *saveChar, _UBYTE **savePos)
{
	size_t bgn;
	_BOOL atSign;
	
	if (pText[0] == '-' ||
		pText[0] == (_UBYTE)KOMMENTAR[0] ||
		pText[0] == (_UBYTE)DEMENTI[0])
	{
		bgn = 0;
		if (pText[0] == '-')
		{
			*id = pText + 1;
		} else
		{
			*id = pText;
			if (IDPos(pText, &bgn))
				*id = pText + bgn;
			else
				return FALSE;
		}
		stampEnd(*id, saveChar, savePos, &atSign, ' ');      /* Ende markieren */
		
		if (atSign)
			return TRUE;
		**savePos = *saveChar; /* zuruecksetzen */
		return FALSE;
	}
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL iGroupPos(CONST _UBYTE *str, size_t *begin, CONST _UBYTE *vgl)
{
	size_t pos;
	
	for (pos = 0; vgl[pos] != '\0'; pos++)
		if (ucase(str[pos]) != vgl[pos])
			return FALSE;
	*begin = pos;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL GroupPos(CONST _UBYTE *str, size_t *begin)
{
	return iGroupPos(str, begin, "IN DER GRUPPE ") ||
		iGroupPos(str, begin, "IN GRUPPE ") ||
		iGroupPos(str, begin, "IN "); 
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL PrepareGroup(_UBYTE *pStr, _UBYTE **str)
{
	size_t bgn;
	_UBYTE old;
	_UBYTE *pos;
	_BOOL atSign;
	
	if (GroupPos(pStr, &bgn))
	{
		*str = pStr + bgn;
		stampEnd(*str, &old, &pos, &atSign, ' ' - 1);
		if (pos != *str && pos[-1] == ':')
		{
			*pos = old;
			pos[-1] = '\0';
		}
		strUpr(*str);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Aus dem in <str> stehenden String die ID und Gruppe herausholen oder nicht.    */
/* <id> zeigt anschliessend auf die nullterminierte ID                             */
/* und <gr> auf die nullterminiert Gruppe innerhalb von <str>                     */
GLOBAL _BOOL GroupComment_PrepareIDnGroup(_UBYTE *str, _UBYTE **id, _UBYTE **gr)
{
	_UBYTE *savePos;
	_UBYTE saveChar;
	
	return  GroupComment_PrepareID(str, id, &saveChar, &savePos) &&
			PrepareGroup(savePos + 1, gr);
}

