/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


/*
 * 30.10.02 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.16h'
 *	Fehler in der Gruppenverwaltung behoben
 *
 * 10.11.02 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.16o'
 *	Beim Bearbeiten der Gruppen wurde ggf. der Name zu "" gesetzt.
 *
 * 26.11.02 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.18c'
 *	Einfgen des YGA beschleunigt
 *
 * 27.11.02 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.19a'
 *	Wurden durch In2Cat Gruppen abbestellt markierte Cat sie 
 *	flschlicherweise als nicht bestellbar
 *
 * 14.2.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.20zf'
 *	das WAI File wurde flschlicherweise im TMP-Ordner statt 
 *	im DATABASE-Ordner gesucht. Und das seit ber 2 Jahren!
 *
 * 12.4.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.26c'
 *	Es waren noch einige Stellen brig, bei denen Gruppennummern in 
 *	_WORD Variablen gespeichert wurden, bei ber 32k Gruppen knallte es.
 *
 * 5.9.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.30g'
 *	Bei der Anzeige 'nur Pseudogruppen' war ein Fehler
 *
 * 1.5.04 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.36d'
 *	Nach dem Lschen einer Gruppe mu buildCatNr2gNr neu
 *	aufgerufen werden.
 *	
 */

#include <groupsel.h>
#include <strconv.h>
#include <mtappl.h>
#include <mtpaths.h>
#include <infofile.h>
#include <mte.h>
#include <datasys.h>
#include <data.h>
#include <catfiles.h>
#include <confvars.h>
#include <varnamel.h>
#include <catlog.h>
#include <protokol.h>
#include <ro_mem.h>
#include <routinec.h>
#include <routines.h>
#include <file_io.h>
#include <s_endian.h>
#include <alias.h>
#include <showimpo.h>
#include <mtrsc.h>

#define CR 0x0D
#define LF 0x0A
#define ITU_NAME "ITU"


/* Magics fuer Header */
#define groupCatMagic      0x43415447L  /* "CATG"                          */
#define groupVersion       4            /* Versionsnummer der CATGROUP.DAT */
#define groupVersionMagic  0x0104       /* Versionsmagic                   */



typedef struct _groupHdr {
	data_offset name;						/* Offsets in Data hinein, der  */
	data_offset netname;					/* erste ist immer 0. Ansonsten heisst */
	data_offset chef;						/* ein offset von 0, dass das Feld nicht */
	data_offset kurztext;					/* vorhanden ist */
	data_offset sysop;
	data_offset followup;
	data_offset herkunft;
	data_offset sprache;
	data_offset alias;
	CAT_GROUP_NUMBER readNumber;
	CAT_GROUP_NUMBER catNumber;
	_UWORD /* rwState */  readState;		/* F-Zeile */
	_UWORD /* rwState */  writeState;		/* F-Zeile */
	_UWORD /* uState */   userState;		/* F-Zeile */
	_UWORD /* netState */ isNet;			/* F-Zeile */
	_WORD /* _BOOL */   hasProg;			/* F-Zeile */
	_WORD /* _BOOL */   withDollar;			/* F-Zeile */
	_WORD /* _BOOL */   selected;			/* Fuer Darstellung in Liste, wird beim Einlesen auf FALSE gesetzt */
	_UWORD /* orderState */ ordered;		/* Fuer An-/Abbestellen von Gruppen */
	_WORD orderDays;						/* Madness-Erweiterung: Gruppe soviel Tage zurueck bestellen */
	data_offset dataLength;					/* Die Laenge des folgenden Datenblocks */
} PACKED groupHdr;

typedef group *groupArrayPtr;


LOCAL _BOOL listModified;
LOCAL groupArrayPtr groupArray;
LOCAL GROUPNR maxGroups;
LOCAL GROUPNR numGroups;
LOCAL GROUPNR *CatNr2gNr=NULL;
CAT_GROUP_NUMBER AnzCatNr=0;
CAT_GROUP_NUMBER MaxCatNr=0;
CAT_GROUP_NUMBER SizeC2G=0;

/*DYJ 5.35c Start*/
/*DYJ  Ende; alt:	
#define LS_GROUP_NAME    0x0001
#define LS_NET_LINE      0x0002
#define LS_KURZTEXT      0x0004
#define LS_CHEF_LINE     0x0008
#define LS_FLAG_LINE     0x0010
#define LS_SYSOP_LINE    0x0020
#define LS_FOLLOWUP_LINE 0x0040
#define LS_HERKUNFT      0x0080
#define LS_SPRACHE       0x0100
#define LS_ALIAS         0x0200
#define LS_CAT_NUMBER    0x0400
#define LS_READ_NUMBER   0x0800
#define LS_ORDERED       0x1000
#define LS_ORDER_DAYS    0x2000
#define LS_FLAG_LINE_Y   0x4000
#define LS_ALL           0x3fff
typedef _UWORD lineSet;
*/

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Alloziert ein neues Gruppenarray und kopiert das alte hinein.
 * FALSE: Nicht genug Speicher!
 */

/*DYJ 5.18c */
LOCAL _BOOL MakeNewGroupArray(_BOOL force, GROUPNR new)
{
	GROUPNR newMax;
	group *newArray;

	if (maxGroups == 0 || (numGroups + 1) >= maxGroups || force)
	{
		if (force)
			newMax = new + 50;
		else if(maxGroups<1000)
			newMax = maxGroups + 250;
		else
			newMax = maxGroups + maxGroups/4;
		newArray = MALLOC(newMax * sizeof(group), "MakeNewGroupArray");
		if (newArray == NULL)
			return FALSE;
		MemSetZero(newArray, newMax * sizeof(group));
		if (groupArray != NULL)
		{
			MemCpy(newArray, groupArray, maxGroups * sizeof(group));
			SFREE(groupArray);
		}
		maxGroups = newMax;
		groupArray = newArray;
	}
	if(!CatNr2gNr)
	{
		CAT_GROUP_NUMBER i;
		SizeC2G=200;
		CatNr2gNr=MALLOC(SizeC2G * sizeof(GROUPNR), "MakeNewGroupArray:CatNr2gNr");
		for(i=0;i<SizeC2G;i++)
			CatNr2gNr[i]=(GROUPNR)dataSys_notSaved;
		AnzCatNr=0;
		MaxCatNr=0;
	}

	return TRUE;
}
/*** ------------------------------------------------------------------------ ***/
/*						*/
/*								*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	25.11.02 von Dimitri Junker	DYJ 5.18c										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.26c */
LOCAL _BOOL MakeNewGroupArrayP(groupArrayPtr **grPtr2list,_BOOL force, GROUPNR new)
{
	GROUPNR newMax,i;
	group *newArray;
	groupArrayPtr *newgrPtrlist;
	
	if (maxGroups == 0 || (numGroups + 1) >= maxGroups || force)
	{
		if (force)
			newMax = new + 200;
		else
			newMax = maxGroups + 1000;
		newArray = MALLOC(newMax * sizeof(group), "MakeNewGroupArrayP");
		if (newArray == NULL)
			return FALSE;
		MemSetZero(newArray, newMax * sizeof(group));
		if (groupArray != NULL)
		{
			MemCpy(newArray, groupArray, maxGroups * sizeof(group));
			SFREE(groupArray);
		}
		newgrPtrlist=MALLOC(newMax * sizeof(groupArrayPtr *), "MakeNewGroupArrayP");
		if (newgrPtrlist == NULL)
			return FALSE;
		for(i=0;i<maxGroups;i++)
			newgrPtrlist[i]=(groupArrayPtr)((_LONG)(*grPtr2list)[i]-(_LONG)groupArray + (_LONG)newArray);
		for(i=maxGroups;i<newMax;i++)
			newgrPtrlist[i]=&(newArray[i]);
		SFREE(grPtr2list);
		*grPtr2list=newgrPtrlist;
		maxGroups = newMax;
		groupArray = newArray;
	}
	if(!CatNr2gNr)
	{
		CAT_GROUP_NUMBER i;
		SizeC2G=200;
		CatNr2gNr=MALLOC(SizeC2G * sizeof(GROUPNR), "MakeNewGroupArrayP:CatNr2gNr");
		for(i=0;i<SizeC2G;i++)
			CatNr2gNr[i]=(GROUPNR)dataSys_notSaved;
		AnzCatNr=0;
		MaxCatNr=0;
	}

	return TRUE;
}
/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.26c */
LOCAL _VOID FreeGroupArray(_VOID)
{
	GROUPNR i;

	if (groupArray != NULL)
	{
		for (i = 0; i < numGroups; i++)
			if (groupArray[i].data != NULL)
				FREE(groupArray[i].data, groupArray[i].dataLength);
		FREE(groupArray, maxGroups * sizeof(group));
		groupArray = NULL;
	}
	numGroups = 0;
	maxGroups = 0;
	AnzCatNr=0;
	MaxCatNr=0;
	SizeC2G=0;
	if(CatNr2gNr)
		SFREE(CatNr2gNr);
	CatNr2gNr=NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ParseFlags(group *entry, CONST _UBYTE *flags,lineSet type)
{
	rwState rwEntry;
	if(type&LS_FLAG_LINE_Y)
	{
		if(*flags=='n' ||*flags=='N')
			entry->readState = entry->writeState = rwImpossible;
		else
			entry->readState = entry->writeState = rwPossible;
		return;
	}

	while (*flags != '\0')
	{
		switch (*flags)
		{
		case 'L':
		case 'S':
			switch (flags[1])
			{
				case '+': rwEntry = rwOn; break;
				case 'B': rwEntry = rwPossible; break;
				case 'C': rwEntry = rwAskChef; break;
				case '-': rwEntry = rwImpossible; break;
				case 'P': rwEntry = rwDefault; break;
				case 'S': rwEntry = rwAskSysop; break;
				default:  rwEntry = rwNA; break;
			}
			if (*flags == 'L')
				entry->readState = rwEntry;
			else
				entry->writeState = rwEntry;
			break;

		case 'V':
			switch (flags[1])
			{
				case '+': entry->isNet = nNet; break;
				case '-': entry->isNet = nLokal; break;
				case '=': entry->isNet = nNetLokal; break;
				case '!': entry->isNet = nNetForbid; break;
				case 'P': entry->isNet = nPrivate; break;
				case 'O': entry->isNet = nOeffentl; break;
				default: entry->isNet = nNA; break;
			}
			break;

		case 'G':
			switch (flags[1])
			{
				case '*': entry->userState = uIsChef; break;
				case '+': entry->userState = uIsUser; break;
				case '-': entry->userState = uNoUser; break;
				default: entry->userState = uNA; break;
			}
			break;

		case 'P':
			entry->hasProg = flags[1] == '+';
			break;

		case '$':
			entry->withDollar = flags[1] == '+';
			break;
		}
		if (*++flags != '\0')
			++flags;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Binaere Suche im Array */
/*DYJ 5.16o */	/*DYJ 5.26c */
LOCAL _BOOL findEntry(CONST _UBYTE *name, GROUPNR *idx)
{
	GROUPNR left, right, mid;
	_BOOL found;
	_WORD res;
	if(!numGroups || !name || !*name)
	{
		*idx=(GROUPNR)dataSys_empty;
		return FALSE;
	}

	found = FALSE;
	left = 0;
	right = numGroups - 1;
	while (left <= right)
	{
		mid = (left + right) / 2;
		res = strSortCmp(name, groupArray[mid].name);
		if (res == 0)
		{
			*idx = mid;
			found = TRUE;
			break;
		} else if (res > 0)
		{
			left = mid + 1;
		} else
		{
			right = mid - 1;
		}
	}
	if (!found)
	{
		/* den kleineren Wert zuweisen, dahinter wird dann ggf. eingefuegt */
		*idx = right;
	}
	return found;
}
/*** ------------------------------------------------------------------------ ***/
/*	_BOOL findEntryP(groupArrayPtr *grPtrlist,CONST _UBYTE *name, GROUPNR *idx)	*/
/*	Binaere Suche im Array 														*/
/*	25.11.02 von Dimitri Junker	DYJ 5.18c										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.26c */
LOCAL _BOOL findEntryP(groupArrayPtr *grPtrlist,CONST _UBYTE *name, GROUPNR *idx)
{
	GROUPNR left, right, mid;
	_BOOL found;
	_WORD res;
	if(!numGroups || !name || !*name)
	{
		*idx=(GROUPNR)dataSys_empty;
		return FALSE;
	}

	found = FALSE;
	left = 0;
	right = numGroups - 1;
	while (left <= right)
	{
		mid = (left + right) / 2;
		res = strSortCmp(name, grPtrlist[mid]->name);
		if (res == 0)
		{
			*idx = mid;
			found = TRUE;
			break;
		} else if (res > 0)
		{
			left = mid + 1;
		} else
		{
			right = mid - 1;
		}
	}
	if (!found)
	{
		/* den kleineren Wert zuweisen, dahinter wird dann ggf. eingefuegt */
		*idx = right;
	}
	return found;
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _BOOL findNetAliasEntry(CONST _UBYTE *name, GROUPNR *idx)
{
	GROUPNR gNr;
	CAT_GROUP_NUMBER cNr;

	for (cNr = 1; cNr <= MaxCatNr; cNr++)
	{
		gNr=CatNr2gNr[cNr];
		if (gNr!=dataSys_notSaved &&
			(groupArray[gNr].netname != NULL &&
			strSortCmp(name, groupArray[gNr].netname) == 0) ||
		    (groupArray[gNr].alias != NULL &&
			strSortCmp(name, groupArray[gNr].alias) == 0))
		{
			*idx = gNr;
			return TRUE;
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/


/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL CAT_GROUP_NUMBER GroupSelect_CountCatGroupsMax(CAT_GROUP_NUMBER *maxNr)				*/
/*	Ersetzt countCatEntries. Zustzlich wird die hchste catNumber zurck gegeben*/
/*	Parameter:	maxNr:	Pointer fr Rckgabe									*/
/*	Ergebnis:	Anzahl Gruppen in Datenbank, und hchste Nr per maxNr			*/		
/*	31.7.01 von Dimitri Junker	DYJ 4.24l										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL CAT_GROUP_NUMBER GroupSelect_CountCatGroupsMax(CAT_GROUP_NUMBER *maxNr)
{
	*maxNr=MaxCatNr;
	return AnzCatNr;
}


/*** ---------------------------------------------------------------------- ***/

		/*DYJ 5.21h */
LOCAL _BOOL findNumberEntry(CAT_GROUP_NUMBER number, GROUPNR *idx)
{
	if(number>MaxCatNr)
		*idx=dataSys_notSaved;
	else
		*idx=CatNr2gNr[number];
	return *idx!=dataSys_notSaved;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL CAT_GROUP_NUMBER GroupSelect_CntCatNum(CAT_GROUP_NUMBER number)	*/
/*	Zhlt wieviele Gruppen die angegebene Catnummer haben, sollte 0 oder 1 sein	*/
/*	Sonst ist ein Fehler in der Datenbank										*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	29.7.01 von Dimitri Junker	DYJ 4.24l										*/
/*** ------------------------------------------------------------------------ ***/

GLOBAL CAT_GROUP_NUMBER GroupSelect_CntCatNum(CAT_GROUP_NUMBER number)
{
	CAT_GROUP_NUMBER anz=0;
	GROUPNR i;

	if(CatNr2gNr[number]==dataSys_notSaved)
		return 0;
	for (i = 0; i < numGroups; i++)
	{
		if ((CAT_GROUP_NUMBER)groupArray[i].catNumber == number)
			anz++;
	}
	return anz;

}

/*** ---------------------------------------------------------------------- ***/


LOCAL _BOOL findReadNumberEntry(CAT_GROUP_NUMBER number, GROUPNR *idx)
{
	GROUPNR i;

	for (i = 0; i < numGroups; i++)
	{
		if (groupArray[i].readNumber == number)
		{
			*idx = i;
			return TRUE;
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Fuellt den Eintrag gr mit den Daten
 */

LOCAL _BOOL MakeEntry(
	group *gr,
	CONST _UBYTE *gName,
	CONST _UBYTE *nName,
	CONST _UBYTE *uLine,
	CONST _UBYTE *cLine,
	CONST _UBYTE *fLine,
	CONST _UBYTE *sLine,
	CONST _UBYTE *rLine,
	CONST _UBYTE *hLine,
	CONST _UBYTE *pLine,
	CONST _UBYTE *aLine,
	CAT_GROUP_NUMBER readNum,
	CAT_GROUP_NUMBER catNum,
	orderState oState,
	_WORD days,
	_BOOL defaultFlags,
	lineSet avail)
{
	_WORD gLen, nLen, uLen, sLen, rLen, cLen, hLen, pLen, aLen;
	_WORD addLen;
	_UBYTE *target;

	/* Laengen herausfinden */
	gLen = (_WORD)strLen(gName) + 1;
	nLen = nName && *nName ? (_WORD)strLen(nName) + 1 : 0;
	uLen = uLine && *uLine ? (_WORD)strLen(uLine) + 1 : 0;
	cLen = cLine && *cLine ? (_WORD)strLen(cLine) + 1 : 0;
	sLen = sLine && *sLine ? (_WORD)strLen(sLine) + 1 : 0;
	rLen = rLine && *rLine ? (_WORD)strLen(rLine) + 1 : 0;
	hLen = hLine && *hLine ? (_WORD)strLen(hLine) + 1 : 0;
	pLen = pLine && *pLine ? (_WORD)strLen(pLine) + 1 : 0;
	aLen = aLine && *aLine ? (_WORD)strLen(aLine) + 1 : 0;
	addLen = gLen + nLen + uLen + cLen + sLen + rLen + hLen + pLen + aLen;

	/* Jetzt Speicher allozieren */
	if (gr->data != NULL)
		FREE(gr->data, gr->dataLength);
	gr->dataLength = 0;
	gr->data = MALLOC(addLen, "GroupSel_MakeEntry");
	if (gr->data == NULL)
		return FALSE;
	gr->dataLength = addLen;

	/* Speicher vorhanden, jetzt Eintraege setzen */
	target = gr->data;

	/* Gruppenname, immer vorhanden */
	gr->name = target;
	while ((*target++ = *gName++) != '\0');
	gr->display_name = gr->name;
	
	/* Netzname */
	if (nLen)
	{
		gr->netname = target;
		while ((*target++ = *nName++) != '\0');
	} else
	{
		gr->netname = NULL;
	}

	/* Gruppenchef */
	if (cLen)
	{
		gr->chef = target;
		while ((*target++ = *cLine++) != '\0');
	} else
	{
		gr->chef = NULL;
	}

	/* Kurztext */
	if (uLen)
	{
		gr->kurztext = target;
		while ((*target++ = *uLine++) != '\0');
	} else
	{
		gr->kurztext = NULL;
	}

	/* Sysop */
	if (sLen)
	{
		gr->sysop = target;
		while ((*target++ = *sLine++) != '\0');
	} else
	{
		gr->sysop = NULL;
	}

	/* Follow-Up */
	if (rLen)
	{
		gr->followup = target;
		while ((*target++ = *rLine++) != '\0');
	} else
	{
		gr->followup = NULL;
	}

	/* Herkunft */
	if (hLen)
	{
		gr->herkunft = target;
		while ((*target++ = *hLine++) != '\0');
	} else
	{
		gr->herkunft = NULL;
	}

	/* Sprache */
	if (pLen)
	{
		gr->sprache = target;
		while ((*target++ = *pLine++) != '\0');
	} else
	{
		gr->sprache = NULL;
	}

	/* Alias */
	if (aLen)
	{
		gr->alias = target;
		while ((*target++ = *aLine++) != '\0');
	} else
	{
		gr->alias = NULL;
	}

	if (avail & LS_READ_NUMBER)
		gr->readNumber = readNum;

	if (avail & LS_CAT_NUMBER)
		gr->catNumber = catNum;

	if (avail & LS_ORDERED)
		gr->ordered = oState;

	if (avail & LS_ORDER_DAYS)
		gr->orderDays = days;

	/* Okay, Texte sind gesetzt, jetzt Flags einsetzen, falls vorhanden */
	/* Default-Einstellung herstellen fuer Flags */
	if (defaultFlags)
	{
		gr->readState = rwNA;
		gr->writeState = rwNA;
		gr->userState = uNA;
		gr->isNet     = nNA;
		gr->hasProg   = FALSE;
		gr->withDollar = FALSE;
		gr->selected  = FALSE;
	}

	if (avail & (LS_FLAG_LINE|LS_FLAG_LINE_Y))
		ParseFlags(gr, fLine,avail);

	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID empty_group(group *gr)
{
	MemSetZeroStruct(*gr);
	gr->readNumber =(CAT_GROUP_NUMBER)dataSys_notSaved;
	gr->catNumber =(CAT_GROUP_NUMBER)dataSys_notSaved;
	gr->readState = rwNA;
	gr->writeState = rwNA;
	gr->userState = uNA;
	gr->isNet = nNA;
	gr->hasProg = FALSE;
	gr->withDollar = FALSE;
	gr->selected = FALSE;
	gr->ordered = oNA;
	gr->orderDays = 0;
	gr->isUpdated = TRUE;
}
/*** ------------------------------------------------------------------------ ***/
/*	_BOOL setCatNr2gNr(GROUPNR gNr,CAT_GROUP_NUMBER cNr)						*/
/*	Setzte einzelne Werte in der CatNr2gNr-Tabelle, die dazu dient schnell von 	*/
/*	CatNumber zur Gruppennummer zu kommen.										*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	19.12.01 von Dimitri Junker	DYJ 4.30zs										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _BOOL setCatNr2gNr(GROUPNR gNr,CAT_GROUP_NUMBER cNr)
{
	GROUPNR *tmp;
	CAT_GROUP_NUMBER i;
	
	if(cNr==(CAT_GROUP_NUMBER)dataSys_notSaved || cNr==(CAT_GROUP_NUMBER)dataSys_empty)
		return TRUE;
	if(cNr>=SizeC2G)
	{
		tmp=MALLOC((cNr+100)*sizeof(GROUPNR),"setCatNr2gNr");
		if(!tmp)
			return FALSE;
		for(i=0;i<SizeC2G;i++)
			tmp[i]=CatNr2gNr[i];
		for(i=SizeC2G;i<(cNr+100);i++)
			tmp[i]=(GROUPNR)dataSys_notSaved;
		SizeC2G=cNr+100;
		SFREE(CatNr2gNr);
		CatNr2gNr=tmp;
	}
	CatNr2gNr[cNr]=gNr;
	if(gNr==dataSys_notSaved)
	{
		AnzCatNr--;
		while(MaxCatNr>0 && CatNr2gNr[MaxCatNr]==dataSys_notSaved)
			MaxCatNr--;
	}
	else
	{
		AnzCatNr++;
		if(MaxCatNr<cNr)
			MaxCatNr=cNr;
	}
	return TRUE;
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID moveCatNr2gNr(GROUPNR start)											*/
/*	macht Platz fr eine neue Gruppe in CatNr2gNr								*/
/*	Parameter:	start: dort soll Platz gemacht werden							*/
/*	Ergebnis:	keines															*/		
/*	20.12.01 von Dimitri Junker	DYJ 4.30zs										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _VOID moveCatNr2gNr(GROUPNR start)
{
	CAT_GROUP_NUMBER i;
	for(i=1;i<=MaxCatNr;i++)
	{
		if(CatNr2gNr[i]>=start && CatNr2gNr[i]!=(GROUPNR)dataSys_notSaved)
			CatNr2gNr[i]++;
	}
}

/*** ---------------------------------------------------------------------- ***/
LOCAL _BOOL UpdateEntry(
	_UBYTE *gName,
	_UBYTE *nName,
	_UBYTE *uLine,
	_UBYTE *cLine,
	_UBYTE *fLine,
	_UBYTE *sLine,
	_UBYTE *rLine,
	_UBYTE *hLine,
	_UBYTE *pLine,
	_UBYTE *aLine,
	CAT_GROUP_NUMBER readNum,
	CAT_GROUP_NUMBER catNum,
	orderState oState,
	_WORD days,
	lineSet avail,
	_BOOL keepOld,
	_BOOL preferLocal)
{
	_BOOL found;
	group *gr;
	GROUPNR grIdx;
	group newGroup;

	/* So, und jetzt suchen wir nur nach dem Namen */
	found = findEntry(gName, &grIdx);
	if (found)
	{
		gr = &groupArray[grIdx];
		/* Jetzt das raussuchen, was ggf. neu gesetzt werden muss
		 * Dabei umgedrehte Logik: Wenn etwas nicht in den neu zu setzenden
		 * Sachen ist, aber in den alten, dann wird der alte Eintrag einfach
		 * fuer den neuen uebernommen, wenn keepOld TRUE ist
		 * (bei GRUPPEN.INF oder IGK der Fall)
		 */
		setCatNr2gNr(dataSys_notSaved,gr->catNumber);
		catNum=gr->catNumber;
		if (preferLocal)
		{
			/* Lokal vorhandene Gruppen bevorzugen */
			if (gr->isUpdated)
			{
				if ((avail & LS_FLAG_LINE) ||
				   ((avail & LS_FLAG_LINE_Y)&& !keepOld))
				{
					newGroup.isNet = nNA;
					ParseFlags(&newGroup, fLine,avail);
					if (newGroup.isNet == nNetLokal || newGroup.isNet == nNetForbid)
					{
						/* Neuer Eintrag ist lokal nicht vorhandene Netzgruppe,
						 * wir behalten einfach den alten
						 */
						return TRUE;
					}
				} else if (gr->isNet != nNetLokal && gr->isNet != nNetForbid && gr->isNet != nNA)
				{
					/* Der alte Eintrag bezeichnet schon eine vorhandene Gruppe */
					return TRUE;
				}
			}
		}
		gr->isUpdated = TRUE;
		if (keepOld)
		{
			if (!(avail & LS_NET_LINE) && gr->netname != NULL)
			{
				/* Netname zuweisen */
				strCpy(nName, gr->netname);
			}
			if (!(avail & LS_KURZTEXT) && gr->kurztext != NULL)
			{
				/* Kurztext zuweisen */
				strCpy(uLine, gr->kurztext);
			}
			if (!(avail & LS_CHEF_LINE) && gr->chef != NULL)
			{
				/* Chef zuweisen */
				strCpy(cLine, gr->chef);
			}
			if (!(avail & LS_SYSOP_LINE) && gr->sysop != NULL)
			{
				/* Sysopinfo zuweisen */
				strCpy(sLine, gr->sysop);
			}
		}
		if (!(avail & LS_FOLLOWUP_LINE) && gr->followup != NULL)
		{
			/* Followup zuweisen */
			strCpy(rLine, gr->followup);
		}
		if (!(avail & LS_HERKUNFT) && gr->herkunft != NULL)
		{
			/* Herkunft zuweisen */
			strCpy(hLine, gr->herkunft);
		}
		if (!(avail & LS_SPRACHE) && gr->sprache != NULL)
		{
			/* Sprache zuweisen */
			strCpy(pLine, gr->sprache);
		}
		if (!(avail & LS_ALIAS) && gr->alias != NULL)
		{
			/* Alias zuweisen */
			strCpy(aLine, gr->alias);
		}

		if (gr->data != NULL)
		{
			FREE(gr->data, gr->dataLength);
			gr->data = NULL;
		}
		gr->dataLength = 0;
	} else
	{
		/* Eintrag loeschen */
		gr = &newGroup;
		empty_group(gr);
	}
	if (!MakeEntry(gr, gName, nName, uLine, cLine, fLine, sLine, rLine, hLine, pLine, aLine, readNum, catNum, oState, days, TRUE, avail))
		return FALSE;

	if (found)
	{
		/* Ist nichts mehr zu tun, da direkt die Adresse des Eintrags uebergeben wurde
		 */
	} else
	{
		/* neuen Eintrag anlegen */
		if (!MakeNewGroupArray(FALSE, 0))
			return FALSE;
		/* hinter dem letzten einsortiern */
		++grIdx;
		/* Alle nachfolgenden verschieben */
		if (numGroups > 0)
		{
			memmove(&groupArray[grIdx+1],&groupArray[grIdx],sizeof(group)*(numGroups-grIdx));
			moveCatNr2gNr(grIdx);
		}
		else
			grIdx=0;
		/* Eintrag uebernehmen */
		groupArray[grIdx] = *gr;
		/* Gruppenanzahl aendern */
		++numGroups;
	}
	setCatNr2gNr(grIdx,catNum);
	/* Eintraege jetzt wieder loeschen */
	*gName = '\0';
	*nName = '\0';
	*uLine = '\0';
	*cLine = '\0';
	*fLine = '\0';
	*sLine = '\0';
	*rLine = '\0';
	*hLine = '\0';
	*pLine = '\0';
	*aLine = '\0';
	listModified = TRUE;
	return TRUE;
}
/*** ------------------------------------------------------------------------ ***/
/*						*/
/*								*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	25.11.02 von Dimitri Junker	DYJ 5.18c										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _BOOL UpdateEntryP(
	groupArrayPtr **grPtr2list,
	_UBYTE *gName,
	_UBYTE *nName,
	_UBYTE *uLine,
	_UBYTE *cLine,
	_UBYTE *fLine,
	_UBYTE *sLine,
	_UBYTE *rLine,
	_UBYTE *hLine,
	_UBYTE *pLine,
	_UBYTE *aLine,
	CAT_GROUP_NUMBER readNum,
	CAT_GROUP_NUMBER catNum,
	orderState oState,
	_WORD days,
	lineSet avail,
	_BOOL keepOld,
	_BOOL preferLocal)
{
	_BOOL found;
	group *gr;
	GROUPNR grIdx;
	group newGroup;

	/* So, und jetzt suchen wir nur nach dem Namen */
	found = findEntryP(*grPtr2list,gName, &grIdx);
	if (found)
	{
		gr = (*grPtr2list)[grIdx];
		/* Jetzt das raussuchen, was ggf. neu gesetzt werden muss
		 * Dabei umgedrehte Logik: Wenn etwas nicht in den neu zu setzenden
		 * Sachen ist, aber in den alten, dann wird der alte Eintrag einfach
		 * fuer den neuen uebernommen, wenn keepOld TRUE ist
		 * (bei GRUPPEN.INF oder IGK der Fall)
		 */
		setCatNr2gNr(dataSys_notSaved,gr->catNumber);
		catNum=gr->catNumber;	
		if (preferLocal)
		{
			/* Lokal vorhandene Gruppen bevorzugen */
			if (gr->isUpdated)
			{
				if ((avail & LS_FLAG_LINE) ||
				   ((avail & LS_FLAG_LINE_Y)&& !keepOld))
				{
					newGroup.isNet = nNA;
					ParseFlags(&newGroup, fLine,avail);
					if (newGroup.isNet == nNetLokal || newGroup.isNet == nNetForbid)
					{
						/* Neuer Eintrag ist lokal nicht vorhandene Netzgruppe,
						 * wir behalten einfach den alten
						 */
						return TRUE;
					}
				} else if (gr->isNet != nNetLokal && gr->isNet != nNetForbid && gr->isNet != nNA)
				{
					/* Der alte Eintrag bezeichnet schon eine vorhandene Gruppe */
					return TRUE;
				}
			}
		}
		gr->isUpdated = TRUE;
		if (keepOld)
		{
			if (!(avail & LS_NET_LINE) && gr->netname != NULL)
			{
				/* Netname zuweisen */
				strCpy(nName, gr->netname);
			}
			if (!(avail & LS_KURZTEXT) && gr->kurztext != NULL)
			{
				/* Kurztext zuweisen */
				strCpy(uLine, gr->kurztext);
			}
			if (!(avail & LS_CHEF_LINE) && gr->chef != NULL)
			{
				/* Chef zuweisen */
				strCpy(cLine, gr->chef);
			}
			if (!(avail & LS_SYSOP_LINE) && gr->sysop != NULL)
			{
				/* Sysopinfo zuweisen */
				strCpy(sLine, gr->sysop);
			}
		}
		if (!(avail & LS_FOLLOWUP_LINE) && gr->followup != NULL)
		{
			/* Followup zuweisen */
			strCpy(rLine, gr->followup);
		}
		if (!(avail & LS_HERKUNFT) && gr->herkunft != NULL)
		{
			/* Herkunft zuweisen */
			strCpy(hLine, gr->herkunft);
		}
		if (!(avail & LS_SPRACHE) && gr->sprache != NULL)
		{
			/* Sprache zuweisen */
			strCpy(pLine, gr->sprache);
		}
		if (!(avail & LS_ALIAS) && gr->alias != NULL)
		{
			/* Alias zuweisen */
			strCpy(aLine, gr->alias);
		}

		if (gr->data != NULL)
		{
			FREE(gr->data, gr->dataLength);
			gr->data = NULL;
		}
		gr->dataLength = 0;
	} else
	{
		/* Eintrag loeschen */
		gr = &newGroup;
		empty_group(gr);
	}
	if (!MakeEntry(gr, gName, nName, uLine, cLine, fLine, sLine, rLine, hLine, pLine, aLine, readNum, catNum, oState, days, TRUE, avail))
		return FALSE;

	if (found)
	{
		/* Ist nichts mehr zu tun, da direkt die Adresse des Eintrags uebergeben wurde
		 */
	} else
	{
		groupArrayPtr nxt;
		/* neuen Eintrag anlegen */
		if (!MakeNewGroupArrayP(grPtr2list,FALSE, 0))
			return FALSE;
		/* hinter dem letzten einsortiern */
		++grIdx;
		nxt=(*grPtr2list)[numGroups];
		/* Alle nachfolgenden verschieben */
		if (numGroups > 0)
		{
			memmove(&(*grPtr2list)[grIdx+1],&(*grPtr2list)[grIdx],sizeof(groupArrayPtr *)*(numGroups-grIdx));
			moveCatNr2gNr(grIdx);
		}
		else
			grIdx=0;
		/* Eintrag uebernehmen */
		(*grPtr2list)[grIdx] =nxt;
		*nxt= *gr;
		/* Gruppenanzahl aendern */
		++numGroups;
	}
	setCatNr2gNr(grIdx,catNum);
	/* Eintraege jetzt wieder loeschen */
	*gName = '\0';
	*nName = '\0';
	*uLine = '\0';
	*cLine = '\0';
	*fLine = '\0';
	*sLine = '\0';
	*rLine = '\0';
	*hLine = '\0';
	*pLine = '\0';
	*aLine = '\0';
	listModified = TRUE;
	return TRUE;
}
/*** ---------------------------------------------------------------------- ***/

typedef enum { sByName, sByReadNum, sByNum, sByOrder, sByNet, sByLang } lSortType;

LOCAL _BOOL readNumComp(CONST _VOID *a1, CONST _VOID *a2, _VOID *private)
{
	CONST group *p1 = *((CONST group * CONST *)a1);
	CONST group *p2 = *((CONST group * CONST *)a2);

	UNUSED(private);
	if (p1->readNumber == p2->readNumber)
		return strSortCmp(p1->name, p2->name) < 0;
	return p1->readNumber < p2->readNumber;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL orderComp(CONST _VOID *a1, CONST _VOID *a2, _VOID *private)
{
	CONST group *p1 = *((CONST group * CONST *)a1);
	CONST group *p2 = *((CONST group * CONST *)a2);
	_BOOL on1, on2;

	UNUSED(private);
	on1 = p1->readState == rwOn || p1->readState == rwDefault;
	on2 = p2->readState == rwOn || p2->readState == rwDefault;
	if (on1 == on2)
		return strSortCmp(p1->name, p2->name) < 0;
	return on1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL nameComp(CONST _VOID *a1, CONST _VOID *a2, _VOID *private)
{
	CONST group *p1 = *((CONST group * CONST *)a1);
	CONST group *p2 = *((CONST group * CONST *)a2);

	UNUSED(private);
	return strSortCmp(p1->display_name, p2->display_name) < 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ExtraktString(CONST _UBYTE *source, _UBYTE *dest)
{
	CONST _UBYTE *p;

	*dest = '\0';
	p = strChr(source, ':');
	if (p != NULL)
		strMcpy(dest, p - source + 1, source);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL netComp(CONST _VOID *a1, CONST _VOID *a2, _VOID *private)
{
	CONST group *p1 = *((CONST group * CONST *)a1);
	CONST group *p2 = *((CONST group * CONST *)a2);
	group_line str1;
	group_line str2;
	_WORD compRes;

	UNUSED(private);
	if (p1->herkunft != NULL && p2->herkunft != NULL)
	{
		ExtraktString(p1->herkunft, str1);
		ExtraktString(p2->herkunft, str2);
		compRes = strSortCmp(str1, str2);
		if (compRes == 0)
			compRes = strSortCmp(p1->display_name, p2->display_name);
		return compRes < 0;
	}
	if (p1->herkunft != NULL)
		return TRUE;
	if (p2->herkunft != NULL)
		return FALSE;
	return strSortCmp(p1->display_name, p2->display_name) < 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL langComp(CONST _VOID *a1, CONST _VOID *a2, _VOID *private)
{
	CONST group *p1 = *((CONST group * CONST *)a1);
	CONST group *p2 = *((CONST group * CONST *)a2);
	group_line str1;
	group_line str2;
	_WORD compRes;

	UNUSED(private);
	if (p1->sprache != NULL && p2->sprache != NULL)
	{
		ExtraktString(p1->sprache, str1);
		ExtraktString(p2->sprache, str2);
		compRes = strSortCmp(str1, str2);
		if (compRes == 0)
			compRes = strSortCmp(p1->display_name, p2->display_name);
		return compRes < 0;
	}
	if (p1->sprache != NULL)
		return TRUE;
	if (p2->sprache != NULL)
		return FALSE;
	return strSortCmp(p1->display_name, p2->display_name) < 0;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID GroupSelect_SortAnyList(List *l, compProc proc)
{
	_ULONG count, i;
	_VOID **sort;

	count = Lists_NoOfEntries(l);
	if (count > 1)
	{
		sort = MALLOC(count * sizeof(_VOID *), "GroupSelect_SortList");
		if (sort != NULL)
		{
			Lists_ResetList(l);
			for (i = 0; i < count; i++)
				sort[i] = Lists_NextEntry(l);
			QuickSort_sortIt(0, count-1, sort, proc, sizeof(_VOID *), QuickSort_noBreak, NULL);
/*DYJ 5.30c Start*/
			/* Liste wieder zurueckbernehmen */
			Lists_ResetList(l);
			for (i = 0; i < count; i++)
				Lists_SetNextEntry(l, sort[i]);
/*DYJ  Ende; alt:	
			/* Liste wieder zurueckuebernehmen */
			Lists_DeleteSimpleList(l, FUNK_NULL, FALSE);
			Lists_ResetList(l);
			for (i = 0; i < count; i++)
				Lists_AppendEntry(l, sort[i]);
*/
			FREE(sort, count * sizeof(_VOID *));
		}
	}
}

/*** ------------------------------------------------------------------------ ***/
/*						*/
/*								*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	19.12.01 von Dimitri Junker	DYJ 4.30zs										*/
/*** ------------------------------------------------------------------------ ***/

_BOOL buildCatNr2gNr(_VOID)
{
	GROUPNR i;
	_BOOL ret=TRUE;
	
	AnzCatNr=0;
	MaxCatNr=0;

	for(i=0;i<SizeC2G;i++)
		CatNr2gNr[i]=dataSys_notSaved;
	for(i=0;i<numGroups;i++)
	{
		if (groupArray[i].catNumber<(CAT_GROUP_NUMBER)dataSys_notSaved)
		{
			if((ret=setCatNr2gNr(i,groupArray[i].catNumber))==FALSE)
				break;
		}
	}
	return ret;
}

/*** ---------------------------------------------------------------------- ***/

/* sortiert eine Gruppenliste */
LOCAL _VOID SortList(List *l, lSortType sortType)
{
	compProc proc;
	switch (sortType)
	{
	case sByName:
		proc = nameComp;
		break;
	case sByReadNum:
		proc = readNumComp;
		break;
	case sByOrder:
		proc = orderComp;
		break;
	case sByNet:
		proc = netComp;
		break;
	case sByLang:
		proc = langComp;
		break;
	default:
		return;
	}
	GroupSelect_SortAnyList(l, proc);
	buildCatNr2gNr();

}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.26c */
LOCAL _VOID ParseITG(TEXTFILENO file)
{
	group_line gName;
	group_line nName;
	group_line uLine;
	group_line cLine;
	group_line fLine;
	group_line sLine;
	group_line rLine;
	group_line hLine;
	group_line aLine;
	group_line pLine;
	group_line scrap;
	_BOOL infoWritten;
	lineSet linesAvail;
	GROUPNR i, idx;

	/* Erstmal alle Eintraege loeschen, die keine CAT-Gruppe sind!
	 */
	idx = 0;
	while (idx < numGroups)
	{
		if (groupArray[idx].catNumber<(CAT_GROUP_NUMBER)dataSys_notSaved)
		{
			groupArray[idx].isUpdated = FALSE;
			++idx;
		} else
		{
			if (groupArray[idx].data != NULL)
				SFREE(groupArray[idx].data);
			for (i = idx+1; i < numGroups; i++)
			{
				/* Eintraege runterkopieren */
				groupArray[i-1] = groupArray[i];
			}
			--numGroups;
		}
	}
	buildCatNr2gNr();

	infoWritten = TRUE;
	linesAvail = LS_ORDERED | LS_ORDER_DAYS;
	*gName = '\0';
	*nName = '\0';
	*uLine = '\0';
	*cLine = '\0';
	*fLine = '\0';
	*sLine = '\0';
	*rLine = '\0';
	*hLine = '\0';
	*pLine = '\0';
	*aLine = '\0';
	while (!mtTextfiles_EndofText(file))
	{
		mtTextfiles_ReadLine(file, scrap, sizeof(scrap));
		mtTextfiles_ReadLn(file);
		switch (scrap[0])
		{
		case 'G':			/* lokaler Gruppenname */
			if (!infoWritten)
			{
				UpdateEntry(gName, nName, uLine, cLine, fLine, sLine, rLine, hLine, pLine, aLine, -1, -1, oNA, 0, linesAvail, FALSE, TRUE);
				linesAvail = LS_ORDERED | LS_ORDER_DAYS;
			}
			strCpy(gName, scrap + 1);
			linesAvail |= LS_GROUP_NAME;
			infoWritten = FALSE;
			break;
		case 'N':			/* Netzgruppenname */
			strCpy(nName, scrap + 1);
			linesAvail |= LS_NET_LINE;
			break;
		case 'U':			/* Kurzbeschreibung */
			strCpy(uLine, scrap + 1);
			linesAvail |= LS_KURZTEXT;
			break;
		case 'C':			/* Gruppenchef */
			strCpy(cLine, scrap + 1);
			linesAvail |= LS_CHEF_LINE;
			break;
		case 'F':			/* Flags */
			strCpy(fLine, scrap + 1);
			linesAvail |= LS_FLAG_LINE;
			break;
		case 'S':			/* Sysopinfo */
			strCpy(sLine, scrap + 1);
			linesAvail |= LS_SYSOP_LINE;
			break;
		case 'R':			/* Default fuer Follow-Up */
			strCpy(rLine, scrap + 1);
			linesAvail |= LS_FOLLOWUP_LINE;
			break;
		case 'H':
			strCpy(hLine, scrap + 1);
			linesAvail |= LS_HERKUNFT;
			break;
		case 'P':
			strCpy(pLine, scrap + 1);
			linesAvail |= LS_SPRACHE;
			break;
		case 'A':
			strCpy(aLine, scrap + 1);
			linesAvail |= LS_ALIAS;
			break;
		case 'E':			/* Aenderungsdatum netzweit */
			break;
		case 'L':			/* Aenderungsdatum lokal */
			break;
		}
	}
	if (!infoWritten)
		UpdateEntry(gName, nName, uLine, cLine, fLine, sLine, rLine, hLine, pLine, aLine, -1, -1, oNA, 0, linesAvail, FALSE, TRUE);
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID ParseYG(TEXTFILENO file,_BOOL keepOld,TEXTFILENO fileA)			*/
/*	Parst YGA oder YGN															*/
/*	Parameter:	file:	das zu parsende File									*/
/*				keepOld:Die alten Daten behalten (YGN)							*/
/*				fileA:	Bei YGN sollen die neuen Eintrge an die YGA angehngt	*/
/*				werden, dazu wird deren fileNumber bentigt						*/
/*	Ergebnis:	keines															*/		
/*	11.10.01 von Dimitri Junker	DYJ 4.30u										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.19j */	/*DYJ 5.18c */	/*DYJ 5.26c */
LOCAL _VOID ParseYG(TEXTFILENO file,_BOOL keepOld,TEXTFILENO fileA)
{
	group_line gName;
	group_line fLine;
	group_line scrap;
	lineSet linesAvail;
	GROUPNR idx,idxNeu=0;
	_LONG dummy1, dummy2;
	_LONG pos,posAlt=0;
	GROUPNR grIdx;
	groupArrayPtr *grPtrlist=NULL;

	/* Erstmal, falls gewnscht, alle Eintraege loeschen, die keine CAT-Gruppe sind!
	 */
	if(!keepOld)
	{
		for(idx =0;idx<numGroups;idx++)
		{
			if (groupArray[idx].catNumber<(CAT_GROUP_NUMBER)dataSys_notSaved)
			{
				groupArray[idx].isUpdated = FALSE;
				groupArray[idxNeu++]=groupArray[idx];
			}
			else
			{
				if (groupArray[idx].data != NULL)
					SFREE(groupArray[idx].data);
			}
		}
		numGroups=idxNeu;
		buildCatNr2gNr();
	}

	else
		grPtrlist= MALLOC(maxGroups * sizeof(groupArrayPtr *), "ParseYG: grPtrlist");
	linesAvail = LS_ORDERED | LS_ORDER_DAYS |LS_GROUP_NAME|LS_FLAG_LINE_Y;
	if(grPtrlist)
	{
		GROUPNR i;
		group *newArray;
		for(i=0;i<maxGroups;i++)
			grPtrlist[i]=&(groupArray[i]);
		while (!mtTextfiles_EndofText(file))
		{
			pos=mtTextfiles_Textpos(file)/1024;
			showImport_SetNew(pos*pos-posAlt*posAlt);
			posAlt=pos;
			mtTextfiles_ReadLine(file, scrap, sizeof(scrap));
			mtTextfiles_ReadLn(file);
			sscanf(scrap,"%s %ld %ld %s",gName,&dummy1,&dummy2,&fLine);
			if(fileA &&!findEntryP(grPtrlist,gName, &grIdx))
			{
				mtTextfiles_WriteLine(fileA, scrap);
				mtTextfiles_WriteLn(fileA);
			}
			UpdateEntryP(&grPtrlist,gName, "", "", "", fLine, "", "", "", "", "", -1, -1, oNA, 0, linesAvail, keepOld, FALSE);
		}
			
		newArray = MALLOC(maxGroups * sizeof(group), "ParseYG");
		if (newArray && groupArray )
		{
			for(i=0;i<maxGroups;i++)
				newArray[i]=*grPtrlist[i];
			SFREE(groupArray);
			SFREE(grPtrlist);
			groupArray = newArray;
		}
	}
	else
	{
		while (!mtTextfiles_EndofText(file))
		{
			pos=mtTextfiles_Textpos(file)/1024;
			showImport_SetNew(pos*pos-posAlt*posAlt);
			posAlt=pos;
			mtTextfiles_ReadLine(file, scrap, sizeof(scrap));
			mtTextfiles_ReadLn(file);
			sscanf(scrap,"%s %ld %ld %s",gName,&dummy1,&dummy2,&fLine);
			if(fileA &&!findEntry(gName, &grIdx))
			{
				mtTextfiles_WriteLine(fileA, scrap);
				mtTextfiles_WriteLn(fileA);
			}
			UpdateEntry(gName, "", "", "", fLine, "", "", "", "", "", -1, -1, oNA, 0, linesAvail, keepOld, FALSE);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isGroupName(CONST _UBYTE *s)
{
	_BOOL valid;

	valid =
		/* 1. Zeichen: Darf Space, + oder - sein. */
		(s[0] == '+' || s[0] == '-' || s[0] == ' ') &&
		/* 2. Zeichen: Darf '*' oder Space sein */
		(s[1] == ' ' || s[1] == '*') &&
		/* 3. Zeichen: Darf nicht Space sein! */
		s[2] != ' ';
	return valid;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ParseIGK(TEXTFILENO file)
{
	group_line gName;
	group_line uLine;
	group_line fLine;
	group_line scrap;
	lineSet linesAvail;
	_UBYTE *p, *s;

	linesAvail = LS_GROUP_NAME | LS_KURZTEXT | LS_FLAG_LINE | LS_ORDERED | LS_ORDER_DAYS;
	*gName = '\0';
	*uLine = '\0';
	*fLine = '\0';
	while (!mtTextfiles_EndofText(file))
	{
		mtTextfiles_ReadLine(file, scrap, sizeof(scrap));
		mtTextfiles_ReadLn(file);
		if (isGroupName(scrap))
		{
			if (scrap[0] == '+')
			{
				strCat(fLine, "L+S+");
			} else if (scrap[0] == '-')
			{
				strCat(fLine, "LBSB");
			}
			if (scrap[1] == '*')
			{
				strCat(fLine, "G*");
			} else if (scrap[0] == '-')
			{
				strCat(fLine, "G+");
			}
			/* Flagzeile ist zusammengebaut */
			if (scrap[2] == '?')
			{
				/* Fragezeichen entfernen (geheime Gruppe) */
				strCpy(scrap + 2, scrap + 3);
			}
			/* Jetzt Gruppennamen rauskopieren. Gruppenname geht bis zum ersten Space
			 * ab der 3. Spalte, aber mindestens 10 Zeichen weit!
			 */
			/* Gruppenname kopieren */
			s = scrap + 2;
			p = gName;
			while (*s != '\0' && *s != ' ')
				*p++ = *s++;
			*p = '\0';
			while (*s == ' ')
				s++;
			/* Rest ist Beschreibung */
			strCpy(uLine, s);
			/* Jetzt Spaces loeschen */
			strdelRightSpace(uLine);
			/* Und jetzt Eintrag updaten */
			*scrap = '\0';
			UpdateEntry(gName, scrap, uLine, scrap, fLine, scrap, scrap, scrap, scrap, scrap, -1, -1, oNA, 0, linesAvail, TRUE, FALSE);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL compGrName(CONST _VOID *a1, CONST _VOID *a2, _VOID *private)
{
	CONST group *p1 = a1;
	CONST group *p2 = a2;

	UNUSED(private);
	return strSortCmp(p1->name, p2->name) < 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID GroupSelect_SortGroupArray(_VOID)
{
	QuickSort_sortIt(0, numGroups - 1, groupArray, compGrName, sizeof(group), QuickSort_noBreak, NULL);
	buildCatNr2gNr();
}
/*** ------------------------------------------------------------------------ ***/
/*	GroupSelect_ParseYGB(_VOID)													*/
/*	Analysiert die YGB, diese beinhaltet die bestellten Gruppen					*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	20.12.01 von Dimitri Junker	DYJ 4.30zs										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.19a */
GLOBAL _VOID GroupSelect_ParseYGB(_VOID)
{
	_UBYTE fname[PATH_MAX],*ptr;
	TEXTFILENO file;
	group_line scrap;
	GROUPNR idx;
	_UBYTE flags[20]; 
	if(Infofiles_GetInfoFilename(YGB_SHORT_NAME, fname)!=0l &&
		mtTextfiles_OpenTextfile(fname, READ, 0, &file) == GERR_OK)
	{
		while (!mtTextfiles_EndofText(file))
		{
			mtTextfiles_ReadLine(file, scrap, sizeof(scrap));
			mtTextfiles_ReadLn(file);
			ptr=strChr(scrap,' ');
			if(ptr)
				*ptr='\0';
			if(strChr("+-",scrap[0])==NULL)
				break;
			strCpy(flags,"L+S+V+G+P-$-");
			if(scrap[0]=='-')
				flags[1]=flags[3]='B';
			if(!strCmp(scrap+1,"*"))		/* alle Eintraege zu oState setzen	 */
			{
				for(idx=0;idx < numGroups;idx++)
					UpdateEntry(groupArray[idx].name, "", "", "", flags, "", "", "", "", "", -1, -1, oNA, 0, LS_FLAG_LINE|LS_ORDERED, TRUE, FALSE);
			}
			else
				UpdateEntry(scrap+1, "", "", "", flags, "", "", "", "", "", -1, -1, oNA, 0, LS_FLAG_LINE|LS_ORDERED, TRUE, FALSE);
		}
		mtTextfiles_CloseTextfile(&file);
	}
	else
		Infofiles_SaveInfoInf();
	Alias_Setnames(groupArray, numGroups);
}
/*** ---------------------------------------------------------------------- ***/

	/*DYJ 5.21b */
GLOBAL _BOOL GroupSelect_ParseInfofile(_BOOL isIncr)
{
	_BOOL parseItg;
	_UBYTE fname[PATH_MAX];
	TEXTFILENO file;

	parseItg = (Infofiles_GetInfoFilename(ITG_SHORT_NAME, fname)!=0l);
	if (parseItg)
	{
		if (mtTextfiles_OpenTextfile(fname, READ, 0, &file) != GERR_OK)
		{
			parseItg = FALSE;
		} else
		{
			ParseITG(file);
			mtTextfiles_CloseTextfile(&file);
			/* Die Liste muss nicht mehr sortiert werden, da die automatisch sortiert
			 * eingefuegt werden
			 */
		}
	}
	if (!parseItg)
	{
		/* mit IGK versuchen */
		parseItg = (Infofiles_GetInfoFilename(IGK_SHORT_NAME, fname)!=0l);
		if (parseItg &&
			mtTextfiles_OpenTextfile(fname, READ, 0, &file) == GERR_OK)
		{
			ParseIGK(file);
			mtTextfiles_CloseTextfile(&file);
			/* Die Liste muss nicht mehr sortiert werden, da die automatisch sortiert
			 * eingefuegt werden
			 */
		} else
		{
			parseItg = FALSE;
		}
	}
	if (!parseItg)
	{
		TEXTFILENO fileA=NULL;
		_ULONG len;
		len=Infofiles_GetInfoFilename(YGA_SHORT_NAME, fname);
		parseItg = (len!=0l);
		if(isIncr)
		{
			if(parseItg && mtTextfiles_OpenTextfile(fname, APPEND, 0, &fileA)!=GERR_OK)	/*neue Gruppen zu YGA zufgen*/
				fileA=NULL;
			len=Infofiles_GetInfoFilename(YGN_SHORT_NAME, fname);
			parseItg = (len!=0l);
		}
		if (parseItg)
		{
			if (mtTextfiles_OpenTextfile(fname, READ, 0, &file) != GERR_OK)
			{
				parseItg = FALSE;
			} else
			{
				_ULONG aktPos,maxPos;
				_UBYTE txt[1024];
				aktPos=showImport_Inq(&maxPos,txt,sizeof(txt));
				if(aktPos!=0xffffffffl)
					showImport_Close();
				len/=1024;
				showImport_Open(len*len,Rsc_WrkString(SIO_GRP));
				ParseYG(file,isIncr,fileA);

				showImport_Close();
				if(aktPos!=0xffffffffl)
				{
					showImport_Open(maxPos,txt);
					showImport_SetNew(aktPos);
				}
				mtTextfiles_CloseTextfile(&file);
				/* Die Liste muss nicht mehr sortiert werden, da die automatisch sortiert
				 * eingefuegt werden
				 */
			}
		}
		if(fileA)
			mtTextfiles_CloseTextfile(&fileA);
	}

	if (!parseItg)
		Infofiles_SaveInfoInf();
	Alias_Setnames(groupArray, numGroups);
	return parseItg;
}
/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD CountInfofileLines(_VOID)
{
	_BOOL parseItg;
	_UBYTE fname[PATH_MAX];
	TEXTFILENO file;
	_UWORD lines;
	_UBYTE ch;

	lines = 0;
	parseItg = (Infofiles_GetInfoFilename(ITG_SHORT_NAME, fname)!=0l);
	if (parseItg)
	{
		if (mtTextfiles_OpenTextfile(fname, READ, 8192, &file) != GERR_OK)
		{
			parseItg = FALSE;
		} else
		{
			while (!mtTextfiles_EndofText(file))
			{
				ch = mtTextfiles_ReadChar(file);
				mtTextfiles_ReadLn (file);
				if (ch == 'G')
				{
					/* Nur Zeilen mit Gruppennamen zaehlen bei der ITG */
					++lines;
				}
			}
			mtTextfiles_CloseTextfile(&file);
		}
	}
	if (!parseItg)
	{
		/* mit IGK versuchen */
		parseItg = (Infofiles_GetInfoFilename(IGK_SHORT_NAME, fname)!=0l);
		if (parseItg &&
			mtTextfiles_OpenTextfile(fname, READ, 8192, &file) == GERR_OK)
		{
			while (!mtTextfiles_EndofText(file))
			{
				mtTextfiles_ReadLn(file);
				++lines;
			}
			mtTextfiles_CloseTextfile(&file);
		} else
		{
			parseItg = FALSE;
		}
	}
	if (!parseItg)
	{
		/* mit YGA versuchen */
		parseItg = (Infofiles_GetInfoFilename(YGA_SHORT_NAME, fname)!=0);
		if (parseItg &&
			mtTextfiles_OpenTextfile(fname, READ, 8192, &file) == GERR_OK)
		{
			while (!mtTextfiles_EndofText(file))
			{
				mtTextfiles_ReadLn(file);
				++lines;
			}
			mtTextfiles_CloseTextfile(&file);
			lines--; 	/*wegen der '.'-Zeile*/
		} else
		{
			parseItg = FALSE;
		}
	}
	if (!parseItg)
		Infofiles_SaveInfoInf();
	return lines;
}
	
/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
LOCAL _BOOL ParseGruppenInf(TEXTFILENO file)
{
	group_line gName;
	group_line voidname;
	lineSet linesAvail;
	_WORD lineNum;
	_WORD i;
	_BOOL sortGroups;

	linesAvail = LS_GROUP_NAME | LS_CAT_NUMBER | LS_READ_NUMBER | LS_ORDERED | LS_ORDER_DAYS;
	*gName = '\0';
	*voidname = '\0';
	lineNum = 1;
	while (!mtTextfiles_EndofText(file))
	{
		mtTextfiles_ReadLine(file, gName, sizeof(gName));
		mtTextfiles_ReadLn(file);
		if (gName[0] != '\0')
		{
			if (strCmp(gName,Rsc_WrkString(ERR_NO_GRP)) &&	
				!UpdateEntry(gName, voidname, voidname, voidname, voidname, voidname, voidname, voidname, voidname, voidname, lineNum, lineNum, oNA, 0, linesAvail, TRUE, FALSE))
				return FALSE;
			++lineNum;
		}
	}
	ConfVars_GetConfDefBool(cSortGroups, &sortGroups, TRUE);
	if (sortGroups)
	{
		/* Einfach die Lesenummern neu vergeben, da die Liste schon beim
		 * erstellen sortiert wird
		 */
		lineNum = 1;
		for (i = 0; i < MaxCatNr; i++)
			if (CatNr2gNr[i]!=(GROUPNR)dataSys_notSaved)
			{
				groupArray[CatNr2gNr[i]].readNumber = lineNum;
				++lineNum;
			}

	}
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----- Eigene Liste schreiben und lesen ------------------------------*/

/* Format der Datei, in der die Liste abgespeichert wird:
 * Pro Eintrag eine Zeile, die einzelnen Elemente sind getrennt durch ein
 * TAB-Zeichen (das kommt definitiv nie in Gruppennamen vor).
 * <name>TAB<netname>TAB<chef>TAB<kurztext>TAB<sysopinfo>TAB<followupto>TAB<readNumber>TAB<catNumber>TAB<flags>
 * Dabei werden die Flags wie im ITG gespeichert.
 */


LOCAL _VOID saveGroupListEntry(CONST group *gr, TEXTFILENO file)
{
	group_line flags;
	_UBYTE *p;

#define rwStateStr " +BCS-P "
#define uStateStr  " *+-    "
#define nStateStr  " +-=!   "
#define TAB '\t'

	if (gr->name != NULL)
		mtTextfiles_WriteLine(file, gr->name);
	mtTextfiles_WriteChar(file, TAB);
	if (gr->netname != NULL)
		mtTextfiles_WriteLine(file, gr->netname);
	mtTextfiles_WriteChar(file, TAB);
	if (gr->chef != NULL)
		mtTextfiles_WriteLine(file, gr->chef);
	mtTextfiles_WriteChar(file, TAB);
	if (gr->kurztext != NULL)
		mtTextfiles_WriteLine(file, gr->kurztext);
	mtTextfiles_WriteChar(file, TAB);
	if (gr->sysop != NULL)
		mtTextfiles_WriteLine(file, gr->sysop);
	mtTextfiles_WriteChar(file, TAB);
	if (gr->followup != NULL)
		mtTextfiles_WriteLine(file, gr->followup);
	mtTextfiles_WriteChar(file, TAB);
	mtTextfiles_WriteInt(file, gr->readNumber, 0);
	mtTextfiles_WriteChar(file, TAB);
	mtTextfiles_WriteInt(file, gr->catNumber, 0);
	mtTextfiles_WriteChar(file, TAB);
	p = flags;
	if (gr->readState != rwNA)
	{
		*p++ = 'L';
		*p++ = rwStateStr[(_WORD)gr->readState];
	}
	if (gr->writeState != rwNA)
	{
		*p++ = 'S';
		*p++ = rwStateStr[(_WORD)gr->writeState];
	}
	if (gr->userState != uNA)
	{
		*p++ = 'G';
		*p++ = uStateStr[(_WORD)gr->userState];
	}
	if (gr->isNet != nNA)
	{
		*p++ = 'V';
		*p++ = nStateStr[(_WORD)gr->isNet];
	}
	*p++ = 'P';
	if (gr->hasProg)
		*p++ = '+';
	else
		*p++ = '-';
	*p++ = '$';
	if (gr->withDollar)
		*p++ = '+';
	else
		*p++ = '-';
	*p = '\0';
	mtTextfiles_WriteLine(file, flags);
	mtTextfiles_WriteChar(file, TAB);
	mtTextfiles_WriteInt(file, (_WORD)gr->ordered, 0);
	mtTextfiles_WriteChar(file, TAB);
	mtTextfiles_WriteInt(file, gr->orderDays, 0);
	mtTextfiles_WriteChar(file, TAB);
	if (gr->herkunft != NULL)
		mtTextfiles_WriteLine(file, gr->herkunft);
	mtTextfiles_WriteChar (file, TAB);
	if (gr->sprache != NULL)
		mtTextfiles_WriteLine(file, gr->sprache);
	mtTextfiles_WriteChar(file, TAB);
	if (gr->alias != NULL)
		mtTextfiles_WriteLine(file, gr->alias);
	mtTextfiles_WriteLn(file);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL saveDataEntry(CONST group *gr, FILENO fhdl)
{
	groupHdr hdr;
	F_ERROR error;
	
#define set_offset(id) \
	if (gr->id != NULL) \
		hdr.id = h2ns((_UWORD)((size_t)(gr->id - (_UBYTE *)(gr->data)))); \
	else \
		hdr.id = 0

	hdr.name = h2ns(0);
	set_offset(netname);
	set_offset(chef);
	set_offset(kurztext);
	set_offset(sysop);
	set_offset(followup);
	set_offset(herkunft);
	set_offset(sprache);
	set_offset(alias);
#undef set_offset

	/* Jetzt die uebrigen Daten setzen */
	hdr.readNumber = h2ns(gr->readNumber);
	hdr.catNumber  = h2ns(gr->catNumber );
	hdr.readState  = h2ns(gr->readState );   /* F-Zeile */
	hdr.writeState = h2ns(gr->writeState);   /* F-Zeile */
	hdr.userState  = h2ns(gr->userState );   /* F-Zeile */
	hdr.isNet      = h2ns(gr->isNet     );   /* F-Zeile */
	hdr.hasProg    = h2ns(gr->hasProg   );   /* F-Zeile */
	hdr.withDollar = h2ns(gr->withDollar);   /* F-Zeile */
	hdr.selected   = h2ns(gr->selected  );   /* Fuer Darstellung in Liste, wird beim Einlesen auf FALSE gesetzt */
	hdr.ordered    = h2ns(gr->ordered   );   /* Fuer An-/Abbestellen von Gruppen */
	hdr.orderDays  = h2ns(gr->orderDays );   /* Madness-Erweiterung: Gruppe soviel Tage zurueck bestellen */
	hdr.dataLength = h2ns(gr->dataLength);   /* Die Laenge des folgenden Datenblocks */
	error = CatFiles_WriteMuch(sizeof(hdr), fhdl, &hdr);
	/* Jetzt die Daten schreiben */
	if (error == NoError)
		error = CatFiles_WriteMuch(hdr.dataLength, fhdl, gr->data);
	return error == NoError;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.26c */
GLOBAL _BOOL GroupSelect_SaveGroupList(_VOID)
{
	_UBYTE fname[PATH_MAX];
	TEXTFILENO file;
	GROUPNR i;
	FileHeaderType dbHdr;
	FILENO fhdl;
	F_ERROR error;
	
	strCpy(fname, DataPath);
	F_Path_Append(fname, groupFileName);
	if (mtTextfiles_OpenTextfile(fname, WRITE, 0, &file) == GERR_OK)
	{
		for (i = 0; i < numGroups; i++)
			saveGroupListEntry(&groupArray[i], file);
		mtTextfiles_CloseTextfile(&file);
		listModified = FALSE;
		/* Jetzt noch das Binaerfile schreiben */
		strBcpy(fname, DataPath);
		F_Path_Append(fname, groupDataName);
		fhdl = CatFiles_CreateFile(fname);
		if (fhdl > 0)
		{
			/* Header schreiben */
			dbHdr.CatMagic = h2nl(groupCatMagic);
			dbHdr.Version  = h2ns(groupVersion);
			dbHdr.VersionMagic = h2ns(groupVersionMagic);
			error = CatFiles_WriteMuch(dbHeaderLength, fhdl, &dbHdr);
			if (error != NoError)
			{
				CatFiles_CloseFile(fhdl);
				return TRUE;
			}
			i = 0;
			while (i < numGroups && saveDataEntry(&groupArray[i], fhdl))
				++i;
			CatFiles_CloseFile(fhdl);
		}
		Protokoll_SendPathUpdate(DataPath);
		return TRUE;
	}
	/* Fehlermeldung rauswerfen */
	sprintf(fname, Rsc_WrkString(MTE_noFileWrite), groupFileName);	/*DYJ 5.21i */
	MTE_info(fname);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Sichert die interne Gruppenliste in die Datei CATGROUP.INF,
 * wenn sie seit dem letzten Einlesen irgendwie geaendert wurde
 */
GLOBAL _BOOL GroupSelect_SaveChangedGroupList(_VOID)
{
 	if (listModified)
		return GroupSelect_SaveGroupList();
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *GetString(_UBYTE *src, _UBYTE *dst)
{
	while (*src != '\0' && *src != TAB)
		*dst++ = *src++;
	*dst = '\0';
	if (*src != '\0')
		++src;
	return src;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.26c */
LOCAL _BOOL parseGroupListLines(TEXTFILENO file)
{
	group *gr;
	GROUPNR lines;
	group_line gName;
	group_line nName;
	group_line uLine;
	group_line cLine;
	group_line fLine;
	group_line sLine;
	group_line rLine;
	group_line hLine;
	group_line aLine;
	group_line pLine;
	group_line oLine;
	group_line oNumber;
	group_line rNumber;
	group_line cNumber;
	group_line scrap;
	_UBYTE *p;
	orderState oState;
	_WORD orderDays;
	CAT_GROUP_NUMBER catNumber;
	CAT_GROUP_NUMBER readNumber;

	/* Erstmal Zeilen zaehlen */
	lines = 0;
	while (!mtTextfiles_EndofText(file))
	{
		mtTextfiles_ReadLn(file);
		++lines;
	}
	/* Jetzt wieder an den Anfang gehen */
	mtTextfiles_Reset(file);
	/* Jetzt allozieren */
	if (!MakeNewGroupArray(TRUE, lines))
		return FALSE;

	lines = 0;
	while (!mtTextfiles_EndofText(file))
	{
		/* Eine Zeile lesen und parsen */
		mtTextfiles_ReadLine(file, scrap, sizeof(scrap));
		mtTextfiles_ReadLn(file);
		p = scrap;
		p = GetString(p, gName);
		p = GetString(p, nName);
		p = GetString(p, cLine);
		p = GetString(p, uLine);
		p = GetString(p, sLine);
		p = GetString(p, rLine);
		p = GetString(p, rNumber);
		p = GetString(p, cNumber);
		p = GetString(p, fLine);
		p = GetString(p, oLine);
		p = GetString(p, oNumber);
		p = GetString(p, hLine);
		p = GetString(p, pLine);
		    GetString(p, aLine);
		
		/* Orderstatus herausfinden */
		if (oLine[0] != '\0')
			oState = (orderState)StrToInt(oLine, NULL, NULL);
		else
			oState = oNA;
		if (oNumber[0] != '\0')
			orderDays = StrToInt(oNumber, NULL, NULL);
		else
			orderDays = 0;
		/* Jetzt die Lesenummern rausfinden */
		if (cNumber[0] != '\0')
			catNumber = StrToInt(cNumber, NULL, NULL);
		else
			catNumber =(CAT_GROUP_NUMBER)dataSys_notSaved;
		if (rNumber[0] != '\0')
			readNumber = StrToInt(rNumber, NULL, NULL);
		else
			readNumber =(CAT_GROUP_NUMBER)dataSys_notSaved;
		/* Jetzt Eintrag setzen */
		gr = &groupArray[lines];
		gr->data = NULL;
		gr->dataLength = 0;
		if (!MakeEntry(gr, gName, nName, uLine, cLine, fLine, sLine, rLine, hLine, pLine, aLine, readNumber, catNumber, oState, orderDays, TRUE, LS_ALL))
			return FALSE;
		setCatNr2gNr(lines,catNumber);
		/* Ok, Eintrag jetzt vollstaendig. Jetzt anhaengen */
		/* Ist durch Array schon angehaengt :-) */
		++lines;
		++numGroups;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/
#include <time.h>
LOCAL _BOOL GroupSelect_parseData(FILENO fhdl)
{
	FileHeaderType dbHdr;
	groupHdr hdr;
	group *gr;
	F_ERROR error;
	_LONG anz;
	/* Aufbau der CATGROUP.DAT:
	 * fileHeader
	 * pro Gruppe:
	 * Header
	 * Textdaten (variabel)
	 */
	if (CatFiles_ReadDbHeader(fhdl, &dbHdr) != NoError)
		return FALSE;
	/* Jetzt ueberpruefen des Headers */
	if (dbHdr.CatMagic != groupCatMagic ||
		dbHdr.Version  != groupVersion ||
		dbHdr.VersionMagic != groupVersionMagic)
	{
		/* Abbrechen */
		return FALSE;
	}
	/* hier eventuell mal zaehlen, wie viele es denn sind */
	ConfVars_GetConfDefLongInt(cGroupNum,&anz, 600);
	if (!MakeNewGroupArray(TRUE,(GROUPNR)anz))
		return FALSE;
	numGroups = 0;
	do
	{
		/* Header lesen */
		error = CatFiles_ReadMuch(sizeof(groupHdr), fhdl, &hdr);
		if (error == NoError)
		{
			/* Ok, Header gelesen */
			if (!MakeNewGroupArray(FALSE, 0))
				return FALSE;
			/* Jetzt Eintrag setzen */
			gr = &groupArray[numGroups];
			gr->data = MALLOC(n2hs(hdr.dataLength), "GroupSelect_parseData");
			if (gr->data == NULL)
				return FALSE;

			/* Jetzt Daten kopieren */

			/* Pointer setzen */
			gr->name = gr->data;
#define set_adr(id) \
	if (hdr.id != 0) \
		gr->id = (_UBYTE *)gr->data + (size_t)(n2hs(hdr.id)); \
	else \
		gr->id = NULL
			set_adr(netname);
			set_adr(chef);
			set_adr(kurztext);
			set_adr(sysop);
			set_adr(followup);
			set_adr(herkunft);
			set_adr(sprache);
			set_adr(alias);
#undef set_adr

			/* Jetzt die uebrigen Daten setzen */
			gr->readNumber = n2hs(hdr.readNumber);
			gr->catNumber  = n2hs(hdr.catNumber );
			gr->readState  = n2hs(hdr.readState );   /* F-Zeile */
			gr->writeState = n2hs(hdr.writeState);   /* F-Zeile */
			gr->userState  = n2hs(hdr.userState );   /* F-Zeile */
			gr->isNet      = n2hs(hdr.isNet     );   /* F-Zeile */
			gr->hasProg    = n2hs(hdr.hasProg   );   /* F-Zeile */
			gr->withDollar = n2hs(hdr.withDollar);   /* F-Zeile */
			gr->selected   = FALSE  ;                /* Fuer Darstellung in Liste, wird beim Einlesen auf FALSE gesetzt */
			gr->ordered    = n2hs(hdr.ordered   );   /* Fuer An-/Abbestellen von Gruppen */
			gr->orderDays  = n2hs(hdr.orderDays );   /* Madness-Erweiterung: Gruppe soviel Tage zurueck bestellen */
			gr->dataLength = n2hs(hdr.dataLength);   /* Die Laenge des folgenden Datenblocks */
			gr->isUpdated  = FALSE;                  /* Eintrag wurde noch nicht upgedatet */
			/* Jetzt Daten lesen */
			error = CatFiles_ReadMuch(gr->dataLength, fhdl, gr->data);
			if (error != NoError)
			{
				/* Lesefehler, abbrechen */
				return FALSE;
			}
			setCatNr2gNr(numGroups,gr->catNumber);
			++numGroups;
		}
	} while (error == NoError);
	ConfVars_SetConfigLongInt(cGroupNum,numGroups);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _BOOL GroupSelect_ReadGroupList(_VOID)
{
	_UBYTE fname[PATH_MAX];
	TEXTFILENO file;
	FILENO fhdl;
	_ULONG size;
	_BOOL exists;
	_UWORD lines;
	_BOOL ok;

	MouseOn();
	MouseBusy();
	FreeGroupArray();
	strCpy(fname, DataPath);
	F_Path_Append(fname, groupDataName);
	size = CatFiles_FileSize(fname, &exists);
	if (exists && size > 0)
	{
		fhdl = CatFiles_OpenFile(fname, CatFiles_readFile);
		if (fhdl > 0)
		{
			if (!GroupSelect_parseData(fhdl))
			{
				/* Einfach danach das CATGROUP.INF parsen,
				 * deshalb alles wieder freigeben
				 */
				CatFiles_CloseFile(fhdl);
				FreeGroupArray();
			} else
			{
				CatFiles_CloseFile(fhdl);
				listModified = FALSE;
				Alias_Setnames(groupArray, numGroups);
				MouseArrow();
				return TRUE;
			}
		}
	}
	
	F_Path_Append(strCpy(fname, DataPath), groupFileName);
	size = CatFiles_FileSize(fname, &exists);
	if (exists && size > 0 && mtTextfiles_OpenTextfile(fname, READ, 0, &file) == GERR_OK)
	{
		if (!parseGroupListLines(file))
		{
			MouseArrow();
			mtTextfiles_CloseTextfile(&file);
			return FALSE;
		}
		mtTextfiles_CloseTextfile(&file);
		listModified = TRUE;
		Alias_Setnames(groupArray, numGroups);
		MouseArrow();
		return TRUE;
	} else
	{
		/* File kann nicht geoeffnet werden. Dann versuchen wir es mal
		 * anders und bauen das neu auf aus der GRUPPEN.INF und der
		 * ITG,YGA bzw. IGK
		 */
		F_Path_Append(strCpy(fname, DataPath), catGroupInf);
		size = CatFiles_FileSize(fname, &exists);
		if (exists && size > 0 && mtTextfiles_OpenTextfile(fname, READ, 0, &file) == GERR_OK)
		{
			/* Erstmal hier die Zeilen zaehlen */
			lines = 0;
			while (!mtTextfiles_EndofText(file))
			{
				mtTextfiles_ReadLn(file);
				++lines;
			}
			/* Jetzt wieder an den Anfang gehen */
			mtTextfiles_Reset(file);
			/* Jetzt die Zeilen im Infofile zaehlen */
			lines += CountInfofileLines();
			/* Und jetzt allozieren wir den Speicher */
			if (!MakeNewGroupArray(TRUE, lines))
			{
				mtTextfiles_CloseTextfile(&file);
				MouseArrow();
				return FALSE;
			}
			ok = ParseGruppenInf(file);
			listModified = TRUE;
			mtTextfiles_CloseTextfile(&file);
			if (ok)
			{
				/* Ok, Gruppen.inf geparst, jetzt noch ITG,YGA bzw. IGK parsen */
				GroupSelect_ParseInfofile(FALSE);
				GroupSelect_SaveGroupList();
				MouseArrow();
				return TRUE;
			} else
			{
				MouseArrow();
				return FALSE;
			}
		} else if (!exists || size == 0)
		{
			/* Keine GRUPPEN.INF da, wahrscheinlich leere Database.
			 * Deshalb jetzt nur die Infofiles parsen
			 */
			lines = CountInfofileLines ();
			/* Und jetzt allozieren wir den Speicher */
			if (!MakeNewGroupArray(TRUE, lines))
			{
				mtTextfiles_CloseTextfile(&file);
				MouseArrow();
				return FALSE;
			}
			GroupSelect_ParseInfofile(FALSE);
			GroupSelect_SaveGroupList();
			MouseArrow();
			return TRUE;
		}
	}
	/* Fehlermeldung rauswerfen */
	MouseArrow();
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Funktionen, die fuer das Zusammenstellen der neuen Listen benoetigt werden */

/* kopiert einen Eintrag in die Liste l
 */
LOCAL _BOOL CopyEntry(group *entry, List *l)
{
	group *new;

	new = NEW(group, "GroupSelect_CopyEntry: group");
	if (new == NULL)
		return FALSE;
	*new = *entry;
	new->data = MALLOC(entry->dataLength, "GroupSelect_CopyEntry: data");
	if (new->data == NULL)
	{
		OFREE(new);
		return FALSE;
	}
	MemCpy(new->data, entry->data, entry->dataLength);
#define copy(id) \
	if (entry->id != NULL) \
		new->id = (_UBYTE *)new->data + ((_UBYTE *)entry->id - (_UBYTE *)entry->data); \
	else \
		new->id = NULL
	
	new->name = new->data;
	copy(display_name);
	copy(netname);
	copy(chef);
	copy(kurztext);
	copy(sysop);
	copy(followup);
	copy(herkunft);
	copy(sprache);
	copy(alias);
#undef copy
	return Lists_AppendEntry(l, new);
}

/*** ---------------------------------------------------------------------- ***/

/* Kopiert die Gruppenliste in eine neue Liste. Dabei wird ein Eintrag
 * genau dann kopiert, wenn doCopy TRUE liefert.
 */
typedef _BOOL (*mayCopyProc)(groupPtr entry, _VOID *param);

/*DYJ 5.26c */
LOCAL _BOOL GroupSelect_CopyList(List *list, mayCopyProc doCopy, _VOID *param)
{
	group *run;
	GROUPNR idx;

	if (!Lists_CreateList(list, "GroupSelect_CopyList"))
		return FALSE;
	for (idx = 0, run = groupArray; idx < numGroups; idx++, run++)
	{
		/* Testen, ob es eine CAT-Gruppe ist */
		if (doCopy(run, param))
		{
			/* Eintrag kopieren(!!) und anhaengen an neue Liste */
			if (!CopyEntry(run, list))
			{
				GroupSelect_DeleteList(list);
				return FALSE;
			}
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* hier kommen jetzt die einzelnen Vergleichsfunktionen fuer GroupSelect_CopyList
 */
LOCAL MSGNR GetNew(CAT_GROUP_NUMBER catNumber)
{
	MSGNR newIdx, lastIdx;

	if (catNumber >=(CAT_GROUP_NUMBER)dataSys_notSaved)
		return 0;
	newIdx = data_FirstNewMsg(catNumber);
	lastIdx = data_LastMsgOfGroup(catNumber);
	if (lastIdx != dataSys_empty && newIdx != dataSys_empty && newIdx <= lastIdx)
		return lastIdx - newIdx + 1;
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL GroupSelect_copyByNum(group *entry, _VOID *param)
{
	UNUSED(param);
	return entry->catNumber <(CAT_GROUP_NUMBER)dataSys_notSaved;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL copyNew(group *entry, _VOID *param)
{
	UNUSED(param);
	return GetNew(entry->catNumber) > 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL copyUnread(group *entry, _VOID *param)
{
	MSGNR unread;
	CAT_GROUP_NUMBER catNumber;

	UNUSED(param);
	catNumber = entry->catNumber;
	if (catNumber >=(CAT_GROUP_NUMBER)dataSys_notSaved)
		return FALSE;
	unread = data_unreadMsgCount(catNumber);
	if (unread == dataSys_empty)
		unread = 0;
	return unread > 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL copyUser(group *entry, _VOID *param)
{
	_BOOL isSysop;

	isSysop = *((_BOOL *)param);
	return entry->userState == uNA ||
		   entry->userState == uIsChef ||
		   (isSysop && !(entry->isNet == nNetLokal || entry->isNet == nNetForbid));
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL copyProgteil(group *entry, _VOID *param)
{
	UNUSED(param);
	return entry->hasProg == TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL copyLokal(group *entry, _VOID *param)
{
	UNUSED(param);
	/* Es werden jetzt wirklich nur lokale Gruppen ausgewaehlt (MH) */
	return entry->isNet == nLokal;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL copyOrdered(group *entry, _VOID *param)
{
	UNUSED(param);
	return ((entry->readState == rwOn || entry->readState == rwDefault) && entry->ordered != oOff);
}

/*** ------------------------------------------------------------------------ ***/
/*	_BOOL copyPseudo(group *entry, _VOID *param)								*/
/*	gibt zurck ob es eine Pseudogruppe ist.									*/
/*	Parameter:	entry:	Gruppe													*/
/*				param:	nicht benutzt											*/
/*	Ergebnis:	Pseudogruppe?													*/		
/*	18.7.01 von Dimitri Junker	DYJ 4.24f										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _BOOL copyPseudo(group *entry, _VOID *param)
{
	UNUSED(param);
/*DYJ 5.30g Start*/
	return entry->isNet < nNA;
/*DYJ  Ende; alt:	
	return entry->isNet <= nNA;
*/
}
/*** ------------------------------------------------------------------------ ***/
/*	_BOOL copyDatenb(group *entry, _VOID *param)								*/
/*	gibt zurck ob die Gruppe in der Datenbank ist.								*/
/*	Parameter:	entry:	Gruppe													*/
/*				param:	nicht benutzt											*/
/*	Ergebnis:	in Datenbank?													*/		
/*	18.7.01 von Dimitri Junker	DYJ 4.24f										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _BOOL copyDatenb(group *entry, _VOID *param)
{
	UNUSED(param);
	return entry->catNumber<(CAT_GROUP_NUMBER)dataSys_notSaved;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL copyWrite(group *entry, _VOID *param)
{
	_BOOL fullList;
	
	fullList = *((_BOOL *)param);
	if (fullList)
	{
 	   return entry->writeState == rwNA || entry->writeState == rwOn || entry->writeState == rwPossible;
	}
	return entry->catNumber > 0 && entry->catNumber<(CAT_GROUP_NUMBER)dataSys_notSaved &&
		(entry->writeState == rwNA || entry->writeState == rwOn || entry->writeState == rwPossible);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL copyAll(group *entry, _VOID *param)
{
	UNUSED(entry);
	UNUSED(param);
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***
 * Hier kommen jetzt Funktionen, die einzelne Listen oder Teillisten
 * den anderen Modulen zur Verfuegung stellen.
 *
 *** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Loescht eine Liste vollstaendig, also auch die Wurzel davon. Danach darf
 * nicht mehr auf die Liste zugegriffen werden.
 */
LOCAL _VOID GroupSelect_DeleteData(_VOID *entry)
{
	group *gr = entry;

	if (gr != NULL)
	{
		if (gr->data != NULL)
			FREE(gr->data, gr->dataLength);
		OFREE(gr);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID GroupSelect_DeleteList(List *list)
{
	MouseBusy();
	Lists_DeleteSimpleList(list, GroupSelect_DeleteData, TRUE);
	MouseArrow();
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
GLOBAL group *create_private_group(_VOID)
{
	group *new;

	new = NEW(group, "create_private_group: group");
	if (new != NULL)
	{
		MemSetZeroStruct(*new);
		new->dataLength = (data_offset)sizeof(Rsc_WrkString(GRP_PRIVATE));
		new->data = MALLOC(new->dataLength, "create_private_group: data");
		if (new->data == NULL)
		{
			OFREE(new);
			new = NULL;
		} else
		{
			new->name = new->data;
			strCpy(new->name, Rsc_WrkString(GRP_PRIVATE));
			new->netname = NULL;
			new->chef    = NULL;
			new->kurztext  = NULL;
			new->sysop     = NULL;
			new->followup  = NULL;
			new->herkunft  = NULL;
			new->sprache   = NULL;
			new->alias     = NULL;
			new->readNumber = 0;
			new->catNumber = dataSys_private;
			new->readState = rwOn;
			new->writeState = rwImpossible;
			new->userState = uNA;
			new->isNet     = nNA;
			new->ordered   = oNA;
			new->orderDays = 0;
			new->hasProg   = FALSE;
			new->withDollar = FALSE;
			new->selected  = FALSE;
			new->isUpdated = FALSE;
			new->display_name = new->name;
		}
	}
	return new;
}

/*** ---------------------------------------------------------------------- ***/

/* Diese Prozedur liefert eine Liste der Gruppen, die in CAT vorhanden
 * und damit lesbar sind. Die Gruppe 'PERSOENLICHE' ist mit enthalten.
 * Die Liste ist nach der Lesereihenfolge sortiert, die Gruppe 'PERSOENLICHE'
 * ist immer als erste Gruppe vorhanden (die Lesenummer davon kann nicht
 * geaendert werden).
 * FALSE: Liste konnte nicht erzeugt werden
 */
GLOBAL _BOOL GroupSelect_GetReadList(List *list, _BOOL byReadNum, _BOOL getNew, _BOOL getUnread)
{
	group *new;
	MSGNR unrdCnt;
	_BOOL copyPers;

	MouseBusy();
	if (getNew && !getUnread)
	{
		if (!GroupSelect_CopyList(list, copyNew, NULL))
		{
			MouseArrow();
			return FALSE;
		}
	} else if (!getNew && getUnread)
	{
		if (!GroupSelect_CopyList(list, copyUnread, NULL))
		{
			MouseArrow();
			return FALSE;
		}
	} else
	{
		if (!GroupSelect_CopyList(list, GroupSelect_copyByNum, NULL))
		{
			MouseArrow();
			return FALSE;
		}
	}
	/* Jetzt haben wir eine neue Liste, die wir nur noch nach der
	 * Lesereihenfolge sortieren muessen
	 */
	if (byReadNum)
	{
		SortList(list, sByReadNum);  /* Hier eventuell noch Sortierprozedur uebergeben */
	} else
	{
		SortList(list, sByName);
	}
	/* So, jetzt haben wir eine sortierte Liste. Jetzt muessen wir nur
	 * noch die persoenliche Gruppe evtl. dazu eintragen und vorne einfuegen.
	 */
	if (getNew && !getUnread)
	{
		copyPers = GetNew(dataSys_private) > 0;
	} else if (!getNew && getUnread)
	{
		unrdCnt = data_unreadMsgCount(dataSys_private);
		copyPers = unrdCnt > 0 && unrdCnt != dataSys_empty;
	} else
	{
		copyPers = TRUE;
	}
	if (copyPers)
	{
		new = create_private_group();
		if (new == NULL)
		{
			GroupSelect_DeleteList(list);
			MouseArrow();
			return FALSE;
		}
		Lists_ResetList(list);
		if (!Lists_InsertEntry(list, new))
		{
			GroupSelect_DeleteData(new);
			GroupSelect_DeleteList(list);
			MouseArrow();
			return FALSE;
		}
	}
	MouseArrow();
	return TRUE;
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _BOOL copyUnkn(group *entry, _VOID *param)							*/
/*	Kopiert die Gruppen die keine CatNumber haben, fr Repair(Adoptieren)		*/
/*	Parameter:	entry:	die ggf zu kopierende Gruppe							*/
/*				param:	unbenutzt												*/
/*	Ergebnis:	keines															*/		
/*	12.8.01 von Dimitri Junker	DYJ 4.24l										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _BOOL copyUnkn(group *entry, _VOID *param)
{

	UNUSED(param);
	return entry->catNumber>=(CAT_GROUP_NUMBER)dataSys_notSaved;
}

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL GroupSelect_GetNoReadList(List *list)							*/
/*	Diese Prozedur liefert eine Liste der Gruppen, die in CAT nicht vorhanden	*/
/*	sind. Die Gruppe 'PERSOENLICHE' ist natrlich nicht enthalten.				*/
/*	Die Liste ist nach den Namen sortiert										*/
/*	Parameter:	list:	da solls hin											*/
/*	Ergebnis:	FALSE: Liste konnte nicht erzeugt werden						*/
/*	12.8.01 von Dimitri Junker	DYJ 4.24l										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL GroupSelect_GetNoReadList(List *list)
{

	MouseBusy();
	if (!GroupSelect_CopyList(list, copyUnkn, NULL))
	{
		MouseArrow();
		return FALSE;
	}
	/* Jetzt haben wir eine neue Liste, die wir nur noch nach dem
	 * Namen sortieren muessen
	 */
	SortList(list, sByName);
	MouseArrow();
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Hiermit wird das Modul aufgefordert, die Lesereihenfolge zu aendern.
 * Es muss als Parameter eine Liste uebergeben werden, die eine geaenderte
 * Lesereihenfolge hat und die ueber GetReadList angefordert wurde.
 * FALSE: Interne Liste konnte nicht upgedatet werden
 */
GLOBAL _BOOL GroupSelect_UpdateReadList(List *list)
{
	group *run;
	GROUPNR grIdx;	

	MouseBusy();
	Lists_ResetList(list);
	while ((run = Lists_NextEntry(list)) != NULL)
	{
		if (run->name != NULL)
			if (findEntry(run->name, &grIdx))
				if (groupArray[grIdx].readNumber != run->readNumber)
				{
					groupArray[grIdx].readNumber = run->readNumber;
					listModified = TRUE;
				}
	}
	MouseArrow();
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Diese Prozedur liefert eine Liste der Gruppen, in denen man Chef ist.
 * Diese wird benoetigt, um User in andere Gruppen einzutragen.
 * FALSE: Liste konnte nicht erzeugt werden
 */
GLOBAL _BOOL GroupSelect_GetUserList(List *list)
{
	_BOOL ok;
	_BOOL isSysop;

	MouseBusy();
	/* Sysop feststellen */
	isSysop = Infofiles_IsInfoFile(ISB_SHORT_NAME, TRUE);
	ok = GroupSelect_CopyList(list, copyUser, &isSysop);
	MouseArrow();
	return ok;
}

/*** ---------------------------------------------------------------------- ***/

/* Diese Prozedur liefert eine Liste der Gruppen, die einen Programm-
 * teil haben. Diese Liste wird fuer die Programmteil-Kommandos benutzt.
 * FALSE: Liste konnte nicht erzeugt werden
 */
GLOBAL _BOOL GroupSelect_GetProgteilList(List *list)
{
	_BOOL ok;

	MouseBusy();
	ok = GroupSelect_CopyList(list, copyProgteil, NULL);
	MouseArrow();
	return ok;
}

/*** ---------------------------------------------------------------------- ***/

/* Diese Prozedur liefert eine Liste der Gruppen, in denen man schreiben
 * kann. Diese Liste wird dann benutzt, wenn man eine neue Nachricht in
 * eine Gruppe schreiben will.
 * FALSE: Liste konnte nicht erzeugt werden
 */
GLOBAL _BOOL GroupSelect_GetWriteList(List *list, _BOOL full)
{
	_BOOL ok;
	group *new;

	MouseBusy();
	ok = GroupSelect_CopyList(list, copyWrite, &full);
	if (!ok)
	{
		MouseArrow();
		return FALSE;
	}
	new = create_private_group();
	if (new == NULL)
	{
		GroupSelect_DeleteList(list);
		MouseArrow();
		return FALSE;
	}
	Lists_ResetList(list);
	ok = Lists_InsertEntry(list, new);
	MouseArrow();
	if (!ok)
	{
		GroupSelect_DeleteData(new);
		GroupSelect_DeleteList(list);
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Diese Prozedur liefert eine Liste der Gruppen, die in der Maus vorhanden
 * sind bzw. alle Gruppen aus dem ITG. Benutzt wird diese Liste fuer die
 * Gruppenauswahl.
 * lokal: Es werden nur die lokal vorhandenen Gruppen geliefert
 * FALSE: Liste konnte nicht erzeugt werden
 */
GLOBAL _BOOL GroupSelect_GetSelectList(List *list, selDispType disp, selSortType sortBy)
{
	mayCopyProc proc;
	
	switch (disp)
	{
		case dAll: proc = copyAll; break;
		case dLocal: proc = copyLokal; break;
		case dOrdered: proc = copyOrdered; break;
		case dPseudo: proc= copyPseudo;break;
		case dDatenb: proc= copyDatenb;break;	/*DYJ 5.35c */
		default: return FALSE;
	}
	MouseBusy();
	if (!GroupSelect_CopyList(list, proc, NULL))
	{
		MouseArrow();
		return FALSE;
	}
	switch (sortBy)
	{
		case sName: SortList(list, sByName); break;
		case sNet: SortList(list, sByNet); break;
		case sLang: SortList(list, sByLang); break;
		case sRead: SortList(list, sByReadNum); break;	/*DYJ 5.35c */
	}
	MouseArrow();
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* (MH)
 * Diese Prozedur liefert eine Liste der User, die in der ITU (Technische
 * Userliste) oder in der USERADR.INF enthalten sind. Benutzt wird diese 
 * Liste fr die Namensauswahl im ITK-Kommando-Modul.
 * FALSE: Liste konnte nicht erzeugt werden (ITU ist nicht vorhanden etc.)
 */
GLOBAL _BOOL GroupSelect_GetUsernameList(List *list)
{
	_BOOL isAdr;
	_UBYTE ituScrap[256];
	_UBYTE filename[PATH_MAX];
	TEXTFILENO tf;
	group newgroup;
	group *gr;
	
	isAdr = FALSE;
	if (!Infofiles_GetInfoFilename(ITU_NAME, filename))
	{
		/* Ist USERADR.INF da? */
		F_Path_Append(strBcpy(filename, DataPath), adrFileName);
	    isAdr = CatFiles_Exists(filename);
	    if (!isAdr)
	    	return FALSE;
	}
	if (!Lists_CreateList(list, "GroupSelect_GetUsernameList"))
		return FALSE;
	*ituScrap = '\0';
	if (mtTextfiles_OpenTextfile(filename, READ, 8192, &tf) == GERR_OK)
	{
		do
		{
			mtTextfiles_ReadLine(tf, ituScrap, sizeof(ituScrap));
			mtTextfiles_ReadLn(tf);
			if (isAdr ||
				(!isAdr && ituScrap[0] == 'N'))
			{
				gr = &newgroup;
				empty_group(gr);
				if (!MakeEntry(gr, isAdr ? ituScrap : ituScrap + 1, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, 0, TRUE, LS_GROUP_NAME) ||
					!CopyEntry(gr, list))
				{
					if (gr->data != NULL)
						FREE(gr->data, gr->dataLength);
					mtTextfiles_CloseTextfile(&tf);
					GroupSelect_DeleteList(list);
					return FALSE;
				}
				FREE(gr->data, gr->dataLength);
			}
		} while (!mtTextfiles_EndofText(tf));
		mtTextfiles_CloseTextfile(&tf);
		/* Jetzt sortieren */
		SortList(list, sByName);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Hiermit wird das Modul aufgefordert, sich zu merken, welche Gruppen
 * beim naechsten Tausch bestellt oder abbestellt werden sollen.
 * Es muss als Parameter eine Liste uebergeben werden,
 * die ueber GetSelectList angefordert wurde
 * FALSE: Interne Liste konnte nicht upgedatet werden
 */
/*DYJ 5.35c Start*/
GLOBAL _BOOL GroupSelect_UpdateSelectList(List *list, _BOOL resort)
{
	group *gr, *run;
	GROUPNR grIdx;
	_UBYTE *oName;
	UNUSED(resort);
	MouseBusy();
	Lists_ResetList(list);
	while ((run = Lists_NextEntry(list)) != NULL)
	{
		if(run->isUpdated&LS_GROUP_NAME)	/* Gruppe wurde umbenannt*/
			oName=run->netname;
		else
			oName=run->name;
		if (oName != NULL)
		{
			if (findEntry(oName, &grIdx))
			{
				gr = &groupArray[grIdx];
				if (gr->ordered != run->ordered || gr->orderDays != run->orderDays)
				{
					gr->ordered = run->ordered;
					gr->orderDays = run->orderDays;
					listModified = TRUE;
				}
				if(run->isUpdated)
				{
					listModified = TRUE;
					if(run->isUpdated&LS_GROUP_NAME)	/* Gruppe wurde umbenannt*/
						GroupSelect_SetGrpPar(gr, run->name,LS_GROUP_NAME);
					if(run->isUpdated&LS_FOLLOWUP_LINE)
						GroupSelect_SetGrpPar(gr, run->followup,LS_FOLLOWUP_LINE);
					if(run->isUpdated&LS_KURZTEXT)
						GroupSelect_SetGrpPar(gr, run->kurztext,LS_KURZTEXT);
					gr->isNet=run->isNet;
					if(run->isUpdated&LS_CHEF_LINE)
					{
						if(run->isUpdated&=LS_SYSOP_LINE)
						{
							/* User als Chef setzen */
							_UBYTE fname[PATH_MAX];
							FILENO out;
							strBcpy(fname, MessagePath);
							F_Path_Append(fname, grSelect);
							out = CatFiles_OpenFile(fname, CatFiles_writeFile);
							if (out < 0)
							{
								MTE_info(Rsc_WrkString(MTE_NoSaveChef));
							} else
							{
								_UBYTE cmd[256];
								CatFiles_Seek(0L, out, CatFiles_end);
								/* Kommandostring bauen */
								strBcpy(cmd, ":GU");
								strBcat(cmd, run->chef);
								strBcat(cmd, "=");
								strBcat(cmd, gr->name);
								CatFiles_WriteMuch(strLen(cmd), out, cmd);
								CatFiles_WriteFile(CR, out);
								CatFiles_WriteFile(LF, out);
								CatFiles_CloseFile(out);
							}
						}
						else
							GroupSelect_SetGrpPar(gr, run->chef,LS_CHEF_LINE);
					}
				}
			}
		}
	}
	MouseArrow();
	return TRUE;
}
/*DYJ  Ende; alt:	
GLOBAL _BOOL GroupSelect_UpdateSelectList(List *list, _BOOL resort)
{
	group *gr, *run;
	GROUPNR grIdx;
	UNUSED(resort);
	MouseBusy();
	Lists_ResetList(list);
	while ((run = Lists_NextEntry(list)) != NULL)
	{
		if (run->name != NULL)
		{
			if (findEntry(run->name, &grIdx))
			{
				gr = &groupArray[grIdx];
				if (gr->ordered != run->ordered || gr->orderDays != run->orderDays)
				{
					gr->ordered = run->ordered;
					gr->orderDays = run->orderDays;
					listModified = TRUE;
				}
				if (run->followup != NULL)
				{
					if ((gr->followup != NULL && strSortCmp(run->followup, gr->followup) != 0) ||
						gr->followup == NULL)
					{
						/* Followup setzen */
						GroupSelect_SetFollowUp(gr, run->followup);
						listModified = TRUE;
					}
				} else if (gr->followup != NULL)
				{
					/* Followup wurde geloescht */
					GroupSelect_SetFollowUp(gr, "");
					listModified = TRUE;
				}
			}
		}
	}
	MouseArrow();
	return TRUE;
}
*/



/******************************************************************************/
/*** ---------------------------------------------------------------------- ***
 * Jetzt noch ein paar Prozeduren fuer data
 *** ---------------------------------------------------------------------- ***/
/******************************************************************************/


/* 'name' in angegebene Datei schreiben, fuer Absender- und Gruppennamen */
/*DYJ 5.21b */
GLOBAL _BOOL GroupSelect_NameToFile(CONST _UBYTE *name, CONST _UBYTE *path, CONST _UBYTE *fname)
{
	FILENO out;
	_UBYTE filename[PATH_MAX];
	F_ERROR error;
	
	strBcpy(filename, path);
	if(fname)	
		F_Path_Append(filename, fname);
	out = CatFiles_OpenFile(filename, CatFiles_writeFile);
	if (FILENO_OK(out))
	{
		CatFiles_Seek(0, out, CatFiles_end);
		error = CatFiles_WriteMuch(strLen(name), out, name);
		if (error == NoError)
			error = CatFiles_WriteFile(CR, out);
		if (error == NoError)
			CatFiles_WriteFile(LF, out);
		CatFiles_CloseFile(out);
		return error == NoError;
	}
	MTE_info(Rsc_WrkString(AL_NAME_NOT_WRITTEN));
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Sichert die Liste der Gruppen komplett
 */
/*DYJ 5.21b */
GLOBAL _VOID GroupSelect_SaveGruppenInf(_VOID)
{
	_UBYTE file[PATH_MAX];
	TEXTFILENO out;
	CAT_GROUP_NUMBER i, num;
	_BOOL firstErr=TRUE;
	GROUPNR idx;
	group_line tmp;
	
	strCpy(file, DataPath);
	F_Path_Append(file, catGroupInf);
	if (mtTextfiles_OpenTextfile(file, WRITE, 4096, &out) == GERR_OK)
	{
		GroupSelect_CountCatGroupsMax(&num);
		for (i = 1; i <= num; i++)
		{
			if (findNumberEntry(i, &idx))
			{
				if (groupArray[idx].name != NULL)
				{
					strCpy(tmp, groupArray[idx].name);
					strUpr(tmp);
					mtTextfiles_WriteLine(out, tmp);
					mtTextfiles_WriteLn(out);
				}
			}
			else 
			{	
				mtTextfiles_WriteLine(out, Rsc_WrkString(ERR_NO_GRP));
				mtTextfiles_WriteLn(out);
				if(firstErr && ProgramName[0] != 'R')
				{
					MTE_info(Rsc_WrkString(AL_GROUPNR_CORR));
					firstErr=FALSE;
				}
			}
		}
		mtTextfiles_CloseTextfile(&out);
		Protokoll_SendPathUpdate(DataPath);
	}
}

/*** ------------------------------------------------------------------------ ***/
/* GLOBAL _BOOL GroupSelect_ChgGroup(CONST _UBYTE *oldName, CONST _UBYTE *newName,CONST _UBYTE newF,CONST _UBYTE *newTxt, _BOOL saveGruppenInf)		*/
/*	Benennt eine Gruppe um und ndert die Flags und die Kurzbeschreibung.		*/
/* 	Die GRUPPEN.INF wird danach direkt neu geschrieben, die Aenderungen in den 	*/
/*	anderen Daten werden am Programmende gesichert								*/
/*																				*/
/*	Parameter:	oldName:	der aktuelle Name der Gruppe						*/
/*				newName:	der neue Name oder ""								*/
/*				newF:		eine F-Zeile										*/
/*				newTxt:		neue Kurzbeschreibung								*/
/*				saveGruppenInf: soll die Gruppen.inf gespeichert werden?		*/
/*	Ergebnis:	erfolgreich?													*/		
/*	18.7.01 von Dimitri Junker	DYJ 4.24f										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.21b */
GLOBAL _BOOL GroupSelect_ChgGroup(CONST _UBYTE *oldName, CONST _UBYTE *newName,CONST _UBYTE newF,CONST _UBYTE *newTxt, _BOOL saveGruppenInf)
{
	group_line gName;
	group_line nName;
	group_line uLine;
	group_line cLine;
	group_line fLine;
	group_line sLine;
	group_line rLine;
	group_line hLine;
	group_line aLine;
	group_line pLine;
	_BOOL found;
	GROUPNR idx,idx2;
	lineSet avail;
	group *gr;

	/* Persoenliche laesst sich nicht umbenennen */
	if (strCaseCmp(oldName, Rsc_WrkString(GRP_PRIVATE)) == 0)
		return FALSE;
	found = findEntry(oldName, &idx);
	if (found)
	{
		if (findEntry(newName, &idx2))
		{
			if (groupArray[idx2].catNumber<(CAT_GROUP_NUMBER)dataSys_notSaved)
			{
				/* Gruppe ist in CAT schon vorhanden */
				return FALSE;
			}
			/* Es sind beide Gruppennamen vorhanden. D.h. der Eintrag newName
			 * erhaelt einfach die CAT-Infos zugewiesen, am Eintrag selber
			 * wird aber nichts geaendert.
			 */
			gr = &groupArray[idx];
			groupArray[idx2].readNumber = gr->readNumber;

			setCatNr2gNr(dataSys_notSaved,groupArray[idx2].catNumber);
			setCatNr2gNr(idx,gr->catNumber);

			groupArray[idx2].catNumber = gr->catNumber;
			gr->readNumber =(CAT_GROUP_NUMBER)dataSys_notSaved;
			gr->catNumber =(CAT_GROUP_NUMBER)dataSys_notSaved;
			listModified = TRUE;
			if (saveGruppenInf)
			{
				/* Jetzt die geaenderte GRUPPEN.INF speichern */
				GroupSelect_SaveGruppenInf();
			}
			/* Jetzt das Array neu sortieren! */
			Alias_Setnames(groupArray, numGroups);
			GroupSelect_SortGroupArray();
			/* Und jetzt raus hier */
			return TRUE;
		}
		gr = &groupArray[idx];
		*nName = '\0';
		*cLine = '\0';
		avail = LS_CAT_NUMBER | LS_READ_NUMBER | LS_ORDERED | LS_ORDER_DAYS;
		if(*newName)
		{
			strCpy(gName, newName);
			avail |= LS_GROUP_NAME;
		}
		else
		{
			*gName = '\0';
			if (gr->name != NULL)
			{
				strCpy(gName, gr->name);
				avail |= LS_GROUP_NAME;
			}
		}
		if(*newTxt)
		{
			strCpy(uLine, newTxt);
			avail |= LS_KURZTEXT;
		}
		else
		{
			*uLine = '\0';
			if (gr->kurztext != NULL)
			{
				strCpy(uLine, gr->kurztext);
				avail |= LS_KURZTEXT;
			}
		}
		if(newF)
		{
			*fLine = 'V';
			fLine[1] = newF;
			fLine[2] = '\0';
			avail |= LS_FLAG_LINE;
		}
		else		
			*fLine = '\0';
		*sLine = '\0';
		*rLine = '\0';
		*hLine = '\0';
		*pLine = '\0';
		*aLine = '\0';
		if (gr->netname != NULL)
		{
			strCpy(nName, gr->netname);
			avail |= LS_NET_LINE;
		}
		if (gr->chef != NULL)
		{
			strCpy(cLine, gr->chef);
			avail |= LS_CHEF_LINE;
		}
		if (gr->sysop != NULL)
		{
			strCpy(sLine, gr->sysop);
			avail |= LS_SYSOP_LINE;
		}
		if (gr->followup != NULL)
		{
			strCpy(rLine, gr->followup);
			avail |= LS_FOLLOWUP_LINE;
		}
		if (gr->herkunft != NULL)
		{
			strCpy(hLine, gr->herkunft);
			avail |= LS_HERKUNFT;
		}
		if (gr->sprache != NULL)
		{
			strCpy(pLine, gr->sprache);
			avail |= LS_SPRACHE;
		}
		if (gr->alias != NULL)
		{
			strCpy(aLine, gr->alias);
			avail |= LS_ALIAS;
		}
		if (!MakeEntry(gr, gName, nName, uLine, cLine, fLine, sLine, rLine, hLine, pLine, aLine, gr->readNumber, gr->catNumber, gr->ordered, gr->orderDays, FALSE, avail))
		{
			return FALSE;
		}
		listModified = TRUE;
		if (saveGruppenInf)
		{
			/* Jetzt die geaenderte GRUPPEN.INF speichern */
			GroupSelect_SaveGruppenInf();
		}
		/* Jetzt das Array neu sortieren! */
		Alias_Setnames(groupArray, numGroups);
		GroupSelect_SortGroupArray();
	}
	return found;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL CAT_GROUP_NUMBER GroupSelect_FreeCatNum(_VOID)						*/
/*	Gibt die kleinste nicht belegte catMumber zurck							*/
/*	Parameter:	keine															*/
/*	Ergebnis:	Nummer															*/		
/*	29.7.01 von Dimitri Junker	DYJ 4.24l										*/
/*** ------------------------------------------------------------------------ ***/

GLOBAL CAT_GROUP_NUMBER GroupSelect_FreeCatNum(_VOID)
{
	GROUPNR idx;
	CAT_GROUP_NUMBER i;
	
	for (i = 1; findNumberEntry(i, &idx); i++);
	return i;
}
/*** ---------------------------------------------------------------------- ***/

/* Neuen Gruppennamen uebernehmen, meckert selber, falls es nicht klappt */
GLOBAL _BOOL GroupSelect_AppendGroup(CONST _UBYTE *groupName)
{
	group *gr;
	_BOOL found;
	_WORD catNums;
	lineSet linesAvail;
	CAT_GROUP_NUMBER lastReadNum;
	GROUPNR idx;
	group newGroup;
	group_line e;

	catNums= GroupSelect_FreeCatNum();
	lastReadNum = 0;
	found = findEntry(groupName, &idx);
	if(found)
	{
		gr = &groupArray[idx];

		gr->catNumber = catNums;
		gr->readNumber = lastReadNum + 1;
	}
	else
	{
		if(idx==dataSys_empty)
			idx=numGroups;
		else
			idx++;
		/* neuen Eintrag anlegen */
		if (!MakeNewGroupArray(FALSE, 0))
			return FALSE;

		gr = &newGroup;
		empty_group(gr);

		/* Hier jetzt einfuegen, da er hierhin passt! */
		linesAvail = LS_GROUP_NAME | LS_CAT_NUMBER | LS_READ_NUMBER | LS_ORDERED;
		*e = '\0';
		if (!MakeEntry(gr, groupName, e, e, e, e, e, e, e, e, e, lastReadNum + 1, catNums, oNA, 0, TRUE, linesAvail))
			return FALSE;
		/* Alle nachfolgenden verschieben */
		if(numGroups>idx)
		{
			memmove(&groupArray[idx+1],&groupArray[idx],sizeof(group)*(numGroups-idx));
			moveCatNr2gNr(idx);
		}
		/* Eintrag jetzt uebernehmen */
		groupArray[idx] = *gr;
		++numGroups;
	}
	setCatNr2gNr(idx,catNums);

	/* Jetzt die Lesenummern anpassen */
	for (idx = 0, gr = groupArray; idx < numGroups; idx++, gr++)
	{
		if (gr->catNumber <(CAT_GROUP_NUMBER)dataSys_notSaved && gr->readNumber > lastReadNum && gr->catNumber != catNums)
			++gr->readNumber;
	}
	listModified = TRUE;
	Alias_Setnames(groupArray, numGroups);
	/* Jetzt Namen an GRUPPEN.INF anhaengen, dort immer gross geschrieben */
	strBcpy(e, groupName);
	strUpr(e);
	return GroupSelect_NameToFile(e, DataPath, catGroupInf);
}

/*** ---------------------------------------------------------------------- ***/

/* Gruppennummer zu einem String feststellen */
/* Wenn FALSE, dann kommt Gruppenanzahl in nr zurueck, Anzahl      */
/* inklusive persoenliche                                         */
/*DYJ 5.21b */
GLOBAL _BOOL GroupSelect_GroupNumber(CONST _UBYTE *gName, CAT_GROUP_NUMBER *nr)
{
	_BOOL found;
	GROUPNR idx;
	CAT_GROUP_NUMBER max;
	found = FALSE;
	if (gName != NULL)
	{
		if (strCaseCmp(gName, Rsc_WrkString(GRP_PRIVATE)) == 0)
		{
			*nr = dataSys_private;
			return TRUE;
		}
		found = findEntry(gName, &idx);
		if (found)
		{
			if (groupArray[idx].catNumber>=(CAT_GROUP_NUMBER)dataSys_notSaved)
			{
				found = FALSE;
			} else
			{
				*nr = groupArray[idx].catNumber;
				return TRUE;
			}
		}
	}
	if (!found)
	{
		*nr = GroupSelect_CountCatGroupsMax(&max) + 1;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert die Zahl der Gruppen in CAT zurueck, inklusive der persoenlichen Gruppe
 */
GLOBAL CAT_GROUP_NUMBER GroupSelect_CountCatGroups(_VOID)
{
	CAT_GROUP_NUMBER dummy;
	return GroupSelect_CountCatGroupsMax(&dummy) + 1;
}

/*** ---------------------------------------------------------------------- ***/

/* Nummer der naechsten Gruppe in der Reihenfolge der momentanen Sortierung
 * feststellen; NO_GROUP falls keine weitere Gruppe vorhanden ist
 */

GLOBAL CAT_GROUP_NUMBER GroupSelect_NextGroupNumber(CAT_GROUP_NUMBER old)
{
	GROUPNR idx;
	CAT_GROUP_NUMBER new;

	new = NO_GROUP;
	if (old != dataSys_private)
	{
		if (findNumberEntry(old, &idx))
			old = groupArray[idx].readNumber;
	}
	if (findReadNumberEntry(old+1, &idx))
	{
		if (groupArray[idx].catNumber<(CAT_GROUP_NUMBER)dataSys_notSaved)
			new = groupArray[idx].catNumber;
	}
	return new;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL CAT_GROUP_NUMBER GroupSelect_NextPrivGroupNumber(CAT_GROUP_NUMBER catNumber)	*/
/*	Gibt die nchste PM-Gruppe zurck											*/
/*	Parameter:	catNumber:	Cat-Nummer der Gruppe ab der gesucht werden soll.	*/
/*	z.B. das Resultat des vorigen Aufrufs										*/
/*	Ergebnis:	Cat-Nummer der nchsten PM-Gruppe oder NO_GROUP					*/		
/*	18.7.01 von Dimitri Junker	DYJ 4.24f										*/
/*** ------------------------------------------------------------------------ ***/

GLOBAL CAT_GROUP_NUMBER GroupSelect_NextPrivGroupNumber(CAT_GROUP_NUMBER catNumber)
{

	GROUPNR idx;
	CAT_GROUP_NUMBER readNumber;

	if (catNumber != dataSys_private)
	{
		if (findNumberEntry(catNumber, &idx))
			readNumber = groupArray[idx].readNumber;
	}
	else
		readNumber= dataSys_private;
	do
	{
		catNumber = NO_GROUP;
		if (findReadNumberEntry(++readNumber, &idx))
		{
			if (groupArray[idx].catNumber<(CAT_GROUP_NUMBER)dataSys_notSaved)
				catNumber = groupArray[idx].catNumber;
		}
		else
			break;
	}while(groupArray[idx].isNet != nPrivate);

	return catNumber;
}
/*** ------------------------------------------------------------------------ ***/
/*	LOBAL _BOOL GroupSelect_isPrivate(CONST CAT_GROUP_NUMBER catNumber)		*/
/*	gibt zurck ob eine Gruppe eine PM-Gruppe ist, also entweder Persnliche	*/
/*	oder eine Pseudogruppe fr PMs												*/
/*	Parameter:	catNumber:	Cat-Nummer der Gruppe								*/
/*	Ergebnis:	TRUE falls PM													*/		
/*	18.7.01 von Dimitri Junker	DYJ 4.24f										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL GroupSelect_isPrivate(CONST CAT_GROUP_NUMBER catNumber)
{
	GROUPNR idx;
	if(catNumber== dataSys_private)
		return TRUE;
	else
	{
		if (findNumberEntry(catNumber, &idx))
			return groupArray[idx].isNet == nPrivate;
	}
	return FALSE;	/* sollte nicht vorkommen*/

}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL GroupSelect_isPseudoOM(CONST CAT_GROUP_NUMBER catNumber)		*/
/*	gibt zurck ob eine Gruppe eine PseudoNews-Gruppe ist, 						*/
/*	Parameter:	catNumber:	Cat-Nummer der Gruppe								*/
/*	Ergebnis:	TRUE falls Pseudo-OM											*/		
/*	28.2.04 von Dimitri Junker	DYJ 5.34d										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL GroupSelect_isPseudoOM(CONST CAT_GROUP_NUMBER catNumber)
{
	GROUPNR idx;
	if(catNumber== dataSys_private)
		return FALSE;
	else
	{
		if (findNumberEntry(catNumber, &idx))
			return groupArray[idx].isNet == nOeffentl;
	}
	return FALSE;	/* sollte nicht vorkommen*/

}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID GroupSelect_setPseudo(CONST CAT_GROUP_NUMBER catNumber,_BOOL private)		*/
/*	gibt zurck ob eine Gruppe eine PseudoNews-Gruppe ist, 						*/
/*	Parameter:	catNumber:	Cat-Nummer der Gruppe								*/
/*				private:	PM oder M-Pseudogruppe?							*/
/*	Ergebnis:	Tkeines															*/		
/*	28.2.04 von Dimitri Junker	DYJ 5.34d										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID GroupSelect_setPseudo(CONST CAT_GROUP_NUMBER catNumber,_BOOL private)
{
	GROUPNR idx;
	if(catNumber!= dataSys_private)
	{
		if (findNumberEntry(catNumber, &idx))
		{
			if(private)
				groupArray[idx].isNet = nPrivate;
			else
				groupArray[idx].isNet = nOeffentl;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Nummer der vorhergehenden Gruppe in der Reihenfolge der momentanen Sortierung */
/* feststellen; 0FFFF == NO_GROUP falls keine weitere Gruppe vorhanden ist      */

GLOBAL CAT_GROUP_NUMBER GroupSelect_PreviousGroupNumber(CAT_GROUP_NUMBER old)
{
	CAT_GROUP_NUMBER new;
	GROUPNR idx;

	new = NO_GROUP;
	if (old != dataSys_private)
	{
		if (findNumberEntry(old, &idx))
			old = groupArray[idx].readNumber;
	} else
	{
		return NO_GROUP;
	}
	if (findReadNumberEntry(old-1, &idx))
	{
		if (groupArray[idx].catNumber<(CAT_GROUP_NUMBER)dataSys_notSaved)
			new = groupArray[idx].catNumber;
	}
	return new;
}

/*** ---------------------------------------------------------------------- ***/

/* Gruppennummer zu einem String mit einem Netzgruppennamen feststellen */
/* Wenn FALSE, dann kommt Gruppenanzahl in nr zurueck, Anzahl      */
/* inklusive persoenliche                                          */
/*DYJ 5.21b */
GLOBAL _BOOL GroupSelect_NetAliasGroupNumber(CONST _UBYTE *gName, CAT_GROUP_NUMBER *nr)
{
	_BOOL found;
	GROUPNR idx;
	CAT_GROUP_NUMBER dummy;

	if (strCaseCmp(gName, Rsc_WrkString(GRP_PRIVATE)) == 0)
	{
		*nr = dataSys_private;
		return TRUE;
	}
	found = findNetAliasEntry(gName, &idx);
	if (found)
	{
		if (groupArray[idx].catNumber>=(CAT_GROUP_NUMBER)dataSys_notSaved)
		{
			found = FALSE;
		} else
		{
			*nr = groupArray[idx].catNumber;
			return TRUE;
		}
	}
	if (!found)
		*nr = GroupSelect_CountCatGroupsMax(&dummy) + 1;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Gruppenname zu einer Nummer feststellen, liefert im Fehlerfall */
/* keinen Eintrag */
/*DYJ 5.21b */
GLOBAL _VOID GroupSelect_GroupName(CAT_GROUP_NUMBER nr, _UBYTE *name, size_t namesize)
{
	GROUPNR idx;

	*name = '\0';
	if (nr == dataSys_private)
	{
		strMcpy(name, namesize, Rsc_WrkString(GRP_PRIVATE));
	} else if (nr != NO_GROUP)
	{
		if (findNumberEntry(nr, &idx))
		{
			if (groupArray[idx].name != NULL)
				strMcpy(name, namesize, groupArray[idx].name);
		}
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID GroupSelect_2GroupNames(CAT_GROUP_NUMBER nr, _UBYTE *name1, _UBYTE *name2, size_t namesize)		*/
/*	Macht das unmgliche, es gibt 2 Namen zu einer catNumber zurck. Das geht	*/
/*	natrlich nur im Fehlerfall.												*/
/*	Parameter:	nr:			gesuchte catNumber									*/
/*				name1,name2:fr die Rckgabe									*/
/*				namesize:	Mindestplatz in name1 und name2						*/
/*	Ergebnis:	nur indirekt ber die beiden Pointer							*/		
/*	31.7.01 von Dimitri Junker	DYJ 4.24l										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.21b */
GLOBAL _VOID GroupSelect_2GroupNames(CAT_GROUP_NUMBER nr, _UBYTE *name1, _UBYTE *name2, size_t namesize)
{
	GROUPNR i;
	_BOOL first=TRUE;
	*name1=*name2 = '\0';
	if (nr == dataSys_private)
	{
		strMcpy(name1, namesize, Rsc_WrkString(GRP_PRIVATE));
	} else if (nr != NO_GROUP)
	{
		for (i = 0; i < numGroups; i++)
		{
			if ((CAT_GROUP_NUMBER)groupArray[i].catNumber == nr)
			{
				if (groupArray[i].name != NULL)
				{
					if(first)
					{
						strMcpy(name1, namesize, groupArray[i].name);
						first=FALSE;
					}
					else
					{
						strMcpy(name2, namesize, groupArray[i].name);
						break;
					}
				}	
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Gruppenname zu einer Nummer feststellen, liefert im Fehlerfall */
/* keinen Eintrag */
/*DYJ 5.21b */
GLOBAL _VOID GroupSelect_GroupDisplayName(CAT_GROUP_NUMBER nr, _UBYTE *name, size_t namesize)
{
	GROUPNR idx;

	*name = '\0';
	if (nr == dataSys_private)
	{
		strMcpy(name, namesize, Rsc_WrkString(GRP_PRIVATE));
	} else if (nr != NO_GROUP)
	{
		if (findNumberEntry(nr, &idx))
		{
			if (groupArray[idx].display_name != NULL)
				strMcpy(name, namesize, groupArray[idx].display_name);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert bei TRUE in str einen String mit +gname oder -gname,
 * je nachdem, ob eine Gruppe angestellt oder abgestellt werden soll
 */
/*DYJ 5.26c */
GLOBAL _BOOL GroupSelect_GetGroupChange(_UBYTE *str, gcMode mode)
{
	group *gr;
	static GROUPNR groupChangeIdx;

	*str = '\0';
	if (mode == GFIRST)
		groupChangeIdx = 0;
	else
		++groupChangeIdx;
	while (groupChangeIdx < numGroups)
	{
		gr = &groupArray[groupChangeIdx];
		if (gr->ordered != oNA && gr->name != NULL)
		{
			if (gr->ordered == oOn)
				*str++ = '+';
			else
				*str++ = '-';
			strCpy(str, gr->name);
			return TRUE;
		}
		++groupChangeIdx;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Loescht eine Gruppenaenderung fuer die Gruppe name
 */
GLOBAL _VOID GroupSelect_ClearGroupChange(CONST _UBYTE *name)
{
	GROUPNR idx;

	if (findEntry(name, &idx))
	{
		if (groupArray[idx].ordered == oOn)
		{
			groupArray[idx].readState = rwOn;
		} else if (groupArray[idx].ordered == oOff)
		{
			groupArray[idx].readState = rwPossible;
		}
		groupArray[idx].ordered = oNA;
		groupArray[idx].orderDays = 0;
		listModified = TRUE;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt einen Parameter fuer die Gruppe gr
 */
/*DYJ 5.35c Start*/
GLOBAL _VOID GroupSelect_SetGrpPar(group *gr, CONST _UBYTE *par,lineSet set)
/*DYJ  Ende; alt:	
GLOBAL _VOID GroupSelect_SetFollowUp(group *gr, CONST _UBYTE *name)
*/
{
	group_line gName;
	group_line nName;
	group_line uLine;
	group_line cLine;
	group_line fLine;
	group_line sLine;
	group_line rLine;
	group_line hLine;
	group_line aLine;
	group_line pLine;
	lineSet avail;

	*gName = '\0';
	*nName = '\0';
	*uLine = '\0';
	*cLine = '\0';
	*fLine = '\0';
	*sLine = '\0';
	*rLine = '\0';
	*hLine = '\0';
	*pLine = '\0';
	*aLine = '\0';
/*DYJ 5.35c Start*/
	avail = LS_CAT_NUMBER | LS_READ_NUMBER | LS_ORDERED | LS_ORDER_DAYS;
/*DYJ  Ende; alt:	
	avail = LS_FOLLOWUP_LINE | LS_CAT_NUMBER | LS_READ_NUMBER | LS_ORDERED | LS_ORDER_DAYS;
*/
	if (gr->name != NULL)
	{
		strBcpy(gName, gr->name);
		avail |= LS_GROUP_NAME;
	}
	if (gr->netname != NULL)
	{
		/* Netname zuweisen */
		strBcpy(nName, gr->netname);
		avail |= LS_NET_LINE;
	}
	if (gr->kurztext != NULL)
	{
		strBcpy(uLine, gr->kurztext);
		avail |= LS_KURZTEXT;
	}
	if (gr->chef != NULL)
	{
		strBcpy(cLine, gr->chef);
		avail |= LS_CHEF_LINE;
	}
	if (gr->sysop != NULL)
	{
		strBcpy(sLine, gr->sysop);
		avail |= LS_SYSOP_LINE;
	}
	if (gr->herkunft != NULL)
	{
		strBcpy(hLine, gr->herkunft);
		avail |= LS_HERKUNFT;
	}
	if (gr->sprache != NULL)
	{
		strBcpy(pLine, gr->sprache);
		avail |= LS_SPRACHE;
	}
	if (gr->alias != NULL)
	{
		strBcpy(aLine, gr->alias);
		avail |= LS_ALIAS;
	}
/*DYJ 5.35c Start*/
	if (gr->followup != NULL)
	{
		strBcpy(rLine, gr->followup);
		avail |= LS_FOLLOWUP_LINE;
	}
	if (par != NULL && *par!='\0')
		avail |= set;
	else
		avail &= ~set;
	switch(set)
	{
		case LS_GROUP_NAME:
			strBcpy(gName, par);
			break;		
		case LS_NET_LINE:
			strBcpy(nName, par);
			break;		
		case LS_KURZTEXT:
			strBcpy(uLine, par);
			break;		
		case LS_CHEF_LINE:
			strBcpy(cLine, par);
			break;		
		case LS_SYSOP_LINE:
			strBcpy(sLine, par);
			break;		
		case LS_HERKUNFT:
			strBcpy(hLine, par);
			break;		
		case LS_SPRACHE:
			strBcpy(pLine, par);
			break;		
		case LS_ALIAS:
			strBcpy(aLine, par);
			break;		
		case LS_FOLLOWUP_LINE:
			strBcpy(rLine, par);
			break;		
	}
/*DYJ  Ende; alt:	
	if (name != NULL)
		strBcpy(rLine, name);
	else
		rLine[0] = '\0';
*/
	if (!MakeEntry(gr, gName, nName, uLine, cLine, fLine, sLine, rLine, hLine, pLine, aLine, gr->readNumber, gr->catNumber, gr->ordered, gr->orderDays, FALSE, avail))
	{
	}
	Alias_Setnames(groupArray, numGroups);
}
/*DYJ 5.35c Start*/
/*** ------------------------------------------------------------------------ ***/
/*	 _VOID GroupSelect_SetGrpParN(CONST _UBYTE *gName, CONST _UBYTE *fup)		*/
/*	Setzt einen der Parameter fuer die Gruppe group								*/
/*	Parameter:	gName:	Gruppenname												*/
/*				fup:	F'up													*/
/*	Ergebnis:	keines															*/		
/*	28.2.04 von Dimitri Junker	DYJ 5.34d										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID GroupSelect_SetGrpParN(CONST _UBYTE *gName, CONST _UBYTE *par,lineSet set)
{
	GROUPNR idx;

	if (findEntry(gName, &idx))
		GroupSelect_SetGrpPar(&groupArray[idx], par,set);
}
/*DYJ  Ende; alt:	

/*** ------------------------------------------------------------------------ ***/
/*	 _VOID GroupSelect_SetFollowUpN(CONST _UBYTE *gName, CONST _UBYTE *fup)		*/
/*	Setzt das Follow-Up fuer die Gruppe group									*/
/*	Parameter:	gName:	Gruppenname												*/
/*				fup:	F'up													*/
/*	Ergebnis:	keines															*/		
/*	28.2.04 von Dimitri Junker	DYJ 5.34d										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID GroupSelect_SetFollowUpN(CONST _UBYTE *gName, CONST _UBYTE *fup)
{
	GROUPNR idx;

	if (findEntry(gName, &idx))
		GroupSelect_SetFollowUp(&groupArray[idx],fup);
}
/*** ------------------------------------------------------------------------ ***/
/*	 _VOID GroupSelect_SetChefN(CONST _UBYTE *gName, CONST _UBYTE *chef)		*/
/*	Setzt den Gruppenchef														*/
/*	Parameter:	gName:	Gruppenname												*/
/*				chef:	Chef													*/
/*	Ergebnis:	keines															*/		
/*	28.2.04 von Dimitri Junker	DYJ 5.34d										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID GroupSelect_SetChefN(CONST _UBYTE *gName, CONST _UBYTE *chef)
{
	GROUPNR idx;

	if (findEntry(gName, &idx))
	{
		group *gr=&groupArray[idx];
		group_line gName;
		group_line nName;
		group_line uLine;
		group_line cLine;
		group_line fLine;
		group_line sLine;
		group_line rLine;
		group_line hLine;
		group_line aLine;
		group_line pLine;
		lineSet avail;
	
		*gName = '\0';
		*nName = '\0';
		*uLine = '\0';
		*cLine = '\0';
		*fLine = '\0';
		*sLine = '\0';
		*rLine = '\0';
		*hLine = '\0';
		*pLine = '\0';
		*aLine = '\0';
		avail = LS_FOLLOWUP_LINE | LS_CAT_NUMBER | LS_READ_NUMBER | LS_ORDERED | LS_ORDER_DAYS;
		if (gr->name != NULL)
		{
			strBcpy(gName, gr->name);
			avail |= LS_GROUP_NAME;
		}
		if (gr->netname != NULL)
		{
			/* Netname zuweisen */
			strBcpy(nName, gr->netname);
			avail |= LS_NET_LINE;
		}
		if (gr->kurztext != NULL)
		{
			strBcpy(uLine, gr->kurztext);
			avail |= LS_KURZTEXT;
		}
		if (gr->followup != NULL)
		{
			strBcpy(rLine, gr->followup);
			avail |= LS_FOLLOWUP_LINE;
		}
		if (gr->sysop != NULL)
		{
			strBcpy(sLine, gr->sysop);
			avail |= LS_SYSOP_LINE;
		}
		if (gr->herkunft != NULL)
		{
			strBcpy(hLine, gr->herkunft);
			avail |= LS_HERKUNFT;
		}
		if (gr->sprache != NULL)
		{
			strBcpy(pLine, gr->sprache);
			avail |= LS_SPRACHE;
		}
		if (gr->alias != NULL)
		{
			strBcpy(aLine, gr->alias);
			avail |= LS_ALIAS;
		}
		if (chef != NULL)
			strBcpy(cLine, chef);
		else
			cLine[0] = '\0';
		if (!MakeEntry(gr, gName, nName, uLine, cLine, fLine, sLine, rLine, hLine, pLine, aLine, gr->readNumber, gr->catNumber, gr->ordered, gr->orderDays, FALSE, avail))
		{
		}
		Alias_Setnames(groupArray, numGroups);
	}
}
*/

/*** ---------------------------------------------------------------------- ***/

/* Liefert das Followup fuer eine Gruppe
 */
GLOBAL _BOOL GroupSelect_GetFollowUp(CONST _UBYTE *gName, _UBYTE *followupTo)
{
	GROUPNR idx;

	if (findEntry(gName, &idx))
	{
		if (groupArray[idx].followup != NULL)
		{
			strCpy(followupTo, groupArray[idx].followup);
			return TRUE;
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert zu einer Gruppe den Netzgruppennamen
 */
GLOBAL _VOID GroupSelect_GetNetName(_UBYTE *name)
{
	GROUPNR idx;

	if (findEntry(name, &idx))
	{
		if (groupArray[idx].netname != NULL)
		{
			strCpy(name, groupArray[idx].netname);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert TRUE zurueck, falls die Gruppe existiert und der
 * Benutzer laut ITG/YGA schreiben darf
 */
GLOBAL _BOOL GroupSelect_GetWriteAccess(CONST _UBYTE *name)
{
	GROUPNR idx;

	if (findEntry(name, &idx))
	{
		return groupArray[idx].writeState == rwNA ||
			groupArray[idx].writeState == rwOn ||
			groupArray[idx].writeState == rwPossible;
	}
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Funktionen fuer CatPutz: */

/* Loescht die Gruppe aus der Liste der in CAT verfuegbaren Gruppen und
 * passt die Lesenummern an. Die Filenummern fuer die Datenbank
 * muessen anders angepasst werden (z.B. mit SetNewCatNumber).
 * Ebenso muss das Gruppen.Pos extern angepasst werden.
 */
GLOBAL _VOID GroupSelect_DeleteGroup(CONST _UBYTE *gName)
{
	GROUPNR idx,i;	
	CAT_GROUP_NUMBER readnum;	

	if (findEntry(gName, &idx))
	{
		readnum = groupArray[idx].readNumber;
		groupArray[idx].readNumber =(CAT_GROUP_NUMBER)dataSys_notSaved;
		groupArray[idx].catNumber =(CAT_GROUP_NUMBER)dataSys_notSaved;
		listModified = TRUE;
		for (i = 0; i < numGroups; i++)
		{
			if (groupArray[i].readNumber > readnum &&
				groupArray[i].readNumber <(CAT_GROUP_NUMBER)dataSys_notSaved)
				--groupArray[i].readNumber;
		}
	}
	buildCatNr2gNr();	/*DYJ 5.36d */
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt eine neue Gruppennummer fuer eine Gruppe.
 * Das GRUPPEN.INF wird danach neu geschrieben
 */
GLOBAL _VOID GroupSelect_SetNewCatNumber(CONST _UBYTE *gName, CAT_GROUP_NUMBER newNumber)
{
	GROUPNR idx;

	if (findEntry(gName, &idx))
	{
		setCatNr2gNr(dataSys_notSaved,groupArray[idx].catNumber);
		setCatNr2gNr(idx,newNumber);
		groupArray[idx].catNumber = newNumber;
		listModified = TRUE;
		/* Jetzt neues GRUPPEN.INF schreiben */
		GroupSelect_SaveGruppenInf();
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL GroupSelect_tstCatNr(_VOID)									*/
/*	Testet ob die Gruppennummerierung OK ist									*/
/*	Parameter:	keine															*/
/*	Ergebnis:	alles OK?														*/		
/*	29.7.01 von Dimitri Junker	DYJ 4.24l										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL GroupSelect_tstCatNr(_VOID)
{
	CAT_GROUP_NUMBER anz,max;
	_BOOL ok=TRUE;
	
	anz=GroupSelect_CountCatGroupsMax(&max);
	if(max!=anz || GroupSelect_FreeCatNum()<anz)
		ok=FALSE;
	return ok;

}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID GroupSelect_makeFilenames(CAT_GROUP_NUMBER group, _BOOL tmpDir, DB_NAMES *dbf,_UBYTE *tmpPath)	*/
/*	hie frher make_filenames und war in putzgrou.c. DB_NAMES neu eingefhrt	*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	31.7.01 von Dimitri Junker	DYJ 4.24l										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.20zf */
GLOBAL _VOID GroupSelect_makeFilenames(CAT_GROUP_NUMBER group, _BOOL tmpDir, DB_NAMES *dbf,_UBYTE *tmpPath)
{
	DB_NAMES tmp;

	data_Number2Name(group, tmp.datName);
	strCpy(tmp.parName, tmp.datName);
	strCpy(tmp.tabName, tmp.datName);
	strCpy(tmp.waiName, tmp.datName);
	if (tmpDir)
	{
		strCat(tmp.datName, ".da");
		strCat(tmp.parName, ".pa");
		strCat(tmp.tabName, ".ta");
		strCpy(dbf->datName, tmpPath);
		F_Path_Append(dbf->datName, tmp.datName);
		strCpy(dbf->parName, tmpPath);
		F_Path_Append(dbf->parName, tmp.parName);
		strCpy(dbf->tabName, tmpPath);
		F_Path_Append(dbf->tabName, tmp.tabName);
		strCat(tmp.waiName, ".wa");
		strCpy(dbf->waiName, tmpPath);
		F_Path_Append(dbf->waiName, tmp.waiName);
	} else
	{
		strCat(tmp.datName, ".dat");
		strCat(tmp.parName, ".par");
		strCat(tmp.tabName, ".tab");
		strCpy(dbf->datName, DataPath);
		F_Path_Append(dbf->datName, tmp.datName);
		strCpy(dbf->parName, DataPath);
		F_Path_Append(dbf->parName, tmp.parName);
		strCpy(dbf->tabName, DataPath);
		F_Path_Append(dbf->tabName, tmp.tabName);
		strCat(tmp.waiName, ".wai");
		strCpy(dbf->waiName, DataPath);
		F_Path_Append(dbf->waiName, tmp.waiName);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID GroupSelect_Exit(_VOID)
{
	FreeGroupArray();
}
