/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



/*
 * 5.4.04 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.35o'
 *	Da war noch einiges in deutsch
 *
 * 9.5.04 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.36l'
 *	Bei Pseudogruppen und im Internetnodus kann der Gruppenchef
 *	auch wieder gelscht werden.
 */
 
#include <gruppena.h>
#include <maustaus.h>
#include <cattypes.h>
#include <catfiles.h>
#include <cathelp.h>
#include <catgloba.h>
#include <mte.h>
#include <grintool.h>
#include <listdl.h>
#include <messages.h>
#include <mtpaths.h>
#include <confvars.h>
#include <varname4.h>
#include <infofile.h>
#include <vdiutil.h>
#include <portvdi.h>
#include <alias.h>

#include <mtappl.h>
#include <mtutils.h>
#include <mtpopups.h>

#include <routinec.h>
#include <routines.h>
#include <file_io.h>
#include <rectfunc.h>
#include <userinfo.h>		/*DYJ 5.19e */
#include <adr.h>		/*DYJ 5.34d */
#include <listhelp.h>		/*DYJ 5.35c */


/*============================================================================
 * Modul:               GruppenAuswahl
 * Autor:               Johannes Goettker-Schnetmann
 * Beschreibung:        Dialoghandling fuer Auswahl der angestellten Gruppen
 *                      Wird gelesen aus kurzer Gruppenliste (IGK.TXT)
 * Version:             2.0
 * letzte Aenderung:    26.7.93
 * ============================================================================
 * Autor Datum     Version  Aenderung
 *  DS   10.6.91   1.2      groe Teile neu geschrieben. Gruppen werden jetzt
 *                          sortiert angezeigt. Umgestellt auf mtTextfiles.
 *                          Teilweise Code auch gestrafft!
 *  DS   25.10.91  1.2x     Anpassung an neues Format von IGK mit Erlaeuterungszeilen
 *                          am Ende.
 *  DS   26.07.93  2.0      Komplett neu geschrieben fuer ITG und interne Gruppenliste
 */




LOCAL List grList;
LOCAL OBJECT *groups;


#define CR 0x0d
#define LF 0x0a

#define ASK 0

#if ASK
#define AL_IS_UNORDERED         "[2][%P:|Diese Gruppe ist bereits abgestellt.|Doch nicht abstellen?][[Ja|:[Nein]"
#define AL_IS_ORDERED           "[2][%P:|Diese Gruppe ist bereits angestellt.|Doch nicht bestellen?][[Ja|:[Nein]"
#endif


typedef struct {
	specialButHdler specials[10];
	_WORD maxGroupNameLength;
	_BOOL isSysop;
	_WORD globalLength;
	_VOID *lastSel;
	group_line Wegen;
	group_line Adressat;
/*DYJ 5.35c Start*/
	netState isNet;
/*DYJ  Ende; alt:	
	_BOOL followUpSelect;
*/
	group *followUpEntry;
	_BOOL buttonOrder;
} GruppenAuswahl;



/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL selGrEntry(_VOID *entry, _VOID *env, _WORD line)
{
	group *gr;
	GRECT r;
	
	UNUSED(env);
	gr = entry;
	if (gr != NULL)
	{
		gr->selected = !gr->selected;
		if (gr->selected && line >= 0)
		{
			SetObjcString(groups, GRNAME, gr->display_name);
			WinDials_WinDialDraw(groups, GRNAME, 0, &r, FALSE);
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID groupToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)
{
	group *gr;
	
	UNUSED(env);
	*str = '\0';
	gr = entry;
	if (gr != NULL)
		strMcpy(str, strsize, gr->display_name);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findSel(_VOID *e, _VOID *i)
{
	group *gr;
	
	UNUSED(i);
	gr = e;
	return gr != NULL && gr->selected;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
/*DYJ 5.30f Start*/
LOCAL _VOID orderGroup(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID orderGroup(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	group *gr;
	GRECT r;
	GruppenAuswahl *auswahl;
	
	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(info);
	auswahl = env;
	*exit = FALSE;
	*draw = TRUE;
	Lists_ResetList(&grList);
	if (Lists_ScanEntries(&grList, Lists_forward, findSel, NULL))
	{
		/* Gruppe an oder abbestellen oder PM-Gruppenstatus ndern*/
		gr = Lists_CurrentEntry(&grList);
		switch (gr->readState)
		{
		case rwOn:
			if (gr->ordered != oNA)
			{
				gr->ordered = oNA;
#if ASK
				if (mtAlerts_Alert(1, AL_IS_UNORDERED) == 1)
					gr->ordered = oNA;
				else
					*draw = FALSE;
#endif
			} else
			{
				gr->ordered = oOff;
			}
			break;
		case rwNA:
		case rwPossible:
			if (gr->ordered != oNA)
			{
				gr->ordered = oNA;
#if ASK
				if (mtAlerts_Alert(1, AL_IS_ORDERED) == 1)
					gr->ordered = oNA;
				else
					*draw = FALSE;
#endif
			} else
			{
				gr->ordered = oOn;
			}
			break;
		case rwAskChef:
			if (mtAlerts_AlertRSC(1, AL_ORDER_CHEF) == 1)
			{
				strBcpy(auswahl->Adressat, "Chef ");
				strBcat(auswahl->Adressat, gr->name);
				strBcpy(auswahl->Wegen, "Gruppe ");
				strBcat(auswahl->Wegen, gr->name);
				*exit = TRUE;
				*draw = FALSE;
			}
			break;
		case rwAskSysop:
			if (mtAlerts_AlertRSC(1, AL_ORDER_SYSOP) == 1)
			{
				strBcpy(auswahl->Adressat, "Sysop");
				strBcpy(auswahl->Wegen, "Gruppe ");
				strBcat(auswahl->Wegen, gr->name);
				*exit = TRUE;
				*draw = FALSE;
			}
			break;
		case rwImpossible:
			MTE_infoRSC(AL_ORDER_IMPOSSIBLE);
			*draw = FALSE;
			break;
		case rwDefault:
			MTE_infoRSC(AL_UNORDER_IMPOSSIBLE);
			*draw = FALSE;
			break;
		case rwNoPermission:
			MTE_infoRSC(AL_ORDER_NO_PERMISSION);
			*draw = FALSE;
			break;
		}
	}
	DelObjState(groups, obj, SELECTED);
	WinDials_WinDialDraw(groups, obj, 1, &r, FALSE);
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.36l Start*/
/*DYJ  Ende; alt:	
/*DYJ 5.21b */
/*DYJ 5.30f Start*/
LOCAL _VOID doChef(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID doChef(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	_UBYTE cmd[256];
	_UBYTE uName[256];
	_WORD but;
	FILENO out;
	OBJECT *wegen;
	group *gr;
	GRECT r;
	_UBYTE fname[PATH_MAX];
#define title "Gruppenchef setzen"
#define cname "Chef:"	/* :language: */

	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(env);
	UNUSED(info);
	/* Neuen Dialog aufmachen und nach Usernamen fragen
	 */
	Lists_ResetList(&grList);
	if (Lists_ScanEntries(&grList, Lists_forward, findSel, NULL))
	{
		/* Followup fuer Gruppe einstellen */
		gr = Lists_CurrentEntry(&grList);
		
		wegen = TreeAddr[WEGEN];
		SetObjcStringAdr(wegen, WGRUPPE, gr->name, sizeof(gr->name));
		SetObjcStringAdr(wegen, OPTSTR,  title, strLen(title) + 1);
		SetObjcStringAdr(wegen, PRESTR,  cname, strLen(cname) + 1);
		SetObjcString(wegen, WEGENSTR, "");
		but = VDIUtil_FormHandle(wegen);
		if (but == WEGENOK)
		{
			ObjcString(wegen, WEGENSTR, uName, sizeof(uName));
			if (uName[0] == '\0')
			{
				MTE_infoRSC(AL_NO_CHEF_SELECTED);
			} else
			{
				/* User als Chef setzen */
				strBcpy(fname, MessagePath);
				F_Path_Append(fname, grSelect);
				out = CatFiles_OpenFile(fname, CatFiles_writeFile);
				if (out < 0)
				{
					MTE_infoRSC(MTE_changeNoSave);
				} else
				{
					CatFiles_Seek(0L, out, CatFiles_end);
					/* Kommandostring bauen */
					strBcpy(cmd, ":GU");
					strBcat(cmd, uName);
					strBcat(cmd, "=");
					strBcat(cmd, gr->name);
					CatFiles_WriteMuch(strLen(cmd), out, cmd);
					CatFiles_WriteFile(CR, out);
					CatFiles_WriteFile(LF, out);
					CatFiles_CloseFile(out);
				}
			}
		}
	}
	DelObjState(groups, obj, SELECTED);
	WinDials_WinDialDraw(groups, obj, 1, &r, FALSE);
	*draw = FALSE;
	*exit = FALSE;
}
*/

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID doHelp(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID doHelp(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;
	
	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(env);
	UNUSED(info);
	CatHelp_DoHelp(CatHelp_selectGroup);
	DelObjState(groups, obj, SELECTED);
	WinDials_WinDialDraw(groups, obj, 1, &r, FALSE);
	*draw = FALSE;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID switchSort(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID switchSort(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	OBJECT *typeSel;
	OBJECT *tree;
	_WORD x, y;
	_WORD select;
	selSortType sort;
	selDispType disp;
	_WORD but;
	GRECT r;
	
	UNUSED(info);

	/* Liste aktualisieren */
	GroupSelect_DeleteList(&grList);
	*lstSel=-1;	/*DYJ 5.30f */
	tree = groups;
	typeSel = TreeAddr[SORTTYPE];
	switch (obj)
	{
	case GRSORT0:
	case GRSORT1:
		but = GRSORT1;
		objc_offset(tree, but, &x, &y);
		select = VDIUtil_GetXState(tree, but);
		select = mtPopups_TreePopup(typeSel, x, y, select) - 1;
		if (select >= 0)
		{
			VDIUtil_SetXState(tree, but, select);
			VDIUtil_SetXString(tree, but, typeSel, select);
			WinDials_WinDialDraw(tree, but, 0, &r, FALSE);
		}
		break;
	case GRSORT2:
		select = VDIUtil_GetXState(tree, GRSORT1);
/*DYJ 5.35c Start*/
		select = (select + 1) % 4;
/*DYJ  Ende; alt:	
		select = (select + 1) % 3;
*/
		VDIUtil_SetXState(tree, GRSORT1, select);
		VDIUtil_SetXString(tree, GRSORT1, typeSel, select);
		WinDials_WinDialDraw(tree, GRSORT1, 0, &r, FALSE);
		break;
	}
	sort = (selSortType)VDIUtil_GetXState(tree, GRSORT1);
	ConfVars_SetConfigInt(cGroupSelectSortType, sort);
	
	disp = (selDispType)VDIUtil_GetXState(tree, GRDISP1);
	GroupSelect_GetSelectList(&grList, disp, sort);

	grList.user = env;
	*draw = TRUE;
	*exit = FALSE;
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _WORD calcLength(GruppenAuswahl *auswahl)								*/
/*	berechnet die maximale Lnge der anzuzeigenden Gruppennamen. Dies war frher*/
/*	Teil der Routine GruppenAuswahl_DoGroupSelect, da es aber jetzt auch noch	*/
/*	an anderer Stelle bentigt wurde habe ich es rausgelst.					*/
/*	Parameter:	auswahl:	Liste der anzuzeigenden Gruppen						*/
/*	Ergebnis:	die maximale Lnge												*/		
/*	1.7.01 von Dimitri Junker	DYJ 4.24c										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _WORD calcLength(GruppenAuswahl *auswahl)
{
	group *group;
	_WORD extent[8];
	_WORD l;
	_WORD maxLength;
	GRECT r;
	/* Maximale Breite eines Gruppennamens herausfinden */
	auswahl->maxGroupNameLength = 0;
	Lists_ResetList(&grList);
	while ((group = Lists_NextEntry(&grList)) != NULL)
	{
		vqt_extent(PrivateWS, group->display_name, extent);
		l = extent[2] - extent[0];
		if (l > auswahl->maxGroupNameLength)
			auswahl->maxGroupNameLength = l;
	}
	
	/* Maximale Breite eines Listeneintrags herausfinden */
	maxLength = 0;
	Lists_ResetList(&grList);
	while ((group = Lists_NextEntry(&grList)) != NULL)
	{
		if (group->kurztext != NULL)
		{
			vqt_extent(PrivateWS, group->kurztext, extent);
			l = extent[2] - extent[0];
			if (l > maxLength)
				maxLength = l;
		}
	}
	maxLength += 6 * CharWidth + auswahl->maxGroupNameLength;
	
	/* Jetzt das Object zusammenbauen */
	CalcArea(groups, GROUPBOX, &r);
	maxLength = Max(maxLength, r.g_w);
	
	auswahl->globalLength = maxLength;
	return 	maxLength;

}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID switchDisp(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID switchDisp(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	selSortType sort;
	selDispType disp;
	OBJECT *typeSel;
	OBJECT *tree;
	_WORD x, y;
	_WORD select;
	_WORD but;
	GRECT r;
	
	UNUSED(info);

	sort = (selSortType)VDIUtil_GetXState(groups, GRSORT1);
	/* Liste aktualisieren */
	GroupSelect_DeleteList(&grList);
	*lstSel=-1;	/*DYJ 5.30f */
	
	tree = groups;
/*DYJ 5.30h Start*/
	if(UserBLK.isInternet)
		typeSel = TreeAddr[DISPTYPEI];
	else
		typeSel = TreeAddr[DISPTYPEM];
/*DYJ  Ende; alt:	
	typeSel = TreeAddr[DISPTYPE];
*/
	switch (obj)
 	{
	case GRDISP0:
	case GRDISP1:
		but = GRDISP1;
		objc_offset(tree, but, &x, &y);
		select = VDIUtil_GetXState(tree, but);
		select = mtPopups_TreePopup(typeSel, x, y, select) - 1;
		if (select >= 0)
		{
			VDIUtil_SetXState(tree, but, select);
			VDIUtil_SetXString(tree, but, typeSel, select);
			WinDials_WinDialDraw(tree, but, 0, &r, FALSE);
		}
		break;
	case GRDISP2:
		select = VDIUtil_GetXState(tree, GRDISP1);
/*DYJ 5.35c Start*/
		select = (select + 1) % 5;
/*DYJ  Ende; alt:	
		select = (select + 1) % 4;
*/
		VDIUtil_SetXState(tree, GRDISP1, select);
		VDIUtil_SetXString(tree, GRDISP1, typeSel, select);
		WinDials_WinDialDraw(tree, GRDISP1, 0, &r, FALSE);
		break;
 	}
	disp = (selDispType)VDIUtil_GetXState(groups, GRDISP1);
	ConfVars_SetConfigInt(cGroupSelectDispType, disp);
	GroupSelect_GetSelectList(&grList, disp, sort);

	grList.user = env;
	calcLength(grList.user);
	*draw = TRUE;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID drawBack(_WORD obj)
{
	GRECT r;

	CalcArea(groups, obj, &r);
	r.g_w += 2;
	r.g_h += 2;
	WinDials_WinDialDraw(groups, ROOT, MAX_DEPTH, &r, TRUE);
}

/*** ---------------------------------------------------------------------- ***/
/*** ------------------------------------------------------------------------ ***/
/*	BOOL followUpTo(netState isNet,group_line followupTo)						*/
/* 	Followup fuer Gruppe einstellen												*/
/*	Parameter:	isNet:	Gruppenart												*/
/*				followupTo: fr Rckgabe										*/
/*	Ergebnis:	OK?															*/		
/*	1.4..04 von Dimitri Junker	DYJ 5.35c										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _BOOL followUpTo(netState isNet,group_line followupTo)
{
	_BOOL newFollowup;
	group_line newFup;
	strCpy(newFup,followupTo);
	if (isNet==nOeffentl)
		newFollowup=adr_SelectName(ADR_MLIST,newFup, sizeof(newFup));
	else if (isNet==nPrivate)
		newFollowup=adr_SelectName(ADR_STDTO,newFup, sizeof(newFup));
	else
	{
		CAT_GROUP_NUMBER ca;
		newFollowup=ListHelp_SelectGroup(newFup, sizeof(newFup), &ca, TRUE, FALSE, FALSE, gsmFollowup);
	}
	if (newFollowup)
		strCpy(followupTo,newFup);
	return newFollowup;
}

/*DYJ 5.35c Start*/
/*DYJ  Ende; alt:	
/*DYJ 5.21b */
/*DYJ 5.30f Start*/
LOCAL _VOID followUpTo(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID followUpTo(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	group *gr;
	_BOOL newFollowup;
	GRECT r;
	_WORD select;
	GruppenAuswahl *auswahl;
	
	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(info);
	auswahl = env;
	*exit = FALSE;
	*draw = FALSE;
	Lists_ResetList(&grList);
	if (Lists_ScanEntries(&grList, Lists_forward, findSel, NULL))
	{
		/* Followup fuer Gruppe einstellen */
		gr = Lists_CurrentEntry(&grList);
		if (gr->followup != NULL)
		{
			select = mtAlerts_AlertRSC(2, AL_FOLLOWUP_DEFINED);
			newFollowup = select == 2;
			if (select == 1)
			{
				/* Followup loeschen */
				GroupSelect_SetFollowUp(gr, "");
				*draw = TRUE;
			}
		} else
		{
			select = mtAlerts_AlertRSC(1, AL_NO_FOLLOWUP);
			newFollowup = select == 1;
		}
/*DYJ 5.34d Start*/
		if (newFollowup && gr->isNet==nOeffentl)
		{
			group_line followupTo;
			if (adr_SelectName(ADR_MLIST,followupTo, sizeof(followupTo)))
				GroupSelect_SetFollowUp(gr, followupTo);
		}
		else if (newFollowup)
/*DYJ  Ende; alt:	
		if (newFollowup)
*/
		{
			/* Neuer Modus! */
			/* Jetzt die beiden Buttons verstecken */
			EnableObjFlags(groups, GRORDER, HIDETREE, TRUE);
			EnableObjFlags(groups, GRCHEF, HIDETREE, TRUE);
			EnableObjFlags(groups, GRFOLLOW, HIDETREE, TRUE);
			drawBack(GRCHEF);
			drawBack(GRORDER);
			drawBack(GRFOLLOW);
			/* Neuen Titel setzen */
			SetObjcString(groups, GRTITLE, MausTauschrsc_WrkString(cSelFollowup));
			WinDials_WinDialDraw(groups, GRTITLE, MAX_DEPTH, &r, FALSE);
			WinDials_WinUpdateTitle(groups);
			MTE_infoRSC(AL_SEL_FOLLOWUP);
			auswahl->followUpSelect = TRUE;
			auswahl->followUpEntry = gr;
		}
	}
	DelObjState(groups, obj, SELECTED);
	WinDials_WinDialDraw(groups, obj, 1, &r, FALSE);
}
*/

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ExtraktValue(_UBYTE *dest, size_t destsize, CONST _UBYTE *source)
{
	CONST _UBYTE *p;
	
	*dest = '\0';
	p = strChr(source, ':');
	if (p != NULL)
		strMcpy(dest, destsize, p + 1);
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
LOCAL _VOID updateButton(_VOID *entry, _VOID *env)
{
	group *gr;
	group_line str;
	group_line str2;
	GRECT r1, r2;
	_BOOL grIsOn;
	selSortType sort;
	GruppenAuswahl *auswahl;
	
	UNUSED(entry);
	auswahl = env;
	Lists_ResetList(&grList);
	if (Lists_ScanEntries(&grList, Lists_forward, findSel, NULL))
	{
		/* Die Textzeilen unten mit Zusatzinfos fuellen */
		gr = Lists_CurrentEntry(&grList);
		/* Erste Zeile: Textinfos */
		str[0] = '\0';
		if (gr->chef != NULL)
		{
			strBcat(str, "Chef: ");/* :language: */
			strBcat(str, gr->chef);
			strBcat(str, "  ");
		}
		if (gr->netname != NULL)
		{
/*DYJ 5.35o Start*/
			strBcat(str, MausTauschrsc_WrkString(NETNAME));
/*DYJ  Ende; alt:	
			strBcat(str, "Netzname: ");
*/
			strBcat(str, gr->netname);
			strBcat(str, "  ");
		}
		if (gr->followup != NULL)
		{
			strBcat(str, "Follow-Up: ");
			strBcat(str, gr->followup);
			strBcat(str, "  ");
		}
		if (gr->alias != NULL)
		{
			if (Alias_ShowAlias(gr->name,TRUE))
			{
/*DYJ 5.35o Start*/
				strBcat(str, MausTauschrsc_WrkString(GRUPPE));
/*DYJ  Ende; alt:	
				strBcat(str, "Gruppe: ");
*/
				strBcat(str, gr->name);
			} else
			{
				strBcat(str, "Alias: ");/* :language: */
				strBcat(str, gr->alias);
			}
			strBcat(str, "  ");
		}
		sort = (selSortType)VDIUtil_GetXState(groups, GRSORT1);
		if (sort == sNet)
		{
			if (gr->herkunft != NULL)
			{
/*DYJ 5.35o Start*/
				strBcat(str, MausTauschrsc_WrkString(NETZ));
/*DYJ  Ende; alt:	
				strBcat(str, "Netz: ");
*/
				ExtraktValue(str2, sizeof(str2), gr->herkunft);
				strBcat(str, str2);
			}
		} else if (sort == sLang)
		{
			if (gr->sprache != NULL)
			{
/*DYJ 5.35o Start*/
				strBcat(str, MausTauschrsc_WrkString(SPRACHE));
/*DYJ  Ende; alt:	
				strBcat(str, "Sprache: ");
*/
				ExtraktValue(str2, sizeof(str2), gr->sprache);
				strBcat(str, str2);
			}
		}
		SetObjcString(groups, GROUPINF1, str);
		
		/* Zweite Zeile: Flags auswerten */
		str[0] = '\0';
		if (gr->isNet != nNetForbid)
		{
			if (gr->readState != rwNA)
			{
				strBcat(str, "Lesen: ");
				switch (gr->readState)
				{
/*DYJ 5.35o Start*/
					case rwOn          : strBcat(str, MausTauschrsc_WrkString(ISTAN)); break;
					case rwPossible    : strBcat(str, MausTauschrsc_WrkString(ANSTELLBAR)); break;
					case rwAskChef     : strBcat(str, MausTauschrsc_WrkString(CHEFFRAGEN)); break;
					case rwAskSysop    : strBcat(str, MausTauschrsc_WrkString(SYSOPFRAGEN)); break;
					case rwImpossible  : strBcat(str, MausTauschrsc_WrkString(UNMOEGLICH)); break;
					case rwDefault     : strBcat(str, MausTauschrsc_WrkString(IMMERAN)); break;
					case rwNoPermission: strBcat(str, MausTauschrsc_WrkString(NICHTERLAUBT)); break;
/*DYJ  Ende; alt:	
					case rwOn          : strBcat(str, "ist an"); break;
					case rwPossible    : strBcat(str, "anstellbar"); break;
					case rwAskChef     : strBcat(str, "Chef fragen"); break;
					case rwAskSysop    : strBcat(str, "Sysop fragen"); break;
					case rwImpossible  : strBcat(str, "unm\224glich"); break;
					case rwDefault     : strBcat(str, "immer an"); break;
					case rwNoPermission: strBcat(str, "nicht erlaubt"); break;
*/
					default: break;
				}
				strBcat(str, "  ");
			}
			if (gr->writeState != rwNA)
			{
				if (gr->writeState != rwNoPermission)
				{
					strBcat(str, "Schreiben: ");
				}
				switch (gr->writeState)
				{
/*DYJ 5.35o Start*/
					case rwOn        : strBcat(str, MausTauschrsc_WrkString(ISTAN)); break;
					case rwPossible  : strBcat(str, MausTauschrsc_WrkString(ANSTELLBAR)); break;
					case rwAskChef   : strBcat(str, MausTauschrsc_WrkString(CHEFFRAGEN)); break;
					case rwAskSysop  : strBcat(str, MausTauschrsc_WrkString(SYSOPFRAGEN)); break;
					case rwImpossible: strBcat(str, MausTauschrsc_WrkString(UNMOEGLICH)); break;
/*DYJ  Ende; alt:	
					case rwOn        : strBcat(str, "ist m\224glich"); break;
					case rwPossible  : strBcat(str, "anstellbar"); break;
					case rwAskChef   : strBcat(str, "Chef fragen"); break;
					case rwAskSysop  : strBcat(str, "Sysop fragen"); break;
					case rwImpossible: strBcat(str, "unm\224glich"); break;
*/
					default: break;
				}
				strBcat(str, "  ");
			}
		}
		if (gr->isNet != nNA)
		{
			switch (gr->isNet)
			{
				case nNetLokal :
/*DYJ 5.35o Start*/
				case nNet      : strBcat(str, MausTauschrsc_WrkString(VERNETZT)); break;
				case nLokal    : strBcat(str, "Lokal"); break;	/* :language: */
				case nNetForbid: strBcat(str, MausTauschrsc_WrkString(NICHTBEST)); break;
/*DYJ  Ende; alt:	
				case nNet      : strBcat(str, "Vernetzt"); break;
				case nLokal    : strBcat(str, "Lokal"); break;
				case nNetForbid: strBcat(str, "Diese Gruppe ist f\201r diese Box nicht bestellbar."); break;
*/
				default: break;
			}
			strBcat(str, "  ");
		}
		SetObjcString(groups, GROUPINF2, str);
		
		/* Button richtig setzen */
/*DYJ 5.35c Start*/
		if (GetObjState(groups, GRORDER, DISABLED))
		{
			EnableObjState(groups, GRORDER, DISABLED, FALSE);
			WinDials_WinDialDraw(groups, GRORDER, 0, &r1, FALSE);
		}
		if(gr->catNumber==(CAT_GROUP_NUMBER)dataSys_notSaved)
			EnableObjState(groups, GR_CHANGE, DISABLED,TRUE);
		else
			EnableObjState(groups, GR_CHANGE, DISABLED,FALSE);
		WinDials_WinDialDraw(groups, GR_CHANGE, 0, &r1, FALSE);
		grIsOn = (gr->readState == rwOn || gr->readState == rwDefault || gr->ordered == oOn) && !(gr->ordered == oOff);
		if (!grIsOn && auswahl->buttonOrder)
		{
			/* Buttontext auf bestellen setzen */
			SetObjcString(groups, GRORDER, MausTauschrsc_WrkString(BUT_ORDER));
			auswahl->buttonOrder = FALSE;
			WinDials_WinDialDraw(groups, GRORDER, 0, &r1, FALSE);
		} else if (grIsOn && !auswahl->buttonOrder)
		{
			/* Buttontext auf Abstellen setzen */
			SetObjcString(groups, GRORDER, MausTauschrsc_WrkString(BUT_SWOFF));
			auswahl->buttonOrder = TRUE;
			WinDials_WinDialDraw(groups, GRORDER, 0, &r1, FALSE);
		}
/*DYJ  Ende; alt:	
		if (!auswahl->followUpSelect && GetObjState(groups, GRORDER, DISABLED))
		{
			EnableObjState(groups, GRORDER, DISABLED, FALSE);
			EnableObjState(groups, GRFOLLOW, DISABLED, FALSE);
			WinDials_WinDialDraw(groups, GRCHEF, 0, &r1, FALSE);
			WinDials_WinDialDraw(groups, GRORDER, 0, &r1, FALSE);
			WinDials_WinDialDraw(groups, GRFOLLOW, 0, &r1, FALSE);
		} else if (auswahl->followUpSelect && !GetObjState(groups, GRORDER, DISABLED))
		{
			EnableObjState(groups, GRCHEF, DISABLED, TRUE);
			EnableObjState(groups, GRORDER, DISABLED, TRUE);
			EnableObjState(groups, GRFOLLOW, DISABLED, TRUE);
			WinDials_WinDialDraw(groups, GRCHEF, 0, &r1, FALSE);
			WinDials_WinDialDraw(groups, GRORDER, 0, &r1, FALSE);
			WinDials_WinDialDraw(groups, GRFOLLOW, 0, &r1, FALSE);
		}
		if (!auswahl->followUpSelect)
		{
			grIsOn = (gr->readState == rwOn || gr->readState == rwDefault || gr->ordered == oOn) && !(gr->ordered == oOff);
			if (!grIsOn && auswahl->buttonOrder)
			{
				/* Buttontext auf bestellen setzen */
				SetObjcString(groups, GRORDER, MausTauschrsc_WrkString(BUT_ORDER));
				auswahl->buttonOrder = FALSE;
				WinDials_WinDialDraw(groups, GRORDER, 0, &r1, FALSE);
			} else if (grIsOn && !auswahl->buttonOrder)
			{
				/* Buttontext auf Abstellen setzen */
				SetObjcString(groups, GRORDER, MausTauschrsc_WrkString(BUT_SWOFF));
				auswahl->buttonOrder = TRUE;
				WinDials_WinDialDraw(groups, GRORDER, 0, &r1, FALSE);
			}
		}
*/
		/* Und jetzt alles zeichnen */
		CalcArea(groups, GROUPINF1, &r1);
		CalcArea(groups, GROUPINF2, &r2);
		rc_union(&r2, &r1);
		WinDials_WinDialDraw(groups, ROOT, MAX_DEPTH, &r1, TRUE);
	} else
	{
		/* Textzeilen unten loeschen */
		str[0] = '\0';
		SetObjcString(groups, GROUPINF1, str);
		SetObjcString(groups, GROUPINF2, str);
		/* Buttons disablen */
		EnableObjState(groups, GRORDER, DISABLED, TRUE);
/*DYJ 5.35c Start*/
		EnableObjState(groups, GR_CHANGE, DISABLED, TRUE);
/*DYJ  Ende; alt:	
		EnableObjState(groups, GRFOLLOW, DISABLED, TRUE);
		EnableObjState(groups, GRCHEF, DISABLED, TRUE);
*/
		/* Und jetzt alles zeichnen */
		CalcArea(groups, GROUPINF1, &r1);
		CalcArea(groups, GROUPINF2, &r2);
		rc_union(&r2, &r1);
		WinDials_WinDialDraw(groups, ROOT, MAX_DEPTH, &r1, TRUE);
		WinDials_WinDialDraw(groups, GRORDER, 0, &r1, FALSE);
/*DYJ 5.35c Start*/
		WinDials_WinDialDraw(groups, GR_CHANGE, 0, &r1, FALSE);
/*DYJ  Ende; alt:	
		WinDials_WinDialDraw(groups, GRFOLLOW, 0, &r1, FALSE);
		WinDials_WinDialDraw(groups, GRCHEF, 0, &r1, FALSE);
*/
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* List-Zwischen-Prozeduren */

LOCAL _BOOL groupIsSelected(_VOID *entry, _VOID *env)
{
	group *gr;
	
	UNUSED(env);
	gr = entry;
	return gr != NULL && gr->selected;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isEnabled(_VOID *entry, _VOID *env)
{
	UNUSED(entry);
	UNUSED(env);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *nextEntry(_VOID *list)
{
	return Lists_NextEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *prevEntry(_VOID *list)
{
	return Lists_PrevEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID resetList(_VOID *list)
{
	Lists_ResetList(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID countEntries(_VOID *l, _LONG *ll, _WORD *ww)
{
	List *lp;
	GruppenAuswahl *auswahl;
	
	lp = l;
	auswahl = lp->user;
	*ww = auswahl->globalLength;
	*ll = Lists_NoOfEntries(lp);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID drawGroupEntry(_VOID *entry, _VOID *env, _WORD x, _WORD y, _WORD offset, CONST GRECT *clip)
{
	group *gr;
	GruppenAuswahl *auswahl;
	_WORD extent[8];
	GRECT fill;
	_WORD pxy[4];
	_BOOL isSel;
	_UWORD eff;
	_WORD fillcol;
	
	UNUSED(offset);
	gr = entry;
	auswahl = env;
	
	fill.g_x = x;
	fill.g_y = y;
	fill.g_w = fill.g_x + auswahl->globalLength - 1;
	fill.g_h = fill.g_y + CharHeight - 1;
	pxy[0] = clip->g_x;
	pxy[1] = clip->g_y;
	pxy[2] = clip->g_x + clip->g_w - 1;
	pxy[3] = clip->g_y + clip->g_h - 1;

	vs_clip(PrivateWS, TRUE, pxy);
	vswr_mode(PrivateWS, MD_REPLACE);

	fillcol = WHITE;
	isSel = entry != NULL && groupIsSelected(entry, env);
	if (isSel)
		fillcol = BLACK;
	MouseOff();
	vsf_color(PrivateWS, fillcol);
	vr_recfl(PrivateWS, (_WORD *)&fill);
	
	if (gr != NULL)
	{
		_UBYTE gtc[20]=GROUP_TYPE_CHAR;
		eff = TXT_NORMAL;
		if (!isEnabled(entry, env))
			eff |= TXT_LIGHT;
		if (isSel)
			vswr_mode(PrivateWS, MD_XOR);
		vst_effects(PrivateWS, eff);

		if ((gr->readState == rwOn || gr->readState == rwDefault) && gr->ordered != oOff)
			v_gtext(PrivateWS, x, y, "\010"); /* Check-Haken */ /* TODO */
		else if (gr->ordered == oOff)
			v_gtext(PrivateWS, x, y, "-");
		else if (gr->ordered == oOn)
			v_gtext(PrivateWS, x, y, "+");
		x += 2 * CharWidth;
/*DYJ 5.35b Start*/
		gtc[gr->isNet-nPrivate+1]='\0';
		v_gtext(PrivateWS, x, y, gtc+gr->isNet-nPrivate);
		x += 2 * CharWidth;
/*DYJ  Ende; */

		v_gtext(PrivateWS, x, y, gr->display_name);

		if (gr->followup != NULL)
		{
			vqt_extent(PrivateWS, gr->display_name, extent);
			v_gtext(PrivateWS, x + extent[2] - extent[0] + CharWidth, y, "\003"); /* Pfeil rechts */ /* TODO */
		}
		x += auswahl->maxGroupNameLength + 3 * CharWidth;
		if (gr->kurztext != NULL)
			v_gtext(PrivateWS, x, y, gr->kurztext);
		vst_effects(PrivateWS, 0);
		vswr_mode(PrivateWS, MD_REPLACE);
	}
	vs_clip(PrivateWS, FALSE, pxy);
	MouseOn();
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
LOCAL _VOID setAndGetGroupValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exitBut)
{
	group *group;	/*DYJ 5.35c */
	GruppenAuswahl *auswahl;
	
	UNUSED(tree);
	auswahl = env;
	if (set)
	{
/*DYJ 5.35c Start*/
		Lists_ResetList(&grList);
		while ((group = Lists_NextEntry(&grList)) != NULL)
			group->isUpdated=0;
/*DYJ  Ende; alt:	
		/* heute nix zu tun */
*/
	} else
	{
		if (exitBut != GROUPABR)
			GroupSelect_UpdateSelectList(&grList, TRUE);
		/* Gruppenliste wieder freigeben */
		GroupSelect_DeleteList(&grList);
		/* Dialog wieder freigeben */
		if (exitBut == GRORDER)
		{
			if (grinTools_BetreffHolen(-1, auswahl->Wegen, sizeof(auswahl->Wegen),
				MausTauschrsc_WrkString(STR_PERS_KEYWORD),
				auswahl->Adressat, MausTauschrsc_WrkString(wgStr), FALSE))
			{
				Messages_NewPersMsg(auswahl->Adressat, "", auswahl->Wegen);
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
/*DYJ 5.35c Start*/
/*DYJ  Ende; alt:	
LOCAL _BOOL checkGroupExit(OBJECT *tree, _VOID *env, _WORD button, _VOID *lastSel, _UWORD kState)
{
	group *gr;
	_BOOL error;
	GRECT r;
	GruppenAuswahl *auswahl;
	
	UNUSED(tree);
	UNUSED(lastSel);
	UNUSED(kState);
	auswahl = env;
	error = FALSE;
	if (!auswahl->followUpSelect)
		return TRUE;
	if (button == GROUPOK)
	{
		/* Selektierten Eintrag suchen */
		Lists_ResetList(&grList);
		if (Lists_ScanEntries(&grList, Lists_forward, findSel, NULL))
		{
			/* Eintrag holen */
			gr = Lists_CurrentEntry(&grList);
			/* Jetzt Namen vergleichen */
			if (strCaseCmp(auswahl->followUpEntry->name, gr->name) == 0)
			{
				MTE_infoRSC(AL_SAME_GROUP);
				error = TRUE;
			} else
			{
				/* Followup neu setzen */
				GroupSelect_SetFollowUp(auswahl->followUpEntry, gr->name);
				auswahl->followUpSelect = FALSE;
			}
		} else
		{
			auswahl->followUpSelect = FALSE;
		}
	} else
	{
		auswahl->followUpSelect = FALSE;
	}
	if (!error)
	{
		EnableObjFlags(groups, GRCHEF, HIDETREE, FALSE);
		EnableObjFlags(groups, GRORDER, HIDETREE, FALSE);
		EnableObjFlags(groups, GRFOLLOW, HIDETREE, FALSE);
		SetObjcString(groups, GRTITLE, MausTauschrsc_WrkString(cSelGroup));
		WinDials_WinUpdateTitle(groups);
		DelObjState(groups, button, SELECTED);
		/* Full Redraw ueber Dialog */
		WinDials_WinDialDraw(groups, ROOT, MAX_DEPTH, &r, FALSE);
	}
	return FALSE;
}
*/

/*** ------------------------------------------------------------------------ ***/
/*	_BOOL hdlGroupBut(netState *isNet,_BOOL isSysop,_WORD but,OBJECT *grchange,group *gr,_BOOL *pIsMausGrp)		*/
/*	Reagiert auf Knopfdrcke im GREDIT-Dialog									*/
/*	Parameter:	isNet:	Gruppenart												*/
/*				isSysop:ist der User Sysop?										*/
/*				but:	gedrckter Knopf										*/
/*				grchange: der Dialog											*/
/*				gr:		die Gruppe												*/
/*				pIsMausGrp:Rckgabe ob Mausgruppe								*/
/*	Ergebnis:	OK?																*/		
/*	1.4..04 von Dimitri Junker	DYJ 5.35c										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _BOOL hdlGroupBut(netState *isNet,_BOOL isSysop,_WORD but,OBJECT *grchange,group *gr,_BOOL *pIsMausGrp)
{
	_BOOL ok=FALSE;
	group_line followupTo;
	ObjcString(grchange, GRE_FUP_EDT, followupTo, sizeof(followupTo));
	
	switch(but)
	{
		case GRE_NG:
			*pIsMausGrp=!UserBLK.isInternet;
			SetObjState(grchange, but, SELECTED);
			SetObjcString(grchange, GRE_FUP_TXT, "Follow-Up:");
			if(strChr(followupTo,'@'))
				SetObjcString(grchange, GRE_FUP_EDT, "");
			*isNet=nNA;

			break;
		case GRE_PMG:
			*pIsMausGrp=FALSE;
			SetObjState(grchange, but, SELECTED);
			SetObjcString(grchange, GRE_FUP_TXT, MausTauschrsc_WrkString(stdTo));
			if(!strChr(followupTo,'@'))
				SetObjcString(grchange, GRE_FUP_EDT, "");
			*isNet=nPrivate;
			break;
		case GRE_OMG:
			SetObjcString(grchange, GRE_FUP_TXT, "ML-Server:");
			SetObjState(grchange, but, SELECTED);
			*pIsMausGrp=FALSE;
			if(!strChr(followupTo,'@'))
				SetObjcString(grchange, GRE_FUP_EDT, "");
			*isNet=nOeffentl;
			break;
		case GRE_FUP_EDT:
			followUpTo(*isNet,followupTo);
			SetObjcString(grchange, GRE_FUP_EDT, followupTo);
			break;
		case GRE_OK:
		{
			group_line line;
			CAT_GROUP_NUMBER nr;
			ok=TRUE;
			if(*isNet!=gr->isNet)
			{
				gr->isNet=*isNet;
				gr->isUpdated|=LS_FLAG_LINE;
			}
			if(strCmp(followupTo,gr->followup))
			{
				GroupSelect_SetGrpPar(gr, followupTo,LS_FOLLOWUP_LINE);
				gr->isUpdated|=LS_FOLLOWUP_LINE;
			}
			ObjcString(grchange, GRE_NAME, line, sizeof(line));
			if(strCmp(line,gr->name))
			{
				ok=FALSE;
				if (line[0] == '\0')
					MTE_infoRSC(AL_EMPTY_GROUP);
				else if ( GroupSelect_GroupNumber(line, &nr))
					MTE_infoRSC(AL_DOUBLE_GROUP);
				else
				{
					ok=TRUE;
					if((gr->isUpdated&LS_GROUP_NAME)==0)
					{
						GroupSelect_SetGrpPar(gr,gr->name,LS_NET_LINE);
						gr->isUpdated|=LS_GROUP_NAME;
					}
					GroupSelect_SetGrpPar(gr,line,LS_GROUP_NAME);
				}
			}

			ObjcString(grchange, GRE_TXT, line, sizeof(line));
			if(strCmp(line,gr->kurztext))
			{
				GroupSelect_SetGrpPar(gr, line,LS_KURZTEXT);
				gr->isUpdated|=LS_KURZTEXT;
			}

			ObjcString(grchange, GRE_CHEF, line, sizeof(line));
			if(strCmp(line,gr->chef))
			{
/*DYJ 5.36l Start*/
				if (line[0] == '\0' && *pIsMausGrp)
				{
					MTE_infoRSC(AL_NO_CHEF_SELECTED);
					ok=FALSE;
				} else
				{
					GroupSelect_SetGrpPar(gr, line,LS_CHEF_LINE);
					gr->isUpdated|=LS_CHEF_LINE;
					if(isSysop && *pIsMausGrp)
						gr->isUpdated|=LS_SYSOP_LINE;
				}
/*DYJ  Ende; alt:	
				if (line[0] == '\0')
				{
					MTE_infoRSC(AL_NO_CHEF_SELECTED);
					ok=FALSE;
				} else
				{
					GroupSelect_SetGrpPar(gr, line,LS_CHEF_LINE);
					gr->isUpdated|=LS_CHEF_LINE;
					if(isSysop)
						gr->isUpdated|=LS_SYSOP_LINE;
				}
*/
			}
			break;
		}
		case GRE_CANCEL:
			ok=TRUE;
			break;
	}
	return ok;
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID changeGroup(List *grList,netState *isNet,_BOOL isSysop)				*/
/*	von changeGroup aus listhelp abgeleitet fr den neuen erweiterten			*/
/*	Gruppen-ndern Dialog														*/
/*	Parameter:	grList:	Die Gruppenliste										*/
/*				isNet:	Gruppenart(nderbar)									*/
/*				isSysop: Ist man Sysop															*/
/*	Ergebnis:	keines															*/		
/*	1.4.04 von Dimitri Junker	DYJ 5.35c										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _WORD GruppenAuswahl_ChangeGroup(List *grList,netState *isNet,_BOOL isSysop)
{
	_BOOL oneSel;
	group *gr;
	OBJECT *grchange;
	group_line oldGruppe,oldChef;
	group_line oldTxt;
	_WORD but;
	_BOOL ok;
	_WORD ret = 0;
	Lists_ResetList(grList);
	oneSel = Lists_ScanEntries(grList, Lists_forward, findSel, NULL);
	if (oneSel)
	{
		_BOOL isMausGrp;
		gr = Lists_CurrentEntry(grList);
		strBcpy(oldGruppe, gr->name);
		strBcpy(oldChef, gr->chef);
		grchange = TreeAddr[GREDIT];
		isMausGrp=!UserBLK.isInternet;
		*isNet=gr->isNet;
		switch(gr->isNet)
		{
		case nPrivate:
			DelObjState(grchange, GRE_NG, SELECTED);
			SetObjState(grchange, GRE_PMG, SELECTED);
			DelObjState(grchange, GRE_OMG, SELECTED);
			DelObjState(grchange, GRE_PMG, DISABLED);
			DelObjState(grchange, GRE_OMG, DISABLED);
			isMausGrp=FALSE;
			SetObjcString(grchange, GRE_FUP_TXT, MausTauschrsc_WrkString(stdTo));
			break;
		case nOeffentl:
			DelObjState(grchange, GRE_NG, SELECTED);
			DelObjState(grchange, GRE_PMG, SELECTED);
			SetObjState(grchange, GRE_OMG, SELECTED);
			SetObjcString(grchange, GRE_FUP_TXT, "ML-Server:");
			DelObjState(grchange, GRE_PMG, DISABLED);
			DelObjState(grchange, GRE_OMG, DISABLED);
			isMausGrp=FALSE;
			break;
		case nNA:
			SetObjState(grchange, GRE_NG, SELECTED);
			DelObjState(grchange, GRE_PMG, SELECTED);
			DelObjState(grchange, GRE_OMG, SELECTED);
			DelObjState(grchange, GRE_PMG, DISABLED);
			DelObjState(grchange, GRE_OMG, DISABLED);
			SetObjcString(grchange, GRE_FUP_TXT, "Follow-Up:");
			break;
		default:
			SetObjState(grchange, GRE_NG, SELECTED);
			SetObjState(grchange, GRE_PMG, DISABLED);
			SetObjState(grchange, GRE_OMG, DISABLED);
			SetObjcStringAdr(grchange, OPTSTR, MausTauschrsc_WrkString(GRCHANGE1), strLen(MausTauschrsc_WrkString(GRCHANGE1)) + 1);
			SetObjcStringAdr(grchange, PRESTR, MausTauschrsc_WrkString(GRCHANGE2), strLen(MausTauschrsc_WrkString(GRCHANGE2)) + 1);
			SetObjcString(grchange, GRE_FUP_TXT, "Follow-Up");
		}
		if(!isMausGrp)
		{
			strBcpy(oldTxt, gr->kurztext);
			SetObjcStringAdr(grchange, GRE_TXT, oldTxt, sizeof(oldTxt) + 1);
		}
		SetObjcString(grchange, GRE_FUP_EDT, gr->followup);
		SetObjcStringAdr(grchange, GRE_NAME, oldGruppe, sizeof(oldGruppe));
		SetObjcStringAdr(grchange, GRE_CHEF, oldChef, sizeof(oldChef));
		/* Sysop feststellen */
		EnableObjFlags(grchange, GRE_CHEF, EDITABLE, isSysop||(!isMausGrp));
		do
		{
			but = VDIUtil_FormHandle(grchange);
			ok = hdlGroupBut(isNet,isSysop,but,grchange,gr,&isMausGrp);
		} while (!ok);
		if(but==GRE_OK)
			ret=1;
		SetObjcStringAdr(grchange, WGRUPPE, "", 1);
		if(gr->isNet==nOeffentl&& !strChr(gr->followup,'@'))
		{
			_UBYTE ul[500];
			group_line followupTo="";
			sprintf(ul,MausTauschrsc_WrkString(AL_POM_FU),gr->name);
			if(mtAlerts_Alert(1,ul)==1 && adr_SelectName(ADR_MLIST,followupTo, sizeof(followupTo)))
				GroupSelect_SetGrpPar(gr, followupTo,LS_FOLLOWUP_LINE);
		}
	}
	else
		ret=-1;

	return ret;
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID changeGroup(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)		*/
/*	Ruft i.W. GruppenAuswahl_ChangeGroup auf und erledigt die Sachen die beim 	*/
/*	Aufruf aus Tausch/Gruppen auswhlen anders sind als bei PM-Filter			*/
/*	Parameter:	obj:	Tausch/Gruppen auswhlen-Dialog							*/
/*				env:	Auswahl													*/
/*				info:	unbenutzt												*/
/*				draw:	mu neu gezeichnet werden?								*/
/*				exit:	Tausch/Gruppen auswhlen-Dialog verlassen?				*/
/*				lstSel:	?														*/
/*	Ergebnis:	keines															*/		
/*	10.4.04 von Dimitri Junker	DYJ 5.35c										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _VOID changeGroup(_WORD obj,_VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
{
	GruppenAuswahl *auswahl;
	GRECT r;
	UNUSED(info);
	auswahl = env;
	*lstSel=-1;
	*draw =GruppenAuswahl_ChangeGroup(&grList,&auswahl->isNet,auswahl->isSysop)>0;
	DelObjState(groups, obj, SELECTED);
	WinDials_WinDialDraw(groups, obj, 1, &r, FALSE);
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.19e */	/*DYJ 5.21b */
GLOBAL _VOID GruppenAuswahl_DoGroupSelect(_VOID)
{
	OBJECT *dial;
	_WORD maxLength;
/*DYJ 5.35c Start*/
/*DYJ  Ende; alt:	
	_BOOL oldFsel;
*/
	ldHandler dialHandler;
	_WORD sort;
	_WORD disp;
	OBJECT *sortTypeSel;
	OBJECT *dispTypeSel;
	GruppenAuswahl *auswahl;
	ldElems elems;
	_BOOL autoClear;
	_WORD intval;
	
	dial = TreeAddr[GROUPS];
	auswahl = NEW(GruppenAuswahl, "GruppenAuswahl_DoGroupSelect");
	if (auswahl == NULL)
		return;
		
/*DYJ 5.35c Start*/
/*DYJ  Ende; alt:	
	auswahl->followUpSelect = FALSE;
	auswahl->followUpEntry = NULL;
*/
	auswahl->buttonOrder = FALSE;
	
	/* Baumadresse setzen */
	groups = dial;
	MausTauschrsc_AdjustDialogSize(dial, GROUPS);
	
	/* Sysop feststellen */
	auswahl->isSysop = Infofiles_IsInfoFile(ISB_SHORT_NAME, TRUE);
	/* Titel setzen */
	SetObjcString(groups, GRTITLE, MausTauschrsc_WrkString(cSelGroup));
	/* Dial erstellen */
	
	/* Sortierung einstellen */
	if(UserBLK.isInternet)
		sort=sName;
	else
		ConfVars_GetConfDefInt(cGroupSelectSortType, &sort, sName);
	sortTypeSel = TreeAddr[SORTTYPE];
	VDIUtil_SetXState(groups, GRSORT1, sort);
	VDIUtil_SetXString(groups, GRSORT1, sortTypeSel, sort);
	
	ConfVars_GetConfDefInt(cGroupSelectDispType, &disp, dAll);
/*DYJ 5.30h Start*/
	if(UserBLK.isInternet)
		dispTypeSel = TreeAddr[DISPTYPEI];
	else
		dispTypeSel = TreeAddr[DISPTYPEM];
/*DYJ  Ende; alt:	
	dispTypeSel = TreeAddr[DISPTYPE];
*/
	VDIUtil_SetXState(groups, GRDISP1, disp);
	VDIUtil_SetXString(groups, GRDISP1, dispTypeSel, disp);

	if (!GroupSelect_GetSelectList(&grList, (selDispType)disp, (selSortType)sort))
		return;

	grList.user = auswahl;

	/* Texte unten leeren */
	/* SetObjcString(groups, GRNAME, ""); */
	SetObjcString(groups, GROUPINF1, "");
	SetObjcString(groups, GROUPINF2, "");
	/* specials bestimmen */
	auswahl->specials[0].objc = GRORDER;
	auswahl->specials[0].proc = orderGroup;
/*DYJ 5.35c Start*/
	auswahl->specials[1].objc = GR_CHANGE;
	auswahl->specials[1].proc = changeGroup;
	auswahl->specials[2].objc = GRHELP;
	auswahl->specials[2].proc = doHelp;
	auswahl->specials[3].objc = GRSORT0;
	auswahl->specials[3].proc = switchSort;
	auswahl->specials[4].objc = GRSORT1;
	auswahl->specials[4].proc = switchSort;
	auswahl->specials[5].objc = GRSORT2;
	auswahl->specials[5].proc = switchSort;
	auswahl->specials[6].objc = GRDISP0;				/* (MH) */
	auswahl->specials[6].proc = switchDisp;
	auswahl->specials[7].objc = GRDISP1;
	auswahl->specials[7].proc = switchDisp;
	auswahl->specials[8].objc = GRDISP2;
	auswahl->specials[8].proc = switchDisp;
/*DYJ  Ende; alt:	
	auswahl->specials[1].objc = GRFOLLOW;
	auswahl->specials[1].proc = followUpTo;
	auswahl->specials[2].objc = GRHELP;
	auswahl->specials[2].proc = doHelp;
	auswahl->specials[3].objc = GRCHEF;
	auswahl->specials[3].proc = doChef;
	auswahl->specials[4].objc = GRSORT0;
	auswahl->specials[4].proc = switchSort;
	auswahl->specials[5].objc = GRSORT1;
	auswahl->specials[5].proc = switchSort;
	auswahl->specials[6].objc = GRSORT2;
	auswahl->specials[6].proc = switchSort;
	auswahl->specials[7].objc = GRDISP0;				/* (MH) */
	auswahl->specials[7].proc = switchDisp;
	auswahl->specials[8].objc = GRDISP1;
	auswahl->specials[8].proc = switchDisp;
	auswahl->specials[9].objc = GRDISP2;
	auswahl->specials[9].proc = switchDisp;
*/
	maxLength=calcLength(auswahl);
	
	EnableObjFlags(dial, GRSORTB, HIDETREE, UserBLK.isInternet);	/*DYJ 5.19e */
	EnableObjFlags(dial, GRSORT0, HIDETREE, UserBLK.isInternet);	/*DYJ 5.19e */
	EnableObjFlags(dial, GRSORT1, HIDETREE, UserBLK.isInternet);	/*DYJ 5.19e */
	EnableObjFlags(dial, GRSORT2, HIDETREE, UserBLK.isInternet);	/*DYJ 5.19e */
	
	
	/* WindowDialog vorbereiten */
	ListDl_BuildLdHandler(&grList,
		resetList,
		nextEntry,
		prevEntry,
		countEntries,
		isEnabled,
		selGrEntry,
		groupToStr,
		groupIsSelected,
		drawGroupEntry,
		FUNK_NULL,
		0, CharHeight,
		8, maxLength,
		&dialHandler);
	ConfVars_GetConfDefInt(cLocateGruMode, &intval, 1);
	elems = ldSelect | ldArrows | ldAutolocate | /* ldShrinkList | */ ldModal;
	if(intval) /* sucht irgendwo im Gruppennamen*/
		elems|=ldTabEqNxt|ldMatchAny;
	
	ConfVars_GetConfDefBool(cAutoClearEdit, &autoClear, FALSE);
	if (autoClear)
		 elems |= ldClearEdit;
/*DYJ 5.35c Start*/
	/* ListWinDialog oeffnen */
	ListDl_WinListDial(groups, elems,
		&dialHandler,
		GRORDBOX,
		GRORDBACK,
		GROUPOK,
		GROUPABR,
		auswahl,		/* env */
		0,				/* doubleClickButton */
		GRNAME,			/* autoEdit */
		updateButton,
		auswahl->specials,
		9,
		ListDl_dummyCheckExit,
		setAndGetGroupValues,
		setAndGetGroupValues,
		&auswahl->lastSel);
/*DYJ  Ende; alt:	
	for (;;)
	{
		/* ListWinDialog oeffnen */
		oldFsel = auswahl->followUpSelect;
		if (ListDl_WinListDial(groups, elems,
			&dialHandler,
			GRORDBOX,
			GRORDBACK,
			GROUPOK,
			GROUPABR,
			auswahl,		/* env */
			0,				/* doubleClickButton */
			GRNAME,			/* autoEdit */
			updateButton,
			auswahl->specials,
			10,
			checkGroupExit,
			setAndGetGroupValues,
			setAndGetGroupValues,
			&auswahl->lastSel))
		{
		}	
		if (auswahl->followUpSelect == oldFsel)
			break;
	}
*/
	SFREE(auswahl);
}

