/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <handlepo.h>
#include <data.h>
#include <mte.h>
#include <maustaus.h>
#include <cat.h>

#include <ro_mem.h>


LOCAL List handles;		/* Liste der Zugriffshandles */

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-- Allgemeine Listenverwaltung ---------------*/

/* Ein Listenelement an die angegebene Liste anhaengen */
GLOBAL _VOID *handlePool_BlankToList(_ULONG size, List *list)
{
	_VOID *new;
	
	new = MALLOC(size, "handlePool: BlankToList");
	if (new != NULL)
	{
		if (!Lists_AppendEntry(list, new))
		{
			FREE(new, size);
			new = NULL;
		} else
		{
			MemSetZero(new, size);
		}
	}
	return new;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
GLOBAL _VOID handlePool_FreeOnePtr(_VOID *ptr, List *list)
{
	if (Lists_FindEntry(list, ptr))
	{
		Lists_RemoveEntry(list, mem_free);
	} else
	{
		MTE_infoRSC(AL_LISTERROR1);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL handlePool_FindEntry(_VOID *data, _BOOL (*condProc)(_VOID *, _VOID *), List *list, _VOID **ptr)
{
	Lists_ResetList(list);
	if (Lists_ScanEntries(list, Lists_forward, condProc, data))
	{
		*ptr = Lists_CurrentEntry(list);
		return TRUE;
	}
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-- Spezielle Datenbankhandleliste ------------*/

LOCAL _BOOL find_group(_VOID *e, _VOID *i)
{
	return ((oneHandleElement *)e)->group == *((CAT_GROUP_NUMBER *)i);
}

/*** ---------------------------------------------------------------------- ***/

/* Handle-Liste nach einem Eintrag der Gruppe <gruppe> durchsuchen */
GLOBAL oneHandleElement *handlePool_SearchDataHandle(CAT_GROUP_NUMBER gruppe)
{
	Lists_ResetList(&handles);
	if (Lists_ScanEntries(&handles, Lists_forward, find_group, &gruppe))
		return Lists_CurrentEntry(&handles);
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

/* Erst die bestehende Liste nach schon offenen Handles durchsuchen,
 * falls dort nicht gefunden, versuchen ein neues zu oeffnen
 */
/*DYJ 5.21b */
oneHandleElement *handlePool_GetOneDatahandle(CAT_GROUP_NUMBER gruppe)
{
	oneHandleElement *handle;

	if ((handle = handlePool_SearchDataHandle(gruppe)) != NULL)
	{
		++handle->open; /* jetzt wird es von einem Fenster mehr benutzt */
	} else
	{
		if ((handle = handlePool_BlankToList(sizeof(oneHandleElement), &handles)) != NULL)
		{
			if (!data_OpenOneGroup(gruppe, 0, FALSE, &handle->Zugriff))
			{
				/* Hier interessiert nicht, was beim zurueckschreiben passiert */
				if (Lists_FindEntry(&handles, handle))
					Lists_RemoveEntry(&handles, mem_free);
				else
					MTE_infoRSC(AL_LISTERROR2);	
				handle = NULL;
			} else
			{
				handle->group = gruppe;
				handle->open  = 1;
			}
		}
	}
	return handle;
}

/*** ---------------------------------------------------------------------- ***/

/* Nachsehen, ob er noch benoetigt wird, sonst aus der Liste entfernen */
/*DYJ 5.21b */
GLOBAL _VOID handlePool_FreeOneDataHandle(oneHandleElement *handle)
{
	if (handle != NULL)
	{
		if (handle->open > 1)		/* == wird noch benoetigt, aber von einem weniger */
		{
			--handle->open;
		} else						/* Wird nicht mehr benoetigt */
		{
			data_CloseOneGroup(&handle->Zugriff);
			/* Hier interessiert nicht, was beim zurueckschreiben passiert */
			if (Lists_FindEntry(&handles, handle))
				Lists_RemoveEntry(&handles, mem_free);
			else
				MTE_infoRSC(AL_LISTERROR2);	
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Wird in grin_grinInit aufgerufen, Fehler auch dort angemeckert und braucht */
/* nicht weiter beachtet zu werden */
GLOBAL _BOOL handlePool_InitPool(_VOID)
{
	if (handles.root == NULL)
		return Lists_CreateList(&handles, "handlePool_InitPool");
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID handlePool_ExitPool(_VOID)
{
	Lists_DeleteSimpleList(&handles, mem_free, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Fragt nach, ob _irgendeine_ Gruppe noch geoeffnet ist. Insbesondere fuer   */
/* parser.i -> Gruppe offen: Einfuegen abbrechen und CAT beenden             */
GLOBAL _BOOL handlePool_isOneGroupOpen(_VOID)
{
	return !Lists_ListEmpty(&handles);
}
