/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#define __HASHING2_IMPLEMENTATION__

#include <hashing2.h>
#ifndef __TOS_H__
#include <tos.h>
#endif

#include <ro_mem.h>
#include <cat_vb.h>
#include <datasys.h>


#define hash_empty 0xFFFFFFFFl /* leerer Listenplatz */

	
struct _hInfo {
	MSGNR *list;
	MSGNR entry;
	MSGNR aAnz;
	MSGNR actual;
	CRC16 hashMax; /* entspricht maxTab! */
	_UWORD anzCrc;		/* Anzahl CRCs pro Message*/
	MSGNR hash[1];
};

LOCAL _UWORD PRIM[] = {
	2503, 3001, 3449, 4001, 4507, 5003,
	5501, 6007, 6521, 7001, 7507, 8009,
	9001,   10007,  11003,
	12007,  13001,  14009,
	15013,  16001,  18013,
	20011,  25013,  30011,
	35023U,  40009U,  45007U,
	50021U,  55001U,  60013U,
	65521U
};
#define maxPrim ((_UWORD)(sizeof(PRIM) / sizeof(PRIM[0])))



/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#define H(crc, maxTab) ((crc) % (maxTab))

/* Gibt die Primzahl zurueck, die in diesem Fall genommen werden soll */
LOCAL _UWORD whatsGood(_ULONG anz)
{
	_UWORD z;
	
	z = 0;
	while (z < (maxPrim-1) && anz >= PRIM[z])
		++z;
	return PRIM[z];
}

/*** ---------------------------------------------------------------------- ***/

/* gibt hashHandle zurueck, die hashcrcs werden in einem Array[0..xx] */
/* uebergeben, dessen Anzahl <anz> ist. <add> gibt die Anzahl der     */
/* noch zu erwartenden Nachrichten an                                */
GLOBAL hashHandle Hashing2_ToHash(CRC16 crcAdr[2], MSGNR anz, MSGNR add,_UWORD anzCrc)
{
	_UWORD i;
	CRC16 *crc;
	hashHandle ptr;
	CRC16 prim,hashAnz;
	MSGNR listAnz,z,*list;
	
	listAnz = anz + add;
	listAnz*=anzCrc;
	if (listAnz > 65534UL)
		hashAnz = 65534U;
	else
		hashAnz = (CRC16)listAnz;
	list = SYSMALLOC(listAnz * sizeof(MSGNR), MEM_NOERR, "Hashing2_ToHash: list");
	if (!list)
		return NULL;
	for(i=3;i>0;i--)
	{
		prim = whatsGood(((_ULONG)hashAnz)*i);
		ptr = SYSMALLOC(sizeof(hInfo) + sizeof(MSGNR) * (prim+1), MEM_NOERR, "Hashing2_ToHash: handle");
		if(ptr)
			break;
	}
	if(ptr)
	{
		ptr->list =list;
		ptr->hashMax = prim;
		ptr->aAnz  = listAnz;
		ptr->anzCrc  = anzCrc;
		ptr->entry = 0l;
		ptr->actual = hash_empty;
		crc = crcAdr;
		/* Hashtabelle loeschen und fllen*/
		for (z = 0; z <= prim; z++)
			ptr->hash[z] = hash_empty;
		for (z = 0; z < anz; z++)
			Hashing2_AddCrc(ptr, &crc[z*anzCrc]);
	}
	else
		SFREE(ptr->list);
	return ptr;
}
/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Hashing2_ReleaseHash(hashHandle ptr)
{
	if (ptr != NULL)
	{
		SFREE(ptr->list);
		SFREE(ptr);
	}
}
LOCAL MSGNR FreeEntry(hashHandle ptr, MSGNR start)
{
	while (ptr->list[start] != hash_empty)
		start = ptr->list[start];
	return start;
}

GLOBAL _VOID Hashing2_AddCrc(hashHandle ptr, CRC16 *crc)
{
	CRC16 nr;
	_WORD i;

	if (ptr == NULL)
		return; /* Neu */
	for(i=0;i<ptr->anzCrc;i++)
	{
		if(crc[i]!=(CRC16)dataSys_empty)
		{
			nr = H(crc[i], ptr->hashMax);
			ptr->list[ptr->entry] = hash_empty;
			if (ptr->hash[nr] == hash_empty)
			{
				ptr->hash[nr] = ptr->entry;
				/* Der Eintrag in der Liste ist <hash_empty> */
			} else
			{
				/* Der Eintrag in der Hashtabelle ist dann schon fertig */
				ptr->list[FreeEntry(ptr, ptr->hash[nr])] = ptr->entry;
				/* !!! sollte man da nicht mal auf Ueberlauf pruefen ??? */
			}
		}
		++ptr->entry;
	}
}
/*** ---------------------------------------------------------------------- ***/
GLOBAL _VOID Hashing2_AddCrcNr(hashHandle ptr, CRC16 crc[2],MSGNR nr)
{
	MSGNR old_entry=ptr->entry;
	ptr->entry=nr*ptr->anzCrc;
	Hashing2_AddCrc(ptr, crc);
	ptr->entry=old_entry;
}
/*** ------------------------------------------------------------------------ ***/
/*						*/
/*								*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	3.10.00 von Dimitri Junker	DYJ 4.50a										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID Hashing2_SubCrcNr(hashHandle ptr, CRC16 crc[2],MSGNR eintrNr)
{
	CRC16 nr;
	_WORD i;
	MSGNR start;

	if (ptr == NULL)
		return; /* Neu */
	eintrNr*=ptr->anzCrc;
	for(i=0;i<ptr->anzCrc;i++)
	{
		if(crc[i]!=(CRC16)dataSys_empty)
		{
			nr = H(crc[i], ptr->hashMax);
			if (ptr->hash[nr] == eintrNr)
			{
				ptr->hash[nr] = ptr->list[eintrNr];
				ptr->list[eintrNr] = hash_empty;
			}
			else
			{
				start=ptr->hash[nr];
				while (ptr->list[start] != eintrNr && ptr->list[start] !=hash_empty)
					start = ptr->list[start];
				if(ptr->list[start] == eintrNr)
					ptr->list[start]=ptr->list[ptr->list[start]];
			}
		}
		++eintrNr;
	}
}
/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20t */
GLOBAL MSGNR Hashing2_GetFirst(hashHandle ptr, CRC16 crc)
{
	if(!ptr)
		return hash_empty;

	ptr->actual = ptr->hash[H(crc, ptr->hashMax)];
	if(ptr->actual ==hash_empty)
		return hash_empty;
	else
		return (ptr->actual/ptr->anzCrc);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL MSGNR Hashing2_GetNext(hashHandle ptr)
{
	if (ptr->actual != hash_empty)
		ptr->actual = ptr->list[ptr->actual];
	if(ptr->actual ==hash_empty)
		return hash_empty;
	else
		return (ptr->actual/ptr->anzCrc);
}
/*** ---------------------------------------------------------------------- ***/

GLOBAL hashHandle Hashing2_getEmptyHash(_VOID)
{
	return (hashHandle) NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Hashing2_emptyHash(hashHandle h)
{
	return h == NULL;
}
