/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <mtappl.h>
#include <mtrsc.h>
#include <mtutils.h>
#include <portvdi.h>
#include <mtdials.h>
#include <mtdir.h>
#include <magiccoo.h>
#include <cattypes.h>
#include <rectfunc.h>
#include <ro_mem.h>
#include <wdwmanag.h>
#include <keyboard.h>
#include <filename.h>

#include <file_io.h>
#include <routine.h>
#include <routinec.h>
#include <debug.h>


/* Dies ist ein einfacher Iconviewer, der die einzelnen Dialoge einer Resource
 * in einem Fenster anzeigt, und zwar inklusive Farbicons
 * Der Einfachheit halber benutze ich die cat-module hierzu.
 */


typedef struct {
	pathStrType fileName;
	String255 infoLine;
	RESOURCE rsc;
	_WORD treeIdx;
	_WORD treeCnt;
	HWND wdw;
	HVDI vdi;
} VIEW, *viewPtr;


GLOBAL _UBYTE ProgramName[] = "Iconview";
GLOBAL _BOOL PrgHasDataBase=FALSE;		/*DYJ 5.26a */

LOCAL pathStrType loadPath;
LOCAL nameStrType loadName;


LOCAL _BOOL FselGet(_UBYTE *InPath, _UBYTE *InSel, CONST _UBYTE *ext, CONST _UBYTE *msg, _BOOL pathOnly)
{
	struct {
		_ULONG id;
		_UWORD version;
	} *fsel;
	pathStrType scrapP;
	nameStrType scrapN;
	_BOOL noMCtrl;

	F_Path_Dirname(InPath);
	strBcpy(scrapP, InPath);
	noMCtrl = FindCookie(COOKIE_FSEL, (_LONG *)&fsel) && fsel && fsel->version >= 0x200;
	strBcpy(scrapN, InSel);
	F_Path_Append(InPath, ext);
	if (!noMCtrl)
		Wind_Update(BEG_MCTRL);
	if (mtDir_GetDir(InPath, InSel, msg))
	{
		if (!noMCtrl)
			Wind_Update(END_MCTRL);
		F_Path_Dirname(InPath);
		if (pathOnly)
			return TRUE;
		return *InSel != '\0';
	}
	if (!noMCtrl)
		Wind_Update(END_MCTRL);
	strMcpy(InPath, PATH_MAX, scrapP);
	strMcpy(InSel, PATH_MAX, scrapN);
	return FALSE;
}

#define WithShift(kstate) (kstate & (K_LSHIFT|K_RSHIFT))
#define WithCtrl(kstate) (kstate & K_CTRL)
#define WithAlt(kstate) (kstate & K_ALT)

/* Prototypen */

static _BOOL OpenOneView(CONST _UBYTE *fname);

/* Fensterhandling */

LOCAL _VOID posView(HWND wdw, OBJECT *tree)
{
	GRECT area, pos, work;
	LGRECT doc;
	
	CalcArea(tree, ROOT, &area);
	ObjcArea(tree, ROOT, &pos);
	WdwManager_GetWdwWork(wdw, &work);
	WdwManager_GetWdwDocument(wdw, &doc);
	tree[ROOT].ob_x = (_WORD)(work.g_x + (pos.g_x - area.g_x) - doc.x);
	tree[ROOT].ob_y = (_WORD)(work.g_y + (pos.g_y - area.g_y) - doc.y);
}


LOCAL _BOOL clickInView(HWND wdw, _VOID *special, _WORD mx, _WORD my, _UWORD kstate, _UWORD buttons, _WORD clicks)
{
	/* Wir machen nix mit Buttons */
	UNUSED(wdw);
	UNUSED(special);
	UNUSED(mx);
	UNUSED(my);
	UNUSED(kstate);
	UNUSED(buttons);
	UNUSED(clicks);
	return FALSE;
}

LOCAL _VOID ChangeTree(viewPtr view, _BOOL next)
{
	GRECT work;
	GRECT work2;
	OBJECT *tree;
	LGRECT doc;

	tree = RscGaddr(view->rsc, R_TREE, view->treeIdx);
	mtDials_DisposeDial(tree);
	if (next)
	{
		++view->treeIdx;
		if (view->treeIdx >= view->treeCnt)
			view->treeIdx = 0;
	} else
	{
		--view->treeIdx;
		if (view->treeIdx < 0)
			view->treeIdx = view->treeCnt - 1;
	}
	tree = RscGaddr(view->rsc, R_TREE, view->treeIdx);
	mtDials_NewDial(tree);
	WdwManager_GetWdwWork(view->wdw, &work);
	WdwManager_SetScrollParms(view->wdw, work.g_w - 16, work.g_h - 16, 16, 16,TRUE);
	posView(view->wdw, tree);
	CalcArea(tree, ROOT, &work);
	doc.x = 0;
	doc.y = 0;
	doc.w = work.g_w;
	doc.h = work.g_h;
	WdwManager_GetWdwWork(view->wdw, &work2);
	work2.g_w = work.g_w;
	work2.g_h = work.g_h;
	WdwManager_SetWdwWork(view->wdw, &work2);
	WdwManager_SetNewDocument(view->wdw, &doc, TRUE);
}


LOCAL _BOOL keyView(HWND wdw, _VOID *special, _WORD taste, _UBYTE ch, _UBYTE scan, _UWORD kstate)
{
	_UWORD gemCh;
	Key key;
	_BOOL done;

	UNUSED(taste);
	gemCh = (scan << 8) | ch;
	GemCharToKey(gemCh, kstate, &key);
	done = TRUE;
	if (IsSpecial(&key))
	{
		switch (SpecialKey(&key))
		{
			case left: WdwManager_ScrollLeft(wdw, 1); break;
			case right: WdwManager_ScrollRight(wdw, 1); break;
			case soln: WdwManager_PageLeft(wdw); break;
			case eoln: WdwManager_PageRight(wdw); break;
			case up: WdwManager_ScrollUp(wdw, 1); break;
			case down: WdwManager_ScrollDown(wdw, 1); break;
			case pgUp: WdwManager_PageUp(wdw); break;
			case pgDown: WdwManager_PageDown(wdw); break;
			case ctrlW: WdwManager_TopNextWindow(); break;
			case ctrlO: OpenOneView(""); break;
			case ctrlU: WdwManager_CloseWindow(wdw, FALSE); break;
			case ctrlQ: WdwManager_CloseAllWindows(FALSE); break;
			case wdLeft: ChangeTree(special, FALSE); break;
			case wdRight: ChangeTree(special, TRUE); break;
			default: done = FALSE;
		}
	} else
	{
		done = FALSE;
	}
	return done;
}


LOCAL _BOOL closeView(HWND wdw, HVDI vdi, _VOID *special, _BOOL force)
{
	viewPtr view;
	OBJECT *tree;

	UNUSED(wdw);
	UNUSED(vdi);
	UNUSED(force);
	view = special;
	tree = RscGaddr(view->rsc, R_TREE, view->treeIdx);
	mtDials_DisposeDial(tree);
	RscFree(view->rsc);
	OFREE(view);
	return TRUE;
}


LOCAL _VOID hideView(HWND wdw, _VOID *special, _BOOL hide)
{
	UNUSED(wdw);
	UNUSED(special);
	UNUSED(hide);
}


LOCAL _VOID snapView(HWND wdw, _VOID *special, GRECT *work)
{
	viewPtr view;
	OBJECT *tree;

	UNUSED(work);
	view = special;
	tree = RscGaddr(view->rsc, R_TREE, view->treeIdx);
	posView(wdw, tree);
}


LOCAL _BOOL timerView(HWND wdw, _VOID *special)
{
	UNUSED(wdw);
	UNUSED(special);
	return FALSE;
}


LOCAL _BOOL topView(HWND wdw, _VOID *special)
{
	UNUSED(wdw);
	UNUSED(special);
	return TRUE;
}


LOCAL _VOID untopView(HWND wdw, _VOID *special)
{
	UNUSED(wdw);
	UNUSED(special);
}


LOCAL _VOID setWork(HWND wdw, _VOID *special, CONST LGRECT *doc, _BOOL slided)
{
	GRECT work;

	UNUSED(special);
	UNUSED(doc);
	UNUSED(slided);
	WdwManager_GetWdwWork(wdw, &work);
	WdwManager_SetScrollParms(wdw, work.g_w - 16, work.g_h - 16, 16, 16,TRUE);
}


LOCAL _VOID getRect(HWND wdw, HVDI vdi, _VOID *special, GRECT *work)
{
	UNUSED(wdw);
	UNUSED(vdi);
	UNUSED(special);
	UNUSED(work);
}


LOCAL _VOID redrawView(HWND wdw, HVDI vdi, _VOID *special, CONST GRECT *clip)
{
	viewPtr view;
	OBJECT *tree;
	GRECT work;

	view = special;
	vswr_mode(vdi, MD_REPLACE);
	vsf_style(vdi, IP_SOLID);
	vsf_interior(vdi, FIS_SOLID);
	vsf_color(vdi, WHITE);
	work = *clip;
	AbsRect(&work);
	vr_recfl(vdi, (_WORD *)&work);
	tree = RscGaddr(view->rsc, R_TREE, view->treeIdx);
	posView(wdw, tree);
	mtDials_DialDraw(tree, ROOT, MAX_DEPTH, clip, TRUE);
}


LOCAL _VOID updateView(HWND wdw, _VOID *special, _BOOL begin)
{
	UNUSED(wdw);
	UNUSED(special);
	if (begin)
	{
		Wind_Update(BEG_UPDATE);
		MouseOff();
	} else
	{
		MouseOn();
		Wind_Update(END_UPDATE);
	}
}



LOCAL _BOOL OpenOneRsc(_UBYTE *fullname, RESOURCE *rsc)
{
	if (*fullname != '\0' || FselGet(loadPath, loadName, "*.rsc", "Resource laden", FALSE))
	{
		if (*fullname == '\0')
		{
			strMcpy(fullname, PATH_MAX, loadPath);
			F_Path_Dirname(fullname);
			F_Path_Append(fullname, F_Path_Basename(loadName));
		}
		if ((*rsc = RscLoad(fullname)) != NULL)
			return TRUE;
	}
	return FALSE;
}


LOCAL viewPtr CreateView(_VOID)
{
	viewPtr view;

	if ((view = NEW(VIEW, "CreateView")) != NULL)
	{
		view->fileName[0] = '\0';
		view->infoLine[0] = '\0';
		view->treeIdx = 0;
		view->treeCnt = 0;
		view->wdw = -1;
		view->vdi = -1;
		view->rsc = NULL;
	}
	return view;
}


static _BOOL OpenOneView(CONST _UBYTE *fname)
{
	_UWORD comps;
	GRECT work, work2;
	viewPtr view;
	OBJECT *tree;
	RSXHDR head;
	LGRECT doc;

	if ((view = CreateView()) != NULL)
	{
		if (*fname != '\0')
			strMcpy(view->fileName, sizeof(view->fileName), fname);
		if (OpenOneRsc(view->fileName, &view->rsc))
		{
			/* Anzahl der Baeume rausfinden */
			RscGetHeader(view->rsc, &head);
			view->treeCnt = (_WORD)head.rsh_ntree;
			/* Ausmasse des dargestellten Baumes rausfinden */
			tree = RscGaddr(view->rsc, R_TREE, view->treeIdx);
			mtDials_NewDial(tree);
			CalcArea(tree, ROOT, &work);
			/* noch clippen */
			work.g_x += 40;
			work.g_y += 40;
			rc_intersect(&deskSize, &work);
			comps = NAME | CLOSER | FULLER | MOVER | SIZER | UPARROW | DNARROW | VSLIDE | LFARROW | RTARROW | HSLIDE;
			Wind_calc(WC_BORDER, comps, &work, &work2);
			/* Fenster oeffnen */
			if (WdwManager_OpenWindow(clickInView, keyView, timerView,
				&deskSize, &work2, comps,
				FALSE, view->infoLine, view->fileName, snapView,
				closeView, redrawView, topView,
				untopView, updateView, setWork, getRect,
				hideView, 0, view, FALSE, TRUE, TRUE,
				TRUE,
				FALSE,
				&view->wdw, &view->vdi))
			{
				/* Scrollparameter setzen */
				WdwManager_SetDocumentParms(view->wdw, 1, 1);
				WdwManager_GetWdwWork(view->wdw, &work);
				WdwManager_SetScrollParms(view->wdw, work.g_w - 16, work.g_h - 16, 16, 16,TRUE);
				CalcArea(tree, ROOT, &work);
				doc.x = 0;
				doc.y = 0;
				doc.w = work.g_w - 1;
				doc.h = work.g_h - 1;
				WdwManager_SetWdwDocument(view->wdw, &doc);
				return TRUE;
			}
		}
		OFREE(view);
	}
	return FALSE;
}


int main(int argc, char *const *argv)
{
	_UWORD kstate;
	_WORD taste;
	_UBYTE ch, scan;
	_UWORD evnt;
	_WORD msgbuff[8];
	HWND topWdw;
	_WORD dummy;
	_UWORD udummy;

#if SPEC_DEBUG
	is_DEBUGGING = FALSE;
#endif
	ApplInit();
	mtDials_Init();
	WdwManager_Init();
	WdwManager_EnableIconify(TRUE);
	if (argc > 1)
	{
		while (--argc)
			OpenOneView(*++argv);
	} else
	{
		OpenOneView("");
	}

	topWdw = WdwManager_GetOwnTopWindow();
	while (topWdw >= 0)
	{
		evnt = evnt_multi(MU_KEYBD|MU_MESAG,
			0, 0, 0,
			0, 0, 0, 0, 0,
			0, 0, 0, 0, 0,
			msgbuff,
			0, 0,
			&dummy, &dummy, &udummy, &kstate, &taste, &dummy);
		ch = taste & 0xff;
		scan = (taste >> 8) & 0xff;
		WdwManager_HandleEvent(&evnt, msgbuff, 0, kstate, ch, scan, taste, 0, 0, 0);
		topWdw = WdwManager_GetOwnTopWindow();
	}
	WdwManager_CloseAllWindows(TRUE);
	ApplTerm(0);
	return 0;
}
