#ifndef __CAT_DOITK_H__
#define __CAT_DOITK_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif
#ifndef __CAT_LISTS_H__
#include <lists.h>
#endif
#ifndef __CAT_LISTDL_H__
#include <listdl.h>
#endif
#ifndef __SET_H__
#include <set.h>
#endif

#define cMaxObjs       256              /* Maximale Anzahl der Objekte im Baum */
#define cMaxChars      256				/* Maximale Zeichenzahl pro Zeile */
#define cMaxText        20				/* Maximale Anzahl Zeilen */
#define cMaxButt        11				/* Maximale Anzahl Buttons */
#define cMaxIcon       256				/* Maximale Anzahl Iconslots */
#define cMaxTeds        20				/* Maximale Anzahl Editfelder */
#define cMaxXTeds cMaxTeds				/* Maximale Anzahl lange Editfelder unter MagiC */

typedef _UBYTE tString[cMaxChars];
typedef SET KeySet;

/* MagiC XTED Struktur. Gehoert nach MagicAES.H */
typedef struct
{
	_UBYTE *xte_ptmplt;
	_UBYTE *xte_pvalid;
	_WORD xte_vislen;
	_WORD xte_scroll;
} XTED;

typedef struct _OneCommand OneCommand;
typedef struct _itk ITK;

#ifdef __ITK_IMPLEMENTATION__

typedef enum _tCmdType
{
	tcSingle,							/* TE */
	tcMultiple,							/* TB */
	tcConfig							/* TK */
} tCmdType;

typedef enum _tParamType
{
	tpConst,							/* C */
	tpEnum,								/* FA */	/* Buttons */
	tpEnumSet,							/* FM */	/* Buttons */
	tpMTDate,							/* FD */	/* Editfeld */
	tpDate,								/* Fd */	/* Editfeld */
	tpString,							/* FS */	/* Editfeld */
	tpPassword,							/* FP */	/* Editfeld */
	tpNewPassword,						/* Fp */	/* Editfeld */
	tpUsername,							/* FU */	/* Editfeld, Button */
	tpInt,								/* FI */	/* Editfeld num */
	tpRange,							/* FIm,n */	/* Editfeld num */
	tpGroup,							/* FG */	/* Editfeld, Button */
	tpChiefGroup,						/* Fg */	/* Editfeld, Button */
	tpPtGroup							/* FT|FGT *//* Editfeld, Button */
} tParamType;

struct _OneCommand
{
	_UBYTE cmdName[256];				/* Name des Kommandos */
	_UBYTE *cmdHelp;					/* Hilfetext zum Kommando,
										 * einzelne Zeilen mit CR/LF
										 * getrennt
										 */
	_WORD cmdVersion;					/* Versionsnummer des Kommandos */
	_WORD cmdIdent;						/* Identifikationsnummer des Kommandos */
	tCmdType cmdType;					/* Kommandotyp */
	List cmdParms;						/* Liste der Parameter */
	_UBYTE cmdGroup[8];					/* Gruppe des Kommandos, Sortierkriterium */
	_BOOL hasParms;
	_BOOL selected;						/* Fuer Darstellung in Listbox */
	_BOOL isInInf;						/* Ist das Kommando gespeichert? */
	_BOOL title;						/* Ist Eintrag eine Kommandogruppenueberschrift? (MH) */
	_BOOL groupVisible;					/* Fuer Titeleintrag: Ist Gruppe sichtbar? (MH) */
	_BOOL entryVisible;					/* Fuer Listeintrag: Ist Eintrag sichtbar? (MH) */
	_BOOL macro;					    /* Ist Eintrag ein ITK-Makro? (MH) */
	List *itkMacro;						/* Liste der Makros (MH) */
	ITK *itk;
};

typedef struct _OneParam
{
	_UBYTE paramName[256];
	_UBYTE *paramHelp;
	tParamType paramType;
	List paramEnum;
	_UBYTE paramDflt[256];				/* Default-Wert fuer Parameter; Wert konstanter Parameter */
	_LONG paramMin;						/* Min-Wert fuer Integer-Bereich, ggf. Maxlaenge des Strings */
	_LONG paramMax;						/* Max-Wert fuer Integer-Bereich */
	_BOOL optional;						/* Parameter ist optional */
	_BOOL hasEnums;						/* Hat Enum-Werte */
	_WORD obLow;						/* Erstes Objekt im Dialog */
	_UBYTE paramValue[256];				/* Eingestellter Wert */
	_BOOL hasDefault;
} OneParam;

typedef struct _OneEnum
{
	_UBYTE enumName[256];				/* Kurzbeschreibung des Enum-Parameters */
	_UBYTE enumValue[256];				/* Werte dieses Enum-Parameters */
	_WORD obLow;						/* Erstes Objekt im Dialog */
	_UBYTE enumCurValue[256];			/* Eingestellter Wert */
} OneEnum;

struct _itk
{
	List *itkCmds;
	_WORD globalLength;
	_WORD groupNameLength;
	OBJECT Tree[cMaxObjs];
	_WORD type[cMaxObjs];
	TEDINFO Teds[cMaxTeds];
	XTED XTeds[cMaxXTeds];
	_WORD currObIdx;
	_WORD tedIdx;
	_WORD lastyPos;
	
	_UBYTE title[80];
	KeySet usedKeys;
	
	OBJECT *CmdBox;
	specialButHdler specials[7];
	
	_WORD exitBut;
	_VOID *lastCmd;
	_WORD sort;
	
	_BOOL inCommand;
	_BOOL runMacro;		/* (MH) */
	_WORD macroExitBut;
	List macroListe;

	_BOOL configModus;		
	_BOOL new;
	_BOOL open;
	_BOOL changed;
	_BOOL listChanged;
	_VOID *macro;
};

#else

struct _tCommandGroupName { int dummy; };
struct _tIntCmdType { int dummy; };
struct _OneCommand { int dummy; };
struct _itk { int dummy; };

#endif


_VOID DoITK_InitMacroList(List *cmdList);
/* Makrolisten initialisieren 
 */

List *DoITK_InitCmdList(_VOID);
/* Kommandoliste aus ITK aufbauen
 */

_VOID DoITK_FreeCmdList(List *itkCmds);
/* Kommandoliste wieder freigeben
 */

_VOID DoITK_SortITK(ITK *itk);
/* Sortiert die Kommandoliste
 */

_BOOL DoITK_HandleCommand(OneCommand *pCmd, _VOID *tMacro);
/* Arbeitet das Kommando ab
 */ 

_VOID DoITK_UpdateButtons(_VOID *entry, _VOID *env);
/* Aktualisiert die Buttons im Kommandodialog
 */

_VOID DoITK_RedrawList(ITK *itk);
/* Zeichnet die komplette Liste der Kommandos neu
 */

_BOOL DoITK_SelectCmd(_VOID);
/* Kommando aus ITK auswaehlen und aufrufen
 * Ruft selber InitCmdList und FreeCmdList
 * auf
 */

_BOOL DoITK_GetCmdVersion(List *itkCmds, _WORD cmd, OneCommand **pCmd, _WORD *version);
/* Liefert die aktuelle Version eines ITK-Kommandos (und den Zeiger (MH))
 * InitCmdList muss vorher aufgerufen werden,
 * und wenn die Daten nicht mehr benoetigt werden, dann FreeCmdList
 */

_BOOL DoITK_TransformCommands(_BOOL deleteOld);

#endif /* __CAT_DOITK_H__ */
