#ifndef __CAT_MSGLIST_H__
#define __CAT_MSGLIST_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */




#ifndef __CAT_DATA_H__
#include <data.h>
#endif

#ifndef __CAT_GRIN_H__
#include <grin.h>
#endif

#ifndef __CAT_CATFILES_H__
#include <catfiles.h>
#endif

extern sortType MsgListLstSort;

_VOID msgList_listOpen(CAT_GROUP_NUMBER gruppe, MSGNR which, HWND grinWindow, grin_openMode mode);
/* Ein Stichwortlistenfenster wird geoeffnet */

_VOID msgList_listOpenWithProc(CAT_GROUP_NUMBER gruppe, getNumberProc whichNr, CONST _UBYTE *ID, HWND grinWdw, grin_openMode mode);
/* Ein Stichwortlistenfenster wird geoeffnet,
 * die Nummer wird ueber eine Prozedur ermittelt
 */

_VOID msgList_listClose(HWND wdw);
/* Ein Stichwortlistenfenster wird geschlossen */

_VOID msgList_listSavePos(_VOID);
/* Sichert die Positionen der Stichwortlistenfenster */

_VOID msgList_listRestorePos(_VOID);
/* Restauriert die Positionen der Stichwortlistenfenster */

_VOID msgList_listCloseAll(_VOID);
/* Alle Stichwortlistenfenster werden geschlossen */

_VOID msgList_listRedrawAll(_VOID);
/* Alle Stichwortlistenfenster werden neu gezeichnet */

_VOID msgList_listFreeAll(_VOID);
/* Schliesst alle Stichwortlistenfenster und gibt auch alle Handles wieder
 * wieder frei
 */

_BOOL msgList_listWindowTop(HWND wdw);
/* Ist ein Stichwortlistenfenster oben? */

_VOID msgList_listSetFont(HWND wdw, _WORD font, _WORD fontSize);
/* Setzt den Font fuer das Stichwortlistenfenster */

_VOID msgList_SelectFont(HWND wdw);

/* Die folgenden Funktionen sind nur fuer die Interaktion
 * mit der Anzeige relevant und werden ueber ein handle angesprochen,
 * welches der Anzeige mit uebergeben wird. Dadurch werden Probleme
 * durch doppelte WdwHandles vermieden, die auftreten koennten.
 */

_VOID msgList_listUnlockWdw(LISTHANDLE hdl);
/* Muss von grin aufgerufen werden, wenn die Gruppe gewechselt wurde
 * oder das Fenster geschlossen wurde. Geht leider nicht anders :-(
 */

_VOID msgList_listCloseByHandle(LISTHANDLE hdl);
/* Ein Stichwortlistenfenster wird ueber das assoziierte Handle
 * geschlossen, wird von grin aufgerufen, wenn ein Fenster mit
 * geschlossen wird, an das eine Stichwortliste gelinkt ist
 */

_BOOL msgList_listChangeGroup(LISTHANDLE hdl, HWND grinWdw, MSGNR mess, CAT_GROUP_NUMBER newGroup, grin_openMode mode);
/* Wechselt die dargestellte Gruppe in der durch wdw spezifizierten
 * Liste und dem Fenster dazu. Ruft selber listUnlockWdw auf, muss
 * also nicht vorher aufgerufen werden.
 * Gibt zurueck, ob die Gruppe gewechselt wurde.
 */

_VOID msgList_listRemoveTree(LISTHANDLE hdl, HWND grinWdw, MSGNR msgNum, _BOOL setReadFlag);
/* Entfernt den Baum mit der Nachricht msgNum aus der Liste im Fenster
 * wdw, falls das grinWdw uebereinstimmt. Wenn setReadFlag TRUE ist,
 * dann wird auch noch fuer alle Nachrichten im Baum das Gelesen-Flag
 * gesetzt.
 */

_VOID msgList_listChangeFlags(CAT_GROUP_NUMBER group, MSGNR msgNum, MSGBITS newBits);
/* grin teilt der Stichwortliste mit, dass sich bei einer Nachricht
 * die Flags geaendert haben
 */


_VOID msgList_listSelectAll(HWND wdw, _BOOL withRedraw);
/* Alle Nachrichten in dem Fenster werden selektiert */

_VOID msgList_listSelectNew(HWND wdw);
/* Alle neuen Nachrichten in dem Fenster werden selektiert */

_BOOL msgList_listSaveAs(HWND wdw, _BOOL asStatus);
/* Speichert die selektierten Nachrichten in einer Datei */

_BOOL msgList_listCopy(HWND wdw, _BOOL append);
/* Alle selektierten Nachrichten werden exportiert*/

_BOOL msgList_listSomethingSelected(HWND wdw);
/* Gibt zurueck, ob in der Stichwortliste Nachrichten markiert
 * sind
 */

_VOID msgList_listSetFlags(HWND wdw, MSGBITS setBits, MSGBITS clearBits);
/* Fuer alle selektierten Nachrichten
 * ein paar Flags setzen oder loeschen
 */
_VOID msgList_listSendState(HWND wdw, _UBYTE newState);
/*	Verschickt State-Nachrichten fr alle selektierten */

_VOID msgList_listSearch(HWND wdw);
/* Nachrichten werden ueber die Suchfunktion selektiert
 */

_BOOL msgList_CheckListLine(CONST _UBYTE *listline);
/* Testet, ob die uebergebene Zeile fuer die Stichwortliste in Ordnung ist */

/* Das Format einer Zeile besteht aus einzelnen Kuerzeln
 * fuer einen Eintrag und darauffolgend in Klammern die Weite dieses Eintrages.
 * Beispiel: D(12),I(10),U(30),W(30),F(2)
 * Erlaubte Zeilenelemente:
 * D: Datum. Das Datum ist maximal 12 Zeichen lang und hat folgende Form:
 *           Mo, 16.11.92
 *    Optional kann man auch einen Formatstring angeben, der folgende Elemente
 *    enthalten darf:
 *     D: Tagesziffer  (d: wird geloescht, wenn nicht auffuellbar)
 *     Y: Jahresziffer (y: wird geloescht, wenn nicht auffuellbar)
 *     M: Monatsziffer (m: wird geloescht, wenn nicht auffuellbar)
 *     G: Monatsbuchstabe Deutsch
 *     F: Monatsbuchstabe Franzoesisch
 *     E: Monatsbuchstabe English
 *     #E (Hinter Ziffern): englische Zahlfortsetzung (1st, 2nd, 3rd, 4th...)
 * T: Zeit. Default-Format fuer die Zeit ist HH:SS, also 06:13.
 *    Optional kann man auch einen Formatstring angeben, der folgendes
 *    enthalten darf:
 *
 *   HH: Stundenzahl ("hh" loescht fuehrende Null)
 *   MM: Minutenzahl ("mm" loescht fuehrende Null)
 *   SS: Sekundenzahl ("ss" loescht fuehrende Null)
 *
 *   #E: AM/PM-Notierung (English) ("e" schreibt "am" bzw. "pm")
 *
 *   Beispiel: Aus 'hh Uhr mm' wird '13 Uhr 5'
 *   Die Angabe von Sekunden in einem Formatstring ist allerdings unsinnig,
 *   da die Zeitangaben zu Mails keine Sekunden enthalten.
 * W: Wochentag. Auch hier kann man einen Formatstring angeben, der die Sprache
 *    spezifiziert, in der der Wochentag ausgegeben wird.
 *
 *     G: Tagesbuchstabe Deutsch
 *     F: Tagesbuchstabe Franzoesisch
 *     E: Tagesbuchstabe English
 *
 *    Es muss fuer die gesamte gewuenschte Laenge ein Formatbuchstabe angegeben
 *    werden.
 *    Wenn das erste Zeichen in der Maske ein # ist, dann wird die
 *    Maske komplett mit dem zweiten Zeichen der Maske gefuellt.
 *    Beispiel: Aus #G wird intern GGGGGGGGGGGGGG
 *
 * I: Message-Id. Im Moment ist diese maximal 10 Zeichen lang, aber ab Maus 9.0
 *                koennen Ids wesentlich laenger werden.
 * R: Reference-Id: Bei Kommentaren oder Antworten die Id der Referenznachricht.
 * U: Username
 * A: Realname (wird ersetzt durch Username, falls nicht vorhanden)
 * W: Stichwort. Vor MAUS 9 maximal 30 Zeichen.
 * F: Flags. Die Flags der Nachrichten, die man in CAT setzen kann.
 *           Reihenfolge wie in der Messageanzeige, also LFITDKB12
 * S: Status. Nur fuer persoenliche Nachrichten, andernfalls wird es ignoriert.
 * C: Kommentare: Anzahl der Kommentare auf die Nachricht, nicht fuer persoenliche Nachrichten.
 * #: interne Nummer der Message. Maximal 5 Stellen.
 */

_VOID msgList_SetListLine(CONST _UBYTE *listline, _BOOL is_private);
/* Setzt die Stichwortlistenzeile fuer die private oder andere Gruppen
 * Wenn die Zeile fehlerhaft ist, wird die vorhandene Zeile nicht
 * veraendert.
 */

_VOID msgList_InitMsgList(_VOID);
/* Initialisiert die Messageliste */

_VOID msgList_ExitMsgList(_VOID);


#endif /* __CAT_MSGLIST_H__ */
