#ifndef __CAT_MSGLISTBASE_H__
#define __CAT_MSGLISTBASE_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif
#ifndef __CAT_LISTS_H__
#include <lists.h>
#endif
#ifndef __CAT_MTUTILS_H__
#include <mtutils.h>
#endif
#ifndef __CAT_GRIN_H__
#include <grin.h>
#endif
#ifndef __CAT_DATA_H__
#include <data.h>
#endif
#ifndef __CAT_CATTYPES_H__
#include <cattypes.h>
#endif
#ifndef __CAT_HANDLEPOOL_H__
#include <handlepo.h>
#endif
#ifndef __CAT_WDWMANAGER_H__
#include <wdwmanag.h>
#endif


#define maxParts        12   /* Anzahl der maximalen verschiedenen Zeilenteile */
#define partFormatLength  32   /* maximale Laenge eines Formatstrings */

/* Einzelne Bits in flags */
#define msglist_Selected        0x0001
#define msglist_inTree          0x0002
#define msglist_notRead         0x0004
#define msglist_doRedraw        0x0008
#define msglist_firstRead       0x0010
#define msglist_initRead        0x0020
#define msglist_msgCorrupt      0x0040
typedef _UWORD MSGFLAGS;


typedef struct _listEntry {
	MSGNR msgNum;
	MSGFLAGS flags;
	union {
		_UBYTE *strPtr;
		_ULONG treeNum;
	} i;
} listEntry;


typedef listEntry *listArray;


typedef enum {
	pUserName,
	pRealName,
	pMailId,
	pRefId,
	pWegen,
	pStatus,
	pDatum,
	pComments,
	pFlags,
	pCount,
	pMId,
	pOwnMess,
	pLength,
	pTime,
	pWeekday,
	pFuzzytime,
	pAttachment,
	pFrom,
	pTo
} partNameType;

typedef _UBYTE partFormat[partFormatLength];

typedef struct partType {
	partNameType type;
	_WORD w;
	partFormat formatStr;
} partType;

typedef partType partArrayType[maxParts];

typedef struct partArrayDesc {
	_WORD lines;
	_WORD width;
	partArrayType parts;
} partArrayDesc;

typedef struct _listWindow {
	HWND  wdw;                   /* Windowhandle */
	HVDI  vdi;                   /* VDI-Handle   */
	_BOOL isHidden;              /* Window is hidden */
	_BOOL isLocked;              /* Redraw wird nicht ausgefuehrt! */
	listEntry *wholeList;        /* Array mit allen Listeneintraegen */
	listEntry *list;             /* Array der Listeneintraege */
	GRECT butWork;               /* Rechteck des Objektbaumes */
	GRECT listWork;              /* Workrectangle des Fensters */
	_WORD number;                /* Nummer des Fensters im Modul */
	_WORD font;
	_WORD fontSize;              /* Font & Groesse */
	MSGNR startPos;              /* aktuelle Top-Position */
	MSGNR maxPos;                /* Anzahl der Messages in der Gruppe */
	_WORD leftOffset;
	_WORD maxWidth;
	_BOOL unreadOrNew;
	oneHandleElement *grHandle;  /* Gruppenhandle */
	_UBYTE grName[256];          /* Gruppenname fuer Titelzeile */
	_WORD charHeight;            /* Zeichensatzvariablen */
	_WORD charWidth;
	_BOOL isFSM;
	_BOOL monoSpaced;
	_WORD usedByGrin;
	grin_openMode openMode;
	HWND grinWindow;             /* Wenn >= 0, dann wurde es von grin aus aufgerufen */
	partArrayDesc partArray;     /* Aufbau der einzelnen Zeilen */
	/* Neu fuer Cursor-Control */
	_LONG cursPos;               /* Indexnummer fuer Cursor; -1 : Nicht sichtbar */
	_WORD entryWidth;            /* Breite in Pixeln einer Zeile */
	_WORD windLines;             /* Zeilen im Fenster */
	/* Fuer Drag & Drop */
	_ULONG dataSize;
	/* Farben */
	_WORD listTextCol;
	_WORD listBackCol;
	_WORD listNotReadCol;
	_WORD listOwnCol;
	_WORD listC2OCol;
	_UWORD ownEff;
	_UWORD c2oEff;
	_UWORD quoteEff;
} listWindow, *listWdwPtr;
typedef listWindow *LISTHANDLE;

typedef enum {
	sAddr,
	sWegen,
	sTree,
	sNum,
	sSize,
	sDate,
	sId,
	sLId,
	sAbs,
	sEmpf,
} sortType;

extern List MsgList_windows;
extern _WORD msglist_lineDist;
extern _WORD msglist_pixOffset;
extern _WORD msglist_globalNumber;


_WORD MsgListBase_FindNum(_VOID);

_VOID MsgListBase_ClearNum(_WORD num);

_BOOL MsgListBase_FindWinCond(_VOID *e, _VOID *i);
/* Abbruchprozedur, wie in <Lists> gefordert */

listWindow *MsgListBase_FindWind(HWND wdw);
listWindow *MsgListBase_FindHandle(LISTHANDLE hdl);
listWindow *MsgListBase_FindGroup(CAT_GROUP_NUMBER gruppe);

_BOOL MsgListBase_FindHandleCond(_VOID *e, _VOID *i);

_BOOL MsgListBase_FindGroupCond(_VOID *e, _VOID *i);

_BOOL MsgListBase_findMsgIdx(listWindow *ptr, MSGNR which, MSGNR *idx);

_BOOL MsgListBase_hasSelectedEntries(listWindow *ptr, MSGNR *first);

_VOID MsgListBase_deselectEntries(listWindow *ptr);

#define MsgListBase_toggleBit(flags, bit) (flags) ^= bit

_VOID MsgListBase_GetLinePart(listWindow *ptr, MSGNR num, MessageType *mess, _UBYTE *line, size_t strsize, CONST partType *type);

_VOID MsgListBase_shrinkToEnd(listWindow *ptr, _BOOL doRedraw);

_BOOL MsgListBase_shrinkList(listWindow *ptr, _BOOL removeSel, _BOOL doRedraw);

_VOID MsgListBase_listSelectFiltered(HWND wdw, _BOOL doSelect, _BOOL doRedraw);

_VOID MsgListBase_freeList(listWindow *ptr);

_BOOL MsgListBase_sortVorw(_VOID);

_BOOL MsgListBase_hideFilt(_VOID);	/*DYJ 5.38e */

_VOID MsgListBase_sethideFilt(_BOOL hideFilt);	/*DYJ 5.38e */

_VOID MsgListBase_setSort(sortType sort,_BOOL vorw);

_VOID MsgListBase_doSort(listWindow *ptr, sortType sort, _BOOL vorw, _BOOL withRedraw);

_VOID MsgListBase_doMark(listWindow *ptr, _BOOL withDial);

_VOID MsgListBase_drawListWdw(HWND wdw, HVDI vdiH, _VOID *special, CONST GRECT *frame);

_BOOL MsgListBase_clickInListWindow(HWND win, _VOID *special, _WORD x, _WORD y, _UWORD kstate, _UWORD buts, _WORD clicks);

_BOOL MsgListBase_handleListTimer(HWND wdw, _VOID *special);

_BOOL MsgListBase_handleListKey(HWND wdw, _VOID *special, _WORD taste, _UBYTE ch, _UBYTE scan, _UWORD kstate);

_VOID MsgListBase_getListScroll(HWND wdw, HVDI vdiH, _VOID *special, GRECT *work);

_BOOL MsgListBase_closeListWindow(HWND wdw, HVDI vdiH, _VOID *special, _BOOL force);

_BOOL MsgListBase_topListWdw(HWND wdw, _VOID *special);

_VOID MsgListBase_untopListWdw(HWND wdw, _VOID *special);

_VOID MsgListBase_updateListWdw(HWND wdw, _VOID *special, _BOOL update);

_VOID MsgListBase_hideListWdw(HWND wdw, _VOID *special, _BOOL hide);

_VOID MsgListBase_setListWork(HWND wdw, _VOID *special, CONST LGRECT *doc, _BOOL slided);

_VOID MsgListBase_snapListWdw(HWND wdw, _VOID *special, GRECT *work);

_BOOL MsgListBase_listGetHeader(HWND wdw, _VOID *special, _WORD idx, _UBYTE *ext, _UBYTE *name, size_t namesize, _ULONG *size);

_VOID MsgListBase_listReadData(HWND wdw, _VOID *special, _VOID **data, _WORD currentExt);

_VOID MsgListBase_MakeTitle(CAT_GROUP_NUMBER gruppe, _UBYTE *title, size_t strsize);


#endif /* __CAT_MSGLISTBASE_H__ */
