/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 * 7.3.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.21p'
 *	Setzen der Terminalkommandozeile und des Anhangpfades in install
 *	berarbeitet
 *
 * 1.5.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.27d'
 *	Install fragt jetzt ab ob es eine Maus- oder Internetinstallation
 *	werden soll, danach werden einige Felder anders vorbelegt.
 */


#include <instrsc.h>
#include <cattypes.h>

#include <mtappl.h>
#include <mtutils.h>
#include <mtdir.h>
#include <mtcomman.h>
#include <mttextfi.h>
#include <mtalerts.h>
#include <mtrsc.h>
#include <mtxobjec.h>
#include <mtdials.h>

#include <portvdi.h>
#include <tos.h>
#include <magicstr.h>
#include <magiccoo.h>

#include <strconv.h>
#include <filename.h>

#include <file_io.h>
#include <dir_io.h>
#include <ro_mem.h>
#include <routinec.h>
#include <routines.h>
#include <catlib.h>
#include <catgloba.h>
#include <catfiles.h>
#include <INSTALL.h>


GLOBAL OBJECT *infoBox;
GLOBAL OBJECT *pathbox;
GLOBAL OBJECT *helppath;
GLOBAL OBJECT *menu;

GLOBAL _UBYTE homePath[PATH_MAX];

LOCAL RESOURCE rsc;

_BOOL WriteCat_Lang=FALSE;

_UBYTE Tmp_CatInf[PATH_MAX]="";

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* image und icon-konvertierung */

LOCAL _VOID vdiFix(MFDB *pfd, _VOID *theAddr, _WORD wb, _WORD h)
{
	pfd->fd_wdwidth = (wb + 1) / 2;
	pfd->fd_w = wb * 8;
	pfd->fd_h = h;
	pfd->fd_nplanes = 1;
	pfd->fd_stand = TRUE;
	pfd->fd_addr = theAddr;
	pfd->fd_r1 = 0;
	pfd->fd_r2 = 0;
	pfd->fd_r3 = 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID vdiTrans(_VOID *source, _WORD sourceWb, _VOID *dest, _WORD destWb, _WORD h, HVDI handle)
{
	MFDB src, dst;

	vdiFix(&src, source, sourceWb, h);
	src.fd_stand = TRUE;
	vdiFix(&dst, dest, destWb, h);
	dst.fd_stand = FALSE;
	vr_trnfm(handle, &src, &dst);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID transBitBlk(BITBLK *spec, HVDI handle)
{
	_VOID *addr;

	addr = spec->bi_pdata;
	if (addr != (_VOID *) - 1)
		vdiTrans(addr, spec->bi_wb, addr, spec->bi_wb, spec->bi_hl, handle);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID transObj(OBJECT *tree, _WORD obj, HVDI handle)
{
	_WORD type;
	_WORD wb, hl;
	OBSPEC *spec;
	ICONBLK *ptrIcn;
	_VOID *tAddr;

	spec = GetObSpec(tree, obj);
	if (spec == (_VOID *) - 1 || spec == NULL)
		return;
	type = GetObtype(tree, obj);
	switch (type & 0xff)
	{
	case G_IMAGE:
		transBitBlk(spec->bitblk, handle);
		break;
	case G_ICON:
		ptrIcn = spec->iconblk;
		hl = ptrIcn->ib_hicon;
		wb = (ptrIcn->ib_wicon + 7) / 8;
		tAddr = ptrIcn->ib_pdata;
		if (tAddr != (_VOID *) - 1)
			vdiTrans(tAddr, wb, tAddr, wb, hl, handle);
		tAddr = ptrIcn->ib_pmask;
		if (tAddr != (_VOID *) - 1)
			vdiTrans(tAddr, wb, tAddr, wb, hl, handle);
		break;
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*  Routinen fuer den Umgang mit Objektbaeumen */
/*  ======================================== */

LOCAL _VOID info(CONST _UBYTE *s)
{
	mtAlerts_Alert(1, s);
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
GLOBAL _VOID InfoAlert(_WORD s1, CONST _UBYTE *s2, _WORD s3)
{
	_UBYTE scrap[512];

	strBcpy(scrap, Inst_WrkString(s1));
	strBcat(scrap, s2);
	strBcat(scrap, Inst_WrkString(s3));
	info(scrap);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD DoSimpleBox(OBJECT *form)
{
	_WORD i;
	GRECT r;

	i = -1;
	if (mtDials_NewDial(form))
	{
		Wind_Update(BEG_UPDATE);
		Wind_Update(BEG_MCTRL);
		mtDials_DialCenter(form, CSCREEN, 0, 0, &r);
		mtDials_DialForm(form, DSTART, &r, &r);
		mtDials_DialDraw(form, ROOT, MAX_DEPTH, &r, TRUE);
		do
			i = mtDials_DialDo(form, 0);
		while (i == -1 || i == -2 || i == -3);
		mtDials_DialForm(form, DFINISH, &r, &r);
		i &= 0x7fff;
		DelObjState(form, i, SELECTED);
		Wind_Update(END_MCTRL);
		Wind_Update(END_UPDATE);
		mtDials_DisposeDial(form);
	}
	return i;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21i */	/*DYJ 5.21o */	/*DYJ 5.22a */	/*DYJ 5.22f */
GLOBAL _BOOL InitResource(_VOID)
{
	_UBYTE rsrcname[PATH_MAX];
	_UBYTE scrap[PATH_MAX];
	_UBYTE nameEnd[20];
	_UWORD screenColumns;

	/* 80-Zeichen Bildschirm wird verlangt! */
	screenColumns = MaxWidth / CharWidth;

	/* Resource laden und Baumaddressen ermitteln */

	F_Path_Get_Program(scrap, "");
	if(!Rsc_EnvLang(scrap,nameEnd))
	{
		WriteCat_Lang=TRUE;		/*DYJ 5.22f */
		switch(mtAlerts_Alert(1,"[2][%P:|Bitte eine Sprache whlen|Please select a language][[Deutsch|[English]"))
		{
			case 1:
				strCpy(nameEnd,LANGUAGE_DE);
				break;
			case 2:
				strCpy(nameEnd,LANGUAGE_EN);
				break;
		}
	}
	strCat(nameEnd,RSC_EXT);

	strCpy(rsrcname, CATL_RSC_BASE);
	strCat(rsrcname,nameEnd);

	if(!Rsc_RscInit(scrap,rsrcname))
	{
		sprintf(scrap,"[3][%P|Konnte %s nicht laden|Could'nt load %s][OK]",rsrcname,rsrcname);
		mtAlerts_Alert(1,scrap);
		return FALSE;
	}
	if (screenColumns < 80)
	{
		mtAlerts_Alert(1, Rsc_WrkString(AL_RESOLUTION));
		return FALSE;
	}

	strCpy(rsrcname, CATI_RSC_BASE);
	strCat(rsrcname,nameEnd);

	if ((rsc = RscLoad(rsrcname)) == NULL)
	{
		F_Path_Append(scrap, rsrcname);
		rsc = RscLoad(scrap);
	}
	if (rsc == NULL)
	{
		sprintf(scrap,Rsc_WrkString(AL_NO_RSC),rsrcname);
		mtAlerts_Alert(1,scrap );
		return FALSE;
	}
	infoBox = RscGaddr(rsc, R_TREE, INFOBOX);
	menu = RscGaddr(rsc, R_TREE, MENU);
	pathbox = TreeAddrLib[PATH_CNF];
	helppath = RscGaddr(rsc, R_TREE, HELPPATH);

	/* Versionsnummer einsetzen */
	SetObjcString(infoBox, VERSION, INST_VERSION);

	/* Bit Images anpassen */

	transObj(infoBox, THECAT0, VDIHandle);
	transObj(infoBox, THECAT1, VDIHandle);
	transObj(infoBox, THECAT2, VDIHandle);

	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID doInfo(_VOID)
{
	DoSimpleBox(infoBox);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ClrRsrc(_VOID)
{
	if (rsc != NULL)
	{
		RscFree(rsc);
		rsc = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ReplaceStr(_UBYTE *str, size_t strsize, CONST _UBYTE *s1, CONST _UBYTE *s2)
{
	size_t p,
	 s1Len;

	s1Len = strLen(s1);	/*DYJ 5.20w */
	p = Pos(s1, str, 0, FALSE);
	while (p != POS_NOT_FOUND)
	{
		Delete(str, p, s1Len);
		Insert(s2, str, strsize, p);
		p = Pos(s1, str, p, FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL FselGet(_UBYTE *InPath, _UBYTE *InSel, CONST _UBYTE *ext, CONST _UBYTE *msg, _BOOL pathOnly)
{
	struct
	{
		_ULONG id;
		_UWORD version;
	} *fsel;
	_UBYTE scrapP[PATH_MAX];
	_UBYTE scrapN[PATH_MAX];
	_BOOL noMCtrl;

	strBcpy(scrapP, InPath);
	noMCtrl = FindCookie(COOKIE_FSEL, (_LONG *)&fsel) && fsel && fsel->version >= 0x200;
	strBcpy(scrapN, InSel);
	F_Path_Append(InPath, ext);
	if (!noMCtrl)
		Wind_Update(BEG_MCTRL);
	if (mtDir_GetDir(InPath, InSel, msg))
	{
		if (!noMCtrl)
			Wind_Update(END_MCTRL);
		F_Path_Dirname(InPath);
		if (pathOnly)
			return TRUE;
		if (*InSel != '\0')
			return TRUE;
	} else
	{
		if (!noMCtrl)
			Wind_Update(END_MCTRL);
	}
	strMcpy(InPath, PATH_MAX, scrapP);
	strMcpy(InSel, PATH_MAX, scrapN);
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

typedef struct
{
	_UBYTE arc_path[PATH_MAX];
	_UBYTE enarc_str[PATH_MAX];
	_UBYTE dearc_str[PATH_MAX];
	_UBYTE import_path[PATH_MAX];
	_UBYTE archive_name[PATH_MAX];
	_UBYTE export_path[PATH_MAX];
	_UBYTE message_path[PATH_MAX];
	_UBYTE data_path[PATH_MAX];
	_UBYTE terminal_path[PATH_MAX];
	_UBYTE terminal_cmd[PATH_MAX];
	_UBYTE attach_path[PATH_MAX];
} configInfo;

#define lzhArc   "a $E$F $M" INFILE_NAME
#define lzhDearc "x $I$O $I " OUTFILE_NAME
#define zipArc   "-a $E$F $M" INFILE_NAME
#define zipDearc "-x $I$O $I " OUTFILE_NAME

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID translateOut(configInfo *confInfo, CONST _UBYTE *src, _UBYTE *dest, size_t destsize)
{
	_UBYTE path[PATH_MAX];
	_UBYTE name[PATH_MAX];
	_UBYTE suff[PATH_MAX];

	strMcpy(dest, destsize, src);
	mtDir_SplitPath(confInfo->archive_name, path, name, suff);
	strBcpy(name, NAME_INFILE);
	strBcat(name, suff);
	ReplaceStr(dest, destsize, "$F", name);						/* Infilename */
	ReplaceStr(dest, destsize, "$O", confInfo->archive_name);	/* Outfilename */
	ReplaceStr(dest, destsize, "$I", confInfo->import_path);	/* Importpath */
	ReplaceStr(dest, destsize, "$E", confInfo->export_path);	/* Exportpath */
	ReplaceStr(dest, destsize, "$M", confInfo->message_path);	/* Messagepath */
	ReplaceStr(dest, destsize, "$D", confInfo->data_path);	/* Databasepath */
	mtDir_SplitPath(confInfo->terminal_path, path, name, suff);
	ReplaceStr(dest, destsize, "$T", path);						/* Terminalpath */
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID getEdits(OBJECT *tree, configInfo *confInfo)
{
	ObjcString(tree, PSARC, confInfo->enarc_str, sizeof(confInfo->enarc_str));
	ObjcString(tree, PSDEARC, confInfo->dearc_str, sizeof(confInfo->dearc_str));
	ObjcString(tree, PSCOMM, confInfo->terminal_cmd, sizeof(confInfo->terminal_cmd));
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID create_path(_UBYTE *path, CONST _UBYTE *newPath, CONST _UBYTE *dir)
{
	strMcpy(path, PATH_MAX, newPath);
	F_Path_Append(path, dir);
	F_Path_Append(path, "");
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21p */	/*DYJ 5.21b */	/*DYJ 5.27d */
GLOBAL _BOOL doConfig(CONST _UBYTE *newPath, _UBYTE *newDataPath)
{

	OBJECT *tree;
	configInfo *confInfo;
	_UBYTE str[256];
	_UBYTE path[PATH_MAX];
	_UBYTE name[PATH_MAX];
	_UBYTE suff[PATH_MAX];
	_WORD but;
	_BOOL res;
	GRECT or;
	TEXTFILENO tf;
	_BOOL nFound;
	_BOOL done;
	_BOOL result;
	_BOOL hattePacker;	
	
	*newDataPath = '\0';
	tree = pathbox;
	confInfo = NEW(configInfo, "doConfig");
	if (confInfo == NULL)
		return FALSE;
	MemSetZeroStruct(*confInfo);
	/* Jetzt Defaults festlegen */
	F_Path_Set(newPath);
	/* Jetzt Unterverzeichnisse anlegen */
	create_path(confInfo->import_path, newPath, "IMPORT");
	create_path(confInfo->export_path, newPath, "EXPORT");
	create_path(confInfo->message_path, newPath, "MESSAGES");
	create_path(confInfo->data_path, newPath, "DATABASE");
	create_path(confInfo->attach_path, newPath, "ATTACHM");
	SetObjcString(tree, PSPACK, confInfo->arc_path);
	SetObjcString(tree, PSARC, confInfo->enarc_str);
	SetObjcString(tree, PSDEARC, confInfo->dearc_str);
	SetObjcString(tree, PSIMPORT, confInfo->import_path);
	if(mtAlerts_Alert(2, Inst_WrkString(AL_MAUS))==2)
	{
		strCpy(confInfo->archive_name,"outfile.txt");
		DelObjFlags(tree, PBPACK, SELECTABLE);
		SetObjState(tree, PBPACK, DISABLED);
	}
	else
	{
		SetObjFlags(tree, PBPACK, SELECTABLE);
		DelObjState(tree, PBPACK, DISABLED);
	}
	SetObjState(tree, PSARC, DISABLED);
	SetObjState(tree, PSDEARC, DISABLED);
	SetObjState(tree, PBARC, DISABLED);
	SetObjState(tree, PBDEARC, DISABLED);
	SetObjcString(tree, PSOUTFIL, confInfo->archive_name);
	SetObjcString(tree, PSEXPORT, confInfo->export_path);
	SetObjcString(tree, PSMESSAG, confInfo->message_path);
	SetObjcString(tree, PSATTCH, confInfo->attach_path);
	SetObjcString(tree, PSDATA, confInfo->data_path);
	SetObjcString(tree, PSTERM, confInfo->terminal_path);
	SetObjcString(tree, PSCOMM, confInfo->terminal_cmd);
	SetObjFlags(tree, PATHSAVE,HIDETREE);	/*DYJ 5.21i */
	result = FALSE;
	if (mtDials_NewDial(tree))
	{
		mtDials_DialCenter(tree, CSCREEN, 0, 0, &or);
		mtDials_DialForm(tree, DSTART, &or, &or);
		mtDials_DialDraw(tree, ROOT, MAX_DEPTH, &or, TRUE);
		done = FALSE;
		while (!done)
		{
			but = mtDials_DialDo(tree, 0);
			but &= 0x7fff;
			switch (but)
			{
			/* Pfad holen */
			/* Nur bei PBOUTFIL nur Dateinamen ohne Pfad nehmen,
			 * bei PBPACK oder PBTERM auch noch Dateinamen an Pfad
			 * anhaengen
			 */
			case PBPACK:
				hattePacker=confInfo->arc_path[0]!='\0';	
				if (confInfo->arc_path[0] != '\0')
				{
					mtDir_SplitPath(confInfo->arc_path, path, name, suff);
				} else
				{
					strBcpy(path, newPath);
					name[0] = '\0';
				}
				res = FselGet(path, name, ALL_FILE_MASK, "Packer ausw\204hlen", FALSE);
				if (res)
				{
					strBcpy(confInfo->arc_path, path);
					if (name[0] != '\0')
						F_Path_Append(confInfo->arc_path, name);
					else
						confInfo->arc_path[0] = '\0';
					/* Defaults fuer die Kommandozeilen und das Outfile eintragen */
					nFound = FALSE;
					if (strCaseStr(name, "LZH") != NULL ||
						strCaseStr(name, "LHA") != NULL)
					{
						/* LHArc gefunden */
						SetObjcString(tree, PSARC, lzhArc);
						SetObjcString(tree, PSDEARC, lzhDearc);
						strBcpy(confInfo->archive_name, "outfile.lzh");
						SetObjcString(tree, PSOUTFIL, confInfo->archive_name);
					} else if (strCaseStr(name, "ZOO") != NULL)
					{
						SetObjcString(tree, PSARC, lzhArc);
						SetObjcString(tree, PSDEARC, lzhDearc);
						strBcpy(confInfo->archive_name, "outfile.zoo");
						SetObjcString(tree, PSOUTFIL, confInfo->archive_name);
					} else if (strCaseStr(name, "ARC") != NULL)
					{
						SetObjcString(tree, PSARC, lzhArc);
						SetObjcString(tree, PSDEARC, lzhDearc);
						strBcpy(confInfo->archive_name, "outfile.arc");
						SetObjcString(tree, PSOUTFIL, confInfo->archive_name);
					} else if (strCaseStr(name, "ZIP") != NULL)
					{
						SetObjcString(tree, PSARC, zipArc);
						SetObjcString(tree, PSDEARC, zipDearc);
						strBcpy(confInfo->archive_name, "outfile.zip");
						SetObjcString(tree, PSOUTFIL, confInfo->archive_name);
					} else
					{
						nFound = TRUE;
					}
					if (!nFound)
					{
						ObjcString(tree, PSARC, confInfo->enarc_str, sizeof(confInfo->enarc_str));
						ObjcString(tree, PSDEARC, confInfo->dearc_str, sizeof(confInfo->dearc_str));
					}
				}
				else
				{
					if(strLen(confInfo->arc_path) && mtAlerts_Alert(1, Rsc_WrkString(AL_NO_PACK))==2)
						confInfo->arc_path[0] = '\0';
				}
				SetObjcString(tree, PSPACK, confInfo->arc_path);
				if(hattePacker!=(strLen(confInfo->arc_path)!=0))
				{
					EnableObjState(tree, PSARC, DISABLED, !strLen(confInfo->arc_path));
					EnableObjState(tree, PSDEARC, DISABLED, !strLen(confInfo->arc_path));
					EnableObjState(tree, PBARC, DISABLED, !strLen(confInfo->arc_path));
					EnableObjState(tree, PBDEARC, DISABLED, !strLen(confInfo->arc_path));
				}
				break;
				
			case PBIMPORT:
				if (confInfo->import_path[0] != '\0')
				{
					mtDir_SplitPath(confInfo->import_path, path, name, suff);
				} else
				{
					strBcpy(path, newPath);
					name[0] = '\0';
				}
				res = GetAndCreatePath(path, name, SEL_IP);
				if (res)
				{
					strBcpy(confInfo->import_path, path);
					SetObjcString(tree, PSIMPORT, confInfo->import_path);
				}
				break;
				
			case PBOUTFIL:
				strBcpy(path, confInfo->import_path);
				strBcpy(name, confInfo->archive_name);
				res = FselGet(path, name, ALL_FILE_MASK, Inst_WrkString(SEL_OF), FALSE);
				if (res)
				{
					strBcpy(confInfo->archive_name, name);
					SetObjcString(tree, PSOUTFIL, confInfo->archive_name);
				}
				break;
				
			case PBEXPORT:
				if (confInfo->import_path[0] != '\0')
				{
					mtDir_SplitPath(confInfo->import_path, path, name, suff);
				} else
				{
					strBcpy(path, newPath);
					name[0] = '\0';
				}
				res = GetAndCreatePath(path, name, SEL_EP);
				if (res)
				{
					strBcpy(confInfo->export_path, path);
					SetObjcString(tree, PSEXPORT, confInfo->export_path);
				}
				break;
				
			case PBMESSAG:
				if (confInfo->message_path[0] != '\0')
				{
					mtDir_SplitPath(confInfo->message_path, path, name, suff);
				} else
				{
					strBcpy(path, newPath);
					name[0] = '\0';
				}
				res = GetAndCreatePath(path, name, SEL_NP);
				if (res)
				{
					strBcpy(confInfo->message_path, path);
					SetObjcString(tree, PSMESSAG, confInfo->message_path);
				}
				break;
				
			case PBATTCH:
				if (confInfo->attach_path[0] != '\0')
				{
					mtDir_SplitPath(confInfo->attach_path, path, name, suff);
				} else
				{
					strBcpy(path, newPath);
					name[0] = '\0';
				}
				res = GetAndCreatePath(path, name, SEL_AP);
				if (res)
				{
					strBcpy(confInfo->attach_path, path);
					SetObjcString(tree, PSATTCH, confInfo->attach_path);
				}
				break;
			case PBDATA:
				if (confInfo->data_path[0] != '\0')
				{
					mtDir_SplitPath(confInfo->data_path, path, name, suff);
				} else
				{
					strBcpy(path, newPath);
					name[0] = '\0';
				}
				res = GetAndCreatePath(path, name, SEL_DP);
				if (res)
				{
					strBcpy(confInfo->data_path, path);
					SetObjcString(tree, PSDATA, confInfo->data_path);
				}
				break;
				
			case PBTERM:
				if (confInfo->terminal_path[0] != '\0')
				{
					mtDir_SplitPath(confInfo->terminal_path, path, name, suff);
				} else
				{
					strBcpy(path, newPath);
					name[0] = '\0';
				}
				res = FselGet(path, name, ALL_FILE_MASK, Inst_WrkString(SEL_TP), FALSE);
				if (res)
				{
					strBcpy(confInfo->terminal_path, path);
					if (name[0] != '\0')
						F_Path_Append(confInfo->terminal_path, name);
					else
						confInfo->terminal_path[0] = '\0';
					SetObjcString(tree, PSTERM, confInfo->terminal_path);
				}
				break;
			
			case PBCMD:
			{
				_BOOL exists;
				if((strCaseStr(confInfo->terminal_path,"in2cat") &&
					CatFiles_FileSize(confInfo->terminal_path, &exists)>120000l) ||
					mtAlerts_Alert(1, Rsc_WrkString(AL_NO_I2C))==1)
				mtAlerts_Alert(1, Inst_WrkString(I2C_FROMCAT));
				break;
			}
			case PATHHELP:
				DoSimpleBox(helppath);
				break;
			
			case PATHOK:
				getEdits(tree, confInfo);
				if (confInfo->import_path[0] == '\0')
				{
					/* Importpfad */
					mtAlerts_Alert(1, Inst_WrkString(UNKN_IP));
				} else if (confInfo->archive_name[0] == '\0')
				{
					mtAlerts_Alert(1, Inst_WrkString(UNKN_OF));
				} else if (confInfo->export_path[0] == '\0')
				{
					/* Exportpfad */
					mtAlerts_Alert(1, Inst_WrkString(UNKN_IP));
				} else if (confInfo->message_path[0] == '\0')
				{
					/* Messagepfad */
					mtAlerts_Alert(1, Inst_WrkString(UNKN_MP));
				} else if (confInfo->data_path[0] == '\0')
				{
					/* Databasepfad */
					mtAlerts_Alert(1, Inst_WrkString(UNKN_DP));
				} else if (confInfo->attach_path[0] == '\0')
				{
					/* Anhangpfad */
					mtAlerts_Alert(1, Inst_WrkString(UNKN_AP));
				} else if (confInfo->terminal_path[0] == '\0')
				{
					/* Terminalprogramm */
					mtAlerts_Alert(1, Inst_WrkString(UNKN_TP));
				} else if (confInfo->arc_path[0] != '\0' && confInfo->enarc_str[0] == '\0')
				{
					/* Leere Kommandozeile fuer Einpacken */
					mtAlerts_Alert(1, Inst_WrkString(UNKN_CI));
				} else if (confInfo->arc_path[0] != '\0' && confInfo->dearc_str[0] == '\0')
				{
					/* Leere Kommandozeile fuer Auspacken */
					mtAlerts_Alert(1, Inst_WrkString(UNKN_CO));
				} else if(strCaseStr(confInfo->terminal_path,"in2cat")==NULL)
				{
					done = TRUE;
					if (confInfo->arc_path[0] == '\0')
					{
						if (mtAlerts_Alert(1, Inst_WrkString(UNKN_PA)) != 1)
							done = FALSE;
					}
					if (done && confInfo->terminal_cmd[0] == '\0')
					{
						if (mtAlerts_Alert(1, Inst_WrkString(UNKN_CT)) != 1)
						done = FALSE;
					}
				}
				else
					done=TRUE;
				break;
	
			case PATHQUIT:
				done = TRUE;
				break;
			}
			DelObjState(tree, but, SELECTED);
			if (!done)
				mtDials_DialDraw(tree, ROOT, MAX_DEPTH, &or, TRUE);
		}
		if (but == PATHOK)
		{
			/* editables holen */
			getEdits(tree, confInfo);
			/* Ordner anlegen */
			F_Path_Create(confInfo->import_path);
			F_Path_Create(confInfo->export_path);
			F_Path_Create(confInfo->message_path);
			F_Path_Create(confInfo->data_path);
			F_Path_Create(confInfo->attach_path);
			/* Strings uebernehmen */
			strBcpy(path, newPath);
			F_Path_Append(path, catinf);
			if (mtTextfiles_OpenTextfile(path, WRITE, 512, &tf) == GERR_OK)
			{
				getEdits(tree, confInfo);	/* editable Strings holen */
				if (confInfo->arc_path[0] != '\0')
				{
					mtDir_SplitPath(confInfo->arc_path, path, name, suff);
					mtTextfiles_WriteLine(tf, path);
					mtTextfiles_WriteLn(tf);	/* Arc-Path */
					mtTextfiles_WriteLine(tf, name);
					mtTextfiles_WriteLn(tf);	/* Arc-Name */
				} else
				{
					mtTextfiles_WriteLn(tf);
					mtTextfiles_WriteLn(tf);
				}
				translateOut(confInfo, confInfo->enarc_str, str, sizeof(str));
				mtTextfiles_WriteLine(tf, str);
				mtTextfiles_WriteLn(tf);	/* arc-Command */
				translateOut(confInfo, confInfo->dearc_str, str, sizeof(str));
				mtTextfiles_WriteLine(tf, str);
				mtTextfiles_WriteLn(tf);	/* dearc-Command */
				mtTextfiles_WriteLine(tf, confInfo->archive_name);
				mtTextfiles_WriteLn(tf);	/* Outfile name */
				mtTextfiles_WriteLn(tf);	/* not supported from dialog */
				mtTextfiles_WriteLn(tf);	/* not supported from dialog */
				mtTextfiles_WriteLine(tf, confInfo->message_path);
				mtTextfiles_WriteLn(tf);	/* Messagepath */
				mtTextfiles_WriteLine(tf, confInfo->import_path);
				mtTextfiles_WriteLn(tf);	/* Importpath */
				mtTextfiles_WriteLine(tf, confInfo->export_path);
				mtTextfiles_WriteLn(tf);	/* Exportpath */
				mtTextfiles_WriteLine(tf, confInfo->data_path);
				mtTextfiles_WriteLn(tf);	/* Databasepath */
				mtDir_SplitPath(confInfo->terminal_path, path, name, suff);
				mtTextfiles_WriteLine(tf, path);
				mtTextfiles_WriteLn(tf);	/* TerminalPath */
				mtTextfiles_WriteLine(tf, name);
				mtTextfiles_WriteLn(tf);	/* TerminalName */
				translateOut(confInfo, confInfo->terminal_cmd, str, sizeof(str));
				mtTextfiles_WriteLine(tf, str);
				mtTextfiles_WriteLn(tf);	/* Terminalcommand */
				mtTextfiles_WriteLine(tf, confInfo->attach_path);
				mtTextfiles_WriteLn(tf);	/* Attachmentpath */
				mtTextfiles_CloseTextfile(&tf);
				result = TRUE;
			} else
			{
				InfoAlert(noFile1, catinf, INST_STR1);
			}
	
			strMcpy(newDataPath, PATH_MAX, confInfo->data_path);
	
		}
		mtDials_DialForm(tree, DFINISH, &or, &or);
		mtDials_DisposeDial(tree);
	}
	OFREE(confInfo);
	return result;
}

/*** ---------------------------------------------------------------------- ***/

typedef struct
{
	_UBYTE username[256];
	_UBYTE passwort[8+1];
	_UBYTE telefon[256];
	_UBYTE device[PATH_MAX];
} terminalInfo;

LOCAL _VOID getTerm(OBJECT *tree, terminalInfo *termInfo)
{
	ObjcString(tree, EUSERNAM, PtrSize(termInfo->username));
	ObjcString(tree, EPASSWORT, PtrSize(termInfo->passwort));
	ObjcString(tree, EDIAL, PtrSize(termInfo->telefon));
	ObjcString(tree, EDEVICE, PtrSize(termInfo->device));
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID readLine(TEXTFILENO tf, _UBYTE *str, size_t maxlen)
{
	mtTextfiles_ReadLine(tf, str, maxlen);
	mtTextfiles_ReadLn(tf);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID writeLine(TEXTFILENO tf, CONST _UBYTE *str)
{
	mtTextfiles_WriteLine(tf, str);
	mtTextfiles_WriteLn(tf);
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
GLOBAL _BOOL doKommata(CONST _UBYTE *newPath,CONST _UBYTE *infName)
{
	OBJECT *tree;
	terminalInfo *termInfo;
	_UBYTE str[256];
	_UBYTE path[PATH_MAX];
	_UBYTE import_path[PATH_MAX];
	_UBYTE export_path[PATH_MAX];
	_WORD but;
	TEXTFILENO tf;
	TEXTFILENO kom;
	_BOOL done;
	_BOOL result;
	GRECT or;
	_UBYTE *p;
	
	tree = RscGaddr(rsc, R_TREE, TERM_CNF);
	termInfo = NEW(terminalInfo, "doConfig");
	if (termInfo == NULL)
		return FALSE;
	MemSetZeroStruct(*termInfo);
	strBcpy(termInfo->device, "U:\\DEV\\MODEM1");
	
	SetObjcString(tree, EUSERNAM, termInfo->username);
	SetObjcString(tree, EPASSWORT, termInfo->passwort);
	SetObjcString(tree, EDIAL, termInfo->telefon);
	SetObjcString(tree, EDEVICE, termInfo->device);
	result = FALSE;
	if (mtDials_NewDial(tree))
	{
		mtDials_DialCenter(tree, CSCREEN, 0, 0, &or);
		mtDials_DialForm(tree, DSTART, &or, &or);
		mtDials_DialDraw(tree, ROOT, MAX_DEPTH, &or, TRUE);
		done = FALSE;
		while (!done)
		{
			but = mtDials_DialDo(tree, 0);
			but &= 0x7fff;
			switch (but)
			{
			case TERMOK:
				getTerm(tree, termInfo);
				done = TRUE;
				break;
	
			case TERMQUIT:
				done = TRUE;
				break;
			}
			DelObjState(tree, but, SELECTED);
			if (!done)
				mtDials_DialDraw(tree, ROOT, MAX_DEPTH, &or, TRUE);
		}
		if (but == TERMOK)
		{
			/* editables holen */
			getTerm(tree, termInfo);
			/* Strings uebernehmen */
			strBcpy(path, newPath);
			F_Path_Append(path, infName);
			if (mtTextfiles_OpenTextfile(path, READ, 512, &tf) == GERR_OK)
			{
				getTerm(tree, termInfo);	/* editable Strings holen */
				strBcpy(path, newPath);
				F_Path_Append(path, "kommata.kom");
				if (mtTextfiles_OpenTextfile(path, WRITE, 512, &kom) == GERR_OK)
				{
					readLine(tf, PtrSize(str)); /* Arc-Path */
					readLine(tf, PtrSize(str)); /* Arc-Name */
					readLine(tf, PtrSize(str)); /* arc-Command */
					readLine(tf, PtrSize(str)); /* dearc-Command */
					readLine(tf, PtrSize(str)); /* Outfile name */
					readLine(tf, PtrSize(str)); /* empty */
					readLine(tf, PtrSize(str)); /* empty */
					readLine(tf, PtrSize(str)); /* message path */
					readLine(tf, PtrSize(import_path)); /* import path */
					readLine(tf, PtrSize(export_path)); /* export path */
					readLine(tf, PtrSize(str)); /* database path */
					readLine(tf, PtrSize(str)); /* terminal path */
					readLine(tf, PtrSize(str)); /* terminal name */
					readLine(tf, PtrSize(str)); /* terminal command */
					
					writeLine(kom, "#INIT");
					writeLine(kom, "ATZ|");
					writeLine(kom, "#DIAL");
					writeLine(kom, termInfo->telefon);
					
					strBcpy(str, termInfo->username);
					p = strChr(str, '@');	/*DYJ 5.20w */
					if (p != NULL)
					{
						*p++ = '\0';
						writeLine(kom, "#BOX");
						mtTextfiles_WriteLine(kom, "Maus");
						writeLine(kom, p);
					}
					writeLine(kom, "#USER");
					writeLine(kom, str);
					
					writeLine(kom, "#PASS");
					writeLine(kom, termInfo->passwort);
					
					strBcpy(path, newPath);
					F_Path_Append(path, "kommata.log");
					writeLine(kom, "#LOG");
					writeLine(kom, path);
					
					strBcpy(path, export_path);
					F_Path_Append(path, INFILE_NAME);
					writeLine(kom, "#IN");
					writeLine(kom, path);
					
					writeLine(kom, "#OUT");
					writeLine(kom, import_path);
					
					writeLine(kom, "#IMP");
					writeLine(kom, import_path);
					
					writeLine(kom, "#DEV");
					writeLine(kom, termInfo->device);
					
					writeLine(kom, "#BAUD");
					writeLine(kom, "19200");
					
					writeLine(kom, "#FLOW");
					writeLine(kom, "RTS/CTS");
					
					writeLine(kom, "#MAX");
					writeLine(kom, "100");
					
					writeLine(kom, "#WAIT");
					writeLine(kom, "300");
					
					writeLine(kom, "#PAUS");
					writeLine(kom, "33");
					
					writeLine(kom, "#INFO");
					writeLine(kom, "JA");
					
					writeLine(kom, "#NCBR");
					writeLine(kom, "JA");
					
					writeLine(kom, "#APDN");
					writeLine(kom, "NEIN");
					
					writeLine(kom, "#GAST");
					writeLine(kom, "NEIN");
					
					writeLine(kom, "#MPC");
					writeLine(kom, magicPC && magIxVer <= 0x600 ? "JA" : "NEIN");
					
					writeLine(kom, "#SYS");
					writeLine(kom, "NEIN");
					
					writeLine(kom, "#TERM");
					writeLine(kom, "NEIN");
					
					writeLine(kom, "#END");
					
					mtTextfiles_CloseTextfile(&kom);
					result = TRUE;
				}
				mtTextfiles_CloseTextfile(&tf);
			} else
			{
				InfoAlert(noFile1, catinf, INST_STR1);
			}
		}
		mtDials_DialForm(tree, DFINISH, &or, &or);
		mtDials_DisposeDial(tree);
	}
	OFREE(termInfo);
	return result;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*DYJ 5.21b */
GLOBAL _BOOL GetAndCreatePath(_UBYTE *path, _UBYTE *name, _WORD title)
{
	_UBYTE gName[PATH_MAX];
	F_STAT st;
	_BOOL res;

	name[0] = '\0';
	if (!FselGet(path, name, ALL_FILE_MASK, Inst_WrkString(title), TRUE))
		return FALSE;
	if (name[0] != '\0')
	{
		/* Testen, ob es ein Dateiname oder Pfadname ist,
		 * ggf. nachfragen und ggf. Pfad anlegen
		 */
		strBcpy(gName, path);
		F_Path_Append(gName, name);
		res = F_Getinfo(gName, &st, TRUE) == GERR_OK;
		if (res && st.st_is_directory)
		{
			/* Name ist ein Ordnername */
			/* Nachfragen! */
			if (mtAlerts_Alert(1, Inst_WrkString(NAME_PATH)) == 2)
			{
				F_Path_Append(path, name);
				F_Path_Append(path, "");
				name[0] = '\0';
			}
		} else if (res)
		{
			/* Nur Dateiname, ignorieren */
		} else
		{
			/* Kein Name, also neuer Ordnername! */
			if (F_Path_Create(gName) != GERR_OK)
			{
				info(Inst_WrkString(NO_CR_FOLDER));
				return FALSE;
			}
			F_Path_Append(path, name);
			F_Path_Append(path, "");
			name[0] = '\0';
		}
	}
	return TRUE;
}
/*** ------------------------------------------------------------------------ ***/
/*	_UBYTE *Inst_WrkString(_WORD nr)											*/
/*	Ermittelt einen freien oder Alertstring aus der inst_*.rsc					*/
/*	Parameter:	nr:	die Stringnummer											*/
/*	Ergebnis:	der String														*/		
/*	24.2.03 von Dimitri Junker	DYJ 5.21i										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _UBYTE *Inst_WrkString(_WORD nr)
{
	static _UBYTE dummy[30];
	_UBYTE *ret;
	ret=*((_UBYTE **)RscGaddr(rsc, R_FRSTR, nr));
	if(!ret)
	{
		ret=dummy;
		sprintf(ret,"[3][NO RSC;Nr:L%d][OK]",nr);
	}
	return ret;
}
