/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 * 4.10.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.30p'
 *	Erstmal nur zum Testen: Man kann jetzt unter Mint die Umleitung
 *	der IO-Filter in eine Pipe abschalten
 */


#include <iofilter.h>
#include <arcstart.h>
#include <catgloba.h>
#include <cattypes.h>
#include <confvars.h>
#include <varname4.h>
#include <varnamel.h>
#include <catlog.h>
#include <mte.h>
#include <mtpaths.h>
#include <userinfo.h>
#include <catopts.h>

#include <routinec.h>
#include <routines.h>
#include <debug.h>
#include <wdwmanag.h>
#include <protokol.h>
#include <vaproto.h>
#include <ro_mem.h>
#include <catfiles.h>

#define AL_IOFILTERMESSAGE_ERROR "[3][%%P:|Fehler bei IO-Filter f\201r Messages:|%s, Returncode = %d.][:[Abbruch]"
#define AL_IOFILTERINFILE_ERROR  "[3][%%P:|Fehler bei IO-Filter f\201r Infile:|%s, Returncode = %d.][:[Abbruch]"
#define AL_IOFILTEROUTFILE_ERROR "[3][%%P:|Fehler bei IO-Filter f\201r Outfile:|%s, Returncode = %d.][:[Abbruch]"

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _VOID buildCmdLine(CONST _UBYTE *inCmd, _UBYTE *cmdLine, size_t len,_WORD id)
{
	_UBYTE *start,*stop,*tmp;
	_UBYTE mausId[128];
	_UBYTE ownName[128];
	_UBYTE *maus;
	
	len -= 1;
	ConfVars_GetConfDefStr(cMausId, mausId, sizeof(mausId), "");
	strCpy(ownName, mausId);
	maus = strChr(mausId, '@');
	if (maus != NULL)
	{
		strCpy(mausId, maus);
		strdelSpace(mausId);
	}
	strMcpy(cmdLine + 1, len, inCmd);
	CatGlobal_ReplaceStr(cmdLine + 1, len, "$I", ImportPath);
	CatGlobal_ReplaceStr(cmdLine + 1, len, "$E", ExportPath);
	CatGlobal_ReplaceStr(cmdLine + 1, len, "$M", MessagePath);
	CatGlobal_ReplaceStr(cmdLine + 1, len, "$D", DataPath);
	CatGlobal_ReplaceStr(cmdLine + 1, len, "$F", INFILE_NAME);
	CatGlobal_ReplaceStr(cmdLine + 1, len, "$O", OUTFILE_NAME);
	CatGlobal_ReplaceStr(cmdLine + 1, len, "$K", mausId);
	CatGlobal_ReplaceStr(cmdLine + 1, len, "$U", ownName);
	start=strStr(cmdLine,"$Q");
	if(start)
	{
		tmp=MALLOC(len,"CMD-Quote");
		if(tmp)
		{
			do
			{
				stop=strStr(start+2,"$Q");
				if(stop)
					*stop='\0';
				VaProto_Quote(tmp,len,start+2,id);
				if(stop)
					strMcat(tmp,len,stop+2);
				*start='\0';
				strMcat(cmdLine,len,tmp);
				start=strStr(cmdLine,"$Q");
			}while(start);
			FREE(tmp,len);
		}
	}
	cmdLine[0] = (_UBYTE)strLen(cmdLine + 1);
}

/*** ---------------------------------------------------------------------- ***/
GLOBAL _VOID IoFilter_ReadFilter(ioFilterStruct *filter, _WORD num)
{
	_UBYTE varname[256];
	_BOOL tmpB;	
	
	/* Filter ist vorhanden, auslesen */
	make_varname(varname, sizeof(varname), cIoFilterInCmdline, num);
	ConfVars_GetConfDefStr(varname, filter->inCmd, sizeof(filter->inCmd), "");
	make_varname(varname, sizeof(varname), cIoFilterOutCmdline, num);
	ConfVars_GetConfDefStr(varname, filter->outCmd, sizeof(filter->outCmd), "");
	make_varname(varname, sizeof(varname), cIoFilterType, num);
	if(!ConfVars_GetConfigInt(varname,(_WORD *) &filter->filtType))
	{		/* Dann die alten Variablen lesen und lschen und neue schreiben	*/
		filter->filtType=ForInfile;	/* als Marke fr ungesetzt */
		make_varname(varname, sizeof(varname), cIoFilterBeforeMsg, num);
		ConfVars_GetConfDefBool(varname, &tmpB, FALSE);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterPastOutfile, num);
		if(tmpB)
			filter->filtType=ForMsg;
		else
		{
			ConfVars_GetConfDefBool(varname, &tmpB, FALSE);
			if(tmpB)
				filter->filtType=PastOutfile;
		}
		ConfVars_DeleteConfigVar(varname);

		make_varname(varname, sizeof(varname), cIoFilterForInfile, num);
		ConfVars_GetConfDefBool(varname, &tmpB, FALSE);
		ConfVars_DeleteConfigVar(varname);

		make_varname(varname, sizeof(varname), cIoFilterForOutfile, num);
		if(filter->filtType==ForInfile)	/* also bisher ungesetzt s.o.*/
		{
			if(!tmpB)
				filter->filtType=ForOutfile;
			ConfVars_GetConfDefBool(varname, &tmpB, FALSE);
			if(tmpB)
				filter->filtType|=ForOutfile;
		}
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterType, num);
		ConfVars_SetConfigInt(varname, filter->filtType);
	}
	make_varname(varname, sizeof(varname), cIoFilterActive, num);

	if(ConfVars_GetConfigBool(varname, &tmpB))
	{
		filter->pass= tmpB? 5 : 0;
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterPass, num);
		ConfVars_SetConfigInt(varname,filter->pass);
	}
	else
	{
		make_varname(varname, sizeof(varname), cIoFilterPass, num);
		ConfVars_GetConfDefInt(varname,&filter->pass,1);
	}
		
	make_varname(varname, sizeof(varname), cIoFilterErrorBreak, num);
	ConfVars_GetConfDefBool(varname, &filter->stopAtErr, FALSE);
	/* Jetzt Programmnamen extrahieren */
	make_varname(varname, sizeof(varname), cIoFilterName, num);
	ConfVars_GetConfDefStr(varname, filter->filtName, sizeof(filter->filtName), "");
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID IoFilter_WriteFilter(CONST ioFilterStruct *filter, _WORD num)
{
	_UBYTE varname[256];
	
	if (filter->progName[0] != '\0')
	{
		make_varname(varname, sizeof(varname), cIoFilterProgram, num);
		ConfVars_SetConfigString(varname, filter->progName);
		make_varname(varname, sizeof(varname), cIoFilterInCmdline, num);
		ConfVars_SetConfigString(varname, filter->inCmd);
		make_varname(varname, sizeof(varname), cIoFilterOutCmdline, num);
		ConfVars_SetConfigString(varname, filter->outCmd);
		make_varname(varname, sizeof(varname), cIoFilterType, num);
		ConfVars_SetConfigInt(varname, filter->filtType);
		make_varname(varname, sizeof(varname), cIoFilterPass, num);
		ConfVars_SetConfigInt(varname, filter->pass);
		make_varname(varname, sizeof(varname), cIoFilterErrorBreak, num);
		ConfVars_SetConfigBool(varname, filter->stopAtErr);
		make_varname(varname, sizeof(varname), cIoFilterName, num);
		ConfVars_SetConfigString(varname, filter->filtName);
	} else
	{
		/* Configvariablen loeschen! */
		make_varname(varname, sizeof(varname), cIoFilterProgram, num);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterInCmdline, num);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterOutCmdline, num);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterBeforeMsg, num);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterForInfile, num);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterForOutfile, num);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterPastOutfile, num);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterActive, num);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterErrorBreak, num);
		ConfVars_DeleteConfigVar(varname);
		make_varname(varname, sizeof(varname), cIoFilterName, num);
		ConfVars_DeleteConfigVar(varname);
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Ruft die definierten Filter auf, die vor dem erstellen von 
 * Infile.txt benutzt werden sollen
 */
GLOBAL _BOOL IoFilter_IoFilterMessage(_BOOL *isError)
{
	ioFilterStruct filter;
	_WORD res, i;
	_UBYTE cmdLine[256];
	_UBYTE varname[256];
	_BOOL done;
	_BOOL redr;
	_WORD pass,nxtPass=1;	
	*isError = FALSE;
	done = FALSE;
	while(nxtPass<1000)
	{
		pass=nxtPass;
		nxtPass=1000;
		for (i = 0; i < MAX_IO_FILTER; i++)
		{
			/* alle Filter durchgehen
			 */
			make_varname(varname, sizeof(varname), cIoFilterProgram, i + 1);
			if (ConfVars_GetConfigString(varname, filter.progName, sizeof(filter.progName)))
			{
				IoFilter_ReadFilter(&filter, i + 1);
				/* Jetzt nachsehen, ob wir den Filter auch benutzen 
				 * muessen 
				 */
				if (filter.filtType==ForMsg && filter.pass==pass)
				{
					int flags=ARCSTART_REDRAW;		/*DYJ 5.30p */
					_BOOL bool;		/*DYJ 5.30p */
					/* Kommandozeile zusammenbauen */
					buildCmdLine(filter.inCmd, cmdLine, sizeof(cmdLine),DO_QUOTE);
					/* Filter aufrufen */
					SPEC_DEBUG_OUT((EO_DEBUG, "Message-Filter: %s %s", filter.progName, cmdLine + 1));
/*DYJ 5.30p Start*/
					ConfVars_GetConfDefBool(cNoRedirectFilter, &bool, FALSE);
					if (bool)
						flags |= ARCSTART_NOREDIRECT;
					res = ARCStarter_ExecProgram(filter.progName, cmdLine, ARCSTART_REDRAW);
/*DYJ  Ende; alt:	
					res = ARCStarter_ExecProgram(filter.progName, cmdLine, ARCSTART_REDRAW);
*/
					SPEC_DEBUG_OUT((EO_DEBUG, "Returncode: %d", res));
					done = TRUE;
					if (res != 0 && filter.stopAtErr)
					{
						*isError = TRUE;
						if (!batchProcess)
						{
							make_varname(varname, sizeof(varname), cIoFilterName, i + 1);
							if (!ConfVars_GetConfigString(varname, filter.filtName, sizeof(filter.filtName)))
								strBcpy(filter.filtName, filter.progName);
							sprintf(cmdLine, AL_IOFILTERMESSAGE_ERROR, filter.filtName, res);
							MTE_info(cmdLine);
						}
						done=FALSE;	
						break;
					}
				}
				else if(filter.filtType==ForMsg && filter.pass>pass && filter.pass<nxtPass)
					nxtPass=filter.pass;
			}
		}
	}
	ConfVars_GetConfDefBool(cIoRedrawAftFilter, &redr, FALSE);	
	if(redr)
		allRedrawWdw();
	return done;
}


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Ruft die definierten Filter auf, die auf dem Infile arbeiten
 */
GLOBAL _BOOL IoFilter_IoFilterInfile(_VOID)
{
	ioFilterStruct filter;
	_WORD i, res;
	_UBYTE cmdLine[256];
	_UBYTE varname[256];
	_BOOL redr,ret=TRUE;
	
	_WORD pass,nxtPass=1;
	_UBYTE infile[PATH_MAX];
#if SPEC_DEBUG
	_BOOL exists;
	size_t len;
#endif
	strCpy( infile,ARCName[0] == '\0' ? ExportPath : MessagePath);
	F_Path_Append(infile, INFILE_NAME);
	while(nxtPass<1000)
	{
		pass=nxtPass;
		nxtPass=1000;
		for (i = 0; i < MAX_IO_FILTER; i++)
		{
			/* alle Filter durchgehen
			 */
			make_varname(varname, sizeof(varname), cIoFilterProgram, i + 1);
			if (ConfVars_GetConfigString(varname, filter.progName, sizeof(filter.progName)))
			{
				/* Filter ist vorhanden, auslesen */
				IoFilter_ReadFilter(&filter, i + 1);
				
				/* Jetzt nachsehen, ob wir den Filter auch benutzen 
				 * muessen 
				 */
				if ((filter.filtType&ForInfile)!=0 && filter.pass==pass)
				{
					int flags=ARCSTART_REDRAW;
					_BOOL bool;		/*DYJ 5.30p */
					/* Kommandozeile zusammenbauen */
					buildCmdLine(filter.inCmd, cmdLine, sizeof(cmdLine),DO_QUOTE);
					/* Filter aufrufen */
					SPEC_DEBUG_OUT((EO_DEBUG, "Infile-Filter: %s %s", filter.progName, cmdLine + 1));
/*DYJ 5.30p Start*/
					ConfVars_GetConfDefBool(cNoRedirectFilter, &bool, FALSE);
					if (bool)
						flags |= ARCSTART_NOREDIRECT;
					res = ARCStarter_ExecProgram(filter.progName, cmdLine, ARCSTART_REDRAW);
/*DYJ  Ende; alt:	
					res = ARCStarter_ExecProgram(filter.progName, cmdLine, ARCSTART_REDRAW);
*/
					SPEC_DEBUG_OUT((EO_DEBUG, "Returncode: %d", res));

#if SPEC_DEBUG
					len=CatFiles_FileSize(infile,&exists);
					if(exists)
						SPEC_DEBUG_OUT((EO_DEBUG, "Das Infile hat die Lnge: %ld", len));
					else
						SPEC_DEBUG_OUT((EO_DEBUG, "Hmmm ich kann das Infile nicht finden"));
#endif
					if (res != 0 && filter.stopAtErr)
					{
						if (!batchProcess)
						{
							make_varname(varname, sizeof(varname), cIoFilterName, i + 1);
							if (!ConfVars_GetConfigString(varname, filter.filtName, sizeof(filter.filtName)))
								strBcpy(filter.filtName, filter.progName);
							sprintf(cmdLine, AL_IOFILTERINFILE_ERROR, filter.filtName, res);
							MTE_info(cmdLine);
						}
						ret=FALSE;
						break;
					}
				}
				else if((filter.filtType&ForInfile)!=0 && filter.pass>pass && filter.pass<nxtPass)
					nxtPass=filter.pass;
			}
		}
	}
	ConfVars_GetConfDefBool(cIoRedrawAftFilter, &redr, FALSE);
	if(redr)
		allRedrawWdw();
	
	return ret;
	
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Ruft die definierten Filter auf, die auf dem Outfile arbeiten
 */
GLOBAL _BOOL IoFilter_IoFilterOutfile(_VOID)
{
	ioFilterStruct filter;
	_WORD i, res;
	_UBYTE cmdLine[256];
	_UBYTE varname[256];
	_BOOL bool;
	_UWORD flags;
	_BOOL redr,ret=TRUE;
	
	_WORD pass,nxtPass=1;
	_UBYTE outfile[PATH_MAX];
#if SPEC_DEBUG
	_BOOL exists;
	size_t len;
#endif
	strCpy( outfile,ImportPath);
	F_Path_Append(outfile, OUTFILE_NAME);
	while(nxtPass<1000)
	{
		pass=nxtPass;
		nxtPass=1000;
		for (i = 0; i < MAX_IO_FILTER; i++)
		{
			/* alle Filter durchgehen
			 */
			make_varname(varname, sizeof(varname), cIoFilterProgram, i + 1);
			if (ConfVars_GetConfigString(varname, filter.progName, sizeof(filter.progName)))
			{
				/* Filter ist vorhanden, auslesen */
				IoFilter_ReadFilter(&filter, i + 1);
				
				/* Jetzt nachsehen, ob wir den Filter auch benutzen 
				 * muessen 
				 */
				if ((filter.filtType&ForOutfile)!=0 && filter.pass==pass)
				{
					/* Kommandozeile zusammenbauen */
					buildCmdLine(filter.outCmd, cmdLine, sizeof(cmdLine),DO_QUOTE);
					/* Filter aufrufen */
					ConfVars_GetConfDefBool(cRedirectPackerToLog, &bool, FALSE);
					CatLog_WriteStringNTime("Rufe IO-Filter auf: ");
					CatLog_WriteLine(filter.progName);
					CatLog_WriteStringNTime("Kommandozeile:      ");
					CatLog_WriteLine(cmdLine + 1);
					SPEC_DEBUG_OUT((EO_DEBUG, "IO-Filter: %s %s", filter.progName, cmdLine + 1));
					flags = ARCSTART_REDRAW;
					if (bool)
						flags |= ARCSTART_TOLOG;
/*DYJ 5.30p Start*/
					ConfVars_GetConfDefBool(cNoRedirectFilter, &bool, FALSE);
					if (bool)
						flags |= ARCSTART_NOREDIRECT;
/*DYJ  Ende; */
					res = ARCStarter_ExecProgram(filter.progName, cmdLine, flags);
					SPEC_DEBUG_OUT((EO_DEBUG, "Returncode: %d", res));
#if SPEC_DEBUG
					len=CatFiles_FileSize(outfile,&exists);
					if(exists)
						SPEC_DEBUG_OUT((EO_DEBUG, "Das Outfile hat die Lnge: %ld", len));
					else
						SPEC_DEBUG_OUT((EO_DEBUG, "Hmmm ich kann das Outfile nicht finden"));
#endif
					if (res != 0)
					{
						CatLog_WriteString("*Returncode ");
						CatLog_WriteInt(res);
						CatLog_WriteString(" bei Aufruf von ");
						CatLog_WriteString(filter.progName);
						CatLog_WriteLine("*");
						if (filter.stopAtErr)
						{
							if (!batchProcess)
							{
								make_varname(varname, sizeof(varname), cIoFilterName, i + 1);
									if (!ConfVars_GetConfigString(varname, filter.filtName, sizeof(filter.filtName)))
									strBcpy(filter.filtName, filter.progName);
								sprintf(cmdLine, AL_IOFILTEROUTFILE_ERROR, filter.filtName, res);
								MTE_info(cmdLine);
							}
							ret=FALSE;
							break;
						}
					}
					CatLog_WriteLineNTime("IO-Filter beendet.");
					CatLog_WriteLn();
				}
				else if ((filter.filtType&ForOutfile)!=0 && filter.pass>pass && filter.pass<nxtPass)
					nxtPass=filter.pass;
			}
		}
	}
	ConfVars_GetConfDefBool(cIoRedrawAftFilter, &redr, FALSE);
	if(redr)
		allRedrawWdw();
	return ret;
	
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Ruft die definierten Filter auf, die nach dem Einfuegen
 * des Outfiles benutzt werden sollen
 */
GLOBAL _BOOL IoFilter_IoFilterPastOutfile(_VOID)
{
	ioFilterStruct filter;
	_WORD i, res;
	_UBYTE cmdLine[256];
	_UBYTE varname[256];
	_BOOL bool;
	_UWORD flags;
	_BOOL redr,ret=TRUE;

	_WORD pass,nxtPass=1;
	while(nxtPass<1000)
	{
		pass=nxtPass;
		nxtPass=1000;
		for (i = 0; i < MAX_IO_FILTER; i++)
		{
			/* alle Filter durchgehen
			 */
			make_varname(varname, sizeof(varname), cIoFilterProgram, i + 1);
			if (ConfVars_GetConfigString(varname, filter.progName, sizeof(filter.progName)))
			{
				/* Filter ist vorhanden, auslesen */
				IoFilter_ReadFilter(&filter, i + 1);
				
				/* Jetzt nachsehen, ob wir den Filter auch benutzen 
				 * muessen 
				 */
				if (filter.filtType==PastOutfile && filter.pass==pass)
				{
					/* Kommandozeile zusammenbauen */
					buildCmdLine(filter.inCmd, cmdLine, sizeof(cmdLine),DO_QUOTE);
					/* Filter aufrufen */
					ConfVars_GetConfDefBool(cRedirectPackerToLog, &bool, FALSE);
					CatLog_WriteStringNTime("Rufe IO-Filter auf: ");
					CatLog_WriteLine(filter.progName);
					CatLog_WriteStringNTime("Kommandozeile:      ");
					CatLog_WriteLine(cmdLine + 1);
					SPEC_DEBUG_OUT((EO_DEBUG, "IO-Filter: %s %s", filter.progName, cmdLine + 1));
					flags = ARCSTART_REDRAW;
					if (bool)
						flags |= ARCSTART_TOLOG;
/*DYJ 5.30p Start*/
					ConfVars_GetConfDefBool(cNoRedirectFilter, &bool, FALSE);
					if (bool)
						flags |= ARCSTART_NOREDIRECT;
/*DYJ  Ende; */
					res = ARCStarter_ExecProgram(filter.progName, cmdLine, flags);
					SPEC_DEBUG_OUT((EO_DEBUG, "Returncode: %d", res));
					if (res != 0)
					{
						CatLog_WriteString("*Returncode ");
						CatLog_WriteInt(res);
						CatLog_WriteString(" bei Aufruf von ");
						CatLog_WriteString(filter.progName);
						CatLog_WriteLine("*");
					}
					CatLog_WriteLineNTime("IO-Filter beendet");
					CatLog_WriteLn();
				}
				else if(filter.filtType==PastOutfile && filter.pass>pass && filter.pass<nxtPass)
					nxtPass=filter.pass;
			}
		}
	}
	ConfVars_GetConfDefBool(cIoRedrawAftFilter, &redr, FALSE);
	if(redr)
		allRedrawWdw();
	return ret;
	
}
