/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


/* Purpose of module: Change MIME text encoding to Atari special characters
 */
#include <strconv.h>
#include <wildcard.h>
#include <mttextfi.h>
#include <filename.h>

#include <file_io.h>
#include <routine.h>

GLOBAL _UBYTE ProgramName[] = "isofilt";
GLOBAL _BOOL PrgHasDataBase=FALSE;		/*DYJ 5.26a */

LOCAL _UBYTE line[32767];
LOCAL _BOOL inOpen;
LOCAL _BOOL outOpen;
LOCAL TEXTFILENO in;
LOCAL TEXTFILENO out;
LOCAL _ULONG charsChanged;
LOCAL _ULONG headLinesChanged;


LOCAL _UBYTE CONST isoTable[128] = {
	128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143,
	144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 225, 159,
	 32, 173, 155, 156, 120, 157, 124, 221,  34,  67, 166, 174, 170,  45,  82, 223,
	248, 241, 253, 252,  39, 230, 227, 249,  44,  49, 167, 175, 172, 171,  47, 168,
	133, 160, 131,  65, 142, 143, 146, 128, 138, 144, 136, 137, 141, 161, 140, 139,
	 68, 165, 149, 162, 147, 111, 153, 120, 237, 151, 163, 150, 154, 121,  80, 225,
	133, 160, 131,  97, 132, 134, 145, 135, 138, 130, 136, 137, 141, 161, 140, 139,
	100, 164, 149, 162, 147, 111, 148, 246, 237, 151, 163, 150, 129, 121, 112, 152
};

#define is_hex(c) (((c) >= '0' && (c) <= '9') || ((c) >= 'A' && (c) <= 'F'))


LOCAL _VOID CloseIt(_VOID)
{
	if (inOpen)
	{
		mtTextfiles_CloseTextfile(&in);
		inOpen = FALSE;
	}
	if (outOpen)
	{
		mtTextfiles_CloseTextfile(&out);
		outOpen = FALSE;
	}
}

LOCAL _VOID error(CONST _UBYTE *s)
{
	fprintf(stderr, "%s: %s\n", ProgramName, s);
	CloseIt();
	exit(1);
}


LOCAL _VOID Usage(_VOID)
{
	printf("Usage: %s inputfile\n", ProgramName);
	CloseIt();
	exit(2);
}

LOCAL _UBYTE GetChar(_WORD val)
{
	if (val >= 128 && val < 256)
		return isoTable[val - 128];
	return (_UBYTE) val;
}


/* Gibt zurueck, ob es sich um eine normale Nachricht oder um einen 
 * Kontrollblock (Infofile etc.) handelt.
 */
LOCAL _BOOL CheckId(CONST _UBYTE *line)
{
	return strchr(line, '@') != NULL;
}


LOCAL _VOID CheckTextLine(_UBYTE *line)
{
	_UBYTE *p;
	_UBYTE num[4];
	_WORD val;
	_BOOL succ;

	p = line;
	for (;;)
	{
		p = strchr(p, '=');
		if (p == NULL)
			break;
		if (is_hex(p[1]) && is_hex(p[2]))
		{
			num[0] = '$';
			num[1] = p[1];
			num[2] = p[2];
			num[3] = '\0';
			val = StrToInt(num, NULL, &succ);
			if (succ && val >= 32)
			{
				strcpy(p + 1, p + 3);
				p[0] = GetChar(val);
				++charsChanged;
			}
		}
		p++;
	}
}


LOCAL _VOID CheckHeadLine(_UBYTE *line)
{
	_UBYTE charSet[256];
	_UBYTE *p, *p2, *pend;
	size_t len;
	
	/* Enthaelt =?irgendwas?=, also wahrscheinlich ISO-Umlaute */
	p = strstr(line, "=?");
	if (p == NULL)
		return;
	/* Zwischen dem ersten Fragezeichen und dem zweiten steht der Name des
	 * Zeichensatzes, extrahieren wir mal, werten wir aber noch nicht weiter aus
	 */
	p += 2;
	pend = strstr(p, "?=");
	if (pend == NULL)
		return;
	p2 = strchr(p, '?');
	if (p2[1] != 'Q' || p2[2] != '?')
		return;
	len = pend - p + 1;
	if (len > sizeof(charSet))
		len = sizeof(charSet);
	strMcpy(charSet, len, p);
	/* Jetzt noch ?= am Ende suchen und entfernen */
	strcpy(p, pend + 2);
	/* Und jetzt noch alle anderen Zeichen wandeln */
	++headLinesChanged;
	CheckTextLine(line);
}


int main(int argc, char **argv)
{
	_BOOL inNormalMessage;
	_UBYTE tmpf[PATH_MAX];
	char *outf;
/*DYJ 4.17h Start*/
	char *pkt;
/*DYJ  Ende */

	inOpen = FALSE;
	outOpen = FALSE;
	charsChanged = 0;
	headLinesChanged = 0;
	if (argc != 2)
	{
		Usage();
	}
	outf = argv[1];
	/* temporaeren Namen basteln */
/*DYJ 4.17i Start*/
	strcpy(tmpf,outf);
	if((pkt=strchr(tmpf,'.'))!=NULL)
		strcpy(pkt, ".tmp");
	else
		strcat(tmpf, ".tmp");
/*DYJ  Ende; alt:
	ConcatName(outf, ".tmp", tmpf);
*/
	if (mtTextfiles_OpenTextfile(outf, READ, 0, &in) != GERR_OK)
	{
		error("Kann Eingabedatei nicht ffnen!");
	}
	inOpen = TRUE;
	/* temporaeres File ffnen */
	if (mtTextfiles_OpenTextfile(tmpf, WRITE, 0, &out) != GERR_OK)
	{
		error("Kann temporaere Datei nicht anlegen!");
	}
	outOpen = TRUE;
	/* Jetzt sind beide Dateien offen, also loslegen */
	inNormalMessage = FALSE;
	while (!mtTextfiles_EndofText(in))
	{
		mtTextfiles_ReadLine(in, line, sizeof(line));
		mtTextfiles_ReadLn(in);
		switch (line[0])
		{
		case 'W':
		case 'N':
		case 'V':
			CheckHeadLine(line);
			break;
		case '#':
			inNormalMessage = CheckId(line);
			break;
		case ':':
			if (inNormalMessage)
			{
				CheckTextLine(line);
			}
			break;
		}
		mtTextfiles_WriteLine(out, line);
		mtTextfiles_WriteLn(out);
	}
	CloseIt();
	/* Jetzt OUTFILE.TXT lschen und tmp-Datei umbenennen */
	if (F_File_Delete(outf) != GERR_OK)
	{
		error("Kann OUTFILE.TXT nicht lschen!");
	}
	if (F_Path_Rename(tmpf, outf) != GERR_OK)
	{
		error("Kann OUTFILE.TMP nicht zu OUTFILE.TXT umbenennen!");
	}
	printf("ISO-Filterung erfolgreich durchgefuehrt.\n");
	printf("%lu Headerzeilen geaendert und %lu Zeichen umgesetzt.\n", headLinesChanged, charsChanged);
	return 0;
}
