/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 * 10.11.02 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.16p'
 *	Man kann jetzt eigene Farben fr PM-Pseudogruppen eingeben.
 *
 * 3.9.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.30f'
 *	Autolocator merkt sich die Position, es sei denn die Liste wird
 *	erneuert.
 *
 * 11.4.04 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.35n'
 *	Den neuen (5.35c) Gruppen-ndern Dialog auch aus dem Gruppenauswahl-
 *	Dialog zur Auswahl der PM-Filter-Gruppe erreichbar gemaucht
 */


#include <listhelp.h>
#include <listdl.h>
#include <data.h>
#include <mte.h>
#include <catgloba.h>
#include <confvars.h>
#include <varname4.h>
#include <cathelp.h>
#include <w_graf.h>
#include <alias.h>

#include <mtappl.h>
#include <mtdials.h>
#include <mtutils.h>
#include <vdiutil.h>
#include <portvdi.h>

#include <maustaus.h>

#include <ro_mem.h>
#include <routinec.h>
#include <routines.h>
#include <strconv.h>
#include <mtrsc.h>
#include <adr.h>	/*DYJ 5.34d */
#include <gruppena.h>	/*DYJ 5.35n */
/*==============================================================*
 * Modul:               Modul zur Verwaltung der Abrufboxen     *
 * Autor:               Dirk Steins, Jo Goettker-Schnetmann     *
 * erstellt am:         10.04.1992                              *
 * letzte Aenderung am: 10.04.1992                              *
 * Version:             1.0                                     *
 * Interne Version:     V#0001                                  *
 *==============================================================*

 *----------------------------------------------------------------------------
 * Datum    Vers. Autor  Aenderung (Arbeitsbericht)
 *----------------------------------------------------------------------------
 * 06.05.98       MH     Benutzername wird aus der ITU ausgelesen und
 *                       kann ausgewaehlt werden 
 * 06.05.1998     MH     Benutzername wird aus der ITU ausgelesen und
 *                       kann ausgewaehlt werden
 * 11.11.1998     MH     Gruppen mit Programmteil koennen ausgewaehlt
 *                       werden 
 *----------------------------------------------------------------------------
 *
 * 5.2.02 Gerhard Stoll (GS) gekennzeichnet mit 'GS 4.36e'
 *	da fehlte ein Komma zwischen 2 Titeln.
 */

LOCAL OBJECT *gruppen;

LOCAL group_line lastGroup;
LOCAL _WORD globalLength;
LOCAL _WORD maxNameLength;
LOCAL _BOOL groupDialIsOpen;

LOCAL List grList;
LOCAL group *clipEntry;

typedef struct {
	groupSelectMode mode;
	_BOOL isReadMode;
	_WORD exitBut;
	_WORD pmCol;
	_WORD omCol;
} envRecord;

LOCAL CONST _WORD titles[gsmNumModes] = {
	GSM_TIT_READBEGIN,		/* Ab Anfang lesen */
	GSM_TIT_READNEW,		/* Neue Nachrichten lesen */
	GSM_TIT_READUNREAD,		/* Ungelesene lesen */
	GSM_TIT_READLASTPOS,	/* Ab letzter Position lesen */
	GSM_TIT_READDATE,		/* Ab Datum lesen */
	GSM_TIT_READID,			/* Ab ID lesen */
	GSM_TIT_WRITE,			/* Neue Nachricht in Gruppe schreiben */
	GSM_TIT_USER,			/* Gruppenname fuer einen Benutzer auswaehlen, fuer MT-Kommandos */
	GSM_TIT_SEL,		/* GUEK schreiben */
	GSM_TIT_MODIFY,			/* Liste aendern */
	GSM_TIT_SEL,			/* Gruppe auswaehlen fuer ITK-Kommando */
	GSM_TIT_SEL,			/* Gruppe fuer Gruppenchef auswaehlen */
	GSM_TIT_PROGTEIL,		/* Gruppen mit Programmteil auswaehlen (MH) */
	GSM_TIT_USERNAME,  		/* Namen aus ITU holen (MH) */
	GSM_TIT_FOLLOWUP,		/* Followup setzen		*/
	GSM_TIT_PMFILT,			/* Gruppe fr PM-Filter*/
	GSM_TIT_PUTZ			/* Gruppe fr Gruppe lschen*/	/*DYJ 5.36c */
};

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL selGrEntry(_VOID *entry, _VOID *env, _WORD line)
{
	group *gr;
	GRECT r;

	UNUSED(env);
	gr = entry;
	if (gr == NULL)
		return FALSE;
	gr->selected = !gr->selected;
	if (gr->selected && line >= 0)
	{
		SetObjcString(gruppen, NEWGROUP, gr->display_name);
		WinDials_WinDialDraw(gruppen, NEWGROUP, 0, &r, FALSE);
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID groupToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)
{
	group *gr;

	UNUSED(env);
	gr = entry;
	if (gr == NULL)
		*str = '\0';
	else
		strMcpy(str, strsize, gr->display_name);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID card2str(MSGNR new, _UBYTE *numStr)
{
	_BOOL appendK;

	if (new != 0)
	{
		if (new >= 1000)
		{
			new /= 1000;
			appendK = TRUE;
		} else
		{
			appendK = FALSE;
		}
		CardToStr(new, 3, numStr, 4);
		if (appendK)
		{
			strCpy(numStr, numStr + 1);
			numStr[2] = 'K';
			numStr[3] = '\0';
		}
	} else
	{
		*numStr = '\0';
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID buildGroup(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize, _UBYTE *unread, _UBYTE *space, _UBYTE *numStr)
{
	group *gr;
	MSGNR newIdx, lastIdx, unrd, new;
	envRecord *modePtr;

	gr = entry;
	modePtr = env;

	strMcpy(str, strsize, gr->display_name);
	/* Anzahl der neuen Nachrichten suchen */
	if (modePtr->isReadMode)
	{
		newIdx = data_FirstNewMsg(gr->catNumber);
		lastIdx = data_LastMsgOfGroup(gr->catNumber);
		if (lastIdx != dataSys_empty && newIdx != dataSys_empty && newIdx <= lastIdx)
			new = lastIdx - newIdx + 1;
		else
			new = 0;
		card2str(new, numStr);

		unrd = data_unreadMsgCount(gr->catNumber);
		if (unrd == dataSys_empty)
			unrd = 0;
		card2str(unrd, unread);

		if (new != 0 && unrd != 0)
			strCpy(space, "|");
		else
			*space = '\0';
	} else
	{
		*unread = '\0';
		*space = '\0';
		*numStr = '\0';
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findSel(_VOID *entry, _VOID *inf)
{
	group *gr = entry;

	UNUSED(inf);
	return gr->selected;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID cutEntry(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID cutEntry(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;
	_BOOL found;

	UNUSED(env);
	*lstSel=-1;	/*DYJ 5.30f */
	/* Selektierten Eintrag in der Liste finden und aus der Liste entfernen */
	Lists_ResetList(&grList);
	if ((found = Lists_ScanEntries(&grList, Lists_forward, findSel, NULL)) != FALSE)
	{
		clipEntry = Lists_CurrentEntry(&grList);
		Lists_RemoveEntry(&grList, FUNK_NULL);
		ListMan_ListScroll2Selection(info, TRUE);
		/* Nun das neue Icon darstellen */
		DelObjFlags(gruppen, GRCFULL, HIDETREE);
		SetObjFlags(gruppen, GRCEMPTY, HIDETREE);
		CalcArea(gruppen, GRCFULL, &r);
		WinDials_WinDialDraw(gruppen, ROOT, MAX_DEPTH, &r, TRUE);
	}
	DelObjState(gruppen, obj, SELECTED);
	WinDials_WinDialDraw(gruppen, obj, 1, &r, FALSE);
	*draw = found;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID pasteEntry(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID pasteEntry(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r, r1, r2;
	_WORD x, y;
	_UWORD kstate, buts;
	_LONG index;
	group *adr;

	UNUSED(obj);
	*lstSel=-1;	/*DYJ 5.30f */
	ObjcArea(gruppen, GRCFULL, &r1);
	ObjcArea(gruppen, ROOT, &r2);
	graf_mkstate(&x, &y, &buts, &kstate);
	if (buts & MOB_LEFT)
	{
		Graf_dragbox(&r1, &r2, &x, &y);
		graf_mkstate(&x, &y, &buts, &kstate);
		index = ListMan_ListClick(info, x, y, kstate, 0);
		/* Persoenliche bleiben immer vorne: */
		if (index == 0)
			++index;
		if (index > 0)
		{
			adr = ListMan_ListFindEntry(info, index);
			if (adr == NULL)
			{
				Lists_AppendEntry(&grList, clipEntry);
			} else
			{
				Lists_PrevEntry(&grList);
				Lists_InsertEntry(&grList, clipEntry);
			}
			/* Jetzt mal schnell alle Eintraege deselektieren */
			Lists_ResetList(&grList);
			while ((adr = Lists_NextEntry(&grList)) != NULL)
				adr->selected = FALSE;
			clipEntry->selected = FALSE;
			selGrEntry(clipEntry, env, 0);
			ListMan_ListScroll2Selection(info, TRUE);
			SetObjFlags(gruppen, GRCFULL, HIDETREE);
			DelObjFlags(gruppen, GRCEMPTY, HIDETREE);
			CalcArea(gruppen, GRCEMPTY, &r);
			WinDials_WinDialDraw(gruppen, ROOT, MAX_DEPTH, &r, TRUE);
			clipEntry = NULL;
			*draw = TRUE;
		} else
		{
			*draw = FALSE;
		}
	} else
	{
		*draw = FALSE;
	}
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
/*DYJ 5.30f Start*/
LOCAL _VOID clipEmpty(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID clipEmpty(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	UNUSED(obj);
	UNUSED(env);
	UNUSED(info);
	UNUSED(lstSel);	/*DYJ 5.30f */
	MTE_infoRSC(AL_CLIPINFO);
	*draw = FALSE;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID helpButton(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID helpButton(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	envRecord *modePtr;
	tHelp topic;
	GRECT r;

	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(info);
	modePtr = env;
	switch (modePtr->mode)
	{
	case gsmReadBegin:
	case gsmReadNew:
	case gsmReadUnread:
	case gsmReadLastPos:
	case gsmReadDate:
	case gsmGroupCom:
	case gsmReadId:
		topic = CatHelp_groupRead;
		break;
	case gsmWrite:
		topic = CatHelp_groupWrite;
		break;
	case gsmUser:
		topic = CatHelp_groupUser;
		break;
	case gsmChief:
	case gsmOther:
	case gsmModify:
		topic = CatHelp_groupsel;
		break;
	case gsmFollowup:			/* Followup setzen		*/
		topic = CatHelp_followup;
		break;
	case gsmPMFilt:			/* PM-FilterGruppe setzen		*/
		topic = CatHelp_pmfilter;
		break;
	default:
		topic = CatHelp_groupRead;
		break;
	}
	CatHelp_DoHelp(topic);
	DelObjState(gruppen, obj, SELECTED);
	WinDials_WinDialDraw(gruppen, obj, 1, &r, FALSE);
	*draw = FALSE;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID updateButton(_VOID *entry, _VOID *env)
{
	_BOOL oneSel;
	group *gr;
	GRECT r;
/*DYJ 5.35n Start*/
	envRecord *modePtr;
	UNUSED(entry);
	modePtr = env;
/*DYJ  Ende; alt:	
	UNUSED(entry);
	UNUSED(env);
*/
	Lists_ResetList(&grList);
	oneSel = Lists_ScanEntries(&grList, Lists_forward, findSel, NULL);
	oneSel = oneSel && (clipEntry == NULL);
	if (oneSel)
	{
		gr = Lists_CurrentEntry(&grList);
		oneSel = oneSel && gr->catNumber != dataSys_private;
	}
	if ((oneSel && (GetObjState(gruppen, GCUT, DISABLED) ||
			GetObjState(gruppen, GCHANGE, DISABLED))) ||
		(!oneSel && (!GetObjState(gruppen, GCUT, DISABLED) ||
			!GetObjState(gruppen, GCHANGE, DISABLED))))
	{
		EnableObjState(gruppen, GCUT, DISABLED, !oneSel);
		WinDials_WinDialDraw(gruppen, GCUT, 0, &r, FALSE);
/*DYJ 5.35n Start*/
		if(modePtr->mode==gsmPMFilt)
		{
			DelObjState(gruppen, GCHANGE, DISABLED);
			if (oneSel)
				SetObjcString(gruppen, GCHANGE, MausTauschrsc_WrkString(ADR_CHANGE));
			else
				SetObjcString(gruppen, GCHANGE, MausTauschrsc_WrkString(ADR_NEW));
			WinDials_WinDialDraw(gruppen, GCHANGE, 0, &r, FALSE);
		}
			EnableObjState(gruppen, GCHANGE, DISABLED, !oneSel && modePtr->mode!=gsmPMFilt );
/*DYJ  Ende; alt:	
		EnableObjState(gruppen, GCHANGE, DISABLED, !oneSel);
*/
		WinDials_WinDialDraw(gruppen, GCHANGE, 0, &r, FALSE);
	}
	oneSel = clipEntry == NULL;
	if ((oneSel && GetObjState(gruppen, GOK, DISABLED)) ||
		(!oneSel && !GetObjState(gruppen, GOK, DISABLED)))
	{
		EnableObjState(gruppen, GOK, DISABLED, !oneSel);
		WinDials_WinDialDraw(gruppen, GOK, 0, &r, FALSE);
		EnableObjFlags(gruppen, GOK, DEFAULT, oneSel);
	}
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID allSelected(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID allSelected(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;

	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(obj);
	UNUSED(env);
	UNUSED(info);
	SetObjcString(gruppen, NEWGROUP, "");
	WinDials_WinDialDraw(gruppen, NEWGROUP, 0, &r, FALSE);
	*draw = FALSE;
	*exit = TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID sortGroupList(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID sortGroupList(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	GRECT r;

	UNUSED(env);
	UNUSED(info);
	*lstSel=-1;	/*DYJ 5.30f */
	GroupSelect_DeleteList(&grList);
	GroupSelect_GetReadList(&grList, FALSE, FALSE, FALSE);
	DelObjState(gruppen, obj, SELECTED);
	WinDials_WinDialDraw(gruppen, obj, 1, &r, FALSE);
	*draw = TRUE;
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.30f Start*/
LOCAL _VOID switchWriteList(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID switchWriteList(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	UNUSED(lstSel);	/*DYJ 5.30f */
	UNUSED(env);
	UNUSED(info);
	GroupSelect_DeleteList(&grList);
	if (GetObjState(gruppen, obj, SELECTED))
	{
		GroupSelect_GetWriteList(&grList, TRUE);
		ConfVars_SetConfigBool(cSmallWriteList, FALSE);
	} else
	{
		/* Nur kleine Liste holen */
		GroupSelect_GetWriteList(&grList, FALSE);
		ConfVars_SetConfigBool(cSmallWriteList, TRUE);
	}
	*draw = TRUE;
	*exit = FALSE;
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID changeGroupX(_WORD obj,_VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)		*/
/*	wie changeGroup in gruppena.c zustzlich wird aber ggf. eine neue Gruppe angelegt	*/
/*	Parameter:	obj:	Button ber den dies aufgerufen wurde					*/
/*				env,info:	werden nicht bentigt								*/
/*				draw:	fr Rckgabe ob neu gezeichnet werden mu				*/
/*				exit:	fr Rckgabe ob Hauptdialog beendet werden soll			*/
/*				lstSel:	?														*/
/*	Ergebnis:	keines															*/		
/*	11.4.04 von Dimitri Junker	DYJ 5.35n										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _VOID changeGroupX(_WORD obj,_VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
{
	GRECT r;
	netState isNet;
	_WORD retGacg;

	UNUSED(info);
	UNUSED(env);
	*lstSel=-1;

	retGacg=GruppenAuswahl_ChangeGroup(&grList,&isNet,FALSE);
	if(retGacg<0)
	{
		group_line groupName;
		ObjcString(gruppen, NEWGROUP, groupName, sizeof(groupName));
		GroupSelect_AppendGroup(groupName);
		GroupSelect_DeleteList(&grList);
		if (GetObjState(gruppen, obj, SELECTED))
			GroupSelect_GetWriteList(&grList, TRUE);
		else			/* Nur kleine Liste holen */
			GroupSelect_GetWriteList(&grList, FALSE);
	}
	*draw=retGacg!=0;
	DelObjState(gruppen, obj, SELECTED);
	WinDials_WinDialDraw(gruppen, obj, 1, &r, FALSE);
	*exit = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
/*DYJ 5.30f Start*/
LOCAL _VOID changeGroup(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit,_LONG *lstSel)
/*DYJ  Ende; alt:	
LOCAL _VOID changeGroup(_WORD obj, _VOID *env, LISTINFO info, _BOOL *draw, _BOOL *exit)
*/
{
	_BOOL oneSel;
	group *gr;
	OBJECT *grchange;
/*DYJ 5.35c Start*/
/*DYJ  Ende; alt:	
	_WORD cancel=GC_CANCEL,gcnew=GCNEW,ggruppe=GGRUPPE;
	netState newtype;
	_UBYTE newFlag;
	group_line oldTxt;
	group_line newTxt;
*/
	group_line oldGruppe;
	group_line gruppe;
	_WORD but;
	_BOOL ok;
	CAT_GROUP_NUMBER nr;
	GRECT r;

	UNUSED(info);
	UNUSED(env);
	*lstSel=-1;	/*DYJ 5.30f */
	*draw = FALSE;
	Lists_ResetList(&grList);
	oneSel = Lists_ScanEntries(&grList, Lists_forward, findSel, NULL);
	if (oneSel)
	{
		gr = Lists_CurrentEntry(&grList);
		strBcpy(gruppe, gr->name);
		strBcpy(oldGruppe, gr->name);
/*DYJ 5.35c Start*/
		grchange = TreeAddr[WEGEN];
		SetObjcStringAdr(grchange, OPTSTR, MausTauschrsc_WrkString(GRCHANGE1), strLen(MausTauschrsc_WrkString(GRCHANGE1)) + 1);
		SetObjcStringAdr(grchange, PRESTR, MausTauschrsc_WrkString(GRCHANGE2), strLen(MausTauschrsc_WrkString(GRCHANGE2)) + 1);
		SetObjcStringAdr(grchange, WGRUPPE, oldGruppe, sizeof(oldGruppe));
		SetObjcString(grchange, WEGENSTR, "");
		do
		{
			but = VDIUtil_FormHandle(grchange);
			*draw = but != WEGEN_CANCEL;
			ok = TRUE;
			if (*draw)
			{
				ObjcString(grchange, WEGENSTR, gruppe, sizeof(gruppe));
/*DYJ  Ende; alt:	
		grchange = TreeAddr[GRCHANGE];
		if(gr->isNet<=nNA)
		{
			if(gr->kurztext)
				strBcpy(newTxt, gr->kurztext);
			else
				*newTxt='\0';
			strBcpy(oldTxt, newTxt);
			SetObjcStringAdr(grchange, GCKTXT, oldTxt, sizeof(oldTxt) + 1);
		}
		switch(gr->isNet)
		{
		case nPrivate:
			DelObjState(grchange, GCMAUS, SELECTED);
			SetObjState(grchange, GCPM, SELECTED);
			DelObjState(grchange, GCOEM, SELECTED);
			break;
		case nOeffentl:
			DelObjState(grchange, GCMAUS, SELECTED);
			DelObjState(grchange, GCPM, SELECTED);
			SetObjState(grchange, GCOEM, SELECTED);
			break;
		case nNA:
			SetObjState(grchange, GCMAUS, SELECTED);
			DelObjState(grchange, GCPM, SELECTED);
			DelObjState(grchange, GCOEM, SELECTED);
			break;
		default:
			cancel=WEGEN_CANCEL;
			gcnew=WEGENSTR;
			ggruppe=WGRUPPE;
			grchange = TreeAddr[WEGEN];
			SetObjcStringAdr(grchange, OPTSTR, MausTauschrsc_WrkString(GRCHANGE1), strLen(MausTauschrsc_WrkString(GRCHANGE1)) + 1);
			SetObjcStringAdr(grchange, PRESTR, MausTauschrsc_WrkString(GRCHANGE2), strLen(MausTauschrsc_WrkString(GRCHANGE2)) + 1);
		}
		SetObjcStringAdr(grchange, ggruppe, oldGruppe, sizeof(oldGruppe));
		SetObjcString(grchange, gcnew, "");
		do
		{
			but = VDIUtil_FormHandle(grchange);
			*draw = but != cancel;
			ok = TRUE;
			if (*draw)
			{
				ObjcString(grchange, gcnew, gruppe, sizeof(gruppe));
*/
				if (gruppe[0] == '\0')
				{
					if(gr->isNet>nNA)
					{
						MTE_infoRSC(AL_EMPTY_GROUP);
						ok = FALSE;
					}
				} else if (GroupSelect_GroupNumber(gruppe, &nr))
				{
					MTE_infoRSC(AL_DOUBLE_GROUP);
					ok = FALSE;
				}
				if(ok)
				{
/*DYJ 5.35c Start*/
					if (!GroupSelect_ChgGroup(gr->name, gruppe,'\0',gr->kurztext, TRUE))
/*DYJ  Ende; alt:	
					if(gr->isNet<=nNA)
					{
						if(GetObjState(grchange, GCMAUS, SELECTED))
						{
							newtype=nNA;
							newFlag='M';
						}
						else
						{
							if(GetObjState(grchange, GCPM, SELECTED))
							{
								newtype=nPrivate;
								newFlag='P';
							}
							else
							{
								newtype=nOeffentl;
								newFlag='O';
							}
						}
						if (gr->isNet==newtype)
							newFlag='\0';
						ObjcString(grchange, GCKTXT, newTxt, sizeof(newTxt));
					}
					if (!GroupSelect_ChgGroup(gr->name, gruppe,newFlag,newTxt, TRUE))
*/
					{
						MTE_infoRSC(AL_NO_REN_GROUP);
					} else
					{
						GroupSelect_DeleteList(&grList);
						GroupSelect_GetReadList(&grList, TRUE, FALSE, FALSE);
					}
				}
			}
		} while (!ok);
		SetObjcStringAdr(grchange, WGRUPPE, "", 1);
/*DYJ 5.34d Start*/
		if(gr->isNet==nOeffentl&& !strChr(gr->followup,'@'))
		{
			_UBYTE ul[500];
			group_line followupTo="";
			sprintf(ul,MausTauschrsc_WrkString(AL_POM_FU),gr->name);
			if(mtAlerts_Alert(1,ul)==1 && adr_SelectName(ADR_MLIST,followupTo, sizeof(followupTo)))
/*DYJ 5.35c Start*/
				GroupSelect_SetGrpPar(gr, followupTo,LS_FOLLOWUP_LINE);
/*DYJ  Ende; alt:	
				GroupSelect_SetFollowUp(gr, followupTo);
*/
		}
/*DYJ  Ende; */
	}

	DelObjState(gruppen, obj, SELECTED);
	WinDials_WinDialDraw(gruppen, obj, 1, &r, FALSE);
	*exit = FALSE;
}
/*DYJ  Ende; alt:	
{
	OBJECT *wegen;
	group_line oldGruppe;
	group_line gruppe;
	_BOOL oneSel;
	_WORD but;
	_BOOL ok;
	group *gr;
	GROUPNR nr;
	GRECT r;

	UNUSED(info);
	UNUSED(env);
	*draw = FALSE;
	wegen = TreeAddr[WEGEN];
	Lists_ResetList(&grList);
	oneSel = Lists_ScanEntries(&grList, Lists_forward, findSel, NULL);
	if (oneSel)
	{
		gr = Lists_CurrentEntry(&grList);
		strBcpy(gruppe, gr->name);
		strBcpy(oldGruppe, gr->name);
		SetObjcStringAdr(wegen, WGRUPPE, oldGruppe, sizeof(oldGruppe));
		SetObjcStringAdr(wegen, OPTSTR, cTitle, strLen(cTitle) + 1);
		SetObjcStringAdr(wegen, PRESTR, cPreStr, strLen(cPreStr) + 1);
		SetObjcString(wegen, WEGENSTR, "");
		do
		{
			but = VDIUtil_FormHandle(wegen);
			*draw = but != WEGEN_CANCEL;
			ok = TRUE;
			if (*draw)
			{
				ObjcString(wegen, WEGENSTR, gruppe, sizeof(gruppe));
				if (gruppe[0] == '\0')
				{
					MTE_info(AL_EMPTY_GROUP);
					ok = FALSE;
				} else if (GroupSelect_GroupNumber(gruppe, &nr))
				{
					MTE_info(AL_DOUBLE_GROUP);
					ok = FALSE;
				} else
				{
					if (!GroupSelect_RenameGroup(gr->name, gruppe, TRUE))
					{
						MTE_info(AL_NO_REN_GROUP);
					} else
					{
						GroupSelect_DeleteList(&grList);
						GroupSelect_GetReadList(&grList, TRUE, FALSE, FALSE);
					}
				}
			}
		} while (!ok);
		SetObjcStringAdr(wegen, WGRUPPE, "", 1);
	}
	DelObjState(gruppen, obj, SELECTED);
	WinDials_WinDialDraw(gruppen, obj, 1, &r, FALSE);
	*exit = FALSE;
}
*/

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* List-Zwischen-Prozeduren */
LOCAL _BOOL groupIsSelected(_VOID *entry, _VOID *env)
{
	group *gr;

	UNUSED(env);
	gr = entry;
	return gr->selected;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *nextEntry(_VOID *list)
{
	return Lists_NextEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *prevEntry(_VOID *list)
{
	return Lists_PrevEntry(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID resetList(_VOID *list)
{
	Lists_ResetList(list);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID countEntries(_VOID *l, _LONG *ll, _WORD *ww)
{
	*ww = globalLength;
	*ll = Lists_NoOfEntries(l);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isEnabled(_VOID *entry, _VOID *env)
{
	UNUSED(entry);
	UNUSED(env);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.16p */	/*DYJ 5.21b */
LOCAL _VOID drawGroupEntry(_VOID *entry, _VOID *env, _WORD x, _WORD y, _WORD offset, CONST GRECT *clip)
{
	group *e;
	group_line str;
	_UBYTE numStr[5];
	_UBYTE unread[5];
	_UBYTE space[3];
	_WORD extent[8];
	GRECT fill;
	_WORD pxy[4];
	_BOOL isSel;
	_UWORD eff;
	_WORD fillcol;
	
	UNUSED(offset);
	e = entry;
	fill.g_x = x;
	fill.g_y = y;
	fill.g_w = fill.g_x + globalLength - 1;
	fill.g_h = fill.g_y + CharHeight - 1;
	pxy[0] = clip->g_x;
	pxy[1] = clip->g_y;
	pxy[2] = clip->g_x + clip->g_w - 1;
	pxy[3] = clip->g_y + clip->g_h - 1;

	vs_clip(PrivateWS, TRUE, pxy);
	vswr_mode(PrivateWS, MD_REPLACE);

	fillcol = WHITE;
	isSel = entry != NULL && groupIsSelected(entry, env);
	if (isSel)
		fillcol = BLACK;
	MouseOff();
	vsf_color(PrivateWS, fillcol);
	vr_recfl(PrivateWS, (_WORD *)&fill);
	
	if (e != NULL)
	{
		envRecord *modePtr;
		modePtr = env;
		if(!strCaseCmp(e->name, Rsc_WrkString(GRP_PRIVATE))|| e->isNet== nPrivate)
			vst_color(PrivateWS, modePtr->pmCol);
		else
			vst_color(PrivateWS, modePtr->omCol);
		eff = TXT_NORMAL;
		if (!isEnabled(entry, env))
			eff |= TXT_LIGHT;
		if (isSel)
			vswr_mode(PrivateWS, MD_XOR);
		vst_effects(PrivateWS, eff);
		/* Jetzt hier den String fuer den Gruppennamen zusammenbauen */
		buildGroup(e, env, str, sizeof(str), unread, space, numStr);
		v_gtext(PrivateWS, x, y, str);
		x += maxNameLength;
		if (unread[0] != '\0')
		{
			vqt_extent(PrivateWS, unread, extent);
			v_gtext(PrivateWS, x + 3 * CharWidth - (extent[2] - extent[0]), y, unread);
		}
		x += 4 * CharWidth;
		v_gtext(PrivateWS, x, y, space);
		x += 2 * CharWidth;
		if (numStr[0] != '\0')
		{
			vqt_extent(PrivateWS, numStr, extent);
			v_gtext(PrivateWS, x + 3 * CharWidth - (extent[2] - extent[0]), y, numStr);
		}
		vst_effects(PrivateWS, 0);
		vswr_mode(PrivateWS, MD_REPLACE);
		vst_color(PrivateWS, BLACK);
	}
	vs_clip(PrivateWS, FALSE, pxy);
	MouseOn();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID setAndGetGroupValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exitBut)
{
	envRecord *iPtr;

	UNUSED(tree);
	exitBut &= 0x7fff;
	if (set)
	{
	} else
	{
		iPtr = env;
		iPtr->exitBut = exitBut;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL CheckStr(_UBYTE *str)
{
	strleft(str);
	strdelRightSpace(str);
	return *str != '\0';
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
LOCAL _BOOL checkGroupExit(OBJECT *tree, _VOID *env, _WORD button, _VOID *group, _UWORD kstate)
{
	envRecord *iPtr;
	group_line groupName;
	_BOOL GroupOk;
	CAT_GROUP_NUMBER nr;
	GRECT r;

	UNUSED(kstate);
	iPtr = env;
	button &= 0x7fff;
	if (button == GOK)
	{
		ObjcString(tree, NEWGROUP, groupName, sizeof(groupName));
		if (group == NULL && iPtr->mode != gsmModify && iPtr->mode != gsmUser && iPtr->mode != gsmChief && iPtr->mode != gsmUsername && iPtr->mode != gsmProgteil)
		{
			/* Hier noch aendern!!! */
			if (!CheckStr(groupName))
			{
				if(iPtr->mode==gsmFollowup)
					GroupOk = TRUE;
				else
				{
					MTE_infoRSC(AL_EMPTY_GNAME);
					GroupOk = FALSE;
				}
			} else  /* IF ~nG THEN */
			{
				GroupOk = (iPtr->mode==gsmFollowup)||(iPtr->mode==gsmPMFilt)||GroupSelect_GroupNumber(groupName, &nr);
				if (!GroupOk /* && iPtr->isReadMode */ )
				{
					MTE_infoRSC(AL_GROUP_NOT_FOUND);
				}
			}
		} else
		{
			GroupOk = TRUE;
		}
		if (!GroupOk)
		{
			DelObjState(tree, button, SELECTED);
			WinDials_WinDialDraw(tree, button, 0, &r, FALSE);
		}
		return GroupOk;
	}
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/


/* Gruppenauswahl...*/
/*DYJ 5.16p */ /*DYJ 5.21b */
GLOBAL _BOOL ListHelp_SelectGroup(_UBYTE *gruppe, size_t strsize, CAT_GROUP_NUMBER *Nr, _BOOL nG, _BOOL clearEdit, _BOOL forSearch, groupSelectMode mode)
{
	_WORD exit;
	specialButHdler specials[6];
	group *gr, *group;
	_BOOL found;
	_BOOL GroupOk;
	_WORD maxLength;
	GRECT r;
	inLoopProc ilProc;
	_BOOL whichList;
	envRecord envRec;
	ldHandler dialHandler;
	_BOOL ok;
	ldElems elems;
	_BOOL autoClear;
	_WORD extent[8];
	_WORD width;
	_WORD intval;
	
	if (groupDialIsOpen)
		return FALSE;

	MausTauschrsc_AdjustDialogSize(gruppen, GRUPPEN);
	
	/* Strings setzen */
	SetObjcString(gruppen, GTITEL, MausTauschrsc_WrkString(titles[mode]));
	SetObjFlags(gruppen, GOK, DEFAULT);
	/* Gruppenmodus bestimmen */
	envRec.mode = mode;
	ConfVars_GetConfDefInt(cGroupListPmCol, &envRec.pmCol, BLUE);
	ConfVars_GetConfDefInt(cGroupListOmCol, &envRec.omCol, BLACK);
	switch (mode)
	{
	case gsmReadBegin:
	case gsmReadNew:
	case gsmReadUnread:
	case gsmReadLastPos:
	case gsmReadDate:
	case gsmReadId:
		envRec.isReadMode = TRUE;
		break;
	case gsmPMFilt:			/* Gruppe fr PM-Filter*/
		strMcpy(lastGroup,sizeof(lastGroup),gruppe);
	default:
		envRec.isReadMode = FALSE;
		break;
	}

/*DYJ 5.35n Start*/
	EnableObjFlags(gruppen, GCHANGE, HIDETREE, mode != gsmModify && mode != gsmPMFilt );
/*DYJ  Ende; alt:	
	EnableObjFlags(gruppen, GCHANGE, HIDETREE, mode != gsmModify);
*/
	EnableObjFlags(gruppen, GNEW,    HIDETREE, TRUE);
	EnableObjFlags(gruppen, GDELETE, HIDETREE, TRUE);
	EnableObjFlags(gruppen, GCUT   , HIDETREE, mode != gsmModify);
	EnableObjFlags(gruppen, GRCEMPTY, HIDETREE, mode != gsmModify);
	/* EnableObjFlags(gruppen, GHELP, HIDETREE, mode != gsmModify); */
	EnableObjFlags(gruppen, GRCFULL, HIDETREE, TRUE);
	EnableObjFlags(gruppen, GBALL,   HIDETREE, mode != gsmWrite && mode != gsmGroupCom);
	EnableObjFlags(gruppen, GBALL,   EXIT, mode == gsmGroupCom);
	EnableObjFlags(gruppen, GBALL,   TOUCHEXIT, mode == gsmWrite);
	EnableObjFlags(gruppen, GSORT,   HIDETREE, mode != gsmModify);
	EnableObjFlags(gruppen, GSORT,   EXIT, mode == gsmModify);
	EnableObjFlags(gruppen, GSORT,   TOUCHEXIT, FALSE);

	if (mode == gsmWrite)
	{
		/* Nachsehen, welche Liste wir zuerst anfordern */
		ConfVars_GetConfDefBool(cSmallWriteList, &whichList, TRUE);
		EnableObjState(gruppen, GBALL, SELECTED, !whichList);
		ConfVars_SetConfigBool(cSmallWriteList, whichList);
	} else if (mode == gsmModify)
	{
		DelObjFlags(gruppen, GSORT, HIDETREE);
		SetObjFlags(gruppen, GBALL, HIDETREE);
		EnableObjState(gruppen, GSORT, SELECTED, FALSE);
	}

	/* Editfeld loeschen */
	if (clearEdit)
		SetObjcString(gruppen, NEWGROUP, "");
	else
		SetObjcString(gruppen, NEWGROUP, lastGroup);
	/* specials bestimmen */
	specials[0].objc = GCUT;
	specials[0].proc = cutEntry;
	specials[1].objc = GRCFULL;
	specials[1].proc = pasteEntry;
	specials[2].objc = GRCEMPTY;
	specials[2].proc = clipEmpty;
	specials[3].objc = mode == gsmModify ? GSORT : GBALL;
	if (mode == gsmWrite)
		specials[3].proc = switchWriteList;
	else if (mode == gsmModify)
		specials[3].proc = sortGroupList;
	else
		specials[3].proc = allSelected;
	specials[4].objc = GCHANGE;
/*DYJ 5.35n Start*/
	if (mode == gsmPMFilt)
		specials[4].proc = changeGroupX;
	else
		specials[4].proc = changeGroup;
/*DYJ  Ende; alt:	
	specials[4].proc = changeGroup;
*/
	specials[5].objc = GHELP;
	specials[5].proc = helpButton;
	clipEntry = NULL;
	if (!forSearch)
		ilProc = updateButton;
	else
		ilProc = ListDl_dummyInLoop;

	/* Jetzt die Liste anfordern beim Listenmodul */
	if (mode == gsmUser || mode == gsmChief)
		ok = GroupSelect_GetUserList(&grList);
	else if (mode == gsmProgteil)
		ok = GroupSelect_GetProgteilList(&grList);	/* (MH) */
	else if (mode == gsmOther)
		ok = GroupSelect_GetSelectList(&grList, dAll, sName);
	else if (mode == gsmUsername)	/* (MH) */
		ok = GroupSelect_GetUsernameList(&grList);
/*DYJ 5.36c Start*/
	else if (envRec.isReadMode || forSearch || mode == gsmModify || mode==gsmPutz)
/*DYJ  Ende; alt:	
	else if (envRec.isReadMode || forSearch || mode == gsmModify)
*/
		ok = GroupSelect_GetReadList(&grList, TRUE, mode == gsmReadNew, mode == gsmReadUnread);
	else if (mode == gsmWrite)
		ok = GroupSelect_GetWriteList(&grList, !whichList);
	else
		ok = GroupSelect_GetWriteList(&grList, TRUE);
	if (!ok)
		return FALSE;

	/* Maximale Breite eines Listeneintrages herausfinden */
	maxLength = 0;
	Lists_ResetList(&grList);
	while ((group = Lists_NextEntry(&grList)) != NULL)
	{
		vqt_extent(PrivateWS, group->display_name, extent);
		width = extent[2] - extent[0];
		if (width > maxLength)
			maxLength = width;
	}
	maxLength += 10 * CharWidth;   /* fuer neue und ungelesene */

	/* Jetzt das Object zusammenbauen */
	CalcArea(gruppen, GROUPBOX, &r);
	maxLength = Max(maxLength, r.g_w);
	maxNameLength = r.g_w - 10 * CharWidth;
	
	globalLength = maxLength;

	groupDialIsOpen = TRUE;

	/* WindowDialog vorbereiten */
	ListDl_BuildLdHandler(&grList,
		resetList,
		nextEntry,
		prevEntry,
		countEntries,
		isEnabled,
		selGrEntry,
		groupToStr,
		groupIsSelected,
		drawGroupEntry,
		FUNK_NULL,
		0, CharHeight,
		8, globalLength,
		&dialHandler);

		ConfVars_GetConfDefInt(cLocateGruMode, &intval, 1);
	/* ListWinDialog oeffnen */
	elems = ldSelect | ldArrows | ldDoubleExit | ldAutolocate | ldModal;
	if(intval)	/* sucht irgendwo im Gruppennamen*/
		elems |= ldTabEqNxt|ldMatchAny;
	ConfVars_GetConfDefBool(cAutoClearEdit, &autoClear, FALSE);
	if (autoClear)
		 elems |= ldClearEdit;
	if (ListDl_WinListDial(gruppen, elems,
		&dialHandler,
		GROUPBOX,
		GROUPBACK,
		GOK,
		GCANCEL,
		&envRec,  /* env */
		GOK,      /* doubleClickButton */
		NEWGROUP, /* autoEdit */
		ilProc,
		specials,
		6,
		checkGroupExit,
		setAndGetGroupValues,
		setAndGetGroupValues,
		(_VOID **)&group))
	{
		exit = envRec.exitBut;
		if (exit == GOK && mode == gsmModify)
		{
			CAT_GROUP_NUMBER nr;
			
			/* Jetzt neue Lesenummern vergeben */
			nr = dataSys_private; /* Da Persoenliche mit drin ist */
			Lists_ResetList(&grList);
			while ((gr = Lists_NextEntry(&grList)) != NULL)
			{
				gr->readNumber = nr;
				++nr;
			}
			GroupSelect_UpdateReadList(&grList);
		}

		if (exit != GCANCEL)
		{
			found = TRUE;
			if ((exit == GBALL || exit == GSORT) && forSearch)
			{
				*gruppe = '\0';
				*Nr = (CAT_GROUP_NUMBER)dataSys_empty;
			} else if (exit == GOK)
			{
				if (group == NULL)
				{
					ObjcString(gruppen, NEWGROUP, gruppe, strsize);
					if (mode != gsmUsername)
					{
						/* Gruppennamen normalisieren */
						CheckStr(gruppe);
						GroupOk = GroupSelect_GroupNumber(gruppe, Nr); /* !!!ALIAS */
						if (!GroupOk)
						{
							if (strCaseCmp(Rsc_WrkString(GRP_PRIVATE), gruppe) == 0)
							{
								*Nr = dataSys_private;
							} else if (!nG)
							{
								*Nr = NO_GROUP;		/* darf so nicht bleiben..??? */
								found = FALSE;
								/* Soll andeuten, dass es sich um eine neue, unbekannte Gruppe handelt */
							} /* sonst ist Nr schon richtig */
						}
					}
				} else
				{
					strMcpy(gruppe, strsize, group->name);
					*Nr = group->catNumber;
				}
			} else
			{
				found = FALSE;
			}
		} else
		{
			found = FALSE;
			if (mode != gsmUsername)	/* (MH) */
			{
				ObjcString(gruppen, NEWGROUP, gruppe, strsize);
			}
		}
		if (group != NULL && group->alias != NULL && Alias_ShowAlias(group->name,TRUE))
			ListHelp_SetLastGroup(group->display_name);
		else
			ListHelp_SetLastGroup(gruppe);
	} else
	{
		found = FALSE;
	}
	groupDialIsOpen = FALSE;
	/* Gruppenliste wieder freigeben */
	if (clipEntry != NULL)
	{
		Lists_AppendEntry(&grList, clipEntry);
		clipEntry = NULL;
	}
	GroupSelect_DeleteList(&grList);
	return found;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListHelp_SetLastGroup(CONST _UBYTE *name)
{
	strBcpy(lastGroup, name);
}

/*** ---------------------------------------------------------------------- ***/

/* Objektbaeume fuer die Gruppenauswahl- und die Adressenauswahlbox */
GLOBAL _VOID ListHelp_Init(_VOID)
{
	gruppen = TreeAddr[GRUPPEN];
}
