/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <searchhe.h>
#include <confvars.h>
#include <varname4.h>

#include <routinec.h>


GLOBAL SearchRecordType suchVar = {
	{ "", "", "", "" },
	{ vUND, vUND, vODER },
	{ sUEBERALL, sUEBERALL, sUEBERALL, sUEBERALL },
	TRUE,		/* show */
	FALSE,		/* change */
	FALSE,		/* statussearch */
	TRUE,		/* ask */
	TRUE,		/* searchtext */
	TRUE,		/* gross */
	FALSE,		/* groupsearch */
	0,			/* sSetBits */
	0,			/* sClearedBits */
	0,			/* setBits */
	0			/* clearBits */
};

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _BOOL SearchHelp_IsValidSearch(CONST _UBYTE *str)
{
	/* Jetzt der eigentliche Test */
	if (*str != '\0')
	{
		while (*str == '*' || *str == '?')
			str++;
		if (*str == '\0')
			return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL SearchHelp_ValidateKonf(SearchRecordType *konf)
{
	_WORD i;

	/* Nur bei grosser Suchbox: Flageinstellungen ueberpruefen */
	
	/* Erster Test: Suchstring leer und Textsuche */
	if (konf->searchStr[0][0] == '\0' && konf->searchText)
		return FALSE;
	if (!konf->searchText && !konf->statussearch)
		return FALSE;
	/* Entweder Show oder Change muessen an sein */
	if (!konf->show && !konf->change)
		return FALSE;
	if (konf->statussearch)
	{
		if ((konf->sSetBits | konf->sClearedBits) == 0)
			return FALSE;
		if (konf->change && (konf->setBits | konf->clearBits) == 0)
			return FALSE;
	}
	
	if (konf->searchText)
	{
		for (i = 0; i < suchstringAnz; i++)
		{
			if (!SearchHelp_IsValidSearch(konf->searchStr[i]))
				return FALSE;
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Suchkonfiguration <nr> abrufen, FALSE -> nicht gesetzt */

LOCAL _BOOL getsetKonf(_BOOL set, MSGNR nr, SearchRecordType *konf)
{
	_UBYTE varname[256];
	_UBYTE varname2[256];
	_UWORD z;
	_WORD scrap;
	_BOOL OneWasThere;
	_BOOL writeOk;

	OneWasThere = FALSE;
	writeOk     = TRUE;
	if (nr >= MAX_SEARCHKONF)
		return FALSE;
	
	make_varname(varname, sizeof(varname), cSearchName, nr);
	make_varname(varname2, sizeof(konf->Name), "Konf", nr+1);
	if (set)
	{
		if(*konf->Name)
			writeOk &= ConfVars_SetConfigString(varname, konf->Name);
		else
			writeOk &= ConfVars_SetConfigString(varname, varname2);
	} else
	{
		ConfVars_GetConfDefStr(varname,konf->Name, sizeof(konf->Name),varname2);
	}
	make_varname(varname, sizeof(varname), cSearchStr, nr);
	strBcat(varname, ".");
	for (z = 0; z < suchstringAnz; z++)
	{
		make_varname(varname2, sizeof(varname2), varname, z);
		if (set)
		{
			writeOk &= ConfVars_SetConfigString(varname2, konf->searchStr[z]);
		} else
		{
			if (ConfVars_GetConfigString(varname2, konf->searchStr[z], sizeof(konf->searchStr[z])))
			{
				OneWasThere |= konf->searchStr[z][0] != '\0';
			} else
			{
				konf->searchStr[z][0] = '\0';
			}
		}
	}
	
	make_varname(varname, sizeof(varname), cSearchLogical, nr);
	strBcat(varname, ".");
	for (z = 0; z < suchstringAnz - 1; z++)
	{
		make_varname(varname2, sizeof(varname2), varname, z);
		if (set)
		{
			writeOk &= ConfVars_SetConfigInt(varname2, konf->verkn[z]);
		} else
		{
			if (ConfVars_GetConfigInt(varname2, &konf->verkn[z]))
				OneWasThere = TRUE;
			else
				konf->verkn[z] = vUND;
		}
	}
	
	make_varname(varname, sizeof(varname), cSearchLocation, nr);
	strBcat(varname, ".");
	for (z = 0; z < suchstringAnz; z++)
	{
		make_varname(varname2, sizeof(varname2), varname, z);
		if (set)
		{
			writeOk &= ConfVars_SetConfigInt(varname2, konf->wo[z]);
		} else
		{
			if (ConfVars_GetConfigInt(varname2, &konf->wo[z]))
				OneWasThere = TRUE;
			else
				konf->wo[z] = sUEBERALL;
		}
	}
	
#define getsetBool(konfname, name, def) \
	make_varname(varname, sizeof(varname), konfname, nr); \
	if (set) \
	{ \
		writeOk &= ConfVars_SetConfigBool(varname, konf->name); \
	} else \
	{ \
		if (ConfVars_GetConfigBool(varname, &konf->name)) \
			OneWasThere = TRUE; \
		else \
			konf->name = def; \
	}
	
	getsetBool(cSearchShow, show, TRUE);
	getsetBool(cSearchChange, change, FALSE);
	getsetBool(cSearchStatus, statussearch, FALSE);
	getsetBool(cSearchAsk, ask, TRUE);
	getsetBool(cSearchText, searchText, TRUE);
	getsetBool(cSearchCasesensitive, gross, TRUE);
	getsetBool(cSearchAllGroups, groupsearch, FALSE);
#undef getsetBool
	
#define getsetBits(konfname, name, def) \
	make_varname(varname, sizeof(varname), konfname, nr); \
	if (set) \
	{ \
		writeOk &= ConfVars_SetConfigInt(varname, konf->name); \
	} else \
	{ \
		if (ConfVars_GetConfigInt(varname, &scrap)) \
		{ \
			konf->name = scrap; \
			OneWasThere = TRUE; \
		} else \
		{ \
			konf->name = def; \
		} \
	}
	
	getsetBits(cSearchSetBits, sSetBits, 0);
	getsetBits(cSearchClearedBits, sClearedBits, 0);
	getsetBits(cSearchToSetBits, setBits, 0);
	getsetBits(cSearchToClearBits, clearBits, 0);
#undef getsetBits
	
	if (set)
		return writeOk;
	return OneWasThere;
}

/*** ---------------------------------------------------------------------- ***/

/* Suchkonfiguration <nr> abrufen, FALSE -> nicht gesetzt */

GLOBAL _BOOL SearchHelp_getKonf(MSGNR nr, SearchRecordType *konf)
{
	return getsetKonf(FALSE, nr, konf);
}

/*** ---------------------------------------------------------------------- ***/

/* Suchkonfiguration <nr> setzen */
GLOBAL _BOOL SearchHelp_setKonf(_UWORD nr, SearchRecordType *konf)
{
	return getsetKonf(TRUE, nr, konf);
}
