/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 */


#include <treelist.h>
#include <data.h>
#include <edittype.h>
#include <mte.h>
#include <vdiutil.h>
#include <maustaus.h>
#include <cattypes.h>
#include <handlepo.h>
#include <confvars.h>
#include <varname1.h>
#include <wdwmanag.h>
#include <fontsele.h>
#include <catgloba.h>
#include <grin.h>
#include <groupsel.h>
#include <catutil.h>

#include <mtappl.h>
#include <mtarea.h>
#include <mtutils.h>
#include <mtdials.h>
#include <mtpopups.h>

#include <keyboard.h>
#include <rectfunc.h>
#include <routinec.h>

#include <mtdir.h>
#include <protokol.h>
#include <uudecode.h>
#include <catfiles.h>
#include <catvers.h>
#include <varn_ede.h>
#include <msglist.h>
#include <userinfo.h>
#include <messages.h>
#include <routines.h>
#include <time.h>
#include <mtrsc.h>


/*-- Konstanten ------------------------------------------------------------*/

#define cElemLines  2
#define cElemHDist  8
#define cElemWDist  32

#define cMinX   8
#define cMinY   8
#define CR '\r'
#define LF '\n'

/*-- Typdefinitionen -------------------------------------------------------*/

typedef enum
{
	zmNormal,
	zmZoom
} zoomModeType;

typedef struct
{
	_WORD x;
	_WORD y;
} Point;
typedef struct
{
	_LONG x;
	_LONG y;
} LongPnt;


typedef struct
{
	Point p1;
	Point p2;
} tLine;




typedef struct
{
	LongPnt p1;
	LongPnt p2;
} tLongLine;

typedef _UBYTE tShortText[20];
typedef struct _treeElem
{
	MSGNR msgNum;
	_BOOL selected;
	_WORD width;
	LGRECT area;
	MSGBITS flags;						/* Die Bits der Nachricht */
	tShortText name;
	tShortText betreff;
	MSGNR up;
	MSGNR down;
	MSGNR left;
	MSGNR right;						/* Indizes in treeArray */
} treeElem;

typedef treeElem *treeArrayPtr;

typedef _LONG *treeOffsetArrayPtr;

typedef struct _treeOffsetType
{
	_LONG size;
	treeOffsetArrayPtr array;
} treeOffsetType;
typedef enum
{
	tVertical,
	tHorizontal
} treeBuildType;
typedef struct _treeWindow
{
	HWND wdw;							/* Windowhandle */
	HVDI vdi;							/* VDI-Handle */
	HWND grinWindow;					/* Das zugehoerige Anzeigefenster */
	GRECT butWork;						/* Rechteck des Objektbaumes */
	GRECT treeWork;						/* Workrectangle des Fensters */
	_WORD number;						/* Nummer des Fensters im Modul */
	_WORD font;
	_WORD fontSize;						/* Font && Groesse */
	LGRECT document;
	oneHandlePtr grHandle;				/* Gruppenhandle */
	_UBYTE grName[256];					/* Gruppenname fuer Titelzeile */
	_WORD charHeight;					/* Zeichensatzvariablen */
	_WORD charWidth;
	_BOOL isFSM;
	treeBuildType treeType;
	zoomModeType zoomMode;
	/* Jetzt die Daten fuer den Baum */
	_WORD elemHeight;
	_WORD elemDist;
	_WORD elemWDist;					/* Groessen und Abstaende der einzelnen Objekte */
	_WORD elemHDist;					/* Groessen und Abstaende der einzelnen Objekte */
	MSGNR treeElems;
	treeArrayPtr tree;
	treeOffsetType *vertical;
	treeOffsetType *horizontal;

	_WORD treeTextCol;
	_WORD treeBackCol;
	_WORD treeLineCol;
	_WORD treeNotReadCol;
	_WORD treeNotReadEff;
	GUEKINF guekInf;
} treeWindow, *treeWdwPtr;


/*-- Globale Variablen -----------------------------------------------------*/

LOCAL List windows;
LOCAL _WORD maxUpLevels = 20;			/* Spaeter durch Configvariablen auslesen */
LOCAL _WORD maxTreeDepth = 40;			/* " */
LOCAL SET treeNums;
LOCAL _WORD globalNumber;
LOCAL MSGNR treeCounter;
LOCAL treeArrayPtr globalTree;
GLOBAL _WORD lastSaveMode = smListe;	

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-------------- Suchfunktionen in Liste -----------------------*/

/* Abbruchprozedur, wie in <Lists> gefordert */
LOCAL _BOOL FindWinCond(_VOID *e, _VOID *i)
{
	return ((treeWdwPtr)e)->wdw == *((HWND *)i);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL treeWdwPtr FindWind(HWND wdw)
{
	_VOID *ptr;

	if (!handlePool_FindEntry(&wdw, FindWinCond, &windows, &ptr))
		return NULL;
	return ptr;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD FindNum(_VOID)
{
	_WORD i;

	if (globalNumber >= 0 && !setin(treeNums, globalNumber))
	{
		setincl(treeNums, globalNumber);
		i = globalNumber;
		globalNumber = -1;
		return i;
	}
	for (i = 0; i < SETMAX; i++)
	{
		if (!setin(treeNums, i))
		{
			setincl(treeNums, i);
			return i;
		}
	}
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ClearNum(_WORD num)
{
	setexcl(treeNums, num);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-- Baum positionieren ------------------------------------------------*/

/*-- Subfunktionen, die dynamische Array verwalten ---------------------*/

LOCAL treeOffsetType *createOffset(_LONG size)
{
	treeOffsetType *offset;

	offset = NEW(treeOffsetType, "createOffset: offset");
	if (offset == NULL)
		return NULL;
	offset->size = size;
	offset->array = M0ALLOC(size * sizeof(*(offset->array)), "createOffset: array");
	if (offset->array == NULL)
	{
		OFREE(offset);
		return NULL;
	}
	return offset;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID freeOffset(treeOffsetType *offset)
{
	SFREE(offset->array);
	OFREE(offset);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID resetOffset(treeOffsetType *offset, _LONG minOff)
{
	_LONG i;

	for (i = 0; i < offset->size; i++)
		offset->array[i] = minOff;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _LONG currentPosition(treeOffsetType *offset, _LONG position)
{
	if (position >= offset->size)
		return 0;
	return offset->array[position];
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID setCurrentPosition(treeOffsetType *offset, _LONG index, _LONG value)
{
	treeOffsetArrayPtr newArray;
	_LONG newSize;

	if (index >= offset->size)
	{
		/* Realloc, Array vergroessern */
		newSize = index + index / 2;
		newArray = MALLOC(newSize * sizeof(*newArray), "setCurrentPosition");
		if (newArray == NULL)
		{
			return;
		}
		MemSetZero(&newArray[offset->size], (newSize - offset->size) * sizeof(*newArray));
		MemCpy(newArray, offset->array, offset->size * sizeof(*newArray));
		SFREE(offset->array);
		offset->size = newSize;
		offset->array = newArray;
	}
	offset->array[index] = value;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _LONG sumOfPositions(treeOffsetType *offset, _LONG index)
{
	_LONG sum;
	_LONG i;
	_LONG stop;

	sum = 0;
	if (index > offset->size)
		stop = offset->size;
	else
		stop = index;
	for (i = 0; i < stop; i++)
	{
		sum += offset->array[i];
	}
	return sum;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Routinen fuer Verticalen (links nach rechts) Baum */

/*-- Horizontale Position setzen ---------------------------------------*/

LOCAL _VOID setPositionsVertical(
	treeWdwPtr ptr, MSGNR idx, _LONG level,
	_LONG *maxWidth, _LONG *maxHeight)
{
	/* Addiere die Summe aller Breiten der Knoten bis zu dieser Tiefe und
	 * benutze es als x-Position
	 */
	ptr->tree[idx].area.x = (level * ptr->elemHDist) + sumOfPositions(ptr->horizontal, level) + cMinX;
	ptr->tree[idx].area.w = currentPosition(ptr->horizontal, level);
	/* Jetzt maxHeight und maxWidth pruefen
	 */
	*maxWidth = LMax(*maxWidth, ptr->tree[idx].area.x + ptr->tree[idx].area.w);
	*maxHeight = LMax(*maxHeight, ptr->tree[idx].area.y + ptr->tree[idx].area.h);
	/* Jetzt werden die Koordinaten von allen subTrees gesetzt */
	idx = ptr->tree[idx].down;
	while (idx < dataSys_notSaved)
	{
		setPositionsVertical(ptr, idx, level + 1, maxWidth, maxHeight);
		idx = ptr->tree[idx].right;
	}
}

/*-- Subtree nach unten verschieben ------------------------------------*/

LOCAL _VOID shiftSubtreeVertical(treeWdwPtr ptr, MSGNR idx, _LONG dif)
{
	MSGNR i;

	ptr->tree[idx].area.y += dif;
	i = ptr->tree[idx].down;
	while (i < dataSys_notSaved)
	{
		shiftSubtreeVertical(ptr, i, dif);
		i = ptr->tree[i].right;
	}
}

/*-- Einen Node positionieren ------------------------------------------*/

LOCAL _LONG computePositionsVertical(treeWdwPtr ptr, MSGNR idx, _LONG level)
{
	_LONG currentHpos;
	_LONG currentVpos;
	MSGNR firstKid;
	MSGNR lastKid;
	MSGNR i;
	_LONG j;
	_LONG depth;
	_LONG offset;
	_LONG pos;
	_LONG top;
	_LONG bottom;

	depth = 0;
	/* Get the current positions for this level */
	currentHpos = currentPosition(ptr->horizontal, level);
	currentVpos = currentPosition(ptr->vertical, level);
	/* set the current horizontal width to the max widths of all
	 * elements at this level
	 */
	setCurrentPosition(ptr->horizontal, level, LMax(currentHpos, ptr->tree[idx].area.w));
	if (ptr->tree[idx].down >= dataSys_notSaved)
	{
		ptr->tree[idx].area.y = currentVpos;
	} else
	{
		/* if the node has subnodes, recursively figure the positions of each
		 * subnode
		 */
		i = ptr->tree[idx].down;
		while (i < dataSys_notSaved)
		{
			depth = LMax(computePositionsVertical(ptr, i, level + 1), depth);
			i = ptr->tree[i].right;
		}
		/* now that the vertical positions of all children are
		 * known, find the vertical extent of all subnodes
		 */
		i = ptr->tree[idx].down;
		firstKid = lastKid = i;
		while (i < dataSys_notSaved)
		{
			lastKid = i;
			i = ptr->tree[i].right;
		}
		top = ptr->tree[firstKid].area.y;
		bottom = ptr->tree[lastKid].area.y + ptr->tree[lastKid].area.h - 1;
		ptr->tree[idx].area.y = (top + bottom - (ptr->elemHeight - 1)) / 2;
		/* if this position is less than the next available
		 * position, correct it to be the next available
		 * position, calculate the amount by which all subnodes
		 * must be shifted, and shift the entire sub-tree.
		 */
		if (ptr->tree[idx].area.y < currentVpos)
		{
			offset = currentVpos - ptr->tree[idx].area.y;
			i = ptr->tree[idx].down;
			while (i < dataSys_notSaved)
			{
				shiftSubtreeVertical(ptr, i, offset);
				i = ptr->tree[i].right;
			}
			/* Adjust the next available space at all levels below
			 * the current level
			 */
			for (j = level + 1; j <= depth; j++)
			{
				pos = currentPosition(ptr->vertical, j);
				setCurrentPosition(ptr->vertical, j, pos + offset);
			}
			/* set the nodes position
			 */
			ptr->tree[idx].area.y = currentVpos;
		}
	}
	/* Record the current vertical position at this level
	 */
	setCurrentPosition(ptr->vertical, level, ptr->tree[idx].area.y + ptr->tree[idx].area.h + ptr->elemDist);
	return LMax(depth, level);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Routinen fuer Horizontalen (oben nach unten) Baum */

/*-- Horizontale Position setzen ---------------------------------------*/

LOCAL _VOID setPositionsHorizontal(
	treeWdwPtr ptr, MSGNR idx, _LONG level,
	_LONG *maxWidth, _LONG *maxHeight)
{
	/* Addiere die Summe aller Hoehen der Knoten bis zu dieser Tiefe und
	 * benutze es als y-Position
	 */
	ptr->tree[idx].area.y = (level * ptr->elemHDist) + sumOfPositions(ptr->vertical, level) + cMinY;
	/* Jetzt maxHeight und maxWidth pruefen
	 */
	*maxWidth = LMax(*maxWidth, ptr->tree[idx].area.x + ptr->tree[idx].area.w);
	*maxHeight = LMax(*maxHeight, ptr->tree[idx].area.y + ptr->tree[idx].area.h);
	/* Jetzt werden die Koordinaten von allen subTrees gesetzt */
	idx = ptr->tree[idx].down;
	while (idx < dataSys_notSaved)
	{
		setPositionsHorizontal(ptr, idx, level + 1, maxWidth, maxHeight);
		idx = ptr->tree[idx].right;
	}
}

/*-- Subtree nach unten verschieben ------------------------------------*/

LOCAL _VOID shiftSubtreeHorizontal(treeWdwPtr ptr, MSGNR idx, _LONG dif)
{
	MSGNR i;

	ptr->tree[idx].area.x += dif;
	i = ptr->tree[idx].down;
	while (i < dataSys_notSaved)
	{
		shiftSubtreeHorizontal(ptr, i, dif);
		i = ptr->tree[i].right;
	}
}

/*-- Einen Node positionieren ------------------------------------------*/

LOCAL _LONG computePositionsHorizontal(treeWdwPtr ptr, MSGNR idx, _LONG level)
{
	_LONG currentHpos;
	_LONG currentVpos;
	MSGNR firstKid;
	MSGNR lastKid;
	MSGNR i;
	_LONG j;
	_LONG depth;
	_LONG offset;
	_LONG pos;
	_LONG top;
	_LONG bottom;

	depth = 0;
	/* Get the current positions for this level */
	currentHpos = currentPosition(ptr->horizontal, level);
	currentVpos = currentPosition(ptr->vertical, level);
	/* set the current horizontal width to the max widths of all
	 * elements at this level
	 */
	setCurrentPosition(ptr->vertical, level, LMax(currentVpos, ptr->tree[idx].area.h));
	if (ptr->tree[idx].down >= dataSys_notSaved)
	{
		ptr->tree[idx].area.x = currentHpos;
	} else
	{
		/* if the node has subnodes, recursively figure the positions of each
		 * subnode
		 */
		i = ptr->tree[idx].down;
		while (i < dataSys_notSaved)
		{
			depth = LMax(computePositionsHorizontal(ptr, i, level + 1), depth);
			i = ptr->tree[i].right;
		}
		/* now that the vertical positions of all children are
		 * known, find the vertical extent of all subnodes
		 */
		i = ptr->tree[idx].down;
		firstKid = lastKid = i;
		while (i < dataSys_notSaved)
		{
			lastKid = i;
			i = ptr->tree[i].right;
		}

		top = ptr->tree[firstKid].area.x;
		bottom = ptr->tree[lastKid].area.x + ptr->tree[lastKid].area.w - 1;
		ptr->tree[idx].area.x = (top + bottom - (ptr->tree[idx].area.w - 1)) / 2;
		/* if this position is less than the next available
		 * position, correct it to be the next available
		 * position, calculate the amount by which all subnodes
		 * must be shifted, and shift the entire sub-tree.
		 */
		if (ptr->tree[idx].area.x < currentHpos)
		{
			offset = currentHpos - ptr->tree[idx].area.x;
			i = ptr->tree[idx].down;
			while (i < dataSys_notSaved)
			{
				shiftSubtreeHorizontal(ptr, i, offset);
				i = ptr->tree[i].right;
			}
			/* Adjust the next available space at all levels below
			 * the current level
			 */
			for (j = level + 1; j <= depth; j++)
			{
				pos = currentPosition(ptr->horizontal, j);
				setCurrentPosition(ptr->horizontal, j, pos + offset);
			}
			/* set the nodes position
			 */
			ptr->tree[idx].area.x = currentHpos;
		}
	}
	/* Record the current vertical position at this level
	 */
	setCurrentPosition(ptr->horizontal, level, ptr->tree[idx].area.x + ptr->tree[idx].area.w - 1 + ptr->elemDist);
	return LMax(depth, level);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID treeList_GetSettings(treeWdwPtr ptr)
{
	_WORD unreadEff;
	
	ConfVars_GetConfDefInt(cTreeTextCol, &ptr->treeTextCol, BLACK);
	ConfVars_GetConfDefInt(cTreeBackCol, &ptr->treeBackCol, WHITE);
	ConfVars_GetConfDefInt(cTreeSubjChangeCol, &ptr->treeLineCol, BLACK);
	ConfVars_GetConfDefInt(cTreeNotReadCol, &ptr->treeNotReadCol, BLACK);
	ConfVars_GetConfDefInt(cTreeNotReadEffect, &unreadEff, 1);

	if (Bitplanes == 1)
	{
		if (ptr->treeTextCol != WHITE)
		{
			ptr->treeBackCol = WHITE;
		} else
		{
			if (ptr->treeBackCol == WHITE)
				ptr->treeBackCol = BLACK;
		}
		ptr->treeLineCol = ptr->treeTextCol;
		ptr->treeNotReadCol = ptr->treeTextCol;
	}
	switch (unreadEff)
	{
	case 0:
		ptr->treeNotReadEff = TXT_NORMAL;
		break;
	case 1:
		ptr->treeNotReadEff = TXT_THICKENED;
		break;
	case 2:
		ptr->treeNotReadEff = TXT_SKEWED;
		break;
	case 3:
		ptr->treeNotReadEff = TXT_LIGHT;
		break;
	default:
		ptr->treeNotReadEff = TXT_NORMAL;
		break;
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-- den ganzen Baum positionieren -------------------------------------*/

LOCAL _VOID posTree(treeWdwPtr ptr)
{
	_LONG maxWidth;
	_LONG maxHeight;
	MSGNR i;
	_WORD maxW;
	_WORD w;

	/* Set elem height und minWidth
	 */
	switch (ptr->zoomMode)
	{
	case zmNormal:
		ptr->elemHeight = ptr->charHeight * cElemLines;
		ptr->elemDist = ptr->charHeight * 2;
		ptr->elemHDist = ptr->charWidth * cElemHDist;
		break;
	case zmZoom:
		ptr->elemHeight = ptr->charHeight;
		ptr->elemDist = ptr->charHeight * 2;
		ptr->elemHDist = ptr->charWidth * 3;
		break;
	}

	maxHeight = 0;
	maxWidth = 0;
	maxW = 0;
	/* set each nodes width and height
	 */
	/* Texteffect setzen */
	vst_effects(ptr->vdi, ptr->treeNotReadEff);
	for (i = 0; i < ptr->treeElems; i++)
	{
		if (ptr->zoomMode == zmNormal)
		{
			w = VDIUtil_InqTextextend(ptr->isFSM, ptr->vdi, ptr->tree[i].betreff);
			w = Max(w, VDIUtil_InqTextextend(ptr->isFSM, ptr->vdi, ptr->tree[i].name));
			maxW = Max(w, maxW);
		} else
		{
			w = ptr->charWidth;
		}
		maxW = Max(w, maxW);

		ptr->tree[i].area.w = w;
		ptr->tree[i].area.h = ptr->elemHeight;
	}
	/* Texteffect zuruecksetzen */
	vst_effects(ptr->vdi, 0);
	if (ptr->treeType == tHorizontal)
	{
		for (i = 0; i < ptr->treeElems; i++)
		{
			ptr->tree[i].area.w = maxW;
		}
	}
	/* Reset the auxiliary tables
	 */
	resetOffset(ptr->vertical, cMinY);
	resetOffset(ptr->horizontal, cMinX);
	if (ptr->treeType == tVertical)
	{
		/* compute each node's x,y position
		 */
		computePositionsVertical(ptr, 0, 0);
		/* Jetzt noch die horizontalen Positionen setzen
		 */
		setPositionsVertical(ptr, 0, 0, &maxWidth, &maxHeight);
		/* set document size
		 */
	} else /* if (ptr->treeType == tHorizontal) */
	{
		/* compute each node's x,y position
		 */
		computePositionsHorizontal(ptr, 0, 0);
		/* Jetzt noch die horizontalen Positionen setzen
		 */
		setPositionsHorizontal(ptr, 0, 0, &maxWidth, &maxHeight);
		/* set document size
		 */
	}
	ptr->document.w = maxWidth + ptr->charWidth - 1;
	ptr->document.h = maxHeight + ptr->charHeight - 1;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* ---------------------- Font-Funktionen ----------------------- */

/* Setzt den Font fuer das Stichwortlistenfenster */
LOCAL _VOID iTreeSetFont(HWND wdw, _WORD fnt, _WORD fntSize, _BOOL redraw)
{
	GRECT r;
	treeWdwPtr ptr;
	_UBYTE varName[256];
	_BOOL mono;
	
	if ((ptr = FindWind(wdw)) != NULL)
	{
		ptr->font = fnt;
		ptr->fontSize = fntSize;
		FontSelect_SetFont(ptr->vdi, &ptr->font, &ptr->fontSize, TRUE, &mono, &ptr->isFSM, &ptr->charWidth, &ptr->charHeight);
		/* Jetzt ist der Font gesetzt. Jetzt noch Window snappen und dann neu
		 * zeichnen lassen
		 */
		make_varname(varName, sizeof(varName), cTreeFont, ptr->number);
		ConfVars_SetConfigInt(varName, ptr->font);
		make_varname(varName, sizeof(varName), cTreeFontSize, ptr->number);
		ConfVars_SetConfigInt(varName, ptr->fontSize);
		make_varname(varName, sizeof(varName), cTreeBuildType, ptr->number);
		ConfVars_SetConfigInt(varName, ptr->treeType);
		make_varname(varName, sizeof(varName), cTreeZoomMode, ptr->number);
		ConfVars_SetConfigInt(varName, ptr->zoomMode);
		/* Jetzt habe ich einen Baum im Speicher, der mit Indizes arbeitet.
		 * Also koennen wir jetzt daran gehen, diese Objekte alle zu plazieren.
		 * Das geht am besten mit einer rekursiven Prozedur, die sowohl die Breite
		 * als auch die Hoehe eines Eintrages zurueckliefert
		 */

		posTree(ptr);
		WdwManager_SetNewDocument(wdw, &ptr->document, FALSE);
		WdwManager_SetDocumentParms(wdw, 1, 1);
		/* Damit die Scrollbereiche neu berechnet werden */
		WdwManager_GetWdwSize(wdw, &r);
		WdwManager_SetWdwSize(wdw, &r);

		if (redraw)
		{
			WdwManager_RedrawWdw(wdw, &ptr->treeWork);
		}
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Ich brauche Prozeduren, die folgende Funktionen erfuellen:
 * - schneidet eine Linie ein Rechteck? (Redraw)
 *   Dies loesen wir einfach ueber Rcintersect. Ist zwar nicht 100%ig exakt,
 *   aber schneller. Und eine vollstaendige Ueberpruefung waere langsamer als
 *   ein paar Linien zusaetzlich ueber das VDI zeichnen zu lassen.
 * - wo verlaesst eine Linie ein Rechteck ? (Linienzeichnen)
 *   Siehe GetIntersection
 * - befindet sich ein Element in einem Rechteck (Rechteckueberschneidung, Redraw)
 *   wird geloest ueber RcIntersect (ClipRect)
 * - Zeichnen eines Elementes und der Verbindungslinien von/zu diesem Element
 */

/*** ---------------------------------------------------------------------- ***/

/* Berechnet den Mittelpunkt eines Rechtecks
 */
LOCAL LongPnt MidPointV(CONST LGRECT *r, _BOOL right)
{
	LongPnt p;

	if (right)
	{
		p.x = r->x + r->w + 3;
	} else
	{
		p.x = r->x - 3;
	}
	p.y = r->y + (r->h / 2);
	return p;
}

/*** ---------------------------------------------------------------------- ***/

/* Berechnet den Mittelpunkt eines Rechtecks
 */
LOCAL LongPnt MidPointH(CONST LGRECT *r, _BOOL down)
{
	LongPnt p;

	if (down)
	{
		p.y = r->y + r->h + 3;
	} else
	{
		p.y = r->y - 3;
	}
	p.x = r->x + (r->w / 2);
	return p;
}

/*** ---------------------------------------------------------------------- ***/

/* clippt das Rechteck frame auf das Rechteck clip
 */
LOCAL LGRECT ClipLongRect(CONST LGRECT *frame, CONST LGRECT *clip)
{
	LGRECT r;
	
	r = *frame;
	if (r.x < clip->x)
	{
		r.w = r.w - clip->x + r.x;
		r.x = clip->x;
	}
	if (r.y < clip->y)
	{
		r.h = r.h - clip->y + r.y;
		r.y = clip->y;
	}
	if (r.w < 0 || r.h < 0)
	{
		r.w = 0;
		r.h = 0;
	}
	if (r.x + r.w > clip->x + clip->w)
	{
		r.w = clip->x + clip->w - r.x;
	}
	if (r.y + r.h > clip->y + clip->h)
	{
		r.h = clip->y + clip->h - r.y;
	}
	if (r.w < 0 || r.h < 0)
	{
		r.w = 0;
		r.h = 0;
	}
	return r;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL GRECT ShortFrame(CONST LGRECT *frame)
{
	GRECT r;

	r.g_x = (_WORD)frame->x;
	r.g_y = (_WORD)frame->y;
	r.g_w = (_WORD)frame->w;
	r.g_h = (_WORD)frame->h;
	return r;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL LGRECT LongFrame(CONST GRECT *frame)
{
	LGRECT r;
	
	r.x = frame->g_x;
	r.y = frame->g_y;
	r.w = frame->g_w;
	r.h = frame->g_h;
	return r;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Diese Routine "clippt" das Rechteck frame auf den Bereich
 * clip, d.h. es wird der Ausschnitt von frame berechnet,
 * der innerhalb von clip liegt.
 */
LOCAL LGRECT LClipRect(CONST LGRECT *frame, CONST LGRECT *clip)
{
	LGRECT r;

	r = *frame;
	if (r.x < clip->x)
	{
		r.w = r.w - clip->x + r.x;
		r.x = clip->x;
	}
	if (r.y < clip->y)
	{
		r.h = r.h - clip->y + r.y;
		r.y = clip->y;
	}
	if (r.w < 0 || r.h < 0)
		r.w = r.h = 0;

	if (r.x + r.w > clip->x + clip->w)
		r.w = clip->x + clip->w - r.x;
	if (r.y + r.h > clip->y + clip->h)
		r.h = clip->y + clip->h - r.y;
	if (r.w < 0 || r.h < 0)
		r.w = r.h = 0;
	return r;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Liefert das kleinste Rechteck, das r1 und r2
 * vollstndig einschliet.
 */
LOCAL LGRECT LFrameRects(CONST LGRECT *r1, CONST LGRECT *r2)
{
	LGRECT r;
	_LONG x1, x2;
	
	if (r1->x > r2->x)
		r.x = r2->x;
	else
		r.x = r1->x;
	if (r1->y > r2->y)
		r.y = r2->y;
	else
		r.y = r1->y;
	x1 = r1->x + r1->w;
	x2 = r2->x + r2->w;
	if (x1 < x2)
		r.w = x2 - r.x;
	else
		r.w = x1 - r.x;
	x1 = r1->y + r1->h;
	x2 = r2->y + r2->h;
	if (x1 < x2)
		r.h = x2 - r.y;
	else
		r.h = x1 - r.y;
	return r;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Liefert das Rechteck, das von p1 und p2 aufgespannt wird.
 */
LOCAL LGRECT LFramePoints(CONST LongPnt *p1, CONST LongPnt *p2)
{
	LGRECT r;
	
	if (p1->x < p2->x)
	{
		r.x = p1->x;
		r.w = p2->x - p1->x + 1;
	} else
	{
		r.x = p2->x;
		r.w = p1->x - p2->x;
	}
	if (p1->y < p2->y)
	{
		r.y = p1->y;
		r.h = p2->y - p1->y + 1;
	} else
	{
		r.y = p2->y;
		r.h = p1->y - p2->y + 1;
	}
	return r;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Liefert das kleinste Rechteck, das r1 und r2
 * vollstndig einschliet.
 */
LOCAL GRECT FrameRects(CONST GRECT *r1, CONST GRECT *r2)
{
	GRECT r;
	_WORD x1, x2;
	
	if (r1->g_x > r2->g_x)
		r.g_x = r2->g_x;
	else
		r.g_x = r1->g_x;
	if (r1->g_y > r2->g_y)
		r.g_y = r2->g_y;
	else
		r.g_y = r1->g_y;
	x1 = r1->g_x + r1->g_w;
	x2 = r2->g_x + r2->g_w;
	if (x1 < x2)
		r.g_w = x2 - r.g_x;
	else
		r.g_w = x1 - r.g_x;
	x1 = r1->g_y + r1->g_h;
	x2 = r2->g_y + r2->g_h;
	if (x1 < x2)
		r.g_h = x2 - r.g_y;
	else
		r.g_h = x1 - r.g_y;
	return r;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-- drawing primitives ----------------------------------------------------*/

LOCAL _VOID DrawLine(HVDI vdi, CONST tLine *line, _WORD col, _WORD style)
{
	/* Erstmal Linewidth setzen */
	vsl_width(vdi, 1);

	/* Linienstil setzen (geht nur bei Dicke 1, wird sonst ignoriert) */
	vsl_type(vdi, style);

	if (col > BLACK && Bitplanes == 1)
	{
		/* Statt Farbe machen wir die Linie dicker */
		vsl_width(vdi, 2);
	} else
	{
		/* Linienfarbe setzen */
		vsl_color(vdi, col);
	}
	v_pline(vdi, 2, (_WORD *)line);
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet einen Rahmen, times bestimmt die Dicke
 * Direkter VDI-Call, damit nicht ein temporaeres Linearray
 * angelegt werden muss
 */
LOCAL _VOID DrawFrame(
	HVDI vdi,
	_WORD col,
	_WORD x, _WORD y, _WORD w, _WORD h,
	_WORD times, _WORD style)
{
	_WORD i;
	_WORD a;
	_WORD b;
	_WORD pxy[100];
	
	/* Erstmal Linewidth setzen */
	vsl_width(vdi, 1);

	/* Linienfarbe setzen */
	vsl_color(vdi, col);

	/* Linienstil setzen */
	vsl_type(vdi, style);

	a = 0;
	b = 0;
	--x;
	--y;
	w += 2;
	h += 2;
	for (i = 0; i < times; i++)
	{
		--x;
		--y;
		w += 2;
		h += 2;
		pxy[0 + a] = x;
		pxy[1 + a] = y;
		pxy[2 + a] = x + w;
		pxy[3 + a] = y;
		pxy[4 + a] = pxy[2 + a];		/* x + w; */
		pxy[5 + a] = y + h;
		pxy[6 + a] = x;
		pxy[7 + a] = pxy[5 + a];		/* y + h; */
		pxy[8 + a] = x;
		pxy[9 + a] = y;
		a += 10;
		b += 5;
	}
	v_pline(vdi, b, pxy);
}

/*** ---------------------------------------------------------------------- ***/

/* Diese Prozedur zeichnet ein Element oder seine Verbindungslinie,
 * wenn es im Cliprechteck liegt
 */
LOCAL _VOID drawElement(treeWdwPtr ptr, MSGNR idx, CONST GRECT *clip)
{
	tLine line;
	tLongLine lline;
	GRECT fill;
	LGRECT lClip;
	LGRECT lfill;
	LGRECT frame;
	LongPnt base;
	_WORD col;
	_WORD style;
	_WORD oldcol=-1,oldstyle=-1;
	_UBYTE anfB[]=" ";
	if (ptr->tree[idx].up < dataSys_notSaved)
	{
		if (ptr->treeType == tVertical)
		{
			/* Linie berechnen */
			lline.p1 = MidPointV(&ptr->tree[ptr->tree[idx].up].area, TRUE);
			lline.p2 = MidPointV(&ptr->tree[idx].area, FALSE);
		} else /* if (treeType == tHorizontal) */
		{
			lline.p1 = MidPointH(&ptr->tree[ptr->tree[idx].up].area, TRUE);
			lline.p2 = MidPointH(&ptr->tree[idx].area, FALSE);
		}
		/* Jetzt Schnittpunkte berechnen */
		/* Aus der Linie auch noch kurz ein Rechteck machen */
		frame = LFramePoints(&lline.p1, &lline.p2);
		/* Jetzt Gesamtbereich des Elements berechnen */
		lfill = ptr->tree[idx].area;
		/* Rahmen ist um zwei Pixel ausserhalb */
		lfill.x -= 2;
		lfill.y -= 2;
		lfill.w += 5;
		lfill.h += 5;
		frame = LFrameRects(&lfill, &frame);
	} else
	{
		frame = ptr->tree[idx].area;
		frame.x -= 2;
		frame.y -= 2;
		frame.w += 5;
		frame.h += 5;
	}
	/* Jetzt aus den relativen Koordinaten die absoluten berechnen */
	frame.x = frame.x - ptr->document.x + ptr->treeWork.g_x;
	frame.y = frame.y - ptr->document.y + ptr->treeWork.g_y;
	base.x = ptr->treeWork.g_x - ptr->document.x;
	base.y = ptr->treeWork.g_y - ptr->document.y;
	/* So, und jetzt nachsehen, ob davon ein Teil in der Cliparea ist */
	lClip = LongFrame(clip);
	frame = LClipRect(&frame, &lClip);
	if (frame.w != 0 || frame.h != 0)
	{
		/* Ok, wir muessen zeichnen */
		col = ptr->treeTextCol;
		style = 1;
		if (ptr->tree[idx].up < dataSys_notSaved)
		{
			lline.p1.x += base.x;
			lline.p1.y += base.y;
			lline.p2.x += base.x;
			lline.p2.y += base.y;
			/* Jetzt Koordinaten auf short wandeln */
			line.p1.x = (_WORD)(LMin(lline.p1.x, 32767));
			line.p1.y = (_WORD)(LMin(lline.p1.y, 32767));
			line.p2.x = (_WORD)(LMin(lline.p2.x, 32767));
			line.p2.y = (_WORD)(LMin(lline.p2.y, 32767));
			/* Linie noch etwas absetzen */
			if (ptr->treeType == tVertical)
			{
				line.p1.x += 3;
				line.p2.x -= 3;
			} else /* if (treeType == tHorizontal) */
			{
				line.p1.y += 3;
				line.p2.y -= 3;
			}
			/* Erstmal sehen, ob das Stichwort geaendert wurde */
			if (strCmp(ptr->tree[idx].betreff, ptr->tree[ptr->tree[idx].up].betreff) != 0)
			{
				if(ptr->treeLineCol!=KEEPCOL)
					col = ptr->treeLineCol;
				if (Bitplanes == 1)
				{
					style = USERLINE;
				}
			}
			DrawLine(ptr->vdi, &line, col, style);
		}
		/* Jetzt das Element */
		/* Farbe setzen */
		vst_color(ptr->vdi, ptr->treeTextCol);

		/* Rahmen um Element zeichnen */
		lfill = ptr->tree[idx].area;
		lfill.x += base.x;
		lfill.y += base.y;
		/* Jetzt daraus ein kleines Rechteck machen */
		fill = ShortFrame(&lfill);	/* Hier eigene Funktion einsetzen */
		DrawFrame(ptr->vdi, col, fill.g_x, fill.g_y, fill.g_w, fill.g_h, 1, style);
		{
			/* Jetzt Text ausgeben */
			col = ptr->treeTextCol;
			style = TXT_NORMAL;
			if (!(ptr->tree[idx].flags & bGelesen))
			{
				/* Texteffect setzen */
				style|=ptr->treeNotReadEff;
				/* Farbe setzen */
				if(ptr->treeNotReadCol!=KEEPCOL)
					col=ptr->treeNotReadCol;
			}
			if(col!=oldcol)
			{
				vst_color(ptr->vdi, col);
				oldcol=col;
			}
			if(style!=oldstyle)
			{
				vst_effects(ptr->vdi, style);
				oldstyle=style;
			}
			vswr_mode(ptr->vdi, MD_TRANS);

			if (ptr->zoomMode == zmNormal)
			{
				VDIUtil_Text(ptr->isFSM, ptr->vdi, (_WORD)(base.x + ptr->tree[idx].area.x), (_WORD)(base.y + ptr->tree[idx].area.y) + ptr->charHeight - 1, ptr->tree[idx].name);
				VDIUtil_Text(ptr->isFSM, ptr->vdi, (_WORD)(base.x + ptr->tree[idx].area.x), (_WORD)(base.y + ptr->tree[idx].area.y) + 2 * ptr->charHeight - 1, ptr->tree[idx].betreff);
			}
			else
			{
				*anfB=*ptr->tree[idx].name;
				VDIUtil_Text(ptr->isFSM, ptr->vdi, (_WORD)(base.x + ptr->tree[idx].area.x), (_WORD)(base.y + ptr->tree[idx].area.y) + ptr->charHeight - 1, anfB);
			}
		}
		/* Wenn es selektiert ist, dann invertieren wir es einfach */
		if (ptr->tree[idx].selected)
		{
			/* Writemode auf XOR und area mit Schwarz fuellen */
			vswr_mode(ptr->vdi, MD_XOR);
			vsf_color(ptr->vdi, BLACK);

			/* fill ist schon gesetzt, nur noch absolutes Rechteck daraus machen
			 */
			fill.g_w = fill.g_x + (_WORD)ptr->tree[idx].area.w;
			fill.g_h = fill.g_y + (_WORD)ptr->tree[idx].area.h;
			vr_recfl(ptr->vdi, (_WORD *)&fill);
			vsf_color(ptr->vdi, ptr->treeBackCol);
		}
		vswr_mode(ptr->vdi, MD_REPLACE);

		if (!(ptr->tree[idx].flags & bGelesen))
		{
			/* Texteffect und Farbe zuruecksetzen */
			/* Texteffect zuruecksetzen */
			vst_effects(ptr->vdi, 0);
			/* Farbe setzen */
			vst_color(ptr->vdi, ptr->treeTextCol);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Rekursive Prozedur, die nur dazu da ist, den Baum zu durchlaufen
 */
LOCAL _VOID drawElemTree(treeWdwPtr ptr, MSGNR idx, CONST GRECT *clip)
{
	while (idx < dataSys_notSaved)
	{
		drawElement(ptr, idx, clip);
		if (ptr->tree[idx].down < dataSys_notSaved)
		{
			/* in die Tiefe gehen und die Teile zeichnen */
			drawElemTree(ptr, ptr->tree[idx].down, clip);
		}
		idx = ptr->tree[idx].right;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID drawTreeWdw(HWND wdw, HVDI vdiH, _VOID *special, CONST GRECT *frame)
{
	treeWdwPtr ptr;
	GRECT clip;
	GRECT fill;

	UNUSED(wdw);
	UNUSED(vdiH);
	ptr = special;
	clip = *frame;
	if (rc_intersect(&ptr->treeWork, &clip))
	{
		clip = rc_clip(&ptr->treeWork, &clip);
		clip.g_w += 2;

		/* Jetzt den Hintergrund loeschen */

		fill = clip;
		fill.g_w = clip.g_x + clip.g_w - 1;
		fill.g_h = clip.g_y + clip.g_h - 1;

		vsf_color(ptr->vdi, ptr->treeBackCol);

		vr_recfl(ptr->vdi, (_WORD *)&fill);

		/* Und jetzt Elemente zeichnen */
		drawElemTree(ptr, 0, &clip);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID centerSelection(treeWdwPtr ptr, _BOOL redraw)
{
	MSGNR i;
	LGRECT lr;
	LGRECT lt;
	LGRECT lc;
	_BOOL visible;

	i = 0;
	while (i < ptr->treeElems && !ptr->tree[i].selected)
		++i;
	if (i < ptr->treeElems)
	{
		/* Wir haben eine selektierte Nachricht */
		lr = ptr->tree[i].area;
		lr.x -= 2;
		lr.y -= 2;
		lr.w += 5;
		lr.h += 5;
		lt = LongFrame(&ptr->treeWork);
		lr.x = lr.x - ptr->document.x + lt.x;
		lr.y = lr.y - ptr->document.y + lt.y;
		/* Jetzt pruefen, ob es sichtbar ist */
		lc = ClipLongRect(&lr, &lt);
		visible = (lc.w != 0) && (lc.h != 0);
		if (!visible)
		{
			/* document.x und .y umsetzen */
			lr = ptr->tree[i].area;
			ptr->document.x = LMax(lr.x - lt.w / 2, 0);
			ptr->document.x = LMax(0, LMin(ptr->document.x, ptr->document.w - lt.w));
			ptr->document.y = LMax(lr.y - lt.h / 2, 0);
			ptr->document.y = LMax(0, LMin(ptr->document.y, ptr->document.h - lt.h));
			if (redraw)
			{
			}
		}
		WdwManager_SetNewDocument(ptr->wdw, &ptr->document, redraw);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID getEntryArea(treeWdwPtr ptr, MSGNR idx, GRECT *r, _BOOL *visible)
{
	LGRECT lr;
	LGRECT lt;
	LGRECT lc;

	lr = ptr->tree[idx].area;
	lr.x -= 2;
	lr.y -= 2;
	lr.w += 5;
	lr.h += 5;
	lt = LongFrame(&ptr->treeWork);
	lr.x = lr.x - ptr->document.x + lt.x;
	lr.y = lr.y - ptr->document.y + lt.y;
	/* Jetzt pruefen, ob es sichtbar ist */
	lc = ClipLongRect(&lr, &lt);
	*visible = (lc.w != 0) && (lc.h != 0);
	if (*visible)
		*r = ShortFrame(&lr);
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet einen Eintrag OHNE seine Linie neu */
LOCAL _VOID redrawEntry(treeWdwPtr ptr, MSGNR entry)
{
	GRECT rect;
	_BOOL vis;

	getEntryArea(ptr, entry, &rect, &vis);
	if (vis)
	{
		WdwManager_RedrawWdw(ptr->wdw, &rect);
		WdwManager_SetClip(ptr->vdi, &ptr->treeWork, TRUE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Alle Nachrichten in dem Fenster werden deselektiert
 */
GLOBAL _VOID treeList_deselectEntries(HWND win)
{
	MSGNR idx;
	GRECT clip;
	GRECT r;
	_BOOL vis;
	_BOOL first;
	treeWdwPtr ptr;
	
	ptr = FindWind(win);
	if (ptr == NULL)
		return;
	first = TRUE;
	for (idx = 0; idx < ptr->treeElems; idx++)
	{
		if (ptr->tree[idx].selected)
		{
			ptr->tree[idx].selected = FALSE;
			getEntryArea(ptr, idx, &r, &vis);
			if (vis)
			{
				if (first)
				{
					clip = r;
					first = FALSE;
				} else
				{
					clip = FrameRects(&clip, &r);
				}
			}
		}
	}
	if (!first)
	{
		WdwManager_RedrawWdw(ptr->wdw, &clip);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findEntry(treeWdwPtr ptr, _WORD x, _WORD y, MSGNR *idx)
{
	MSGNR i;
	GRECT r;
	_BOOL vis;

	for (i = 0; i < ptr->treeElems; i++)
	{
		getEntryArea(ptr, i, &r, &vis);
		if (vis && rc_inside(x, y, &r))
		{
			*idx = i;
			return TRUE;
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID openSelected(treeWdwPtr ptr, MSGNR idx, _BOOL doNotTop, _BOOL withCtrl)
{
	_BOOL isOpen;
	HWND window;
	CAT_GROUP_NUMBER group;
	MSGNR mess;
	_ULONG amount;
	
	if (ptr->grinWindow >= 0)
	{
		isOpen = FALSE;
		if (!withCtrl &&
			grin_grinWindowTop(ptr->grinWindow))
		{
			grin_ActualGroupNr(ptr->grinWindow, &group);
			if (group == ptr->grHandle->group)
			{
				isOpen = grin_grinSwitchTo(ptr->grinWindow, ptr->tree[idx].msgNum, grin_grinNextMess, 0, doNotTop);
			}
		}
		if (!isOpen)
		{
			window = grin_grinOpenMessage(ptr->grHandle->group, ptr->tree[idx].msgNum, grin_grinNextMess, 0, mOther);
			/* Nun Anfragen, welches Grinfenster nun top ist */
			if (window >= 0 &&
				grin_grinWindowTop(window) &&
				grin_getBasicInfo(window, &group, &mess, &amount) &&
				group == ptr->grHandle->group &&
				mess == ptr->tree[idx].msgNum)
			{
				ptr->grinWindow = window;
			} else
			{
				ptr->grinWindow = -1;
			}
		}
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID openGuek(treeWdwPtr ptr, MSGNR idx, _BOOL doNotTop, _BOOL withCtrl)	*/
/*	Bei GKs wird als Wurzel eine Pseudomessage angezeigt, will man diese ffnen*/
/*	versucht diese Funktion die Ursprungsmail in  der anderen Gruppe zu finden	*/
/*	abgeleitet von openSelected													*/
/*	Parameter:	ptr:	Alle Infos zum Baum										*/
/*				idx:	Nr im Baum, ich vermute immer 0							*/
/*				doNotTop: soll das Fenster getoppt werden?						*/
/*				withCtrl: war Ctrl gedrckt?									*/
/*	Ergebnis:	keines															*/		
/*	31.7.01 von Dimitri Junker	DYJ 4.24o										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.23k */
LOCAL _VOID openGuek(treeWdwPtr ptr, MSGNR idx, _BOOL doNotTop, _BOOL withCtrl)
{
	_BOOL isOpen;
	HWND window;
	CAT_GROUP_NUMBER group;
	MSGNR msgnr;
	_ULONG amount;
	_UBYTE *id=NULL;
	MessageType mess;
	F_ERROR error;
	MemSetZeroStruct(mess);

	msgnr=ptr->tree[idx].down;
	if(msgnr<dataSys_notSaved)
		msgnr=ptr->tree[msgnr].msgNum;
	if (msgnr<dataSys_notSaved && ptr->grinWindow >= 0)
	{
		isOpen = FALSE;
		/* Message lesen */
		
		error = data_ReadHeader(ptr->grHandle->Zugriff, msgnr, &mess,TRUE);
		if (error == NoError)
		{
			id=mess.KommentierteID;
			if(mess.rid && strLen(mess.rid)>strLen(id))
				id=mess.rid;
		}
	}
	if(id)
	{
		if (!withCtrl &&
			grin_grinWindowTop(ptr->grinWindow))
		{
			grin_ActualGroupNr(ptr->grinWindow, &group);
			if (group == ptr->guekInf.guekNr)
			{
				msgnr=data_NumberOfID(ptr->grHandle->Zugriff, id);
				isOpen = grin_grinSwitchTo(ptr->grinWindow,msgnr , grin_grinNextMess, 0, doNotTop);
			}
		}
		if (!isOpen)
		{
			window = grin_grinOpenWithProc(ptr->guekInf.guekNr, data_NumberOfID, id, grin_grinNextMess, 0, mOther);
			/* Nun Anfragen, welches Grinfenster nun top ist */
			if (window >= 0 &&
				grin_grinWindowTop(window) &&
				grin_getBasicInfo(window, &group, &msgnr, &amount) &&
				group == ptr->grHandle->group &&
				msgnr == ptr->tree[idx].msgNum)
			{
				ptr->grinWindow = window;
			}
		}
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID selectArea(treeWdwPtr ptr, _WORD x, _WORD y)
{
	MSGNR i;
	GRECT r;
	GRECT rect;
	_BOOL first;
	GRECT clip;
	_BOOL vis;

	rect.g_x = x;
	rect.g_y = y;
	rect.g_w = 0;
	rect.g_h = 0;
	MouseFinger();
	graf_rubberbox(x, y, -32767, -32767, &rect.g_w, &rect.g_h);
	MouseArrow();

	if (rect.g_w < 0)
	{
		rect.g_x += rect.g_w;
		rect.g_w = -rect.g_w;
	}
	if (rect.g_h < 0)
	{
		rect.g_y += rect.g_h;
		rect.g_h = -rect.g_h;
	}

	/* clippen */
	rect = rc_clip(&rect, &ptr->treeWork);
	rect.g_h = Min(rect.g_h, ptr->treeWork.g_y + ptr->treeWork.g_h - 1 - rect.g_y);

	first = TRUE;
	for (i = 0; i < ptr->treeElems; i++)
	{
		/* Jetzt pruefen, ob das Element beruehrt wird */
		getEntryArea(ptr, i, &r, &vis);
		if (vis&& ptr->tree[i].msgNum!=dataSys_notSaved)	/*nicht vorhandene Messages nicht selektieren*/	
		{
			if (rc_intersect(&rect, &r))
			{
				/* Eintrag selektieren und zeichnen */
				ptr->tree[i].selected = !ptr->tree[i].selected;
				getEntryArea(ptr, i, &r, &vis);
				if (vis)
				{
					if (first)
					{
						clip = r;
						first = FALSE;
					} else
					{
						clip = FrameRects(&clip, &r);
					}
				}
			}
		}
	}
	if (!first)
	{
		WdwManager_RedrawWdw(ptr->wdw, &clip);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL clickInTreeWindow(HWND win, _VOID *special, _WORD x, _WORD y, _UWORD kstate, _UWORD buts, _WORD clicks)	
{
	treeWdwPtr ptr;
	_UWORD but;
	Point p;
	MSGNR idx;

	ptr = special;
	{
		if (rc_inside(x, y, &ptr->treeWork))
		{
			if (!CatGlobal_WithShift(kstate))
			{
				/* Alte deselektieren */
				treeList_deselectEntries(win);
			}
			/* Selektierten Eintrag finden */
			if (findEntry(ptr, x, y, &idx))
			{
				if(ptr->tree[idx].msgNum!=dataSys_notSaved)			/*nicht vorhandene Messages nicht selektieren*/	
				{
					/* Eintrag selektieren */
					ptr->tree[idx].selected = !ptr->tree[idx].selected;
					redrawEntry(ptr, idx);
					/* Warten auf loslassen */
					Bounce();
					if (clicks > 1 || (!(buts & MOB_LEFT) && (buts & MOB_RIGHT)))
					{
						openSelected(ptr, idx, !(buts & MOB_LEFT) && (buts & MOB_RIGHT), CatGlobal_WithCtrl(kstate));
					}
				}
				else if(ptr->guekInf.guekNr!=NO_GROUP)
				{
					/* Warten auf loslassen */
					Bounce();
					if (clicks > 1 || (!(buts & MOB_LEFT) && (buts & MOB_RIGHT)))
					{
						openGuek(ptr, idx, !(buts & MOB_LEFT) && (buts & MOB_RIGHT), CatGlobal_WithCtrl(kstate));
					}
				}

			} else
			{
				graf_mkstate(&p.x, &p.y, &but, &kstate);
				if (but & MOB_LEFT)
				{
					/* Graf-Rubberbox aufrufen */
					selectArea(ptr, x, y);
				}
			}
			return TRUE;
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL handleTreeKey(HWND wdw, _VOID *special, _WORD taste, _UBYTE ch, _UBYTE scan, _UWORD kstate)	
{
	treeWdwPtr ptr;
	_UWORD gemCh;
	Key key;
	_BOOL done;

	UNUSED(wdw);
	UNUSED(taste);
	ptr = special;
	gemCh = (scan << 8) + ch;
	GemCharToKey(gemCh, kstate, &key);
	done = TRUE;
	if (IsSpecial(&key))
	{
		switch (SpecialKey(&key))
		{
		case left:
			WdwManager_ScrollLeft(ptr->wdw, 1);
			break;
		case soln:
			WdwManager_PageLeft(ptr->wdw);
			break;
		case right:
			WdwManager_ScrollRight(ptr->wdw, 1);
			break;
		case eoln:
			WdwManager_PageRight(ptr->wdw);
			break;
		case up:
			WdwManager_ScrollUp(ptr->wdw, 1);
			break;
		case down:
			WdwManager_ScrollDown(ptr->wdw, 1);
			break;
		case pgUp:
			WdwManager_PageUp(ptr->wdw);
			break;
		case pgDown:
			WdwManager_PageDown(ptr->wdw);
			break;
		case altC:
			ptr->document.x = 0;
			ptr->document.y = 0;
			if (ptr->treeType == tVertical)
			{
				ptr->treeType = tHorizontal;
			} else
			{
				ptr->treeType = tVertical;
			}
			iTreeSetFont(ptr->wdw, ptr->font, ptr->fontSize, FALSE);
			centerSelection(ptr, TRUE);
			break;
		case altZ:
			ptr->document.x = 0;
			ptr->document.y = 0;
			if (ptr->zoomMode == zmNormal)
			{
				ptr->zoomMode = zmZoom;
			} else
			{
				ptr->zoomMode = zmNormal;
			}
			iTreeSetFont(ptr->wdw, ptr->font, ptr->fontSize, FALSE);
			centerSelection(ptr, FALSE);
			WdwManager_FullRedrawWdw(ptr->wdw);
			break;
		default:
			done = FALSE;
			break;
		}
	} else
	{
		done = FALSE;
	}
	return done;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL handleTreeTimer(HWND wdw, _VOID *special)	
{
	UNUSED(wdw);
	UNUSED(special);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL closeTreeWindow(HWND wdw,HVDI vdiH, _VOID *special, _BOOL force)
{
	treeWdwPtr ptr;
	_UBYTE varName[256];
	GRECT full;

	UNUSED(wdw);
	UNUSED(vdiH);
	UNUSED(force);
	ptr = special;
	make_varname(varName, sizeof(varName), cTreeWindowPos, ptr->number);
	WdwManager_GetWdwSize(ptr->wdw, &full);	/* Groesse des Fensters abfragen */
	ConfVars_SetConfigRect(varName, &full);
	make_varname(varName, sizeof(varName), cTreeFont, ptr->number);
	ConfVars_SetConfigInt(varName, ptr->font);
	make_varname(varName, sizeof(varName), cTreeFontSize, ptr->number);
	ConfVars_SetConfigInt(varName, ptr->fontSize);
	make_varname(varName, sizeof(varName), cTreeBuildType, ptr->number);
	ConfVars_SetConfigInt(varName, ptr->treeType);
	make_varname(varName, sizeof(varName), cTreeZoomMode, ptr->number);
	ConfVars_SetConfigInt(varName, ptr->zoomMode);
	ClearNum(ptr->number);
	FontSelect_UnloadFonts(ptr->vdi);
	/* Speicher freigeben */
	SFREE(ptr->tree);
	freeOffset(ptr->horizontal);
	freeOffset(ptr->vertical);
	handlePool_FreeOneDataHandle(ptr->grHandle);
	handlePool_FreeOnePtr(ptr, &windows);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL topTreeWdw(HWND wdw, _VOID *special)	
{
	UNUSED(wdw);
	UNUSED(special);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID untopTreeWdw(HWND wdw, _VOID *special)	
{
	UNUSED(wdw);
	UNUSED(special);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID updateTreeWdw(HWND wdw, _VOID *special, _BOOL update)	
{
	UNUSED(wdw);
	UNUSED(special);
	if (update)
	{
		Wind_Update(BEG_UPDATE);
		MouseOff();
	} else
	{
		MouseOn();
		Wind_Update(END_UPDATE);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID hideTreeWdw(HWND wdw, _VOID *special, _BOOL hide)
{
	UNUSED(wdw);
	UNUSED(special);
	UNUSED(hide);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID setTreeWork(HWND wdw, _VOID *special, CONST LGRECT *doc, _BOOL slided)	
{
	treeWdwPtr ptr;

	UNUSED(wdw);
	UNUSED(slided);
	ptr = special;
	ptr->document = *doc;
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID snapTreeWdw(HWND wdw,_VOID *special, GRECT *work)	
{
#  define minWindWidth 128
#  define minWindHeight 128
	treeWdwPtr ptr;

	UNUSED(wdw);
	ptr = special;
	if (ptr->font >= 0)
	{
		ptr->treeWork = *work;
		WdwManager_SetScrollParms(ptr->wdw, work->g_w - ptr->charWidth, work->g_h - ptr->charHeight, ptr->charWidth, ptr->charHeight,TRUE);
	}
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID getTreeScroll(HWND wdw,HVDI vdiH, _VOID *special, GRECT *work)
{
	treeWdwPtr ptr;
	_UBYTE varName[256];
	GRECT full;

	UNUSED(wdw);
	UNUSED(vdiH);
	ptr = special;
	WdwManager_SetScrollParms(wdw, work->g_w - ptr->charWidth, work->g_h - ptr->charHeight, ptr->charWidth, ptr->charHeight,TRUE);
	ptr->treeWork = *work;
	if (ptr->wdw >= 0)
	{
		make_varname(varName, sizeof(varName), cTreeWindowPos, ptr->number);
		WdwManager_GetWdwSize(ptr->wdw, &full);	/* Groesse des Fensters abfragen */
		ConfVars_SetConfigRect(varName, &full);
	}
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID MakeTitle(CAT_GROUP_NUMBER gruppe, _UBYTE *title, size_t strsize)
{
	GroupSelect_GroupDisplayName(gruppe, title, strsize);
	insert_box_name(title, strsize);	
	insert_app_name(title, strsize);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL treeCount(OneGroupHandle hdl, MSGNR msgIdx, pBlockPtr mess, _VOID *private)
{
	UNUSED(hdl);
	UNUSED(msgIdx);
	UNUSED(mess);
	UNUSED(private);
	++treeCounter;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CheckAssign(CONST _UBYTE *strPtr, _UBYTE *str, size_t strsize)
{
	if (strPtr != NULL)
		strMcpy(str, strsize, strPtr);
	else
		*str = '\0';
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.23k */
LOCAL _BOOL treeBuild(OneGroupHandle hdl, MSGNR msgIdx, pBlockPtr msg, _VOID *private)
{
	_UBYTE tmpStr[256];
	tShortText line;
	MessageType mess;
	size_t maxLen;
	treeElem *elem;
	
	CAT_GROUP_NUMBER *gueknr;
	
	MemSetZeroStruct(mess);
	gueknr=private;
	CatGlobal_busyMouse();
	elem = &globalTree[treeCounter];
	elem->msgNum = msgIdx;
	elem->selected = FALSE;
	/* Erstmal die Messagenummern eintragen */
	elem->up = msg->upMess;
	elem->down = msg->downMess;
	elem->right = msg->rightMess;
	elem->left = msg->leftMess;
	/* Jetzt die Daten lesen */
	if(msgIdx==dataSys_notSaved)
	{
		/* Flags zuweisen */
		elem->flags = bGelesen;
		if(*gueknr!=NO_GROUP)
		{
			strBcpy(elem->name,    "Msg. evtl. in");
			strBcpy(elem->betreff, "anderer Gruppe");
		}
		else
		{
			strBcpy(elem->name,    "Msg. nicht");
			strBcpy(elem->betreff, "vorhanden");
		}
		elem->width = (_WORD)strLen(elem->name);
		/* Speicher wieder freigeben */
	}
	else
	{		
		if (data_ReadHeader(hdl, msgIdx, &mess,TRUE) == GERR_OK)
		{
			/* Flags zuweisen */
			elem->flags = mess.StatusBits;
			/* Jetzt die Texte */
			if (GroupSelect_isPrivate(mess.Gruppe) && mess.EigeneNachricht)
			{
				strBcpy(line, "An: ");
				CheckAssign(mess.Empfaenger, tmpStr, sizeof(tmpStr));
				maxLen = sizeof(line) - 4;
			} else
			{
				CheckAssign(mess.Absender, tmpStr, sizeof(tmpStr));
				line[0] = '\0';
				maxLen = sizeof(line);
			}
			CatGlobal_KurzName(tmpStr, tmpStr, maxLen);
			strBcat(line, tmpStr);
			strBcpy(elem->name, line);
			elem->width = (_WORD)strLen(elem->name);
			/* Jetzt noch das Stichwort */
			CheckAssign(mess.Betreff, tmpStr, sizeof(tmpStr));
			maxLen = sizeof(line);
			if (strLen(tmpStr) >= maxLen)
			{
				/* Zu lang, hinten Pfeil nach rechts anhaengen */
				tmpStr[maxLen - 2] = '\003';
				tmpStr[maxLen - 1] = '\0';
			}
			elem->width = Max((_WORD)strLen(tmpStr), elem->width);
			strBcpy(elem->betreff, tmpStr);
			/* Speicher wieder freigeben */
			data_freeMessage(&mess);
		} else
		{
			return FALSE;
		}
	}
	++treeCounter;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID getTreeDepth(treeArrayPtr tree, MSGNR idx, _UWORD depth, _UWORD *maxDepth)
{
	MSGNR i;

	++depth;
	if (depth > *maxDepth)
		*maxDepth = depth;
	i = idx;
	while (i < dataSys_notSaved)
	{
		if (tree[i].down < dataSys_notSaved)
		{
			getTreeDepth(tree, tree[i].down, depth, maxDepth);
		}
		i = tree[i].right;
	}
	/* --depth; */
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID findMsgIdx(treeWdwPtr ptr, MSGNR *idx)
{
	MSGNR i;

	for (i = 0; i < treeCounter; i++)
	{
		if (ptr->tree[i].msgNum == *idx)
		{
			*idx = i;
			return;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL buildTree(treeWdwPtr ptr, oneHandlePtr handle, MSGNR msg)
{
	MSGNR i;
	_ULONG mem;

	/* Erstmal Speicher fuer die Offsetarray allozieren */
	ptr->vertical = createOffset(128);
	if (ptr->vertical == NULL)
	{
		return FALSE;
	}
	ptr->horizontal = createOffset(128);
	if (ptr->horizontal == NULL)
	{
		freeOffset(ptr->vertical);
		return FALSE;
	}
	/* Konfigvariablen auslesen */
	ConfVars_GetConfDefInt(cTreeUpLevels, &maxUpLevels, 20);
	ConfVars_GetConfDefInt(cTreeMaxDepth, &maxTreeDepth, 40);
	/* Objekte im Baum zaehlen */
	treeCounter = 0;
	data_WalkTree2(handle->Zugriff, msg, maxUpLevels, maxTreeDepth, treeCount, NULL);
	/* Jetzt Speicher allozieren */
	ptr->treeElems = treeCounter;
	mem = treeCounter * sizeof(treeElem);
	ptr->tree = MALLOC(mem, "buildTree");
	if (ptr->tree == NULL)
	{
		freeOffset(ptr->horizontal);
		freeOffset(ptr->vertical);
		return FALSE;
	}
	/* Speicher loeschen, damit die Strings nicht einzeln geloescht werden
	 * muessen
	 */
	MemSetZero(ptr->tree, mem);
	/* Speicher ist da, jetzt Baum aufbauen. Dabei muss ich danach die Indizes
	 * nochmal umrechnen!
	 */
	treeCounter = 0;
	globalTree = ptr->tree;
	data_WalkTree2(handle->Zugriff, msg, maxUpLevels, maxTreeDepth, treeBuild, &ptr->guekInf);
	MouseArrow();
	/* Wir haben jetzt einen Baum. Jetzt muessen wir die Indizes noch anpassen,
	 * daher gebe ich die jetzt erstmal aus.
	 */
	/* so, jetzt muessen wir die Indizes noch anpassen */
	for (i = 0; i < treeCounter; i++)
	{
		findMsgIdx(ptr, &ptr->tree[i].up);
		findMsgIdx(ptr, &ptr->tree[i].down);
		findMsgIdx(ptr, &ptr->tree[i].right);
		findMsgIdx(ptr, &ptr->tree[i].left);
	}
	/* Jetzt muessen noch alle Treeindizes, die nicht im erlaubten Bereich
	 * liegen (0..treeElems-1) auf empty gesetzt werden, da ja nur ein
	 * Teilbaum vorhanden ist
	 */
	for (i = 0; i < treeCounter; i++)
	{
		if (ptr->tree[i].up >= treeCounter)
			ptr->tree[i].up = dataSys_empty;
		if (ptr->tree[i].down >= treeCounter)
			ptr->tree[i].down = dataSys_empty;
		if (ptr->tree[i].left >= treeCounter)
			ptr->tree[i].left = dataSys_empty;
		if (ptr->tree[i].right >= treeCounter)
			ptr->tree[i].right = dataSys_empty;
		if (ptr->tree[i].msgNum == msg)
			ptr->tree[i].selected = TRUE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL treeList_treeChangeFont(HWND wdw, _WORD font, _WORD fontSize, _BOOL isMono)
{
	UNUSED(isMono);
	if (FindWind(wdw) != NULL)
	{
		treeList_treeSetFont(wdw, font, fontSize);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL treeList_treeChangeColor(HWND wdw, _WORD color)
{
	if (FindWind(wdw) != NULL)
	{
		ConfVars_SetConfigInt(cTreeTextCol, color);
		treeList_treeRedrawAll();
		return TRUE;
	}
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*DYJ 5.21b */
GLOBAL _VOID treeList_treeOpen(CAT_GROUP_NUMBER gruppe, MSGNR which, HWND grinWdw)
{
	treeWdwPtr ptr;
	oneHandlePtr handlePtr;
	_UBYTE varName[128];
	_UWORD comps;
	_WORD num;
	_UBYTE infoLine[128];

	if (which >= dataSys_notSaved)
		return;
	if (windows.root == NULL)
		if (!Lists_CreateList(&windows, "treeList_treeOpen"))
			return;
	if ((ptr = handlePool_BlankToList(sizeof(treeWindow), &windows)) != NULL)
	{
		if ((handlePtr = handlePool_GetOneDatahandle(gruppe)) != NULL)
		{
			ptr->wdw = -1;
			ptr->grHandle = handlePtr;
			ptr->grinWindow = grinWdw;
			/* Fensternummer dieses Moduls herausfinden */
			ptr->number = FindNum();
			make_varname(varName, sizeof(varName), cTreeWindowPos, ptr->number);
			if (!ConfVars_GetConfigRect(varName, &ptr->treeWork))
			{
				if (ptr->number > 0)
				{
					num = ptr->number - 1;
					make_varname(varName, sizeof(varName), cTreeWindowPos, num);
				}
				ConfVars_GetConfDefRect(varName, &ptr->treeWork, &deskSize);
				if (!rc_equal(&ptr->treeWork, &deskSize))
				{
					ptr->treeWork.g_x += 2 * CharWidth;
					ptr->treeWork.g_y += CharHeight;
				}
				make_varname(varName, sizeof(varName), cTreeWindowPos, ptr->number);
				ConfVars_SetConfigRect(varName, &ptr->treeWork);
			}
			treeList_GetSettings(ptr);
			/* Fensterkomponenten setzen */
			comps = NAME | CLOSER | FULLER | MOVER | INFO | SIZER | UPARROW | DNARROW | VSLIDE | LFARROW | RTARROW | HSLIDE;
			MakeTitle(gruppe, ptr->grName, sizeof(ptr->grName));
			if (data_LastMsgOfGroup(gruppe) == dataSys_empty)
			{
				/* Gruppe ist leer! */
				handlePool_FreeOneDataHandle(handlePtr);
				handlePool_FreeOnePtr(ptr, &windows);
				ClearNum(ptr->number);
				return;
			}
			/* Jetzt den Baum aufbauen */
			if (!buildTree(ptr, handlePtr, which))
			{
				handlePool_FreeOneDataHandle(handlePtr);
				handlePool_FreeOnePtr(ptr, &windows);
				ClearNum(ptr->number);
				return;
			}
			/* Infozeile basteln */
			if(ptr->tree[0].msgNum==dataSys_notSaved)
			{		
				sprintf(infoLine, MausTauschrsc_WrkString(FS_NUM_MESSAGES),
					ptr->treeElems-1,
					ptr->treeElems == 2 ? "" : MausTauschrsc_WrkString(FS_PLURAL1));
			}
			else
			{		
				sprintf(infoLine, MausTauschrsc_WrkString(FS_NUM_MESSAGES),
					ptr->treeElems,
					ptr->treeElems == 1 ? "" : MausTauschrsc_WrkString(FS_PLURAL1));
			}
			/* Fenster oeffnen */
			if (WdwManager_OpenWindow(clickInTreeWindow,
				handleTreeKey, handleTreeTimer,
				&deskSize, &ptr->treeWork, comps, TRUE,
				infoLine, ptr->grName, snapTreeWdw, closeTreeWindow,
				drawTreeWdw, topTreeWdw, untopTreeWdw,
				updateTreeWdw,
				setTreeWork, getTreeScroll, hideTreeWdw,
				0, ptr, FALSE, TRUE, TRUE, TRUE,
				FALSE,		/* hasIdem  */
				&ptr->wdw, &ptr->vdi))
			{
				_WORD val;
				
				/* Font aus Configvariablen lesen und setzen */
				make_varname(varName, sizeof(varName), cTreeFont, ptr->number);
				if (!ConfVars_GetConfigInt(varName, &ptr->font))
				{
					num = Max(ptr->number - 1, 0);
					make_varname(varName, sizeof(varName), cTreeFont, num);
					ConfVars_GetConfDefInt(varName, &ptr->font, 1);
					make_varname(varName, sizeof(varName), cTreeFontSize, num);
					ConfVars_GetConfDefInt(varName, &ptr->fontSize, 10);
					make_varname(varName, sizeof(varName), cTreeBuildType, num);
					ConfVars_GetConfDefInt(varName, &val, tVertical);
					ptr->treeType = (treeBuildType) val;
					make_varname(varName, sizeof(varName), cTreeZoomMode, num);
					ConfVars_GetConfDefInt(varName, &val, zmNormal);
					ptr->zoomMode = (zoomModeType) val;
					
					make_varname(varName, sizeof(varName), cTreeFont, ptr->number);
					ConfVars_SetConfigInt(varName, ptr->font);
					make_varname(varName, sizeof(varName), cTreeFontSize, ptr->number);
					ConfVars_SetConfigInt(varName, ptr->fontSize);
					make_varname(varName, sizeof(varName), cTreeBuildType, ptr->number);
					ConfVars_SetConfigInt(varName, ptr->treeType);
					make_varname(varName, sizeof(varName), cTreeZoomMode, ptr->number);
					ConfVars_SetConfigInt(varName, ptr->zoomMode);
				} else
				{
					make_varname(varName, sizeof(varName), cTreeFontSize, ptr->number);
					ConfVars_GetConfDefInt(varName, &ptr->fontSize, 10);
					make_varname(varName, sizeof(varName), cTreeBuildType, ptr->number);
					ConfVars_GetConfDefInt(varName, &val, tVertical);
					ptr->treeType = (treeBuildType) val;
					make_varname(varName, sizeof(varName), cTreeZoomMode, ptr->number);
					ConfVars_GetConfDefInt(varName, &val, zmNormal);
					ptr->zoomMode = (zoomModeType) val;
				}
				/* Noch ein bisschen was fuer die Window-Library */
				WdwManager_SetNewDocument(ptr->wdw, &ptr->document, FALSE);
				/* Fonts laden */
				FontSelect_LoadFonts(ptr->vdi);

				/* Font setzen */
				ptr->document.x = 0;
				ptr->document.y = 0;
				iTreeSetFont(ptr->wdw, ptr->font, ptr->fontSize, FALSE);

				/* Jetzt noch ein paar andere VDI-Variablen setzen */
				vsf_style(ptr->vdi, IP_SOLID);
				vsf_interior(ptr->vdi, FIS_SOLID);
				vsf_color(ptr->vdi, ptr->treeBackCol);

				vsl_udsty(ptr->vdi, 0xF0F0);
				/* Schreibmodus setzen */
				vswr_mode(ptr->vdi, MD_REPLACE);

				/* Und jetzt noch die erste selektierte Nachricht zentrieren
				 */
				centerSelection(ptr, FALSE);
				WdwManager_AcceptFont(ptr->wdw, treeList_treeChangeFont);
				WdwManager_AcceptColor(ptr->wdw, treeList_treeChangeColor);
			} else
			{
				/* Fenster konnte nicht geoeffnet werden */
				SFREE(ptr->tree);	/* Speicher freigeben */
				mtAlerts_AlertRSC(1, AL_NO_TREE_WIN);
				handlePool_FreeOneDataHandle(handlePtr);
				handlePool_FreeOnePtr(ptr, &windows);
				ClearNum(ptr->number);
			}
		} else
		{
			/* Gruppe konnte nicht geoeffnet werden */
			handlePool_FreeOnePtr(ptr, &windows);
		}
	} else
	{
		/* Kein Platz mehr fuer Fenstereintrag */
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Ein Baumfenster wird geschlossen */
GLOBAL _VOID treeList_treeClose(HWND wdw)
{
	if (FindWind(wdw) != NULL)
		WdwManager_CloseWindow(wdw, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Schliesst alle Fenster des Moduls
 */
GLOBAL _VOID treeList_treeCloseAll(_VOID)
{
	/* Alle Sichwortlistenfenster werden geschlossen */
	treeWdwPtr lauf;

	Lists_ResetList(&windows);
	while ((lauf = Lists_NextEntry(&windows)) != NULL)
		treeList_treeClose(lauf->wdw);
	Lists_DeleteList(&windows);
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet alle Fenster des Moduls neu
 */
GLOBAL _VOID treeList_treeRedrawAll(_VOID)
{
	treeWdwPtr lauf;

	Lists_ResetList(&windows);
	while ((lauf = Lists_NextEntry(&windows)) != NULL)
	{
		treeList_GetSettings(lauf);
		WdwManager_FullRedrawWdw(lauf->wdw);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Ist ein Kommentarbaumfenster oben? */
GLOBAL _BOOL treeList_treeWindowTop(HWND wdw)
{
	return FindWind(wdw) != NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID treeList_SelectFont(HWND wdw)
{
	treeWdwPtr p;
	
	p = FindWind(wdw);
	if (p != NULL)
	{
		VDIUtil_SelectFont(p->vdi, wdw, FALSE, p->font, p->fontSize);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt den Font fuer das Kommentarbaumfenster */
GLOBAL _VOID treeList_treeSetFont(HWND wdw, _WORD font, _WORD fontSize)
{
	iTreeSetFont(wdw, font, fontSize, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Alle Nachrichten mit geaenderten Flags neu darstellen */
LOCAL _VOID iTreeSetFlags(treeWdwPtr p, MSGBITS setBits, MSGBITS clearBits, _BOOL writeBits, MSGNR msgNum)
{
	MSGBITS bits;
	MSGBITS newBits;
	MSGNR i;
	MSGNR counter;
	_BOOL vis;
	_BOOL first;
	GRECT r;
	GRECT clip;
	_WORD readChg=0;
	MSGBITS maske=bGelesen;
	MSGNR count;
	if(UserBLK.ignoreFiltered)
		maske|=bFiltered;
	count = data_unreadMsgCount(p->grHandle->group);
	
	if (writeBits)
		CatGlobal_busyMouse();
	counter = 0;
	first = TRUE;
	for (i = 0; i < p->treeElems; i++)
	{
		if ((writeBits && p->tree[i].selected) ||
			(!writeBits && p->tree[i].msgNum == msgNum))
		{
			if (!writeBits ||
				data_ReadState(p->grHandle->Zugriff, p->tree[i].msgNum, &bits) == GERR_OK)
			{
				/* Neue Flags berechnen */
				newBits = (bits & ~clearBits) | setBits;
				/* Flags setzen */
				if ((!writeBits ||
					 data_SetBits(p->grHandle->Zugriff, p->tree[i].msgNum, newBits) == GERR_OK) &&
					newBits != p->tree[i].flags)
				{
					if((bits & maske) && !(newBits & maske))
						readChg=3;
					else
					{
						if(!(bits & maske) && (newBits & maske))
						{
							count--;
							readChg|=1;
						}
					}
					/* Und auch den Nachrichtenfenstern und der Messageliste Bescheid sagen */
					msgList_listChangeFlags(p->grHandle->group, p->tree[i].msgNum, newBits);
					grin_grinChangeFlags(p->grHandle->group, p->tree[i].msgNum, newBits);
					p->tree[i].flags = newBits;
					getEntryArea(p, i, &r, &vis);
					if (vis)
					{
						if (first)
						{
							clip = r;
							first = FALSE;
						} else
						{
							clip = FrameRects(&clip, &r);
						}
					}
					if(((bits^newBits)&bOwnMessage)!=0)
						grin_setCoO(p->grHandle->Zugriff, p->tree[i].msgNum, newBits&bOwnMessage);
				}
				++counter;
				if (writeBits && (counter % 50) == 0)
				{
					CatGlobal_busyMouse();
				}
			}
		}
	}
	if(readChg==3)
		data_forceUnreadRefresh(p->grHandle->Zugriff);
	else
		data_SetUnreadMsgCount(p->grHandle->group, count);
	if(readChg)
		grin_grinChangeUngel(p->grHandle->group);
	if (writeBits)
		MouseArrow();
	if (!first)
	{
		WdwManager_RedrawWdw(p->wdw, &clip);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Fuer alle selektierten Nachrichten
 * ein paar Flags setzen oder loeschen
 */
GLOBAL _VOID treeList_treeSetFlags(HWND wdw, MSGBITS setBits, MSGBITS clearBits)
{
	treeWdwPtr p;

	if ((p = FindWind(wdw)) != NULL)
		iTreeSetFlags(p, setBits, clearBits, TRUE, dataSys_empty);
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID treeList_treeSendState(HWND wdw, _UBYTE newState)						*/
/*	Verschickt State-Nachrichten fr alle selektierten 							*/
/*	Parameter:	wdw:		Baum-Fenster										*/
/*				newState:	z.B. 'X'											*/
/*	Ergebnis:	keines															*/		
/*	9.12.01 von Dimitri Junker	DYJ 4.30zo										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.23k */
GLOBAL _VOID treeList_treeSendState(HWND wdw, _UBYTE newState)
{
	treeWdwPtr p;
	MSGNR  i;
	MessageType mess;
	
	MemSetZeroStruct(mess);
	if ((p = FindWind(wdw)) != NULL)
	{
		CatGlobal_busyMouse();
		for (i = 0; i < p->treeElems; i++)
		{
			if (p->tree[i].selected)
			{
				if (data_ReadHeader(p->grHandle->Zugriff, p->tree[i].msgNum, &mess,TRUE) == GERR_OK)
					Messages_SendState(mess.MailID, newState);
			}
		}
		MouseArrow();
		/* Und jetzt vorsichtshalber Redraw ausloesen */
		WdwManager_FullRedrawWdw(wdw);
	}
}
/*** ---------------------------------------------------------------------- ***/

/* die Stichwortliste teil uns mit, dass sich bei einer Nachricht
 * die Flags geaendert haben
 */

typedef struct _treeChangeInfo {
	CAT_GROUP_NUMBER group;
	MSGNR msgNum;
	MSGBITS newBits;
} treeChangeInfo;

LOCAL _BOOL iTreeChangeFlags(_VOID *e, _VOID *i)
{
	treeChangeInfo *info;
	treeWdwPtr p;
	
	info = i;
	p = e;
	if (p->grHandle->group == info->group)
		iTreeSetFlags(e, info->newBits, ~0, FALSE, info->msgNum);
	return FALSE;
}

GLOBAL _VOID treeList_treeChangeFlags(CAT_GROUP_NUMBER group, MSGNR msgNum, MSGBITS newBits)
{
	treeChangeInfo info;
	
	info.group = group;
	info.msgNum = msgNum;
	info.newBits = newBits;
	Lists_ResetList(&windows);
	Lists_ScanEntries(&windows, Lists_forward, iTreeChangeFlags, &info);
}

/*** ---------------------------------------------------------------------- ***/

/* Alle Nachrichten in dem Fenster werden selektiert
 */
GLOBAL _VOID treeList_treeSelectAll(HWND wdw)
{
	treeWdwPtr ptr;
	MSGNR idx;
	_BOOL first;
	GRECT r;
	GRECT clip;
	_BOOL vis;

	if ((ptr = FindWind(wdw)) != NULL)
	{
		first = TRUE;
		for (idx = 0; idx < ptr->treeElems; idx++)
		{
			if (!ptr->tree[idx].selected)
			{
				ptr->tree[idx].selected = TRUE;
				getEntryArea(ptr, idx, &r, &vis);
				if (vis)
				{
					if (first)
					{
						clip = r;
						first = FALSE;
					} else
					{
						clip = FrameRects(&clip, &r);
					}
				}
			}
		}
		if (!first)
		{
			WdwManager_RedrawWdw(wdw, &clip);
		}
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL treeList_hasSelectedEntries(treeWindow *ptr, MSGNR *first)		*/
/*	Gibt zurck ob die Liste selektierte Elemente hat und ggf. das 1.			*/
/*	Parameter:	ptr:	der Baum												*/
/*				first	fr die Rckgabe des ersten selektierten Eintrags		*/
/*	Ergebnis:	TRUE falls mindestens ein selektierter Eintrag					*/
/*	Abgeleitet von MsgListBase_hasSelectedEntries								*/
/*	10.10.00 von Dimitri Junker	DYJ 4.18za										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL treeList_hasSelectedEntries(treeWindow *ptr, MSGNR *first)
{
	MSGNR i;
	
	for (i = 0; i < ptr->treeElems; i++)
		if (ptr->tree[i].selected )
		{
			*first = i;
			return TRUE;
		}
	return FALSE;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL treeList_listSomethingSelected(HWND wdw)						*/
/*	Gibt zurck ob die Liste im angegebenen Fenster selektierte Elemente hat.	*/
/*	Parameter:	wdw:	das Fenster												*/
/*	Ergebnis:	TRUE falls mindestens ein selektierter Eintrag					*/
/*	Abgeleitet von msgList_listSomethingSelected								*/
/*	10.10.00 von Dimitri Junker	DYJ 4.18za										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL treeList_listSomethingSelected(HWND wdw)
{
	treeWdwPtr p;
	MSGNR firstSel;
	
	if ((p = FindWind(wdw)) != NULL)
		return treeList_hasSelectedEntries(p, &firstSel);
	return FALSE;
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID treeSaveDecode(treeWindow *ptr)									*/
/*	Dekodiert selektierte Messages.												*/
/*	Parameter:	ptr:	der Baum												*/
/*	Ergebnis:	keines															*/
/*	Abgeleitet von listSaveDecode												*/
/*	10.10.00 von Dimitri Junker	DYJ 4.18za										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.21b */	/*DYJ 5.23k */
LOCAL _VOID treeSaveDecode(treeWindow *ptr)
{
	MSGNR firstSel, msgNum, idx;
	MessageType mess;
	_UBYTE *text, *chPtr;
	size_t textIdx;
	_UBYTE saveCh;
	F_ERROR error;
	
	MemSetZeroStruct(mess);
	if (treeList_hasSelectedEntries(ptr, &firstSel))
	{
		CatGlobal_busyMouse();
		UUDecode_BeginDecode();
		msgNum = ptr->tree[firstSel].msgNum;
		idx = firstSel;
		while (msgNum < dataSys_empty)
		{
			CatGlobal_busyMouse();
			/* Message lesen */
			error = data_ReadMessage(ptr->grHandle->Zugriff, msgNum, &mess);
			if (error == NoError && mess.Text != NULL && mess.InfoStrings != NULL)
			{
				textIdx = 0;
				chPtr = mess.Text;
				while (textIdx < mess.textLen)
				{
					/* Startadresse festlegen */
					text = chPtr;
					/* Zeilenende suchen */
					while (textIdx < mess.textLen && *chPtr != CR && *chPtr != LF)
					{
						++textIdx;
						++chPtr;
					}
					saveCh = *chPtr;
					*chPtr = '\0';
					if ((error = UUDecode_DecodeLine(text)) != GERR_OK)
					{
						if (error == -1)
							MTE_infoRSC(AL_UUDECODE_ERROR);
						else
							F_ErrorAlert(error);
						data_freeMessage(&mess);
						UUDecode_EndDecode();
						MouseArrow();
						return;
					}
					*chPtr = saveCh;
					++textIdx;
					++chPtr;
					/* Naechsten Zeilenanfang finden */
					while (textIdx < mess.textLen && (*chPtr == CR || *chPtr == LF))
					{
						++textIdx;
						++chPtr;
					}
				}
				/* Speicher freigeben */
				data_freeMessage(&mess);
			}
			/* Naechste selektierte finden */
			++idx;
			while (idx < ptr->treeElems && !(ptr->tree[idx].selected ))
				++idx;
			if (idx < ptr->treeElems)
				msgNum = ptr->tree[idx].msgNum;
			else
				msgNum = dataSys_empty;
		}
		UUDecode_EndDecode();
		MouseArrow();
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID treeSaveList(treeWindow *ptr, TEXTFILENO f)						*/
/*	Schreibt selektierte Messages als Liste in ein File.						*/
/*	Parameter:	ptr:	der Baum												*/
/*				f:		das File in das geschrieben werden soll					*/
/*	Ergebnis:	keines															*/
/*	Abgeleitet von listSaveList													*/
/*	10.10.00 von Dimitri Junker	DYJ 4.18za										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _VOID treeSaveList(treeWindow *ptr, TEXTFILENO f)
{
	MSGNR firstSel, msgNum, idx;
	
	if (treeList_hasSelectedEntries(ptr, &firstSel))
	{
		msgNum = ptr->tree[firstSel].msgNum;
		idx = firstSel;
		while (msgNum < dataSys_empty)
		{
			mtTextfiles_WriteLine(f, ptr->tree[idx].name);
			mtTextfiles_WriteLn(f);
			mtTextfiles_WriteLine(f, ptr->tree[idx].betreff);
			mtTextfiles_WriteLn(f);
			mtTextfiles_WriteLn(f);
			/* Naechste selektierte finden */
			++idx;
			while (idx < ptr->treeElems && !(ptr->tree[idx].selected))
				++idx;
			if (idx < ptr->treeElems)
				msgNum = ptr->tree[idx].msgNum;
			else
				msgNum = dataSys_empty;
		}
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID treeSaveOutfile(treeWindow *ptr, TEXTFILENO f, _BOOL append)	*/
/*	Schreibt selektierte Messages als Outfile in ein File.						*/
/*	Parameter:	ptr:	der Baum												*/
/*				f:		das File in das geschrieben werden soll					*/
/*				append:	Fals TRUE mu die alte Endmarkierung gelscht werden	*/
/*	Ergebnis:	keines															*/
/*	Abgeleitet von listSaveOutfile												*/
/*	10.10.00 von Dimitri Junker	DYJ 4.18za										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.00b */	/*DYJ 5.19b */	/*DYJ 5.21b */	/*DYJ 5.23k */
LOCAL _VOID treeSaveOutfile(treeWindow *ptr, TEXTFILENO f, _BOOL append)
{
	MSGNR firstSel, msgNum, idx;
	MessageType mess;
	group_line groupName;
	_UBYTE dateLine[256];
	_ULONG fPos;
	F_ERROR error;
	
	MemSetZeroStruct(mess);
	if (treeList_hasSelectedEntries(ptr, &firstSel))
	{
		if (append)
		{
			/* #+CR+LF am Dateiende entfernen */
			fPos = mtTextfiles_Textpos(f);
			/* Ein Test ist hier nicht noetig, append wird nur
			 * gesetzt, wenn es moeglich ist
			 */
			mtTextfiles_SetTextpos(f, fPos-3);
		}
		else
		{
			time_t timet;
			char datum[30];
			/* Outfile-Header ausgeben */
			data_LineOut(f, "#","HEAD");
			data_LineOut(f, ":T",CatDate);
			data_LineOut(f, ":V",CatVersion);
			data_LineOut(f, ":B",CAT_PROGNAME);
			time(&timet);
			strftime(datum,sizeof(datum),"%Y%m%d%H%M\n",localtime(&timet));
			data_LineOut(f, ":D",datum);
			if(*userRName)
				data_LineOut(f, ":U", userRName);
			else
				data_LineOut(f, ":U", userAddr);
			data_LineOut(f, ":S","G");
			data_LineOut(f, ":Z","ATARI");
		}
		/* Gruppennamen rausfinden */
		if (ptr->grHandle->group == dataSys_private)
			*groupName = '\0';
		else
		{
			if (GroupSelect_isPrivate(ptr->grHandle->group) &&
				mtAlerts_AlertRSC(1, AL_PSGP)==2)
				*groupName = '\0';
			else
				GroupSelect_GroupName(ptr->grHandle->group, groupName, sizeof(groupName));
		}
		msgNum = ptr->tree[firstSel].msgNum;
		idx = firstSel;
		while (msgNum < dataSys_empty)
		{
			/* Message lesen */
			error = data_ReadMessage(ptr->grHandle->Zugriff, msgNum, &mess);
			if (error == NoError)
			{
				/* Header ausgeben */
				data_LineOut(f, "#", mess.MailID);
				if (!(*groupName) && mess.EigeneNachricht &&
					(mess.Absender == NULL || mess.Absender[0] == '\0'))
				{
					data_LineOut(f, "V", userAddr);
				} else
				{
					data_LineOut(f, "V", mess.Absender);
				}
				data_LineOut(f, "W", mess.Betreff);
				data_LineOut(f, "A", mess.Empfaenger);
				data_LineOut(f, "G", groupName);
				/* Datum noch holen */
				data_buildDate(MessDate(mess), dateLine, sizeof(dateLine));
				data_LineOut(f, "E", dateLine);
				data_LineOut(f, "-", mess.KommentierteID);
				/* Jetzt noch den Usenet-Krempel */
				data_LineOut(f, "N", mess.name);
				data_LineOut(f, "I", mess.mid);
				data_LineOut(f, "R", mess.rid);
				data_LineOut(f, "O", mess.box);
				data_LineOut(f, "M", mess.mime);
				data_LineOut(f, "F", mess.followupTo);
				data_LineOut(f, "S", mess.sender);
				data_LineOut(f, "T", mess.replyTo);
				data_LineOut(f, "Y", mess.gate);
				/* Distribution rauspusten */
				if (mess.distribution != distNone)
				{
					dateLine[0] = "KLMN"[mess.distribution];
					dateLine[1] = '\0';
					data_LineOut(f, "D", dateLine);
				}
				/* Bei der persoenlichen Gruppe muss eine Statuszeile ausgegeben werden
				 */
				if (!(*groupName))
				{
					data_buildDate(mess.statusDate,NULL, dateLine+1, sizeof(dateLine)-1);
					dateLine[0] = mess.Status;
					data_LineOut(f, "B", dateLine);
				}
				/* Jetzt noch die Statuszeile */
				data_buildFlagLine(mess.StatusBits, dateLine);
				data_LineOut(f, "s", dateLine);
				/* Und jetzt koennen wir den Text ausgeben */
				data_messageOut(f, &mess, ":");
				/* Speicher freigeben */
				data_freeMessage(&mess);
			}
			/* Naechste selektierte finden */
			++idx;
			while (idx < ptr->treeElems && !(ptr->tree[idx].selected))
				++idx;
			if (idx < ptr->treeElems)
				msgNum = ptr->tree[idx].msgNum;
			else
				msgNum = dataSys_empty;
		}
		/* Jetzt noch das Logfile schreiben */
		data_LineOut(f, "#", "LOG");
		data_LineOut(f, ":!V", CatDate);
		data_LineOut(f, ":!CAT ", CatVersion);
		if(*userRName)
			data_LineOut(f, ":!Database von ", userRName);
		else
			data_LineOut(f, ":!Database von ", userAddr);
		mtTextfiles_WriteLine(f, "#");
		mtTextfiles_WriteLn(f);
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID lineOut(TEXTFILENO file, _WORD strNr, CONST _UBYTE *str)				*/
/*	Eine abgewandelte Version von data_LineOut hier wird kein String, sondern	*/
/*	Die Nummer des Strings im RSC bergeben										*/
/*	Parameter:	file:	darein schreiben										*/
/*				strNr:	Stringnummer im RSC										*/
/*				str:	2. String												*/
/*	Ergebnis:	keines															*/		
/*	26.2.03 von Dimitri Junker	DYJ 5.21b										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _VOID lineOut(TEXTFILENO file, _WORD strNr, CONST _UBYTE *str)
{
	if(strNr<0)
	{
		_UBYTE tmpLine[30];
		strBcpy(tmpLine,MausTauschrsc_WrkString(-strNr));
		strBcat(tmpLine," ");
		data_LineOut(file, tmpLine, str);
	}
	else
		data_LineOut(file, MausTauschrsc_WrkString(strNr), str);
}

/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID treeSaveText(treeWindow *ptr, TEXTFILENO f, _BOOL dataMode)		*/
/*	Schreibt selektierte Messages als Text in ein File.							*/
/*	Parameter:	ptr:	der Baum												*/
/*				f:		das File in das geschrieben werden soll					*/
/*				dataMode:	Fals TRUE ohne Kopf,...								*/
/*	Ergebnis:	keines															*/
/*	Abgeleitet von listSaveText													*/
/*	10.10.00 von Dimitri Junker	DYJ 4.18za										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.21b */	/*DYJ 5.23k */
LOCAL _VOID treeSaveText(treeWindow *ptr, TEXTFILENO f, _BOOL dataMode)
{
	MSGNR firstSel, msgNum, idx;
	MessageType mess;
	group_line groupName;
	_UBYTE tmpLine[1024];
	_WORD emptyLines, i;
	F_ERROR error;
	
	MemSetZeroStruct(mess);
	if (treeList_hasSelectedEntries(ptr, &firstSel))
	{
		ConfVars_GetConfDefInt(cPrintEmptyLines, &emptyLines, 0);
		/* Gruppennamen rausfinden */
		GroupSelect_GroupDisplayName(ptr->grHandle->group, groupName, sizeof(groupName));
		msgNum = ptr->tree[firstSel].msgNum;
		idx = firstSel;
		while (msgNum < dataSys_empty)
		{
			/* Message lesen */
			error = data_ReadMessage(ptr->grHandle->Zugriff, msgNum, &mess);
			if (error == NoError && mess.Text != NULL && mess.InfoStrings != NULL)
			{
				if (!dataMode)
				{
					/* Header ausgeben */
					data_LineOut(f, "ID  : ", mess.MailID);
					lineOut(f, -msgGroup, groupName);
					data_LineOut(f, "Followup-To: ", mess.followupTo);
					lineOut(f, FS_COMMENT, mess.KommentierteID);
					if (mess.Absender != NULL)
					{
						strBcpy(tmpLine, mess.Absender);
						strBcat(tmpLine, " (");
						strBcat(tmpLine, mess.Datum);
						strBcat(tmpLine, ")");
						lineOut(f, -HEAD_VON, tmpLine);
					}
					lineOut(f, -HEAD_NAME, mess.name);
					data_LineOut(f, "Sender: ", mess.sender);
					data_LineOut(f, "Reply-To: ", mess.replyTo);
					lineOut(f, -HEAD_WG, mess.Betreff);
					if (mess.Empfaenger != NULL)
					{
						strBcpy(tmpLine, mess.Empfaenger);
						if (mess.Gruppe == dataSys_private && mess.EigeneNachricht)
						{
							strBcat(tmpLine, " (");
							strBcat(tmpLine, mess.Datum);
							strBcat(tmpLine, ")");
						}
						lineOut(f, -msgAn, tmpLine);
					}
					/* Jetzt noch den Usenet-Krempel */
					data_LineOut(f, "Box : ", mess.box);
					data_LineOut(f, "MId : ", mess.mid);
					data_LineOut(f, "RId : ", mess.rid);
					data_LineOut(f, "Gate: ", mess.gate);
					data_LineOut(f, "MIME: ", mess.mime);
					mtTextfiles_WriteLn(f);
				}
				
				/* Und jetzt koennen wir den Text ausgeben */
				data_messageOut(f, &mess, "");
				
				if (!dataMode)
				{
					/* Jetzt noch ein paar Trennzeilen */
					for (i = 0; i < emptyLines; i++)
						mtTextfiles_WriteLn(f);
				}
				
				/* Speicher freigeben */
				data_freeMessage(&mess);
			}
			/* Naechste selektierte finden */
			++idx;
			while (idx < ptr->treeElems && !(ptr->tree[idx].selected))
				++idx;
			if (idx < ptr->treeElems)
				msgNum = ptr->tree[idx].msgNum;
			else
				msgNum = dataSys_empty;
		}
	}
}
/*-- Ausgabefunktionen -------------------------------------------------*/

/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID treeSaveStatus(treeWindow *ptr, TEXTFILENO f)					*/
/*	Schreibt Status der selektierten Messages in ein File.						*/
/*	Parameter:	ptr:	der Baum												*/
/*				f:		das File in das geschrieben werden soll					*/
/*	Ergebnis:	keines															*/
/*	Abgeleitet von listSaveStatus												*/
/*	10.10.00 von Dimitri Junker	DYJ 4.18za										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.23k */
LOCAL _VOID treeSaveStatus(treeWindow *ptr, TEXTFILENO f)
{
	MSGNR firstSel, msgNum, idx;
	MessageType mess;
	F_ERROR error;
	MemSetZeroStruct(mess);
	
	if (treeList_hasSelectedEntries(ptr, &firstSel))
	{
		msgNum = ptr->tree[firstSel].msgNum;
		idx = firstSel;
		while (msgNum < dataSys_empty)
		{
			/* Header der Message lesen */
			error = data_ReadHeader(ptr->grHandle->Zugriff, msgNum, &mess,TRUE);
			if (error == NoError)
			{
				if(mess.MailID)		
				{
					/* MsgId ausgeben */
					mtTextfiles_WriteLine(f, "#");
					mtTextfiles_WriteLine(f, mess.MailID);
					mtTextfiles_WriteLn(f);
					/* Status ausgeben */
					mtTextfiles_WriteLine(f, "BG");
					mtTextfiles_WriteLn(f);
				}
				data_freeMessage(&mess);
			}
			/* Naechste selektierte finden */
			++idx;
			while (idx < ptr->treeElems && !(ptr->tree[idx].selected))
				++idx;
			if (idx < ptr->treeElems)
				msgNum = ptr->tree[idx].msgNum;
			else
				msgNum = dataSys_empty;
		}
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID treeSave(treeWdwPtr ptr, CONST _UBYTE *path, CONST _UBYTE *name,*/
/*							 _WORD saveMode, _BOOL askOverwrite, _BOOL append)	*/
/*	Schreibt selektierte Messages in ein File.									*/
/*	Parameter:	ptr:		der Baum											*/
/*				path:		der Pfad des Files									*/
/*				name:		der Name des Files									*/
/*				saveMode:	Liste, Outfile,...									*/
/*				askOverwrite:	Sicherheitsabfrage zum berschreiben?			*/
/*				append:		Anhngen ode berschreiben							*/
/*	Ergebnis:	keines															*/
/*	Abgeleitet von listSaveStatus												*/
/*	10.10.00 von Dimitri Junker	DYJ 4.18za										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.21b */	/*DYJ 5.22c */
LOCAL _VOID treeSave(treeWdwPtr ptr, CONST _UBYTE *path, CONST _UBYTE *name, _WORD saveMode, _BOOL askOverwrite, _BOOL append)
{
	_UBYTE fname[PATH_MAX];
	Textmode fileOpenMode;
	_BOOL exists;
	TEXTFILENO out;
	_BOOL isOutfile;
	_ULONG fPos;
	_WORD i;
	_WORD select;
	_UBYTE tmp[4];
	F_ERROR err;
	
	/* Dateinamen basteln */
	strBcpy(fname, path);
	F_Path_Append(fname, name);
	/* Erstmal feststellen, ob die Datei existiert */
	fPos = CatFiles_FileSize(fname, &exists);
	fileOpenMode = WRITE;
	if (exists)
	{
		if (askOverwrite)
		{
			select = mtAlerts_Alert(2, Rsc_WrkString(AL_OVERWRITE3));
			if (select == 2)
				fileOpenMode = APPEND;
			else if (select == 1)
				fileOpenMode = WRITE;
			else
				return;
		} else
		{
			if (append)
				fileOpenMode = APPEND;
			else
				fileOpenMode = WRITE;
		}
	}
	isOutfile = FALSE;
	if (fileOpenMode == APPEND)
	{
		/* Erstmal kurz die letzten drei Byte lesen */
		if (mtTextfiles_OpenTextfile(fname, READ, 0, &out) == GERR_OK)
		{
			/* In fPos steht schon die Groesse */
			if (fPos >= 3)
			{
				mtTextfiles_SetTextpos(out, fPos - 3);
				for (i = 0; i < 3; i++)
					tmp[i] = mtTextfiles_ReadChar(out);
				tmp[3] = '\0';
				/* Jetzt Test auf #+CR+LF */
				if (strCmp(tmp, "#\r\n") == 0)
					isOutfile = TRUE;
			}
			mtTextfiles_CloseTextfile(&out);
		}
	}
	
	/* So, jetzt die Datei oeffnen */
	if ((err = mtTextfiles_OpenTextfile(fname, fileOpenMode, 0, &out)) == GERR_OK)
	{
		MouseBusy();
		switch (saveMode)
		{
			case smListe :   treeSaveList(ptr, out); break;
			case smOutfile:  treeSaveOutfile(ptr, out, isOutfile); break;
			case smText  :   treeSaveText(ptr, out, FALSE); break;
			case smData  :   treeSaveText(ptr, out, TRUE); break;
			case smStatus:   treeSaveStatus(ptr, out); break;
		}
		mtTextfiles_CloseTextfile(&out);
		MouseArrow();
	} else
	{
		F_ErrorAlert(err);
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL treeList_treeSaveAs(HWND wdw, _BOOL asStatus)					*/
/*	Speichert die selektierten Nachrichten in einer Datei 						*/
/*	Parameter:	wdw:		Fenster in dem der baum ist							*/
/*				asStatus:	Status schreiben?									*/
/*	Ergebnis:	Fehlerfrei->TRUE												*/		
/*	10.10.00 von Dimitri Junker	DYJ 4.18za										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.21b */
GLOBAL _BOOL treeList_treeSaveAs(HWND wdw, _BOOL asStatus)
{
	treeWdwPtr p;
	_WORD saveMode;
	MSGNR firstSel;
	_WORD select;
	
	if ((	p = FindWind(wdw)) != NULL)
	{
		if (treeList_hasSelectedEntries(p, &firstSel))
		{
			saveMode = mtAlerts_AlertRSC(lastSaveMode, AL_LISTMODE);
			if (saveMode == 5)
				return FALSE;
			lastSaveMode = saveMode;
			if (saveMode == smListe && asStatus)
			{
				select = mtAlerts_AlertRSC(1, AL_AS_STATUS);
				if (select == 1)
					saveMode = smStatus;
			}
			if (saveMode == smDecode)
			{
				treeSaveDecode(p);
			} else
			{
				/* Jetzt Datei auswaehlen */
				if (!mtDir_FselGet(SavePath, SaveName, "*.txt", MausTauschrsc_WrkString(FS_MESSAGES_SAVEAS), FALSE))
					return FALSE;
				/* Ok, Name ist ausgewaehlt */
				treeSave(p, SavePath, SaveName, saveMode, TRUE, FALSE);
				Protokoll_SendPathUpdate(SavePath);
			}
		}
		return TRUE;
	}
	return FALSE;
}
