/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <watchdog.h>
#include <portaes.h>
#include <protokol.h>
#include <catproto.h>
#include <xacc.h>
#include <vaproto.h>
#include <userinfo.h>
#include <wdwmanag.h>
#include <catgloba.h>

#include <mtappl.h>
#include <routines.h>
#include <cat.h>
#include <maustaus.h>

#include <debug.h>

#define watchDog   "WATCHDOG"
#define watchFound 0x0815

LOCAL _WORD watch = -1;	/* Ist ein WatchDog installiert? */
LOCAL _BOOL isWatch;	/* Soll der auch gefragt werden? */

#define enableWatchDog  0
#define disableWatchDog 1
#define CatAffirm       2

/* GutenTag */
#define HelloWatchDog  0x8000

/*** ---------------------------------------------------------------------- ***/

/* WatchDog-Initialisierungsmsg von Cat */
LOCAL _VOID SendToWatchDog(_VOID)
{
	_WORD message[8];

	message[0] = HelloWatchDog;
	message[1] = gl_apid;
	message[2] = 0;
	message[3] = CatGreetings;
	message[4] = CatProtoVersion;
	message[5] = 0;
	message[6] = 0;
	message[7] = 0;
	appl_write(watch, 16, message);
	Wind_Update(END_UPDATE);
	Wind_Update(BEG_UPDATE);
	isWatch = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Ist das ein WatchDog? Ja -> Begruessung losschicken */
/*DYJ 5.21b */
LOCAL _VOID WatchDog_CheckName(CONST _UBYTE *str, _WORD id)
{
	if (strCmp(str, watchDog) == 0)
	{
		watch = id;
		if (UserBLK.watchBox)
			mtAlerts_AlertRSC(1, AL_WATCHDOG_JIPPIE);
		SendToWatchDog();
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Diese Prozedur wird von parser.i benutzt, um herauszufinden, ob die gerade
 * reinkommende Msg passieren darf
 */
/*DYJ 5.21b */
GLOBAL watchType WatchDog_AcceptThis(PtrRecord *ptr)
{
	_WORD message[8];
	_UWORD event;
	_UWORD retry;
	_UWORD noMess;
	_VOID *mem;
	_WORD dummy;
	_UWORD udummy;

	if (!Protokoll_Init())
		return pCatAccept;
	
	if (!isWatch)
		return pCatAccept;
	ASSERT(memDescrSize >= sizeof(*ptr));
	mem = memPtr;
	/* Block umkopieren in globalen Speicher */
	*((PtrRecord *)mem) = *ptr;

#define is_watchdog_message() \
	((_UWORD)message[0] == HelloWatchDog && \
		(message[3] == CatForget || message[3] == CatAccept || message[3] == CatFiltered))

	Protokoll_SendMess(HelloWatchDog, CatAsk, (_ULONG)(mem), 0, 0, watch);
	retry = 0;
	noMess = 0;
	do /* Eine kleine Schleife mit TimeOut und Retry-Zaehler */
	{
		event = evnt_multi(MU_MESAG | MU_TIMER,
			0, 0, 0,
			0, 0, 0, 0, 0,
			0, 0, 0, 0, 0,
			message,
			1000, 0,
			&dummy, &dummy, &udummy, &udummy, &dummy, &dummy);

		if ((event & MU_MESAG) && !is_watchdog_message())
		{
			/* Standardmessages direkt verarbeiten */
			if (!WdwManager_HandleEvent(&event, message, 0, 0, '\0', '\0', 0, 0, 0, 0))
			{
				/* Nicht verarbeitet? Aufbewahren, wird vielleicht spaeter
				 * mal verarbeitet
				 */
				appl_write(gl_apid, 16, message);
			}

			++noMess;
			if (noMess > 100)
			{
				event  = MU_TIMER;
				noMess = 0;
			}
		}

		if (event == MU_TIMER) /* entweder sowieso oder von oben */
		{
			++retry;
			if (retry > 5)
			{
				isWatch = FALSE;
			} else if (retry > 2)
			{
				if (mtAlerts_AlertRSC(1, AL_WATCHDOG) != 1)
					isWatch = FALSE;
			}
		}

	} while (!is_watchdog_message() && isWatch);

	if (isWatch)
	{
		switch (message[3])
		{
			case CatAccept: return pCatAccept;
			case CatForget: return pCatForget;
			case CatFiltered: return pCatFiltered;
			default: return pCatAccept;
		}
	}
	return pCatAccept;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID WatchDog_SendTerminate(_VOID)
{
	if (watch >= 0)
		Protokoll_SendMess(HelloWatchDog, CatTerminate, 0, 0, 0, watch);
}

/*** ---------------------------------------------------------------------- ***/

/* Ist der WatchDog angemeldet und aktiv? */
GLOBAL _VOID WatchDog_isWatchDog(_BOOL *installed, _BOOL *active)
{
	*installed = watch >= 0;
	*active    = isWatch;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CheckNameXACC(CONST _UBYTE *name, _WORD id)
{
	if (name > (CONST _UBYTE *)0xf000)
		WatchDog_CheckName(name, id);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CheckNameVA(CONST _UBYTE *name, _WORD id)
{
	if (name > (CONST _UBYTE *)0xf000)
		WatchDog_CheckName(name, id);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL WatchDog_HandleMessage(CONST _WORD *message)
{
	CONST struct {
		_WORD messId;
		_WORD apId;
		_WORD over;
		_WORD catType;
		union {
			_WORD data[4];
			struct {
				_WORD d0;
				_VOID *adr;
				_WORD d1;
			} a;
		} d;
	} *mess;
	_UBYTE *pName;
	
	switch (message[0])
	{
	case ACC_ID:
		pName = *((_UBYTE **)(&message[4]));    /* Pointer auf Text */
		CheckNameXACC(pName, message[1]);
		break;
	
	case ACC_ACC:
		if (multiTask)
		{
			pName  = *((_UBYTE **)(&message[4]));    /* Pointer auf Text */
			CheckNameXACC(pName, message[1]);
		}
		break;
	
	case AV_PROTOKOLL:
		pName = *((_UBYTE **)(&message[6]));
		CheckNameVA(pName, message[1]);
		break;
		
	default:
		if ((_UWORD)message[0] == CatMsg)
		{
			mess = (CONST _VOID *)message;
			switch (mess->catType)
			{
			case enableWatchDog:
				if (watch >= 0)
				{
					isWatch = TRUE;
					Protokoll_SendMessInt(CatMsg, CatAffirm, 0, 0, 0, 1, watch);
				}
				break;
		
			case disableWatchDog:
				if (watch >= 0)
				{
					isWatch = FALSE;
					Protokoll_SendMessInt(CatMsg, CatAffirm, 0, 0, 0, 0, watch);
				}
				break;
			default:
				return FALSE;
			}
		} else
		{
			return FALSE;
		}
		break;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID WatchDog_Init(_VOID)
{
	Protokoll_Handle_Message(WatchDog_HandleMessage);
}
