/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#include <wiederv2.h>
#include <ostruct.h>
#include <tos.h>
#include <data.h>
#include <mte.h>
#include <catgloba.h>
#include <catfiles.h>
#include <mtpaths.h>
#include <quicksor.h>
#include <confvars.h>
#include <mtrsc.h>
#include <mtappl.h>

#include <routinec.h>
#include <routines.h>
#include <s_endian.h>

#include <debug.h>

/* Magics fuer Header */
#define resubCatMagic     0x43415457L  /* "CATW"                           */
#define resubVersion      1            /* Versionsnummer der Wiedervorlage */
#define resubVersionMagic 0x0100       /* Versionsmagic                    */


typedef struct {
	data_offset group;
	data_offset msgId;
	data_offset from;
	data_offset topic;
	struct {					/* eigentlich Date */
		_UWORD day;
		_UWORD month;
		_WORD year;
	} date;
	struct {					/* eigentlich Time */
		_UWORD second;
		_UWORD minute;
		_UWORD hour;
	} time;
	struct {					/* eigentlich Date */
		_UWORD day;
		_UWORD month;
		_WORD year;
	} resubDate;
	data_offset dataLength;		/* Die Laenge des folgenden Datenblocks */
} PACKED resubHdr;


GLOBAL _BOOL listModified;

GLOBAL resubRec *resubArray;
LOCAL _ULONG maxResubs;
GLOBAL _ULONG resubs;            /* aktueller Index in Array */
GLOBAL _ULONG currIdx;
GLOBAL _WORD globalLength;
GLOBAL _WORD whichType;


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Alloziert ein neues Gruppenarray und kopiert das alte hinein.
 * FALSE: Nicht genug Speicher!
 */
GLOBAL _BOOL MakeNewResubArray(_BOOL force, _ULONG new)
{
	_ULONG newMax;
	resubRec *newArray;

	if (maxResubs == 0 || resubs >= maxResubs || force)
	{
		if (!force)
			new = maxResubs;
		if (new > (4294967295LU - 20u))
			return FALSE;
		newMax = new + 20;
		newArray = M0ALLOC(newMax * sizeof(*newArray), "MakeNewResubArray");
		if (newArray == NULL)
			return FALSE;
		if (resubArray != NULL)
		{
			MemCpy(newArray, resubArray, maxResubs * sizeof(*newArray));
			SFREE(resubArray);
		}
		maxResubs = newMax;
		resubArray = newArray;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Fuellt den Eintrag rs mit den Daten
 */
/*ME 4.36a*/
GLOBAL _BOOL MakeEntry(resubRec *rs,
	CONST _UBYTE *gName,
	CONST _UBYTE *mId,
	CONST _UBYTE *fromL,
	CONST _UBYTE *topicL,
	CONST Date *dt,
	CONST Time *ti,
	CONST Date *resubDt)
{
	size_t gLen, mLen, fLen, tLen, addLen;
	_UBYTE *target;

	SPEC_DEBUG_OUT((EO_DEBUG, "Start: MakeEntry()"));

	/* Laengen herausfinden */
/*Hier hatte Martin Elssser in der 4.36 einen Test auf NULL
eingebaut, dies hat sich durch die Einfuhr der Funktion strLen erbrigt,
und war der Auslser zu deren Einfuhr.*/
	gLen = strLen(gName) + 1;
	if (mId != NULL)
		mLen = strLen(mId) + 1;
	else
		mLen = 0;
	if (fromL != NULL)
		fLen = strLen(fromL) + 1;
	else
		fLen = 0;
	if (topicL != NULL)
		tLen = strLen(topicL) + 1;
	else
		tLen = 0;
	addLen = gLen + mLen + fLen + tLen;
	/* Jetzt Speicher allozieren */
	rs->dataLength = 0;
	rs->data = MALLOC(addLen, "Resub: MakeEntry");
	if (rs->data == NULL)
		return FALSE;
	rs->dataLength = addLen;
	
	/* Speicher vorhanden, jetzt Eintraege setzen */
	target = rs->data;
	if( gLen>0 )
	{
		MemCpy(target, gName, gLen);
		rs->group = target;
		target += gLen;
	}
	else
		rs->group = NULL;
	
	if (mLen > 0)
	{
		MemCpy(target, mId, mLen);
		rs->msgId = target;
		target += mLen;
	} else
	{
		rs->msgId = NULL;
	}
	
	if (fLen > 0)
	{
		MemCpy(target, fromL, fLen);
		rs->from = target;
		target += fLen;
	} else
	{
		rs->from = NULL;
	}
	
	if (tLen > 0)
	{
		MemCpy(target, topicL, tLen);
		rs->topic = target;
		/* target += tLen; */
	} else
	{
		rs->topic = NULL;
	}
	rs->date = *dt;
	rs->time = *ti;
	rs->resubDate = *resubDt;
	rs->selected = FALSE;
	SPEC_DEBUG_OUT((EO_DEBUG, "MakeEntry() == TRUE"));
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID freeResubArray(_VOID)
{
	_ULONG i;
	
	if (resubArray != NULL)
	{
		for (i = 0; i < resubs; i++)
			if (resubArray[i].data != NULL)
				SFREE(resubArray[i].data);
		SFREE(resubArray);
		resubArray = NULL;
	}
	resubs = 0;
	maxResubs = 0;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----- Eigene Liste schreiben und lesen ------------------------------*/

GLOBAL _BOOL SaveDataEntry(resubRec *rs, FILENO fHdl)
{
	resubHdr hdr;
	
	hdr.group = 0;
	hdr.msgId = 0;
	hdr.from = 0;
	hdr.topic = 0;
	if (rs->msgId != NULL)
		hdr.msgId = h2ns((data_offset)(rs->msgId - rs->data));
	if (rs->from != NULL)
		hdr.from = h2ns((data_offset)(rs->from - rs->data));
	if (rs->topic != NULL)
		hdr.topic = h2ns((data_offset)(rs->topic - rs->data));
	/* Jetzt die uebrigen Daten setzen */
	hdr.date.day = h2ns(rs->date.day);
	hdr.date.month = h2ns(rs->date.month);
	hdr.date.year = h2ns(rs->date.year);
	hdr.time.second = h2ns(rs->time.second);
	hdr.time.minute = h2ns(rs->time.minute);
	hdr.time.hour = h2ns(rs->time.hour);
	hdr.resubDate.day = h2ns(rs->resubDate.day);
	hdr.resubDate.month = h2ns(rs->resubDate.month);
	hdr.resubDate.year = h2ns(rs->resubDate.year);
	hdr.dataLength = h2ns((data_offset)(rs->dataLength));
	if (CatFiles_WriteMuch(sizeof(hdr), fHdl, &hdr) != NoError)
		return FALSE;
	/* Jetzt die Daten schreiben */
	if (CatFiles_WriteMuch(rs->dataLength, fHdl, rs->data) != NoError)
	{
		SPEC_DEBUG_OUT((EO_DEBUG, "SaveDataEntry() == FALSE"));
		return FALSE;
	}
	SPEC_DEBUG_OUT((EO_DEBUG, "SaveDataEntry() == TRUE"));
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL WiederVorlage_SaveResubList(_WORD modus)
{
	_UBYTE fname[PATH_MAX];
	_ULONG i;
	FileHeaderType dbHdr;
	FILENO fHdl;

	strBcpy(fname, DataPath);
 	
 	/* Je nach Modus unterschiedliche Dateien oeffnen (MH) */
 	if (modus == WVL)
 	{
 		F_Path_Append(fname, resubDataName);
 	} else
 	{
 		F_Path_Append(fname, cmtsDataName);
 	}
 	
	listModified = FALSE;
	/* Jetzt das Binaerfile schreiben */
	fHdl = CatFiles_CreateFile(fname);
	if (FILENO_OK(fHdl))
	{
		/* Header schreiben */
		dbHdr.CatMagic = h2nl(resubCatMagic);
		dbHdr.Version  = h2ns(resubVersion);
		dbHdr.VersionMagic = h2ns(resubVersionMagic);
		if (CatFiles_WriteMuch(dbHeaderLength, fHdl, &dbHdr) != NoError)
		{
 			sprintf(fname, Rsc_WrkString(MTE_noFileOpen), modus == WVL ? resubDataName : cmtsDataName);	/*DYJ 5.21i */
 			MTE_info(fname);
			CatFiles_CloseFile(fHdl);
			return FALSE;
		}
		i = 0;
		while (i < resubs && SaveDataEntry(&resubArray[i], fHdl))
			++i;
		CatFiles_CloseFile(fHdl);
		return i == resubs;
	}
	/* Fehlermeldung rauswerfen */
	sprintf(fname, Rsc_WrkString(MTE_noFileOpen), modus == WVL ? resubDataName : cmtsDataName);	/*DYJ 5.21i */
	MTE_info(fname);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL WiederVorlage_parseData(FILENO fhdl)
{
	FileHeaderType dbHdr;
	resubHdr hdr;
	resubRec *rs;
	F_ERROR error;
	
	if (CatFiles_ReadDbHeader(fhdl, &dbHdr) != NoError)
		return FALSE;
	/* Jetzt ueberpruefen des Headers */
	if (dbHdr.CatMagic != resubCatMagic ||
		dbHdr.Version  != resubVersion ||
		dbHdr.VersionMagic != resubVersionMagic)
	{
		/* Abbrechen */
		return FALSE;
	}
	/* hier eventuell mal zaehlen, wie viele es denn sind */
	if (!MakeNewResubArray(TRUE, 50))
		return FALSE;
	resubs = 0;
	do
	{
		/* Header lesen */
		error = CatFiles_ReadMuch(sizeof(hdr), fhdl, &hdr);
		if (error == NoError)
		{
			/* Ok, Header gelesen */
			if (!MakeNewResubArray(FALSE, 0))
				return FALSE;
			/* Jetzt Eintrag setzen */
			rs = &resubArray[resubs];
			rs->data = MALLOC(n2hs(hdr.dataLength), "Resub: parseData");
			if (rs->data == NULL)
				return FALSE;
			/* Jetzt Daten kopieren */
			/* Pointer setzen */
			rs->group = rs->data;
			rs->msgId = NULL;
			rs->from = NULL;
			rs->topic = NULL;
			if (hdr.msgId != 0)
				rs->msgId = rs->data + n2hs(hdr.msgId);
			if (hdr.from != 0)
				rs->from = rs->data + n2hs(hdr.from);
			if (hdr.topic != 0)
				rs->topic = rs->data + n2hs(hdr.topic);
			/* Jetzt die uebrigen Daten setzen */
			rs->date.day = n2hs(hdr.date.day);
			rs->date.month = n2hs(hdr.date.month);
			rs->date.year = n2hs(hdr.date.year);
			rs->time.hour = n2hs(hdr.time.hour);
			rs->time.minute = n2hs(hdr.time.minute);
			rs->time.second = n2hs(hdr.time.second);
			rs->resubDate.day = hdr.resubDate.day;          /* F-Zeile */
			rs->resubDate.month = hdr.resubDate.month;
			rs->resubDate.year = n2hs(hdr.resubDate.year);
			rs->selected = FALSE;                           /* Fuer Darstellung in Liste, wird beim Einlesen auf FALSE gesetzt */
			rs->dataLength = n2hs(hdr.dataLength);          /* Die Laenge des folgenden Datenblocks */
			/* Jetzt Daten lesen */
			if (CatFiles_ReadMuch(rs->dataLength, fhdl, rs->data) != NoError)
			{
				/* Lesefehler, abbrechen */
				SFREE(rs->data);
				return FALSE;
			}
			++resubs;
		}
	} while (error == NoError);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL WiederVorlage_ReadResubList(_WORD modus)
{
	_UBYTE fname[PATH_MAX];
	FILENO fhdl;
	_ULONG size;
	_BOOL exists;

	MouseOn();
	MouseBusy();
	freeResubArray();
	strBcpy(fname, DataPath);
	
	/* Je nach Modus unterschiedliche Dateien oeffnen (MH) */
	if (modus == WVL)
	{
		F_Path_Append(fname, resubDataName);
	} else
	{
		F_Path_Append(fname, cmtsDataName);
	}
	
	size = CatFiles_FileSize(fname, &exists);
	if (exists && size > 0)
	{
		fhdl = CatFiles_OpenFile(fname, CatFiles_readFile);
		if (fhdl >= 0)
		{
			if (!WiederVorlage_parseData(fhdl))
			{
				freeResubArray();
				CatFiles_CloseFile(fhdl);
				MouseArrow();	/* (MH) */
				return FALSE;
			} else
			{
				CatFiles_CloseFile(fhdl);
				listModified = FALSE;
				MouseArrow();	/* (MH) */
				return TRUE;
			}
		}
	} else
	{
		/* Leeres Array anlegen */
		if (!MakeNewResubArray(TRUE, 10))
			return FALSE;
		resubs = 0;
	}
	MouseArrow();	/* (MH) */
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _ULONG WiederVorlage_ResubListSize(_VOID)
{
	_ULONG size;
	_ULONG i;
	
	size = 0;
	for (i = 0; i < resubs; i++)
	{
		size += resubArray[i].dataLength + 6 + 15 + 9;
	}
	return size + 1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID WiederVorlage_GetResubList(_UBYTE *ptr)
{
	_ULONG i;
	CONST _UBYTE *src;
	resubRec *rs;
	
	for (i = 0; i < resubs; i++)
	{
		rs = &resubArray[i];
		src = rs->group;
		while ((*ptr++ = *src++) != '\0')
			;
		src = rs->msgId;
		if (src == NULL)
			src = "\001";
		while ((*ptr++ = *src++) != '\0')
			;
		src = rs->from;
		if (src == NULL)
			src = "\001";
		while ((*ptr++ = *src++) != '\0')
			;
		src = rs->topic;
		if (src == NULL)
			src = "\001";
		while ((*ptr++ = *src++) != '\0')
			;
		sprintf(ptr, "%04d%02d%02d%02d%02d%02d",
			rs->date.year,
			rs->date.month,
			rs->date.day,
			rs->time.hour,
			rs->time.minute,
			rs->time.second);
		ptr += 15;
		sprintf(ptr, "%04d%02d%02d",
			rs->resubDate.year,
			rs->resubDate.month,
			rs->resubDate.day);
		ptr += 9;
	}
	*ptr = '\0';
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID DeleteEntry(_ULONG idx)
{
	_ULONG i;

	if (idx >= resubs)
		return;
	SFREE(resubArray[idx].data);
	--resubs;
	for (i = idx; i < resubs; i++)
	{
		/* Eintraege runterkopieren */
		resubArray[i] = resubArray[i+1];
	}
	listModified = TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----------------------------------------------------------------------
 * "Listen"prozeduren
 *----------------------------------------------------------------------*/

LOCAL _BOOL sortDate(CONST _VOID *e1, CONST _VOID *e2, _VOID *private)
{
	CONST resubRec *pResub1;
	CONST resubRec *pResub2;

	UNUSED(private);
	pResub1 = e1;
	pResub2 = e2;
	return ConvertDate_PackDate(&pResub1->resubDate) < ConvertDate_PackDate(&pResub2->resubDate);
}

/* (MH) */
LOCAL _BOOL sortDateGroupAndName(CONST _VOID *e1, CONST _VOID *e2, _VOID *private)
{
	CONST resubRec *pResub1;
	CONST resubRec *pResub2;
	_WORD res;

	UNUSED(private);
	pResub1 = e1;
	pResub2 = e2;
	
	res = 0;
	
	if (ConvertDate_PackDate(&pResub1->date) == ConvertDate_PackDate(&pResub2->date))
	{
		if ((res = strSortCmp(pResub1->group, pResub2->group)) == 0)
			return strSortCmp(pResub1->from, pResub2->from) < 0;
	}
	return res < 0;
}

/*** ---------------------------------------------------------------------- ***/

/* (MH) */
GLOBAL _VOID SortResubs(_WORD modus)
{
	if (modus == WVL)
	{
		QuickSort_sortIt(0, resubs-1, resubArray, sortDate, sizeof(resubRec), QuickSort_noBreak, NULL);
	} else
	{
		QuickSort_sortIt(0, resubs-1, resubArray, sortDateGroupAndName, sizeof(resubRec), QuickSort_noBreak, NULL);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID BuildLine1(resubRec *pResub, _UBYTE *str, size_t strsize)
{
	if (pResub->from != NULL)
		CatGlobal_KurzName(pResub->from, str, 26);
	else
		CatGlobal_KurzName("Anonym", str, 26);
	strMcat(str, strsize, "  ");
	if (pResub->group != NULL)
		strMcat(str, strsize, pResub->group);
	strMcat(str, strsize, " ");
	if (pResub->resubDate.day   == 99 &&
		pResub->resubDate.month == 99 &&
		pResub->resubDate.year  == 9999)
	{
		strMcat(str, strsize, "(gehalten) ");
	}		
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID BuildLine2(resubRec *pResub, _UBYTE *str, size_t strsize)
{
	_UBYTE tmpStr[40];
	
	if (whichType == WVL)	/* (MH) */
	{
		strMcpy(str, strsize, "  Wv: ");
		T_Strlftime(tmpStr, sizeof(tmpStr), "%d.%m.%Y", T_DosToSystime(ConvertDate_PackDate(&pResub->resubDate), 0));
		strMcat(str, strsize, tmpStr);
		strMcat(str, strsize, "  Vom: ");
	} else
	{
		strMcpy(str, strsize, "  Vom: ");
	}
	T_Strlftime(tmpStr, sizeof(tmpStr), "%1D", T_DosToSystime(ConvertDate_PackDate(&pResub->date), ConvertDate_PackTime(&pResub->time)));
	strMcat(str, strsize, tmpStr);
	strMcat(str, strsize, "  ");
	if (pResub->topic != NULL)
		strMcat(str, strsize, pResub->topic);
	strMcat(str, strsize, " ");
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CountEntries(_VOID *l, _LONG *ll, _WORD *ww)
{
	UNUSED(l);
	*ww = globalLength;
	*ll = resubs;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID *NextEntry(_VOID *l)
{
	UNUSED(l);
	if (currIdx < resubs)
	{
		++currIdx;
		return &resubArray[currIdx-1u];
	}
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID *PrevEntry(_VOID *l)
{
	UNUSED(l);
	if (currIdx > 0)
	{
		--currIdx;
		return &resubArray[currIdx];
	}
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ResetResubs(_VOID *l)
{
	UNUSED(l);
	currIdx = 0;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID resubToStr(_VOID *entry, _VOID *env, _UBYTE *str, size_t strsize)
{
	UNUSED(entry);
	UNUSED(env);
	UNUSED(str);
	UNUSED(strsize);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL selEntry(_VOID *entry, _VOID *env, _WORD line)
{
	resubRec *e;
	
	UNUSED(env);
	UNUSED(line);
	e = entry;
	if (e != NULL)
		e->selected = !e->selected;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL isSelected(_VOID *entry, _VOID *env)
{
	resubRec *ent;
	
	UNUSED(env);
	ent = entry;
	return ent->selected;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL isEnabled(_VOID *entry, _VOID *env)
{
	UNUSED(entry);
	UNUSED(env);
	return TRUE;
}
GLOBAL _VOID WiederVorlage_Exit(_VOID)
{
	freeResubArray();
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _UBYTE **WV_getMsgIDs(_LONG *anz)									*/
/*	Erstellt eine Liste aller MsgIDs die in der Wiedervorlage sind				*/
/*	Dies wird von CATPUTZ bentigt												*/
/*	Parameter:	anz: fr Rckgabe: Anzahl IDs									*/
/*	Ergebnis:	Pointer auf die Liste											*/		
/*	4.4.00 von Dimitri Junker		DYJ 4.14k									*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _UBYTE **WV_getMsgIDs(_LONG *anz)
{
	_UBYTE **ret;
	_WORD dummy;
	_LONG i;
	*anz=0l;
	if(!WiederVorlage_ReadResubList(WVL))
		return NULL;
	CountEntries(NULL,anz, &dummy);
	if(!(*anz))
		return NULL;
	ret=MALLOC(*anz*sizeof(_UBYTE *), "Resub: IDs");
	if(ret)
	{
		for(i=0;i<*anz;i++)
			ret[i]=resubArray[i].msgId;
	}
	return ret;
}


