/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <xacc.h>
#include <protokol.h>
#include <termproc.h>

#include <portaes.h>
#include <mtappl.h>

#include <ro_mem.h>
#include <debug.h>

/* Um sich alle installierten Accs beim XAcc-Protokoll zu merken wird hier ein */
/* kleines Array angelegt. Das ist natuerlich statisch, aber darueber wuerde ich  */
/* mir erstmal noch keine Sorgen machen.. :-)                                  */

#define maxAcc 80       /* Wegen MTOS von 6 auf 80 erweitert. Zwar kann
                         * man unter MTOS auch mehr als 80 ACCs und Applikationen
                         * haben, aber das waere ziemlich krank */


LOCAL _UWORD accCount;
LOCAL _WORD acc[maxAcc];
LOCAL _UBYTE *xaccName;

LOCAL XaccKeyMsg keyMsg;
LOCAL XaccTextMsg textMsg;

#define TRACE_PROT 1

#if TRACE_PROT
#  define TRACE(x) SPEC_DEBUG_OUT(x)
#else
#  define TRACE(x) 
#endif

#define MY_VERSION 0x40
#define MY_PROTO   0x01


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID Xacc_Exit(_VOID)
{
	if (xaccName != NULL)
	{
		SFREE(xaccName);
		xaccName = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL Xacc_Mem_Init(CONST _UBYTE *appName, size_t len)
{
	Xacc_Exit();
	if (appName != NULL && len > 0)
	{
		xaccName = SYSMALLOC(len, MEM_NOERR | MEM_SHARABLE, "Xacc_Mem_Init");
		if (xaccName == NULL)
			return FALSE;
		MemCpy(xaccName, appName, len);
		InstallTermproc(Xacc_Exit);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Das wuerde noetig werden, wenn wir XAcc Stufe 1 koennen */
GLOBAL _VOID Xacc_SendTextReady(_VOID)
{
}

/*** ---------------------------------------------------------------------- ***/

/* Prueft, ob schon ein Prozess mit der ID registriert ist */
LOCAL _BOOL checkXaccId(_WORD id)
{
	_UWORD i;

	for (i = 0; i < accCount; i++)
		if (acc[i] == id)
			return FALSE;
	return id != gl_apid;
}

/*** ---------------------------------------------------------------------- ***/

/* XAcc-Protokoll, accInit behandeln */
LOCAL _VOID accInit(CONST _VOID *mess)
{
	CONST struct _access {
		_WORD messId;
		_WORD apId;
		_WORD over;
		_UBYTE version;
		_UBYTE protoStep;
		_UBYTE *name;
		_WORD menuId;
		_WORD accId;
	} *access;
	struct _access message;
	_UWORD i;
	
	access = mess;
	if (multiTask)
	{
		if (accCount < maxAcc && checkXaccId(access->apId))
		{
			acc[accCount]     = access->apId; /* ID des Acc, das sich gerade anmeldet */
			if (access->messId != ACC_ACC)
			{
				message.messId    = ACC_ACC;
				message.apId      = gl_apid;
				message.over      = 0;
				message.version   = MY_VERSION;
				message.protoStep = MY_PROTO;
				message.name      = xaccName;
				message.menuId    = 0;
				message.accId     = 0;
				
				appl_write(access->apId, 16, (_WORD *)&message);
				Wind_Update(END_UPDATE); /* Abschicken */
				Wind_Update(BEG_UPDATE);
			}
			++accCount;
		}
	} else
	{
		if (accCount < maxAcc && checkXaccId(access->apId))
		{
			acc[accCount]     = access->apId; /* ID des Acc, das sich gerade anmeldet */
			message.messId    = ACC_ACC;
			message.apId      = gl_apid;
			message.over      = 0;
			message.version   = access->version;
			message.protoStep = access->protoStep;
			message.name      = access->name;
			message.menuId    = access->menuId;
			message.accId     = access->accId;

			/* Alle anderen Accs benachrichtigen */
			for (i = 0; i < accCount; i++)
			{
				appl_write(acc[i], 16, (_WORD *)&message);
				Wind_Update(END_UPDATE); /* Abschicken */
				Wind_Update(BEG_UPDATE);
			}

			message.messId    = ACC_ID;
			message.apId      = gl_apid;
			message.over      = 0;
			message.version   = MY_VERSION;
			message.protoStep = MY_PROTO;
			message.name      = xaccName;
			message.menuId    = 0;
			message.accId     = 0;

			appl_write(acc[accCount], 16, (_WORD *)&message);
			++accCount;

			Wind_Update(END_UPDATE); /* Abschicken */
			Wind_Update(BEG_UPDATE);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID RemoveXaccAcc(_WORD apId)
{
	_UWORD i;

	for (i = 0; i < accCount && acc[i] != apId; i++)
		;
	if (i < accCount)
	{
		--accCount;
		for (; i < accCount; i++)
			acc[i] = acc[i + 1];
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL Xacc_Handle_Message(CONST _WORD *message)
{
	switch (message[0])
	{
	case ACC_ID: 	/* hier behandelt, WatchDog suchen */
		TRACE((EO_TRACE, "ACC_ID: %d, %d", message[1], message[6]));
		accInit(message);      /* Xacc-Protokoll behandeln */
		break;

	case ACC_ACC:
		TRACE((EO_TRACE, "ACC_ACC: %d, %d", message[1], message[6]));
		if (multiTask)	/* hier behandelt, WatchDog suchen */
		{
			accInit(message);
		}
		break;

	case ACC_EXIT:                    /* XACC Acc_Exit */
		/* remove Accessorie from acc list */
		TRACE((EO_TRACE, "ACC_EXIT: %d", message[1]));
		RemoveXaccAcc(message[1]);
		break;

		/* AccOpen und AccClose werden einfach ueberhoert! */
	case ACC_OPEN:
		TRACE((EO_TRACE, "ACC_OPEN: %d", message[1]));
		break;

	case ACC_CLOSE:
		TRACE((EO_TRACE, "ACC_CLOSE: %d", message[1]));
		break;
		
	case ACC_ACK:
		TRACE((EO_TRACE, "ACC_ACK: %d", message[1]));
		break;
	
	case ACC_KEY:                     /* Tastendruck durchreichen */
		/* Genau umgedreht zu AV_SENDKEY */
		TRACE((EO_TRACE, "ACC_KEY: $%04x, $%03x", message[3], message[4]));
		if (keyMsg != FUNK_NULL &&
			keyMsg(message, message[3], message[4]))
		{
			Xacc_SendAcknowledge(message[1], TRUE);
		} else
		{
			Xacc_SendAcknowledge(message[1], FALSE);
		}
		break;

	case ACC_TEXT:
		/* get text via XACC */
		TRACE((EO_TRACE, "ACC_TEXT: %s", *((_UBYTE **)(&message[4]))));
		if (textMsg != FUNK_NULL &&
			textMsg(message, *((_UBYTE **)(&message[4]))))    /* Pointer auf Text */
		{
			Xacc_SendAcknowledge(message[1], TRUE);
		} else
		{
			Xacc_SendAcknowledge(message[1], FALSE);
		}
		break;
	
	default:
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Xacc_Init(CONST _UBYTE *appName, size_t len)
{
	struct {
		_WORD messId;
		_WORD apId;
		_WORD over;
		_UBYTE version;
		_UBYTE protoStep;
		_UBYTE *name;
		_WORD menuId;
		_WORD accId;
	} access;

	if (!Xacc_Mem_Init(appName, len))
		return;
	Protokoll_Handle_Message(Xacc_Handle_Message);
	access.messId    = ACC_ID;
	access.apId      = gl_apid;
	access.over      = 0;
	access.version   = MY_VERSION;
	access.protoStep = MY_PROTO;
	access.name      = xaccName;
	access.menuId    = 0;
	access.accId     = 0;
	Protokoll_Broadcast(&access);
}

/*** ---------------------------------------------------------------------- ***/

/* dem Acc mitteilen, das ACC_TEXT oder ACC_KEY geklappt haben */
GLOBAL _VOID Xacc_SendAcknowledge(_WORD apId, _BOOL succ)
{
	Protokoll_SendMess(ACC_ACK, succ ? 1 : 0, 0L, 0, 0, apId);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Xacc_Register_Key(XaccKeyMsg func)
{
	keyMsg = func;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Xacc_Register_Text(XaccTextMsg func)
{
	textMsg = func;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Xacc_SendTerminate(_BOOL terminate)
{
	struct {
		_UWORD messId;
		_WORD apId;
		_WORD over;
		_WORD r[5];
	} m;
	_UWORD i;

	if (terminate)
	{
		m.messId = ACC_EXIT;
		m.apId   = gl_apid;
		m.over   = 0;
		m.r[0] = 0;
		m.r[1] = 0;
		m.r[2] = 0;
		m.r[3] = 0;
		m.r[4] = 0;
		Wind_Update(END_UPDATE);
		for (i = 0; i < accCount; i++)
			appl_write(acc[i], 16, (_WORD *)&m);
		Wind_Update(BEG_UPDATE);
	} else
	{
		accCount = 0;
	}
}
