/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <editbase.h>
#include <mtutils.h>
#include <routines.h>

#include <ro_mem.h>

#define CR 0x0d
#define LF 0x0a


#define fillPercent 95L

GLOBAL EditError eReason;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#if EDIT_DEBUG
/* Debug - Funktionen, fliegen in der endgueltigen Version raus */

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID TestStructure(EDITOR *ed)
{
	aBuffer *buf;
	_WORD bufNum;
	_LONG i, used, numLines;

	ErrorOut(EO_DEBUG, "Teste Struktur und F\201llungsgrad ...");
	buf = ed->firstBuff;
	bufNum = 1;
	while (buf != NULL)
	{
		used = 0;
		numLines = buf->toLine - buf->fromLine + 1;
		for (i = 0; i < numLines; i++)
			used += buf->lines[i];
		ErrorOut(EO_DEBUG, "Buffer Nummer: %d", bufNum);
		ErrorOut(EO_DEBUG, "usedMem: %d", buf->usedMem);
		ErrorOut(EO_DEBUG, "used by count: %ld", used);
		ErrorOut(EO_DEBUG, "MaxLines: %d", buf->maxLine);
		ErrorOut(EO_DEBUG, "fromLine: %ld", buf->fromLine);
		ErrorOut(EO_DEBUG, "toLine: %ld", buf->toLine);
		ErrorOut(EO_DEBUG, "usedLines: %ld", numLines);
		buf = buf->next;
		++bufNum;
	}
	ErrorOut(EO_DEBUG, "totalLineNr: %ld", ed->totalLineNr);
}

#endif /* EDIT_DEBUG */

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _BOOL findLineNum(EDITOR *ed, EDLINE lineNum)
{
	aBuffer *buff;

	/* Erst mal Buffer suchen und CurrentBuff entsprechend setzen */
	if (ed->firstBuff == NULL)
		return FALSE;
	buff = ed->currentBuff;
	if (buff == NULL)
		buff = ed->firstBuff;
	if (lineNum > buff->toLine)
	{
		while (buff != NULL && lineNum > buff->toLine)
			buff = buff->next;
	} else if (lineNum < buff->fromLine)
	{
		/* Nach vorne suchen (rueckwaerts) */
		while (buff != NULL && lineNum < buff->fromLine)
			buff = buff->prev;
	}
	if (buff == NULL)
	{
#if EDIT_DEBUG
		if (lineNum != ed->totalLineNr)
		{
			ErrorOut(EO_DEBUG, "Fehler bei Anforderung von Zeilenadresse");
			ErrorOut(EO_DEBUG, "Angeforderte Zeile: %ld, maxZeile: %ld", lineNum, ed->totalLineNr);
		}
#endif /* EDIT_DEBUG */
		ed->currentBuff = ed->firstBuff;
		return FALSE;
	}
	ed->currentBuff = buff;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Neuen Verwaltungsblock anlegen */
LOCAL aBuffer *createBuffer(_WORD defSize)
{
	aBuffer *newBuff;

	if (defSize == 0)
		defSize = memBlockSize;
	newBuff = NEW(aBuffer, "createBuffer: aBuffer");
	/* Wenn wir jetzt nicht genug Speicher haben, dann ist eh alles vorbei */
	if (newBuff == NULL)
		return NULL;
	/* Textspeicher allozieren */
	newBuff->buffer = MALLOC(defSize, "createBuffer: textbuffer");
	if (newBuff->buffer == NULL)
	{
		OFREE(newBuff);
		return NULL;
	}
	MemSetZero(newBuff->buffer, defSize);
	newBuff->bufSize = defSize;
	newBuff->fromLine = 0;
	newBuff->toLine = 0;
	newBuff->usedMem = 0;
	newBuff->maxLine = 0;
	newBuff->lines = NULL;
	newBuff->next = NULL;
	newBuff->prev = NULL;
	return newBuff;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID disposeBuffer(aBuffer *buff)
{
	if (buff->lines != NULL)
		SFREE(buff->lines);
	if (buff->buffer != NULL)
		SFREE(buff->buffer);
	OFREE(buff);
}

/*** ---------------------------------------------------------------------- ***/

/* Kopiert aus dem aktuellen Buffer ab Zeile line in neuen Buffer newBuff.
 * Wenn line < 0 ist, dann wird nach Prozenten gesplittet.
 */
LOCAL _BOOL copyBuffer(EDITOR *ed, EDLINE line, aBuffer **pnewBuff, EDBUFLINE *copyLines, _BOOL copyOnly)
{
	EDBUFLINE i, usedLines;
	EDBUFSIZE offSet2;
	aBuffer *buff, *newBuff;

	newBuff = *pnewBuff = createBuffer(0);
	if (newBuff == NULL)
		return FALSE;
	buff = ed->currentBuff;
	/* Nun Kopierposition feststellen */
	offSet2 = 0;
	usedLines = (EDBUFLINE)(buff->toLine - buff->fromLine);
	if (!(copyOnly || line >= 0))
	{
		/* Jetzt Bytes und Zeilen zum Splitten feststellen */
		i = 0;
		while (offSet2 < buff->usedMem / 2 && i <= usedLines)
		{
			offSet2 += buff->lines[i];
			++i;
		}
		if (i > usedLines && i > 0)
		{
			/*
			 * Dann war die letzte Zeile wohl ein bisschen lang,
			 * und wir splitten vorher
			 */
			--i;
			offSet2 -= buff->lines[i];
		}
	} else
	{
		if (line > buff->toLine)
		{
			eReason = notFound;
			return FALSE;
		}
		usedLines = (EDBUFLINE)(line - buff->fromLine);
		for (i = 0; i <= usedLines; i++)
			offSet2 += buff->lines[i];
		i = usedLines + 1;
	}
	*copyLines = i;
	/* In i steht die relative Zeilennummer im Buffer, ab der in den
	 * neuen Buffer kopiert wird.
	 */
	newBuff->maxLine = Max(200, (EDBUFLINE)(buff->toLine - buff->fromLine - i) * 2);
	newBuff->lines = MALLOC(newBuff->maxLine * sizeof(*(newBuff->lines)), "copyBuffer: lines");
	if (newBuff->lines == NULL)
	{
		disposeBuffer(newBuff);
		*pnewBuff = NULL;
		return FALSE;
	}
	/* So, jetzt koennen wir auch kopieren */
	/* Text kopieren */
	MemCpy(newBuff->buffer, &buff->buffer[offSet2], buff->usedMem - offSet2);
	newBuff->usedMem = buff->usedMem - offSet2;
	if (!copyOnly)
		buff->usedMem = offSet2;
	/* Zeilenarray kopieren */
	MemCpy(newBuff->lines, &buff->lines[i], (buff->toLine - buff->fromLine - i + 1) * sizeof(EDBUFSIZE));
	/* Zeilennummern in neuen Buffer schreiben */
	newBuff->fromLine = buff->fromLine + i;
	newBuff->toLine = buff->toLine;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Splittet den aktuellen Buffer auf und haengt einen neuen
 * Buffer hinter den aktuellen.
 * Falls line >= 0 ist, wird hinter line gesplittet. Line ist die
 * Zeilennummer in diesem Buffer!
 * FALSE: Nicht genug Speicher, eReason wird gesetzt
 */
LOCAL _BOOL splitBuffer(EDITOR *ed, EDLINE line)
{
	aBuffer *newBuff;
	EDBUFLINE i;

	if (!copyBuffer(ed, line, &newBuff, &i, FALSE))
		return FALSE;
	/* Jetzt haben wir alles alloziert, jetzt koennen wir den neuen
	 * Buffer in die Liste einhaengen
	 */
	newBuff->prev = ed->currentBuff;
	newBuff->next = ed->currentBuff->next;
	if (newBuff->next != NULL)
		newBuff->next->prev = newBuff;
	ed->currentBuff->next = newBuff;
	newBuff->fromLine = ed->currentBuff->fromLine + i;
	newBuff->toLine = ed->currentBuff->toLine;
	ed->currentBuff->toLine = ed->currentBuff->fromLine + i - 1;
	/* Jetzt noch die Buffer ueberpruefen */
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Bringt Daten aus dem zweiten Buffer in den ersten, so dass dieser zu
 * ca. fillPercent gefuellt wird
 */
LOCAL _BOOL compactTwo(aBuffer *first, aBuffer *second, _BOOL *freedSecond)
{
	EDBUFSIZE freeInFirst;
	EDBUFSIZE usedInSecond;
	EDBUFLINE secIdx, usedLines;
	EDBUFLINE i;
	EDBUFLINE newLines;
	EDBUFSIZE *newArray;

	*freedSecond = FALSE;
	if ((first->usedMem * 100L) / first->bufSize < fillPercent)
	{
		freeInFirst = (EDBUFSIZE)(first->bufSize * fillPercent / 100L) - first->usedMem;
		/* Jetzt in zweitem Buffer suchen, bis freeInFirst Bytes ueberschritten */
		usedInSecond = 0;
		secIdx = 0;
		usedLines = (EDBUFLINE)(second->toLine - second->fromLine);
		while (secIdx <= usedLines && usedInSecond + second->lines[secIdx] <= freeInFirst)
		{
			usedInSecond += second->lines[secIdx];
			++secIdx;
		}
		/* Jetzt haben wir die Anzahl Bytes, die zu verschieben sind */
		if (usedInSecond == 0 || freeInFirst == 0)
			return TRUE;
		/* Jetzt nachsehen, ob das LineArray in first gross genug ist */
		newLines = (EDBUFLINE)(first->toLine - first->fromLine) + secIdx;
		if (newLines >= first->maxLine)
		{
			/* lineArray zu klein, neu allozieren! */
			if (newLines < 16384)
			{
				newLines = Max(200, newLines * 2);
			} else if (newLines < first->bufSize - 100)
			{
				newLines += 100;
			} else
			{
				/* Zu viele Zeilen in einem Block! */
				HALT();
			}
			newArray = MALLOC(newLines * sizeof(*newArray), "compactTwo: lines");
			if (newArray == NULL)
				return FALSE;
			/* Ok, wir haben ein neues lineArray */
			usedLines = (EDBUFLINE)(first->toLine - first->fromLine + 1);
			MemCpy(newArray, first->lines, usedLines * sizeof(*newArray));
			first->maxLine = newLines;
			SFREE(first->lines);
			first->lines = newArray;
		}
		/* Jetzt lineArray aus zweitem Buffer kopieren */
		for (i = 0; i < secIdx; i++)
			first->lines[(first->toLine - first->fromLine) + i + 1] = second->lines[i];
		/* Jetzt Speicherbereich umkopieren */
		MemCpy(&first->buffer[first->usedMem], second->buffer, usedInSecond);
		/* Jetzt lineArray in zweitem Buffer umkopieren */
		usedLines = (EDBUFLINE)(second->toLine - second->fromLine);
		for (i = secIdx; i <= usedLines; i++)
			second->lines[i - secIdx] = second->lines[i];
		/* Jetzt Speicher intern umkopieren */
		MemCpy(second->buffer, &second->buffer[usedInSecond], second->usedMem - usedInSecond);
		/* Jetzt noch die usedMems und die Zeilen anpassen */
		first->usedMem += usedInSecond;
		second->usedMem -= usedInSecond;
		first->toLine += secIdx;
		second->fromLine += secIdx;
		/* Jetzt noch testen, ob evtl. der zweite Buffer leer geworden ist: */
		if (second->usedMem == 0)
		{
			/* Buffer komplett freigeben und aus Liste aushaengen */
			first->next = second->next;
			if (second->next != NULL)
				second->next->prev = second->prev;
			disposeBuffer(second);
			*freedSecond = TRUE;
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL deleteLineBlock(EDITOR *ed, EDLINE lineFrom, EDLINE lineTo)
{
	EDBUFLINE i, lineIdx, lineCount;
	EDBUFSIZE offset;
	EDBUFSIZE len;
	aBuffer *buff;

	buff = ed->currentBuff;
	if (lineFrom < buff->fromLine ||
		lineTo < buff->fromLine ||
		lineTo > buff->toLine ||
		lineFrom > buff->toLine)
	{
		return FALSE;
	}
	offset = 0;
	lineCount = (EDBUFLINE)(lineFrom - buff->fromLine);
	for (i = 0; i < lineCount; i++)
		offset += buff->lines[i];
	lineCount = (EDBUFLINE)(lineTo - lineFrom) + 1;
	lineIdx = (EDBUFLINE)(lineFrom - buff->fromLine);
	len = 0;
	for (i = lineIdx; i < lineIdx + lineCount; i++)
		len += buff->lines[i];
	/* LineArray anpassen */
	MemCpy(&buff->lines[lineIdx], &buff->lines[lineIdx + lineCount], (buff->toLine - lineTo + 1) * sizeof(EDBUFSIZE));
	/* Im Speicher verschieben */
	MemCpy(&buff->buffer[offset], &buff->buffer[offset + len], buff->usedMem - offset - len);
	/* usedMem anpassen */
	buff->usedMem -= len;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _ULONG findLineEnd(CONST _UBYTE *buffer, _ULONG start, _ULONG max)
{
	_UBYTE ch;

	buffer += start;
	while (start < max)
	{
		ch = *buffer++;
		if (ch == '\0' || ch == CR || ch == LF)
			break;
		++start;
	}
	return start;
}

/*** ---------------------------------------------------------------------- ***/

/* Zaehlt die Zeilen in einem Buffer */
LOCAL EDBUFLINE countLines(aBuffer *buff)
{
	EDBUFLINE lineCount;
	EDBUFSIZE idx2;
	_UBYTE ch;

	/* Zeilen zaehlen */
	lineCount = 0;
	idx2 = 0;
	while (idx2 < buff->usedMem)
	{
		idx2 = (EDBUFSIZE)findLineEnd(buff->buffer, idx2, buff->usedMem);
		ch = '\0';
		if (idx2 >= buff->usedMem || (ch = buff->buffer[idx2]) == LF || ch == CR || ch == '\0')
		{
			++lineCount;
			++idx2;
			if (idx2 < buff->usedMem)
			{
				if ((buff->buffer[idx2] == LF || buff->buffer[idx2] == CR) && buff->buffer[idx2] != ch)
					++idx2;
			}
		}
	}
	return lineCount;
}

/*** ---------------------------------------------------------------------- ***/

/* Fuellt das Zeilenarray zu einem Buffer */
LOCAL _VOID fillLineArray(aBuffer *buff)
{
	EDBUFLINE lineCount;
	EDBUFSIZE idx2;
	EDBUFSIZE lastIdx;
	EDBUFSIZE lineLen;
	_UBYTE ch;

	/* Jetzt nur noch das ZeilenArray fuellen */
	lineCount = 0;
	idx2 = 0;
	lastIdx = 0;
	while (idx2 < buff->usedMem)
	{
		idx2 = (EDBUFSIZE)findLineEnd(buff->buffer, idx2, buff->usedMem);
		ch = '\0';
		if (idx2 >= buff->usedMem || (ch = buff->buffer[idx2]) == LF || ch == CR || ch == '\0')
		{
			++idx2;
			if (idx2 > buff->usedMem)
				idx2 = buff->usedMem;
			lineLen = idx2 - lastIdx;
			if (idx2 < buff->usedMem)
			{
				if ((buff->buffer[idx2] == LF || buff->buffer[idx2] == CR) && buff->buffer[idx2] != ch)
				{
					++idx2;
					++lineLen;
				}
			}
			buff->lines[lineCount] = lineLen;
			++lineCount;
			/* Notbremse */
			if (lineCount >= buff->maxLine)
			{
#if EDIT_DEBUG
				ErrorOut(EO_ERROR, "fillLineArray: overflow");
#endif
				return;
			}
			lastIdx = idx2;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID FixLine(EDITOR *ed, EDLINE line)
{
	EDBUFSIZE len, l;
	_UBYTE *txt;

	EditBase_GetLineLength(ed, line, &len);
	txt = MALLOC(len + 5, "FixLine");
	if (txt == NULL)
		return;
	EditBase_GetLine(ed, line, txt, len + 4, &len);
	/* Jetzt Zeile testen */
	l = (EDBUFSIZE)strLen(txt);
	/* ??? */
	if (txt[l] != CR && txt[l] != LF)
	{
		/* CR/LF anhaengen */
		txt[l] = CR;
		txt[l+1] = LF;
		txt[l+2] = '\0';
		l += 2;
		EditBase_PutLine(ed, line, txt, l);
	}
	/* Speicher wieder freigeben */
	SFREE(txt);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID FixBuffer(EDITOR *ed)
{
	EDBUFLINE oldLines, lineCount;
	EDLINE addLines, oldTo;
	aBuffer *tBuf, *buff;
	_BOOL doHalt;		/* Halt am Ende ? */

	doHalt = FALSE;
	/* So, und jetzt die Struktur des Buffers nochmal neu aufbauen */
	buff = ed->currentBuff;
	lineCount = countLines(buff);
	/* Wir haben also jetzt lineCount Zeilen in dem Buffer.
	 * Jetzt nachsehen, ob wir das Zeilenarray neu allozieren muessen
	 */
	if (lineCount >= buff->maxLine)
	{
		oldLines = buff->maxLine;
		buff->maxLine = Max(100,(EDBUFLINE)( (lineCount * 3L) / 2L));
		SFREE(buff->lines);
		buff->lines = MALLOC(buff->maxLine * sizeof(*(buff->lines)), "FixBuffer");
		if (buff->lines == NULL)
		{
			/* Shit! Kein Speicher mehr! */
			buff->maxLine = lineCount + 1;
			buff->lines = MALLOC(buff->maxLine * sizeof(*(buff->lines)), "FixBuffer");
			if (buff->lines == NULL)
			{
				/* Wir haben ein echtes Problem: Das alte Zeilenarray ist futsch,
				 * und fuer das neue ist nicht genug Platz da!
				 * Da kann ich nur hoffen, dass das nie auftritt!
				 * Erstmal alten Speicher wieder allozieren und dann
				 * das Zeilenarray soweit fuellen, wie es geht, und dann
				 * die Notbremse treten.
				 */
				buff->lines = MALLOC(oldLines * sizeof(*(buff->lines)), "FixBuffer");
				if (buff->lines == NULL)
				{
					/* Auch fuer das alte Array kein Speicher mehr?
					 * Dann ist jetzt alles vorbei, es hat auch  keinen
					 * Zweck mehr, irgendwelche Rettungsversuche zu
					 * unternehmen.
					 */
					HALT();
				} else
				{
					doHalt = TRUE;
				}
			}
		}
	}
	oldTo = buff->toLine;
	buff->toLine = buff->fromLine + lineCount - 1;
	/* neue toLine berechnen */
	addLines = buff->toLine - oldTo;

	fillLineArray(buff);

	/* Jetzt noch Zeilennummern in folgenden Buffern
	 * korrigieren.
	 */
	tBuf = buff->next;
	while (tBuf != NULL)
	{
		tBuf->fromLine += addLines;
		tBuf->toLine += addLines;
		tBuf = tBuf->next;
	}
	ed->totalLineNr += addLines;

	/* Jetzt die Notbremse ziehen */
	if (doHalt)
	{
		HALT();
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Exportierte Prozeduren */

/* Es wird ein Text aus einem Buffer in ed an der aktuellen Stelle eingefuegt.
 * Vorgehensweise:
 * Die Einfuegeposition wird gemerkt, ebenso der Zeilenindex davon. Dann wird
 * die Einfuegeposition im Buffer berechnet und nachgesehen, wieviel Platz
 * noch im Buffer ist. Wenn noch genuegend Platz in dem Buffer ist, wird
 * der Teil hinter der Einfuegeposition um len Bytes nach hinten verschoben
 * und danach das Zeilenarray einfach komplett neu aufgebaut.
 * Sollte nicht genuegend Platz vorhanden sein, dann wird hinter der Einfuegestelle
 * der Buffer gesplittet und solange neue Buffer angelegt, bis der Text komplett hinein-
 * passt. Danach wird dann der buffer einkopiert und das Zeilenarray ab dem ersten Buffer,
 * in den eingefuegt wurde, neu aufgebaut.
 * Es wird auch noch festgestellt, wieviele Zeilen eingefuegt wurden, so dass bei Bloecken
 * die Endposition bestimmt werden kann.
 * Umbruch wird keiner vorgenommen.
 */
LOCAL _BOOL appendToBuffer(EDITOR *ed, _UBYTE *buf, _ULONG len, EDBUFSIZE offset, _ULONG startByte, _ULONG freeBytes)
{
	_UBYTE *ptr;
	_ULONG idx;
	_ULONG i;
	_BOOL first;
	_UBYTE ch;

	/* Jetzt ersten Teil aus Buffer einkopieren */
	first = TRUE;
	ptr = buf;
	idx = 0;   /* Index in Buffer */
	do
	{
		if (!first)
		{
			if (!EditBase_GetBuffer(ed, 0))
				return FALSE;
			startByte = (ed->currentBuff->bufSize * fillPercent) / 100L; /* Wir wollen den Buffer zu 95% fuellen */
			freeBytes = ed->currentBuff->bufSize;
		}
		if (idx + startByte < len)
		{
			/* Von Index idx+startByte vorwaerts nach letztem Zeilenende suchen */
			i = findLineEnd(buf, idx + startByte, idx + freeBytes);
			ch = ptr[i];
			if (i >= idx + freeBytes - 2) /* -2 fuer CR/LF! */
			{
				/* Shit, keines mehr gefunden */
				i = idx + startByte;
				ch = ptr[i];
				while (ch != CR && ch != LF && i > idx)
				{
					--i;
					ch = ptr[i];
				}
				if (i == idx)
					return FALSE;
			} else
			{
				if ((ptr[i+1] == CR || ptr[i+1] == LF) && ptr[i+1] != ch)
					++i;
			}
			++i;
		} else
		{
			i = len;
		}
		if (first)
		{
			MemCpy(&ed->currentBuff->buffer[offset], &ptr[idx], i - idx);
			ed->currentBuff->usedMem += (EDBUFSIZE)(i - idx);
		} else
		{
			MemCpy(ed->currentBuff->buffer, &ptr[idx], i - idx);
			ed->currentBuff->usedMem = (EDBUFSIZE)(i - idx);
		}
		idx = i;
		first = FALSE;
	} while (idx < len);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL EditBase_InsertBuffer(EDITOR *ed, _UBYTE **buff, _ULONG len, _BOOL freeBuffer)
{
	EDBUFLINE lineCount;
	EDLINE addLines, oldTo;
	EDBUFSIZE insPos, freeInBuff;
	aBuffer *bBuf, *cBuf, *cNext, *tBuf, *bf;
	EDBUFSIZE bUsed, startByte;
	_UBYTE *tmpBuf;
	EDBUFSIZE tLen;
	_BOOL first;
	EDLINE startCount;

	/* Jetzt Offset in Buffer bestimmen und currentBuff setzen */
	insPos = EditBase_GetOffset(ed);
	if (insPos < 0)
	{
		/* Zeile nicht gefunden, am Ende anfuegen */
		ed->currentBuff = ed->firstBuff;
		while (ed->currentBuff->next != NULL)
			ed->currentBuff = ed->currentBuff->next;
		insPos = ed->currentBuff->usedMem;
	}
	/* Jetzt nachsehen, wieviel Platz in currentBuff noch drin ist */
	freeInBuff = ed->currentBuff->bufSize - ed->currentBuff->usedMem;
	if (len < freeInBuff)
	{
		/* Gesamter Buffer passt noch in aktuellen Buffer, nur ein
		 * paar Zeilen verschieben
		 */
		bf = ed->currentBuff;
		/* Text hinter Einfuegeposition verschieben */
		tLen = bf->usedMem - insPos;
		MemMove(&bf->buffer[insPos + len], &bf->buffer[insPos], tLen);
		/* Text aus buff einfuegen */
		MemCpy(&bf->buffer[insPos], *buff, len);
		/* Jetzt Speicher direkt freigeben */

		if (freeBuffer)
		{
			SFREE(*buff);
			*buff = NULL;
		}

		bf->usedMem += (EDBUFSIZE)len;
		/* Jetzt Zeilenarray neu berechnen.
		 * Erst mal Zeilen zaehlen in Buffer
		 */

		lineCount = countLines(bf);
		/* Wir haben also jetzt lineCount Zeilen in dem Buffer.
		 * Jetzt nachsehen, ob wir das Zeilenarray neu allozieren muessen
		 */
		if (lineCount >= bf->maxLine)
		{
			bf->maxLine = Max(100, (EDBUFLINE)(LMin(32000L, (lineCount * 3L))) / 2);
			if (bf->lines != NULL)
				SFREE(bf->lines);
			bf->lines = MALLOC(bf->maxLine * sizeof(*(bf->lines)), "InsertBuffer: lines1");
			if (bf->lines == NULL)
			{
				/* Shit! Kein Speicher mehr! */
				/* Also wieder zurueckkopieren und zurueck */
				MemCpy(&bf->buffer[insPos], &bf->buffer[insPos + len], tLen);
				bf->usedMem -= (EDBUFSIZE)len;
				return FALSE;
			}
		}
		oldTo = bf->toLine;
		bf->toLine = bf->fromLine + lineCount - 1;
		/* neue toLine berechnen */
		addLines = bf->toLine - oldTo;

		fillLineArray(bf);

		/* Jetzt noch Blockende ggf. feststellen und Zeilennummern in folgenden Buffern
		 * korrigieren.
		 */
		tBuf = bf->next;
		while (tBuf != NULL)
		{
			tBuf->fromLine += addLines;
			tBuf->toLine += addLines;
			tBuf = tBuf->next;
		}
		ed->totalLineNr += addLines;

		/* Jetzt Blockende holen */
		if (ed->block)
		{
			EditBase_GetPos(ed, insPos + (EDBUFSIZE)len, &ed->blocks[0].blockEnd.line, &ed->blocks[0].blockEnd.row);
		}
		ed->currentBuff = ed->firstBuff;
		findLineNum(ed, ed->currLineNr);
	} else
	{
		/* Buffer passt nicht in currentBuff.
		 * Jetzt machen wir das folgendermassen: Wir kopieren alles hinter der Einfuegepositon
		 * aus dem aktuellen Buffer in einen temporaeren Buffer, dann kopieren wir den ersten
		 * Teil des Textes in den aktuellen Buffer. Danach haengen wir weitere Buffer an, bis der
		 * Text komplett drin ist und haengen dann noch den temporaeren Buffer an.
		 */
		bf = ed->currentBuff;
		tLen = bf->usedMem - insPos;
		if (tLen > 0)
		{
			tmpBuf = MALLOC(tLen, "InsertBuffer: tmp");
			if (tmpBuf == NULL)
				return FALSE;
			/* Jetzt ist ein temporaere Buffer da. */
			MemCpy(tmpBuf, &bf->buffer[insPos], tLen);
			bf->usedMem = insPos;
		} else
		{
			tmpBuf = NULL;
		}
		cBuf = bf;
		cNext = bf->next;
		MemSetZero(&bf->buffer[insPos], bf->bufSize - insPos);

		startByte = bf->bufSize - insPos - 100;
		if (startByte < 0)
			startByte = bf->bufSize - insPos;

		appendToBuffer(ed, *buff, len, insPos, startByte, bf->bufSize - insPos);
		/* Jetzt ist der Text aufgesplittet in mehrere Buffer.
		 * Nun haengen wir noch den temporaeren Buffer wieder dran,
		 * Vorgehensweise ist die gleiche wie gerade, nur die Startposition ist
		 * leicht veraendert, naemlich jetzt das letzte Byte im aktuellen Buffer
		 * (usedMem).
		 */
		if (freeBuffer)
		{
			SFREE(*buff);
			*buff = NULL;
		}

		/* Hier diese Position muss ich mir noch merken, um das Blockende festzustellen
		 */
		bBuf = ed->currentBuff;
		bUsed = bBuf->usedMem;

		if (tLen > 0)
		{
			startByte = bBuf->bufSize - bBuf->usedMem - 1000;
			if (startByte < 0)
				startByte = bBuf->bufSize - bBuf->usedMem - 100;
			if (startByte < 0)
				startByte = bBuf->bufSize - bBuf->usedMem;

			appendToBuffer(ed, tmpBuf, tLen, bBuf->usedMem, startByte, bBuf->bufSize - bBuf->usedMem);
		}

		/* So, jetzt setzen wir den currentBuff wieder auf den ersten, in
		 * den eingefuegt wurde, und bauen die Zeilenstruktur neu auf!
		 */

		ed->currentBuff = cBuf;
		first = TRUE;
		addLines = 0;
		startCount = 0;
		while (ed->currentBuff != cNext)
		{
			bf = ed->currentBuff;
			lineCount = countLines(bf);
			/* Wir haben also jetzt lineCount Zeilen in dem Buffer.
			 * Jetzt nachsehen, ob wir das Zeilenarray neu allozieren muessen
			 */
			if (lineCount >= bf->maxLine || bf->lines == NULL)
			{
				bf->maxLine = Max(200, (EDBUFLINE)(LMin(32000L, (lineCount * 3L))) / 2);
				if (bf->lines != NULL)
					SFREE(bf->lines);
				bf->lines = MALLOC(bf->maxLine * sizeof(*(bf->lines)), "InsertBuffer: lines2");
				if (bf->lines == NULL)
				{
					/* Shit! Kein Speicher mehr! */
					/* Also wieder zurueckkopieren und zurueck */
					MemCpy(&cBuf->buffer[insPos], tmpBuf, tLen);
					cBuf->usedMem = insPos + tLen;
					/* Jetzt noch neu angehaengte Bloecke loeschen */
					/* YYYY Hier muessen alle neuen Bloecke freigegeben werden und
					 * noch was zurueck kopiert werden.
					 */
					return FALSE;
				}
			}
			if (first)
			{
				/* neue toLine berechnen */
				oldTo = bf->toLine;
				bf->toLine = bf->fromLine + lineCount - 1;
				startCount = bf->toLine;
				addLines = bf->toLine - oldTo;
			} else
			{
				bf->fromLine = startCount + 1;
				bf->toLine = startCount + lineCount;
				addLines += lineCount;
				startCount += lineCount;
			}
			fillLineArray(bf);
			first = FALSE;
			ed->currentBuff = bf->next;
		}

		if (tmpBuf != NULL)
			SFREE(tmpBuf);

		/* So, jetzt alle weiteren Zeilennummern anpassen */
		tBuf = cNext;
		while (tBuf != NULL)
		{
			tBuf->fromLine += addLines;
			tBuf->toLine += addLines;
			tBuf = tBuf->next;
		}
		ed->totalLineNr += addLines;

		/* Jetzt noch Position vom Blockende feststellen */
		ed->currentBuff = bBuf;

		if (ed->block)
		{
			EditBase_GetPos(ed, bUsed, &ed->blocks[0].blockEnd.line, &ed->blocks[0].blockEnd.row);
		}
		/* Und jetzt noch wieder komprimieren */
		EditBase_CompactBuffer(ed);
	}
	/* Und jetzt noch aktuellen Buffer restaurieren */
	/* Das kann evtl. boese in die Hose gehen, falls der Buffer wegkomprimiert wurde */
	/*    currentBuff = cBuf;       */
#if EDIT_DEBUG
	TestStructure(ed);
#endif
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Kopieren von einem EditBuffer in einen Speicherbereich.
 * Es wird aus dem Editor in addBuf der Bereich von
 * tStartLine bis tEndLine kopiert.
 * Neue Fassung, die alles in einen Block kopiert und nicht in n kleine Bloecke.
 */
GLOBAL _BOOL EditBase_CopyToBuff(EDITOR *ed, EDLINE tStartLine, EDLINE tEndLine, _UBYTE **addBuf, _ULONG *len)
{
	aBuffer *tBuf, *buff;
	EDBUFLINE i, usedLines;
	_LONG tBytes, cBytes, bytes;
	EDBUFSIZE offset;
	_UBYTE *fromAddr, *toAddr;

	/* Startzeile suchen */
	if (tStartLine > 0)
	{
		if (!findLineNum(ed, tStartLine - 1))
			return FALSE;
	} else
	{
		ed->currentBuff = ed->firstBuff;
	}
	if (tEndLine > ed->totalLineNr - 1)
		tEndLine = ed->totalLineNr - 1;
	/* Bytes zum Allozieren zaehlen */
	buff = ed->currentBuff;
	bytes = buff->usedMem;
	offset = 0;
	usedLines = (EDBUFLINE)(tStartLine - buff->fromLine);
	for (i = 0; i < usedLines; i++)
	{
		bytes -= buff->lines[i];
		offset += buff->lines[i];
	}
	tBuf = buff;
	while (tBuf != NULL && tEndLine > tBuf->toLine)
	{
		tBuf = tBuf->next;
		if (tBuf != NULL)
			bytes += tBuf->usedMem;
	}
	usedLines = (EDBUFLINE)(tBuf->toLine - tBuf->fromLine);
	for (i = (EDBUFLINE)(tEndLine - tBuf->fromLine + 1); i <= usedLines; i++)
		bytes -= tBuf->lines[i];
	if (bytes <= 0)
	{
#if EDIT_DEBUG
		ErrorOut(EO_ERROR, "CopyToBuff: no bytes to copy");
#endif
		return FALSE;
	}
	bytes += 10;        /* ein bisschen Sicherheit */ /* hmmm */
	*addBuf = MALLOC(bytes, "CopyToBuff");
	if (*addBuf == NULL)
		return FALSE;

	/* Jetzt den ganzen Kram in den Buffer kopieren */
	tBuf = ed->currentBuff;
	tBytes = bytes - 10;
	toAddr = *addBuf;
	fromAddr = &tBuf->buffer[offset];
	if (tBytes < tBuf->usedMem - offset)
	{
		MemCpy(toAddr, fromAddr, tBytes);
	} else
	{
		MemCpy(toAddr, fromAddr, tBuf->usedMem - offset);
		tBytes -= tBuf->usedMem - offset;
		toAddr += tBuf->usedMem - offset;
		do
		{
			tBuf = tBuf->next;
			if (tBuf != NULL)
			{
				cBytes = LMin(tBytes, tBuf->usedMem);
				MemCpy(toAddr, tBuf->buffer, cBytes);
				toAddr += cBytes;
				tBytes -= cBytes;
			}
		} while (tBuf != NULL && tBytes > 0);
	}
	*len = bytes - 10;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt Buffer auf CurrentLine und berechnet Offset in
 * currentBuffer fuer currLineNr und currRow
 */
GLOBAL EDBUFSIZE EditBase_GetOffset(EDITOR *ed)
{
	EDBUFSIZE offSet;
	EDBUFLINE i, usedLines;
	aBuffer *buff;

	if (!findLineNum(ed, ed->currLineNr))
		return -1;
	/* Aktueller Buffer enthaelt lineNum */
	buff = ed->currentBuff;
	offSet = 0;
	usedLines = (EDBUFLINE)(ed->currLineNr - buff->fromLine);
	for (i = 0; i < usedLines; i++)
		offSet += buff->lines[i];
	offSet += ed->currRow;
	return offSet;
}

/*** ---------------------------------------------------------------------- ***/

/* Berechnet aus Offset in currentBuff die aktuelle
 * Zeile und Spalte
 */
GLOBAL _VOID EditBase_GetPos(EDITOR *ed, EDBUFSIZE offset, EDLINE *line, EDCOLUMN *row)
{
	EDBUFSIZE offset2;
	EDBUFLINE i, usedLines;
	aBuffer *buff;

	/* Aktueller Buffer enthaelt lineNum */
	i = 0;
	offset2 = 0;
	buff = ed->currentBuff;
	usedLines = (EDBUFLINE)(buff->toLine - buff->fromLine);
	while (i <= usedLines && offset2 + buff->lines[i] <= offset)
	{
		offset2 += buff->lines[i];
		++i;
	}
	*line = buff->fromLine + i;
	*row = offset - offset2;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert die Laenge der Zeile lineNum
 * FALSE: Zeile nicht vorhanden.
 */
GLOBAL _BOOL EditBase_GetLineLength(EDITOR *ed, EDLINE lineNum, EDCOLUMN *len)
{
	if (lineNum >= ed->totalLineNr || lineNum < 0)
	{
		*len = 0;
		eReason = notFound;
		return FALSE;
	}
	/* Erst mal Buffer suchen und CurrentBuff entsprechend setzen */
	if (!findLineNum(ed, lineNum))
		return FALSE;
	*len = ed->currentBuff->lines[(EDBUFLINE)(lineNum - ed->currentBuff->fromLine)];
	if (*len == 0)
	{
		/* Sollte nicht passieren, also Struktur korrigieren */
		FixBuffer(ed);
		return EditBase_GetLineLength(ed, lineNum, len);
	}
	if (lineNum == ed->currLineNr)
		*len = ed->editLen;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert die Adresse der Zeile und die Laenge.
 * Nur fuer Lesezugriff!
 * FALSE: Zeile nicht vorhanden
 */
GLOBAL _BOOL EditBase_GetLineAdr(EDITOR *ed, EDLINE lineNum, _UBYTE **text, EDCOLUMN *len)
{
	EDBUFLINE i, usedLines;
	EDBUFSIZE offSet;
	aBuffer *buff;

	if (lineNum >= ed->totalLineNr || lineNum < 0)
	{
		*len = 0;
		eReason = notFound;
		return FALSE;
	}
	if (lineNum == ed->currLineNr && ed->editChanged)
	{
		*text = ed->editLine.text;
		*len = ed->editLen;
		return TRUE;
	}
	/* Erst mal Buffer suchen und CurrentBuff entsprechend setzen */
	if (!findLineNum(ed, lineNum))
		return FALSE;
	/* Aktueller Buffer enthaelt lineNum */
	offSet = 0;
	buff = ed->currentBuff;
	usedLines = (EDBUFLINE)(lineNum - buff->fromLine);
	for (i = 0; i < usedLines; i++)
		offSet += buff->lines[i];
	*len = buff->lines[i];
	*text = &buff->buffer[offSet];
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL EditBase_GetLine(EDITOR *ed, EDLINE lineNum, _UBYTE *str, size_t strsize, EDCOLUMN *len)
{
	_UBYTE *text;
	_BOOL found;

	if (lineNum == ed->currLineNr && ed->editChanged)
	{
		*len = ed->editLen;
		text = ed->editLine.text;
	} else
	{
		if (!EditBase_GetLineAdr(ed, lineNum, &text, len))
		{
			*str = '\0';
			*len = 0;
			return FALSE;
		}
	}
	eReason = lineToLong;
	if (*len >= strsize)
	{
		MemCpy(str, text, strsize - 1);
		str[strsize - 1] = '\0';
		return FALSE;
	}
	MemCpy(str, text, *len);
	strsize -= *len;
	found = FALSE;
	if (*len >= 1)
	{
		if (*len >= 2)
			if (str[*len-2] == CR || str[*len-2] == LF)
				found = TRUE;
		if (str[*len-1] == CR || str[*len-1] == LF)
			found = TRUE;
	}
	if (!found)
	{
		if (--strsize != 0)
		{
			str[*len] = CR;
			++(*len);
			if (--strsize != 0)
			{
				str[*len] = LF;
				++(*len);
			}
		}
		if (strsize == 0)
		{
			str[*len - 1] = '\0';
			return FALSE;
		}
	}
	str[*len] = '\0';
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL EditBase_PutLine(EDITOR *ed, EDLINE lineNum, CONST _UBYTE *str, EDBUFSIZE len)
{
	aBuffer *buff;
	EDBUFSIZE oldLen;
	EDBUFSIZE offSet;
	EDBUFLINE i, usedLines;
	
	if (lineNum >= ed->totalLineNr || lineNum < 0)
	{
		eReason = notFound;
		return FALSE;
	}
	/* Erst mal Buffer suchen und CurrentBuff entsprechend setzen */
	if (!findLineNum(ed, lineNum))
		return FALSE;
	/* Aktueller Buffer enthaelt lineNum */
	offSet = 0;
	buff = ed->currentBuff;
	usedLines = (EDBUFLINE)(lineNum - buff->fromLine);
	for (i = 0; i < usedLines; i++)
		offSet += buff->lines[i];
	oldLen = buff->lines[i];
	if (len <= oldLen)
	{
		/* Easy, Zeile nur kopieren, und evtl. was nachschieben von hinten */
		MemCpy(&buff->buffer[offSet], str, len);
		if (len < oldLen)
		{
			MemCpy(&buff->buffer[offSet+len], &buff->buffer[offSet+oldLen], buff->usedMem - offSet - oldLen);
			/* Benutzten Speicher updaten */
			buff->usedMem -= oldLen - len;
		}
		buff->lines[usedLines] = len;
		return TRUE;
	} else
	{
		/* Jetzt wird es echt schwierig: Ich muss erst nachsehen, ob
		 * noch genug Platz im aktuellen Buffer ist. Wenn das nicht
		 * der Fall ist, dann muss ich einen neuen Buffer allozieren
		 * und ca. 50% des Blocks auslagern. Wenn das auch nicht mehr
		 * geht (kein Speicher mehr fuer Buffer), dann muss ich eine
		 * Garbage Collection machen (Alle Bloecke zusammenschieben)
		 * und dann das nochmal versuchen.
		 */
		eReason = notEnoughMem;
		if (buff->usedMem - oldLen < buff->bufSize - len)
		{
			/* Ok, passt noch rein */
			/* Bytes nach hinten schieben */
			MemMove(&buff->buffer[offSet+len], &buff->buffer[offSet+oldLen], buff->usedMem - offSet - oldLen);
			/* Zeile einfuegen */
			MemCpy(&buff->buffer[offSet], str, len);
			/* Benutzen Speicher im Block hochzaehlen */
			buff->usedMem += len - oldLen;
			buff->lines[usedLines] = len;
			return TRUE;
		} else
		{
			/* Nicht genug Platz, also neuen Buffer anhaengen */
			if (!splitBuffer(ed, -1))
			{
				return FALSE;
			}
			/* Und jetzt muessen wir nur noch die eigentliche Zeile einfuegen.
			 * Da ich faul bin, mache ich das einfach durch eine Rekursion
			 */
			return EditBase_PutLine(ed, lineNum, str, len);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Trennt die Zeile lineNum an der Stelle row auf.
 * FALSE: Kein Speicherplatz mehr frei.
 */
GLOBAL _BOOL EditBase_InsertCr(EDITOR *ed, EDLINE lineNum, EDCOLUMN row)
{
	EDBUFLINE lineIdx;
	EDBUFSIZE *newArray;
	aBuffer *buff;
	_BOOL doFixBuf;

	if (lineNum >= ed->totalLineNr || lineNum < 0)
	{
		eReason = notFound;
		return FALSE;
	}
	/* Wir suchen einfach nach der Zeile und fuegen
	 * noch einen Eintrag im LineArray an.
	 */
	eReason = notEnoughMem;
	buff = ed->currentBuff;
	lineIdx = (EDBUFLINE)(lineNum - buff->fromLine);
	if ((EDBUFLINE)(buff->toLine - buff->fromLine) + 1 >= buff->maxLine)
	{
		/* LineArray neu allozieren */
		newArray = MALLOC((buff->maxLine + 20) * sizeof(*(buff->lines)), "InsertCr");
		if (newArray == NULL)
			return FALSE;
		MemCpy(newArray, buff->lines, buff->maxLine * sizeof(*(buff->lines)));
		SFREE(buff->lines);
		buff->lines = newArray;
		buff->maxLine += 20;
	}
	/* Platz im LineArray ist jetzt garantiert da! */
	/* Zeilenindex einfuegen */
	doFixBuf = FALSE;
	/* Pruefen, ob eine Zeile 0 Byte lang werden wuerde */
	if (buff->lines[lineIdx] == row || row == 0)
	{
#if EDBIT_DEBUG
		TestStructure(ed);
#endif
		FixLine(ed, lineNum);
		return EditBase_InsertCr(ed, lineNum, row);
	}
	MemMove(&buff->lines[lineIdx+1], &buff->lines[lineIdx], (buff->toLine - buff->fromLine - lineIdx + 1) * sizeof(*(buff->lines)));
	buff->lines[lineIdx+1] = buff->lines[lineIdx] - row;
	if (buff->lines[lineIdx+1] == 0)
		doFixBuf = TRUE;
	buff->lines[lineIdx] = row;
	if (buff->lines[lineIdx] == 0)
		doFixBuf = TRUE;
	++buff->toLine;
	++ed->totalLineNr;
	buff = buff->next;
	while (buff != NULL)
	{
		++buff->fromLine;
		++buff->toLine;
		buff = buff->next;
	}
	if (doFixBuf)
		FixBuffer(ed);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Fuegt die Zeile lineNum im Buffer ein.
 * Wenn pre == TRUE ist, dann wird die Zeile vor die alte Zeile lineNum gesetzt,
 * ansonsten hinter die Zeile lineNum.
 * Ergebnis: FALSE: Nicht genug Speicher mehr frei
 */
GLOBAL _BOOL EditBase_InsertLine(EDITOR *ed, EDLINE lineNum, CONST _UBYTE *str, _BOOL pre, EDBUFSIZE len)
{
	aBuffer *buff;
	EDBUFSIZE *newArray;
	EDBUFSIZE offset;
	EDBUFLINE lineIdx, i;

	if (lineNum > ed->totalLineNr || lineNum < 0)
	{
		eReason = notFound;
		return FALSE;
	}
	/* Wir fuegen immer hinter der Zeile ein. Bei pre wird
	 * einfach lineNum dekrementiert
	 */
	eReason = notEnoughMem;
	if (pre)
		--lineNum;
	if (lineNum == -1)
	{
		buff = ed->currentBuff = ed->firstBuff;
		offset = 0;
		lineIdx = 0;
	} else
	{
		if (!findLineNum (ed, lineNum))
			return FALSE;
		buff = ed->currentBuff;
		offset = 0;
		lineIdx = (EDBUFLINE)(lineNum - buff->fromLine + 1);
		for (i = 0; i < lineIdx; i++)
			offset += buff->lines[i];
	}
	/* Jetzt sind offset und Buffer bestimmt */
	if (buff->usedMem < buff->bufSize - len)
	{
		/* Ok, passt noch rein */
		if ((EDBUFLINE)(buff->toLine - buff->fromLine + 4) >= buff->maxLine - 1)
		{
			/* LineArray neu allozieren */
			newArray = MALLOC((buff->maxLine + 20) * sizeof(*(buff->lines)), "InsertLine");
			if (newArray == NULL)
				return FALSE;
			MemCpy(newArray, buff->lines, buff->maxLine * sizeof(*(buff->lines)));
			SFREE(buff->lines);
			buff->lines = newArray;
			buff->maxLine += 20;
		}
		/* Platz im LineArray ist jetzt garantiert da! */
		/* Zeilenindex einfuegen */
		/*
		MemCpy(&buff->lines[lineIdx+1], &buff->lines[lineIdx], (toLine - fromLine + 1 - lineIdx + 1) * sizeof(*(buff->lines)));
		*/
		for (i = (EDBUFLINE)(buff->toLine - buff->fromLine + 1); i >= lineIdx; i--)
			buff->lines[i+1] = buff->lines[i];
		buff->lines[lineIdx] = len;
		MemCpy(&buff->buffer[offset+len], &buff->buffer[offset], buff->usedMem - offset);
		MemCpy(&buff->buffer[offset], str, len);
		buff->usedMem += len;
		/* Jetzt Zeilenverweise updaten in allen folgenden Buffern */
		++buff->toLine;
		++ed->totalLineNr;
		buff = buff->next;
		while (buff != NULL)
		{
			++buff->fromLine;
			++buff->toLine;
			buff = buff->next;
		}
	} else
	{
		/* Nicht genug Platz im aktuellen Buffer! Also das alte Spiel:
		 * Splitten und dann nochmal versuchen
		 */
		if (!splitBuffer(ed, -1))
			return FALSE;
		return EditBase_InsertLine(ed, lineNum, str, FALSE, len);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Loescht einen Bereich von Zeilen ueber mehrere Bloecke hinweg
 * Dabei wird mittels deleteLineBlock in den aeusseren Bloecken
 * geloescht und die inneren Bloecke entfernt.
 */
GLOBAL _BOOL EditBase_DeleteSomeLines(EDITOR *ed, EDLINE from, EDLINE to)
{
	aBuffer *tBuf, *buff;
	EDBUFLINE lineCount;

	/* Buffer suchen zu Zeile */
	if (!findLineNum(ed, from))
		return FALSE;
	to = LMin(to, ed->totalLineNr - 1);
	buff = ed->currentBuff;
	if (buff->toLine >= to)
	{
		if (!deleteLineBlock(ed, from, to))
			return FALSE;
	} else
	{
		if (from == buff->fromLine)
		{
			/* currentBuff komplett loeschen */
			tBuf = buff;
			if (tBuf->prev != NULL)
				tBuf->prev->next = tBuf->next;
			if (tBuf->next != NULL)
				tBuf->next->prev = tBuf->prev;
			if (ed->firstBuff == tBuf)
				ed->firstBuff = tBuf->next;
			ed->currentBuff = tBuf->next;
			disposeBuffer(tBuf);
		} else
		{
			if (!deleteLineBlock(ed, from, buff->toLine))
				return FALSE;
			ed->currentBuff->toLine = from - 1;
			ed->currentBuff = ed->currentBuff->next;
		}
		while (ed->currentBuff != NULL && to > ed->currentBuff->toLine)
		{
			/* Buffer komplett freigeben */
			tBuf = ed->currentBuff;
			if (tBuf->prev != NULL)
				tBuf->prev->next = tBuf->next;
			if (tBuf->next != NULL)
				tBuf->next->prev = tBuf->prev;
			if (ed->firstBuff == tBuf)
				ed->firstBuff = tBuf->next;
			ed->currentBuff = tBuf->next;
			disposeBuffer(tBuf);
		}
		/* Jetzt noch restliche Zeilen loeschen */
		if (!deleteLineBlock(ed, ed->currentBuff->fromLine, to))
			return FALSE;
		ed->currentBuff->fromLine = from;
	}
	buff = ed->currentBuff;
	lineCount = (EDBUFLINE)(to - from) + 1;
	/* Zeilenverweise anpassen */
	ed->totalLineNr -= lineCount;
	buff->toLine -= lineCount;
	tBuf = buff->next;
	while (tBuf != NULL)
	{
		tBuf->fromLine -= lineCount;
		tBuf->toLine -= lineCount;
		tBuf = tBuf->next;
	}
#if EDIT_DEBUG
	TestStructure(ed);
#endif
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Loescht die Zeile lineNum im Buffer
 * FALSE: Zeile nicht vorhanden
 */
GLOBAL _BOOL EditBase_DeleteLine(EDITOR *ed, EDLINE lineNum)
{
	EDBUFLINE lineCount;
	aBuffer *buff;
	_BOOL freedSecond;
	
	if (lineNum > ed->totalLineNr || lineNum < 0)
	{
		eReason = notFound;
		return FALSE;
	}
	if (!findLineNum(ed, lineNum))
		return FALSE;
	/* Zeile loeschen */
	if (!deleteLineBlock(ed, lineNum, lineNum))
		return FALSE;
	buff = ed->currentBuff;
	lineCount = 1;
	/* Zeilenverweise anpassen */
	ed->totalLineNr -= lineCount;
	if (ed->totalLineNr == 0)
	{
		ed->totalLineNr = 1;
		lineCount = (EDBUFLINE)buff->toLine;
	}
	buff->toLine -= lineCount;
	buff = buff->next;
	while (buff != NULL)
	{
		buff->fromLine -= lineCount;
		buff->toLine -= lineCount;
		buff = buff->next;
	}
	buff = ed->currentBuff;
	if (buff->usedMem < buff->bufSize / 4)
	{
		/* weniger als 25 % belegt,
		 * Buffer mit folgendem oder vorherigem zusammenlegen
		 */
		if (buff->next != NULL)
		{
			compactTwo(buff, buff->next, &freedSecond);
		} else if (buff->prev != NULL)
		{
			compactTwo(buff->prev, buff, &freedSecond);
			if (freedSecond)
			{
				if (buff == ed->currentBuff)
					ed->currentBuff = ed->firstBuff;
				if (buff == ed->firstBuff)
				{
					ed->firstBuff = NULL;
					ed->currentBuff = NULL;
				}
			}
		}
	}
	if (ed->currLineNr == ed->editLineNr)
		ed->editChanged = FALSE;
	if (ed->currLineNr >= ed->totalLineNr)
		ed->currLineNr = ed->totalLineNr - 1;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Kompaktiert den Bufferspeicher. FALSE: Nicht mehr moeglich, da
 * gar kein Speicher mehr frei war.
 */
GLOBAL _BOOL EditBase_CompactBuffer(EDITOR *ed)
{
	aBuffer *buffOne;
	_BOOL freedSecond, success;

	buffOne = ed->firstBuff;
	success = TRUE;
	while (buffOne != NULL && buffOne->next != NULL)
	{
		success &= compactTwo(buffOne, buffOne->next, &freedSecond);
		if (!freedSecond)
			buffOne = buffOne->next;
	}
	ed->currentBuff = ed->firstBuff;
	return success;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert die aktuelle Zeile, in der der Cursor steht.
 */
GLOBAL _VOID EditBase_GetCurrLine(EDITOR *ed, _UBYTE *str, size_t strsize, EDBUFSIZE *len)
{
	EditBase_GetLine(ed, ed->currLineNr, str, strsize, len);
}

/*** ---------------------------------------------------------------------- ***/

/* Schiebt die aktuelle Zeile, in der der Cursor steht, wieder in den Buffer
 */
GLOBAL _BOOL EditBase_PutCurrLine(EDITOR *ed)
{
	_BOOL res;

	if (ed->editChanged)
	{
#if EDIT_DEBUG
		if (ed->editLen != strLen(ed->editLine.text))
			ErrorOut(EO_ERROR, "PutCurrLine: Line length mismatch");
#endif
		if (ed->editLen == 0)
		{
			ed->editLine.text[0] = CR;
			ed->editLine.text[1] = LF;
			ed->editLine.text[2] = '\0';
			ed->editLen = 2;
		}
		res = EditBase_PutLine(ed, ed->currLineNr, ed->editLine.text, ed->editLen);
		if (!res)
		{
			if (!EditBase_CompactBuffer(ed))
				return FALSE;
			if (!EditBase_PutLine(ed, ed->currLineNr, ed->editLine.text, ed->editLen))
				return FALSE;
		}
		ed->editChanged = FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Erzeugt einen neuen Buffer hinter currentBuff
 * Es wird nur der Textbuffer und der Descriptor alloziert,
 * das LineArray noch nicht.
 */
GLOBAL _BOOL EditBase_GetBuffer(EDITOR *ed, EDBUFSIZE defSize)
{
	aBuffer *newBuff;
	
	newBuff = createBuffer(defSize);
	if (newBuff == NULL)
		return FALSE;
	/* Jetzt kommt das Einhaengen in die Liste */
	if (ed->firstBuff == NULL)
	{
		newBuff->prev = NULL;
		newBuff->next = NULL;
		ed->firstBuff = newBuff;
		ed->currentBuff = ed->firstBuff;
	} else
	{
		if (ed->currentBuff == NULL)
			ed->currentBuff = ed->firstBuff;
		newBuff->prev = ed->currentBuff;
		newBuff->next = ed->currentBuff->next;
		if (newBuff->next != NULL)
			newBuff->next->prev = newBuff;
		ed->currentBuff->next = newBuff;
		ed->currentBuff = newBuff;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditBase_FreeBuffer(EDITOR *ed)
{
	aBuffer *buff;
	
	buff = ed->currentBuff;
	if (buff == NULL)
		return;
	if (buff->prev != NULL)
		buff->prev->next = buff->next;
	if (buff->next != NULL)
		buff->next->prev = buff->prev;
	if (ed->firstBuff == buff)
		ed->firstBuff = buff->next;
	ed->currentBuff = buff->next;
	disposeBuffer(buff);
}

/*** ---------------------------------------------------------------------- ***/

/* Zaehlt die Bytes, die von dem Text zwischen den beiden Textmarken belegt werden
 */
GLOBAL _VOID EditBase_CountBytes(EDITOR *ed, EDLINE lineFrom, EDCOLUMN rowFrom, EDLINE lineTo, EDCOLUMN rowTo, _ULONG *bytes)
{
	aBuffer *buff;
	EDBUFSIZE offset;
	EDBUFLINE i, linesUsed;

	if (!findLineNum(ed, lineTo) || !findLineNum(ed, lineFrom))
	{
		*bytes = 0;
		return;
	}
	buff = ed->currentBuff;
	offset = 0;
	linesUsed = (EDBUFLINE)(lineFrom - buff->fromLine);
	for (i = 0; i < linesUsed; i++)
		offset += buff->lines[i];
	offset += rowFrom;
	*bytes = buff->usedMem - offset;
	/* Jetzt Endebuffer suchen */
	while (buff != NULL && lineTo > buff->toLine)
	{
		*bytes += buff->usedMem;
		buff = buff->next;
	}
	if (buff != NULL)
	{
		offset = 0;
		linesUsed = (EDBUFLINE)(lineTo - buff->fromLine);
		for (i = 0; i < linesUsed; i++)
			offset += buff->lines[i];
		if (rowTo > buff->lines[i])
			rowTo = buff->lines[i];
		offset += rowTo;
		if (buff != ed->currentBuff)
			*bytes += offset;
		else
			*bytes -= buff->usedMem - offset;
	} else
	{
		*bytes = 0;
	}
	ed->currentBuff = buff;
}
