/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <editdraw.h>
#include <editglob.h>
#include <edittool.h>
#include <editbase.h>
#include <wdwmanag.h>
#include <confvars.h>

#include <mtalerts.h>
#include <mtappl.h>
#include <mtdials.h>
#include <mtutils.h>
#include <portvdi.h>

#include <keyboard.h>
#include <rectfunc.h>

#include <routinec.h>
#include <routine.h>

#define TAB 0x09
#define CR  0x0d
#define LF  0x0a



/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _VOID EditDraw_GetBlockRects(EDITOR *ed, GRECT blockRects[], _WORD *count, EDLINE oldLine, EDCOLUMN oldRow, EDLINE newLine, EDCOLUMN newRow)
{
	_WORD i;
	EDLINE viewLine;

	/* OldLine muss kleiner sein, da es als Anfang genommen wird. */
	/* Erstmal Zeilen auf gueltige Werte bringen */
	if (oldLine < 0)
		oldLine = 0;
	if (newLine < 0)
		newLine = 0;
	if (newLine > ed->totalLineNr)		/* - 1 */
		newLine = ed->totalLineNr;		/* - 1 */
	if (oldLine > ed->totalLineNr)		/* - 1 */
		oldLine = ed->totalLineNr;		/* - 1 */
	/* Notfalls swappen! */
	if (oldLine > newLine)
	{
		EDLINE tmp;
		EDCOLUMN row;

		tmp = newLine;
		newLine = oldLine;
		oldLine = tmp;
		row = newRow;
		newRow = oldRow;
		oldRow = row;
	} else if (oldLine == newLine && oldRow > newRow)
	{
		EDCOLUMN row;

		row = newRow;
		newRow = oldRow;
		oldRow = row;
	}
	i = 0;
	/* Jetzt gewuenschten Bereich malen */
	if (oldLine >= 0 && newLine >= 0 && newLine >= oldLine)
	{
		/* Block ist vorhanden */
		if (!((oldLine < ed->StartLine && newLine < ed->StartLine) || (oldLine > ed->StartLine + ed->windLines)))
		{
			/* Block ist zumindest teilweise im Fenster.
			 * Es werden maximal drei Rechtecke berechnet, die den Block im Fenster komplett beschreiben.
			 */
			if (oldLine >= ed->StartLine)
			{
				EditTools_getCharPos(ed, oldLine, oldRow, &blockRects[i].g_x, &blockRects[i].g_y);
				if (blockRects[i].g_x <= ed->editWork.g_x + pixOff)
				{
					blockRects[i].g_x = ed->editWork.g_x + pixOff;
				}
				if (newLine == oldLine)
				{
					EditTools_getCharPos(ed, newLine, newRow, &blockRects[i].g_w, &blockRects[i].g_h);
					--blockRects[i].g_w;
					blockRects[i].g_h += ed->charHeight - 1;
				} else
				{
					blockRects[i].g_w = ed->editWork.g_x + ed->editWork.g_w - 1;
					blockRects[i].g_h = blockRects[i].g_y + ed->charHeight - 1;
				}
				++i;
			}
			/* Komplette Zeilen zwischen Start und Ende */
			if (newLine > oldLine + 1)
			{
				/* mindestens eine Zeile im Fenster */
				viewLine = LMax(oldLine + 1, ed->StartLine);
				EditTools_getCharPos(ed, viewLine, 0, &blockRects[i].g_x, &blockRects[i].g_y);
				viewLine = LMin(newLine - 1, ed->StartLine + ed->windLines);
				EditTools_getCharPos(ed, viewLine, 0, &blockRects[i].g_w, &blockRects[i].g_h);

				blockRects[i].g_h += ed->charHeight - 1;
				blockRects[i].g_x = ed->editWork.g_x + pixOff;
				blockRects[i].g_w = ed->editWork.g_x + ed->editWork.g_w - 1;
				++i;
			}
			if (newLine >= ed->StartLine && newLine <= ed->StartLine + ed->windLines && newLine != oldLine)
			{
				/* Blockende im Fenster und nicht in Startzeile */
				blockRects[i].g_x = ed->editWork.g_x + pixOff;
				EditTools_getCharPos(ed, newLine, newRow, &blockRects[i].g_w, &blockRects[i].g_y);
				--blockRects[i].g_w;
				blockRects[i].g_h = blockRects[i].g_y + ed->charHeight - 1;
				++i;
			}
		}
	}
	*count = i;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditDraw_MarkArea(EDITOR *ed, EDLINE oldLine, EDCOLUMN oldRow, EDLINE newLine, EDCOLUMN newRow, _BOOL clipIt)
{
	GRECT work;
	GRECT c;
	_WORD i, j;
	GRECT blockRects[3];

	EditGlobals_HideMouse();
	if (clipIt)
	{
		EditGlobals_HideCursor(ed);
		/* selber clippen */
		EditGlobals_ClipWork(ed);
	}
	EditDraw_GetBlockRects(ed, blockRects, &i, oldLine, oldRow, newLine, newRow);

	if (i > 0)
	{
		vswr_mode(ed->hdl, MD_XOR);
		vsf_color(ed->hdl, BLACK);

		/* Jetzt Rechteckliste abfragen und auf Rechtecke clippen */
		if (clipIt)
		{
			if (WdwManager_RectList(ed->wdw, 0, &work))
			{
				do
				{
					/* kleinsten Redrawbereich bestimmen */
					c = rc_clip(&work, &deskSize);
					c = rc_clip(&c, &ed->editWork);
					if (c.g_w > 0)
					{
						c.g_w += c.g_x - 1;
						c.g_h += c.g_y - 1;
						vs_clip(ed->hdl, TRUE, (_WORD *)&c);
						for (j = 0; j < i; j++)
						{
							if (blockRects[j].g_w > blockRects[j].g_x && blockRects[j].g_h > blockRects[j].g_y)
								vr_recfl(ed->hdl, (_WORD *)&blockRects[j]);
						}
						EditGlobals_ClipWork(ed);
					}
				} while (WdwManager_RectList(ed->wdw, 1, &work));
			}
		} else
		{
			for (j = 0; j < i; j++)
			{
				if (blockRects[j].g_w > blockRects[j].g_x && blockRects[j].g_h > blockRects[j].g_y)
					vr_recfl(ed->hdl, (_WORD *)&blockRects[j]);
			}
		}
		vswr_mode(ed->hdl, MD_REPLACE);
		vsf_color(ed->hdl, ed->backCol);
	}
	if (clipIt)
		EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/* Vorhandener Block wird angezeigt */
GLOBAL _VOID EditDraw_ShowBlockMark(EDITOR *ed, _BOOL clipIt)
{
	_WORD i;

	for (i = 0; i < maxBlocks; i++)
	{
		if (ed->blocks[i].blockStart.line >= 0 &&
			ed->blocks[i].blockEnd.line >= 0 /* &&
			ed->blocks[i].blockEnd.line >= ed->blocks[i].blockStart.line */ )
		{
			EditDraw_MarkArea(ed,
				ed->blocks[i].blockStart.line, ed->blocks[i].blockStart.row,
				ed->blocks[i].blockEnd.line, ed->blocks[i].blockEnd.row, clipIt);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID MakeDrawLine(EDITOR *ed, EDLINE lineNr, _UBYTE *drawLine, size_t linesize, EDCOLUMN *l)
{
	_UBYTE tmpLine[4096];
	_BOOL gotLine;
	size_t len;
	
	gotLine = EditBase_GetLine(ed, lineNr, drawLine, linesize, l);
	if (gotLine || *l >= linesize)
	{
		if (*l > 0 && drawLine[*l - 1] == LF)
		{
			--(*l);
			drawLine[*l] = '\0';
		}
		if (!ed->umbruch || !ed->EditShowLineEnds)
		{
			if (*l > 0 && drawLine[*l - 1] == CR)
			{
				--(*l);
				drawLine[*l] = '\0';
			}
		}
		if (*l > 0 && drawLine[*l - 1] == LF)
		{
			--(*l);
			drawLine[*l] = '\0';
		}
		if (ed->realTabs)
		{
			strBcpy(tmpLine, drawLine);
			len = Str_ConvertTabs(tmpLine, drawLine, linesize, ed->TabSize);
			*l = (EDCOLUMN)len;
		}
	} else
	{
		drawLine[0] = '\0';
		*l = 0;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Geht davon aus, dass RectFuncs.ClipRect gesetzt ist und nur Zeilen zu zeichnen sind!
 */
LOCAL _VOID rawRedraw(EDITOR *ed, CONST GRECT *clip)
{
	_UBYTE *text;
	_WORD i;
	_WORD width;
	_WORD xOutput;
	_WORD yOutput;
	EDCOLUMN l;
	EDLINE topLine;

	vst_color(ed->hdl, ed->textCol);
	vswr_mode(ed->hdl, MD_TRANS);

	topLine = ed->StartLine;
	i = 0;							/* Zeilenzaehler im Fenster */
	/* Nicht im Redrawbereich liegende Zeilen ueberspringen */
	while (topLine < ed->totalLineNr && ((ed->editWork.g_y + (i + 1) * ed->charHeight) - 1 < clip->g_y))
	{
		++topLine;
		++i;
	}
	xOutput = ed->editWork.g_x + pixOff - ed->leftOffset;
	yOutput = ed->editWork.g_y + i * ed->charHeight;
	/* Jetzt zeichnen */
	while (topLine < ed->totalLineNr &&
		i < ed->editWork.g_h / ed->charHeight &&
		yOutput <= clip->g_h)
	{
		MakeDrawLine(ed, topLine, theDrawLine, sizeof(theDrawLine), &l);
		text = theDrawLine;
		width = EditGlobals_OutputLine(ed, xOutput, yOutput, text, l, 30000);
		if (width < 30000)
		{
			ed->maxWidth = Max(ed->maxWidth, width + ed->charWidth);
		}
		++i;
		++topLine;
		yOutput += ed->charHeight;
	}
	vswr_mode(ed->hdl, MD_REPLACE);

	EditDraw_ShowBlockMark(ed, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/* Alle redraw-Funktionen */
GLOBAL _VOID EditDraw_redrawWdw(HWND wdw, HVDI vdiH, _VOID *special, CONST GRECT *frame)
{
	GRECT clip;
	EDITOR *ed;
	_WORD oldMax;
	GRECT currSize;
	
	UNUSED(wdw);
	UNUSED(vdiH);
	ed = special;
	/* Clipping auf Rectangle setzen */
	if (ed->hasUserProc)
	{
		clip = *frame;
		if (rc_intersect(&ed->userRect, &clip))
		{
			WdwManager_GetWdwWork(ed->wdw, &currSize);
			clip = currSize;
			ed->drawUserArea(ed->wdw, ed->hdl, &clip, frame, ed->textCol, ed->backCol);
		}
	}
	clip = *frame;
	if (rc_intersect(&ed->editWork, &clip))
	{
		oldMax = ed->maxWidth;
		clip = rc_clip(frame, &ed->editWork);
		clip.g_w = clip.g_x + clip.g_w - 1;
		clip.g_h = clip.g_y + clip.g_h - 1;

		vsf_color(ed->hdl, ed->backCol);
		vr_recfl(ed->hdl, (_WORD *)&clip);

		if (clip.g_x == ed->editWork.g_x)
		{
			clip.g_x += pixOff;
		}
		clip.g_x = Min(clip.g_x, clip.g_w);

		WdwManager_SetClip(ed->hdl, &clip, FALSE);

		rawRedraw(ed, &clip);

		if (oldMax != ed->maxWidth)
			EditGlobals_SetDocument(ed);
	}
	EditGlobals_ClipWork(ed);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditDraw_redrawLines(EDITOR *ed, _WORD fromLine)
{
	GRECT frame;

	EditGlobals_HideCursor(ed);
	frame = ed->editWork;
	frame.g_y = ed->editWork.g_y + fromLine * ed->charHeight;
	WdwManager_RedrawWdw(ed->wdw, &frame);
	EditGlobals_ShowCursor(ed);
}

/*** ---------------------------------------------------------------------- ***/

/* Hier werden richtige Zeilennummern uebergeben, nicht die Bildschirmzeilen */
GLOBAL _VOID EditDraw_redrawLineArea(EDITOR *ed, EDLINE fromLine, EDLINE toLine)
{
	GRECT frame;

	if ((fromLine >= ed->StartLine && fromLine < ed->StartLine + ed->windLines) ||
		(toLine >= ed->StartLine && toLine < ed->StartLine + ed->windLines))
	{
		/* Nur Bereich neu zeichnen */
		EditGlobals_HideCursor(ed);
		frame = ed->editWork;
		if (fromLine > ed->StartLine)
		{
			frame.g_y = ed->editWork.g_y + (_WORD)(fromLine - ed->StartLine) * ed->charHeight;
		}
		if (toLine < ed->StartLine + ed->windLines)
		{
			frame.g_h = ed->editWork.g_y + (_WORD)(toLine - ed->StartLine + 1) * ed->charHeight;
			frame.g_h = frame.g_h - frame.g_y + 1;
		}
		WdwManager_RedrawWdw(ed->wdw, &frame);
		EditGlobals_ShowCursor(ed);
	} else if (fromLine < ed->StartLine && toLine > ed->StartLine)
	{
		/* komplett neu zeichnen */
		EditGlobals_HideCursor(ed);
		WdwManager_RedrawWdw(ed->wdw, &ed->editWork);
		EditGlobals_ShowCursor(ed);
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Scroll-Funktionen */

GLOBAL _VOID EditDraw_scrollRegion(EDITOR *ed, _WORD from, _WORD to, _WORD amount)
{
	GRECT copy;
	_WORD dir;

	if (from == to)
		return;
	copy.g_x = Min(MaxWidth - 1, ed->editWork.g_x);
	copy.g_y = Min(MaxHeight - 1, ed->editWork.g_y + from * ed->charHeight);
	copy.g_w = Min(MaxWidth - 1, ed->editWork.g_w);
	if (from < to)
	{
		copy.g_h = Min(MaxHeight - 1, (to - from + 1) * ed->charHeight);
	} else
	{
		copy.g_h = Min(MaxHeight - 1, (to - from + 1) * ed->charHeight);
	}
	if (amount > 0)
	{
		dir = UP;
	} else
	{
		dir = DOWN;
		amount = -1 * amount;
	}
	amount = amount * ed->charHeight;
	WdwManager_ScrollInWdw(ed->wdw, &copy, dir, amount);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL IsInBlock(EDITOR *ed, EDLINE lnr)
{
	_WORD i;

	for (i = 0; i < maxBlocks; i++)
	{
		if (ed->blocks[i].blockStart.line <= lnr && ed->blocks[i].blockEnd.line >= lnr)
			return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditDraw_drawLine(EDITOR *ed, _WORD screenLine)
{
	GRECT fill;
	_WORD width;
	_UBYTE *text;
	EDCOLUMN len;
	GRECT clipr;

	if (screenLine < 0)
		return;
	/* Fillrectangle berechnen */
	fill.g_x = Min(MaxWidth - 1, ed->editWork.g_x);
	fill.g_y = Min(MaxHeight - 1, ed->editWork.g_y + screenLine * ed->charHeight);
	fill.g_w = Min(MaxWidth - 1, ed->editWork.g_x + ed->editWork.g_w - 1);
	fill.g_h = Min(MaxHeight - 1, ed->editWork.g_y + (screenLine + 1) * ed->charHeight - 1);
	if (WdwManager_RectList(ed->wdw, 0, &clipr))
	{
		do
		{
			WdwManager_SetClip(ed->hdl, &clipr, TRUE);
			vr_recfl(ed->hdl, (_WORD *)&fill);
			vswr_mode(ed->hdl, MD_TRANS);

			/* String ausgeben */
			if (ed->StartLine + screenLine < ed->totalLineNr)
			{
				MakeDrawLine(ed, ed->StartLine + screenLine, theDrawLine, sizeof(theDrawLine), &len);
				text = theDrawLine;
				width = EditGlobals_OutputLine(ed, fill.g_x + pixOff - ed->leftOffset, fill.g_y, text, len, 30000);
				if (width > ed->maxWidth)
				{
					ed->maxWidth = width;
				}
			}
			/* Jetzt noch testen, ob es im Block war */
			vswr_mode(ed->hdl, MD_REPLACE);

			if (IsInBlock(ed, ed->StartLine + screenLine))
			{
				vs_clip(ed->hdl, TRUE, (_WORD *)&fill);
				EditDraw_ShowBlockMark(ed, FALSE);
				EditGlobals_ClipWork(ed);
			}
		} while (WdwManager_RectList(ed->wdw, 1, &clipr));
	}
}

/*** ---------------------------------------------------------------------- ***/

/* zeichnet currLine von currRow1 bis Ende neu */
LOCAL _VOID redrawLineFSM(EDITOR *ed)
{
	_WORD width;
	_WORD x, y;							/* aktuelle Zeichenposition */
	GRECT fill;							/* Fuellrechteck */
	_UBYTE *text;
	EDCOLUMN len;
	GRECT clipr;
	_WORD w;
	
	/* Zeilenanfang holen */
	MakeDrawLine(ed, ed->currLineNr, theDrawLine, sizeof(theDrawLine), &len);
	text = theDrawLine;
	if (ed->currRow > 0)
	{
		w = ed->currRow - 1;
	} else
	{
		w = 0;
	}
	w = EditGlobals_InqNTextextend(ed, text, w, FALSE, 30000);
	x = ed->editWork.g_x + w + pixOff - ed->leftOffset;
	y = ed->editWork.g_y + (_WORD)(ed->currLineNr - ed->StartLine) * ed->charHeight;

	fill.g_x = Min(MaxWidth - 1, x);
	fill.g_y = Min(MaxHeight - 1, y);
	fill.g_w = Min(MaxWidth - 1, ed->editWork.g_x + ed->editWork.g_w - 1);
	fill.g_h = Min(MaxHeight - 1, y + ed->charHeight - 1);
	if (WdwManager_RectList(ed->wdw, 0, &clipr))
	{
		do
		{
			WdwManager_SetClip(ed->hdl, &clipr, TRUE);
			vr_recfl(ed->hdl, (_WORD *)&fill);
			/* String ausgeben */
			fill.g_x = Min(MaxWidth - 1, ed->editWork.g_x);
			vswr_mode(ed->hdl, MD_TRANS);
			width = EditGlobals_OutputLine(ed, fill.g_x + pixOff - ed->leftOffset, fill.g_y, text, len, 30000);
			if (width + ed->charWidth > ed->maxWidth)
			{
				ed->maxWidth = width + ed->charWidth;
			}
			if (IsInBlock(ed, ed->currLineNr))
			{
				vs_clip(ed->hdl, TRUE, (_WORD *)&fill);
				EditDraw_ShowBlockMark(ed, FALSE);
				EditGlobals_ClipWork(ed);
			}
			vswr_mode(ed->hdl, MD_REPLACE);
		} while (WdwManager_RectList(ed->wdw, 1, &clipr));
	}
}

/*** ---------------------------------------------------------------------- ***/

/* zeichnet currLine von currRow1 bis Ende neu,
 * fuer monospaced font
 */
LOCAL _VOID redrawLineMono(EDITOR *ed, _BOOL insChar)
{
	_WORD width;
	_WORD x, y;								/* aktuelle Zeichenposition */
	GRECT fill;								/* Fuellrechteck */
	_UBYTE lastCh[8];
	_WORD copy[8];
	GRECT tmp;
	_UBYTE *strAdr;
	EDCOLUMN le, row;
	MFDB pscrMFDB;
	MFDB pdesMFDB;
	GRECT clipr;

	pscrMFDB.fd_addr = 0;
	pdesMFDB.fd_addr = 0;
	/* Fenster nach links aus dem Bildschirm raus mit WinX? */
	if (ed->editWork.g_x < 0)
	{
		redrawLineFSM(ed);
		return;
	}
	MakeDrawLine(ed, ed->currLineNr, theDrawLine, sizeof(theDrawLine), &le);
	/* Zeichen unter Cursor */
	lastCh[0] = theDrawLine[ed->currRow];
	lastCh[1] = theDrawLine[ed->currRow + 1];
	lastCh[2] = '\0';
	/* zeichenposition berechnen */
	x = ed->editWork.g_x + pixOff - ed->leftOffset + ed->currRow * ed->charWidth;
	y = ed->editWork.g_y + (_WORD)(ed->currLineNr - ed->StartLine) * ed->charHeight;

	fill.g_x = Min(MaxWidth - 1, x);
	fill.g_y = Min(MaxHeight - 1, y);
	fill.g_w = Min(MaxWidth - 1, x + ed->charWidth - 1);
	fill.g_h = Min(MaxHeight - 1, y + ed->charHeight - 1);
	if (WdwManager_RectList(ed->wdw, 0, &clipr))
	{
		do
		{
			WdwManager_SetClip(ed->hdl, &clipr, TRUE);

			if (ed->insertMode || !insChar || theDrawLine[ed->currRow] < 32)
			{
				/* Bitblocktransfer der uebrigen Zeichen nach hinten */
				copy[0] = Min(MaxWidth - 1, x);
				copy[1] = Min(MaxHeight - 1, y);
				copy[2] = Min(MaxWidth - 1, ed->editWork.g_x + ed->editWork.g_w - ed->charWidth - 1);
				copy[3] = Min(MaxHeight - 1, y + ed->charHeight - 1);
				copy[4] = Min(MaxWidth - 1, x + ed->charWidth);
				copy[5] = copy[1];
				copy[6] = Min(MaxWidth - 1, ed->editWork.g_x + ed->editWork.g_w - 1);
				copy[7] = copy[3];
				/* Beim Loeschen BitBlk-Transfer um 1 Zeichen zurueck */
				if (!insChar)
				{
					tmp = *((GRECT *)&copy[0]);
					*((GRECT *)&copy[0]) = *((GRECT *)&copy[4]);
					*((GRECT *)&copy[4]) = tmp;
				}
				vro_cpyfm(ed->hdl, S_ONLY, copy, &pscrMFDB, &pdesMFDB);
				/* Noch Zeilenbreite bestimmen */
				width = le * ed->charWidth;
				if (width + ed->charWidth > ed->maxWidth)
				{
					ed->maxWidth = width + ed->charWidth;
				}
			}
			if (!insChar)
			{
				/* Zeilenende noch loeschen und ggf neuzeichnen */
				copy[0] = copy[2] - ed->charWidth;
				vr_recfl(ed->hdl, copy);
				row = ((copy[0] - ed->editWork.g_x + ed->leftOffset - pixOff) / ed->charWidth);
				if (row < le && row >= 0)
				{
					strAdr = &theDrawLine[row];
					vswr_mode(ed->hdl, MD_TRANS);
					copy[0] = (ed->editWork.g_x - ed->leftOffset + pixOff) + row * ed->charWidth;
					EditGlobals_NText(ed, copy[0], copy[1], strAdr, le - row);
				}
			} else
			{
				vr_recfl(ed->hdl, (_WORD *)&fill);
				/* Zeichen ausgeben */
				vswr_mode(ed->hdl, MD_TRANS);
				EditGlobals_Text(ed, fill.g_x, fill.g_y, lastCh);
			}
			/* Blockmarkierung erneuern */
			if (IsInBlock(ed, ed->currLineNr))
			{
				vs_clip(ed->hdl, TRUE, (_WORD *)&fill);
				EditDraw_ShowBlockMark(ed, FALSE);
				EditGlobals_ClipWork(ed);
			}
			vswr_mode(ed->hdl, MD_REPLACE);
		} while (WdwManager_RectList(ed->wdw, 1, &clipr));
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditDraw_redrawLine(EDITOR *ed, _BOOL insChar, _BOOL full)
{
	if (ed->wdw != theTopWindow)
	{
		redrawLineFSM(ed);
	} else if (ed->monoSpaced && !full && !ed->realTabs && !ed->isFSM)	/* added "&& !isFSM" for Monospaced 821 Font */
	{
		redrawLineMono(ed, insChar);
	} else if ((ed->realTabs || full) && !ed->isFSM)
	{
		EditDraw_drawLine(ed, (_WORD)(ed->currLineNr - ed->StartLine));
	} else
	{
		redrawLineFSM(ed);
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Blaetterfunktionen */

GLOBAL _VOID EditDraw_CenterCurrline(EDITOR *ed)
{
	_WORD maxOffset;
	_WORD oldOffset;
	_WORD x, y;
	_BOOL centered;

	centered = FALSE;
	if (ed->currLineNr < ed->StartLine || ed->currLineNr > ed->StartLine + ed->windLines - 1)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		/* Zentrieren ist noetig! */
		EditTools_SetStartLine(ed, LMax(0, ed->currLineNr - ed->windLines / 2), TRUE);
		centered = TRUE;
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
	EditTools_getCharPos(ed, ed->currLineNr, ed->currRow, &x, &y);
	if (x < ed->editWork.g_x ||
		x >= ed->editWork.g_x + ed->editWork.g_w ||
		(ed->editWork.g_w > ed->maxWidth && ed->leftOffset > 0))
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		/* Zentrieren ist noetig! */
		oldOffset = ed->leftOffset;
		maxOffset = Max(0, ed->maxWidth - ed->editWork.g_w);
		/* neuen offset berechnen */
		ed->leftOffset = Max(0, oldOffset + (x - ed->editWork.g_x) - ed->charWidth * 5);
		ed->leftOffset = Min(ed->leftOffset, maxOffset);
		if (x + oldOffset < ed->editWork.g_w - ed->charWidth * 2)
		{
			ed->leftOffset = 0;
		}
		EditGlobals_SetDocument(ed);
		centered = TRUE;
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
	if (centered)
	{
		EditTools_SetInfoLine(ed);
	}
}
