/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 * 31.1.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.20k'
 *	Trat beim mimedekodieren eines Blocks ein Fehler auf, so
 *	wurde die Fehlermeldung einmal pro Zeile angezeigt
 */
 

#include <editfunc.h>
#include <clip.h>
#include <fontsele.h>
#include <editglob.h>
#include <edittool.h>
#include <editdraw.h>
#include <editor.h>
#include <editbase.h>
#include <wdwmanag.h>
#include <printer.h>
#include <confvars.h>
#include <varn_ede.h>
#include <catgloba.h>
#include <uudecode.h>

#include <mtalerts.h>
#include <mtappl.h>
#include <mtdials.h>
#include <mtutils.h>
#include <mttextfi.h>

#include <tos.h>
#include <strconv.h>

#include <routinec.h>
#include <routines.h>
#include <routine.h>
#include <lists.h>
#include <mime.h>
#include <mtrsc.h>
#include <catlib.h>


GLOBAL EDITOR *editList;
GLOBAL _UWORD rotateFaktor = 13;

#define compileMsg  " Compiling..."

#define CR      0x0d
#define LF      0x0a
#define TAB     0x09


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID MTE_noWrite(_VOID)
{
	mtAlerts_Alert(1, Rsc_WrkString(AL_NO_WRITE));
}

/*** ---------------------------------------------------------------------- ***/

/* "Suchfunktionen" */

GLOBAL EDITOR *EditFuncs_FindEditor(HWND wdw)
{
	EDITOR *ed;

	for (ed = editList; ed != NULL; ed = ed->nextEdit)
		if (ed->wdw == wdw)
			return ed;
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt die interne Nummer des Editors zurueck */
GLOBAL _WORD EditFuncs_GetEditNumber(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor (wdw);
	if (ed != NULL)
		return ed->number;
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL EDCOLUMN EditFuncs_findNext(CONST _UBYTE *text, EDCOLUMN fromPos, EDCOLUMN l, SET tSet)
{
	EDCOLUMN z;

	z = fromPos;
	while (z < l && text[z] != '\0' && text[z] != CR && text[z] != LF && !setin(tSet, text[z]))
		++z;
	while (z < l && text[z] != '\0' && text[z] != CR && text[z] != LF && setin(tSet, text[z]))
		++z;
	return z;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL EDCOLUMN EditFuncs_FindNext(EDITOR *ed, _BOOL umbruch, EDCOLUMN l)
{
	if (umbruch)
		return EditFuncs_findNext(ed->editLine.text, ed->currRow, l, UmbruchSet);
	return EditFuncs_findNext(ed->editLine.text, ed->currRow, l, TrennSet);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL EDCOLUMN EditFuncs_findLast(CONST _UBYTE *text, EDCOLUMN fromPos, SET tSet)
{
	EDCOLUMN z;

	z = fromPos;
	while (z > 0 && setin(tSet, text[z-1]))
		--z;
	while (z > 0 && !setin(tSet, text[z-1]))
		--z;
	return z;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL EDCOLUMN EditFuncs_FindLast(EDITOR *ed, _BOOL umbruch)
{
	if (umbruch)
		return EditFuncs_findLast(ed->editLine.text, ed->currRow, UmbruchSet);
	return EditFuncs_findLast(ed->editLine.text, ed->currRow, TrennSet);
}

/*** ---------------------------------------------------------------------- ***/
	
GLOBAL _VOID EditFuncs_CursorDown(EDITOR *ed, _BOOL ctrl)
{
	EDCOLUMN prefRow;

	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	if (ed->block && ed->readOnly==MODE_RW)
	{
		EditTools_SetCurrLine(ed, ed->blocks[0].blockEnd.line);
		ed->currRow = ed->blocks[0].blockEnd.row;
		EditGlobals_FixCursorPos (ed);
		EditTools_ClearBlock(ed);
	}
	/* hier nur ganze Zeile */
	if ((ed->currLineNr - ed->StartLine == ed->windLines - 1) ||
		(ed->readOnly!=MODE_RW && !ed->multiBlockMark && !ed->listMode))
	{
		if (ctrl)
			WdwManager_PageDown(ed->wdw);
		else
			WdwManager_ScrollDown(ed->wdw, 1);
	} else
	{
		if (ctrl)
			EditTools_SetCurrLine(ed, ed->StartLine + ed->windLines - 1);
		else
			EditTools_SetCurrLine(ed, ed->currLineNr + 1);
		ed->currRow = Min(ed->prefRow, ed->editLen);
		prefRow=ed->prefRow;
		EditGlobals_FixCursorPos(ed);
		ed->prefRow=prefRow;
	}
	EditDraw_CenterCurrline(ed);
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

	
GLOBAL _VOID EditFuncs_CursorUp(EDITOR *ed, _BOOL ctrl)
{
	EDCOLUMN prefRow;

	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	if (ed->block && ed->readOnly==MODE_RW)
	{
		EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
		ed->currRow = ed->blocks[0].blockStart.row;
		EditGlobals_FixCursorPos(ed);
		EditTools_ClearBlock(ed);
	}
	if ((ed->currLineNr == ed->StartLine && ed->currLineNr > 0) ||
		(ed->readOnly!=MODE_RW && !ed->multiBlockMark && !ed->listMode))
	{
		if (ctrl)
			WdwManager_PageUp(ed->wdw);
		else
			WdwManager_ScrollUp(ed->wdw, 1);
	} else if (ed->currLineNr - ed->StartLine != 0 && ed->currLineNr > 0)
	{
		if (ctrl)
			EditTools_SetCurrLine(ed, ed->StartLine);
		else
			EditTools_SetCurrLine(ed, ed->currLineNr - 1);
		ed->currRow = Min(ed->prefRow, ed->editLen);
		prefRow=ed->prefRow;
		EditGlobals_FixCursorPos(ed);
		ed->prefRow=prefRow;
	}
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _VOID EditFuncs_CursorLeft(EDITOR *ed, _UWORD kstate)
{
	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	if (ed->block && ed->readOnly==MODE_RW)
	{
		EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
		ed->currRow = ed->blocks[0].blockStart.row;
		EditGlobals_FixCursorPos(ed);
		EditTools_ClearBlock(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
		return;
	}
	if (ed->readOnly!=MODE_RW && !ed->multiBlockMark)
	{
		if (kstate & K_SHIFT)
			WdwManager_PageLeft(ed->wdw);
		else
			WdwManager_ScrollLeft(ed->wdw, 1);
	} else if (ed->currRow == 0)
	{
		if (ed->currLineNr > ed->StartLine)
		{
			EditTools_SetCurrLine(ed, ed->currLineNr - 1);
			ed->currRow = ed->editLen;
		} else if (ed->StartLine > 0)
		{
			WdwManager_ScrollUp(ed->wdw, 1);
			ed->currRow = ed->editLen;
		}
	} else if (kstate & K_CTRL)
	{
		ed->currRow = EditFuncs_FindLast(ed, FALSE);
	} else if (kstate & K_SHIFT)
	{
		ed->currRow = 0;
	} else if (ed->currRow > 0)
	{
		--ed->currRow;
	}
	EditGlobals_FixCursorPos(ed);
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _VOID EditFuncs_CursorRight(EDITOR *ed, _UWORD kstate)
{
	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	if (ed->block && ed->readOnly==MODE_RW)
	{
		EditTools_SetCurrLine(ed, ed->blocks[0].blockEnd.line);
		ed->currRow = ed->blocks[0].blockEnd.row;
		EditGlobals_FixCursorPos(ed);
		EditTools_ClearBlock(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
		return;
	}
	if (ed->readOnly!=MODE_RW && !ed->multiBlockMark)
	{
		if (kstate & K_SHIFT)
			WdwManager_PageRight(ed->wdw);
		else
			WdwManager_ScrollRight(ed->wdw, 1);
	} else if (ed->editLine.text[ed->currRow] == CR ||
		ed->editLine.text[ed->currRow] == LF ||
		ed->editLine.text[ed->currRow] == '\0')
	{
		/* Zeilenende */
		if (ed->currLineNr - ed->StartLine < ed->windLines-1 &&
			ed->currLineNr < ed->totalLineNr - 1)
		{
			EditTools_SetCurrLine(ed, ed->currLineNr + 1);
			ed->currRow = 0;
		} else if (ed->currLineNr < ed->totalLineNr - 1)
		{
			WdwManager_ScrollDown(ed->wdw, 1);
			ed->currRow = 0;
		}
	} else if (kstate & K_CTRL)
	{
		ed->currRow = EditFuncs_FindNext(ed, FALSE, ed->editLen);
	} else if (kstate & K_SHIFT)
	{
		ed->currRow = ed->editLen;
	} else
	{
		++ed->currRow;
	}
	EditGlobals_FixCursorPos(ed);
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_AllUp(_UBYTE *text)
{
	strUpr(text);
	return GERR_OK;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_AllLower(_UBYTE *text)
{
	strLwr(text);
	return GERR_OK;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_SwapUpperLower(_UBYTE *text)
{
	while (*text != '\0')
	{
		if (is_upper(*text))
			*text = lcase(*text);
		else if (is_lower(*text))
			*text = ucase(*text);
		++text;
	}
	return GERR_OK;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_Capitalise(_UBYTE *text)
{
	for (;;)
	{
		while (*text != '\0' && setin(TrennSet, *text))
			++text;
		if (*text == '\0')
			break;
		*text = ucase(*text);
		text++;
		while (*text != '\0' && !setin(TrennSet, *text))
		{
			*text = lcase(*text);
			++text;
		}
	}
	return GERR_OK;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_Mirror(_UBYTE *text)
{
	_UBYTE ch;

	while ((ch = *text) != '\0')
	{
		if (ch >= 'A' && ch <= 'Z')
			ch = 'A' + 'Z' - ch;
		else if (ch >= 'a' && ch <= 'z')
			ch = 'a' + 'z' - ch;
		*text++ = ch;
	}
	return GERR_OK;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_Rot18(_UBYTE *text)
{
	_UBYTE ch;
	CONST _UBYTE *from = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz";
	CONST _UBYTE *to   = "STUVWXYZ0123456789ABCDEFGHIJKLMNOPQRnopqrstuvwxyzabcdefghijklm";
	CONST _UBYTE *p;

	while ((ch = *text) != '\0')
	{
		if ((p = strChr(from, ch)) != NULL)
			*text = to[p - from];
		text++;
	}
	return GERR_OK;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_RotXX(_UBYTE *text)
{
	_UBYTE ch;

	while ((ch = *text) != '\0')
	{
		if (ch >= 'A' && ch <= 'Z')
			ch = ((ch - 'A' + rotateFaktor) % 26) + 'A';
		else if (ch >= 'a' && ch <= 'z')
			ch = ((ch - 'a' + rotateFaktor) % 26) + 'a';
		*text++ = ch;
	}
	return GERR_OK;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_Protect(_UBYTE *text)
{
	size_t l;

	l = strLen(text);
	while (l > 0 && (text[l-1] == CR || text[l-1] == LF))
		--l;
	text[l++] = CR;
	text[l++] = LF;
	text[l] = '\0';
	return GERR_OK;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID EditFuncs_UnProtect(_UBYTE *text)								*/
/*	Umkehrfunktion zur vorigen													*/
/*	Parameter:	text: der zu ndernde Text															*/
/*	Ergebnis:	keines															*/		
/*	6.3.01 von Dimitri Junker	DYJ 4.23n										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_UnProtect(_UBYTE *text)
{
	size_t l;

	l = strLen(text);
	while (l > 0 && (text[l-1] == CR || text[l-1] == LF))
		--l;
	text[l++] = LF;
	text[l] = '\0';
	return GERR_OK;
}


/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_XXDecode(_UBYTE *text)
{
	size_t l;
	_UBYTE saveCh;
	F_ERROR err;
	
	l = strLen(text);
	while (l > 0 && (text[l-1] == CR || text[l-1] == LF))
		--l;
	saveCh = text[l];
	text[l] = '\0';
	if ((err = UUDecode_DecodeLine(text)) != GERR_OK)
		F_ErrorAlert(err);
	text[l] = saveCh;
	return GERR_OK;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID EditFuncs_MimeDecode(_UBYTE *text)								*/
/*	wie die vorige aber fr MIME												*/
/*	Parameter:	text:	der zu dekodierende Text								*/
/*	Ergebnis:	keines															*/		
/*	20.11.01 von Dimitri Junker	DYJ 4.30ze										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.20k */
GLOBAL F_ERROR EditFuncs_MimeDecode(_UBYTE *text)
{
	size_t l;
	_UBYTE saveCh;
	F_ERROR err;
	
	l = strLen(text);
	while (l > 0 && (text[l-1] == CR || text[l-1] == LF))
		--l;
	saveCh = text[l];
	text[l] = '\0';
	err=Mime_DecodeLine(text);
	text[l] = saveCh;
	return err;
}

/*** ---------------------------------------------------------------------- ***/

/* Ruft fuer jede Zeile im Block die Funktion trans auf
 * Anfang und Ende werden entsprechend umgeformt!
 */
	/*DYJ 5.20k */
GLOBAL _VOID EditFuncs_TransformBlock(EDITOR *ed, formFunc trans, _BOOL wholeLines)
{
	_UBYTE *ch2;
	_UBYTE save2;
	_UBYTE *t1;
	EDCOLUMN le;
	EDLINE l;
	aMark blS, blE;
	EDLINE oldCurrLine;
	EDCOLUMN oldRow;

	oldCurrLine = ed->currLineNr;
	oldRow = ed->currRow;
	EditTools_ShowFileChanged(ed);
	if (wholeLines)
	{
		blS = ed->blocks[0].blockStart;
		blE = ed->blocks[0].blockEnd;
		ed->blocks[0].blockStart.row = 0;
		if (ed->blocks[0].blockEnd.line == ed->totalLineNr - 1)
		{
			ed->blocks[0].blockEnd.row = EditTools_ToLineEnd(ed, ed->blocks[0].blockEnd.line);
		} else
		{
			ed->blocks[0].blockEnd.row = 0;
			++ed->blocks[0].blockEnd.line;
		}
	}
	EditTools_SaveOp(ed, blockPaste, FALSE);
	EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
	ed->currRow = ed->blocks[0].blockStart.row;
	EditGlobals_FixCursorPos(ed);
	if (ed->blocks[0].blockStart.line == ed->blocks[0].blockEnd.line)
	{
		if (!wholeLines)
		{
			/* Nur Block in einer Zeile */
			t1 = &ed->editLine.text[ed->blocks[0].blockStart.row];
			ch2 = &ed->editLine.text[ed->blocks[0].blockEnd.row];
			save2 = *ch2;
			*ch2 = '\0';
			trans(t1);
			*ch2 = save2;
		} else
		{
			t1 = ed->editLine.text;
			trans(t1);
		}
		ed->editChanged = TRUE;
		ed->editLen = (EDCOLUMN)strLen(ed->editLine.text);
		EditBase_PutCurrLine(ed);
	} else
	{
		/* mehrzeiliger Block */
		l = ed->blocks[0].blockStart.line;
		while (l <= ed->blocks[0].blockEnd.line)
		{
			le = ed->editLen;
			if (wholeLines)
				EditGlobals_AllocNew(&ed->editLine, ed->editLen + 3);
			if (l == ed->blocks[0].blockStart.line &&
				ed->blocks[0].blockStart.row > 0 &&
				!wholeLines)
			{
				/* Nur Teilweise bearbeiten */
				t1 = &ed->editLine.text[ed->blocks[0].blockStart.row];
				trans(t1);
			} else if (l == ed->blocks[0].blockEnd.line &&
				ed->blocks[0].blockEnd.row < le &&
				!wholeLines)
			{
				/* ebenfalls nur teilweise bearbeiten */
				ch2 = &ed->editLine.text[ed->blocks[0].blockEnd.row];
				save2 = *ch2;
				*ch2 = '\0';
				trans(ed->editLine.text);
				*ch2 = save2;
			} else
			{
			if(trans(ed->editLine.text)!=GERR_OK)
				break;
			}
			ed->editChanged = TRUE;
			ed->editLen = (EDCOLUMN)strLen(ed->editLine.text);
			EditTools_SetCurrLine(ed, ed->currLineNr + 1);
			++l;
		}
	}
	/* Noch Blockstart fuer Undobuffer merken */
	EditTools_NewBlockToUndoBuffer(ed);
	if (wholeLines)
	{
		ed->blocks[0].blockStart = blS;
		ed->blocks[0].blockEnd = blE;
	}
	ed->prefRow = ed->currRow = oldRow;
	EditTools_SetCurrLine(ed, oldCurrLine);
	/* Jetzt noch redraw von blockstart bis blockende */
	EditDraw_redrawLineArea(ed, ed->blocks[0].blockStart.line, ed->blocks[0].blockEnd.line);
}

/*** ---------------------------------------------------------------------- ***/

	
GLOBAL _VOID EditFuncs_Home(EDITOR *ed, _BOOL ende)
{
	EDLINE newLine;

	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	if (ed->block && ed->readOnly==MODE_RW)
		EditTools_ClearBlock(ed);
	if (ende)
	{
		newLine = LMax(0, ed->totalLineNr - ed->windLines);
		if (newLine != ed->StartLine && newLine != 0)
			EditTools_SetStartLine(ed, newLine, TRUE);
		EditTools_SetCurrLine(ed, ed->totalLineNr - 1);
		ed->currRow = ed->editLen;
		EditGlobals_FixCursorPos(ed);
	} else
	{
		if (ed->StartLine != 0)
			EditTools_SetStartLine(ed, 0, TRUE);
		EditTools_SetCurrLine(ed, 0);
		ed->prefRow = ed->currRow = 0;
	}
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditFuncs_SwapLines(EDITOR *ed, _BOOL withUp)
{
	EDLINE l;
	aLineDesc swap;
	EDCOLUMN len;
	_WORD screenLine;
	
	if (ed->block)
		return;
	if (withUp)
		l = ed->currLineNr - 1;
	else
		l = ed->currLineNr + 1;
	if (!EditBase_GetLineLength(ed, l, &len))
		return;
	EditTools_ShowFileChanged(ed);
	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	swap.text = MALLOC(len+2, "SwapLines");
	if (swap.text != NULL)
	{
		EditBase_GetLine(ed, l, swap.text, len + 2, &len);
		if (len > ed->editLen)
		{
			EditBase_PutLine(ed, l, ed->editLine.text, ed->editLen);
			EditBase_PutLine(ed, ed->currLineNr, swap.text, len);
		} else
		{
			EditBase_PutLine(ed, ed->currLineNr, swap.text, len);
			EditBase_PutLine(ed, l, ed->editLine.text, ed->editLen);
		}
		ed->editChanged = FALSE;
		screenLine = (_WORD)(ed->currLineNr - ed->StartLine);
		EditDraw_drawLine(ed, screenLine);
		if (withUp)
		{
			EditDraw_drawLine(ed, screenLine - 1);
			EditTools_SetCurrLine(ed, ed->currLineNr - 1);
		} else
		{
			EditDraw_drawLine(ed, screenLine + 1);
			EditTools_SetCurrLine(ed, ed->currLineNr + 1);
		}
		SFREE(swap.text);
		EditDraw_CenterCurrline(ed);
	}
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

typedef struct {
	_UBYTE *buffer;
	_ULONG bufPos;
	_ULONG bufSize;
	tResponse Response;
} writeBuf;

/* IO-Routinen */

LOCAL _LONG WriteBuffered(FILENO fHdl, _ULONG l, _VOID *buf, writeBuf *buffer)
{
	_ULONG remain;
	_ULONG wrBytes;
	_UBYTE *ptr;

	remain = l;
	ptr = buf;
	l = 0;
	while (remain + buffer->bufPos > buffer->bufSize)
	{
		MemCpy(&buffer->buffer[buffer->bufPos], ptr, buffer->bufSize - buffer->bufPos);
		wrBytes = Fwrite(fHdl, buffer->bufSize, buffer->buffer);
		if (wrBytes != buffer->bufSize)
		{
			if ((_LONG)wrBytes >= 0)
			{
				l += wrBytes - buffer->bufPos;
				buffer->bufPos = wrBytes;
			}
			return l;
		}
		ptr += buffer->bufSize - buffer->bufPos;
		remain -= buffer->bufSize - buffer->bufPos;
		l += buffer->bufSize - buffer->bufPos;
		buffer->bufPos = 0;
	}
	MemCpy(&buffer->buffer[buffer->bufPos], ptr, remain);
	buffer->bufPos += remain;
	l += remain;
	return l;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _LONG WritePrinter(FILENO fHdl, _ULONG l, _VOID *buf, writeBuf *buffer)
{
	_UBYTE *str;
	
	UNUSED(fHdl);
	if (l >= buffer->bufSize)
	{
		str = buf;
		if (str[0] == CR)
		{
			if (buffer->Response == Printer_done)
				buffer->Response = Printer_WriteLn();
		} else
		{
			if (buffer->Response == Printer_done)
				buffer->Response = Printer_WriteString(str);
		}
	} else
	{
		MemCpy(buffer->buffer, buf, l);
		str = buffer->buffer;
		str[l] = '\0';
		if (l > 0 && str[l-1] == CR)
			str[l-1] = '\0';
		if (str[0] == CR)
		{
			if (buffer->Response == Printer_done)
				buffer->Response = Printer_WriteLn();
		} else
		{
			if (buffer->Response == Printer_done)
				buffer->Response = Printer_WriteString(str);
		}
	}
	return l;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _LONG WriteThrough(FILENO fHdl, _ULONG l, _VOID *buf, writeBuf *buffer)
{
	UNUSED(buffer);
	return Fwrite(fHdl, l, buf);
}

/*** ---------------------------------------------------------------------- ***/

/* allgemeine Saveroutine */


GLOBAL _BOOL EditFuncs_outputText(
	EDITOR *ed,
	EDLINE fromLine, EDCOLUMN fromRow,
	EDLINE toLine, EDCOLUMN toRow,
	FILENO fHdl,
	_BOOL umbruch, _BOOL usePrintMod)
{
	EDLINE line;
	_BOOL saveall;
	EDLINE i;
	_ULONG wlen, len;
	EDCOLUMN length;
	_UBYTE *writeAdr;
	_LONG (*writeProc)(FILENO, _ULONG, _VOID *, writeBuf *);
	_BOOL writeCR;
	_BOOL writeLF;
	aBuffer *buff;
	_UBYTE s[256];
	EDCOLUMN margin;
	_UBYTE marginStr[13];
	writeBuf buffer;
	_UBYTE crlf[3];
	_UBYTE cr[2];
	_UBYTE lf[2];
	
	MouseBusy();
	saveall = toLine < fromLine;
	buffer.buffer = NULL;
	buffer.bufSize = 0xF000L;
	while (buffer.buffer == NULL && buffer.bufSize > 0)
	{
		buffer.buffer = SYSMALLOC(buffer.bufSize, MEM_NOERR, "outputText");
		if (buffer.buffer == NULL)
			buffer.bufSize -= 0x100;
	}
	if (buffer.bufSize == 0)
	{
		writeProc = WriteThrough;
	} else
	{
		if (usePrintMod)
		{
			writeProc = WritePrinter;
			ConfVars_GetConfDefInt(cPrintLeftMargin, &margin, 0);
			if (margin > 0)
			{
				if (margin >= sizeof(marginStr))
					margin = (_WORD)sizeof(marginStr) - 1;
				MemSet(marginStr, ' ', margin);
				marginStr[margin] = '\0';
			}
		} else
		{
			writeProc = WriteBuffered;
		}
	}
	buffer.bufPos = 0;
	buffer.Response = Printer_done;
	crlf[0] = CR;
	crlf[1] = LF;
	crlf[2] = '\0';
	cr[0] = CR;
	cr[1] = '\0';
	lf[0] = LF;
	lf[1] = '\0';
	if (saveall && !(usePrintMod || ed->readOnly!=MODE_RW))
	{
		buff = ed->firstBuff;
		while (buff != NULL)
		{
			len = buff->usedMem;
			len = writeProc(fHdl, len, buff->buffer, &buffer);
			if (len != buff->usedMem)
			{
				/* Schreibfehler! */
				MouseArrow();
				MTE_noWrite();
				if (buffer.bufSize > 0)
					SFREE(buffer.buffer);
				return FALSE;
			}
			buff = buff->next;
		}
	} else
	{
		/* Nur langsames, zeilenweises sichern */
		if (saveall && (usePrintMod || ed->readOnly!=MODE_RW))
		{
			fromLine = 0; 
			fromRow = 0;
			toLine = ed->totalLineNr - 1;
			EditBase_GetLineLength(ed, toLine, &toRow);
			/* Extinfo ausgeben, falls vorhanden */
			if (ed->readOnly!=MODE_RW && !usePrintMod &&
				ed->userPrint != FUNK_NULL &&
				ed->userPrint(ed->wdw, LFIRST, s, sizeof(s)))
			{
				/* erste Zeile ausgeben */
				writeAdr = s;
				do
				{
					len = strLen(s);
					wlen = writeProc(fHdl, len, writeAdr, &buffer);
					if (wlen != len)
					{
						/* Schreibfehler! */
						MouseArrow();
						MTE_noWrite();
						if (buffer.bufSize > 0)
							SFREE(buffer.buffer);
						return FALSE;
					}
					/* CR/LF schreiben */
					wlen = writeProc(fHdl, 2, crlf, &buffer);
					if (wlen != 2)
					{
						MouseArrow();
						MTE_noWrite();
						if (buffer.bufSize > 0)
							SFREE(buffer.buffer);
						return FALSE;
					}
				} while (ed->userPrint(ed->wdw, LNEXT, s, sizeof(s)));
			}
		}
		line = fromLine;
		i = fromLine;
		while (line <= toLine)
		{
			EditBase_GetLineAdr(ed, line, &writeAdr, &length);
			len = length;
			if (i == fromLine)
			{
				writeAdr += fromRow;
				len = LMax(0, len - fromRow);
			}
			if (i == toLine)
			{
				len -= length - toRow;
			}
			/* CR/LFs am Zeilenende entfernen */
			writeLF = FALSE;
			writeCR = FALSE;
			while (len > 0 && (writeAdr[len-1] == CR || writeAdr[len-1] == LF))
			{
				if (writeAdr[len-1] == LF)
					writeLF = TRUE;
				else
					writeCR = TRUE;
				--len;
			}
			if (ed->readOnly!=MODE_RW)
			{
				writeLF = TRUE;
				writeCR = TRUE;
			}
			if (i >= fromLine && i <= toLine)
			{
				if (usePrintMod && margin > 0)
				{
					if (buffer.Response == Printer_done)
						buffer.Response = Printer_WriteString(marginStr);
				}
				wlen = writeProc(fHdl, len, writeAdr, &buffer);
				if (wlen != len)
				{
					/* Schreibfehler! */
					MouseArrow();
					MTE_noWrite();
					if (buffer.bufSize > 0)
						SFREE(buffer.buffer);
					return FALSE;
				}
			}
			if (i >= fromLine && i <= toLine)
			{
				if (writeLF && writeCR)
				{
					len = 2;
					writeAdr = crlf;
				} else if (writeLF)
				{
					len = 1;
					writeAdr = lf;
				} else if ((writeCR && !umbruch) || (writeCR && usePrintMod))
				{
					len = 1;
					writeAdr = cr;
				} else
				{
					len = 0;
				}
				if (usePrintMod)
				{
					if (buffer.Response == Printer_done)
						buffer.Response = Printer_WriteLn();
				} else if (!(i == toLine && toRow != length) && len > 0)
				{
					wlen = writeProc(fHdl, len, writeAdr, &buffer);
					if (len != wlen)
					{
						/* Schreibfehler! */
						MouseArrow();
						MTE_noWrite();
						if (buffer.bufSize > 0)
							SFREE(buffer.buffer);
						return FALSE;
					}
				}
			}
			++line;
			++i;
		}
	}
	if (buffer.bufSize > 0 && buffer.bufPos > 0)
	{
		if (!usePrintMod)
		{
			/* Buffer noch flushen */
			wlen = Fwrite(fHdl, buffer.bufPos, buffer.buffer);
			if (wlen != buffer.bufPos)
			{
				MouseArrow();
				MTE_noWrite();
				if (buffer.bufSize > 0)
					SFREE(buffer.buffer);
				return FALSE;
			}
		}
	}
	if (buffer.bufSize > 0)
		SFREE(buffer.buffer);
	MouseArrow();
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _VOID EditFuncs_KillEditor(EDITOR *ed)
{
	EDITOR *edi;
	
	if ((edi = ed) == NULL)
		return;
	while (edi->firstBuff != NULL)
	{
		edi->currentBuff = edi->firstBuff;
		EditBase_FreeBuffer(edi);
	}
	if (edi->editLine.text != NULL)
		SFREE(edi->editLine.text);
	if (edi->fileName.text != NULL)
		SFREE(edi->fileName.text);
	if (editList != edi)
	{
		edi = editList;
		while (edi->nextEdit != ed)
		{
			/* ed suchen in Liste */
			edi = edi->nextEdit;
		}
		edi->nextEdit = ed->nextEdit;
	} else
	{
		editList = edi->nextEdit;
	}
	OFREE(ed);
	if (editList == NULL)
	{
		/* letzter Editor geschlossen */
		EditTools_FreeUndoBuffer();
	}
}

/*** ---------------------------------------------------------------------- ***/

	
GLOBAL EDITOR *EditFuncs_CreateEditor(_VOID)
{
	EDITOR *ed;
	_WORD i;
	
	ed = NEW(EDITOR, "CreateEditor");
	if (ed == NULL)
		return NULL;
	MemSetZeroStruct(*ed);
	ed->wdw = -1;
	ed->StartLine = 0;
	ed->currLineNr = 0;
	ed->totalLineNr = 1;
	ed->TabSize = 8;
	ed->prefRow = ed->currRow = 0;
	ed->currentBuff = NULL;
	ed->editLine.text = NULL;
	ed->editLine.text = MALLOC(16, "CreateEditor: editLine");
	if (ed->editLine.text == NULL)
	{
		OFREE(ed);
		return NULL;
	}
	ed->editLine.length = 16;
	ed->editLine.text[0] = '\0';
	ed->firstBuff = NULL;
	ed->multiBlockMark = FALSE;
	ed->multiBlockIdx = -1;
	ed->editChanged = FALSE;
	ed->editLen = 0;
	ed->nextEdit = NULL;
	ed->DisplayEffects = FALSE;
	ed->userDD = FALSE;
	ed->isEnriched = FALSE;
	ed->leftOffset = 0;
	ed->readOnly=MODE_RW;
	ed->isGrin = FALSE;
	ed->alienCoords = FALSE;
	ed->changed = FALSE;
	ed->insertMode = TRUE;
	ed->cursor = 1;
	ed->block = FALSE;
	ed->blockIndent = FALSE;
	for (i = 0; i < maxBlocks; i++)
	{
		ed->blocks[i].blockStart.line = -1;
		ed->blocks[i].blockEnd.line = -1;
	}
	ed->backup = FALSE;
	ed->wdwIsOpen = FALSE;
	ed->wasOpen = FALSE;
	ed->listModeProc = FUNK_NULL;
	ed->listMode = FALSE;
	ed->hOffset = 0;
	ed->charHeight = 16;
	ed->charWidth = 8;
	ed->maxWidth = 0;
	ed->AllowLowerQuoteInitials = TRUE;
	ed->autoIndent = TRUE;
	ed->realTabs = FALSE;
	ed->isEnriched = FALSE;
	ed->umbruch = FALSE;
	/* Error-Parameter setzen */
	ed->errLine = -1;
	ed->errRow = 0;
	strBcpy(ed->errMsg, "Kein Fehler aufgetreten");
	
	ed->nextEdit = editList;
	editList = ed;
	return ed;
}

/*** ---------------------------------------------------------------------- ***/

	
GLOBAL _BOOL EditFuncs_ReadBuffer(EDITOR *ed, _UBYTE **buf, _ULONG len, _BOOL freeBuffer, ErrReason *err)
{
	EDBUFSIZE allocSize;
	
	*err = memError;
	if (ed->firstBuff == NULL)
	{
		if (ed->readOnly!=MODE_RW && len + 50 < memBlockSize)
			allocSize = (EDBUFSIZE)(len + 50);
		else
			allocSize = memBlockSize;
		if (!EditBase_GetBuffer(ed, allocSize))
			return FALSE;
		ed->currentBuff = ed->firstBuff;
		ed->currentBuff->fromLine = 0;
		ed->currentBuff->toLine = 0;
		ed->currentBuff->usedMem = 0;
		ed->currLineNr = 0;
		ed->prefRow = ed->currRow = 0;
		ed->totalLineNr = 1;
	}
	if (ed->editChanged)
		EditBase_PutCurrLine(ed);
	if (EditBase_InsertBuffer(ed, buf, len, freeBuffer))
	{
		EditTools_UpdateCurrLine(ed, ed->currLineNr);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID CheckMM2Format(EDITOR *ed)
{
	EDLINE clLine, cLine;
	EDCOLUMN cRow;
	EDCOLUMN i, spaces;
	
	/* Position sichern */
	clLine = ed->currLineNr;
	cRow = ed->currRow;
	/* Anpassung an MM2-Kurzformat */
	cLine = 0;
	EditTools_SetCurrLine(ed, cLine);
	/* Erstmal pruefen, ob ueberhaupt was gewandelt werden muss! */
	if (ed->editLine.text[0] == 0x10)
	{
		/* Ok, ist MM2-Kurzformat, wandeln */
		while (cLine < ed->totalLineNr)
		{
			EditTools_SetCurrLine(ed, cLine);
			if (ed->editLen > 1 && ed->editLine.text[0] == 0x10 && ed->editLine.text[1] >= 32)
			{
				/* change line */
				spaces = ed->editLine.text[1] - 32;
				EditGlobals_AllocNew(&ed->editLine, ed->editLen + spaces + 2);
				strCpy(ed->editLine.text, ed->editLine.text + 2);
				if (spaces > 0 && spaces < 80)
				{
					MemMove(ed->editLine.text + spaces, ed->editLine.text, ed->editLen + spaces - 2 + 1);
					MemSet(ed->editLine.text, ' ', spaces);
				}
				/* Zeilenende anpassen */
				ed->editLen = ed->editLen + spaces - 2;
				i = ed->editLen - 1;
				while (i >= 0 && (ed->editLine.text[i] == CR || ed->editLine.text[i] == LF))
					--i;
				++i;
				ed->editLine.text[i++] = CR;
				ed->editLine.text[i++] = LF;
				ed->editLine.text[i] = '\0';
				ed->editLen = i;
				ed->editChanged = TRUE;
			}
			++cLine;
		}
	}
	/* Position restaurieren */
	EditTools_SetCurrLine(ed, clLine);
	ed->prefRow = ed->currRow = cRow;
}

/*** ---------------------------------------------------------------------- ***/

	/*DYJ 5.21b */
GLOBAL _BOOL EditFuncs_readText(EDITOR *ed, _BOOL fastRead, _ULONG flen, FILENO *fHdl, EDCOLUMN right, _UBYTE **readBuf, _BOOL inLine, _BOOL fullText)
{
	_ULONG readBytes;
	EDLINE clLine;
	EDCOLUMN cRow;
	ErrReason err;
	_ULONG bufPos, rb;
	_BOOL wasBlock;
	aMark bEnd;
	_UBYTE *text;
	_UBYTE ch;
#define smallBufSize 28000L

	UNUSED(inLine);
	if (fastRead)
	{
		readBytes = Fread(*fHdl, flen, *readBuf);
		if (readBytes != flen)
		{
			/* nicht alles gelesen! */
			if ((_WORD)readBytes == -37)/*!ERROR*/
				F_ErrorAlert(-37);
			else
				mtAlerts_Alert(1, Rsc_WrkString(readErr));
			(void)Fclose(*fHdl);
			*fHdl = -1;
			return FALSE;
		}
		F_ErrorAlert(Fclose(*fHdl));
		EditBase_PutCurrLine(ed);
		*fHdl = -1;
		/* Text aus Buffer in Editor Zeilenweise uebertragen */
		EditGlobals_HideCursor(ed);
		MouseBusy();
		ed->rightMargin = right;
		/*
		 * Keine Dateien einfuegen, die nur aus Ctrl-Z bestehen
		 */
		if (flen != 1 || (*readBuf)[0] != 0x1a)
		{
			{
				if (!EditFuncs_ReadBuffer(ed, readBuf, flen, TRUE, &err))
				{
					if (*readBuf != NULL)
					{
						SFREE(*readBuf);
						*readBuf = NULL;
					}
					EditGlobals_HideMouse();
					EditGlobals_ShowCursor(ed);
					EditGlobals_ShowMouse(FALSE);
					MouseArrow();
					switch (err)
					{
						case memError: /* Fehlermeldung kam schon */ break;
						case readError: /* schon behandelt, tritt nicht auf */ break;
						case noTextError: /* Fehlermeldung kam schon */ break;
					}
					return FALSE;
				}
			}
		}
		/* 2 Buffer wieder freigeben */
		if (*readBuf != NULL)
		{
			SFREE(*readBuf);
			*readBuf = NULL;
		}
		if (fullText)
			CheckMM2Format(ed);
		EditGlobals_SetDocument(ed);
		EditGlobals_HideMouse();
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
		MouseArrow();
	} else
	{
		/* langsames Lesen des Textes */
		if (*readBuf != NULL)
		{
			SFREE(*readBuf);
			*readBuf = NULL;
		}
		*readBuf = MALLOC(smallBufSize, "readText: small readbuf");
		if (*readBuf == NULL)
			return FALSE;
		EditBase_PutCurrLine (ed);
		/* Text aus Buffer in Editor Zeilenweise uebertragen */
		EditGlobals_HideCursor(ed);
		MouseBusy();
		ed->rightMargin = right;
		bufPos = 0;
		clLine = ed->currLineNr;
		cRow = ed->currRow;
		wasBlock = ed->block;
		bEnd = ed->blocks[0].blockEnd;
		ed->block = TRUE;
		while (bufPos < flen)
		{
			Fseek(bufPos, *fHdl, SEEK_SET);
			readBytes = LMin(flen - bufPos, smallBufSize);
			rb = readBytes;
			readBytes = Fread(*fHdl, rb, *readBuf);
			if (readBytes != rb)
			{
				/* nicht alles gelesen! */
				if ((_WORD)readBytes == -37)/*!ERROR*/
					F_ErrorAlert(-37);
				else
					mtAlerts_Alert(1, Rsc_WrkString(readErr));
				(void)Fclose(*fHdl);
				EditGlobals_HideMouse();
				EditGlobals_ShowCursor(ed);
				EditGlobals_ShowMouse(FALSE);
				MouseArrow();
				*fHdl = -1;
				return FALSE;
			}
			/* Jetzt ausgehend vom Ende des Buffers ein Zeilenende suchen
			 * und nur bis dahin einlesen
			 */
			if (bufPos + readBytes < flen)
			{
				text = *readBuf;
				ch = text[readBytes - 1];
				while (ch != CR && ch != LF && readBytes > 0)
				{
					--readBytes;
					ch = text[readBytes - 1];
				}
			}
			if (!EditFuncs_ReadBuffer(ed, readBuf, readBytes, FALSE, &err))
			{
				if (*readBuf != NULL)
				{
					SFREE(*readBuf);
					*readBuf = NULL;
				}
				EditGlobals_HideMouse();
				EditGlobals_ShowCursor(ed);
				EditGlobals_ShowMouse(FALSE);
				MouseArrow();
				switch (err)
				{
					case memError: /* Fehlermeldung kam schon */ break;
					case readError: /* schon behandelt, tritt nicht auf */ break;
					case noTextError: /* Fehlermeldung kam schon */ break;
				}
				F_ErrorAlert(Fclose(*fHdl));
				*fHdl = -1;
				return FALSE;
			}
			if (ed->blocks[0].blockEnd.line > ed->totalLineNr - 1)
			{
				ed->blocks[0].blockEnd.line = ed->totalLineNr - 1;
				EditBase_GetLineLength(ed, ed->blocks[0].blockEnd.line, &ed->blocks[0].blockEnd.row);
			}
			EditTools_SetCurrLine(ed, ed->blocks[0].blockEnd.line);
			ed->currRow = ed->blocks[0].blockEnd.row;
			bufPos += readBytes;
		}
		F_ErrorAlert(Fclose(*fHdl));
		SFREE(*readBuf);
		*readBuf = NULL;
		/* Position restaurieren */
		ed->block = wasBlock;
		ed->blocks[0].blockEnd = bEnd;
		EditTools_SetCurrLine(ed, clLine);
		ed->prefRow = ed->currRow = cRow;
		if (fullText)
			CheckMM2Format(ed);
		EditGlobals_SetDocument(ed);
		EditGlobals_HideMouse();
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
		MouseArrow();
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditFuncs_Goto(EDITOR *ed, EDLINE line, EDCOLUMN row)
{
	if (line > 0)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		EditTools_SetCurrLine(ed, line - 1);
		ed->currRow = Min(row, ed->editLen);
		EditGlobals_FixCursorPos(ed);
		EditDraw_CenterCurrline(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditFuncs_showError(EDITOR *ed)
{
	if (ed->errLine >= 0)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		EditTools_ClearBlock(ed);
		EditFuncs_Goto(ed, ed->errLine, ed->errRow);
		EditTools_SetInfoLine(ed);
		WdwManager_SetWdwInfoline(ed->wdw, ed->errMsg);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	} else
	{
		WdwManager_SetWdwInfoline(ed->wdw, ed->errMsg);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditFuncs_callComp(EDITOR *ed)
{
	UNUSED(ed);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Funktionen und Typen fuer QED-kompatible Kuerzeldateien
 */

LOCAL List shortList;
LOCAL _BOOL shortListRead;

typedef struct {
	_UWORD val;  /* ARRAY [0..1] OF _UBYTE interpreted als Wort */
	_WORD len;   /* Laenge des Textes */
	_UBYTE *text;   /* Der Pointer auf den Text */
} shortElem, *shortPtr;


/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findValue(_VOID *entry, _VOID *info)
{
	shortElem *theVar;
	_UWORD *theName;
	
	theVar = entry;
	theName = info;
	return theVar->val == *theName;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findShortKey(_UWORD key, shortElem **var)
{
	Lists_ResetList(&shortList);
	if (Lists_ScanEntries(&shortList, Lists_forward, findValue, &key))
	{
		*var = Lists_CurrentEntry(&shortList);
		return *var != NULL;
	}
	*var = NULL;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* veraendert eine vorhandene Variable oder erzeugt eine neue
 */
LOCAL _BOOL setKey(shortElem *theVar)
{
	shortElem *newVar;

	if (!findShortKey(theVar->val, &newVar))
	{
		newVar = NEW(shortElem, "setKey");
		if (newVar == NULL)
			return FALSE;
		newVar->val  = theVar->val;
		newVar->len  = theVar->len;
		newVar->text = theVar->text;
		if (!Lists_AppendEntry(&shortList, newVar))
		{
			OFREE(newVar);
			return FALSE;
		}
	} else
	{
		SFREE(newVar->text);
		newVar->val  = theVar->val;
		newVar->len  = theVar->len;
		newVar->text = theVar->text;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID prep(CONST _UBYTE *in, _UBYTE *name, size_t namesize, _UBYTE *arg, size_t argsize)
{
	_UBYTE *p;
	size_t len;
	
	p = strChr(in, '=');
	if (p != NULL)
	{
		len = (CONST _UBYTE *)p - in + 1;
		if (len > namesize)
			len = namesize;
		strMcpy(name, len, in);
		strMcpy(arg, argsize, p + 1);
		strleft(arg);
	} else
	{
		strMcpy(name, namesize, in);
		*arg = '\0';
	}
	if (name[0] != '\0' && name[strLen(name)-1] == '*')
		name[strLen(name) - 1] = '\0';
	strleft(name);
	strdelRightSpace(name);
	/* YYYY Hier noch Semikolon-Abfrage aendern */
	p = strrChr(arg, ';');
	if (p != NULL)
		*p = '\0';
}

/*** ---------------------------------------------------------------------- ***/

LOCAL varType getNameAndType(CONST _UBYTE *line, _UWORD *val, _UBYTE *argStr, size_t argsize)
{
	_UBYTE name[128];
	
	prep(line, name, sizeof(name), argStr, argsize);
	if (name[0] == ';' || name[0] == '#' || name[0] == '\0')
		return comment;
	if (argStr[0] == '\0')
		return noVar;
	strleft(name);
	*val = name[0] * 255 + name[1];
	return string;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL getInfoStr(CONST _UBYTE *arg, _UBYTE *str, size_t strsize)
{
	_UBYTE numStr[10];
	_UBYTE *pos;
	
	if (*arg == '\0')
		return FALSE;
	strMcpy(str, strsize, arg);
	/* Sonderzeichen uebersetzen */
	pos = str;
	while (pos != NULL)
	{
		pos = strChr(pos, '\\');
		if (pos != NULL)
		{
			/* Feststellen, ob die naechsten drei Zahlen sind */
			strMcpy(numStr, 4, pos + 1);
			if (is_digit(numStr[0]) &&
				is_digit(numStr[1]) &&
				is_digit(numStr[2]))
			{
				*pos = (_UBYTE)StrToCard(numStr, NULL, NULL);
				strCpy(pos + 1, pos + 4);
				pos++;
			} else if (numStr[0] == '\\')
			{
				/* Ein \ entfernen */
				strCpy(pos, pos + 1);
				++pos;
			} else
			{
				++pos;
			}
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL processLine(CONST _UBYTE *fname, CONST _UBYTE *line)
{
	shortElem theVar;
	_BOOL ok;
	_UBYTE argStr[1024];
	_UBYTE valStr[1024];

	UNUSED(fname);
	switch (getNameAndType(line, &theVar.val, argStr, sizeof(argStr)))
	{
	case string:
		ok = getInfoStr(argStr, valStr, sizeof(valStr));
		if (ok)
		{
			theVar.len = (_WORD)strLen(valStr);
			theVar.text = STRDUP(valStr, "processLine");
			if (theVar.text == NULL)
				return FALSE;
		}
		break;
	case noVar:
	case comment:
		return TRUE;
	default:
		ok = FALSE;
		break;
	}
	/* Variable richtig eingelesen, also weitermachen */
	if (ok)
		return setKey(&theVar);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditFuncs_FreeShortkeyList(_VOID)
{
	shortElem *theVar;

	Lists_ResetList(&shortList);
	while ((theVar = Lists_NextEntry(&shortList)) != NULL)
	{
		Lists_RemoveEntry(&shortList, FUNK_NULL);
		SFREE(theVar->text);
		OFREE(theVar);
	}
	Lists_DeleteList(&shortList);
	shortListRead = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditFuncs_ReadShortkeyFile(CONST _UBYTE *fname)
{
	_UBYTE s[1024];
	TEXTFILENO tf;
	_BOOL cont;

	EditFuncs_FreeShortkeyList();
	if (mtTextfiles_OpenTextfile(fname, READ, 512, &tf) != GERR_OK)
		return;
	if (shortList.root == NULL)
		if (!Lists_CreateList(&shortList, "ReadShortKeyFile"))
			return;
	/* Jetzt Zeilenweise einlesen und dann die einzelnen Zeilen parsen */
	do
	{
		mtTextfiles_ReadLine(tf, s, sizeof(s));
		mtTextfiles_ReadLn(tf);
		cont = processLine(fname, s);
	} while (!mtTextfiles_EndofText(tf) && cont);
	mtTextfiles_CloseTextfile(&tf);
	shortListRead = TRUE;
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID SetEditShort(EDITOR *ed, shortElem *theKey)
{
	EDLINE cLine, cl2;
	EDCOLUMN cRow, cr2;
	_WORD scrLine;
	
	/* Jetzt Position sichern */
	cLine = ed->currLineNr;
	cRow = ed->currRow;
	if (EditFuncs_putLineInEditor(ed, theKey->text, theKey->len, FALSE))
	{
		cl2 = ed->currLineNr;
		cr2 = ed->currRow;
		/* Jetzt die Tilde suchen und den Cursor dort positionieren */
		if (EditFuncs_searchAndReplace(ed, "~", "", fromPos, sdir_backward, searchOne, 1, FALSE,
			FALSE, FALSE, FALSE, FALSE, NULL))
		{
			/* Erstmal nachsehen, ob das auch im neuen Bereich ist */
			if (ed->currLineNr > cLine || (ed->currLineNr == cLine && ed->currRow >= cRow))
			{
				/* Tilde entfernen */
				strCpy(ed->editLine.text + ed->currRow, ed->editLine.text + ed->currRow + 1);
				--ed->editLen;
				ed->editChanged = TRUE;
				EditBase_PutCurrLine(ed);
			} else
			{
				/* Blockmarkierung loeschen */
				ed->blocks[0].blockStart.line = -1;
				ed->blocks[0].blockEnd.line = -1;
				ed->block = FALSE;
				EditTools_SetCurrLine(ed, cl2);
				ed->prefRow = ed->currRow = cr2;
			}
			/* Blockmarkierung loeschen */
			ed->blocks[0].blockStart.line = -1;
			ed->blocks[0].blockEnd.line = -1;
			ed->block = FALSE;
		} else
		{
			EditTools_SetCurrLine(ed, cl2);
			ed->prefRow = ed->currRow = cr2;
		}
		/* Redraw ausloesen */
		if (cLine == cl2)
		{
			EditDraw_redrawLineArea(ed, cLine, cLine);
		} else
		{
			EditDraw_CenterCurrline(ed);
			scrLine = Max(0, (_WORD)(cLine - ed->StartLine));
			EditDraw_redrawLines(ed, scrLine);
		}
		EditGlobals_ShowCursor(ed);
	}
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _BOOL EditFuncs_InsertShortkey(EDITOR *ed)
{
	_UWORD kurzVal;
	shortElem *theKey;

	if (ed == NULL)
		return FALSE;
	/* Erstmal nachsehen, ob wir so ein Kuerzel haben */
	if (ed->currRow == 0 || ed->block)
		return FALSE;
	if (!shortListRead)
		return FALSE;
	if (ed->currRow > 1 && ed->editLine.text[ed->currRow-2] != ' ')
	{
		/* Erstmal nach Zwei-Zeichen-Kuerzel suchen */
		kurzVal = ed->editLine.text[ed->currRow-2] * 255 + ed->editLine.text[ed->currRow-1];
		if (findShortKey(kurzVal, &theKey))
		{
			EditGlobals_HideCursor(ed);
			/* gefunden, ersetzen */
			strCpy(ed->editLine.text + ed->currRow - 2, ed->editLine.text + ed->currRow);
			ed->editLen -= 2;
			ed->currRow -= 2;
			ed->prefRow = ed->currRow;
			ed->editChanged = TRUE;
			EditBase_PutCurrLine(ed);
			/* Jetzt einsetzen */
			SetEditShort(ed, theKey);
			EditGlobals_ForceCursor(ed);
			return TRUE;
		}
	}
	/* Jetzt nach Ein-Zeichen-Kuerzel suchen */
	kurzVal = ed->editLine.text[ed->currRow-1] * 255;
	if (findShortKey(kurzVal, &theKey))
	{
		EditGlobals_HideCursor(ed);
		/* gefunden, ersetzen */
		strCpy(ed->editLine.text + ed->currRow - 1, ed->editLine.text + ed->currRow);
		--ed->editLen;
		--ed->currRow;
		ed->prefRow = ed->currRow;
		ed->editChanged = TRUE;
		EditBase_PutCurrLine(ed);
		/* Jetzt einsetzen */
		SetEditShort(ed, theKey);
		EditGlobals_ForceCursor(ed);
		return TRUE;
	}
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Erweiterte Funktionen fuer Search & Replace */

/*DYJ 5.21b */
LOCAL _WORD AskForReplace(_VOID)
{
	_WORD moX, moY;
	_UWORD moButton;
	_WORD kReturn;
	_UWORD moKState;
	_UBYTE ch;
	_WORD bReturn;
	_UWORD event;
	_WORD message[8];
	_BOOL valid;
	_UBYTE	*answ;
	answ=Rsc_WrkString(JA_NEIN_ABBR);

	valid = FALSE;
	ch = '\0';
	for (;;)
	{
		/* 5 Sekunden Zeit, um eine Taste zu druecken */
		event = evnt_multi(MU_KEYBD | MU_TIMER,
			2, 0, 0,
			0, 0, 0, 0, 0,
			0, 0, 0, 0, 0,
			message,
			5000, 0,
			&moX, &moY, &moButton, &moKState, &kReturn, &bReturn);
		if (event & MU_TIMER)
		{
			valid = FALSE;
			event &= ~MU_TIMER;
		}
		if (event & MU_KEYBD)
		{
			ch = ucase(kReturn & 0xff);
			valid = strChr(answ,ch)!=NULL;
			/* Timer-Event loeschen! */
			EventTimer(1);
		}
		if (valid)
			break;
		return mtAlerts_Alert(1, Rsc_WrkString(askRepl));
	}
	if (ch == answ[0])
		return 1;
	if (ch == answ[1])
		return 2;
	return 3;
}

/*** ---------------------------------------------------------------------- ***/

/* Suchoperationen:
 *
 * Start:       aktuelle Position, Top, Ende
 * Richtung:    nach oben, nach unten
 * Anzahl:      erstes, ntes, letztes Vorkommen
 * Modus:       grossklein, nurWort
 */

GLOBAL _BOOL EditFuncs_searchAndReplace(
	EDITOR *ed,
	CONST _UBYTE *w,
	CONST _UBYTE *r,
	searchPos pos,
	searchDir direction,
	searchCount count,
	_WORD num,		/* wird nur bei count == nmal benutzt */
	_BOOL ignoreCase,
	_BOOL onlyWord,
	_BOOL replace,
	_BOOL ask,
	_BOOL again,
	CONST _UBYTE *bingEvent)
{
	_BOOL found;
	_BOOL f2;
	_WORD fc;
	_UBYTE rw[1024];
	_WORD select;
	
	MouseBusy();
	EditGlobals_HideCursor(ed);
	EditBase_PutCurrLine(ed);
	ed->findMode = 0;
	if (ignoreCase)
		ed->findMode |= GrossKlein;
	if (onlyWord)
		ed->findMode |= NurWort;
	if (replace && !ask && count != searchOne)
		ed->findMode |= silent;
	if (bingEvent == NULL)
		ed->findMode |= silent;
	if (pos == fromStart)
		EditFuncs_Home(ed, FALSE);
#if OLD
	if (pos == fromStart)
	{
		if (inBlock)
			EditTools_ToBlockmark(ed, FALSE);
		else
			EditFuncs_Home(ed, FALSE);
	} else if (pos == fromEnd)
	{
		if (inBlock)
			EditTools_ToBlockmark(ed, TRUE);
		else
			EditFuncs_Home(ed, TRUE);
	} else
	{
		if (!inBlock && BlockIsMarked(ed->wdw))
			again = TRUE;
	}
#endif
	found = FALSE;
	if (!EditTools_InitSearch(ed, direction, w, rw, replace))
	{
		EditGlobals_ShowCursor(ed);
		MouseArrow();
		return FALSE;
	}
	if (count == searchOne)
	{
		EditTools_SearchWord(ed, w, direction, &found, rw, sizeof(rw), again);
		if (!found && bingEvent != NULL)
		{
			CatGlobal_Bing(bingEvent);
		} else
		{
			if (replace)
			{
				if (ask)
				{
					EditDraw_CenterCurrline(ed);
					select = AskForReplace();
					if (select == 1)
					{
						EditTools_ReplaceWord(ed, rw, r);
					}
				} else
				{
					EditTools_ReplaceWord(ed, rw, r);
				}
			}
		}
	} else
	{
		found = FALSE;
		fc = 0;
		for (;;)
		{
			if (replace)
				EditTools_SearchWord(ed, w, direction, &f2, rw, sizeof(rw), FALSE);
			else
				EditTools_SearchWord(ed, w, direction, &f2, rw, sizeof(rw), fc != 0 || again);
			if (f2)
			{
				++fc;
				found = TRUE;
				if (replace)
				{
					if (ask)
					{
						EditDraw_CenterCurrline(ed);
						select = AskForReplace();
						if (select == 1)
						{
							EditTools_ReplaceWord(ed, rw, r);
						} else if (select == 3)
						{
							/* Cancel! */
							replace = FALSE;
							f2 = FALSE;
						} else
						{
							/* um naechstes zu finden */
							++ed->currRow;
							ed->prefRow = ed->currRow;
						}
					} else
					{
						EditTools_ReplaceWord(ed, rw, r);
					}
				}
			}
			if ((fc >= num && count == searchNum) || !f2)
				break;
			if (!replace)
			{
				++ed->currRow;
			}
			EditGlobals_FixCursorPos(ed);
		}
		if (((count == searchNum && !f2) || !found) && bingEvent != NULL)
			CatGlobal_Bing(bingEvent);
		if (found && !f2 && !replace)
			--ed->currRow;
	}
	if (found)
	{
		EditDraw_CenterCurrline(ed);
		if (replace)
			WdwManager_RedrawWdw(ed->wdw, &ed->editWork);
	}
	/* Suchmuster wieder freigeben */
	EditTools_EndSearch();
	EditGlobals_ShowCursor(ed);
	MouseArrow();
	return found;
}

/*** ---------------------------------------------------------------------- ***/

/* Fuegt die aktuelle und die folgende Zeile zu einer zusammen
 */
LOCAL _VOID JoinLines(EDITOR *ed, _BOOL appendSpace)
{
	_UBYTE *newText, *tx2;
	EDCOLUMN newLen, len, len2;
	EDLINE putLine;

	if (ed->currLineNr < ed->totalLineNr - 1)
	{
		if (EditBase_GetLineLength(ed, ed->currLineNr + 1, &len))
		{
			len2 = len;
			/* Neue Laenge berechnen */
			newLen = len + ed->editLen;
			++newLen;
			newText = MALLOC(newLen + 2, "joinLines: newText");
			if (newText == NULL)
				return;
			/* Zeile holen */
			EditBase_GetLine(ed, ed->currLineNr, newText, newLen + 4, &len);
			/* Jetzt CR/LFs am Ende der Zeile loeschen */
			--len;
			while (len >= 0 && (newText[len] == CR || newText[len] == LF))
			{
				newText[len] = '\0';
				--len;
			}
			/* Eventuell ein Leerzeichen anhaengen */
			++len;
			if (len > 0 && appendSpace && newText[len-1] != ' ' && newText[len-1] != '-')
			{
				newText[len++] = ' ';
				newText[len] = '\0';
			}
			/* Jetzt naechste Zeile holen */
			tx2 = MALLOC(len2 + 4, "JointLines: tx2");
			if (tx2 == NULL)
			{
				SFREE(newText);
				return;
			}
			EditBase_GetLine(ed, ed->currLineNr + 1, tx2, len2 + 4, &len2);
			if (ed->umbruch)
				strleft(tx2);
			strCat(newText, tx2);
			newLen = (EDCOLUMN)strLen(newText);
			putLine = ed->currLineNr;
			EditBase_DeleteLine(ed, ed->currLineNr + 1);
			EditBase_PutLine(ed, putLine, newText, newLen);
			ed->editChanged = FALSE;
			EditTools_UpdateCurrLine(ed, putLine);
			SFREE(newText);
			SFREE(tx2);
		}
	}
#if OLD
	if (ed->currLineNr < ed->totalLineNr - 1)
	{
		EditTools_SetCurrLine(ed, ed->currLineNr + 1);
		if (EditBase_GetLineLength(ed, ed->currLineNr - 1, &len)
		{
			newLen = len + ed->editLen;
			++newLen;
			newText = MALLOC(newLen + 2, "JoinsLines: newText");
			if (newText == NULL)
				return;
			EditBase_GetLine(ed, currLineNr - 1, newText, newLen + 2, &len);
			/* Jetzt CR/LFs am Ende der Zeile loeschen */
			--len;
			while (len >= 0 && (newText[len] == CR || newText[len] == LF))
			{
				newText[len] = '\0';
				--len;
			}
			++len;
			if (len > 0 && appendSpace && newText[len-1] != ' ' && newText[len-1] != '-')
			{
				newText[len++] = ' ';
				newText[len] = '\0';
			}
			if (ed->umbruch)
			{
				strleft(ed->editLine.text);
				ed->editLen = (EDCOLUMN)strLen(ed->editLine.text);
			}
			strCat(newText, ed->editLine.text);
			newLen = (EDCOLUMN)strLen(newText);
			putLine = ed->currLineNr - 1;
			EditBase_DeleteLine(ed, currLineNr);
			EditBase_PutLine(ed, putLine, newText, newLen);
			ed->editChanged = FALSE;
			EditTools_UpdateCurrLine(ed, putLine);
			SFREE(newText);
		}
	}
#endif
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CheckUmbruch(EDITOR *ed)
{
	EDCOLUMN l, l2;
	_UBYTE *text;
	EDCOLUMN edLen;

	if (ed->umbruch)
	{
		if (ed->currLineNr < ed->totalLineNr - 1 &&
			EditBase_GetLineLength(ed, ed->currLineNr + 1, &l))
		{
			/* Test, ob aktuelle Zeile mit CR/LF terminiert ist */
			if (ed->editLen > 0 && (ed->editLine.text[ed->editLen-1] == CR || (ed->editLen > 1 && ed->editLine.text[ed->editLen-2] == CR)))
				return;
			/* Testen, ob noch ein Wort in die aktuelle Zeile passt */
			EditBase_GetLineAdr(ed, ed->currLineNr + 1, &text, &l);
			l2 = 0;
			while (l2 < l && text[l2] == ' ')
				l2++;
			if (l2 == l)
				return;
			if (ed->realTabs)
				edLen = EditGlobals_RowToIndex(ed->editLine.text, ed->editLen, ed->TabSize);
			else
				edLen = ed->editLen;
			if (edLen + EditFuncs_findNext(text, l2, l, UmbruchSet) - l2 <= ed->rightMargin)
				EditFuncs_Format0(ed, FALSE);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Neu implementiert fuer neue Textstruktur 24.6.92
 */
/*DYJ 5.21b */
GLOBAL _VOID EditFuncs_Delete(EDITOR *ed, _BOOL ctrl, _BOOL saveLine)
{
	_BOOL join;
	EDCOLUMN delChars;
	EDCOLUMN l2;

	/* saveLine = saveLine && !ed->block; */
	UNUSED(saveLine);
	if (ed->block)
	{
		EditTools_SaveOp(ed, blockCut, FALSE);
		EditFuncs_cutEdit(ed, FALSE);
		return;
	}
	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	/* Test auf Zeilenende */
	join = ed->editLine.text[ed->currRow] == CR || ed->editLine.text[ed->currRow] == LF || ed->editLine.text[ed->currRow] == '\0';
	EditTools_ShowFileChanged(ed); /* Anzeigen, dass Text geaendert */
	if (join && ed->currLineNr < ed->totalLineNr - 1)
	{
		/* Zeile mit naechster vereinigen */
		EditBase_GetLineLength(ed, ed->currLineNr + 1, &l2);
		if (ed->editLen + l2 < ed->rightMargin)
		{
			/* ok, Folgezeile passt komplett rein */
			if (!EditBase_PutCurrLine(ed))
			{
				EditGlobals_ShowCursor(ed);
				EditGlobals_ShowMouse(FALSE);
				return; /* HALT */
			}
			ed->editChanged = FALSE;
			JoinLines(ed, FALSE);
			EditDraw_redrawLine(ed, FALSE, TRUE); /* CurrLine neu zeichnen */
			if (ed->currLineNr < ed->StartLine + ed->windLines)
			{
				/* Scrollen bis zu einer Zeile unter currLine */
				if (ed->currLineNr - ed->StartLine + 2 < ed->windLines)
					EditDraw_scrollRegion(ed, (_WORD)(ed->currLineNr - ed->StartLine) + 1, ed->windLines, -1);
				else
					EditDraw_drawLine(ed, ed->windLines - 1);
			}
			EditGlobals_SetDocument(ed);
			CheckUmbruch(ed);
		} else if (ed->umbruch)
		{
			/* Folgezeile passt nicht komplett */
			EditFuncs_Format0(ed, TRUE);
		} else
		{
			/* auch kein Umbruch, dann meckern */
			EditGlobals_SaveMouse();
			EditGlobals_ShowMouse(TRUE);
			mtAlerts_Alert(1, Rsc_WrkString(AL_NO_FIT));
			EditGlobals_RestoreMouse();
		}
	} else if (!join)
	{
		/* Loeschen in einer Zeile */
		if (ctrl)
		{
			/* Anzahl der zu loeschenden Zeichen feststellen */
			l2 = ed->currRow;
			ed->currRow = EditFuncs_findNext(ed->editLine.text, ed->currRow, ed->editLen, TrennSet);
			EditGlobals_FixCursorPos(ed);
			delChars = ed->currRow;
			ed->prefRow = ed->currRow = l2;
			delChars -= ed->currRow;
			if (delChars <= 0)
				delChars = 1;
			if (delChars + ed->currRow > ed->editLen)
				delChars = ed->editLen - ed->currRow /* HALT */;
		} else
		{
			delChars = 1;
		}
		strCpy(ed->editLine.text + ed->currRow, ed->editLine.text + ed->currRow + delChars);
		ed->editLen -= delChars;
		ed->editChanged = TRUE;
		EditDraw_redrawLine(ed, FALSE, ctrl);
		CheckUmbruch(ed);
	}
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/* CursorLeft & dann Delete */
GLOBAL _VOID EditFuncs_Backspace(EDITOR *ed, _BOOL ctrl)
{
	_BOOL wasBlock;
	
	wasBlock = ed->block;
	if (ed->block)
	{
		EditTools_SaveOp(ed, blockCut, FALSE);
		EditFuncs_cutEdit(ed, FALSE);
		return;
	}
	if (ed->currLineNr == 0 && ed->currRow == 0)
		return;
	if (ctrl)
		EditFuncs_CursorLeft(ed, K_CTRL);
	else
		EditFuncs_CursorLeft(ed, 0);
	EditDraw_CenterCurrline(ed);
	EditFuncs_Delete(ed, ctrl, !wasBlock);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditFuncs_DupLine(EDITOR *ed)
{
	EDLINE clLineNr;

	if (ed->block)
		return;
	EditGlobals_HideCursor(ed);
	EditGlobals_HideMouse();
	EditTools_ShowFileChanged(ed);
	EditBase_PutCurrLine(ed);
	if (EditBase_InsertLine(ed, ed->currLineNr, ed->editLine.text, FALSE, ed->editLen))
	{
		if (ed->currLineNr < ed->StartLine + ed->windLines)
		{
			/* Scrollen bis zu einer Zeile unter currLine */
			if (ed->currLineNr - ed->StartLine + 2 < ed->windLines)
				EditDraw_scrollRegion(ed, (_WORD)(ed->currLineNr - ed->StartLine) + 1, ed->windLines, 1);
		}
		EditGlobals_SetDocument(ed);
		clLineNr = ed->currLineNr;
		if (ed->currLineNr >= ed->StartLine + ed->windLines)
			WdwManager_ScrollDown(ed->wdw, 1);
		EditTools_SetCurrLine(ed, clLineNr);
		/* und zeichnen */
		EditDraw_drawLine(ed, (_WORD)(ed->currLineNr - ed->StartLine) + 1);
	}
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditFuncs_Tab(EDITOR *ed, _BOOL onlyIndent)
{
	_UBYTE *prevText;
	EDCOLUMN i, j, spacesToInsert, l;

	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	EditDraw_CenterCurrline(ed);
	if (onlyIndent)
	{
		/* Alle Whitespaces in Zeile darueber ueberspringen und bis dahin einruecken
		 */
		if (ed->currLineNr > 0)
		{
			EditBase_GetLineAdr(ed, ed->currLineNr - 1, &prevText, &l);
			/* Jetzt nach erstem Buchstaben suchen */
			i = 0;
			while (i < l && (prevText[i] == TAB || prevText[i] == ' '))
				++i;
			if (i > 0)
			{
				for (j = 0; j < i; j++)
					EditFuncs_Insert(ed, prevText[j]);
			}
		}
	} else
	{
		/* Einfach entweder ein TAB einfuegen oder mit Spaces auffuellen bis zur naechsten Marke
		 */
		if (ed->realTabs)
		{
			EditFuncs_Insert(ed, TAB);
		} else
		{
			i = ((ed->currRow / ed->TabSize) + 1) * ed->TabSize;
			spacesToInsert = i - ed->currRow;
			if (spacesToInsert < 0)
				spacesToInsert = 0;
			for (i = 0; i < spacesToInsert; i++)
				EditFuncs_Insert(ed, ' ');
		}
	}
#if OLD
	spacesToInsert = -1;
	if (ed->currLineNr > 0)
	{
		EditBase_GetLineAdr(ed, ed->currLineNr - 1, &prevText, &l);
		/* naechsten Wortanfang in Prevline suchen */
		i = ed->currRow;
		if (ed->realTabs)
			i = EditGlobals_RowToIndex(ed->editLine.text, ed->currRow, ed->TabSize);
		if (i < l)
		{
			/* ok, koennte noch ein Wort kommen */
			if (!onlyIndent)
			{
				while (i < l && prevText[i] != ' ')
					++i;
			}
			while (i < l && prevText[i] == ' ')
				++i;
			if (ed->realTabs)
				spacesToInsert = i - EditGlobals_RowToIndex(ed->editLine.text, ed->currRow, ed->TabSize) - 1;
			else
				spacesToInsert = i - ed->currRow - 1;
		}
	}
	if (ed->realTabs)
	{
		if (spacesToInsert <= 0 && !onlyIndent)
		{
			tabsToInsert = 1;
			spacesToInsert = 0;
		} else
		{
			++spacesToInsert;
			tabsToInsert = spacesToInsert / ed->TabSize;
			spacesToInsert -= tabsToInsert * ed->TabSize;
		}
		for (i = 0; i < tabsToInsert; i++)
			EditFuncs_Insert(ed, TAB);
		for (i = 0; i < spacesToInsert; i++)
			EditFuncs_Insert(ed, ' ');
	} else
	{
		if (spacesToInsert <= 0 && !onlyIndent)
		{
			i = ((ed->currRow / ed->TabSize) + 1) * ed->TabSize;
			spacesToInsert = i - ed->currRow;
		}
		for (i = 0; i <= spacesToInsert; i++)
			EditFuncs_Insert(ed, ' ');
	}
#endif
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditFuncs_InsertCR(EDITOR *ed)
{
	EDLINE clLineNr;

	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	EditTools_ShowFileChanged(ed);
	if (!EditBase_PutCurrLine(ed))
	{
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
		return;
	}
	clLineNr = ed->currLineNr;
	if (!EditBase_InsertCr(ed, ed->currLineNr, ed->currRow))
	{
		EditTools_SetCurrLine(ed, clLineNr);
		/* und zeichnen */
		EditDraw_drawLine(ed, (_WORD)(ed->currLineNr - ed->StartLine));
		EditDraw_CenterCurrline(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
		return;
	}
	/* Zeilen unter currLine kopieren */
	if (ed->currLineNr < ed->StartLine + ed->windLines)
	{
		/* Scrollen bis zu einer Zeile unter currLine */
		if (ed->currLineNr - ed->StartLine + 2 < ed->windLines)
			EditDraw_scrollRegion(ed, (_WORD)(ed->currLineNr - ed->StartLine) + 1, ed->windLines, 1);
	}
	EditGlobals_SetDocument(ed);
	EditTools_SetCurrLine(ed, ed->currLineNr + 1);
	EditDraw_drawLine(ed, (_WORD)(ed->currLineNr - ed->StartLine - 1));
	clLineNr = ed->currLineNr;
	if (ed->currLineNr >= ed->StartLine + ed->windLines)
		WdwManager_ScrollDown(ed->wdw, 1);
	EditTools_SetCurrLine(ed, clLineNr);
	/* und zeichnen */
	EditDraw_drawLine(ed, (_WORD)(ed->currLineNr - ed->StartLine));
	EditDraw_CenterCurrline(ed);
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#define isEnde(ch) ((ch) == CR || (ch) == '.' || (ch) == '!' || (ch) == ':' || (ch) == ')')

typedef struct {
	_BOOL forcedJoin;
	aLineDesc tmp;
	_BOOL tmpValid;
	_BOOL absatzEnde;
	EDCOLUMN row;
	EDCOLUMN cRow;
	EDCOLUMN newRow;
	EDLINE clNr;
} formatInfo;

LOCAL _VOID IsAbsatzEnde(EDITOR *ed, EDCOLUMN row, _BOOL *ende)
{
	_UBYTE *tBuf;
	EDCOLUMN z;
	
	if (row > ed->editLen)
		row = ed->editLen;
	if (row == 0)
	{
		*ende = TRUE;
		return;
	}
	*ende |= isEnde(ed->editLine.text[row-1]) || (row > 1 && isEnde(ed->editLine.text[row-2])) ||
		(ed->currLineNr + 1 >= ed->totalLineNr);
	if (!*ende)
	{
		EditBase_GetLineAdr(ed, ed->currLineNr + 1, &tBuf, &z);
		*ende = TRUE;
		--z;
		while (z >= 0 && (tBuf[z] == LF || tBuf[z] == CR))
			--z;
		while (z >= 0 && *ende)
		{
			*ende = tBuf[z] == ' ';
			--z;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID JoinSomeLines(EDITOR *ed, formatInfo *info)
{
	info->row = ed->editLen;
	info->absatzEnde = FALSE;
	IsAbsatzEnde(ed, info->row, &info->absatzEnde);
	/* Solange Zeilen zusammensetzen, bis rightMargin (Zeilenlaenge) ueberschritten ist */
	while ((!info->absatzEnde && info->newRow < ed->rightMargin) || info->forcedJoin)
	{
		/* Check auf row > 0 nicht noetig, da dann schon Absatzende ist! */
		if (info->row > 0 && !info->forcedJoin)
		{
			JoinLines(ed, TRUE);
		} else
		{
			JoinLines(ed, FALSE);
			info->forcedJoin = FALSE;
		}
		info->row = ed->editLen;
		info->absatzEnde=FALSE;
		IsAbsatzEnde(ed, info->row, &info->absatzEnde);
		while (info->row > 0 && (ed->editLine.text[info->row-1] == CR || ed->editLine.text[info->row-1] == LF))
			--info->row;
		info->newRow = info->row;
	}
	while (info->row > 0 && (ed->editLine.text[info->row-1] == CR || ed->editLine.text[info->row-1] == LF))
		--info->row;
	info->newRow = info->row;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID SplitLine(EDITOR *ed, formatInfo *info)
{
	_UBYTE termCh;
	EDCOLUMN len;
	
	/* Alles hinter Umbruchstelle in tmp kopieren */
	len = info->row - info->newRow;
	EditGlobals_AllocNew(&info->tmp, len + 2);
	MemCpy(info->tmp.text, ed->editLine.text + info->newRow, len);
	info->tmp.text[len] = '\0';
	info->tmpValid = TRUE;
	
	/* Aktuelle Zeile terminieren */
	strCpy(ed->editLine.text + info->newRow, ed->editLine.text + info->newRow + len);
	if (info->newRow >= ed->editLen)
		info->newRow = ed->editLen - 2;
	ed->editLine.text[info->newRow] = LF;
	ed->editLine.text[info->newRow + 1] = '\0';
	ed->editLen = info->newRow + 1;
	
	ed->editChanged = TRUE;
	EditBase_PutCurrLine(ed);
	
	/* Jetzt tmp-String bearbeiten:
	 * Erst mal CR/LF am Zeilenende loeschen.
	 * Wenn kein Space und kein '-' am Ende ist, dann ein Space anfuegen
	 */
	--len;
	while (len >= 0 && (info->tmp.text[len] == CR || info->tmp.text[len] == LF))
	{
		info->tmp.text[len] = '\0';
		--len;
	}
	if (len >= 0)
	{
		termCh = info->tmp.text[len];
		if (termCh != ' ' && termCh != '-' && !isEnde(termCh) && !info->absatzEnde)
		{
			++len;
			info->tmp.text[len++] = ' ';
			info->tmp.text[len] = '\0';
		}
	}
	if (ed->currLineNr == info->clNr && info->cRow > info->newRow)
	{
		++info->clNr; /* Saveposition inkrementieren */
		/* neue aktuelle Spalte bestimmen */
		info->cRow = info->cRow - info->newRow;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL maySplit(EDITOR *ed, CONST _UBYTE *text, EDCOLUMN row, EDCOLUMN idx)
{
	if (row >= ed->rightMargin || (ed->realTabs && idx >= ed->rightMargin))
		return TRUE;
	return (text[row] != '\0' && text[row] != CR && text[row] != LF) &&  /* Zeilenende, nicht splitten! */
		( ( row > 0 &&
		  ( (text[row-1] == '(' || text[row-1] == '[' || text[row-1] == '{') ||
		  ( (text[row-1] == '"' || text[row-1] == '\'') && row > 1 && text[row-2] == ' ') ) ) ||
		 ( row > 0 && (text[row] == ')' || text[row] == ']' || text[row] == '}') ) );
}

/*** ---------------------------------------------------------------------- ***/

/* Formatiert einen Absatz neu. Dabei werden zu lange Zeilen
 * auf die maximale Zeilenlaenge verkuerzt, Autoindent wird beachtet,
 * und Leerzeichen am Zeilenanfang werden entfernt bzw. uebersprungen
 */
 

GLOBAL _VOID EditFuncs_Format0(EDITOR *ed, _BOOL forcedJoin)
{
	_UBYTE saveCh;
	_UBYTE saveCh2;
	EDCOLUMN rowAdd, z, l, idx;
	EDLINE rdrStart, aEnde, lastLine;
	formatInfo info;
	
	info.tmp.length = 16;
	info.tmp.text = MALLOC(info.tmp.length, "Format0: tmp");
	if (info.tmp.text == NULL)
		return;
	info.tmp.text[0] = '\0';
	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	/* Position merken */
	info.forcedJoin = forcedJoin;
	info.clNr = ed->currLineNr;
	info.absatzEnde = FALSE;
	info.cRow = ed->currRow;
	rdrStart = info.clNr;
	idx = 0;
	/*
	 * Dafuer merke ich mir die zeilennummer des Absatzanfangs und
	 * des Absatzendes
	 */
	while (!info.absatzEnde)
	{
		IsAbsatzEnde(ed, ed->editLen, &info.absatzEnde);
		if (!info.absatzEnde)
			EditTools_SetCurrLine(ed, ed->currLineNr + 1);
	}
	aEnde = ed->currLineNr;
	/* Wenn Zeilen wirklich vereinigt werden sollen,
	 * dann nochmal checken
	 */
	if (info.forcedJoin && aEnde == info.clNr)
	{
		EditTools_SetCurrLine(ed, ed->currLineNr + 1);
		info.absatzEnde = FALSE;
		while (!info.absatzEnde)
		{
			IsAbsatzEnde(ed, ed->editLen, &info.absatzEnde);
			if (!info.absatzEnde)
				EditTools_SetCurrLine(ed, ed->currLineNr + 1);
		}
		aEnde = ed->currLineNr;
	}
	info.absatzEnde = FALSE;
	EditTools_SetCurrLine(ed, info.clNr);             /* Position restaurieren */
	info.tmpValid = FALSE;
	while (!info.absatzEnde /* (length >= ed->rightMargin) && !insertNew */ || info.forcedJoin)
	{
		JoinSomeLines(ed, &info);
		
		/* Wenn jetzt die Laenge der aktuellen Zeile < rightMargin ist,
		 * dann war ein anderer Grund fuer das Absatzende da und es muss
		 * nichts mehr gemacht werden, auch kein Einfuegen von irgendwelchen
		 * temporaeren Teilen
		 */
		/* In row steht die Laenge der Zeile */
		if (ed->realTabs)
			idx = EditGlobals_RowToIndex(ed->editLine.text, info.newRow, ed->TabSize);
		
		if (info.newRow >= ed->rightMargin || (ed->realTabs && idx >= ed->rightMargin))
		{
			/* Jetzt Stelle zum Splitten der aktuellen Zeile finden */
			while (maySplit(ed, ed->editLine.text, info.newRow, idx))
			{
				info.newRow = EditFuncs_findLast(ed->editLine.text, info.newRow, UmbruchSet);
				if (ed->realTabs)
					idx = EditGlobals_RowToIndex(ed->editLine.text, info.newRow, ed->TabSize);
			}
			/* Wenn Zeile nur aus einem Wort besteht,
			 * dann harten Umbruch einfuegen
			 */
			if (info.newRow == 0)
			{
				info.newRow = Min((EDCOLUMN)strLen(ed->editLine.text), ed->rightMargin - 2);
				if (info.newRow >= info.row)
					info.newRow = info.row - 2;
			}
			
			/* Aktuelle Zeile wird mit richtigem Umbruch terminiert
			 * und der Rest wird in tmp kopiert
			 */
			SplitLine(ed, &info);
			
			/* Wenn wir am Absatzende sind, dann muss tmp als neue
			 * Zeile eingefuegt werden. Andernfalls wird es vor die naechste
			 * Zeile gehaengt und dann erneut die Schleife durchlaufen
			 */
			if (info.absatzEnde)
			{
				/* insertNew = TRUE; */
			} else
			{
				/* Nicht Absatzende, also tmp in naechste Zeile kopieren */
				ed->currRow = 0;
				/* Indent feststellen von aktueller Zeile */
				if (ed->autoIndent)
				{
					while (ed->currRow <= ed->editLen && ed->editLine.text[ed->currRow] == ' ')
						++ed->currRow;
				}
				
				/* Naechste Zeile zu aktueller Zeile machen */
				z = ed->currRow;
				EditTools_SetCurrLine(ed, ed->currLineNr + 1);
				ed->prefRow = ed->currRow = z;
				
				/* Cursorspalte evtl. korrigieren */
				if (info.clNr == ed->currLineNr)
					info.cRow += ed->currRow;
				
				l = ed->editLen; /* Laenge aktueller Zeile */
				z = (EDCOLUMN)strLen(info.tmp.text); /* tmp ist 0C terminiert ohne CR/LF */
				/* Jetzt genaue Einfuegeposition bestimmen
				 * currRow zeigt Indent von vorheriger Zeile an, newRow
				 * hiernach Indent der aktuellen Zeile
				 */
				
				info.newRow = 0;
				if (ed->autoIndent)
				{
					while (ed->editLine.text[info.newRow] == ' ')
						++info.newRow;
				}
				
				/* Wenn indent der oberen Zeile groesser dem der unteren ist,
				 * dann muss noch was hinzugefuegt werden
				 */
				if (ed->currRow > info.newRow)
					rowAdd = ed->currRow - info.newRow;
				else
					rowAdd = 0;
				
				/* Jetzt feststellen, wie lang die Zeile danach wird und
				 * dann evtl. neu allozieren
				 */
				EditGlobals_AllocNew(&ed->editLine, l + z + rowAdd);
				
				if (ed->currRow > info.newRow)
				{
					/* In vorheriger Zeile mehr eingerueckt: Zeile einruecken */
					EditGlobals_AllocNew(&info.tmp, z + rowAdd);
					MemMove(info.tmp.text + rowAdd, info.tmp.text, z + 1);
					MemSet(info.tmp.text, ' ', rowAdd);
					/* tmp einfuegen */
					MemMove(ed->editLine.text + info.newRow + z + rowAdd, ed->editLine.text + info.newRow, l - info.newRow + 1);
					MemCpy(ed->editLine.text + info.newRow, info.tmp.text, z + rowAdd);
					ed->editLen = z + rowAdd + l;
				} else if (info.newRow > ed->currRow)
				{
					/* In vorheriger Zeile weniger Spaces am Anfang: Spaces loeschen */
					rowAdd = info.newRow - ed->currRow;
					strCpy(ed->editLine.text, ed->editLine.text + rowAdd);
					l -= rowAdd;
					/* tmp einfuegen */
					MemMove(ed->editLine.text + ed->currRow + z, ed->editLine.text + ed->currRow, l - ed->currRow + 1);
					MemCpy(ed->editLine.text + ed->currRow, info.tmp.text, z);
					ed->editLen = l + z;
				} else
				{
					/* dritter Fall und muss jedesmal gemacht werden: tmp einfuegen */
					MemMove(ed->editLine.text + ed->currRow + z, ed->editLine.text + ed->currRow, l - ed->currRow + 1);
					MemCpy(ed->editLine.text + ed->currRow, info.tmp.text, z);
					ed->editLen = l + z;
				}
				
				ed->editChanged = TRUE;
				EditBase_PutCurrLine(ed);
				info.tmpValid = FALSE;
			}
		}
		/* length = ed->editLen; */
	}
	
	if (info.tmpValid)
	{
		l = (EDCOLUMN)strLen(info.tmp.text);
		while (l > 0)
		{
			z = 0;
			if (ed->autoIndent)
			{
				saveCh = ed->editLine.text[z];
				/* Genuegend Spaces einfuegen */
				while (saveCh == ' ')
				{
					++z;
					saveCh = ed->editLine.text[z];
				}
				if (saveCh != '\0' && saveCh != CR && saveCh != LF)
				{
					/* Autoindent nur, wenn die Zeile nicht nur aus Space besteht */
					EditGlobals_AllocNew(&info.tmp, l + z);
					MemMove(info.tmp.text + z, info.tmp.text, l + 1);
					MemSet(info.tmp.text, ' ', z);
					l += z;
				} else
				{
					z = 0;
				}
			}
			/* Jetzt neue Zeile einfuegen */
			info.newRow = Min(ed->rightMargin + 5, l);
			if (ed->realTabs)
				idx = EditGlobals_RowToIndex(info.tmp.text, info.newRow, ed->TabSize);
			while (maySplit(ed, info.tmp.text, info.newRow, idx))
			{
				info.newRow = EditFuncs_findLast(info.tmp.text, info.newRow, UmbruchSet);
				if (ed->realTabs)
					idx = EditGlobals_RowToIndex(info.tmp.text, info.newRow, ed->TabSize);
			}
			if (info.newRow == 0)
			{
				/* Endlosschleife verhindern, harter Umbruch! */
				info.newRow = Min(l, ed->rightMargin - 1);
			}
			if (info.newRow < l)
			{
				saveCh = info.tmp.text[info.newRow];
				saveCh2 = info.tmp.text[info.newRow+1];
				info.tmp.text[info.newRow] = LF;
				info.tmp.text[info.newRow+1] = '\0';
				if (!EditBase_InsertLine(ed, ed->currLineNr, info.tmp.text, FALSE, info.newRow + 1))
				{
					/* HALT */;
				}
				EditTools_SetCurrLine(ed, ed->currLineNr + 1);
				/* ++linesAdded; */
				info.tmp.text[info.newRow] = saveCh;
				info.tmp.text[info.newRow+1] = saveCh2;
			} else
			{
				EditGlobals_AllocNew(&info.tmp, info.newRow + 2);
				info.tmp.text[info.newRow++] = CR;
				info.tmp.text[info.newRow++] = LF;
				info.tmp.text[info.newRow] = '\0';
				l = info.newRow;
				if (!EditBase_InsertLine(ed, ed->currLineNr, info.tmp.text, FALSE, l))
				{
					/* HALT */;
				}
				EditTools_SetCurrLine(ed, ed->currLineNr + 1);
			}
			strCpy(info.tmp.text, info.tmp.text + info.newRow);
			l -= info.newRow;
			ed->currRow = z;
			if (ed->currLineNr == info.clNr)
			{
				info.cRow += ed->currRow;
			}
		}
	}
	lastLine = ed->currLineNr;
	SFREE(info.tmp.text);
	EditTools_SetCurrLine(ed, info.clNr);
	EditGlobals_SetDocument(ed);
	ed->prefRow = ed->currRow = info.cRow;
	/* Jetzt noch redraw von Absatzstart bis Absatzende */
	if (lastLine != aEnde)
	{
		EditDraw_redrawLines(ed, (_WORD)(info.clNr - ed->StartLine) - 1);
	} else
	{
		if (lastLine == rdrStart)
			++lastLine;
		EditDraw_redrawLineArea(ed, rdrStart, lastLine);
	}
	EditTools_ShowFileChanged(ed);
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichen ohne Umbruch einfuegen */
LOCAL _VOID InsertOne(EDITOR *ed, EDCOLUMN l, _UBYTE ch)
{
	EDCOLUMN newLength;
	_UBYTE lastCh;
	_UBYTE *tBuf;
	
	if (ed->editLine.length - 2 <= l || ed->currRow >= ed->editLine.length - 2)
	{
		/* Shit, Zeile zu kurz, umspeichern */
		newLength = ((ed->editLine.length / 16) + 1) * 16;
		if (newLength <= ed->editLine.length)
			newLength += 16;
		tBuf = SYSMALLOC(newLength, MEM_NOERR, "InsertOne");
		if (tBuf == NULL)
		{
			EditBase_CompactBuffer(ed);
			tBuf = MALLOC(newLength, "InsertOne");
			if (tBuf == NULL)
			{
				/* kein Speicher mehr frei */
				/* Zeichen einfach nicht einfuegen */
				return;
			}
		}
		strCpy(tBuf, ed->editLine.text);
		SFREE(ed->editLine.text);
		ed->editLine.text = tBuf;
		ed->editLine.length = newLength;
	}
	lastCh = ed->editLine.text[ed->currRow];
	if (ed->insertMode || lastCh < 32)
	{
		++ed->editLen;
		MemMove(ed->editLine.text + ed->currRow + 1, ed->editLine.text + ed->currRow, ed->editLen - ed->currRow);
	}
	ed->editLine.text[ed->currRow] = ch;
	ed->editChanged = TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Die wichtigste Prozedur ueberhaupt:
 * ein Zeichen an der Cursorposition einfuegen.
 * Neu geschrieben am 11.1.92 fuer Unterstuetzung von
 * Fliesstextmodus und geringere Fehleranfaelligkeit.
 */

GLOBAL _VOID EditFuncs_Insert(EDITOR *ed, _UBYTE ch)
{
	EDCOLUMN idx, z, l;

	/* Cursor und Mouse ausschalten */
	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	/* Aktuelle Zeile sichern fuer Undo */
	/* EditTools_SaveLine(ed->currLine, ed->currLineNr); */
	/* Bei markiertem Block diesen ersetzen und Operation fuer Undo merken */
	if (ed->block)
	{
		EditTools_SaveOp(ed, blockToChar, FALSE);
		EditFuncs_cutEdit(ed, FALSE);
	}
	/* Zeilenlaenge feststellen */
	l = ed->editLen;
	/* Anzeigen, das File geaendert */
	EditTools_ShowFileChanged(ed);
	/* Wenn Zeilenlaenge nicht zu gross ist, dann einfach einfuegen */
	if (ed->realTabs)
		idx = EditGlobals_RowToIndex(ed->editLine.text, l, ed->TabSize);
	else
		idx = l;
	if (idx < ed->rightMargin || ch == CR || ch == LF)
	{
		/* Zeichen ohne Umbruch einfuegen */
		InsertOne(ed, l, ch);
		if (ch != CR && ch != LF)
			EditDraw_redrawLine(ed, TRUE, FALSE);
		++ed->currRow;
	} else if (ed->umbruch)
	{
		/* Hier Absatzumbruch vornehmen! */
		/* Jetzt aktuelle Zeile umbrechen */
		if (ed->currRow >= l && ch == ' ' && ed->editLine.text[l-1] == ' ')
		{
			/* Am Ende der Zeile Space einfuegen ==> Sprung in naechste Zeile
			 * und dort Space einfuegen
			 */
			if (ed->currLineNr - 1 < ed->totalLineNr)
			{
				if (ed->autoIndent)
				{
					/* findFirstWord in CurrLine */
					ed->currRow = 0;
					while (ed->editLine.text[ed->currRow] == ' ')
						++ed->currRow;
				} else
				{
					ed->currRow = 0;
				}
				z = ed->currRow;
				/* naechste Zeile zur aktuellen machen */
				EditTools_SetCurrLine(ed, ed->currLineNr + 1);
				ed->currRow = z;
			} else
			{
				/* Zeile einfuegen mit Autoindent-Beachtung */
				EditFuncs_Insert(ed, LF);
				EditFuncs_InsertCR(ed);
				ed->currRow = 0;
				if (ed->autoIndent)
					EditFuncs_Tab(ed, TRUE);
			}
			EditFuncs_Insert(ed, ch);
		} else
		{
			/* Zeichen einfach einfuegen und danach Absatzformatierung ausloesen */
			InsertOne(ed, l, ch);
			++ed->currRow;
			EditFuncs_Format0(ed, FALSE);
		}
	} else
	{
		/* Zeile zu lang und kein Umbruch! */
		/* Ein Bing ausgeben */
		CatGlobal_Bing(cBingLineTooLong);
	}
	ed->prefRow = ed->currRow;
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Clipboard-Funktionen */


GLOBAL _BOOL EditFuncs_cutBlock(EDITOR *ed, _BOOL toClip, _BOOL redraw)
{
	EDLINE cLine;
	EDLINE l;
	_WORD rdrStart;
	EDLINE rdrLines;
	_BOOL ende, addCr, oneLine;
	EDLINE bStart, blEnd, blStart;
	EDCOLUMN len;
	_UBYTE *text;

	if (!EditBase_PutCurrLine(ed))
		return FALSE;
	addCr = FALSE;
	bStart = ed->blocks[0].blockStart.line;
	if (ed->blocks[0].blockStart.line >= 0 &&
		ed->blocks[0].blockEnd.line >= 0)
	{
		/* Jetzt erstmal Blockmarkierung auf gueltige Bereiche beschraenken */
		if (ed->blocks[0].blockEnd.line > ed->totalLineNr - 1)
		{
			EditTools_SetCurrLine(ed, ed->totalLineNr - 1);
			ed->currRow = ed->editLen;
			ed->blocks[0].blockEnd.row = ed->currRow;
			ed->blocks[0].blockEnd.line = ed->currLineNr;
		}
		
		/* Redraw-Bereich feststellen */
		if (ed->blocks[0].blockStart.line >= ed->StartLine &&
			ed->blocks[0].blockStart.line < ed->StartLine + ed->windLines)
		{
			rdrStart = (_WORD)(ed->blocks[0].blockStart.line - ed->StartLine);
		} else
		{
			rdrStart = -1;
		}
		/* Block auf Clipboard schieben */
		if (toClip)
			if (!Editor_EditCopy(ed->wdw))
				return FALSE;
		
		EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
		ed->prefRow = ed->currRow = ed->blocks[0].blockStart.row;
		rdrLines = ed->blocks[0].blockEnd.line - ed->blocks[0].blockStart.line;
		
		/* Jetzt Block loeschen */
		if (ed->blocks[0].blockStart.line == ed->blocks[0].blockEnd.line ||
			(ed->blocks[0].blockEnd.line == ed->blocks[0].blockStart.line + 1 &&
			 ed->blocks[0].blockEnd.row == 0 &&
			 ed->blocks[0].blockStart.row == 0))
		{
			/* Flag fuer Redraw setzen! */
			oneLine = TRUE;
			/* Nur Block in einer Zeile */
			if ((ed->blocks[0].blockStart.row == 0 &&
				 ed->blocks[0].blockEnd.row == ed->editLen) ||
				(ed->blocks[0].blockStart.row == 0 &&
				 ed->blocks[0].blockEnd.row == 0 &&
				 ed->blocks[0].blockStart.line == ed->blocks[0].blockEnd.line - 1))
			{
				/* Wenn die geloeschte Zeile ein Absatzende enthaelt und die
				 * darueber nicht, dann muss an die darueber ein Absatzende gesetzt
				 * werden
				 */
				ende = FALSE;
				IsAbsatzEnde(ed, ed->editLen, &ende);
				if (ende && ed->blocks[0].blockStart.line > 0)
				{
					/* Jetzt die Zeile vorher kontrollieren */
					EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line - 1);
					ende = FALSE;
					IsAbsatzEnde(ed, ed->editLen, &ende);
					if (!ende)
						addCr = TRUE;
					EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
				}
				/* Blockmarkierung loeschen */
				EditTools_ClearBlock(ed);
				/* Genau eine Zeile markiert */
				EditTools_DeleteLine(ed, TRUE);
				/* Redraw wird schon bei DeleteLine gemacht */
				redraw = FALSE;
			} else
			{
				len = ed->blocks[0].blockEnd.row - ed->blocks[0].blockStart.row;
				text = ed->editLine.text + ed->blocks[0].blockStart.row;
				strCpy(text, text + len);
				ed->editChanged = TRUE;
				ed->editLen -= len;
			}
		} else
		{
			oneLine = FALSE;
			/* mehrzeiliger Block */
			l = ed->blocks[0].blockStart.line;
			if (ed->blocks[0].blockEnd.line - ed->blocks[0].blockStart.line - 2 > 0)
			{
				blEnd = ed->blocks[0].blockEnd.line;
				blStart = ed->blocks[0].blockStart.line;
				/* Erstmal Ende und Anfang bearbeiten */
				EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
				if (ed->blocks[0].blockStart.row > 0)
				{
					/* Nur Teilweise loeschen */
					len = ed->editLen - ed->blocks[0].blockStart.row;
					text = ed->editLine.text + ed->blocks[0].blockStart.row;
					strCpy(text, text + len);
					ed->editLen -= len;
					ed->editChanged = TRUE;
					EditBase_PutCurrLine(ed);
				} else
				{
					--blStart;
				}
				EditTools_SetCurrLine(ed, blEnd);
				if (ed->blocks[0].blockEnd.row < ed->editLine.length)
				{
					/* ebenfalls nur teilweise loeschen */
					strCpy(ed->editLine.text, ed->editLine.text + ed->blocks[0].blockEnd.row);
					ed->editLen -= ed->blocks[0].blockEnd.row;
					ed->editChanged = TRUE;
					EditBase_PutCurrLine(ed);
				} else
				{
					ende = FALSE;
					IsAbsatzEnde(ed, ed->editLen, &ende);
					if (ende && ed->blocks[0].blockStart.line > 0)
					{
						/* Jetzt die Zeile vorher kontrollieren */
						EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line - 1);
						ende = FALSE;
						IsAbsatzEnde(ed, ed->editLen, &ende);
						if (!ende)
							addCr = TRUE;
					}
					++blEnd;
				}
				/* Und jetzt die grobe Harke */
				EditBase_DeleteSomeLines(ed, blStart+1, blEnd-1);
				EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
				ed->prefRow = ed->currRow = ed->blocks[0].blockStart.row;
			} else
			{
				blEnd = ed->blocks[0].blockEnd.line;
				while (l <= blEnd)
				{
					EditTools_SetCurrLine(ed, l);
					if (l == ed->blocks[0].blockStart.line &&
						ed->blocks[0].blockStart.row > 0)
					{
						/* Nur Teilweise loeschen */
						len = ed->editLen - ed->blocks[0].blockStart.row;
						text = ed->editLine.text + ed->blocks[0].blockStart.row;
						strCpy(text, text + len);
						ed->editLen -= len;
						ed->editChanged = TRUE;
						++l;
					} else if (l == blEnd && ed->blocks[0].blockEnd.row < ed->editLine.length)
					{
						/* ebenfalls nur teilweise loeschen */
						strCpy(ed->editLine.text, ed->editLine.text + ed->blocks[0].blockEnd.row);
						ed->editLen -= ed->blocks[0].blockEnd.row;
						ed->editChanged = TRUE;
						++l;
					} else
					{
						EditTools_DeleteLine(ed, FALSE);
						--blEnd;
					}
				}
			}
			/* Join lines if necessary */
			if (ed->blocks[0].blockStart.row > 0)
			{
				EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
				ed->prefRow = ed->currRow = ed->blocks[0].blockStart.row;
				EditTools_ClearBlock(ed);
				EditFuncs_Delete(ed, FALSE, FALSE);
			}
		}
		
		EditBase_PutCurrLine(ed);
		
		/* Jetzt nachsehen, ob ein CR hinzugefuegt werden muss */
		if (addCr)
		{
			cLine = ed->currLineNr;
			EditTools_SetCurrLine(ed, bStart - 1);
			EditGlobals_AllocNew(&ed->editLine, ed->editLen + 4);
			EditFuncs_Protect(ed->editLine.text);
			ed->editChanged = TRUE;
			ed->editLen = (EDCOLUMN)strLen(ed->editLine.text);
			EditBase_PutCurrLine(ed);
			if (!redraw)
			{
				redraw = bStart - 1 >= ed->StartLine;
				rdrStart = (_WORD)(bStart - 1 - ed->StartLine);
				oneLine = TRUE;
			} else if (rdrStart > 0)
			{
				--rdrStart;
			}
			EditTools_SetCurrLine(ed, cLine);
		}
		/* Blockmarken loeschen */
		EditTools_ClearBlock(ed);
		EditGlobals_SetDocument(ed);
		
		EditTools_UpdateCurrLine(ed, ed->currLineNr);
		/* Jetzt nur noch neu Zeichnen */
		if (redraw)
		{
			if (rdrStart >= 0)
			{
				if (oneLine)
				{
					EditDraw_drawLine(ed, rdrStart);
				} else if (rdrLines < ed->windLines)
				{
					EditDraw_redrawLines(ed, rdrStart);
				} else
				{
					EditDraw_CenterCurrline(ed);
					WdwManager_RedrawWdw(ed->wdw, &ed->editWork);
				}
			} else
			{
				EditDraw_CenterCurrline(ed);
				WdwManager_RedrawWdw(ed->wdw, &ed->editWork);
			}
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _BOOL EditFuncs_cutEdit(EDITOR *ed, _BOOL fromOuter)
{
	_BOOL res;
	
	if (ed->readOnly!=MODE_RW)
		return FALSE;
	if (fromOuter)
	{
		EditTools_SaveOp(ed, blockCut, FALSE);
	}
	EditTools_ShowFileChanged(ed); /* Anzeigen, dass Text geaendert */
	res = EditFuncs_cutBlock(ed, fromOuter, TRUE);
	EditDraw_CenterCurrline(ed);
	if (ed->showInfo)
		EditTools_SetInfoLine(ed);
	return res;
}

/*** ---------------------------------------------------------------------- ***/

/* Fuegt einen Text an adr an die aktuelle Position ein!
 */
	
GLOBAL _BOOL EditFuncs_putLineInEditor(EDITOR *ed, _UBYTE *adr, _ULONG len, _BOOL redraw)
{
	EDLINE blStart;
	_WORD scrLine;
	_BOOL wasBlock;
	_UBYTE *adr2;

	if (ed->readOnly!=MODE_RW || len == 0)
		return FALSE;
	/* Der Text an adr wird hier dupliziert, da der Speicher
	 * danach in der Leseroutine freigegeben wird
	 */
	adr2 = MALLOC(len, "putLineInEditor");
	if (adr2 == NULL)
		return FALSE;
	MemCpy(adr2, adr, len);
	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	EditBase_PutCurrLine(ed);
	wasBlock = ed->block;
	if (ed->block)
	{
		EditTools_SaveOp(ed, blockPaste, FALSE);
		EditFuncs_cutBlock(ed, FALSE, FALSE);
	} else
	{
		EditTools_SaveOp(ed, normalPaste, FALSE);
	}
	EditTools_SetBlockstart(ed);
	ed->block = TRUE;
	if (!EditBase_InsertBuffer(ed, &adr2, len, TRUE))
	{
		/* Fehlermeldung */
		ed->blocks[0].blockStart.line = -1;
		ed->blocks[0].blockEnd.line = -1;
		ed->block = FALSE;
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
		return FALSE;
	}
	EditTools_NewBlockToUndoBuffer(ed);
	blStart = ed->blocks[0].blockStart.line;
	EditTools_SetCurrLine(ed, ed->blocks[0].blockEnd.line);
	ed->prefRow = ed->currRow = ed->blocks[0].blockEnd.row;
	if (!wasBlock)
	{
		/* normal Paste */
		ed->block = FALSE;
		ed->blocks[0].blockStart.line = -1;
		ed->blocks[0].blockEnd.line = -1;
	}
	EditTools_UpdateCurrLine(ed, ed->currLineNr);
	EditGlobals_FixCursorPos(ed);
	EditGlobals_SetDocument(ed);
	EditTools_ShowFileChanged(ed);
	if (redraw)
	{
		EditDraw_CenterCurrline(ed);
		scrLine = Max(0, (_WORD)(blStart - ed->StartLine));
		EditDraw_redrawLines(ed, scrLine);
	}
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL EditFuncs_ReadEditBuffer(EDITOR *ed, _UBYTE *buf, _ULONG len, _BOOL inLine, CONST _UBYTE *pre)
{
	UNUSED(ed);
	UNUSED(buf);
	UNUSED(len);
	UNUSED(inLine);
	UNUSED(pre);
	return FALSE;
}
