/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


/*
 * 23.1.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.20b'
 *	Teilweise wurde bei der Ersetzung des Eurozeichens nicht der Euro-Cookie verwendet.
 *
 * 31.1.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.20j'
 *	Erweiterung zur vorletzten nderung (4.43d). Jetzt knnen den beiden
 *	Quotearten verschiedene Farben/Effekte zugeordnet werden.
 */

#include <editglob.h>
#include <editbase.h>
#include <wdwmanag.h>
#include <vdiutil.h>

#include <mtappl.h>
#include <mtutils.h>
#include <portvdi.h>
#include <tos.h>
#include <rectfunc.h>

#include <ro_mem.h>
#include <routinec.h>
#include <routines.h>
#include <routine.h>
#include <file_io.h>
#include <debug.h>
#include <mime.h>


#define CR      0x0d
#define LF      0x0a
#define TAB     0x09


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Window-Funktionen fuer Darstellung auf Bildschirm */

/* Setzt neues virtuelles Fenster */
GLOBAL _VOID EditGlobals_SetDocument(EDITOR *ed)
{
	LGRECT doc;

	doc.x = ed->leftOffset;
	doc.y = ed->StartLine;
	doc.w = ed->maxWidth;
	doc.h = ed->totalLineNr - 1;
	WdwManager_SetWdwDocument(ed->wdw, &doc);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_AllocNew(aLineDesc *line, EDCOLUMN newLen)
{
	EDCOLUMN newLength;
	_UBYTE *tBuf;

	if (newLen >= line->length - 1)
	{
		newLength = ((newLen + 16) / 16 ) * 16;
		if (newLength < newLen)
			newLength += 16;
		tBuf = SYSMALLOC(newLength, MEM_NOERR, "AllocNew");
		if (tBuf == NULL)
		{
			tBuf = MALLOC(newLength, "AllocNew");
			if (tBuf == NULL)
			{
				HALT(); /* kein Speicher mehr frei */
			}
		}
		MemCpy(tBuf, line->text, line->length);
		MemSetZero(tBuf + line->length, newLength - line->length);
		FREE(line->text, line-<length);
		line->text = tBuf;
		line->length = newLength;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* GrafGrowbox und GrafShrinkbox neu, da das AES bei kleinen Boxgroessen anscheinend Muell macht */
GLOBAL _VOID EditGlobals_GrowAndShrink(EDITOR *ed, CONST GRECT *from)
{
	_WORD pts[10];
	_WORD i, j, k;
	_WORD diffw, diffh;

	WdwManager_SetClip(cursorHandle, &ed->editWork, TRUE);
	vsl_type(cursorHandle, USERLINE);
	diffw = Max(4, from->g_w / 2);
	diffh = from->g_h / 2;
	for (j = 0; j < 4; j++)
	{
		for (k = 0; k < 8; k++)
		{
			for (i = 0; i < 11; i++)
			{
				pts[0] = from->g_x - diffw * i;
				pts[1] = from->g_y - diffh * i;
				pts[2] = from->g_x + from->g_w - 1 + diffw * i;
				pts[3] = pts[1];
				pts[4] = pts[2];
				pts[5] = from->g_y + from->g_h + diffh * i - 1;
				pts[6] = pts[0];
				pts[7] = pts[5];
				pts[8] = pts[0];
				pts[9] = pts[1];
				v_pline(cursorHandle, 5, pts);
			}
			/* Hier irgendeine Wartefunktion einbauen */
			EventTimer(0);
		}
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* VDI Text Funktionen    */

GLOBAL _VOID EditGlobals_Text(EDITOR *ed, _WORD x, _WORD y, CONST _UBYTE *string)
{
	_WORD len, olen;
	
#define MAX_TEXT_LEN 126
	
	if (string[0] != '\0')
	{
		if (ed->isFSM || !ed->monoSpaced)
		{
			VDIUtil_Text(ed->isFSM, ed->hdl, x, y + ed->charHeight - 1, string);
		} else
		{
			len = (_WORD)strLen(string);
			while (len != 0)
			{
				olen = len;
				if (olen > MAX_TEXT_LEN)
					olen = MAX_TEXT_LEN;
				v_gntext(ed->hdl, x, y + ed->charHeight - 1, string, olen);
				len -= olen;
				string += olen;
				x += ed->charWidth * olen;
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_NText(EDITOR *ed, _WORD x, _WORD y, CONST _UBYTE *string, _WORD len)
{
	_WORD olen;
	
#define MAX_TEXT_LEN 126
	
	if (len > 0)
	{
		if (ed->isFSM || !ed->monoSpaced)
		{
			VDIUtil_NText(ed->isFSM, ed->hdl, x, y + ed->charHeight - 1, string, len);
		} else
		{
			while (len != 0)
			{
				olen = len;
				if (olen > MAX_TEXT_LEN)
					olen = MAX_TEXT_LEN;
				v_gntext(ed->hdl, x, y + ed->charHeight - 1, string, olen);
				len -= olen;
				string += olen;
				x += ed->charWidth * olen;
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20j */
GLOBAL _WORD EditGlobals_isQuote(EDITOR *ed, CONST _UBYTE *str)
{
	size_t j;
	_UBYTE ch;
	
	while (*str == ' ')
		str++;
	j = 0;
	while (j < 6 && str[j] != '>' && str[j] != '|')
	{
		ch = str[j];
		if (j >= 2)
			ch = ucase(ch);
		if (ed->AllowLowerQuoteInitials)
		{
			if (!is_upper(ch) && !is_lower(ch))
				return FALSE;
		} else
		{
			if (!is_upper(ch))
				return FALSE;
		}
		++j;
	}
	if(j<6 && str[j]=='>')
		return 1;
	else if(j<6 && str[j]=='|')
		return 2;
	else
		return 0;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_GetQuote(EDITOR *ed, CONST _UBYTE *str, _UBYTE *quote, size_t strsize)
{
	size_t j;
	_BOOL isQ;
	_UBYTE ch;
	
	while (*str == ' ')
		str++;
	j = 0;
	isQ = TRUE;
	while (j < 6 && str[j] != '>' && str[j] != '|')
	{
		ch = str[j];
		if (j >= 2)
			ch = ucase(ch);
		if (ed->AllowLowerQuoteInitials)
		{
			if (!is_upper(ch) && !is_lower(ch))
			{
				isQ = FALSE;
				break;
			}
		} else
		{
			if (!is_upper(ch))
			{
				isQ = FALSE;
				break;
			}
		}
		++j;
	}
	isQ &= j < 6;
	if (isQ)
	{
		j += 2; /* +1 fuer '>', +1 fuer '\0' */
		if (j > strsize)
			j = strsize;
		strMcpy(quote, j, str);
	} else
	{
		*quote = '\0';
	}
}

/*** ---------------------------------------------------------------------- ***/

#define simpleOutput() \
	/* Zeile einfach ausgeben */ \
	if (doOutput) \
		EditGlobals_Text(ed, x, y, line); \
	if (!ed->monoSpaced || ed->isFSM) /* added "OR isFSM" for Monospaced 821 Font */ \
	{ \
		/* Textadvance und damit xOffset berechnen */ \
		xOff += EditGlobals_InqNTextextend(ed, line, maxExtIdx, FALSE, 30000); \
	} else \
	{ \
		xOff += maxExtIdx * ed->charWidth; \
	}

#define is_normal_char(ch) ( \
	((ch) >= 'A' && (ch) <= 'Z') || \
	((ch) >= 'a' && (ch) <= 'z') || \
	((ch) >= '0' && (ch) <= '9') || \
	(ch) == 0x84 /* ae */ || \
	(ch) == 0x94 /* oe */ || \
	(ch) == 0x81 /* ue */ || \
	(ch) == 0x9e /* ss */ || \
	(ch) == 0x8e /* Ae */ || \
	(ch) == 0x99 /* Oe */ || \
	(ch) == 0x9a /* Ue */ || \
	(ch) == '"' || \
	(ch) == '\'' \
	)

#define is_delimiter(ch) ( \
	(ch) == ' ' || \
	(ch) == '-' || \
	(ch) == '>' || \
	(ch) == '(' || \
	(ch) == '[' || \
	(ch) == '{' || \
	(ch) == '"' || \
	(ch) == '\'' \
	)

#define is_word_end(ch) ( \
	(ch) == '\0' || \
	(ch) == CR || \
	(ch) == LF || \
	(ch) == ' ' || \
	(ch) == '.' || \
	(ch) == ',' || \
	(ch) == ';' || \
	(ch) == '!' || \
	(ch) == '-' || \
	(ch) == '?' || \
	(ch) == ':' || \
	(ch) == '"' || \
	(ch) == '\'' || \
	(ch) == '}' || \
	(ch) == ']' || \
	(ch) == ')' \
	)
	
/*** ---------------------------------------------------------------------------- ***/
/*	GLOBAL _WORD EditGlobals_LinkType(const _UBYTE *string,const _UBYTE **linkRet)	*/
/*	Analysiert einen String und gibt zurck um was fr einen Link es sich handelt	*/
/*	Auerdem gibt es auf Wunsch zurck wo der eigentliche Link beginnt, also z.B.	*/
/*	ohne '<' und 'mailto:'															*/
/*	Parameter:	string:		der zu analysierende String								*/
/*				linkRet:	Rckgabe:hier beginnt der eigentliche Link. 			*/
/*							darf NULL sein,dann keine Rckgabe						*/
/*	Ergebnis:	NoLNK:		kein Link												*/
/*				Http,Ftp:	Link ins WWW oder FTP									*/
/*				EmailI,EmailM:Email adresse im Mausnet oder sonstigem Internet		*/
/*				MausId,MessId:Link auf Message ID im Mausnet oder sonstigem Internet*/
/*				EmailOrIdM,EmailOrIdI: Email oder Message ID?						*/		
/*	24.8.00 von Dimitri Junker	DYJ 4.18h											*/
/*** ---------------------------------------------------------------------------- ***/
/*GS 5.28a */
GLOBAL _WORD EditGlobals_LinkType(const _UBYTE *string,const _UBYTE **linkRet)
{
	_UBYTE *at,*end,*pkt=NULL,*tmp,*blk;
	const _UBYTE *link;
	_WORD type=NoLNK;

	link=string;
	while(strChr("< ",*link))
		link++;
	if(!strnCaseCmp(link,"a href",6))	/* Ein bischen html*/
	{
		tmp=strChr(link,'\"');
		if(tmp)
			link=tmp+1;
	}
	end=strChr(link,'>');
	at=strChr(link,'@');
	blk=strChr(link,' ');
	if(end && ((_ULONG)end<(_ULONG)at))
		at=NULL;
	if(at)
	{
		pkt=strChr(at,'.');
		if(end && ((_ULONG)end<(_ULONG)pkt))
			pkt=NULL;
	}
		
	switch(*link)
	{
		case 'g':
		case 'G':
			if(!strnCaseCmp(link,"gopher://",9))
				type=Gopher;
			break;
		case 'h':
		case 'H':
			if(!strnCaseCmp(link,"http://",7) || !strnCaseCmp(link,"https://",8))
				type=Http;
			break;
		case 'w':
		case 'W':
			if(!strnCaseCmp(link,"www.",4))
				type=Http;
			else if(!strnCaseCmp(link,"wais://",7))
				type=Wais;
			break;
		case 't':
		case 'T':
			if(!strnCaseCmp(link,"telnet://",9))
				type=Telnet;
			break;
		case 'f':
		case 'F':
			if(!strnCaseCmp(link,"ftp://",6))
				type=Ftp;
			break;
		case 'n':
		case 'N':
			if(!strnCaseCmp(link,"news:",5) ||!strnCaseCmp(link,"nntp:",5))
			{
				link+=5;
				if(pkt)
					type=MessId;
				else
					type=MausId;
			}
			break;
		case 'm':
		case 'M':
			if(!strnCaseCmp(link,"mailto:",7))
			{
				link+=7;
				if(pkt)
					type=EmailI;
				else
					type=EmailM;
			}
			break;
	}
	if(type==NoLNK && at)
	{
		if(blk && blk<at)
			type=EmailM;	/* nur da darf eine Leerstelle im Teil vor dem '@' sein */
		else
		{
			if(pkt)
				type=EmailOrIdI;
			else
				type=EmailOrIdM;
		}
		if(pkt)
			type=EmailOrIdI;
		else
			type=EmailOrIdM;
	}
	if(linkRet)
		*linkRet=link;
	return type;
}
		
/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20b */	/*DYJ 5.20j */
LOCAL _WORD handleEffectLine(EDITOR *ed, _WORD x, _WORD y,
	CONST _UBYTE *line, EDCOLUMN len, _BOOL doOutput, _WORD maxExtIdx, _WORD maxWdwWidth)
{
	_WORD isQ;
	_UBYTE display[4096];
	_WORD xOff;
	_WORD maxI, i, j, idx;
	_BOOL inEffect;
	_UWORD eff;
	_UBYTE effChar;
	_UBYTE endChar;
	_WORD col;
	_WORD colL,effL;
	_WORD euroChar;
	euroChar=Mime_EuroChar();

	xOff = 0;
	if (maxExtIdx >= sizeof(display))
	{
		/* Laenge ueberschreitet maximal moegliche Testlaenge,
		 * daher Breite als MAXINT zurueckliefern
		 */
		return 30000 /* INT_MAX */;
		/* INT_MAX ist zu viel, weil noch der linke Fensterrand dazugerechnet
		 * und das Ergebnis dann negativ wird
		 */
	}
	if (ed->readOnly!=MODE_RW)
	{
		isQ = EditGlobals_isQuote(ed, line);
		colL=ed->textCol;
		effL=TXT_NORMAL;
		if (isQ==1)
		{
			/* Farbe umsetzen auf Quotefarbe */
			if(ed->quoteCol!=KEEPCOL)
				colL=ed->quoteCol;
			/* Texteffekt umsetzen */
			effL|=ed->quoteEff;
		}
		else if (isQ==2)
		{
			/* Farbe umsetzen auf Quotefarbe */
			if(ed->quoteCol2!=KEEPCOL)
				colL=ed->quoteCol2;
			/* Texteffekt umsetzen */
			effL|=ed->quoteEff2;
		}
		col=colL;
		eff=effL;
		vst_color(ed->hdl, colL);
		vst_effects(ed->hdl, effL);
		if (ed->DisplayEffects)
		{
			/* Zeile parsen und ggf. Effekte einstellen */
			i = 0;
			idx = 0;
			xOff = 0;
			inEffect = FALSE;
			effChar = '\0';
			maxI = maxExtIdx;  /* vorher: maxI = len */
			while (i < maxI && line[i] != '\0')
			{
				/* Solange Teilstrings kopieren, bis Sonderzeichen gefunden wurde */
				if (is_eff_char(line[i]) || 
					(!inEffect && line[i]=='<' && EditGlobals_LinkType(&line[i+1],NULL) ||
					( inEffect && line[i]=='>'  ) ))
				{
					if (inEffect)
					{
						/* Testen, ob Effect nicht beendet wird */
						if (i < len-1 && !setin(TrennSet, line[i+1]) && effChar == '_')
						{
							if (line[i] == '_')
								display[idx] = ' ';
							else
								display[idx] = line[i];
							++idx;
						} else if (effChar == line[i] && is_word_end(line[i+1]))
						{
							/* Effekt beendet */
							if(effChar=='>')
								display[idx++] =effChar ;
							display[idx] = '\0';
							if (doOutput)
							{
								/* Letzten Teil ausgeben */
								EditGlobals_NText(ed, x + xOff, y, display, idx);
							}
							/* Textadvance und damit xOffset berechnen */
							xOff += EditGlobals_InqNTextextend(ed, display, idx, FALSE, maxWdwWidth - xOff);
							/* Effekt wieder abstellen */
							switch(effChar)
							{
								case '>':
									col=colL;
									vst_color(ed->hdl, col);
									eff &= ~ed->linkEff;
									eff|=effL&ed->linkEff;
									break;
								case '*':
									eff &= ~TXT_THICKENED;
									eff|=effL&TXT_THICKENED;
									break;
								case '/':
									eff &= ~TXT_SKEWED;
									eff|=effL&TXT_SKEWED;
									break;
								case '_':
									eff &= ~TXT_UNDERLINED;
									eff|=effL&TXT_UNDERLINED;
									break;
								case '#':
									eff &= ~TXT_LIGHT;
									eff|=effL&TXT_LIGHT;
									break;
							}
							vst_effects(ed->hdl, eff);
							idx = 0;
							inEffect = FALSE;
							if (xOff > maxWdwWidth)	
								break;
						} else
						{
							display[idx] = line[i];
							++idx;
						}
					} else
					{
						/* Nicht in Effekt, vielleicht ein neuer? */
						endChar=(line[i]=='<' ? '>' : line[i]);
						if (i < len - 1 && (is_normal_char(line[i+1])||line[i]=='<') &&

/*DYJ  Ende; alt:	
						if (i < len - 1 && is_normal_char(line[i+1]) &&
*/
							(i == 0 || is_delimiter(line[i-1])))
						{
							/* Jetzt noch testen, ob das Zeichen in der Zeile nochmal kommt */
							j = i;
							do
							{
								do
									++j;
								while (j < len && line[j] != endChar);
							} while (j < len && !is_word_end(line[j+1]));
							if (j < len)
							{
								/* Beginn eines Effects */
								inEffect = TRUE;
								/* Letzten Teil ausgeben */
								display[idx] = '\0';
								if (doOutput)
								{
									/* Textstueck ausgeben */
									EditGlobals_NText(ed, x + xOff, y, display, idx);
								}
								/* Textadvance und damit xOffset berechnen */
								xOff += EditGlobals_InqNTextextend(ed, display, idx, FALSE, maxWdwWidth - xOff);
								/* Jetzt Effekt setzen */
								switch(line[i])
								{
									case '<':
										eff |=ed->linkEff;
										if(ed->linkCol!=KEEPCOL)
										{
											col=ed->linkCol;
											vst_color(ed->hdl, col);
										}
										break;
									case '*':
										eff |=TXT_THICKENED;
										break;
									case '/':
										eff |=TXT_SKEWED;
										break;
									case '_':
										eff |= TXT_UNDERLINED;
										eff|=effL&TXT_UNDERLINED;
										break;
									case '#':
										eff |=TXT_LIGHT;
										break;
								}
								vst_effects(ed->hdl, eff);
								effChar = endChar;
								idx = 0;
								if(line[i]=='<')
									display[idx++] ='<' ;
								if (xOff > maxWdwWidth)	
									break;
							} else
							{
								display[idx] = line[i];
								++idx;
							}
						} else
						{
							display[idx] = line[i];
							++idx;
						}
					}
				} else
				{
					display[idx] = line[i];
					++idx;
				}
				++i;
				if((_WORD)display[idx-1]==euroChar)
				{
					display[idx-1] = 'E';
					display[idx++] = 'U';
					display[idx++] = 'R';
				}
			}
			if (idx > 0)
			{
				/* Text noch nicht ausgegeben */
				display[idx] = '\0';
				if (doOutput)
				{
					/* Text noch ausgeben */
					EditGlobals_NText(ed, x + xOff, y, display, idx);
				}
				/* Textadvance und damit xOffset berechnen */
				xOff += EditGlobals_InqNTextextend(ed, display, idx, FALSE, maxWdwWidth - xOff);
			}
		} else
		{
			/* Zeile einfach ausgeben */
			vst_effects(ed->hdl, TXT_NORMAL);
			simpleOutput();
		}
	} else
	{
		vst_effects(ed->hdl, TXT_NORMAL);
		simpleOutput();
	}
	return xOff;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD EditGlobals_OutputLine(EDITOR *ed, _WORD x, _WORD y, CONST _UBYTE *line, EDCOLUMN len, _WORD maxWdwWidth)
{
	return handleEffectLine(ed, x, y, line, len, TRUE, len, maxWdwWidth);
}

/*** ---------------------------------------------------------------------- ***/

/* Diese Prozedur parst den Text immer.
 * Idx gibt die Position an, bis zu der der
 * String maximal zusammengesetzt wird!
 * len ist die tatsaechliche Laenge.
 * maxWdwWidth ist die Pixelbreite bei der
 * wir aufhoeren koennen zu parsen
 */
GLOBAL _WORD EditGlobals_GetTextWidth(EDITOR *ed, CONST _UBYTE *string, EDCOLUMN idx, EDCOLUMN len, _WORD maxWdwWidth)
{
	return handleEffectLine(ed, 0, 0, string, len, FALSE, idx, maxWdwWidth);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD EditGlobals_InqTextextend(EDITOR *ed, CONST _UBYTE *string, _BOOL DisplayEffects, _WORD maxWdwWidth)
{
	EDCOLUMN l;
	_WORD width;
	
	if (string[0] == '\0')
	{
		width = 0;
	} else
	{
		if (!DisplayEffects)
		{
			width = VDIUtil_InqTextextend(ed->isFSM, ed->hdl, string);
		} else
		{
			l = (EDCOLUMN)strLen(string);
			width = handleEffectLine(ed, 0, 0, string, l, FALSE, l, maxWdwWidth);
		}
	}
	return width;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD EditGlobals_InqNTextextend(EDITOR *ed, CONST _UBYTE *string, _WORD l, _BOOL DisplayEffects, _WORD maxWdwWidth)
{
	_WORD width;
	
	if (l == 0)
	{
		width = 0;
	} else
	{
		if (!DisplayEffects)
		{
			width = VDIUtil_InqNTextextend(ed->isFSM, ed->hdl, string, l);
		} else
		{
			width = handleEffectLine(ed, 0, 0, string, l, FALSE, l, maxWdwWidth);
		}
	}
	return width;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL EDCOLUMN EditGlobals_RowToIndex(CONST _UBYTE *str, EDCOLUMN row, EDCOLUMN TabSize)
{
	EDCOLUMN i, j;

	j = 0;
	for (i = 0; i < row; i++)
	{
		if (str[i] == TAB)
			j = ((j / TabSize) + 1) * TabSize;
		else
			++j;
	}
	return j;
}

/*** ---------------------------------------------------------------------- ***/

/* Passt auf, das der Cursor nie hinter einem CR/LF steht
 */

GLOBAL _VOID EditGlobals_FixCursorPos(EDITOR *ed)
{
	_UBYTE *tPtr;
	_UBYTE ch;

	if (ed->currRow > 0)
	{
		tPtr = ed->editLine.text;
		for (;;)
		{
			ch = tPtr[ed->currRow - 1];
			if (ch != CR && ch != LF)
				break;;
			--ed->currRow;
			if (ed->currRow <= 0)
				break;
		}
	}
	ed->prefRow = ed->currRow;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Cursor-Zeichenfunktionen */

GLOBAL _VOID EditGlobals_ClipWork(EDITOR *ed)
{
	GRECT clip;

	clip = rc_clip(&deskSize, &ed->editWork);
	WdwManager_SetClip(ed->hdl, &clip, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL getCursorRect(EDITOR *ed, _WORD xy[4])
{
	GRECT c;
	_WORD width;
	_UBYTE ch[2];
	_UBYTE saveCh;
	_UBYTE *charAdr;
	_UBYTE *text;
	EDCOLUMN len;
	EDCOLUMN drawLen;
	
	if (ed->currLineNr >= ed->totalLineNr || (ed->readOnly!=MODE_RW && !ed->multiBlockMark))
		return FALSE;
	if (!EditBase_GetLineAdr(ed, ed->currLineNr, &text, &len))
		return FALSE;
	if (ed->currRow > len)
		ed->prefRow = ed->currRow = len;
	charAdr = &text[ed->currRow];
	saveCh = *charAdr;
	if (ed->currRow != ed->cursPos.row || ed->currLineNr != ed->cursPos.line)
	{
		if (!ed->monoSpaced || ed->realTabs || ed->isFSM)   /* added "OR isFSM" for Monospaced 821 Font */
		{
			/* String terminieren */
			*charAdr = '\0';
			drawLen = ed->currRow;
			if (ed->realTabs)
			{
				drawLen = (EDCOLUMN)Str_ConvertTabs(text, theDrawLine, sizeof(theDrawLine), ed->TabSize);
				text = theDrawLine;
			}
			/* Cursorposition bestimmen */
			width = EditGlobals_InqNTextextend(ed, text, drawLen, FALSE, 30000);
			/* Aenderung wieder rueckgaengig machen */
			*charAdr = saveCh;
			ed->cursPos.x = width;
		} else
		{
			ed->cursPos.x = ed->currRow * ed->charWidth;
			width = ed->cursPos.x;
		}
		ed->cursPos.row = ed->currRow;
		ed->cursPos.line = ed->currLineNr;
	} else
	{
		width = ed->cursPos.x;
	}
	/*----*/
	c.g_x = ed->editWork.g_x + width + pixOff - ed->leftOffset;
	c.g_y = ed->editWork.g_y + (_WORD)(ed->currLineNr - ed->StartLine) * ed->charHeight;
	xy[0] = c.g_x;
	xy[1] = c.g_y - 1;
	xy[3] = c.g_y + ed->charHeight + 1;
	if (ed->insertMode)
	{
		xy[2] = xy[0] + 1;
	} else
	{
		/* Zeichenausmasse holen */
		ch[1] = '\0';
		ch[0] = saveCh;
		if (ch[0] == '\0' || (ed->monoSpaced && !ed->isFSM))
		{
			xy[2] = c.g_x + ed->charWidth;
		} else
		{
			width = EditGlobals_InqNTextextend(ed, ch, 1, FALSE, 30000);
			xy[2] = c.g_x + width - 1;
		}
		++xy[1];
		--xy[3];
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_growCursor(EDITOR *ed)
{
	_WORD xy[4];
	GRECT r;

	if (getCursorRect(ed, xy))
	{
		r.g_x = xy[0];
		r.g_y = xy[1];
		r.g_w = xy[2] - xy[0] + 1;
		r.g_h = xy[3] - xy[1] + 1;
		EditGlobals_GrowAndShrink(ed, &r);
	}
}


/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID EditGlobals_drawCursor(EDITOR *ed)
{
	_WORD xy[4];
	GRECT r;

	if (getCursorRect(ed, xy))
	{
		if (WdwManager_RectList(ed->wdw, 0, &r))
		{
			do
			{
				WdwManager_SetClip(cursorHandle, &r, TRUE);
				vr_recfl(cursorHandle, xy);
			} while (WdwManager_RectList(ed->wdw, 1, &r));
		}
		WdwManager_SetClip(cursorHandle, &ed->editWork, TRUE);
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----------------------------------------------------------------------*
 *                            Maeusetreiberei                            *
 *----------------------------------------------------------------------*/

LOCAL _UWORD mouseCount;
LOCAL _UWORD smCount;
LOCAL _WORD smForm;
LOCAL MFORM *smUser;
LOCAL _WORD MausForm;
LOCAL MFORM *Usermaus;

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID EditGlobals_SetMouse(_VOID)
{
	if (MausForm == USER_DEF)
		graf_mouse(MausForm, Usermaus);
	else
		graf_mouse(MausForm, NULL);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_MouseArrow(_VOID)
{
	MausForm = ARROW;
	EditGlobals_SetMouse();
	/* ShowMouse(FALSE); */
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_MouseCursor(_VOID)
{
	MausForm = TEXT_CRSR;
	EditGlobals_SetMouse();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_MouseBusy(_VOID)
{
	MausForm = BUSY_BEE;
	EditGlobals_SetMouse();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_MouseFinger(_VOID)
{
	MausForm = POINT_HAND;
	EditGlobals_SetMouse();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_MouseHand(_VOID)
{
	MausForm = FLAT_HAND;
	EditGlobals_SetMouse();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_MouseThincross(_VOID)
{
	MausForm = THIN_CROSS;
	EditGlobals_SetMouse();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_MouseThickcross(_VOID)
{
	MausForm = THICK_CROSS;
	EditGlobals_SetMouse();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_MouseOutline(_VOID)
{
	MausForm = OUTLN_CROSS;
	EditGlobals_SetMouse();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_MouseUser(_VOID)
{
	MausForm = USER_DEF;
	EditGlobals_SetMouse();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_UserMouse(MFORM *form)
{
	Usermaus = form;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_SaveMouse(_VOID)
{
	smCount = mouseCount;
	smUser = Usermaus;
	smForm = MausForm;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_RestoreMouse(_VOID)
{
	Usermaus = smUser;
	MausForm = smForm;
	if (mouseCount > 0 && smCount == 0)
	{
		EditGlobals_SetMouse();
		EditGlobals_ShowMouse(TRUE);
	} else if (mouseCount == 0 && smCount > 0)
	{
		EditGlobals_SetMouse();
		EditGlobals_HideMouse();
		mouseCount = smCount;
	} else
	{
		mouseCount = smCount;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL EditGlobals_MouseIsOn(_VOID)
{
	return mouseCount == 0;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_HideMouse(_VOID)
{
	if (mouseCount == 0)
		graf_mouse(M_OFF, NULL);
	++mouseCount;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_ShowMouse(_BOOL force)
{
	if (mouseCount > 0 || force)
	{
		if (mouseCount <= 0)
			return;
		--mouseCount;
		if (force)
			mouseCount = 0;
		if (mouseCount == 0)
			graf_mouse(M_ON, NULL);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL EditGlobals_CursorIsOn(EDITOR *ed)
{
	return ed->cursor == 0;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditGlobals_HideCursor(EDITOR *ed)
{
	if (ed->isHidden || ed->inForce)
		return;
	if (ed->cursor == 0)
	{
		if ((ed->readOnly==MODE_RW || ed->multiBlockMark) &&
			(!ed->block || ed->blockIndent))
		{
			EditGlobals_drawCursor(ed);
		}
	}
	++ed->cursor;
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _VOID EditGlobals_ShowCursor(EDITOR *ed)
{
	if (ed->isHidden || ed->inForce)
		return;
	if (ed->cursor < 0)
	{
		ErrorOut(EO_ERROR, "ShowCursor: cursor < 0");
		return;
	}
	if (ed->cursor > 0)
	{
		--ed->cursor;
		if (ed->cursor == 0 &&
			(ed->readOnly==MODE_RW || ed->multiBlockMark) &&
			(!ed->block || ed->blockIndent))
		{
			EditGlobals_drawCursor(ed);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _VOID EditGlobals_ForceCursor(EDITOR *ed)
{
	_WORD xy[4];
	GRECT r;

	if (ed->isHidden)
		return;
	ed->cursor = 0;
	if (getCursorRect(ed, xy))
	{
		ed->inForce = TRUE;
		r.g_x = xy[0];
		r.g_y = xy[1];
		r.g_w = xy[2] - xy[0] + 1;
		r.g_h = xy[3] - xy[1] + 1;
		WdwManager_RedrawWdw(ed->wdw, &r);
		ed->inForce = FALSE;
		if ((ed->readOnly==MODE_RW || ed->multiBlockMark) &&
			(!ed->block || ed->blockIndent))
		{
			EditGlobals_drawCursor(ed);
		}
	}
}
