/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 */


#include <editor.h>
#include <catgloba.h>
#include <clip.h>
#include <fontsele.h>
#include <protokol.h>

#include <edittype.h>
#include <editglob.h>
#include <edittool.h>
#include <editdraw.h>
#include <editbase.h>
#include <editfunc.h>
#include <editutil.h>

#include <wdwmanag.h>
#include <confvars.h>
#include <varn_edi.h>
#include <printer.h>
#include <uudecode.h>
#include <rectfunc.h>
#include <vdiutil.h>

#include <mtalerts.h>
#include <mtappl.h>
#include <mtdials.h>
#include <mtutils.h>
#include <mttextfi.h>
#include <mintutil.h>
#ifndef __MINTBIND_H__
#include <mintbind.h>
#endif
#include <filename.h>
#include <magicstr.h>

#include <keyboard.h>
#include <routinec.h>
#include <routines.h>
#include <dd.h>
#include <ro_mem.h>
#include <mime.h>

#include <mtrsc.h>
#include <catlib.h>
#define TAB 0x09
#define CR  0x0d
#define LF  0x0a

LOCAL _BOOL lastOpWasCtrlY;
LOCAL SET nums;


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#if EDIT_DEBUG

LOCAL _BOOL debugMode;

EXTERN _BOOL __CDECL validAdr(_VOID *adr);

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID TestStructure(EDITOR *ed)
{
	aBuffer *buf;
	_WORD bufNum;
	EDBUFSIZE used;
	EDBUFLINE i;

	ErrorOut(EO_DEBUG, "Teste Struktur und F\201llungsgrad ...");
	buf = ed->firstBuff;
	bufNum = 1;
	while (buf != NULL)
	{
		used = 0;
		for (i = 0; i <= (EDBUFLINE)(buf->toLine - buf->fromLine); i++)
		{
			used += buf->lines[i];
		}
		ErrorOut(EO_DEBUG, "Buffer Nummer: %ld", (_LONG)bufNum);
		ErrorOut(EO_DEBUG, "usedMem: %ld", (_LONG)buf->usedMem);
		ErrorOut(EO_DEBUG, "used by count: %ld", (_LONG)used);
		ErrorOut(EO_DEBUG, "MaxLines: %ld", (_LONG)buf->maxLine);
		ErrorOut(EO_DEBUG, "fromLine: %ld", (_LONG)buf->fromLine);
		ErrorOut(EO_DEBUG, "toLine: %ld", (_LONG)buf->toLine);
		ErrorOut(EO_DEBUG, "usedLines: %ld", (_LONG)(buf->toLine - buf->fromLine + 1));
		buf = buf->next;
		++bufNum;
	}
}

/*** ---------------------------------------------------------------------- ***/


#if EDIT_DEBUG >= 2
LOCAL _VOID doDump(EDITOR *ed)
{
	EDLINE fromLine, toLine, cLine;
	EDCOLUMN cRow;
	EDLINE lines, l2;

	cLine = ed->currLineNr;
	cRow = ed->currRow;
	fromLine = 0;
	toLine = ed->totalLineNr - 1;
	EditTools_SetCurrLine(ed, fromLine);
	fromLine = ed->currLineNr;
	while (fromLine <= toLine && fromLine == ed->currLineNr)
	{
		/* Zeile testen */
		if (strLen(ed->editLine.text) >= ed->editLine.length)
		{
			ErrorOut(EO_DEBUG, "Zeile zu lang: %ld (%s)", ed->currLineNr, ed->editLine.text);
		}
		++fromLine;
		EditTools_SetCurrLine(ed, fromLine);
	}
	ErrorOut(EO_DEBUG, "Von unten nach oben: %ld lines", fromLine);
	l2 = fromLine;
	fromLine = 0;
	toLine = ed->totalLineNr - 1;
	EditTools_SetCurrLine(ed, toLine);
	lines = 0;
	while (toLine >= fromLine && toLine == ed->currLineNr)
	{
		--toLine;
		EditTools_SetCurrLine(ed, toLine);
		++lines;
	}
	ErrorOut(EO_DEBUG, "Von oben nach unten: %ld lines", lines);
	if (l2 != lines)
	{
		;
	}
#if DEBUG_ALLOC >= 2
	mem_debug_check_all("doDump")er;
#endif
	EditTools_SetCurrLine(ed, cLine);
	ed->prefRow =ed->currRow = cRow;
}
#endif /* EDIT_DEBUG >= 2 */
#endif /* EDIT_DEBUG */

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _WORD FindNum(_VOID)
{
	_WORD i;

	for (i = 0; i < SETMAX; i++)
	{
		if (!setin(nums, i))
		{
			setincl(nums, i);
			return i;
		}
	}
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ClearNum(_WORD num)
{
	setexcl(nums, num);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD Editor_GetEditNumber(HWND wdw)
{
	return EditFuncs_GetEditNumber(wdw);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Liest eine Kuerzeldatei ein
 */
GLOBAL _VOID Editor_ReadShortkeyFile(CONST _UBYTE *fname)
{
	EditFuncs_ReadShortkeyFile(fname);
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt eine Kuerzeldatei wieder frei
 */
GLOBAL _VOID Editor_FreeShortkeyList(_VOID)
{
	EditFuncs_FreeShortkeyList();
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Sichert die Position des Editors in den
 * Configvariablen
 */
GLOBAL _VOID Editor_EditSaveWdwPos(HWND wdw)
{
	EditUtil_EditSaveWdwPos(wdw);
}

/*** ---------------------------------------------------------------------- ***/

/* Sichert die Position und den Inhalt aller offenen Editorfenster
 * (nicht ReadOnly-Fenster
 */
GLOBAL _VOID Editor_EditSavePos(_VOID)
{
	EditUtil_EditSavePos();
}

/*** ---------------------------------------------------------------------- ***/

/* Restauriert den Inhalt und die Position der EditorFenster
 */
GLOBAL _VOID Editor_EditRestorePos(_BOOL doOpen)
{
	_UBYTE varName[256];
	EDCOLUMN margin;
	_BOOL info;
	_BOOL umbruch;
	_UBYTE fullname[PATH_MAX];
	_UBYTE path[PATH_MAX];
	_UBYTE name[PATH_MAX];
	_WORD i;
	HWND wdw;
	
	for (i = 0; i < 256; i++)
	{
		make_varname(varName, sizeof(varName), cEditFile, i);
		if (ConfVars_GetConfigString(varName, fullname, sizeof(fullname)))
		{
			ConfVars_DeleteConfigVar(varName);
			/* Name splitten */
			strBcpy(path, fullname);
			F_Path_Dirname(path);
			F_Path_Append(path, "");
			strBcpy(name, F_Path_Basename(fullname));
			make_varname(varName, sizeof(varName), cEditMargin, i);
			ConfVars_GetConfDefInt(varName, &margin, 80);
			ConfVars_DeleteConfigVar(varName);
			make_varname(varName, sizeof(varName), cEditInfo, i);
			ConfVars_GetConfDefBool(varName, &info, TRUE);
			ConfVars_DeleteConfigVar(varName);
			make_varname(varName, sizeof(varName), cEditWrap, i);
			ConfVars_GetConfDefBool(varName, &umbruch, FALSE);
			ConfVars_DeleteConfigVar(varName);
			if (doOpen)
			{
				if (!Editor_OpenEditor(path, name, NULL, FALSE, FALSE,
					FUNK_NULL, FUNK_NULL, FUNK_NULL, FUNK_NULL,
					FUNK_NULL, FUNK_NULL, FUNK_NULL, FUNK_NULL,
					margin, info, umbruch, i, &wdw))
				{
					return;
				}
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Editor_Search(
	HWND wdw,
	CONST _UBYTE *w,
	searchPos pos,
	searchDir direction,
	searchCount count,
	_WORD num,
	_BOOL ignoreCase,
	_BOOL onlyWord,
	_BOOL again,
	CONST _UBYTE *bingEvent)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		return EditFuncs_searchAndReplace(ed, w, "", pos, direction, count, num,
			ignoreCase, onlyWord, FALSE, TRUE, again, bingEvent);
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Editor_SearchAndReplace(
	HWND wdw,
	CONST _UBYTE *w,
	CONST _UBYTE *r,
	searchPos pos,
	searchDir direction,
	searchCount count,
	_WORD num,
	_BOOL ignoreCase,
	_BOOL onlyWord,
	_BOOL ask,
	_BOOL again)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		return EditFuncs_searchAndReplace(ed, w, r, pos, direction, count, num,
			ignoreCase, onlyWord, TRUE, ask, again, cBingNothingFound);
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_IndentBlock(HWND wdw)
{
	EditUtil_IndentBlock(wdw);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Editor_EditCut(HWND wdw, _BOOL toClip)
{
	EDITOR *ed;
	_BOOL res;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		res = EditFuncs_cutEdit(ed, toClip);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
		return res;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Fuegt einen Text mit Filename ein
 */
GLOBAL _VOID Editor_EditPasteFile(HWND wdw, _BOOL isFilter, CONST _UBYTE *path, CONST _UBYTE *name)
{
	EditUtil_EditPasteFile(wdw, isFilter, path, name);
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _VOID Editor_EditPaste(HWND wdw)
{
	EDITOR *ed;
	_UBYTE ScrapPath[PATH_MAX];

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->readOnly!=MODE_RW)
			return;
		if (!Clip_GetScrap(ScrapPath))
			return;
		EditUtil_EditPasteFile(wdw, FALSE, ScrapPath, SCRAP_TXT);
	}
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
GLOBAL _BOOL Editor_EditCopy(HWND wdw)
{
	_BOOL append;
	_UWORD state;
	_UBYTE fullName[PATH_MAX];
	FILENO fHdl;
	EDITOR *ed;
	_UBYTE ScrapPath[PATH_MAX];
	_WORD i;
	_WORD dummy;
	_UWORD udummy;
	
	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (!Clip_GetScrap(ScrapPath))
			return FALSE;
		graf_mkstate(&dummy, &dummy, &udummy, &state);
		append = (state & K_SHIFT) != 0;	
		strBcpy(fullName, ScrapPath);
		F_Path_Append(fullName, SCRAP_TXT);
		if (append)
		{
			/* Anhaengen */
			fHdl = (FILENO)Fopen(fullName, FO_WRITE);
			if (fHdl < 0)
			{
				Clip_ScrapClear("", "");
				fHdl = (FILENO)Fcreate(fullName, 0);
				if (fHdl < 0)
				{
					mtAlerts_Alert(1, Rsc_WrkString(AL_NO_SCRAP));
					return FALSE;
				}
			} else
			{
				Clip_ScrapClear("", SCRAP_TXT);
			}
			Fseek(0L, fHdl, SEEK_END);
		} else
		{
			Clip_ScrapClear("", "");
			fHdl = (FILENO)Fcreate(fullName, 0);
			if (fHdl < 0)
			{
				mtAlerts_Alert(1, Rsc_WrkString(AL_NO_SCRAP));
				return FALSE;
			}
		}
		/* Nun ist das Clipboardfile offen */
		if (ed->block)
		{
			for (i = 0; i < maxBlocks; i++)
			{
				if (ed->blocks[i].blockStart.line >= 0 && ed->blocks[i].blockEnd.line >= 0)
				{
					EditFuncs_outputText(ed,
						ed->blocks[i].blockStart.line, ed->blocks[i].blockStart.row,
						ed->blocks[i].blockEnd.line, ed->blocks[i].blockEnd.row,
						fHdl, ed->umbruch, FALSE);
				}
			}
		} else
		{
			EditFuncs_outputText(ed, 0, 0, -1, 0, fHdl, ed->umbruch, FALSE);
		}
		(void)Fclose(fHdl);

		/* Clipboard-Modul Bescheid geben, das was drin ist */
		Clip_ScrapWritten(SCF_TEXT, ".TXT");
		Protokoll_SendPathUpdate(ScrapPath);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
LOCAL _BOOL PrinterErrorProc(tResponse errCode)
{
	_BOOL cont;

	cont = TRUE;
	switch (errCode)
	{
	case Printer_done:
		break;
	case Printer_notdone:
		cont = FALSE;
		break;
	case Printer_noMemory:
		break;							/* werden woanders behandelt */
	case Printer_CFGnotFound:
		break;							/* werden woanders behandelt */
	case Printer_wrongIdent:
		break;							/* werden woanders behandelt */
	case Printer_ErrorInStreams:
		mtAlerts_Alert(1, Rsc_WrkString(AL_PRINT_DOS_ERR));
		cont = FALSE;
		break;
	case Printer_Cancelled:
		mtAlerts_Alert(1, Rsc_WrkString(AL_PRINT_CANCEL));
		cont = FALSE;
		break;
	case Printer_PrnDeviceBusy:
	case Printer_AuxDeviceBusy:
	case Printer_ConDeviceBusy:
		cont = mtAlerts_Alert(1, Rsc_WrkString(AL_PRINT_BUSY)) == 1;
		break;
	}
	return cont;
}

/*** ---------------------------------------------------------------------- ***/

/* Druckt entweder den ganzen Text oder nur den markierten Block */
	/*DYJ 5.21b */
GLOBAL _VOID Editor_EditPrint(HWND wdw, _BOOL prBlock, _BOOL askCancel)
{
	EDITOR *ed;
	FILENO fHdl = 0;
	_UBYTE alt[512];
	_WORD val;
	_UWORD effects;
	_BOOL formFeed;
	_BOOL boolVal;
	_UBYTE s[256];
	EDCOLUMN margin;
	_UBYTE marginStr[80];
	_WORD i;
	_WORD select;
	tResponse Response;
	
	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		ConfVars_GetConfDefBool(cPrintFormFeed, &formFeed, FALSE);
		if(!askCancel)
		{
			if(!formFeed)
				formFeed=mtAlerts_Alert(1, Rsc_WrkString(AL_FORM_FEED))==1;
		}
		else
		{
			if (prBlock)
				strBcpy(alt,Rsc_WrkString( AL_BLOCK_PRINT2));
			else
				strBcpy(alt, Rsc_WrkString(AL_PRINT_ALL));
			if (!formFeed)
			{
				strBcat(alt, Rsc_WrkString(blockButF));
				select = mtAlerts_Alert(2, alt);
			} else
			{
				strBcat(alt, Rsc_WrkString(blockBut));
				select = mtAlerts_Alert(1, alt)+1;
			}
			switch(select)
			{
				case 1:
					formFeed = TRUE;
					break;
				case 3:
					return;
			}
		}
		MouseBusy(); /* from mtAppl */
		/* Drucker oeffnen */
		ConfVars_GetConfDefInt(cPrintPort, &val, 0);
		if (val == 0)
			Response = Printer_Open(cPrn);
		else
			Response = Printer_Open(cAux);
		if (Response == Printer_done)
		{
			ConfVars_GetConfDefBool(cPrintNLQ, &boolVal, FALSE);
			Printer_SendInit(boolVal, formFeed);
		}
		ConfVars_GetConfDefInt(cPrintStyle, &val, 0);
		Response = Printer_SetMode(val);
		/* Nun ist der Drucker offen */
		/* Jetzt den Text drucken */
		if (prBlock)
		{
			effects = cFat;
			if (Response == Printer_done)
				Response = Printer_Effect(effects, TRUE);
			if (Response == Printer_done)
				Response = Printer_Effect(effects, FALSE);
			for (i = 0; i < maxBlocks; i++)
			{
				if (ed->blocks[i].blockStart.line >= 0 && ed->blocks[i].blockEnd.line >= 0)
				{
					if (Response == Printer_done)
						if (!EditFuncs_outputText(ed,
							ed->blocks[i].blockStart.line, ed->blocks[i].blockStart.row,
							ed->blocks[i].blockEnd.line, ed->blocks[i].blockEnd.row,
							fHdl, FALSE, TRUE))
							Response = Printer_notdone;
				}
			}
		} else
		{
			/* Header nur drucken, wenn alles gedruckt wird */
			/* Init fuer Header durchfuehren */
			effects = 0;
			ConfVars_GetConfDefInt(cPrintHeaderType, &val, 0);
			switch (val)
			{
			case 0:
				break;
			case 1:
				effects |= cFat;
				break;
			default:
				effects |= 1 << val;
				break;
			}
			if (Response == Printer_done)
				Response = Printer_Effect(effects, TRUE);
			/* erste Zeile ausgeben */
			if (ed->userPrint != FUNK_NULL &&
				ed->userPrint(wdw, LFIRST, s, sizeof(s)))
			{
				ConfVars_GetConfDefInt(cPrintLeftMargin, &margin, 0);
				if (margin > 0)
				{
					if (margin < 0)
						margin = 0;
					if (margin >= sizeof(marginStr))
						margin = (EDCOLUMN)sizeof(marginStr) - 1;
					MemSet(marginStr, ' ', margin);
					marginStr[margin] = '\0';
				}
				do
				{
					if (margin > 0)
						if (Response == Printer_done)
							Response = Printer_WriteString(marginStr);
					if (Response == Printer_done)
						Response = Printer_WriteString(s);
					if (Response == Printer_done)
						Response = Printer_WriteLn();
				} while (ed->userPrint(wdw, LNEXT, s, sizeof(s)));
			}
			/* Texteffekte zuruecksetzen */
			if (Response == Printer_done)
				Response = Printer_Effect(effects, FALSE);
			if (Response == Printer_done)
				if (!EditFuncs_outputText(ed, 0, 0, -1, 0, fHdl, FALSE, TRUE))
					Response = Printer_notdone;
		}
		if (formFeed)
		{
			if (Response == Printer_done)
				Response = Printer_FormFeed();
			if (Response == Printer_done)
				Printer_SendExit();
		} else
		{
			ConfVars_GetConfDefInt(cPrintEmptyLines, &margin, 0);
			for (val = 0; val < margin; val++)
			{
				if (Response == Printer_done)
					Response = Printer_WriteLn();
			}
		}
		Printer_Close();
		MouseArrow(); /* from mtAppl */
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_EditClear(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		EditTools_ClearBlock(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID handleUpdate(HWND wdw, _VOID *special, _BOOL update)	
{
	EDITOR *ed;

	UNUSED(wdw);
	ed = special;
	if (update)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
	} else
	{
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID docChanged(HWND wdw, _VOID *special, CONST LGRECT *doc, _BOOL slided)	
{
	EDITOR *ed;
	EDLINE oldLine;
	EDCOLUMN prefRow;
	
	UNUSED(wdw);
	ed = special;
	if (doc->y < 0)
	{
	}
	oldLine = ed->StartLine;
	ed->leftOffset = (EDCOLUMN)(doc->x);
	EditTools_SetStartLine(ed, doc->y, FALSE);
	if (slided)
	{
		EditTools_SetCurrLine(ed, ed->currLineNr + (ed->StartLine - oldLine));
		ed->currRow = Min(ed->prefRow, ed->editLen);
		prefRow=ed->prefRow;
		EditGlobals_FixCursorPos(ed);
		ed->prefRow=prefRow;
	}
}

/*** ---------------------------------------------------------------------- ***/

	
LOCAL _VOID getScrollRect(HWND wdw, HVDI vdiH, _VOID *special, GRECT *work)
{
	EDITOR *ed;
	GRECT full;
	_BOOL fullRedraw;
	_UBYTE varName[256];
	_BOOL lstPgF;

	ed = special;
	if (ed->hasUserProc)
	{
		ed->getUserRect(wdw, work, &ed->userRect, &ed->editWork);
		/* Ist etwas unsauber, da angenommen wird, dass das UserRect am oberen Rand ist! */
		ed->hOffset = (ed->userRect.g_y - ed->editWork.g_y) % ed->charHeight;

		/* Hier muss nicht gesnapt werden, das wird in WindowSnap gemacht! */
		*work = ed->editWork;
	} else
	{
		ed->editWork = *work;
	}
	ed->windLines = (Min(work->g_y + work->g_h - 1, MaxHeight - 1) - work->g_y + 1) / ed->charHeight;
	ConfVars_GetConfDefBool(cEditLstPgF, &lstPgF,TRUE);
	WdwManager_SetScrollParms(wdw, ed->editWork.g_w - ed->charWidth, ed->windLines - 1, ed->charWidth, 1,lstPgF);
	/* Umbruch */
	if (ed->wdwIsOpen && ed->readOnly!=MODE_RW)
	{
		fullRedraw = EditUtil_WrapText(ed, ed->isEnriched);
	} else
	{
		fullRedraw = FALSE;
	}
	/* Parameter fuer Cursor setzen */
	if (wdw == theTopWindow)
	{
		WdwManager_SetClip(cursorHandle, work, TRUE);
	}
	WdwManager_SetClip(vdiH, work, TRUE);
	EditGlobals_SetDocument(ed);
	if (!ed->alienCoords && wdw >= 0)
	{
		make_varname(varName, sizeof(varName), cEditWindowPos, ed->number);
		WdwManager_GetWdwSize(wdw, &full);	/* Groesse des Fensters abfragen */
		ConfVars_SetConfigRect(varName, &full);
	}
	if (fullRedraw)
	{
		WdwManager_FullRedrawWdw(wdw);
	}
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID snapWindow(HWND wdw, _VOID *special, GRECT *work)	
{
#define minWindWidth  80
#define minWindHeight 80
	_BOOL lstPgF;
	EDITOR *ed;

	ed = special;
	if (ed->hasUserProc)
	{
		ed->getUserRect(wdw, work, &ed->userRect, &ed->editWork);
		/* Ist etwas unsauber, da angenommen wird, dass das UserRect am oberen Rand ist! */
		ed->hOffset = (ed->userRect.g_y - ed->editWork.g_y) % ed->charHeight;
	} else
	{
		ed->hOffset = 0;
	}
	/* snappen */
	work->g_h += ed->charHeight / 2;
	work->g_w += ed->charWidth / 2;		/* Um Rundungsfehler zu vermeiden */
	work->g_h -= Abs(ed->hOffset);
	work->g_w -= pixOff;
	work->g_h = (work->g_h / ed->charHeight) * ed->charHeight + Abs(ed->hOffset);
	work->g_w = (work->g_w / ed->charWidth) * ed->charWidth + pixOff;
	while (work->g_w < minWindWidth)
		work->g_w += ed->charWidth;
	while (work->g_h < minWindHeight)
		work->g_h += ed->charHeight;
	if (ed->hasUserProc)
	{
		ed->getUserRect(wdw, work, &ed->userRect, &ed->editWork);
	} else
	{
		ed->editWork = *work;
	}
	if (wdw == theTopWindow)
	{
		WdwManager_SetClip(cursorHandle, work, TRUE);
	}
	ed->windLines = (Min(ed->editWork.g_y + ed->editWork.g_h - 1, MaxHeight - 1) - ed->editWork.g_y + 1) / ed->charHeight;
	ConfVars_GetConfDefBool(cEditLstPgF, &lstPgF,TRUE);
	WdwManager_SetScrollParms(wdw, ed->editWork.g_w - ed->charWidth, ed->windLines - 1, ed->charWidth, 1,lstPgF);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL topEditor(HWND wdw, _VOID *special)	
{
	EDITOR *ed;

	ed = special;
	if (ed->isHidden)
		return TRUE;
	WdwManager_SetClip(cursorHandle, &ed->editWork, TRUE);
	EditGlobals_ClipWork(ed);
	EditGlobals_HideMouse();
	EditGlobals_ForceCursor(ed);
	EditGlobals_ShowMouse(FALSE);
	if (ed->hasUserProc && ed->userTop != FUNK_NULL)
	{
		ed->userTop(wdw);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID untopEditor(HWND wdw, _VOID *special)	
{
	EDITOR *ed;

	ed = special;
	if (ed->isHidden)
		return;
	if (EditGlobals_CursorIsOn(ed))
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
	WdwManager_SetClip(cursorHandle, &deskSize, TRUE);
	if (ed->hasUserProc && ed->userUntop != FUNK_NULL)
	{
		ed->userUntop(wdw);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Ein Timerevent fuer dieses Fenster ist aufgetreten */
LOCAL _BOOL timerEditor(HWND wdw, _VOID *special)		
{
	EDITOR *ed;

	ed = special;
	if (ed->isHidden)
	{
		return FALSE;
	}
	if (ed->showInfo && wdw == ownTopWindow)
	{
		EditTools_SetInfoLine(ed);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Das Fenster wird evtl. versteckt */
LOCAL _VOID hideEditor(HWND wdw, _VOID *special, _BOOL hide)	
{
	EDITOR *ed;

	UNUSED(wdw);
	ed = special;
	ed->isHidden = hide;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Drag && Drop Support */

LOCAL _BOOL closeEditor(HWND wdw, HVDI vdiH, _VOID *special, _BOOL force);


typedef _UBYTE editExtsType[DD_NUMEXTS][DD_EXTLEN];

#define editExts ".ASC.TXT.CSVARGS\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"
#define DDEXT_ASC 0
#define DDEXT_TXT 1
#define maxSendExts   2
#define DDEXT_CSG 2
#define DDEXT_ARGS 3
#define maxUserExts   (DD_NUMEXTS - maxSendExts)

LOCAL _WORD ddMaxSendExts;

LOCAL _VOID editGetExts(HWND wdw, _VOID *special, _UBYTE *exts, size_t bufsize)
{
	EDITOR *ed;
	size_t max;
	editExtsType supportedExts;
	_WORD i;
	_WORD additional;

	ed = special;
	MemCpy(&supportedExts[0][0], editExts, sizeof(supportedExts));
	if (ed->userDD)
	{
		additional = ed->userDDGetMaxExts(wdw);
		additional = Min(maxUserExts, additional);
		for (i = 0; i < additional; i++)
			ed->userDDGetExt(wdw, i, supportedExts[ddMaxSendExts + i]);
	}
	max = DD_EXTSIZE;
	if (max > bufsize)
		max = bufsize;
	MemCpy(exts, &supportedExts[0][0], max);
#if DD_TEST
	printf("editGetExts: %s\n", exts);
#endif
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL editAcceptData(HWND wdw, _VOID *special, _VOID *hdr)
{
	struct
	{
		_UBYTE dtype[DD_EXTLEN];
		_ULONG dlen;
		_UBYTE dname[1];
	} *header;
	_WORD i;

	UNUSED(wdw);
	UNUSED(special);
	header = hdr;
	/* Mal nachsehen, ob die Daten in dem von uns unterstuetzten Format vorhanden
	 * sind
	 */
	i = 0;
	while (i < DD_EXTSIZE && MemCmp(header->dtype, &editExts[i], DD_EXTLEN) != 0)
	{
		i += DD_EXTLEN;
	}
#if DD_TEST
	printf("editAcceptData: %c%c%c%c, size %ld: %s\n",
		header->dtype[0], header->dtype[1], header->dtype[2], header->dtype[3],
		header->dlen,
		i < DD_EXTSIZE ? "yes" : "no");
#endif
	return i < DD_EXTSIZE;
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID editWriteData(HWND wdw, _VOID *special, _VOID *data, _ULONG dlen)
{
	EDITOR *ed;
	_BOOL wasBlock;
	ErrReason err;
	_UBYTE *ch;

	ed = special;
	ch = data;
#if DD_TEST
	{
		_ULONG i;

		printf("editWriteData: len=%lu, data: \"", dlen);
		for (i = 0; i < dlen; i++)
		{
			if (ch[i] < 0x20)
				printf("\\%o", ch[i]);
			else
				putchar(ch[i]);
		}
		printf("\"\n");
	}
#endif
	if (dlen >= 1)
	{
		ch += dlen - 1;
		if (ch[0] == '\0')
		{
			--ch;
			--dlen;
		}
		if (dlen >= 1)
		{
			if (ch[0] == LF && (dlen == 1 || ch[-1] != CR))
			{
				/* Ein einzelnes LF am Ende */
				--dlen;
			}
		}
	}
	if (dlen == 0)
		return;
	wasBlock = ed->block;
	if (ed->block)
	{
		EditTools_SaveOp(ed, blockPaste, FALSE);
		EditFuncs_cutBlock(ed, FALSE, FALSE);
		ed->blocks[0].blockStart.line = ed->currLineNr;
		ed->blocks[0].blockStart.row = ed->currRow;
		ed->block = TRUE;
	} else
	{
		EditTools_SetBlockstart(ed);
		EditTools_SaveOp(ed, normalPaste, FALSE);
		ed->block = TRUE;
	}
	if (!EditFuncs_ReadBuffer(ed, (_UBYTE **)&data, dlen, FALSE, &err))
	{
	}
	EditTools_ShowFileChanged(ed);
	EditTools_NewBlockToUndoBuffer(ed);
	/* Jetzt Cursorposition anpassen an Ende des gepasteten Stuecks
	 */
	EditTools_SetCurrLine(ed, ed->blocks[0].blockEnd.line);
	ed->prefRow = ed->currRow = ed->blocks[0].blockEnd.row;
	EditGlobals_SetDocument(ed);
	if (!wasBlock)
	{
		/* normal Paste */
		ed->block = FALSE;
		ed->blocks[0].blockStart.line = -1;
		ed->blocks[0].blockEnd.line = -1;
	}
	if (ed->showInfo)
	{
		EditTools_SetInfoLine(ed);
	}
	WdwManager_FullRedrawWdw(wdw);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL editGetHeader(HWND wdw, _VOID *special, _WORD currentExt, _UBYTE *ext, _UBYTE *name, size_t namesize, _ULONG *size)
{
#define cBlockName  " Editorblock"
	EDITOR *ed;
	_UBYTE *data;
	editExtsType supportedExts;
	_WORD additional;
	_WORD i;

	ed = special;
	if (!ed->block)
	{
		if (!ed->userDD)
			return FALSE;
		ddMaxSendExts = 0;
	} else
	{
		ddMaxSendExts = maxSendExts;
	}
	if (currentExt >= ddMaxSendExts)
	{
		if (ed->userDD)
		{
			additional = ed->userDDGetMaxExts(wdw);
			additional = Min(maxUserExts, additional);
			if (currentExt >= ddMaxSendExts + additional)
			{
				return FALSE;
			}
			for (i = 0; i < additional; i++)
				ed->userDDGetExt(wdw, i, supportedExts[ddMaxSendExts + i]);
			MemCpy(ext, supportedExts[currentExt], DD_EXTLEN);
			ed->userDDGetExtName(wdw, currentExt - ddMaxSendExts, name, namesize);
			*size = ed->userDDGetSize(wdw, currentExt - ddMaxSendExts);
#if DD_TEST
			printf("editGetHeader(user): size = %lu\n", *size);
#endif
			return TRUE;
		} else
		{
			return FALSE;
		}
	}
	MemCpy(ext, &editExts[currentExt * DD_EXTLEN], DD_EXTLEN);
	strMcpy(name, namesize, ProgramName);
	strMcat(name, namesize, cBlockName);
	if (!Editor_GetBlock(wdw, currentExt == DDEXT_ASC, currentExt == DDEXT_TXT, &data, size))
		return FALSE;
	if (data != NULL)
		SFREE(data);
#if DD_TEST
	printf("editGetHeader: %c%c%c%c size = %lu\n", ext[0], ext[1], ext[2], ext[3], *size);
#endif
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID editReadData(HWND wdw, _VOID *special, _VOID **data, _WORD currentExt)
{
	EDITOR *ed;
	_ULONG size;
	
	ed = special;
	if (currentExt < ddMaxSendExts)
	{
		Editor_GetBlock(wdw, currentExt == DDEXT_ASC, currentExt == DDEXT_TXT, (_UBYTE **)data, &size);
#if DD_TEST
		{
			_ULONG i;
			_UBYTE *ch;
			
			printf("editReadData: ext=%d, len=%lu, data: \"", currentExt, size);
			ch = *data;
			for (i = 0; i < size; i++)
			{
				if (ch[i] < 0x20)
					printf("\\%o", ch[i]);
				else
					putchar(ch[i]);
			}
			printf("\"\n");
		}
#endif
	} else
	{
		if (ed->userDD)
		{
			ed->userDDGetData(wdw, currentExt - ddMaxSendExts, data);
		}
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Basisfunktionen */

LOCAL _VOID SetFont(EDITOR *ed, _WORD font, _WORD fontSize);

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL Editor_ChangeFont(HWND wdw, _WORD font, _WORD fontSize, _BOOL isMono)
{
	UNUSED(isMono);
	if (EditFuncs_FindEditor(wdw) != NULL)
	{
		Editor_SelectEditFont(wdw, font, fontSize);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL Editor_ChangeColor(HWND wdw, _WORD color)
{
	if (EditFuncs_FindEditor(wdw) != NULL)
	{
		ConfVars_SetConfigInt(cEditTextCol, color);
		Editor_RedrawAll();
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20j */
LOCAL _VOID Editor_GetSettings(EDITOR *ed)
{
	_WORD viewQuoteEff,viewQuoteEff2;
	_WORD viewLinkEff,txtEffs[]={TXT_NORMAL,TXT_THICKENED,TXT_SKEWED,TXT_LIGHT};
	_BOOL wasTmp;
	
	wasTmp = ConfVars_ConfigConfigModule(FALSE);

	if (ed->readOnly!=MODE_RW)
	{
		ConfVars_GetConfDefInt(cMsgTextCol, &ed->textCol, BLACK);
		ConfVars_GetConfDefInt(cMsgBackCol, &ed->backCol, WHITE);
		ConfVars_GetConfDefInt(cMsgQuoteCol, &ed->quoteCol, BLACK);
		ConfVars_GetConfDefInt(cMsgQuoteEffect, &viewQuoteEff, 1);
		if(viewQuoteEff>-1 && viewQuoteEff<4)
			ed->quoteEff = txtEffs[viewQuoteEff];
		ConfVars_GetConfDefInt(cMsgQuoteCol2, &ed->quoteCol2, ed->quoteCol);
		ConfVars_GetConfDefInt(cMsgQuoteEffect2, &viewQuoteEff2, 1);
		if(viewQuoteEff2>-1 && viewQuoteEff2<4)
			ed->quoteEff2 = txtEffs[viewQuoteEff2];
		ConfVars_GetConfDefInt(cMsgLinkCol, &ed->linkCol, BLUE);
		ConfVars_GetConfDefInt(cMsgLinkEffect, &viewLinkEff, 1);
		if(viewLinkEff>-1 && viewLinkEff<5)
			ed->linkEff = txtEffs[viewLinkEff];
	} else
	{
		ConfVars_GetConfDefInt(cEditTextCol, &ed->textCol, BLACK);
		ConfVars_GetConfDefInt(cEditBackCol, &ed->backCol, WHITE);
		ed->quoteCol = ed->quoteCol2 = ed->textCol;
		ed->quoteEff = ed->quoteEff2 = TXT_NORMAL;
	}
	/* Jetzt ggf. Farben bei monochrom ignorieren und nur von Hintergrundfarbe
	 * ausgehen
	 */
	if (Bitplanes == 1)
	{
		if (ed->textCol != WHITE)
		{
			ed->backCol = WHITE;
		} else
		{
			if (ed->backCol == WHITE)
				ed->backCol = BLACK;
		}
		ed->quoteCol = ed->quoteCol2 = ed->textCol;
	}
	ConfVars_ConfigConfigModule(wasTmp);
}

/*** ---------------------------------------------------------------------- ***/

/* oeffnet einen Editor im Fenster. Font und Fontsize und gewuenschte
 * Workarea muessen uebergeben werden, wdwHandle dieses Editors wird
 * im Erfolgsfall zurueckgeliefert.
 */

GLOBAL _BOOL Editor_OpenEditor(
	CONST _UBYTE *path,
	CONST _UBYTE *FileName,
	_VOID /* List */ *SrcPaths,
	_BOOL New,
	_BOOL useUserProc,
	userProcType theUserProc,
	userGetRectProc theUserRectProc,
	userDrawProc theDrawUserProc,
	userCloseProc theCloseProc,
	userKeyProc theKeyProc,
	userTopProc theTopProc,
	userUntopProc theUntopProc,
	userPrePrintProc thePrePrintProc,
	EDCOLUMN right,
	_BOOL showInfoLine,
	_BOOL wrap,
	_WORD num,
	HWND *wdwHandle)
{
	aLineDesc fn;
	FILENO fHdl;
	_ULONG flen;
	EDITOR *ed;
	_UWORD comps;
	_UBYTE *readBuf;
	_BOOL fastRead;
	_UBYTE varName[256];
	_WORD edFont;
	_WORD edFontSize;
	GRECT full;
	EDLINE cLine;
	_BOOL lstPgF;

/* Dealloziert allozierte Pointer und gibt sonstiges Zeugs frei
 */
#define FreeStuff() \
	if (*wdwHandle >= 0) { WdwManager_CloseWindow(*wdwHandle, TRUE); *wdwHandle = -1; } \
	else if (ed != NULL) EditFuncs_KillEditor(ed); \
	if (readBuf != NULL) SFREE(readBuf); \
	if (fHdl >= 0) F_ErrorAlert(Fclose(fHdl)); \
	if (fn.text != NULL) SFREE(fn.text)

	/* Variablen initialisieren, damit FreeStuff auch funktioniert! */
	ed = NULL;
	readBuf = NULL;
	fn.text = NULL;
	*wdwHandle = -1;
	fHdl = -1;
	flen = 0;
	/* Filenamen basteln, Datei oeffnen und Laenge feststellen */
	if (!EditUtil_getFileParms(path, FileName, SrcPaths, New, TRUE, &fn, &flen, &fHdl))
	{
		FreeStuff();
		return FALSE;
	}
	if (flen == 0)
		New = TRUE;
	/* Speicher feststellen und auf flen testen */
	if (!EditUtil_testMem(flen, &fastRead, &readBuf))
	{
		FreeStuff();
		return FALSE;
	}
	/* Fensterkomponenten bestimmen */
	comps =	NAME | CLOSER | FULLER | MOVER | SIZER | UPARROW | DNARROW | VSLIDE | LFARROW | RTARROW | HSLIDE;
	if (showInfoLine)
		comps |= INFO;
	/* Editor erstellen */
	if ((ed = EditFuncs_CreateEditor()) == NULL)
	{
		/* Fehlermeldung: not enough memory */
		FreeStuff();
		return FALSE;
	}

	/* Configvariablen lesen */
	if (num >= 0 && !setin(nums, num))
	{
		setincl(nums, num);
		ed->number = num;
	} else
	{
		ed->number = FindNum();
	}
	make_varname(varName, sizeof(varName), cEditWindowPos, ed->number);
	if (!ConfVars_GetConfigRect(varName, &full))
	{
		num = Max(ed->number - 1, 0);
		make_varname(varName, sizeof(varName), cEditWindowPos, num);
		ConfVars_GetConfDefRect(varName, &full, &deskSize);
		if (ed->number > 0 && !rc_equal(&full, &deskSize))
		{
			full.g_x += 2 * CharWidth;
			full.g_y += CharHeight;
		}
		make_varname(varName, sizeof(varName), cEditFont, num);
		ConfVars_GetConfDefInt(varName, &edFont, 1);
		make_varname(varName, sizeof(varName), cEditFontSize, num);
		ConfVars_GetConfDefInt(varName, &edFontSize, 10);
	} else
	{
		make_varname(varName, sizeof(varName), cEditFont, ed->number);
		ConfVars_GetConfDefInt(varName, &edFont, 1);
		make_varname(varName, sizeof(varName), cEditFontSize, ed->number);
		ConfVars_GetConfDefInt(varName, &edFontSize, 10);
	}
	/* Editorvariablen setzen */
	ed->fileName = fn;
	fn.text = NULL;
	ed->new = New;
	ed->rightMargin = right;
	ed->showInfo = showInfoLine;
	ConfVars_GetConfDefBool(cEditAutoIndent, &ed->autoIndent, TRUE);
	ConfVars_GetConfDefBool(cHardTabs, &ed->realTabs, FALSE);
	ConfVars_GetConfDefBool(cShowLineEnds, &ed->EditShowLineEnds, TRUE);
	/* Jetzt noch die User-Parameter */
	ed->hasUserProc = useUserProc;
	ed->userProc = theUserProc;
	ed->getUserRect = theUserRectProc;
	ed->drawUserArea = theDrawUserProc;
	ed->userClose = theCloseProc;
	ed->userKey = theKeyProc;
	ed->userTop = theTopProc;
	ed->userUntop = theUntopProc;
	ed->userPrint = thePrePrintProc;
	
	Editor_GetSettings(ed);
	
	/* Fenster mit Window Library oeffnen */
	if (!WdwManager_OpenWindow(
		Editor_ClickInEditWindow,
		Editor_EditChar, timerEditor,
		&deskSize, &full, comps, TRUE, "", "",
		snapWindow, closeEditor, EditDraw_redrawWdw, topEditor, untopEditor,
		handleUpdate, docChanged, getScrollRect, hideEditor,
		pixOff,
		ed,		/* special */
		FALSE,	/* onlyThisTop */
		TRUE,	/* showIt */
		TRUE,	/* supportIconify */
		TRUE,	/* openWorkstation */
		FALSE,	/* hasIdem*/
		&ed->wdw, &ed->hdl))
	{
		/* Kein Fenster oder keine Workstation mehr frei! */
		mtAlerts_Alert(1, Rsc_WrkString(AL_NO_EDIT_OPEN));
		ClearNum(ed->number);
		FreeStuff();
		return FALSE;
	}
	/* VDI-Operationen fuer neuen Editor */
	*wdwHandle = ed->wdw;
	ed->isHidden = FALSE;
	/* Text einlesen */
	ed->umbruch = wrap;
	if (flen != 0)
	{
		ed->currRow = 0;
		ed->cursor = 1;
		if (!EditFuncs_readText(ed, fastRead, flen, &fHdl, right, &readBuf, FALSE, TRUE))
		{
			ClearNum(ed->number);
			FreeStuff();
			return FALSE;
		}
	} else
	{
		/* neuen Text anlegen */
		if (!EditBase_GetBuffer(ed, 0))
		{
			ClearNum(ed->number);
			FreeStuff();
			return FALSE;
		}
		ed->firstBuff->lines = MALLOC(200L * sizeof(EDCOLUMN), "OpenEditor: lines");
		if (ed->firstBuff->lines == NULL)
		{
			ClearNum(ed->number);
			FreeStuff();
			return FALSE;
		} else
		{
			MemSetZero(ed->firstBuff->lines, 200 * sizeof(EDCOLUMN));
			ed->firstBuff->maxLine = 200;
			ed->firstBuff->fromLine = 0;
			ed->firstBuff->toLine = 0;
		}
		ed->totalLineNr = 1;
		ed->editLine.text[0] = CR;
		ed->editLine.text[1] = LF;
		ed->editLine.text[2] = '\0';
		ed->editLen = 2;
		ed->editChanged = TRUE;
		ed->currLineNr = 0;
		EditBase_PutCurrLine(ed);
	}
	/* Wenn wir bis hierhin gekommen sind, dann kann man auch editieren. */
	/* Fonts laden */
	FontSelect_LoadFonts(ed->hdl);

	/* Font setzen */
	SetFont(ed, edFont, edFontSize);
	ed->font = edFont;
	ed->fontSize = edFontSize;
	ed->wasOpen = TRUE;
	vsf_style(ed->hdl, IP_SOLID);
	vsf_interior(ed->hdl, FIS_SOLID);
	/* Schreibmodus setzen */
	vswr_mode(ed->hdl, MD_REPLACE);

	ed->editChanged = FALSE;
	/* Und jetzt noch eventuell vorhandene Configvariablen wieder einlesen */
	make_varname(varName, sizeof(varName), cEditTabs, ed->number);
	ConfVars_GetConfDefBool(varName, &ed->realTabs, ed->realTabs);
	ConfVars_DeleteConfigVar(varName);
	make_varname(varName, sizeof(varName), cEditShowLineEnds, ed->number);
	ConfVars_GetConfDefBool(varName, &ed->EditShowLineEnds, ed->EditShowLineEnds);
	ConfVars_DeleteConfigVar(varName);
	make_varname(varName, sizeof(varName), cEditTabSize, ed->number);
	ConfVars_GetConfDefInt(varName, &ed->TabSize, 4);
	ConfVars_DeleteConfigVar(varName);
	make_varname(varName, sizeof(varName), cEditLine, ed->number);
	ConfVars_GetConfDefLongInt(varName, &ed->currLineNr, ed->currLineNr);
	ConfVars_DeleteConfigVar(varName);
	make_varname(varName, sizeof(varName), cEditRow, ed->number);
	ConfVars_GetConfDefInt(varName, &ed->currRow, ed->currRow);
	ConfVars_DeleteConfigVar(varName);
	ConfVars_GetConfDefBool(cAllowLowerQuoteInitials, &ed->AllowLowerQuoteInitials, TRUE);
	cLine = LMin(ed->currLineNr, ed->totalLineNr - 1);
	cLine = LMax(cLine, 0);
	ed->currLineNr = -1;
	EditTools_SetCurrLine(ed, cLine);
	ed->cursor = 1;
	ed->prefRow = ed->currRow = Max(Min(ed->currRow, ed->editLen), 0);
	EditGlobals_FixCursorPos(ed);

	/* Drag und Drop Support einschalten */
	WdwManager_WdwInstallDDClient(ed->wdw, editGetExts, editAcceptData, editWriteData);
	WdwManager_WdwInstallDDServer(ed->wdw, editGetHeader, editReadData);
	
	WdwManager_AcceptFont(ed->wdw, Editor_ChangeFont);
	WdwManager_AcceptColor(ed->wdw, Editor_ChangeColor);
	/* Fensterinhalte setzen */
	EditTools_SetEditTitle(ed);
	if (ed->showInfo)
	{
		EditTools_SetInfoLine(ed);
	}
	/* Fenstergroesse berechnen */
	WdwManager_SetWdwSize(ed->wdw, &full);
	WdwManager_SetDocumentParms(ed->wdw, 1, ed->charHeight);
	ConfVars_GetConfDefBool(cEditLstPgF, &lstPgF,TRUE);
	WdwManager_SetScrollParms(ed->wdw, ed->editWork.g_w - ed->charWidth, ed->windLines - 1, ed->charWidth, 1,lstPgF);

	/* Document fuer Window setzen */
	EditGlobals_SetDocument(ed);
	/* Clipping setzen fuer Cursorworkstation */
	WdwManager_SetClip(cursorHandle, &ed->editWork, TRUE);
	/* Cursor anzeigen */
	EditDraw_CenterCurrline(ed);
	EditGlobals_ShowCursor(ed);
	ed->wdwIsOpen = TRUE;

#undef FreeStuff
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Installiert den DD-Server der Userprozedur
 */
GLOBAL _VOID Editor_InstallUserDD(
	HWND wdw,
	userDDGetMaxExtProc userDDGetMaxExts,
	userDDGetExtProc userDDGetExt,
	userDDGetExtNameProc userDDGetExtName,
	userDDGetSizeProc userDDGetSize,
	userDDGetDataProc userDDGetData)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		ed->userDD = TRUE;
		ed->userDDGetMaxExts = userDDGetMaxExts;
		ed->userDDGetExt = userDDGetExt;
		ed->userDDGetExtName = userDDGetExtName;
		ed->userDDGetSize = userDDGetSize;
		ed->userDDGetData = userDDGetData;
	}
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
GLOBAL _VOID Editor_EditAbandonText(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (mtAlerts_Alert(2, Rsc_WrkString(AL_EDIT_ABANDON)) == 1)	
			Editor_EditReloadText(wdw);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Fragt nach, ob die letzte Fassung des Textes geladen werden soll und macht das
 * dann
 */

GLOBAL _VOID Editor_EditReloadText(HWND wdw)
{
	EDITOR *ed;
	_UBYTE path[PATH_MAX];
	_UBYTE name[PATH_MAX];
	_ULONG flen;
	_UBYTE *readBuf;
	_BOOL fastRead;
	aLineDesc fn;
	FILENO fHdl;
	_WORD i;
	LGRECT doc;

/* Dealloziert allozierte Pointer und gibt sonstiges Zeugs frei
 */
#define FreeStuff() \
	if (readBuf != NULL) SFREE(readBuf); \
	if (fHdl >= 0) F_ErrorAlert(Fclose(fHdl)); \
	if (fn.text != NULL) SFREE(fn.text)

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->fileName.text == NULL)
		{
			return;
		}
		SplitPath(ed->fileName.text, path, name);
		while (ed->firstBuff != NULL)
		{
			ed->currentBuff = ed->firstBuff;
			EditBase_FreeBuffer(ed);
		}
		SFREE(ed->fileName.text);
		ed->changed = FALSE;
		ed->editChanged = FALSE;
		if (!(							/* Dateinamen basteln, Datei oeffnen und Laenge feststellen */
			EditUtil_getFileParms(path, name, NULL, FALSE, FALSE, &fn, &flen, &fHdl) &&
			/* Speicher feststellen und auf flen testen */
			 EditUtil_testMem(flen, &fastRead, &readBuf) &&
			/* Text einlesen und einfuegen */
			 EditFuncs_readText(ed, fastRead, flen, &fHdl, ed->rightMargin, &readBuf, FALSE, TRUE)
			))
		{
			FreeStuff();
			WdwManager_CloseWindow(ed->wdw, FALSE);
			EditGlobals_ShowCursor(ed);
			EditGlobals_ShowMouse(FALSE);
			return;
		}
		ed->changed = FALSE;
		ed->fileName = fn;
		ed->currLineNr = -1;
		ed->editChanged = FALSE;
		ed->StartLine = 0;
		EditTools_SetCurrLine(ed, 0);
		ed->prefRow = ed->currRow = 0;
		ed->leftOffset = 0;
		ed->maxWidth = 0;
		for (i = 0; i < maxBlocks; i++)
		{
			ed->blocks[i].blockStart.line = -1;
			ed->blocks[i].blockEnd.line = -1;
		}
		ed->block = FALSE;
		EditTools_SetEditTitle(ed);
		EditTools_SetInfoLine(ed);
		doc.x = ed->leftOffset;
		doc.y = ed->StartLine;
		doc.w = ed->maxWidth;
		doc.h = ed->totalLineNr - 1;
		WdwManager_SetNewDocument(wdw, &doc, TRUE);
		EditDraw_CenterCurrline(ed);
		WdwManager_FullRedrawWdw(ed->wdw);
	}
#undef FreeStuff
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_EditMerge(HWND wdw, CONST _UBYTE *path, CONST _UBYTE *name)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->block)
			EditTools_ClearBlock(ed);
		Editor_EditPasteFile(wdw, FALSE, path, name);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* nachfragen, wenn File schon vorhanden */
	
GLOBAL _BOOL Editor_EditSave(HWND wdw, CONST _UBYTE *path, CONST _UBYTE *FileName, _BOOL takeName, _BOOL saveBlock)
{
	EDITOR *ed;
	_BOOL saveas;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		saveas = path != NULL && path[0] != '\0' && FileName != NULL && FileName[0] != '\0';
		if (!(saveas || ed->readOnly!=MODE_RW) && !ed->changed)
		{
			return TRUE;
		}
		return EditUtil_SaveText(ed, saveas || ed->readOnly!=MODE_RW, takeName, path, FileName,saveBlock);
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.21b */
LOCAL _WORD askForSave(EDITOR *ed, _BOOL mayCancel)
{
	_UBYTE changedAlt[512];
	_UBYTE tmp[31];
	
	if (mayCancel)
		strBcpy(changedAlt, Rsc_WrkString(AL_TEXT_CHANGED));
	else
		strBcpy(changedAlt, Rsc_WrkString(AL_TEXT_NOT_SAVED));
	if (ed->fileName.length > 30)
	{
		strBcpy(tmp, "...");
		strBcat(tmp, ed->fileName.text + strLen(ed->fileName.text) - 25);
		CatGlobal_ReplaceStr(changedAlt, sizeof(changedAlt), "&", tmp);
	} else
	{
		if (ed->fileName.length == 0)
		{
			CatGlobal_ReplaceStr(changedAlt, sizeof(changedAlt), "&", cNoname);
		} else
		{
			CatGlobal_ReplaceStr(changedAlt, sizeof(changedAlt), "&", ed->fileName.text);
		}
	}
	return mtAlerts_Alert(1, changedAlt);
}

/*** ---------------------------------------------------------------------- ***/

/* Fragt nach, falls der Text veraendert wurde, ob dieser gesichert
 * werden soll.
 */

GLOBAL _BOOL Editor_AskForSave(HWND wdw, _BOOL mayCancel)
{
	EDITOR *ed;
	
	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->changed)
		{
			switch (askForSave(ed, mayCancel))
			{
			case 1:
				/* Text noch speichern */
				return EditUtil_SaveText(ed, ed->fileName.length == 0, FALSE, "", "",TRUE);
			case 2:
				/* nicht speichern */
				break;
			case 3:
				/* Abbruch */
				return FALSE;
			}
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL closeEditor(HWND wdw, HVDI vdiH, _VOID *special, _BOOL force)
{
	EDITOR *ed;
	_UBYTE varName[256];
	GRECT full;
	_WORD select;
	
	UNUSED(special);
	ed = EditFuncs_FindEditor(wdw);
	if (ed == NULL)
		return TRUE;
	if (ed->changed)
	{
		if (!force)
		{
			select = askForSave(ed, TRUE);
		} else
		{
			/* force mode */
			ed->backup = TRUE;
			select = 1;
		}
		if (select == 3)
		{							/* Abbruch gewaehlt */
			return FALSE;
		}
		if (select == 1)
		{
			/* Text noch speichern */
			if (!EditUtil_SaveText(ed, ed->fileName.length == 0, FALSE, "", "",TRUE) && !force)
				return FALSE;
		}
	}
	/* Hier wird jetzt definitiv das Fenster geschlossen und der Editor geloescht! */
	if (ed->userClose != FUNK_NULL)
	{
		if (!ed->userClose(wdw))
			return FALSE;
	}
	WdwManager_AcceptFont(ed->wdw, FUNK_NULL);
	WdwManager_AcceptColor(ed->wdw, FUNK_NULL);
	if (!ed->alienCoords && ed->wasOpen)
	{
		/* Parameter noch sichern */
		make_varname(varName, sizeof(varName), cEditWindowPos, ed->number);
		WdwManager_GetWdwSize(wdw, &full);	/* Groesse des Fensters abfragen */
		ConfVars_SetConfigRect(varName, &full);
		make_varname(varName, sizeof(varName), cEditFont, ed->number);
		ConfVars_SetConfigInt(varName, ed->font);
		make_varname(varName, sizeof(varName), cEditFontSize, ed->number);
		ConfVars_SetConfigInt(varName, ed->fontSize);
		ClearNum(ed->number);
	}
	WdwManager_SetClip(cursorHandle, &deskSize, TRUE);
	FontSelect_UnloadFonts(vdiH);
	EditFuncs_KillEditor(ed);
	EditTools_CheckAndFreeUndo(wdw);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Editor_CloseEditor(HWND wdw, _BOOL force)
{
	return WdwManager_CloseWindow(wdw, force);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID editListMode(EDITOR *ed, _WORD clicks)
{
	/* Jetzt die Zeile markieren, die selektiert wurde */
	EditTools_MarkLine(ed);
	if (ed->listModeProc != FUNK_NULL && clicks > 1)
	{
		/* Prozedur aufrufen mit dem Inhalt der Zeile */
		ed->listModeProc(ed->wdw, ed->editLine.text);
	}
	if (ed->showInfo)
	{
		EditTools_SetInfoLine(ed);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Wertet alle Tastendruecke aus und fuegt ggf. Zeichen ein. */
	
GLOBAL _BOOL Editor_EditChar(HWND wdw, _VOID *special, _WORD taste, _UBYTE ch, _UBYTE scan, _UWORD kstate)		
{
#define Chars " !\"#$%&&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_abcdefghijklmnopqrstuvwxyz{break;}!\204\224\201\216\231\232\236\335\341"
	EDITOR *ed;
	_UWORD gemCh;
	Key key;
	_BOOL ctrl;
	_BOOL done;
	_BOOL moreChars;
	_BOOL insMod;
	_BOOL done2;

	UNUSED(taste);
	ed = special;
	if (WdwManager_WindowIsShaded(wdw))
		return FALSE;
	if (ed != NULL)
	{
#if EDIT_DEBUG >= 2
		if (!validAdr(ed->editLine.text))
			HALT();
#endif /* EDIT_DEBUG >= 2 */
		do
		{
			moreChars = FALSE;
			gemCh = (scan << 8) + ch;

			GemCharToKey(gemCh, kstate, &key);
			if (ed->userKey != FUNK_NULL)
			{
				if (ed->userKey(wdw, &scan, &ch, &kstate, &moreChars))
					return TRUE;
			}
			ctrl = (kstate & K_CTRL) != 0;
			done = TRUE;
			if (IsSpecial(&key))
			{
				EditGlobals_HideMouse();
				switch (SpecialKey(&key))
				{
				case left:
				case wdLeft:
				case soln:
					EditFuncs_CursorLeft(ed, kstate);
					break;
					
				case right:
				case wdRight:
				case eoln:
					EditFuncs_CursorRight(ed, kstate);
					break;
				case pgUp:
					if(WdwManager_PageUp(ed->wdw))
					{
						if (ed->listMode)
							editListMode(ed, 0);
						break;
					}
					ctrl=TRUE;
				case up:
					EditFuncs_CursorUp(ed, ctrl);
					if (ed->listMode)
						editListMode(ed, 0);
					break;
					
				case pgDown:
					if(WdwManager_PageDown(ed->wdw))
					{
						if (ed->listMode)
							editListMode(ed, 0);
						break;
					}
					ctrl=TRUE;
				case down:
					EditFuncs_CursorDown(ed, ctrl);
					if (ed->listMode)
						editListMode(ed, 0);
					break;
					
				case home:
					EditFuncs_Home(ed, FALSE);
					if (ed->listMode)
						editListMode(ed, 0);
					break;
					
				case clr:
					EditFuncs_Home(ed, TRUE);
					if (ed->listMode)
						editListMode(ed, 0);
					break;
				case return_:
				case enter:
					if (ed->listMode)
						editListMode(ed, 2);
					else
						done = FALSE;
					break;
					
				default:
					done = FALSE;
					break;
				}
				if (ed->readOnly!=MODE_RW)
				{
					if (!done)
					{
						if (SpecialKey(&key) == backspace)
						{
							EditTools_ClearBlock(ed);
							done = TRUE;
						}
					}
					if (ed->showInfo)
					{
						EditTools_SetInfoLine(ed);
					}
					EditGlobals_ShowMouse(FALSE);
					return done;
				}
				EditDraw_CenterCurrline(ed);
#if 0
				EditGlobals_ClipWork(ed);
#endif
				done2 = TRUE;
				switch (SpecialKey(&key))
				{
				case esc:
					/* Nur fuer FRED: Kuerzel suchen und auswerten */
					if (!EditFuncs_InsertShortkey(ed))
					{
						EditBase_PutCurrLine(ed);
						WdwManager_FullRedrawWdw(wdw);
					}
					break;
					
				case undo:
					lastOpWasCtrlY = FALSE;
					EditTools_Undo(ed);
					break;
					
				case insert:
					if (ctrl)
					{
						Editor_ToggleMode(wdw, indentMode);
					} else
					{
						Editor_ToggleMode(wdw, insMode);
					}
					break;
					
				case insLine:
					if (ed->block)
					{
						EditGlobals_ShowMouse(FALSE);
						return done;
					}
					lastOpWasCtrlY = FALSE;
					if ((kstate & K_LSHIFT) && (kstate & K_RSHIFT))
					{
						EditFuncs_Format0(ed, FALSE);
					} else if ((kstate & K_SHIFT) && 
							   (kstate & K_CTRL))
					{
						EditGlobals_ShowMouse(FALSE);
						insMod = mtDials_DialCharTable(&ch);
						EditGlobals_HideMouse();
						if (insMod && ch != CR && ch != LF)
						{
							lastOpWasCtrlY = FALSE;
							EditFuncs_Insert(ed, ch);
						}
					} else if (kstate & K_SHIFT)
					{
						EditFuncs_DupLine(ed);
					}
					break;
					
				case backspace:
					lastOpWasCtrlY = FALSE;
					if (ed->insertMode)
					{
						EditGlobals_HideCursor(ed);
						EditFuncs_Backspace(ed, ctrl);
						EditGlobals_ShowCursor(ed);
					} else
					{
						EditFuncs_CursorLeft(ed, kstate);
					}
					break;
					
				case delete:
					EditGlobals_HideCursor(ed);
					if (kstate & K_SHIFT)
					{
						if (ed->block)
						{
							lastOpWasCtrlY = FALSE;
							EditTools_ClearBlock(ed);
						}
						EditBase_PutCurrLine(ed);
						EditTools_MarkLine(ed);
						EditTools_SaveOp(ed, blockCut, FALSE);
						EditFuncs_cutEdit(ed, FALSE);
						EditGlobals_ShowCursor(ed);
						lastOpWasCtrlY = TRUE;
					} else if (ctrl)
					{
						lastOpWasCtrlY = FALSE;
						if (ed->block)
						{
							EditGlobals_ShowMouse(FALSE);
							return done;
						}
						EditFuncs_Delete(ed, TRUE, TRUE);
					} else
					{
						lastOpWasCtrlY = FALSE;
						EditFuncs_Delete(ed, FALSE, TRUE);
					}
					EditGlobals_ShowCursor(ed);
					break;
					
				case delLine:		/* Ctrl-Delete */
					lastOpWasCtrlY = FALSE;
					if (ed->block)
					{
						EditGlobals_ShowMouse(FALSE);
						return done;
					}
					EditFuncs_Delete(ed, TRUE, TRUE);
					break;
					
				case tab:
					lastOpWasCtrlY = FALSE;
					if (ctrl)
						Editor_MarkWord(ed->wdw);
					else
						EditFuncs_Tab(ed, FALSE);
					break;
					
				case return_:
				case enter:
					lastOpWasCtrlY = FALSE;
					EditGlobals_HideCursor(ed);
					if (ed->block)
					{
						EditTools_SaveOp(ed, blockToChar, FALSE);
						EditFuncs_cutEdit(ed, FALSE);
					}
					if (ed->insertMode || ed->currLineNr == ed->totalLineNr - 1)
					{
						insMod = ed->insertMode;
						ed->insertMode = TRUE;
						EditFuncs_Insert(ed, CR);
						EditFuncs_Insert(ed, LF);
						EditFuncs_InsertCR(ed);
						ed->currRow = 0;
						if (ed->autoIndent && !ctrl)
						{
							EditFuncs_Tab(ed, TRUE);
						}
						ed->insertMode = insMod;
					} else
					{
						EditTools_SetCurrLine(ed, ed->currLineNr + 1);
						if (ed->autoIndent && !ctrl)
						{
							ed->currRow = 0;
							while (ed->editLine.text[ed->currRow] == ' ' &&
								   ed->editLine.text[ed->currRow] < 0x20)
							{
								++ed->currRow;
							}
						} else
						{
							ed->currRow = 0;
						}
					}
					ed->prefRow = ed->currRow;
					EditGlobals_ShowCursor(ed);
					break;
					
#if EDIT_DEBUG
				case ctrlH:
					TestStructure(ed);
					done = FALSE;
					break;
					
				case ctrlD:
					EditBase_CompactBuffer(ed);
					done = FALSE;
					break;
#endif /* EDIT_DEBUG */

				case ctrlJ:
					lastOpWasCtrlY = FALSE;
					EditGlobals_HideCursor(ed);
					EditTools_SetBlockstart(ed);
					ed->block = Editor_BlockIsMarked(ed->wdw);
					EditGlobals_ShowCursor(ed);
					break;
					
				case ctrlK:
					lastOpWasCtrlY = FALSE;
					EditGlobals_HideCursor(ed);
					EditTools_SetBlockend(ed);
					ed->block = Editor_BlockIsMarked(ed->wdw);
					EditGlobals_ShowCursor(ed);
					break;
				
#if EDIT_DEBUG
				case ctrlZ:
					debugMode = !debugMode;
					break;
#endif /* EDIT_DEBUG */

				case ctrlY:
					EditGlobals_HideCursor(ed);
					if (ed->block)
					{
						lastOpWasCtrlY = FALSE;
						EditTools_ClearBlock(ed);
					}
					EditBase_PutCurrLine(ed);
					EditTools_MarkLine(ed);
					EditFuncs_cutEdit(ed, TRUE);
					EditGlobals_ShowCursor(ed);
					lastOpWasCtrlY = TRUE;
					break;
				
				default:
					done2 = FALSE;
					break;
				}
				done = done || done2;
				EditGlobals_ShowMouse(FALSE);
			} else 	if (ed->readOnly!=MODE_RW)
			{
				/* Keine Textveraenderung bei Read-Only Texten */
				EditGlobals_ShowMouse(FALSE);
				if (ed->showInfo)
				{
					EditTools_SetInfoLine(ed);
				}
				return FALSE;
			} else if (scan == 0x39 && ctrl && !ed->block)
			{
				/* control space! */
				EditGlobals_HideMouse();
				EditGlobals_HideCursor(ed);
				EditGlobals_growCursor(ed);
				EditGlobals_ShowCursor(ed);
				EditGlobals_ShowMouse(FALSE);
			} else if (scan == 0x2b && (ctrl || (kstate & K_ALT)))
			{
				EditFuncs_SwapLines(ed, ctrl);
			} else if (ch != '\0' && !ctrl)
			{
				lastOpWasCtrlY = FALSE;
				EditFuncs_Insert(ed, ch);
			} else
			{
				done = FALSE;
			}
			EditDraw_CenterCurrline(ed);
			if (ed->showInfo)
			{
				EditTools_SetInfoLine(ed);
			}
			ed->block = Editor_BlockIsMarked(wdw);
		} while (moreChars);
		return done;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Fuegt einen Text an adr an die aktuelle Position ein!
 */
GLOBAL _BOOL Editor_PutLine(HWND wdw, _UBYTE *adr, size_t len)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		return EditFuncs_putLineInEditor(ed, adr, len, TRUE);
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Fuer Quoten und anderes.
 * Fuegt einen Text an adr an die aktuelle Position ein! pre wird vor jede Zeile
 * gesetzt.
 * width: Maximale Laenge einer Zeile, bei der umbebrochen werden soll.
 * -1: Einstellung aus Editor uebernehmen
 */
GLOBAL _BOOL Editor_QuoteSomething(HWND wdw, CONST _UBYTE *pre, CONST _UBYTE *adr, EDCOLUMN len, _WORD width)
{
	EDITOR *ed;
	CONST _UBYTE *txt;
	_WORD scrLine;
	aLineDesc theLine;
	EDCOLUMN startIdx, next, idx;
	EDCOLUMN li;
	EDCOLUMN preLen;
	_UBYTE ch;
	_BOOL insPre;
	EDLINE clNr;
	EDCOLUMN clRow;
	EDCOLUMN rMg;
	EDCOLUMN allocLen;
	_WORD addcrlf;

#define maySplit(ch) \
	(ch == '(' || ch == '[' || ch == '{' || ch == ')' || ch == ']' || ch == '}' || ch == '"' || ch == '\'')

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditTools_SaveOp(ed, normalPaste, FALSE);
		EditTools_SetBlockstart(ed);

		if (width < 10)
			rMg = ed->rightMargin;
		else
			rMg = width;
		if (ed->readOnly!=MODE_RW || len == 0)
			return TRUE;
		if (ed->rightMargin > width)
			allocLen = ed->rightMargin + 5;
		else
			allocLen = width + 5;
		theLine.text = MALLOC(allocLen, "QuoteSomething");
		if (theLine.text == NULL)
			return FALSE;
		clRow = ed->currRow;
		theLine.length = allocLen;
		scrLine = (_WORD)(ed->currLineNr - ed->StartLine);
		txt = adr;
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		EditBase_PutCurrLine(ed);
		insPre = TRUE;
		if (ed->currRow != 0)
		{
			clNr = ed->currLineNr;
			EditTools_SetCurrLine(ed, ed->currLineNr + 1);
			if (clNr == ed->currLineNr)
			{
				insPre = FALSE;
			}
		}
		preLen = (EDCOLUMN)strLen(pre);
		idx = 0;
		len = Min(len, (EDCOLUMN)strLen(txt));
		ConfVars_GetConfDefInt(cAddPreQuoteCRLFNum, &addcrlf,0);
		if (addcrlf && ed->currLineNr)
		{
			li = 0;
			theLine.text[li++] = CR;
			theLine.text[li++] = LF;
			theLine.text[li] = '\0';
			while(addcrlf)
			{
				EditBase_InsertLine(ed, ed->currLineNr, theLine.text, insPre, li);
				EditTools_SetCurrLine(ed, ed->currLineNr + 1);
				addcrlf--;
			}
		}
		while (idx < len)
		{
			startIdx = idx;
			do
			{
				next = EditFuncs_findNext(txt, idx, len, UmbruchSet);
				ch = txt[next];
				if (next - startIdx + preLen < rMg)
				{
					idx = next;
				}
			} while (next < len && next - startIdx + preLen < rMg && ch != CR && ch != LF && ch != '\0');
			/* Jetzt noch testen, ob irgendwelche Klammern, Anfuehrungszeichen oder sonstiges
			 * verletzt wird und solange dekrementieren
			 */
			if (idx > startIdx && txt[idx] != CR && txt[idx] != LF)
			{
				next = idx;
				while (next >= startIdx && maySplit(txt[idx]))
				{
					next = EditFuncs_findLast(txt, idx, UmbruchSet);
					if (next >= startIdx)
					{
						idx = next;
					}
				}
				/* Jetzt noch testen, ob Effekte verletzt werden */
				if (idx > 0 && !setin(TrennSet, txt[idx]) && is_eff_char(txt[idx - 1]))
				{
					/* Effektzeichen mit umbrechen */
					--idx;
				}
			}
			if (idx == startIdx && txt[idx] != CR && txt[idx] != LF)
			{
				/* Wort laenger als Umbruchabstand!! */
				idx = Min(startIdx + rMg - preLen, len);
			}
			Copy(txt, startIdx, idx - startIdx, theLine.text, theLine.length);
			strdelRightSpace(theLine.text);
			Insert(pre, theLine.text, theLine.length, 0);
			ch = txt[idx];
			if (ch == CR || ch == LF)
			{
				++idx;
				if ((txt[idx] == CR || txt[idx] == LF) && ch != txt[idx])
				{
					++idx;
				}
			}
			li = (EDCOLUMN)strLen(theLine.text);
			while (li > 0 && (theLine.text[li - 1] == CR || theLine.text[li - 1] == LF))
			{
				--li;
			}
			theLine.text[li++] = CR;
			theLine.text[li++] = LF;
			theLine.text[li] = '\0';
			EditBase_InsertLine(ed, ed->currLineNr, theLine.text, insPre, li);
			EditTools_SetCurrLine(ed, ed->currLineNr + 1);
		}

		/*
			vj: Nun noch optional eine CR/LF-Zeile anhngen
		*/		
		ConfVars_GetConfDefInt(cAddQuoteCRLFNum, &addcrlf,0);
		if (addcrlf)
		{
			li = 0;
			theLine.text[li++] = CR;
			theLine.text[li++] = LF;
			theLine.text[li] = '\0';
			while(addcrlf)
			{
				EditBase_InsertLine(ed, ed->currLineNr, theLine.text, insPre, li);
				EditTools_SetCurrLine(ed, ed->currLineNr + 1);
				addcrlf--;
			}
		}
		
		ed->blocks[0].blockEnd.line = ed->currLineNr;
		ed->blocks[0].blockEnd.row = ed->currRow;
		EditTools_NewBlockToUndoBuffer(ed);
		/* Und Block wieder loeschen */
		ed->blocks[0].blockStart.line = -1;
		ed->blocks[0].blockEnd.line = -1;
		ed->block = FALSE;
		SFREE(theLine.text);
		EditGlobals_SetDocument(ed);
		EditTools_SetCurrLine(ed, ed->currLineNr);
		ed->currRow = clRow;
		EditGlobals_FixCursorPos(ed);
		EditTools_ShowFileChanged(ed);
		EditDraw_redrawLines(ed, scrLine);
		EditDraw_CenterCurrline(ed);
		if (ed->showInfo)
		{
			EditTools_SetInfoLine(ed);
		}
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Kopiert den gesamten Block an einem Stueck
 * in einen Speicherbereich.
 * Dieser Bereich wird von dieser Routine angelegt und muss
 * bei Nichtgebrauch wieder freigegeben werden
 * Wenn withCrs == TRUE ist, dann werden die vorhandenen Zeilenumbrueche
 * uebernommen, andernfalls werden die Absaetze umgebrochen, soweit es geht.
 */
GLOBAL _BOOL Editor_GetBlock(HWND wdw, _BOOL unChanged, _BOOL withCrs, _UBYTE **adr, _ULONG *buffLen)
{
	_UBYTE *text;
	EDITOR *ed;
	_BOOL absatzEnde;
	_ULONG pos;
	_ULONG bLen;
	_ULONG blockLen;
	EDLINE saveLine;
	EDCOLUMN saveRow;
	EDLINE i;
	EDCOLUMN len;
	EDCOLUMN len2;
	_UBYTE *writeAdr;
	_UBYTE *text2;
	_UBYTE *tBuf;
	EDCOLUMN z;
	_WORD bl;
	_WORD crs,c;

	ed = EditFuncs_FindEditor(wdw);
	*adr = NULL;
	if (ed != NULL)
	{
		ConfVars_GetConfDefInt(cCRbetweenBlocks, &crs, 3);
		EditBase_PutCurrLine(ed);
		if (!ed->block)
			return FALSE;
		blockLen = 0;
		for (bl = 0; bl < maxBlocks; bl++)
		{
			if (ed->blocks[bl].blockStart.line >= 0 &&
				ed->blocks[bl].blockEnd.line >= 0)
			{
				EditBase_CountBytes(ed,
					ed->blocks[bl].blockStart.line, ed->blocks[bl].blockStart.row,
					ed->blocks[bl].blockEnd.line, ed->blocks[bl].blockEnd.row, &bLen);
			} else
			{
				bLen = 0;
			}
			if (bLen > 0)
				bLen += 2+(2*crs);
			blockLen += bLen;
		}
		if (blockLen == 0)
		{
			return FALSE;
		}
		blockLen += 256;		/* Sicherheitsgrenze */
		text = MALLOC(blockLen + 1, "GetBlock");
		if (text == NULL)
			return FALSE;
		/* Bereich loeschen */
		MemSetZero(text, blockLen);

		saveLine = ed->currLineNr;
		saveRow = ed->currRow;
		pos = 0;

		for (bl = 0; bl < maxBlocks; bl++)
		{
			if (ed->blocks[bl].blockStart.line >= 0 &&
				ed->blocks[bl].blockEnd.line >= 0)
			{
				if (bl > 0)
				{
					/* Noch drei Leerzeilen in den Buffer einfuegen */
					for(c=0;c<crs;c++)
					{
						text[pos++] = CR;
						text[pos++] = LF;
					}
				}
				EditTools_SetCurrLine(ed, ed->blocks[bl].blockStart.line);
				i = ed->currLineNr;
				while (i <= ed->blocks[bl].blockEnd.line && pos < blockLen)
				{
					EditBase_GetLineAdr(ed, ed->currLineNr, &text2, &len2);
					len = len2;

					absatzEnde = (len <= 2);
					if (ed->blocks[bl].blockEnd.line > ed->blocks[bl].blockStart.line)
					{
						if (ed->currLineNr < ed->totalLineNr - 1 && !absatzEnde)
						{
							EditBase_GetLineAdr(ed, ed->currLineNr + 1, &tBuf, &z);
							if (!unChanged)
							{
								--z;
								while (z > 0 && (tBuf[z] == CR || tBuf[z] == LF))
								{
									--z;
								}
								absatzEnde = TRUE;
								while (z > 0 && absatzEnde)
								{
									absatzEnde = tBuf[z] == ' ';
									--z;
								}
							}
						}
					}
					writeAdr = text2;
					if (i == ed->blocks[bl].blockStart.line)
					{
						len = Max(0, len - ed->blocks[bl].blockStart.row);
						writeAdr += ed->blocks[bl].blockStart.row;
					}
					if (i == ed->blocks[bl].blockEnd.line)
					{
						len -= len2 - ed->blocks[bl].blockEnd.row;
						if (len < 0)
							len = 0;
					}
					tBuf = writeAdr;
					if (!unChanged)
					{
						while (len > 0 && (tBuf[len - 1] == CR || tBuf[len - 1] == LF))
						{
							--len;
						}
						if (!withCrs)
						{
							while (len > 0 && tBuf[0] == ' ')
							{
								--len;
								++tBuf;
							}
						}
					}
					writeAdr = tBuf;

					/* Zeile ausgeben */
					if (len + pos > blockLen)
						len = (EDCOLUMN)(blockLen - pos);
					MemCpy(&text[pos], writeAdr, len);
					pos += len;

					/* Bei Absatz noch CR/LF ausgeben */
					if (!unChanged)
					{
						if (!(i == ed->blocks[bl].blockEnd.line && ed->blocks[bl].blockEnd.row == 0))
						{
							if ((absatzEnde || withCrs) && (pos + 2) <= blockLen)
							{
								text[pos++] = CR;
								text[pos++] = LF;
							} else
							{
								if (pos > 0 && text[pos - 1] != ' ' && text[pos - 1] != CR && pos < blockLen)
								{
									text[pos++] = ' ';
								}
							}
						}
					}
					EditTools_SetCurrLine(ed, ed->currLineNr + 1);
					++i;
				}
			}
		}
		if (pos > 0 && text[pos - 1] == ' ' && !unChanged)
			--pos;
		text[pos] = '\0';
		*buffLen = pos;

		/* Hier noch Speicherblock verkleinern */
		EditTools_SetCurrLine(ed, saveLine);
		ed->currRow = saveRow;
		EditGlobals_FixCursorPos(ed);
		*adr = text;
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Kopiert den gesamten Text an einem Stueck
 * in einen Speicherbereich.
 */
GLOBAL _BOOL Editor_GetText(HWND wdw, _UBYTE *adr, _ULONG buffLen)
{
	_UBYTE *text;
	EDITOR *ed;
	_ULONG pos;
	_ULONG blockLen;
	EDLINE saveLine;
	EDCOLUMN saveRow;
	EDLINE i;
	EDCOLUMN len;
	_UBYTE *text2;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditBase_PutCurrLine(ed);
		EditBase_CountBytes(ed,
			0, 0,
			ed->totalLineNr - 1, 32000, &blockLen);
		if (blockLen > buffLen)
			blockLen = buffLen;
		
		saveLine = ed->currLineNr;
		saveRow = ed->currRow;
		pos = 0;
		text = adr;
		i = 0;
		while (i < ed->totalLineNr && pos < blockLen)
		{
			EditTools_SetCurrLine(ed, i);
			EditBase_GetLineAdr(ed, i, &text2, &len);
			/* Zeile ausgeben */
			if (len + pos > blockLen)
				len = (EDCOLUMN)(blockLen - pos);
			MemCpy(&text[pos], text2, len);
			pos += len;
			++i;
		}

		/* Hier noch Speicherblock verkleinern */
		EditTools_SetCurrLine(ed, saveLine);
		ed->currRow = saveRow;
		EditGlobals_FixCursorPos(ed);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Editor_GetTextSize(HWND wdw, _ULONG *buffLen)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditBase_PutCurrLine(ed);
		EditBase_CountBytes(ed,
			0, 0,
			ed->totalLineNr - 1, 32000, buffLen);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt zurueck, ob die Position x, y im Blockbereich liegt
 */
GLOBAL _BOOL Editor_ClickIsInBlock(HWND wdw, _WORD x, _WORD y)
{
	return EditUtil_ClickIsInBlock(wdw, x, y);
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _BOOL Editor_ClickInEditWindow(HWND wdw, _VOID *special, _WORD x, _WORD y, _UWORD kstate, _UWORD buts, _WORD clicks)	
{
	EDITOR *ed;
	GRECT work;
	EDLINE clLine;
	EDCOLUMN clRow;
	_WORD mx, my;
	_UWORD but;
	_UWORD kshift;
	EDLINE oLine;
	EDCOLUMN oRow;
	_BOOL init;
	_BOOL whichMark;
	GRECT currSize;

	UNUSED(special);
	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
#if EDIT_DEBUG >= 2
		doDump(ed);
		mem_debug_check_all("ClickInEditWindow");
#endif /* EDIT_DEBUG >= 2 */
		if (!(buts & MOB_LEFT))
		{
			return FALSE;
		}
		ed->block = Editor_BlockIsMarked(wdw);
		oRow = ed->currRow;
		oLine = ed->currLineNr;
		WdwManager_GetWdwWork(wdw, &work);
		WdwManager_GetWdwSize(wdw, &currSize);
		if (ed->hasUserProc && rc_inside(x, y, &ed->userRect))
		{
			ed->userProc(wdw, ed->hdl, x, y, kstate, &work);
			lastOpWasCtrlY = FALSE;
			EditGlobals_ClipWork(ed);
			return TRUE;
		}
		if (rc_inside(x, y, &ed->editWork))
		{
			/* Listwindow im Editor nur in FRED */
			if (ed->listMode)
			{
				EditTools_ClearBlock(ed);
				/* Jetzt Zeile und Spalte bestimmen */
				EditTools_GetClickLineAndRow(ed, x, y, &clLine, &clRow);
				EditTools_SetCurrLine(ed, clLine);
				editListMode(ed, clicks);
			} else
			{
				/* Cursor ausschalten */
				EditGlobals_HideMouse();
				EditGlobals_HideCursor(ed);
				/* Jetzt Zeile und Spalte bestimmen */
				EditTools_GetClickLineAndRow(ed, x, y, &clLine, &clRow);
				/* Jetzt nachsehen, ob die Maustaste noch gedrueckt ist */
				graf_mkstate(&mx, &my, &but, &kshift);
				whichMark = FALSE;
				if (kstate & K_SHIFT)
				{
					lastOpWasCtrlY = FALSE;
					if (ed->block)
					{
						/* Block ist markiert, Markierung erweitern */
						/* Wenn es ein Klick in den Block ist und die Maustaste
						 * festgehalten wird, dann ist es Drag && Drop!
						 */
						if (ed->currLineNr < ed->blocks[0].blockStart.line ||
							(ed->currLineNr == ed->blocks[0].blockStart.line && ed->currRow < ed->blocks[0].blockStart.row))
						{
							if (ed->wordBlock)
							{
								ed->currRow = EditTools_ToWordStart(ed, ed->currLineNr, ed->currRow);
							} else if (ed->lineBlock)
							{
								ed->currRow = EditTools_ToLineStart(ed, ed->currLineNr);
							}
							EditTools_SetBlockstart(ed);
							whichMark = TRUE;
						} else
						{
							if (ed->wordBlock)
							{
								ed->currRow = EditTools_ToWordEnd(ed, ed->currLineNr, ed->currRow);
							} else if (ed->lineBlock)
							{
								if (ed->currLineNr < ed->totalLineNr - 1)
								{
									++ed->currLineNr;
									EditTools_SetCurrLine(ed, ed->currLineNr);
									ed->currRow = EditTools_ToLineStart(ed, ed->currLineNr);
								} else
								{
									ed->currRow = EditTools_ToLineEnd(ed, ed->currLineNr);
								}
							}
							if (ed->currRow > 0 && ed->editLine.text[ed->currRow - 1] == CR)
							{
								--ed->currRow;
							}
							EditTools_SetBlockend(ed);
							whichMark = FALSE;
						}
					} else
					{
						/* noch keiner markiert, also jetzt markieren */
						if (ed->currLineNr > oLine || (ed->currLineNr == oLine && ed->currRow > oRow))
						{
							whichMark = FALSE;
							EditTools_SetBlockend(ed);
							EditTools_SetCurrLine(ed, oLine);
							ed->currLineNr = oLine;
							ed->currRow = oRow;
							EditTools_SetBlockstart(ed);
							EditTools_SetCurrLine(ed, clLine);
							ed->currLineNr = clLine;
							ed->currRow = clRow;
						} else
						{
							whichMark = TRUE;
							EditTools_SetBlockstart(ed);
							EditTools_SetCurrLine(ed, oLine);
							ed->currLineNr = oLine;
							ed->currRow = oRow;
							EditTools_SetBlockend(ed);
							EditTools_SetCurrLine(ed, clLine);
							ed->currLineNr = clLine;
							ed->currRow = clRow;
						}
						ed->block = TRUE;
					}
					ed->prefRow = ed->currRow;
				}
				if (but & MOB_LEFT)
				{
					/* Wenn es ein Klick in den Blockbereich ist und die
					 * Shift-Taste nicht gedrueckt ist, dann ist es eine Drag && Drop Action
					 */
					if (ed->block &&
						!(kstate & K_SHIFT) &&
						((ed->currLineNr > ed->blocks[0].blockStart.line) ||
						 ((ed->currLineNr == ed->blocks[0].blockStart.line) && (ed->currRow >= ed->blocks[0].blockStart.row))) &&
						((ed->currLineNr < ed->blocks[0].blockEnd.line) ||
						 ((ed->currLineNr == ed->blocks[0].blockEnd.line) && (ed->currRow <= ed->blocks[0].blockEnd.row))))
					{
						/* click ist in Block und nicht mit Shift!
						 * Nur in dem Fall ist es Drag && Drop
						 */
						EditUtil_PerformDragDrop(ed, x, y, kstate, buts);
					} else
					{
						lastOpWasCtrlY = FALSE;
						/* Taste noch gedrueckt, in Blockmarkmodus gehen */
						if (!(kstate & K_SHIFT))
						{
							/* eventuell vorhandenen Block loeschen */
							EditTools_ClearBlock(ed);
						}
						init = !ed->block;
						if (!ed->block)
						{
							ed->wordBlock = (clicks == 2) && !(kstate & K_CTRL);
							ed->lineBlock = (clicks > 2) || ((clicks == 2) && (kstate & K_CTRL));
							if (ed->wordBlock)
							{
								EditTools_MarkWord(ed);
								whichMark = FALSE;
							} else if (ed->lineBlock)
							{
								EditTools_MarkLine(ed);
								whichMark = FALSE;
							} else
							{
								EditTools_SetBlockstart(ed);
								EditTools_SetBlockend(ed);
								whichMark = FALSE;
							}
						}
						EditTools_MarkBlock(ed, x, y, init, !whichMark);
						EditTools_TestBlockMarksAndSwap(ed);
						ed->block = Editor_BlockIsMarked(wdw);
					}
				} else
				{
					if (!(kstate & K_SHIFT))
					{
						lastOpWasCtrlY = FALSE;
						if (ed->block)
						{
							EditTools_ClearBlock(ed);
						} else
						{
							ed->blocks[0].blockStart.line = -1;
							ed->blocks[0].blockEnd.line = -1;
						}
						ed->wordBlock = (clicks == 2) && !(kstate & K_CTRL);
						ed->lineBlock = (clicks > 2) || ((clicks == 2) && (kstate & K_CTRL));
						if (ed->wordBlock)
						{
							EditTools_MarkWord(ed);
						} else if (ed->lineBlock)
						{
							EditTools_MarkLine(ed);
						}
					}
					ed->block = Editor_BlockIsMarked(wdw);
				}
				EditGlobals_ShowCursor(ed);
				EditGlobals_ShowMouse(FALSE);
				if (ed->showInfo)
				{
					EditTools_SetInfoLine(ed);
				}
			}
			return TRUE;
		} else
		{
			/* War falscher Aufruf */
			return FALSE;
		}
	}
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Font-Funktionen */

LOCAL _VOID SetFont(EDITOR *ed, _WORD font, _WORD fontSize)
{
	_UBYTE varName[256];

	ed->font = font;
	ed->fontSize = fontSize;
	ed->cursPos.row = -1;
	FontSelect_SetFont(ed->hdl, &font, &ed->fontSize, FALSE, &ed->monoSpaced, &ed->isFSM, &ed->charWidth, &ed->charHeight);
	ed->maxWidth = 0;
	if (!ed->alienCoords)
	{
		make_varname(varName, sizeof(varName), cEditFont, ed->number);
		ConfVars_SetConfigInt(varName, ed->font);
		make_varname(varName, sizeof(varName), cEditFontSize, ed->number);
		ConfVars_SetConfigInt(varName, ed->fontSize);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt den neuen Font fuer den Editor
 */
GLOBAL _VOID Editor_SelectEditFont(HWND wdw, _WORD font, _WORD fontSize)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		SetFont(ed, font, fontSize);
		/* Jetzt ist der Font gesetzt. Jetzt noch Window snappen und dann neu
		 * zeichnen lassen
		 */
		WdwManager_SetDocumentParms(wdw, 1, ed->charHeight);
		WdwManager_SnapWdw(wdw);
		WdwManager_SetClip(cursorHandle, &ed->editWork, TRUE);
		WdwManager_RedrawWdw(wdw, &ed->editWork);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_SelectFont(HWND wdw)
{
	EDITOR *ed;
	
	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		VDIUtil_SelectFont(ed->hdl, wdw, FALSE, ed->font, ed->fontSize);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Nachfrage-Funktionen */

/* fragt nach, ob ein Editor das Top-Window hat
 */
GLOBAL _BOOL Editor_IsEditTop(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	return ed != NULL && !ed->isHidden;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt zurueck, ob ein Block markiert ist oder nicht
 */
GLOBAL _BOOL Editor_BlockIsMarked(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		return (ed->blocks[0].blockStart.line >= 0) && (ed->blocks[0].blockEnd.line >= 0) &&
			(((ed->blocks[0].blockStart.line == ed->blocks[0].blockEnd.line)
			  && (ed->blocks[0].blockEnd.row > ed->blocks[0].blockStart.row))
			 || (ed->blocks[0].blockStart.line < ed->blocks[0].blockEnd.line));
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert Handle der Workstation des Fensters */
GLOBAL _VOID Editor_GetEditVDIHandle(HWND wdw, HVDI *hdl)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		*hdl = ed->hdl;
	else
		*hdl = -1;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Anzeige-Funktionen des Editors! */

/* Oeffnet einen Editor und liest Text aus Buffer
 */
/*DYJ 5.21b */
GLOBAL _BOOL Editor_OpenEditBuffer(
	CONST _UBYTE *title,
	CONST _UBYTE *infoline,
	_BOOL showInfoLine,
	_UBYTE **buff,
	_ULONG buffLen,
	_BOOL freeBuffer,
	_BOOL txtIsEnriched,
	_WORD font,
	_WORD fontSize,
	GRECT *curr,
	_BOOL useUserProc,
	userProcType theUserProc,
	userGetRectProc theUserRectProc,
	userDrawProc theDrawUserProc,
	userCloseProc theCloseProc,
	userKeyProc theKeyProc,
	userTopProc theTopProc,
	userUntopProc theUntopProc,
	userPrePrintProc thePrePrintProc,
	HWND *wdwHandle)
{
	EDITOR *ed;
	_UWORD comps;
	ErrReason err;
	_WORD bits;
	_BOOL withoutHScroll;
	_BOOL withoutVScroll;
	_BOOL MsgMarkBackground;
	_WORD dummy;
	_BOOL lstPgF;
	
/* Dealloziert allozierte Pointer und gibt sonstiges Zeugs frei
 */
#define FreeStuff() \
	if (*wdwHandle >= 0) WdwManager_CloseWindow(*wdwHandle, TRUE); \
	else if (ed != NULL) EditFuncs_KillEditor(ed); \

	/* Variablen initialisieren, damit FreeStuff auch funktioniert! */
	*wdwHandle = -1;
	/* Fensterkomponenten bestimmen */
	comps =	NAME | CLOSER | FULLER | MOVER | SIZER | UPARROW | DNARROW | VSLIDE | LFARROW | RTARROW | HSLIDE;
	if (showInfoLine)
		comps |= INFO;
	ConfVars_GetConfDefBool(cMsgNoScrollbar, &withoutHScroll, FALSE);
	if (withoutHScroll)
	{
		comps &= ~(LFARROW | RTARROW | HSLIDE);
	}
	ConfVars_GetConfDefBool(cMsgNoVScrollbar, &withoutVScroll, FALSE);
	if (withoutVScroll)
	{
		comps &= ~(UPARROW | DNARROW | VSLIDE);
	}
	if (withoutHScroll && withoutVScroll)
		comps &= (SIZER | FULLER);
	
	/* Editor erstellen */
	if ((ed = EditFuncs_CreateEditor()) == NULL)
	{
		/* Fehlermeldung: not enough memory */
		FreeStuff();
		return FALSE;
	}
	/* Editorvariablen setzen */
	ConfVars_GetConfDefBool(cDisplayEffects, &ed->DisplayEffects, TRUE);
	ConfVars_GetConfDefBool(cAllowLowerQuoteInitials, &ed->AllowLowerQuoteInitials, TRUE);
	ed->isEnriched = txtIsEnriched;
	ed->readOnly = MODE_SHOW;
	ed->isGrin = TRUE;
	ed->alienCoords = TRUE;
	ed->realTabs = TRUE;

	ed->fileName.text = NULL;
	ed->fileName.length = 0;
	ed->showInfo = showInfoLine;
	ed->windComps = comps;
	/* Strings uebernehmen */
	if (title != NULL)
		strBcpy(ed->WindowTitle, title);
	else
		ed->WindowTitle[0] = '\0';
	if (infoline != NULL)
		strBcpy(ed->WindowInfo, infoline);
	else
		ed->WindowInfo[0] = '\0';
	/* Jetzt noch die User-Parameter */
	ed->hasUserProc = useUserProc;
	ed->userProc = theUserProc;
	ed->getUserRect = theUserRectProc;
	ed->drawUserArea = theDrawUserProc;
	ed->userClose = theCloseProc;
	ed->userKey = theKeyProc;
	ed->userTop = theTopProc;
	ed->userUntop = theUntopProc;
	ed->userPrint = thePrePrintProc;

	Editor_GetSettings(ed);
	
	/* Fenster mit Window Library oeffnen */
	if (!WdwManager_OpenWindow(
		Editor_ClickInEditWindow,
		Editor_EditChar, FUNK_NULL,
		&deskSize, curr, comps, TRUE, "", "",
		snapWindow, closeEditor, EditDraw_redrawWdw, topEditor, untopEditor,
		handleUpdate, docChanged, getScrollRect, hideEditor,
		pixOff,
		ed,		/* special */
		FALSE,	/* onlyThisTop */
		TRUE,	/* showIt */
		TRUE,	/* supportIconify */
		TRUE,	/* openWorkstation */
		FALSE,		/* hasIdem */
		&ed->wdw, &ed->hdl))
	{
		/* Kein Fenster oder keine Workstation mehr frei! */
		FreeStuff();
		mtAlerts_Alert(1, Rsc_WrkString(AL_NO_BUFF_OPEN));
		return FALSE;
	}
	ConfVars_GetConfDefBool(cMsgMarkBackground, &MsgMarkBackground, FALSE);
	if (MsgMarkBackground)
	{
		if (appl_xgetinfo(APG_WINDOWS, &bits, &dummy, &dummy, &dummy) == 1 &&
			(bits & 0x20))
		{
			wind_set(ed->wdw, WF_BEVENT, 1, 0, 0, 0);
		}
	}
	*wdwHandle = ed->wdw;
	ed->isHidden = FALSE;
	/* Text einlesen */
	ed->currRow = 0;
	ed->rightMargin = 32767;
	/* Text einlesen */
	if (!EditFuncs_ReadBuffer(ed, buff, buffLen, freeBuffer, &err))
	{
		FreeStuff();
		return FALSE;
	}
	ed->editWork = deskSize;
	ed->wasOpen = TRUE;

	ed->editChanged = FALSE;
	ed->currLineNr = -1;
	EditTools_SetCurrLine(ed, 0);

	/* Pointer noch richtig setzen */
	ed->currentBuff = ed->firstBuff;
	ed->currLineNr = 0;
	ed->prefRow = ed->currRow = 0;
	/* Als Drag && Drop Server installieren */
	WdwManager_WdwInstallDDServer(ed->wdw, editGetHeader, editReadData);
	/* Fonts laden */
	FontSelect_LoadFonts(ed->hdl);
	/* Font setzen */
	SetFont(ed, font, fontSize);
	ed->font = font;

	WdwManager_AcceptFont(ed->wdw, Editor_ChangeFont);
	WdwManager_AcceptColor(ed->wdw, Editor_ChangeColor);
	/* Fuellparameter setzen */
	vsf_style(ed->hdl, IP_SOLID);
	vsf_interior(ed->hdl, FIS_SOLID);
	/* Schreibmodus setzen */
	vswr_mode(ed->hdl, MD_REPLACE);

	/* Umbruchbreite zuruecksetzen */
	ed->lastWrapWidth = 0;
	/* Fensterinhalte setzen */
	EditTools_SetEditTitle(ed);
	if (ed->showInfo)
	{
		EditTools_SetInfoLine(ed);
	}
	/* Fenstergroesse berechnen */
	WdwManager_SetWdwSize(ed->wdw, curr);
	/* Text umbrechen */
	EditUtil_WrapText(ed, ed->isEnriched);
	WdwManager_SetDocumentParms(ed->wdw, 1, ed->charHeight);
	ConfVars_GetConfDefBool(cEditLstPgF, &lstPgF,TRUE);
	WdwManager_SetScrollParms(ed->wdw, ed->editWork.g_w - ed->charWidth, ed->windLines - 1, ed->charWidth, 1,lstPgF);
	/* Document fuer Window setzen */
	EditGlobals_SetDocument(ed);
	/* Cursor anzeigen */
	EditGlobals_ShowCursor(ed);
	ed->wdwIsOpen = TRUE;
	return TRUE;
#undef FreeStuff
}

/*** ---------------------------------------------------------------------- ***/

/* Schliesst Nachrichtenfenster */
GLOBAL _VOID Editor_CloseEditBuffer(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		ed->changed = FALSE;
		Editor_CloseEditor(ed->wdw, FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Zeigt neuen Text im Anzeigefenster an! */
	
GLOBAL _VOID Editor_ShowNewBuffer(HWND wdw, CONST _UBYTE *title, CONST _UBYTE *infoline,
	_UBYTE **buff, _ULONG buffLen, _BOOL freeBuffer, _BOOL isEnriched)
{
	EDITOR *ed;
	ErrReason err;
	_WORD i;
	LGRECT doc;
	
	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		while (ed->firstBuff != NULL)
		{
			ed->currentBuff = ed->firstBuff;
			EditBase_FreeBuffer(ed);
		}
		ConfVars_GetConfDefBool(cDisplayEffects, &ed->DisplayEffects, TRUE);
		ed->isEnriched = isEnriched;
		ed->editChanged = FALSE;
		ed->editLine.text[0] = '\0';
		ed->currRow = 0;
		ed->totalLineNr = 0;
		ed->currLineNr = 0;
		ed->readOnly = MODE_SHOW;
		ed->isGrin = TRUE;
		if (!EditFuncs_ReadBuffer(ed, buff, buffLen, freeBuffer, &err))
		{
			return;
		}
		ed->currentBuff = ed->firstBuff;
		ed->block = FALSE;
		ed->alienCoords = TRUE;
		ed->prefRow = ed->currRow = 0;
		ed->StartLine = 0;
		for (i = 0; i < maxBlocks; i++)
		{
			ed->blocks[i].blockStart.line = -1;
			ed->blocks[i].blockEnd.line = -1;
		}
		ed->leftOffset = 0;
		ed->maxWidth = 0;
		ed->editChanged = FALSE;
		ed->currLineNr = -1;
		EditTools_SetCurrLine(ed, 0);
		if (title != NULL)
			strBcpy(ed->WindowTitle, title);
		else
			ed->WindowTitle[0] = '\0';
		if (infoline != NULL)
			strBcpy(ed->WindowInfo, infoline);
		else
			ed->WindowInfo[0] = '\0';
		/* neu zeichnen und Slider setzen */
		if (ed->showInfo)
		{
			EditTools_SetInfoLine(ed);
		}
		EditTools_SetEditTitle(ed);
		/* Umbruchbreite zuruecksetzen */
		ed->lastWrapWidth = 0;
		/* Text umbrechen */
		EditUtil_WrapText(ed, isEnriched);
		doc.x = ed->leftOffset;
		doc.y = ed->StartLine;
		doc.w = ed->maxWidth;
		doc.h = ed->totalLineNr - 1;
		WdwManager_SetNewDocument(wdw, &doc, TRUE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* schliesst alle Editoren, gibt TRUE zurueck, falls
 * erfolgreich
 */
GLOBAL _BOOL Editor_CloseAllEditors(_BOOL force)
{
	while (editList != NULL)
	{
		EditGlobals_ShowMouse(TRUE);
		if (!Editor_CloseEditor(editList->wdw, force))
			return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Alle Fenster neu zeichnen */
GLOBAL _VOID Editor_RedrawAll(_VOID)
{
	EDITOR *ed;

	for (ed = editList; ed != NULL; ed = ed->nextEdit)
	{
		Editor_GetSettings(ed);
		WdwManager_FullRedrawWdw(ed->wdw);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Fuehrt Undo durch */
GLOBAL _VOID Editor_DoUndo(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		lastOpWasCtrlY = FALSE;
		EditTools_Undo(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Markiert das Wort beim Cursor als Block */
GLOBAL _VOID Editor_MarkWord(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		EditTools_ClearBlock(ed);
		EditTools_MarkWord(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Markiert den ganzen Text als Block */

GLOBAL _VOID Editor_MarkAll(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		EditTools_SetCurrLine(ed, ed->totalLineNr - 1);
		ed->currRow = ed->editLen;
		EditTools_SetBlockend(ed);
		EditTools_SetCurrLine(ed, 0);
		ed->prefRow = ed->currRow = 0;
		EditTools_SetBlockstart(ed);
		ed->block = TRUE;
		EditDraw_CenterCurrline(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt den Cursor an die Position offset im aktuellen Buffer
 * Wird nur fuer CAT gebraucht!
 */
GLOBAL _VOID Editor_EditSetOffset(HWND wdw, _ULONG offset)
{
	EDITOR *ed;
	EDLINE line;
	EDCOLUMN row;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditBase_GetPos(ed, (EDBUFSIZE)offset, &line, &row);
		EditFuncs_Goto(ed, line, row);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt den Cursor an die angegebene Position, falls moeglich
 */
GLOBAL _VOID Editor_EditSetLineAndRow(HWND wdw, EDLINE line, EDCOLUMN row)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		EditFuncs_Goto(ed, line, row);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* neue Funktionen fuer Eddix */

/* Geht zur Zeile nr, falls moeglich */
GLOBAL _VOID Editor_ToLineNr(HWND wdw, EDLINE nr)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		EditFuncs_Goto(ed, nr, ed->currRow);
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert den kompletten Filenamen, falls das Fenster ein Editor
 * ist
 */
GLOBAL _BOOL Editor_GetCompInfo(HWND wdw, _UBYTE *fname, _BOOL *saved)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->fileName.length != 0)
		{
			strMcpy(fname, PATH_MAX, ed->fileName.text);
		} else
		{
			strMcpy(fname, PATH_MAX, cNoname);
		}
		*saved = !ed->changed;
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt Informationen ueber den aktuellen Text raus
 */

GLOBAL _BOOL Editor_FileInfos(HWND wdw, _ULONG *lines, _ULONG *bytes, _WORD *edits, _ULONG *totalMem, _UBYTE *fName)
{
	EDITOR *ed;
	EDLINE clLine;
	EDCOLUMN clRow;
	aBuffer *aBuf;

	*edits = 0;
	ed = editList;
	while (ed != NULL)
	{
		++(*edits);
		ed = ed->nextEdit;
	}
	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		*lines = ed->totalLineNr;
		*bytes = 0;
		*totalMem = 0;
		clLine = ed->currLineNr;
		clRow = ed->currRow;
		EditGlobals_MouseBusy();
		aBuf = ed->firstBuff;
		while (aBuf != NULL)
		{
			*bytes += aBuf->usedMem;
			*totalMem += aBuf->bufSize;
			*totalMem += aBuf->maxLine * sizeof(*(aBuf->lines));
			*totalMem += sizeof(aBuffer);
			aBuf = aBuf->next;
		}
		*totalMem += sizeof(EDITOR);
		EditTools_SetCurrLine(ed, clLine);
		ed->prefRow = ed->currRow = clRow;
		if (ed->fileName.length != 0)
		{
			strMcpy(fName, PATH_MAX, F_Path_Basename(ed->fileName.text));
		} else
		{
			strMcpy(fName, PATH_MAX, cNoname);
		}
		EditGlobals_MouseArrow();
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt Blockanfang auf Cursorposition
 */
GLOBAL _VOID Editor_SetBlockStart(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->block)
			return;
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		EditTools_SetBlockstart(ed);
		ed->block = Editor_BlockIsMarked(ed->wdw);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt Blockende auf Cursorposition
 */
GLOBAL _VOID Editor_SetBlockEnd(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->block)
			return;
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		EditTools_SetBlockend(ed);
		ed->block = Editor_BlockIsMarked(ed->wdw);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Loescht Blockmarkierung
 */
GLOBAL _VOID Editor_ClearBlock(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		EditTools_ClearBlock(ed);
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt Marke which an Cursorposition
 */
GLOBAL _VOID Editor_SetMark(HWND wdw, _WORD which)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (which < MAX_MARKS)
		{
			ed->markPoints[which].line = ed->currLineNr + 1;
			ed->markPoints[which].row = ed->currRow;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Springt zur Marke which
 */
GLOBAL _VOID Editor_ToMark(HWND wdw, _WORD which)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (which < MAX_MARKS)
			EditFuncs_Goto(ed, ed->markPoints[which].line, ed->markPoints[which].row);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Springt zum Blockanfang
 */
GLOBAL _VOID Editor_ToBlockStart(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		EditTools_ToBlockmark(ed, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/* Springt zum Blockende
 */
GLOBAL _VOID Editor_ToBlockEnd(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		EditTools_ToBlockmark(ed, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt TabSize auf size
 */
GLOBAL _VOID Editor_SetTabSize(HWND wdw, EDCOLUMN size)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		ed->TabSize = size;
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt size auf TabSize
 */
GLOBAL _VOID Editor_GetTabSize(HWND wdw, EDCOLUMN *size)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		*size = ed->TabSize;
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die Raender und man kann den Umbruch ein-
 * und ausschalten
 */
GLOBAL _VOID Editor_SetMargins(HWND wdw, EDCOLUMN right, _BOOL umbruch)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		ed->rightMargin = right;
		if (ed->rightMargin < 1)
			ed->rightMargin = 1;
		ed->umbruch = umbruch;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die Raender und man kann den Umbruch ein-
 * und ausschalten
 */
GLOBAL _VOID Editor_GetMargins(HWND wdw, EDCOLUMN *right)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		*right = ed->rightMargin;
}

/*** ---------------------------------------------------------------------- ***/

/* Schaltet einen Zustand um */

GLOBAL _VOID Editor_ToggleMode(HWND wdw, editMode mode)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		switch (mode)
		{
		case insMode:
			ed->insertMode = !ed->insertMode;
			break;
		case indentMode:
			ed->autoIndent = !ed->autoIndent;
			break;
		case autoBackup:
			ed->backup = !ed->backup;
			break;
		case wrapMode:
			ed->umbruch = !ed->umbruch;
			break;
		case tabMode:
			ed->realTabs = !ed->realTabs;
			ed->cursPos.row = -1;
			break;
		case crMode:
			ed->EditShowLineEnds = !ed->EditShowLineEnds;
			break;
		case roMode:
			if(ed->readOnly!=MODE_RO)
				ed->readOnly = MODE_SHOW+MODE_RW-ed->readOnly;
			break;
		case effMode:
			ed->DisplayEffects = !ed->DisplayEffects;
			break;
		case errorMode:
			ed->listMode = !ed->listMode;
			break;
		case msgMode:
			ed->isGrin = !ed->isGrin;
			break;
		}
		Editor_GetSettings(ed);
		EditTools_SetInfoLine(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID Editor_SetRW(HWND wdw,_WORD val)								*/
/*	Setzt den RW Zustand. Im Gegensatz zu Editor_SetMode als WORD				*/
/*	Parameter:	wdw:	Editorfenster											*/
/*				val:	neuer Wert												*/
/*	Ergebnis:	keines															*/		
/*	22.12.00 von Dimitri Junker	DYJ 4.22l										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID Editor_SetRW(HWND wdw,_WORD val)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		ed->readOnly = val;
		Editor_GetSettings(ed);
		EditTools_SetInfoLine(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt einen Zustand
 */

GLOBAL _VOID Editor_SetMode(HWND wdw, editMode mode, _BOOL val)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		EditGlobals_HideMouse();
		EditGlobals_HideCursor(ed);
		switch (mode)
		{
		case insMode:
			ed->insertMode = val;
			break;
		case indentMode:
			ed->autoIndent = val;
			break;
		case autoBackup:
			ed->backup = val;
			break;
		case wrapMode:
			ed->umbruch = val;
			break;
		case tabMode:
			ed->realTabs = val;
			ed->cursPos.row = -1;
			break;
		case crMode:
			ed->EditShowLineEnds = val;
			break;
		case roMode:
			if(ed->readOnly!=MODE_RO)
				ed->readOnly = val? MODE_SHOW : MODE_RW;
			break;
		case effMode:
			ed->DisplayEffects = val;
			break;
		case errorMode:
			ed->listMode = val;
			break;
		case msgMode:
			ed->isGrin = val;
			break;
		}
		Editor_GetSettings(ed);
		EditTools_SetInfoLine(ed);
		EditGlobals_ShowCursor(ed);
		EditGlobals_ShowMouse(FALSE);
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _WORD Editor_GetModeRW(HWND wdw)										*/
/*	Fragt den RW-Zustand ab. Im Gegensatz zu Editor_GetMode als WORD			*/
/*	Parameter:	wdw:Editorfenster												*/
/*	Ergebnis:	Wert															*/		
/*	22.12.00 von Dimitri Junker	DYJ 4.22l										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _WORD Editor_GetModeRW(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		return ed->readOnly;
	return MODE_RO;
}

/*** ---------------------------------------------------------------------- ***/

/* Fragt einen Zustand ab.
 */
GLOBAL _BOOL Editor_GetMode(HWND wdw, editMode mode)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		switch (mode)
		{
		case insMode:
			return ed->insertMode;
		case indentMode:
			return ed->autoIndent;
		case autoBackup:
			return ed->backup;
		case wrapMode:
			return ed->umbruch;
		case tabMode:
			return ed->realTabs;
		case crMode:
			return ed->EditShowLineEnds;
		case roMode:
			return ed->readOnly!=MODE_RW;
		case effMode:
			return ed->DisplayEffects;
		case errorMode:
			return ed->listMode;
		case msgMode:
			return ed->isGrin;
		}
	}
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Setzt den Listenmodus fuer ein Fenster. Dabei wird der Cursor
 * ausgeschaltet, das Fenster auf Readonly gesetzt und der Listen-
 * modus angeschaltet
 */
GLOBAL _VOID Editor_SetListMode(HWND wdw)
{
	Editor_SetMode(wdw, roMode, TRUE);
	Editor_SetMode(wdw, errorMode, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die ListModeProc fuer ein Fenster. Um alles einzuschalten,
 * sollte man einfach SetListMode aufrufen.
 */
GLOBAL _BOOL Editor_SetListProc(HWND wdw, userListProc listProc)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		ed->listModeProc = listProc;
	return ed != NULL;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt bei einem uebergebenem Dateinamen zurueck, ob diese Datei
 * schon geladen ist. Beachtet auch Case-Sensitivitaet von Filesystemen
 * Im Falle von TRUE wird in wdw das Fensterhandle des Editors
 * zurueckgeliefert
 */
GLOBAL _BOOL Editor_FileIsEditor(CONST _UBYTE *fname, HWND *wdw)
{
	EDITOR *ed;

	ed = editList;
	while (ed != NULL)
	{
		if (ed->fileName.text != NULL)
		{
			if (F_Path_Equal(fname, ed->fileName.text))
			{
				*wdw = ed->wdw;
				return TRUE;
			}
		}
		ed = ed->nextEdit;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_CompileText(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		EditFuncs_callComp(ed);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_ShowError(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		EditFuncs_showError(ed);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_SetError(HWND wdw, EDLINE errorLine, EDCOLUMN errorRow, CONST _UBYTE *errorMsg, _BOOL show)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		ed->errRow = errorRow;
		ed->errLine = errorLine;
		strBcpy(ed->errMsg, errorMsg);
		if (show)
			Editor_ShowError(wdw);
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Block Transformationen */

GLOBAL _VOID Editor_BlockUp(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		if (ed->block)
			EditFuncs_TransformBlock(ed, EditFuncs_AllUp, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_BlockDown(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		if (ed->block)
			EditFuncs_TransformBlock(ed, EditFuncs_AllLower, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_BlockCapitals(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		if (ed->block)
			EditFuncs_TransformBlock(ed, EditFuncs_Capitalise, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_BlockSwap(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		if (ed->block)
			EditFuncs_TransformBlock(ed, EditFuncs_SwapUpperLower, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_BlockRot18(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		if (ed->block)
			EditFuncs_TransformBlock(ed, EditFuncs_Rot18, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_BlockRot(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->block)
		{
			rotateFaktor = 13;
			EditFuncs_TransformBlock(ed, EditFuncs_RotXX, FALSE);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_BlockDecode(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->block)
		{
			UUDecode_BeginDecode();
			EditFuncs_TransformBlock(ed, EditFuncs_XXDecode, FALSE);
			UUDecode_EndDecode();
		}
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID Editor_BlockMime(HWND wdw)											*/
/*	Mime-dekodiert einen Block, abgeleitet von Editor_BlockDecode				*/
/*	Parameter:	wdw:	das Fenster mit dem Block								*/
/*	Ergebnis:	keines															*/		
/*	12.11.01 von Dimitri Junker	DYJ 4.30ze										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID Editor_BlockMime(HWND wdw)
{
	EDITOR *ed;
	EDLINE i;
	size_t len=200;		/*Ggf. mu noch ein Header generiert werden*/

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		if (ed->block)
		{
			for(i=ed->blocks[0].blockStart.line;i<=ed->blocks[0].blockEnd.line;i++)
			{
				EditTools_SetCurrLine(ed, i);
				len+=ed->editLen+2;
			}
			if(Mime_BeginDecode(len))
			{
				EditFuncs_TransformBlock(ed, EditFuncs_MimeDecode, FALSE);
				Mime_EndDecode();
			}
		}
	}
}
/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_BlockMirror(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		if (ed->block)
			EditFuncs_TransformBlock(ed, EditFuncs_Mirror, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_BlockProtect(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		if (ed->block)
			EditFuncs_TransformBlock(ed, EditFuncs_Protect, TRUE);
}
/*** ------------------------------------------------------------------------ ***/
/*	LOBAL _VOID Editor_BlockUnProtect(HWND wdw)									*/
/*	Die Umkehrfunktion zur vorigen												*/
/*	Parameter:	wdw: das Fenster in dem der Block entschtzt werden soll		*/
/*	Ergebnis:	keines															*/		
/*	6.3.01 von Dimitri Junker	DYJ 4.23n										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID Editor_BlockUnProtect(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		if (ed->block)
			EditFuncs_TransformBlock(ed, EditFuncs_UnProtect, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Editor_UndoPossible(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		return EditTools_PossibleUndo(ed);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert die Laenge der Zeile line im zum wdw gehoerenden Editor zurueck.
 * FALSE: Fenster ist kein Editor oder Zeile nicht vorhanden
 */
GLOBAL _BOOL Editor_GetTextLineLength(HWND wdw, EDLINE line, EDCOLUMN *len)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		return EditBase_GetLineLength(ed, line, len);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert die Zeile line im zum wdw gehoerenden Editor zurueck.
 * FALSE: Fenster ist kein Editor oder Zeile nicht vorhanden
 */
GLOBAL _BOOL Editor_GetTextLine(HWND wdw, EDLINE line, _UBYTE *str, size_t maxlen, size_t *len)
{
	EDITOR *ed;
	EDCOLUMN maxcol;
	_BOOL res;
	
	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		res = EditBase_GetLine(ed, line, str, maxlen, &maxcol);
		*len = maxcol;
		return res;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Liefert die Adresse des ersten Buffers im zum wdw gehoerenden Editor zurueck.
 * In len steht der benutzte Speicherplatz.
 * FALSE: Fenster ist kein Editor
 */
GLOBAL _BOOL Editor_GetEditBuffer(HWND wdw, _UBYTE **adr, EDBUFSIZE *len)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		*adr = ed->firstBuff->buffer;
		*len = ed->firstBuff->usedMem;
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt den Cursor an das Ende eines Textes
 */
GLOBAL _VOID Editor_ToEnd(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		EditFuncs_Home(ed, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Holt die Wildcardzeichen
 */
GLOBAL _VOID Editor_GetWildChars(_UBYTE one[2], _UBYTE all[2])
{
	one[0] = wildOneChar;
	one[1] = '\0';
	all[0] = wildAllChar;
	all[1] = '\0';
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die Wildcardzeichen
 */
GLOBAL _VOID Editor_SetWildChars(_UBYTE one, _UBYTE all)
{
	wildOneChar = one;
	wildAllChar = all;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Editor_MarkMultiple(HWND wdw)
{
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
		EditUtil_MarkMultiple(ed);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Editor_Init(_VOID)
{
	_BOOL addcrlf;
		if(ConfVars_GetConfigBool("AddQuoteCRLF", &addcrlf))
		{
			ConfVars_DeleteConfigVar("AddQuoteCRLF");
			ConfVars_SetConfigInt(cAddQuoteCRLFNum, addcrlf ? 1 : 0);
		}
	setclr(nums);
	ErrorProc = PrinterErrorProc;
	return TRUE;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID Editor_InsChr(HWND wdw)										*/
/*	Sonderzeicheneingabe per Zeichentabelle										*/
/*	Parameter:	wdw:	das Zielfenster											*/
/*	Ergebnis:	keines															*/		
/*	26.10.00 von Dimitri Junker	DYJ 4.18zl										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID Editor_InsChr(HWND wdw)
{
	_UBYTE ch;
	_BOOL insMod;
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		insMod = mtDials_DialCharTable(&ch);
		if (insMod && ch != CR && ch != LF)
		{
			lastOpWasCtrlY = FALSE;
			EditFuncs_Insert(ed, ch);
		}
	}
}
