/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 */


#include <edittool.h>
#include <editglob.h>
#include <editbase.h>
#include <editdraw.h>
#include <find2.h>
#include <confvars.h>
#include <catgloba.h>
#include <wdwmanag.h>
#include <catutil.h>

#include <mtappl.h>
#include <mtalerts.h>
#include <mtdials.h>
#include <mtutils.h>

#include <magicstr.h>
#include <tos.h>
#include <strconv.h>
#include <keyboard.h>

#include <routinec.h>
#include <routine.h>
#include <debug.h>
#include <ro_mem.h>		
#include <routines.h>
#include <atari.h>	
#include <time.h>	

#define TAB 0x09
#define CR  0x0d
#define LF  0x0a

typedef struct
{
	undoType whatToUndo;
	_UBYTE *buffer;
	_ULONG bufSize;
	aMark oldStart;
	aMark oldEnd;
	aMark newStart;
	aMark newEnd;
	aLineDesc undoLine;
	EDLINE lines;
	_BOOL atEndOfText;
	HWND wdw;
} undoBufferType;

LOCAL undoBufferType undoBuffer = { nothingToDo };
LOCAL aLinePtr undoList;
LOCAL aLineDesc saveLine = { 0, NULL };
LOCAL EDLINE saveLineNr = -1;

LOCAL _ULONG lastInfoLine;
LOCAL HWND lastInfoWin;
LOCAL EDLINE lastLine;
LOCAL EDLINE lastTotal;
LOCAL EDCOLUMN lastRow;
LOCAL _BOOL lastInsert;
LOCAL _BOOL lastIndent;

/* Variablen fuer Suchen und Ersetzen mit Wildcards */

/* In diesen Arrays wird fuer jedes Wildcardzeichen gespeichert,
 * das wievielte Wildcardzeichen es insgesamt ist!
 */
#define MAX_WILD 10
LOCAL _WORD allWildNumbers[MAX_WILD];
LOCAL _WORD oneWildNumbers[MAX_WILD];

/* Das compilerte Suchpattern */
LOCAL tPattern *pPatt;

/* Suchmuster initialisiert: */
LOCAL _BOOL searchInited;

LOCAL CONST _UBYTE *editWIndowInfo = " Zeile: %ld/%ld   Spalte: %d  %s%s";
LOCAL CONST _UBYTE *editInsertOn = "Einf\201gen";
LOCAL CONST _UBYTE *editInsertOff = "\232berschreiben";
LOCAL CONST _UBYTE *editIndentOn = "/Einr\201cken";
LOCAL CONST _UBYTE *editIndentOff = "";



/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _UONG read_b_hz_200(_VOID)											*/
/*	liest die Systemvariable:					 								*/
/*	b_hz_200         LONG $4ba      bisherige Anzahl der 200-Hz Interrupts.		*/
/*	Ursprnglich war dies ein Teil der Funktion SystemTimer die intern Super()	*/
/*	benutzte.																	*/
/*	Ich habe die Super entfernt, wodurch diese Funktion jetzt mit Supexec		*/
/*	aufgerufen werden mu. Auerdem wurde deshalb der Rckgabetyp zu _LONG		*/
/*	Parameter:	keine															*/
/*	Ergebnis:	'Zeit' als long													*/		
/*	11.5.00 von Dimitri Junker	DYJ 4.16f										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _LONG read_b_hz_200(_VOID)
{
	_LONG hz;
	hz = *((_LONG *)0x4ba);
	return hz;
}
	
/*** ---------------------------------------------------------------------- ***/
GLOBAL _ULONG SystemTimer(_VOID)
{

	_ULONG hz;
	
	if(CanDoSuper())
	{
		hz = Ssystem(S_GETLVAL, 0x4ba, 0);
		if ((_LONG)hz != -32)
			return hz;
		hz=(_ULONG)Supexec(read_b_hz_200);	
	}
	else
		hz=clock();
	return hz;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_SetInfoLine(EDITOR *ed)
{
	_UBYTE *text;
	EDCOLUMN col;
	_ULONG system_time;
	
	if (!ed->showInfo)
	{
		return;
	}
	system_time = SystemTimer();
	if ((system_time < 100 || system_time - lastInfoLine > 100) &&
		(lastInfoWin != ed->wdw ||
		 ed->currLineNr != lastLine ||
		 ed->currRow != lastRow ||
		 ed->autoIndent != lastIndent ||
		 ed->insertMode != lastInsert ||
		 ed->totalLineNr != lastTotal))
	{
		lastInfoLine = system_time;
		lastLine = ed->currLineNr;
		lastTotal = ed->totalLineNr;
		lastRow = ed->currRow;
		lastIndent = ed->autoIndent;
		lastInsert = ed->insertMode;
		lastInfoWin = ed->wdw;
		if (ed->readOnly==MODE_RW || ed->listMode)
		{
			if (ed->realTabs)
			{
				EditBase_GetLineAdr(ed, ed->currLineNr, &text, &col);
				col = EditGlobals_RowToIndex(text, ed->currRow, ed->TabSize);
			} else
			{
				col = ed->currRow;
			}
			sprintf(ed->WindowInfo, editWIndowInfo,
				ed->currLineNr + 1,
				ed->totalLineNr,
				col,
				ed->insertMode ? editInsertOn : editInsertOff,
				ed->autoIndent ? editIndentOn : editIndentOff);
		}
		WdwManager_SetWdwInfoline(ed->wdw, ed->WindowInfo);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_SetEditTitle(EDITOR *ed)
{
	if (ed->readOnly==MODE_RW)
	{
		ed->WindowTitle[0] = '\0';
		/* Changed anzeigen mit '*' vor Dateinamen */
		if (ed->changed)
			strBcat(ed->WindowTitle, "*");
		if (ed->fileName.length != 0)
			strBcat(ed->WindowTitle, ed->fileName.text);
		else
			strBcat(ed->WindowTitle, cNoname);
		insert_app_name(ed->WindowTitle, sizeof(ed->WindowTitle));
	}
	WdwManager_SetWdwTitle(ed->wdw, ed->WindowTitle);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_ShowFileChanged(EDITOR *ed)
{
	if (!ed->changed && ed->readOnly==MODE_RW)
	{
		ed->changed = ed->readOnly==MODE_RW;
		EditTools_SetEditTitle(ed);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_UpdateCurrLine(EDITOR *ed, EDLINE line)
{
	saveLineNr = -1;
	ed->currLineNr = LMin(ed->totalLineNr - 1, line);
	ed->currLineNr = LMax(0, line);
	EditBase_GetLineLength(ed, ed->currLineNr, &ed->editLen);
	if (ed->editLen	>= ed->editLine.length)
	{
		SFREE(ed->editLine.text);
		ed->editLine.length = ((ed->editLen + 16) / 16) * 16;
		ed->editLine.text = SYSMALLOC(ed->editLine.length, MEM_NOERR, "UpdateCurrLine");
		if (ed->editLine.text == NULL)
		{
			/* Kein Speicher frei, kompaktieren */
			EditBase_CompactBuffer(ed);
			ed->editLine.text = MALLOC(ed->editLine.length, "UpdateCurrLine");
			if (ed->editLine.text == NULL)
			{
				HALT();
			}
		}
	}
	MemSetZero(ed->editLine.text, ed->editLine.length);
	EditBase_GetCurrLine(ed, ed->editLine.text, ed->editLine.length, &ed->editLen);
	ed->editChanged = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_SetCurrLine(EDITOR *ed, EDLINE line)
{
	if (line == ed->currLineNr)
	{
		return;
	}
	line = LMin(ed->totalLineNr - 1, line);
	line = LMax(0, line);
	if (ed->editChanged)
	{
		/*
		if (ed->editLen != strLen(editLine.text)) { HALT(); }
		*/
		EditBase_PutCurrLine(ed);
		ed->editChanged = FALSE;
	}
	EditTools_UpdateCurrLine(ed, line);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_SetStartLine(EDITOR *ed, EDLINE line, _BOOL docUpdate)
{
	line = LMin(ed->totalLineNr - 1, line);
	line = LMax(0, line);
	ed->StartLine = line;
	if (docUpdate)
	{
		EditGlobals_SetDocument(ed);
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Abfrage-Funktionen */

GLOBAL _VOID EditTools_getCharPos(EDITOR *ed, EDLINE line, EDCOLUMN row, _WORD *x, _WORD *y)
{
	EDLINE saveLine;
	_UBYTE saveCh;
	_UBYTE *charAdr;
	_BOOL restoreLine;
	_UBYTE *text;
	EDCOLUMN len;
	_WORD width;
	
	if (line != ed->currLineNr)
	{
		restoreLine = TRUE;
		saveLine = ed->currLineNr;
		EditTools_SetCurrLine(ed, line);
	} else
	{
		restoreLine = FALSE;
		saveLine = 0;
	}
	*y = ed->editWork.g_y + (_WORD)(ed->currLineNr - ed->StartLine) * ed->charHeight;
	if (row < 0)
	{
		*x = ed->editWork.g_x;
		return;
	}
	if (!ed->monoSpaced	|| ed->realTabs || ed->DisplayEffects)
	{
		/* So geht's schneller: */
		if (!EditBase_GetLineAdr(ed, line, &text, &len))
		{
			*x = 0;
			*y = 0;
		} else
		{
			charAdr = &text[len];
			saveCh = *charAdr;
			/* String terminieren */
			*charAdr = '\0';
			/* Cursorposition bestimmen */
			if (ed->realTabs)
			{
				len = (EDCOLUMN)Str_ConvertTabs(text, theDrawLine, sizeof(theDrawLine), ed->TabSize);
				row = EditGlobals_RowToIndex(text, row, ed->TabSize);
				text = theDrawLine;
			}
			if (row > len)
			{
				row = len;
			}
			/* Jetzt Breite bestimmen */
			width = EditGlobals_GetTextWidth(ed, text, row, len, 30000);
			*charAdr = saveCh;
			*x = ed->editWork.g_x + width + pixOff - ed->leftOffset;
		}
	} else
	{
		*x = ed->editWork.g_x + pixOff - ed->leftOffset + row * ed->charWidth;
	}
	if (restoreLine)
		EditTools_SetCurrLine(ed, saveLine);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Neues Konzept fuer UNDO:
 * Es werden pro Datei 64 Operationen gemerkt. Eine Operation ist dabei 
 * folgendes:
 * 
 * - Einfuegen eines Wortes (== alle Zeichen bis zu einem Space oder Befehl)
 * - Einfuegen eines oder mehrerer Zeilenumbrueche
 * - Einfuegen, Ersetzen oder Loeschen eines Blocks (oder Files)
 *
 * Implementation: 
 * Es wird ein Array von Undobuffern implementiert, in dem die Operationen
 * alle festgehalten werden. Dieses Array stellt einen Ring dar, wobei ein
 * globaler Index festhaelt, welches Element das erste im Ring ist.
 * In jedem Undobuffer wird festgehalten, die wievielte Operation dies
 * im Ring ist, sowie alle benoetigten Daten zur Rueckgaengigmachung der
 * Aktion. Dazu wird folgendes festgehalten:
 * Die Anzahl der Zeichen, die eingefuegt oder geloescht wurden.
 * Die Zeichen, die eingefuegt oder geloescht wurden (in einem Buffer)
 * Die aktuelle Cursorposition
 * Die Position der Blockmarken vor und nach der Operation
 * 
 *
 * Da diese Operationen teilweise sehr speicherintensiv sein koennen,
 * ist es notwendig, einen Teil der Implementation in EditBase vorzunehmen, 
 * damit bei einer Speicherknappheit die aeltesten Undooperationen 
 * auch geloescht werden koennen.
 * Da die Implementation pro Datei erfolgen soll, ist es notwendig, den 
 * Undobuffer in den Typ EDITOR mit aufzunehmen. Dadurch waechst der Platz-
 * bedarf der Editor-Struktur nochmal deutlich an.
 */

#  if 0
/* neue Implementation, erstmal als Kommentar. */
#define maxUndo 32

typedef struct
{
	_ULONG operationNr;
	undoType whatToUndo;
	_UBYTE *buffer;
	_ULONG bufSize;
	aMark oldStart;
	aMark oldEnd;
	aMark newStart;
	aMark newEnd;
	HWND wdw;
} undoBufferType;

typedef undoBufferType undoArrayType[maxUndo];

typedef struct
{
	_WORD undoIndex;
	_WORD undoMax;
	undoArrayType undoBuffers;
	_BOOL undoDone;
} undoDescriptor;

#endif

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID freeUndoBuffer(_VOID)
{
	if (undoBuffer.buffer != NULL)
	{
		SFREE(undoBuffer.buffer);
		undoBuffer.buffer = NULL;
	}
	if (undoBuffer.undoLine.text != NULL)
	{
		SFREE(undoBuffer.undoLine.text);
		undoBuffer.undoLine.text = NULL;
	}
	undoBuffer.bufSize = 0;
	undoBuffer.undoLine.text = NULL;
	undoBuffer.buffer = NULL;
	undoBuffer.lines = 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ClearUndoBuffer(_VOID)
{
	if (undoBuffer.whatToUndo != nothingToDo)
	{
		freeUndoBuffer();
		undoBuffer.whatToUndo = nothingToDo;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_SaveLine(aLinePtr l, EDLINE lNr)
{
	if (lNr != saveLineNr)
	{
		if (saveLine.text != NULL)
		{
			SFREE(saveLine.text);
			saveLine.text = NULL;
		}
		if (l->zeile.length <= 0)
		{
			ErrorOut(EO_ERROR, "SaveLine: illegal length");
			return;
		}
		saveLine.text = MALLOC(l->zeile.length, "SaveLine");
		if (saveLine.text == NULL)
		{
			saveLineNr = -1;
			saveLine.length = 0;
		} else
		{
			saveLineNr = lNr;
			saveLine.length = l->zeile.length;
			strCpy(saveLine.text, l->zeile.text);
		}
		ClearUndoBuffer();
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_PutInUndoList(aLinePtr line)
{
	/* Zeile in Undopuffer haengen */
	line->next = undoList;
	line->prev = NULL;
	undoList = line;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_DeleteLine(EDITOR *ed, _BOOL redraw)
{
	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	EditTools_ShowFileChanged(ed);
	if (ed->currLineNr == ed->totalLineNr - 1)
	{
		/* Letzte Zeile: Nur Inhalt loeschen! */
		ed->editLine.text[0] = CR;
		ed->editLine.text[1] = LF;
		ed->editLine.text[2] = '\0';
		ed->editLen = 2;
		ed->editChanged = TRUE;
		EditBase_PutCurrLine(ed);
		EditDraw_redrawLine(ed, FALSE, TRUE);
	} else
	{
		ed->editChanged = FALSE;
		if (!EditBase_DeleteLine(ed, ed->currLineNr))
		{
			return;
		}
		EditTools_UpdateCurrLine(ed, ed->currLineNr);
		if (redraw)
		{
			/* Bildschirm neu aufbauen */
			if ((ed->currLineNr - ed->StartLine) + 1 < ed->windLines)
			{
				EditDraw_scrollRegion(ed, (_WORD)(ed->currLineNr - ed->StartLine), ed->windLines, -1);
				/* drawLine(ed, ed->windLines-1); */
				/* drawLine(ed, ed->windLines); */
			} else
			{
				/* nur unterste Zeile neu zeichnen */
				EditDraw_drawLine(ed, ed->windLines - 1);
			}
		}
	}
	EditGlobals_SetDocument(ed);
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_CheckAndFreeUndo(HWND wdw)
{
	if (wdw == undoBuffer.wdw)
	{
		ClearUndoBuffer();
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_FreeUndoBuffer(_VOID)
{
	if (saveLine.text != NULL)
	{
		SFREE(saveLine.text);
		saveLine.text = NULL;
	}
	saveLineNr = -1;
	saveLine.length = 0;
	ClearUndoBuffer();
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _VOID EditTools_Undo(EDITOR *ed)
{
	aMark m1;
	_UBYTE *secondBuff;
	_ULONG secondSize;
	EDLINE blEnd;

	EditGlobals_HideCursor(ed);
	if (ed->editChanged)
	{
		ed->editChanged = FALSE;
		EditTools_UpdateCurrLine(ed, ed->currLineNr);
	} else
	{
		if (ed->wdw != undoBuffer.wdw)
		{
			CatGlobal_Bing(cBingNoUndo);
			EditGlobals_ShowCursor(ed);
			return;
		}
		switch (undoBuffer.whatToUndo)
		{
		case blockToChar:
		case blockCut:
			/* Es wird der gesicherte Block an der alten Stelle wieder eingefuegt. 
			 * Da die kompletten Start- und Endzeilen gesichert wurden, muss die
			 * aktuelle Zeile nur gesichert werden und dann geloescht und dann der 
			 * alte Block eingefuegt werden
			 */
			
			/* Alte Position wieder herstellen */
			EditTools_SetCurrLine(ed, undoBuffer.oldStart.line);
			/* Erstmal aktuelle Zeile in Undobuffer sichern */
			undoBuffer.undoLine.text = MALLOC(ed->editLine.length, "Edit_Undo");
			if (undoBuffer.undoLine.text == NULL)
			{
				undoBuffer.whatToUndo = nothingToDo;
			} else
			{
				undoBuffer.undoLine.length = ed->editLine.length;
				strCpy(undoBuffer.undoLine.text, ed->editLine.text);
				undoBuffer.newStart.line = ed->currLineNr;
				undoBuffer.newEnd.line = ed->currLineNr;
				undoBuffer.lines = 1;
			}
			/* Aktuelle Zeile in Buffer schreiben (Datenkonsistenz) */
			EditBase_PutCurrLine(ed);
			ed->currRow = undoBuffer.oldStart.row;
			/* Aktuelle und naechste Zeile loeschen */
			if (!undoBuffer.atEndOfText)
			{
				if (ed->currLineNr < ed->totalLineNr - 1)
				{
					EditBase_DeleteLine(ed, ed->currLineNr + 1);
				}
			}
			EditBase_DeleteLine(ed, ed->currLineNr);
			/* Undobuffer jetzt einkopieren */
			ed->currLineNr = undoBuffer.oldStart.line;
			ed->currRow = 0;
			ed->block = TRUE;
			EditBase_InsertBuffer(ed, &undoBuffer.buffer, undoBuffer.bufSize, TRUE);
			ed->currRow = undoBuffer.oldStart.row;
			undoBuffer.buffer = NULL;
			undoBuffer.bufSize = 0;
			/* Jetzt neuen Undotyp setzen */
			if (undoBuffer.whatToUndo == blockCut ||
				undoBuffer.whatToUndo == blockToChar)	/* kann vorher veraendert worden sein */
			{
				undoBuffer.whatToUndo = cutUndo;
			}
			/* Jetzt Blockmarken wiederherstellen */
			ed->blocks[0].blockStart = undoBuffer.oldStart;
			ed->blocks[0].blockEnd = undoBuffer.oldEnd;
			ed->block = TRUE;
			break;
		
		case blockPaste:
			/* Vorgehensweise: Aktuellen Block sichern und dann ausschneiden,
			 * dann letzten Block wieder einfuegen. Blockmarken umsetzen.
			 */
			if (ed->blocks[0].blockEnd.line < 0 || ed->blocks[0].blockStart.line < 0)
			{
				EditGlobals_ShowCursor(ed);
				return;
			}
			undoBuffer.newStart = ed->blocks[0].blockStart;
			undoBuffer.newEnd = ed->blocks[0].blockEnd;
			/* gesamten Block duplizieren */
			if (!EditBase_CopyToBuff(ed, ed->blocks[0].blockStart.line, ed->blocks[0].blockEnd.line + 1, &secondBuff, &secondSize))
			{
				undoBuffer.whatToUndo = nothingToDo;
			}
			/* Jetzt diesen Block ausschneiden! */
			if (ed->blocks[0].blockEnd.line >= ed->totalLineNr - 1)
			{
				blEnd = ed->blocks[0].blockEnd.line;
			} else
			{
				blEnd = ed->blocks[0].blockEnd.line + 1;
			}
			EditBase_DeleteSomeLines(ed, ed->blocks[0].blockStart.line, blEnd);
			/* Und jetzt den alten wieder einfuegen */
			EditTools_SetCurrLine(ed, undoBuffer.oldStart.line);
			ed->currRow = undoBuffer.oldStart.row;
			/* Undobuffer jetzt einkopieren */
			EditBase_InsertBuffer(ed, &undoBuffer.buffer, undoBuffer.bufSize, TRUE);
			ed->currRow = undoBuffer.oldStart.row;
			if (undoBuffer.whatToUndo == blockPaste)
			{
				undoBuffer.buffer = secondBuff;
				undoBuffer.bufSize = secondSize;
			} else
			{
				undoBuffer.buffer = NULL;
				undoBuffer.bufSize = 0;
			}
			/* Blockmarken setzen */
			ed->blocks[0].blockStart = undoBuffer.oldStart;
			ed->blocks[0].blockEnd = undoBuffer.oldEnd;
			/* Jetzt noch Blockmarken swappen, falls blockPaste */
			if (undoBuffer.whatToUndo == blockPaste)
			{
				m1 = undoBuffer.newStart;
				undoBuffer.newStart = undoBuffer.oldStart;
				undoBuffer.oldStart = m1;
				m1 = undoBuffer.newEnd;
				undoBuffer.newEnd = undoBuffer.oldEnd;
				undoBuffer.oldEnd = m1;
			}
			/* SaveOp fuer naechstes Undo setzen */
			ed->block = TRUE;
			break;
		
		case cutUndo:
		case normalPaste:		/* Blockbereich komplett loeschen und einfach alte Zeile restaurieren */
			/* gesamten Block duplizieren */
			EditTools_SetCurrLine(ed, undoBuffer.oldStart.line);
			if (undoBuffer.oldEnd.line >= ed->totalLineNr - 1)
			{
				blEnd = undoBuffer.oldEnd.line;
			} else
			{
				blEnd = undoBuffer.oldEnd.line + 1;
			}
			if (undoBuffer.buffer != NULL)
			{
				SFREE(undoBuffer.buffer);
				undoBuffer.buffer = NULL;
			}
			undoBuffer.bufSize = 0;
			if (!EditBase_CopyToBuff(ed, undoBuffer.oldStart.line, blEnd, &undoBuffer.buffer, &undoBuffer.bufSize))
			{
				undoBuffer.whatToUndo = nothingToDo;
				freeUndoBuffer();
				EditGlobals_ShowCursor(ed);
				return;
			}
			/* Blockbereich loeschen */
			EditBase_DeleteSomeLines(ed, undoBuffer.oldStart.line, undoBuffer.oldEnd.line);
			/* Alte gesicherte Zeile einfuegen */
			EditBase_InsertLine(ed, undoBuffer.oldStart.line, undoBuffer.undoLine.text, TRUE, (EDBUFSIZE)strLen(undoBuffer.undoLine.text));
			undoBuffer.whatToUndo = blockCut;
			ed->currRow = undoBuffer.oldStart.row;
			SFREE(undoBuffer.undoLine.text);
			undoBuffer.undoLine.text = NULL;
			ed->block = FALSE;
			ed->blocks[0].blockStart.line = -1;
			ed->blocks[0].blockEnd.line = -1;
			break;
		
		case charToBlock:
		case nothingToDo:
			break;
		}
		EditTools_UpdateCurrLine(ed, ed->currLineNr);
		EditGlobals_SetDocument(ed);
	}
	ed->prefRow =ed->currRow = Max(0, Min(ed->editLen, ed->currRow));
	EditGlobals_FixCursorPos(ed);
	/*
	countLines();
	*/
	ed->cursPos.row = -1;			/* Neue Positionsberechnung fuer Cursor erzwingen */
	EditTools_ShowFileChanged(ed);
	WdwManager_RedrawWdw(ed->wdw, &ed->editWork);
	EditDraw_CenterCurrline(ed);
	EditGlobals_SetDocument(ed);
	EditGlobals_ShowCursor(ed);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_SaveOp(EDITOR *ed, undoType op, _BOOL append)
{
	EDLINE blEnd;

	UNUSED(append);
	EditBase_PutCurrLine(ed);
	undoBuffer.whatToUndo = op;
	saveLineNr = -1;
	undoBuffer.wdw = ed->wdw;
	switch (op)
	{
	case blockPaste:
	case blockCut:
	case blockToChar:
		if (ed->blocks[0].blockStart.line < 0 ||
			ed->blocks[0].blockEnd.line < 0 ||
			ed->blocks[0].blockEnd.line < ed->blocks[0].blockStart.line)
		{
			return;
		}
		/* alten Undobuffer freigeben, wird nicht mehr gebraucht! */
		freeUndoBuffer();
		/* Blockmarken merken */
		undoBuffer.oldStart = ed->blocks[0].blockStart;
		undoBuffer.oldEnd = ed->blocks[0].blockEnd;
		/* gesamten Block duplizieren */
		if (ed->blocks[0].blockEnd.line >= ed->totalLineNr - 1)
		{
			blEnd = ed->blocks[0].blockEnd.line;
			undoBuffer.atEndOfText = TRUE;
		} else
		{
			blEnd = ed->blocks[0].blockEnd.line + 1;
			undoBuffer.atEndOfText = FALSE;
		}
		if (!EditBase_CopyToBuff(ed, ed->blocks[0].blockStart.line, blEnd, &undoBuffer.buffer, &undoBuffer.bufSize))
		{
			undoBuffer.whatToUndo = nothingToDo;
			freeUndoBuffer();
			return;
		}
		/* Cursorposition restaurieren */
		break;
	
	case normalPaste:
		/* erstmal alten buffer freigeben, wird nicht mehr gebraucht
		 */
		freeUndoBuffer();
		/* Kann ich nicht so leicht machen, bessere Idee gehabt:
		 * nachher noch einen Aufruf mit den entsprechenden Werten in den BlockMarken
		 * Die sollten sowieso drin sein, da die auch bei blockPaste drinstehen muessen.
		 */
		/* Ich muss mir nur die aktuelle Zeile merken, falls die gesplittet wird */
		/* currLine merken und undoBuffer darauf setzen! */
		if (undoBuffer.undoLine.text != NULL)
		{
			SFREE(undoBuffer.undoLine.text);
		}
		undoBuffer.undoLine.text = MALLOC(ed->editLine.length, "Edit_SaveOp");
		if (undoBuffer.undoLine.text == NULL)
		{
			undoBuffer.whatToUndo = nothingToDo;
			return;
		} else
		{
			undoBuffer.undoLine.length = ed->editLine.length;
		}
		strCpy(undoBuffer.undoLine.text, ed->editLine.text);
		undoBuffer.newStart.line = ed->currLineNr;
		undoBuffer.newEnd.line = ed->currLineNr;
		undoBuffer.lines = 1;
		break;
	
	case charToBlock:
	case cutUndo:
	case nothingToDo:
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_NewBlockToUndoBuffer(EDITOR *ed)
{
	switch (undoBuffer.whatToUndo)
	{
	case blockPaste:
		undoBuffer.newStart = ed->blocks[0].blockStart;
		undoBuffer.newEnd = ed->blocks[0].blockEnd;
		break;
	case normalPaste:
		undoBuffer.oldStart = ed->blocks[0].blockStart;
		undoBuffer.oldEnd = ed->blocks[0].blockEnd;
		break;
	default:
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL EditTools_PossibleUndo(EDITOR *ed)
{
	return ed->editChanged || (ed->wdw == undoBuffer.wdw && undoBuffer.whatToUndo != nothingToDo);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Blockfunktionen I */

LOCAL _VOID Blockstart_Unmark(EDITOR *ed, EDLINE oldLine, EDCOLUMN oldRow)
{
	ed->blocks[0].blockStart.line = oldLine;
	ed->blocks[0].blockStart.row = oldRow;
	EditDraw_ShowBlockMark(ed, TRUE);
	ed->blocks[0].blockStart.line = ed->currLineNr;
	ed->blocks[0].blockStart.row = ed->currRow;
}

GLOBAL _VOID EditTools_SetBlockstart(EDITOR *ed)
{
	EDLINE oldLine;
	EDCOLUMN oldRow;

	if (ed->multiBlockIdx > 0)
	{
		return;
	}
	oldLine = ed->blocks[0].blockStart.line;
	oldRow = ed->blocks[0].blockStart.row;
	ed->blocks[0].blockStart.line = ed->currLineNr;
	ed->blocks[0].blockStart.row = ed->currRow;
	if (ed->blocks[0].blockEnd.line >= ed->blocks[0].blockStart.line)
	{
		if (oldLine >= 0)
		{
			/* Block erweitern */
			Blockstart_Unmark(ed, oldLine, oldRow);
			EditDraw_ShowBlockMark(ed, TRUE);
		} else
		{
			EditDraw_ShowBlockMark(ed, TRUE);
		}
	} else
	{
		if (ed->blocks[0].blockEnd.line >= oldLine)
		{
			/* alten Block loeschen */
			Blockstart_Unmark(ed, oldLine, oldRow);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID Blockend_Unmark(EDITOR *ed, EDLINE oldLine, EDCOLUMN oldRow)
{
	ed->blocks[0].blockEnd.line = oldLine;
	ed->blocks[0].blockEnd.row = oldRow;
	EditDraw_ShowBlockMark(ed, TRUE);
	ed->blocks[0].blockEnd.line = ed->currLineNr;
	ed->blocks[0].blockEnd.row = ed->currRow;
}

GLOBAL _VOID EditTools_SetBlockend(EDITOR *ed)
{
	EDLINE oldLine;
	EDCOLUMN oldRow;

	if (ed->multiBlockIdx > 0)
	{
		return;
	}
	oldLine = ed->blocks[0].blockEnd.line;
	oldRow = ed->blocks[0].blockEnd.row;
	ed->blocks[0].blockEnd.line = ed->currLineNr;
	ed->blocks[0].blockEnd.row = ed->currRow;
	if (ed->blocks[0].blockEnd.line >= ed->blocks[0].blockStart.line)
	{
		if (oldLine >= 0 &&
			ed->blocks[0].blockStart.line >= 0)
		{
			/* Block erweitern */
			Blockend_Unmark(ed, oldLine, oldRow);
			EditDraw_ShowBlockMark(ed, TRUE);
		} else
		{
			EditDraw_ShowBlockMark(ed, TRUE);
		}
	} else
	{
		if (oldLine >= ed->blocks[0].blockStart.line)
		{
			/* alte Blockmarkierung loeschen */
			Blockend_Unmark(ed, oldLine, oldRow);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Loescht Blockmarkierung */
GLOBAL _VOID EditTools_ClearBlock(EDITOR *ed)
{
	_WORD i;

	EditDraw_ShowBlockMark(ed, TRUE);
	for (i = 0; i < maxBlocks; i++)
	{
		ed->blocks[i].blockStart.line = -1;
		ed->blocks[i].blockEnd.line = -1;
	}
	ed->multiBlockIdx = -1;
	ed->block = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_GetClickLineAndRow(EDITOR *ed, _WORD x, _WORD y, EDLINE *clLine, EDCOLUMN *clRow)
{
	EDCOLUMN sl;
	_UBYTE *text;
	EDCOLUMN i, j;
	_BOOL takeLast;
	EDCOLUMN xSub;
	_WORD w;
	
	/* Feststellen, in welche Zeile geklickt wurde */
	*clLine = ed->StartLine + (y - ed->editWork.g_y) / ed->charHeight;
	if (*clLine < 0)
	{
		*clLine = 0;
		*clRow = 0;
		return;
	}
	if (*clLine > ed->totalLineNr - 1)
	{
		*clLine = ed->totalLineNr - 1;
		takeLast = TRUE;
	} else
	{
		takeLast = FALSE;
	}
	/* Zeile zur aktuellen machen */
	EditTools_SetCurrLine(ed, *clLine);
	/* Jetzt Spalte bestimmen */
	if (!ed->insertMode)
	{
		if (ed->monoSpaced)
		{
			xSub = ed->charWidth / 2;
		} else
		{
			xSub = 0;
		}
	} else
	{
		xSub = 0;
	}
	x += pixOff;					/* PixelOffset addieren */
	*clRow = (x + ed->leftOffset - ed->editWork.g_x - xSub) / ed->charWidth;

	sl = ed->editLen;
	if (*clRow < 0)
		*clRow = 0;
	/* clRow ist nur Naeherung, mal suchen gehen */
	if (takeLast)
	{
		ed->currRow = sl;
		EditGlobals_FixCursorPos(ed);
		*clRow = ed->currRow;
		return;
	}
	if (ed->realTabs)
	{
		sl = (EDCOLUMN)Str_ConvertTabs(ed->editLine.text, theDrawLine, sizeof(theDrawLine), ed->TabSize);
		text = theDrawLine;
	} else
	{
		text = ed->editLine.text;
	}
	*clRow = Max(0, Min(*clRow, sl));
	if (x <= ed->editWork.g_x)
	{
		*clRow = 0;					/* Sonderfall! */
	} else if (!ed->monoSpaced || ed->DisplayEffects)
	{
		/* Nachsehen, ob berechnete Position zu klein! */
		w = 0;
		do
		{
			if (*clRow < sl)
			{
				++(*clRow);
				w = EditGlobals_GetTextWidth(ed, text, *clRow, sl, 30000);
			}
		} while (x - ed->editWork.g_x + ed->leftOffset >= w && *clRow < sl);
		++(*clRow);
		do
		{
			--(*clRow);
			if (*clRow > 0 && *clRow <= sl)
			{
				w = EditGlobals_GetTextWidth(ed, text, *clRow, sl, 30000);
			}
		} while (x - ed->editWork.g_x + ed->leftOffset <= w && *clRow != 0);
	}
	*clRow = Min(*clRow, sl);
	if (ed->realTabs)
	{
		EditBase_GetLineAdr(ed, ed->currLineNr, &text, &sl);
		j = 0;
		i = 0;
		while (j <= *clRow)
		{
			if (text[i] == TAB)
			{
				j = ((j / ed->TabSize) + 1) * ed->TabSize;
			} else
			{
				++j;
			}
			++i;
		}
		if (i > 0)
			*clRow = i - 1;
		else
			*clRow = i;
	}
	/* Jetzt noch CR abfangen */
	if (*clRow > 0 && text[*clRow - 1] == CR)
		--(*clRow);
	ed->currRow = *clRow;
	EditGlobals_FixCursorPos(ed);
	*clRow = ed->currRow;
}

/*** ---------------------------------------------------------------------- ***/

#define is_word_end(ch) ( \
	(ch) == '\0' || \
	(ch) == CR || \
	(ch) == LF || \
	(ch) == ' ' || \
	(ch) == '.' || \
	(ch) == ',' || \
	(ch) == ';' || \
	(ch) == '!' || \
	(ch) == '-' || \
	(ch) == '?' || \
	(ch) == ':' || \
	(ch) == '"' || \
	(ch) == '\'' || \
	(ch) == '}' || \
	(ch) == ']' || \
	(ch) == ')' \
	)

GLOBAL _VOID EditTools_TestBlockMarksAndSwap(EDITOR *ed)
{
	EDCOLUMN i, j;
	_WORD bl;
	EDLINE l;
	EDCOLUMN col;
	
	for (bl = 0; bl < maxBlocks; bl++)
	{
		if (ed->blocks[bl].blockStart.line >= 0 &&
			ed->blocks[bl].blockEnd.line >= 0)
		{
			if (ed->blocks[bl].blockEnd.line < ed->blocks[bl].blockStart.line)
			{
				l = ed->blocks[bl].blockEnd.line;
				ed->blocks[bl].blockEnd.line = ed->blocks[bl].blockStart.line;
				ed->blocks[bl].blockStart.line = l;
				col = ed->blocks[bl].blockEnd.row;
				ed->blocks[bl].blockEnd.row = ed->blocks[bl].blockStart.row;
				ed->blocks[bl].blockStart.row = col;
			} else if (ed->blocks[bl].blockEnd.line == ed->blocks[bl].blockStart.line &&
					   ed->blocks[bl].blockEnd.row < ed->blocks[bl].blockStart.row)
			{
				col = ed->blocks[bl].blockEnd.row;
				ed->blocks[bl].blockEnd.row = ed->blocks[bl].blockStart.row;
				ed->blocks[bl].blockStart.row = col;
			}
			/* Jetzt noch Spezialbehandlung fuer Effekte */
			if (ed->DisplayEffects && ed->blocks[bl].blockStart.row > 0)
			{
				EditTools_SetCurrLine(ed, ed->blocks[bl].blockStart.line);
				if (is_eff_char(ed->editLine.text[ed->blocks[bl].blockStart.row - 1]))
				{
					/* Check for second occurence of effect char in line */
					i = ed->blocks[bl].blockStart.row - 1;
					j = i;
					do
					{
						do
							++j;
						while (j < ed->editLen && ed->editLine.text[j] != ed->editLine.text[i]);
					} while (j < ed->editLen && !is_word_end(ed->editLine.text[j + 1]));
					if (j < ed->editLen)
					{
						/* Ok, wirklich ein Effektzeichen, also hinzunehmen */
						--ed->blocks[bl].blockStart.row;
					}
				}
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL EDCOLUMN EditTools_ToWordEnd(EDITOR *ed, EDLINE line, EDCOLUMN row)
{
	EDCOLUMN l, z;
	_UBYTE *text;

	if (!EditBase_GetLineAdr(ed, line, &text, &l))
	{
		return row;
	}
	if (l > 0 && text[l - 1] == CR)
	{
		--l;
	}
	z = row;
	while (z < l && !setin(TrennSet, text[z]))
	{
		++z;
	}
	if (z < l && is_eff_char(text[z]))
	{
		++z;
	}
	return z;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL EDCOLUMN EditTools_ToWordStart(EDITOR *ed, EDLINE line, EDCOLUMN row)
{
	EDCOLUMN l, z;
	_UBYTE *text;

	if (!EditBase_GetLineAdr(ed, line, &text, &l))
	{
		return row;
	}
	z = Min(l, row);
	while (z > 0 && !setin(TrennSet, text[z]))
	{
		--z;
	}
	if (z != 0 && z + 1 < row)
	{
		++z;
	}
	if (z > 0 && is_eff_char(text[z - 1]))
	{
		--z;
	}
	return z;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL EDCOLUMN EditTools_ToLineStart(EDITOR *ed, EDLINE line)
{
	/* Ist nur gedacht fuer andere moegliche Implementationen, deshalb 
	 * als Funktion
	 */
	UNUSED(ed);
	UNUSED(line);
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt die Cursorposition fuer das Zeilenende zurueck */

GLOBAL EDCOLUMN EditTools_ToLineEnd(EDITOR *ed, EDLINE line)
{
	EDCOLUMN cRow;
	EDCOLUMN ret;

	cRow = ed->currRow;
	if (EditBase_GetLineLength(ed, line, &ed->currRow))
	{
		EditGlobals_FixCursorPos(ed);
		ret = ed->currRow;
	} else
	{
		ret = cRow;
	}
	ed->prefRow =ed->currRow = cRow;
	return ret;
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _VOID EditTools_MarkLine(EDITOR *ed)
{
	EDCOLUMN clRow;
	EDLINE cLine;

	clRow = ed->currRow;
	ed->currRow = 0;
	EditTools_SetBlockstart(ed);
	if (ed->currLineNr == ed->totalLineNr - 1)
	{
		EditBase_GetLineLength(ed, ed->currLineNr, &ed->currRow);
		EditGlobals_FixCursorPos(ed);
		EditTools_SetBlockend(ed);
	} else
	{
		cLine = ed->currLineNr;
		EditTools_SetCurrLine(ed, ed->currLineNr + 1);
		EditTools_SetBlockend(ed);
		EditTools_SetCurrLine(ed, cLine);
	}
	ed->prefRow =ed->currRow = clRow;
	ed->block = TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_MarkWord(EDITOR *ed)
{
	typedef enum { none, all, line } wordRange;

	EDCOLUMN clRow;
	EDLINE clLine;
	_UBYTE opChar;
	_UBYTE ch;
	_UBYTE *text;
	EDCOLUMN l;
	EDCOLUMN depth;
	_BOOL found;
	searchDir dir;
	wordRange range;

	clRow = ed->currRow;
	clLine = ed->currLineNr;
	/* Feststellen, welches Zeichen unter Cursor ist */
	ch = ed->editLine.text[ed->currRow];
	found = FALSE;
	/* gegengesetztes Zeichen feststellen */
	switch (ch)
	{
	case '{':
		opChar = '}';
		dir = sdir_forward;
		range = all;
		break;
	case '}':
		opChar = '{';
		dir = sdir_backward;
		range = all;
		break;
	case '(':
		opChar = ')';
		dir = sdir_forward;
		range = all;
		break;
	case ')':
		opChar = '(';
		dir = sdir_backward;
		range = all;
		break;
	case '[':
		opChar = ']';
		dir = sdir_forward;
		range = all;
		break;
	case ']':
		opChar = '[';
		dir = sdir_backward;
		range = all;
		break;
	case '<':
		opChar = '>';
		dir = sdir_forward;
		range = all;
		break;
	case '>':
		opChar = '<';
		dir = sdir_backward;
		range = all;
		break;
	case '\'':
	case '"':
		opChar = ch;
		range = line;
		dir = sdir_forward;
		break;
	default:
		range = none;
		opChar = '\0';
		dir = sdir_forward;
		break;
	}
	if (range == line)
	{
		/* nur aktuelle Zeile durchsuchen, vorrangig vorwaerts! */
		text = ed->editLine.text;
		l = ed->editLen;
		++ed->currRow;
		for (;;)
		{
			if (ed->currRow >= l)
				break;
			if (text[ed->currRow] == opChar)
			{
				found = TRUE;
				break;
			}
			++ed->currRow;
		}
		if (!found && ed->currRow > 0)
		{
			/* Rueckwaerts suchen */
			ed->currRow = clRow - 1;
			for (;;)
			{
				if (ed->currRow < 0)
					break;
				if (text[ed->currRow] == opChar)
				{
					found = TRUE;
					break;
				}
				--ed->currRow;
			}
		}
		if (!found)
		{
			ed->currRow = clRow;
		} else
		{
			ed->block = TRUE;
			if (ed->currRow < clRow)
			{
				EditTools_SetBlockstart(ed);
				ed->currRow = clRow + 1;
				EditTools_SetBlockend(ed);
			} else
			{
				++ed->currRow;
				EditTools_SetBlockend(ed);
				ed->currRow = clRow;
				EditTools_SetBlockstart(ed);
			}
		}
	} else if (range == all)
	{
		depth = 0;
		if (dir == sdir_forward)
		{
			++ed->currRow;
			for (;;)
			{
				text = ed->editLine.text;
				l = ed->editLen;
				for (;;)
				{
					if (text[ed->currRow] == opChar)
					{
						if (depth == 0)
						{
							found = TRUE;
							break;
						} else
						{
							--depth;
						}
					} else if (text[ed->currRow] == ch)
					{
						++depth;
					}
					if (ed->currRow >= l)
					{
						break;
					}
					++ed->currRow;
				}
				if (found)
					break;
				if (ed->currLineNr == ed->totalLineNr - 1)
				{
					break;
				} else
				{
					EditTools_SetCurrLine(ed, ed->currLineNr + 1);
					ed->currRow = 0;
				}
			}
			if (found)
			{
				++ed->currRow;
				EditTools_SetBlockend(ed);
				EditTools_SetCurrLine(ed, clLine);
				ed->currRow = clRow;
				EditTools_SetBlockstart(ed);
				ed->block = TRUE;
			}
		} else
		{							/* dir == sdir_backward */
			--ed->currRow;
			text = ed->editLine.text;
			for (;;)
			{
				for (;;)
				{
					if (ed->currRow < 0)
						break;
					if (text[ed->currRow] == opChar)
					{
						if (depth == 0)
						{
							found = TRUE;
							break;
						} else
						{
							--depth;
						}
					} else if (text[ed->currRow] == ch)
					{
						++depth;
					}
					--ed->currRow;
				}
				if (found)
					break;
				if (ed->currLineNr == 0)
				{
					break;
				} else
				{
					EditTools_SetCurrLine(ed, ed->currLineNr - 1);
					l = ed->editLen;
					while (l > 0 && text[l - 1] < 0x20)
					{
						--l;
					}
					if (l > 0)
						ed->currRow = l - 1;
					else
						ed->currRow = l;
				}
			}
			if (found)
			{
				EditTools_SetBlockstart(ed);
				EditTools_SetCurrLine(ed, clLine);
				ed->currRow = clRow + 1;
				EditTools_SetBlockend(ed);
				ed->block = TRUE;
			}
		}
	}
	if (!found)
	{
		EditTools_SetCurrLine(ed, clLine);
		ed->currRow = clRow;
		ed->currRow = EditTools_ToWordEnd(ed, ed->currLineNr, ed->currRow);
		EditTools_SetBlockend(ed);
		if (ed->currRow > 0)
		{
			--ed->currRow;
		}
		ed->currRow = EditTools_ToWordStart(ed, ed->currLineNr, ed->currRow);
		EditTools_SetBlockstart(ed);
		ed->block = TRUE;
	}
	ed->prefRow = ed->currRow;
}

/*** ---------------------------------------------------------------------- ***/

/* Wird mit gedrueckter linker Taste aufgerufen, uebernimmt die Blockmarkierung 
 */
GLOBAL _VOID EditTools_MarkBlock(EDITOR *ed, _WORD ox, _WORD oy, _BOOL init, _BOOL fromTop)
{
	_UWORD b;
	GRECT work;
	EDLINE newLine;
	EDLINE oldLine;
	EDCOLUMN oldRow;
	EDCOLUMN oldPixRow;
	EDCOLUMN newPixRow;
	EDCOLUMN newRow;
	aMark newEnd;
	aMark newStart;
	aMark saveEnd;
	aMark saveStart;
	_WORD mx, my;
	_WORD lines;
	_BOOL isRight;
	_BOOL isLeft;
	_BOOL oldIsRight;
	_BOOL oldIsLeft;
	_WORD dummy;
	_UWORD udummy;
	
	/* Arbeitsweise: Invertiert den von der Maus ueberstrichenen Bereich und scrollt notfalls das Fenster.
	 * Dabei wird jeweils der Block neu festgestellt, damit das auch beim Scrollen noch funktioniert.
	 */
	/* Erst mal die Maus anschalten und auf Textcursor setzen */
	EditGlobals_SaveMouse();
	EditGlobals_MouseCursor();
	EditGlobals_ShowMouse(TRUE);
	Wind_Update(BEG_MCTRL);			/* Wir haben jetzt die komplette Mauskontrolle          */
	EditGlobals_ClipWork(ed);		/* Clipping auf Work-bereich vom Editor legen           */
	work = ed->editWork;			/* Work-Bereich als xy-Paar berechnen                   */
	work.g_w = work.g_x + work.g_w - 1;
	work.g_h = work.g_y + work.g_h - 1;
	EditTools_GetClickLineAndRow(ed, ox, oy, &oldLine, &oldRow);
	newStart = ed->blocks[0].blockStart;
	newEnd = ed->blocks[0].blockEnd;
	if (init)
	{
		saveStart = ed->blocks[0].blockStart;
		saveEnd = ed->blocks[0].blockEnd;
	} else
	{
		if (fromTop)
		{
			saveStart = ed->blocks[0].blockStart;
			oldRow = ed->blocks[0].blockEnd.row;
			oldLine = ed->blocks[0].blockEnd.line;
			if (ed->wordBlock)
			{
				saveEnd.line = saveStart.line;
				saveEnd.row = EditTools_ToWordEnd(ed, saveEnd.line, saveStart.row);
			} else if (ed->lineBlock)
			{
				if (saveStart.line < ed->totalLineNr - 1)
				{
					saveEnd.line = saveStart.line + 1;
					saveEnd.row = EditTools_ToLineStart(ed, saveEnd.line);
				} else
				{
					saveEnd.line = saveStart.line;
					saveEnd.row = EditTools_ToLineEnd(ed, saveEnd.line);
				}
			}
		} else
		{
			saveEnd = ed->blocks[0].blockEnd;
			oldRow = ed->blocks[0].blockStart.row;
			oldLine = ed->blocks[0].blockStart.line;
			if (ed->wordBlock)
			{
				saveStart.line = saveEnd.line;
				saveStart.row = EditTools_ToWordStart(ed, saveStart.line, saveEnd.row);
				if (saveStart.row == saveEnd.row && saveStart.row > 0)
				{
					saveStart.row = EditTools_ToWordStart(ed, saveStart.line, saveStart.row - 1);
				}
			} else if (ed->lineBlock)
			{
				if (saveEnd.line > 0)
				{
					saveStart.line = saveEnd.line - 1;
					saveStart.row = EditTools_ToLineStart(ed, saveStart.line);
				} else
				{
					saveStart.line = 0;
					saveStart.row = EditTools_ToLineStart(ed, saveStart.line);
				}
			}
		}
	}
	EditTools_getCharPos(ed, oldLine, oldRow, &oldPixRow, &dummy);

	graf_mkstate(&mx, &my, &b, &udummy);
	do
	{
		if (my < work.g_y)
		{
			/* Oberhalb vom Work-Bereich, scrollen */
			/* Und jetzt scrollen, eigentlich sollte der Rest funktionieren */
			if (my + 2 * ed->charHeight < work.g_y)
				lines = 5;
			else if (my + ed->charHeight < work.g_y)
				lines = 3;
			else
				lines = 1;
			EditGlobals_HideMouse();
			WdwManager_ScrollUp(ed->wdw, lines);
			EditGlobals_ShowMouse(FALSE);
		} else if (my > work.g_h)
		{
			/* Unterhalb von Work-Bereich, scrollen */
			if (my - 2 * ed->charHeight > work.g_h)
				lines = 5;
			else if (my - ed->charHeight > work.g_h)
				lines = 3;
			else
				lines = 1;
			EditGlobals_HideMouse();
			WdwManager_ScrollDown(ed->wdw, lines);
			EditGlobals_ShowMouse(FALSE);
		}
		if (mx <= work.g_x || mx <= 0)
		{
			/* Nach links scrollen */
			if (mx + 2 * ed->charWidth <= work.g_x)
				lines = 5;
			else if (mx + ed->charWidth <= work.g_x)
				lines = 3;
			else
				lines = 1;
			WdwManager_ScrollLeft(ed->wdw, lines);
		} else if (mx > work.g_w)
		{
			/* Nach rechts scrollen */
			if (mx - 2 * ed->charWidth > work.g_w)
				lines = 5;
			else if (mx - ed->charWidth > work.g_w)
				lines = 3;
			else
				lines = 1;
			WdwManager_ScrollRight(ed->wdw, lines);
		}							/* Bewegungsauswertung */
		/* So, jetzt koennte es funktionieren */
		EditTools_GetClickLineAndRow(ed, mx, my, &newLine, &newRow);
		/* Jetzt haben wir Zeile und Spalte. Jetzt muessen noch die Nebenbedingungen
		 * beachtet werden, d.h. Wort- bzw. Zeilenmarkierung
		 * und das mindestens ein Wort bzw. eine Zeile markiert bleibt!
		 */
		if (ed->wordBlock)
		{
			/* Wordposition berechnen */
			if (newLine > saveStart.line ||
				(newLine == saveStart.line && newRow > saveStart.row))
			{
				newRow = EditTools_ToWordEnd(ed, newLine, newRow);
			} else
			{
				newRow = EditTools_ToWordStart(ed, newLine, newRow);
			}
		} else if (ed->lineBlock)
		{
			/* Zeilenposition neu berechnen */
			if (newLine > saveStart.line)
			{
				++newLine;
			}
			if (newLine >= ed->totalLineNr)
			{
				newRow = EditTools_ToLineEnd(ed, ed->totalLineNr - 1);
				newLine = ed->totalLineNr - 1;
			} else
			{
				newRow = 0;
			}
		}
		newLine = LMin(ed->totalLineNr - 1, LMax(0, newLine));
		/* Jetzt feststellen, welche Marke neu gesetzt wird */
		isRight = newLine < saveStart.line || (newLine == saveStart.line && newRow < saveStart.row);
		isLeft = newLine > saveEnd.line || (newLine == saveEnd.line && newRow > saveEnd.row);
		oldIsRight = oldLine < saveStart.line || (oldLine == saveStart.line && oldRow < saveStart.row);
		oldIsLeft = oldLine > saveEnd.line || (oldLine == saveEnd.line && oldRow > saveEnd.row);
		/* Jetzt haben wir die neuen Positionen, jetzt daraus die neuen 
		 * Pixelpositionen berechnen und dann vergleichen 
		 */

		EditTools_getCharPos(ed, newLine, newRow, &newPixRow, &dummy);

		if (isRight)
		{
			if (oldIsRight)
			{
				/* Einfach neuen Bereich markieren */
				if (oldRow != newRow || oldLine != newLine)		/* && (oldLine >= 0) && (oldLine <= totalLineNr) */
				{
					/* Es hat sich was geaendert, also mal neu zeichnen */
					EditDraw_MarkArea(ed, oldLine, oldRow, newLine, newRow, TRUE);
				}
			} else if (oldIsLeft)
			{
				/* Jetzt wird's spassiger, aber es hat sich garantiert was geaendert! 
				 */
				EditDraw_MarkArea(ed, oldLine, oldRow, saveEnd.line, saveEnd.row, TRUE);
				EditDraw_MarkArea(ed, newLine, newRow, saveStart.line, saveStart.row, TRUE);
				newEnd.line = saveEnd.line;
				newEnd.row = saveEnd.row;
			} else
			{
				/* Markieren von Blockanfang bis zu neuem Anfang */
				EditDraw_MarkArea(ed, newLine, newRow, saveStart.line, saveStart.row, TRUE);
			}
			newStart.row = newRow;
			newStart.line = newLine;
		} else if (isLeft)
		{
			if (oldIsLeft)
			{
				/* Einfach neuen Bereich markieren */
				if (oldRow != newRow || oldLine != newLine)		/* && (oldLine >= 0) && (oldLine <= totalLineNr) */
				{
					/* Es hat sich was geaendert, also mal neu zeichnen */
					EditDraw_MarkArea(ed, oldLine, oldRow, newLine, newRow, TRUE);
				}
			} else if (oldIsRight)
			{
				/* Jetzt wird's spassiger, aber es hat sich garantiert was geaendert! 
				 */
				EditDraw_MarkArea(ed, oldLine, oldRow, saveStart.line, saveStart.row, TRUE);
				EditDraw_MarkArea(ed, newLine, newRow, saveEnd.line, saveEnd.row, TRUE);
				newStart.line = saveStart.line;
				newStart.row = saveStart.row;
			} else
			{
				/* Markieren von Blockende bis zu neuer Position */
				EditDraw_MarkArea(ed, newLine, newRow, saveEnd.line, saveEnd.row, TRUE);
			}
			newEnd.row = newRow;
			newEnd.line = newLine;
		} else
		{
			/* in saveArea! */
			if (oldIsRight)
			{
				/* Bis zum Rand der Savearea markieren */
				EditDraw_MarkArea(ed, oldLine, oldRow, saveStart.line, saveStart.row, TRUE);
				newStart.line = saveStart.line;
				newStart.row = saveStart.row;
			} else if (oldIsLeft)
			{
				EditDraw_MarkArea(ed, oldLine, oldRow, saveEnd.line, saveEnd.row, TRUE);
				newEnd.line = saveEnd.line;
				newEnd.row = saveEnd.row;
			} else
			{
				/* old ist ebenfalls in SaveArea, do nothing! */
			}
		}
		/* jetzt alte Werte sichern */
		oldLine = newLine;
		oldRow = newRow;
		oldPixRow = newPixRow;
		ed->blocks[0].blockStart = newStart;
		ed->blocks[0].blockEnd = newEnd;
		graf_mkstate(&mx, &my, &b, &udummy);
	} while (b & MOB_LEFT);
	EditGlobals_RestoreMouse();
	Wind_Update(END_MCTRL);			/* Mauskontrolle wieder freigeben       */
	EditGlobals_ClipWork(ed);		/* Und clipping wieder auf Workbereich  */
	ed->block = TRUE;
}

/*** ---------------------------------------------------------------------- ***/


GLOBAL _VOID EditTools_ToBlockmark(EDITOR *ed, _BOOL ende)
{
	if (!ed->block)
		return;
	EditGlobals_HideMouse();
	EditGlobals_HideCursor(ed);
	if (ende)
	{
		EditTools_SetCurrLine(ed, ed->blocks[0].blockEnd.line);
		ed->currRow = ed->blocks[0].blockEnd.row;
	} else
	{
		EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
		ed->currRow = ed->blocks[0].blockStart.row;
	}
	ed->prefRow = ed->currRow;
	EditDraw_CenterCurrline(ed);
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Suchen und Ersetzen      */

/* Basisfunktionen fuer Search and Replace */

/* Compiliert das Suchmuster und setzt ein paar Variablen fuer
 * Suchen && Ersetzen mit Wildcards, falls ersetzt werden soll
 * Gibt zurueck, ob das Muster compiliert werden konnte.
 */
GLOBAL _BOOL EditTools_InitSearch(EDITOR *ed, searchDir direction, CONST _UBYTE *search, CONST _UBYTE *repl, _BOOL doRepl)
{
	_BOOL gross;
	_BOOL up;
	EDCOLUMN M;
	EDCOLUMN i;
	_WORD oneC;
	_WORD allC;
	_WORD wildC;

	UNUSED(repl);
	
	if (searchInited)
		EditTools_EndSearch();

	/* Jetzt erstmal alle Wildcardzeichen am Anfang und am Ende loeschen,
	 * die sind ueberfluessig
	 */

	while (search[0] == wildOneChar || search[0] == wildAllChar)
	{
		++search;
	}
	M = (EDCOLUMN)strLen(search);	/* Anzahl der Zeichen im Suchstring */
	while (M > 1 && (search[M - 1] == wildAllChar || search[M - 1] == wildOneChar))
	{
		--M;
	}

	if (M == 0)
	{
		return FALSE;
	}
	up = direction == sdir_backward;
	/* Soll gross == klein sein? Wenn nicht gesetzt, dann ja! */
	gross = (ed->findMode & GrossKlein) != 0;

	/* Suchmuster compilieren */
	pPatt = Find2_Compile(search, M, wildOneChar, 0x01, wildAllChar, LF, gross, up);

	if (doRepl)
	{
		/* Nach Wildcards im Suchstring suchen und Positionen merken */
		oneC = 0;
		allC = 0;
		wildC = 1;
		for (i = 0; i < M; i++)
		{
			if (search[i] == wildOneChar)
			{
				if (oneC < MAX_WILD)
				{
					oneWildNumbers[oneC] = wildC;
					++oneC;
				}
				++wildC;
			} else if (search[i] == wildAllChar)
			{
				if (allC < MAX_WILD)
				{
					allWildNumbers[allC] = wildC;
					++allC;
				}
				++wildC;
			}
		}
	}
	searchInited = TRUE;
	return pPatt != NULL;
}

/*** ---------------------------------------------------------------------- ***/

/* findx ist die Startposition des Suchstrings, findx + wLen - 1 die Endp. */
/* L ist die Laenge der untersuchten Zeile */
LOCAL _BOOL WortTest(EDITOR *ed, CONST _UBYTE *s, EDCOLUMN cRow, size_t wLen)
{
	_BOOL links, rechts;

	if (!(ed->findMode & NurWort))
	{
		return TRUE;
	} else
	{
		links = cRow == 0 || setin(TrennSet, s[cRow]) ||
			setin(TrennSet, s[cRow - 1]);
		if (!links)
		{
			return FALSE;
		} else
		{
			rechts = (cRow + wLen > ed->editLen) ||
				setin(TrennSet, s[cRow + wLen - 1]) ||
				setin(TrennSet, s[cRow + wLen]);
			return rechts;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Neue Suchroutine, benutzt jetzt Find-Modul von Andreas Alich. Reine Stringsuche
 * also ausgelagert.
 */

GLOBAL _VOID EditTools_SearchWord(EDITOR *ed, CONST _UBYTE *w, searchDir direction,
			  _BOOL *gefunden, _UBYTE *foundWord, size_t wsize, _BOOL again)
{
	EDBUFSIZE startx;					/* Position, ab der in der
										   Zeile gesucht werden soll */
	EDBUFSIZE findx;					/* gefundene Position */
	_BOOL up;
	EDBUFSIZE L;						/* Laenge der untersuchten Zeile */
	_BOOL found;						/* Suchstring gefunden? */
	EDLINE cLine;						/* temporaere Variable fuer gefundene Zeilennummer */
	EDCOLUMN cRow;						/* temporaere Variable fuer gefundene Spaltennummer */
	EDLINE saveLine;
	EDCOLUMN saveRow;
	size_t len;
	_UBYTE *patPos;
	_WORD patLen;
	_WORD patIdx;
	
	UNUSED(w);
	/* SucheWort */
	MouseOff();
	EditTools_ClearBlock(ed);
	MouseOn();
	up = direction == sdir_backward;

	if (!searchInited)
		*gefunden = FALSE;
	
	/* Suchen in neuem Editor: 
	 * Suchen in jedem Buffer, ggf. Startposition und Buffer berechnen, 
	 * falls ab aktueller Zeile. 
	 * Wenn gefunden, dann nachpruefen, ob gueltig (in Block),
	 * dann aus Offset Zeile und Spalte berechnen.
	 */
	saveLine = ed->currLineNr;
	saveRow = ed->currRow;
	startx = EditBase_GetOffset(ed);
	if (again)
	{
		if (up)
		{
			--startx;
			if (startx < 0)
				startx = 0;
		} else
		{
			++startx;
		}
	}
	if (up)
		L = startx;
	else
		L = ed->currentBuff->usedMem;
	found = FALSE;
	for (;;)
	{
		if (Find2_Find(ed->currentBuff->buffer, L, pPatt, startx))
		{
			/* neu:  
			   Falls das Muster gefunden wurde, sind die Eintraege 'len' und 'pMatch'
			   in den tPat-Records gueltig. 'pMatch' zeigt auf den Anfang des zum
			   Teilmuster passenden Textteils. 'len' gibt die Laenge des Teilmusters an.
			 */
			patIdx = 0;
			Find2_GetMatch(pPatt, patIdx, &patPos, &patLen);
			findx = (EDBUFSIZE)(patPos - ed->currentBuff->buffer);

			EditBase_GetPos(ed, findx, &cLine, &cRow);
			EditTools_SetCurrLine(ed, cLine);
			ed->currRow = cRow;
			/* So, jetzt die Laenge des gefundenen Testes rausfinden und dann den Worttest machen! */
			patIdx = 0;
			while (Find2_GetMatch(pPatt, patIdx, &patPos, &patLen))
			{
				findx = (EDBUFSIZE)(patPos - ed->currentBuff->buffer);
				EditBase_GetPos(ed, findx, &cLine, &cRow);
				cRow += patLen;
				++patIdx;
			}
			/*
			 * !!! Vorsicht! Das Ende des gefundenen Textes kann auch in einer anderen Zeile liegen
			 */
			if (cLine == ed->currLineNr)
				len = cRow - ed->currRow + 1;
			else
				len = ed->editLen - ed->currRow + 1;
			if (len > wsize)
				len = wsize;
			strMcpy(foundWord, len, ed->editLine.text + ed->currRow);
			--len;
			if (WortTest(ed, ed->editLine.text, ed->currRow, len))
			{
				found = TRUE;
				break;
			}
			patIdx = 0;
			Find2_GetMatch(pPatt, patIdx, &patPos, &patLen);
			findx = (EDBUFSIZE)(patPos - ed->currentBuff->buffer);
			if (up)
			{
				startx = findx - 1;
				if (startx < 0)
					startx = 0;
			} else
			{
				startx = findx + 1;
			}
		} else
		{
			/* Nicht in dem Buffer gefunden */
			if (up)
			{
				if (ed->currentBuff->prev != NULL)
				{
					ed->currentBuff = ed->currentBuff->prev;
					startx = ed->currentBuff->usedMem;
					L = ed->currentBuff->usedMem;
				} else
				{
					break;
				}
			} else
			{
				if (ed->currentBuff->next != NULL)
				{
					ed->currentBuff = ed->currentBuff->next;
					startx = 0;
					L = ed->currentBuff->usedMem;
				} else
				{
					break;
				}
			}
		}
	}

	if (found)
	{
		if (!(ed->findMode & silent))
		{
			MouseOff();
			/* Nicht globales Ersetzen, also Block markieren! */
			EditTools_SetBlockstart(ed);
			cRow = ed->currRow;
			patIdx = 0;
			while (Find2_GetMatch(pPatt, patIdx, &patPos, &patLen))
			{
				findx = (EDBUFSIZE)(patPos - ed->currentBuff->buffer);
				EditBase_GetPos(ed, findx, &cLine, &ed->currRow);
				ed->currRow += patLen;
				patIdx++;
			}
			saveLine = ed->currLineNr;
			EditTools_SetCurrLine(ed, cLine);
			EditTools_SetBlockend(ed);
			EditTools_SetCurrLine(ed, saveLine);
			ed->currRow = cRow;
			ed->block = TRUE;
			EditDraw_CenterCurrline(ed);
			MouseOn();
		}
	} else
	{
		EditTools_SetCurrLine(ed, saveLine);
		ed->currRow = saveRow;
	}
	ed->prefRow = ed->currRow;
	*gefunden = found;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditTools_ReplaceWord(EDITOR *ed, CONST _UBYTE *s, CONST _UBYTE *t)
{
	aLineDesc Zeile;
	EDCOLUMN sl, tl, zl, el;
	EDCOLUMN tmpRow;
	EDCOLUMN i;
	_UBYTE replW[1024];					/* Laenger kann unser Ersatzwort halt nicht werden */
	_UBYTE numC;
	EDCOLUMN wIdx;
	EDCOLUMN pIdx;
	EDCOLUMN idx;
	EDBUFSIZE findx;
	EDLINE cLine;
	EDCOLUMN cRow;
	EDCOLUMN clRow;
	_UBYTE *patPos;
	_WORD patLen;
	_BOOL patFound;
	
	/* Erstmal Wildcards in t durch richtige Zeichen ersetzen */
	tl = (EDCOLUMN)strLen(t);
	tmpRow = 0;
	i = 0;
	while (i < tl && tmpRow < sizeof(replW) - 1)
	{
		if (t[i] == wildOneChar || t[i] == wildAllChar)
		{
			/* Ein-Zeichen-Wildcard gefunden */
			numC = t[i + 1];
			if (numC >= '0' && numC <= '9')
			{
				wIdx = numC - '0';
				/* Jetzt den Index in Patternliste rausfinden */
				if (t[i] == wildOneChar)
					idx = oneWildNumbers[wIdx];
				else
					idx = allWildNumbers[wIdx];
				++i;					/* Damit das Indexzeichen auch uebersprungen wird */
				/* Jetzt zum Ende des vorhergehenden Patterns gehen und dann 
				 * alle Zeichen bis zum Anfang des naechsten Patterns in das Replaceword
				 * setzen 
				 */

				pIdx = 0;
				patFound = Find2_GetMatch(pPatt, pIdx, &patPos, &patLen);
				while (patFound && pIdx < idx)
				{
					findx = (EDBUFSIZE)(patPos - ed->currentBuff->buffer);
					EditBase_GetPos(ed, findx, &cLine, &cRow);
					cRow += patLen;
					++pIdx;
					patFound = Find2_GetMatch(pPatt, pIdx, &patPos, &patLen);
				}
				/* Jetzt haben wir die Position im Text, jetzt muessen wir nur noch herausfinden,
				 * wieviele Zeichen wir kopieren koennen. Da es alles in einer Zeile ist,
				 * kann ich zum rauskopieren auf editLine.text zugreifen!
				 */
				if (patFound)
				{
					/* Endposition steht praktisch schon in ptrPatt drin, muss nur ausgelesen
					 * werden
					 */
					findx = (EDBUFSIZE)(patPos - ed->currentBuff->buffer);
					EditBase_GetPos(ed, findx, &cLine, &clRow);
					while (cRow < clRow)
					{
						replW[tmpRow] = ed->editLine.text[cRow];
						++cRow;
						++tmpRow;
					}
				}
			} else
			{
				replW[tmpRow] = t[i];
				++tmpRow;
			}
		} else
		{
			replW[tmpRow] = t[i];
			++tmpRow;
		}
		++i;
	}
	replW[tmpRow] = '\0';

	/* ersetzt s durch replW in Zeile */
	sl = (EDCOLUMN)strLen(s);
	tl = (EDCOLUMN)strLen(replW);
	tmpRow = ed->currRow;
	zl = ed->editLen;
	el = zl + tl - sl;
	if (el >= ed->editLine.length - 1)
	{
		el = ((el + 16) / 16) * 16;
		if (el < 16)
		{
			el = 16;
		}
		Zeile.text = MALLOC(el, "Edit_ReplaceWord");
		if (Zeile.text == NULL)
		{
			return;
		}
		Zeile.length = el;
		strCpy(Zeile.text, ed->editLine.text);
		strCpy(Zeile.text + tmpRow, Zeile.text + tmpRow + sl);
		Insert(replW, Zeile.text, el, tmpRow);
	} else
	{
		strCpy(ed->editLine.text, ed->editLine.text);
		strCpy(ed->editLine.text + tmpRow, ed->editLine.text + tmpRow + sl);
		Insert(replW, ed->editLine.text, ed->editLine.length, tmpRow);
		Zeile.text = NULL;
	}
	ed->editChanged = TRUE;
	EditTools_ShowFileChanged(ed);
	if (Zeile.text != NULL)
	{
		SFREE(ed->editLine.text);
		ed->editLine.text = Zeile.text;
		ed->editLine.length = Zeile.length;
	}
	ed->editLen = (EDCOLUMN)strLen(ed->editLine.text);
	EditBase_PutCurrLine(ed);
	/* currRow hinter Wort setzen */
	ed->currRow += tl;
	ed->prefRow = ed->currRow;
	if (!(ed->findMode & silent))
	{
		EditGlobals_HideMouse();
		EditDraw_CenterCurrline(ed);
		EditTools_ClearBlock(ed);
		EditDraw_drawLine(ed, (_WORD)(ed->currLineNr - ed->StartLine));
		EditGlobals_ShowMouse(FALSE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt das compilierte Suchmuster wieder frei
 */
GLOBAL _VOID EditTools_EndSearch(_VOID)
{
	_WORD i;

	if (searchInited)
	{
		Find2_Dispose(&pPatt);
		searchInited = FALSE;
		for (i = 0; i < MAX_WILD; i++)
		{
			oneWildNumbers[i] = 0;
			allWildNumbers[i] = 0;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Prueft die Konsistenz von Wildcards und ihren Gegenstuecken im Such-
 * und Ersetzstring
 */
GLOBAL _BOOL EditTools_CheckWildAssign(CONST _UBYTE *search, CONST _UBYTE *repl)
{
	_UBYTE one, all;
	_UBYTE numC;
	EDCOLUMN i;
	_UWORD oneC;
	_UWORD allC;
	EDCOLUMN M;

	one = wildOneChar;
	all = wildAllChar;

	/* Jetzt erstmal alle Wildcardzeichen am Anfang und am Ende loeschen,
	 * die sind ueberfluessig
	 */

	while (search[0] == one || search[0] == all)
		search++;
	M = (EDCOLUMN)strLen(search);	/* Anzahl der Zeichen im Suchstring */
	while (M > 1 && (search[M - 1] == all || search[M - 1] == one))
	{
		--M;
	}

	oneC = 0;
	allC = 0;
	for (i = 0; i < M; i++)
	{
		if (search[i] == one)
			++oneC;
		else if (search[i] == all)
			++allC;
	}
	/* Jetzt haben wir die Anzahl der Wildcards im Suchstring */
	if (oneC > MAX_WILD || allC > MAX_WILD)
	{
		return FALSE;
	}
	/* Jetzt den Ersetzstring pruefen */
	for (i = 0; repl[i] != '\0'; i++)
	{
		/* Naechstes Zeichen testen, ob es eine Nummer ist */
		numC = repl[i + 1];
		if (numC >= '0' && numC <= '9')
		{
			if (repl[i] == one)
			{
				if (numC - '0' >= oneC)
					return FALSE;
			} else if (repl[i] == all)
			{
				if (numC - '0' >= allC)
					return FALSE;
			}
		}
	}
	return TRUE;
}
