/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


/*
 * 23.1.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.20a'
 *	Bei WrapLine kamen bei Zeilen mit sehr vielen EURO-Zeichen negative Werte vor.
 *
 * 12.3.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.22c'
 *	Beim Versuch ein File zu berschreiben wurde das falsche Alert gezeigt
 */
#include <editutil.h>
#include <editglob.h>
#include <edittool.h>
#include <editdraw.h>
#include <editbase.h>
#include <editfunc.h>
#include <editor.h>
#include <catgloba.h>
#include <wdwmanag.h>
#include <confvars.h>
#include <varn_edi.h>
#include <quicksor.h>
#include <protokol.h>

#include <mtappl.h>
#include <mtalerts.h>
#include <mtutils.h>
#include <mtdir.h>
#include <rectfunc.h>

#include <portvdi.h>
#include <tos.h>
#include <magicstr.h>
#include <keyboard.h>
#include <filename.h>

#include <routinec.h>
#include <routines.h>
#include <dir_io.h>
#include <mtrsc.h>
#include <catlib.h>



#define TAB 0x09
#define CR  0x0d
#define LF  0x0a

LOCAL _BOOL fastWrap;


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Sichert die Position des Editors in den
 * Configvariablen
 */
GLOBAL _VOID EditUtil_EditSaveWdwPos(HWND wdw)
{
	_UBYTE varName[256];
	GRECT full;
	EDITOR *ed;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		make_varname(varName, sizeof(varName), cEditWindowPos, ed->number);
		WdwManager_GetWdwSize(ed->wdw, &full);	/* Groesse des Fensters abfragen */
		ConfVars_SetConfigRect(varName, &full);
		make_varname(varName, sizeof(varName), cEditFont, ed->number);
		ConfVars_SetConfigInt(varName, ed->font);
		make_varname(varName, sizeof(varName), cEditFontSize, ed->number);
		ConfVars_SetConfigInt(varName, ed->fontSize);
		make_varname(varName, sizeof(varName), cEditTabs, ed->number);
		ConfVars_SetConfigBool(varName, ed->realTabs);
		make_varname(varName, sizeof(varName), cEditTabSize, ed->number);
		ConfVars_SetConfigInt(varName, ed->TabSize);;
		make_varname(varName, sizeof(varName), cEditWrap, ed->number);
		ConfVars_SetConfigBool(varName, ed->umbruch);
		make_varname(varName, sizeof(varName), cEditMargin, ed->number);
		ConfVars_SetConfigInt(varName, ed->rightMargin);
		make_varname(varName, sizeof(varName), cEditInfo, ed->number);
		ConfVars_SetConfigBool(varName, ed->showInfo);
		make_varname(varName, sizeof(varName), cEditLine, ed->number);
		ConfVars_SetConfigLongInt(varName, ed->currLineNr);
		make_varname(varName, sizeof(varName), cEditRow, ed->number);
		ConfVars_SetConfigInt(varName, ed->currRow);
		make_varname(varName, sizeof(varName), cEditShowLineEnds, ed->number);
		ConfVars_SetConfigBool(varName, ed->EditShowLineEnds);
		make_varname(varName, sizeof(varName), cEditReadOnly, ed->number);
		ConfVars_SetConfigInt(varName, ed->readOnly);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Sichert die Position und den Inhalt aller offenen Editorfenster
 * (nicht ReadOnly-Fenster)
 */
GLOBAL _VOID EditUtil_EditSavePos(_VOID)
{
	EDITOR *ed;
	_UBYTE varName[256];

	ed = editList;
	while (ed != NULL)
	{
		if (ed->fileName.length > 0)
		{
			make_varname(varName, sizeof(varName), cEditFile, ed->number);
			ConfVars_SetConfigString(varName, ed->fileName.text);
			EditUtil_EditSaveWdwPos(ed->wdw);
		}
		ed = ed->nextEdit;
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Einfuege- und Loeschoperationen */

/* Maximale Zeilenlaenge fuer Einruecken */
#define MAX_INDENT 512


LOCAL _VOID Ruecke(EDITOR *ed, _BOOL links, EDCOLUMN index, CONST _UBYTE *blank)
{
	EDCOLUMN l;

	EditTools_ShowFileChanged(ed);			/* Anzeigen, dass Text geaendert */
	if (index >= MAX_INDENT)
	{
		return;
	}
	EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
	do
	{
		/* YYYY Hier noch Anpassung auf CR/LF am Zeilenende einbauen! */
		l = ed->editLen;
		while (l > 0 && (ed->editLine.text[l - 1] == CR || ed->editLine.text[l - 1] == LF))
		{
			--l;
		}
		ed->editChanged = TRUE;
		if (index > l && !links)
		{
			EditGlobals_AllocNew(&ed->editLine, index + 4);
			MemMove(ed->editLine.text + index, ed->editLine.text + l, ed->editLen - l + 1);
			MemSet(ed->editLine.text + l, ' ', index - l);
			l = index;
		}
		if (l >= index)
		{
			if (links)
			{
				Delete(ed->editLine.text, index, 1);
			} else
			{
				EditGlobals_AllocNew(&ed->editLine, index + (EDCOLUMN)strLen(blank) + 2);
				Insert(blank, ed->editLine.text, ed->editLine.length, index);
			}
			l = (EDCOLUMN)strLen(ed->editLine.text);
			ed->editLen = l;
			EditBase_PutLine(ed, ed->currLineNr, ed->editLine.text, l);
		}
		if (ed->currLineNr != ed->blocks[0].blockEnd.line)
		{
			EditTools_SetCurrLine(ed, ed->currLineNr + 1);
		}
	} while (ed->currLineNr != ed->blocks[0].blockEnd.line);
	EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
	ed->prefRow =ed->currRow = index;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
GLOBAL _VOID EditUtil_IndentBlock(HWND wdw)
{
	EDCOLUMN index, i, t;
	_BOOL Doit, Schirm;
	_UBYTE blank[81];
	EDITOR *ed;
	_WORD taste;
	Key key;

	ed = EditFuncs_FindEditor(wdw);
	if (ed == NULL)
	{
		return;
	}
	if (!ed->block)
	{
		return;
	}
	if (!ed->monoSpaced)
	{
		mtAlerts_Alert(1, Rsc_WrkString(AL_ONLY_MONO_FONT));
		return;
	}
	/* Block sichern fuer Undo */
	EditTools_SaveOp(ed, blockPaste, FALSE);
	if (ed->showInfo)
	{
		/* Infozeile setzen */
		strBcpy(ed->WindowInfo, Rsc_WrkString(FS_INDENT));
		WdwManager_SetWdwInfoline(wdw, ed->WindowInfo);
	}
	/* Currline auf Blockanfang setzen */
	EditTools_SetCurrLine(ed, ed->blocks[0].blockStart.line);
	ed->prefRow =ed->currRow = ed->blocks[0].blockStart.row;
	EditDraw_CenterCurrline(ed);
	/* Variablen initialisieren */
	blank[0] = ' ';
	blank[1] = '\0';
	index = ed->currRow;
	/* Menuzeile sperren und Window-Events verhindern */
	Wind_Update(BEG_UPDATE);
	Wind_Update(BEG_MCTRL);
	EditGlobals_HideMouse();
	/* Cursor zeigen */
	EditGlobals_HideCursor(ed);
	ed->blockIndent = TRUE;
	EditGlobals_ShowCursor(ed);
	do
	{
		Doit = FALSE;
		Schirm = FALSE;
		taste = evnt_keybd();
		GemCharToKey(taste, 0, &key);
		if (IsSpecial(&key))
		{
			switch (SpecialKey(&key))
			{
			case tab:
				t = ed->TabSize * ((index / ed->TabSize) + 1);
				for (i = index; i < t; i++)
				{
					blank[i - index] = ' ';
				}
				blank[t - index] = '\0';
				if (ed->realTabs)
				{
					blank[0] = TAB;
					blank[1] = '\0';
				}
				Ruecke(ed, FALSE, index, blank);
				EditGlobals_HideCursor(ed);
				if (ed->realTabs)
				{
					++ed->currRow;
				} else
				{
					ed->currRow += t - index;
				}
				ed->prefRow = ed->currRow;
				EditGlobals_ShowCursor(ed);
				Doit = TRUE;
				Schirm = TRUE;
				index = ed->currRow;
				break;
			case left:
			case wdLeft:
			case soln:
				if (ed->currRow > 0)
				{
					EditGlobals_HideCursor(ed);
					--ed->currRow;
					ed->prefRow = ed->currRow;
					EditGlobals_ShowCursor(ed);
					Doit = TRUE;
					index = ed->currRow;
				}
				break;
			case backspace:
				if (ed->currRow > 0)
				{
					EditGlobals_HideCursor(ed);
					--ed->currRow;
					ed->prefRow = ed->currRow;
					EditGlobals_ShowCursor(ed);
					Doit = TRUE;
					Schirm = TRUE;
					index = ed->currRow;
					Ruecke(ed, TRUE, index, blank);
				}
				break;
			case right:
			case wdRight:
			case eoln:
				if (ed->currRow < MAX_INDENT)
				{
					EditGlobals_HideCursor(ed);
					++ed->currRow;
					ed->prefRow = ed->currRow;
					EditGlobals_ShowCursor(ed);
					Doit = TRUE;
					index = ed->currRow;
				}
				break;
			case delete:
				Ruecke(ed, TRUE, index, blank);
				Doit = TRUE;
				Schirm = TRUE;
				break;
			default:
				break;
			}
		} else if (IsCharacter(&key))
		{
			blank[0] = taste & 0xff;
			blank[1] = '\0';
			Ruecke(ed, FALSE, index, blank);
			Doit = TRUE;
			Schirm = TRUE;
			if (ed->currRow < MAX_INDENT)
			{
				EditGlobals_HideCursor(ed);
				++ed->currRow;
				ed->prefRow = ed->currRow;
				EditGlobals_ShowCursor(ed);
				index = ed->currRow;
			}
		}
		if (Doit)
		{
			EditDraw_CenterCurrline(ed);
			EditTools_ShowFileChanged(ed);	/* Anzeigen, dass Text geaendert */
			if (Schirm)
			{
				/* Hier noch richtigen Redrawaufruf hinsetzen */
				EditDraw_redrawLineArea(ed, ed->blocks[0].blockStart.line, ed->blocks[0].blockEnd.line);
			}
		}
	} while (!IsSpecial(&key) || SpecialKey(&key) != return_);
	/* Noch Blockstart fuer Undobuffer merken */
	EditTools_NewBlockToUndoBuffer(ed);
	/* Cursor zeigen */
	EditGlobals_HideCursor(ed);
	ed->blockIndent = FALSE;
	/* Blockmarken noch loeschen */
	EditTools_ClearBlock(ed);
	Wind_Update(END_MCTRL);
	Wind_Update(END_UPDATE);
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
	if (ed->showInfo)
	{
		EditTools_SetInfoLine(ed);
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Hilfsfunktionen fuer OpenEditor und EditMerge */

#define FSelSymbol '?'
#define HomeSymbol '*'

LOCAL _VOID ReplaceHome(_UBYTE *name)
{
	size_t idx;
	
	idx = name[0] == FSelSymbol ? 1 : 0;
	if (name[idx] == '*')
	{
		strCpy(name + idx, name + idx + 1);
		if (name[idx] == '\\' && HomePath[0] != '\0' && HomePath[strLen(HomePath)-1] == '\\')
		{
			/* um aus "*\DAT" nicht "D:\\DAT" zu machen */
			strCpy(name + idx, name + idx + 1);
		}
		Insert(HomePath, name, PATH_MAX, idx);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL SearchPrepare(_UBYTE *name)
{
	_BOOL fsel;
	
	ReplaceHome(name);
	fsel = name[0] == FSelSymbol;
	if (fsel)
		strCpy(name, name + 1);
	return fsel;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL SearchFind(CONST _UBYTE *path, CONST _UBYTE *name, _UBYTE *resultName)
{
	F_STAT st;
	
	strMcpy(resultName, PATH_MAX, path);
	F_Path_Append(resultName, name);
	if (F_Getinfo(resultName, &st, TRUE) == GERR_OK)
	{
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL SearchFile(CONST _UBYTE *iname, List *path, _UBYTE *resultName)
{
	_UBYTE pathName[PATH_MAX];
	_UBYTE firstPath[PATH_MAX];
	_UBYTE firstname[PATH_MAX];
	_UBYTE name[PATH_MAX];
	_UBYTE *p;
	_BOOL fsel;
	
	strBcpy(firstname, iname);
	fsel = SearchPrepare(firstname);
	SplitPath(firstname, firstPath, name);
	if (name[0] == '\0')
	{
		*resultName = '\0';
		return FALSE;
	}
	
	if (fsel)
	{
		strBcpy(pathName, firstPath);
		if (!mtDir_FselGet(pathName, name, ALL_FILE_MASK, NULL, FALSE))
			return FALSE;
		return SearchFind(pathName, name, resultName);
	}

	pathName[0] = '\0';
	Lists_ResetList(path);
	if (Lists_ListEmpty(path) || firstPath[0] != '\0')
	{
		/* Zuerst angegebenen Pfad pruefen */
		if (SearchFind(firstPath, name, resultName))
			return TRUE;
	}
	/* Jetzt die PathList durchsuchen */
	while ((p = Lists_NextEntry(path)) != NULL)
	{
		strBcpy(pathName, p);
		fsel = SearchPrepare(pathName);
		if (fsel)
		{
			if (!mtDir_FselGet(pathName, name, ALL_FILE_MASK, NULL, FALSE))
				return FALSE;
			return SearchFind(pathName, name, resultName);
		}
		if (!F_Path_Equal(firstPath, pathName) &&
			SearchFind(pathName, name, resultName))
			return TRUE;
	}
	strMcpy(resultName, PATH_MAX, firstname);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
GLOBAL _BOOL EditUtil_getFileParms(CONST _UBYTE *path, CONST _UBYTE *FileName, _VOID /* List */ *SrcPaths, _BOOL new, _BOOL alerts, aLineDesc *fn, _ULONG *flen, FILENO *fHdl)
{
	_UBYTE newName[PATH_MAX];
	_UBYTE newPath[PATH_MAX];
	
	*flen = 0;
	*fHdl = -1;
	if (path[0] == '\0')				/* Pfad leer, aktuellen Pfad nehmen */
	{
		F_Path_Get(newPath);
	} else
	{
		strBcpy(newPath, path);
	}
	/* Dateinamen zusammenbauen */
	if (FileName[0] != '\0')
	{
		strBcpy(newName, newPath);
		F_Path_Append(newName, FileName);
		fn->length = (EDCOLUMN)strLen(newName) + 1;
		fn->text = MALLOC(fn->length, "getFileParms");
		if (fn->text == NULL)
		{
			/* not enough memory!! */
			return FALSE;
		}
		strCpy(fn->text, newName);
	} else
	{
		fn->length = 0;
	}
	/* File oeffnen */
	*fHdl = (FILENO)Fopen(fn->text, FO_READ);
	/* Suchen im Pfad */
	if (*fHdl < 0 && SrcPaths != NULL)
	{
		/* file nicht geoeffnet */
		if (*fHdl == -33)/*!ERROR*/
		{
			if (fn->length != 0)
			{
				if (SearchFile(FileName, SrcPaths, newName))
				{
					_UBYTE *new;
					
					new = STRDUP(newName, "getFileParms: newName");
					if (new == NULL)
						return FALSE;
					SFREE(fn->text);
					fn->text = new;
					*fHdl = (FILENO)Fopen(fn->text, FO_READ);
				}
			}
		}
	}
	if (*fHdl < 0)
	{
		if (*fHdl == -33)/*!ERROR*/
		{
			if (alerts && !new)
			{
				if (mtAlerts_Alert(1, Rsc_WrkString(AL_NEW_FILE)) == 1)
					return TRUE;
				return FALSE;
			} else
			{
				return TRUE;
			}
		} else
		{
			fn->length = 0;
			SFREE(fn->text);
			fn->text = NULL;
			if (alerts)
			{
				F_ErrorAlert(*fHdl);
			}
			return FALSE;
		}
	}
	/* Filegroesse feststellen */
	*flen = Fseek(0, *fHdl, SEEK_END);
	Fseek(0, *fHdl, SEEK_SET);
	if (*flen == 0 && !new)
	{
		/* Lohnt sich nicht :-) */
		if (mtAlerts_Alert(1, Rsc_WrkString(AL_EMPTY_FILE)) != 1)
		{
			/* Abbruch gewaehlt */
			return FALSE;
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

#if OLD
GLOBAL _BOOL EditUtil_testMem(_ULONG flen, _ULONG *memSize, _BOOL *fastRead)
{
	/* freien Speicher feststellen, mindestens 30 kB frei! */
	*memSize = MemAvail();
	if (*memSize < (2 * flen + ((flen / 100) * 115) + 4096L))
	{
		if (*memSize < (flen / 100) * 115 + 4096L)
		{
			/* not enough memory, just don't try any more */
			EditFuncs_OutOfMem();
			return FALSE;
		} else
		{
			/* koennte klappen, versuchen wir mal die langsame Methode! */
			*fastRead = FALSE;
		}
	} else
	{
		*fastRead = TRUE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL EditUtil_getReadBuff(_ULONG flen, _UBYTE **readBuf)
{
	_VOID *voidBuf;
	_ULONG bufSize;
	_ULONG memsize;
	
	/* 2 * Max(TextSize, 10 kB) allozieren */
	memSize = MemAvail();
	if (memSize > 4 * flen)
	{
		/* wir haben VIEL Speicher */
		if (memSize > flen + 32000)
		{
			bufSize = LMax(memSize - flen - 32000, 10 * 1024);
		} else
		{
			bufSize = 10 * 1024;
		}
	} else
	{
		bufSize = LMax((flen / 100) * 115 + 8192L, 10 * 1024);
		if (memSize > flen + 32000)
		{
			bufSize = LMin(bufSize, memSize - flen - 32000);
		}
	}
	voidBuf = MALLOC(bufSize, "getReadBuff: voidbuf");
	if (voidBuf == NULL)
	{
		/* Not enough memory */
		return FALSE;
	}
	*readBuf = MALLOC(flen + 2, "getReadBuff");
	if (*readBuf == NULL)
	{
		/* Not enough memory */
		SFREE(voidBuf);
		return FALSE;
	}
	/* 1 Buffer wieder freigeben */
	SFREE(voidBuf);
	return TRUE;
}

#else

GLOBAL _BOOL EditUtil_testMem(_ULONG flen, _BOOL *fastRead, _UBYTE **readBuff)
{
	*readBuff = NULL;
	*fastRead = FALSE;
	if (flen != 0)
	{
		/*
		 * wird nur in Faellen benutzt, wo anschliessend eine
		 * alternative Routine mit kleineren Speicherbloecken
		 * arbeitet, also wollen wir erst mal keine Fehlermeldung
		 * falls dies hier nicht klappt
		 */
		*readBuff = SYSMALLOC(flen + 2, MEM_NOERR, "testMem");
		if (*readBuff == NULL)
			return FALSE;
		*fastRead = TRUE;
	}
	return TRUE;
}

#endif

/*** ---------------------------------------------------------------------- ***/

/* Fuegt einen Text mit Filename ein
 */
GLOBAL _VOID EditUtil_EditPasteFile(HWND wdw, _BOOL isFilter, CONST _UBYTE *path, CONST _UBYTE *name)
{
	_ULONG flen;
	EDITOR *ed;
	_UBYTE *readBuf;
	_BOOL fastRead;
	aLineDesc fn;
	FILENO fHdl;
	_BOOL wasBlock;
	_BOOL inLine;

	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		readBuf = NULL;
		if (ed->readOnly!=MODE_RW && !isFilter)
		{
			return;
		}
		fn.text = NULL;
		fn.length = 0;
		if (EditUtil_getFileParms(path, name, NULL, TRUE, FALSE, &fn, &flen, &fHdl) && flen > 0)
		{
			SFREE(fn.text);
			(void)Fclose(fHdl);

			if (flen > 0)
			{
				wasBlock = ed->block;
				if (ed->block)
				{
					EditTools_SaveOp(ed, blockPaste, FALSE);
					EditFuncs_cutBlock(ed, FALSE, FALSE);
					ed->blocks[0].blockStart.line = ed->currLineNr;
					ed->blocks[0].blockStart.row = ed->currRow;
					ed->block = TRUE;
				} else
				{
					EditTools_SetBlockstart(ed);
					EditTools_SaveOp(ed, normalPaste, FALSE);
					ed->block = TRUE;
				}
				inLine = ed->currRow != 0;
				if (!(
					/* Dateinamen basteln, Datei oeffnen und Laenge feststellen */
					EditUtil_getFileParms(path, name, NULL, FALSE, FALSE, &fn, &flen, &fHdl) &&
					/* Speicher feststellen und auf flen testen */
					EditUtil_testMem(flen, &fastRead, &readBuf) &&
					/* Text einlesen und einfuegen */
					EditFuncs_readText(ed, fastRead, flen, &fHdl, ed->rightMargin, &readBuf, !inLine, FALSE)
					))
				{
					if (readBuf != NULL)
					{
						SFREE(readBuf);
					}
					if (fHdl >= 0)
					{
						F_ErrorAlert(Fclose(fHdl));
					}
					if (fn.text != NULL)
					{
						SFREE(fn.text);
					}
					return;
				}
				EditGlobals_HideMouse();
				EditGlobals_HideCursor(ed);
				EditTools_UpdateCurrLine(ed, ed->currLineNr);
				EditTools_ShowFileChanged(ed);
				if (fn.text != NULL)
				{
					SFREE(fn.text);
				}
				EditTools_NewBlockToUndoBuffer(ed);
				/* Jetzt Cursorposition anpassen an Ende des gepasteten Stuecks
				 */
				EditTools_SetCurrLine(ed, ed->blocks[0].blockEnd.line);
				ed->prefRow =ed->currRow = ed->blocks[0].blockEnd.row;
				EditGlobals_SetDocument(ed);
				if (!wasBlock)
				{
					/* normal Paste */
					ed->block = FALSE;
					ed->blocks[0].blockStart.line = -1;
					ed->blocks[0].blockEnd.line = -1;
				}
				WdwManager_RedrawWdw(wdw, &ed->editWork);
				EditDraw_CenterCurrline(ed);
				if (ed->showInfo)
				{
					EditTools_SetInfoLine(ed);
				}
				EditGlobals_ShowCursor(ed);
				EditGlobals_ShowMouse(FALSE);
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */	/*DYJ 5.22c */
GLOBAL _BOOL EditUtil_SaveText(EDITOR *ed, _BOOL saveas, _BOOL takeName, CONST _UBYTE *fpath, CONST _UBYTE *fname,_BOOL saveBlock)
{
	_UBYTE path[PATH_MAX];
	_UBYTE name[PATH_MAX];
	_UBYTE fullName[PATH_MAX];
	FILENO fHdl;
	_BOOL appended;
	_WORD i;
	_WORD select;
	
	fHdl = -1;
	appended = FALSE;
	if (ed->editChanged)
	{
		EditBase_PutCurrLine(ed);
	}
	if (ed->new && !saveas)
	{
		if (ed->fileName.length == 0)
		{
			F_Path_Get(path);
			strBcpy(name, cNoname);
			if (mtDir_FselGet(path, name, ALL_FILE_MASK, Rsc_WrkString(FS_SAVE_AS), FALSE))
			{
				strBcpy(fullName, path);
				F_Path_Append(fullName, name);
				fHdl = (FILENO)Fopen(fullName, FO_WRITE);
				if (fHdl != -33)/*!ERROR*/
				{
					if (fHdl >= 0)
					{
						select = mtAlerts_Alert(2, Rsc_WrkString(AL_OVERWRITE3));
						if (select == 2)
						{
							Fseek(0L, fHdl, SEEK_END);
							appended = TRUE;
						} else if (select != 1)
						{
							F_ErrorAlert(Fclose(fHdl));
							return FALSE;
						} else
						{
							/* Schliessen und Fcreate machen */
							(void)Fclose(fHdl);
							fHdl = (FILENO)Fcreate(fullName, 0);
							if (fHdl < 0)
							{
								F_ErrorAlert(fHdl);
								return FALSE;
							}
						}
					} else
					{
						F_ErrorAlert(fHdl);
						return FALSE;
					}
				} else
				{
					/* File not found: Create */
					fHdl = (FILENO)Fcreate(fullName, 0);
					if (fHdl < 0)
					{
						F_ErrorAlert(fHdl);
						return FALSE;
					}
				}
			} else
			{
				return FALSE;
			}
		}
	}
	if (fHdl < 0)
	{
		if (fpath != NULL && fname != NULL && fname[0] != '\0')
		{
			strBcpy(fullName, fpath);
			F_Path_Append(fullName, fname);
		} else
		{
			strBcpy(fullName, ed->fileName.text);
		}
		fHdl = (FILENO)Fopen(fullName, FO_WRITE);
		if (fHdl < 0)
		{
			if (fHdl == -33)/*!ERROR*/
			{
				fHdl = (FILENO)Fcreate(fullName, 0);
				if (fHdl < 0)
				{
					F_ErrorAlert(fHdl);
					return FALSE;
				}
			} else
			{
				F_ErrorAlert(fHdl);
				return FALSE;
			}
		} else
		{
			if (saveas)
			{
				select = mtAlerts_Alert(2, Rsc_WrkString(AL_OVERWRITE3));

				if (select == 2)
				{
					/* Anhaengen */
					(void)Fseek(0L, fHdl, SEEK_END);
					appended = TRUE;
				} else if (select != 1)
				{
					/* Abbruch */
					F_ErrorAlert(Fclose(fHdl));
					return FALSE;
				} else
				{
					/* Ueberschreiben */
					/* Schliessen und Fcreate machen */
					(void)Fclose(fHdl);
					fHdl = (FILENO)Fcreate(fullName, 0);
					if (fHdl < 0)
					{
						F_ErrorAlert(fHdl);
						return FALSE;
					}
				}
			} else
			{
				/* Direkt ueberschreiben */
				/* Schliessen und Fcreate machen */
				(void)Fclose(fHdl);
				if (ed->backup)
				{
					if (!CatGlobal_CreateBackup(fullName))
					{
						mtAlerts_Alert(1, Rsc_WrkString(AL_NO_BACKUP));

						return FALSE;
					}
				}
				fHdl = (FILENO)Fcreate(fullName, 0);
				if (fHdl < 0)
				{
					F_ErrorAlert(fHdl);
					return FALSE;
				}
			}
		}
	}
	/* So, jetzt ist garantiert ein File offen, wir koennen endlich schreiben */
	if (ed->block && saveas && saveBlock)
	{
		for (i = 0; i < maxBlocks; i++)
		{
			if (ed->blocks[i].blockStart.line >= 0 && ed->blocks[i].blockEnd.line >= 0)
			{
				EditFuncs_outputText(ed,
					ed->blocks[i].blockStart.line, ed->blocks[i].blockStart.row,
					ed->blocks[i].blockEnd.line, ed->blocks[i].blockEnd.row,
					fHdl, ed->umbruch, FALSE);
			}
		}
	} else
	{
		EditFuncs_outputText(ed, 0, 0, -1, -1, fHdl, ed->umbruch, FALSE);
	}
	i = Fclose(fHdl);
	F_ErrorAlert(i);

	if (i != 0)
	{
		return FALSE;
	} else
	{
		if (takeName &&
			((saveas && !ed->block && fname != NULL && fname[0] != '\0') ||
			 (ed->new && !ed->block)) &&
			!appended && ed->readOnly==MODE_RW)
		{
			_UBYTE *newName;
			
			/* hier noch Assign auf Filename */
			newName = STRDUP(fullName, "SaveText: newName");
			if (newName == NULL)
			{
				/* not enough memory!! */
				return TRUE;
			}
			if (ed->fileName.text != NULL)
			{
				SFREE(ed->fileName.text);
			}
			ed->fileName.text = newName;
			ed->fileName.length = (EDCOLUMN)strLen(fullName) + 1;
		}
		if (!(ed->block && saveas))
		{
			ed->changed = FALSE;
		}
	}
	EditTools_SetEditTitle(ed);
	F_Path_Dirname(fullName);
	Protokoll_SendPathUpdate(fullName);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditUtil_PerformDragDrop(EDITOR *ed, _WORD x, _WORD y, _UWORD kstate, _UWORD buts)
{
	_WORD outline[maxBlocks][MAX_MARKS * 4 + 1];
	GRECT outRects[maxBlocks][4];
	_WORD count;
	_WORD pIdx[maxBlocks];
	_WORD i;
	_WORD deltaX;
	_WORD deltaY;
	_WORD oldX;
	_WORD oldY;
	_WORD style;
	_WORD bl;

	/* get block rectangles */
	for (bl = 0; bl < maxBlocks; bl++)
	{
		if (ed->blocks[bl].blockStart.line >= 0 &&
			ed->blocks[bl].blockEnd.line >= 0)
		{
			EditDraw_GetBlockRects(ed, outRects[bl], &count,
				ed->blocks[bl].blockStart.line, ed->blocks[bl].blockStart.row,
				ed->blocks[bl].blockEnd.line, ed->blocks[bl].blockEnd.row);
		} else
		{
			count = 0;
		}
		/* now calculate outline points of rectangles */
		pIdx[bl] = 0;
		if (count > 0)
		{
			outline[bl][pIdx[bl]+0] = outRects[bl][0].g_x;
			outline[bl][pIdx[bl]+1] = outRects[bl][0].g_y;
			pIdx[bl] += 2;
			outline[bl][pIdx[bl]+0] = outRects[bl][0].g_w + 1;
			outline[bl][pIdx[bl]+1] = outRects[bl][0].g_y;
			pIdx[bl] += 2;
			outline[bl][pIdx[bl]+0] = outRects[bl][0].g_w + 1;
			outline[bl][pIdx[bl]+1] = outRects[bl][0].g_h + 1;
			pIdx[bl] += 2;
			/* walk down right edge */
			for (i = 1; i < count; i++)
			{
				outline[bl][pIdx[bl]+0] = outRects[bl][i].g_w + 1;
				outline[bl][pIdx[bl]+1] = outRects[bl][i].g_y;
				pIdx[bl] += 2;
				outline[bl][pIdx[bl]+0] = outRects[bl][i].g_w + 1;
				outline[bl][pIdx[bl]+1] = outRects[bl][i].g_h + 1;
				pIdx[bl] += 2;
			}
			/* Jetzt rueckwaerts die linke Kante hoch */
			for (i = count - 1; i >= 0; i--)
			{
				outline[bl][pIdx[bl]+0] = outRects[bl][i].g_x;
				outline[bl][pIdx[bl]+1] = outRects[bl][i].g_h + 1;
				pIdx[bl] += 2;
				outline[bl][pIdx[bl]+0] = outRects[bl][i].g_x;
				outline[bl][pIdx[bl]+1] = outRects[bl][i].g_y;
				pIdx[bl] += 2;
			}
		}
		pIdx[bl] /= 2;
	}
	/* Jetzt haben wir eine Outline, da sind zwar moeglicherweise doppelte
	 * Punkte drin, aber das ist egal
	 */
	oldX = x;
	oldY = y;
	Wind_Update(BEG_MCTRL);
	EditGlobals_ShowMouse(FALSE);
	EditGlobals_MouseHand();
	/* Writemode auf XOR setzen */
	vswr_mode(ed->hdl, MD_XOR);

	/* Linestyle etc. setzen */
	vsl_type(ed->hdl, USERLINE);

	if (x & 1)
		style = 0xAAAA;
	else
		style = 0x5555;
	vsl_udsty(ed->hdl, style);
	vsl_width(ed->hdl, 1);
	vsl_color(ed->hdl, BLACK);

	/* Clipping setzen */
	WdwManager_SetClip(ed->hdl, &deskSize, TRUE);
	/* Linie zeichnen */
	EditGlobals_HideMouse();
	for (bl = 0; bl < maxBlocks; bl++)
	{
		if (pIdx[bl] > 0)
		{
			v_pline(ed->hdl, pIdx[bl], outline[bl]);
		}
	}
	EditGlobals_ShowMouse(FALSE);
	do
	{
		graf_mkstate(&x, &y, &buts, &kstate);
		if (x != oldX || y != oldY)
		{
			EditGlobals_HideMouse();
			for (bl = 0; bl < maxBlocks; bl++)
			{
				if (pIdx[bl] > 0)
				{
					/* Linie wieder wegzeichnen an alten Koordinaten */
					v_pline(ed->hdl, pIdx[bl], outline[bl]);
					/* Koordinaten anpassen */
					deltaX = x - oldX;
					deltaY = y - oldY;
					for (i = 0; i < pIdx[bl]; i++)
					{
						outline[bl][i * 2 + 0] += deltaX;
						outline[bl][i * 2 + 1] += deltaY;
					}
					/* Neue Outline zeichnen */
					v_pline(ed->hdl, pIdx[bl], outline[bl]);
				}
			}
			EditGlobals_ShowMouse(FALSE);
			oldX = x;
			oldY = y;
		}
	} while (buts & MOB_LEFT);
	/* Outline wieder wegzeichnen */
	EditGlobals_HideMouse();
	for (bl = 0; bl < maxBlocks; bl++)
	{
		if (pIdx[bl] > 0)
		{
			v_pline(ed->hdl, pIdx[bl], outline[bl]);
		}
	}
	EditGlobals_ShowMouse(FALSE);
	/* Writemode zuruecksetzen */
	vswr_mode(ed->hdl, MD_REPLACE);

	/* Clipping auf Workbereich setzen */
	EditGlobals_ClipWork(ed);
	Wind_Update(END_MCTRL);
	/* Drag und Drop ausfuehren */
	Wind_Update(END_UPDATE);
	WdwManager_WdwDDServe(ed->wdw, x, y, kstate);
	Wind_Update(BEG_UPDATE);
	/* Mauszeiger zuruecksetzen */
	EditGlobals_MouseArrow();
	EditGlobals_HideMouse();
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt zurueck, ob die Position x, y im Blockbereich liegt
 */

GLOBAL _BOOL EditUtil_ClickIsInBlock(HWND wdw, _WORD x, _WORD y)
{
	EDITOR *ed;
	EDLINE clLine;
	EDCOLUMN clRow;
	EDLINE oLine;
	EDCOLUMN oRow;

	if (!Editor_BlockIsMarked(wdw))
	{
		return FALSE;
	}
	ed = EditFuncs_FindEditor(wdw);
	if (ed != NULL)
	{
		oRow = ed->currRow;
		oLine = ed->currLineNr;
		if (rc_inside(x, y, &ed->editWork))
		{
			EditTools_GetClickLineAndRow(ed, x, y, &clLine, &clRow);
			EditTools_SetCurrLine(ed, oLine);
			ed->prefRow =ed->currRow = oRow;
			/* WITH ed->blocks[0]. */ 
			{
				if (((clLine > ed->blocks[0].blockStart.line) ||
					 ((clLine == ed->blocks[0].blockStart.line) && (clRow >= ed->blocks[0].blockStart.row))) &&
					((clLine < ed->blocks[0].blockEnd.line) ||
				  ((clLine == ed->blocks[0].blockEnd.line) && (clRow <= ed->blocks[0].blockEnd.row))))
				{
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID DoEvent(_UBYTE *scan, _UBYTE *ch, _WORD *taste, _UWORD *kstate)
{
	_WORD mBuff[8];
	_WORD moX, moY;
	_UWORD moButton;
	_WORD bReturn;

	evnt_multi(MU_KEYBD,
		0, 0, 0,
		0, 0, 0, 0, 0,
		0, 0, 0, 0, 0,
		mBuff,
		0, 0,
		&moX, &moY, &moButton, kstate, taste, &bReturn);
	*ch = (*taste) & 0xff;
	*scan = (*taste >> 8) & 0xff;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL SortMark(CONST _VOID *a1, CONST _VOID *a2, _VOID *private)
{
	CONST aMark *m1;
	CONST aMark *m2;

	UNUSED(private);
	m1 = a1;
	m2 = a2;
	if (m1->line < 0)
		return FALSE;
	return m1->line < m2->line || (m1->line == m2->line && m1->row < m2->row);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID RemoveTwoMarks(CONST aMark *which, aMark *marks, _WORD *highMark, _WORD *lastMark)
{
	_WORD i, k;
	_WORD rmvCount;

	i = 0;
	rmvCount = 0;
	while (i <= *highMark && rmvCount < 2)
	{
		if (which->line == marks[i].line && which->row == marks[i].row)
		{
			if (i == *lastMark)
			{
				++i;
			} else
			{
				/* marks[i] entfernen */
				for (k = i; k < *highMark; k++)
				{
					marks[k] = marks[k + 1];
				}
				--(*highMark);
				if (*lastMark > i)
				{
					--(*lastMark);
				}
				++rmvCount;
			}
		} else
		{
			++i;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID MarkToBlocks(EDITOR *ed, aMark *markSort, aMark *marks, _WORD *highMark, _WORD *lastMark)
{
	_WORD i;
	_WORD high;

	if ((*highMark) & 1)
	{
		high = *highMark;
	} else
	{
		high = *highMark - 1;
	}
	for (i = 0; i <= high; i++)
		markSort[i] = marks[i];
	QuickSort_sortIt(0, high, markSort, SortMark, sizeof(aMark), QuickSort_noBreak, NULL);
	for (i = 0; i < maxBlocks; i++)
	{
		ed->blocks[i].blockStart.line = -1;
		ed->blocks[i].blockEnd.line = -1;
	}
	for (i = 0; i <= high; i += 2)
	{
		if (markSort[i].line != markSort[i + 1].line ||
			markSort[i].row != markSort[i + 1].row)
		{
			ed->blocks[i / 2].blockStart = markSort[i];
			ed->blocks[i / 2].blockEnd = markSort[i + 1];
			ed->multiBlockIdx = i / 2;
		} else
		{
			RemoveTwoMarks(&markSort[i], marks, highMark, lastMark);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EditUtil_MarkMultiple(EDITOR *ed)
{
#define maxMarks ((maxBlocks - 1) * 2)

	_WORD i;
	_WORD taste;
	_UBYTE scan, ch;
	_UWORD kstate;
	Key key;
	EDLINE oldLine;
	EDCOLUMN oldRow;
	_BOOL marked;
	_BOOL wasMarked;
	_BOOL ctrl;
	_WORD lastMark;
	_WORD highMark;
	aMark marks[maxMarks];
	aMark markSort[maxMarks];

	if (ed == NULL)
		return;
	/* vorhandene Blockmarkierung loeschen */
	EditTools_ClearBlock(ed);
	ed->block = FALSE;
	ed->multiBlockIdx = 0;
	lastMark = 0;
	highMark = 0;
	EditDraw_CenterCurrline(ed);
	/* Variablen initialisieren */
	/* Menuzeile sperren und Window-Events verhindern */
	Wind_Update(BEG_UPDATE);
	Wind_Update(BEG_MCTRL);
	EditGlobals_HideMouse();
	/* Cursor zeigen */
	EditGlobals_HideCursor(ed);
	ed->blockIndent = TRUE;
	ed->multiBlockMark = TRUE;
	wasMarked = FALSE;
	for (i = 0; i < maxMarks; i++)
	{
		marks[i].line = -1;
		marks[i].row = -1;
	}
	marks[0].line = ed->currLineNr;
	marks[0].row = ed->currRow;
	EditGlobals_ShowCursor(ed);
	do
	{
		oldLine = ed->currLineNr;
		oldRow = ed->currRow;
		/* Event holen */
		DoEvent(&scan, &ch, &taste, &kstate);
		GemCharToKey(taste, kstate, &key);
		marked = (kstate & K_SHIFT) != 0;
		if (marked != wasMarked)
		{
			++lastMark;
			if (lastMark >= maxMarks || highMark >= maxMarks)
			{
				--lastMark;
				/* Abbrechen, maximale Anzahl Marken erreicht */
				CatGlobal_Bing(cBingError);
				key.ch = 'Q';
				key.ctrl = 0;
				key.scan = 0;
			}
			highMark = Max(highMark, lastMark);
		}
		kstate &= K_CTRL;
		ctrl = kstate != 0;
		if (IsSpecial(&key))
		{
			switch (SpecialKey(&key))
			{
			case home:
				EditFuncs_Home(ed, FALSE);
				break;
			case clr:
				marked = FALSE;		/* Nicht markieren! */
				EditFuncs_Home(ed, TRUE);
				break;
			case soln:
			case wdLeft:
			case left:
				EditFuncs_CursorLeft(ed, kstate);
				break;
			case eoln:
			case wdRight:
			case right:
				EditFuncs_CursorRight(ed, kstate);
				break;
			case pgUp:
			case up:
				EditFuncs_CursorUp(ed, ctrl);
				break;
			case pgDown:
			case down:
				EditFuncs_CursorDown(ed, ctrl);
				break;
			default:
				break;
			}
		}
		if (marked && (oldLine != ed->currLineNr || oldRow != ed->currRow))
		{
			EditDraw_MarkArea(ed, oldLine, oldRow, ed->currLineNr, ed->currRow, TRUE);
		}
		marks[lastMark].line = ed->currLineNr;
		marks[lastMark].row = ed->currRow;
		wasMarked = marked;
		if (marked && (oldLine != ed->currLineNr || oldRow != ed->currRow))
		{
			MarkToBlocks(ed, markSort, marks, &highMark, &lastMark);
		}
	} while (!IsCharacter(&key) || ucase(key.ch) != 'Q');

	/* Cursor zeigen */
	EditGlobals_HideCursor(ed);
	ed->multiBlockMark = FALSE;
	ed->blockIndent = FALSE;
	ed->block = Editor_BlockIsMarked(ed->wdw);
	EditGlobals_ShowCursor(ed);
	EditGlobals_ShowMouse(FALSE);
	Wind_Update(END_MCTRL);
	Wind_Update(END_UPDATE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL EDLINE JoinLine(EDITOR *ed)
{
	EDCOLUMN qLen1;
	EDCOLUMN len;
	EDCOLUMN newLen;
	EDCOLUMN qLen;
	_UBYTE quoteStr[128],quoteStr1[128];
	_UBYTE *newText;

	if (EditBase_GetLineLength(ed, ed->currLineNr + 1, &len))
	{
		/* temporaere Zeile allozieren */
		newText = MALLOC(len + 2, "EditUtil_JoinLine");
		if (newText == NULL)
		{
			return 0;
		}
		/* Neue Laenge berechnen */
		newLen = len + ed->editLen;
		EditGlobals_AllocNew(&ed->editLine, newLen + 2);

		/* naechste Zeile holen */
		EditBase_GetLine(ed, ed->currLineNr + 1, newText, len + 2, &len);

		/* Quote holen */
		EditGlobals_GetQuote(ed, ed->editLine.text, quoteStr1, sizeof(quoteStr1));
		qLen1 = (EDCOLUMN)strLen(quoteStr);
		EditGlobals_GetQuote(ed, newText, quoteStr, sizeof(quoteStr));
		qLen = (EDCOLUMN)strLen(quoteStr);
		if(qLen1>0 && qLen==0)
			return 1;
		if(strCmp(quoteStr1,quoteStr))
			qLen=0;
		ed->editLine.text[ed->editLen - 1] = '\0';
		strCpy(newText, newText + qLen);
		strCat(ed->editLine.text, newText);
		ed->editLen = (EDCOLUMN)strLen(ed->editLine.text);

		ed->editChanged = TRUE;

		EditBase_PutCurrLine(ed);
		EditBase_DeleteLine(ed, ed->currLineNr + 1);
		if (ed->currLineNr < ed->StartLine)
		{
			--ed->StartLine;
		}
		SFREE(newText);
	}
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID BuildOriginalText(EDITOR *ed)
{
	EDLINE cLine;

	/* Jetzt die Zeilen durchgehen */
	cLine = 0;
	EditTools_SetCurrLine(ed, 1);
	while (cLine < ed->totalLineNr)
	{
		EditTools_SetCurrLine(ed, cLine);
		if (ed->editLine.text[ed->editLen - 1] == CR)
		{
			/* Hier haben wir umgebrochen */
			cLine+=JoinLine(ed);
		} else
		{
			++cLine;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD TextWidth(EDITOR *ed, CONST _UBYTE *str, EDCOLUMN idx, EDCOLUMN len, _WORD maxWdwWidth)
{
	if (fastWrap)
	{
		if (idx < 30000 / ed->charWidth)
		{
			return idx * ed->charWidth;
		} else
		{
			return 30000;
		}
	}
	return EditGlobals_GetTextWidth(ed, str, idx, len, maxWdwWidth);
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.20a */
LOCAL _VOID WrapLine(EDITOR *ed, _WORD maxWdwWidth)
{
	_WORD width;
	EDCOLUMN lastIdx;
	EDCOLUMN idx, l, ql;
	_UBYTE quoteStr[128];

	idx = l = ed->editLen;
	do
	{
		width = TextWidth(ed, ed->editLine.text, idx, l, maxWdwWidth);
		if (width > maxWdwWidth)
		{
			/* umbrechen */
			lastIdx = idx;
			if (ed->monoSpaced)
			{
				idx = (maxWdwWidth / ed->charWidth) - 1;
			} else
			{
				if (width >= 30000)
				{
					idx = Min((maxWdwWidth * 7 / 5) / ed->charWidth, idx);
				} else
				{
					/* Erstmal eine Annaeherung berechnen */
					idx -= (width - maxWdwWidth) / ed->charWidth;
					if(idx<0)
						idx=lastIdx/2; /* mal einfach versuchen */
				}
			}
			idx = Min(idx, lastIdx);
			--idx;
			while (idx > 0 && ed->editLine.text[idx] != ' ')
			{
				--idx;
			}
			if (idx == 0)
			{
				/* Zwangsumbrechen im Wort */
				idx = Min(lastIdx, (maxWdwWidth * 7 / 5) / ed->charWidth);
				while (width > maxWdwWidth && idx > 0)
				{
					--idx;
					width = TextWidth(ed, ed->editLine.text, idx, l, maxWdwWidth);
				}
				if (idx == 0)
				{
					idx = lastIdx;	/* kein Umbruch */
					width = maxWdwWidth;
				}
			}
		}

		if(idx>10 && EditGlobals_isQuote(ed,ed->editLine.text+idx+1))
			maxWdwWidth=width-1;
	} while (width > maxWdwWidth);
	if (idx < l)
	{
		/* umbrechen, Zeile bei idx trennen */
		++idx;
		EditGlobals_GetQuote(ed, ed->editLine.text, quoteStr, sizeof(quoteStr));
		/* Wir basteln den Quote nur zusammen, wenn das */
		ql = (EDCOLUMN)strLen(quoteStr);
		EditGlobals_AllocNew(&ed->editLine, l + ql + 5);

		if (idx > (ql+2))
		{
			MemMove(ed->editLine.text + idx + ql + 1, ed->editLine.text + idx, l - idx + 1);
			ed->editLine.text[idx] = CR;
			MemCpy(ed->editLine.text + idx + 1, quoteStr, ql);
			ed->editLen += ql + 1;
		} else
		{
			MemMove(ed->editLine.text + idx + 2, ed->editLine.text + idx, l - idx + 1);			
			ed->editLine.text[idx] = CR;			
			ed->editLine.text[idx+1] = '>';			
			ed->editLen += 1;
		}
		/* Zeile in Buffer schreiben */
		ed->editChanged = TRUE;
		EditBase_PutCurrLine(ed);
		/* Zeile trennen */
		EditBase_InsertCr(ed, ed->currLineNr, idx + 1);
		if (ed->currLineNr < ed->StartLine)
		{
			++ed->StartLine;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL EditUtil_WrapText(EDITOR *ed, _BOOL force)
{
	EDLINE cLine;
	EDLINE saveLine;
	_WORD mWidth;
	_WORD wdwWidth;
	_BOOL viewWrap;
	
	/* Umbruch ueberhaupt eingeschaltet */
	if (!force)
	{
		if (ed->fileName.text != NULL)
		{
			return FALSE;
		}
		ConfVars_GetConfDefBool(cViewWrap, &viewWrap, TRUE);
		if (!viewWrap)
		{
			return FALSE;
		}
	}
	ConfVars_GetConfDefBool(cViewFastWrap, &fastWrap, FALSE);

	/* Reservebreite holen */
	mWidth = TextWidth(ed, "   ", 2, 3, 30000);

	/* Umbruchbreite berechnen */
	wdwWidth = ed->editWork.g_w - pixOff - mWidth;

	/* Keine Aenderung? Raus hier! */
	if (wdwWidth == ed->lastWrapWidth)
	{
		return FALSE;
	}
	EditGlobals_MouseBusy();

	if (ed->block)
	{
		EditTools_ClearBlock(ed);
	}
	/* Text wieder in Originalform bringen */
	saveLine = ed->currLineNr;
	BuildOriginalText(ed);

	/* Jetzt die Zeilen durchgehen */
	/* Umbruchbreite merken */
	ed->lastWrapWidth = wdwWidth;
	cLine = 0;
	while (cLine < ed->totalLineNr)
	{
		EditTools_SetCurrLine(ed, cLine);
		WrapLine(ed, wdwWidth);
		++cLine;
	}
	ed->maxWidth = 0;
	EditTools_SetCurrLine(ed, saveLine);
	ed->currLineNr = saveLine;

	EditGlobals_MouseArrow();
	return TRUE;
}
