/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#define __FIND2_IMPLEMENTATION__
#include <find2.h>
#include <regex.h>
#include <mtalerts.h>

#include <ro_mem.h>
#include <routine.h>

#define MAX_WILD 10

#define AL_REGEX_ERROR "[%P:|Das Suchmuster enth\204lt Fehler]][:[Abbruch]"


struct _tPattern {
	struct re_pattern_buffer re_pattern;
	_UBYTE *searchString;
	_LONG searchStart;
	/* wir brauchen ein Array-Element mehr,
	 * weil Register 0 immer die Werte des
	 * des gesamten Match-Strings beinhaltet
	 */
	regoff_t startOffset[MAX_WILD+1];
	regoff_t endOffset[MAX_WILD+1];
	struct re_registers regs;
	_BOOL backwards;
	_UBYTE pattern[1];
};


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL tPattern *Find2_Compile(CONST _UBYTE *pattern, size_t patlen, _UBYTE exist, _UBYTE all, _UBYTE all_but_eol, _UBYTE eol, _BOOL cap, _BOOL backward)
{
	tPattern *pPatt;
	reg_syntax_t syntax;
	
	UNUSED(eol);
	UNUSED(exist);
	
	pPatt = MALLOC(sizeof(tPattern) + patlen, "Find2_Compile");
	if (pPatt == NULL)
		return NULL;
	MemSetZeroStruct(*pPatt);
	
	strMcpy(pPatt->pattern, patlen + 1, pattern);
	pPatt->pattern[patlen] = '\0';
	pPatt->backwards = backward;
	
	syntax = RE_SYNTAX_POSIX_BASIC;
	if (all != all_but_eol)
	{
		syntax &= ~RE_DOT_NEWLINE;
		syntax |= RE_HAT_LISTS_NOT_NEWLINE;
		pPatt->re_pattern.newline_anchor = 1;
	} else
	{
		pPatt->re_pattern.newline_anchor = 0;
	}
	pPatt->re_pattern.no_sub = FALSE;
	pPatt->re_pattern.regs_allocated = REGS_FIXED;
	
	if (cap)
		pPatt->re_pattern.translate = get_lowercase_table();
	
	re_set_syntax(syntax);
	/* re_compile_pattern((pPatt->pattern, patlen, &pPatt->rePattern); */
	
	switch (regex_compile(pPatt->pattern, patlen, syntax, &pPatt->re_pattern))
	{
	case REG_ESPACE:
		err_memory(0);
		break;
	default:
		mtAlerts_Alert(1, AL_REGEX_ERROR);
		break;
	case REG_NOERROR:
		pPatt->regs.num_regs = MAX_WILD + 1;
		pPatt->regs.start = pPatt->startOffset;
		pPatt->regs.end = pPatt->endOffset;
		return pPatt;
	}
	OFREE(pPatt);
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Find2_Find(_UBYTE *Text, _LONG TextLen, tPattern *p, _LONG Start)
{
	p->searchString = Text;
	p->searchStart = Start;
	if (p->backwards)
		return re_search_2(&p->re_pattern, NULL, 0, Text, (int)TextLen, (int)Start, (int)(-Start), &p->regs, 0) >= 0;
	/* else searching forwards */
	return re_search_2(&p->re_pattern, NULL, 0, Text, (int)TextLen, (int)Start, (int)(TextLen - Start), &p->regs, (int)TextLen) >= 0;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Find2_Dispose(tPattern **pPattern)
{
	if (*pPattern != NULL)
	{
		regfree(&(*pPattern)->re_pattern);
		OFREE(*pPattern);
		*pPattern = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Find2_GetMatch(tPattern *pPattern, _WORD patIdx, _UBYTE **patPos, _WORD *patLen)
{
	if (patIdx < 0 || patIdx >= MAX_WILD || pPattern == NULL)
		return FALSE;
	if (patIdx == 0 && pPattern->regs.start[1] < 0)
	{
		/* Dann waren wohl keine Wildcards im Suchstring,
		 * und wir nehmen das Gesamt-Ergebnis
		 */
	} else
	{
		++patIdx;
	}
	if (pPattern->regs.start[patIdx] < 0)
		return FALSE;
	*patPos = pPattern->searchString + pPattern->regs.start[patIdx];
	*patLen = pPattern->regs.end[patIdx] - pPattern->regs.start[patIdx];
	return TRUE;
}
