/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#define __FIND2_IMPLEMENTATION__
#include <find2.h>

#include <ro_mem.h>
#include <routinec.h>
#include <routines.h>

/*
 * (c) 3/1992 Andreas Alich
 *
 * C-Implementation 1/1998 Thorsten Otto
 */

typedef struct _tPat {
	struct _tPat *pNext;	/* naechstes Teilmuster, ggf. NIL */
	struct _tPat *pPrev;	/* vorheriges Teilmuster, ggf. NIL */
	_UBYTE *pMatch;			/* Zeiger auf gefundene Textstelle */
	_WORD len;				/* Laenge des Teilmusters */
	/* interne Werte: */
	_UBYTE *pLimit;
	_BOOL no_eol;
	_UBYTE eol;
	_WORD idx;
#define PAT_IGNORECASE 0x01
#define PAT_HAS_EXIST  0x02
	_UBYTE d[256];
	_UBYTE pattern[1];		/* formaler Typ */
} tPat;

struct _tPattern {
	tPat *pFirst;			/* erstes Teilmuster, ggf. NIL */
	tPat *pLast;			/* letztes Teilmuster, ggf. NIL */
	/* interne Werte: */
	_UBYTE any;
	_BOOL backward;
};


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL tPattern *Find2_Compile(CONST _UBYTE *pattern, size_t patlen, _UBYTE any, _UBYTE all, _UBYTE all_but_eol, _UBYTE eol, _BOOL cap, _BOOL backward)
{
	tPattern *pPattern;
	tPat **next;				/* A5 */	/* Adresse des Nachfolgerzeigers */
	CONST _UBYTE *scan;			/* A0 */
	tPat *pat;					/* A2 */	/* aktuelles Pat */
	CONST _UBYTE *startpos;		/* D4 */	/* Anfang des Teilmusters */
	CONST _UBYTE *pEnd;						/* Ende des Teilmusters */
	size_t len;					/* D2 */	/* Laenge des Teilmusters */
	_UBYTE *lastpos;			/* D5 */	/* Position+1 des letzten/ersten ?, das nicht an
         									   letzter/erster Stelle steht */
	size_t copylen;
	_UBYTE *p;					/* A3 */
	_UBYTE ch;
	
	pPattern = NEW(tPattern, "Find2_Compile: pPattern");
	if (pPattern == NULL)
		return NULL;
	pPattern->pFirst = NULL;
	pPattern->pLast = NULL;
	pPattern->any = any;
	pPattern->backward = backward;
	next = &pPattern->pFirst;
	pEnd = pattern;
	pat = NULL;
	for (;;)
	{
	/* com_1: */
		scan = pEnd;
		lastpos = NULL;		/* Initialisierung, falls Muster leer */
		/*
		 * fuehrende '@' und '*' ueberlesen
		 */
	/* com_2: */
		while (patlen && (*scan == all || *scan == all_but_eol))
		{
			patlen--;
			scan++;
		}
		startpos = scan;
	/* com_4: */
		/*
		 * weiter bis zum Ende des Teilmusters
		 */
		while (patlen && (*scan != all && *scan != all_but_eol))
		{
			patlen--;
			scan++;
		}
	/* com_8: */
		pEnd = scan;
		len = scan - startpos;
		if (len == 0)
			break;
		*next = pat = MALLOC(sizeof(tPat) + len, "Find2_Compile: tPat");
		if (pat == NULL)
		{
			Find2_Dispose(&pPattern);
			return NULL;
		}
		pat->pPrev = NULL;
		pat->pNext = NULL;
		pat->len = (_WORD)len;
		pat->eol = eol;
		pat->idx = 0;
		next = &pat->pNext;
		p = pat->pattern;
		if (!backward)
			*p++ = '\0';
	/* com_62: */
		scan = startpos;
		copylen = len;
		if (!cap)
		{
	/* com_61: */
			do
				*p++ = *scan++;
			while (--copylen != 0);
		} else
		{
	/* com_65: */
			CONST _UBYTE *tab = get_uppercase_table();
			
	/* com_66: */
			do
			{
				ch = *scan++;
				if (ch != any)
					ch = tab[ch];
				*p++ = ch;
			} while (--copylen != 0);
		}
	/* com_69: */
		if (backward)
			*p = '\0';
	/* com_63: */
		scan = pEnd;
		pat->no_eol = patlen != 0 && *scan == all_but_eol;		/* '*' als Trennsymbol */
		
		if (!backward)
		{
			p = &pat->pattern[1];			/* fuehrendes '\0'! */
			/* letztes Zeichen mit ? vgl. */
			if (p[len - 1] == any)
				pat->idx |= PAT_HAS_EXIST;	/* Muster enthaelt ? */
	/* com_19: */
			if (len >= 2)
			{
				copylen = len - 1;
	/* com_11: */
				do
				{
					if (*p++ == any)
					{
						lastpos = p;		/* Position des (letzten) ? merken */
						pat->idx |= PAT_HAS_EXIST;	/* Muster enthaelt ? */
					}
	/* com_17: */
				} while (--copylen);
			}
	/* com_21: */
			if (lastpos != NULL)
				len = &pat->pattern[len + 1] - lastpos;
	/* com_10: */
			MemSet(pat->d, (_UBYTE)len, sizeof(pat->d));
			if (--len == 0)
			{
				if (cap)
					pat->idx |= PAT_IGNORECASE;
			} else
			{
	/* com_13: */
				p = &pat->pattern[1];		/* fuehrendes '\0'! */
				if (lastpos != NULL)
					p = lastpos;
	/* com_14: */
				if (!cap)
				{
					/* d gemaess den Zeichen nach dem letzten ? anpassen */
	/* com_16: */
					do
					{
						pat->d[*p++] = len;
					} while (--len);
				} else
				{
	/* com_20: */
					CONST _UBYTE *tab = get_lowercase_table();
					
					/* gro/klein beim Vgl. ignorieren */
					pat->idx |= PAT_IGNORECASE;
					
					/* d gemaess den Zeichen nach dem letzten ? anpassen */
	/* com_26: */
					do
					{
						pat->d[*p] = len;
						pat->d[tab[*p++]] = len;
					} while (--len);
				}
			}
		} else
		{
	/* com_40: */
			/* Rueckwaertssuche */
			
			p = &pat->pattern[0];
			/* erstes Zeichen mit ? vgl. */
			if (*p == any)
				pat->idx |= PAT_HAS_EXIST;	/* Muster enthaelt ? */
	/* com_43: */
			if (len >= 2)
			{
				p += len;
				copylen = len - 1;
	/* com_41: */
				do
				{
					if (*--p == any)
					{
						lastpos = p;		/* Position des (letzten) ? merken */
						pat->idx |= PAT_HAS_EXIST;	/* Muster enthaelt ? */
					}
	/* com_47: */
				} while (--copylen);
			}
	/* com_48: */
			if (lastpos != NULL)
				len = lastpos - &pat->pattern[0];
	/* com_49: */
			MemSet(pat->d, (_UBYTE)len, sizeof(pat->d));
			if (--len == 0)
			{
				if (cap)
					pat->idx |= PAT_IGNORECASE;
			} else
			{
	/* com_45: */
				p = &pat->pattern[len + 1];	/* len wurde um 1 verringert */
				if (!cap)
				{
					/* d gemaess den Zeichen nach dem letzten ? anpassen */
	/* com_46: */
					do
					{
						pat->d[*--p] = len;
					} while (--len);
				} else
				{
	/* com_50: */
					CONST _UBYTE *tab = get_lowercase_table();
					
					/* gro/klein beim Vgl. ignorieren */
					pat->idx |= PAT_IGNORECASE;
					
					/* d gemaess den Zeichen nach dem letzten ? anpassen */
					do
					{
	/* com_56: */
						pat->d[*--p] = len;
						pat->d[tab[*p]] = len;
					} while (--len);
				}
			}
		}
	}
	/* com_30: */
	
	/* Rueckwaertsverkettung erstellen */
	pPattern->pLast = pat;
	if (pPattern->pFirst != NULL)
	{
		pat = pPattern->pFirst;
	/* com_32: */
		while (pat->pNext != NULL)
		{
			pat->pNext->pPrev = pat;
			pat = pat->pNext;
		} 
	} else
	{
	/* com_35: */
		/* kein Pat vorhanden */
		next = &pPattern->pFirst;
		*next = pat = MALLOC(sizeof(tPat), "Find2_Compile: tPat");
		if (pat != NULL)
		{
			pat->pPrev = NULL;
			pat->pNext = NULL;
			pPattern->pLast = pat;
			pat->len = 0;
			pat->pattern[0] = '\0';
			pat->eol = eol;
			pat->idx = 0;
			pat->no_eol = FALSE;
		} else
		{
			Find2_Dispose(&pPattern);
			return NULL;
		}
	}
	/* com_34: */
	/* Sprungtabellen initialisieren: */
	/* Find2_Find(NULL, 0L, NULL, 0L); */
	
	return pPattern;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Find2_Find(
	_UBYTE *Text,					/* Anfangsadresse des zu durchsuchenden Textes */
	_LONG TextLen,					/* Gesamtlaenge des Textes */
	tPattern *p,					/* Zeiger auf vorcompiliertes Suchmuster */
	_LONG Start)					/* Such-Startposition im Text */
{
	tPat *pat;				/* A0 */
	_UBYTE *pPatternEnd;	/* D6 */
	_UBYTE *pEnd;			/* D7 */
	_UBYTE *scanstart;		/* A1 */
	_UBYTE *startpos;
	_LONG searchLen;		/* Textlaenge ab Suchstart */
	_LONG patternLen;		/* Gesamtlaenge der Teilmuster */
	_LONG len;
	CONST _UBYTE *upperTab;	/* A4 */
	tPat *pPat;				/* A5 */
	_BOOL no_eol;			/* D1 */
	_UBYTE *pattern;
	_UBYTE any;				/* D3 */
	_UBYTE *scan;			/* A0 */
	
	pat = p->pFirst;
	if (pat->len == 0)
	{
		/* leeres Muster */
		if (Text == NULL)
			return FALSE;
		pat->pMatch = Text + Start;
		return TRUE;
	}
	/* f_1: */
	pEnd = Text;
	if (!p->backward)
	{
		scanstart = Text + Start;
		startpos = scanstart;
	/* ff_0: */
		do
		{
			scanstart += pat->len;		/* zunaechst Zeiger hinter Match */
			pat->pMatch = scanstart;
			pat = pat->pNext;
		} while (pat != NULL);
		pEnd += TextLen;
		searchLen = TextLen - Start;
		if (searchLen <= 0)
			return FALSE;
		patternLen = scanstart - startpos;
		if (searchLen < patternLen)
			return FALSE;				/* Muster laenger als Rest des Textes */
		
		any = p->any;
		upperTab = get_uppercase_table();
		pPat = p->pLast;
		pPat->pLimit = pEnd;
		no_eol = FALSE;
	/* ff_3: */
		for (;;)
		{
			scanstart = pPat->pMatch;
			pEnd = pPat->pLimit;
			if (no_eol)
			{
				scan = pEnd;
	/* ff_4: */
				for (;;)
				{
					if (*--scan == pPat->eol)
					{
						scanstart = scan + 1;
						break;
					}
					if (scan <= scanstart)
						break;
				}
			}
	/* ff_2: */
	f_rescan:
			scan = pPat->pattern + pPat->len - 1;
			pPatternEnd = scan + 2;
			switch (pPat->idx)
			{
			default:
				/* Fwd */
	/* ff_14: */
				do
				{
					scan = scanstart;
					pattern = pPatternEnd;
	/* ff_15: */
					do
					{
						if (*--pattern == '\0')
							goto f_match;
					} while (*--scan == *pattern);
					scanstart += pPat->d[scanstart[-1]];
				} while (scanstart < pEnd);
				break;
			case PAT_IGNORECASE:
				/* FwdCap */
	/* ff_24: */
				do
				{
					scan = scanstart;
					pattern = pPatternEnd;
	/* ff_25: */
					do
					{
						if (*--pattern == '\0')
							goto f_match;
					} while (upperTab[*--scan] == *pattern);
					scanstart += pPat->d[scanstart[-1]];
				} while (scanstart < pEnd);
				break;
			case PAT_HAS_EXIST:
				/* FwdAny */
	/* ff_34: */
				do
				{
					scan = scanstart;
					pattern = pPatternEnd;
	/* ff_35: */
					for (;;)
					{
						if (*--pattern == '\0')
							goto f_match;
						if (*pattern == any)
						{
							--scan;
						} else
						{
							if (*--scan != *pattern)
								break;
						}
					}
					scanstart += pPat->d[scanstart[-1]];
				} while (scanstart < pEnd);
				break;
			case PAT_HAS_EXIST|PAT_IGNORECASE:
				/* FwdCapAny */
	/* ff_34: */
				do
				{
					scan = scanstart;
					pattern = pPatternEnd;
	/* ff_35: */
					for (;;)
					{
						if (*--pattern == '\0')
							goto f_match;
						if (*pattern == any)
						{
							--scan;
						} else
						{
							if (upperTab[*--scan] != *pattern)
								break;
						}
					}
					scanstart += pPat->d[scanstart[-1]];
				} while (scanstart < pEnd);
				break;
			}
	/* ff_7: */
			do
			{
				/* Keine Uebereinstimmung */
				if (pPat->pNext == NULL)
					return FALSE;			/* kein Nachfolger -> Suche erfolglos */
				pPat = pPat->pNext;
				scanstart = pPat->pMatch;
				pEnd = pPat->pLimit;
				scanstart += pPat->d[scanstart[-1]];
			} while (scanstart < pEnd);
			goto f_rescan;
	/* ff_6: */
	f_match:
			/* Uebereinstimmung */
			pPat->pMatch = scanstart;
			if (pPat->pPrev == NULL)
				break;
			scan = scanstart - pPat->len;
			pPat = pPat->pPrev;
			no_eol = pPat->no_eol;
			pPat->pLimit = scan;
			/* Suche mit Vorgaenger fortsetzen */
		}
	/* ff_10: */
		/* Alle Teilmuster passen */
	/* ff_11: */
		do
		{
			pPat->pMatch -= pPat->len;
			pPat = pPat->pNext;
		} while (pPat != NULL);
	} else
	{
	/* fb_1: */
		/* Rueckwaertssuche */
		if (Start < 0)
			return FALSE;
		len = TextLen;
	/* fb_8: */
		do
		{
			len -= pat->len;
			pat = pat->pNext;
		} while (pat != NULL);
		if (Start > len)
		{
			/* Startposition nach unten korrigieren (Muster laenger als Rest) */
			Start = len;
			if (Start < 0)
				return FALSE;
		}
	/* fb_9: */
		scanstart = Text + Start;
		pat = p->pFirst;
	/* fb_0: */
		do
		{
			pat->pMatch = scanstart;
			scanstart += pat->len;
			pat = pat->pNext;
		} while (pat != NULL);
		
		any = p->any;
		upperTab = get_uppercase_table();
		pPat = p->pFirst;
		pPat->pLimit = pEnd;
		no_eol = FALSE;
	/* fb_3: */
		for (;;)
		{
			scanstart = pPat->pMatch;
			pEnd = pPat->pLimit;
			if (no_eol)
			{
				scan = pEnd;
	/* fb_4: */
				for (;;)
				{
					if (*scan++ == pPat->eol)
					{
						scanstart = scan - 1;
						break;
					}
					if (scan >= scanstart)
						break;
				}
			}
	/* fb_2: */
	b_rescan:
			scan = pPat->pattern;
			pPatternEnd = scan;
			switch (pPat->idx)
			{
			default:
				/* Bwd */
	/* fb_14: */
				do
				{
					scan = scanstart;
					pattern = pPatternEnd;
	/* fb_15: */
					do
					{
						if (*pattern == '\0')
							goto b_match;
					} while (*scan++ == *pattern++);
					scanstart -= pPat->d[scanstart[0]];
				} while (scanstart >= pEnd);
				break;
			case PAT_IGNORECASE:
				/* BwdCap */
	/* fb_24: */
				do
				{
					scan = scanstart;
					pattern = pPatternEnd;
	/* fb_25: */
					do
					{
						if (*pattern == '\0')
							goto b_match;
					} while (upperTab[*scan++] == *pattern++);
					scanstart -= pPat->d[scanstart[0]];
				} while (scanstart >= pEnd);
				break;
			case PAT_HAS_EXIST:
				/* BwdAny */
	/* fb_34: */
				do
				{
					scan = scanstart;
					pattern = pPatternEnd;
	/* fb_35: */
					for (;;)
					{
						if (*pattern == '\0')
							goto b_match;
						if (*pattern == any)
						{
							scan++;
							pattern++;
						} else
						{
							if (*scan++ != *pattern++)
								break;
						}
					}
					scanstart -= pPat->d[scanstart[0]];
				} while (scanstart >= pEnd);
				break;
			case PAT_HAS_EXIST|PAT_IGNORECASE:
				/* BwdCapAny */
	/* fb_34: */
				do
				{
					scan = scanstart;
					pattern = pPatternEnd;
	/* fb_35: */
					for (;;)
					{
						if (*pattern == '\0')
							goto b_match;
						if (*pattern == any)
						{
							scan++;
							pattern++;
						} else
						{
							if (upperTab[*scan++] != *pattern++)
								break;
						}
					}
					scanstart -= pPat->d[scanstart[0]];
				} while (scanstart >= pEnd);
				break;
			}
	/* fb_7: */
			do
			{
				/* Keine Uebereinstimmung */
				if (pPat->pPrev == NULL)
					return FALSE;			/* kein Nachfolger -> Suche erfolglos */
				pPat = pPat->pPrev;
				scanstart = pPat->pMatch;
				pEnd = pPat->pLimit;
				scanstart -= pPat->d[scanstart[0]];
			} while (scanstart >= pEnd);
			goto b_rescan;
	/* fb_6: */
	b_match:
			/* Uebereinstimmung */
			pPat->pMatch = scanstart;
			if (pPat->pNext == NULL)
				break;
			scan = scanstart + pPat->len;
			pPat = pPat->pNext;
			no_eol = pPat->no_eol;
			pPat->pLimit = scan;
			/* Suche mit Nachfolger fortsetzen */
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Find2_Dispose(tPattern **ppPattern)
{
	tPattern *pPattern;
	tPat *pat, *next;
	
	pPattern = *ppPattern;
	if (pPattern != NULL)
	{
		pat = pPattern->pFirst;
		while (pat != NULL)
		{
			next = pat->pNext;
			SFREE(pat);
			pat = next;
		}
		OFREE(pPattern);
	}
	*ppPattern = NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Find2_GetMatch(tPattern *pPattern, _WORD patIdx, _UBYTE **patPos, _WORD *patLen)
{
	tPat *ptrPatt;
	
	if (pPattern == NULL)
		return FALSE;
	ptrPatt = pPattern->pFirst;
	while (ptrPatt != NULL && patIdx > 0)
	{
		ptrPatt = ptrPatt->pNext;
		patIdx--;
	}
	if (ptrPatt == NULL)
		return FALSE;
	*patPos = ptrPatt->pMatch;
	*patLen = ptrPatt->len;
	return TRUE;
}
