#ifndef __EDITBASE_H__
#define __EDITBASE_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif


#ifndef __EDITTYPES_H__
#include <edittype.h>
#endif


typedef enum {
	notFound,
	notEnoughMem,
	lineToLong
} EditError;


#define memBlockSize    0x7fe0   /* Groesse der einzelnen Bloecke im Editor */


extern EditError eReason;


_BOOL EditBase_InsertBuffer(EDITOR *ed, _UBYTE **buff, _ULONG len, _BOOL freeBuffer);
/* Es wird ein Text aus einem Buffer in ed an der aktuellen Stelle eingefuegt.
 */

_BOOL EditBase_CopyToBuff(EDITOR *ed, EDLINE tStartLine, EDLINE tEndLine, _UBYTE **addBuf, _ULONG *len);
/* Kopieren von einem EditBuffer in einen anderen.
 * Es wird der aus dem Editor in addBuf der Bereich von
 * tStartLine bis tEndLine kopiert.
 * Neue Fassung, die alles in einen Block kopiert und nicht in n kleine Bloecke.
 */

EDBUFSIZE EditBase_GetOffset(EDITOR *ed);
/* Setzt Buffer auf CurrentLine und berechnet Offset in
 * currentBuffer fuer currLineNr und currRow
 */

_VOID EditBase_GetPos(EDITOR *ed, EDBUFSIZE offset, EDLINE *line, EDCOLUMN *row);
/* Berechnet aus Offset in currentBuff die aktuelle
 * Zeile und Spalte
 */

_BOOL EditBase_GetLineLength(EDITOR *ed, EDLINE lineNum, EDCOLUMN *len);
/* Liefert die Laenge der Zeile lineNum
 * FALSE: Zeile nicht vorhanden.
 */

_BOOL EditBase_GetLineAdr(EDITOR *ed, EDLINE lineNum, _UBYTE **text, EDCOLUMN *len);
/* Liefert die Adresse der Zeile und die Laenge.
 * Nur fuer Lesezugriff!
 * FALSE: Zeile nicht vorhanden
 */

_BOOL EditBase_GetLine(EDITOR *ed, EDLINE lineNum, _UBYTE *str, size_t strsize, EDCOLUMN *len);
/* Holt die Zeile lineNum aus der Bufferliste
 * FALSE: Entweder ist der String str fuer die Zeile zu kurz,
 *        dann steht in len schon die benoetigte Laenge,
 *        oder die Zeile existiert nicht, dann ist len = 0.
 */

_BOOL EditBase_PutLine(EDITOR *ed, EDLINE lineNum, CONST _UBYTE *str, EDBUFSIZE len);
/* Ersetzt eine Zeile im Buffer.
 * FALSE: Entweder Zeile ist nicht vorhanden, oder es ist nicht mehr
 * genug Speicher da, um die Zeile einzufuegen.
 * Im zweiten Fall sollte man ein CompactBuffer aufrufen und
 * es dann nochmal versuchen.
 * Grund steht in der globalen Variable eReason.
 */

_BOOL EditBase_InsertCr(EDITOR *ed, EDLINE lineNum, EDCOLUMN row);
/* Trennt die Zeile lineNum an der Stelle row auf.
 * FALSE: Kein Speicherplatz mehr frei.
 */

_BOOL EditBase_InsertLine(EDITOR *ed, EDLINE lineNum, CONST _UBYTE *str, _BOOL pre, EDBUFSIZE len);
/* Fuegt die Zeile lineNum im Buffer ein.
 * Wenn pre = TRUE ist, dann wird die Zeile vor die alte Zeile lineNum gesetzt,
 * ansonsten hinter die Zeile lineNum.
 * Ergebnis: FALSE: Nicht genug Speicher mehr frei
 */

_BOOL EditBase_DeleteSomeLines(EDITOR *ed, EDLINE from, EDLINE to);
/* Loescht einen Bereich von Zeilen ueber mehrere Bloecke hinweg
 * Dabei wird mittels deleteLineBlock in den aeusseren Bloecken
 * geloescht und die inneren Bloecke entfernt.
 */

_BOOL EditBase_DeleteLine(EDITOR *ed, EDLINE lineNum);
/* Loescht die Zeile lineNum im Buffer
 * FALSE: Zeile nicht vorhanden
 */

_BOOL EditBase_CompactBuffer(EDITOR *ed);
/* Kompaktiert den Bufferspeicher. FALSE: Nicht mehr moeglich, da
 * gar kein Speicher mehr frei war.
 */

_VOID EditBase_GetCurrLine(EDITOR *ed, _UBYTE *str, size_t strsize, EDBUFSIZE *len);
/* Liefert die aktuelle Zeile, in der der Cursor steht.
 */

_BOOL EditBase_PutCurrLine(EDITOR *ed);
/* Schiebt die aktuelle Zeile, in der der Cursor steht, wieder in den Buffer.
 * FALSE: Kein Speicher mehr frei
 */

_BOOL EditBase_GetBuffer(EDITOR *ed, EDBUFSIZE defSize);
/* Erzeugt einen neuen Buffer hinter currentBuff
 * Es wird nur der Textbuffer und der Descriptor alloziert,
 * das LineArray noch nicht.
 * Der Buffer erhaelt die Groesse defSize, falls dieser Wert sinnvoll ist.
 */

_VOID EditBase_FreeBuffer(EDITOR *ed);
/* Haengt aktuellen Buffer (currentBuff) aus Liste aus
 * und gibt alle Speicherbloecke frei.
 * Es wird keine Pruefung vorgenommen, ob noch Text enthalten ist!
 */

_VOID EditBase_CountBytes(EDITOR *ed, EDLINE fromLine, EDCOLUMN fromRow, EDLINE toLine, EDCOLUMN toRow, _ULONG *bytes);
/* Zaehlt die Bytes, die von dem Text zwischen den beiden Textmarken belegt werden
 */


#endif /* __EDITBASE_H__ */
