#ifndef __EDITOR_H__
#define __EDITOR_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif

#ifndef __EDITTYPES_H__
#include <edittype.h>
#endif

typedef _VOID (*KillProcType)(CONST _UBYTE *path, CONST _UBYTE *name);

extern KillProcType KillProc;



_VOID Editor_SelectEditFont(HWND wdw, _WORD font, _WORD fontSize);
/* Setzt den neuen Font fuer den Editor
 */

_VOID Editor_SelectFont(HWND wdw);

/* Anzeige-Funktionen des Editors! */
/* User-Proc Parameter */
/*!! Hier aufpassen!*/
/* Edit-Parameter */
_BOOL Editor_OpenEditBuffer(
	CONST _UBYTE *title,
	CONST _UBYTE *infoline,
	_BOOL showInfo,
	_UBYTE **buff,
	_ULONG buffLen,
	_BOOL freeBuffer,
	_BOOL isEnriched,
	_WORD font,
	_WORD fontSize,
	GRECT *curr,
	_BOOL useUserProc,
	userProcType theUserProc,
	userGetRectProc theUserRectProc,
	userDrawProc theDrawUserProc,
	userCloseProc theCloseProc,
	userKeyProc theKeyProc,
	userTopProc theTopProc,
	userUntopProc theUntopProc,
	userPrePrintProc thePrePrintProc,
	HWND *wdwHandle);
/* Oeffnet einen Editor und liest Text aus Buffer */

_VOID Editor_CloseEditBuffer(HWND wdw);
/* Schliesst Nachrichtenfenster */

_VOID Editor_EditAbandonText(HWND wdw);
/* Verwirft alle Aenderungen und laedt den Text neu nach Rueckfrage
 */

_VOID Editor_EditReloadText(HWND wdw);
/* Verwirft alle Aenderungen und laedt den Text neu ohne Rueckfrage
 */

_VOID Editor_ShowNewBuffer(HWND wdw, CONST _UBYTE *title, CONST _UBYTE *infoline,
	_UBYTE **buff, _ULONG buffLen, _BOOL freeBuffer, _BOOL isEnriched);
/* Zeigt neuen Text im Anzeigefenster an! */

/* User-Proc Parameter */
/*!! Hier aufpassen!*/
/* Edit-Parameter */
/* Wird als interne Editornummer benutzt, falls nicht schon vergeben und > 0 */
_BOOL Editor_OpenEditor(
	CONST _UBYTE *path,
	CONST _UBYTE *FileName,
	_VOID /* List */ *SrcPaths,
	_BOOL New,
	_BOOL useUserProc,
	userProcType theUserProc,
	userGetRectProc theUserRectProc,
	userDrawProc theDrawUserProc,
	userCloseProc theCloseProc,
	userKeyProc theKeyProc,
	userTopProc theTopProc,
	userUntopProc theUntopProc,
	userPrePrintProc thePrePrintProc,
	EDCOLUMN right,
	_BOOL showInfo,
	_BOOL wrap,
	_WORD num,
	HWND *wdwHandle);

/* oeffnet einen Editor im Fenster. Font und Fontsize und gewuenschte
 * Workarea muessen uebergeben werden, wdwHandle dieses Editors wird
 * im Erfolgsfall zurueckgeliefert.
 * Ueber die User-Procs kann man noch einen Bereich im Editor-Fenster
 * festlegen, in denen Benutzerdefinierte Sachen dargestellt werden koennen.
 * Die User-Routine wird angesprungen, falls ein Button-Event in diesem Bereich
 * auftrat. Die RectProc wird nach jeder Groessen und Positionsaenderung aufgerufen.
 * Die DrawUserProc wird aufgerufen, wenn Fensterteile neu gezeichnet werden
 * muessen.
 * !!! Im Moment ist es nur moeglich, einen Bereich am oberen Ende des Fensters
 * als UserBereich zu benutzen!!!
 * Die Parameter fuer die Userprozeduren sind als Adressen aufgefuehrt, damit es
 * leichter ist, keine Userproc zu uebergeben! Deshalb kann aber der Compiler
 * das auch nicht ueberpruefen, also Vorsicht beim Aufruf!
 */

_VOID Editor_InstallUserDD(
	HWND wdw,
	userDDGetMaxExtProc userDDGetMaxExts,
	userDDGetExtProc userDDGetExt,
	userDDGetExtNameProc userDDGetExtName,
	userDDGetSizeProc userDDGetSize,
	userDDGetDataProc userDDGetData);
/* Installiert den DD-Server der Userprozedur
 */

_VOID Editor_EditPrint(HWND wdw, _BOOL block, _BOOL askCancel);

/* Druckt entweder den ganzen Text oder nur den markierten Block */

_BOOL Editor_EditSave(HWND wdw, CONST _UBYTE *path, CONST _UBYTE *FileName, _BOOL takeName, _BOOL saveBlock);

/* nachfragen, wenn File schon vorhanden */

_VOID Editor_EditMerge(HWND wdw, CONST _UBYTE *path, CONST _UBYTE *name);

/* Blockfunktionen */
_BOOL Editor_EditCut(HWND wdw, _BOOL toClip);

_BOOL Editor_EditCopy(HWND wdw);

_VOID Editor_EditPaste(HWND wdw);

_VOID Editor_EditClear(HWND wdw);

_VOID Editor_EditPasteFile(HWND wdw, _BOOL isFilter, CONST _UBYTE *path, CONST _UBYTE *name);
/* Fuegt einen Text mit Filename ein
 */

_BOOL Editor_CloseEditor(HWND wdw, _BOOL force);
/* wird aufgerufen, wenn Editor geschlossen werden soll. Falls Text geaendert
 * wurde, wird nachgefragt und ggf. FALSE zurueckgegeben.
 * Bei force = TRUE wird nicht nachgefragt, sondern der Text gespeichert und
 * in jedem Fall TRUE zurueckgegeben.
 */

_BOOL Editor_IsEditTop(HWND wdw);
/* fragt nach, ob ein Editor das Top-Window hat
 */

_BOOL Editor_EditChar(HWND wdw, _VOID *special, _WORD taste, _UBYTE ch, _UBYTE scan, _UWORD kstate);

_BOOL Editor_ClickInEditWindow(HWND win, _VOID *special, _WORD x, _WORD y, _UWORD kstate, _UWORD buts, _WORD clicks);

/* Fragt nach, ob die letzte Fassung des Textes geladen werden soll und macht das
 * dann
 */

_BOOL Editor_GetBlock(HWND wdw, _BOOL unChanged, _BOOL withCrs, _UBYTE **adr, _ULONG *buffLen);
/* Kopiert den gesamten Block an einem Stueck
 * in einen Speicherbereich. Nur am Absatzende wird ein CR/LF
 * angefuegt.
 * Dieser Bereich wird von dieser Routine angelegt und muss
 * bei Nichtgebrauch wieder freigegeben werden
 */

_BOOL Editor_GetText(HWND wdw, _UBYTE *adr, _ULONG buffLen);
_BOOL Editor_GetTextSize(HWND wdw, _ULONG *buffLen);

_BOOL Editor_PutLine(HWND win, _UBYTE *adr, size_t len);
/* Voll? */
/* Fuer Floskeltasten und aehnliches
 */

_BOOL Editor_QuoteSomething(HWND win, CONST _UBYTE *pre, CONST _UBYTE *adr, EDCOLUMN len, _WORD width);
/* Voll? */
/* Fuer Quoten.
 */


/* Umfangreiche Such- und Ersetzroutinen.
 *
 * Start:       aktuelle Position, Top, Ende
 * Richtung:    nach oben, nach unten
 * Anzahl:      erstes, ntes, letztes Vorkommen
 * Modusflags:  grossklein, nurWort
 */
/* wird nur bei count = nmal benutzt */
_BOOL Editor_Search(
	HWND wdw,
	CONST _UBYTE *w,
	searchPos pos,
	searchDir direction,
	searchCount count,
	_WORD num,
	_BOOL ignoreCase,
	_BOOL onlyWord,
	_BOOL again,
	CONST _UBYTE *bingEvent);
/* Nur suchen, gefundene Stelle wird angezeigt */

/* wird nur bei count = nmal benutzt */
_BOOL Editor_SearchAndReplace(
	HWND wdw,
	CONST _UBYTE *w,
	CONST _UBYTE *r,
	searchPos pos,
	searchDir direction,
	searchCount count,
	_WORD num,
	_BOOL ignoreCase,
	_BOOL onlyWord,
	_BOOL ask,
	_BOOL again);
/* Suchen und ersetzen, evtl. mit Rueckfrage
 */

_BOOL Editor_BlockIsMarked(HWND wdw);
/* Ist Ersatz fuer EditMenu(); Das MUSS in CAT gemacht werden!
 */

_BOOL Editor_ClickIsInBlock(HWND wdw, _WORD x, _WORD y);
/* Gibt zurueck, ob die Position x, y im Blockbereich liegt
 */

_VOID Editor_GetEditVDIHandle(HWND wdw, HVDI *hdl);
/* Liefert Handle der Workstation des Fensters */




_BOOL Editor_CloseAllEditors(_BOOL force);
/* schliesst alle Editoren, gibt TRUE zurueck, falls
 * erfolgreich
 * Wenn force = TRUE ist, dann werden alle Texte ohne Rueckfrage
 * gespeichert!
 */

_VOID Editor_RedrawAll(_VOID);
/* Zeichnet alle Fenster neu */

_WORD Editor_GetEditNumber(HWND wdw);
/* Gibt die interne Nummer des Editors zurueck */

_VOID Editor_EditSaveWdwPos(HWND wdw);
/* Sichert die Position des Editors in den
 * Configvariablen
 */

_VOID Editor_EditSavePos(_VOID);
/* Sichert die Position und den Inhalt aller offenen Editorfenster
 * (nicht ReadOnly-Fenster
 */

_VOID Editor_EditRestorePos(_BOOL doOpen);
/* Restauriert den Inhalt und die Position der EditorFenster
 */

_VOID Editor_EditSetOffset(HWND wdw, _ULONG offset);
/* Setzt den Cursor an die Position offset im aktuellen Buffer
 * Wird nur fuer CAT gebraucht im Messagemanager!
 */

_VOID Editor_EditSetLineAndRow(HWND wdw, EDLINE line, EDCOLUMN row);
/* Setzt den Cursor an die angegebene Position, falls moeglich
 */

_VOID Editor_DoUndo(HWND wdw);
/* Fuehrt Undo durch */

_VOID Editor_MarkAll(HWND wdw);
/* Markiert den ganzen Text als Block */

_VOID Editor_MarkWord(HWND wdw);
/* Markiert das Wort beim Cursor als Block */

_VOID Editor_SetBlockStart(HWND wdw);
/* Setzt den Blockstart an der aktuellen
 * Cursorposition
 */

_VOID Editor_SetBlockEnd(HWND wdw);
/* Setzt das Blockende an der aktuellen
 * Cursorposition
 */

_VOID Editor_SetMark(HWND wdw, _WORD number);
/* Setzt Marke number an Cursorposition
 */

_VOID Editor_ClearBlock(HWND wdw);
/* Loescht Blockmarkierung
 */

_VOID Editor_ToMark(HWND wdw, _WORD number);
/* Springt zur Marke number
 */

_VOID Editor_ToBlockStart(HWND wdw);
/* Springt zum Blockanfang
 */

_VOID Editor_ToBlockEnd(HWND wdw);
/* Springt zum Blockende
 */

_VOID Editor_SetTabSize(HWND wdw, EDCOLUMN size);
/* Setzt TabSize auf size
 */

_VOID Editor_GetTabSize(HWND wdw, EDCOLUMN *size);
/* Setzt size auf TabSize
 */

_VOID Editor_SetMargins(HWND wdw, EDCOLUMN right, _BOOL umbruch);
/* Setzt die Raender und man kann den Umbruch ein-
 * und ausschalten
 */

_VOID Editor_GetMargins(HWND wdw, EDCOLUMN *right);
/* Gibt den eingestellten rechten Rand zurueck.
 */

/* Modus-Einstellungen */

typedef enum {
	insMode,
	indentMode,
	autoBackup,
	wrapMode,
	tabMode,
	crMode,
	roMode,
	effMode,
	errorMode,
	msgMode
} editMode;


_VOID Editor_ToggleMode(HWND wdw, editMode mode);
/* Schaltet einen Zustand um */

_VOID Editor_SetMode(HWND wdw, editMode mode, _BOOL val);
/* Setzt einen Zustand
 */
_VOID Editor_SetRW(HWND wdw, _BOOL val);
_WORD Editor_GetModeRW(HWND wdw);

_BOOL Editor_GetMode(HWND wdw, editMode mode);
/* Fragt einen Zustand ab.
 */

/* Block Transformationen */
_VOID Editor_BlockUp(HWND wdw);

_VOID Editor_BlockDown(HWND wdw);

_VOID Editor_BlockCapitals(HWND wdw);

_VOID Editor_BlockSwap(HWND wdw);

_VOID Editor_BlockRot18(HWND wdw);

_VOID Editor_BlockRot(HWND wdw);

_VOID Editor_BlockDecode(HWND wdw);

_VOID Editor_BlockMime(HWND wdw);

_VOID Editor_BlockMirror(HWND wdw);

_VOID Editor_BlockProtect(HWND wdw);

_VOID Editor_BlockUnProtect(HWND wdw);

_BOOL Editor_UndoPossible(HWND wdw);

_BOOL Editor_GetTextLineLength(HWND wdw, EDLINE line, EDCOLUMN *len);
/* Liefert die Laenge der Zeile line im zum wdw gehoerenden Editor zurueck.
 * FALSE: Fenster ist kein Editor oder Zeile nicht vorhanden
 */

_BOOL Editor_GetTextLine(HWND wdw, EDLINE line, _UBYTE *str, size_t maxlen, size_t *len);
/* Liefert die Zeile line im zum wdw gehoerenden Editor zurueck.
 * FALSE: Fenster ist kein Editor oder Zeile nicht vorhanden
 */

_BOOL Editor_GetEditBuffer(HWND wdw, _UBYTE **adr, EDBUFSIZE *len);
/* Liefert die Adresse des ersten Buffers im zum wdw gehoerenden Editor zurueck.
 * In len steht der benutzte Speicherplatz.
 * FALSE: Fenster ist kein Editor
 */

_BOOL Editor_AskForSave(HWND wdw, _BOOL mayCancel);
/* Fragt nach, falls der Text veraendert wurde, ob dieser gesichert
 * werden soll.
 */

_VOID Editor_ToEnd(HWND wdw);
/* Setzt den Cursor an das Ende eines Textes
 */

_VOID Editor_GetWildChars(_UBYTE one[2], _UBYTE all[2]);
/* Holt die Wildcardzeichen
 */

_VOID Editor_SetWildChars(_UBYTE one, _UBYTE all);
/* Setzt die Wildcardzeichen
 */

_VOID Editor_MarkMultiple(HWND wdw);
/* Startet den multiplen Blockmarkiermodus
 */



/* neue Funktionen fuer Eddix */

_VOID Editor_ReadShortkeyFile(CONST _UBYTE *fname);
/* Liest eine Kuerzeldatei ein
 */

_VOID Editor_FreeShortkeyList(_VOID);
/* Gibt eine Kuerzeldatei wieder frei
 */

_VOID Editor_ToLineNr(HWND wdw, EDLINE nr);
/* Geht zur Zeile nr, falls moeglich */

_BOOL Editor_GetCompInfo(HWND wdw, _UBYTE *fname, _BOOL *saved);
/* Liefert den kompletten Filenamen, falls das Fenster ein Editor
 * ist
 */

_BOOL Editor_FileInfos(HWND wdw, _ULONG *lines, _ULONG *bytes, _WORD *edits, _ULONG *totalMem, _UBYTE *fName);
/* Gibt Informationen ueber den aktuellen Text raus
 */

_VOID Editor_SetListMode(HWND wdw);
/* Setzt den Listenmodus fuer ein Fenster. Dabei wird der Cursor
 * ausgeschaltet, das Fenster auf Readonly gesetzt und der Listen-
 * modus angeschaltet
 */

_BOOL Editor_SetListProc(HWND wdw, userListProc listProc);
/* Setzt die ListModeProc fuer ein Fenster. Um alles einzuschalten,
 * sollte man einfach SetListMode aufrufen.
 */

_BOOL Editor_FileIsEditor(CONST _UBYTE *fname, HWND *wdw);
/* Gibt bei einem uebergebenem Dateinamen zurueck, ob diese Datei
 * schon geladen ist. Beachtet auch Case-Sensitivitaet von Filesystemen
 * Im Falle von TRUE wird in wdw das Fensterhandle des Editors
 * zurueckgeliefert
 */

/* Fuer Compilierung ein paar Prozeduren */
_VOID Editor_CompileText(HWND wdw);

_VOID Editor_ShowError(HWND wdw);

_VOID Editor_SetError(HWND wdw, EDLINE errorLine, EDCOLUMN errorRow, CONST _UBYTE *errorMsg, _BOOL show);

/* Block bearbeiten wie in GME */
_VOID Editor_IndentBlock(HWND wdw);

_BOOL Editor_Init(_VOID);

_VOID Editor_InsChr(HWND wdw);


#endif /* __EDITOR_H__ */
