#ifndef __EDITTYPES_H__
#define __EDITTYPES_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */




#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __CAT_WDWMANAGER_H__
#include <wdwmanag.h>
#endif
#ifndef __SET_H__
#include <set.h>
#endif

#define EDIT_DEBUG 0

#define maxBlocks       50
#define MAX_MARKS		5


#define cNoname "namenlos.txt"


typedef enum {
	LFIRST, LNEXT
} lineMode;

typedef _WORD EDCOLUMN;
typedef _LONG EDLINE;
typedef _WORD EDBUFSIZE;
typedef _WORD EDBUFLINE;

typedef struct aMark {
	EDLINE line;
	EDCOLUMN row;
} aMark;   /* aMark */

typedef struct aBlock {
	aMark blockStart;
	aMark blockEnd;
} aBlock;


typedef struct aLineDesc {
	EDCOLUMN length;
	_UBYTE *text;
} aLineDesc;   /* aLineDesc */


typedef struct aBuffer {
	_UBYTE *buffer;				/* Buffer fuer alle Zeilen */
	EDBUFSIZE bufSize;			/* Groesse des Textbuffers */
	EDBUFSIZE *lines;			/* Laengen der einzelnen Zeilen, inclusive CR/LF aber ohne EOS */
	EDBUFSIZE usedMem;			/* bis jetzt belegter Speicher */
	EDBUFLINE maxLine;			/* Groesse des lines-Array */
	EDLINE fromLine;			/* erste Zeile im Buffer */
	EDLINE toLine;				/* letzte Zeile im Buffer */
	struct aBuffer *next;
	struct aBuffer *prev;
} aBuffer, *aBufferPtr;


typedef struct aLine {
	struct aLine *next;
	struct aLine *prev;
	aLineDesc zeile;
} aLine, *aLinePtr;   /* aLine */

typedef _VOID (*userProcType)(HWND wdw, HVDI vdi, _WORD mx, _WORD my, _UWORD kstate, CONST GRECT *work);
typedef _VOID (*userGetRectProc)(HWND wdw, CONST GRECT *wdwRect, GRECT *user, GRECT *editwork);
typedef _VOID (*userDrawProc)(HWND wdw, HVDI vdi, CONST GRECT *work, CONST GRECT *clip, _WORD textCol, _WORD backCol);


/* Wird dann aufgerufen, wenn der Editor definitiv geschlossen wird! */
typedef _BOOL (*userCloseProc)(HWND wdw);


/* Falls diese Prozedur existiert, wird sie vor jeder Tastenbehandlung aufgerufen.
 * Man muss diese Prozedur aber nicht uebergeben, NIL geht auch!
 */
typedef _BOOL (*userKeyProc)(HWND wdw, _UBYTE *scan, _UBYTE *ch, _UWORD *kstate, _BOOL *moreChars);

/* Wird aufgerufen, bevor die Taste vom Editor behandelt wird.
 * Wenn TRUE zurueckgegeben wird, wird die Taste danach NICHT vom Editor behandelt.
 * Wenn moreChars auf TRUE gesetzt wird, wird nach der Behandlung durch den
 * Editor die Prozedur solange aufgerufen und die Tasten aus scan und c behandelt,
 * bis moreChars auf FALSE gesetzt wird.
 */

typedef _VOID (*userTopProc)(HWND wdw);
/* Wird aufgerufen, wenn das Fenster getoppt wird. Dann kann das Usermodul
 * ggf. noch andere dazu passende Aktionen unternehmen (Menuzeile umgestalten)
 * Man kann auch statt der Prozedur NIL uebergeben
 */

typedef _VOID (*userUntopProc)(HWND wdw);
/* Wird aufgerufen, wenn das Fenster enttoppt wird. Dann kann das Usermodul
 * ggf. noch andere dazu passende Aktionen unternehmen (Menuzeile umgestalten)
 * Man kann auch statt der Prozedur NIL uebergeben
 */

typedef _BOOL (*userPrePrintProc)(HWND wdw, lineMode which, _UBYTE *string, size_t strsize);
/* Falls diese Prozedur uebergeben wird, dann wird diese vor dem Drucken
 * eines Textes oder Blocks solange aufgerufen, bis sie als Ergebnis FALSE
 * liefert. Beim ersten Aufruf wird sie mit dem Parameter LFIRST und
 * bei den folgenden mit dem Parameter LNEXT aufgerufen.
 * Man kann auch NIL an den Editor uebergeben.
 */

typedef _VOID (*userListProc)(HWND wdw, CONST _UBYTE *line);
/* Diese Prozedur wird im Listenmodus benutzt, wenn eine Zeile
 * doppelgeclickt wird
 */

/* Die folgenden Prozedurtypen sind fuer die Unterstuetzung von zusaetzlichen
 * Drag&Drop-Typen als Server durch die Userprozeduren
 */

typedef _WORD (*userDDGetMaxExtProc)(HWND wdw);
/* Liefert die Anzahl der zusaetzlich verfuegbaren Extensions
 * fuer das Drag&Drop
 */

typedef _BOOL (*userDDGetExtProc)(HWND wdw, _WORD idx, _UBYTE ext[4]);
/* Liefert die erste oder weitere zusaetzliche Extensions.
 * idx ist der Index der Userextension, von 0 an gezaehlt.
 * In ext muss die Prozedur die Daten reinschreiben, 4 Zeichen lang!
 */

typedef _VOID (*userDDGetExtNameProc)(HWND wdw, _WORD idx, _UBYTE *name, size_t namsize);
/* Muss den Namen der Extension liefern
 * idx: Index der Userextension, von 0 an gezaehlt
 */

typedef _ULONG (*userDDGetSizeProc)(HWND wdw, _WORD idx);
/* Liefert die Groesse der zu verschickenden Daten
 * idx: Index der Userextension, von 0 an gezaehlt
 */

typedef _VOID (*userDDGetDataProc)(HWND wdw, _WORD idx, _VOID **data);
/* Liefert die zu verschickenden Daten
 * idx: Index der Userextension, von 0 an gezaehlt
 * Der Speicher wird spaeter vom Drag&Drop-Teil wieder
 * freigegeben
 */

typedef enum {
	fromPos,
	fromStart
} searchPos;

typedef enum {
	sdir_forward,
	sdir_backward
} searchDir;

typedef enum {
	searchOne,
	searchNum,
	searchAll
} searchCount;

typedef enum {
	GrossKlein = 0x0001,
	NurWort = 0x0002,
	silent = 0x0004
} SuchFlags;

typedef _UWORD SuchModus;

enum{ MODE_RW,MODE_SHOW,MODE_RO};	

typedef struct EDITOR {
	HWND wdw;								/* window handle */
	HVDI hdl;								/* VDI-Handle    */
	_WORD font;								/* font id */
	_WORD fontSize;							/* font point size */
	_UWORD windComps;
	_WORD charHeight;
	_WORD charWidth;						/* charWidht only for */
	_BOOL wasOpen;
	_BOOL isFSM;							/* ist FSM-Font?      */
	_BOOL monoSpaced;						/* monospaced fonts   */
	GRECT editWork;							/* Der reine Edit-Bereich */
	_BOOL isFull;							/* window fulled? */
	_BOOL isHidden;							/* window is hidden */
	_BOOL inForce;
	/* userProc fuer Events, die vom User gemanaget werden */
	_BOOL hasUserProc;
	userProcType userProc;
	userGetRectProc getUserRect;
	userDrawProc drawUserArea;
	userCloseProc userClose;
	userKeyProc userKey;
	GRECT userRect;
	userTopProc userTop;
	userUntopProc userUntop;
	userPrePrintProc userPrint;
	userListProc listModeProc;
	/* User-Drag&Drop */
	_BOOL userDD;
	userDDGetMaxExtProc userDDGetMaxExts;
	userDDGetExtProc userDDGetExt;
	userDDGetExtNameProc userDDGetExtName;
	userDDGetSizeProc userDDGetSize;
	userDDGetDataProc userDDGetData;
	/* -------- */
	_WORD hOffset;							/* Unterschied zwischen charHeight und userRect.h */
	/* nun die Editor-Variablen */
	_WORD windLines;						/* Zeilen pro Fensterseite */
	_BOOL new;
	_BOOL changed;
	_BOOL showInfo;
	aBuffer *firstBuff;
	aBuffer *currentBuff;
	EDLINE StartLine;						/* oberste Zeile im Fenster */
	EDLINE currLineNr;
	EDLINE totalLineNr;
	aLineDesc editLine;
	EDLINE editLineNr;
	EDCOLUMN editLen;
	_BOOL editChanged;
	_WORD maxWidth;							/* Breite in Pixeln des Textes */
	_WORD leftOffset;						/* Pixeloffset fuer linken Rand */
	EDCOLUMN currRow;
	EDCOLUMN prefRow;						/* Da wr man gerne (scrollen)	*/
	_WORD TabSize;
	_BOOL lineBlock;						/* Block wurde Zeilenweise markiert */
	_BOOL wordBlock;						/* Block wurde im Wortmodus markiert */
	aBlock blocks[maxBlocks];
	aMark markPoints[MAX_MARKS];			/* 5 Marken koennen gesetzt werden */
	EDCOLUMN rightMargin;
	_BOOL realTabs;							/* Echte Tabs ja/nein */
	_BOOL umbruch;							/* Umbruch ja/nein */
	_BOOL backup;							/* automatisch ein Backup erzeugen beim Speichern */
	_WORD readOnly;							/* Text darf nicht geaendert werden! */
	_BOOL isGrin;
	_BOOL EditShowLineEnds;
	_BOOL autoIndent;
	_BOOL insertMode;
	_BOOL listMode;							/* Die Anzeige ist nur eine Liste von Zeilen */
	_BOOL DisplayEffects;					/* Bei Anzeige auf Effekte parsen */
	_BOOL alienCoords;						/* Fensterkoordinaten werden von Fremdmodul verwaltet */
	_WORD number;							/* Die Nummer fuer die Editorvariablen */
	_WORD cursor;							/* cursor an/aus */
	_BOOL multiBlockMark;					/* Gerade im markieren multipler Bloecke */
	_BOOL blockIndent;						/* Gerade im Block einruecken */
	_BOOL block;							/* Ein Block ist an! */
	_WORD multiBlockIdx;					/* Nummer des Blocks, der gerade markiert wird */
	struct {
		EDLINE line;
		EDCOLUMN row;
		_WORD x;
		_WORD y;
	} cursPos;
	aLineDesc fileName;
	SuchModus findMode;
	_UBYTE WindowInfo[140];
	_UBYTE WindowTitle[140];
	/* Fuer Fehlermeldungen */
	EDLINE errLine;
	EDCOLUMN errRow;
	_UBYTE errMsg[128];
	struct EDITOR *nextEdit;
	/* Ab 2.02 fuer Farben etc. */
	_UWORD quoteEff;						/* Effekt fuer Quotes, nur Monochrom */
	_WORD quoteCol;							/* Farbe fuer Quotes */
	_UWORD quoteEff2;						/* Effekt fuer |-Quotes, nur Monochrom */
	_WORD quoteCol2;						/* Farbe fuer |-Quotes */
	_WORD textCol;							/* Textfarbe */
	_WORD backCol;							/* Hintergrundfarbe */
	_UWORD linkEff;						/* Effekt fuer Links, nur Monochrom */
	_WORD linkCol;							/* Farbe fuer Links */
	/* fuer Umbruch von langen Nachrichten */
	EDCOLUMN lastWrapWidth;
	_BOOL isEnriched;						/* Fuer Anzeige. Wenn TRUE, dann wird immer umbrochen */
	_BOOL wdwIsOpen;
	_BOOL AllowLowerQuoteInitials;				/* Kleinbuchstaben als Quoteinitialen? (MH) */
} EDITOR;


typedef enum {
	blockToChar,
	charToBlock,
	blockCut,
	cutUndo,
	blockPaste,
	normalPaste,
	nothingToDo
} undoType;


#define HALT() /* hmmm.. */

#define pixOff          2   /* pixelOffset fuer linken Rand          */

#define is_eff_char(ch) ((ch) == '*' || (ch) == '_' || (ch) == '/' || (ch) == '#')	/*ME 4.18b +4.18h */


extern SET TrennSet;
extern SET UmbruchSet;
extern SET NormalSet;
extern SET DelimiterSet;
extern SET WordEndSet;

extern _UBYTE theDrawLine[4096];

extern _UBYTE wildAllChar;   /* Alle Zeichen, Default: '*' */
extern _UBYTE wildOneChar;   /* Ein Zeichen, Default: '?' */

extern HVDI cursorHandle;   /* Handle der Workstation des Cursors */


_BOOL EditType_Init(_VOID);

#endif /* __EDITTYPES_H__ */
