#ifndef __FIND2_H__
#define __FIND2_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif


/* (c) Copyright 1992, Andreas Alich */


typedef struct _tPattern tPattern;

#ifndef __FIND2_IMPLEMENTATION__
struct _tPattern { int dummy; };
#endif


/* Muster */
/* Joker fuer genau ein beliebiges Zeichen */
/* Joker fuer beliebig viele beliebige Zeichen,
                  auch keines und Zeilenende */
/* Joker fuer beliebig viele Zeichen
                             ungleich Zeilenende ('eol'),
                             auch keines */
/* Zeichen, das bei 'all_but_eol' als Zeilenende
                  gewertet wird */
/* gross/klein ignorieren; nationale Sonderzeichen
          werden beruecksichtigt */
/* rueckwaerts suchen */
tPattern *Find2_Compile(CONST _UBYTE *pattern, size_t patlen, _UBYTE exist, _UBYTE all, _UBYTE all_but_eol, _UBYTE eol, _BOOL cap, _BOOL backward);
/*
    Compiliert das Muster vor fuer die eigentliche Suche mit Find. Die Werte
    in den Records duerfen nicht veraendert werden!
    Die Records werden mittels Storage alloziert. Falls der Speicher nicht
    ausreicht, wird Exception.RAISE (Exception.eNoMemory) aufgerufen. Falls
    kein Handler fuer Exception installiert ist, wird im Fehlerfall NIL
    zurueckgegeben.
    Die einzelnen tPat-Records repraesentieren ein Teilmuster, das durch einen
    Allquantor vom naechsten Teilmuster im Muster getrennt ist. Mehrere
    aufeinanderfolgende Allquantoren werden zu einem zusammengefat.
    Allquantoren am Anfang und am Ende des Musters werden ignoriert.
    Existenzquantoren sind im Teilmuster enthalten.
*/

/* Anfangsadresse des zu durchsuchenden Textes */
/* Gesamtlaenge des Textes */
/* Zeiger auf vorcompiliertes Suchmuster */
/* Such-Startposition im Text */
_BOOL Find2_Find(_UBYTE *Text, _LONG TextLen, tPattern *p, _LONG Start);
/*
    Durchsucht den Text ab Start nach dem bei Compile festgelegten Muster.
    Falls das Muster gefunden wurde, wird TRUE geliefert, sonst FALSE.
    Falls das Muster gefunden wurde, sind die Eintraege 'len' und 'pMatch'
    in den tPat-Records gueltig. 'pMatch' zeigt auf den Anfang des zum
    Teilmuster passenden Textteils. 'len' gibt die Laenge des Teilmusters an.
    Bei einem leeren Muster wird immer TRUE geliefert; in diesem Fall ist
    'pPattern^.pFirst' = 'pPattern^.pLast' und in 'pPattern^.pFirst^.pMatch'
    wird ein Zeiger auf die Startposition der Suche geliefert.
    'pPattern^.pFirst^.len' ist in diesem Fall gleich Null.
*/

_VOID Find2_Dispose(tPattern **pPattern);
/*
    Gibt den Speicher, der bei 'Compile' alloziert wurde, wieder frei. Falls
    'pPattern' NIL ist, so passiert nichts.
*/

_BOOL Find2_GetMatch(tPattern *pPattern, _WORD patIdx, _UBYTE **patPos, _WORD *patLen);
/*
 * Liefert Position und Laenge des n'ten Patterns nach einer
 * erfolgreichen Suche
 */

#endif /* __FIND2_H__ */
