#ifndef __PRINTER_H__
#define __PRINTER_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif


#define cCon            "CON:"
#define cAux            "AUX:"
#define cPrn            "PRN:"

#define cPica           0   /* 10 cpi */
#define cElite          1   /* 12 cpi */
#define cCondensed      2   /* 17 cpi */
#define cExpanded       3   /*  6 cpi */

#define cFat            0x01 /* fett          */
#define cLightened      0x02 /* hell          */
#define cSlanted        0x04 /* kursiv        */
#define cUnderlined     0x08 /* unterstrichen */

/* Color-Farbbaender: */
#define cBlack          0
#define cMagenta        1
#define cCyan           2
#define cYellow         3


typedef enum {
	Printer_done,
	Printer_notdone,
	Printer_noMemory,
	Printer_CFGnotFound,
	Printer_wrongIdent,
	Printer_ErrorInStreams,
	Printer_Cancelled,
	Printer_PrnDeviceBusy,
	Printer_AuxDeviceBusy,
	Printer_ConDeviceBusy
} tResponse;
typedef _BOOL (*tErrorProc)(tResponse);



extern tErrorProc ErrorProc;   /* ist mit 'DummyError' initialisiert */


tResponse Printer_LoadCFGFile(CONST _UBYTE *name);
/* Eroeffnung einer CFG-Datei. Der komplette Dateiname muss uebergeben werden.
   Falls keine Eroeffnung moeglich war, wird 'Response' auf 'CFGnotFound' gesetzt.
   Die Extension wird ggfs. automatisch korrigiert.
   Die Datei wird nach Eroeffnung auf die Dateikennung in den ersten 8 Bytes
   ueberprueft ('GST-CFG:'). Falls die Kennung nicht uebereinstimmt, wird
   die Datei wieder geschlossen und 'Response' auf 'wrongIdent' gesetzt.
   Im Erfolgsfall enthaelt 'Response' 'done'.
*/

_VOID Printer_FreeCFGFile(_VOID);
/* Loescht eine bereits geladene CFG-Datei aus dem Speicher
 */

tResponse Printer_QueryCFGFile(_UBYTE *path, _UBYTE *name, CONST _UBYTE *InfoText);
/* erfragt eine CFG-Datei per FileSelectBox.
   InfoText ist fuer die FileSelectBox gedacht.
   Die Endung wird fest auf '.CFG' festgeschrieben.
   Die Funktion ruft anschliessend OpenCFGFile auf.
*/
_VOID Printer_GetPrnName(_UBYTE *Printername, size_t strsize);
/* Eingetragenen Druckernamen liefern. Falls keiner vorhanden ist, dann
   wird ein Leerstring ('') geliefert.
 */

tResponse Printer_SendExit(_VOID);
/*  Es wird Eintrag 21H (Allgemeiner Druckerreset am Textende) gesendet. */

tResponse Printer_SendInit(_BOOL NLQ, _BOOL hAndVInit);
/* Drucker-Initialisierung senden. Es werden nacheinander folgende
   Eintraege geschickt (in Klammern die NLQ-Eintraege):
   - Eintrag 20H       Horizontale Initialisierung (nur bei hAndVInit = TRUE)
   - Eintrag 1FH       Vertikale Initialisierung (nur bei hAndVInit = TRUE)
   - Eintrag  7H ( 9H) Fettschrift aus.
   - Eintrag  BH ( DH) Kursivschrift aus.
   - Eintrag  FH (11H) Light aus.
   - Eintrag 13H (15H) Superscript aus.
   - Eintrag 17H (19H) Subscript aus.
   - Eintrag 1BH (1DH) Unterstreichung aus.
   Falls vorhanden, werden noch die folgenden Sequenzn geschickt:
   - Eintrag 28H (29H) PICA (10 CPI).
   - Eintrag 30H       Color-Einstellung: Black.
   - Eintrag 35H       Proportional-Schrift wird ausgeschaltet.
 */

tResponse Printer_SetMode(_WORD Mode);
/*  Zeichenbreite einstellen.
    Achtung: vorher 'SendInit' durchfuehren!
    Unterstuetzt werden:
    - cPica      = 0;    (* 10 cpi *)
    - cElite     = 1;    (* 12 cpi *)
    - cCondensed = 2;    (* 17 cpi *)
    - cExpanded  = 3;    (*  6 cpi *)
 */

tResponse Printer_SetColor(_WORD Color);
/*  Farbe einstellen. Unterstuetzt werden folgende Farben:
    - cBlack     = 0;
    - cMagenta   = 1;
    - cCyan      = 2;
    - cYellow    = 3;
*/
tResponse Printer_SetPropSize(_BOOL Prop, _WORD PropSize);
/* Ein/Ausschalten der Proportionalschrift sowie Setzen einer mittleren
   Zeichenbreite (fuer Tabulierung notwendig)
*/

tResponse Printer_Bold(_BOOL Setit);
/* Fettdruck ein/ausschalten          */

tResponse Printer_Light(_BOOL Setit);
/* Helle Schrift ein/ausschalten      */

tResponse Printer_Italic(_BOOL Setit);
/* Schraegschrift ein/ausschalten      */

tResponse Printer_Underline(_BOOL Setit);
/* Unterstreichen ein/ausschalten     */

tResponse Printer_Superscript(_BOOL Setit);
/* Hochstellen ein/ausschalten        */

tResponse Printer_Subscript(_BOOL Setit);
/* Tiefstellen ein/ausschalten        */


tResponse Printer_Effect(_UWORD Effects, _BOOL Setit);
/* Texteffekte setzen/ruecksetzen.
   Die Wertung im BITSET ist wie folgt:
   0 : fett,
   1 : hell,
   2 : kursiv,
   3 : unterstrichen
 */

tResponse Printer_FormFeed(_VOID);
/* Seitenvorschub ausgeben            */

_VOID Printer_SetTabSize(_WORD Tab);
/* Tabulatorweite einstellen          */

tResponse Printer_WriteLn(_VOID);
/* Zeilenvorschub generieren          */

tResponse Printer_Write(_UBYTE ch);

tResponse Printer_WriteString(CONST _UBYTE *Str);

tResponse Printer_Open(CONST _UBYTE *Name);
/* Eroeffnung eines Druckerkanals; wenn ein Fehler auftritt, wird 'Response'
   auf 'ErrorInStreams' gestellt; die eigentliche Fehlerursache ist dann bei
   'Streams.Response' zu suchen.
*/

_VOID Printer_Close(_VOID);
/* Schliessung eines Druckerkanals; wenn ein Fehler auftritt, wird 'Response'
   auf 'ErrorInStreams' gestellt; die eigentliche Fehlerursache ist dann bei
   'Streams.Response' zu suchen.
*/

_BOOL Printer_DummyError(tResponse ErrNr);
/* Mit dieser Fehlerprozedur ist 'ErrorProc' initialisiert. Sie besteht nur
   aus einem 'RETURN TRUE'.
   Mit Hilfe der ProzedurVariablen 'ErrorProc' kann eine eigene Fehlerausgabe
   angegeben werden.
 */


#endif /* __PRINTER_H__ */
