/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <printer.h>
#include <tos.h>
#include <ro_mem.h>
#include <mtdir.h>

#include <file_io.h>
#include <routines.h>
#include <routinec.h>
#include <edittool.h>


/*****************************************************************************
  Projekt:
  Datei  : PRINTER.M
  Datum  : 21.12.1991
  Autor  : Uwe Mindrup, Wolfsgrubenstrasse 23, 74379 Ingersheim, Tel: 07142/64385
           Uwe Mindrup @ S3
******************************************************************************/

typedef struct {
	_WORD len;
	_WORD pos;
} tTableEntry;

#define cMaxSequence 80
#define cMaxTranslations 256

      /* es folgen die Konstanten fuer den Tabelleneingang der Funktionen: */
#define cVertPos         0x05      /* absolute vertikale Positionierung */
#define cBold            0x06
#define cItalic          0x0A
#define cLight           0x0E
#define cSuperscript     0x12
#define cSubscript       0x16
#define cUnderline       0x1A
#define cHorzInit        0x1F
#define cVertInit        0x20
#define cPrinterInit     0x21

#define c1stType         0x28
#define c1stColor        0x30
#define cProp            0x35
#define cFormFeed        0x1E
#define cWriteLn         0x01
#define cTab             0x09

GLOBAL tErrorProc ErrorProc = Printer_DummyError;   /* ist mit 'DummyError' initialisiert */

LOCAL tTableEntry SeqTable[cMaxSequence];
LOCAL tTableEntry TransTable[cMaxTranslations];
LOCAL _UBYTE *CFG;
LOCAL _WORD TabSize = 8;
LOCAL _WORD NLQSet;
LOCAL _WORD CharPos;                 /* Position des Zeichens in der aktuellen Zeile */
LOCAL _WORD NrSpaces;                /* Anzahl Leerzeichen */
LOCAL _WORD PropSize;
LOCAL FILENO PrnHdl;
LOCAL _BOOL Proportional;
LOCAL _WORD AktColor;
LOCAL _WORD AktMode;

/* Ersatz fuer das Streams-Modul: Die Variablen */
LOCAL _WORD currentOut;             /* Das Handle des Kanals */


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Tabellen initialisieren: */
LOCAL _VOID InitTables(_VOID)
{
	_WORD i;

	/* Sequence- und Translationtable loeschen: */
	for (i = 0; i < cMaxSequence; i++)
		SeqTable[i].len = 0;
	for (i = 0; i < cMaxTranslations; i++)
		TransTable[i].len = 0;
	CharPos = 0;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Ausgabe eines Fehlers; die globale Variable 'Response' wird gesetzt und
 * die benutzerdefinierbare Fehlerfunktion wird aufgerufen.
 */
LOCAL _VOID Printer_Error(tResponse Err)
{
	/* Response = Err; */
	ErrorProc(Err);
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Ausgabe eines Fehlers; die globale Variable 'Response' wird gesetzt und
 * die benutzerdefinierbare Fehlerfunktion wird aufgerufen.
 */
LOCAL _BOOL QError(tResponse Err)
{
	/* Response = Err; */
	return ErrorProc(Err);
}

/*** ---------------------------------------------------------------------- ***/

/* Einfachste Form einer Fehlerbehandlung ist die: nichts tun */
GLOBAL _BOOL Printer_DummyError(tResponse Err)
{
	UNUSED(Err);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/*  Ausgabe eines einzelnen Zeichens; es wird VOR der Ausgabe geprueft, ob der
 *  Kanal ueberhaupt 'ready' ist.
 */
LOCAL tResponse Out(_UBYTE ch)
{
	_BOOL Ok, Cancel;
	_ULONG Timer, len;
	tResponse Response;
	
	do
	{
		Cancel = FALSE;
		Timer = 1000 + SystemTimer();	
		if (currentOut == 2)
		{
			do
				Ok = Cauxos();
			while (!Ok && Timer >= SystemTimer());	
			if (!Ok)
				Cancel = !QError(Printer_AuxDeviceBusy);
		} else if (currentOut == 3)
		{
			do
				Ok = Cprnos();
			while (!Ok && Timer >= SystemTimer());	
			if (!Ok)
				Cancel = !QError(Printer_PrnDeviceBusy);
		} else
		{
			Ok = TRUE;
		}
	} while (!Ok && !Cancel);
	if (Ok)
	{
		len = Fwrite(PrnHdl, 1, &ch);
		if (len == 1)
		{
			Response = Printer_done;
		} else
		{
			if (currentOut == 2)
			{
				Response = Printer_AuxDeviceBusy;
				Cancel = !QError(Response);
			} else if (currentOut == 3)
			{
				Response = Printer_PrnDeviceBusy;
				Cancel = !QError(Response);
			} else
			{
				Response = Printer_Cancelled;
				Cancel = TRUE;
			}
			if (Cancel)
			{
				Printer_Error(Response);
			}
		}
	} else
	{
		Printer_Error(Response = Printer_Cancelled);
	}
	return Response;
}

/*** ---------------------------------------------------------------------- ***/

/* Loescht eine bereits geladene CFG-Datei aus dem Speicher
 */
GLOBAL _VOID Printer_FreeCFGFile(_VOID)
{
	if (CFG != NULL)
	{
		/* noch ein Printertreiber eingehaengt; ganz frech einfach freigeben: */
		SFREE(CFG);
		CFG = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Eroeffnung einer CFG-Datei. Der komplette Dateiname muss uebergeben werden.
 * Falls keine Eroeffnung moeglich war, wird 'Response' auf 'CFGnotFound' gesetzt.
 * Die Datei wird nach Eroeffnung auf die Dateikennung in den ersten 8 Bytes
 * ueberprueft ('GST-CFG:'). Falls die Kennung nicht uebereinstimmt, wird
 * die Datei wieder geschlossen und 'Response' auf 'wrongIdent' gesetzt.
 * Im Erfolgsfall enthaelt 'Response' dann 'done'.
 */
GLOBAL tResponse Printer_LoadCFGFile(CONST _UBYTE *name)
{
	FILENO f;
	_ULONG Length;
	_UWORD i, j, Pos, Len;

	f = (FILENO)Fopen(name, 0);
	if (f < 0)
	{
		Printer_Error(Printer_CFGnotFound);
		return Printer_CFGnotFound;
	}
	
	Printer_FreeCFGFile();
	
	Length = Fseek(0L, f, SEEK_END);
	
	CFG = MALLOC(Length + 1, "Printer_LoadCFGFile");
	if (CFG == NULL)
	{
		Printer_Error(Printer_noMemory);
		(void)Fclose(f);
		return Printer_noMemory;
	}
	InitTables();
	
	Fseek(0L, f, SEEK_SET);
	Fread(f, Length, CFG);
	(void)Fclose(f);
	CFG[Length] = '\0';
	
	/* Kennung ueberpruefen: */
	if (Length < 8 || strnCmp(CFG, "GST-CFG:", 8) != 0)
	{
		Printer_Error(Printer_wrongIdent); /* Keine GST-CFG-Datei !!! */
		Printer_FreeCFGFile();
		return Printer_wrongIdent;
	}
	/* Jetzt muessen die Tabellen aufgebaut werden: */
	i = 8; /* Lesezeiger auf den Beginn des Druckernamens setzen:  */
	while (i < Length && CFG[i] != '\0')	/* Ende des Druckernamens gefunden. */
		++i;
	
	i += 7; /* Druckeranpassungen interessieren nicht.  */
	
	/* zunaechst die Tabelle der Druckerbesonderheiten:  */
	while (i < Length && CFG[i] != '\0')  /* Tabelle wird mit einem NUL-Byte beendet */
	{
		Len = CFG[i];
		Pos = CFG[i+1];  /* Tabelleneintrag */
		if (Len >= 2)
			SeqTable[Pos].len  = Len - 2;
		else
			SeqTable[Pos].len = 0;
		SeqTable[Pos].pos  = i + 2;
		i += Len;
	}
	
	++i; /* Tabellenende ueberspringen */
	/* jetzt die UebersetzungsTabelle:  */
	while (i < Length && CFG[i] != '\0') /* Tabelle wird mit einem NUL-Byte beendet */
	{
		Len = CFG[i];
		Pos = CFG[i+1];  /* Tabelleneintrag */
		if (Pos < cMaxTranslations)
		{
			if (Len >= 2)
				TransTable[Pos].len  = Len - 2;
			else
				TransTable[Pos].len = 0;
			TransTable[Pos].pos = i + 2;
		} else
		{
			/* ??? Warnung ausgeben? */
		}
		i += Len;
	}
	
	/* Optimierung der Druckercharakteristik-Tabelle: */
	/* 1. die Texteffekte: */
	for (i = cBold; i <= cUnderline; i += 4)
	{
		for (j = i; j <= i+1; j++)
		{
			if (SeqTable[j].len == 0 && SeqTable[j+2].len != 0)
				SeqTable[j] = SeqTable[j+2];
			if (SeqTable[j+2].len == 0 && SeqTable[j].len != 0)
				SeqTable[j+2] = SeqTable[j];
		}
	}
	/* 2. die Schriftarten: */
	for (i = c1stType; i < c1stColor; i += 2)
	{
		if (SeqTable[i].len == 0 && SeqTable[i+1].len != 0)
			SeqTable[i] = SeqTable[i+1];
		if (SeqTable[i+1].len == 0 && SeqTable[i].len != 0)
			SeqTable[i+1] = SeqTable[i];
	}
	return Printer_done;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * erfragt eine CFG-Datei per FileSelectBox.
 * InfoText ist fuer die FileSelectBox gedacht.
 * Die Endung wird fest auf '*.CFG' festgeschrieben.
 */
GLOBAL tResponse Printer_QueryCFGFile(_UBYTE *path, _UBYTE *name, CONST _UBYTE *InfoText)
{
	_UBYTE fullname[PATH_MAX];
	
	if (mtDir_FselGet(path, name, "*.cfg", InfoText, FALSE))
	{
		strBcpy(fullname, path);
		F_Path_Append(fullname, name);
		return Printer_LoadCFGFile(fullname);
	}
	return Printer_notdone;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Eingetragenen Druckernamen liefern. Falls keiner vorhanden ist, dann
 * wird ein Leerstring ('') geliefert.
 */
GLOBAL _VOID Printer_GetPrnName(_UBYTE *Printername, size_t strsize)
{
	size_t i;
	
	if (strsize == 0)
		return; /* mit sowas fangen wird gar nicht erst an */
	--strsize;
	if (CFG != NULL)
	{
		i = 8;
		while (CFG[i] != '\0' && strsize)
		{
			*Printername++ = CFG[i++];
			strsize--;
		}
	}
	*Printername = '\0';
}

/*** ---------------------------------------------------------------------- ***/

LOCAL tResponse PrintSeq(_WORD Which)
{
	_WORD i, end;
	tResponse Response;

	end = SeqTable[Which].pos + SeqTable[Which].len;
	for (i = SeqTable[Which].pos; i < end; i++)
	{
		if ((Response = Out(CFG[i])) != Printer_done)
			return Response;
	}
	return Printer_done;
}

/*** ---------------------------------------------------------------------- ***/

/*  Ausgabe der angewaehlten Steuersequenz. */
LOCAL tResponse Print(_WORD Entry, _BOOL Set)
{
	if (!Set)
		++Entry;
	return PrintSeq(Entry + NLQSet);
}

/*** ---------------------------------------------------------------------- ***/

/* Drucker zuruecksetzen */
GLOBAL tResponse Printer_SendExit(_VOID)
{
	return PrintSeq(cPrinterInit);
}

/*** ---------------------------------------------------------------------- ***/

/* Drucker initialisieren */
GLOBAL tResponse Printer_SendInit(_BOOL NLQ, _BOOL hAndVInit)
{
	tResponse Response;
	
	NLQSet = 0;
	if (NLQ)
		NLQSet = 2;
	if (hAndVInit)
	{
		/* Horizontale Initialisierung */
		if ((Response = PrintSeq(cHorzInit)) != Printer_done)
			return Response;
		/* Vertikale Initialisierung   */
		if ((Response = PrintSeq(cVertInit)) != Printer_done)
			return Response;
	}
	/* Fettschrift aus.            */
	if ((Response = Print(cBold, FALSE)) != Printer_done)
		return Response;
	/* Kursivschrift aus.          */
	if ((Response = Print(cItalic, FALSE)) != Printer_done)
		return Response;
	/* Light aus.                  */
	if ((Response = Print(cLight, FALSE)) != Printer_done)
		return Response;
	/* Superscript aus.            */
	if ((Response = Print(cSuperscript, FALSE)) != Printer_done)
		return Response;
	/* Subscript aus.              */
	if ((Response = Print(cSubscript, FALSE)) != Printer_done)
		return Response;
	/* Unterstreichung aus.        */
	if ((Response = Print(cUnderline, FALSE)) != Printer_done)
		return Response;
	
	AktMode  = -1;
	/* PICA (10 CPI)               */
	if ((Response = PrintSeq(c1stType)) != Printer_done)
		return Response;

	AktColor = -1;
	/* Color-Einstellung: Black.   */
	if ((Response = PrintSeq(c1stColor)) != Printer_done)
		return Response;

	Proportional = FALSE;
	/* Proportional-Schrift aus.   */
	Response = PrintSeq(cProp + 1);
	return Response;
}

/*** ---------------------------------------------------------------------- ***/

/* Fettdruck ein/ausschalten          */
GLOBAL tResponse Printer_Bold(_BOOL Set)
{
	return Print(cBold, Set);
}

/*** ---------------------------------------------------------------------- ***/

/* Schraegschrift ein/ausschalten      */
GLOBAL tResponse Printer_Italic(_BOOL Set)
{
	return Print(cItalic, Set);
}

/*** ---------------------------------------------------------------------- ***/

/* Helle Schrift ein/ausschalten      */
GLOBAL tResponse Printer_Light(_BOOL Set)
{
	return Print(cLight, Set);
}

/*** ---------------------------------------------------------------------- ***/

/* Hochstellen ein/ausschalten        */
GLOBAL tResponse Printer_Superscript(_BOOL Set)
{
	return Print(cSuperscript, Set);
}

/*** ---------------------------------------------------------------------- ***/

/* Tiefstellen ein/ausschalten        */
GLOBAL tResponse Printer_Subscript(_BOOL Set)
{
	return Print(cSubscript, Set);
}

/*** ---------------------------------------------------------------------- ***/

/* Unterstreichen ein/ausschalten     */
GLOBAL tResponse Printer_Underline(_BOOL Set)
{
	return Print(cUnderline, Set);
}

/*** ---------------------------------------------------------------------- ***/

/* Texteffekte setzen/ruecksetzen */
GLOBAL tResponse Printer_Effect(_UWORD Effects, _BOOL Set)
{
	tResponse Response;
	
	if (Effects & cFat)
		if ((Response = Printer_Bold(Set)) != Printer_done)
			return Response;
	if (Effects & cLightened)
		if ((Response = Printer_Light(Set)) != Printer_done)
			return Response;
	if (Effects & cItalic)
		if ((Response = Printer_Italic(Set)) != Printer_done)
			return Response;
	if (Effects & cUnderlined)
		if ((Response = Printer_Underline(Set)) != Printer_done)
			return Response;
	return Printer_done;
}

/*** ---------------------------------------------------------------------- ***/

/*  Seitenvorschub ausgeben */
tResponse Printer_FormFeed(_VOID)
{
	CharPos = 0;
	if (CFG != NULL)
		return PrintSeq(cFormFeed);
	return Out('\f');
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Printer_SetTabSize(_WORD Tab)
{
	TabSize = Tab;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Ein/Ausschalten der Proportionalschrift sowie Setzen einer mittleren
 * Zeichenbreite (fuer Tabulierung notwendig)
 */
GLOBAL tResponse Printer_SetPropSize(_BOOL Prop, _WORD PSize)
{
	_WORD nlq;
	tResponse Response;
	
	nlq = NLQSet;
	NLQSet = 0;
	Response = Print(cProp, Prop);
	NLQSet = nlq;
	PropSize = PSize;
	Proportional = Prop;
	return Response;
}

/*** ---------------------------------------------------------------------- ***/

/*  Zeilenvorschub ausgeben */
GLOBAL tResponse Printer_WriteLn(_VOID)
{
	tResponse Response;
	
	CharPos = 0;
	if (CFG != NULL)
		return PrintSeq(cWriteLn);
	if ((Response = Out(0x0d)) == Printer_done)
		Response = Out(0x0a);
	return Response;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Zeichenbreite einstellen.
 * ACHTUNG: vorher Druckmodus 'DRAFT' oder 'NLQ' einstellen!
 */
GLOBAL tResponse Printer_SetMode(_WORD Mode)
{
	if (Mode >= cPica && Mode <= cExpanded)
	{
		if (Mode != AktMode)
		{
			AktMode = Mode;
			return PrintSeq(2 * Mode + c1stType + (NLQSet / 2));
		}
	}
	return Printer_done;
}

/*** ---------------------------------------------------------------------- ***/

/*  Farbe einstellen. */
GLOBAL tResponse Printer_SetColor(_WORD Color)
{
	if (Color >= cBlack && Color <= cYellow)
	{
		if (Color != AktColor)
		{
			AktColor = Color;
			return PrintSeq(Color + c1stColor);
		}
	}
	return Printer_done;
}

/*** ---------------------------------------------------------------------- ***/

/*  Druckkopf neu positionieren */
LOCAL tResponse SetHead(_VOID)
{
	_WORD i, end;
	tResponse Response;
	
	end = SeqTable[cVertPos].pos + SeqTable[cVertPos].len;
	Response = Printer_done;
	for (i = SeqTable[cVertPos].pos; Response == Printer_done && i < end; i++)
	{
		if (CFG[i] == 0x80)
			Response = Out((CharPos * 60 / PropSize) & 0xff);
		else if (CFG[i] == 0x81)
			Response = Out((CharPos * 60 / PropSize) / 256);
		else
			Response = Out(CFG[i]);
	}
	NrSpaces = 0;
	return Response;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL tResponse WriteChar(_UBYTE ch)
{
	_WORD i, end;
	tResponse Response;
	
	if (TransTable[ch].len > 0)
	{
		end = TransTable[ch].pos + TransTable[ch].len;
		for (i = TransTable[ch].pos; i < end; i++)
		{
			if ((Response = Out(CFG[i])) != Printer_done)
				return Response;
			++CharPos;
		}
	} else
	{
		/* Zeichen ohne Uebersetzung ausgeben. */
		if ((Response = Out(ch)) != Printer_done)
			return Response;
		++CharPos;
	}
	return Printer_done;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Zentrale Prozedur fuer die Uebersetzung der einzelnen Zeichen in eine
 * Zeichenfolge.
 */
GLOBAL tResponse Printer_Write(_UBYTE ch)
{
	_WORD i;
	tResponse Response;
	
	Response = Printer_done;
	if (ch == cTab || ch == ' ')
	{
		if (Proportional)
		{
			if (ch == ' ')
			{
				++CharPos;
				++NrSpaces;
			} else
			{
				CharPos += TabSize;
				NrSpaces += TabSize;
			}
		} else
		{
			if (ch == ' ')
			{
				Response = WriteChar(' ');
			} else
			{
				for (i = 0; i < TabSize; i++)
				{
					if ((Response = WriteChar(' ')) != Printer_done)
						return Response;
				}
			}
		}
	} else
	{
		if (NrSpaces > 1)
		{
			Response = SetHead();
		} else if (NrSpaces == 1)
		{
			Response = WriteChar(' ');
			NrSpaces = 0;
		}
		if (Response == Printer_done)
			Response = WriteChar(ch);
	}
	return Response;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL tResponse Printer_WriteString(CONST _UBYTE *Str)
{
	tResponse Response;
	
	while (*Str != '\0')
	{
		if ((Response = Printer_Write(*Str)) != Printer_done)
			return Response;
		Str++;
	}
	return Printer_done;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Eroeffnung eines Druckerkanals; wenn ein Fehler auftritt, wird 'Response'
 * auf 'ErrorInStreams' gestellt; die eigentliche Fehlerursache ist dann bei
 * 'Streams.Response' zu suchen.
 */
GLOBAL tResponse Printer_Open(CONST _UBYTE *Name)
{
	_WORD Device;
	_LONG handle;
	
	/* Device feststellen */
	Device = 0;
	if (strCaseCmp(Name, cCon) == 0)
		Device = 1;
	else if (strCaseCmp(Name, cAux) == 0)
		Device = 2;
	else if (strCaseCmp(Name, cPrn) == 0)
		Device = 3;

	if (Device == 0)     /* Wird wohl ein File sein */
		handle = Fcreate(Name, 0);
	else
		handle = Fopen(Name, 1);
	if (handle < 0)
	{
		Printer_Error(Printer_ErrorInStreams);
		return Printer_ErrorInStreams;
	}
	PrnHdl = (FILENO)handle;
	if (PrnHdl < 0 && PrnHdl > -6)
	{
		/* Dann war's wohl doch ein GEMDOS-Device */
		Device = -PrnHdl;
	}
	currentOut = Device;
	return Printer_done;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Schliessung eines Druckerkanals
 */
GLOBAL _VOID Printer_Close(_VOID)
{
	(void)Fclose(PrnHdl);
}
