/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <uudecode.h>
#include <protokol.h>
#include <mttextfi.h>
#include <mtdir.h>
#include <mte.h>

#include <routinec.h>
#include <routines.h>
#include <file_io.h>
#include <mtrsc.h>

#define TEXTFILES 1

typedef enum { noErr, lineToShort, illCharInLine } decodeErrType;

#define offset ' '

LOCAL _UBYTE decodePath[PATH_MAX];
LOCAL _UBYTE lastLine[80];
LOCAL _BOOL checkLastLine;
#if TEXTFILES
LOCAL TEXTFILENO decodeFile;
#else
LOCAL FILENO decodeFile;
#endif;
LOCAL decodeErrType decodeErr;

typedef struct {
	CONST _UBYTE *line;
	_WORD lineIndex;
	_WORD len;
	_WORD byteNum;
	_UBYTE hunk[3];
} decodeInfo;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Initialisiert die Dekodierung
 */
GLOBAL _VOID UUDecode_BeginDecode(_VOID)
{
#if TEXTFILES
	decodeFile = NULL;
#else
	decodeFile = -1;
#endif
	decodeErr = noErr;
	checkLastLine = FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE nextch(decodeInfo *info)
{
	_UBYTE ch;
	
	if (info->lineIndex >= info->len)
		return '\0'; /* abort('Line too short.'); */
	ch = info->line[info->lineIndex];
	if (ch == '\0')
		return '\0';
	++info->lineIndex;
	if (ch < ' ' || ch > '`')
	{
		decodeErr = illCharInLine;
		return '\0'; /* abort('Illegal character in line.'); */
	}
	if (ch == '`')
		return '\0';
	return ch - offset;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID GetNextHunk(decodeInfo *info)
{
	_UBYTE chars[4];
	
	chars[0] = nextch(info);
	chars[1] = nextch(info);
	chars[2] = nextch(info);
	chars[3] = nextch(info);
	info->hunk[0] = (chars[0] << 2) + (chars[1] >> 4);
	info->hunk[1] = (chars[1] << 4) + (chars[2] >> 2);
	info->hunk[2] = (chars[2] << 6) + chars[3];
	info->byteNum = 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID DecodeByte(decodeInfo *info)
{
	if (info->byteNum == 3)
	{
		GetNextHunk(info);
		if (decodeErr != noErr)
			return;
	}
#if TEXTFILES
	mtTextfiles_WriteChar(decodeFile, info->hunk[info->byteNum]);
#else
	CatFiles_WriteFile(info->hunk[info->byteNum], decodeFile);
#endif
	++info->byteNum;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL close_file(_VOID)
{
	/* Ende eines Files */
#if TEXTFILES
	if (decodeFile != NULL)
	{
		mtTextfiles_CloseTextfile(&decodeFile);
		Protokoll_SendPathUpdate(decodePath);
		return TRUE;
	}
#else
	if (decodeFile > 0)
	{
		CatFiles_CloseFile(decodeFile);
		decodeFile = -1;
		return TRUE;
	}
#endif
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Dekodiert eine Zeile
 */
/*DYJ 5.21b */
LOCAL F_ERROR iDecodeLine(CONST _UBYTE *line, _BOOL validCheck)
{
	decodeInfo info;
	_WORD count;
	_WORD calcLen;
	_BOOL processLine;
	_UBYTE fname[PATH_MAX];
	CONST _UBYTE *p;
	F_ERROR err;
	
	info.line = line;
	info.len = (_WORD)strLen(line);
	/* Erstmal feststellen, was fuer eine Zeile es ist */
	if (strnCmp(line, "begin", 5) == 0)
	{
		/* Anfang eines Files */
		/* Haben wir noch eines offen? Das muessen wir dann zumachen! */
		if (close_file())
		{
			/* CheckLastLine muessen wir hier nicht machen,
			 * vor einem begin kann keine gueltige Zeile eines
			 * anderen UUs kommen
			 */
			checkLastLine = FALSE;
		}
		/* Anfang eines Files */
		/* Dateinamen Suchen */
		p = line + 5;
		while (*p == ' ')
			++p;
		p = strChr(p, ' ');
		if (p == NULL)
			return -1;
		while (*p == ' ')
			++p;
		strBcpy(fname, p);
		/* Datei zur Auswahl anbieten */
		if (mtDir_FselGet(decodePath, fname, ALL_FILE_MASK, Rsc_WrkString(FS_DECODE_TO), FALSE))
		{
			/* Datei erzeugen */
			_UBYTE path[PATH_MAX];
			
			strBcpy(path, decodePath);
			F_Path_Append(path, fname);
			
#if TEXTFILES
			if ((err = mtTextfiles_OpenTextfile(path, WRITE, 0, &decodeFile)) != GERR_OK)
				return err;
#else
			decodeFile = CatFiles_CreateFile(path);
			if (!FILENO_OK(decodeFile))
				return TO_HANDLE(decodeFile);
#endif
		}
	} else if (strnCmp(line, "end", 3) == 0)
	{
		/* Ende eines Files */
#if TEXTFILES
		if (decodeFile != NULL)
#else
		if (FILENO_OK(decodeFile))
#endif
		{
			if (checkLastLine)
			{
				checkLastLine = FALSE;
				if ((err = iDecodeLine(lastLine, FALSE)) != GERR_OK)
				{
					return err;
				}
			}
			close_file();
		}
	} else
	{
#if TEXTFILES
		if (decodeFile != NULL)
#else
		if (FILENO_OK(decodeFile))
#endif
		{
			/* Moeglicherweise eine UU-Zeile */
			info.lineIndex = 0;
			info.byteNum = 3;
			count = nextch(&info);
			calcLen = (count / 3) * 4;
			if (validCheck && calcLen == info.len)
			{
				/* Ein Byte fehlt am Ende, Laenge auffuellen, die MAUS hat
				 * ein Leerzeichen abgebissen
				 */
				++info.len;
			}
			processLine = !validCheck || (calcLen == info.len - 1) || (calcLen == info.len - 2);
			if (info.len == 1)
			{
				processLine = FALSE;
			} else if (!processLine)
			{
				/* Zeile noch genauer untersuchen */
				if (strnCmp(line, "-----", 5) != 0)      /* definitiv ein Trenner, wenn identisch */
				{
					/* also kein Trenner, mal optimistisch sein */
					processLine = (info.len <= 64) && /* Wir nehmen in dem Fall mal an, dass es eine UU-Zeile ist */
						(line[0] >= ' ' && line[0] <= 'R');
				}
			}
			if (processLine)
			{
				if (checkLastLine)
				{
					checkLastLine = FALSE;
					if ((err = iDecodeLine(lastLine, FALSE)) != GERR_OK)
					{
						return err;
					}
				}
				/* wohl UU-Zeile, verarbeiten */
				while (count > 0)
				{
					DecodeByte(&info);
					if (decodeErr != noErr)
						return -1;
					count--;
				}
			} else if ((count / 3) * 4 > info.len && info.len < 80 && info.len > 1)
			{
				/* moeglicherweise Leerzeichen abgeschnitten */
				strBcpy(lastLine, line);
				checkLastLine = TRUE;
			}
		}
	}
	return GERR_OK;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL F_ERROR UUDecode_DecodeLine(_UBYTE *line)
{
	return iDecodeLine(line, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Beendet die Dekodierung, schliesst ggf. offene Dateien
 */
/*DYJ 5.21b */
GLOBAL _VOID UUDecode_EndDecode(_VOID)
{
	if(close_file())
		MTE_info(Rsc_WrkString(AL_NO_END));

}
