/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <seproto.h>
#include <protokol.h>

#include <portaes.h>

#include <ro_mem.h>
#include <routinec.h>
#include <debug.h>

#define SE_INIT      0x4200
    /* Die Shell fragt an, ob ein Editor das SE-Protokoll
     * versteht.
     * Wort 3: Ein Bitset, welche Nachrichten die Shell versendet
     * Wort 4+5: Ein Bitset, welche Editorkommandos verstanden werden
     * Wort 6: Unterstuetzte Versionsnummer des Protokolls
     * Als Antwort erhaelt es ein ES_OK vom  Editor.
     */

#define SE_OK        0x4201
    /* Die Shell sagt dem Editor, das sie das Protokoll
     * versteht.
     * Wort 3: Ein Bitset, welche Nachrichten die Shell versendet
     * Wort 4+5: Ein Bitset, welche Editorkommandos verstanden werden
     * Wort 6: Unterstuetzte Versionsnummer des Protokolls
     * Wort 7: Die ApId des Programmes, dessen Nachricht beantwortet wird
     */

#define SE_ACK       0x4202
    /* Die Shell bestaetigt den Empfang eines Editorkommandos und gibt zurueck,
     * ob das Kommando ausgefuehrt wird.
     * Wort 3: TRUE: Kommando wird verstanden und ausgefuehrt.
     *         FALSE: Das Kommando wird nicht verstanden
     * Eine SE_ACK mit TRUE sagt nichts darueber aus, ob das Kommando erfolg-
     * reich ausgefuehrt wurde. Es sagt nur, dass die Shell das Kommando
     * versteht und ausfuehren wird!
     */

#define SE_OPEN      0x4203
    /* Die Shell sagt dem Editor, dass er einen Text oeffnen
     * soll. Als Antwort erhaelt die Shell ein ES_ACK
     * Wort 3+4: Ein Zeiger auf den Filenamen des zu oeffnenden Files
     * Wort 5+6: Cursorposition (Zeilennummer)
     * Wort 7  : Cursorposition (Spaltennummer)
     */

#define SE_ERROR     0x4204
    /* Es ist ein Fehler beim Compilieren aufgetreten.
     * Wort 3+4: Ein Zeiger auf eine Infostruktur, die wie folgt
     *           aufgebaut ist:
     *              Wort 0+1: Ein Zeiger auf den Namen des compilierten Textes.
     *              Wort 2+3: Ein Zeiger auf den Text der Fehlermeldung
     *              Wort 4  : Die Fehlernummer
     *              Wort 5+6: Die fehlerhafte Zeile
     *              Wort 7  : Die Spalte mit dem Fehler (oder 0)
     * Mit ES_ACK bestaetigt der Editor die Meldung.
     */

#define SE_ERRFILE   0x4205
    /* Es sind Fehler aufgetreten. Die Fehlermeldungen stehen in einem
     * Errorfile, welches in der Message spezifiziert wird.
     * Wort 3+4: Ein Zeiger auf den Filenamen des Errorfiles mit den
     *           Fehlermeldungen
     * Wort 5+6: Ein Zeiger auf den Namen des compilierten Textes
     * Mit ES_ACK bestaetigt der Editor die Meldung.
     */

#define SE_PROJECT   0x4206
    /* Die Shell teilt dem Editor mit, dass das Projekt
     * geaendert wurde. Der Filename des aktuellen
     * Projektfiles wird als Parameter uebergeben
     * Wort 3+4: Ein Zeiger auf den Namen des Projektfiles
     * Der Editor bestaetigt mit ES_ACK.
     * Eine vernuenftige Reaktion des Editors waere es in dem Fall,
     * ebenfalls das Projekt zu wechseln, sofern er diese unterstuetzt.
     */

#define SE_QUIT      0x4207
    /* Die Shell teilt dem Editor mit, dass sie jetzt beendet wird.
     * Der Editor sollte in dem Fall die Shell als Kommunikationspartner
     * vergessen.
     * Parameter gibt es keine.
     * Es wird keine Antwort erwartet!
     */

#define SE_TERMINATE 0x4208
    /* Die Shell teilt dem Editor mit, dass dieser sich beenden soll.
     * Der Editor sollte in dem Fall sich selbst beenden und seinen ganz
     * normalen Terminierungsprozess durchlaufen (und in dem auch ein ES_QUIT
     * schicken!). Der Grund fuer so eine Meldung der Shell koennte zum Beispiel
     * zu wenig Speicher zum compilieren sein.
     * Mit ES_ACK bestaetigt der Editor die Meldung.
     */

#define SE_CLOSE     0x4209
    /* Die Shell teilt dem Editor mit, dass dieser alle Texte sichern soll.
     * Der Editor sollte bei geaenderten Texten vorher nachfragen.
     * Wort 3+4: Ein Zeiger auf den Namen einer Datei oder eine Dateimaske.
     *           '*.*' steht fuer alle Textfenster (entspricht also dem SE_CLOSE
     *           der Protokoll-Version 0x100).
     * Wort 5  : 0 = nur sichern
     *           1 = sichern und schliessen
     *           2 = schliessen ohne sichern
     * Mit ES_ACK bestaetigt der Editor die Meldung.
     */

#define SE_MENU      0x4210
    /* Die Shell teilte dem Editor mit, was er fuer die einzelnen Programme
     * in sein Menue eintragen soll.
     * Ein Element darf auch NULL sein.
     * Word 3+4: Ein Zeiger auf die Stuktur mit den Eintraegen.
     * Mit ES_ACK bestaetigt der Editor die Meldung.
     */

#define ES_INIT      0x4240
    /* Ein Editor fragt an, ob eine Shell das SE-Protokoll versteht.
     * Wort 3: Ein Bitset, welche Shellnachrichten verstanden werden
     * Wort 4+5: Ein Bitset, welche Editorkommandos versendet werden
     * Wort 6: Unterstuetzte Versionsnummer des Protokolls
     * Als Antwort erhaelt er SE_OK von der Shell
     */

#define ES_OK        0x4241
    /* Der Editor beantwortet die Anfrage der Shell nach dem Protokoll.
     * Wort 3: Ein Bitset, welche Shellnachrichten verstanden werden
     * Wort 4+5: Ein Bitset, welche Editorkommandos versendet werden
     * Wort 6: Unterstuetzte Versionsnummer des Protokolls
     * Wort 7: Die ApId des Programmes, dessen Nachricht beantwortet wird
     */

#define ES_ACK       0x4242
    /* Der Editor bestaetigt den Empfang des Kommandos
     * Wort 3: TRUE: Kommando wird verstanden und ausgefuehrt.
     *         FALSE: Das Kommando wird nicht verstanden
     * Eine ES_ACK mit TRUE sagt nichts darueber aus, ob das Kommando erfolg-
     * reich ausgefuehrt wurde. Es sagt nur, dass der Editor das Kommando
     * versteht und ausfuehren wird!
     */

#define ES_COMPILE   0x4243
    /* Der Editor sagt der Shell, dass sie ein File uebersetzen soll.
     * Ein Pointer auf den Dateinamen wird in der Message uebergeben.
     * Wort 3+4: Zeiger auf den Namen der zu compilierenden Datei.
     * Diese Nachricht muss mit SE_ACK bestaetigt werden.
     */

#define ES_MAKE      0x4244
    /* Der Editor sagt der Shell, dass sie ein Make ausfuehren soll.
     * Ein Filename kann in der Message uebergeben werden, muss aber
     * nicht gesetzt sein und muss von der Shell auch nicht beachtet
     * werden!
     * Wort 3+4: Zeiger auf den Namen des Makefiles (oder NULL)
     * Die Shell bestaetigt mit SE_ACK.
     */

#define ES_MAKEALL   0x4245
    /* Der Editor sagt der Shell, dass ein komplettes Make All ausgefuehrt
     * werden soll. Ein Filename fuer das Makefile kann (muss nicht) in
     * der Message uebergeben werden.
     * Wort 3+4: Zeiger auf den Namen des Makefiles (oder NULL)
     * Die Shell bestaetigt mit SE_ACK.
     */

#define ES_LINK      0x4246
    /* Der Editor sagt der Shell, dass das Programm
     * gelinkt werden soll. Ein Filename kann
     * in der Message uebergeben werden, muss aber
     * nicht unbedingt beachtet werden von der Shell!
     * Wort 3+4: Zeiger auf den Namen der Source, die gelinkt werden soll
     *           (oder NULL)
     * Die Shell bestaetigt mit SE_ACK.
     */

#define ES_EXEC      0x4247
    /* Der Editor sagt der Shell, dass das Programm
     * zu der Source ausgefuehrt werden soll. Ein
     * Filename kann uebergeben werden, muss von der
     * Shell aber nicht beachtet werden.
     * Wort 3+4: Zeiger auf den Namen des auszufuehrenden Files (oder NULL)
     *           Bei einer Sourcedatei ist diese ggf. noch zu
     *           compilieren und/oder zu linken
     * Die Shell bestaetigt mit SE_ACK.
     */

#define ES_MAKEEXEC  0x4248
    /* Die Shell soll ein Make ausfuehren und danach das Programm ausfuehren.
     * Ein Filename fuer das Makefile kann (muss nicht) in der Message
     * uebergeben werden.
     * Wort 3+4: Zeiger auf den Namen des Makefiles (oder NULL)
     * Die Shell bestaetigt mit SE_ACK.
     */

#define ES_PROJECT   0x4249
    /* Der Editor teilt der Shell mit, dass das Project
     * geaendert/gewechselt wurde. Der Filename des
     * Projektfiles wird als Parameter in der Message
     * uebergeben.
     * Wort 3+4: Zeiger auf den Namen des Projektfiles
     * Die Shell bestaetigt mit SE_ACK.
     * Eine vernuenftige Reaktion der Shell waere in dem Fall, ebenfalls
     * das Projekt zu wechseln, sofern es diese unterstuetzt.
     */

#define ES_QUIT      0x424A
    /* Der Editor teilt der Shell mit, dass er jetzt beendet wird.
     * Die Shell sollte in dem Fall den Editor als Kommunikationspartner
     * vergessen.
     * Parameter gibt es keine.
     * Es wird keine Antwort erwartet!
     */

#define ES_SHLCTRL   0x424B
    /*
     * Sonstige Kontrolle der Shell durch den Editor.
     * Wort 3+4    Zeiger auf den Namen des obersten Fensters oder NULL.
     * Wort 5      Steuer-Flag:
     *                0: Shell soll sich toppen (Menue/Fenster).
     * Die Shell bestaetigt mit SE_ACK.
     */

#define SEPROTOVERSION  0x105 /* Die Versionsnummer des Protokolls.
                               * Highbyte ist die Major Number
                               * Lowbyte ist die Lower Number
                               */

/* Typdefinitionen fuer die Nachrichten
 */

/* Dies definiert die einzelnen Bits, welche Nachrichten unterstuetzt werden
 */
#define _SEINIT      0x0001
#define _SEOK        0x0002
#define _SEACK       0x0004
#define _SEOPEN      0x0008
#define _SEERROR     0x0010
#define _SEERRFILE   0x0020
#define _SEPROJECT   0x0040
#define _SEQUIT      0x0080
#define _SETERMINATE 0x0100
#define _SECLOSE     0x0200
#define _SEMENU      0x0400

#define _ESINIT      0x0001UL
#define _ESOK        0x0002UL
#define _ESACK       0x0004UL
#define _ESCOMPILE   0x0008UL
#define _ESMAKE      0x0010UL
#define _ESMAKEALL   0x0020UL
#define _ESLINK      0x0040UL
#define _ESEXEC      0x0080UL
#define _ESMAKEEXEC  0x0100UL
#define _ESPROJECT   0x0200UL
#define _ESQUIT      0x0400UL
#define _ESSHLCTRL   0x0800UL
/* Bits 12-31 reserved */

typedef _UWORD tShellSet;
typedef _ULONG tEditSet;


        /* Ueber diese Recorddefinition kann man alle
         * Messages des Protokolls behandeln
         */

typedef struct _messRec {
	_WORD msg;						/* Messagenummer, Wort 0 */
	_WORD apId;						/* Applikation ID des Senders, Wort 1 */
	_WORD overLen;					/* Ueberlaenge der Nachricht (immer 0 fuer SE-Protokoll), Wort 2 */
	union {
		struct {
			tShellSet shellBits;	/* Bits, die angeben, welche Shell-Nachrichten unterstuetzt werden (Wort 3) */
			tEditSet editBits;		/* Bits, die angeben, welche Editor-Nachrichten unterstuetzt werden (Worte 4 und 5) */
			_UWORD version;			/* Unterstuetzte Versionsnummer des Protokoll. Aktuell: $100 */
		} se_init;
		struct {
			tShellSet shellBits;	/* Bits, die angeben, welche Shell-Nachrichten unterstuetzt werden (Wort 3) */
			tEditSet editBits;		/* Bits, die angeben, welche Editor-Nachrichten unterstuetzt werden (Worte 4 und 5) */
			_UWORD version;			/* Unterstuetzte Versionsnummer des Protokoll. Aktuell: $100 */
		} es_init;
		struct {
			tShellSet okShellBits;	/* Bits, die angeben, welche Shell-Nachrichten unterstuetzt werden (Wort 3) */
			tEditSet okEditBits;	/* Bits, die angeben, welche Editor-Nachrichten unterstuetzt werden (Worte 4 und 5) */
			_UWORD okVersion;		/* Unterstuetzte Versionsnummer des Antwortenden. Aktuell: $100 */
			_WORD otherId;			/* Die ApId des Senders, auf das dies eine Antwort ist (Wort 7) */
		} se_ok;
		struct {
			tShellSet okShellBits;	/* Bits, die angeben, welche Shell-Nachrichten unterstuetzt werden (Wort 3) */
			tEditSet okEditBits;	/* Bits, die angeben, welche Editor-Nachrichten unterstuetzt werden (Worte 4 und 5) */
			_UWORD okVersion;		/* Unterstuetzte Versionsnummer des Antwortenden. Aktuell: $100 */
			_WORD otherId;			/* Die ApId des Senders, auf das dies eine Antwort ist (Wort 7) */
		} es_ok;
		struct {
			_BOOL acknowledge;		/* TRUE: Kommando wurde akzeptiert, FALSE: Kommando konnte nicht ausgefuehrt werden */
		} se_ack;
		struct {
			_BOOL acknowledge;		/* TRUE: Kommando wurde akzeptiert, FALSE: Kommando konnte nicht ausgefuehrt werden */
		} es_ack;
		struct {
			_UBYTE *namePtr;		/* Zeiger auf den Namen des zu ladenden Textes (Worte 3 und 4) */
			_LONG iLineNum;			/* Worte 5 und 6: Zeilennummer mit Position zum Oeffnen */
			_WORD iColNum;			/* Wort 7: Spaltennummer */
		} se_open;
		struct {
			_UBYTE *namePtr;		/* Zeiger auf den Namen der zu schliessenden Fenster (Worte 3 und 4) */
			_WORD control;			/* Wort 5 */
		} se_close;
		struct {
			SEERRINFO *errorPtr;	/* Zeiger auf Speicherbereich, der die Fehlermeldung spezifiziert. */
		} se_error;
		struct {
			SEMENUINFO *menuInfoPtr;/* Zeiger auf Speicherbereich mit Menuinformationen */
		} se_menu;
		struct {
			_UBYTE *errPtr;			/* Zeiger auf den Namen des Errorfiles */
			_UBYTE *fnamePtr;		/* Zeiger auf den Namen der compilierten Datei */
		} se_errfile;
		struct {
			_UBYTE *filePtr;		/* Zeiger auf den Filenamen des Files, welches behandelt werden soll */
		} se_project;
		struct {
			_UBYTE *filePtr;		/* Zeiger auf den Filenamen des Files, welches behandelt werden soll */
		} es_compile;
		struct {
			_UBYTE *filePtr;		/* Zeiger auf den Filenamen des Files, welches behandelt werden soll */
		} es_make;
		struct {
			_UBYTE *filePtr;		/* Zeiger auf den Filenamen des Files, welches behandelt werden soll */
		} es_makeall;
		struct {
			_UBYTE *filePtr;		/* Zeiger auf den Filenamen des Files, welches behandelt werden soll */
		} es_link;
		struct {
			_UBYTE *filePtr;		/* Zeiger auf den Filenamen des Files, welches behandelt werden soll */
		} es_exec;
		struct {
			_UBYTE *filePtr;		/* Zeiger auf den Filenamen des Files, welches behandelt werden soll */
		} es_makeexec;
		struct {
			_UBYTE *filePtr;		/* Zeiger auf den Filenamen des Files, welches behandelt werden soll */
		} es_project;
		struct {
			_UBYTE *filePtr;		/* Zeiger auf den Namen des obersten Fensters oder NULL */
			_WORD control;			/* 0: Shell soll sich toppen (Menue/Fenster) */
		} es_shlctrl;
		
		_WORD data[5];				/* Definition fuer Maximallaenge des Records */
	} data;
} tMessRec, *tPtrMessRec;

#define supportedSEProtoVersion 0x105

#define maxSe 3

typedef struct _sePartner {
	_WORD apId;
	_UWORD protoVersion;
	tShellSet shell;
	tEditSet editor;
	_UBYTE project[PATH_MAX];
} sePartner;

LOCAL sePartner seList[maxSe];
LOCAL _UWORD seCount;
LOCAL SeOpen se_Open;
LOCAL SeError se_Error;
LOCAL SeErrfile se_Errfile;
LOCAL SeClose se_Close;
LOCAL SeProject se_Project;
LOCAL SeTerminate se_Terminate;
LOCAL SeMenu se_Menu;
LOCAL tShellSet ourShellBits = _SEINIT | _SEOK | _SEACK | _SEQUIT;
LOCAL tEditSet ourEditBits = _ESINIT | _ESOK | _ESACK | _ESCOMPILE | _ESMAKE | _ESMAKEALL | _ESLINK | _ESEXEC | _ESMAKEEXEC | _ESPROJECT | _ESQUIT | _ESSHLCTRL;

#define TRACE_PROT 1

#if TRACE_PROT
#  define TRACE(x) SPEC_DEBUG_OUT(x)
#else
#  define TRACE(x) 
#endif

/*** ---------------------------------------------------------------------- ***/

/*
 * An alle SE-Applikationen ein ES_QUIT schicken
 */
GLOBAL _VOID SeProto_SendTerminate(_BOOL terminate)
{
	tMessRec mess;
	_UWORD i;
	
	UNUSED(terminate);
	mess.msg = ES_QUIT;
	mess.apId = gl_apid;
	mess.overLen = 0;
	mess.data.data[0] = 0;
	mess.data.data[1] = 0;
	mess.data.data[2] = 0;
	mess.data.data[3] = 0;
	mess.data.data[4] = 0;
	for (i = 0; i < seCount; i++)
	{
		appl_write(seList[i].apId, 16, (_WORD *)&mess);
		Wind_Update(END_UPDATE);
		Wind_Update(BEG_UPDATE);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID SendSeAcknowledge(_WORD apId, _BOOL res)
{
	tMessRec mess;
	
	mess.msg = ES_ACK;
	mess.apId = gl_apid;
	mess.overLen = 0;
	mess.data.data[0] = 0;
	mess.data.data[1] = 0;
	mess.data.data[2] = 0;
	mess.data.data[3] = 0;
	mess.data.data[4] = 0;
	mess.data.es_ack.acknowledge = res;
	appl_write(apId, 16, (_WORD *)&mess);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL SendEsMessage(_WORD messNum, CONST _UBYTE *adr)
{
	tMessRec mess;
	_UBYTE *fPtr;
	_UWORD i;
	_BOOL found;
	
	if (adr != NULL)
	{
		fPtr = memPtr;
		strMcpy(fPtr, PATH_MAX, adr);
	} else
	{
		fPtr = NULL;
	}
	found = FALSE;
	for (i = 0; i < 1 && i < seCount; i++)
	{
		if (seList[i].editor & (((tEditSet)1) << (messNum - ES_INIT)))
		{
			mess.msg = messNum;
			mess.apId = gl_apid;
			mess.overLen = 0;
			mess.data.data[0] = 0;
			mess.data.data[1] = 0;
			mess.data.data[2] = 0;
			mess.data.data[3] = 0;
			mess.data.data[4] = 0;
			mess.data.es_project.filePtr = fPtr;
			appl_write(seList[i].apId, 16, (_WORD *)&mess);
			Wind_Update(END_UPDATE);
			Wind_Update(BEG_UPDATE);
			found = TRUE;
			if (messNum == ES_PROJECT)
			{
				if (fPtr != NULL)
					strBcpy(seList[i].project, fPtr);
				else
					seList[i].project[0] = '\0';
				if (se_Project != FUNK_NULL)
					se_Project(seList[i].project);
			}
		}
	}
	return found;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL SeProto_SendCompile(CONST _UBYTE *fname)
{
	return SendEsMessage(ES_COMPILE, fname);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL SeProto_SendMakeall(CONST _UBYTE *fname)
{
	return SendEsMessage(ES_MAKEALL, fname);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL SeProto_SendMake(CONST _UBYTE *fname)
{
	return SendEsMessage(ES_MAKE, fname);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL SeProto_SendExec(CONST _UBYTE *fname)
{
	return SendEsMessage(ES_EXEC, fname);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL SeProto_SendMakeexec(CONST _UBYTE *fname)
{
	return SendEsMessage(ES_MAKEEXEC, fname);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL SeProto_SendLink(CONST _UBYTE *fname)
{
	return SendEsMessage(ES_LINK, fname);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL SeProto_SendProject(CONST _UBYTE *fname)
{
	return SendEsMessage(ES_PROJECT, fname);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL addToList(_WORD apId, _UWORD version, tShellSet shellBits, tEditSet editBits)
{
	_UWORD i;
	
	for (i = 0; i < seCount; i++)
		if (seList[i].apId == apId)
			break;
	/* Eintragen in SeProtoliste */
	if (i >= seCount)
	{
		if (i >= maxSe)
			return FALSE;
		seCount++;
	}
	/* Minimale Daten erstellen */
	seList[i].apId = apId;
	seList[i].protoVersion = version;
	seList[i].shell = shellBits;
	seList[i].editor = editBits;
	++seCount;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID InitSeProto(CONST tMessRec *mess)
{
	tMessRec newMess;
	
	/* Antwort zusammenbasteln */
	newMess.msg = ES_OK;
	newMess.apId = gl_apid;
	newMess.overLen = 0;
	newMess.data.data[0] = 0;
	newMess.data.data[1] = 0;
	newMess.data.data[2] = 0;
	newMess.data.data[3] = 0;
	newMess.data.data[4] = 0;
	newMess.data.es_ok.okShellBits = ourShellBits;
	newMess.data.es_ok.okEditBits = ourEditBits;
	newMess.data.es_ok.okVersion = supportedSEProtoVersion;
	newMess.data.es_ok.otherId = mess->apId;
	if (addToList(mess->apId, mess->data.se_init.version, mess->data.se_init.shellBits, mess->data.se_init.editBits))
	{
		/* Jetzt Antwort schicken */
		TRACE((EO_DEBUG, "ES_OK to %d", mess->apId));
		appl_write(mess->apId, 16, (_WORD *)&newMess);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ActivateSeProto(CONST tMessRec *mess)
{
	if (mess->data.se_ok.otherId == gl_apid)
	{
		addToList(mess->apId,  mess->data.se_ok.okVersion, mess->data.se_ok.okShellBits, mess->data.se_ok.okEditBits);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID RemoveSeProto(_WORD apId)
{
	_UWORD i;
	
	i = 0;
	while (i < seCount && seList[i].apId != apId)
		++i;
	if (i < seCount)
	{
		--seCount;
		for (; i < seCount; i++)
			seList[i] = seList[i+1];
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL SeProto_HandleMessage(CONST _WORD *message)
{
	CONST tMessRec *mess;
	CONST _UBYTE *pCmd;
	_UWORD i;
	
	mess = (CONST tMessRec *)message;
	switch (mess->msg)
	{
	case SE_INIT:
		TRACE((EO_DEBUG, "SE_INIT from %d", message[1]));
		InitSeProto(mess);
		break;

	case SE_OK:
		TRACE((EO_DEBUG, "SE_OK from %d", message[1]));
		ActivateSeProto(mess);
		break;

	case SE_ACK:
		TRACE((EO_DEBUG, "SE_ACK from %d", message[1]));
		/* nichts weiter machen */
		break;

	case SE_OPEN:
		/* Analog VA_START behandeln */
		TRACE((EO_DEBUG, "SE_OPEN %s", mess->data.se_open.namePtr ? (const char *)mess->data.se_open.namePtr : "<null>"));
		if (se_Open != FUNK_NULL)
		{
			_LONG line;
			_WORD column;
			
			pCmd = NULL;
			line = 0;
			column = 0;
			for (i = 0; i < seCount; i++)
			{
				if (seList[i].apId == mess->apId)
				{
					pCmd = mess->data.se_open.namePtr;
					if (seList[i].protoVersion >= 0x102)
					{
						line = mess->data.se_open.iLineNum;
						column = mess->data.se_open.iColNum;
					} else
					{
						line = 1;
						column = 1;
					}
					break;
				}
			}
			if (pCmd != NULL)
				se_Open(pCmd, line, column);
			SendSeAcknowledge(mess->apId, TRUE);
		} else
		{
			SendSeAcknowledge(mess->apId, FALSE);
		}
		break;
	
	case SE_ERROR:
		TRACE((EO_DEBUG, "SE_ERROR %s", mess->data.se_error.errorPtr && mess->data.se_error.errorPtr->errFile ? (const char *)mess->data.se_error.errorPtr->errFile : "<null>"));
		if (se_Error != FUNK_NULL)
		{
			se_Error(mess->data.se_error.errorPtr);
			SendSeAcknowledge(mess->apId, TRUE);
		} else
		{
			SendSeAcknowledge(mess->apId, FALSE);
		}
		break;
	
	case SE_ERRFILE:
		TRACE((EO_DEBUG, "SE_ERRFILE '%s' '%s'",
			mess->data.se_errfile.errPtr ? (const char *)mess->data.se_errfile.errPtr : "<null>",
			mess->data.se_errfile.fnamePtr ? (const char *)mess->data.se_errfile.fnamePtr : "<null>"));
		if (se_Errfile != FUNK_NULL)
		{
			se_Errfile(mess->data.se_errfile.fnamePtr, mess->data.se_errfile.errPtr);
			SendSeAcknowledge(mess->apId, TRUE);
		} else
		{
			SendSeAcknowledge(mess->apId, FALSE);
		}
		break;
	
	case SE_CLOSE:
		TRACE((EO_DEBUG, "SE_CLOSE from %d", message[1]));
		if (se_Close != FUNK_NULL)
		{
			_WORD control;
			
			pCmd = NULL;
			control = 1;
			for (i = 0; i < seCount; i++)
			{
				if (seList[i].apId == mess->apId)
				{
					if (seList[i].protoVersion >= 0x101)
					{
						control = mess->data.se_close.control;
						pCmd = mess->data.se_close.namePtr;
						if (pCmd == NULL || *pCmd == '\0')
							pCmd = "*.*";
					} else
					{
						pCmd = "*.*";
						control = 1;
					}
					break;
				}
			}
			if (pCmd != NULL)
				se_Close(pCmd, control);
		}
		SendSeAcknowledge(mess->apId, TRUE);
		break;

	case SE_TERMINATE:
		TRACE((EO_DEBUG, "SE_TERMINATE from %d", message[1]));
		if (se_Terminate != FUNK_NULL)
		{
			se_Terminate();
			SendSeAcknowledge(mess->apId, TRUE);
		} else
		{
			SendSeAcknowledge(mess->apId, FALSE);
		}
		break;

	case SE_QUIT:
		TRACE((EO_DEBUG, "SE_QUIT from %d", message[1]));
		RemoveSeProto(mess->apId);
		break;

	case SE_PROJECT:
		pCmd = mess->data.se_project.filePtr;
		TRACE((EO_DEBUG, "SE_PROJECT(%d) %s", mess->apId, pCmd ? (const char *)pCmd : "<null>"));
		if (se_Project != FUNK_NULL)
		{
			for (i = 0; i < seCount; i++)
			{
				if (seList[i].apId == mess->apId)
				{
					if (pCmd != NULL)
						strBcpy(seList[i].project, pCmd);
					else
						seList[i].project[0] = '\0';
					se_Project(seList[i].project);
					break;
				}
			}
			SendSeAcknowledge(mess->apId, TRUE);
		} else
		{
			SendSeAcknowledge(mess->apId, FALSE);
		}
		break;

	case SE_MENU:
		TRACE((EO_DEBUG, "SE_MENU from %d", message[1]));
		if (se_Menu != FUNK_NULL)
		{
			SEMENUINFO info;
			SEMENUINFO *ptr;
			
			for (i = 0; i < seCount; i++)
			{
				if (seList[i].apId == mess->apId)
				{
					ptr = mess->data.se_menu.menuInfoPtr;
					info.compStr = ptr->compStr;
					info.makeStr = ptr->makeStr;
					info.makeAllStr = ptr->makeAllStr;
					info.linkStr = ptr->linkStr;
					info.execStr = ptr->execStr;
					info.makeExecStr = ptr->makeExecStr;
					info.progName = NULL;
					info.shellCtrlStr = NULL;
					if (seList[i].protoVersion >= 0x104)
					{
						info.progName = ptr->progName;
						if (seList[i].protoVersion >= 0x105)
							info.shellCtrlStr = ptr->shellCtrlStr;
					}
					se_Menu(&info);
					break;
				}
			}
		}
		SendSeAcknowledge(mess->apId, TRUE);
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SeProto_Register_Open(SeOpen func)
{
	se_Open = func;
	ourShellBits |= _SEOPEN;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SeProto_Register_Error(SeError func)
{
	se_Error = func;
	ourShellBits |= _SEERROR;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SeProto_Register_Errfile(SeErrfile func)
{
	se_Errfile = func;
	ourShellBits |= _SEERRFILE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SeProto_Register_Project(SeProject func)
{
	se_Project = func;
	ourShellBits |= _SEPROJECT;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SeProto_Register_Terminate(SeTerminate func)
{
	se_Terminate = func;
	ourShellBits |= _SETERMINATE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SeProto_Register_Close(SeClose func)
{
	se_Close = func;
	ourShellBits |= _SECLOSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SeProto_Register_Menu(SeMenu func)
{
	se_Menu = func;
	ourShellBits |= _SEMENU;
}

/*** ---------------------------------------------------------------------- ***/

/* SE-Protokoll Initialisierung unter MTOS */
GLOBAL _VOID SeProto_Init(_VOID)
{
	tMessRec mess;
	
	Protokoll_Handle_Message(SeProto_HandleMessage);
	mess.msg = ES_INIT;
	mess.apId = gl_apid;
	mess.overLen = 0;
	mess.data.data[0] = 0;
	mess.data.data[1] = 0;
	mess.data.data[2] = 0;
	mess.data.data[3] = 0;
	mess.data.data[4] = 0;
	mess.data.es_init.shellBits = ourShellBits;
	mess.data.es_init.editBits = ourEditBits;
	mess.data.es_init.version = supportedSEProtoVersion;
	Protokoll_Broadcast(&mess);
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt zurueck, ob das SE-Protokoll aktiv ist
 */
GLOBAL _BOOL SeProto_Activ(_VOID)
{
	return seCount > 0;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt zurueck, ob ein Projekt aktiv ist
 */
GLOBAL _BOOL SeProto_Project_Activ(_UBYTE *filename)
{
	_UWORD i;
	
	for (i = 0; i < seCount; i++)
		if (seList[i].project[0] != '\0')
		{
			if (filename != NULL)
				strMcpy(filename, PATH_MAX, seList[i].project);
			return TRUE;
		}
	return FALSE;
}
