#ifndef __CAT_CATGLOBAL_H__
#define __CAT_CATGLOBAL_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif
#ifndef __PATH_MAX_H__
#include <path_max.h>
#endif

/* Einfache Basisfunktionen, die in mehreren Modulen gebraucht werden.
 * Am 2.11. CatGlobal und CatUtils zusammengelegt in CatGlobal,
 * da zwei so Minimodule nicht noetig sind! DS.
 */


#ifndef __CAT_CATTYPES_H__
#include <cattypes.h>
#endif

extern pathStrType SavePath;
extern nameStrType SaveName;
extern nameStrType OpenName;
extern pathStrType infPath;
extern nameStrType infName;
extern pathStrType prgPath;
extern nameStrType prgName;
extern String255 userAddr;
extern String255 userRName;

/* Positionen sichern? */
extern _BOOL SaveAllPositions;

/* Nur zum Lesen! */
extern _BOOL minTos14;					/* mindestens TOS 1.4 vorhanden */
extern _BOOL tos3DLook;					/* Hat das AES eine aktive 3D-Darstellung */
extern _BOOL isMiNTDomain;				/* Wir befinden uns in der Mint-Domain */

extern _UBYTE HomePath[PATH_MAX];

_VOID CatGlobal_GetUser(_VOID);
/* Setzt den Benutzernamen auf den in den Configvariablen
 * eingestellten Wert
 */

_VOID CatGlobal_busyMouse(_VOID);
/* Stellt rotierenden Ball dar, bei jedem Aufruf dreht er sich weiter */

_VOID FullScreenRedraw(_VOID);
/* Nur fuer nicht-MultiTOS geeignet, loest ein Redraw des gesamten */
/* Bildschirms beim naechsten Durchlaufen der Hauptschleife aus   */

_BOOL CatGlobal_WithShift(_UWORD kstate);
/* Gibt zurueck, ob eine der Shifttasten gedrueckt wurde */

_BOOL CatGlobal_WithCtrl(_UWORD kstate);
/* Gibt zurueck, ob Ctrl gedrueckt wurde */

_BOOL CatGlobal_WithAlt(_UWORD kstate);
/* Gibt zurueck, ob Alternate gedrueckt wurde */

_VOID CatGlobal_ReplaceStr(_UBYTE *str, size_t strsize, CONST _UBYTE *s1, CONST _UBYTE *s2);
/* Ersetzt in str alle Vorkommen der Zeichenkette s1 durch s2
 */

_VOID CatGlobal_ReplaceChar(_UBYTE *str, _UBYTE c1, _UBYTE c2);

_VOID CatGlobal_MakeStr(CONST _UBYTE *source, _UBYTE *dest, size_t destsize);
/* Setzt einen neuen String in dest ein und behaelt die
 * Shortcuts von dest bei
 */

_BOOL CatGlobal_CreateBackup(CONST _UBYTE *fname);
/* Wenn die Datei existiert, dann wird aus dieser ein Backup
 * gemacht.
 */

_VOID CatGlobal_Bing(CONST _UBYTE *name);
/* Erzeugt einen Sound. Die Ereignisse muessen vorher initialisiert werden */
#define cBingError           "BingError"
#define cBingNoUndo          "BingNoUndo"
#define cBingComToOwnMessage "BingComToOwnMessage"
#define cBingFlagsNoWork     "BingFlagsNoWork"
#define cBingAutoNextGroup   "BingAutoNextGroup"
#define cBingNoGroup         "BingNoGroup"
#define cBingNoMessageFound  "BingNoMessageFound"
#define cBingTextNotFound    "BingTextNotFound"
#define cBingNothingFound    "BingNothingFound"
#define cBingLineTooLong     "BingLineTooLong"
#define cBingTerminalStart   "BingTerminalStart"
#define cBingProgramStart    "BingProgramStart"
#define cBingNewMails        "BingNewMails"
#define cBingNoMails         "BingNoMails"
#define cBingEmergencyBrake  "BingEmergencyBrake"
#define cBingAttachment  	 "BingAttachment"


_VOID CatGlobal_SetStrings(OBJECT *tree, _WORD from, _WORD to, CONST _UBYTE *string);
/* Verteilt den String auf verschiedene Textobjekte
 */

_VOID CatGlobal_GetStrings(OBJECT *tree, _WORD from, _WORD to, _UBYTE *string, size_t strsize);
/* Umkehrung zu SetStrings, liest die einzelnen Objekte aus und
 * setzt den String zusammen
 */

_VOID CatGlobal_SetAppl(CONST _UBYTE *str);
/* Setzt den Namen der Applikation fuer interne Zwecke
 * Der Name muss ohne Extension angegeben werden.
 */

_VOID CatGlobal_GetApplPath(_UBYTE *path);
/* liefert den Pfad der Applikation, initialisiert dabei die interne
 * Variable. Vorher muss einmal SetAppl aufgerufen werden
 */

_VOID CatGlobal_ApplPath(_UBYTE *path);
/* liefert den Pfad der Applikation
 */

_VOID CatGlobal_ApplName(_UBYTE *name);
/* liefert den Namen der Applikation
 */

_VOID CatGlobal_Konjugiere(CONST _UBYTE *str1, CONST _UBYTE *single, CONST _UBYTE *plural, _ULONG count, _UBYTE *out, size_t strsize);
/* konjugiert ein Wort nach Singular oder Plural
 */

_VOID CatGlobal_KurzName(CONST _UBYTE *name, _UBYTE *ziel, size_t maxLen);
/* Erzeugt aus einem Benutzernamen ggf. einen auf maxLen
 * verkuerzten Namen, der sinnvoll abgekuerzt ist
 */

_VOID CatGlobal_Init(_VOID);

#endif /* __CAT_CATGLOBAL_H__ */
