#ifndef __CAT_CONVERTDATE_H__
#define __CAT_CONVERTDATE_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __RO_TIME_H__
#include <ro_time.h>
#endif
#ifndef __CAT_TYPES_H__
#include <cattypes.h>
#endif

typedef struct Date {
	_UBYTE day;
	_UBYTE month;
	_WORD year;
} Date;

typedef struct Time {
	_UBYTE second;
	_UBYTE minute;
	_UBYTE hour;
} Time;

typedef enum {
	Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday
} tWeekday;


CATDATE ConvertDate_CurrentDate(_VOID);
/* Liefert das aktuelle GEMDOS-Datum im CAT-Format zurueck
 */

_BOOL ConvertDate_TextToDate(CONST _UBYTE *s, Date *d);
/* Die eigentliche Hauptfunktion: Sie wandelt sowohl ins Datum und
 * ueberprueft auch, ob das Datum gueltig ist. Die wird auch von den
 * Funktionen DateOK und StrToDate aufgerufen.
 */

_VOID ConvertDate_StrToMTDate(CONST _UBYTE *str, _UBYTE *res, size_t strsize);
/* Eingabe: String mit relativ beliebigem Format.
 * Ausgabe: Datum im Maustausch-Format, Uhrzeit wird auf 0 Uhr gesetzt
 */

CATDATE ConvertDate_StrToDate(CONST _UBYTE *str);
/* Eingabe: String mit relativ beliebigem Format.
 * Ausgabe: LONGCARD mit folgenden Format:
 *          jjmmddhhmm, Jahr minus 1990.
 *          0000  immer, da Zeit nicht eingegeben wird.
 * Anmerkung: Diese Funktion gibt nur bis 2032 das korrekte Datum
 *            zurueck, da danach ein Ueberlauf des 32-Bit Wertes
 *            auftritt. Spaetestens dann ist diese Funktion bzw.
 *            das Datumsformat zu ueberarbeiten :-)
 */

_BOOL ConvertDate_DateOk(CONST _UBYTE *str);
/* DateOk prueft nach, ob ein Datum in einem String ein gueltiges Datum ist.
 * Wenn nicht, wird FALSE zurueckgegeben, andernfalls ...
 */

_ULONG ConvertDate_TextDateToNum(CONST _UBYTE *date);
/* Wandelt einen Datumsstring in Anzahl Tage um
 */

_ULONG ConvertDate_DateInNum(_UWORD date);
/* Wandelt ein Datum im GEMDOS-Format in eine Zahl um
 */

_UWORD ConvertDate_NumInDate(_ULONG num);
/* Wandelt eine Zahl in ein GEMDOS-Datum um
 */

CATDATE ConvertDate_UTC(CATDATE cd,_UBYTE *zeitX);
_VOID ConvertDate_CatDate2Datim(CATDATE cd,_UBYTE *zeitX, F_TIME *t);
/* Wandelt ein CAT-Datum in ein Datum und eine Uhrzeit um
 */

Date ConvertDate_UnpackDate(_UWORD date);
/* Wandelt GEMDOS-Datum in Datum um
 */

_UWORD ConvertDate_PackDate(CONST Date *date);
/* Wandelt Datum in GEMDOS-Datum um
 */

Time ConvertDate_UnpackTime(_UWORD time);
/* Wandelt GEMDOS-Zeit in Zeit um
 */

_UWORD ConvertDate_PackTime(CONST Time *time);
/* Wandelt Zeit in GEMDOS-Zeit um
 */

_ULONG ConvertDate_DateToJulian(CONST Date *d);
/* Wandelt ein Datum in eine Zahl (Tage seit 1.1.1980) um
 */

tWeekday ConvertDate_WeekDay(CONST Date *d);
/* Liefert den aktuellen Wochentag zurueck
 */

_BOOL ConvertDate_WeekEnd(CONST Date *d);
/* Liefert TRUE, wenn das Datum am Wochenende ist
 */

_BOOL ConvertDate_Holiday(CONST Date *d);
/* Gibt zurueck, ob das Datum ein deutscher Feiertag ist
 */

_VOID ConvertDate_BussBet(_WORD year, Date *d);
/* Liefert das Datum vom Buss- und Bettag
 */

_VOID ConvertDate_KarFreitag(_WORD year, Date *d);
/* Liefert das Datum von Karfreitag
 */

_ULONG ConvertDate_KarFreitagNumber(_WORD year);
/* Liefert das Datum von Karfreitag als Zahl
 * Damit kann man einige andere variable Feiertage berechnen:
 *
 *  Aschermittoch = Karfreitag - 44;
 *  Ostersonntag = Karfreitag + 2
 *  Ostermontag  = Karfreitag + 3
 *  Christi Himmelfahrt  = Karfreitag + 41
 *  Pfingstmontag  = Karfreitag + 52
 */

_VOID ConvertDate_TimeToText(CONST F_TIME *date, CONST _UBYTE *mask, _UBYTE *t, size_t strsize);
/*
 * Wandelt die Zeit 's' in den String 't'.
 * 't' muss mindestens so gross wie die uebergebene Maske sein, sonst
 * tritt ein Laufzeitfehler auf! (Empfohlen werden mind. 12 Zeichen!)
 *
 * Die Form wird durch 'mask' bestimmt:
 *
 *   HH: Stundenzahl ("hh" loescht fuehrende Null)
 *   MM: Minutenzahl ("mm" loescht fuehrende Null)
 *   SS: Sekundenzahl ("ss" loescht fuehrende Null)
 *
 *   #E: AM/PM-Notierung (English) ("e" schreibt "am" bzw. "pm")
 *
 *   Beispiele fuer:            >13 Uhr 50, 5s<     >2 Uhr 50, 10s<
 *    'hh:mm:ss'                '13:50:5'           '2:50:10'
 *    'HH:MM:SS'                '13:50:05'          '02:50:10'
 *    'hh:MM:SS'                '13:50:05'          '2:50:10'
 *    'hh:MM #e'                '1:50 pm'           '2:50 am'
 *    'hh Uhr mm'               '13 Uhr 5'          '2 Uhr 10'
 *
 * Ist 'mask' ein Leerstring, wird 'HH:SS:MM' verwendet.
 */

_VOID ConvertDate_DateToText(CONST F_TIME *s, CONST _UBYTE *mask, _UBYTE *d, size_t strsize);
/*
 * Wandelt Datum 's' in String 'd'.
 * 'd' muss mindestens so gross wie die uebergebene Maske sein, sonst
 * tritt ein Laufzeitfehler auf! (Empfohlen werden mind. 12 Zeichen!)
 *
 * Die Form wird durch 'mask' bestimmt:
 *
 *   D: Tagesziffer  (d: wird geloescht, wenn nicht auffuellbar)
 *   Y: Jahresziffer (y: wird geloescht, wenn nicht auffuellbar)
 *   M: Monatsziffer (m: wird geloescht, wenn nicht auffuellbar)
 *   G: Monatsbuchstabe Deutsch
 *   F: Monatsbuchstabe Franzoesisch
 *   E: Monatsbuchstabe English
 *   U: Benutzerdefinierter Monatsbuchstabe (s. 'MOSConfig.NameOfMonth')
 *   #E (Hinter Ziffern): s. Bsp. (#G, #F gehen z.Zt. nicht !)
 *
 *   Beispiele fuer:                >13.5.1988<          >1.9.2002<
 *   'DD.MM.YY'                ->  '13.05.88'           '01.09.02'
 *   'mm-dd-yy'                ->  '5-13-88'            '9-1-2'
 *   'dd. GGG yyyy'            ->  '13. Mai 1988'       '1. Sep 2002'
 *   'yyyy, d#E of EEEEEEEEE'  ->  '1988, 13th of May'  '2002, 1st of September'
 *   'GGGGGGGGGGGG'            ->  'Mai'                'September'
 *
 * Ist 'mask' ein Leerstring, wird 'DD.MM.YY' verwendet.
 */

_VOID ConvertDate_DayToText(CONST F_TIME *date, CONST _UBYTE *mask, _UBYTE *dest, size_t destsize);
/*
 * Wandelt Datum 's' in Wochentagsstring 'd'.
 * 'd' muss mindestens so gross wie die uebergebene Maske sein, sonst
 * tritt ein Laufzeitfehler auf! (Empfohlen werden mind. 12 Zeichen!)
 *
 * Die Form wird durch 'mask' bestimmt:
 *
 *   G: Tagesbuchstabe Deutsch
 *   F: Tagesbuchstabe Franzoesisch
 *   E: Tagesbuchstabe English
 */

_VOID ConvertDate_FuzzyTime(CONST F_TIME *date, CONST _UBYTE *mask, _UBYTE *ftime, size_t strsize);
/* Wandelt eine Uhrzeit in einen String mit einer Fuzzy-Uhrzeit um.
 * Von der Maske wird nur das erste Zeichen ausgewertet.
 * G: Deutsche Fuzzytime
 * E: Englische Fuzzytime
 */

_VOID ConvertDate_FuzzyDate(CONST F_TIME *date, CONST _UBYTE *mask, _UBYTE *str, size_t strsize);
/* Wandelt ein Datum und Uhrzeit in einen String mit einem Fuzzydatum um.
 */

_VOID ConvertDate_MakeFreeDate(_UWORD cDt, _UWORD cTi, _UBYTE *str, size_t strsize);
/* Kodiert das uebergebene Datum und die uebergebene Uhrzeit
 * Im MausTausch-Format
 * Das Datum und die Uhrzeit wird im GEMDOS-Format uebergeben
 */
_VOID ConvertDate_MakeInDate(_UWORD cDt, _UWORD cTi, _UBYTE *str, size_t strsize);
/* Wie die vorige aber im Internet-Format*/

_VOID ConvertDate_MakeDate(_UBYTE *str, size_t strsize);
/* Kodiert das aktuelle Datum und die aktuelle Uhrzeit
 * Im MausTausch-Format
 */

_BOOL ConvertDate_MTTextToDate(CONST _UBYTE *s, Date *d);
/* Prueft einen String im MausTausch-Format und konvertiert ihn in ein Datum (MH)
 */

_BOOL ConvertDate_MTTextToTime(CONST _UBYTE *s, Time *t);
/* Prueft einen String im MausTausch-Format und konvertiert ihn in eine Zeit (MH)
 */

#endif /* __CAT_CONVERTDATE_H__ */
