#ifndef __CAT_DATA_H__
#define __CAT_DATA_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __MOS_LISTS_H__
#include <lists.h>
#endif
#ifndef __CAT_CATTYPES_H__
#include <cattypes.h>
#endif
#ifndef __CAT_DATASYS_H__
#include <datasys.h>
#endif
#ifndef __MAGICTOOL_TEXTFILES_H__
#include <mttextfi.h>
#endif
#ifndef __FILE_IO_H__
#include <file_io.h>
#endif

#include <cat_vb.h>

typedef enum {
	nextMess,
	previousMess,
	nextKom,
	prevKom,
	downKom,
	upKom
} PosType;


/* Achtung, Aenderungen hier betreffen auch das Acc-Protokoll!! */


typedef enum {
	distNone,
	distLokal,
	distMausNet,
	distNet
} tDistribution;

typedef struct _MessageType {
	MSGNR MailNr;
	MSGNR MailAnz;
	String255 KommentierteID;
	_BOOL fromOther;					/* Wurde die ID von der anderen Msg gelesen? */
	_UBYTE *MailID;
	_UBYTE *Betreff;
	_UBYTE *Absender;
	_UBYTE *Empfaenger;
	_UBYTE *mid;
	_UBYTE *rid;
	_UBYTE *box;
	_UBYTE *gate;
	_UBYTE *mime;
	_UBYTE *followupTo;
	_UBYTE *replyTo;
	_UBYTE *sender;
	_UBYTE *name;						/* zeigen auf Positionen in "InfoStrings" */
	_UBYTE *attachment;
	_UBYTE *allReceivers;
	_UBYTE *cc;
	_UBYTE *bcc;
	_UBYTE *eZeitX;
	_UBYTE *ref;
	_UBYTE *retPath;
	_UBYTE *keyw;
	_UBYTE *resent;
	_UBYTE *trace;
	_UBYTE *received;
	_UBYTE *path;
	_UBYTE *mailer;
	_UBYTE *unknown;
	CAT_GROUP_NUMBER Gruppe;
	DateType Datum;
	/* nur persoenliche */
	DateType StatusDatum;
	_UWORD infoLen;						/* letztes Byte in <InfoStrings> */
	size_t textLen;						/* letztes Byte im <Text> */

	MSGBITS StatusBits;

	MSGNR up;
	MSGNR down;
	MSGNR left;
	MSGNR right;
	MSGNR KommentarAnzahl;
	CATDATE statusDate;					/* Bei persoenlichen: Statusdatum im CAT-Format */
	CATDATE tauschDate;					/* Datum im CAT-Format */
	_BOOL EigeneNachricht;
	_UBYTE Status;
	_UBYTE *Text;
	_UBYTE *InfoStrings;				/* Freigeben nicht vergessen.. :-) */
	tDistribution distribution;
	_BOOL corrupt;
} MessageType;
#define MessDate(aa) ((aa).tauschDate), ((aa).eZeitX)


struct _oneGroupHandle;
typedef struct _oneGroupHandle oneGroupHandle, *OneGroupHandle;
/*	Definition hinter: maxDatBuffer	*/

#ifndef __CAT_GROUP_IMPLEMENTATION__
struct _oneGroupHandle { int dummy; };
#endif



CATDATE data_mDateStr2Long(CONST _UBYTE *str, _UWORD start);

_VOID data_Long2DateStr(CATDATE date,_UBYTE *zeitX, _UBYTE *str, size_t strsize);


/* Initialisieren, Fehlermeldung wird selber ausgegeben */
_BOOL data_InitDataBase(_VOID);
_VOID data_ResetDataBase(_VOID);

/* Datenbank schliessen, abmelden am Ende */
_VOID data_CloseBase(_VOID);

_VOID data_PrepareToWrite(_VOID);
/* setzt die Positionen so, dass nach dem Einfuegen etwas huebsches rauskommt :-) */

_VOID data_Number2Name(CAT_GROUP_NUMBER nr, _UBYTE *name);
/* Aus der Gruppennummer den Namensanfang (ohne Extension) erstellen */

_BOOL data_OpenOneGroup(CAT_GROUP_NUMBER group, MSGNR add, _BOOL wannaWrite, OneGroupHandle *handle);
/* FALSE, wenn nicht geklappt */
/* add gibt an, fuer wieviele neue Nachrichten Platz benoetigt wird wenn wannaWrite */
/* TRUE ist, dann wird auch versucht ein Schreibpuffer fuer die DAT anzulegen */

_VOID data_CloseOneGroup(OneGroupHandle *handle);
/* Gruppe, die zum Lesen geoeffnet war schliessen */

_VOID data_CloseOneWriteGroup(OneGroupHandle *handle, MSGNR newMsg, _BOOL write, _BOOL *abort);
/* Gruppe, die zum schreiben geoeffnet war schliessen */
/* newMsg -> Anzahl der neuen (d.h. nicht gefilterten oder ignorierten) Msgs */
/* write  -> Puffer wirklich zurueckschreiben? Normalerweise nicht bei Abbruch */
/* abort  -> etwas ist beim Schliessen schiefgelaufen */

_VOID data_SetLogDate(_UBYTE *date, size_t strsize);
/* Datum aus dem Logfile uebernehmen */


_BOOL data_ReadPos(_VOID);
_BOOL data_WritePos(_VOID);
/* Positionstabelle einlesen und schreiben, Fehlermeldung wird ausgegeben */

F_ERROR data_ReadBlock(OneGroupHandle handle, MSGNR nr, pBlockPtr *ptr);
/* Liest einen Block aus der PAR-Datei */

F_ERROR data_ReadFromPar(FILENO parH,_WORD parV,MSGNR nr,MSGNR anz, pBlock *pPb);
/* Liest Daten aus Par*/

F_ERROR data_ReadBlockCrc(OneGroupHandle handle, MSGNR nr, pBlockPtr *ptr);
/* Block mit crc-Check einlesen */

F_ERROR data_GetCrc(OneGroupHandle handle, MSGNR nr, CRC16 crc[2]);
/* CRC aus TAB lesen */

F_ERROR data_PutCrc(OneGroupHandle handle, MSGNR nr, const CRC16 crc[2]);

/* CRC in TAB schreiben */

F_ERROR data_CpyFromDat(OneGroupHandle handle, _ULONG start, _ULONG len, FILENO zielF);
/* Teil der Dat in anderes File kopieren	*/

F_ERROR data_WriteBlock(OneGroupHandle handle, MSGNR nr, pBlock *ptr);
F_ERROR data_WriteBlockCrc(OneGroupHandle handle, MSGNR nr, pBlock *ptr);



_VOID data_GetCheckArea(onePos *area);
/* Holt die Pruefarea fuer das Datum aus dem GRUPPEN.POS
 */

_VOID data_SetCheckArea(CONST onePos *area);
/* Setzt die Pruefarea fuer das Datum in dem GRUPPEN.POS
 */

MSGNR data_FirstNewMsg(CAT_GROUP_NUMBER group);
/* erste Msg aus dem letzten Outfile in dieser Gruppe */
/* "empty", wenn keine neue in dieser Gruppe ist      */

MSGNR data_LastMsgOfGroup(CAT_GROUP_NUMBER group);
/* letzte Msg der Gruppe */

MSGNR data_lastReadMsgOfGroup(CAT_GROUP_NUMBER group);
/* letzte vom User gelesene Msg dieser Gruppe */

MSGNR data_unreadMsgCount(CAT_GROUP_NUMBER group);
/* Anzahl der ungelesenen Msgs in der Gruppe */

MSGNR data_unreadMsgPos(CAT_GROUP_NUMBER group);
/* Position der ersten ungelesenen Msg in der Gruppe */

_VOID data_forceUnreadRefresh(OneGroupHandle handle);
/* erste Ungelesene suchen und durchzaehlen */

_VOID data_SetLastReadMsg(CAT_GROUP_NUMBER group, MSGNR nr);
/* neue letzte vom User gelesene Msg setzen */
_VOID data_SetUnreadMsgCount(CAT_GROUP_NUMBER group, MSGNR count);
/* neue Anzahl letzter vom User gelesene Msg setzen */

/* Fehlerrueckgabe der naechsten 2 Positionsfunktionen: noError/notFound */

MSGNR data_NumberOfID(OneGroupHandle handle, CONST _UBYTE *ID);
/* MsgNummer zu einer ID feststellen ab 4.50 wird dabei nicht mehr zwischen
kurzer und langer ID unterschieden*/

MSGNR data_NumberOfDate(OneGroupHandle handle, CATDATE date);
/* date im Cat-Format, umwandeln mit ConvertDate.StrToDate */
/* MsgNummer zu einem Datum suchen */

/* Moegliche Fehler: noError, notFound, IOError */

F_ERROR data_ReadMessage(OneGroupHandle handle, MSGNR nr, MessageType *mess);
/* Eine Msg aus der Datenbank lesen */

F_ERROR data_ReadSmallHeader(OneGroupHandle handle, MSGNR nr, MessageType *mess);
/* Liest nur die Headerinformationen aus der .PAR-Datei. Alle anderen Daten
 * sind *nicht* initialisiert
 */

F_ERROR data_ReadHeader(OneGroupHandle handle, MSGNR nr, MessageType *mess,_BOOL showErr);
/* Nur einen Teil der Msg lesen, evtl. weitere Funktionen */

F_ERROR data_ReadState(OneGroupHandle handle, MSGNR nr, MSGBITS *flags);
/* Nur die Flags lesen, fuer "Gelesene ignorieren" */

F_ERROR data_ReadPersState(OneGroupHandle handle, MSGNR nr, _UBYTE *state, _BOOL *own, MSGBITS *bits);
/* Nur den Status einer persoenlichen Msg lesen */

_VOID data_ReadRightNumber(OneGroupHandle handle, MSGNR nr, MSGNR *right);
/* Nummer der Nachricht in der Kommentarverkettung nach rechts erfragen */

F_ERROR data_ReadText(OneGroupHandle handle, MSGNR nr, _VOID *adr, size_t bufsize);
/* den Text zur msg in den Puffer lesen, muss natuerlich gross genug sein! */

F_ERROR data_SetBits(OneGroupHandle handle, MSGNR nr, MSGBITS bits);
/* Statusbits setzen */

F_ERROR data_ChangeState(OneGroupHandle handle, MSGNR nr, _UBYTE NewState);
/* StatusChar setzen */

_BOOL data_HasAnswer(OneGroupHandle handle, MSGNR nr, MSGBITS *bits);
/* Testet, ob diese Msg noch eine Antwort hat, fuer grin.grinResetState */

/* Idee: Bei Fehler Einfuegen abbrechen und Fehlerdatei schreiben;
 * Fehlerbehandlung ueber eigenes Programm
 */


typedef enum {
	inBits = 0x0001,
	inText = 0x0002,
	changeBits = 0x0004,
	dontShow = 0x0008,
	ask = 0x0010,
	reverse = 0x0020
} SearchType;

/* Die ersten drei geben je an, wo gesucht werden soll. changeBits sagt, */
/* dass die Bits der gefundenen Msg gemaess der Setzmaske gesetzt werden    */
/* sollen. Bei dontShow werden alle Msgs bis zum Ende behandlet. Mit     */
/* reverse kann man die Suchrichtung auf rueckwaerts stellen               */


typedef _UWORD SearchSet;

typedef struct _SearchInfo {
	OneGroupHandle handle;
	MSGNR start;					/* StartNachricht         */
	MSGBITS setBits;				/* Zusammen Suchmaske     */
	MSGBITS clearBits;
	CONST _UBYTE *str;				/* Zu suchender String    */
	CONST _UBYTE *str2;
	CONST _UBYTE *str3;
	CONST _UBYTE *str4;
	_WORD wo1;						/* Wo sollen sie stehen?  */
	_WORD wo2;
	_WORD wo3;
	_WORD wo4;
	_WORD verkn1;					/* logische Verknuepfung   */
	_WORD verkn2;
	_WORD verkn3;
	_BOOL gross;					/* gross=klein?            */
	MSGBITS toSetBits;				/* Zusammen Setzmaske     */
	MSGBITS toClearBits;
	SearchSet what;					/* Was machen?            */
	_BOOL *break_;					/* Abbruch durch esc?     */
} SearchInfo;

typedef	struct
{
	CAT_GROUP_NUMBER guekNr;						/* bei Bumen die mit einem GK starten steht hier die Nummer der Vorgngergruppe */
	_UBYTE guekId[20];
}GUEKINF;


_BOOL data_ComplexSearch(SearchInfo *search, MSGNR *nr);


/* Nummer, falls gefunden */
/* Sucht eine Nachricht nach den angegebenen Werten, falls sie gefunden wird */
/* dann ist found TRUE und es wird die Nummer dieser Nachricht zurueckgegeben */
/* str wird evtl. wegen gross-klein-Unterscheidung geaendert, deswegen         */
/* vielleicht einen Backup-String uebergeben.                                 */
/* wo1..wo4, verkn1..verkn3 wie in SearchHelp.. */

/* Uebergabetypen fuer Appendmessage */

/* Reihenfolge wichtig! */
#define bId           0x00000001L
#define bEZeit        0x00000002L
#define bGruppe       0x00000004L
#define bBSZeit       0x00000008L
#define bWegen        0x00000010L
#define bVon          0x00000020L
#define bAn           0x00000040L
#define bMId          0x00000080L
#define bRId          0x00000100L
#define bBox          0x00000200L
#define bName         0x00000400L
#define bRefNr        0x00000800L
#define bDistribution 0x00001000L
#define bGate         0x00002000L
#define bMime         0x00004000L
#define bFollowupTo   0x00008000L
#define bReplyTo      0x00010000L
#define bSender       0x00020000L
#define	bAttachment	  0x00040000L
#define bCc     	  0x00080000L	
#define bBcc 		  0x00100000L
#define bEZeitX 	  0x00200000L
#define bRef   		  0x00400000L
#define bRetPath      0x00800000L
#define bPath         0x01000000L
#define bMailer       0x02000000L
#define bLastStr      0x02000000L
#define bDistribution 0x00001000L



enum {
	bmKeyW=0x01,
	bmResent=0x02,
	bmTrace=0x04,
	bmReceived=0x08,
	bmUnknown=0x10};	/* mu letztes sein*/

enum {
	fText=0x01,
	fTextDatei=0x02,
	fDelDatei=0x04,
	fWegenIsInf=0x08,
	fIsHeader=0x10,
	fmMime=0x20,
	fMultipartM=0x40,		/*Multipart/mixed*/
	fMultipartA=0x80,		/*Multipart/alternative*/
	fMultipartR=0xC0,		/*Multipart/related*/
	fMultipartD=0x100,		/*Multipart/digest*/
	fMultipartS=0x140,		/*Multipart/signed*/
	fMultipart1=0x180,		/*kein Multipart aber auch kein Mailtext*/		/*DYJ 5.33b */
	fMultipartE=0x1C0,		/*Multipart/encrypted*/	/*DYJ 5.36h */
	fMultipartMask=0x1C0,	/*Maske um Multipart-Typ zu extrahieren*/
	fFlags= 0x0200,
	fType= 0x0400,
	fMailingList=0x800,		/*DYJ 5.34d */			
	};
/*DYJ  Ende; alt:	
#define bText         0x08000000L
#define bTextDatei    0x10000000L
#define bDelDatei     0x20000000L
#define bWegenIsInf   0x40000000L
#define bIsHeader     0x80000000L
*/

typedef _LONG stringSet, stringTypes;

/*Fr Zeilentypen die mehrfach vorkommen knnen.*/
typedef struct  {
	_UWORD	maxZeilen;
	_UWORD	anzZeilen;
	_UBYTE  **adr;
}MultiLines;

typedef struct sVERK_BLK {
	struct sVERK_BLK	*nxt;
	_UBYTE	data;
}	VERK_BLK;


typedef enum {
	msg_own,
	msg_personal,
	msg_status,
	msg_normal,
	msg_garbage
} WhatsThat;

/* Neue Reihenfolge fuer WatchDog-Protokoll, wegen Kompatibilitaet */
/* WatchDog darf nur gefragt werden, wenn es keine persoenliche   */
/* Msg ist, sonst ist evtl. z.B. pText NIL                       */

typedef _UWORD RECFLAGS;


typedef struct _PtrRecord {
	_UBYTE *pGruppe;					/* Gruppe                               */
	_UBYTE *pVon;						/* Absender                             */
	_UBYTE *pWegen;						/* Betreff                              */
	_UBYTE *pText;						/* MsgText                              */
	_UBYTE *pDelDatei;					/* File das ggf gelscht werden soll	*/
	mess_len TextMax;
	_UBYTE *pId;						/* Maus-ID der Mitteilung               */
	_UBYTE *pRefNr;						/* kommentierte Msg                     */
	_UBYTE *pAn;						/* Empfaenger                           */
	_UBYTE *pEZeit;						/* Eingabezeit                          */
	_UBYTE *pBSZeit;					/* Bearb.status+Zeit                    */
	_UBYTE *pMId;						/* MessageID                            */
	_UBYTE *pRId;						/* RId gem. Def                         */
	_UBYTE *pBox;						/* Box gem. Def.                        */
	_UBYTE *pName;						/* Name gem. Def falls # Absenderangabe */
	_UBYTE *pGate;						/* Gateway                              */
	_UBYTE *pMime;						/* MIME-Zeile von Maus                  */
	_UBYTE *pFollowupTo;				/* FollowUp-To - Zeile                  */
	_UBYTE *pReplyTo;					/* Reply-To - Zeile                     */
	_UBYTE *pSender;					/* Sender - Zeile                       */
	_UBYTE *pFlags;						/* s-Zeile */
	_UBYTE *pCc;						/* Carbon-Copy-Zeile					*/
	_UBYTE *pBcc;						/* Blind-Carbon-Copy-Zeile				*/
	_UBYTE *pEZeitX;					/* Eingabezeit Ergnzung(Zeitzone,Jahr,Sekunde):YYYY SS +ZZZZ    */
	_UBYTE *pRef;						/* REFERENCES-Zeile						*/
	_UBYTE *pRetPath;					/* RETURN-PATH-Zeile					*/
	_UBYTE *pPath;						/*PATH-Zeile							*/
	_UBYTE *pMailer;					/*X-Mailer oder X-Newsreader			*/
	_UBYTE *pLenO;						/*Lnge des ursprnglichen Textes(vor dem Abtrennen von Anhngen)		*/
	MultiLines mKeyW;					/* KEYWORDS-Zeile						*/
	MultiLines mResent;					/* Resent-Zeilen						*/
	MultiLines mTrace;					/* TRACE-Zeile							*/
	MultiLines mReceived;				/* Received-Zeilen						*/
	MultiLines mMime;					/* Mime-Zeilen aus Internet				*/
	MultiLines mAttachment;				/* Anhnge								*/
	MultiLines mUnknown;				/* Unbekannte-Zeile						*/

	/*Achtung falls neue Multilines eingefgt werden mssen sie in 
		freeMultilines freigegeben werden*/
	/* Achtung, die Groesse dieses Strings wird bei der Verkettung */
	/* per RId benoetigt -> bei Aenderungen beruecksichtigen */
	TEXTFILENO txt;
	stringSet whatsThere;
	_UBYTE whatsThereM;
/*DYJ 5.34g Start*/
	RECFLAGS prFlags;
/*DYJ  Ende; alt:	
	RECFLAGS flags;
*/
	WhatsThat msgType;
	
	_UBYTE *pDistribution;				/* Distribution                         */
	VERK_BLK	*tmpMem;
	FILENO relatedFN;
} PtrRecord;

#define UnknID "???"		/*DYJ 5.39c */

/* Falls das Bit bTextDatei gesetzt ist, dann soll der Text aus der uebergebenen */
/* Datei uebernommen werden, er steht dann nicht im Speicher                     */
/* !!Achtung: Weiterhin ist dieses Bit der Anzeiger dafuer, dass es sich bei     */
/* dieser  Message um eine eigene handelt, die gerade eingefuegt werden soll.    */


/*DYJ 5.39c Start*/
WhatsThat data_PreCheck(PtrRecord *mess,OneGroupHandle handle);
/*DYJ  Ende; alt:	
WhatsThat data_PreCheck(PtrRecord *mess);
*/

/* Testet die Nachricht einmal durch, damit der parser schon einen Teil der */
/* Fehlerbehandlung sowie die WatchDog-Behandlung machen kann               */

/* Bits, die beim Einfuegen schon gesetzt werden sollen; bFiltered bisher */
/*DYJ 5.39d Start*/
MSGNR data_AppendMessage(
/*DYJ  Ende; alt:	
_BOOL data_AppendMessage(
*/
	OneGroupHandle handle,
	MSGBITS bitWishes,
	PtrRecord *mess,
	_WORD *waisenStatus,
	_BOOL *isOneDupe,
	_BOOL *isCommentToOwn,		/* (MH) */
	_BOOL *abort,
	_WORD defCharset,
	_BOOL setLastRead);
/* Eine Message bearbeiten/ abort -> sofort abbrechen, schwerer Fehler */
/* Returnwert zeigt an, ob die Nachricht geschrieben wurde */
/* Es koennen natuerlich nur die Typen own, personal, status und normal geschrieben */
/* werden. Bei Missbrauch: Garbage in possible crash out! */

CAT_GROUP_NUMBER data_GetGroupNr(OneGroupHandle handle);

_VOID data_dumpMess(CONST PtrRecord *mess);
/* Fehlermeldung zu dieser Nachricht in catlog.txt ausgeben */

_ULONG data_EstimateNecessaryMemory(MSGNR additional);
/* Versucht den fuer den naechsten Einfuegevorgang noetigen Platz zu schaetzen */

typedef _BOOL (*treeProc)(OneGroupHandle handle, MSGNR msgIdx, pBlock *mess, _VOID *private);


_VOID data_WalkTree(OneGroupHandle handle, MSGNR mess, _BOOL fromTop, treeProc proc, _VOID *private);
/* Allgemeine Prozedur, um einen Baum zu durchlaufen */

_VOID data_WalkTree2(OneGroupHandle handle, MSGNR mess, _WORD maxUp, _WORD maxDepth, treeProc proc, GUEKINF *guekinf);
/* Etwas weniger allgemeine Prozedur, um einen Baum zu durchlaufen
 * Es wird maximal um maxUp Ebenen nach oben gegangen, und von dort aus
 * maximal maxDepth Ebenen nach unten. Jetzt wird auch noch bei GK die
 * Gruppennummer der Vorgngergruppe zurckgegeben.
 */
typedef _VOID (*updateProc)(CAT_GROUP_NUMBER group,MSGNR msgIndex, MSGBITS newFlags);
typedef struct {
	MSGBITS clearbits;
	MSGBITS setbits;
	_BOOL readChg;
} changeInfo;
typedef MSGBITS (*flagchangeProc)(MSGBITS oldBits, CONST changeInfo *info);


_VOID data_TreeFlags(OneGroupHandle handle, MSGNR mess, _BOOL fromTop, updateProc updt, flagchangeProc flagchange, CONST changeInfo *changeInfo);
/* Flags eines Baumes bearbeiten */

GLOBAL _VOID data_ChldFlags(OneGroupHandle handle, MSGNR mess, updateProc updt, flagchangeProc flagchange, CONST changeInfo *changeInfo);
/* Flags der Kinder einer Msg bearbeiten */

_VOID data_LineOut(TEXTFILENO file, CONST _UBYTE *ch, CONST _UBYTE *str);
/* Eine Zeile in ein Outfile schreiben */
_VOID data_buildDate(CATDATE date,_UBYTE *eZeitX, _UBYTE *str, size_t strsize);
/* Datum fr Outfile erzeugen */
_VOID data_buildFlagLine(MSGBITS bits, _UBYTE *str);
/* Flagzeile fr Outfile erzeugen */
_VOID data_messageOut(TEXTFILENO f, CONST MessageType *mess, CONST _UBYTE *pre);
/* Message in Outfile schreiben */

GLOBAL _VOID data_CopyPos(CAT_GROUP_NUMBER groupZ,CAT_GROUP_NUMBER groupQ);

GLOBAL _VOID data_updateGrPos(CAT_GROUP_NUMBER cGrp,MSGNR *newNums,MSGNR msgCounter,MSGNR del_n_bad,MSGNR firstUnread, MSGNR unreadCounter);

_VOID data_freeMessage(MessageType *mess);

_BOOL data_delAllAttach(CAT_GROUP_NUMBER grCatNr);

_VOID data_delAttach(pBlock *param,_UBYTE *buf);

extern _UBYTE Bcc_unbek[];

#endif /* __CAT_DATA_H__ */
