#ifndef __CAT_DATASYS_H__
#define __CAT_DATASYS_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */

#include <cat_vb.h>



#ifndef __CAT_TYPES_H__
#include <cattypes.h>
#endif

#define dataSys_private 0

#define dataSys_empty    (MSGNR)0xffffffffl		/* kein Verweis	*/
#define dataSys_notSaved (MSGNR)0xfffffffel		/* verweist auf nicht bekannte Message*/

typedef _UWORD MSGBITS;
#define bGelesen        0x0001
#define bFiltered       0x0002
#define bInteressant    0x0004
#define bTeilloeschung  0x0008
#define bTotalloeschung 0x0010
#define bKommentieren   0x0020
#define bAntworten      0x0040
#define bUser1          0x0080
#define bUser2          0x0100
#define bVererben       0x0200 /* bFiltered = 10; */
/* Statusbits einer Msg */
#define bComToOwnMessage 0x0400

/* Vorlaeufig bis bei der Maus die Message-Ids diesen Namen auch verdienen: */

/* Neu, eigene Messages werden gekennzeichnet */
#define bOwnMessage     0x4000
#define bOldComToOwnMessage  0x2000

#define bDelMaus		0x1000
/* da einige Betaversionen das so hatten, wird in einer Uebergangsphase beides */
/* erkannt und ggf. 13 entfernt und dafuer 10 gesetzt */



#define maxGroup        0xfffe


/* Header fuer gruppen.pos und *.par */

typedef struct FileHeaderType {
	_ULONG CatMagic;
	_UWORD Version;
	_UWORD VersionMagic;
} PACKED FileHeaderType;


#define dbHeaderLength  (sizeof(FileHeaderType))

#define dbCatMagic      0x43415420L   /* "CAT "                    */

#define dbVersion       2   /* Versionsnummer der data.i */

#define dbVersionMagic  0x123   /* Versionsmagic             */
#define grPosVersionMagic  0x124   /* Versionsmagic             */
#define dbVersMin       1   /* Kleinste Versionsnummer die verstanden wird */
#define dbVersMax       2   /* Grte Versionsnummer die verstanden wird */


/*--------------------------------------------------------------------
  Cat-Version    Magic        Bemerkungen
  0001           0123H        Erste Version der neuen Datenbank
  0002           0123H        Zweite Version der neuen Datenbank ab 4.50
 --------------------------------------------------------------------*/

#ifndef _CRC16
#define _CRC16
typedef _UWORD CRC16;
#endif

typedef enum {
	aktuellePos,
	neuePos,
	letztePos,
	unreadPos,
	unreadCount
} posType;
typedef struct {
	MSGNR aktuellePos;
	MSGNR neuePos;
	MSGNR letztePos;
	MSGNR unreadPos;
	MSGNR unreadCount;
} PACKED onePos;

typedef struct grPosType {
	FileHeaderType head;
	onePos save;
	onePos *pos;
	CAT_GROUP_NUMBER posGroups;
	CAT_GROUP_NUMBER usedGroups;
} grPosType;


/* BITS in items */
#define itemVon 			0x0001
#define itemAn 				0x0002
#define itemMId 			0x0004
#define itemRId 			0x0008
#define itemBox 			0x0010
#define itemName 			0x0020
#define itemRefNr 			0x0040
#define itemDistribution 	0x0080
#define itemGate 			0x0100
#define itemMime 			0x0200
#define itemFollowup 		0x0400
#define itemReplyTo 		0x0800
#define itemSender 			0x1000
#define itemAttachment		0x2000

#define itemCc				0x00004000L
#define itemBcc	    		0x00008000L
#define itemEZeitX 	  		0x00010000L
#define itemRef   	  		0x00020000L
#define itemRetPath		 	0x00040000L
#define itemPath     		0x00080000L
#define itemMailer			0x00100000L

#define itemKeyW     		0x00200000L
#define itemResent			0x00400000L
#define itemTrace		  	0x00800000L
#define itemReceived		0x01000000L
#define itemLast			0x01000000L

#define itemUnknown			0x40000000L
#define itemPrivateBytes	0x80000000l
typedef _ULONG ITEMBITS;

/*-------------------------------------------------------------------------------
   Achtung, es ist wichtig, dass die crc an erster Stelle steht und zwei Bytes
   lang ist, davon geht data.i an mehreren Stellen aus. Ausserdem muss der
   dupeInfoType auf den pBlock passen!
--------------------------------------------------------------------------------*/

/*-- Aus Redundanzgruenden doppelt gespeicherte Information --*/

typedef struct dupeInfoType {
	CATDATE Datum;			/* Datum im CAT-format */
	ITEMBITS items;			/* In der DAT vorhandene Dinge, */
	/* in der Reihenfolge wie oben angegeben */
	MSGBITS bits;
	data_len hLength;		/* Laenge des MessageHeaders */
	data_len idLength;		/* ID-Laenge im Header       */
	mess_len Length;		/* Laenge der Mitteilung     */
	_ULONG Start;			/* Offset in der Datendatei  */
	/*-- Bis hier genau wie in pBlock --*/
	_UWORD setTerminator;	/* beendet den Datensatz */
} dupeInfoType;


#define Terminator      0xaa00
/* Kommt nach meinen Tests recht selten vor und ergibt damit einen gewissen Anhaltspunkt */

/*-- Parameterblock fuer die Parameterfiles --*/

typedef struct pBlock {
	CRC16 crc;				/* crc ueber den Rest des Parameterblocks */
	CATDATE Datum;			/* Datum im CAT-Format */
	ITEMBITS items;			/* In der DAT vorhandene Dinge, */
	/* in der Reihenfolge wie oben angegeben */
	MSGBITS bits;
	data_len hLength;		/* Laenge des MessageHeaders */
	data_len idLength;		/* ID-Laenge im Header       */
	mess_len Length;		/* Laenge der Mitteilung     */
	_ULONG Start;			/* Offset in der Datendatei  */
	MSGNR upMess;
	MSGNR downMess;
	MSGNR rightMess;
	MSGNR leftMess;
	MSGNR KomCount;
} PACKED pBlock;			/* 44 Byte */
typedef pBlock *pBlockPtr;
typedef struct pBlock1 {
	CRC16 crc;				/* crc ueber den Rest des Parameterblocks */
	CATDATE Datum;			/* Datum im CAT-Format */
	_UWORD items;			/* In der DAT vorhandene Dinge, */
	/* in der Reihenfolge wie oben angegeben */
	MSGBITS bits;
	data_len hLength;		/* Laenge des MessageHeaders */
	data_len idLength;		/* ID-Laenge im Header       */
	_UWORD Length;		/* Laenge der Mitteilung     */
	_ULONG Start;			/* Offset in der Datendatei  */
	_UWORD upMess;
	_UWORD downMess;
	_UWORD rightMess;
	_UWORD leftMess;
	_UWORD KomCount;
} PACKED pBlock_X;			/* 30 Byte */
typedef struct {
	_UWORD aktuellePos;
	_UWORD neuePos;
	_UWORD letztePos;
	_UWORD unreadPos;
	_UWORD unreadCount;
} PACKED onePos_X;


/*-- Bei persoenlichen Msgs zusaetzlich gespeicherte Infos --*/


typedef struct pInfoType {
	CATDATE LeseDatum;
	_UBYTE Status;
	_UBYTE locked;   /* wenn # 0 dann Status nicht vom User aenderbar */
} PACKED pInfoType;
typedef pInfoType *pInfoPtr;

typedef struct
{
	MSGNR	msgNr;		/* Die Messagenummer oder dataSys_empty	*/
	MSGNR 	kommentarAnzahl;
	CATDATE	ignAft;		/*	Verfallsdatum	*/
	CRC16	crc[2];		/*	CRC der RefNr (Referenz auf MausID)	und RId (Referenz auf Message ID)*/
}WAISEN;



/*--------------------------------------------------------------------
  Neuer Aufbau eines *.DAT-Eintrages:
          +=========================+ \
          | Header, max. 16kB       |  |
          |                         |  \  Erlaeuterung unten
          |                         |  /  Laenge in "hLength"
          |                         |  |
       /  +=========================+ /
Gesamt-|  | Messagetext, max 64kB   |     Messagetext, Strings, die mit lf getrennt
Laenge <  |                         |     sind. Demnaechst einmal noch nullterminiert
in     |  |                         |     am Ende.
Length \  +=========================+ \
          | long Datum              |  |
          +-------------------------+  |
          | word items              |  |
          +-------------------------+  \  Aus Redundanzgruenden
          | word bits               |  /  nochmal hier; fuer Reparaturzwecke;
          +-------------------------+  |  dieselben Werte, wie in pBlock
          | word strLength          |  |
          +-------------------------+  |
          | word Length             |  |
          +-------------------------+ /



  In Header steht:
  +-------------------------+
  | Message-ID              | Laenge in idLength, sicher vorhanden
  +=========================+ <- Ein Byte eingeschoben, falls idLength ungerade ist
  | Anzahl der Eintraege    | Anzahl der noch folgenden Strings
  +-------------------------+
  | Laenge Eintrag # 1      | Laenge von String Nummer 1 (Betreff)
  +-------------------------+
  .                         .
  .                         .
  +-------------------------+
  | Laenge Eintrag # n      |
  +=========================+
  | Betreff                 | sicher vorhanden
  +=========================+
  | Absender                | wahrscheinlich vorhanden
  +-------------------------+
  | Empfaenger              | sporadisch vorhanden
  +-------------------------+
  | MID                     | Message-ID bei Messages ueber ein Gateway
  +-------------------------+
  | RID                     | Kommentar-ID bei Messages ueber ein Gateway
  +-------------------------+
  | BOX                     | Hinweise auf Absenderbox
  +-------------------------+
  | Name                    | Name des Absenders, falls nicht gleich "Absender"
  +=========================+
  | Weitere Textinformation | Strings der Maus, die Cat noch nicht kennt
  +=========================+
  | private-Bytes           | Statusdatum, locked, Status
  +-------------------------+

Bis auf die beiden letzten Eintraege sind alle Eintraege nullterminierte
Strings und nur vorhanden, wenn in "items" angegeben. Die Reihenfolge gemaess
"items" wird eingehalten.

Also einmal mit Hausnummern:
items = 00000000 00101101 = 2D hex heisst, dass es ausser der Message-ID und
dem Betreff noch einen Absender, eine Gateway-Message-ID, eine Kommentar-ID
und eine Namensangabe in dieser Reihenfolge im Header gibt.

In der Sparte "Weiter Textinformation" stehen Informationen, die von
der Maus geliefert werden, aber die Cat noch nicht kennt. Es handelt sich
dabei um eine Liste nullterminierter Strings, die mit einem leeren
String beendet wird. Der erste Buchstabe der Strings entspricht dem
vom MausTausch-Protokoll uebergebenen.

private-Bytes enthaelt die bisherige Sonderinfo aus dem Parameterblock bei
persoenlichen Msgs. Der Eintrag ist immer 6 Bytes lang und folgendermassen
aufgebaut:

----------------------------------------------------------------------*/


#endif /* __CAT_DATASYS_H__ */
