/*****************************************************************************
 *		DIR_IO.H
 *****************************************************************************/

#ifndef __DIR_IO_H__
#define __DIR_IO_H__

#ifndef __RO_TIME_H__
#include <ro_time.h>
#endif
#ifndef __FILE_IO_H__
#include <file_io.h>
#endif

/* file access modes for user, group, and other*/
#define S_IRUSR         0x0100
#define S_IWUSR         0x0080
#define S_IXUSR         0x0040
#define S_IRGRP         0x0020
#define S_IWGRP         0x0010
#define S_IXGRP         0x0008
#define S_IROTH         0x0004
#define S_IWOTH         0x0002
#define S_IXOTH         0x0001

typedef struct my_fstat {
	_UBYTE st_is_directory;			/* directory */
	_UBYTE st_is_regular;			/* regular file */
	_UBYTE st_is_char_special;		/* character special file */
	_UBYTE st_is_block_special;		/* block special file */
	_UBYTE st_is_lnk;				/* symbolic link */
	_UBYTE st_is_fifo;				/* named pipe */
	_UBYTE st_is_socket;			/* socket */
	_UBYTE st_is_mem;				/* memory region or process */
	_ULONG st_mode;					/* access mode bits */
	_ULONG st_size;					/* file size */
	SYSTIME st_mtime;				/* modification time (seconds since 1.1.1900 00:00) */
	SYSTIME st_atime;				/* access time (seconds since 1.1.1900 00:00) */
	SYSTIME st_ctime;				/* creation time (seconds since 1.1.1900 00:00) */
	_LONG st_nlink;					/* number of links */
	_LONG st_uid;					/* user id of owner */
	_LONG st_gid;					/* group id of owner */
	_LONG st_dev;
	_LONG st_ino;
} F_STAT;

typedef struct my_dirent {
	F_STAT	d_st;
	_WORD	d_namlen;				/* length of d_name */
	struct  my_dirent *d_next;
	_UBYTE	d_name[1];
} DirectoryEntry;

typedef struct my_dirdesc {
	_LONG	dd_fd;	/* uniquely identify each open directory */
	_LONG	dd_loc;	/* where we are in directory entry is this */
	_UBYTE   *dd_buf; /* directory block */
	DirectoryEntry  *dd_cur;
	struct  my_dirdesc *dd_next;
	struct  my_dirdesc *dd_link;
	_UBYTE  *dd_wd;
} Directory;

typedef struct my_filesys {
	_UBYTE fs_mount_point[PATH_MAX];
	_BOOL  fs_removable;
	_ULONG fs_size;
	_UBYTE fs_description[PATH_MAX];
	_UBYTE fs_type[PATH_MAX];
} FILESYSTEM;

#define MAX_FILESYSTEMS 32


#ifndef _
#  define _(x) x
#endif

extern _BOOL _readdir_tolower;

extern	Directory		*D_Opendir _(( CONST _UBYTE * ));
extern	DirectoryEntry	*D_Readdir _(( Directory * ));
extern	_VOID			D_Seekdir _(( Directory *, _LONG ));
#define	D_Rewinddir(dirp)	D_Seekdir(dirp, 0L)
extern	_LONG			D_Telldir _(( Directory * ));
extern	_VOID			D_Closedir _(( Directory * ));

EXTERN F_ERROR F_Getinfo _(( CONST _UBYTE *name, F_STAT *st, _BOOL followLinks ));

typedef struct {
	_ULONG block_size;
	_ULONG total_size;
	_DOUBLE free_size;		/* total free space */
	_DOUBLE avail_size;		/* free space available to user */ 
	_UBYTE label[PATH_MAX+1];
} DiskINFO;

EXTERN _BOOL D_Info (CONST _UBYTE *path, DiskINFO *d);
EXTERN _LONG D_GetDiskFree (CONST _UBYTE *path);
EXTERN _WORD D_Filesystems(FILESYSTEM *fs, _WORD max_fs, _BOOL all);
EXTERN _BOOL D_SameDevice(CONST _UBYTE *path1, CONST _UBYTE *path2);



#endif /* __DIR_IO_H__ */
