#ifndef __CAT_GROUPSELECT_H__
#define __CAT_GROUPSELECT_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */




#ifndef __MOS_LISTS_H__
#include <lists.h>
#endif

#ifndef __CAT_CATTYPES_H__
#include <cattypes.h>
#endif

#ifndef __CAT_QUICKSORT_H__
#include <quicksor.h>
#endif

/*DYJ 5.35c Start*/
typedef _UWORD lineSet;
#define LS_GROUP_NAME    0x0001
#define LS_NET_LINE      0x0002
#define LS_KURZTEXT      0x0004
#define LS_CHEF_LINE     0x0008
#define LS_FLAG_LINE     0x0010
#define LS_SYSOP_LINE    0x0020
#define LS_FOLLOWUP_LINE 0x0040
#define LS_HERKUNFT      0x0080
#define LS_SPRACHE       0x0100
#define LS_ALIAS         0x0200
#define LS_CAT_NUMBER    0x0400
#define LS_READ_NUMBER   0x0800
#define LS_ORDERED       0x1000
#define LS_ORDER_DAYS    0x2000
#define LS_FLAG_LINE_Y   0x4000
#define LS_ALL           0x3fff
/*DYJ  Ende;*/

typedef enum _selSortType {
	sName,
	sNet,
	sLang,
	sRead		/*DYJ 5.35c */
} selSortType;

typedef enum _selDispType {
	dAll,
	dLocal,
	dOrdered,
	dPseudo,
	dDatenb			/*DYJ 5.35c */
} selDispType;

typedef enum _rwState {
	rwNA,
	rwOn,
	rwPossible,
	rwAskChef,
	rwAskSysop,
	rwImpossible,
	rwDefault,
	rwNoPermission
} rwState;

typedef enum _uState {
	uNA,
	uIsChef,
	uIsUser,
	uNoUser
} uState;

typedef enum _netState {
	nPrivate=-2,
	nOeffentl,
	nNA,
	nNet,
	nLokal,
	nNetLokal,
	nNetForbid
} netState;
/* Fr jeden NetState steht im nchsten String ein P wenn es eine Persnliche,
ein n wenn es eine Pseudo-Newsgroup, ein N fr eine normale Newsgroup
und bei nNA ein ? */
#define GROUP_TYPE_CHAR "pn NNNN" 	/*DYJ 5.35b */
typedef enum _orderState {
	oNA,
	oOn,
	oOff
} orderState;

typedef struct group {
	_UBYTE *name;   		/* G-Zeile, Gruppenname */
	_UBYTE *netname;		/* N-Zeile, Netzweiter Name */ 	/*DYJ 5.35c wird bei Tausch/Gruppen als alter name mibraucht*/
	_UBYTE *chef;			/* C-Zeile, Gruppen-Chef */
	_UBYTE *kurztext;		/* :-Zeile, Kurzbeschreibung */
	_UBYTE *sysop;			/* S-Zeile, Sysop */
	_UBYTE *followup;		/* R-Zeile, Follow-Up */
	_UBYTE *herkunft;		/* H-Zeile */
	_UBYTE *sprache;		/* P-Zeile */
	_UBYTE *alias;			/* A-Zeile */
	CAT_GROUP_NUMBER readNumber;
	CAT_GROUP_NUMBER catNumber;
	rwState readState;		/* F-Zeile */
	rwState writeState;		/* F-Zeile */
	uState userState;		/* F-Zeile */
	netState isNet;			/* F-Zeile */
	_BOOL hasProg;			/* F-Zeile */
	_BOOL withDollar;		/* F-Zeile */
	_BOOL selected;			/* Fuer Darstellung in Liste */
	orderState ordered;		/* Fuer An-/Abbestellen von Gruppen */
	_WORD orderDays;		/* Madness-Erweiterung: Gruppe soviel Tage zurueck bestellen */
/*DYJ 5.35c Start*/
	_UWORD isUpdated;		/* Eintrag wurde aktualisiert, nur fuer ITG und fr Tausch/Gruppen*/
/*DYJ  Ende; alt:	
	_BOOL isUpdated;		/* Eintrag wurde aktualisiert, nur fuer ITG */
*/
	_UBYTE *display_name;	/* Entweder Name oder Alias-Name */
	data_len dataLength;	/* Die Laenge des folgenden Datenblocks */
	_VOID *data;			/* Block mit Daten fuer Pointer oben */
} group, *groupPtr;
/* Achtung nderungen hier auch an GROUPTREE durchfhren*/

typedef struct{
	_UBYTE datName[PATH_MAX];
	_UBYTE parName[PATH_MAX];
	_UBYTE tabName[PATH_MAX];
	_UBYTE waiName[PATH_MAX];
} DB_NAMES;

typedef _UBYTE group_line[256];
typedef _UBYTE group_lineXXL[4096];


GLOBAL _VOID GroupSelect_SortAnyList(List *list, compProc proc);

_BOOL GroupSelect_ParseInfofile(_BOOL isIncr);
/* Parst die ITG bzw. IGK (wenn ITG nicht da) und updatet die Gruppenliste
 */

_VOID GroupSelect_ParseYGB(_VOID);
/* Parst YGB*/

_BOOL GroupSelect_SaveGroupList(_VOID);
/* Sichert die interne Gruppenliste in die Datei CATGROUP.INF
 */

_BOOL GroupSelect_SaveChangedGroupList(_VOID);
/* Sichert die interne Gruppenliste in die Datei CATGROUP.INF,
 * wenn sie seit dem letzten Einlesen irgendwie geaendert wurde
 */

_BOOL GroupSelect_ReadGroupList(_VOID);
/* Liest die interne Gruppenliste, gibt Fehlercode zurueck, falls nicht
 * moeglich. Falls die Liste nicht gelesen werden kann, muss CAT terminieren!
 */

/* Hier kommen jetzt Funktionen, die einzelne Listen oder Teillisten
 * den anderen Modulen zur Verfuegung stellen.
 */

group *create_private_group(_VOID);



_BOOL GroupSelect_GetReadList(List *list, _BOOL byReadNum, _BOOL getNew, _BOOL getUnread);
/* Diese Prozedur liefert eine Liste der Gruppen, die in CAT vorhanden
 * und damit lesbar sind. Die Gruppe 'PERSOENLICHE' ist mit enthalten.
 * Die Liste ist nach der Lesereihenfolge sortiert, die Gruppe 'PERSOENLICHE'
 * ist immer als erste Gruppe vorhanden (die Lesenummer davon kann nicht
 * geaendert werden).
 * FALSE: Liste konnte nicht erzeugt werden
 */
GLOBAL _BOOL GroupSelect_GetNoReadList(List *list);
/* Diese Prozedur liefert eine Liste der Gruppen, die in CAT nicht vorhanden
 * sind. Die Gruppe 'PERSOENLICHE' ist natrlich nicht enthalten.
 * Die Liste ist nach dem Namen sortiert
 * FALSE: Liste konnte nicht erzeugt werden
 */

_BOOL GroupSelect_UpdateReadList(List *list);
/* Hiermit wird das Modul aufgefordert, die Lesereihenfolge zu aendern.
 * Es muss als Parameter eine Liste uebergeben werden, die eine geaenderte
 * Lesereihenfolge hat und die ueber GetReadList angefordert wurde.
 * FALSE: Interne Liste konnte nicht upgedatet werden
 */

_BOOL GroupSelect_GetUserList(List *list);
/* Diese Prozedur liefert eine Liste der Gruppen, in denen man Chef ist.
 * Diese wird benoetigt, um User in andere Gruppen einzutragen.
 * FALSE: Liste konnte nicht erzeugt werden
 */

_BOOL GroupSelect_GetProgteilList(List *list);
/* Diese Prozedur liefert eine Liste der Gruppen, die einen Programm-
 * teil haben. Diese Liste wird fuer die Programmteil-Kommandos benutzt.
 * FALSE: Liste konnte nicht erzeugt werden
 */
 
_BOOL GroupSelect_GetWriteList(List *list, _BOOL full);
/* Diese Prozedur liefert eine Liste der Gruppen, in denen man schreiben
 * kann. Diese Liste wird dann benutzt, wenn man eine neue Nachricht in
 * eine Gruppe schreiben will.
 * FALSE: Liste konnte nicht erzeugt werden
 */

_BOOL GroupSelect_GetSelectList(List *list, selDispType disp, selSortType sortBy);
/* Diese Prozedur liefert eine Liste der Gruppen, die in der Maus vorhanden
 * sind bzw. alle Gruppen aus dem ITG. Benutzt wird diese Liste fuer die
 * Gruppenauswahl.
 * lokal: Es werden nur die lokal vorhandenen Gruppen geliefert
 * FALSE: Liste konnte nicht erzeugt werden
 */

_BOOL GroupSelect_GetUsernameList(List *list);
/* (MH)
 * Diese Prozedur liefert eine Liste der User, die in der ITU (Technische
 * Userliste) enthalten sind. Benutzt wird diese Liste fr die Namensauswahl
 * im ITK-Kommando-Modul.
 * FALSE: Liste konnte nicht erzeugt werden (ITU ist nicht vorhanden etc.)
 */

_BOOL GroupSelect_UpdateSelectList(List *list, _BOOL resort);
/* Hiermit wird das Modul aufgefordert, sich zu merken, welche Gruppen
 * beim naechsten Tausch bestellt oder abbestellt werden sollen.
 * Es muss als Parameter eine Liste uebergeben werden,
 * die ueber GetSelectList angefordert wurde
 * FALSE: Interne Liste konnte nicht upgedatet werden
 */

_VOID GroupSelect_DeleteList(List *list);
/* Loescht eine Liste vollstaendig, also auch die Wurzel davon. Danach darf
 * nicht mehr auf die Liste zugegriffen werden.
 */

_BOOL GroupSelect_NameToFile(CONST _UBYTE *name, CONST _UBYTE *path, CONST _UBYTE *fname);
/* Schreibt den Namen 'name' an das Ende des Files 'path\fname'
 */

GLOBAL _BOOL GroupSelect_ChgGroup(CONST _UBYTE *oldName, CONST _UBYTE *newName,CONST _UBYTE newF,CONST _UBYTE *newTxt, _BOOL saveGruppenInf);
/* Benennt eine Gruppe um,ndert den Netstatus und den Kurztext. Die GRUPPEN.INF wird danach direkt neu
 * geschrieben, die Aenderungen in den anderen Daten werden
 * am Programmende gesichert
 */

_BOOL GroupSelect_AppendGroup(CONST _UBYTE *groupName);
/* Neuen Gruppennamen uebernehmen, meckert selber, falls es nicht klappt */

_BOOL GroupSelect_GroupNumber(CONST _UBYTE *gName, CAT_GROUP_NUMBER *nr);
/* Gruppennummer zu einem String feststellen */
/* Wenn FALSE, dann kommt Gruppenanzahl in nr zurueck, Anzahl      */
/* inklusive persoenliche                                          */

CAT_GROUP_NUMBER GroupSelect_CountCatGroups(_VOID);
/* Liefert die Zahl der Gruppen in CAT zurueck, inklusive der persoenlichen Gruppe
 */

CAT_GROUP_NUMBER GroupSelect_CountCatGroupsMax(CAT_GROUP_NUMBER *maxNr);
/* Liefert die Zahl der Gruppen und die mit der hchsten Nummer in CAT
 zurueck, exclusiv der persoenlichen Gruppe
 */
LOCAL CAT_GROUP_NUMBER GroupSelect_CntCatNum(CAT_GROUP_NUMBER number);
/* Zhlt wieviele Gruppen die angegebene Catnummer haben, sollte 0 oder 1 sein*/

CAT_GROUP_NUMBER GroupSelect_NextGroupNumber(CAT_GROUP_NUMBER old);
/* Nummer der naechsten Gruppe in der Reihenfolge der momentanen Sortierung */
/* feststellen; 0FFFF = empty falls keine weiter Gruppe vorhanden ist      */

CAT_GROUP_NUMBER GroupSelect_PreviousGroupNumber(CAT_GROUP_NUMBER old);
/* Nummer der vorhergehenden Gruppe in der Reihenfolge der momentanen Sortierung */
/* feststellen; 0FFFF = empty falls keine weiter Gruppe vorhanden ist      */

_BOOL GroupSelect_NetAliasGroupNumber(CONST _UBYTE *gName, CAT_GROUP_NUMBER *nr);
/* Gruppennummer zu einem String mit einem Netzgruppennamen oder Alias feststellen */
/* Wenn FALSE, dann kommt Gruppenanzahl in nr zurueck, Anzahl      */
/* inklusive persoenliche                                          */

_VOID GroupSelect_GroupName(CAT_GROUP_NUMBER nr, _UBYTE *name, size_t namesize);
/* Gruppenname zu einer Nummer feststellen, liefert im Fehlerfall */
/* keinen Eintrag sondern leeren Namen */

_VOID GroupSelect_2GroupNames(CAT_GROUP_NUMBER nr, _UBYTE *name1, _UBYTE *name2, size_t namesize);
/* 2 Gruppennamen zu einer Nummer feststellen, liefert im Fehlerfall */
/* keinen Eintrag sondern leere Namen, nur sinnvoll bei fehlerbedingter
	doppelter Zuordnung einer Nummer */

_VOID GroupSelect_GroupDisplayName(CAT_GROUP_NUMBER nr, _UBYTE *name, size_t namesize);
/* Gruppenname zu einer Nummer feststellen, liefert im Fehlerfall */
/* keinen Eintrag sondern leeren Namen */

/* Fuer die Gruppenaenderungen, die an die Maus geschickt werden */

typedef enum {
	GFIRST, GNEXT
} gcMode;


_BOOL GroupSelect_GetGroupChange(_UBYTE *str, gcMode mode);
/* Liefert bei TRUE in str einen String mit +gname oder -gname,
 * je nachdem, ob eine Gruppe angestellt oder abgestellt werden soll
 */

_VOID GroupSelect_ClearGroupChange(CONST _UBYTE *name);
/* Loescht eine Gruppenaenderung fuer die Gruppe name
 */

/*DYJ 5.35c Start*/
_VOID GroupSelect_SetGrpPar(group *gr, CONST _UBYTE *name,lineSet set);
_VOID GroupSelect_SetGrpParN(CONST _UBYTE *gName, CONST _UBYTE *name,lineSet set);
/* Setzt einen Parameter fuer die Gruppe gr */
/*DYJ  Ende; alt:	
_VOID GroupSelect_SetFollowUp(group *gr, CONST _UBYTE *name);
_VOID GroupSelect_SetFollowUpN(CONST _UBYTE *gName, CONST _UBYTE *name); 	/*DYJ 5.34d */
/* Setzt das Follow-Up fuer die Gruppe group
 */
_VOID GroupSelect_SetChefN(CONST _UBYTE *gName, CONST _UBYTE *chef);	/*DYJ 5.34d */
/* Setzt den Gruppenchef
 */
*/

_BOOL GroupSelect_GetFollowUp(CONST _UBYTE *gName, _UBYTE *followupTo);
/* Setzt das Follow-Up fuer die Gruppe group
 */

_VOID GroupSelect_GetNetName(_UBYTE *name);
/* Liefert zu einer Gruppe den Netzgruppennamen
 */

_BOOL GroupSelect_GetWriteAccess(CONST _UBYTE *name);
/* Liefert TRUE zurueck, falls die Gruppe existiert und der
 * Benutzer laut ITG schreiben darf
 */

/* Funktionen fuer CatPutz: */

_VOID GroupSelect_DeleteGroup(CONST _UBYTE *gName);
/* Loescht die Gruppe aus der Liste der in CAT verfuegbaren Gruppen und
 * passt die Lesenummern an. Die Filenummern fuer die Datenbank
 * muessen anders angepasst werden (z.B. mit SetNewCatNumber).
 * Ebenso muss das Gruppen.Pos angepasst werden.
 */

_VOID GroupSelect_SetNewCatNumber(CONST _UBYTE *gName, CAT_GROUP_NUMBER newNumber);
/* Setzt eine neue Gruppennummer fuer eine Gruppe.
 * Das GRUPPEN.INF wird danach neu geschrieben
 */

_VOID GroupSelect_SaveGruppenInf(_VOID);
/* Sichert die Liste der Gruppen komplett
 */

GLOBAL _BOOL GroupSelect_isPrivate(CONST CAT_GROUP_NUMBER idx);
/* Gibt zurck ob es eine PM Gruppe ist
*/

GLOBAL _BOOL GroupSelect_isPseudoOM(CONST CAT_GROUP_NUMBER idx);	/*DYJ 5.34d */
/* Gibt zurck ob es eine PseudoNews Gruppe ist
*/

 _VOID GroupSelect_setPseudo(CONST CAT_GROUP_NUMBER catNumber,_BOOL private);	/*DYJ 5.34d */
/* Setzt eine Gruppe auf Pseude-PM oder Pseudo-M
*/

GLOBAL CAT_GROUP_NUMBER GroupSelect_NextPrivGroupNumber(CAT_GROUP_NUMBER old);
/* Gibt nchste private Gruppe nach idx zurck oder NO_GROUP
*/
 _BOOL	GroupSelect_tstCatNr(_VOID);
CAT_GROUP_NUMBER GroupSelect_FreeCatNum(_VOID);

LOCAL _VOID GroupSelect_makeFilenames(CAT_GROUP_NUMBER group, _BOOL tmpDir, DB_NAMES *dbf,_UBYTE *tmpPath);

_VOID GroupSelect_Exit(_VOID);


#endif /* __CAT_GROUPSELECT_H__ */
