#ifndef __CAT_INFOFILES_H__
#define __CAT_INFOFILES_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */




#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __CAT_LISTS_H__
#include <lists.h>
#endif
#ifndef __CAT_DATASYS_H__
#include <datasys.h>
#endif


typedef enum {
	IFIRST, INEXT
} infMode;

typedef enum {
	cNa, cWithCrc, cNoCrc
} tCrcState;

typedef union infEntry {
	struct {
		CRC16 crc;
		_WORD /* _BOOL */ new;
		_WORD /* _BOOL */ avail;
		_WORD /* _BOOL */ isMaus;
		_WORD /* _BOOL */ ordered;
		_WORD orderIntervall;			/* -1: Einmalig, 0: Immer, 1: taeglich,
		                                 * 0-6 bei ITI_ORDERED_WDAY (MH)
		                                 */
#define ORDER_ONCE -1
#define ORDER_ALWAYS 0
		_ULONG lastOrdered;				/* Datum in Tagen seit dem 1.1.1900 */
		_WORD /* _BOOL */ selected;
		_UWORD /* tCrcState */ hasCrc;
		_UWORD res;
#define ITI_IS_IN_ITI     0x0001		/* Ist der Eintrag gerade in der ITI gekommen? */
#define ITI_TITLE         0x0002		/* Ist der Eintrag eine Titelueberschrift? */
#define ITI_GROUP_VISIBLE 0x0004		/* Ueberschrift -> Gruppe sichtbar? */
#define ITI_ENTRY_VISIBLE 0x0008		/* Eintrag -> Eintrag sichtbar? */
#define ITI_GROUP_MASK    0x00f0		/* Titelgruppe */
#define ITI_GROUP_SHIFT   4
#define   ITI_INFOFILE_I  0x0000
#define   ITI_INFOFILE_J  0x0010
#define   ITI_INFOFILE_O  0x0020
#define   ITI_INFOFILE_S  0x0030
#define   ITI_INFOFILE_U  0x0040
#define ITI_ORDERED_NOW   0x0200		/* Sofort bestellen? (MH) */
#define ITI_ORDERED_WDAY  0x0400		/* Anforderung nach Wochentag? (MH) */
#define   ITI_INFOFILE_Y  0x0800
/* restliche 5 Bits sind reserviert */

		_UBYTE mausShort[10];
		_UBYTE mausName[256];
		_UBYTE filename[256 /* PATH_MAX */];
	} PACKED i;
	_UBYTE data[1024];
} infEntry;

#define INF_ALLOC_SIZE(fname) (offsetof(infEntry, i.filename) + strLen(fname) + 1)

#define DEFAULT_OUTBLOCK "OUT-Block"

extern List infList;


GLOBAL _WORD Infofiles_GetInfoGroupNumber(infEntry *info);
/* Gibt die Infofile-Gruppennummer zu einem infEntry zurueck 
 */

_BOOL Infofiles_IsNeededInfo(CONST _UBYTE *str);
/* Gibt zurueck, ob es ein Infofile ist, das von CAT immer
 * benoetigt wird
 */

_BOOL Infofiles_LoadInfoInf(_BOOL dbChanged, _BOOL clearNew);

_BOOL Infofiles_SaveInfoInf(_VOID);

_VOID Infofiles_SetInfoCrc(CONST _UBYTE *inf, CONST _UBYTE *str);
/* Setzt die CRC fuer das Infofile inf. Falls das Infofile nicht
 * gefunden wird, dann wird gar nichts gesetzt.
 */

_BOOL Infofiles_GetInfoCrc(_UBYTE *str, infMode mode);
/* Liefert den Namen eines Infofiles inkl. Crc zurueck. Bei Mode = IFIRST wird das erste Infofile
 * zurueckgeliefert, bei Mode = INEXT wird der naechste zurueckgeliefert.
 * Wenn alle Infostrings geliefert wurden, dann wird FALSE zurueckgegeben
 * (kann auch bei IFIRST passieren).
 * Zwischen den Aufrufen sollte nicht anders auf die Liste
 * zugegriffen werden, da ansonsten evtl. nicht alle angefordert
 * werden oder einige doppelt angefordert werden.
 */

_BOOL Infofiles_AddEntry(
	_BOOL isMausInf,
	_BOOL order,
	_BOOL avail,
	_BOOL new,
	CRC16 crc,
	_WORD inter,
	_ULONG lastOrder,
	CONST _UBYTE *shortname,
	CONST _UBYTE *name,
	CONST _UBYTE *file,
	_BOOL saveInf,
	tCrcState withCrc);
/* Fuegt einen Eintrag fuer ein Infofile an die Liste an.
 * Bei einem Mausinfofile ist die Angabe des Namens ueberfluessig,
 * da dort immer das Kuerzel mit der Extension .TXT genommen wird.
 */

_BOOL Infofiles_DeleteEntry(CONST _UBYTE *shortname);
/* Entfernt ein Infofile aus der Liste */

_ULONG Infofiles_GetInfoFilename(CONST _UBYTE *shortname, _UBYTE *fname);
/* Liefert zum Kuerzel des Infofiles den Filenamen.
 * FALSE: Infofile nicht vorhanden
 */

_BOOL Infofiles_IsInfoFile(CONST _UBYTE *shortname, _BOOL onlyMaus);
/* Liefert zurueck, ob ein Kuerzel ein Infofile ist oder nicht
 */

_BOOL Infofiles_Available(CONST _UBYTE *shortname, _BOOL onlyMaus);
/* Liefert zurueck, ob ein Infofile verfuegbar ist
 */

_VOID Infofiles_CleanUpOutBlocks(_VOID);
/* Aktualisiert die Liste der OUT-Bloecke (MH)
 */

_VOID Infofiles_ResetOrderedNowFlag(_VOID);
/* Setzt das ORDERED_NOW-Flag zurueck (MH)
 */

_ULONG Infofiles_BuildName(infEntry *info, _UBYTE *fname);

_VOID Infofiles_SortList(List *l);

_VOID Infofiles_Exit(_VOID);

#endif /* __CAT_INFOFILES_H__ */
