#ifndef __MINTBIND_H__
#define __MINTBIND_H__

#ifndef _OSBIND_H
#include <osbind.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __XATTR
#define __XATTR
typedef struct
{
        unsigned short mode;
        long           index;
        unsigned short dev;
        unsigned short rdev;
        unsigned short nlink;
        unsigned short uid;
        unsigned short gid;
        long           size;
        long           blksize;
        long           nblocks;
        short          mtime;
        short          mdate;
        short          atime;
        short          adate;
        short          ctime;
        short          cdate;
        short          attr;
        short          reserved2;
        long           reserved3[2];
} XATTR;
#endif

#ifndef __MSG
#define __MSG
typedef struct
{
        long     msg1;
        long     msg2;
        short    pid;
} MSG;
#endif

/* The requests for Dpathconf() */
#define DP_IOPEN	0	/* internal limit on # of open files */
#define DP_MAXLINKS	1	/* max number of hard links to a file */
#define DP_PATHMAX	2	/* max path name length */
#define DP_NAMEMAX	3	/* max length of an individual file name */
#define DP_ATOMIC	4	/* # of bytes that can be written atomically */
#define DP_TRUNC	5	/* file name truncation behavior */
#	define	DP_NOTRUNC	0	/* long filenames give an error */
#	define	DP_AUTOTRUNC	1	/* long filenames truncated */
#	define	DP_DOSTRUNC	2	/* DOS truncation rules in effect */
#define DP_CASE		6
#	define	DP_CASESENS	0	/* case sensitive */
#	define	DP_CASECONV	1	/* case always converted */
#	define	DP_CASEINSENS	2	/* case insensitive, preserved */
#define DP_MODEATTR		7
#	define	DP_ATTRBITS	0x000000ffL	/* mask for valid TOS attribs */
#	define	DP_MODEBITS	0x000fff00L	/* mask for valid Unix file modes */
#	define	DP_FILETYPS	0xfff00000L	/* mask for valid file types */
#	define	DP_FT_DIR	0x00100000L	/* directories (always if . is there) */
#	define	DP_FT_CHR	0x00200000L	/* character special files */
#	define	DP_FT_BLK	0x00400000L	/* block special files, currently unused */
#	define	DP_FT_REG	0x00800000L	/* regular files */
#	define	DP_FT_LNK	0x01000000L	/* symbolic links */
#	define	DP_FT_SOCK	0x02000000L	/* sockets, currently unused */
#	define	DP_FT_FIFO	0x04000000L	/* pipes */
#	define	DP_FT_MEM	0x08000000L	/* shared memory or proc files */
#ifndef DP_XATTRFIELDS
#define DP_XATTRFIELDS 8		/* information about supported extended attributes */
#  define   DP_INDEX    (0x0001)    /* index field unique for every file on the fs */
#  define   DP_DEV      (0x0002)    /* device field valid */
#  define   DP_RDEV     (0x0004)    /* rdev field valid (and not identical to dev) */
#  define   DP_NLINK    (0x0008)    /* number of links valid */
#  define   DP_UID      (0x0010)    /* user id valid */
#  define   DP_GID      (0x0020)    /* group id valid */
#  define   DP_BLKSIZE  (0x0040)    /* block size valid */
#  define   DP_SIZE     (0x0080)    /* size field valid (and meaningful!) */
#  define   DP_NBLOCKS  (0x0100)    /* number of blocks valid */
#  define   DP_ATIME    (0x0200)    /* file system has last access time */
#  define   DP_CTIME    (0x0400)    /* file system has last status change time */
#  define   DP_MTIME    (0x0800)    /* file system has last modification time */
#endif
#define DP_VOLNAMEMAX 9         /* maximum length of a volume name (0 if volume names not supported) */
#define DP_MAXREQ	9 /* Dpathconf(-1) */			/* highest legal request */

/* Dpathconf and Sysconf return this when a value is not limited
   (or is limited only by available memory) */

#define UNLIMITED	0x7fffffffL

/* Dcntl requests */

#define MFS_BASE	0x100
#define MFS_VERIFY	(MFS_BASE)	/* Return minixfs magic number */
#define MFS_SYNC	(MFS_BASE|0x01)	/* Sync the filesystem */
#define MFS_CINVALID	(MFS_BASE|0x02)	/* Invalidate cache entries */
#define MFS_FINVALID	(MFS_BASE|0x03)	/* Invalidate Fileptrs */
#define MFS_INFO	(MFS_BASE|0x04)	/* Get info about filesystem */
#define MFS_USAGE	(MFS_BASE|0x05)	/* Get block allocation of a file */
#define MFS_IMODE	(MFS_BASE|0x06)	/* Change all bits in an inode mode */
#define MFS_GTRANS	(MFS_BASE|0x07) /* Get filename translation mode */ 
#define MFS_STRANS	(MFS_BASE|0x08) /* Set filename translation mode */
#define MFS_PHYS	(MFS_BASE|0x09) /* Get physical partition info */
#define MFS_IADDR	(MFS_BASE|0x0a) /* Get start address of minixfs */
#define MFS_UPDATE	(MFS_BASE|0x0b) /* Update daemon controls */
#define MFS_MOUNT	(MFS_BASE|0x0c) /* Mount a filesystem */
#define MFS_UMOUNT	(MFS_BASE|0x0d) /* Umount a filesystem */
#define MFS_LOPEN	(MFS_BASE|0x0e) /* Get list of open files on device */
#define MFS_MKNOD	(MFS_BASE|0x0f) /* Make special file */

#define MFS_MAGIC	0x18970431L		/* Magic number from MFS_VERIFY */

#define DEV_NEWTTY	0xde00
#define DEV_NEWBIOS	0xde01
#define DEV_INSTALL	0xde02

#define FS_INSTALL    0xf001  /* let the kernel know about the file system */
#define FS_MOUNT      0xf002  /* make a new directory for a file system */
#define FS_UNMOUNT    0xf003  /* remove a directory for a file system */
#define FS_UNINSTALL  0xf004  /* remove a file system from the list */


#define EXT2FS_BASE		0x100
#define EXT2FS_VERIFY	(EXT2FS_BASE)		/* Return ext2fs magic number */
#define EXT2FS_SYNC		(EXT2FS_BASE|0x01)	/* Sync the filesystem */
#define EXT2FS_CINVALID	(EXT2FS_BASE|0x02)	/* Invalidate cache entries */
#define EXT2FS_FINVALID	(EXT2FS_BASE|0x03)	/* Invalidate Fileptrs */
#define EXT2FS_INFO		(EXT2FS_BASE|0x04)	/* Get info about filesystem */
#define EXT2FS_USAGE	(EXT2FS_BASE|0x05)	/* Get block allocation of a file */
#define EXT2FS_IMODE	(EXT2FS_BASE|0x06)	/* Change all bits in an inode mode */
#define EXT2FS_GTRANS	(EXT2FS_BASE|0x07)	/* Get filename translation mode */ 
#define EXT2FS_STRANS	(EXT2FS_BASE|0x08) 	/* Set filename translation mode */
#define EXT2FS_PHYS		(EXT2FS_BASE|0x09) 	/* Get physical partition info */
#define EXT2FS_IADDR	(EXT2FS_BASE|0x0a)	/* Get start address of EXT2FS */
#define EXT2FS_UPDATE	(EXT2FS_BASE|0x0b)	/* Update daemon controls */
#define EXT2FS_MOUNT	(EXT2FS_BASE|0x0c)	/* Mount a filesystem */
#define EXT2FS_UMOUNT	(EXT2FS_BASE|0x0d)	/* Umount a filesystem */
#define EXT2FS_LOPEN	(EXT2FS_BASE|0x0e)	/* Get list of open files on device */
#define EXT2FS_MKNOD	(EXT2FS_BASE|0x0f)	/* Make special file */
#define EXT2FS_LINODES	(EXT2FS_BASE|0x20)	/* list cached inodes */

#define EXT2FS_MAGIC	0x81794013l			/* Magic number from EXT2FS_VERIFY */

/* Sysconf() values */
#ifndef _SC_LAST
#define _SC_LAST	-1	/* max limit # for sysconf - NOT POSIX */
#define _SC_MEMR_MAX	0	/* memory regions per process - NOT POSIX */
#define _SC_ARG_MAX	1	/* max length of cmdln - NOT quite POSIX */
#define _SC_OPEN_MAX	2	/* max # of open files per process - POSIX */
#define _SC_NGROUPS_MAX	3	/* max # supp gids - POSIX */
#define _SC_CHILD_MAX	4	/* max # processes/user - NOT quite POSIX */
#endif


/* Psemaphore */

#define PSEM_CRGET       0                                  /* MTOS/MagiC 3.0 */
#define PSEM_DESTROY     1
#define PSEM_GET         2
#define PSEM_RELEASE     3


/* Pget/setpriotity modes */
# ifndef MIN_NICE
# define MIN_NICE -20
# endif
# ifndef MAX_NICE
# define MAX_NICE 20
# endif

# define PRIO_MIN MIN_NICE   /* The smallest valid priority value.  */
# define PRIO_MAX MAX_NICE   /* The highest valid priority value.  */

/* Values for the first argument to `getpriority' and `setpriority'.  */
#define PRIO_PROCESS 0	/* Read or set the priority of one process. */
#define PRIO_PGRP    1	/* Read or set the priority of one process group. */
#define PRIO_USER    2	/* Read or set the priority of one user's processes */

/* Ssystem modes */
#define S_OSNAME		0
#define S_OSXNAME	1
#define S_OSVERSION	2
#define S_OSHEADER	3
#define S_OSBUILDDATE	4
#define S_OSBUILDTIME	5
#define S_OSCOMPILE	6
#define S_OSFEATURES	7
#define S_GETCOOKIE	8
#define S_SETCOOKIE	9
#define S_GETLVAL	10
#define S_GETWVAL	11
#define S_GETBVAL	12
#define S_SETLVAL	13
#define S_SETWVAL	14
#define S_SETBVAL	15
#define S_SECLEVEL	16
#define S_RUNLEVEL	17	/* currently disabled, reserved */
#define S_TSLICE		18
#define S_FASTLOAD	19
#define S_SYNCTIME	20	/* reserved, use Dcntl */
#define S_RESERVED_21	21	/* reserved, use Dcntl */
#define S_CLOCKMODE	100
#define S_TIOCMGET  0x54f8 /* reserved for MiNT-Lib */

#define FEAT_MP	1	/* memory protection enabled */
#define FEAT_VM	2	/* virtual memory manager enabled */


/* MagiC 5.20 Share Library Support */
typedef struct _shared_lib { int dummy; } *SHARED_LIB;
typedef long __CDECL (*SLB_EXEC)(SHARED_LIB slb, long fn, int nwords, ...);

#if defined(__TURBOC__) || (defined(__GNUC__) && !defined(__mc68000__))

/* we supply a library of bindings for TurboC / PureC */

int Syield( void );				/* GEMDOS 0xff */
int Fpipe( short *ptr );			/* GEMDOS 0x100 */
long Fcntl( int f, long arg, int cmd);		/* GEMDOS 0x104 */
#define Fcntl(f, arg, cmd) Fcntl(f, (long)(arg), cmd)
long Finstat( int f );				/* GEMDOS 0x105 */
long Foutstat( int f );				/* GEMDOS 0x106 */
long Fgetchar(int f, int mode);			/* GEMDOS 0x107 */
long Fputchar( int f, long c, int mode ); 	/* GEMDOS 0x108 */
long Pwait( void );				/* GEMDOS 0x109 */
int Pnice( int delta );				/* GEMDOS 0x10a */
int Pgetpid( void );				/* GEMDOS 0x10b */
int Pgetppid( void );				/* GEMDOS 0x10c */
int Pgetpgrp( void );				/* GEMDOS 0x10d */
int Psetpgrp(int pid, int newgrp);		/* GEMDOS 0x10e */
int Pgetuid( void );				/* GEMDOS 0x10f */
int Psetuid( int id );				/* GEMDOS 0x110 */
int Pkill( int pid, int sig );			/* GEMDOS 0x111 */
long Psignal(int sig, long handler);		/* GEMDOS 0x112 */
#define Psignal(sig, handler) Psignal(sig, (long)(handler))
long Pvfork( void );				/* GEMDOS 0x113 */
int Pgetgid( void );				/* GEMDOS 0x114 */
int Psetgid(int id);				/* GEMDOS 0x115 */
long Psigblock(unsigned long mask);		/* GEMDOS 0x116 */
long Psigsetmask(unsigned long mask);		/* GEMDOS 0x117 */
long Pusrval(long arg);				/* GEMDOS 0x118 */
int Pdomain(int newdom);			/* GEMDOS 0x119 */
__VOID Psigreturn( void );			/* GEMDOS 0x11a */
long Pfork( void );				/* GEMDOS 0x11b */
long Pwait3(int flag, long *rusage);		/* GEMDOS 0x11c */
int Fselect(unsigned int timeout, long *rfds, long *wfds, long *xfds); /* GEMDOS 0x11d */
int Prusage( long r[8] );			/* GEMDOS 0x11e */
long Psetlimit(int lim, long value);		/* GEMDOS 0x11f */
long Talarm( long secs );			/* GEMDOS 0x120 */
__VOID Pause( void );				/* GEMDOS 0x121 */
long Sysconf( int n );				/* GEMDOS 0x122 */
long Psigpending( void );			/* GEMDOS 0x123 */
long Dpathconf( const char *name, int n );	/* GEMDOS 0x124 */
long Pmsg( int mode, long mbox, void *msg ); 	/* GEMDOS 0x125 */
long Fmidipipe( int pid, int in, int out );	/* GEMDOS 0x126 */
int Prenice( int pid, int delta );		/* GEMDOS 0x127 */
long Dopendir( const char *name, int flag );	/* GEMDOS 0x128 */
long Dreaddir( int buflen, long dir, char *buf); /*GEMDOS 0x129 */
#define Dreaddir(buflen, dir, buf) Dreaddir(buflen, (long)(dir), buf)
long Drewinddir( long dir );			/* GEMDOS 0x12a */
#define Drewinddir(dir) Drewinddir((long)(dir))
long Dclosedir( long dir );			/* GEMDOS 0x12b */
#define Dclosedir(dir) Dclosedir((long)(dir))
long Fxattr( int flag, const char *name, XATTR *buf );	/* GEMDOS 0x12c */
long Flink( const char *oldname, const char *newname );	/* GEMDOS 0x12d */
long Fsymlink( const char *oldname, const char *newname );	/* GEMDOS 0x12e */
long Freadlink( int siz, char *buf, const char *name); /*GEMDOS 0x12f */
long Dcntl( int cmd, const char *name, long arg );	/* GEMDOS 0x130 */
long Fchown( const char *name, int uid, int gid);	/* GEMDOS 0x131 */
long Fchmod( const char *name, int mode );		/* GEMDOS 0x132 */
int Pumask( int mask );				/* GEMDOS 0x133 */
long Psemaphore(int mode, long id, long timeout); /* GEMDOS 0x134 */
int Dlock( int mode, int drive );		/* GEMDOS 0x135 */
long Psigpause( unsigned long mask);		/* GEMDOS 0x136 */
long Psigaction(int sig, long act, long oact);	/* GEMDOS 0x137 */
#define Psigaction(sig, act, oact) Psigaction(sig, (long)(act), (long)(oact));
int Pgeteuid( void );				/* GEMDOS 0x138 */
int Pgetegid( void );				/* GEMDOS 0x139 */
long Pwaitpid(int pid, int flag, long *rusage);	/* GEMDOS 0x13a */
long Dgetcwd(char *path, int drv, int size);	/* GEMDOS 0x13b */
long Salert(const char *msg);				/* GEMDOS 0x13c */
unsigned long Tmalarm(unsigned long millisecs);				/* GEMDOS 0x13d */
long    Psigintr(int vec, int sig);					/* GEMDOS 0x13e */
long	Suptime(unsigned long *cur_uptime, unsigned long loadave[3]); /* GEMDOS 0x13f */
long	Dxreaddir(int len, long handle, char *buf, XATTR *attr, long *xret); /* GEMDOS 0x142 */
long	Pseteuid(int id); /* GEMDOS 0x143 */
long	Psetegid(int id); /* GEMDOS 0x144 */
long	Pgetauid(void); /* GEMDOS 0x145 */
long	Psetauid(int id); /* GEMDOS 0x146 */
long	Pgetgroups(int gidsetlen, _gid_t gidset[]); /* GEMDOS 0x147 */
long	Psetgroups(int ngroups, _gid_t gidset[]); /* GEMDOS 0x148 */
/*long	Tsetitimer(int which, long *interval, long *value, long *ointeral, long *ovalue); /* GEMDOS 0x149 */
DYJ: Stimmt nicht mit TOS.HYP berein, bevor man es benutzt sollte das abgeklrt werden*/				
long	Scookie(int action, void *yummy); /* GEMDOS 0x14a */
long	Psetreuid(int rid, int eid); /* GEMDOS 0x14e */
long	Psetregid(int rid, int eid); /* GEMDOS 0x14f */
long	Sync(void); /* GEMDOS 0x150 */
long	Shutdown(long restart); /* GEMDOS 0x151 */
long	Dreadlabel(const char *path, char *label, int maxlen); /* GEMDOS 0x152 */
long	Dwritelabel(const char *path, const char *label); /* GEMDOS 0x153 */
long    Ssystem(int mode, long arg1, long arg2); /* GEMDOS 0x154 */
#ifdef __TIME_H__
long    Tgettimeofday(struct timeval *tv, struct timezone *tz); /* GEMDOS 0x155 */
long    Tsettimeofday(struct timeval *tv, struct timezone *tz);  /* GEMDOS 0x156 */
long    Tadjtime(long adj); /* GEMDOS 0x157 */
#endif
long    Pgetpriority(int which, int who); /* GEMDOS 0x158 */
long    Psetpriority(int which, int who, int pri); /* GEMDOS 0x159 */
long    Srealloc(long size); /* GEMDOS 0x15 */


/* KAOS 1.2 */
long	Sconfig(int mode, long value);
long	Fshrink(int handle);
/* #define Fshrink(a)      Fwrite(a, 0L, (void *) -1L) */
long	Mgrow(void *block, long newsize);
/* #define Mgrow Mshrink */
long	Mblavail(void *block);
/* #define Mblavail(block) Mshrink(block, -1l) */

/* MagiC 5.20 Share Library Support */
long Slbopen(const char *name, const char *path, long min_ver, SHARED_LIB *slb, SLB_EXEC *slbexec); /* GEMDOS 0x16 */
long Slbclose(SHARED_LIB slb); /* GEMDOS 0x17 */

#endif /* __TURBOC__ */



#ifdef __LATTICE__

void _vmv(int);
short _smv(int);
int _imv(int);
long _lmv(int);

int _im8(int,long[8]);
void _vml(int,long);
short _sms(int,int);
short _smr(int,short *);
long _lms(int,int);
long _lml(int,long);
long _lmu(int,unsigned long);
long _lmp(int,void *);
long _lmC(int,const char *);

void _vmls(int,long,int);
short _smss(int,int,int);
short _smsl(int,int,long);
short _smls(int,long,int);

long _lmss(int,int,int);
long _lmsl(int,int,long);
long _lmsp(int,int,void *);
long _lmps(int,void *,int);
long _lmCs(int,const char *,short);
long _lmpp(int,void *,void *);
long _lmCC(int,const char *,const char *);

short _smsls(int,int,long,int);
int _imssq(int,int,int,long *);
long _lmsss(int,int,int,int);
long _lmsls(int,int,long,int);
long _lmsll(int,int,int,long);
long _lmslp(int,int,long,void *);
long _lmspl(int,int,void *,long);
long _lmsCl(int,int,const char *,long);
long _lmspp(int,int,void *,void *);
long _lmspC(int,int,void *,const char *);
long _lmlss(int,long,int,int);
long _lmCss(int,const char *,int,int);
long _lmcss(int,char *,int,int);
short _smsqqq(int,int,long *,long *,long *);

#pragma inline _vmv((short))	{register d2,a2; "4e41";}
#pragma inline d0=_smv((short))	{register d2,a2; "4e41";}
#pragma inline d0=_imv((short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmv((short))	{register d2,a2; "4e41";}

#pragma inline d0=_im8((short),)	{register d2,a2; "4e41";}
#pragma inline _vml((short),)	{register d2,a2; "4e41";}
#pragma inline d0=_sms((short),(short))	{register d2,a2; "4e41";}
#pragma inline d0=_smr((short),)	{register d2,a2; "4e41";}
#pragma inline d0=_lms((short),(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lml((short),)	{register d2,a2; "4e41";}
#pragma inline d0=_lmu((short),)	{register d2,a2; "4e41";}
#pragma inline d0=_lmp((short),)	{register d2,a2; "4e41";}
#pragma inline d0=_lmC((short),)	{register d2,a2; "4e41";}

#pragma inline _vmls((short),,(short))	{register d2,a2; "4e41";}
#pragma inline d0=_smss((short),(short),(short))	{register d2,a2; "4e41";}
#pragma inline d0=_smsl((short),(short),)	{register d2,a2; "4e41";}
#pragma inline d0=_smls((short),,(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmss((short),(short),(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmsl((short),(short),)	{register d2,a2; "4e41";}
#pragma inline d0=_lmsp((short),(short),)	{register d2,a2; "4e41";}
#pragma inline d0=_lmps((short),,(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmCs((short),,(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmpp((short),,)	{register d2,a2; "4e41";}
#pragma inline d0=_lmCC((short),,)	{register d2,a2; "4e41";}

#pragma inline d0=_smsls((short),(short),,(short))	{register d2,a2; "4e41";}
#pragma inline d0=_imssq((short),(short),(short),)	{register d2,a2; "4e41";}
#pragma inline d0=_lmsss((short),(short),(short),(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmsls((short),(short),,(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmsll((short),(short),(short),)	{register d2,a2; "4e41";}
#pragma inline d0=_lmlss((short),,(short),(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmslp((short),(short),,)	{register d2,a2; "4e41";}
#pragma inline d0=_lmspl((short),(short),,)	{register d2,a2; "4e41";}
#pragma inline d0=_lmsCl((short),(short),,)	{register d2,a2; "4e41";}
#pragma inline d0=_lmspp((short),(short),,)	{register d2,a2; "4e41";}
#pragma inline d0=_lmspC((short),(short),,)	{register d2,a2; "4e41";}
#pragma inline d0=_lmlss((short),,(short),(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmCss((short),,(short),(short))	{register d2,a2; "4e41";}
#pragma inline d0=_lmcss((short),,(short),(short))	{register d2,a2; "4e41";}
#pragma inline d0=_smsqqq((short),(short),,,)	{register d2,a2; "4e41";}

#define Syield() _smv(0xff)
#define Fpipe(a) _smr(0x100, a)
#define Fcntl(a, b, c) _smsls(0x104, a, (long)(b), c)
#define Finstat(a) _lms(0x105, a)
#define Foutstat(a) _lms(0x106, a)
#define Fgetchar(a, b) _lmss(0x107, a, b)
#define Fputchar(a, b, c) _lmsls(0x108, a, b, c)
#define Pwait() _lmv(0x109)
#define Pnice(a) _sms(0x10a, a)
#define Pgetpid() _smv(0x10b)
#define Pgetppid() _smv(0x10c)
#define Pgetpgrp() _smv(0x10d)
#define Psetpgrp(a, b) _smss(0x10e, a, b)
#define Pgetuid() _smv(0x10f)
#define Psetuid(a) _sms(0x110, a)
#define Pkill(a, b) _smss(0x111, a, b)
#define Psignal(a, b) _lmsl(0x112, a, b)
#define Pvfork() _smv(0x113)
#define Pgetgid() _smv(0x114)
#define Psetgid(a) _sms(0x115, a)
#define Psigblock(a) _lmu(0x116, a)
#define Psigsetmask(a) _lmu(0x117, a)
#define Pusrval(a) _lml(0x118, a)
#define Pdomain(a) _sms(0x119, a)
#define Psigreturn() _vmv(0x11a)
#define Pfork() _lmv(0x11b)
#define Pwait3(a, b) _lmsp(0x11c, a, b)
#define Fselect(a, b, c, d) _smsqqq(0x11d, a, b, c, d)
#define Prusage(a) _im8(0x11e, a)
#define Psetlimit(a, b) _lmsl(0x11f, a, b)
#define Talarm(a) _lml(0x120, a)
#define Pause() _vmv(0x121)
#define Sysconf(a) _lms(0x122, a)
#define Psigpending() _lmv(0x123)
#define Dpathconf(a, b) _lmCs(0x124, a, b)
#define Pmsg(a, b, c) _lmslp(0x125, a, b, c)
#define Fmidipipe(a, b, c) _lmsss(0x126, a, b, c)
#define Prenice(a, b) _smss(0x127, a, b)
#define Dopendir(a, b) _lmps(0x128, a, b)
#define Dreaddir(a, b, c) _lmslp(0x129, a, b, c)
#define Drewinddir(a) _lml(0x12a, a)
#define Dclosedir(a) _lml(0x12b, a)
#define Fxattr(a, b, c) _lmspp(0x12c, a, b, c)
#define Flink(a, b) _lmCC(0x12d, a, b)
#define Fsymlink(a, b) _lmCC(0x12e, a, b)
#define Freadlink(a, b, c) _lmspC(0x12f, a, b, c)
#define Dcntl(a, b, c) _lmsCl(0x130, a, b, (long)c)
#define Fchown(a, b, c) _lmCss(0x131, a, b, c)
#define Fchmod(a, b) _lmCs(0x132, a, b)
#define Pumask(a) _sms(0x133, a)
#define Psemaphore(a, b, c) _lmsll(0x134, a, b, c)
#define Dlock(a, b) _smss(0x135, a, b)
#define Psigpause(a) _vml(0x136,a)
#define Psigaction(a, b, c) _lmsll(0x137, a, (long)b, (long)c)
#define Pgeteuid() _imv(0x138)
#define Pgetegid() _imv(0x139)
#define Pwaitpid(a, b, c) _imssq(0x13a, a, b, c)
#define Dgetcwd(a, b, c) _lmcss(0x13b, a, b, c)
#define Salert(a) _lmC(0x13c, a)
#define Tmalarm(a) _lml(0x13d, a)

#endif /* !__LATTICE__ */

#if defined(__GNUC__) && defined(__mc68000__)

#define	Syield()						\
		(int)trap_1_w(0xff)
#define Fpipe(ptr)						\
		(int)trap_1_wl(0x100, (long)(ptr))
#define Fcntl(f, arg, cmd)					\
		trap_1_wwlw(0x104, (short)(f), (long)(arg), (short)(cmd))
#define Finstat(f)						\
		trap_1_ww(0x105, (short)(f))
#define Foutstat(f)						\
		trap_1_ww(0x106, (short)(f))
#define Fgetchar(f, mode)					\
		trap_1_www(0x107, (short)(f), (short)(mode))
#define Fputchar(f, ch, mode)					\
		trap_1_wwlw(0x108, (short)(f), (long)(ch), (short)(mode))

#define Pwait()							\
		trap_1_w(0x109)
#define Pnice(delta)						\
		(int)trap_1_ww(0x10a, (short)(delta))
#define Pgetpid()						\
		(int)trap_1_w(0x10b)
#define Pgetppid()						\
		(int)trap_1_w(0x10c)
#define Pgetpgrp()						\
		(int)trap_1_w(0x10d)
#define Psetpgrp(pid, grp)					\
		(int)trap_1_www(0x10e, (short)(pid), (short)(grp))
#define Pgetuid()						\
		(int)trap_1_w(0x10f)
#define Psetuid(id)						\
		(int)trap_1_ww(0x110, (short)(id))
#define Pkill(pid, sig)						\
		(int)trap_1_www(0x111, (short)(pid), (short)(sig))
#define Psignal(sig, handler)					\
		trap_1_wwl(0x112, (short)(sig), (long)(handler))
/* #define Pvfork()	trap_1_w(0x113) */
/* this is handled separately in vfork.cpp */
extern long Pvfork(void);
#define Pgetgid()						\
		(int)trap_1_w(0x114)
#define Psetgid(id)						\
		(int)trap_1_ww(0x115, (short)(id))
#define Psigblock(mask)						\
		trap_1_wl(0x116, (unsigned long)(mask))
#define Psigsetmask(mask)					\
		trap_1_wl(0x117, (unsigned long)(mask))
#define Pusrval(arg)						\
		trap_1_wl(0x118, (long)(arg))
#define Pdomain(arg)						\
		(int)trap_1_ww(0x119, (short)(arg))
#define Psigreturn()						\
		trap_1_w(0x11a)
#define Pfork()							\
		trap_1_w(0x11b)
#define Pwait3(flag, rusage)					\
		trap_1_wwl(0x11c, (short)(flag), (long)(rusage))
#define Fselect(time, rfd, wfd, xfd)				\
		(int)trap_1_wwlll(0x11d, (unsigned short)(time), (long)(rfd), \
				(long)(wfd), (long)(xfd))
#define Prusage(rsp)						\
		(int)trap_1_wl(0x11e, (long)(rsp))
#define Psetlimit(i, val)					\
		trap_1_wwl(0x11f, (short)(i), (long)(val))

#define Talarm(sec)						\
		trap_1_wl(0x120, (long)(sec))
#define Pause()							\
		trap_1_w(0x121)
#define Sysconf(n)						\
		trap_1_ww(0x122, (short)(n))
#define Psigpending()						\
		trap_1_w(0x123)
#define Dpathconf(name, which)					\
		trap_1_wlw(0x124, (long)(name), (short)(which))

#define Pmsg(mode, mbox, msg)					\
		trap_1_wwll(0x125, (short)(mode), (long)(mbox), (long)(msg))
#define Fmidipipe(pid, in, out)					\
		trap_1_wwww(0x126, (short)(pid), (short)(in),(short)(out))
#define Prenice(pid, delta)					\
		(int)trap_1_www(0x127, (short)(pid), (short)(delta))
#define Dopendir(name, flag)					\
		trap_1_wlw(0x128, (long)(name), (short)(flag))
#define Dreaddir(len, handle, buf)				\
		trap_1_wwll(0x129, (short)(len), (long)(handle), (long)(buf))
#define Drewinddir(handle)					\
		trap_1_wl(0x12a, (long)(handle))
#define Dclosedir(handle)					\
		trap_1_wl(0x12b, (long)(handle))
#define Fxattr(flag, name, buf)					\
		trap_1_wwll(0x12c, (short)(flag), (long)(name), (long)(buf))
#define Flink(old, new)						\
		trap_1_wll(0x12d, (long)(old), (long)(new))
#define Fsymlink(old, new)					\
		trap_1_wll(0x12e, (long)(old), (long)(new))
#define Freadlink(siz, buf, linknm)				\
		trap_1_wwll(0x12f, (short)(siz), (long)(buf), (long)(linknm))
#define Dcntl(cmd, name, arg)					\
		trap_1_wwll(0x130, (short)(cmd), (long)(name), (long)(arg))
#define Fchown(name, uid, gid)					\
		trap_1_wlww(0x131, (long)(name), (short)(uid), (short)(gid))
#define Fchmod(name, mode)					\
		trap_1_wlw(0x132, (long)(name), (short)(mode))
#define Pumask(mask)						\
		(int)trap_1_ww(0x133, (short)(mask))
#define Psemaphore(mode, id, tmout)				\
		trap_1_wwll(0x134, (short)(mode), (long)(id), (long)(tmout))
#define Dlock(mode, drive)					\
		(int)trap_1_www(0x135, (short)(mode), (short)(drive))
#define Psigpause(mask)						\
		trap_1_wl(0x136, (unsigned long)(mask))
#define Psigaction(sig, act, oact)					\
		trap_1_wwll(0x137, (short)(sig), (long)(act), (long)(oact))
#define Pgeteuid()						\
		(int)trap_1_w(0x138)
#define Pgetegid()						\
		(int)trap_1_w(0x139)
#define Pwaitpid(pid,flag, rusage)				\
		trap_1_wwwl(0x13a, (short)(pid), (short)(flag), (long)(rusage))
#define Dgetcwd(path, drv, size)				\
		trap_1_wlww(0x13b, (long)(path), (short)(drv), (short)(size))
#define Salert(msg)						\
		trap_1_wl(0x13c, (long)(msg))
/* The following are not yet official... */
#define Tmalarm(ms)						\
		trap_1_wl(0x13d, (long)(ms))
#define Psigintr(vec, sig)					\
		trap_1_www(0x13e, (short)(vec), (short)(sig))
#define Suptime(uptime, avenrun)				\
		trap_1_wll(0x13f, (long)(uptime), (long)(avenrun))
#define Dxreaddir(len, handle, buf, attr, xret) \
		trap_1_wwllll(0x142, (short)(len), (long)(handle), (long)(buf), (long)(attr), (long)(xret))
#define Pseteuid(id) \
		trap_1_ww(0x143, (short)(id))
#define Psetegid(id) \
		trap_1_ww(0x144, (short)(id))
#define Pgetauid() \
		trap_1_w(0x145)
#define Psetauid(id) \
		trap_1_ww(0x146, (short)(id))
#define Pgetgroups(gidsetlen, gidset) \
		trap_1_wwl(0x147, (short)(gidsetlen), (long)(gidset))
#define Psetgroups(ngroups, gidset) \
		trap_1_wwl(0x148, (short)(ngroups), (long)(gidset))
/*#define Tsetitimer(which, interval, value, ointerval, ovalue) \
		trap_1_wwllll(0x149, (short)(which), (long)(interval), (long)(value), (long)(ointerval), (long)(ovalue))
DYJ: Stimmt nicht mit TOS.HYP berein, bevor man es benutzt sollte das abgeklrt werden*/				
#define Scookie(action, yummy) \
		trap_1_wwl(0x14a, (short)(action), (long)(yummy))
#define Psetreuid(rid, eid) \
		trap_1_www(0x14e, (short)(rid), (short)(eid))
#define Psetregid(rid, eid) \
		trap_1_www(0x14f, (short)(rid), (short)(eid))
#define Sync()   \
		trap_1_w(0x150)
#define Shutdown(restart)  \
		trap_1_wl(0x151, (long)(restart))
#define Dreadlabel(path, label, maxlen)  \
		trap_1_wllw(0x152, (long)(path), (long)(label), (short)(maxlen))
#define Dwritelabel(path, label)  \
		trap_1_wll(0x153, (long)(path), (long)(label))
#define Ssystem(mode, arg1, arg2) \
		trap_1_wwll(0x154, (short)(mode), (long)(arg1), (long)(arg2))
#define Tgettimeofday(tv, tz) \
		trap_1_wll(0x155, (long)(tv), (long)(tz))
#define Tsettimeofday(tv, tz) \
		trap_1_wll(0x156, (long)(tv), (long)(tz))
#define Tadjtime(adj) \
		trap_1_wl(0x157, (long)(adj))
#define Pgetpriority(which, who) \
		trap_1_www(0x158, (short)(which), (short)(who))
#define Psetpriority(which, who, pri) \
		trap_1_wwww(0x159, (short)(which), (short)(who), (short)(pri))
#define Srealloc(size) \
		trap_1_wl(0x15, (long)(size))

#define Sconfig(mode, value) \
		trap_1_wwl(0x33, (short)(mode), (long)(value))
#define Mgrow(block, newsize) \
		trap_1_wwll(0x4a, (short)0, (long)(block), (long)(newsize))
#define Mblavail(block) \
		trap_1_wwll(0x4a, (short)0, (long)(block), (long)(-1))
#define Fshrink(a)      Fwrite(a, 0L, (void *) -1L)         /* KAOS 1.2 */

#define Slbopen(name, path, min_ver, slb, slbexec) \
		trap_1_wlllll(0x16, (long)(name), (long)(path), (long)(min_ver), (long)(slb), (long)(slbexec))
#define Slbclose(slb) \
		trap_1_wl(0x17, (long)(slb))

#endif /* __GNUC__ */

#ifdef __cplusplus
}
#endif

#endif /* __MINTBIND_H__ */
