#ifndef _OSTRUCT_H
#define _OSTRUCT_H

#ifndef _COMPILER_H
#include <compiler.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/*
 * General OS specific codes here
 *
 */

/*
 * GEMDOS defines and structures
 */

/* Structure used by Dfree() */
typedef struct {
    unsigned long b_free;	/* number of free clusters */
    unsigned long b_total;	/* total number of clusters */
    unsigned long b_secsiz;	/* number of bytes per sector */
    unsigned long b_clsiz;	/* number of sectors per cluster */
} _DISKINFO;

/* Structure returned by Fdatime() */
typedef struct {
  unsigned short time;
  unsigned short date;
} _DOSTIME;

/* Structure used by Cconrs */
typedef struct
{
        unsigned char maxlen;
        unsigned char actuallen;
        char    buffer[255];
} _CCONLINE;


/* Structure used by Fgetdta(), Fsetdta(), Fsfirst(), Fsnext() */
typedef struct _dta {
    char 	    dta_buf[21];	/* reserved */
    char            dta_attribute;	/* file attribute */
    unsigned short  dta_time;		/* file time stamp */
    unsigned short  dta_date;		/* file date stamp */
    long            dta_size;		/* file size */
    char            dta_name[14];	/* file name */
} _DTA;

/****** GEMDOS *************************************************************/

/* GEMDOS- Device- Handles */

#define HDL_CON -1                                          /* TOS */
#define HDL_AUX -2                                          /* TOS */
#define HDL_PRN -3                                          /* TOS */
#define HDL_NUL -4                                          /* KAOS 1.2 */

/* GEMDOS- Standard- Handles */

#define STDIN   0                                           /* TOS */
#define STDOUT  1                                           /* TOS */
#define STDAUX  2                                           /* TOS */
#define STDPRN  3                                           /* TOS */
#define STDERR  4                                           /* TOS */
#define STDXTRA 5                                           /* TOS */

/* Codes used with Fcreate()/Fsfirst() */

#define        FA_RDONLY           0x01
#define        FA_READONLY         FA_RDONLY
#define        FA_HIDDEN           0x02
#define        FA_SYSTEM           0x04
#define        FA_LABEL            0x08
#define        FA_VOLUME           FA_LABEL
#define        FA_DIR              0x10
#define        FA_SUBDIR           FA_DIR
#ifndef FA_DIREC
#define        FA_DIREC            FA_DIR
#endif
#define        FA_CHANGED          0x20
#define        FA_ARCHIVE          FA_CHANGED
#ifndef FA_ARCH
#define        FA_ARCH             FA_CHANGED
#endif
#define        FA_SYMLINK          0x40			/* MagiC 3.0 */

/* Fopen modes: */

#define FO_READ         0
#define FO_WRITE        1
#define FO_RW           2


/* Codes used with Pexec */

#define        PE_LOADGO           0           /* load & go */
#define        PE_LOAD             3           /* just load */
#define        PE_GO               4           /* just go */
#define        PE_CBASEPAGE        5           /* just create basepage */
/* Tos 1.4: like 4, but memory ownership changed to child, and freed
   on exit
 */
#define        PE_GO_FREE          6           /* just go, then free */

#ifdef __MINT__
#define        PE_CBASEPAGE_FLAGS  7           /* create basepage defining the program flags */
#define        PE_ASYNC_LOADGO	   100	       /* load and asynchronously go */
#define        PE_ASYNC_GO         104	       /* asynchronously go	     */
#define        PE_ASYNC_GO_FREE    106	       /* asynchronously go and free */
#define        PE_OVERLAY	       200	       /* load and overlay	     */
#define        PE_OVERLAY_GO       204         /* overlay and go */
#define        PE_OVERLAY_GO_FREE  206         /* overlay, go and free */
#endif

/****** KAOS 1.2 Gemdos Extension ***************************************/

/* Sconfig - Modes */

#define SC_GET   0                                          /* KAOS 1.2 */
#define SC_SET   1                                          /* KAOS 1.2 */
#define SC_VARS  2                                          /* KAOS 1.4 */

/* Sconfig Bits */

#define SCB_PTHCK   0x001                                   /* KAOS 1.2 */
#define SCB_DSKCH   0x002                                   /* KAOS 1.2 */
#define SCB_BREAK   0x004                                   /* KAOS 1.2 */
#define SCB_NCTLC   0x008                                   /* KAOS 1.2 */
#define SCB_NFAST   0x010                                   /* KAOS 1.2 */
#define SCB_CMPTB   0x020                                   /* KAOS 1.4 */
#define SCB_NSMRT   0x040                                   /* KAOS 1.4 */
#define SCB_NGRSH   0x080                                   /* KAOS 1.4 */
#define SCB_NHALT   0x100                                   /* KAOS 1.4 */
#define SCB_RESVD   0x200                                   /* KAOS 1.4 */
#define SCB_PULLM   0x400                                   /* KAOS 1.4 */
#define SCB_FLPAR   0x800                                   /* MagiC 3.0 */

/* Sconfig(2) -> */

#ifndef _DOSVARS
#define _DOSVARS
typedef struct
   {
   char      *in_dos;                 /* Adresse der DOS- Semaphore */
   short     *dos_time;               /* Adresse der DOS- Zeit      */
   short     *dos_date;               /* Adresse des DOS- Datums    */
   long      res1;                    /*                            */
   long      res2;                    /*                            */
   long      res3;                    /* ist 0L                     */
   void      *act_pd;                 /* Laufendes Programm         */
   long      res4;                    /*                            */
   short     res5;                    /*                            */
   void      *res6;                   /*                            */
   void      *res7;                   /* interne DOS- Speicherliste */
   void      (*resv_intmem)(void);    /* DOS- Speicher erweitern    */
   long      __CDECL (*etv_critic)(short err);         /* etv_critic des GEMDOS      */
   char *    ((*err_to_str)(char e)); /* Umrechnung Code->Klartext  */
   long      res8;                    /*                            */
   long      res9;                    /*                            */
   long      res10;                   /*                            */
   } DOSVARS;
#endif

#ifndef _AESVARS
#define _AESVARS
typedef struct
{
	long 	magic;								/* muss $87654321 sein */
	void 	*membot;		   					/* Ende der AES- Variablen	*/
	void 	*aes_start;		   				/* Startadresse */
	long 	magic2;			 					/* ist 'MAGX' */
	long 	date; 								/* Erstelldatum ttmmjjjj */
	void 	(*chgres)(short res, short txt);/* Aufloesung aendern */
	long 	(**shel_vector)(void);  		/* residentes Desktop */
	char 	*aes_bootdrv; 		 				/* von hieraus wurde gebootet	*/
	short  *vdi_device;						/* vom AES benutzter VDI-Treiber */
	void	*reservd1;
	void 	*reservd2;
	void 	*reservd3;
	short 	version;		   					/* z.B. $0201 ist V2.1 */
	short	release;		   					/* 0=alpha..3=release */
} AESVARS;
#endif

#ifndef _MAGX_COOKIE
#define _MAGX_COOKIE
typedef struct
{
	long		config_status;
	DOSVARS	*dosvars;
	AESVARS	*aesvars;
} MAGX_COOKIE;
#endif

/****** BIOS *************************************************************/

/* Device codes for Bconin(), Bconout(), Bcostat(), Bconstat() */
#define _PRT    0
#define _AUX    1
#define _CON    2
#define _MIDI   3
#define _IKBD   4
#define _RAWCON 5

/* Structure returned by Getbpb() */
typedef struct {
  short recsiz;         /* bytes per sector */
  short clsiz;          /* sectors per cluster */
  short clsizb;         /* bytes per cluster */
  short rdlen;          /* root directory size */
  short fsiz;           /* size of file allocation table */
  short fatrec;         /* startsector of second FAT */
  short datrec;         /* first data sector */
  short numcl;          /* total number of clusters */
  short bflags;         /* some flags */
} _BPB;

/* Structures used by Getmpb() */

/* Memory descriptor */
typedef struct _md {
    struct _md	*md_next;	/* next descriptor in the chain */
    long	 md_start;	/* starting address of block */
    long	 md_length;	/* length of the block */
    long	 md_owner;	/* owner's process descriptor */
} _MD;

/* Memory parameter block */
typedef struct {
    _MD *mp_free;		/* free memory chunks */
    _MD *mp_used;		/* used memory descriptors */
    _MD *mp_rover;		/* rover memory descriptor */
} _MPB;

/****** XBIOS *************************************************************/

/* Codes used with Cursconf() */
#define CURS_HIDE   	0
#define CURS_SHOW   	1
#define CURS_BLINK  	2
#define CURS_NOBLINK    3
#define CURS_SETRATE    4
#define CURS_GETRATE    5

/* Structure returned by Iorec() */
typedef struct {
    char    *ibuf;
    short   ibufsiz;
    volatile short   ibufhd;
    volatile short   ibuftl;
    short   ibuflow;
    short   ibufhi;
} _IOREC;

/* Structure used by Initmouse() */
typedef struct {
	char	topmode;
	char	buttons;
	char	xparam;
	char	yparam;
	short	xmax;
	short	ymax;
	short	xstart;
	short	ystart;
} _PARAM;

/*
 * Structure returned by Kbdvbase()
 * no need to declare the funtions cdecl,
 * since the arg is passed in a0 AND on stack
 */
typedef struct {
    void    (*midivec)	__PROTO((void));
    void    (*vkbderr)	__PROTO((void));
    void    (*vmiderr)	__PROTO((void));
    void    (*statvec)	__PROTO((void *));
    void    (*mousevec)	__PROTO((void *));
    void    (*clockvec)	__PROTO((void *));
    void    (*joyvec)	__PROTO((void *));
    long    (*midisys)	__PROTO((void));
    long    (*ikbdsys)	__PROTO((void));
    char    kbstate;
} _KBDVECS;

/* Structure returned by Keytbl() */
typedef struct {
    void *unshift;	/* pointer to unshifted keys */
    void *shift;	/* pointer to shifted keys */
    void *caps;		/* pointer to capslock keys */
} _KEYTAB;

/* Structure used by Prtblk() */
typedef struct
{
        void    *pb_scrptr;
        short   pb_offset;
        short   pb_width;
        short   pb_height;
        short   pb_left;
        short   pb_right;
        short   pb_screz;
        short   pb_prrez;
        void    *pb_colptr;
        short   pb_prtype;
        short   pb_prport;
        void    *pb_mask;
} _PBDEF;

typedef struct
{
        short (*Bconstat) __PROTO((void));
        long  (*Bconin) __PROTO((void));
        short (*Bcostat) __PROTO((void));
        long  (*Bconout) __PROTO((short));
        long  (*Rsconf) __PROTO((short, short, short, short, short, short));
        _IOREC *iorec;
} _MAPTAB;

typedef struct
{
        _MAPTAB *maptab;
        short  maptabsize;
} _BCONMAP;

/***********************************************************************/

/* os header */
/* system variable _sysbase (0x4F2L) points to this structure         */
typedef struct _osheader 
{
    unsigned short	os_entry;   	 /* 0x00  BRA to reset handler	*/
    unsigned short	os_version; 	 /* 0x02  TOS version		*/
    void		(*reseth) __PROTO((void)); /* 0x04 -> reset handler	*/
    struct _osheader	*os_beg;	 /* 0x08 -> base of OS		*/
    void		*os_end;	 /* 0x0c -> end of OS ram usage */
    char		*os_rsv1;	 /* 0x10 reserved		*/
    char		*os_magic;	 /* 0x14 GEM memory usage param */
    long		os_date;	 /* 0x18 Build date 0xMMDDYYYY	*/
    unsigned short	os_conf;	 /* 0x1c OS conf bits		*/
    unsigned short	os_dosdate;	 /* 0x1e DOS format build date  */
    /* the following available on TOS version >= 1.2 */
    char		**p_root;	 /* 0x20 -> base of OS pool	*/
    char		**pkbshift;	 /* 0x24 -> kbd shift state var */
    char		**p_run;	 /* 0x28 -> PID of current proc */
    char		*p_rsv2;	 /* 0x2c reserved		*/
} OSHEADER;
    
#ifdef __cplusplus
}
#endif

#endif /* _OSTRUCT_H */
