/*****************************************************************************
 *		PORTAES.H
 *****************************************************************************/

#ifndef __PORTAES_H__
#define __PORTAES_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif

EXTERN_C_BEG

extern int _app;
extern _WORD gl_apid;
extern _WORD gl_ap_version;
extern _WORD gl_ap_count;

/* Keybord states */

#define K_RSHIFT        0x0001
#define K_LSHIFT        0x0002
#define K_SHIFT			(K_LSHIFT|K_RSHIFT)
#define K_CTRL          0x0004
#define K_ALT           0x0008
#define K_CAPSLOCK		0x0010

/* mouse buttons */

#define MOB_LEFT		0x01
#define MOB_RIGHT		0x02
#define MOB_MIDDLE		0x04
#define MOB_BUTTON4		0x08
#define MOB_BUTTON5		0x10

/* SC_CHANGED formats */
#define SCF_INDEF 		0		/* nothing specified */
#define SCF_DATABASE    0x0001	/* data to be loaded into a database (".DBF", ".CSV", ...) */
#define SCF_DBASE       SCF_DATABASE
#define SCF_TEXT        0x0002	/* text files (".TXT", ".ASC", ".RTF", ".DOC", ...) */
#define SCF_VECTOR      0x0004	/* vector graphics (".GEM", ".EPS", ".CVG", ".DXF", ...) */
#define SCF_RASTER      0x0008	/* bitmap graphics (".IMG", ".TIF", ".GIF", ".PCX", ".IFF", ...) */
#define SCF_SPREADSHEET 0x0010	/* spreadsheet data (".DIF", ".WKS", ...) */
#define SCF_SAMPLE      0x0020	/* samples, MIDI files, sound, ... (".MOD", ".SND", ...) */
#define SCF_SOUND       SCF_SAMPLE
#define SCF_ARCHIVE     0x0040	/* archives (".ZIP", ".LZH", ...) */
#define SCF_SYSTEM      0x8000	/* system files */


typedef struct
{
	_WORD	m_out;
	_WORD	m_x;
	_WORD	m_y;
	_WORD	m_w;
	_WORD	m_h;
} MOBLK;

#ifndef __GRECT
#define __GRECT
typedef struct
{
	_WORD	g_x;
	_WORD	g_y;
	_WORD	g_w;
	_WORD	g_h;
} GRECT;
#endif

typedef struct orect
{
        struct  orect   *o_link;
        _WORD     o_x;
        _WORD     o_y;
        _WORD     o_w;
        _WORD     o_h;
} ORECT;


/* Object structures */

/*
 * Macros to manipulate a TEDINFO color word
 */

#define COLSPEC_GET_FRAMECOL(color)    (((color) >> 12) & 0x0f)
#define COLSPEC_GET_TEXTCOL(color)     (((color) >>  8) & 0x0f)
#define COLSPEC_GET_TEXTMODE(color)    (((color) >>  7) & 0x01)
#define COLSPEC_GET_FILLPATTERN(color) (((color) >>  4) & 0x07)
#define COLSPEC_GET_INTERIORCOL(color) (((color)      ) & 0x0f)

#define COLSPEC_SET_FRAMECOL(color, framecol)       color = ( ((color) & 0x0fff) | (((framecol)    & 0x0f) << 12) )
#define COLSPEC_SET_TEXTCOL(color, textcol)         color = ( ((color) & 0xf0ff) | (((textcol)     & 0x0f) <<  8) )
#define COLSPEC_SET_TEXTMODE(color, textmode)       color = ( ((color) & 0xff7f) | (((textmode)    & 0x01) <<  7) )
#define COLSPEC_SET_FILLPATTERN(color, fillpattern) color = ( ((color) & 0xff8f) | (((fillpattern) & 0x07) <<  4) )
#define COLSPEC_SET_INTERIORCOL(color, interiorcol) color = ( ((color) & 0xfff0) | (((interiorcol) & 0x0f)      ) )

#define COLSPEC_MAKE(framecol, textcol, textmode, fillpattern, interiorcol) \
	((((framecol)    & 0x0f) << 12) | \
	 (((textcol)     & 0x0f) <<  8) | \
	 (((textmode)    & 0x01) <<  7) | \
	 (((fillpattern) & 0x07) <<  4) | \
	 (((interiorcol) & 0x0f)      ))

typedef struct _tedinfo
{
        char            *te_ptext;      /* ptr to text (must be 1st)    */
        char            *te_ptmplt;     /* ptr to template              */
        char            *te_pvalid;     /* ptr to validation            */
        _WORD            te_font;       /* font                         */
        _WORD            te_fontid;     /* junk WORD                    */
        _WORD            te_just;       /* justification: left, right...*/
        _UWORD           te_color;      /* color information            */
        _WORD            te_fontsize;   /* junk WORD                    */
        _WORD            te_thickness;  /* border thickness             */
        _WORD            te_txtlen;     /* text string length           */
        _WORD            te_tmplen;     /* template string length       */
} PACKED TEDINFO;
#define te_junk1 te_fontid
#define te_junk2 te_fontsize

/*
 * Macros to manipulate a ICONBLK color word
 */
#define ICOLSPEC_GET_DATACOL(color)   ( ((color) >> 12) & 0x0f )
#define ICOLSPEC_GET_MASKCOL(color)   ( ((color) >>  8) & 0x0f )
#define ICOLSPEC_GET_CHARACTER(color) ( ((color)      ) & 0xff )

#define ICOLSPEC_SET_DATACOL(color, datacol) color = ( ((color) & 0x0fff) | (((datacol) & 0x0f) << 12) )
#define ICOLSPEC_SET_MASKCOL(color, maskcol) color = ( ((color) & 0xf0ff) | (((maskcol) & 0x0f) <<  8) )
#define ICOLSPEC_SET_CHARACTER(color, ch)    color = ( ((color) & 0xff00) | (((ch)      & 0xff)      ) )

#define ICOLSPEC_MAKE(datacol, maskcol, ch) \
	((((datacol)    & 0x0f) << 12) | \
	 (((maskcol)    & 0x0f) <<  8) | \
	 (((ch)         & 0xff)      ))

typedef struct _iconblk
{
        _WORD     *ib_pmask;
        _WORD     *ib_pdata;
        char    *ib_ptext;
        _UWORD    ib_char;
        _WORD     ib_xchar;
        _WORD     ib_ychar;
        _WORD     ib_xicon;
        _WORD     ib_yicon;
        _WORD     ib_wicon;
        _WORD     ib_hicon;
        _WORD     ib_xtext;
        _WORD     ib_ytext;
        _WORD     ib_wtext;
        _WORD     ib_htext;
} PACKED ICONBLK;


typedef struct _bitblk
{
        _WORD     *bi_pdata;              /* ptr to bit forms data        */
        _WORD     bi_wb;                  /* width of form in bytes       */
        _WORD     bi_hl;                  /* height in lines              */
        _WORD     bi_x;                   /* source x in bit form         */
        _WORD     bi_y;                   /* source y in bit form         */
        _WORD     bi_color;               /* foreground color             */
} PACKED BITBLK;


typedef struct _cicon {
    _WORD    num_planes;            /* number of planes in the following data */
    _WORD   *col_data;             /* pointer to color bitmap in standard form */
    _WORD   *col_mask;             /* pointer to single plane mask of col_data */
    _WORD   *sel_data;             /* pointer to color bitmap of selected icon */
    _WORD   *sel_mask;             /* pointer to single plane mask of selected icon */
    struct _cicon *next_res;   /* pointer to next icon for a different resolution */
} PACKED CICON; /* AES >= 3.3 */

typedef struct _ciconblk {
    ICONBLK monoblk;	          /* default monochrome icon */
    CICON *mainlist;	          /* list of color icons for different resolutions */
} PACKED CICONBLK; /* AES >= 3.3 */

#define CICON_STR_SIZE 12

#ifdef __TURBOC__
#  define _CDECL cdecl
#endif
#ifdef __SOZOBONX__
#  define _CDECL 
#endif
#ifndef _CDECL
#  define _CDECL /**/
#endif

struct __parmblk;

typedef _UWORD _CDECL (*PARMBLKFUNC)(struct __parmblk *pb);
typedef struct
{
        PARMBLKFUNC ub_code;
        _LONG ub_parm;
} PACKED USERBLK;

typedef struct
{
        unsigned character   :  8;
        signed   framesize   :  8;
        unsigned framecol    :  4;
        unsigned textcol     :  4;
        unsigned textmode    :  1;
        unsigned fillpattern :  3;
        unsigned interiorcol :  4;
} bfobspec;

/*
 * Macros to manipulate a OBSPEC info
 */
#define OBSPEC_GET_CHARACTER(obspec)   ((_UBYTE) ( (((obspec).index) >> 24) & 0xff ))
#define OBSPEC_GET_FRAMESIZE(obspec)   (( _BYTE) ( (((obspec).index) >> 16) & 0xff ))
#define OBSPEC_GET_FRAMECOL(obspec)    ((_UBYTE) ( (((obspec).index) >> 12) & 0x0f ))
#define OBSPEC_GET_TEXTCOL(obspec)     ((_UBYTE) ( (((obspec).index) >>  8) & 0x0f ))
#define OBSPEC_GET_TEXTMODE(obspec)    ((_UBYTE) ( (((obspec).index) >>  7) & 0x01 ))
#define OBSPEC_GET_FILLPATTERN(obspec) ((_UBYTE) ( (((obspec).index) >>  4) & 0x07 ))
#define OBSPEC_GET_INTERIORCOL(obspec) ((_UBYTE) ( (((obspec).index)      ) & 0x0f ))

#define OBSPEC_SET_CHARACTER(obspec, ch)            (obspec).index = ( (((obspec).index) & 0x00ffffffl) | ((((_ULONG)((ch)          & 0xff)) << 24)) )
#define OBSPEC_SET_FRAMESIZE(obspec, framesize)     (obspec).index = ( (((obspec).index) & 0xff00ffffl) | ((((_ULONG)((framesize)   & 0xff)) << 16)) )
#define OBSPEC_SET_FRAMECOL(obspec, framecol)       (obspec).index = ( (((obspec).index) & 0xffff0fffl) | ((((_ULONG)((framecol)    & 0x0f)) << 12)) )
#define OBSPEC_SET_TEXTCOL(obspec, textcol)         (obspec).index = ( (((obspec).index) & 0xfffff0ffl) | ((((_ULONG)((textcol)     & 0x0f)) <<  8)) )
#define OBSPEC_SET_TEXTMODE(obspec, textmode)       (obspec).index = ( (((obspec).index) & 0xffffff7fl) | ((((_ULONG)((textmode)    & 0x01)) <<  7)) )
#define OBSPEC_SET_FILLPATTERN(obspec, fillpattern) (obspec).index = ( (((obspec).index) & 0xffffff8fl) | ((((_ULONG)((fillpattern) & 0x07)) <<  4)) )
#define OBSPEC_SET_INTERIORCOL(obspec, interiorcol) (obspec).index = ( (((obspec).index) & 0xfffffff0l) | ((((_ULONG)((interiorcol) & 0x0f))      )) )

#define OBSPEC_MAKE(ch, framesize, framecol, textcol, textmode, fillpattern, interiorcol) \
	   ( \
		((((_ULONG)((ch)          & 0xff)) << 24)) | \
		((((_ULONG)((framesize)   & 0xff)) << 16)) | \
		((((_ULONG)((framecol)    & 0x0f)) << 12)) | \
		((((_ULONG)((textcol)     & 0x0f)) <<  8)) | \
		((((_ULONG)((textmode)    & 0x01)) <<  7)) | \
		((((_ULONG)((fillpattern) & 0x07)) <<  4)) | \
		((((_ULONG)((interiorcol) & 0x0f))      )) )

#define OBSPEC_SET_OBSPEC(obspec, ch, framesize, framecol, textcol, textmode, fillpattern, interiorcol) \
	obspec.index = OBSPEC_MAKE(ch, framesize, framecol, textcol, textmode, fillpattern, interiorcol)


typedef struct {
     char  *string;                 /* etwa "TOS|KAOS|MAG!X"                */
     _WORD  num;                     /* Nr. der aktuellen Zeichenkette       */
     _WORD  maxnum;                  /* maximal erlaubtes <num>              */
} PACKED SWINFO;

typedef struct {
     struct _object *tree;         /* Popup- Menue                          */
     _WORD   obnum;                   /* aktuelles Objekt von <tree>          */
} PACKED POPINFO;

typedef union obspecptr
{
        _LONG    index;
        union obspecptr *indirect;
        bfobspec obspec;
        TEDINFO  *tedinfo;
        ICONBLK  *iconblk;
        BITBLK   *bitblk;
        USERBLK  *userblk;
        CICONBLK *ciconblk;
        _UBYTE   *free_string;
} PACKED OBSPEC;


typedef struct _object
{
        _WORD            ob_next;        /* -> object's next sibling     */
        _WORD            ob_head;        /* -> head of object's children */
        _WORD            ob_tail;        /* -> tail of object's children */
        _UWORD           ob_type;        /* object type: BOX, CHAR,...   */
        _UWORD           ob_flags;       /* object flags                 */
        _UWORD           ob_state;       /* state: SELECTED, OPEN, ...   */
        OBSPEC          ob_spec;        /* "out": -> anything else      */
        _WORD            ob_x;           /* upper left corner of object  */
        _WORD            ob_y;           /* upper left corner of object  */
        _WORD            ob_width;       /* object width                 */
        _WORD            ob_height;      /* object height                */
} PACKED OBJECT;


typedef struct __parmblk
{
        OBJECT  *pb_tree;
        _WORD    pb_obj;
        _UWORD   pb_prevstate;
        _UWORD   pb_currstate;
        _WORD    pb_x, pb_y, pb_w, pb_h;
        _WORD    pb_xc, pb_yc, pb_wc, pb_hc;
        _LONG    pb_parm;
} PARMBLK;

/****** Object definitions **********************************************/

/* graphic types of obs */
#define G_BOX           20
#define G_TEXT          21
#define G_BOXTEXT       22
#define G_IMAGE         23
#define G_USERDEF       24
#define G_IBOX          25
#define G_BUTTON        26
#define G_BOXCHAR       27
#define G_STRING        28
#define G_FTEXT         29
#define G_FBOXTEXT      30
#define G_ICON          31
#define G_TITLE         32
#define G_CICON         33 /* AES >= 3.3 */
#define G_SWBUTTON      34                                 /* MAG!X       */
#define G_POPUP         35                                 /* MAG!X       */
#define G_RESVD1        36

/* Object flags */
#define NONE            0x0000
#define SELECTABLE      0x0001
#define DEFAULT         0x0002
#define EXIT            0x0004
#define EDITABLE        0x0008
#define RBUTTON         0x0010
#define LASTOB          0x0020
#define TOUCHEXIT       0x0040
#define HIDETREE        0x0080
#define INDIRECT        0x0100

/* 3D objects AES 3.4	*/
#define	FL3DMASK	0x0600
#define	  FL3DNONE	0x0000
#define	  FL3DIND	0x0200	/* Indicator */
#define	  FL3DBAK	0x0400	/* Background */
#define	  FL3DACT	0x0600	/* Activator */


/* Object states */
#define NORMAL          0x00
#define SELECTED        0x01
#define CROSSED         0x02
#define CHECKED         0x04
#define DISABLED        0x08
#define OUTLINED        0x10
#define SHADOWED        0x20
#define WHITEBAK        0x40
#define DRAW3D			0x80


/* Object colors */
#if !defined(__COLORS)
#define __COLORS        /*
                                                using AES-colors and BGI-colors
                                                is not possible
                                        */
enum {WHITE, BLACK, RED, GREEN, BLUE, CYAN, YELLOW, MAGENTA,
	 LWHITE,LBLACK,LRED,LGREEN,LBLUE,LCYAN,LYELLOW,LMAGENTA,
	 KEEPCOL};
#endif /* __COLORS */

#ifndef NIL
# define NIL (-1)
#endif
#define ROOT             0
#define MAX_LEN         81              /* max string length */
#define MAX_DEPTH        8              /* max depth of search or draw */


/* font types */
#define GDOS_PROP        0 /* Speedo GDOS font */
#define GDOS_MONO        1 /* Speedo GDOS font, force monospace output */
#define GDOS_BITM        2 /* GDOS bit map font */
#define IBM              3
#define SMALL            5
#define TE_FONT_MASK     7


/* editable text field definitions */
#define ED_START        0
#define ED_INIT         1
#define ED_CHAR         2
#define ED_END          3
#define ED_CRSR         100                                /* MAG!X       */
#define ED_DRAW         103                                /* MAG!X 2.00  */

/* editable text justification */
#define TE_LEFT         0
#define TE_RIGHT        1
#define TE_CNTR         2
#define TE_JUST_MASK    3

/* inside patterns */
#define IP_HOLLOW		 0
#define IP_1PATT		 1
#define IP_2PATT		 2
#define IP_3PATT		 3
#define IP_4PATT		 4
#define IP_5PATT		 5
#define IP_6PATT		 6
#define IP_SOLID		 7


/* data structure types */
#define R_TREE           0
#define R_OBJECT         1
#define R_TEDINFO        2
#define R_ICONBLK        3
#define R_BITBLK         4
#define R_STRING         5              /* gets pointer to free strings */
#define R_IMAGEDATA      6              /* gets pointer to free images */
#define R_OBSPEC         7
#define R_TEPTEXT        8              /* sub ptrs in TEDINFO */
#define R_TEPTMPLT       9
#define R_TEPVALID      10
#define R_IBPMASK       11              /* sub ptrs in ICONBLK */
#define R_IBPDATA       12
#define R_IBPTEXT       13
#define R_BIPDATA       14              /* sub ptrs in BITBLK */
#define R_FRSTR         15              /* gets addr of ptr to free strings */
#define R_FRIMG         16              /* gets addr of ptr to free images  */

typedef struct rshdr
{
        _UWORD     rsh_vrsn;
        _UWORD     rsh_object;
        _UWORD     rsh_tedinfo;
        _UWORD     rsh_iconblk;    /* list of ICONBLKS */
        _UWORD     rsh_bitblk;
        _UWORD     rsh_frstr;
        _UWORD     rsh_string;
        _UWORD     rsh_imdata;     /* image data */
        _UWORD     rsh_frimg;
        _UWORD     rsh_trindex;
        _UWORD     rsh_nobs;       /* counts of various structs */
        _UWORD     rsh_ntree;
        _UWORD     rsh_nted;
        _UWORD     rsh_nib;
        _UWORD     rsh_nbb;
        _UWORD     rsh_nstring;
        _UWORD     rsh_nimages;
        _UWORD     rsh_rssize;     /* total bytes in resource */
} PACKED RSHDR;



#if ATARI || defined(ATARI_EMULATOR)
EXTERN _BOOL Wind_Update(_WORD begend);
EXTERN _VOID Wind_New(_VOID);
#else
#define Wind_Update(begend) 
#define Wind_New()
#endif

/************************************************************************/
/* end of portable definitions											*/
/************************************************************************/

#if ATARI || defined(ATARI_EMULATOR) || defined(IN_ATARI_EMULATOR)

/****** GEMparams *******************************************************/

typedef struct
{
    _WORD   contrl[15];
    _WORD   global[80];
    _WORD   intin[128];
    _WORD   intout[45];
    _WORD   ptsout[128];
    _VOID   *addrin[128];
    _VOID   *addrout[6];
    _WORD   ptsin[128];
    _WORD   acontrl[5];   /* MO */
    _WORD   aintin[32];   /* MO */
    _WORD   aintout[32];  /* MO */
} GEMPARBLK;

typedef struct  /* MO */
{
	_WORD *contrl;
	_WORD *global;
	_WORD *intin;
	_WORD *intout;
	_VOID **addrin;
	_VOID **addrout;
} AESPB;

extern  AESPB _AesParBlk;  /* MO */

extern int  vq_aes( _VOID );
extern _VOID aes( AESPB *aespb );
extern _WORD _aes(_WORD dummy, _LONG code);


#ifndef _DOSVARS
#define _DOSVARS
typedef struct
{
	_BYTE		*in_dos;		 	  			/* Adresse der DOS- Semaphore */
	_WORD		*dos_time; 						/* Adresse der DOS- Zeit */
	_WORD		*dos_date; 						/* Adresse des DOS- Datums */
	_LONG		res1;
	_LONG		res2;
	_LONG		res3;
	_VOID		*act_pd;		 	  			/* Laufendes Programm */
	_LONG		res4;
	_WORD		res5;
	_VOID		*res6;
	_VOID		*res7;							/* interne DOS- Speicherliste */
	_VOID		(*resv_intmem)(_VOID);		 	/* DOS- Speicher erweitern */
	_LONG		(*etv_critic)(_VOID);			/* etv_critic des GEMDOS */
	_BYTE *	((*err_to_str)(_BYTE e)); 	/* Umrechnung Code->Klartext */
	_LONG		res8;
	_LONG		res9;
	_LONG		res10;
} DOSVARS;
#endif

#ifndef _AESVARS
#define _AESVARS
typedef struct
{
	_LONG 	magic;								/* muss $87654321 sein */
	_VOID 	*membot;		   					/* Ende der AES- Variablen	*/
	_VOID 	*aes_start;		   				/* Startadresse */
	_LONG 	magic2;			 					/* ist 'MAGX' */
	_LONG 	date; 								/* Erstelldatum ttmmjjjj */
	_VOID 	(*chgres)(_WORD res, _WORD txt);/* Aufloesung aendern */
	_LONG 	(**shel_vector)(_VOID);  		/* residentes Desktop */
	_BYTE 	*aes_bootdrv; 		 				/* von hieraus wurde gebootet	*/
	_WORD  *vdi_device;						/* vom AES benutzter VDI-Treiber */
	_VOID 	*reservd1;
	_VOID 	*reservd2;
	_VOID 	*reservd3;
	_WORD 	version;		   					/* z.B. $0201 ist V2.1 */
	_WORD	release;		   					/* 0=alpha..3=release */
} AESVARS;
#endif

#ifndef _MAGX_COOKIE
#define _MAGX_COOKIE
typedef struct
{
	_LONG		config_status;
	DOSVARS	*dosvars;
	AESVARS	*aesvars;
} MAGX_COOKIE;
#endif


/****** Application definitions *****************************************/

/* appl_search modes */
#define APS_FIRST 0  /* MO */
#define APS_NEXT  1  /* MO */
#define APS_SHEL  2  /* MO */

/* application types */
#define APK_SYS   1  /* MO */
#define APK_APP   2  /* MO */
#define APK_ACC   4  /* MO */

#define APPL_AESFIND( mintid ) \
  appl_find( (const char *)(0xFFFF0000L | (_ULONG)mintid))
  /* MO */
#define APPL_MINTFIND( aesid ) \
  appl_find( (const char *)(0xFFFE0000L | (_ULONG)aesid))
  /* MO */
#define APPL_CURRFIND() appl_find( NULL)
  /* MO */

/* appl_getinfo modes */
#define APG_FONT    0   /* Get AES regular font information
                         * o1: font height, o2: font id,
                         * o3: font type: 0 - system font
                         *     1 - fsm font
                         *     2 and on to be defined in the future */
#define APG_SMLFONT 1   /* Get AES small font information. see above */
#define APG_REZ     2   /* Get AES current resolution number and the number
                         * of color is being supported by the object library
                         * o1: resolution number
                         * o2: number of color supported by AES object library
                         * o3: color icons: 0 - Not supported 1 - supported
                         */
#define APG_LANG    3   /* o1: currently used language (see below) */

#define L_ENGLISH    0    /* Konstanten fuer die einzelnen Sprachen */
#define L_GERMAN     1
#define L_FRENCH     2
#define L_SPANISH    4
#define L_ITALIAN    5
#define L_SWEDISH    6

#define APG_GLOBAL1    4
        /* Get general AES environment info #1
           ap_gout1 - 0 - non-pre-emptive multitasking
                      1 - pre-emptive multitasking
           ap_gout2 - 0 - appl_find cannot convert from MiNT to AES ids
                  1 - extended appl_find modes supported
           ap_gout3 - 0 - appl_search not implemented
                  1 - appl_search implemented
           ap_gout4 - 0 - rsrc_rcfix not implemented
                  1 - rsrc_rcfix implemented
        */

#define APG_GLOBAL2    5
        /*
        5 - General AES environment info #2

           ap_gout1 - 0 - objc_xfind not implemented
                      1 - objc_xfind implemented
           ap_gout2 - 0 - reserved, always 0
           ap_gout3 - 0 - GEM/3 menu_click not implemented
                  1 - menu_click implemented
           ap_gout4 - 0 - GEM/3 shel_r/wdef not implemented
                  1 - shel_r/wdef implemented
        */

#define APG_GLOBAL3    6
        /*
        6 - General AES environment info #3

           ap_gout1 - 0 - appl_read(-1) not implemented
                      1 - appl_read(-1) implemented
           ap_gout2 - 0 - shel_get(-1) not implemented
                  1 - shel_get(-1) implemented
           ap_gout3 - 0 - menu_bar(-1) not implemented
                  1 - menu_bar(-1) implemented
           ap_gout4 - 0 - menu_bar(MENU_INSTL) not implemented
                  1 - menu_bar(MENU_INSTL) implemented
        */

#define APG_RESVD  7

        /*
        7 - Reserved for OS extensions. MultiTOS sets
            ap_gout1,2,3,4 to 0.
        */

#define APG_MOUSE  8

        /*
        8 - Mouse support

           ap_gout1 - 0 - graf_mouse modes 258-260 not supported
                      1 - graf_mouse modes 258-260 supported
           ap_gout2 - 0 - application must maintain mouse form
                  1 - mouse form maintained by OS on a per-application
                  basis
        */

#define APG_MENUS  9

        /*
        9 - Menu support

           ap_gout1 - 0 - submenus not supported
                      1 - MultiTOS style submenus
           ap_gout2 - 0 - popup menus not supported
                  1 - MultiTOS style popup menus
           ap_gout3 - 0 - scrollable menus not supported
                  1 - MultiTOS style scrollable menus
           ap_gout4 - 0 - extended MN_SELECTED not supported
                  1 - words 5/6/7 in MN_SELECTED message give extra info
        */

#define APG_SHELLW     10

        /*
        10 - shel_write info

           ap_gout1 - shel_write modes supported:
                bit 0-7: indicates highest legal value for
                    (sh_wdoex & 0x00ff)
                bit 8-15: indicates which bits in (sh_wdoex & 0xFF00)
                     are supported as in MultiTOS
           ap_gout2 - 0 - shel_write(0) launches an application
                  1 - shel_write(0) cancels previous shel_write
           ap_gout3 - 0 - shel_write(1) launches an application immediately
                  1 - shel_write(1) takes effect after current application
                  exits (like TOS 1.4)
           ap_gout4 - 0 - ARGV parameter passing not possible
                  1 - sh_wiscr controls ARGV parameter passing
        */

#define APG_WINDOWS    11

        /*
        11 - window support

           ap_gout1 - extended WF_ functions available in wind_get/set
                  (0=not available, 1=available)
                bit 0: WF_TOP returns window below current one
                bit 1: wind_get(WF_NEWDESK) supported
                bit 2: WF_COLOR get/set supported
                bit 3: WF_DCOLOR get/set supported
                bit 4: WF_OWNER supported in wind_get
                bit 5: WF_BEVENT get/set supported
                bit 6: WF_BOTTOM supported
                bit 7: WF_ICONIFY supported
                bit 8: WF_UNICONIFY supported
                bits 9-15 reserved, 0
           ap_gout2 - reserved, 0
           ap_gout3 - new gadgets supported:
                  (0=supported, 1=not supported)
                    bit 0: iconifier
                bit 1: explicit "bottomer" gadget
                bit 2: shift+click to send window to bottom
                bit 3: "hot" close box
                all other bits: reserved, 0
           ap_gout4 - 0 - wind_update check and set not allowed
                  1 - wind_update check and set allowed
        */

#define APG_MESSAGES   12

        /*
        12 - messages sent to applications

           ap_gout1 - bit field of extra messages supported (1)
                  (0=no,1=yes)
                bit 0: WM_NEWTOP message meaningful
                bit 1: WM_UNTOPPED message sent
                bit 2: WM_ONTOP message sent
                bit 3: AP_TERM message sent
                bit 4: MultiTOS shutdown and resolution change
                       messages supported
                bit 5: AES sends CH_EXIT
                bit 6: WM_BOTTOM message sent
                bit 7: WM_ICONIFY message sent
                bit 8: WM_UNICONIFY message sent
                bit 9: WM_ALLICONIFY message sent
           ap_gout2 - bit field of extra messages supported (2)
                (currently all bits are reserved and 0)
           ap_gout3 - message behaviour
                bit 0: WM_ICONIFY message gives coordinates (0=no,1=yes)
        */

#define APG_OBJECTS    13

        /*
        13 - object information

           ap_gout1 - 0 - no 3D objects
                      1 - 3D objects supported via objc_flags
           ap_gout2 - 0 - objc_sysvar supported
                      1 - MultiTOS 1.01 objc_sysvar
                      2 - extended objc_sysvar
           ap_gout3 - 0 - only system font for TEDINFO structures
                      1 - SPEEDO and GDOS fonts allowed in TEDINFO
           ap_gout4 - reserved for OS extensions (MultiTOS always
                      sets this to 0)
                      suggestion: use this to indicate presence
                      of new object types like radio buttons
        */

#define APG_FORMS      14

        /*
        14 - form library information

           ap_gout1 - 0 - no flying dialogs
                      1 - flying dialogs supported
           ap_gout2 - 0 - keyboard tables not supported
                      1 - Mag!X style keyboard tables
           ap_gout3 - 0 - last cursor position not returned
                      1 - last cursor position returned
           ap_gout4 - reserved, 0
         */


/* appl_control[ap_cwhat]:                      */
#define APC_APP         0               /* Zeiger auf APP-Struktur liefern          */
#define APC_VANISHED    1               /* Applikation ist MiNT-maessig terminiert  */
#define APC_HIDE        10              /* Anwendung ausblenden                     */
#define APC_SHOW        11              /* Anwendung einblenden                     */
#define APC_TOP         12              /* Anwendung in den Vordergrund bringen     */
#define APC_HIDENOT     13              /* Alle anderen ausblenden                  */

_WORD appl_init( _VOID );
_WORD appl_read( _WORD ap_rid, _WORD ap_rlength, _VOID *ap_rpbuff );
_WORD appl_write( _WORD ap_wid, _WORD ap_wlength, const _VOID *ap_wpbuff );
_WORD appl_find( const char *ap_fpname );
_WORD appl_tplay( _VOID *ap_tpmem, _WORD ap_tpnum, _WORD ap_tpscale );
_WORD appl_trecord( _VOID *ap_trmem, _WORD ap_trcount );
_WORD appl_exit( _VOID );
_WORD appl_search( _WORD ap_smode, char *ap_sname, _WORD *ap_stype, _WORD *ap_sid ); /* MTOS */
_WORD appl_getinfo( _WORD ap_gtype, _WORD *ap_gout1, _WORD *ap_gout2, _WORD *ap_gout3, _WORD *ap_gout4); /* MTOS */
_WORD appl_xgetinfo( _WORD ap_gtype, _WORD *ap_gout1, _WORD *ap_gout2, _WORD *ap_gout3, _WORD *ap_gout4); /* MTOS */
_WORD appl_control(_WORD ap_cid, _WORD ap_cwhat, _VOID *ap_cout);


/****** Event definitions ***********************************************/

#define MU_KEYBD        0x0001
#define MU_BUTTON       0x0002
#define MU_M1           0x0004
#define MU_M2           0x0008
#define MU_MESAG        0x0010
#define MU_TIMER        0x0020

#define MN_SELECTED     10
#define WM_REDRAW       20
#define WM_TOPPED       21
#define WM_CLOSED       22
#define WM_FULLED       23
#define WM_ARROWED      24
#define WM_HSLID        25
#define WM_VSLID        26
#define WM_SIZED        27
#define WM_MOVED        28
#define WM_NEWTOP       29
#define WM_UNTOPPED     30 /* MTOS */
#define WM_ONTOP        31 /* MTOS */
#define WM_OFFTOP       32
#define PR_FINISH       33
#define WM_BOTTOM		33
#define WM_BOTTOMED		WM_BOTTOM
#define WM_ICONIFY      34
#define WM_UNICONIFY    35
#define WM_ALLICONIFY   36
#define WM_TOOLBAR      37

#define WM_MOUSEWHEEL   0x0930

#define AC_OPEN         40
#define AC_CLOSE        41
#define CT_UPDATE       50
#define CT_MOVE         51
#define CT_NEWTOP       52
#define CT_KEY			53
#define AP_TERM         50 /* MTOS */
#define AP_TFAIL        51 /* MTOS */
#define AP_RESCHG       57 /* MTOS */
#define SHUT_COMPLETED  60 /* MTOS */
#define RESCH_COMPLETED 61 /* MTOS */
#define AP_DRAGDROP		63 /* MTOS */
#define SH_EXIT         68 /* MTOS */
#define SH_START        69 /* MTOS */
#define SH_WDRAW        72 /* MTOS */
#define SC_CHANGED      80
#define PRN_CHANGED     82
#define FNT_CHANGED     83
#define THR_EXIT        88
#define PA_EXIT         89
#define CH_EXIT         90 /* MTOS */
#define WM_M_BDROPPED   100                                /* KAOS 1.4    */
#define SM_M_SPECIAL    101                                /* MAG!X       */
#define SM_M_RES2       102                                /* MAG!X       */
#define SM_M_RES3       103                                /* MAG!X       */
#define SM_M_RES4       104                                /* MAG!X       */
#define SM_M_RES5       105                                /* MAG!X       */
#define SM_M_RES6       106                                /* MAG!X       */
#define SM_M_RES7       107                                /* MAG!X       */
#define SM_M_RES8       108                                /* MAG!X       */
#define SM_M_RES9       109                                /* MAG!X       */

/* SM_M_SPECIAL codes */
#define SMC_TIDY_UP     0                           /* MagiC 2  */
#define SMC_TERMINATE   1                           /* MagiC 2  */
#define SMC_SWITCH      2                           /* MagiC 2  */
#define SMC_FREEZE      3                           /* MagiC 2  */
#define SMC_UNFREEZE    4                           /* MagiC 2  */
#define SMC_RES5        5                           /* MagiC 2  */
#define SMC_UNHIDEALL   6                           /* MagiC 3.1    */
#define SMC_HIDEOTHERS  7                           /* MagiC 3.1    */
#define SMC_HIDEACT     8                           /* MagiC 3.1    */


typedef struct _mevent
{
	_UWORD e_flags;
	_UWORD e_bclk;
	_UWORD e_bmsk;
	_UWORD e_bst;
	_UWORD e_m1flags;
	GRECT e_m1;
	_UWORD e_m2flags;
	GRECT e_m2;
	_WORD  *e_mepbuf;
	_ULONG e_time;
	_WORD  e_mx;
	_WORD  e_my;
	_UWORD e_mb;
	_UWORD e_ks;
	_UWORD e_kr;
	_UWORD e_br;
	_UWORD e_m3flags;
	GRECT e_m3;
	_WORD  e_xtra0;
	_WORD  *e_smepbuf;
	_ULONG e_xtra1;
	_ULONG e_xtra2;
} MEVENT;

_WORD evnt_keybd( _VOID );
_WORD evnt_button( _WORD ev_bclicks, _WORD ev_bmask, _WORD ev_bstate,
                 _WORD *ev_bmx, _WORD *ev_bmy, _UWORD *ev_bbutton,
                 _UWORD *ev_bkstate );
_WORD evnt_mouse( _WORD ev_moflags, _WORD ev_mox, _WORD ev_moy,
                _WORD ev_mowidth, _WORD ev_moheight, _WORD *ev_momx,
                _WORD *ev_momy, _UWORD *ev_mobutton,
                _UWORD *ev_mokstate );
_WORD evnt_mesag( _WORD *ev_mgpbuff );
_WORD evnt_timer( _WORD ev_tlocount, _WORD ev_thicount );
_WORD evnt_multi( _WORD ev_mflags, _WORD ev_mbclicks, _WORD ev_mbmask,
                _WORD ev_mbstate, _WORD ev_mm1flags, _WORD ev_mm1x,
                _WORD ev_mm1y, _WORD ev_mm1width, _WORD ev_mm1height,
                _WORD ev_mm2flags, _WORD ev_mm2x, _WORD ev_mm2y,
                _WORD ev_mm2width, _WORD ev_mm2height,
                _WORD *ev_mmgpbuff, _WORD ev_mtlocount,
                _WORD ev_mthicount, _WORD *ev_mmox, _WORD *ev_mmoy,
                _UWORD *ev_mmbutton, _UWORD *ev_mmokstate,
                _WORD *ev_mkreturn, _WORD *ev_mbreturn );
_WORD evnt_event( MEVENT *mevent );
_WORD evnt_dclick( _WORD ev_dnew, _WORD ev_dgetset );


/* this is our special invention to increase evnt_multi performance */

typedef struct /* Special type for EventMulti */
{
        /* input parameters */
        _WORD     ev_mflags, ev_mbclicks, ev_bmask, ev_mbstate, ev_mm1flags,
                ev_mm1x, ev_mm1y, ev_mm1width, ev_mm1height, ev_mm2flags,
                ev_mm2x, ev_mm2y, ev_mm2width, ev_mm2height, ev_mtlocount,
                ev_mthicount;
        /* output parameters */
        _WORD     ev_mwich, ev_mmox, ev_mmoy;
        _UWORD    ev_mmobutton, ev_mmokstate;
        _WORD     ev_mkreturn, ev_mbreturn;
        /* message buffer */
        _WORD     ev_mmgpbuf[8];
} EVENT;

_WORD EvntMulti( EVENT *evnt_struct );


/****** Menu definitions ************************************************/

/* menu_bar modes */
#define MENU_ERASE    0				/* MO */
#define MENU_DISPLAY  1		/* MO */
#define MENU_INQUIRY (-1)	/* MO */
#define MENU_HIDE        0                                  /* TOS         */
#define MENU_SHOW        1                                  /* TOS         */
#define MENU_INSTL       100                                /* MAG!X       */
#define MENU_GETMODE    3               /* Menueleistenmodus abfragen                */
#define MENU_SETMODE    4               /* Menueleistenmodus setzen (nicht implem.)  */
#define     MENU_HIDDEN     0x0001      /* Menueleiste nur bei Bedarf sichtbar       */
#define     MENU_PULLDOWN   0x0002      /* Pulldown-Menue (aufklappen bei Mausklick) */
#define     MENU_SHADOWED   0x0004      /* Menueleistenboxen mit Schatten            */
#define MENU_UPDATE     5               /* Update des Systemteils der Menueleiste    */

typedef struct
{
	OBJECT *mn_tree;		/*  - the object tree of the menu 	*/
	_WORD	mn_menu;	 	/* - the parent object of the menu items 	*/
	_WORD	mn_item;	 	/* - the starting menu item	*/
	_WORD	mn_scroll; 	/* - the scroll field status of the menu	*/
					/* 0	- The menu will not scroll	*/
					/* !0 - it will scroll if the number of menu
					 *	 items exceed the menu scroll height. The 
					 * NOTE: If the scroll field status is !0, the menu
					 *	 items must consist entirely of G_STRINGS.
					 */
	_UWORD	mn_keystate;	/* - The CTRL, ALT, SHIFT Key state at the time the	*/
} MENU;


typedef struct
{
        _LONG   Display;
        _LONG   Drag;
        _LONG   Delay;
        _LONG   Speed;
        _WORD   Height;
} MN_SET;

_WORD menu_bar( OBJECT *me_btree, _WORD me_bshow );
_WORD menu_icheck( OBJECT *me_ctree, _WORD me_citem, _WORD me_ccheck );
_WORD menu_ienable( OBJECT *me_etree, _WORD me_eitem,
                  _WORD me_eenable );
_WORD menu_tnormal( OBJECT *me_ntree, _WORD me_ntitle,
                  _WORD me_nnormal );
_WORD menu_text( OBJECT *me_ttree, _WORD me_titem,
               const char *me_ttext );
_WORD menu_register( _WORD me_rapid, const char *me_rpstring );
_WORD menu_unregister( _WORD me_rapid ); /* GEM 2.x */
_WORD menu_popup( MENU *me_menu, _WORD me_xpos, _WORD me_ypos,
                MENU *me_mdata ); /* MTOS */
_WORD menu_attach( _WORD me_flag, OBJECT *me_tree, _WORD me_item,
                 MENU *me_mdata ); /* MTOS */
_WORD menu_istart( _WORD me_flag, OBJECT *me_tree, _WORD me_imenu,
                 _WORD me_item ); /* MTOS */
_WORD menu_settings( _WORD me_flag, MN_SET *me_values ); /* MTOS */
_WORD menu_click ( _WORD val, _WORD setit ); /* GEM 3.x */


/****** Object prototypes ************************************************/

/* the objc_sysvar ob_swich values 	*/
#define	LK3DIND		1
#define LK3DACT		2
#define INDBUTCOL	3
#define ACTBUTCOL	4
#define BACKGRCOL	5
#define AD3DVALUE	6
#define MX_ENABLE3D  10                                     /* MagiC 3.0   */

#define OB_GETVAR 0
#define OB_SETVAR 1

_WORD objc_add( OBJECT *ob_atree, _WORD ob_aparent, _WORD ob_achild );
_WORD objc_delete( OBJECT *ob_dltree, _WORD ob_dlobject );
_WORD objc_draw( OBJECT *ob_drtree, _WORD ob_drstartob,
               _WORD ob_drdepth, _WORD ob_drxclip, _WORD ob_dryclip,
               _WORD ob_drwclip, _WORD ob_drhclip );
_WORD objc_find( OBJECT *ob_ftree, _WORD ob_fstartob, _WORD ob_fdepth,
               _WORD ob_fmx, _WORD ob_fmy );
_WORD objc_offset( OBJECT *ob_oftree, _WORD ob_ofobject,
                 _WORD *ob_ofxoff, _WORD *ob_ofyoff );
_WORD objc_order( OBJECT *ob_ortree, _WORD ob_orobject,
                _WORD ob_ornewpos );
_WORD objc_edit( OBJECT *ob_edtree, _WORD ob_edobject,
               _WORD ob_edchar, _WORD *ob_edidx, _WORD ob_edkind );
_WORD objc_change( OBJECT *ob_ctree, _WORD ob_cobject,
                 _WORD ob_cresvd, _WORD ob_cxclip, _WORD ob_cyclip,
                 _WORD ob_cwclip, _WORD ob_chclip,
                 _WORD ob_cnewstate, _WORD ob_credraw );
_WORD objc_sysvar( _WORD ob_svmode, _WORD ob_svwhich,  /* MTOS */
                 _WORD ob_svinval1, _WORD ob_svinval2,
                 _WORD *ob_svoutval1, _WORD *ob_svoutval2); /* MTOS */
_WORD objc_xfind( OBJECT *ob_ftree, _WORD ob_fstartob, _WORD ob_fdepth,
               _WORD ob_fmx, _WORD ob_fmy );
_VOID objc_wchange( OBJECT *ob_ctree, _WORD ob_cobject,
                 _WORD ob_cnewstate, GRECT *clip, _WORD whandle);
_VOID objc_wdraw( OBJECT *ob_drtree, _WORD ob_drstartob,
               _WORD ob_drdepth, GRECT *clip, _WORD whandle);
_WORD objc_wedit( OBJECT *ob_edtree, _WORD ob_edobject,
               _WORD ob_edchar, _WORD *ob_edidx, _WORD ob_edkind, _WORD whandle );


/****** Form definitions ************************************************/

#define FMD_START       0
#define FMD_GROW        1
#define FMD_SHRINK      2
#define FMD_FINISH      3

_WORD form_do( OBJECT *fo_dotree, _WORD fo_dostartob );
_WORD form_dial( _WORD fo_diflag, _WORD fo_dilittlx,
               _WORD fo_dilittly, _WORD fo_dilittlw,
               _WORD fo_dilittlh, _WORD fo_dibigx,
               _WORD fo_dibigy, _WORD fo_dibigw, _WORD fo_dibigh );
_WORD form_alert( _WORD fo_adefbttn, const char *fo_astring );
_WORD form_error( _WORD fo_enum );
_WORD form_center( OBJECT *fo_ctree, _WORD *fo_cx, _WORD *fo_cy,
                 _WORD *fo_cw, _WORD *fo_ch );
_WORD form_keybd( OBJECT *fo_ktree, _WORD fo_kobject, _WORD fo_kobnext,
                _WORD fo_kchar, _WORD *fo_knxtobject, _WORD *fo_knxtchar );
_WORD form_wkeybd( OBJECT *fo_ktree, _WORD fo_kobject, _WORD fo_kobnext,
                _WORD fo_kchar, _WORD *fo_knxtobject, _WORD *fo_knxtchar, _WORD whandle );
_WORD form_button( OBJECT *fo_btree, _WORD fo_bobject, _WORD fo_bclicks,
                _WORD *fo_bnxtobj );
_WORD form_wbutton( OBJECT *fo_btree, _WORD fo_bobject, _WORD fo_bclicks,
                _WORD *fo_bnxtobj, _WORD whandle );

/* MagiC */

typedef struct
{
	_UBYTE scancode;
	_UBYTE nclicks;
	_WORD objnr;
} PACKED SCANX;

typedef struct
{
	SCANX *unsh;
	SCANX *shift;
	SCANX *ctrl;
	SCANX *alt;
	_VOID  *resvd;
} PACKED XDO_INF;

_WORD form_popup(OBJECT *tree, _WORD x, _WORD y);
_WORD form_xdial( _WORD fo_diflag, _WORD fo_dilittlx,
               _WORD fo_dilittly, _WORD fo_dilittlw,
               _WORD fo_dilittlh, _WORD fo_dibigx,
               _WORD fo_dibigy, _WORD fo_dibigw, _WORD fo_dibigh,
               _VOID **flydial );
_WORD form_xdo( OBJECT *tree, _WORD startob, _WORD *lastcrsr, XDO_INF *tabs, _VOID *flydial);
_WORD form_xerr(_LONG errcode, const char *errfile);


/****** Graph definitions ************************************************/


/* Mouse forms */

#define ARROW             0
#define TEXT_CRSR         1
#define HOURGLASS         2
#define BUSY_BEE          HOURGLASS
#define POINT_HAND        3
#define FLAT_HAND         4
#define THIN_CROSS        5
#define THICK_CROSS       6
#define OUTLN_CROSS       7
#define USER_DEF        255
#define M_OFF           256
#define M_ON            257
#define	M_SAVE	        258  /* MO */
#define	M_RESTORE       259  /* MO */
#define	M_PREV          260  /* MO */
#define M_FORCE      0x8000  /* MO */


/* Mouse form definition block */

typedef struct mfstr
{
        _WORD     mf_xhot;
        _WORD     mf_yhot;
        _WORD     mf_nplanes;
        _WORD     mf_fg;
        _WORD     mf_bg;
        _WORD     mf_mask[16];
        _WORD     mf_data[16];
} MFORM;


_WORD graf_rubberbox( _WORD gr_rx, _WORD gr_ry, _WORD gr_minwidth,
                    _WORD gr_minheight, _WORD *gr_rlastwidth,
                    _WORD *gr_rlastheight );
_WORD graf_rubbox( _WORD gr_rx, _WORD gr_ry, _WORD gr_minwidth,
                    _WORD gr_minheight, _WORD *gr_rlastwidth,
                    _WORD *gr_rlastheight );
_WORD graf_dragbox( _WORD gr_dwidth, _WORD gr_dheight,
                  _WORD gr_dstartx, _WORD gr_dstarty,
                  _WORD gr_dboundx, _WORD gr_dboundy,
                  _WORD gr_dboundw, _WORD gr_dboundh,
                  _WORD *gr_dfinishx, _WORD *gr_dfinishy );
_WORD graf_movebox( _WORD gr_mwidth, _WORD gr_mheight,
                  _WORD gr_msourcex, _WORD gr_msourcey,
                  _WORD gr_mdestx, _WORD gr_mdesty );
_WORD graf_mbox( _WORD gr_mwidth, _WORD gr_mheight,
                  _WORD gr_msourcex, _WORD gr_msourcey,
                  _WORD gr_mdestx, _WORD gr_mdesty );
_WORD graf_growbox( _WORD gr_gstx, _WORD gr_gsty,
                  _WORD gr_gstwidth, _WORD gr_gstheight,
                  _WORD gr_gfinx, _WORD gr_gfiny,
                  _WORD gr_gfinwidth, _WORD gr_gfinheight );
_WORD graf_shrinkbox( _WORD gr_sfinx, _WORD gr_sfiny,
                    _WORD gr_sfinwidth, _WORD gr_sfinheight,
                    _WORD gr_sstx, _WORD gr_ssty,
                    _WORD gr_sstwidth, _WORD gr_sstheight );
_WORD graf_watchbox( OBJECT *gr_wptree, _WORD gr_wobject,
                   _WORD gr_winstate, _WORD gr_woutstate );
_WORD graf_wwatchbox( OBJECT *gr_wptree, _WORD gr_wobject,
                   _WORD gr_winstate, _WORD gr_woutstate, _WORD whandle );
_WORD graf_slidebox( OBJECT *gr_slptree, _WORD gr_slparent,
                   _WORD gr_slobject, _WORD gr_slvh );
_WORD graf_handle( _WORD *gr_hwchar, _WORD *gr_hhchar,
                 _WORD *gr_hwbox, _WORD *gr_hhbox );
_WORD graf_mouse( _WORD gr_monumber, MFORM *gr_mofaddr );
_WORD graf_mkstate( _WORD *gr_mkmx, _WORD *gr_mkmy,
                  _UWORD *gr_mkmstate, _UWORD *gr_mkkstate );
_WORD graf_xhandle( _WORD *gr_hwchar, _WORD *gr_hhchar,
                 _WORD *gr_hwbox, _WORD *gr_hhbox, _WORD *dev );


/****** Scrap definitions ***********************************************/

_WORD scrp_read( char *sc_rpscrap );
_WORD scrp_write( char *sc_wpscrap );
_WORD scrp_clear ( _VOID );               /* GEM 2.x */


/****** File selector definitions ***************************************/

_WORD fsel_input( char *fs_iinpath, char *fs_iinsel,
                _WORD *fs_iexbutton );
_WORD fsel_exinput( char *fs_einpath, char *fs_einsel,
                _WORD *fs_eexbutton, const char *fs_elabel );
_WORD fsel_boxinput( char *fs_einpath, char *fs_einsel,
                _WORD *fs_eexbutton, const char *fs_elabel, _VOID _CDECL (*hndl_message)(_WORD *message) );


/****** Window definitions **********************************************/

#define NAME    0x0001
#define CLOSER  0x0002
#define FULLER  0x0004
#define MOVER   0x0008
#define INFO    0x0010
#define SIZER   0x0020
#define UPARROW 0x0040
#define DNARROW 0x0080
#define VSLIDE  0x0100
#define LFARROW 0x0200
#define RTARROW 0x0400
#define HSLIDE  0x0800
#define HOTCLOSEBOX      0x1000                             /* GEM 2.x     */
#define BACKDROP         0x2000                             /* KAOS 1.4    */
#define SMALLER 0x4000

#define WF_KIND          1  /* - */
#define WF_NAME          2  /* S */
#define WF_INFO          3  /* S */
#define WF_WORKXYWH      4  /* G */
#define WF_CURRXYWH      5  /* G&S */
#define WF_PREVXYWH      6  /* G */
#define WF_FULLXYWH      7  /* G */
#define WF_HSLIDE        8  /* G&S */
#define WF_VSLIDE        9  /* G&S */
#define WF_TOP          10  /* G&S */
#define WF_FIRSTXYWH    11  /* G */
#define WF_NEXTXYWH     12  /* G */
#define WF_RESVD        13  /* S */
#define WF_NEWDESK      14  /* G&S */
#define WF_HSLSIZE      15  /* G&S */
#define WF_VSLSIZE      16  /* G&S */
#define WF_SCREEN       17  /* G */
#define WF_COLOR        18  /* G&S */
#define WF_DCOLOR       19  /* G&S */
#define WF_OWNER        20  /* G  MTOS */
#define WF_BEVENT       24  /* G&S  MTOS */
#define WF_BOTTOM       25  /* G&S  MTOS */
#define WF_ICONIFY       26                                 /* AES 4.1     */
#define WF_UNICONIFY     27                                 /* AES 4.1     */
#define WF_UNICONIFYXYWH 28                                 /* AES 4.1     */
#define WF_TOOLBAR       30
#define WF_FTOOLBAR      31
#define WF_NTOOLBAR      32
#define WF_M_BACKDROP    100                                /* KAOS 1.4    */
#define WF_M_OWNER       101                                /* KAOS 1.4    */
#define WF_M_WINDLIST    102                                /* KAOS 1.4    */

/* wind_set(WF_DCOLOR) */

#define W_BOX            0
#define W_TITLE          1
#define W_CLOSER         2
#define W_NAME           3
#define W_FULLER         4
#define W_INFO           5
#define W_DATA           6
#define W_WORK           7
#define W_SIZER          8
#define W_VBAR           9
#define W_UPARROW       10
#define W_DNARROW       11
#define W_VSLIDE        12
#define W_VELEV         13
#define W_HBAR          14
#define W_LFARROW       15
#define W_RTARROW       16
#define W_HSLIDE        17
#define W_HELEV         18
#define W_SMALLER       19                                 /* AES 4.1     */
#define W_BOTTOMER      20                                 /* MagiC 3     */


#define WA_UPPAGE 0
#define WA_DNPAGE 1
#define WA_UPLINE 2
#define WA_DNLINE 3
#define WA_LFPAGE 4
#define WA_RTPAGE 5
#define WA_LFLINE 6
#define WA_RTLINE 7

/* wind calc flags */
#define WC_BORDER 0
#define WC_WORK   1

/* update flags */
#define END_UPDATE 0
#define BEG_UPDATE 1
#define END_MCTRL  2
#define BEG_MCTRL  3
#define BEG_CHECK  0x100   /* MO */

/* AP_DRAGDROP return codes */
#define DD_OK        0
#define DD_NAK       1
#define DD_EXT       2
#define DD_LEN       3
#define DD_TRASH     4
#define DD_PRINTER   5
#define DD_CLIPBOARD 6


_WORD wind_create( _WORD wi_crkind, _WORD wi_crwx, _WORD wi_crwy,
                 _WORD wi_crww, _WORD wi_crwh );
_WORD wind_open( _WORD wi_ohandle, _WORD wi_owx, _WORD wi_owy,
               _WORD wi_oww, _WORD wi_owh );
_WORD wind_close( _WORD wi_clhandle );
_WORD wind_delete( _WORD wi_dhandle );
_WORD wind_get( _WORD wi_ghandle, _WORD wi_gfield, ... );
_WORD wind_set( _WORD wi_shandle, _WORD wi_sfield, ... );
_WORD wind_find( _WORD wi_fmx, _WORD wi_fmy );
_WORD wind_update( _WORD wi_ubegend );
_WORD wind_calc( _WORD wi_ctype, _WORD wi_ckind, _WORD wi_cinx,
               _WORD wi_ciny, _WORD wi_cinw, _WORD wi_cinh,
               _WORD *coutx, _WORD *couty, _WORD *coutw,
               _WORD *couth );
_VOID wind_new( _VOID );
_WORD wind_draw(_WORD wi_dhandle, _WORD wi_dstartob);


/****** Resource definitions ********************************************/

_WORD rsrc_load( const char *re_lpfname );
_WORD rsrc_free( _VOID );
_WORD rsrc_gaddr( _WORD re_gtype, _WORD re_gindex, _VOID *gaddr );
_WORD rsrc_saddr( _WORD re_stype, _WORD re_sindex, _VOID *saddr );
_WORD rsrc_obfix( OBJECT *re_otree, _WORD re_oobject );
_WORD rsrc_rcfix( RSHDR *rc_header ); /* MTOS */
_BOOL rsrc_flip(_VOID *data, _LONG size);


/****** Shell definitions ***********************************************/

/* tail for default shell */

typedef struct
     {
     _WORD  dummy;                   /* ein Nullwort               */
     _LONG  magic;                   /* 'SHEL', wenn ist Shell     */
     _WORD  isfirst;                 /* erster Aufruf der Shell    */
     _LONG  lasterr;                 /* letzter Fehler             */
     _WORD  wasgr;                   /* Programm war Grafikapp.    */
     } SHELTAIL;

/* shel_write modes for parameter "isover" */

#define SHW_IMMED      0  /* PC-GEM 2.x */
#define SHW_CHAIN      1  /* TOS */
#define SHW_DOS        2  /* PC-GEM 2.x */
#define SHW_PARALLEL 100  /* MAG!X */
#define SHW_SINGLE   101  /* MAG!X */

/* shel_write modes for parameter "doex" */

#define SHW_NOEXEC    0  /* MO,PC-GEM 2.x */
#define SHW_EXEC      1  /* MO */
#define SHW_ACCEXEC   3  /* MO */
#define SHW_SHUTDOWN  4  /* MO */
#define SHW_RESCHG    5  /* MO */
#define SHW_GLOBMSG   7  /* MO */
#define SHW_SETENV    8  /* MO */
#define SHW_MSGREC    9  /* MO */
#define SHW_SYSMSG   10  /* MO */
#define SHW_THR_CREATE 20

/* shel_write extended mode flags */
#define SHWF_LIMIT 0x0100  /* MO */
#define SHWF_NICE  0x0200  /* MO */
#define SHWF_DIR   0x0400  /* MO */
#define SHWF_ENV   0x0800  /* MO */

typedef struct   /* MO */
{
        char *prgname;
        _LONG psetlimit;
        _LONG prenice;
        char *directory;
        _VOID *environment;
} SHWPARBLK;

typedef struct {
    _LONG _CDECL (*proc)(_VOID *par);
    _VOID *user_stack;
    _ULONG stacksize;
    _WORD mode;      /* immer auf 0 setzen! */
    _LONG res1;      /* immer auf 0L setzen! */
} THREADINFO;

_WORD shel_read( char *sh_rpcmd, char *sh_rptail );
_WORD shel_write( _WORD sh_wdoex, _WORD sh_wisgr, _WORD sh_wiscr,
                const char *sh_wpcmd, const char *sh_wptail );
#define shel_writeX(a,b,c,d,f)	shel_write(a,b,c,(const char *)d,f)
_WORD shel_get( char *sh_gaddr, _WORD sh_glen );
_WORD shel_put( char *sh_paddr, _WORD sh_plen );
_WORD shel_find( char *sh_fpbuff );
_WORD shel_envrn( char **sh_epvalue, char *sh_eparm );
_WORD shel_rdef ( char *cmd, char *dir ); /* GEM 2.x */
_WORD shel_wdef ( char *cmd, char *dir ); /* GEM 2.x */

#define SHP_HELP 0
_WORD shel_help (_WORD sh_hmode, const char *sh_hfile, const char *sh_hkey);


/****** Xgrf definitions ***********************************************/

extern _WORD xgrf_stepcalc(_WORD orgw, _WORD orgh, _WORD xc, _WORD yc, _WORD wc, _WORD hc,
    _WORD *pxc, _WORD *pyc, _WORD *pcnt, _WORD *pxstep, _WORD *pystep);
extern _WORD xgrf_2box(_WORD xc, _WORD yc, _WORD w, _WORD h, _WORD corners, _WORD cnt,
    _WORD xstep, _WORD ystep, _WORD doubled);


/************************************************************************/

#endif /* ATARI */

EXTERN_C_END

#endif /* __PORTAES_H__ */
