/*****************************************************************************
 * ENDIAN.C
 *****************************************************************************/

#include <portab.h>
#include <s_endian.h>

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD get_byte_order(_VOID)
{
	LOCAL _WORD byte_order;

	if (byte_order == 0)
	{
		union
		{
			_UBYTE  c[4];
			_LONG   l;
		} a;

		a.l = 0x11223344l;
		if (a.c[0] == 0x11 && a.c[1] == 0x22 && a.c[2] == 0x33 && a.c[3] == 0x44)
			byte_order = BYTE_ORDER_BIG_ENDIAN;
		else if (a.c[0] == 0x44 && a.c[1] == 0x33 && a.c[2] == 0x22 && a.c[3] == 0x11)
			byte_order = BYTE_ORDER_LITTLE_ENDIAN;
		else if (a.c[0] == 0x22 && a.c[1] == 0x11 && a.c[2] == 0x44 && a.c[3] == 0x33)
			byte_order = BYTE_ORDER_PDP_ENDIAN;
		/* else ??? */
	}
	return byte_order;
}

/*** ---------------------------------------------------------------------- ***/

#undef h2ns
GLOBAL _UWORD h2ns(_UWORD val)
{
	switch (HOST_BYTE_ORDER)
	{
	case BYTE_ORDER_BIG_ENDIAN:
		break;
	default:
		{
			_UBYTE c, *adr = (_UBYTE *)&val;
	
			c = adr[0];
			adr[0] = adr[1];
			adr[1] = c;
		}
		break;
	}
	return val;
}

/*** ---------------------------------------------------------------------- ***/

#undef n2hs
GLOBAL _UWORD n2hs(_UWORD val)
{
	switch (HOST_BYTE_ORDER)
	{
	case BYTE_ORDER_BIG_ENDIAN:
		break;
	default:
		{
			_UBYTE c, *adr = (_UBYTE *)&val;
	
			c = adr[0];
			adr[0] = adr[1];
			adr[1] = c;
		}
		break;
	}
	return val;
}

/*** ---------------------------------------------------------------------- ***/

#undef h2nl
GLOBAL _ULONG h2nl(_ULONG val)
{
	switch (HOST_BYTE_ORDER)
	{
	case BYTE_ORDER_LITTLE_ENDIAN:
		{
			_UBYTE *to, *from;
			_ULONG buf = val;
			
			to = (_UBYTE *)&val;
			from = (_UBYTE *)&buf;
			to[0] = from[3];
			to[1] = from[2];
			to[2] = from[1];
			to[3] = from[0];
		}
		break;
	case BYTE_ORDER_PDP_ENDIAN:
		{
			_UBYTE *to, *from;
			_ULONG buf = val;
			
			to = (_UBYTE *)&val;
			from = (_UBYTE *)&buf;
			to[0] = from[1];
			to[1] = from[0];
			to[2] = from[3];
			to[3] = from[2];
		}
		break;
	}
	return val;
}

/*** ---------------------------------------------------------------------- ***/

#undef n2hl
GLOBAL _ULONG n2hl(_ULONG val)
{
	switch (HOST_BYTE_ORDER)
	{
	case BYTE_ORDER_LITTLE_ENDIAN:
		{
			_UBYTE *to, *from;
			_ULONG buf = val;
			
			to = (_UBYTE *)&val;
			from = (_UBYTE *)&buf;
			to[0] = from[3];
			to[1] = from[2];
			to[2] = from[1];
			to[3] = from[0];
		}
		break;
	case BYTE_ORDER_PDP_ENDIAN:
		{
			_UBYTE *to, *from;
			_ULONG buf = val;
			
			to = (_UBYTE *)&val;
			from = (_UBYTE *)&buf;
			to[0] = from[1];
			to[1] = from[0];
			to[2] = from[3];
			to[3] = from[2];
		}
		break;
	}
	return val;
}



#ifdef MAIN
int main(void)
{
	_WORD byte_order;
	
	byte_order = get_byte_order();
    printf("Byteorder: %d (%s)\n", byte_order,
    	byte_order == BYTE_ORDER_BIG_ENDIAN ? "big endian" :
    	byte_order == BYTE_ORDER_LITTLE_ENDIAN ? "little endian" :
    	byte_order == BYTE_ORDER_PDP_ENDIAN ? "pdpb endian" :
    	"unknown");
    printf("sizeof(short):  %d\n", (int)sizeof(short));
    printf("sizeof(int):    %d\n", (int)sizeof(int));
    printf("sizeof(long):   %d\n", (int)sizeof(long));
    printf("sizeof(ptr):    %d\n", (int)sizeof(void *));
    printf("sizeof(_WORD):  %d\n", (int)sizeof(_WORD));
    printf("sizeof(_LONG):  %d\n", (int)sizeof(_LONG));
    printf("sizeof(float):  %d\n", (int)sizeof(float));
    printf("sizeof(double): %d\n", (int)sizeof(double));
    printf("sizeof(long double): %d\n", (int)sizeof(long double));
    return 0;
}
#endif
