/*****************************************************************************
 *		MENU_KEY.C
 *****************************************************************************/

#include <menu_key.h>
#include <keycode.h>
#include <mtappl.h>

#include <tos.h>
#include <mtutils.h>
#include <ro_mem.h>
#include <routinec.h>
#include <routines.h>
#include <routine.h>
#include <termproc.h>

#define KEY_STATEMASK (KEY_ALT | KEY_CTRL | KEY_SHIFT)
#define KEY_KEYMASK (~KEY_STATEMASK)


/* Menue-Indices */
#define menu_the_bar(menu) (menu[ROOT].ob_head)
#define menu_the_active(menu) (menu[menu_the_bar(menu)].ob_head)
#define menu_the_first(menu) (menu[menu_the_active(menu)].ob_head)
#define menu_the_last(menu) (menu[menu_the_active(menu)].ob_tail)

#define SYM_SHIFT 0x1
#define SYM_ALT   0x7
#define SYM_CTRL  0x5e

#define OBTYPEMASK 0xff


typedef struct
{
	_UWORD value;
	CONST _UBYTE txt[8];
} KEY_TAB;


RLOCAL CONST KEY_TAB KeyTable[] =
{
	{KEY_F1,  {'F', '1', '\0'}},
	{KEY_F2,  {'F', '2', '\0'}},
	{KEY_F3,  {'F', '3', '\0'}},
	{KEY_F4,  {'F', '4', '\0'}},
	{KEY_F5,  {'F', '5', '\0'}},
	{KEY_F6,  {'F', '6', '\0'}},
	{KEY_F7,  {'F', '7', '\0'}},
	{KEY_F8,  {'F', '8', '\0'}},
	{KEY_F9,  {'F', '9', '\0'}},
	{KEY_F10, {'F', '1', '0', '\0'}},

	{KEY_F1 | KEY_SHIFT, {SYM_SHIFT, 'F', '1', '\0'}},
	{KEY_F2 | KEY_SHIFT, {SYM_SHIFT, 'F', '2', '\0'}},
	{KEY_F3 | KEY_SHIFT, {SYM_SHIFT, 'F', '3', '\0'}},
	{KEY_F4 | KEY_SHIFT, {SYM_SHIFT, 'F', '4', '\0'}},
	{KEY_F5 | KEY_SHIFT, {SYM_SHIFT, 'F', '5', '\0'}},
	{KEY_F6 | KEY_SHIFT, {SYM_SHIFT, 'F', '6', '\0'}},
	{KEY_F7 | KEY_SHIFT, {SYM_SHIFT, 'F', '7', '\0'}},
	{KEY_F8 | KEY_SHIFT, {SYM_SHIFT, 'F', '8', '\0'}},
	{KEY_F9 | KEY_SHIFT, {SYM_SHIFT, 'F', '9', '\0'}},
	{KEY_F10 | KEY_SHIFT, {SYM_SHIFT, 'F', '1', '0', '\0'}},

	{KEY_F1 | KEY_SHIFT, {'s', 'F', '1', '\0'}},
	{KEY_F2 | KEY_SHIFT, {'s', 'F', '2', '\0'}},
	{KEY_F3 | KEY_SHIFT, {'s', 'F', '3', '\0'}},
	{KEY_F4 | KEY_SHIFT, {'s', 'F', '4', '\0'}},
	{KEY_F5 | KEY_SHIFT, {'s', 'F', '5', '\0'}},
	{KEY_F6 | KEY_SHIFT, {'s', 'F', '6', '\0'}},
	{KEY_F7 | KEY_SHIFT, {'s', 'F', '7', '\0'}},
	{KEY_F8 | KEY_SHIFT, {'s', 'F', '8', '\0'}},
	{KEY_F9 | KEY_SHIFT, {'s', 'F', '9', '\0'}},
	{KEY_F10 | KEY_SHIFT, {'s', 'F', '1', '0', '\0'}},

	{KEY_F1 | KEY_CTRL, {'^', 'F', '1', '\0'}},
	{KEY_F2 | KEY_CTRL, {'^', 'F', '2', '\0'}},
	{KEY_F3 | KEY_CTRL, {'^', 'F', '3', '\0'}},
	{KEY_F4 | KEY_CTRL, {'^', 'F', '4', '\0'}},
	{KEY_F5 | KEY_CTRL, {'^', 'F', '5', '\0'}},
	{KEY_F6 | KEY_CTRL, {'^', 'F', '6', '\0'}},
	{KEY_F7 | KEY_CTRL, {'^', 'F', '7', '\0'}},
	{KEY_F8 | KEY_CTRL, {'^', 'F', '8', '\0'}},
	{KEY_F9 | KEY_CTRL, {'^', 'F', '9', '\0'}},
	{KEY_F10 | KEY_CTRL, {'^', 'F', '1', '0', '\0'}},

	{KEY_DC,     {'D', 'e', 'l', 'e', 't', 'e', '\0'}},
	{KEY_DC,     {'D', 'e', 'l', '\0'}},
	{KEY_CANCEL, {'E', 's', 'c', 'a', 'p', 'e', '\0'}},
	{KEY_CANCEL, {'E', 's', 'c', '\0'}},
	{KEY_IC,     {'I', 'n', 's', 'e', 'r', 't', '\0'}},
	{KEY_IC,     {'I', 'n', 's', '.', '\0'}},
	{KEY_STAB,   {'T', 'a', 'b', '\0'}},
	{KEY_UNDO,   {'U', 'N', 'D', 'O', '\0'}},
	{KEY_ENTER,  {'R', 'e', 't', 'u', 'r', 'n', '\0'}},
	{KEY_ENTER,  {'R', 'e', 't', '.', '\0'}},
	{KEY_HELP,   {'H', 'e', 'l', 'p', '\0'}},
	{KEY_HOME,   {'H', 'o', 'm', 'e', '\0'}},
	{KEY_END,    {'C', 'l', 'r', 'H', 'o', 'm', 'e', '\0'}},

	{KEY_KP_ENTER, {'E', 'n', 't', 'e', 'r', '\0'}},
	{KEY_KP_IC, {'K', 'P', '-', '0', '\0'}},
	
	{0, {0}}
};

LOCAL _WORD *menu_keytab;

RLOCAL CONST _UWORD TransTable[256] = {
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			KEY_BACKSPACE,
			KEY_STAB,
/* 0x10 */	0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			KEY_ENTER,
			0x00,
			0x00,
			0x00,
/* 0x20 */	0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			KEY_LEFT | KEY_SHIFT,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
/* 0x30 */	0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			KEY_RIGHT | KEY_SHIFT,
			0x00,
			0x00,
			0x00,
			0x00,
			KEY_F1,
			KEY_F2,
			KEY_F3,
			KEY_F4,
			KEY_F5,
/* 0x40 */	KEY_F6,
			KEY_F7,
			KEY_F8,
			KEY_F9,
			KEY_F10,
			0x00,
			0x00,
			KEY_HOME,
			KEY_UP,
			KEY_PPAGE,
			0x00,
			KEY_LEFT,
			0x00,
			KEY_RIGHT,
			0x00,
			KEY_END,
/* 0x50 */	KEY_DOWN,
			KEY_NPAGE,
			KEY_IC,
			KEY_DC,
			KEY_F1 | KEY_SHIFT,
			KEY_F2 | KEY_SHIFT,
			KEY_F3 | KEY_SHIFT,
			KEY_F4 | KEY_SHIFT,
			KEY_F5 | KEY_SHIFT,
			KEY_F6 | KEY_SHIFT,
			KEY_F7 | KEY_SHIFT,
			KEY_F8 | KEY_SHIFT,
			KEY_F9 | KEY_SHIFT,
			KEY_F10 | KEY_SHIFT,
			0x00,
			0x00,
/* 0x60 */	0x00,
			KEY_UNDO,
			KEY_HELP,
			0x00,
			0x00,
			0x00,
			0x00,
			KEY_KP_HOME,
			KEY_KP_UP,
			KEY_KP_PPAGE,
			KEY_KP_LEFT,
			KEY_KP_CENTER,
			KEY_KP_RIGHT,
			KEY_KP_END,
			KEY_KP_DOWN,
			KEY_KP_NPAGE,
/* 0x70 */	KEY_KP_IC,
			KEY_KP_DECIMAL,
			KEY_KP_ENTER,
			KEY_LEFT | KEY_CTRL,
			KEY_RIGHT | KEY_CTRL,
			KEY_END | KEY_CTRL,
			KEY_NPAGE | KEY_CTRL,
			KEY_HOME | KEY_CTRL,
			'1' | KEY_ALT,
			'2' | KEY_ALT,
			'3' | KEY_ALT,
			'4' | KEY_ALT,
			'5' | KEY_ALT,
			'6' | KEY_ALT,
			'7' | KEY_ALT,
			'8' | KEY_ALT,
/* 0x80 */	'9' | KEY_ALT,
			'0' | KEY_ALT,
			0x9e | KEY_ALT,
			0x27 | KEY_ALT,
			KEY_PPAGE | KEY_CTRL,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
			0x00,
/* 0x90 */	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL FindKeycode(CONST _UBYTE *str, _WORD *keycode)
{
	CONST KEY_TAB *useTab;

	for (useTab = KeyTable; useTab->value != 0; useTab++)
	{
		if (strCaseCmp(useTab->txt, str) == 0)
		{
			*keycode = useTab->value;
			return TRUE;
		}
	}
	*keycode = 0;
	while (*str != '\0')
	{
		if (*str == SYM_SHIFT)
			*keycode |= KEY_SHIFT;
		else if (*str == SYM_CTRL)
			*keycode |= KEY_CTRL;
		else if (*str == SYM_ALT)
			*keycode |= KEY_ALT;
		else if ((*keycode & KEY_KEYMASK) == 0)
			*keycode |= ucase(*str);
		else
			return FALSE;
		str++;
	}
	if ((*keycode & KEY_KEYMASK) != 0 && *str == '\0')
		return TRUE;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/
/*
 * return index of title belonging to menu entry
 */
GLOBAL _WORD MenuKey_FindTitle(OBJECT *menu, _WORD search_idx)
{
	_WORD   title, menui, idx, titlebox;

	titlebox = menu[menu[ROOT].ob_head].ob_head;
	title = menu[titlebox].ob_head;
	menui = menu[menu[ROOT].ob_tail].ob_head;
	do
	{
		idx = menu[menui].ob_head;
		do
		{
			if (idx == search_idx)
				return title;
			if (menu[idx].ob_head != NIL)
			{
				_WORD idx2, submenu;
				
				submenu = menu[idx].ob_head;
				idx2 = menu[submenu].ob_head;
				do
				{
					if (idx2 == search_idx)
						return title;
					idx2 = menu[idx2].ob_next;
				} while (idx2 != submenu);
			}
			idx = menu[idx].ob_next;
		} while (idx != menui);
		menui = menu[menui].ob_next;
		title = menu[title].ob_next;
	} while (title != titlebox);
	return NIL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL MenuKey_FindKeycode(CONST _UBYTE *str, _WORD *keycode, _WORD *offset)
{
	_UBYTE *menueStr, *fiStr;
	_BOOL found;

	found = FALSE;
	if ((menueStr = STRDUP(str, "MenuKey_FindKeycode")) != NULL)
	{
		for (;;)
		{
			fiStr = strrChr(menueStr, ' ');
			if (fiStr == NULL)
				break;
			if (*(fiStr + 1) == '\0')
				*fiStr = '\0';
			else
				break;
		}

		if (fiStr != NULL)
		{
			while (*fiStr == ' ')
				fiStr++;
			if (*fiStr != '\0')
			{
				found = FindKeycode(fiStr, keycode);
				if (found)
				{
					if (offset != NULL)
						*offset = (_WORD)(fiStr - menueStr);
					while (fiStr != menueStr && fiStr[-1] == ' ')
						--fiStr;
					if (fiStr == menueStr)
						found = FALSE;
				}
			}
		}
		SFREE(menueStr);
	}
	return found;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL MenuKey_KeyToStr(_WORD key, _UBYTE *str, size_t strsize)
{
	CONST KEY_TAB *useTab;
	_BOOL found;
	CONST _UBYTE *txt;
	
	found = FALSE;
	if (strsize == 0)
		return FALSE;
	--strsize;
	for (useTab = KeyTable; useTab->value != 0; useTab++)
	{
		if ((_WORD)useTab->value == key)
		{
			txt = useTab->txt;
			while (*txt != '\0' && strsize)
			{
				*str++ = *txt;
				txt++;
				strsize--;
			}
			found = TRUE;
			break;
		}
	}
	
	if (!found)
	{
		if (key & KEY_SHIFT)
		{
			if (strsize != 0)
			{
				*str++ = SYM_SHIFT;
				strsize--;
			}
		}
		if (key & KEY_ALT)
		{
			if (strsize != 0)
			{
				*str++ = SYM_ALT;
				strsize--;
			}
		}
		if (key & KEY_CTRL)
		{
			if (strsize != 0)
			{
				*str++ = SYM_CTRL;
				strsize--;
			}
		}
	
		if ((key & KEY_KEYMASK) < 256)
		{
			if (strsize)
			{
				*str++ = ucase(key & KEY_KEYMASK);
				strsize--;
			}
			found = TRUE;
		}
	}
	*str = '\0';
	return found;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD *workMenue(_WORD *menueTab, _WORD *menuePos, OBJECT *tree, _WORD obj)
{
	_WORD key;

	if (MenuKey_FindKeycode(tree[obj].ob_spec.free_string, &key, NULL))
	{
		if (menueTab == NULL)
		{
			*menuePos += 3;
		} else
		{
			*menuePos = key;
			menuePos++;
			*menuePos = obj;
			menuePos++;
			*menuePos = MenuKey_FindTitle(tree, obj);
			menuePos++;
		}
	}
	return menuePos;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID MenuKey_Exit(_VOID)
{
	if (menu_keytab != NULL)
	{
		SFREE(menu_keytab);
		menu_keytab = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID MenuKey_Init(OBJECT *tree)
{
	size_t tabsize;
	_WORD *menuePos;
	_WORD title, menui, idx, titlebox;
	_WORD count;
	_BOOL deskmenu;
	
	MenuKey_Exit();
	if (tree != NULL)
	{
		/*
		 * einmal durchgehen zum zaehlen
		 */
		count = 0;

		deskmenu = TRUE;
		titlebox = tree[tree[ROOT].ob_head].ob_head;
		title = tree[titlebox].ob_head;
		menui = tree[tree[ROOT].ob_tail].ob_head;
		do
		{
			idx = tree[menui].ob_head;
			do
			{
				if ((tree[idx].ob_type & OBTYPEMASK) == G_STRING)
					workMenue(NULL, &count, tree, idx);
				if (tree[idx].ob_head != NIL)
				{
					_WORD idx2, submenu;
					
					submenu = tree[idx].ob_head;
					idx2 = tree[submenu].ob_head;
					do
					{
						if ((tree[idx2].ob_type & OBTYPEMASK) == G_STRING)
							workMenue(NULL, &count, tree, idx2);
						idx2 = tree[idx2].ob_next;
					} while (idx2 != submenu);
				}
				idx = tree[idx].ob_next;
				if (deskmenu)
				{
					idx = menui;
					deskmenu = FALSE;
				}
			} while (idx != menui);
			menui = tree[menui].ob_next;
			title = tree[title].ob_next;
		} while (title != titlebox);
	
		count++;
		tabsize = (size_t) (count) * sizeof(_WORD);
		if ((menu_keytab = MALLOC(tabsize, "MenueKey_Init")) != NULL)
		{
			InstallTermproc(MenuKey_Exit);
			/*
			 * nochmal durchgehen und in Tabelle eintragen
			 */
			menuePos = menu_keytab;
			deskmenu = TRUE;
			titlebox = tree[tree[ROOT].ob_head].ob_head;
			title = tree[titlebox].ob_head;
			menui = tree[tree[ROOT].ob_tail].ob_head;
			do
			{
				idx = tree[menui].ob_head;
				do
				{
					if ((tree[idx].ob_type & OBTYPEMASK) == G_STRING)
						menuePos = workMenue(menu_keytab, menuePos, tree, idx);
					if (tree[idx].ob_head != NIL)
					{
						_WORD idx2, submenu;
						
						submenu = tree[idx].ob_head;
						idx2 = tree[submenu].ob_head;
						do
						{
							if ((tree[idx2].ob_type & OBTYPEMASK) == G_STRING)
								menuePos = workMenue(menu_keytab, menuePos, tree, idx2);
							idx2 = tree[idx2].ob_next;
						} while (idx2 != submenu);
					}
					idx = tree[idx].ob_next;
					if (deskmenu)
					{
						idx = menui;
						deskmenu = FALSE;
					}
				} while (idx != menui);
				menui = tree[menui].ob_next;
				title = tree[title].ob_next;
			} while (title != titlebox);
			*menuePos = 0;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UWORD MenuKey_TranslateKey(_WORD keycode, _UWORD kbshift)
{
	_KEYTAB *keytab;
	_UBYTE *tab;
	_UBYTE scan;
	_UBYTE ch;
	_UWORD newcode;
	
	keytab = Keytbl((_VOID *)-1, (_VOID *)-1, (_VOID *)-1);
	scan = (keycode >> 8) & 0xff;
	ch = keycode & 0xff;
	newcode = TransTable[scan];
	if (kbshift & (K_LSHIFT | K_RSHIFT))
		kbshift |= K_LSHIFT | K_RSHIFT;
	kbshift = ((kbshift >> 1) & 0x07) << 13;
	if (newcode != 0)
	{
		newcode |= kbshift;
	} else
	{
		if (kbshift & KEY_SHIFT)
		{
			tab = keytab->shift;
			ch = tab[scan];
			if (is_alpha(ch))
				newcode = ucase(ch) | (kbshift & ~KEY_SHIFT);
			else
				newcode = ucase(ch) | kbshift;
		} else if (kbshift & KEY_ALT)
		{
			tab = keytab->unshift;
			ch = tab[scan];
			if (ch == 0)
				newcode = 0;
			else
				newcode = ucase(ch) | kbshift;
		} else if (kbshift & KEY_CTRL)
		{
			tab = keytab->unshift;
			ch = tab[scan];
			newcode = ucase(ch) | kbshift;
		} else
		{
			if (ch != 0)
				newcode = ch | kbshift;
			else
				newcode = 0;
		}
	}
	return newcode;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL MenuKey_Test(OBJECT *menu_tree, _WORD keycode, _UWORD kbshift, menuProc hdlMenu, _BOOL showMouse)
{
	_WORD *tab;
	_BOOL found;
	
	found = FALSE;
	tab = menu_keytab;
	if (tab == NULL)
	{
		if (menu_tree != NULL)
			MenuKey_Init(menu_tree);
		tab = menu_keytab;
		if (tab == NULL)
			return FALSE;
	}
	keycode = MenuKey_TranslateKey(keycode, kbshift);
	while (*tab != 0)
	{
		if (*tab == keycode)
		{
			_WORD title;

			tab++;
			title = tab[1];
			if (!GetObjState(menu_tree, *tab, DISABLED) &&
				!GetObjState(menu_tree, title, DISABLED))
			{
				if (title == NIL)
					title = menu_the_first(menu_tree);
				menu_tnormal(menu_tree, title, 0);
				if (showMouse)
					MouseOn();
				hdlMenu(*tab, title, kbshift);
				found = TRUE;
			}
			break;
		}
		tab += 3;
	}
	return found;
}
