/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <quicksor.h>
#include <ro_mem.h>


#define Limit 20   /* Ab Limit wird auf InsertSort umgeschaltet. */

typedef _VOID (*copyProc)(_VOID *to, CONST _VOID *from, size_t size);


typedef struct {
	_VOID *ptrelem;
	size_t entrySize;
	compProc comp;
	copyProc memCopy;
	_BOOL breakJN;
	breakProc brk;
	_VOID *tmpBuf;
	_VOID *valueBuf;
	_VOID *private;
} QS_PARA;

LOCAL _VOID ChooseSort(_LONG l, _LONG r, QS_PARA *para);


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID copyByte(_VOID *to, CONST _VOID *from, size_t size)
{
	UNUSED(size);
	*(_UBYTE *)to = *(_UBYTE *)from;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID copyWord(_VOID *to, CONST _VOID *from, size_t size)
{
	UNUSED(size);
	*(_WORD *)to = *(_WORD *)from;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID copyLong(_VOID *to, CONST _VOID *from, size_t size)
{
	UNUSED(size);
	*(_LONG *)to = *(_LONG *)from;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID copy6(_VOID *to, CONST _VOID *from, size_t size)
{
	UNUSED(size);
	*((_LONG *)to)++ = *((_LONG *)from)++;
	*(_WORD *)to = *(_WORD *)from;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID copyDlong(_VOID *to, CONST _VOID *from, size_t size)
{
	UNUSED(size);
	*((_LONG *)to)++ = *((_LONG *)from)++;
	*(_LONG *)to = *(_LONG *)from;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID copy10(_VOID *to, CONST _VOID *from, size_t size)
{
	UNUSED(size);
	*((_LONG *)to)++ = *((_LONG *)from)++;
	*((_LONG *)to)++ = *((_LONG *)from)++;
	*(_WORD *)to = *(_WORD *)from;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID copy12(_VOID *to, CONST _VOID *from, size_t size)
{
	UNUSED(size);
	*((_LONG *)to)++ = *((_LONG *)from)++;
	*((_LONG *)to)++ = *((_LONG *)from)++;
	*(_LONG *)to = *(_LONG *)from;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID copy16(_VOID *to, CONST _VOID *from, size_t size)
{
	UNUSED(size);
	*((_LONG *)to)++ = *((_LONG *)from)++;
	*((_LONG *)to)++ = *((_LONG *)from)++;
	*((_LONG *)to)++ = *((_LONG *)from)++;
	*(_LONG *)to = *(_LONG *)from;
}

/*** ---------------------------------------------------------------------- ***/

#define getIdx(idx)  ((_VOID *)((_UBYTE *)(para->ptrelem) + (idx) * para->entrySize))
#define assign(i, j) para->memCopy(getIdx(i), getIdx(j), para->entrySize)
#define Push(i)      para->memCopy(para->tmpBuf, getIdx(i), para->entrySize)
#define PushValue(i) para->memCopy(para->valueBuf, getIdx(i), para->entrySize)
#define Pop(i)       para->memCopy(getIdx(i), para->tmpBuf, para->entrySize)
#define Swap(i, j) \
	para->memCopy(para->tmpBuf, getIdx(i), para->entrySize), \
	para->memCopy(getIdx(i), getIdx(j), para->entrySize), \
	para->memCopy(getIdx(j), para->tmpBuf, para->entrySize)

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID InsertSort(_LONG l, _LONG r, QS_PARA *para)
{
	_LONG i, j;
	_BOOL b;

	for (i = (l + 1); i <= r; i++)
	{
		Push(i);
		j = i - 1;
		b = para->comp(para->tmpBuf, getIdx(j), para->private);

		while (b)
		{
			assign(j+1, j);
			--j;
			b = j >= l;

			if (b)
				b = para->comp(para->tmpBuf, getIdx(j), para->private);
		}
		Pop(j+1);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID QuickSort(_LONG l, _LONG r, QS_PARA *para)
{
	_LONG i, j;

	i = l;
	j = r;
	PushValue((l+r) / 2);
	do
	{
		while (para->comp(getIdx(i), para->valueBuf, para->private))
			++i;
		while (para->comp(para->valueBuf, getIdx(j), para->private))
			--j;
		if (i <= j)
		{
			Swap(i, j);
			++i;
			--j;
		}
	} while (i <= j);
	if (l < j)
		ChooseSort(l, j, para);
	if (para->breakJN)
		return;
	if (i < r)
		ChooseSort(i, r, para);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ChooseSort(_LONG l, _LONG r, QS_PARA *para)
{
	if (para->breakJN)
		return;
	if ((r - l) > Limit)
		QuickSort(l, r, para);
	else
		InsertSort(l, r, para);
	if (para->brk())
		para->breakJN = TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL QuickSort_noBreak(_VOID)
{
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL QuickSort_sortIt(_LONG low, _LONG high, _VOID *ptrelem, compProc comp, _LONG entrySize, breakProc brk, _VOID *private)
{
	QS_PARA para;

	para.tmpBuf = MALLOC(entrySize, "QuickSort: tmpBuf");
	if (para.tmpBuf == NULL)
		return FALSE;
	para.valueBuf = MALLOC(entrySize, "QuickSort: valueBuf");
	if (para.valueBuf == NULL)
	{
		FREE(para.tmpBuf, entrySize);
		return FALSE;
	}
	para.ptrelem = ptrelem;
	para.entrySize = entrySize;
	para.comp = comp;
	if (brk == FUNK_NULL)
		brk = QuickSort_noBreak;
	para.brk = brk;
	para.breakJN = FALSE;
	para.private = private;
	switch ((int)entrySize)
	{
		case 1: para.memCopy = copyByte; break;
		case 2: para.memCopy = copyWord; break;
		case 4: para.memCopy = copyLong; break;
		case 6: para.memCopy = copy6; break;
		case 8: para.memCopy = copyDlong; break;
		case 10: para.memCopy = copy10; break;
		case 12: para.memCopy = copy12; break;
		case 16: para.memCopy = copy16; break;
		default: para.memCopy = (copyProc)memcpy; break;
	}
	ChooseSort(low, high, &para);
	FREE(para.valueBuf, entrySize);
	FREE(para.tmpBuf, entrySize);
	return TRUE;
}
