/*****************************************************************************
 *		ROUTINE.C
 *****************************************************************************/

#include <portab.h>
#include <routines.h>
#include <string.h>


/*** ---------------------------------------------------------------------- ***/


GLOBAL CONST _UBYTE *strCaseStr(CONST _UBYTE _HUGE *s, CONST _UBYTE *wanted)
{
	CONST _UBYTE _HUGE *scan;
	size_t len;
	_UBYTE firstc;
	
	/*
	 * The odd placement of the two tests is so "" is findable.
	 * Also, we inline the first char for speed.
	 * The ++ on scan has been moved down for optimization.
	 */
	if(!s)
		return NULL;
	if(!wanted)
		return strChr(s,'\0');
	firstc = ucase(*wanted);
	len = strLen(wanted);
	for (scan = s; ucase(*scan) != firstc ||
		strnCaseCmp((CONST _UBYTE *)scan, wanted, len) != 0; )
	{
		if (*scan++ == '\0')
			return NULL;
	}
	return (CONST _UBYTE *)scan;
}

/*** ------------------------------------------------------------------------ ***/
/*	_UBYTE  *strStr(CONST _UBYTE _HUGE *s, CONST _UBYTE *wanted)				*/
/*	wie strstr nur ohne die Abstrze,abgeleitet von strCaseStr					*/
/*	Parameter:	s,wanted: String in dem Unterstring wanted gesucht wird			*/
/*	Ergebnis:	Pointer auf das erste Vorkommen von wanted						*/		
/*	3.8.02 von Dimitri Junker	DYJ 4.41n										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _UBYTE  *strStr(CONST _UBYTE _HUGE *s, CONST _UBYTE *wanted)
{
	CONST _UBYTE _HUGE *scan;
	size_t len;
	_UBYTE firstc;
	
	/*
	 * The odd placement of the two tests is so "" is findable.
	 * Also, we inline the first char for speed.
	 * The ++ on scan has been moved down for optimization.
	 */
	if(!s)
		return NULL;
	if(!wanted)
		return strChr(s,'\0');
	firstc = *wanted;
	len = strLen(wanted);
	for (scan = s; *scan != firstc ||
		strnCmp((CONST _UBYTE *)scan, wanted, len) != 0; )
	{
		if (*scan++ == '\0')
			return NULL;
	}
	return (_UBYTE *)scan;
}
/*** ------------------------------------------------------------------------ ***/
/*	CONST _UBYTE *strCaseWord(CONST _UBYTE _HUGE *s, CONST _UBYTE *wanted)		*/
/*	Wie strCaseStr, allerdings mu vor und hinter dem Wort Leerzeichen o.. sein*/
/*	Parameter:	s:		Darin suchen											*/
/*				wanted:	dies suchen												*/
/*	Ergebnis:	Start oder NULL													*/		
/*	3.12.01 von Dimitri Junker	DYJ 4.30zm										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL CONST _UBYTE *strCaseWord(CONST _UBYTE _HUGE *s, CONST _UBYTE *wanted)
{
	CONST _UBYTE *ret;
	size_t len;
	_UBYTE trennZ[]="\n\r\ ,;!-?:\"'\t";
	if(!wanted)	/* Der Fall s=NULL wird durch strCaseStr abgedeckt*/
		return NULL;

	len = strLen(wanted);
	ret=strCaseStr(s,wanted);
	while(ret)
	{
		if( (ret==s || strChr(trennZ,ret[-1])) &&		/*Entweder am Anfang oder ein Trennzeichen davor*/
			strChr(trennZ,ret[len]))					/*Ein Trennzeichen dahinter ('\0' klappt auch*/
			break;
		ret=strCaseStr(ret+1,wanted);
	}
	return ret;
}

/*** ------------------------------------------------------------------------ ***/
/*	_UBYTE  *strChr( CONST _UBYTE *s, int c )									*/
/*	wie strchr nur ohne die Abstrze											*/
/*	Parameter:	s,c: String in dem das Zeichen c gesucht wird					*/
/*	Ergebnis:	Pointer auf das erste Vorkommen von c							*/		
/*	3.8.02 von Dimitri Junker	DYJ 4.41n										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _UBYTE  *strChr( CONST _UBYTE *s, int c )
{
	CONST _UBYTE _HUGE *scan;
	
	if(!s)
		return NULL;
	for (scan = s; *scan && *scan!=c; scan++);
	if(*scan==c)
		return (_UBYTE *)scan;
	else
		return NULL;
}
/*** ------------------------------------------------------------------------ ***/
/*	size_t strCnt( CONST _UBYTE *s, int c )										*/
/*	Zhlt das Vorkommen eines Zeichens in einem String							*/
/*	Parameter:	s: der String													*/
/*				c: das Zeichen													*/
/*	Ergebnis:	Anzahl c in s													*/		
/*	23.3.03 von Dimitri Junker	DYJ 5.23i										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL size_t strCnt( CONST _UBYTE *s, int c )
{
	size_t anz=0;
	if(s)
	{
		do
		{
			if((*s)==c)
				anz++;
		}while(*(s++));
	}
	return anz;
}

/*** ------------------------------------------------------------------------ ***/
/*	_UBYTE  *strChr( CONST _UBYTE *s, int c )									*/
/*	wie strrchr nur ohne die Abstrze											*/
/*	Parameter:	s,c: String in dem das Zeichen c gesucht wird					*/
/*	Ergebnis:	Pointer auf das erste Vorkommen von c							*/		
/*	3.8.02 von Dimitri Junker	DYJ 4.41n										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _UBYTE  *strrChr( CONST _UBYTE *s, int c )
{
	CONST _UBYTE _HUGE *scan;
	
	if(!s)
		return NULL;
	for (scan = strChr(s,'\0'); scan!=s && *scan!=c; scan--);
	if(*scan==c)
		return (_UBYTE *)scan;
	else
		return NULL;
}

/*** ------------------------------------------------------------------------ ***/
/*	_UBYTE  *strCaseChr( CONST _UBYTE *s, int c )								*/
/*	caseinsensiteve Versin von strChr											*/
/*	Parameter:	s,c: String in dem das Zeichen c caseinsensitiv gesucht wird	*/
/*	Ergebnis:	Pointer auf das erste Vorkommen von c							*/		
/*	3.8.02 von Dimitri Junker	DYJ 4.41n										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _UBYTE  *strCaseChr( CONST _UBYTE *s, int c )
{
	CONST _UBYTE _HUGE *scan;
	c=ucase(c);
	if(!s)
		return NULL;
	for (scan = s; *scan && ucase(*scan)!=c; scan++);
	if(ucase(*scan)==c)
		return (_UBYTE *)scan;
	else
		return NULL;
}
/*** ------------------------------------------------------------------------ ***/
/*	_UBYTE  *strPbrk( CONST _UBYTE *s1, CONST _UBYTE *s2 )						*/
/*	wie strpbrk nur ohne die Abstrze											*/
/*	10.8.02 von Dimitri Junker	DYJ 4.41n										*/
/*** ------------------------------------------------------------------------ ***/
/*DYJ 5.20w */
char    *strpbrk( const char *s, const char *set );	
GLOBAL _UBYTE  *strPbrk( CONST _UBYTE *s1, CONST _UBYTE *s2 )
{
	if(!s1)
		return NULL;
	if(!s2)
		return strChr(s1,'\0');
	return strpbrk(s1,s2 );
}
/*** ------------------------------------------------------------------------ ***/
/*	size_t  strSpn( const char *s, const char *set )						*/
/*	wie strspn nur ohne die Abstrze											*/
/*	9.2.03 von Dimitri Junker	DYJ 5.20w										*/
/*** ------------------------------------------------------------------------ ***/
size_t  strspn( const char *s, const char *set );
size_t  strSpn( const char *s, const char *set )
{
	if(!s)
		return 0;
	if(!set)
		set="";
	return strspn(s,set);
}
	
/*** ------------------------------------------------------------------------ ***/
/*	size_t  strcSpn( const char *s, const char *set )					*/
/*	wie strcspn nur ohne die Abstrze											*/
/*	9.2.03 von Dimitri Junker	DYJ 5.20w										*/
/*** ------------------------------------------------------------------------ ***/
size_t  strcspn( const char *s, const char *set );
size_t  strcSpn( const char *s, const char *set )
{
	if(!s)
		return 0;
	if(!set)
		set="";
	return strcspn(s,set);
}