/*****************************************************************************
 *	RO_TIME.C
 *	system independant time functions
 *****************************************************************************/

#include <portab.h>
#include <ro_time.h>

/* no. of days to beginning of month for each month */
LOCAL _UWORD dsboy[12] =  { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};

#define is_leapyear(year) \
	((year) % 4 == 0 && \
	 ((year) % 100 != 0 || (year) % 400 == 0))

/*** ---------------------------------------------------------------------- ***/

GLOBAL SYSTIME T_Calctime(CONST F_TIME *t)
{
	_ULONG  daycount;
	SYSTIME longtime;

	if (T_IsZero(t))
		return 0;
	/* Calculate days since 1900/01/01 */
	daycount = dsboy[t->tm_mon] +                      /* days since beginning of this year */
		t->tm_mday;                                    /* days since beginning of month */
	if (t->tm_year > BASEYEAR)
		daycount += 365l * (t->tm_year - BASEYEAR) +   /* days due to whole years */
			(t->tm_year - LEAPBASEYEAR) / 4;           /* days due to leap years */

	if (is_leapyear (t->tm_year) && t->tm_mon >= 2)
		daycount++;

	/* Knowing the days, we can find seconds */
	longtime = daycount * SECS_PER_DAY +
		t->tm_hour * SECS_PER_HOUR +
		t->tm_min * SECS_PER_MIN +
		t->tm_sec;
	return longtime;
}
