/*****************************************************************************
 *	RO_TIME.C
 *	system independant time functions
 *****************************************************************************/

#include <portab.h>
#include <ro_time.h>

/* no. of days to beginning of month for each month */
LOCAL _UWORD dsboy[12] =  { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
LOCAL _UWORD dsboly[12] = { 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335};

#define is_leapyear(year) \
	((year) % 4 == 0 && \
	 ((year) % 100 != 0 || (year) % 400 == 0))


/*** ---------------------------------------------------------------------- ***/

LOCAL F_TIME static_t;

GLOBAL F_TIME *T_Converttime(SYSTIME ltime, F_TIME *t)
{
	_ULONG  day;

	if (t == NULL)
		t = &static_t;
	if (ltime == 0)
	{
		T_SetZero(t);
		return t;
	}
	day = ltime / SECS_PER_DAY;
	t->tm_wday = (_UWORD) ((day + (BASEYEAR == 1900 ? 1 : 4)) % 7);
	t->tm_year = BASEYEAR;
	for (;;)
	{
		if (is_leapyear(t->tm_year))
		{
			if (ltime < SECS_PER_LEAPYEAR)
				break;
			ltime -= SECS_PER_LEAPYEAR;
		} else
		{
			if (ltime < SECS_PER_YEAR)
				break;
			ltime -= SECS_PER_YEAR;
		}
		t->tm_year++;
	}
	day = ltime / SECS_PER_DAY;
	ltime -= day * SECS_PER_DAY;
	t->tm_yday = (_UWORD) day;
	t->tm_mon = 0;
	if (is_leapyear(t->tm_year))
	{
		while (t->tm_mon < 11 && day >= dsboly[t->tm_mon + 1])
			t->tm_mon++;
		t->tm_mday = (_UWORD) day - dsboly[t->tm_mon];
	} else
	{
		while (t->tm_mon < 11 && day >= dsboy[t->tm_mon + 1])
			t->tm_mon++;
		t->tm_mday = (_UWORD) day - dsboy[t->tm_mon];
	}
	t->tm_hour = (_UWORD) (ltime / SECS_PER_HOUR);
	ltime -= t->tm_hour * SECS_PER_HOUR;
	t->tm_min = (_UWORD) (ltime / SECS_PER_MIN);
	ltime -= t->tm_min * SECS_PER_MIN;
	t->tm_sec = (_UWORD) ltime;
	return t;
}
