/*****************************************************************************
 *	RO_TIME.C
 *	system independant time functions
 *****************************************************************************/

#include <portab.h>
#include <ro_time.h>
#include <var_args.h>
#include <routines.h>

/*** ---------------------------------------------------------------------- ***/

/* Return the week in the year of the time in TM, with the weeks
   starting on Sundays. */

LOCAL _WORD sun_week(CONST F_TIME *tm)
{
	_WORD dl;

  /* Set `dl' to the day in the year of the last day of the week previous
     to the one containing the day specified in TM.  If the day specified
     in TM is in the first week of the year, `dl' will be negative or 0.
     Otherwise, calculate the number of complete weeks before our week
     (dl / 7) and add any partial week at the start of the year (dl % 7). */
	dl = tm->tm_yday - tm->tm_wday;
	return dl <= 0 ? 0 : dl / 7 + (dl % 7 != 0);
}

/* Return the week in the year of the time in TM, with the weeks
   starting on Mondays. */

LOCAL _WORD mon_week (CONST F_TIME *tm)
{
	_WORD dl, wday;

	if (tm->tm_wday == 0)
		wday = 6;
	else
    	wday = tm->tm_wday - 1;
	dl = tm->tm_yday - wday;
	return dl <= 0 ? 0 : dl / 7 + (dl % 7 != 0);
}

/*** ---------------------------------------------------------------------- ***/
/* :language: */
LOCAL CONST _UBYTE *C_mth_name[] =
{
	"January", "February", "March", "April", "May", "June",
	"July", "August", "September", "October", "November", "December"
};

LOCAL CONST _UBYTE *C_day_name[] =
{
	"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"
};

LOCAL CONST _UBYTE *german_mth_name[] =
{
	"Januar", "Februar", "M\204rz", "April", "Mai", "Juni", "Juli", "August",
	"September", "Oktober", "November", "Dezember"
};

LOCAL CONST _UBYTE *german_day_name[] =
{
	"Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"
};

LOCAL CONST _UBYTE **_LC_Day_name = german_day_name;
LOCAL CONST _UBYTE **_LC_Mth_name = german_mth_name;

/*
   Performs `%' substitutions similar to those in printf.  Except
   where noted, substituted fields have a fixed size; numeric fields are
   padded if necessary.  Padding is with zeros by default; for fields
   that display a single number, padding can be changed or inhibited by
   following the `%' with one of the modifiers described below.  Unknown
   field specifiers are copied as normal characters.  All other
   characters are copied to the output without change.

   Supports a superset of the ANSI C field specifiers.

   Literal character fields:
   %	%
   n	newline
   t	tab

   Numeric modifiers (a nonstandard extension):
   -	do not pad the field
   _	pad the field with spaces

   Time fields:
   %H	hour (00..23)
   %I	hour (01..12)
   %k	hour ( 0..23)
   %l	hour ( 1..12)
   %M	minute (00..59)
   %p	locale's AM or PM
   %r	time, 12-hour (hh:mm:ss [AP]M)
   %R	time, 24-hour (hh:mm)
   %s	time in seconds since 00:00:00, Jan 1, 1970 (a nonstandard extension)
   %S	second (00..61)
   %T	time, 24-hour (hh:mm:ss)
   %X	locale's time representation (%H:%M:%S)
   %Z	time zone (EDT), or nothing if no time zone is determinable

   Date fields:
   %a	locale's abbreviated weekday name (Sun..Sat)
   %0a  C locale's abbreviated weekday name (Sun..Sat)
   %A	locale's full weekday name, variable length (Sunday..Saturday)
   %0A  C locale's full weekday name, variable length (Sunday..Saturday)
   %b	locale's abbreviated month name (Jan..Dec)
   %0b	C locale's abbreviated month name (Jan..Dec)
   %B	locale's full month name, variable length (January..December)
   %0B	C locale's full month name, variable length (January..December)
   %c	locale's date and time (Sat Nov 04 12:02:33 EST 1989)
   %C	century (00..99)
   %d	day of month (01..31)
   %e	day of month ( 1..31)
   %D	date (mm/dd/yy)
   %h	same as %b
   %j	day of year (001..366)
   %m	month (01..12)
   %U	week number of year with Sunday as first day of week (00..53)
   %w	day of week (0..6)
   %W	week number of year with Monday as first day of week (00..53)
   %x	locale's date representation (mm/dd/yy)
   %y	last two digits of year (00..99)
   %Y	year (1970...)
*/

/* Types of padding for numbers in date and time. */
enum padding { none, blank, zero };

#define add_char(c) if (length < maxsize) string[length++] = (c)

LOCAL size_t add_num2 (_UBYTE *string, size_t length, size_t maxsize, _WORD num, enum padding pad)
{
	_WORD top = num / 10;

	if (top == 0 && pad == blank)
	{
    	add_char (' ');
	} else if (top != 0 || pad == zero)
	{
    	add_char (top + '0');
	}
	add_char (num % 10 + '0');
	return length;
}


LOCAL size_t add_num3 (_UBYTE *string, size_t length, size_t maxsize, _WORD num, enum padding pad)
{
	_WORD top = num / 100;
	_WORD mid = (num - top * 100) / 10;

	if (top == 0 && pad == blank)
	{
    	add_char (' ');
	} else if (top != 0 || pad == zero)
	{
    	add_char(top + '0');
	}
	if (mid == 0 && top == 0 && pad == blank)
	{
    	add_char (' ');
	} else if (mid != 0 || top != 0 || pad == zero)
	{
	add_char (mid + '0');
	}
	add_char (num % 10 + '0');
	return length;
}


LOCAL size_t add_str (_UBYTE *string, size_t length, size_t maxsize, CONST _UBYTE *from)
{
	while (*from != '\0')
	{
		add_char(*from);
		from++;
	}
	return length;
}


LOCAL size_t add_num_time_t (_UBYTE *string, size_t length, size_t maxsize, SYSTIME num)
{
	/* This buffer is large enough to hold the character representation
	   (including the trailing NUL) of any unsigned decimal quantity
       whose binary representation fits in 128 bits.  */
	_UBYTE buf[40];

	sprintf (buf, "%lu", (unsigned long) num);
	return add_str (string, length, maxsize, buf);
}

LOCAL size_t t_strftime(_UBYTE *string, size_t maxsize, CONST _UBYTE *format, F_TIME *ts, __c_va_list args)
{
	enum padding pad;
	size_t length = 0; /* characters put in string so far */
	int arg;

	for (; *format != '\0' && length < maxsize; format++)
	{
		if (*format != '%')
		{
			add_char(*format);
			continue;
		}
		
		++format;
		if (*format == '-')
		{
			pad = none;
			++format;
		} else if (*format == '_' || *format == ' ')
		{
			pad = blank;
			++format;
		} else
		{
			pad = zero;
		}
		arg = -1;

#define is_digit(c) ((c) >= '0' && (c) <= '9')

		if (is_digit(*format))
		{
			arg = 0;
			while (is_digit(*format))
				arg = arg * 10 + (*format++ - '0');
		} else if (*format == '*')
		{
			arg = __c_va_arg(args, int);
			format++;
		}
		
		switch (*format)
		{
		/* literal characters: */
		
		case '%':
			add_char('%');
			break;
		case 'n':						/* same as \n */
			add_char('\n');
			break;
		case 't':						/* same as \t */
			add_char('\t');
			break;
		default:
			add_char(*format);
			break;
		
		/* time fields: */
		
		case 'H':	/* hour, 00-23 */
			length = add_num2(string, length, maxsize, ts->tm_hour, pad);
			break;
		case 'k':	/* hour, 00-23, zero padded */
			length = add_num2(string, length, maxsize, ts->tm_hour, zero);
			break;
		case 'I':	/* hour, 01-12 */
		case 'l':	/* hour, 01-12, blank padded */
			{
				_WORD n = ts->tm_hour;
				
				if (n == 0)
					n = 12;
				else if (n > 12)
					n -= 12;
				length = add_num2(string, length, maxsize, n, *format == 'I' ? pad : blank);
			}
			break;
		case 'M': /* minute, 00-59 */
			length = add_num2(string, length, maxsize, ts->tm_min, pad);
			break;
		case 'p': /* am or pm based on 1-hour clock */
			add_char(ts->tm_hour < 12 ? 'A' : 'P');
			add_char('M');
			break;
		case 'r':	/* time as %I:%M:%S %p */
			length += T_Strftime(string+length, maxsize-length, "%I:%M:%S %p", ts);
			break;
		case 'R':	/* time as %H:%M */
			length += T_Strftime(string+length, maxsize-length, "%H:%M", ts);
			break;
		case 's':
			length = add_num_time_t(string, length, maxsize, T_Mktime(ts) + T_DIFFTIME);
			break;
		case 'S':	/* second, 00-59 */
			length = add_num2(string, length, maxsize, ts->tm_sec, pad);
			break;
		case 'T':	/* time as %H:%M:%S */
			length += T_Strftime(string+length, maxsize-length, "%H:%M:%S", ts);
			break;
		case 'X':	/* appropriate time representation */
			/* once again - format may be locale dependent */
			length += T_Strftime(string+length, maxsize-length, "%H:%M:%S", ts);
			break;
		case 'Z':
			if(ts->tm_tZone==TZ_LOCALTIME)
				break;
			else
			{
				_BOOL fertig=TRUE;
				switch(ts->tm_tZone)
				{
					case 0:
						length = add_str(string, length, maxsize, "GMT");
						break;
					case -240:
						length = add_str(string, length, maxsize, "EDT");
						break;
					case -300:
						length = add_str(string, length, maxsize, "EST");
						break;
					case -360:
						length = add_str(string, length, maxsize, "CST");
						break;
					case -420:
						length = add_str(string, length, maxsize, "MST");
						break;
					case -480:
						length = add_str(string, length, maxsize, "PST");
						break;
					case 60:
						length = add_str(string, length, maxsize, "MEZ");
						break;
					case 120:
						length = add_str(string, length, maxsize, "MESZ");
						break;
					default:
						fertig=FALSE;
				}
				if(fertig)	/* sonst unbekannte Zeitzone im Standardformat anzeigen*/
					break;
			}
		case 'z':
			if(ts->tm_tZone==TZ_LOCALTIME)
				length = add_str(string, length, maxsize, "-0000");
			else
			{
				_WORD st,min;
				_UBYTE tz[6];
				st=ts->tm_tZone/60;
				min=ts->tm_tZone-st*60;
				min+=st*100;				/*damit haben wir jetzt eine Zahl, deren letzten 2 Ziffern die min sind*/
				sprintf(tz,"%+05d",min);
				length = add_str(string, length, maxsize, tz);
			}
			break;
		
		/* date fields: */
		
		case 'a': /* abbreviated weekday name */
			/*
			 * cases 'a' and 'b' are still wrong -
			 * in some locale short names do not have to be
			 * 3 characters wide - still works in North America.
			 */
			{
				CONST _UBYTE *s;
				CONST _UBYTE **tab;
				
				switch (arg % 100)
				{
					case 0: tab = C_day_name; break;
					case 1: tab = german_day_name; break;
					default: tab = _LC_Day_name; break;
				}
				s = (_WORD) ts->tm_wday < 0 || ts->tm_wday > 6 ? (CONST _UBYTE *)"???" : tab[ts->tm_wday];
				
				add_char(s[0]);
				add_char(s[1]);
				if (arg < 100)
					add_char(s[2]);
			}
			break;
		case 'A':	/* full weekday name */
			{
				CONST _UBYTE *s;
				CONST _UBYTE **tab;
				
				switch (arg)
				{
					case 0: tab = C_day_name; break;
					case 1: tab = german_day_name; break;
					default: tab = _LC_Day_name; break;
				}
				s = (_WORD)ts->tm_wday < 0 || ts->tm_wday > 6 ? (CONST _UBYTE *)"?" : tab[ts->tm_wday];
				
				length = add_str(string, length, maxsize, s);
			}
			break;
		case 'b':	/* abbreviated month name */
		case 'h':	/* same as 'b' */
			{
				CONST _UBYTE *s;
				CONST _UBYTE **tab;
				
				switch (arg)
				{
					case 0: tab = C_mth_name; break;
					case 1: tab = german_mth_name; break;
					default: tab = _LC_Mth_name; break;
				}
				s = (_WORD) ts->tm_mon < 0 || ts->tm_mon > 11 ? (CONST _UBYTE *)"???" : tab[ts->tm_mon];
				
				add_char(s[0]);
				add_char(s[1]);
				add_char(s[2]);
			}
			break;
		case 'B':	/* full month name */
			{
				CONST _UBYTE *s;
				CONST _UBYTE **tab;
				
				switch (arg)
				{
					case 0: tab = C_mth_name; break;
					case 1: tab = german_mth_name; break;
					default: tab = _LC_Mth_name; break;
				}
				s = (_WORD) ts->tm_mon < 0 || ts->tm_mon > 11 ? (CONST _UBYTE *)"???" : tab[ts->tm_mon];
							
				length = add_str(string, length, maxsize, s);
			}
			break;
		case 'c':	/* appropriate date and time representation */
			/* this format should be set in setlocale.c */
			length += T_Strftime(string+length, maxsize-length, "%a %b %d %H:%M:%S %Z %Y", ts);
			break;
		case 'C':	/* century */
			length = add_num2(string, length, maxsize, ts->tm_year / 100, pad);
			break;
		case 'd':	/* day of month, 01-31 */
			length = add_num2(string, length, maxsize, ts->tm_mday+1, pad);
			break;
		case 'D':	/* date as %m/%d/%y */
			switch (arg)
			{
			case 0:
			default:
				length += T_Strftime(string+length, maxsize-length, "%m/%d/%y", ts);
				break;
			case 1:
				length += T_Strftime(string+length, maxsize-length, "%d.%m.%y", ts);
				break;
			}
			break;
		case 'e':	/* day of month, blank padded */
			length = add_num2(string, length, maxsize, ts->tm_mday+1, blank);
			break;
		case 'E':	/* POSIX locale extensions, ignored for now */
			break;
		case 'j':	/* day of the year, 001-366 */
			length = add_num2(string, length, maxsize, ts->tm_yday+1, pad);
			break;
		case 'm':	/* month, 01-12 */
			length = add_num2(string, length, maxsize, ts->tm_mon+1, pad);
			break;
		case 'O':	/* POSIX locale extensions, ignored for now */
			break;
		case 'U':	/* week of year - starting Sunday */
			length = add_num2(string, length, maxsize, sun_week(ts), pad);
			break;
		case 'V':	/* date as dd-bbb-YYYY */
			length += T_Strftime(string+length, maxsize-length, "%d-%m-%Y", ts);
			if (length >= 11)
			{
				string[length-8] = ucase(string[length-8]);
				string[length-7] = ucase(string[length-7]);
				string[length-6] = ucase(string[length-6]);
			}
			break;
		case 'W':	/* week of year - starting Monday */
			length = add_num2(string, length, maxsize, mon_week(ts), pad);
			break;
		case 'w':	/* day of week, Sunday == 0 */
			length = add_num2(string, length, maxsize, ts->tm_wday, pad);
			break;
		case 'x':	/* appropriate date representation */
			/* once again - format may be locale dependent */
			length += T_Strftime(string+length, maxsize-length, "%b %d %Y", ts);
			break;
		case 'y':	/* year without a century, 00-99 */
			length = add_num2(string, length, maxsize, ts->tm_year % 100, pad);
			break;
		case 'Y':	/* year with century */
			add_char(ts->tm_year / 1000 + '0');
			length = add_num3(string, length, maxsize, ts->tm_year % 1000, zero);
			break;
		}
	}
	if (length >= maxsize)
	{
		if (maxsize == 0)
			return 0;
		--length;
	}
	string[length] = '\0';
	return length;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL size_t T_Strftime(_UBYTE *string, size_t maxsize, CONST _UBYTE *format, F_TIME *ts __c_va_alist)
{
	__c_va_list args;
	size_t length;
	
	__c_va_start(args, ts);
	length = t_strftime(string, maxsize, format, ts, args);
	__c_va_end(args);
	return length;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL size_t T_Strlftime(_UBYTE *str, size_t maxsize, CONST _UBYTE *format, SYSTIME ltime __c_va_alist)
{
	__c_va_list args;
	size_t length;
	
	__c_va_start(args, ltime);
	length = t_strftime(str, maxsize, format, T_Localtime(ltime, NULL), args);
	__c_va_end(args);
	return length;
}

