/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <wildcard.h>
#include <routines.h>



LOCAL _BOOL WildMat(CONST _UBYTE *s, CONST _UBYTE *p, _BOOL ignoreCase);

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL Star(CONST _UBYTE *source, CONST _UBYTE *pattern, _BOOL ignoreCase)
{
	while (!WildMat(source, pattern, ignoreCase))
	{
		++source;
		if (*source == '\0')
			return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL WildMat(CONST _UBYTE *source, CONST _UBYTE *pattern, _BOOL ignoreCase)
{
	_UWORD last;
	_BOOL match, reverse;
	CONST _UBYTE *pt;

	while (*pattern != '\0')
	{
		/* und am Ende source und pattern inkrementieren */

		switch (*pattern)
		{
		case '\\':
			/* Literal match with following character */
			++pattern;
			if (*pattern == '\0' || *source != *pattern)
				return FALSE;
			break;

		/* so, jetzt kommen die interessanten Sachen! */
		case '?':
			if (*source == '\0')
				return FALSE;
			break;

		case '*': /* Schon etwas komplizierter zu uebersetzen */
			++pattern;
			if (*pattern == '\0')
				return TRUE;
			return Star(source, pattern, ignoreCase);

		/* So, und jetzt wird's richtig lustig */
		case '[':
			pt = pattern;
			++pt;
			reverse = (*pt == '^');
			if (reverse)
				++pattern;

			last = 0x100;
			match = FALSE;
			++pattern;
			while (*pattern != '\0' && *pattern != ']')
			{
				if (*pattern == '-')
				{
					/* erster Fall: match == true wenn *s<==*++p && *s >=last
					 */
					++pattern;
					if (ignoreCase)
					{
						match |= (ucase(*source) <= ucase(*pattern) &&
							*source >= last);
					} else
					{
						match |= *source <= *pattern && *source >= last;
					}
				} else
				{
					/* zweiter Fall: match == true wenn *s==*p */
					if (ignoreCase)
						match |= ucase(*source) == ucase(*pattern);
					else
						match |= *source == *pattern;
				}
				last = *pattern;
				++pattern;
			}

			if (match == reverse)
				return FALSE;
			break;

		default:
			if (ignoreCase)
			{
				if (ucase(*source) != ucase(*pattern))
					return FALSE;
			} else
			{
				if (*source != *pattern)
					return FALSE;
			}
			break;
		}
		++pattern;
		++source;
	}

	return *source == '\0';
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL WildCards_NameMatching(CONST _UBYTE *source, CONST _UBYTE *pattern)
{
	return WildMat(source, pattern, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL WildCards_MatchName(CONST _UBYTE *source, CONST _UBYTE *pattern, _BOOL ignoreCase)
{
	return WildMat(source, pattern, ignoreCase);
}
