/*****************************************************************************
 *	TOS/DIR_IO.C
 *****************************************************************************/


#include "_file_io.h"
#include <ro_mem.h>
#include <debug.h>
#include <routines.h>
#include <routine.h>

#define FA_RDONLY       0x01
#define FA_HIDDEN       0x02
#define FA_SYSTEM       0x04
#define FA_LABEL        0x08
#ifndef FA_DIREC
#define FA_DIREC        0x10
#endif
#ifndef FA_ARCH
#define FA_ARCH         0x20
#endif


#define	ATTRIBUTES		(FA_DIREC | FA_HIDDEN | FA_SYSTEM)


#ifndef	NULL
# define	NULL	0
#endif	/* NULL */

#define NULL_DIR ((Directory *)NULL)
#define NULL_ENT ((DirectoryEntry *)NULL)


LOCAL _WORD dir_fd;
LOCAL _WORD max_fd;
LOCAL Directory *dir_list;

GLOBAL _BOOL _readdir_tolower = 0;


/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *D_Getcwd(_VOID)
{
	_UBYTE pathbuf[PATH_MAX+1];
	_UBYTE *path;
	size_t bufsize;

	if (Dgetpath(pathbuf, 0) != 0)
	{
		return NULL;
	}
	bufsize = PATH_MAX + 1;
	if ((path = (_UBYTE *)SYSMALLOC(bufsize, MEM_NOERR, "D_Getcwd")) == 0)
	{
		return NULL;
	}
	strCpy(path, pathbuf);
	return path;
}

/*** ---------------------------------------------------------------------- ***/

#undef S_IFMT
#undef S_IFREG
#undef S_IFCHR
#undef S_IFDIR
#undef S_IFIFO
#undef S_IMEM
#undef S_IFLNK

#define	S_IFMT		0xF000	/* type of file */
#define S_IAMB		0x1FF	/* access mode bits */
#define	S_IFREG		0x8000	/* regular */
#define	S_IFCHR		0x2000	/* character special */
#define	S_IFBLK		0x3000	/* block special */
#define	S_IFDIR		0x4000	/* directory */
#define	S_IFIFO		0xA000	/* fifo */
#define	S_IFSOCK	0x7000	/* socket */
#define S_IMEM      0xC000  /* memory region or process */
#define	S_IFLNK		0xE000	/* symbolic link */


LOCAL _VOID copy_attr(F_STAT *st, XATTR *xattr, _LONG modeattr)
{
	_UWORD filetype;
	
	filetype = (xattr->mode & S_IFMT);
	/*
	 * mindestens Gemulator setzt den Filetyp nicht,
	 * also nehmen wir den nur wenn er auch gueltig ist
	 */
	if ((modeattr & DP_FT_DIR) && filetype != 0)
		st->st_is_directory = filetype == S_IFDIR;
	else if (modeattr & DP_ATTRBITS)
		st->st_is_directory = (xattr->attr & FA_DIREC) != 0;
	else
		st->st_is_directory = FALSE;
	if ((modeattr & DP_FT_REG) && filetype != 0)
		st->st_is_regular = filetype == S_IFREG;
	else if (modeattr & DP_ATTRBITS)
		st->st_is_regular = (xattr->attr & FA_DIREC) == 0;
	else
		st->st_is_regular = FALSE;
	if (modeattr & DP_FT_CHR)
		st->st_is_char_special = filetype == S_IFCHR;
	else
		st->st_is_char_special = FALSE;
	if (modeattr & DP_FT_BLK)
		st->st_is_block_special = filetype == S_IFBLK;
	else
		st->st_is_block_special = FALSE;
	if (modeattr & DP_FT_LNK)
		st->st_is_lnk = filetype == S_IFLNK;
	else
		st->st_is_lnk = FALSE;
	if (modeattr & DP_FT_FIFO)
		st->st_is_fifo = filetype == S_IFIFO;
	else
		st->st_is_fifo = FALSE;
	if (modeattr & DP_FT_SOCK)
		st->st_is_socket = filetype == S_IFSOCK;
	else
		st->st_is_socket = FALSE;
	if (modeattr & DP_FT_MEM)
		st->st_is_mem = filetype == S_IMEM;
	else
		st->st_is_mem = FALSE;
	st->st_mode = xattr->mode & S_IAMB;
	st->st_size = xattr->size;
	st->st_mtime = T_DosToSystime(xattr->mdate, xattr->mtime);
	st->st_atime = T_DosToSystime(xattr->adate, xattr->atime);
	st->st_ctime = T_DosToSystime(xattr->cdate, xattr->ctime);
	st->st_nlink = xattr->nlink;
	st->st_uid = xattr->uid;
	st->st_gid = xattr->gid;
	st->st_dev = xattr->dev;
	st->st_ino = xattr->index;
}

/*** ---------------------------------------------------------------------- ***/


LOCAL _VOID copy_dattr(F_STAT *st, _DTA *dta)
{
	if (dta->dta_attribute & FA_DIREC)
	{
		st->st_is_directory = TRUE;
		st->st_is_regular = FALSE;
		st->st_mode = 0x1ff;
		st->st_nlink = 2;
	} else
	{
		st->st_is_directory = FALSE;
		st->st_is_regular = TRUE;
		st->st_mode = 0x16d;
		if (!(dta->dta_attribute & FA_RDONLY))
			st->st_mode |= (S_IWUSR|S_IWGRP|S_IWOTH);
		st->st_nlink = 1;
	}
	st->st_is_char_special = FALSE;
	st->st_is_block_special = FALSE;
	st->st_is_lnk = FALSE;
	st->st_is_fifo = FALSE;
	st->st_is_socket = FALSE;
	st->st_is_mem = FALSE;
	st->st_size = dta->dta_size;
	st->st_mtime = T_DosToSystime(dta->dta_date, dta->dta_time);
	st->st_atime = st->st_mtime;
	st->st_ctime = st->st_mtime;
	if (dta->dta_attribute & FA_ARCHIVE)
		--(st->st_ctime);
	st->st_uid = 0;
	st->st_gid = 0;
	st->st_dev = 2;
}

/*** ---------------------------------------------------------------------- ***/
/*DYJ 5.20w */
GLOBAL Directory *D_Opendir(CONST _UBYTE * name)
{
	Directory	*dirp;
	_UBYTE		c;
	_UBYTE		nbuf[PATH_MAX + 5];
	size_t		len;
	_DTA		*olddta;
	_DTA		dtabuf;
	DirectoryEntry		*head;
	DirectoryEntry		*dp;
	DirectoryEntry		*last;
	_LONG		result;
	_LONG		ino;
	_UBYTE		*namep;
	_UBYTE		*p;
	Directory	*search;

	/* This bit of nastyness is due to the fact that Fattrib()
	 * does not do as it is ment to - i.e. it always fails when
	 * used on directories.  Piss me off!
	 */
	if (strChr(name, '*') || strChr(name, '?'))
		return NULL_DIR;

	(_VOID) strnCpy(nbuf, name, PATH_MAX);
	nbuf[PATH_MAX] = '\0';
	len = strLen(nbuf);
	while (len > 1 && ((c = nbuf[len - 1]) == '\\' || c == '/'))
		nbuf[--len] = '\0';
	if (len == 0)
	{
		nbuf[len++] = '.';
	} else if (len == 2 && nbuf[1] == ':')
	{
		nbuf[len++] = '\\';
	}
	nbuf[len] = '\0';
	dirp = (Directory *)SYSMALLOC(sizeof(Directory), MEM_NOERR, "D_Opendir: dir");
	if (dirp == NULL_DIR)
	{
		return dirp;
	}
	if ((dirp->dd_wd = D_Getcwd()) == NULL)
	{
		SYSFREE((_UBYTE *)dirp, -1);
		return NULL_DIR;
	}
	namep = dirp->dd_wd + strLen(dirp->dd_wd);
	while (namep[-1] == '\\')
		*--namep = '\0';
	if (namep == dirp->dd_wd)
		strCpy(namep, "\\");

	namep = nbuf;
	if (*namep == '\\' || (is_alpha(*namep) && namep[1] == ':'))
	{
		strnCpy(dirp->dd_wd, namep, PATH_MAX);
		dirp->dd_wd[PATH_MAX] = '\0';
	} else
	{
		while (*namep)
		{
			if (namep[0] == '.' && namep[1] == '.' &&
				(namep[2] == '\0' || namep[2] == '\\'))
			{
				p = strrChr(dirp->dd_wd, '\\');
				if (p != NULL)
					*p = '\0';
				namep += 2;
				while (*namep == '\\')
					++namep;
			} else
			{
				if (namep[0] == '.' && (namep[1] == '\0' || namep[1] == '\\'))
				{
					++namep;
					while (*namep == '\\')
						++namep;
				}
			}
			if (*namep)
			{
				p = dirp->dd_wd + strLen(dirp->dd_wd);
				if (p < (dirp->dd_wd + PATH_MAX))
					*p++ = '\\';
				while (*namep && *namep != '\\')
				{
					if (p < (dirp->dd_wd + PATH_MAX))
						*p++ = *namep;
					namep++;
				}
				while (*namep == '\\')
					++namep;
				*p = '\0';
			}
		}
	}
	
	dirp->dd_loc = 0;
	dirp->dd_link = NULL;
	
	for (search = dir_list; search != NULL; search = search->dd_next)
	{
		if (F_Path_Equal(search->dd_wd, dirp->dd_wd))
		{
			while (search->dd_link != NULL)
				search = search->dd_link;
			search->dd_link = dirp;
			dirp->dd_buf = search->dd_buf;
			dirp->dd_cur = (DirectoryEntry *)search->dd_buf;
			++dir_fd;
			if (dir_fd > max_fd)
				max_fd = dir_fd;
			dirp->dd_fd = dir_fd;
			return dirp;
		}
	}

	dirp->dd_next = dir_list;
	dir_list = dirp;
	
	dirp->dd_fd = Dopendir(nbuf, 0);
	if ((dirp->dd_fd & 0xff000000l) != 0xff000000l)
	{
		_BOOL do_lower = _readdir_tolower && Dpathconf(nbuf, DP_CASE) != DP_CASESENS;
		_BOOL do_upper = !_readdir_tolower && Dpathconf(nbuf, DP_CASE) == DP_CASECONV && Pdomain(-1) == 1;
		_LONG modeattr;
		XATTR xattr;
		_LONG xret;
		
		if (Dpathconf(nbuf, -1) >= DP_MODEATTR)
			modeattr = Dpathconf(nbuf, DP_MODEATTR);
		else
			modeattr = DP_ATTRBITS | DP_MODEBITS | DP_FILETYPS;
		head = NULL_ENT;
		last = NULL_ENT;
		if (Dxreaddir(PATH_MAX, dirp->dd_fd, nbuf, &xattr, &xret) == 0 && xret == 0)
		{
			do {
				len = strLen(nbuf+4);
				if ((dp = (DirectoryEntry *)SYSMALLOC(sizeof(DirectoryEntry)+len, MEM_NOERR, "D_Opendir: entry")) == NULL_ENT)
				{
					break;
				}
				MemSetZeroStruct(*dp);
				(_VOID) strCpy(dp->d_name, nbuf+4);
				if (do_lower)
					strLwr(dp->d_name);
				if (do_upper)
					strUpr(dp->d_name);
				dp->d_namlen = (_WORD)len;
				copy_attr(&dp->d_st, &xattr, modeattr);
				dp->d_st.st_ino = *(long *)nbuf;
				dp->d_next = NULL_ENT;
				if (head == NULL_ENT)
				{
					head = dp;
				} else
				{
					last->d_next = dp;
				}
				last = dp;
			} while (Dxreaddir(PATH_MAX, dirp->dd_fd, nbuf, &xattr, &xret) == 0);
		} else if (Dreaddir(PATH_MAX, dirp->dd_fd, nbuf) == 0)
		{
			_LONG ino;
			
			do {
				len = strLen(nbuf+4);
				if ((dp = (DirectoryEntry *)SYSMALLOC(sizeof(DirectoryEntry)+len, MEM_NOERR, "D_Opendir: entry")) == NULL_ENT)
				{
					break;
				}
				MemSetZeroStruct(*dp);
				(_VOID) strCpy(dp->d_name, nbuf+4);
				if (do_lower)
					strLwr(dp->d_name);
				if (do_upper)
					strUpr(dp->d_name);
				dp->d_namlen = (_WORD)len;
				dp->d_next = NULL_ENT;
				ino = *(long *)nbuf;
				strCat(strCat(strCpy(nbuf, dirp->dd_wd), "\\"), dp->d_name);
				if (Fxattr(1, nbuf, &xattr) == 0)
				{
					copy_attr(&dp->d_st, &xattr, modeattr);
					dp->d_st.st_ino = ino;
				}
				if (head == NULL_ENT)
				{
					head = dp;
				} else
				{
					last->d_next = dp;
				}
				last = dp;
			} while (Dreaddir(PATH_MAX, dirp->dd_fd, nbuf) == 0);
		}
		Dclosedir(dirp->dd_fd);
		dirp->dd_buf = (_UBYTE *)head;
		dirp->dd_cur = head;
	} else if (dirp->dd_fd != -32)
	{
		dir_list = dirp->dd_next;
		SYSFREE((_UBYTE *)dirp, -1);
		dirp = NULL_DIR;
	} else
	{
		if (len > 0 && (nbuf[len-1] != '\\' && nbuf[len-1] != '/'))
			strCat(nbuf, "\\");
		strCat(nbuf, ALL_FILE_MASK);
		olddta = Fgetdta();
		Fsetdta(&dtabuf);
		head = NULL_ENT;
		last = NULL_ENT;
		if ((result = Fsfirst(nbuf, ATTRIBUTES)) == 0)
		{
			ino = ((dtabuf.dta_buf[18] << 8) & 0xff00) + (dtabuf.dta_buf[19] & 0xff);
			ino <<= 16;
			do {
				len = strLen(dtabuf.dta_name);
				if ((dp = (DirectoryEntry *)SYSMALLOC(sizeof(DirectoryEntry)+len, MEM_NOERR, "D_Opendir: entry")) == NULL_ENT)
				{
					break;
				}
				MemSetZeroStruct(*dp);
				dp->d_st.st_ino = ino++;
				copy_dattr(&dp->d_st, &dtabuf);
				(_VOID) strCpy(dp->d_name, dtabuf.dta_name);
				if (_readdir_tolower)
					strLwr(dp->d_name);
				dp->d_namlen = (_WORD)len;
				dp->d_next = NULL_ENT;
				if (head == NULL_ENT)
				{
					head = dp;
				} else
				{
					last->d_next = dp;
				}
				last = dp;
			} while (Fsnext() == 0);
		} else
		{
			if (result == -33 || result == -34 || result == -46)
			{
				dir_list = dirp->dd_next;
				SYSFREE((_UBYTE *)dirp, -1);
				dirp = NULL_DIR;
			}
		}
		Fsetdta(olddta);
		if (dirp != NULL_DIR)
		{
			dirp->dd_buf = (_UBYTE *)head;
			dirp->dd_cur = head;
			++dir_fd;
			if (dir_fd > max_fd)
				max_fd = dir_fd;
			dirp->dd_fd = dir_fd;
		}
	}
	return dirp;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID D_Closedir(Directory *dirp)
{
	DirectoryEntry *dp, *odp;
	Directory *search, *link, *last;

	if (dirp == NULL_DIR)
	{
		return;
	}
	last = NULL_DIR;
	for (search = dir_list; search != NULL_DIR; search = search->dd_next)
	{
		if (search == dirp)
		{
			if (last == NULL)
			{
				if (dirp->dd_link == NULL)
				{
					dir_list = dirp->dd_next;
				} else
				{
					dirp->dd_link->dd_next = dirp->dd_next;
					dir_list = dirp->dd_link;
				}
			} else
			{
				if (dirp->dd_link == NULL)
				{
					last->dd_next = dirp->dd_next;
				} else
				{
					dirp->dd_link->dd_next = dirp->dd_next;
					last->dd_next = dirp->dd_link;
				}
			}
			dp = (DirectoryEntry *)(dirp->dd_buf);
			while (dp)
			{
				odp = dp;
				dp = dp->d_next;
				SYSFREE((_UBYTE *)odp, -1);
			}
			if (dirp->dd_fd == max_fd)
			{
				--max_fd;
				--dir_fd;
			}
			SYSFREE(dirp->dd_wd, -1);
			SYSFREE(dirp, -1);
			return;
		}
		if (search->dd_link != NULL)
		{
			link = search;
			while (link != NULL)
			{
				if (link->dd_link == dirp)
				{
					link->dd_link = link->dd_link->dd_link;
					return;
				}
				link = link->dd_link;
			}
		}
		last = search;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG D_Telldir(Directory *dirp)
{
	return dirp->dd_loc;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL DirectoryEntry *D_Readdir(Directory *dirp)
{
	DirectoryEntry *dp;

	if ((dp = dirp->dd_cur) == NULL_ENT)
		return NULL_ENT;
	dirp->dd_cur = dp->d_next;
	++dirp->dd_loc;
	return dp;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID D_Seekdir(Directory *dirp, _LONG off)
{
	DirectoryEntry *dp;
	_LONG i;
	
	if (off < 0)
		return;
	dp = (DirectoryEntry *)(dirp->dd_buf);
	i = off;
	while (--i >= 0 && dp)
		dp = dp->d_next;
	dirp->dd_loc = off - (i + 1);
	dirp->dd_cur = dp;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL F_ERROR F_Getinfo(CONST _UBYTE *name, F_STAT *st, _BOOL followLinks)
{
	XATTR xattr;
	_DTA *olddta;
	_DTA dta;
	_LONG res;
	_LONG modeattr;
	
	res = Fxattr(followLinks ? 0 : 1, name, &xattr);
	SPEC_DEBUG_OUT((EO_TRACE, "Fxattr(%s): res=%ld, mode=$%04x, size=%ld, attr=$%04x",
		name, res,
		xattr.mode,
		xattr.size,
		xattr.attr));
	if (res == 0)
	{
		if (Dpathconf(name, -1) >= DP_MODEATTR)
			modeattr = Dpathconf(name, DP_MODEATTR);
		else
			modeattr = DP_ATTRBITS | DP_MODEBITS | DP_FILETYPS;
		/* SPEC_DEBUG_OUT((EO_TRACE, "modeattr: $%08lx", modeattr)); */
		copy_attr(st, &xattr, modeattr);
	} else
	{
		if (res == -32 /* EINVAL */)
		{
			olddta = Fgetdta();
			Fsetdta(&dta);
			res = Fsfirst(name, FA_DIREC | FA_HIDDEN | FA_SYSTEM);
			SPEC_DEBUG_OUT((EO_TRACE, "Fsfirst(%s): res=%ld, size=%ld, attr=$%02x", name, res, dta.dta_size, dta.dta_attribute));
			if (res == 0)
				copy_dattr(st, &dta);
			Fsetdta(olddta);
		}
	}
	return sys_errno((_WORD) res);
}
