/*****************************************************************************
 *	TOS/FIO_FREE.C
 *****************************************************************************/

#include "_file_io.h"
#include <routinec.h>
#include <routines.h>

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE get_real_drive(CONST _UBYTE *path)
{
	_UBYTE link[PATH_MAX];
	_UBYTE pathbuf[PATH_MAX];
	
	strBcpy(pathbuf, path);
	while (strLen(pathbuf) > 2)
	{
		if (Freadlink((int)sizeof(link), link, pathbuf) >= 0)
		{
			strCpy(pathbuf, link);
		} else
		{
			F_Path_Dirname(pathbuf);
		}
	}
	return ucase(pathbuf[0]);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL D_Info(CONST _UBYTE *path, DiskINFO *d)
{
	_UBYTE root[8];
	_DISKINFO buf;
	_DTA *olddta;
	_DTA dta;
	_ULONG cluster_size;
	_UBYTE drive = get_real_drive(path);
	
	strCpy(root, "A:\\");
	root[0] = drive;
	d->label[0] = '\0';
	if (Dreadlabel(root, d->label, (_WORD)sizeof(d->label)) != 0)
	{
		olddta = Fgetdta();
		Fsetdta(&dta);
		strCat(root, ALL_FILE_MASK);
		if (Fsfirst(root, FA_LABEL) == 0)
			strCpy(d->label, dta.dta_name);
		Fsetdta(olddta);
	}
	if (Dfree(&buf, drive - 'A' + 1) != 0)
	{
		return FALSE;
	}
	cluster_size = (_ULONG)buf.b_secsiz * (_ULONG)buf.b_clsiz;
	d->block_size = cluster_size;
	d->free_size = (_DOUBLE)cluster_size * (_DOUBLE)buf.b_free;
	d->avail_size = d->free_size;
	d->total_size = cluster_size * buf.b_total;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL D_SameDevice(CONST _UBYTE *path1, CONST _UBYTE *path2)
{
	return get_real_drive(path1) == get_real_drive(path2);
}
