#include "_file_io.h"
#include <routines.h>

#define _REALO_APPEND	0x008		/* this is what MiNT uses */
#define _REALO_NDELAY	0x100
#define _REALO_CREAT	0x200
#define _REALO_TRUNC	0x400
#define _REALO_EXCL		0x800

#define FTRUNCATE	(('F'<< 8) | 4)

/*** ---------------------------------------------------------------------- ***/

LOCAL F_ERROR doFopen(CONST _UBYTE *filename, _WORD mode, DOSFNO *rv)
{
	_LONG fcbuf;				/* a temporary buffer for Fcntl */
	_WORD umode;
	
	switch (mode & FO_ACCMODE)
	{
		default:
		case FO_RDONLY: umode = 0; break;
		case FO_WRONLY: umode = 1; break;
		case FO_RDWR: umode = 2; break;
	}
	if (mode & FO_APPEND)
		umode |= _REALO_APPEND;
	if (mode & FO_CREAT)
		umode |= _REALO_CREAT;
	if (mode & FO_TRUNC)
		umode |= _REALO_TRUNC;
	if (mode & FO_SYNC)
	{
		; /* not implemented yet in MiNT */
	}
	if (mode & FO_EXCL)
		umode |= _REALO_EXCL;
	if (mode & FO_NDELAY)
		umode |= _REALO_NDELAY;
	if ((mode & FO_SHMODE) == FO_DENYNONE)
		umode |= 0x0040; /* O_DENYNONE */
	
	if (strCaseCmp(filename, "PRN:") == 0 ||
		strCaseCmp(filename, "AUX:") == 0 ||
		strCaseCmp(filename, "CON:") == 0 ||
		Fattrib(filename, 0, 0) >= 0)		/* file exists */
	{
		if ((mode & (FO_CREAT | FO_EXCL)) == (FO_CREAT | FO_EXCL))
		{
			return GERR_EXIST;
		}
		*rv = Fopen(filename, umode);
		if (*rv == -32)
		{
			umode &= FO_ACCMODE;
			*rv = Fopen(filename, umode);
		}
		if ((mode & FO_TRUNC) && (*rv >= 0))
		{
			/* Give up if the mode flags conflict */
			if ((mode & FO_ACCMODE) == FO_RDONLY)
			{
				(void)Fclose((_WORD)(*rv));
				return GERR_ACCESS;
			}
			/*
			 * Try the FTRUNCATE first. If it fails, have GEMDOS
			 * truncate it, then reopen with the correct modes.
			 */
			fcbuf = 0L;
			if (Fcntl((_WORD)(*rv), (long) &fcbuf, FTRUNCATE) < 0)
			{
				(void)Fclose((_WORD)(*rv));
				*rv = Fcreate(filename, 0x00);
				if (*rv < 0)
				{
					return sys_errno((_WORD)(*rv));
				}
				(void)Fclose((_WORD)(*rv));
				*rv = Fopen(filename, umode);
			}
		}
	} else					/* file doesn't exist */
	{
		if (mode & FO_CREAT)
		{
			*rv = Fcreate(filename, 0x00);
			if (*rv >= 0)
			{
				(void)Fclose((_WORD)(*rv));
				*rv = Fopen(filename, umode);
				if (*rv == -32)
				{
					umode &= FO_ACCMODE;
					*rv = Fopen(filename, umode);
				}
			}
		} else
		{
			return GERR_NOENT;
		}
	}

	if (LEGAL_DOSFNO(*rv))
	{
		if ((mode & FO_APPEND))
			(void)Fseek(0L, (_WORD)(*rv), SEEK_END);
		return GERR_OK;
	}
	return sys_errno((_WORD)(*rv));
}

/*** ---------------------------------------------------------------------- ***/

OS_GLOBAL F_ERROR FiBufOpen(CONST _UBYTE *datei, _WORD mode, _LONG offset, DOSFNO *handle)
{
	_WORD ii;
	F_ERROR res;
	
	SavePD();	

	for (;;)
	{
		ii = 0;
		while ((res = doFopen(datei, mode, handle)) == GERR_ACCESS)
		{
			if (!NetzwerkDa)
				break;
			ii++;
			if (ii > 5)
			{
				ii = 0;
	
				break;
			}
			NETWAIT();
		}
		if (res != GERR_MFILE) /* file table overflow */
			break;
		for (ii = 0; ii < fi_buf_size; ii++)
		{
			if (FiBuf[ii].used && FiBuf[ii].open)
			{
				sysFclose(&FiBuf[ii]);
				break;
			}
		}
		if (ii >= fi_buf_size)
			break;
	}
	
#ifdef DEBUG
	if (res == GERR_OK)
		OpenFiles++;
#endif

	if (res == GERR_OK)
	{
		if (offset > 0)
			Fseek(offset, (_WORD)(*handle), 0);
	}
	RestorePD();

	return res;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL FILENO F_File_Open(CONST _UBYTE *datei, _WORD mode)
{
	FILENO retV;
	F_ERROR err;
	DOSFNO handle;
	
	if ((err = FiBufOpen(datei, mode, 0, &handle)) == GERR_OK)
		retV = FiBufSave(handle, datei, mode);
	else
		retV = TO_FILENO(err);
#ifdef DEBUG_TXT
	ErrorOut(EO_TRACE, "F_File_Open %s: %ld", datei, (_LONG)TO_HANDLE(retV));
#endif
	return retV;
}
