#include "_file_io.h"
#include <portaes.h>
#ifdef __GNUC__
#include <unistd.h>
#endif
#include <signal_.h>

/*** ---------------------------------------------------------------------- ***/

/* _clock() has a rez of CLOCKS_PER_SEC ticks/sec */

#define MSEC_PER_TICK (1000L / ((unsigned long)CLK_TCK))
#define	MSEC_TO_CLOCK_TICKS(us)	((us) / MSEC_PER_TICK )


static clock_t VOLATILE now;

/* this must execute in supervisor mode; it fetches the system variable
 * containing the number of 200HZ ticks since the system was booted
 */


static long getnow(_VOID)
{
	now = *((unsigned long *) 0x4baL);
	return 0;
}
static clock_t _get_clock(_VOID)
{
	
	now = Ssystem(S_GETLVAL, 0x4ba, 0);
	if ((_LONG)now != -32)
		return now;
	(void)Supexec(getnow);
	return (now);
}


static unsigned long _alarm(unsigned long msecs)
{
	unsigned long r;

	r = Tmalarm(msecs);
	if ((long) r == -32)
		r = Talarm(msecs * 1000l);
	return r;
}


static void alarm_catch(_VOID)
{
}


GLOBAL _VOID EvntWait(_ULONG msecs)
{
	clock_t start = _get_clock();
	clock_t aktClock;

	do
	{
		(void)Syield();
		aktClock = _get_clock();
		if (aktClock < start)
			start = aktClock;
	} while ((aktClock - start) * MSEC_PER_TICK < msecs);
}


static _ULONG _usleep(_ULONG n)
{
	_ULONG stop;
	long old_alarm_func;
	long old_sigmask;
	_ULONG alarm_msec;
	long remain;

	/* Clear any existing alarm, but save its expire time.
	   Then block all signals and save original mask.
	   Install our alarm handler, and remask (since Psignal()
	   just unmasked SIGALRM.)
	*/
	n = (n + 999l) / 1000l;
	alarm_msec = Tmalarm(0l);
	if ((long) alarm_msec == -32)
	{
		alarm_msec = (long) Talarm(0L);
		if ((long) alarm_msec != -32)
			alarm_msec *= 1000;
	}
	
	if ((long) alarm_msec == -32)
	{
		while (n > 32000l)
		{
			alarm_msec = Fselect(32000, 0L, 0L, 0L);
			if ((long) alarm_msec == -32)
				break;
			n -= 32000l;
		}
		if (n > 0 && (long) alarm_msec != -32)
		{
			alarm_msec = Fselect((_UWORD) n, 0L, 0L, 0L);
			n = 0;
		}
		if (n != 0)
		{
			EvntWait(n);
		}
		return 0;
	}
	old_sigmask = Psigblock(~0L);
	old_alarm_func = Psignal(SIGALRM, (long) alarm_catch);
	(void) Psigblock(~0L);
	/* Figure out when we would normally wake up */
	stop = _get_clock() + n / MSEC_PER_TICK;
	/* But only sleep as long as the original alarm clock */
	if (alarm_msec && (alarm_msec < n))
		(void) _alarm(alarm_msec);
	else
		(void) _alarm((unsigned long) n);
	/* Go to sleep with the original mask but always
	   allow SIGALRM or we might never wake up.
	   */
	Psigpause(old_sigmask & ~(sigmask(SIGALRM)));
	/* Cancel any time left on the alarm clock */
	(void) _alarm(0L);
	/* Eat any alarms that occurred before we cancelled.
	   The Syield ensure we enter the kernel and handle the
	   signal.
	   */
	(void) Psignal(SIGALRM, (long) alarm_catch);
	(void) Syield();
	/* Remask again after the Psignal() */
	(void) Psigblock(~0L);
	remain = ((long) stop - (long) _get_clock()) * MSEC_PER_TICK;
	if (alarm_msec)
	{
		/* Restore the original timer (or if it expired,
		   post the signal now)
		   */
		alarm_msec -= ((long) n - remain);
		if (alarm_msec <= 0)
			(void) Pkill(Pgetpid(), SIGALRM);
		else
			(void) _alarm(alarm_msec);
	}
	/* Restore the original alarm handler (which also
	   allows it to be delivered!)  Then restore the
	   original signal mask (if the signal handler
	   does a longjmp() we might lose).
	   */
	(void) Psignal(SIGALRM, (long) old_alarm_func);
	(void) Psigsetmask(old_sigmask);
	(void) Syield();
	return ((remain > 0) ? (unsigned) remain : 0);
}


void usleep(unsigned long usec)
{
	_usleep(usec);
}


unsigned int sleep(unsigned int n)
{
	return (unsigned int)(_usleep(n * 1000000l) / 1000000l);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID EventTimer(_ULONG millisecs)
{
#if GUI_VERSION
	evnt_timer((_UWORD) (millisecs & 0xffffl), (_UWORD)(millisecs >> 16));
#else
	usleep(millisecs * (_ULONG)1000l);
#endif
}
