/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */

/*
	GEMScript-Modul fuer CAT
	
	Achtung: Aenderungen an GSCRIPT.C und GSCRIPT.H, sowie der GEMScript
	Funktionalitaet von CAT gehen bitte zuerst an Volker Janzen @ UL.
	Von ihm werden Aenderungen an Thorsten Otto weitergeleitet.
	
	Written by Volker Janzen
	
	History:
	
	24.01.98: -Erstes GEMScript-Modul das in CAT eingehaengt ist
	31.01.98: -Portierung in Sourceversion 4.02 (vj)
			  -Sourcecode etwas ueberarbeitet
			  -Bug gefunden: SendGSQuit ist fuer den Absturz
			   verantwortlich
	08.02.98: -Portierung in Sourceversion 4.03 (vj)
			  -Ueberfluessigen Parameter bei GScript_IsProtokoll()
			   entfernt
			  -Korrekte Klammerung wind_update-Klammerung fuer
			   SendGSQuit()
	21.02.98: -Kleinere Korrekturen
	24.02.98: -Uebername in Sourceversion 4.05 (vj)
	          -Neue KonfigVariable "CATLongName", die den Rueckgabewert
	           von AppGetLongName bestimmt
	          -Debuginformationen jetzt per Praeprozessor de-/aktivierbar
	          -GEMScript_Command fuegt ein GEMScript-Kommando mit Handlerfunktion in
	           eine verkettete Liste ein
	          -Aus dem uebergebenen String wird ein argc/argv-Konstruct gebildet
	          -HandleGSCommand sucht nun auch in der verketteten Liste nach einem
	           Kommando und ruft die passende Handlerfunktion
	25.02.98: -"CATLongName" wird jetzt mit Default benutzt. Das Default wird in
	           gscript.h ueber "GS_APPLONGNAME" festgelegt
	          -Das Kommando "Quit" wird jetzt ueber ein intern gesendetes AP_TERM
	           realisiert, so brauche ich mich nicht um den genauen Mechanismus
	           zu kuemmern
	10.03.98: -"CATLongName" heisst jetzt "APPLongName" wegen der
	           Portabilitaet dieses Moduls mit Fred
	          -Bugfix: Beim GS-Kommando Quit() wird nun nicht mehr
	           unbekannter Befehl zurueckgegeben
	15.03.98: -Uebernahme in vorlaeufigen Source 4.07
			  -CONST bei GEMScript_Command rausgenommen
			  -Fix: GEMScript_Command wandelt nun ein Kommando komplett
			   in Kleinschrift um, damit es beim Stringvergleich keine
			   Probleme gibt
	08.04.98: -Uebernahme in Source v4.08 (vj)
			  -Bugfix: es wird nun bei allen Speicheranforderungen auf
			   Erfolg geprueft
			  -Fix: GEMScript_Command ist GLOBAL
	12.04.98: -Bei einem leeren oder reservierten Parameter ('\1' bis '\6') wird in
			   gs_argv[] ein Zeiger auf einen Leerstring uebergeben (Achtung: bei einer
			   Aenderung dieses Verhaltens (ausser bei '\1') muss GS_VERSION angepasst
			   werden (z.B. bei Unterstuetzung von '\2')
	
*/

#include <portab.h>
#include <portaes.h>
#include <protokol.h>
#include <portaes.h>
#include <ro_mem.h>
#include <confvars.h>
#include <gscript.h>
#include <debug.h>
#include <routines.h>

/*
	Folgende Zeile auskommentieren, wenn das GEMScript-Modul ohne
	Debuginformationen erzeugt werden soll
*/
#define GS_DEBUG 1

#define GS_VERSION	0x0120						/* Eigene unterstuetzte GEMScript-Version */
#define GS_MSGS  (GSM_COMMAND | GSM_HEXCODING)	/* Eigene verstandene Messages */

/*
	Wieviele Parameter kann es maximal geben?
*/
#define GS_MAX_PAR 10

/* GEMScript-Nachrichten: */

#define GS_REQUEST		0x1350
#define GS_REPLY		0x1351
#define GS_COMMAND		0x1352
#define GS_ACK			0x1353
#define GS_QUIT			0x1354
#define GS_OPENMACRO	0x1355
#define GS_MACRO		0x1356
#define GS_WRITE		0x1357
#define GS_CLOSEMACRO	0x1358

/* Masken fuer unterstuetzte Nachrichten (msgs): */

#define GSM_COMMAND		0x0001  /* kann GS_COMMAND empfangen */
#define GSM_MACRO		0x0002  /* (Interpreter) kann GS_OPENMACRO, GS_WRITE und GS_CLOSEMACRO empfangen, GS_MACRO verschicken */
#define GSM_WRITE		0x0004  /* (aufnahmefaehige App) kann GS_OPENMACRO, GS_WRITE und GS_CLOSEMACRO verschicken, GS_MACRO empfangen */
#define GSM_HEXCODING   0x0008  /* Hex-Codierung wird verstanden */

/* Returnwerte bei GS_ACK (msg[7]): */

#define GSACK_OK		0
#define GSACK_UNKNOWN	1
#define GSACK_ERROR		2

typedef struct GSC_LISTE {
	CONST _UBYTE *name;
	GSC_Command_Handler func;
	struct GSC_LISTE *next;
} GSC_LISTE;

typedef struct GS_PARTNER {
	_WORD id;  /* AES-ID des Kommunikationspartners */
	_WORD gid; /* ID aus GS_REQUEST */
	struct GS_PARTNER *next;
} GS_PARTNER;

typedef struct {
	_LONG len;       /* Laenge der Struktur in Bytes                     */
	_WORD version;   /* Versionsnummer des Protokolles beim Sender
                        (z.Z. 0x0120 = 1.20)                             */
	_WORD msgs;      /* Bitmap der unterstuetzten Nachrichten (GSM_xxx)
	                    und Faehigkeiten                                 */
	_LONG ext;       /* benutzte Endung, etwa '.SIC'                     */
} GS_INFO;

/* GEMScriptstrukturen schon initialisiert? */
LOCAL _BOOL gs_initialised;
/* GEMScriptkommunikation moeglich? */
LOCAL _BOOL gs_possible;
/* Zaehler fuer Enable/DisableGEMScript */
LOCAL _WORD gsp_counter;
/* Hat gsp_counter den folgenden Wert ist GEMScript durch einen internen Fehler gesperrt */
#define GS_OFFLINE 0x0FFF
/* Zeiger auf globalen Speicher fuer GS_INFO */
LOCAL _VOID *gsi_mem;
/* Anker fuer die verkettete Liste der Kommunikationspartner */
LOCAL GS_PARTNER *gp_liste;
/* Anker fuer die GEMScript-Kommandoliste */
LOCAL GSC_LISTE *gs_commands;
/* Default-Name fuer AppLongName */
LOCAL CONST _UBYTE *gsAppName;

#if GS_DEBUG
#  define TRACE(x) SPEC_DEBUG_OUT(x)
#else
#  define TRACE(x) 
#endif

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*
	Anfragefunktion: wurde GEMScript schon initialisiert?
*/
GLOBAL _BOOL IsGEMScriptInitialised(_VOID)
{
	return gs_initialised;
}

/*** ---------------------------------------------------------------------- ***/

/*
	Anfragefunktion: ist GEMScript grade moeglich?
*/
GLOBAL _BOOL IsGEMScriptPossible(_VOID)
{
	return gs_possible;
}

/*** ---------------------------------------------------------------------- ***/

/*
	GEMScript sperren (es wird mitgezaehlt: erst entsperren wenn alle meinen es geht)
*/
GLOBAL _BOOL DisableGEMScript(_VOID)
{
	/* Zaehler nur weiter hochsetzen, wenn GEMScript nicht offline ist */
	if (gsp_counter != GS_OFFLINE)
	{
		gsp_counter++;
	}
	
	if (gsp_counter == 0)
	{
		gs_possible = TRUE;
	} else
	{
		gs_possible = FALSE;
	}
	
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/*
	GEMScript entsperren (es wird mitgezaehlt: erst entsperren wenn alle meinen es geht)
*/
GLOBAL _BOOL EnableGEMScript(_VOID)
{
	gsp_counter--;

	if (gsp_counter < 0)
	{
		/*
			Interner Fehler! Irgendwas stimmt da mit dem sperren/entsperren nicht!
			Programmcode pruefen! GEMScript zur Sicherheit moeglichst ganz sperren!
		*/
		TRACE((EO_DEBUG, "[EnableGEMScript] Internal Error: setting GEMScript offline"));
		
		gsp_counter = GS_OFFLINE; /* Da kann man viele EnableGEMScript aufrufen bevor das wieder geht */
	}

	if (gsp_counter == 0)
	{
		gs_possible = TRUE;
	} else
	{
		gs_possible = FALSE;
	}
	
	return gs_possible;
}

/*** ---------------------------------------------------------------------- ***/

/*
	Neues GEMScript-Kommando in die Liste aufnehmen
*/
GLOBAL _BOOL GEMScript_Command(CONST _UBYTE *command, GSC_Command_Handler func)
{
	GSC_LISTE *temp;

	temp = MALLOC(sizeof(GSC_LISTE), "GEMScript_Command");
	if (temp != NULL)
	{
		temp->name = command;
		temp->func = func;
		temp->next = gs_commands;
		gs_commands = temp;
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*
	Neuen GEMScript Kommunikationspartner in die Liste aufnehmen
*/
LOCAL _WORD NewGSPartner(_WORD id, _WORD gid)
{
	GS_PARTNER *check;

	/* Speicher fuer neue Struktur anfordern und Daten eintragen */
	check = SYSMALLOC(sizeof(GS_PARTNER), MEM_NOERR, "NewGSPartner (New)");
	if (check != NULL)
	{
		check->id = id;
		check->gid = gid;
		check->next = gp_liste;
	} else
	{
		TRACE((EO_DEBUG, "[NewGSPartner] out of memory: malloc for partner failed"));
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/*
	Diese Procedure entfernt einen Kommunikationspartner aus
	der verketteten Liste
*/
LOCAL _BOOL DeleteGSPartner(_WORD id, _WORD gid)
{
	GS_PARTNER **search;
	GS_PARTNER *temp;
	
	search = &gp_liste;
	while ((temp = *search) != NULL)
	{
		/*
			Testen ob AES-ID und GS-ID mit dem zu
			entfernenden Partner uebereinstimmen
		*/
		if (temp->id == id && temp->gid == gid)
		{
			/*
				Diesen Partner entfernen
			*/
			*search = temp->next;
			SFREE(temp);
			return TRUE;
		}
		search = &temp->next;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*
	GS_QUIT an Partner senden
*/
LOCAL _VOID SendGSQuit(_WORD id, _WORD gid)
{
	_WORD msg[8];
	
	msg[0] = GS_QUIT;
	msg[1] = gl_apid;
	msg[2] = 0;
	msg[3] = 0;
	msg[4] = 0;
	msg[5] = 0;
	msg[6] = 0;
	msg[7] = gid;
	
	wind_update(END_UPDATE);
	appl_write(id, 16, msg);
	wind_update(BEG_UPDATE);

}

/*** ---------------------------------------------------------------------- ***/

/*
	Diese Procedure sendet AP_TERM an uns selbst, damit wir uns beenden. :-)
	So spare ich mir Einzelheiten des Beendigungsvorgang. :-)
*/
LOCAL _VOID QuitProc(_VOID)
{
	_WORD msg[8];
	
	msg[0] = AP_TERM;
	msg[1] = gl_apid;
	msg[2] = 0;
	msg[3] = 0;
	msg[4] = 0;
	msg[5] = 0; /* 0 statt AP_TERM -> Indikator fuer intern gesendete Message */
	msg[6] = 0;
	msg[7] = 0;
	
	appl_write(gl_apid, 16, msg);
}

/*** ---------------------------------------------------------------------- ***/

/*
	GEMScript Message GS_COMMAND auswerten
*/
LOCAL _BOOL HandleGSCommand(CONST _UBYTE *cmdln, _UBYTE **retln)
{
	size_t i;
	GSC_LISTE *temp;
	/* Zeiger fuer Parameterliste */
	_UBYTE *gs_argv[GS_MAX_PAR];
	_BOOL gs_arg_allocated[GS_MAX_PAR];
	/* Counter fuer Parameterliste */
	_WORD gs_argc;
	_BOOL found;
	
	TRACE((EO_DEBUG, "[HandleGSCommand] cmdln:='%s'", cmdln));

	/*
		Default-Return:
		-Kein Resultatstring=Kommando unbekannt
	*/
	*retln = NULL;

	/*
		Un nu gucken wir uns mal die Parameter an
	*/
	i = 0;
	gs_argc = 0;
	while (cmdln[i] != '\0' && gs_argc < GS_MAX_PAR)
	{
		gs_arg_allocated[gs_argc] = FALSE;
		if (cmdln[i] != '\0')
		{
			if (cmdln[i] >= '\1' && cmdln[i] <= '\6')
			{
				if (cmdln[i] == '\2')
				{
					/* Hex-Codierung */
					size_t start, count;
					_UBYTE c1, c2;
					size_t l;
					
					i++;
					start = i;
					while (cmdln[i] != '\0')
						i++;
					count = (i - start) / 2;
					gs_argv[gs_argc] = SYSMALLOC(count + 1, MEM_NOERR, "HandleGSCommand: Hex-String");
					if (gs_argv[gs_argc] != NULL)
					{
						i = start;
						l = 0;
						for (l = 0; l < count; l++)
						{
							c1 = cmdln[i++];
							c2 = cmdln[i++];
							gs_argv[gs_argc][l] = hextoc(c1) * 16 + hextoc(c2); 
						}
						gs_argv[gs_argc][l] = '\0';
						
						if (cmdln[i] != '\0')
							i++; /* Hm, ungerade Anzahl Hex-Ziffern */
						gs_arg_allocated[gs_argc] = TRUE;
					}
				} else
				{
					/* Leerer bzw. reservierter Parameter -> Zeiger auf Leerstring */
					
					/* Auf Nullbyte "vorspulen" */
					while (cmdln[i] != '\0')
						i++;
					/* und den Zeiger darauf uebergeben */
					gs_argv[gs_argc] = (_UBYTE *)NO_CONST(cmdln) + i;
				}
			} else
			{
				/* "gueltiger" Parameter -> uebertragen */
				gs_argv[gs_argc] = (_UBYTE *)NO_CONST(cmdln) + i;

				while (cmdln[i] != '\0')
					i++;
			}

			TRACE((EO_DEBUG, "[HandleGSCommand] Reading Parameter %d: '%s'", gs_argc, gs_argv[gs_argc]));
		}
		gs_argc++;
		i++;
	}

#if GS_DEBUG
	TRACE((EO_TRACE, "[HandleGSCommand] Checking parsed Parameters:"));
	TRACE((EO_TRACE, "argc:=%d", gs_argc));
	/* Mit dem ifdef wird ggf. die nutzlose Schleife rausgeworfen */
	for (i = 0; i < gs_argc; i++)
		TRACE((EO_TRACE, "argv[%ld]:=%s", i, gs_argv[i]));
#endif

	found = FALSE;
	if (gs_argc == 0)
	{
		/* Nanu? Ueberhaupt kein Kommando? */
	} else
	{
		/*
			Nun folgen Standardkommandos, die universell fuer alle APPs
			sind, die dieses Modul verwenden
		*/
		/* AppGetLongName */
		if (strCaseCmp(gs_argv[0], "appgetlongname") == 0)
		{
			_UBYTE LongName[256];
			size_t len;
			
			ConfVars_GetConfDefStr("APPLongName", LongName, sizeof(LongName), gsAppName);
			len = strLen(LongName);
			*retln = SYSMALLOC(len + 2, MEM_NOERR | MEM_SHARABLE, "HandleGSCommand");

			if (*retln != NULL)
			{
				/* Variable in Rueckgabestring kopieren */
				strCpy(*retln, LongName);
				(*retln)[len + 1] = '\0';
			} else
			{
				TRACE((EO_DEBUG, "[HandleGSCommand] malloc for AppGetLongName() response failed"));
				/* retln=NULL; Is ja schon so :-) */
			}
			found = TRUE;
		}
		/* Quit */
		if (strCaseCmp(gs_argv[0], "quit") == 0)
		{
			*retln = SYSMALLOC(3, MEM_NOERR | MEM_SHARABLE, "HandleGSCommand");
			if (*retln != NULL)
			{
				QuitProc();
				MemCpy(*retln, "1\0", 3);
			} else
			{
				TRACE((EO_DEBUG, "[HandleGSCommand] malloc for quit() response failed"));
				/* retln=NULL; Is ja schon so :-) */
			}
			found = TRUE;
		}
	
		/*
			Nu gugge mer mal ob der Befehl in der verketteten Liste steht...
		*/
		temp = gs_commands;
		while (temp != NULL)
		{
			/* kommando wegen caseinsensitiv! argv[0]=casesensitiv */
			if (strCaseCmp(gs_argv[0], temp->name) == 0)
			{
				/* Befehl gefunden: Handler aufrufen... */
				/*(*GSC_Command_Handler)(_WORD argc, _UBYTE *argv[], _UBYTE **erg);*/
				temp->func(gs_argc, gs_argv, retln);
				found = TRUE;
				/* ...und tschuess */
				break;
			}
			temp = temp->next;
		}
	}
	
	for (i = 0; i < gs_argc; i++)
		if (gs_arg_allocated[i])
			SFREE(gs_argv[i]);
	
	TRACE((EO_DEBUG, "[HandleGSCommand] Reply: $%08lx %s", (_ULONG)(*retln), *retln ? (const char *)(*retln) : "<null>"));

	return found;
}

/*** ---------------------------------------------------------------------- ***/

/*
	Mit dieser Funktion werden GEMScript Nachrichten behandelt. Sie wird in
	PROTOCOL.C verwendet, um das GEMScript Modul seine Messages auswerten
	zu lassen.
*/
LOCAL _BOOL GScript_Handle_Message(CONST _WORD *message)
{
	/* Messagearry fuer GEMScript-Antwort */
	_WORD gmes[8];
	_WORD ret;
	_VOID *memp1;
	_VOID *memp2;
	_UBYTE *cmdln, *retln;
	_BOOL found;
	
	switch (message[0])
	{
	/*
		GS_REPLY auswerten: zu dieser APP wollten wir eine Kommunikation aufbauen
	*/
	case GS_REQUEST:
		TRACE((EO_DEBUG, "GS_REQUEST from %d", message[1]));
		gmes[0] = GS_REPLY;
		gmes[1] = gl_apid;
		gmes[2] = 0;
		/* Pointer auf GS_INFO-Struktur */
		gmes[3] = (_WORD)((_LONG)(gsi_mem)>>16);
		gmes[4] = (_WORD)((_LONG)gsi_mem & 0xFFFF);
		gmes[5] = 0;
		/* GEMScript Kommunikation zur Zeit moeglich? */
		if (gs_possible)
		{	/* Ja */
			gmes[6] = 0;
		} else
		{	/* Nein */
			gmes[6] = 1;
		}
		gmes[7] = message[7];
		appl_write(message[1], 16, gmes);
		if (gs_possible)
			NewGSPartner(message[1], message[7]); /* Partner in Liste aufnehmen */
		break;
	
	case GS_COMMAND:
		cmdln = *((_VOID **)&message[3]);
		found = HandleGSCommand(cmdln, &retln);
		if (found == FALSE)
		{
			ret = GSACK_UNKNOWN;
		} else if (retln == NULL)
		{
			ret = GSACK_ERROR;
		} else
		{
			ret = GSACK_OK;
		}
		gmes[0] = GS_ACK;
		gmes[1] = gl_apid;
		gmes[2] = 0;
		gmes[3] = message[3];
		gmes[4] = message[4];
		gmes[5] = (_WORD)((_LONG)(retln)>>16);
		gmes[6] = (_WORD)((_LONG)retln & 0xFFFF);
		gmes[7] = ret;
		TRACE((EO_TRACE, "GS_ACK to %d: $%08lx $%08lx %d", message[1], (_ULONG)cmdln, (_ULONG)retln, ret));
		while(*retln)
		{
			TRACE((EO_TRACE, "GS_ACK Parameter:%s",retln));
			retln+=strLen(retln)+1;
		}
		appl_write(message[1], 16, gmes);
		break;
	
	case GS_ACK:
		memp1 = *((_VOID **)&message[3]);
		memp2 = *((_VOID **)&message[5]);
		TRACE((EO_DEBUG, "GS_ACK from %d: $%08lx $%08lx", message[1], (_ULONG)memp1, (_ULONG)memp2));
		/*
			Messagetyp: nur Speicher freigeben
		*/
		if (memp1 == NULL && memp2 != NULL)
		{	/* Speicher freigeben */
			SFREE(memp2);
		}
		/*
			Messagetyp: Speicher freigeben und ggf. replyen
		*/
		if (memp1 != NULL)
		{	/* Speicher freigeben */
			SFREE(memp1);
			if (memp2 != NULL)
			{
				/*
					Es liegt ein Ergebnis vor, ggf. sollte man das
					Ergebniss sichern, damit es ausgewertet werden
					kann
				*/
				TRACE((EO_TRACE, "GS_ACK to %d: $%08lx", message[1], (_ULONG)memp2));
				gmes[0] = GS_ACK;
				gmes[1] = gl_apid;
				gmes[2] = 0;
				gmes[3] = 0;
				gmes[4] = 0;
				gmes[5] = message[5];
				gmes[6] = message[6];
				gmes[7] = 0;
				appl_write(message[1], 16, gmes);
			}
		}
		break;
	
	case GS_QUIT:
		TRACE((EO_DEBUG, "GS_QUIT from %d", message[1]));
		DeleteGSPartner(message[1], message[7]); /* Partner aus der Liste entfernen */
		break;
	
	default:
		return FALSE;
	}
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*
	GEMScript Internas initialisieren
*/
GLOBAL _BOOL GScript_Init(CONST _UBYTE *AppName)
{
	GS_INFO *gsi;

	gs_initialised = FALSE;
	gs_possible = FALSE;
	gsp_counter = 0;
	gs_commands = NULL;
	gsAppName = AppName;
	/*
		GS_INFO Struktur global anfordern
	*/	
	gsi_mem = SYSMALLOC(sizeof(GS_INFO), MEM_NOERR | MEM_SHARABLE, "InitGScript: global GS_INFO");
	if (gsi_mem != NULL)
	{
		gsi = (GS_INFO *)gsi_mem;
		/*
			Globale GS_INFO-Struktur fuellen
		*/
		gsi->len = sizeof(GS_INFO);
		gsi->version = GS_VERSION;
		gsi->msgs = GS_MSGS;
		gsi->ext = 0;
		
		gs_initialised = TRUE;
		gs_possible = TRUE;
		Protokoll_Handle_Message(GScript_Handle_Message);
		return TRUE;
	}

	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*
	GEMScript Internas deinitialisieren
*/
GLOBAL _VOID GScript_Exit(void)
{
	GSC_LISTE *temp, *temp2;

	gs_initialised = FALSE;
	gs_possible = FALSE;
	
	/*
		Wir verabschieden uns von allen Kommunikationspartnern
		und geben den Speicher frei
	*/
	
	while (gp_liste != NULL)
	{
		SendGSQuit(gp_liste->id, gp_liste->gid);
		DeleteGSPartner(gp_liste->id, gp_liste->gid);
	}
	/*
		Speicher fuer die Kommandoliste freigeben
	*/
	temp = gs_commands;
	while (temp != NULL)
	{
		temp2 = temp->next; /* Adresse des naechsten Elements sichern */
		SFREE(temp);        /* Dieses Element freigeben */
		temp = temp2;       /* Next Step: naechstes Element */
	}

	if (gsi_mem != NULL)
	{
		/*
			GS_INFO Speicher wieder freigeben
		*/
		SFREE(gsi_mem);
		gsi_mem = NULL;
	}
}
