/*
	GEMScript-Headerfile fuer CAT
	
	Written by Volker Janzen
	
	History:
	
	24.01.98: -Erste Version des Headerfiles fuer erstes GEMScript
			   Modul in CAT
	08.04.98: -GLOBAL-Fixes wie in GSCRIPT.C
			  -Neue Funktionen IsGEMScriptPossible, IsGEMScriptInitialised,
			   DisableGEMScript, EnableGEMScript eingebaut
			  -Programmierdoku in dieses Headerfile aufgenommen
*/
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __CAT_GSCRIPT_H__
#define __CAT_GSCRIPT_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif

/*
	GScript_Init initialisiert das GEMScript-Modul. Die Funktion wird von
	CAT.C aufgerufen und sollte dann nicht mehr benutzt werden. AppName
	gibt den Defaultnamen fuer AppGetLongName an.
*/
GLOBAL _BOOL GScript_Init(CONST _UBYTE *AppName);

/*
	Diese Procedure deinitialisiert das GEMScript-Modul. Sie wird von CAT.C
	in der Terminirungsroutine aufgerufen und sollte sonst nicht verwendet
	werden
*/
GLOBAL _VOID GScript_Exit(_VOID);

/*
	Die Funktion IsGEMScriptPossible signalisiert ob zur Zeit Kommandos
	ueber GEMScript ausgefuehrt werden koennen. (TRUE=ja, FALSE=nein)
*/
GLOBAL _BOOL IsGEMScriptPossible(_VOID);

/*
	IsGEMScriptInitialised liefert TRUE wenn das GEMScriptmodul initialisiert
	wurde, sonst FALSE
*/
GLOBAL _BOOL IsGEMScriptInitialised(_VOID);

/*
	In manchen Faellen kann es sinnvoll sein GEMScript Kommandos nicht zu
	erlauben. Diesen Zustand teilt man Modul mit folgender Funktion mit.
	GSC_COMMAND-Messages werden dann Protokollkonform abgewiesen. Das
	Resultat ist zur Zeit immer TRUE.
*/
GLOBAL _BOOL DisableGEMScript(_VOID);

/*
	Mit der folgenden Funktion teilt man dem Modul mit, das wieder
	GEMScript Kommandos moeglich sind. Die beiden Funktionen zaehlen die
	Aufrufe von DisableGEMScript und EnableGEMScript mit, GEMScript
	Kommandos werden also wirklich erst wieder erlaubt, wenn auf alle
	DisableGEMScript Aufrufe ein EnableGEMScript Aufruf folgte. Aus diesem
	Grund liefert die Funktion TRUE oder FALSE zurueck, je nachdem ob nach
	dem Funktionsaufruf wieder GEMScript Kommandos moeglich sind, oder nicht.
*/
GLOBAL _BOOL EnableGEMScript(_VOID);

/*
	So sieht der GSC_COMMAND Kommandohandler aus:
	argc, argv ist analog zur C-Kommandozeile aufgebaut
	Achtung: es koennen Parameter vorkommen, die gemaess der GEMScript-Doku
	mit \1 bis \6 anfangen, dies muss vom Handler ausgewertet werden.
	erg ist ein Zeiger auf den Antwortstring. Er muss von der
	Handlerfunktion selbst GLOBAL angefordert werden und wird von GEMScript
	Modul bei der entsprechenden GS_ACK Message wieder freigegeben.
*/
typedef _VOID (*GSC_Command_Handler)(_WORD argc, _UBYTE *argv[], _UBYTE **erg);

/*
	Mit dieser Funktion koennen die Module ihre GEMScriptfunktionen
	anmelden:
	command ist der Kommandoname (Gross/Kleinschreibung ist egal)
	func ist die Handlerfunktion
*/
_BOOL GEMScript_Command(CONST _UBYTE *command, GSC_Command_Handler func);

#endif /* __CAT_GSCRIPT_H__ */
