/*****************************************************************************
 *		TOS/MEMORY.C
 *****************************************************************************/

#include <portab.h>
#include <var_args.h>
#include <tos.h>
#include <ro_mem.h>
#include <debug.h>
#include <routines.h>
#include <catlib.h>	/*DYJ 5.34o */
#include <catlibOL.h>	/*DYJ 5.36b */
#include <mtrsc.h>	/*DYJ 5.34o */

#ifndef GUI_VERSION
#  define GUI_VERSION 1
#endif

#if GUI_VERSION
#  include <portaes.h>
#  include <mtalerts.h>
#define VA_START			0x4711
#endif
/*
 * 4.12.02 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.19k'
 *	Der Test ob die memory-Protection Bits bei Mxalloc verwendet werden
 *	sollen soll sicherer ber Sysconf gehen.
 */
LOCAL ERR_MEM_FUNC p_err_memory = FUNK_NULL;

OS_GLOBAL _BOOL rtx_found;

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ErrorStrOut(CONST _UBYTE *str)
{
	LOCAL _LONG debug_pipe = -1;

	if (debug_pipe == -1)
	{
		debug_pipe = Fopen("U:\\PIPE\\DEBUG", 2);
	}
	if (debug_pipe >= 0)
	{
		Fwrite((_WORD)debug_pipe, strLen(str), str);
		return;
	}
#if GUI_VERSION
	{
		LOCAL _WORD apid = -2;

		if (apid == -2)
			apid = appl_find("DEBUG   ");

		if (apid >= 0)
		{
			_WORD len;
			char buf[512+16];
			_WORD *message = (_WORD *)buf;

			len = (_WORD)strLen(str);
			if (len > 512)
				len = 512;
			message[0] = VA_START+1;
			message[1] = gl_apid;
			message[2] = len;
			message[3] = 0;
			message[4] = 0;
			message[5] = 0;
			message[6] = 0;
			message[7] = 0;
			memcpy(buf+16, str, len);
			appl_write(apid, 16+len, message);
			return;
		}
	}
#endif
	{
		fputs(str, stderr);
		fflush(stderr);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ErrorOut(EO_MODUS modus, CONST char *str __c_va_alist)
{
	_BOOL alert_error;
	_UBYTE	box[512];
	_UBYTE *ptrPrgName = ProgramName;
	__c_va_list args;

/*
	if (modus == EO_DEBUG && !is_MASTER)
		return;
*/

	__c_va_start(args, str);
	vsprintf(box, str, args);
	__c_va_end(args);

	alert_error = modus == EO_ERROR || modus == EO_FATAL;
	if (!error_std_out(modus, ptrPrgName, box))
		alert_error = FALSE;
	if (alert_error)
	{
#if GUI_VERSION
		mtAlerts_Alert(1, box);
#endif
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL ERR_MEM_FUNC set_err_memory(ERR_MEM_FUNC p_err)
{
	ERR_MEM_FUNC old = p_err_memory;
	p_err_memory = p_err;
	return old;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID err_memory(_ULONG size)
{
	if (p_err_memory != FUNK_NULL)
	{
		(*p_err_memory)(size);
	} else
	{
#if GUI_VERSION
/*DYJ 5.34o Start*/
		ErrorOut(EO_FATAL, Rsc_WrkString(AL_NO_MEM));
/*DYJ  Ende; alt:	
		ErrorOut(EO_FATAL, "[3][%%P:|F\201r die gew\201nschte|Operation steht|zuwenig Speicher|zur Verf\201gung.][[Abbruch]");
*/
#else
		ErrorOut(EO_FATAL, "not enough memory for %ld bytes", size);
#endif /* GUI_VERSION */
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID *sysMemGet(_ULONG size, _UWORD *type)
{
	_VOID *ptr;

	if (*type & MEM_SHARABLE)
		*type |= MEM_SYSTEM;
	{
		if (*type & MEM_SYSTEM)
		{
			_WORD mode = 0x03;
			if ((*type & MEM_SHARABLE) && rtx_found)
				mode |= 0x40;
			if (*type & MEM_NOT_BANKED)
				mode &= 0xfffc;
			ptr = Mxalloc(size, mode);
			if (ptr == (_VOID *)-32)
				ptr = Malloc(size);
		} else
		{
			ptr = malloc(size);
		}
	}
	
	return ptr;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID sysMemFree(_VOID *ptr, _UWORD type)
{
	if (type & MEM_SYSTEM)
		Mfree(ptr);
	else
		free(ptr);
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.19k*/
LIB_GLOBAL _VOID sys_mem_init(_BOOL init)
{
	if (init)
	{
#if GUI_VERSION
		rtx_found=Sysconf(-1)!=-32;
#endif
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG mem_max(_VOID)
{
	_ULONG coresize;
	_ULONG thissize;
	_VOID **buf;
	_VOID **list;

	coresize = 0;
	list = NULL;
	while ((thissize = (_ULONG)Malloc(-1l)) >= sizeof(_VOID *))
	{
		buf = (_VOID **)Malloc(thissize);
		if (buf == NULL)
			continue;
		coresize += thissize;
		*buf = list;
		list = buf;
	}
	while (list != NULL)
	{
		buf = *list;
		Mfree(list);
		list = buf;
	}
	return coresize;
}

/*** ---------------------------------------------------------------------- ***/

#define BEL 0x07

GLOBAL _VOID Pling(_VOID)
{
#if GUI_VERSION
#if GEM & (GEM2 | GEM3 | XGEM)
	v_sound(phys_handle, 550, 3);
#else
	Cconout(BEL);
	/* Bconout (2, BEL); */
#endif
#else
	/* putc(BEL, stderr);
	fflush(stderr); */
#endif
}

