/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <olga.h>
#include <mtappl.h>
#include <tos.h>
#include <magiccoo.h>
#include <portaes.h>
#include <protokol.h>
#include <termproc.h>
#include <routinec.h>

#include <ro_mem.h>


#define OLE_INIT							0x4950
#define OLE_EXIT							0x4951
#define OLE_NEW 							0x4952

#define OLGA_INIT 							0x1236
#define OLGA_UPDATE 						0x1238
#define OLGA_ACK							0x1239
#define OLGA_RENAME 						0x123A
#define OLGA_OPENDOC						0x123B
#define OLGA_CLOSEDOC 						0x123C
#define OLGA_LINK 							0x123D
#define OLGA_UNLINK 						0x123E
#define OLGA_UPDATED						0x123F
#define OLGA_RENAMELINK 					0x1240
#define OLGA_LINKRENAMED					0x1241
#define OLGA_GETOBJECTS 					0x1242
#define OLGA_OBJECTS						0x1243
#define OLGA_BREAKLINK						0x1244
#define OLGA_LINKBROKEN 					0x1245
#define OLGA_START							0x1246
#define OLGA_GETINFO						0x1247
#define OLGA_INFO 							0x1248
#define OLGA_IDLE 							0x1249
#define OLGA_ACTIVATE 						0x124A
#define OLGA_EMBED							0x124B
#define OLGA_EMBEDDED 						0x124C
#define OLGA_UNEMBED						0x124D
#define OLGA_GETSETTINGS					0x124E
#define OLGA_SETTINGS 						0x124F
#define OLGA_REQUESTNOTIFICATION			0x1250
#define OLGA_RELEASENOTIFICATION			0x1251
#define OLGA_NOTIFY 						0x1252
#define OLGA_NOTIFIED 						0x1253
#define OLGA_SERVERTERMINATED	 			0x1254
#define OLGA_CLIENTTERMINATED 				0x1255
#define OLGA_INPLACEUPDATE					0x1256
#define OLGA_ID4UPDATE						0x1257
#define OLGA_GETEXTENSION					0x1258
#define OLGA_EXTENSION						0x1259
#define OLGA_GETSERVERPATH					0x125A
#define OLGA_SERVERPATH						0x125B
#define OLGA_IE_BUTTON						0x125C
#define OLGA_IE_KEY							0x125D

#define OL_OEP								0x0001

#define OLS_TYPE							1
#define OLS_EXTENSION 						2
#define OLS_NAME							3

#define OL_SRV_ID4							0x0001

typedef struct {
	_LONG  magic;
	_UWORD version;
	_UWORD skip;
} OLGAInfHeader;

typedef struct {
	_ULONG id;
	_ULONG length;
} OLGABlockHeader;

#define OLGA_INF_MAGIC		'OLGA'

#define OLGA_BLOCK_REM		'REM '
#define OLGA_BLOCK_AUTHOR	'AUTH'
#define OLGA_BLOCK_KEYWORD	'KEYW'
#define OLGA_BLOCK_DATE		'DATE'
#define OLGA_BLOCK_ICON		'ICON'
#define OLGA_BLOCK_EOF		0L


typedef struct {
	_WORD red;
	_WORD green;
	_WORD blue;
} OLGARGB;

typedef struct {
	_WORD Count;
	OLGARGB colors[1];
} OLGAColorTable;

typedef struct _ObjectInfo ObjectInfo;

typedef _VOID _CDECL (*OLGADraw)(ObjectInfo *objectinfo, _WORD outScreen, _WORD outHandle, _WORD outDevID, GRECT *Size, GRECT *clip);
typedef _VOID _CDECL (*OLGAUnembed)(ObjectInfo *objectinfo);
typedef _VOID _CDECL (*OLGAXDraw)(ObjectInfo *objectinfo, _WORD outScreen, _WORD outHandle, _WORD outDevId, GRECT *Size, GRECT *clip, _LONG Width_mm1000, _LONG Height_mm1000, _LONG Scale);

struct _ObjectInfo {
	_UBYTE			*Filename;
	AESPB			*ClientGEMPB;
	_LONG			 CLientData;
	_LONG			 ServerData;
	_WORD			 CBLock;
	_WORD			 CBCount;
	OLGADraw		 CBDraw;
	OLGAUnembed		 CBUnembed;
	OLGAXDraw		 CBXDraw;
	OLGAColorTable	*cbColorTable;
	_WORD			 cbClientID;
	_WORD			 cbServerID;
};


typedef struct {
	_UWORD code;
	_WORD apid;
	_UWORD oversize;
	union {
		struct {
			tsProgType progType;
			_UWORD ProtoVersion;
			_UWORD oep;
			_UWORD ini_res;
			_UWORD xacctype;
		} ole_init;
		struct {
			tsProgType olenew_progType;
			_UWORD olenew_ProtoVersion;
			_UWORD olenew_oep;
			_UWORD olenew_res;
			_UWORD olenew_managerversion;
		} ole_new;
		struct {
			_UWORD initmanager;
			_UWORD olgainit_Protoversion;
			_UWORD olgainit_res5;
			_UWORD olgainit_res6;
			_UWORD CanOL;
		} olga_init;
		struct {
			_UWORD initmanager;
			_UWORD olgainit_Protoversion;
			_UWORD olgainit_res5;
			_UWORD olgainit_res6;
			_UWORD CanOL;
		} ole_exit;
		struct {
			_UBYTE *update_pName;
			_UWORD Index;
		} olga_update;
		struct {
			_UWORD ack_res[4];
			_UWORD whichacknowledge;
		} olga_ack;
		struct {
			_UBYTE *link_pName;
			_UWORD link_Group;
		} olga_link;
		struct {
			_UBYTE *link_pName;
			_UWORD link_Group;
		} olga_unlink;
		struct {
			_UWORD open_res[2];
			_UWORD open_Group;
		} olga_opendoc;
		struct {
			_UWORD open_res[2];
			_UWORD open_Group;
		} olga_closedoc;
		struct {
			_UBYTE *upd_pName;
			_UWORD upd_Index;
			_UWORD upd_serverid;
			_UWORD upd_group;
		} olga_updated;
		struct {
			_UBYTE *rename_old;
			_UBYTE *rename_new;
			_UWORD rename_group;
		} olga_rename;
		struct {
			_UBYTE *name;
			_UWORD group;
		} olga_broken;
		struct {
			_WORD serverId;
		} olga_terminated;
	} c;
} tMsg;

typedef struct _tHandler {
	tHandlerType Type;
	tHandleProc Call;
} tHandler;

typedef struct _tHandlerTab tHandlerTab;
struct _tHandlerTab {
	tHandler Handler;
	tHandlerTab *next;
};


LOCAL tsProgType MyProgType;
LOCAL tObjType MyObjType;
LOCAL _BOOL ServerCanOL = FALSE;
LOCAL _WORD OLGAApId = -1;
LOCAL tHandlerTab *HandlerTab = NULL;
LOCAL _UBYTE *pathbuf;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-----------------------------------------------------------------------
 *
 * Setzen/Loeschen von Handlern
 *
 *-----------------------------------------------------------------------*/

GLOBAL _BOOL Olga_SetHandler(tHandlerType type, tHandleProc proc)
{
	tHandlerTab *pHandler;

	pHandler = NEW(tHandlerTab, "Olga_SetHandler");
	if (pHandler == NULL)
		return FALSE;

	pHandler->Handler.Type = type;
	pHandler->Handler.Call = proc;

	/* und einketten */
	pHandler->next = HandlerTab;
	HandlerTab = pHandler;

	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Olga_RemoveHandler(tHandlerType type, tHandleProc proc)
{
	tHandlerTab *pHandler, *RemoveHandler;
	
	pHandler = HandlerTab;

	/* ist es der erste in der Kette? */
	if (HandlerTab != NULL &&
		HandlerTab->Handler.Call == proc &&
		HandlerTab->Handler.Type == type)
	{
		HandlerTab = HandlerTab->next;
		OFREE(pHandler);
	} else
	{
		while (pHandler != NULL)
		{
			RemoveHandler = pHandler->next;
			if (RemoveHandler != NULL &&
				RemoveHandler->Handler.Call == proc &&
				RemoveHandler->Handler.Type == type)
			{
				pHandler->next = RemoveHandler->next;
				OFREE(RemoveHandler);
				return;
			}
			pHandler = pHandler->next;
		}
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-----------------------------------------------------------------------
 *
 * Abzusetzende Mitteilungen
 *
 *-----------------------------------------------------------------------*/

LOCAL _BOOL get_path_buf(_VOID)
{
	if (pathbuf == NULL)
	{
		pathbuf = SYSMALLOC(PATH_MAX, MEM_NOERR|MEM_SHARABLE, "Olga: get_path_buf");
		if (pathbuf == NULL)
			return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID DoInit(_WORD ApId, tsProgType progtype, tObjType objtype)
{
	tMsg Msg;
	
	OLGAApId = ApId;
	Msg.code = OLE_INIT;
	Msg.apid = gl_apid;
	Msg.oversize = 0;
	Msg.c.ole_init.progType = progtype;
	Msg.c.ole_init.ProtoVersion = 0;
	Msg.c.ole_init.oep = 0;
	Msg.c.ole_init.ini_res = 0;
#define XACCTYPE(a,b) (((a) << 8) | (b))
	switch (objtype)
	{
		case OLGA_Text:      Msg.c.ole_init.xacctype = XACCTYPE('W', 'P'); break;
		case OLGA_DTP:       Msg.c.ole_init.xacctype = XACCTYPE('D', 'P'); break;
		case OLGA_Editor:    Msg.c.ole_init.xacctype = XACCTYPE('E', 'D'); break;
		case OLGA_Database:  Msg.c.ole_init.xacctype = XACCTYPE('D', 'B'); break;
		case OLGA_Tabcalc:   Msg.c.ole_init.xacctype = XACCTYPE('S', 'S'); break;
		case OLGA_Raster:    Msg.c.ole_init.xacctype = XACCTYPE('R', 'G'); break;
		case OLGA_Vektor:    Msg.c.ole_init.xacctype = XACCTYPE('V', 'G'); break;
		case OLGA_Graphics:  Msg.c.ole_init.xacctype = XACCTYPE('G', 'G'); break;
		case OLGA_Music:     Msg.c.ole_init.xacctype = XACCTYPE('M', 'U'); break;
		case OLGA_CAD:       Msg.c.ole_init.xacctype = XACCTYPE('C', 'D'); break;
		case OLGA_Comm:      Msg.c.ole_init.xacctype = XACCTYPE('D', 'C'); break;
		case OLGA_Desk:      Msg.c.ole_init.xacctype = XACCTYPE('D', 'T'); break;
		case OLGA_ProgShell: Msg.c.ole_init.xacctype = XACCTYPE('P', 'E'); break;
		default: /* HALT; */ return;   /* ganz schlecht!!! */
	}
#undef XACCTYPE
	appl_write(OLGAApId, 16, (_WORD *)&Msg);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL Olga_Run_Manager(_BOOL force)
{
	_WORD olgaapid;
	_UBYTE *OlePath;
	
	if (OLGAApId < 0)
	{
		olgaapid = appl_locate("OLGA", FALSE);
		/* oepapid  = appl_find("OEP_SERV"); */
	
		if (olgaapid < 0 && force)
		{
			/* den Managername holen */
			if ((OlePath = getenv("OLGAMANAGER")) != NULL)
			{
				olgaapid = appl_locate(OlePath, TRUE);
			} else if ((OlePath = getenv("OEPSERVER")) != NULL)
			{
				olgaapid = appl_locate(OlePath, TRUE);
			}
		}
	
		if (olgaapid >= 0)
		{
			DoInit(olgaapid, MyProgType, MyObjType);
		}
	}
	return OLGAApId >= 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL Olga_Start_Manager(_VOID)
{
	return Olga_Run_Manager(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Olga_Update(CONST _UBYTE *name)
{
	if (Olga_Start_Manager())
	{
		if (get_path_buf())
		{
			strMcpy(pathbuf, PATH_MAX, name);
			Protokoll_Send(OLGAApId, OLGA_UPDATE, PROT_NAMEPTR(pathbuf), 0, 0, 0);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Olga_OpenDoc(_UWORD GroupId)
{
	if (Olga_Start_Manager())
	{
		Protokoll_Send(OLGAApId, OLGA_OPENDOC, GroupId, 0, 0, 0, 0);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Olga_CloseDoc(_UWORD GroupId)
{
	if (Olga_Start_Manager())
	{
		Protokoll_Send(OLGAApId, OLGA_CLOSEDOC, GroupId, 0, 0, 0, 0);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Olga_Link(CONST _UBYTE *name, _UWORD GroupId)
{
	if (Olga_Start_Manager())
	{
		if (get_path_buf())
		{
			strMcpy(pathbuf, PATH_MAX, name);
			Protokoll_Send(OLGAApId, OLGA_LINK, PROT_NAMEPTR(pathbuf), GroupId, 0, 0);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Olga_Unlink(CONST _UBYTE *name, _UWORD GroupId)
{
	if (Olga_Start_Manager())
	{
		if (get_path_buf())
		{
			strMcpy(pathbuf, PATH_MAX, name);
			Protokoll_Send(OLGAApId, OLGA_UNLINK, PROT_NAMEPTR(pathbuf), GroupId, 0, 0);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Olga_Start(CONST _UBYTE *filename)
{
	_WORD message[8];
	
	if (Olga_Run_Manager(TRUE))
	{
		if (get_path_buf())
		{
			strMcpy(pathbuf, PATH_MAX, filename);
			Protokoll_Send(OLGAApId, OLGA_GETEXTENSION, PROT_NAMEPTR(pathbuf), 0, 0, 0);
			if (Protokoll_Waitfor(OLGA_EXTENSION, 2000, message))
			{
				Protokoll_Send(OLGAApId, OLGA_START, OLS_EXTENSION, message[5], message[6], PROT_NAMEPTR(pathbuf));
				return TRUE;
			}
		}
	}
	return FALSE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-----------------------------------------------------------------------
 *
 * Handling von Nachrichten
 *
 *-----------------------------------------------------------------------*/

LOCAL _VOID Acknowledge(CONST tMsg *Msg)
{
	switch (Msg->c.olga_ack.whichacknowledge)
	{
	case OLGA_UPDATE:
	case OLGA_UNLINK:
	case OLGA_LINK:
		if (Msg->c.olga_update.update_pName == pathbuf &&
			pathbuf != NULL)
		{
			SFREE(pathbuf);
			pathbuf = NULL;
		}
		break;
	case OLGA_ACTIVATE:
		/* TODO */
		break;
	case OLGA_INFO:
		/* TODO */
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID Updated(CONST tMsg *Msg)
{
	tHandlerTab *pHandler;
	
	pHandler = HandlerTab;
	while (pHandler != NULL)
	{
		if (pHandler->Handler.Type == cUpdated &&
			pHandler->Handler.Call(Msg->c.olga_updated.upd_pName,
				Msg->c.olga_updated.upd_Index,
				Msg->c.olga_updated.upd_group,
				Msg->c.olga_updated.upd_serverid))
		{
			return; 	/* bearbeitet */
		}
		pHandler = pHandler->next;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ServerTerminated(_WORD ServerID)
{
	/* TODO */
	UNUSED(ServerID);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ClientTerminated(_WORD ClientID)
{
	/* TODO */
	UNUSED(ClientID);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID Olga_Link_Renamed(_UBYTE *old, _UBYTE *new, _UWORD group)
{
	/* TODO */
	Protokoll_Send(OLGAApId, OLGA_LINKRENAMED, PROT_NAMEPTR(old), PROT_NAMEPTR(new), group);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID Olga_Link_Broken(_UBYTE *name, _UWORD group)
{
	/* TODO */
	Protokoll_Send(OLGAApId, OLGA_UNLINK, PROT_NAMEPTR(name), group, 0, 0);
}

/*------------------------------------------------------------------
 *
 *Message verarbeiten
 *
 *Es werden z.Zt. folgende Msgs (aus EvntMulti) verarbeitet:
 *
 *OLE_NEW
 *OLGA_INIT
 *OLGA_UPDATED (an Handler)
 *OLGA_ACK (von UPDATE, LINK, LINK_RENAMED
 *
 *
 *RETURN :
 *
 * TRUE  : Msg wurde verarbeitet
 * FALSE : Msg geht mich nix an.
 *
 *------------------------------------------------------------------*/

LOCAL _BOOL Olga_Msg(CONST _WORD MsgBuffer[])
{
	CONST tMsg *pMsg;

	pMsg = (CONST tMsg *)MsgBuffer;
	switch (pMsg->code)
	{
	case OLE_NEW:	/* Empfang eines nachgestarteten Managers */
		/* Antwort */
		DoInit(pMsg->apid, MyProgType, MyObjType);
		break;
	case OLE_EXIT:
		if (pMsg->apid == OLGAApId)
			OLGAApId = -1;
		break;
	case OLGA_INIT:
		ServerCanOL = pMsg->c.olga_init.CanOL != 0;
		OLGAApId = pMsg->apid;
		break;
	
	case OLE_INIT:
	case OLGA_UPDATE:
	case OLGA_OPENDOC:
	case OLGA_CLOSEDOC:
	case OLGA_RENAME:
	case OLGA_UNLINK:
	case OLGA_LINKRENAMED:
	case OLGA_START:
	case OLGA_BREAKLINK:
	case OLGA_GETEXTENSION:
	case OLGA_GETSERVERPATH:
	case OLGA_REQUESTNOTIFICATION:
	case OLGA_RELEASENOTIFICATION:
	case OLGA_NOTIFIED:
		/* Nachrichten an Manager, interessieren uns nicht */
		break;

	case OLGA_ACK:
		Acknowledge(pMsg);
		break;
	case OLGA_UPDATED:
		Updated(pMsg);
		break;
	case OLGA_IDLE:
		if (MsgBuffer[3] != 0)
			Protokoll_Send(pMsg->apid, OLGA_IDLE, 0, MsgBuffer[4], MsgBuffer[5], MsgBuffer[6], MsgBuffer[7]);
		break;
	case OLGA_RENAMELINK:
		Olga_Link_Renamed(pMsg->c.olga_rename.rename_old, pMsg->c.olga_rename.rename_new, pMsg->c.olga_rename.rename_group);
		break;
	case OLGA_LINKBROKEN:
		Olga_Link_Broken(pMsg->c.olga_broken.name, pMsg->c.olga_broken.group);
		break;
	case OLGA_SERVERTERMINATED:
		ServerTerminated(pMsg->c.olga_terminated.serverId);
		break;
	case OLGA_CLIENTTERMINATED:
		ClientTerminated(pMsg->c.olga_terminated.serverId);
		break;
	case OLGA_GETINFO:
		/* TODO */
		break;
	case OLGA_INFO:
		/* TODO */
		Protokoll_Send(pMsg->apid, OLGA_ACK, MsgBuffer[3], MsgBuffer[4], MsgBuffer[5], MsgBuffer[6], OLGA_INFO);
		break;
	
	case OLGA_GETOBJECTS:
	case OLGA_OBJECTS:
	case OLGA_ACTIVATE:
	case OLGA_EMBED:
	case OLGA_ID4UPDATE:
	case OLGA_EMBEDDED:
	case OLGA_UNEMBED:
	case OLGA_INPLACEUPDATE:
		/* ID4-Nachrichten, NYI */
		break;
	
	case OLGA_SERVERPATH:
		/* Konfigurationsabfragen, NYI */
		Protokoll_Send(pMsg->apid, OLGA_ACK, 0, 0, MsgBuffer[5], MsgBuffer[6], OLGA_SERVERPATH);
		break;
	case OLGA_SETTINGS:
	case OLGA_EXTENSION:
		/* Konfigurationsabfragen, NYI */
		break;
	
	case OLGA_NOTIFY:
		Protokoll_Send(pMsg->apid, OLGA_NOTIFIED, MsgBuffer[3], MsgBuffer[4], MsgBuffer[5], MsgBuffer[6], MsgBuffer[7]);
		break;
	
	default:
		return FALSE;
	}
	return TRUE;
}

/*-----------------------------------------------------------------------
 *
 * Aktivierung und Beendigung
 *
 *-----------------------------------------------------------------------*/

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID DoExit(_VOID)
{
	if (OLGAApId >= 0)
	{
		Protokoll_Send(OLGAApId, OLE_EXIT, 0, 0, 0, 0, 0);
		OLGAApId = -1;
	}
	if (pathbuf != NULL)
	{
		SFREE(pathbuf);
		pathbuf = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Olga_Init(tsProgType progtype, tObjType objtype)
{
	MyProgType = progtype | OL_IDLE;
	MyObjType  = objtype;

	Protokoll_Handle_Message(Olga_Msg);
	InstallTermproc(DoExit);

	if (progtype & OL_SERVER)
		Olga_Run_Manager(TRUE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Olga_Exit(_VOID)
{
	DoExit();
}
