#ifndef __CAT_MTOLGA_H__
#define __CAT_MTOLGA_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif


/****************************************************************************
 *
 * Handling des Olga-Protokolles
 *
 *
 * $Source$
 *
 * $Revision$
 *
 * $Author$
 *
 * $Date$
 *
 * $State$
 *
 *****************************************************************************
 * History:
 *
 * $Log$
 *
 *
 ****************************************************************************/


#define OL_SERVER 							0x0001
#define OL_CLIENT 							0x0002
#define OL_PEER 							(OL_SERVER | OL_CLIENT)
#define OL_CONF								0x0400
#define OL_IDLE 							0x0800
#define OL_PIPES							0x1000
#define OL_START							0x2000
#define OL_MANAGER							0x4000

typedef _UWORD tsProgType;


typedef enum {
	OLGA_Text,
	OLGA_DTP,
	OLGA_Editor,
	OLGA_Database,
	OLGA_Tabcalc,
	OLGA_Raster,
	OLGA_Vektor,
	OLGA_Graphics,
	OLGA_Music,
	OLGA_CAD,
	OLGA_Comm,
	OLGA_Desk,
	OLGA_ProgShell
} tObjType;


/*-----------------------------------------------------------------------
 *
 * OLGA/OLE initialisieren
 * startet OLGA/OLE, wenn nicht vorhanden und meldet sich bei OLGA/OLE an
 * Die Antworten werden erst von Msg verarbeitet
 *
 *-----------------------------------------------------------------------*/


_VOID Olga_Init(tsProgType progtype, tObjType objtype);

_VOID Olga_Exit(_VOID);


typedef enum {
	cUpdated
} tHandlerType;

typedef _BOOL (*tHandleProc)(CONST _UBYTE *fname, _UWORD index, _UWORD Group, _UWORD serverid);

/*-----------------------------------------------------------------------
 *
 * einen Handler installieren
 * Die Liste dieser Handler wird von Msg abgearbeitet. Wird ein passender
 * Handler gefunden, wird dieser aufgerufen.
 * liefert der Handler TRUE gilt die Nachricht als verarbeitet, bei FALSE
 * wird nach weiteren Handler des passenen Types gesucht.
 * Damit koennen mehrere Handler fuer einen Typ installiert werden, die nur
 * die sie betreffenden Teile auswerten.
 * Die Liste wird LIFO bearbeitet, wer als letztes installiert wurde,
 * bekommt als erstes angeboten.
 *
 * Ergebnis TRUE: Handler wurde aufgenommen
 *
 *-----------------------------------------------------------------------*/


_BOOL Olga_SetHandler(tHandlerType type, tHandleProc proc);

/*-----------------------------------------------------------------------
 *
 * Einen Handler wieder entfernen
 * Es wird nur ein passender Handler deinstalliert. Damit ist es moeglich, einen
 * Handler mehrfach zu installieren. ei der Deinstallation muessen gleich viele
 * Deinstallationen durchgefuehrt werden, wie Installationen, bis der Handler
 * wirklich weg ist.
 *
 *-----------------------------------------------------------------------*/
_VOID Olga_RemoveHandler(tHandlerType type, tHandleProc proc);


/*-----------------------------------------------------------------------
 *
 * Ein Dokument wird geoeffnet
 *
 *-----------------------------------------------------------------------*/
_VOID Olga_OpenDoc(_UWORD GroupId);


/*-----------------------------------------------------------------------
 *
 * Ein Dokument wird geschlossen
 *
 *-----------------------------------------------------------------------*/
_VOID Olga_CloseDoc(_UWORD GroupId);


/*-----------------------------------------------------------------------
 *
 * Ein Dokument soll ueberwacht werden (Client)
 *
 *-----------------------------------------------------------------------*/
_VOID Olga_Link(CONST _UBYTE *name, _UWORD GroupId);


/*-----------------------------------------------------------------------
 *
 * Die Ueberwachung kann beendet werden
 *
 *-----------------------------------------------------------------------*/
_VOID Olga_Unlink(CONST _UBYTE *name, _UWORD GroupId);


/*-----------------------------------------------------------------------
 *
 * Eine Datei wurde gespeichert, Mitteilung an OLGA versenden
 * Der Name darf fluechtig sein, zur Uebergabe wird eine Kopie erstellt, die
 * automatisch von Msg wieder freigegeben wird.
 *
 *-----------------------------------------------------------------------*/
_VOID Olga_Update(CONST _UBYTE *name);

/*-----------------------------------------------------------------------
 *
 * Server fuer Dateinamen starten
 *
 *-----------------------------------------------------------------------*/
_BOOL Olga_Start(CONST _UBYTE *filename);

#endif /* __CAT_MTOLGA_H__ */
