/*****************************************************************************
 *	TOS/SOUND.C
 *****************************************************************************/

/*
 * Diese Datei ist Teil der GEM-Library von Thorsten Otto,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */


#include <sound.h>
#include <termproc.h>
#include <ro_mem.h>
#include <debug.h>
#include <mtappl.h>
#include <tos.h>
#include <protokol.h>
#include <vaproto.h>
#include <portvdi.h>
#include <olga.h>
#include <confvars.h>
#include <routines.h>

typedef struct _sound SOUND;

struct _sound {
	SOUND *next;
	_UBYTE *event;
	_WORD type;
	union {
		_WORD nr;
		_UBYTE *filename;
	} data;
};

LOCAL SOUND *sounds;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#define BEGIN   0x80
#define CHANNEL 0x81
#define PAUSE   0x82     /* followed by time in 1/50 seconds */
#define TERMI   0xFF     /* Sound        */
#define NATE    0x00     /*   terminator */
enum REGS { R0, R1, R2, R3, R4, R5, R6, R7, R8, R9, R10, R11, R12, R13 };
/* 14 parameters for the registers 0-13 and */
/* tone variations: channel, start, +/-step, end */

LOCAL _UBYTE bong1[] = {
	R8, 0x10,     /* VolA = 0x10 */
	R9, 0x18,     /* VolB = 0x18 */
	R10, 0x00,    /* VolC = 0x00 */
	R11, 0x00,    /* EnvPer = 0x2000 */
	R12, 0x20,
	R13, 0x09,    /* EnvForm */
	R7,  0xf8,
	R0, 0x88,     /* PerA = 0x0e8 */
	R1, 0x00,
	R2, 0x20,     /* PerB = 0x220 */
	R3, 0x02,
	PAUSE, 0x28,  /* 0.8 second delay */
	R8, 0x00,     /* VolA = 0 */
	R9, 0x00,     /* VolB = 0 */
	TERMI,NATE
};
LOCAL _UBYTE bong2[] = {
    0x08, 0x10, 0x09, 0x18, 0x0a, 0x00, 0x0b, 0x00, 0x0c, 0x20, 0x0d, 0x09,
    0x07, 0xf8, 0x00, 0xe8, 0x01, 0x01, 0x02, 0x20, 0x03, 0x02, 0x82, 0x28,
    0x08, 0x00, 0x09, 0x00,
	TERMI,NATE
};
LOCAL _UBYTE bong3[] = {
    0x08, 0x10, 0x09, 0x18, 0x0a, 0x00, 0x0b, 0x00, 0x0c, 0x20, 0x0d, 0x09,
    0x07, 0xf8, 0x00, 0xe8, 0x01, 0x03, 0x02, 0x20, 0x03, 0x01, 0x82, 0x28,
    0x08, 0x00, 0x09, 0x00,
	TERMI,NATE
};
LOCAL _UBYTE bong4[] = {
    0x08, 0x10, 0x09, 0x18, 0x0a, 0x00, 0x0b, 0x00, 0x0c, 0x20, 0x0d, 0x09,
    0x07, 0xf8, 0x00, 0x68, 0x01, 0x00, 0x02, 0xf0, 0x03, 0x00, 0x82, 0x28,
    0x08, 0x00, 0x09, 0x00,
	TERMI,NATE
};
LOCAL _UBYTE bong5[] = {
    0x08, 0x10, 0x09, 0x18, 0x0a, 0x00, 0x0b, 0x00, 0x0c, 0x20, 0x0d, 0x09,
    0x07, 0xf8, 0x00, 0x28, 0x01, 0x00, 0x02, 0x80, 0x03, 0x00, 0x82, 0x28,
    0x08, 0x00, 0x09, 0x00,
	TERMI,NATE
};
LOCAL _UBYTE bong6[] = {
    0x08, 0x10, 0x09, 0x18, 0x0a, 0x00, 0x0b, 0x00, 0x0c, 0x20, 0x0d, 0x09,
    0x07, 0xf8, 0x00, 0x28, 0x01, 0x04, 0x02, 0x80, 0x03, 0x06, 0x82, 0x28,
    0x08, 0x00, 0x09, 0x00,
	TERMI,NATE
};
LOCAL _UBYTE bong7[] = {
	0x01, 0x01, 0x00, 0x5E, 0x08, 0x1F, 0x03, 0x00, 0x02, 0x00, 0x09, 0x00, 0x05, 0x00, 0x04, 0x00,
	0x0A, 0x00, 0x06, 0x00, 0x07, 0xFE, 0x0D, 0x01, 0x0C, 0x3C, 0x0B, 0xAF,
	TERMI,NATE
};

LOCAL _UBYTE bounce3[] = {
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,27,
       R7,248, R8,16, R9,16, R10,16, R11,35, R12,95, R13,0,
       BEGIN,255, CHANNEL,3,-1,116,
       PAUSE,255,
       TERMI,NATE };
LOCAL _UBYTE bounce4[] = {
       R0,0, R1,216, R2,4, R3,219, R4,5, R5,0, R6,0,
       R7,254, R8,16, R9,0, R10,0, R11,223, R12,118, R13,0,
       BEGIN,0, CHANNEL,0,22,0,
       TERMI,NATE };
LOCAL _UBYTE bounce5[] = {
       R0,82, R1,2, R2,251, R3,13, R4,34, R5,0, R6,0,
       R7,248, R8,16, R9,0, R10,0, R11,0, R12,86, R13,0,
       BEGIN,0, CHANNEL,0,11,0,
       TERMI,NATE };
LOCAL _UBYTE bounce6[] = {
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,0,
       R7,252, R8,16, R9,0, R10,0, R11,20, R12,70, R13,0,
       BEGIN,0, CHANNEL,1,1,255,
       TERMI,NATE };
LOCAL _UBYTE tingeling[] = {
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,23,
       R7,248, R8,16, R9,16, R10,16, R11,130, R12,62, R13,0,
       BEGIN,255, CHANNEL,0,-106,116,
       PAUSE,255,
       TERMI,NATE };
LOCAL _UBYTE tingeling2[] = {
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,0,
       R7,254, R8,16, R9,0, R10,0, R11,207, R12,88, R13,0,
       BEGIN,0, CHANNEL,0,41,0,
       TERMI,NATE };
LOCAL _UBYTE siren2[] = {
       R0,0, R1,1, R2,0, R3,0, R4,0, R5,0, R6,0,
       R7,8, R8,15, R9,0, R10,0, R11,0, R12,0, R13,0, /* 0,10,??? */
       BEGIN,1, CHANNEL,0,10,0,
       TERMI,NATE };
LOCAL _UBYTE chopper[] = {
       R0,176, R1,0, R2,238, R3,1, R4,119, R5,12, R6,26,
       R7,227, R8,16, R9,11, R10,14, R11,111, R12,2, R13,12,
       TERMI,NATE };
LOCAL _UBYTE surf[] = {
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,31,
       R7,199, R8,16, R9,16, R10,16, R11,0, R12,32, R13,14,
       TERMI,NATE };
LOCAL _UBYTE ting2[] = {
       R0,200, R1,0, R2,201, R3,0, R4,100, R5,0, R6,0,
       R7,248, R8,16, R9,16, R10,16, R11,0, R12,20, R13,0,
       TERMI,NATE };
LOCAL _UBYTE ding[] = {
       R0,64, R1,1, R2,56, R3,1, R4,0, R5,0, R6,0,
       R7,252, R8,16, R9,16, R10,0, R11,20, R12,20, R13,0,
       TERMI,NATE };
LOCAL _UBYTE ploink[] = {
       R0,2, R1,15, R2,2, R3,15, R4,0, R5,0, R6,0,
       R7,252, R8,16, R9,16, R10,0, R11,20, R12,20, R13,10,
       BEGIN,240, CHANNEL,1,-16,0,
       PAUSE,10,
       R0,2, R1,15, R2,2, R3,15, R4,0, R5,0, R6,0,
       R7,252, R8,0, R9,0, R10,0, R11,20, R12,20, R13,10,
       TERMI,NATE };
LOCAL _UBYTE toing[] = {
       R0,140, R1,1, R2,216, R3,4, R4,219, R5,5, R6,0,
       R7,254, R8,16, R9,0, R10,0, R11,207, R12,88, R13,0,
       BEGIN,0, CHANNEL,0,41,0,
       TERMI,NATE };
LOCAL _UBYTE bell[] = {
       R0,64, R1,0, R2,120, R3,0, R4,0, R5,0, R6,0,
       R7,252, R8,16, R9,16, R10,0, R11,20, R12,20, R13,0,
       TERMI,NATE };
LOCAL _UBYTE gong[] = {
       R0,1, R1,5, R2,0, R3,5, R4,2, R5,5, R6,0,
       R7,248, R8,16, R9,16, R10,16, R11,0, R12,20, R13,1,
       TERMI,NATE };
LOCAL _UBYTE gong2[] = {
       R0,210, R1,3, R2,232, R3,3, R4,209, R5,7, R6,0,
       R7,248, R8,16, R9,16, R10,16, R11,111, R12,163, R13,0,
       TERMI,NATE };
LOCAL _UBYTE gong3[] = {
       R0,0, R1,13, R2,0, R3,12, R4,0, R5,15, R6,0,
       R7,248, R8,16, R9,16, R10,16, R11,200, R12,200, R13,2,
       PAUSE,250,
       TERMI,NATE };
LOCAL _UBYTE dingdong[] = {
	0x01, 0x01,
	0x00, 0x1C,
	0x08, 0x1F, 
	0x06, 0x00,
	0x0D, 0x01,
	0x0C, 0x3C,
	0x0B, 0xAF,
	0x07, 0xFE,
	0x82, 0x28,
	0x01, 0x01,
	0x00, 0x5E,
	0x08, 0x1F,
	0x07, 0xFE,
	0x06, 0x00,
	0x0D, 0x01,
	0x0C, 0x3C,
	0x0B, 0xAF,
	0x07, 0xFE,
	TERMI,NATE
};
LOCAL _UBYTE pieuw[] = {
       R0,1, R1,0, R2,0, R3,0, R4,0, R5,0, R6,0,
       R7,254, R8,16, R9,0, R10,0, R11,0, R12,35, R13,1,
       BEGIN,50, CHANNEL,0,1,100,
       TERMI,NATE };
LOCAL _UBYTE poing[] = {
       R0,1, R1,0, R2,0, R3,0, R4,0, R5,0, R6,0,
       R7,254, R8,16, R9,0, R10,0, R11,0, R12,35, R13,1,
       BEGIN,100, CHANNEL,0,-1,50,
       TERMI,NATE };
LOCAL _UBYTE zap[] = {
       R0,0, R1,16, R2,0, R3,0, R4,0, R5,0, R6,0,
       R7,252, R8,15, R9,0, R10,0, R11,20, R12,0, R13,4,
       BEGIN,0, CHANNEL,1,1,15,
       PAUSE,1,
       R0,0, R1,16, R2,0, R3,0, R4,0, R5,0, R6,0,
       R7,252, R8,0, R9,0, R10,0, R11,20, R12,0, R13,4,
       TERMI,NATE };
LOCAL _UBYTE dlink[] = {
       R0,8, R1,2, R2,12, R3,4, R4,0, R5,0, R6,0,
       R7,252, R8,16, R9,16, R10,0, R11,20, R12,20, R13,9,
       BEGIN,200, CHANNEL,0,-20,0,
       PAUSE,1,
       R0,8, R1,2, R2,12, R3,4, R4,0, R5,0, R6,0,
       R7,252, R8,0, R9,0, R10,0, R11,20, R12,20, R13,9,
       TERMI,NATE };
LOCAL _UBYTE shot[] = {
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,15,
       R7,199, R8,16, R9,16, R10,16, R11,0, R12,16, R13,0,
       PAUSE,25,
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,15,
       R7,199, R8,0, R9,0, R10,0, R11,0, R12,16, R13,0,
       TERMI,NATE };
LOCAL _UBYTE explosion1[] = {
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,31,
       R7,199, R8,16, R9,16, R10,16, R11,0, R12,50, R13,9,
       TERMI,NATE };
LOCAL _UBYTE explosion2[] = {
       R0,0, R1,0, R2,100, R3,0, R4,200, R5,0, R6,31,
       R7,198, R8,16, R9,16, R10,16, R11,207, R12,88, R13,0,
       BEGIN,255, CHANNEL,6,0,0,
       TERMI,NATE };
LOCAL _UBYTE laser[] = {
       R0,100, R1,0, R2,200, R3,0, R4,50, R5,0, R6,31,
       R7,220, R8,16, R9,0, R10,16, R11,127, R12,37, R13,0,
       BEGIN,0, CHANNEL,0,137,200,
       PAUSE,128,
       TERMI,NATE };
LOCAL _UBYTE fft[] = {
       R0,42, R1,2, R2,88, R3,4, R4,164, R5,8, R6,0,
       R7,199, R8,16, R9,16, R10,16, R11,106, R12,10, R13,4,
       BEGIN,124, CHANNEL,4,54,164,
       PAUSE,1,
       R8,0, R9,0, R10,0, R11,0, R12,16, R13,9,
       TERMI,NATE };
LOCAL _UBYTE steam[] = {
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,10,
       R7,199, R8,16, R9,16, R10,16, R11,0, R12,80, R13,0,
       PAUSE,20,
       R0,0, R1,0, R2,0, R3,0, R4,0, R5,0, R6,10,
       R7,255, R8,0, R9,0, R10,0, R11,0, R12,80, R13,100,
       TERMI,NATE };
LOCAL _UBYTE thrill[] = {
       R0,86, R1,0, R2,86, R3,0, R4,0, R5,0, R6,0,
       R7,252, R8,16, R9,15, R10,0, R11,50, R12,1, R13,10,
       BEGIN,16, CHANNEL,9,-1,0,
       PAUSE,0,
       R0,86, R1,0, R2,86, R3,0, R4,0, R5,0, R6,0,
       R7,252, R8,0, R9,15, R10,0, R11,50, R12,1, R13,10,
       TERMI,NATE };
LOCAL _UBYTE jingle[] = {
       R0,100, R1,4, R2,101, R3,4, R4,0, R5,0, R6,0,
       R7,252, R8,15, R9,15, R10,0, R11,0, R12,30, R13,0,
       PAUSE,5,
       R0,100, R1,3, R2,101, R3,3, R4,0, R5,0, R6,0,
       R7,252, R8,15, R9,15, R10,0, R11,0, R12,30, R13,0,
       PAUSE,5,
       R0,100, R1,2, R2,101, R3,2, R4,0, R5,0, R6,0,
       R7,252, R8,15, R9,15, R10,0, R11,0, R12,30, R13,0,
       PAUSE,5,
       R0,100, R1,1, R2,101, R3,1, R4,0, R5,0, R6,0,
       R7,252, R8,15, R9,15, R10,0, R11,0, R12,30, R13,0,
       PAUSE,5,
       R0,100, R1,0, R2,101, R3,0, R4,0, R5,0, R6,0,
       R7,252, R8,16, R9,16, R10,0, R11,0, R12,30, R13,0,
       TERMI,NATE };
LOCAL _UBYTE systembeep[] = {
       R0,52, R1,0, R2,0, R3,0, R4,0, R5,0, R6,0,
       R7,254, R8,16, R9, 0, R10,0, R11,0, R12,16, R13,9,
       TERMI,NATE };
LOCAL _UBYTE klonk[] = {
	0x00, 0x01, 0x01, 0x05, 0x02, 0x00, 0x03, 0x05, 0x04, 0x02, 0x05, 0x05, 0x07, 0xF8, 0x08, 0x10,
	0x09, 0x10, 0x0A, 0x10, 0x0B, 0x00, 0x0C, 0x14, 0x0D, 0x01, 0xFF, 0x00
};
LOCAL _UBYTE bumm[] = {
	0x00, 0x00, 0x01, 0x00, 0x02, 0x00, 0x03, 0x00, 0x04, 0x00, 0x05, 0x00, 0x06, 0x1F, 0x07, 0xC7,
	0x08, 0x10, 0x09, 0x10, 0x0A, 0x10, 0x0B, 0x00, 0x0C, 0x32, 0x0D, 0x09, 0xFF, 0x00
};
LOCAL _UBYTE dudel[] = {
	0x08, 0x00, 0x09, 0x00, 0x0A, 0x00, 0x0B, 0x01, 0x0C, 0x00, 0x0D, 0x09, 0x07, 0xF8, 0x00, 0x8D,
	0x01, 0x00, 0x08, 0x0F, 0x82, 0x01, 0x00, 0x5E, 0x82, 0x01, 0x00, 0x8D, 0x82, 0x01, 0x00, 0xD3,
	0x82, 0x01, 0x00, 0x8D, 0x82, 0x01, 0x00, 0x5E, 0x82, 0x01, 0x00, 0x8D, 0x82, 0x01, 0x08, 0x00,
	0x82, 0x01, 0x07, 0xFF, 0x00, 0x09, 0x00, 0x0A, 0x00, 0x00, 0x34
};
LOCAL _UBYTE tusch[] = {
	0x00, 0x41,
	0x01, 0x01,
	0x02, 0x40,
	0x03, 0x01,
	0x04, 0x42,
	0x05, 0x01,
	0x07, 0xF8,
	0x08, 0x10,
	0x09, 0x10,
	0x0A, 0x10,
	0x0B, 0xFF,
	0x0C, 0x46,
	0x0D, 0x01,
	0xC8, 0x28,
	0x02, 0xF2,
	0x03, 0x00,
	0x04, 0x7F,
	0x05, 0x01,
	0x0C, 0x64,
	0x0D, 0x01,
	0xFF, 0x00 
};

LOCAL _VOID *bongArray[] = {
	bong1,
	bong2,
	bong3,
	bong4,
	bong5,
	bong6,
	bong7,
	bounce3,
	bounce4,
	bounce5,
	bounce6,
	tingeling,
	tingeling2,
	ting2,
	ding,
	ploink,
	toing,
	bell,
	gong,
	gong2,
	gong3,
	pieuw,
	poing,
	zap,
	dlink,
	jingle,
	shot,
	explosion1,
	explosion2,
	laser,
	fft,
	steam,
	thrill,
	siren2,
	chopper,
	surf,
	dingdong,
	klonk,
	bumm,
	dudel,
	tusch,
	systembeep,
};
#define maxBing ((_WORD)(sizeof(bongArray) / sizeof(bongArray[0])))

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _BOOL Sound_Init(_VOID)
{
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Sound_Stop(_VOID)
{
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID Sound_Exit(_VOID)
{
	SOUND *sound, *next;
	
	sound = sounds;
	while (sound != NULL)
	{
		next = sound->next;
		SFREE(sound);
		sound = next;
	}
	sounds = NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Sound_Unregister(CONST _UBYTE *event)
{
	SOUND **sound, *s;
	
	sound = &sounds;
	while ((s = *sound) != NULL)
	{
		if (strCmp(event, s->event) == 0)
		{
			*sound = s->next;
			SFREE(s);
			return;
		}
		sound = &(s->next);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Sound_Register(CONST _UBYTE *event, _WORD type, CONST _UBYTE *data)
{
	size_t len;
	SOUND *sound;
	
	Sound_Unregister(event);
	len = sizeof(SOUND) + strLen(event) + 1;
	if (type == SOUND_FILENAME)
		len += strLen(data) + 1;
	sound = MALLOC(len, "Sound_Register");
	if (sound == NULL)
		return FALSE;
	sound->type = type;
	sound->event = (_UBYTE *)(sound + 1);
	strCpy(sound->event, event);
	sound->next = sounds;
	sounds = sound;
	switch (type)
	{
	case SOUND_INTERN:
		sound->data.nr = atoi(data);
		if (sound->data.nr == 0)
			sound->type = SOUND_DEFAULT;
		break;
	case SOUND_FILENAME:
		sound->data.filename = sound->event + strLen(sound->event) + 1;
		strCpy(sound->data.filename, data);
		break;
	}
	InstallTermproc(Sound_Exit);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Sound_Bing(_VOID)
{
	Pling();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Sound_Play_Intern(_WORD type)
{
	if (type > 0 && type <= maxBing && !magicMac)
	{
		Dosound(bongArray[type-1]);
	} else
	{
		Sound_Bing();
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Sound_Play_Filename(CONST _UBYTE *filename,_WORD avModus)
{
	_WORD apid;
	_BOOL ret=FALSE;
	_UBYTE filenameQ[PATH_MAX];
	_WORD quote_id;

	if(Protokoll_Init() && avModus && (apid = FindAVServerFor(AV_STARTPROG))>= 0) 
	{
		if(avModus==2)
			quote_id=apid;
		else
			quote_id=DONT_QUOTE;
		VaProto_Quote(filenameQ,PATH_MAX,filename,quote_id);
		Protokoll_SendPath(AV_STARTPROG,apid, filenameQ, NULL);
		ret= TRUE;
	}
	else
	{
		apid = appl_locate_args(getenv("GEMJINGPATH"), "-p");
		if (apid < 0)
			apid = appl_locate("GEMJING", FALSE);
		if (apid < 0)
			apid = appl_locate("PAULA", FALSE);
		if (apid >= 0)
		{
			VaProto_Quote(filenameQ,PATH_MAX,filename,apid);
			Protokoll_SendPath(VA_START,apid, filenameQ, NULL);
			ret= TRUE;
		}
	}
	if(!ret)
		ret=Olga_Start(filename);
	return ret;
}
/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Sound_Flash(_VOID)
{
	_WORD pxy[8];
	MFDB screen;
	
	Wind_Update(BEG_UPDATE);
	pxy[0] = 0;
	pxy[1] = 0;
	pxy[2] = MaxWidth - 1;
	pxy[3] = MaxHeight - 1;
	pxy[4] = 0;
	pxy[5] = 0;
	pxy[6] = MaxWidth - 1;
	pxy[7] = MaxHeight - 1;
	screen.fd_addr = 0;
	MouseOff();
	vs_clip(VDIHandle, 0, pxy);
	vro_cpyfm(VDIHandle, D_INVERT, pxy, &screen, &screen);
	evnt_timer(500, 0);
	vro_cpyfm(VDIHandle, D_INVERT, pxy, &screen, &screen);
	MouseOn();
	Wind_Update(END_UPDATE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Sound_Play(CONST _UBYTE *event,_WORD avModus)	
{
	SOUND *sound;
	
	for (sound = sounds; sound != NULL; sound = sound->next)
	{
		if (strCmp(event, sound->event) == 0)
			break;
	}
	if (sound == NULL)
		return FALSE;
	switch (sound->type)
	{
	case SOUND_NONE:
		break;
	case SOUND_INTERN:
		Sound_Play_Intern(sound->data.nr);
		break;
	case SOUND_DEFAULT:
		Sound_Bing();
		break;
	case SOUND_FLASH:     /* naja, kein Sound :-) */
		Sound_Flash();
		break;
	case SOUND_FILENAME:
		if (!Sound_Play_Filename(sound->data.filename,avModus))	
			Sound_Bing();
		break;
	}
	return TRUE;
}
