/*****************************************************************************
 *	TOS/SYSTIME.C
 *	system dependant time functions
 *****************************************************************************/

#include <portab.h>
#include <tos.h>
#include <ro_time.h>
#undef _VOID
#include <time.h>
#ifdef __GNUC__
#include <sys/time.h>
#endif
#define _VOID void

/*** ---------------------------------------------------------------------- ***/

GLOBAL SYSTIME T_Tzadjust(SYSTIME ltime, _BOOL to_local)
{
#if defined(__GNUC__) && defined(__MINGW32__)
	if (ltime != 0)
	{
		if (to_local)
			ltime += _timezone;
		else
			ltime -= _timezone;
	}
#elif defined(__GNUC__)
	struct timezone tz;
	struct timeval tv;
	
	gettimeofday(&tv, &tz);
	if (ltime != 0)
	{
		if (to_local)
		{
			ltime += tz.tz_minuteswest;
			if (tz.tz_dsttime)
				ltime += SECS_PER_HOUR;
		} else
		{
			ltime -= tz.tz_minuteswest;
			if (tz.tz_dsttime)
				ltime -= SECS_PER_HOUR;
		}
	}
#else
	if (ltime != 0)
	{
		if (to_local)
			ltime += timezone;
		else
			ltime -= timezone;
	}
#endif
	return ltime;
}
	
/*** ---------------------------------------------------------------------- ***/

GLOBAL SYSTIME T_Systime(_VOID)
{
	_UWORD date, time;
	
	date = Tgetdate();
	time = Tgettime();
	return T_DosToSystime(date, time);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID T_Set_Systime(SYSTIME t)
{
	_UWORD date, time;
	
	T_SysToDostime(t, &date, &time);
	Tsetdate(date);
	Tsettime(time);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UBYTE *T_SystimeToString (_UBYTE * string, _UWORD strSize, _UWORD mode, SYSTIME tt)
{
	mode |= TIME_TS_DEFAULT_MODE;
	return T_TimeToString(string, strSize, mode, ':', tt);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UBYTE *T_SysdateToString (_UBYTE * string, _UWORD strSize, _UWORD mode, SYSTIME tt)
{
	mode |= DATE_TS_DEFAULT_MODE;
	return T_DateToString(string, strSize, mode, '.', tt);
}
