#include <xbra.h>
#include <ro_mem.h>
#include <tos.h>
#include <debug.h>
#include <atari.h>

typedef struct xbra
{
	_ULONG		xbra_magic;
	_ULONG		xbra_id;
	struct xbra	*next;
	_WORD		jump;
	_VOID		(*thisfunc)(_VOID);
} PACKED XBRA;

#define XBRA_MAGIC	0x58425241UL /* "XBRA" */
#define JMP_OPCODE	0x4EF9

#ifdef __PUREC__
static _ULONG get_sp(_VOID) 0x200f;			/* move.l sp,d0 */
static _VOID  set_sp(_ULONG sp) 0x2e40;		/* move.l d0,sp */
static _VOID  get_adr0(_VOID) 0x41fa;		
static _VOID *get_adr1(_VOID) 0x0024;
static _VOID *get_adr2(_VOID) 0x002c;
#define get_adr() (get_adr0(), get_adr1())	/* lea buserror(pc),a0 */
                                            /* Der Offset ist abhaengig von der Schleife!!! */
#define get_adr2() (get_adr0(), get_adr2())	/* lea buserror(pc),a0 */
#define XBRA_OK
#endif


#ifdef __GNUC__
#ifdef atarist
#define get_sp() \
__extension__ \
({ \
	register long retvalue __asm__("d0"); \
	\
	__asm__ volatile \
	("\
		movl	sp,%0 " \
	: "=r"(retvalue)	/* outputs */ \
	:					/* no inputs */ \
						/* clobbered regs */ \
	); \
	retvalue; \
})

#define set_sp(sp) \
__extension__ \
({ \
	register long _sp = sp; \
	\
	__asm__ volatile \
	("\
		movl	%0,sp " \
	: 					/* no outputs */ \
	: "r"(_sp)			/* inputs */ \
	: "sp"				/* clobbered regs */ \
	  AND_MEMORY \
	); \
})
#define XBRA_OK
#endif
#endif /* __GNUC__ */


#ifdef XBRA_OK


_ULONG Id,Vec;
_VOID (*Func)(_VOID);

_LONG xbra_linkSup(_VOID)
{
	_ULONG new_sp;
	_BOOL found;
	XBRA  *xbra, *new;
	_VOID (*old_buserror)(_VOID);	

	new = NULL;
	if (Func != FUNK_NULL)
	{
		new = NEW(XBRA, "xbra_link");
		if (new == NULL)
			return FALSE;
	}
	found = FALSE;
	new_sp = get_sp();
	old_buserror = Setexc(2,(void(*)())-1);
	
#ifdef __PUREC__
	Setexc(2,(void(*)())get_adr());
#endif
#ifdef __GNUC__
	Setexc(2,&&buserror);
#endif
	xbra = (XBRA *)(Vec - 8);
	for (;;)
	{
		xbra = (XBRA *)((_ULONG)xbra->next - 12);
		if (xbra->xbra_magic != XBRA_MAGIC)
			break;
		if (xbra->xbra_id == Id)
		{
			found = TRUE;
			break;
		}
	}
	
#ifdef __GNUC__
buserror:
#endif
	if (get_sp() != new_sp)
	{
		/* wir sind auf einen Bus-Error gelaufen */
		
		set_sp(new_sp);
		found = FALSE;
	}

	Setexc(2,old_buserror);
	
	if (!found && Func != FUNK_NULL)
	{
		new->xbra_magic = XBRA_MAGIC;
		new->xbra_id = Id;
		new->next = *((XBRA **)Vec);
		new->jump = JMP_OPCODE;
		new->thisfunc = Func;
		*((_WORD **)Vec) = &new->jump;
		found = TRUE;
	}
	if (!found && Func != FUNK_NULL)
		OFREE(new);
	return found;
}
_BOOL xbra_link(_ULONG id, _ULONG vec, _VOID (*func)(_VOID))
{
	Id=id;
	Vec=vec;
	Func=func;
	return (_BOOL)Supexec(xbra_linkSup);
}


_LONG xbra_unlinkSup(_VOID)
{
	_ULONG new_sp;
	_BOOL found;
	XBRA  *xbra, *last;
	_VOID *old_buserror;
	
	found = FALSE;
	new_sp = get_sp();
	old_buserror = Setexc(2,(void(*)())-1);
	
#ifdef __PUREC__
	Setexc(2,(void(*)())get_adr2());
#endif
#ifdef __GNUC__
	Setexc(2,&&buserror);
#endif
	xbra = (XBRA *)(Vec - 8);
	for (;;)
	{
		last = xbra;
		xbra = (XBRA *)((_ULONG)xbra->next - 12);
		if (xbra->xbra_magic != XBRA_MAGIC)
			break;
		if (xbra->xbra_id == Id)
		{
			found = TRUE;
			last->next = xbra->next;
			break;
		}
	}

#ifdef __GNUC__
buserror:
#endif
	if (get_sp() != new_sp)
	{
		/* wir sind auf einen Bus-Error gelaufen */
		
		set_sp(new_sp);
		found = FALSE;
	}

	*((_VOID **)8) = old_buserror;
	
	if (found)
		OFREE(xbra);
	return found;
}
_BOOL xbra_unlink(_ULONG id, _ULONG vec )
{
	Id=id;
	Vec=vec;
	return (_BOOL)Supexec(xbra_unlinkSup);
}

#else

_BOOL xbra_link(_ULONG id, _ULONG vec, _VOID (*func)(_VOID))
{
	if (func == FUNK_NULL)
	{
		if (id == 0x53544e58L && vec == 0x88) /* STonX */
			return FALSE;
		if (id == 0x46536d70L && vec == 0x88) /* FSmp */
			return TRUE;
	}
	return FALSE;
}
#endif /* XBRA_OK */
