/*****************************************************************************
 *		TOS/XUFSL.H
 *****************************************************************************/

#ifndef __XUFSL_H__
#define __XUFSL_H__

/**************************************************************************/
/* Kennung des xUfsl                                                      */
/**************************************************************************/
#define xUFS_ID          0x78554653L  /* 'xUFS' */
#define UFLS_ID          0x5546534CL  /* 'UFLS' */

/**************************************************************************/
/* Rckgabewerte des xUfsl                                                */
/**************************************************************************/
#define UFS_RES_ERROR    -7       /* Auflsung kleiner als 640*400 Punkte */
#define UFS_NO_HANDLE    -6       /* Kein Handle frei                     */
#define UFS_NO_WINDOW    -5       /* Kein Fenster frei                    */
#define UFS_NO_FONTS     -4       /* Keine Fonts geladen                  */
#define UFS_NO_FONTSIZE  -3       /* Fontgre nicht identifizierbar      */
#define UFS_ACTIVE       -2       /* Fontselektor ist bereits aktiv       */
#define UFS_ERROR        -1       /* Speichermangel o..                  */
#define UFS_STOP          0       /* <Abbruch> gewhlt                    */
#define UFS_OK            1       /* <Ok> gewhlt                         */

/**************************************************************************/
/* Dialogtypen --> dialtyp, s.u.                                          */
/**************************************************************************/
#define UFS_DIALOG        0       /* Modaler Dialog                       */
#define UFS_WINDOW        1       /* Unmodaler Dialog                     */

/**************************************************************************/
/* Flags fr xUFSL --> flags, s.u.                                        */
/**************************************************************************/
#define UFS_SIZE_CHANGE   0x00000001  /* Schriftgre nderbar?           */
#define UFS_WIDTH_CHANGE  0x00000002  /* Schriftbreite nderbar?          */
#define UFS_ATTR_CHANGE   0x00000004  /* Schriftattribute whlbar?        */
#define UFS_SKEW_CHANGE   0x00000008  /* Schriftneigung nderbar?         */
#define UFS_COLOR_CHANGE  0x00000010  /* Schriftfarbe nderbar?           */
#define UFS_SIZE_DEFAULT  0x00000020  /* Schriftgre "DEFAULT" erlaubt?  */
#define UFS_ATTR_INVERS   0x00000040  /* Sonderattribut "invers" whlbar? */
#define UFS_KERNING_INFO  0x00000080  /* Kerning-Informationen nderbar?  */
#define UFS_GDOS_FONTS    0x00000100  /* GDOS-Fonts erlaubt?              */
#define UFS_PROP_FONTS    0x00000200  /* Proportionalfonts erlaubt?       */
#define UFS_BITMAP_FONTS  0x00000400  /* Nur Bitmap-Fonts erlaubt?        */
#define UFS_VECTOR_FONTS  0x00000800  /* Nur Vektor-Fonts erlaubt?        */
#define UFS_WIND_DIALOG   0x00001000  /* Dialog im Fenster?               */
#define UFS_ERROR_MSG     0x00008000  /* Fehler als Alertbox anzeigen?    */
#define UFS_INDIRECT      0x80000000L /* Wird eine XUFSL_LIST verwendet?  */

/**************************************************************************/
/* UFSL-Initialisierungsfunktion, nur aus Kompatibilittsgrnden imple-   */
/* mentiert. Liefert immer SUCCESS.                                       */
/**************************************************************************/
typedef _WORD _CDECL (*UFSL_INIT)(_VOID);

/**************************************************************************/
/* Standard-Fontselektor                                                  */
/**************************************************************************/
typedef _WORD _CDECL (*UFSL)(
	_WORD   handle,          /* Handle der VDI-Workstation                */
	_WORD   fonts,           /* Gesamtzahl der geladenen Fonts            */
	_WORD   flag,            /* 0=Alle Fonts, 1=Nur unproportionale Fonts */
	CONST _UBYTE  *info,     /* Infotext                                  */
	_WORD   *fontid,         /* Gewhlte Font-ID                          */
	_WORD   *fontsize        /* Gewhlte Font-Gre                       */
);

/**************************************************************************/
/* Neuer Standard-Fontselektor ab UFSL 0.97                               */
/**************************************************************************/
typedef _WORD _CDECL (*NUFSL)(
	_WORD   handle,          /* Handle der VDI-Workstation                */
	_WORD   flag,            /* 0=Alle Fonts, 1=Nur unproportionale Fonts */
	CONST _UBYTE *info,      /* Infotext                                  */
	_WORD   *fontid,         /* Gewhlte Font-ID                          */
	_WORD   *fontsize        /* Gewhlte Font-Gre                       */
);

/**************************************************************************/
/* Benutzerdefinierte Hilfefunktion, sofern gewnscht. Nur wenn diese     */
/* ungleich NULL ist, wird der Hilfe-Knopf berhaupt aktiviert.           */
/**************************************************************************/
typedef _WORD _CDECL (*UFSL_HELP)(void);

/**************************************************************************/
/* Redraw-Funktion                                                        */
/* Die Redraw-Funktion wird NICHT bei folgenden Ereignissen aufgerufen:   */
/* WM_TOPPED, WM_NEWTOP, WM_CLOSED.                                       */
/**************************************************************************/
typedef void _CDECL (*UFSL_MESSAGE)(
	_UWORD event,            /* Event                                     */
	_WORD msgbuff[]          /* Message-Puffer                            */
);

/**************************************************************************/
/* Schriftbreiten-Informationsstruktur ('FontWidthInfo')                  */
/* pair   Pair-Kerning (0=nein, 1=ja): "Zusammenschieben" bestimmter      */
/*        Buchstabenpaare (z.B. "Fa", "Ve").                              */
/* track  Art des Track-Kernings (0=ohne, 1=normal, 2=eng, 3=sehr eng):   */
/*        Hiermit wird der generelle Abstand zwischen den Buchstaben be-  */
/*        stimmt.                                                         */
/* width  Schriftbreite (0..MAX_CHAR_WIDTH Punkte)                        */
/**************************************************************************/
#define MAX_CHAR_WIDTH  8191

/*  Schriftbreiten-Informationsstruktur,
 *  wird nur fr das Font-Protokoll (Nachricht XFONT_CHANGED) bentigt
 */
typedef _UWORD FW_INFO;

#define FW_INFO_PAIR(fw)  (((fw) >> 15) & 0x0001)
#define FW_INFO_TRACK(fw) (((fw) >> 13) & 0x0003)
#define FW_INFO_WIDTH(fw) (((fw)        & 0x1fff)

/**************************************************************************/
/* Bei gesetztem UFS_INDIRECT-Bit wird statt in info ein Zeiger auf eine  */
/* XUFSL_LIST bergeben. Die Anzahl der Eintrge dieser Liste mu im Pa-  */
/* rameter fontid bergeben werden.                                       */
/**************************************************************************/
typedef struct {
	_ULONG        flags;     /* Flags zur Funktionsberwachung            */
	_UBYTE        *entry;    /* Popup-Eintrag, mu eindeutig sein!        */
	_WORD         fontid;    /* Voreingestellte/gewhlte ID               */
	_WORD         fontsize;  /* Voreingestellte/gewhlte Gre            */
	FW_INFO       fontwidth; /* Voreingestellte/gewhlte Breite+Kerning   */
	_WORD         fontattr;  /* Voreingestellte/gewhlte Attribute        */
	_WORD         fontcol;   /* Voreingestellte/gewhlte Farbe            */
	_WORD         fontskew;  /* Voreingestellte/gewhlte Neigung          */
} XUFSL_LIST;

/**************************************************************************/
/* Extended-Fontselektor                                                  */
/**************************************************************************/
typedef _WORD _CDECL (*XUFSL)(
	_WORD         handle,    /* Handle der VDI-Workstation                */
	_ULONG        flags,     /* Flags zur Funktionsberwachung            */
	CONST _UBYTE  *title,    /* Fenstertitel, Default: Copyright-Info     */
	CONST _UBYTE  *info,     /* Fensterinfo, Default: 'Schriftauswahl'    */
	                         /* bzw. Zeiger auf eine XUFSL_LIST           */
	_WORD         smallid,   /* ID SMALL-Zeichensatz im xUFSL-Dialog      */
	_WORD         ibmid,     /* ID IBM-Zeichensatz im xUFSL-Dialog        */
	_WORD         *fontid,   /* Voreingestellte/gewhlte ID               */
	                         /* bzw. Anzahl der Eintrge der XUFSL_LIST   */
	_WORD         *fontsize, /* Voreingestellte/gewhlte Gre            */
	                         /* bzw. Aufsatzpunkt in der XUFSL_LIST       */
	FW_INFO       *fontwidth,/* Voreingestellte/gewhlte Breite und       */
	                         /* Kerning-Informationen                     */
	_WORD         *fontattr, /* Voreingestellte/gewhlte Attribute        */
	_WORD         *fontcol,  /* Voreingestellte/gewhlte Farbe            */
	_WORD         *fontskew  /* Voreingestellte/gewhlte Neigung          */
);

/**************************************************************************/
/* Eigentliche (x)UFSL-Struktur                                             */
/**************************************************************************/
typedef struct {
	_ULONG id;               /* Kennung des xUFSL (xUFS)                  */
	_UWORD version;          /* Version (BCD-Format)                      */
	_UWORD dialtyp;          /* Dialogtyp                                 */
	UFSL_INIT fontsel_init;  /* Initialisierungsfunktion (Dummy)          */
	UFSL fontsel_input;      /* Standard-Selektorfunktion                 */
	OBJECT *helpbutton;      /* Hilfe-Knopf (BOXTEXT), maximale           */
	                         /* Textlnge 9 Zeichen (OHNE Prfung!).      */
	UFSL_HELP helpfunc;      /* Eigene Hilfefunktion                      */
	/* Ab Version 0.91 */
	CONST _UBYTE *examplestr;/* Beispieltext fr Fontdarstellung          */
	/* Ab Version 0.96 */
	UFSL_MESSAGE msgfunc;    /* Redrawfunktion                            */
	/* Ab Version 0.97 */
	NUFSL fontsel_exinput;   /* Neuer Standard-Selektor                   */
	/* bei Kennung xUFS */
	XUFSL font_selexinput;   /* Extended-Selektorfunktion                 */
} xUFSL;

#endif /* __XUFSL_H__ */
