/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 2000, Dimitri Junker
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */



#define __CAT_BTREES_IMPLEMENTATION__
#include <btrees.h>
#include <ro_mem.h>


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL BTree_ResetBTree(BTree *bTree)									*/
/*	Setzt current auf root und berprft ob der Baum berhaupt existiert		*/
/*	Parameter:	bTree:	der Baum												*/
/*	Ergebnis:	0 falls bTree==NULL	(kein Baum)									*/		
/*				1 falls bTree!=NULL aber bTree->root=NULL	(leerer Baum)		*/
/*				2 falls bTree!=NULL und bTree->root!=NULL	(voller Baum)		*/
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _WORD BTree_ResetBTree(BTree *bTree)
{
	if (bTree != NULL)
	{
		if((bTree->current = bTree->root)==NULL)
			return 1;
		else
			return 2;
	}
	else
		return 0;
}

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL BTree_CreateBTree(BTree *bTree,bTcmpElem cmpElem)				*/
/*	initialisiert die Variablen													*/
/*	Parameter:	bTree:	der Baum												*/
/*				cmpElem:Callbackfunktion zum Vergleich von 2 Datas				*/
/*	Ergebnis:	FALSE falls bTree == NULL										*/		
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL BTree_CreateBTree(BTree *bTree,bTcmpElem cmpElem)
{
	if (bTree == NULL)
		return FALSE;
	bTree->root = NULL;
	bTree->current = NULL;
	bTree->first = NULL;
	bTree->last = NULL;
	bTree->cmpElem = cmpElem;
	return TRUE;
}

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID BTree_DeleteBTree(BTree *bTree, btrees_delete_proc delete)		*/
/*	lscht einen ganzen Baum													*/
/*	Parameter:	bTree:	der zu lschende Baum									*/
/*				delete:	Lschroutine fr data									*/
/*	Ergebnis:	keines															*/		
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID BTree_DeleteBTree(BTree *bTree, btrees_delete_proc delete)
{
	BTCarrier akt,nxt;
	_VOID *entry;
	if(!bTree)
		return;
	akt=bTree->first;
	while(akt)
	{
		nxt=akt->next;
		entry = akt->data;
		if (delete != FUNK_NULL)
			delete(entry);
		OFREE(akt);
		akt=nxt;
	}
	bTree->root = NULL;
	bTree->current = NULL;
	bTree->first = NULL;
	bTree->last = NULL;
}

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID *BTree_FindEntry(BTree *bTree,CONST _VOID *entry,_WORD *cmp)	*/
/* 	Durchsucht einen BTree nach einem Eintrag.									*/
/*	Faengt dabei bei der Wurzel an. Wird der Eintrag gefunden, zeigt 			*/
/*	'bTree.current' darauf und es wird cmp=0 zurck gegeben. Sonst wird			*/
/*	'bTree.current' auf das Element gesetzt an das das neue angehngt werden 	*/
/*	kann. Je nach Rckgabewert in cmp als smaller oder bigger					*/
/*	Parameter:	bTree:	der Baum												*/
/*				entry:	das zu suchende Element, mu nur soweit gefllt sein, 	*/
/*						da die Vergleichsfunktion funktioniert					*/
/*						cmp: fr Rckgabe										*/
/*	Ergebnis:	!=NULL:	data des gefundenen Eintrags; cmp=0						*/
/*				NULL:	nicht gefunden weitere info in cmp						*/
/*				cmp<0:	ist kleiner als current									*/
/*				cmp=0:	Baum ist leer											*/
/*				cmp>0:	ist grer als current									*/
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID *BTree_FindEntry(BTree *bTree,CONST _VOID *entry,_WORD *cmp)
{
	_VOID *currVar;
	*cmp=0;
	if(!BTree_ResetBTree(bTree))
		return NULL;
	while ((currVar = BTree_NextEntry(bTree,*cmp)) != NULL)
	{
		if ((*cmp=bTree->cmpElem(currVar,entry)) == 0)
			break;
	}
	return currVar;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL BTree_AppendEntry(BTree *bTree, CONST _VOID *entry,_WORD cmp)	*/
/*	fgt Eintrag an Baum an														*/
/*	Parameter:	bTree:	der Baum												*/
/*				entry:	das Einzufgende										*/
/*				cmp:	=0: Einfgeort noch suchen								*/
/*						!=0:An current anhngen, bei cmp<0 als smaller			*/
/*	Ergebnis:	erfolgreich?													*/		
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL BTree_AppendEntry(BTree *bTree, CONST _VOID *entry,_WORD cmp)
{
	BTCarrier p;
	
	if (bTree == NULL )
		return FALSE;
	if ((p = NEW(Carrier, "BTree_AppendEntry")) == NULL)
		return FALSE;
	if(bTree->root == NULL)
	{
		bTree->root=p;
		p->up=NULL;
		bTree->first=p;
	}
	else
	{
		if(!cmp)
		{
			BTree_FindEntry(bTree,entry,&cmp);
			if(!cmp)
				return FALSE;
		}
		if(cmp<0)
			bTree->current->smaller=p;
		else
			bTree->current->bigger=p;
		p->up=bTree->current;
		bTree->last->next=p;
	}
	p->smaller = NULL;
	p->bigger = NULL;
	p->data = NO_CONST(entry);
	p->next=NULL;
	bTree->current=p;
	bTree->last=p;
	return TRUE;
}

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL BTree_RemoveEntry(BTree *bTree, btrees_delete_proc delete)		*/
/*	lscht einen Eintrag (current) aus dem Baum. Der Knoten selbst wird nicht 	*/
/*	gelscht, denn in der unsortierten einfach verkettetetn Liste(evL) bleibt 	*/
/*	er, nur data wird gelscht. Dies ist sinnvoll solange wenig gelscht wird. 	*/
/*	Um den Knoten auch aus der evL zu lschen wre viel Suchen ntig, oder 		*/
/*	besser die Liste doppelt verketten. Ausnahme: wird das 1. Element gelscht, */
/*	so wird	auch der Knoten gelscht											*/
/*	Parameter:	bTree:	der Baum												*/
/*				delete: Lschroutine fr data									*/
/*	Ergebnis:	erfolgreich?													*/		
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL BTree_RemoveEntry(BTree *bTree, btrees_delete_proc delete)
{
	BTCarrier *ptr,del,nxtSml,*nxtSmlP;

	if (bTree == NULL ||
		bTree->root == NULL)
		return FALSE;
	if(bTree->current->up==NULL)	/*	ist es root?	*/
		ptr=&bTree->root;
	else
	{
		if(bTree->current->up->smaller==bTree->current)	/* welcher Nachfahre ist es?	*/
			ptr=&(bTree->current->up->smaller);
		else
			ptr=&(bTree->current->up->bigger);
	}
	if(bTree->current->smaller)
	{
		if(bTree->current->bigger)	/* Hat 2 Nachfahren ->nchst kleineres Element an die Stelle setzen*/
		{
			nxtSml=bTree->current->smaller;
			nxtSmlP=&bTree->current->smaller;
			while(nxtSml->bigger)
			{
				nxtSmlP=&nxtSml->bigger;
				nxtSml=nxtSml->bigger;
			}
			*nxtSmlP=nxtSml->smaller;		/*Aus alter Position lschen	*/
			if(nxtSml->smaller)
				nxtSml->smaller->up=nxtSml->up;
			*ptr=nxtSml;							/*Verketung der neuen Position bernehmen */
			nxtSml->smaller=bTree->current->smaller;
			if(nxtSml->smaller)	/* kann evtl. durch '*nxtSmlP=nxtSml->smaller' zu NULL geworden sein	*/
				nxtSml->smaller->up=nxtSml;
			nxtSml->bigger=bTree->current->bigger;	/* Ist nicht NULL!*/
			nxtSml->bigger->up=nxtSml;
			nxtSml->up=bTree->current->up;
		}
		else		/* Hat nur einen kleineren Nachfahren ->diesen an die Stelle setzen*/
		{
			*ptr=bTree->current->smaller;
			(*ptr)->up=bTree->current->up;
		}
	}
	else
	{
		if(bTree->current->bigger)	/* Hat nur einen greren Nachfahren ->diesen an die Stelle setzen*/
		{
			*ptr=bTree->current->bigger;
			(*ptr)->up=bTree->current->up;
		}
		else	/* Hat keine Nachkommen ->enfach lschen*/
			*ptr=NULL;
	}
	
				
	
	if (delete != FUNK_NULL)
	{
		delete(bTree->current->data);
		bTree->current->data=NULL;	/* so ist es auch erkennbar */
	}
	while(bTree->first && !bTree->first->data)
	{
		del=bTree->first;
		bTree->first=bTree->first->next;
		OFREE(del);
	}
	bTree->current=bTree->root;
	return TRUE;
}

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID *BTree_NextEntry(BTree *bTree,_WORD dir)						*/
/*	verschiebt current und gibt das Data des neuen zurck						*/
/*	Parameter:	bTree:	der Baum												*/
/*				dir:	<0: Verschieben zu smaller								*/
/*						=0: nicht verschieben									*/
/*						>0: Verschieben zu bigger								*/
/*	Ergebnis:	wenn gewnschter Nachfolger existiert dessen data sonnst NULL	*/		
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID *BTree_NextEntry(BTree *bTree,_WORD dir)
{
	BTCarrier nxt;
	if (bTree == NULL)
		return NULL;
	if(dir>0)
		nxt = bTree->current->bigger;
	else
	{
		if(!dir)
			nxt = bTree->current;
		else
			nxt = bTree->current->smaller;
	}
	if (nxt)
	{	
		bTree->current =nxt;
		return bTree->current->data;
	}
	else
		return NULL;
}

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL BTree_ScanEntriesUnSort(BTree *bTree, BTCondProc cond, CONST _VOID *info)	*/
/*	Geht die einfach verkettete Liste durch und macht irgedwas (cond)			*/
/*	Dabei wird jedesmal die Prozedur 'cond' aufgerufen und ihr der aktuelle		*/
/*	Eintrag uebergeben. Diese Prozedur kann dann entscheiden, ob weitergesucht	*/
/*	werden soll, indem sie FALSE (weiter) oder TRUE (gefunden, abbrechen)		*/
/*	zurueckgibt. Ihr kann, z.B. als Zeiger auf einen Vergleichswert, in 'info'	*/
/*	ein Pointer uebergeben werden.												*/
/*	Die an 'cond' uebergebene Procedure darf lokal sein!						*/
/*	Nebenbei werden auch gleich die gelschten Knoten aus der Liste entfernt 	*/
/*	und freigegeben																*/
/*	Parameter:	bTree:	der Baum												*/
/*				cond:	die fr jedes Element aufzurufende Funktion				*/
/*				info:	irgendeine info fr cond								*/
/*	Ergebnis:	Abbruch durch cond?												*/		
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL BTree_ScanEntriesUnSort(BTree *bTree, BTCondProc cond, CONST _VOID *info)
{
	BTCarrier *lst,akt;
	if (bTree == NULL)
		return FALSE;
	lst=&bTree->first;
	akt=bTree->first;
	while(akt)
	{
		if(akt->data)
		{
			if (cond(akt->data, NO_CONST(info)))
				return TRUE;
			lst=&akt->next;
			akt = akt->next;
		}
		else
		{
			*lst=akt->next;
			OFREE(akt);
			akt = *lst;
		}
	}
	return FALSE;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _BOOL BTree_ScanEntriesSort(BTree *bTree, BTCondProc cond, CONST _VOID *info)	*/
/*	Geht den Baum durch (also sortiert) und macht irgedwas (cond)				*/
/*	Weitere Info zu cond s.BTree_ScanEntriesUnSort								*/
/*	Achtung rekursiv! da derzeit nur zu debugzwecken bei Bumen mit ~500 		*/
/*	Eintrgen benutz OK, sonst ggf. ndern										*/
/*	Parameter:	bTree:	der Baum												*/
/*				cond:	die fr jedes Element aufzurufende Funktion				*/
/*				info:	irgendeine info fr cond								*/
/*	Ergebnis:	Abbruch durch cond?												*/		
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
#include <portaes.h>
GLOBAL _BOOL BTree_ScanEntriesSort(BTree *bTree, BTCondProc cond, CONST _VOID *info)
{
	BTCarrier akt;
	akt=bTree->current;
	if(bTree->current->smaller)
	{
		bTree->current=bTree->current->smaller;
		if(BTree_ScanEntriesSort(bTree,cond,info))
			return TRUE;
		bTree->current=bTree->current->up;
		if(akt!=bTree->current)
			form_alert(1,"[3][up falsch][Mist]");
	}
	if(bTree->current->data)
	{
		if (cond(bTree->current->data, NO_CONST(info)))
			return TRUE;
	}
	if(bTree->current->bigger)
	{
		bTree->current=bTree->current->bigger;
		if(BTree_ScanEntriesSort(bTree,cond,info))
			return TRUE;
		bTree->current=bTree->current->up;
		if(akt!=bTree->current)
			form_alert(1,"[3][up falsch][Mist]");
	}
	return FALSE;
}
			