/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */

/*
 * 29.11.02 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.19d'
 *	Die Conf-Variablen UsenetChaining und ConfigRequestFile gelscht.
 */

#include <confvars.h>
#include <ro_mem.h>
#include <btrees.h>
#include <debug.h>
#include <strconv.h>
#include <mtappl.h>
#include <mtalerts.h>
#include <mttextfi.h>
#include <routinec.h>
#include <routines.h>
#include <rectfunc.h>
#include <filename.h>
#include <file_io.h>
#include <termproc.h>
#include <mtrsc.h>
#include <catlib.h>


#define configVersion "1.3 Dec 16 1996,"
#define copyRight     "  1992-1997 Dirk Steins"
#define whatString    "@(#)ConfigVars " configVersion  copyRight

#define equalSignCol  30

typedef _UBYTE MaxStr[512];
typedef _UBYTE *strPtr;

typedef union {
	_WORD intV;
	_LONG lintV;
	strPtr strV;
	GRECT rectV;
	_BOOL boolV;
} varVal;

typedef struct {
	varType type;
	varVal val;
	_UBYTE name[1];
} confVar;


typedef enum { noError, noBool, noInt, noRect, noString, syntaxErr, typeErr } ErrorIndicator;

LOCAL BTree *confVars;
LOCAL BTree normalVars;
LOCAL BTree tmpVars;
LOCAL _BOOL usingTmpVars;

LOCAL _BOOL reqF;
LOCAL TEXTFILENO reqFile;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _WORD ConfVars_cmpVar(_VOID *curr,_VOID *search)						*/
/*	Vergleicht per strCmp die Namen von 2 Variablen als Callback fr BTree		*/
/*	Parameter:	curr:	z.B. die aktuelle Variable								*/
/*				search:	z.B. die gesuchte										*/
/*	Ergebnis:	<0 search<curr													*/		
/*				=0 search=curr													*/		
/*				>0 search>curr													*/		
/*	6.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _WORD ConfVars_cmpVar(_VOID *curr,_VOID *search)
{
	confVar *theVar1,*theVar2;
	_UBYTE qs1=0,qs2=0,*ptr;
	theVar1=curr;
	theVar2=search;
	for(ptr=theVar1->name;*ptr;ptr++)
		qs1^=*ptr;
	for(ptr=theVar2->name;*ptr;ptr++)
		qs2^=*ptr;
	if(qs1!=qs2)
		return((_WORD)qs2-(_WORD)qs1);
	return(strCmp(theVar2->name,theVar1->name));
}
/*** ---------------------------------------------------------------------- ***/

LOCAL confVar *findConfVar(CONST _UBYTE *name)
{
	confVar *theVar=NULL,*search;
	_WORD cmp=0;
	search=(confVar *)((size_t)name-offsetof(confVar,name));
	theVar=BTree_FindEntry(confVars,search,&cmp);
	if(!cmp)
		return theVar;
	return NULL;
	
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID make_varname(_UBYTE *varname, size_t namesize, CONST _UBYTE *id, _LONG number)
{
	_UBYTE numStr[20];
	
	strMcpy(varname, namesize, id);
	IntToStr(number, 0, numStr, sizeof(numStr));
	strMcat(varname, namesize, numStr);
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Umrechnungsprozeduren */

LOCAL _LONG lMin(_LONG x, _LONG y)
{
	return x < y ? x : y;
}

/*** ---------------------------------------------------------------------- ***/

#define ndcBase 32767L

LOCAL _VOID RC2NDC(GRECT *r)
{
	_WORD tw, th;

	tw = deskSize.g_x + deskSize.g_w;
	th = deskSize.g_y + deskSize.g_h;
#define RDC2NDCVAL(x, w) \
	x = (_WORD)lMin(ndcBase, ((ndcBase * (_LONG)(x) * 10L) / (_LONG)(w) + 5) / 10)

	RDC2NDCVAL(r->g_x, tw);
	RDC2NDCVAL(r->g_y, th);
	RDC2NDCVAL(r->g_w, tw);
	RDC2NDCVAL(r->g_h, th);
#undef RDC2NDCVAL
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID NDC2RC(GRECT *r)
{
	_WORD tw, th;

	tw = deskSize.g_x + deskSize.g_w;
	th = deskSize.g_y + deskSize.g_h;
#define NDC2RDCVAL(x, w) \
	x = (_WORD)lMin(ndcBase, (((_LONG)(w) * (_LONG)(x) * 10L) / ndcBase + 5) / 10)

	NDC2RDCVAL(r->g_x, tw);
	NDC2RDCVAL(r->g_y, th);
	NDC2RDCVAL(r->g_w, tw);
	NDC2RDCVAL(r->g_h, th);
#undef NDC2RDCVAL
}

/*** ---------------------------------------------------------------------- ***/

/*
 * veraendert eine vorhandene Variable oder erzeugt eine neue
 */
LOCAL _BOOL setVar(CONST confVar *theVar, CONST _UBYTE *name)
{
	confVar *newVar;
	confVar *search;
	_WORD cmp=0;
	search=(confVar *)((size_t)name-offsetof(confVar,name));	/*Erstmal temporr nur mit name zur Suche*/
	newVar=BTree_FindEntry(confVars,search,&cmp);
	if (!newVar)
	{
		newVar = MALLOC(sizeof(confVar) + strLen(name), "ConfVars_setVar");
		if (newVar == NULL)
			return FALSE;
		newVar->type = theVar->type;
		newVar->val = theVar->val;
		strCpy(newVar->name, name);
		if (!BTree_AppendEntry(confVars, newVar,cmp))
		{
			OFREE(newVar);
			return FALSE;
		}
	} else
	{
		if (newVar->type == lint && theVar->type == sint)
		{
			/* Kompatibilitaet von lInt und int */
			newVar->type = theVar->type;
		} else if (newVar->type == sint && theVar->type == lint)
		{
			/* Bereichspruefung und dann zuweisen */
			newVar->type = theVar->type;
		} else if (newVar->type != theVar->type)
		{
			return FALSE;
		}
		if (theVar->type == string)
		{
			SFREE(newVar->val.strV);
			newVar->val.strV = theVar->val.strV;
		} else
		{
			newVar->val = theVar->val;
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID prep(CONST _UBYTE *in, _UBYTE *name, size_t namesize, _UBYTE *arg, size_t argsize)
{
	_UBYTE *p;
	CONST _UBYTE *scan;

#define is_space(c) ((c) == ' ' || (c) == '\t' || (c) == '\r' || (c) == '\n')

	--argsize;
	--namesize;
	while (is_space(*in))
		in++;
	while (*in != '\0' && !is_space(*in))
	{
		if (namesize)
		{
			*name++ = *in;
			namesize--;
		}
		in++;
	}
	*name = '\0';

	while (is_space(*in))
		in++;
	scan = in;
	while (*scan != '\0' && *scan != '=')
		scan++;
	if (*scan != '\0')
		in = scan + 1;
	while (is_space(*in))
		in++;
	p = arg;
	while (*in != '\0' && *in != ';' && argsize)
	{
		*p++ = *in++;
		argsize--;
	}
	*p = '\0';
	while (p != arg && is_space(p[-1]))
		*--p = '\0';
}

/*** ---------------------------------------------------------------------- ***/

LOCAL varType getNameAndType(
	CONST _UBYTE *line,
	_UBYTE *name, size_t namesize,
	_UBYTE *arg, size_t argsize,
	ErrorIndicator *errorType)
{
	varType theType;

	prep(line, name, namesize, arg, argsize);
	if (*name == '\0' || *name == '#' || *name == ';')
		return comment;
	if (*arg == '\0')
	{
		*errorType = syntaxErr;
		return noVar;
	}

	theType = noVar;
	switch (*arg)
	{
	case '0': case '1': case '2': case '3': case '4':
	case '5': case '6': case '7': case '8': case '9':
	case '-':
		theType = lint;
		break;
	case '{':
		theType = rect;
		break;
	case 'O': case 'o':
	case 'T': case 't':
	case 'F': case 'f':
		theType = bool;
		break;
	case '"':
	case '\'':
		if (arg[strLen(arg) - 1] == arg[0])
			theType = string;
		break;
	}
	if (theType == noVar)
		*errorType = typeErr;
	return theType;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL getInfoBool(CONST _UBYTE *arg, ErrorIndicator *errorType)
{
	if (strCaseCmp(arg, "ON") == 0)
		return TRUE;
	if (strCaseCmp(arg, "TRUE") == 0)
		return TRUE;
	if (strCaseCmp(arg, "OFF") == 0)
		return FALSE;
	if (strCaseCmp(arg, "FALSE") == 0)
		return FALSE;
	*errorType = noBool;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _LONG getInfoInt(CONST _UBYTE *arg, ErrorIndicator *errorType)
{
	_LONG t;
	_BOOL valid;

	t = StrToLInt(arg, NULL, &valid);
	if (!valid)
	{
		*errorType = noInt;
		t = 0;
	}
	return t;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID getInfoRect(CONST _UBYTE *arg, ErrorIndicator *errorType, GRECT *r)
{
	size_t p;
	_WORD i;
	_WORD ra[4];
	_BOOL valid;

	p = 0;
	for (i = 0; i < 4; i++)
	{
		while (arg[p] == ' ' || arg[p] == '{' || arg[p] == ',')
			++p;
		ra[i] = StrToInt(arg, &p, &valid);
		if (!valid)
		{
			*errorType = noRect;
			return;
		}
	}

	r->g_x = ra[0];
	r->g_y = ra[1];
	r->g_w = ra[2];
	r->g_h = ra[3];

	NDC2RC(r);
	rc_intersect(&deskSize, r);
	if (r->g_w < 0)
		r->g_w = 80;
	if (r->g_h < 0)
		r->g_h = 120;
	rc_intersect(&deskSize, r);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID getInfoStr(CONST _UBYTE *arg, ErrorIndicator *errorType, _UBYTE *str, size_t strsize)
{
	_UBYTE *p;
	_UBYTE numStr[4];

	if (*arg == '\0')
	{
		*errorType = noString;
		return;
	}
	strMcpy(str, strsize, arg + 1);	/* vorderes Anfuehrungszeichen ueberspringen */
	strsize = strLen(str);
	if (strsize && str[strsize - 1] == arg[0])
		str[--strsize] = '\0';		/* hinteres Anfuehrungszeichen loeschen */
	/* Sonderzeichen uebersetzen */
	p = str;
	while ((p = strChr(p, '\\')) != NULL)
	{
		/* Feststellen, ob die naechsten drei Zahlen sind */
#define is_digit(c) ((c) >= '0' && (c) <= '9')
		if (is_digit(p[1]) && is_digit(p[2]) && is_digit(p[3]))
		{
			strMcpy(numStr, 4, p + 1);
			*p++ = StrToCard(numStr, NULL, NULL);
			strCpy(p, p + 3);
		} else if (p[1] == '\\')
		{
			/* Ein \ entfernen */
			strCpy(p, p + 1);
			p++;
		} else
		{
			p++;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.21b */
LOCAL _BOOL processLine(CONST _UBYTE *fname, CONST _UBYTE *line, _LONG lineNumber)
{
	confVar theVar;
	ErrorIndicator errorType;
	MaxStr alt;
	MaxStr argStr;
	MaxStr valStr;
	MaxStr varname;
	errorType = noError;
	switch (getNameAndType(line, varname, sizeof(varname), argStr, sizeof(argStr), &errorType))
	{
	case lint:
		theVar.val.lintV = getInfoInt(argStr, &errorType);
		if (theVar.val.lintV <= 32767L && theVar.val.lintV >= -32768L)
		{
			theVar.val.intV = (_WORD)theVar.val.lintV;
			theVar.type = sint;
		} else
		{
			theVar.type = lint;
		}
		break;
	case bool:
		theVar.val.boolV = getInfoBool(argStr, &errorType);
		theVar.type = bool;
		break;
	case rect:
		getInfoRect(argStr, &errorType, &theVar.val.rectV);
		theVar.type = rect;
		break;
	case string:
		getInfoStr(argStr, &errorType, valStr, sizeof(valStr));
		if (errorType == noError)
		{
			theVar.val.strV = STRDUP(valStr, "ConfVars_processLine");
			if (theVar.val.strV == NULL)
				return FALSE;
			theVar.type = string;
		}
		break;
	case comment:
		return TRUE;
	default:
		errorType = typeErr;
		break;
	}

	if (errorType != noError)
	{
		_WORD err;

		/* Fehlermeldung ausgeben */
		switch (errorType)
		{
			case noInt:	err = ST_NO_INT; break;
			case noBool: err = ST_NO_BOOL; break;
			case noString: err = ST_NO_STRING; break;
			case noRect: err = ST_NO_RECT; break;
			case syntaxErr: err = ST_SYNTAX_ERR; break;
			case typeErr: err = ST_TYPE_ERR; break;
			default: err = -1; break;
		}
		sprintf(alt, Rsc_WrkString(AL_INFFILE_ERROR), fname, err<0?"":Rsc_WrkString(err), lineNumber);
		mtAlerts_SetIconColor(RED);
		mtAlerts_Alert(1, alt);
		mtAlerts_SetIconColor(BLACK);
		return TRUE; /* An der Stelle koennen wird ruhig weitermachen */
	}
	/* Variable richtig eingelesen, also weitermachen */
	return setVar(&theVar, varname);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL CONST _UBYTE *buildBool(_BOOL value)
{
	return value ? "TRUE" : "FALSE";
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID writeName(TEXTFILENO inf, CONST _UBYTE *name)
{
	_WORD len;

	mtTextfiles_WriteLine(inf, name);
	len = (_WORD)strLen(name);
	if (len < equalSignCol)
		mtTextfiles_WriteLine(inf, "                              " + len);
	mtTextfiles_WriteLine(inf, " := ");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID writeInfoBool(TEXTFILENO inf, CONST _UBYTE *name, _BOOL value)
{
	writeName(inf, name);
	mtTextfiles_WriteLine(inf, buildBool(value));
	mtTextfiles_WriteChar(inf, ';');
	mtTextfiles_WriteLn(inf);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID writeInfoInt(TEXTFILENO inf, CONST _UBYTE *name, _LONG value)
{
	writeName(inf, name);
	mtTextfiles_WriteLongInt(inf, value, 0);
	mtTextfiles_WriteChar(inf, ';');
	mtTextfiles_WriteLn(inf);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID writeStr(TEXTFILENO inf, CONST _UBYTE *value)
{
	_UBYTE str[4];
	
	mtTextfiles_WriteLine(inf, "\"");
	while (*value != '\0')
	{
		if (*value >= ' ' && *value != ';')
		{
			if (*value == '\\')
				mtTextfiles_WriteChar(inf, '\\');
			mtTextfiles_WriteChar(inf, *value);
		} else
		{
			mtTextfiles_WriteChar(inf, '\\');
			NumToStr(*value, 10, 3, '0', str, 4);
			mtTextfiles_WriteLine(inf, str);
		}
		value++;
	}
	mtTextfiles_WriteLine(inf, "\"");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID writeInfoStr(TEXTFILENO inf, CONST _UBYTE *name, CONST _UBYTE *value)
{
	writeName(inf, name);
	writeStr(inf, value);
	mtTextfiles_WriteChar(inf, ';');
	mtTextfiles_WriteLn(inf);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *buildRectStr(CONST GRECT *value, _UBYTE *str)
{
	GRECT r;

	r = *value;
	RC2NDC(&r);
	sprintf(str, "{%d, %d, %d, %d}", r.g_x, r.g_y, r.g_w, r.g_h);
	return str;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID writeInfoRect(TEXTFILENO inf, CONST _UBYTE *name, CONST GRECT *value)
{
	MaxStr str;

	writeName(inf, name);
	mtTextfiles_WriteLine(inf, buildRectStr(value, str));
	mtTextfiles_WriteChar(inf, ';');
	mtTextfiles_WriteLn(inf);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL confVar *reqConfBool(CONST _UBYTE *name, _BOOL withDef, _BOOL def)
{
	confVar *theVar;

	theVar = findConfVar(name);
	if (reqF)
	{
		writeName(reqFile, name);
		if (theVar != NULL)
			mtTextfiles_WriteLine(reqFile, buildBool(theVar->val.boolV));
		mtTextfiles_WriteLine(reqFile, ";  [");
		if (withDef)
			mtTextfiles_WriteLine(reqFile, buildBool(def));
		mtTextfiles_WriteLine(reqFile, "]");
		mtTextfiles_WriteLn(reqFile);
	}
	return theVar;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL confVar *reqConfStr(CONST _UBYTE *name, _BOOL withDef, CONST _UBYTE *def)
{
	confVar *theVar;

	theVar = findConfVar(name);
	if (reqF)
	{
		writeName(reqFile, name);
		if (theVar != NULL && theVar->type == string)
		{
			writeStr(reqFile, theVar->val.strV);
		}
		mtTextfiles_WriteLine(reqFile, ";  [");
		if (withDef)
		{
			writeStr(reqFile, def);
		}
		mtTextfiles_WriteLine(reqFile, "]");
		mtTextfiles_WriteLn(reqFile);
	}
	return theVar;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL confVar *reqConfInt(CONST _UBYTE *name, _BOOL withDef, _LONG def)
{
	confVar *theVar;

	theVar = findConfVar(name);
	if (reqF)
	{
		writeName(reqFile, name);
		if (theVar != NULL)
		{
			if (theVar->type == lint)
				mtTextfiles_WriteLongInt(reqFile, theVar->val.lintV, 0);
			else
				mtTextfiles_WriteLongInt(reqFile, theVar->val.intV, 0);
		}
		mtTextfiles_WriteLine(reqFile, ";  [");
		if (withDef)
			mtTextfiles_WriteLongInt(reqFile, def, 0);
		mtTextfiles_WriteLine(reqFile, "]");
		mtTextfiles_WriteLn(reqFile);
	}
	return theVar;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL confVar *reqConfRect(CONST _UBYTE *name, _BOOL withDef, CONST GRECT *def)
{
	confVar *theVar;
	MaxStr str;

	theVar = findConfVar(name);
	if (reqF)
	{
		writeName(reqFile, name);
		if (theVar != NULL)
			mtTextfiles_WriteLine(reqFile, buildRectStr(&theVar->val.rectV, str));
		mtTextfiles_WriteLine(reqFile, ";  [");
		if (withDef)
			mtTextfiles_WriteLine(reqFile, buildRectStr(def, str));
		mtTextfiles_WriteLine(reqFile, "]");
		mtTextfiles_WriteLn(reqFile);
	}
	return theVar;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL writeEntry(_VOID *entry, _VOID *info)
{
	CONST confVar *theVar;
	CONST TEXTFILENO *inf;

	theVar = entry;
	inf = info;
	switch (theVar->type)
	{
	case string:
		writeInfoStr(*inf, theVar->name, theVar->val.strV);
		break;
	case lint:
		writeInfoInt(*inf, theVar->name, theVar->val.lintV);
		break;
	case sint:
		writeInfoInt(*inf, theVar->name, theVar->val.intV);
		break;
	case rect:
		writeInfoRect(*inf, theVar->name, &theVar->val.rectV);
		break;
	case bool:
		writeInfoBool(*inf, theVar->name, theVar->val.boolV);
		break;
	default:
		break;
	}
	return FALSE;
}
/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID delete_var(_VOID *entry)
{
	confVar *theVar = entry;
	if(theVar)
	{
		if (theVar->type == string)
			SFREE(theVar->val.strV);
		OFREE(theVar);
	}
}


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*-------- exportierte Funktionen ---------------*/

GLOBAL _VOID ConfVars_Exit(_VOID)
{
	if (normalVars.root != NULL)
		BTree_DeleteBTree(&normalVars,delete_var);
	if (tmpVars.root != NULL)
		BTree_DeleteBTree(&tmpVars,delete_var);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL ConfVars_Init(_VOID)
{
	if (normalVars.root == NULL)
	{
		if (!BTree_CreateBTree(&normalVars,ConfVars_cmpVar))
			return FALSE;
		InstallTermproc(ConfVars_Exit);
		confVars = &normalVars;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL ConfVars_GetConfigString(CONST _UBYTE *name, _UBYTE *val, size_t valsize)
{
	confVar *theVar;

	if ((theVar = reqConfStr(name, FALSE, "")) == NULL)
		return FALSE;
	if (theVar->type != string)
		return FALSE;
	strMcpy(val, valsize, theVar->val.strV);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL ConfVars_GetConfigInt(CONST _UBYTE *name, _WORD *val)
{
	confVar *theVar;

	if ((theVar = reqConfInt(name, FALSE, 0)) == NULL)
		return FALSE;
	if (theVar->type == lint)
	{
		if (theVar->val.lintV > 32767L || theVar->val.lintV < -32768L)
			return FALSE;
		*val = (_WORD)theVar->val.lintV;
	} else if (theVar->type == sint)
	{
		*val = theVar->val.intV;
	} else
	{
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL ConfVars_GetConfigLongInt(CONST _UBYTE *name, _LONG *val)
{
	confVar *theVar;

	if ((theVar = reqConfInt(name, FALSE, 0)) == NULL)
		return FALSE;
	if (theVar->type == lint)
	{
		*val = theVar->val.lintV;
	} else if (theVar->type == sint)
	{
		*val = theVar->val.intV;
	} else
	{
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL ConfVars_GetConfigBool(CONST _UBYTE *name, _BOOL *val)
{
	confVar *theVar;

	if ((theVar = reqConfBool(name, FALSE, FALSE)) == NULL)
		return FALSE;
	if (theVar->type != bool)
		return FALSE;
	*val = theVar->val.boolV;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL ConfVars_GetConfigRect(CONST _UBYTE *name, GRECT *val)
{
	confVar *theVar;

	if ((theVar = reqConfBool(name, FALSE, FALSE)) == NULL)
		return FALSE;
	if (theVar->type != rect)
		return FALSE;
	*val = theVar->val.rectV;
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*------------- Variablen lesen mit Default-Wert -------------------*/

/* Liest eine String-Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */
GLOBAL _VOID ConfVars_GetConfDefStr(CONST _UBYTE *name, _UBYTE *val, size_t valsize, CONST _UBYTE *default_)
{
	confVar *theVar;

	if ((theVar = reqConfStr(name, TRUE, default_)) != NULL && theVar->type == string)
		strMcpy(val, valsize, theVar->val.strV);
	else
		strMcpy(val, valsize, default_);
}

/*** ---------------------------------------------------------------------- ***/

/* Liest eine Integer-Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */
GLOBAL _VOID ConfVars_GetConfDefInt(CONST _UBYTE *name, _WORD *val, _WORD default_)
{
	confVar *theVar;

	if ((theVar = reqConfInt(name, TRUE, default_)) == NULL)
	{
		*val = default_;
		return;
	}
	if (theVar->type == lint)
	{
		if (theVar->val.lintV > 32767L || theVar->val.lintV < -32768L)
		{
			*val = default_;
			return;
		}
		*val = (_WORD)theVar->val.lintV;
	} else if (theVar->type == sint)
	{
		*val = theVar->val.intV;
	} else
	{
		*val = default_;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Liest eine Integer-Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */
GLOBAL _VOID ConfVars_GetConfDefLongInt(CONST _UBYTE *name, _LONG *val, _LONG default_)
{
	confVar *theVar;

	if ((theVar = reqConfInt(name, TRUE, default_)) == NULL)
	{
		*val = default_;
		return;
	}
	if (theVar->type == lint)
	{
		*val = theVar->val.lintV;
	} else if (theVar->type == sint)
	{
		*val = theVar->val.intV;
	} else
	{
		*val = default_;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Liest eine Bool'sche Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */
GLOBAL _VOID ConfVars_GetConfDefBool(CONST _UBYTE *name, _BOOL *val, _BOOL default_)
{
	confVar *theVar;

	if ((theVar = reqConfBool(name, TRUE, default_)) == NULL || theVar->type != bool)
	{
		*val = default_;
	} else
	{
		*val = theVar->val.boolV;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Liest eine Rechteck-Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */
GLOBAL _VOID ConfVars_GetConfDefRect(CONST _UBYTE *name, GRECT *val, CONST GRECT *default_)
{
	confVar *theVar;

	if ((theVar = reqConfRect(name, TRUE, default_)) == NULL || theVar->type != rect)
	{
		*val = *default_;
	} else
	{
		*val = theVar->val.rectV;
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*------------- Variablen schreiben -------------------*/

/* Schreibt eine String-Variable. Bei TRUE konnte die Variable angelegt werden.
 */
GLOBAL _BOOL ConfVars_SetConfigString(CONST _UBYTE *name, CONST _UBYTE *val)
{
	confVar theVar;

	theVar.type = string;
	theVar.val.strV = STRDUP(val, "ConfVars_SetConfigStr");
	if (theVar.val.strV == NULL)
		return FALSE;
	if (!setVar(&theVar, name))
	{
		SFREE(theVar.val.strV);
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Schreibt eine Integer-Variable. Bei TRUE konnte die Variable angelegt werden.
 */
GLOBAL _BOOL ConfVars_SetConfigInt(CONST _UBYTE *name, _WORD val)
{
	confVar theVar;

	theVar.type = sint;
	theVar.val.intV = val;
	return setVar(&theVar, name);
}

/*** ---------------------------------------------------------------------- ***/

/* Schreibt eine Integer-Variable. Bei TRUE konnte die Variable angelegt werden.
 */
GLOBAL _BOOL ConfVars_SetConfigLongInt(CONST _UBYTE *name, _LONG val)
{
	confVar theVar;

	theVar.type = lint;
	theVar.val.lintV = val;
	return setVar(&theVar, name);
}

/*** ---------------------------------------------------------------------- ***/

/* Schreibt eine Bool'sche Variable. Bei TRUE konnte die Variable angelegt werden.
 */
GLOBAL _BOOL ConfVars_SetConfigBool(CONST _UBYTE *name, _BOOL val)
{
	confVar theVar;

	theVar.type = bool;
	theVar.val.boolV = val;
	return setVar(&theVar, name);
}

/*** ---------------------------------------------------------------------- ***/

/* Schreibt eine Rechteck-Variable. Bei TRUE konnte die Variable angelegt werden.
 */
GLOBAL _BOOL ConfVars_SetConfigRect(CONST _UBYTE *name, CONST GRECT *val)
{
	confVar theVar;

	theVar.type = rect;
	theVar.val.rectV = *val;
	return setVar(&theVar, name);
}

/*** ------------------------------------------------------------------------ ***/
/*	 _BOOL isSort(_VOID *entry, _VOID *info)									*/
/*	Callback Funktion zur berprfung der Reihenfolge im BTree					*/
/*	Parameter:	entry:	der aktuelle zu prfende Eintrag						*/
/*				info:	Zwischenspeicher fr den vorigen Eintrag, sollte beim	*/
/*						1. Aufruf auf einen Leerstring zeigen.					*/
/*	Ergebnis:	TRUE:	bei Fehler!												*/		
/*	6.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _BOOL isSort(_VOID *entry, _VOID *info)
{
	CONST confVar *theVar;
	_UBYTE **pLst;
	_BOOL ret;
	pLst=info;
	theVar = entry;
	ret=strCmp(*pLst,theVar->name)>=0;
	*pLst=theVar->name;
	
	return ret;
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _VOID ConfVars_Test(_VOID)											*/
/*	Testet die Reihenfolge der confVars											*/
/*	Achtung BTree_ScanEntriesSort ist rekursiv!									*/
/*	Parameter:	keine															*/
/*	Ergebnis:	keines															*/		
/*	5.11.00 von Dimitri Junker	DYJ 4.18zo										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _VOID ConfVars_Test(_VOID)
{
	_UBYTE empty[]="",*lst;
	lst=empty;
	if(BTree_ResetBTree(confVars)<2)
		return;
	if(BTree_ScanEntriesSort(confVars, isSort, &lst))
	{
		SPEC_DEBUG_OUT((EO_FATAL, "ConfVars not alphabeticly sortet"));
		form_alert(1,"[3][Variablen in Baum falsch sortiert][Mist]");
	}
	else
		SPEC_DEBUG_OUT((EO_INFO, "ConfVars alphabeticly sortet"));
}
/*** ---------------------------------------------------------------------- ***/

/* Loescht die Config-Variable name, falls vorhanden,
 * und dealloziert saemtlichen benoetigten Speicher
 */
GLOBAL _BOOL ConfVars_DeleteConfigVar(CONST _UBYTE *name)
{
	if (findConfVar(name) != NULL)
	{
		BTree_RemoveEntry(confVars, delete_var);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Loescht alle Variablen der aktuellen Variablenliste
 */
GLOBAL _VOID ConfVars_DeleteAllVars(_VOID)
{
	BTree_DeleteBTree(confVars, delete_var);	
}

/*** ---------------------------------------------------------------------- ***/

/* Wird aufgerufen, wenn das Programm beendet wird.
 * Dadurch wird ein eventuell vorhandenes Request-File
 * geschlossen
 */
GLOBAL _VOID ConfVars_CloseConfig(_VOID)
{
	if (reqF)
	{
		mtTextfiles_CloseTextfile(&reqFile);
		reqF = FALSE;
	}
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*------------- File I/O ------------------------- */

/*DYJ 5.19d */	/*DYJ 5.21b */
GLOBAL _BOOL ConfVars_ReadConfigFile(CONST _UBYTE *filename)
{
	TEXTFILENO inf;
	_BOOL cont;
	_LONG line;
	MaxStr s;
	_UBYTE *fname;

	if (mtTextfiles_OpenTextfile(filename, READ, 0, &inf) != GERR_OK)
		return FALSE;
	fname = F_Path_Basename(filename);
	line = 0;
	do
	{
		mtTextfiles_ReadLine(inf, s, sizeof(s));
		mtTextfiles_ReadLn(inf);
		++line;
		cont = processLine(fname, s, line);
	} while (!mtTextfiles_EndofText(inf) && cont);
	if (!cont)
		mtAlerts_Alert(1, Rsc_WrkString(AL_CONF_FILE_MEM));
	mtTextfiles_CloseTextfile(&inf);
	return cont;
}

/*** ---------------------------------------------------------------------- ***/

/* Leere Infoproc, gibt nichts aus */
GLOBAL _VOID ConfVars_emptyInf(TEXTFILENO inf)
{
	UNUSED(inf);
}
/*** ---------------------------------------------------------------------- ***/

/* Schreibt alle Config-Variablen in eine Datei. Diese Datei wird
 * dabei angelegt.
 * Ueber die infoProc koennen am Anfang der Datei noch zusaetzliche
 * Informationen abgespeichert werden. Dazu muessen die Ausgabeprozeduren von
 * mtTextfiles benutzt werden.
 */
GLOBAL _BOOL ConfVars_WriteConfigFile(CONST _UBYTE *filename, infoProc info)
{
	TEXTFILENO inf;

	if (mtTextfiles_OpenTextfile(filename, WRITE, 16384, &inf) != GERR_OK)
		return FALSE;
	if (info != FUNK_NULL)
		info(inf);

	mtTextfiles_WriteLine (inf, "# " whatString);
	mtTextfiles_WriteLn (inf);
	BTree_ScanEntriesUnSort(confVars, writeEntry, &inf);
	mtTextfiles_CloseTextfile(&inf);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Konfiguriert das Modul um, welche Variablenliste benutzt
 * wird. Default sind die normalen Variablen.
 */
GLOBAL _BOOL ConfVars_ConfigConfigModule(_BOOL useTmpVars)
{
	_BOOL wasTmp;
	
	if (useTmpVars)
	{
		if (tmpVars.root == NULL)
		{
			BTree_CreateBTree(&tmpVars,ConfVars_cmpVar);
		}
		confVars = &tmpVars;
	} else
	{
		confVars = &normalVars;
	}
	wasTmp = usingTmpVars;
	usingTmpVars = useTmpVars;
	return wasTmp;
}

/*** ---------------------------------------------------------------------- ***/

/* Konfiguriert das Modul um, welche Variablenliste benutzt
 * wird.
 */
GLOBAL _VOID ConfVars_SetConfigVarList(BTree *l)
{
	confVars = l;
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die interne Variablenliste wieder auf die
 * normalen Configvariablen zurueck.
 */
GLOBAL _VOID ConfVars_ResetConfigVarList(_VOID)
{
	ConfVars_ConfigConfigModule(FALSE);
}

/*** ---------------------------------------------------------------------- ***/

typedef struct {
	confWalkProc proc;
	_VOID *private;
} walkInfo;

LOCAL _BOOL walkVars(_VOID *entry, _VOID *info)
{
	CONST confVar *theVar;
	walkInfo *inf;
	_BOOL res;
	
	theVar = entry;
	inf = info;
	switch (theVar->type)
	{
	case bool:
		res = inf->proc(theVar->name, 0, &theVar->val.boolV, inf->private);
		break;
	case sint:
		res = inf->proc(theVar->name, 1, &theVar->val.intV, inf->private);
		break;
	case lint:
		res = inf->proc(theVar->name, 2, &theVar->val.lintV, inf->private);
		break;
	case string:
		res = inf->proc(theVar->name, 3, theVar->val.strV, inf->private);
		break;
	case rect:
		res = inf->proc(theVar->name, 4, &theVar->val.rectV, inf->private);
		break;
	default:
		res = FALSE;
		break;
	}
	return !res;
}

/*** ---------------------------------------------------------------------- ***/
GLOBAL _BOOL ConfVars_Walk(confWalkProc proc, _VOID *private)
{
	walkInfo inf;
	
	inf.proc = proc;
	inf.private = private;
	BTree_ResetBTree(confVars);
	if (BTree_ScanEntriesUnSort(confVars, walkVars, &inf))	
		return FALSE;
	return TRUE;
}

