/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */

#define __DIALSLIDER_IMPLEMENTATION__

#include <dialslid.h>
#include <mtappl.h>
#include <mtrsc.h>
#include <mtutils.h>
#include <listutil.h>
#include <ro_mem.h>
#include <termproc.h>


LOCAL OBJECT *vertSlidTree;
LOCAL OBJECT *horiSlidTree;
LOCAL RESOURCE rsc;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _UBYTE RscData[] = {
#include <sliddial.inc>
};
#include <sliddial.h>

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

#define SLIDER_OBJS 5
typedef OBJECT SliderTree[SLIDER_OBJS];

struct _slidspec {
	_BOOL hori;
	_BOOL drawn;
	_WORD accel;
	_LONG scale;
	_LONG position;
	_WORD size;
	_WORD step;
	SliderTree ob;
};

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID calcSlider(SLIDER sl, _LONG pos)
{
	GRECT sFrame;
	GRECT pFrame;

	if (sl->hori)
	{
		/* calculate slider size */
		ObjcRect(sl->ob, GHSLID, &sFrame);
		ObjcRect(sl->ob, GHBAR, &pFrame);
		if (sl->scale > sl->size)
		{
			sFrame.g_w = pFrame.g_w;
			sFrame.g_w = (_WORD)(((((_LONG)sl->size * 100) / sl->scale) * (_LONG)pFrame.g_w) / 100);
			if (sFrame.g_w > pFrame.g_w)
				sFrame.g_w = pFrame.g_w;
			if (sFrame.g_w < CharWidth * 2)
				sFrame.g_w = CharWidth * 2;
		} else
		{
			sFrame.g_w = pFrame.g_w;
		}
		sFrame.g_x = 0;
		SetObjcRect(sl->ob, GHSLID, &sFrame);
		if (sl->scale > sl->size)
			sl->step = (_WORD)(((100L * (pFrame.g_w - sFrame.g_w)) / (sl->scale - sl->size) + 50) / 100);
		else
			sl->step = 0;
	} else
	{
		/* calculate slider size */
		ObjcRect(sl->ob, GVSLID, &sFrame);
		ObjcRect(sl->ob, GVBAR, &pFrame);
		if (sl->scale > sl->size)
		{
			sFrame.g_h = pFrame.g_h;
			sFrame.g_h = (_WORD)(((((_LONG)sl->size * 100) / sl->scale) * (_LONG)pFrame.g_h) / 100);
			if (sFrame.g_h > pFrame.g_h)
				sFrame.g_h = pFrame.g_h;
			if (sFrame.g_h < CharHeight)
				sFrame.g_h = CharHeight;
		} else
		{
			sFrame.g_h = pFrame.g_h;
		}
		sFrame.g_y = 0;
		SetObjcRect(sl->ob, GVSLID, &sFrame);
		if (sl->scale > sl->size)
			sl->step = (_WORD)(((100L * (pFrame.g_h - sFrame.g_h)) / (sl->scale - sl->size) + 50) / 100);
		else
			sl->step = 0;
	}
	sl->position = pos;
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet den Slider sl im Rechtecht r (oder ganz,
 * wenn clip FALSE ist)
 */
GLOBAL _VOID DialSlider_DrawSlider(SLIDER sl, CONST GRECT *r, _BOOL clip)
{
	if (sl != NULL)
	{
		if (clip)
		{
			if (r->g_w > 0 && r->g_h > 0)
				Objc_draw(sl->ob, ROOT, MAX_DEPTH, r);
		} else
		{
			Objc_draw(sl->ob, ROOT, MAX_DEPTH, &deskSize);
		}
		sl->drawn = TRUE;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID DialSlider_Exit(_VOID)
{
	if (rsc != NULL)
	{
		RscFree(rsc);
		rsc = NULL;
		vertSlidTree = NULL;
		horiSlidTree = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL DialSlider_Init(_VOID)
{
	if (vertSlidTree == NULL)
	{
		if ((rsc = RscReloc(RscData, sizeof(RscData))) == NULL)
			return FALSE;
		vertSlidTree = RscGaddr(rsc, R_TREE, SLID_VERTICAL);
		horiSlidTree = RscGaddr(rsc, R_TREE, SLID_HORIZONTAL);
		InstallTermproc(DialSlider_Exit);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Erzeugt einen neuen Slider. hori gibt an, ob es sich um einen
 * horizontalen Slider handelt, und accel gibt die Zeilenzahl beim
 * scrollen an (normalerweise 1)
 */
GLOBAL _BOOL DialSlider_CreateSlider(SLIDER *psl, _BOOL hori, _WORD accel, _WORD slSize, _LONG scale, _LONG slPos)
{
	_WORD i;
	SLIDER sl;
	OBJECT *tree;

	*psl = NULL;
	if (!DialSlider_Init())
		return FALSE;
	sl = NEW(SLIDSPEC, "DialSlider_CreateSlider");
	if (sl == NULL)
		return FALSE;
	sl->hori = hori;
	sl->accel = accel;
	sl->scale = scale;
	sl->position = slPos;
	sl->size     = slSize;
	sl->step     = 0;
	sl->drawn    = FALSE;
	if (hori)
		tree = horiSlidTree;
	else
		tree = vertSlidTree;
	for (i = 0; i < SLIDER_OBJS; i++)
	{
		sl->ob[i] = tree[i];
		rsrc_obfix(sl->ob, i);
	}
	*psl = sl;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt einen Slider wieder frei. Danach darf nicht mehr
 * darauf zugegriffen werden
 */
GLOBAL _VOID DialSlider_FreeSlider(SLIDER *sl)
{
	if (*sl != NULL)
	{
		OFREE(*sl);
		*sl = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die Groesse des Sliders (Anzahl der angezeigten Eintraege)
 */
GLOBAL _VOID DialSlider_SetSliderSize(SLIDER sl, _WORD size)
{
	if (sl != NULL && sl->size != size)
	{
		sl->size = size;
		calcSlider(sl, 0);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die Skalierung fuer den Slider (Anzahl Elemente)
 */
GLOBAL _VOID DialSlider_SetSliderScale(SLIDER sl, _LONG scale)
{
	if (sl != NULL && sl->scale != scale)
	{
		sl->scale = scale;
		calcSlider(sl, 0);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die aktuelle Position des Sliders
 */
GLOBAL _VOID DialSlider_SetSliderPos(SLIDER sl, _LONG pos, _BOOL draw)
{
	_LONG scale;

	if (sl != NULL && sl->position != pos)
	{
		scale = sl->scale - sl->size;
		sl->position = pos;
		if (sl->hori)
		{
			ListUtils_formSetSlider(sl->ob, GHBAR, GHSLID, FALSE,
				scale, sl->position, sl->drawn && draw);
		} else
		{
			ListUtils_formSetSlider(sl->ob, GVBAR, GVSLID, TRUE,
				scale, sl->position, sl->drawn && draw);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Plaziert den Slider an die richtige Stelle
 * Sollte direkt nach dem Erzeugen aufgerufen werden
 */
GLOBAL _VOID DialSlider_SliderExtents(SLIDER sl, _WORD x, _WORD y, _WORD len, _BOOL draw)
{
	_LONG scl;
	_WORD l2;

	if (sl != NULL)
	{
		/* Anpassung an 1 Pixel Rahmen der Slider-Objekte */
		++x;
		++y;
		len -= 2;
		/* Jetzt testen, ob sich was geaendert hat */
		if (sl->hori)
			l2 = sl->ob[ROOT].ob_width;
		else
			l2 = sl->ob[ROOT].ob_height;
		if (x == sl->ob[ROOT].ob_x && y == sl->ob[ROOT].ob_y && len == l2)
			return;
		/* Basisobjekt */
		sl->ob[ROOT].ob_x = x;
		sl->ob[ROOT].ob_y = y;
		if (sl->hori)
		{
			sl->ob[ROOT].ob_width = len;
			sl->ob[ROOT].ob_height = CharHeight;
		} else
		{
			sl->ob[ROOT].ob_height = len;
			sl->ob[ROOT].ob_width = CharWidth * 2;
		}
		if (sl->hori)
		{
			/* Erster Arrow */
			sl->ob[GLEFT].ob_x = 0;
			sl->ob[GLEFT].ob_y = 0;
			sl->ob[GLEFT].ob_width = CharWidth * 2;
			sl->ob[GLEFT].ob_height = CharHeight;
			/* Zweiter Arrow */
			sl->ob[GRIGHT].ob_x = len - CharWidth * 2;
			sl->ob[GRIGHT].ob_y = 0;
			sl->ob[GRIGHT].ob_width = CharWidth * 2;
			sl->ob[GRIGHT].ob_height = CharHeight;
			/* Slidebar */
			sl->ob[GHBAR].ob_x = CharWidth * 2;
			sl->ob[GHBAR].ob_y = 0;
			sl->ob[GHBAR].ob_height = CharHeight;
			sl->ob[GHBAR].ob_width  = len - CharWidth * 4;
			/* Slider */
			sl->ob[GHSLID].ob_x = 0;
			sl->ob[GHSLID].ob_y = 0;
			sl->ob[GHSLID].ob_height = CharHeight;
			sl->ob[GHSLID].ob_width  = len - CharWidth * 4;
		} else
		{
			/* Erster Arrow */
			sl->ob[GUP].ob_x = 0;
			sl->ob[GUP].ob_y = 0;
			sl->ob[GUP].ob_width = CharWidth * 2;
			sl->ob[GUP].ob_height = CharHeight;
			/* Zweiter Arrow */
			sl->ob[GDOWN].ob_x = 0;
			sl->ob[GDOWN].ob_y = len - CharHeight;
			sl->ob[GDOWN].ob_width = CharWidth * 2;
			sl->ob[GDOWN].ob_height = CharHeight;
			/* Slidebar */
			sl->ob[GVBAR].ob_x = 0;
			sl->ob[GVBAR].ob_y = CharHeight;
			sl->ob[GVBAR].ob_height = len - CharHeight * 2;
			sl->ob[GVBAR].ob_width  = CharWidth * 2;
			/* Slider */
			sl->ob[GVSLID].ob_x = 0;
			sl->ob[GVSLID].ob_y = 0;
			sl->ob[GVSLID].ob_height = len - CharHeight * 2;
			sl->ob[GVSLID].ob_width  = CharWidth * 2;
		}

		calcSlider(sl, sl->position);
		scl = sl->scale - sl->size;
		if (sl->hori)
			ListUtils_formSetSlider(sl->ob, GHBAR, GHSLID, FALSE, scl, sl->position, sl->drawn && draw);
		else
			ListUtils_formSetSlider(sl->ob, GVBAR, GVSLID, TRUE, scl, sl->position, sl->drawn && draw);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID formSlider(SLIDER sl, sliderUpdateProc update, _VOID *env)
{
	_LONG newVal;

	GRECT pAbsFrame;
	GRECT sAbsFrame;
	GRECT pFrame;
	GRECT sFrame;
	GRECT xFrame;

	_WORD xOff;
	_WORD yOff;
	_WORD X, Y;
	_UWORD button;
	_UWORD kstate;
	_LONG oldStart;

	if (sl->hori)
	{
		ObjcArea(sl->ob, GHBAR, &pAbsFrame);
		ObjcArea(sl->ob, GHSLID, &sAbsFrame);
		ListUtils_CalcAbsArea(sl->ob, GHSLID, &xFrame);

		ObjcRect(sl->ob, GHBAR, &pFrame);
		ObjcRect(sl->ob, GHSLID, &sFrame);
	} else
	{
		ObjcArea(sl->ob, GVBAR, &pAbsFrame);
		ObjcArea(sl->ob, GVSLID, &sAbsFrame);
		ListUtils_CalcAbsArea(sl->ob, GVSLID, &xFrame);

		ObjcRect(sl->ob, GVBAR, &pFrame);
		ObjcRect(sl->ob, GVSLID, &sFrame);
	}

	MouseHand();
	oldStart = sl->position;
	newVal = oldStart;
	graf_mkstate(&X, &Y, &button, &kstate);
	xOff = X - sAbsFrame.g_x - 1;
	yOff = Y - sAbsFrame.g_y - 1;
	for (;;)
	{
		graf_mkstate(&X, &Y, &button, &kstate);
		if (!(button & MOB_LEFT))
			break;

		if (!sl->hori)
		{
			if (pAbsFrame.g_h <= sAbsFrame.g_h)
			{
				newVal = 0;
			} else
			{
				/*
				Epp = ((scale - size) * 1000L) / (pFrame.g_h - sFrame.g_h);
				newVal = (((Y - yOff - pAbsFrame.g_y) * Epp) / 1000L);
				*/
				newVal = ((Y - yOff - pAbsFrame.g_y) * (sl->scale - sl->size) ) / (pAbsFrame.g_h - sAbsFrame.g_h);
			}
		} else
		{
			if (pFrame.g_w <= sFrame.g_w)
				newVal = 0;
			else
				newVal = ((X - xOff - pAbsFrame.g_x) * (sl->scale - sl->size)) / (pAbsFrame.g_w - sAbsFrame.g_w);
		}
		if (newVal > sl->scale - sl->size)
			newVal = sl->scale - sl->size;
		if (newVal < 0)
			newVal = 0;
		if (sl->accel > 0)
			newVal = (newVal / sl->accel) * sl->accel;

		/* Slider neu zeichnen und Liste updaten */
		if (newVal != oldStart)
		{
			update(sl, newVal, env);
			oldStart = newVal;
			DialSlider_SetSliderPos(sl, newVal, TRUE);
		}
	}
	MouseArrow();
	sl->position = newVal;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID SelectButton(OBJECT *tree, _WORD objc, _BOOL doSel)
{
	GRECT r;

	EnableObjState(tree, objc, SELECTED, doSel);
	CalcArea(tree, objc, &r);
	Objc_draw(tree, ROOT, 2, &r);
}

/*** ---------------------------------------------------------------------- ***/

/* In: Mauskoordinate. Out: Neue Position. Bei einem Mausklick
 * ausserhalb des Sliders wird -1 zurueckgegeben
 */
GLOBAL _LONG DialSlider_SliderClick(SLIDER sl, _WORD x, _WORD y, _WORD clicks, sliderUpdateProc update, _VOID *env)
{
	_WORD objc;
	_UWORD kstate;
	_UWORD buts;
	GRECT r;
	_LONG newPos, oldPos;

	if (sl != NULL && clicks != 0)
	{
		objc = objc_find(sl->ob, ROOT, MAX_DEPTH, x, y);
		if (objc > 0)
		{
			/* Klick behandeln */
			switch (objc)
			{
			case GLEFT: /* Up- bzw. Leftarrow */
				SelectButton(sl->ob, objc, TRUE);
				oldPos = sl->position;
				newPos = sl->position;
				if (clicks > 1)
					newPos = 0;
				do
				{
					newPos -= sl->accel;
					if (newPos < 0)
						newPos = 0;
					if (newPos != oldPos)
					{
						update(sl, newPos, env);
						DialSlider_SetSliderPos(sl, newPos, TRUE);
					}
					oldPos = newPos;
					graf_mkstate(&x, &y, &buts, &kstate);
				} while (buts & 1);
				SelectButton(sl->ob, objc, FALSE);
				break;

			case GRIGHT: /* Down- bzw. Rightarrow */
				SelectButton(sl->ob, objc, TRUE);
				oldPos = sl->position;
				newPos = sl->position;
				if (clicks > 1)
					newPos = sl->scale - sl->size;
				do
				{
					newPos += sl->accel;
					if (newPos > sl->scale - sl->size)
						newPos = sl->scale - sl->size;
					if (newPos != oldPos)
					{
						update(sl, newPos, env);
						DialSlider_SetSliderPos(sl, newPos, TRUE);
					}
					oldPos = newPos;
					graf_mkstate(&x, &y, &buts, &kstate);
				} while (buts & 1);
				SelectButton(sl->ob, objc, FALSE);
				break;

			case GHBAR: /* Slidebar */
				CalcArea(sl->ob, GHSLID, &r);
				oldPos = sl->position;
				newPos = sl->position;
				if ((sl->hori & (x < r.g_x)) || (!sl->hori && (y < r.g_y)))
				{
					do
					{
						/* Page Up/Right */
						newPos -= sl->size;
						if (newPos < 0)
							newPos = 0;
						if (newPos != oldPos)
						{
							update(sl, newPos, env);
							DialSlider_SetSliderPos(sl, newPos, TRUE);
						}
						oldPos = newPos;
						graf_mkstate(&x, &y, &buts, &kstate);
					} while (buts & 1);
				} else
				{
					do
					{
						/* Page Down/Left */
						newPos += sl->size;
						if (newPos > sl->scale - sl->size)
							newPos = sl->scale - sl->size;
						if (newPos != oldPos)
						{
							update(sl, newPos, env);
							DialSlider_SetSliderPos(sl, newPos, TRUE);
						}
						oldPos = newPos;
						graf_mkstate(&x, &y, &buts, &kstate);
					} while (buts & 1);
				}
				break;

			case GHSLID: /* Slider */
				formSlider(sl, update, env);
				break;
			}
			/*
			update(sl, position);
			*/
			return sl->position;
		} else
		{
			return -1;
		}
	}
	return -1;
}

