/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 2000, Dimitri Junker
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */



#ifndef __CAT_BTREES_H__
#define __CAT_BTREES_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif

/*
 * Allgemeine Binrbaumverwaltung.
 *
 * Erstellt 6.11.2000
 */


typedef struct _btcarrier Carrier;
typedef struct _btcarrier *BTCarrier;

struct _btcarrier {
	BTCarrier smaller;	/* kleineres Folgeelement im Baum	*/
	BTCarrier bigger;	/* greres Folgeelement im Baum	*/
	BTCarrier next;		/* unsortierte Verkettung als Liste	*/
	BTCarrier up;		/* Vorgnger im Baum				*/
	_VOID *data;		/* Daten, je nach Verwendung		*/ 
};

typedef _WORD (*bTcmpElem)(_VOID *current,_VOID *search);
typedef _VOID (*btrees_delete_proc)(_VOID *entry);

typedef struct BTree {
	BTCarrier current;		/* 	darf nur NULL sein wenn root NULL ist*/
	BTCarrier root;			/* Start des Binren Baums	*/
	BTCarrier first;		/* Start der einfach verketteten Liste, kann nach lschen von root !=root sein*/
	BTCarrier last;			/* Ende der einfach verketteten Liste	*/
	bTcmpElem cmpElem;		/* Callback Funktion um 2 Daten zu vergleichen	*/
} BTree;

typedef _BOOL (*BTCondProc)(_VOID *entry, _VOID *info);


_BOOL BTree_CreateBTree(BTree *bTree,bTcmpElem cmpElem);
/*
/*	initialisiert die Variablen													*/
 */

_WORD BTree_ResetBTree(BTree *bTree);
/*
 * Ernennt die Wurzel (bTree.root) zum aktuellen Eintrag.
 * 'error': 0: bTree=NULL 1: bTree->root=NULL 2: sonst
 */


_BOOL BTree_RemoveEntry(BTree *bTree, btrees_delete_proc delete);
/*
 * Entfernt aktuellen Eintrag. 'current' wird auf root gesetzt.
 * 'error':= Leerer Baum oder sonstiges Problem
 */

_VOID BTree_DeleteBTree(BTree *l, btrees_delete_proc delete);
/*
 *	lscht einen ganzen Baum
 */

_VOID *BTree_FindEntry(BTree *bTree,CONST _VOID *entry,_WORD *cmp);
/*
 * 	Durchsucht einen BTree nach einem Eintrag.
 */

_BOOL BTree_AppendEntry(BTree *bTree, CONST _VOID *entry,_WORD cmp);
/*
 * Fuegt Eintrag in BTree ein; 'bTree.current' wird auf das neue gesetzt.
 * 'error':= "Kein Speicher mehr frei"
 */

_VOID *BTree_NextEntry(BTree *bTree,_WORD dir);
/*
 * Liefert naechsten Eintrag, setzt 'current' darauf; liefert NIL, wenn
 * nicht vorhanden.
 */

_BOOL BTree_ScanEntriesSort(BTree *bTree, BTCondProc cond, CONST _VOID *info);
_BOOL BTree_ScanEntriesUnSort(BTree *bTree, BTCondProc cond, CONST _VOID *info);
/*
 * 2 Routinen um den Baum zu durchwandern, einmal sortiert also ber die Baumverkettung,
 * das andere Mal unsortiert ber die einfachverkettete Liste
 */


#endif /*__CAT_BTREES_H__*/
