#ifndef __CAT_CONFVARS_H__
#define __CAT_CONFVARS_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */




/* Dieses Modul verwaltet Konfigurationsvariablen innerhalb einer Liste.
 * Fuer jede Variable wird ein Eintrag angelegt, in dem der Name, der Typ
 * und der Wert festgehalten wird.
 * Der Zugriff ist jederzeit ueber die Funktionen dieses Moduls moeglich.
 * Beim Schreiben einer Variable wird der alte Inhalt ersetzt. Eine Typwandlung
 * einer Variablen ist nicht moeglich!
 */

#ifndef __PORTAB_H__
#include <portab.h>
#endif

#ifndef __CAT_MTTEXTFILES_H__
#include <mttextfi.h>
#endif
#ifndef __GRAFBASE_H__
#include <grafbase.h>
#endif

typedef enum { noVar, comment, bool, sint, lint, string, rect} varType;

_VOID make_varname(_UBYTE *varname, size_t namesize, CONST _UBYTE *id, _LONG number);

/*------------- Variablen lesen ohne Default-Wert -------------------*/

_BOOL ConfVars_GetConfigString(CONST _UBYTE *name, _UBYTE *val, size_t valsize);
/* Liest eine String-Variable. Liefert bei TRUE den Wert in val zurueck
 */

_BOOL ConfVars_GetConfigInt(CONST _UBYTE *name, _WORD *val);
/* Liest eine Integer-Variable. Liefert bei TRUE den Wert in val zurueck
 */

_BOOL ConfVars_GetConfigLongInt(CONST _UBYTE *name, _LONG *val);
/* Liest eine LongInteger-Variable. Liefert bei TRUE den Wert in val zurueck
 */

_BOOL ConfVars_GetConfigBool(CONST _UBYTE *name, _BOOL *val);
/* Liest eine Bool'sche Variable. Liefert bei TRUE den Wert in val zurueck
 */

_BOOL ConfVars_GetConfigRect(CONST _UBYTE *name, GRECT *val);
/* Liest eine Rechteck-Variable. Liefert bei TRUE den Wert in val zurueck
 */

/*------------- Variablen lesen mit Default-Wert -------------------*/

_VOID ConfVars_GetConfDefStr(CONST _UBYTE *name, _UBYTE *val, size_t valsize, CONST _UBYTE *default_);
/* Liest eine String-Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */

_VOID ConfVars_GetConfDefInt(CONST _UBYTE *name, _WORD *val, _WORD default_);
/* Liest eine Integer-Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */

_VOID ConfVars_GetConfDefLongInt(CONST _UBYTE *name, _LONG *val, _LONG default_);
/* Liest eine Integer-Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */

_VOID ConfVars_GetConfDefBool(CONST _UBYTE *name, _BOOL *val, _BOOL default_);
/* Liest eine Bool'sche Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */

_VOID ConfVars_GetConfDefRect(CONST _UBYTE *name, GRECT *val, CONST GRECT *default_);
/* Liest eine Rechteck-Variable. Wenn die Variable nicht gefunden wird,
 * dann wird der Default-Wert zurueckgegeben.
 */

/*------------- Variablen schreiben -------------------*/

_BOOL ConfVars_SetConfigString(CONST _UBYTE *name, CONST _UBYTE *val);
/* Schreibt eine String-Variable. Bei TRUE konnte die Variable angelegt werden.
 */

_BOOL ConfVars_SetConfigInt(CONST _UBYTE *name, _WORD val);
/* Schreibt eine Integer-Variable. Bei TRUE konnte die Variable angelegt werden.
 */

_BOOL ConfVars_SetConfigLongInt(CONST _UBYTE *name, _LONG val);
/* Schreibt eine Integer-Variable. Bei TRUE konnte die Variable angelegt werden.
 */

_BOOL ConfVars_SetConfigBool(CONST _UBYTE *name, _BOOL val);
/* Schreibt eine Bool'sche Variable. Bei TRUE konnte die Variable angelegt werden.
 */

_BOOL ConfVars_SetConfigRect(CONST _UBYTE *name, CONST GRECT *val);
/* Schreibt eine Rechteck-Variable. Bei TRUE konnte die Variable angelegt werden.
 */

/*------------- Variable loeschen -------------------*/

_BOOL ConfVars_DeleteConfigVar(CONST _UBYTE *name);
/* Loescht die Config-Variable name, falls vorhanden,
 * und dealloziert saemtlichen benoetigten Speicher
 */

_VOID ConfVars_DeleteAllVars(_VOID);
/* Loescht alle Variablen der aktuellen Variablenliste
 */

_VOID ConfVars_CloseConfig(_VOID);
/* Wird aufgerufen, wenn das Programm beendet wird.
 * Dadurch wird ein eventuell vorhandenes Request-File
 * geschlossen
 */

/*------------- File I/O -------------------*/

_BOOL ConfVars_ReadConfigFile(CONST _UBYTE *filename);


/* Liest ein Config-File ein. Danach stehen alle Variablen aus dem File
 * in der internen Liste, falls das Lesen erfolgreich war.
 */

/* info */
typedef _VOID (*infoProc)(TEXTFILENO inf);
typedef _BOOL (*confWalkProc)(CONST _UBYTE *name, _WORD type, CONST _VOID *value, _VOID *private);


_VOID ConfVars_emptyInf(TEXTFILENO inf);
/* Leere Infoproc, gibt nichts aus */

_BOOL ConfVars_WriteConfigFile(CONST _UBYTE *filename, infoProc info);
/* Schreibt alle Config-Variablen in eine Datei. Diese Datei wird
 * dabei angelegt.
 * Ueber die infoProc koennen am Anfang der Datei noch zusaetzliche
 * Informationen abgespeichert werden. Dazu muessen die Ausgabeprozeduren von
 * mtTextfiles benutzt werden.
 */


/*------------- Modulkonfiguration ---------------*/

_BOOL ConfVars_ConfigConfigModule(_BOOL useTmpVars);
/* Konfiguriert das Modul um, welche Variablenliste benutzt
 * wird. Default sind die normalen Variablen.
 */
#ifdef __CAT_BTREES_H__
_VOID ConfVars_SetConfigVarList(BTree *l);	
/* Konfiguriert das Modul um, welche Variablenliste benutzt
 * wird.
 */
#endif

_VOID ConfVars_ResetConfigVarList(_VOID);
/* Setzt die interne Variablenliste wieder auf die
 * normalen Configvariablen zurueck.
 */

_WORD ConfVars_cmpVar(_VOID *curr,_VOID *search);

_BOOL ConfVars_Walk(confWalkProc proc, _VOID *private);


_BOOL ConfVars_Init(_VOID);
_VOID ConfVars_Exit(_VOID);

#endif /* __CAT_CONFVARS_H__ */
