/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __CAT_DIALSLIDER_H__
#define __CAT_DIALSLIDER_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif

struct _slidspec;
typedef struct _slidspec SLIDSPEC, *SLIDER;
#ifndef __DIALSLIDER_IMPLEMENTATION__
struct _slidspec { int dummy; };
#endif


_VOID DialSlider_DrawSlider(SLIDER sl, CONST GRECT *r, _BOOL clip);
/* Zeichnet den Slider sl im Rechtecht r (oder ganz,
 * wenn clip FALSE ist)
 */

_BOOL DialSlider_CreateSlider(SLIDER *sl, _BOOL hori, _WORD accel, _WORD slSize, _LONG slScale, _LONG slPos);
/* Erzeugt einen neuen Slider. hori gibt an, ob es sich um einen
 * horizontalen Slider handelt, und accel gibt die Zeilenzahl beim
 * scrollen an (normalerweise 1)
 */

_VOID DialSlider_FreeSlider(SLIDER *sl);
/* Gibt einen Slider wieder frei. Danach darf nicht mehr
 * darauf zugegriffen werden
 */

_VOID DialSlider_SetSliderSize(SLIDER sl, _WORD size);
/* Setzt die Groesse des Sliders
 */

_VOID DialSlider_SetSliderScale(SLIDER sl, _LONG scale);
/* Setzt die Skalierung fuer den Slider (Anzahl Elemente)
 */

_VOID DialSlider_SetSliderPos(SLIDER sl, _LONG pos, _BOOL draw);
/* Setzt die aktuelle Position des Sliders
 */

_VOID DialSlider_SliderExtents(SLIDER sl, _WORD x, _WORD y, _WORD len, _BOOL draw);


/* Plaziert den Slider an die richtige Stelle
 * Sollte direkt nach dem Erzeugen aufgerufen werden
 */

/* sl : */
/* pos: */
typedef _VOID (*sliderUpdateProc)(SLIDER sl, _LONG pos, _VOID *env);


_LONG DialSlider_SliderClick(SLIDER sl, _WORD x, _WORD y, _WORD clicks, sliderUpdateProc update, _VOID *env);
/* In: Mauskoordinate. Out: Neue Position. Bei einem Mausklick
 * ausserhalb des Sliders wird -1 zurueckgegeben
 * Falls sich die Koordinate geaendert hat, wird ausserdem vorher noch
 * die UpdateProc update aufgerufen mit dem neuen Wert.
 * Ausserdem wird beim Scrolling auch die
 * Updateproc staendig aufgerufen, falls sich der Wert aendert.
 */


#endif /* __CAT_DIALSLIDER_H__ */
