/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __MAGIC_FILESYS_H__
#define __MAGIC_FILESYS_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __TOS_H__
#include <tos.h>
#endif
#ifndef __MINTBIND_H__
#include <mintbind.h>
#endif


struct filesys;		/* forward declaration */
struct devdrv;		/* ditto */

typedef struct filesys *FILESYSPTR;
typedef struct devdrv *DEVDRVPTR;

#ifndef ushort
#  define ushort unsigned short
#endif

#ifndef __CDECL
#  define __CDECL cdecl
#endif
#define ARGS_ON_STACK __CDECL



typedef struct f_cookie {
	FILESYSPTR fs;		/* filesystem that knows about this cookie */
	ushort	dev;		/* device info (e.g. Rwabs device number) */
	ushort	aux;		/* extra data that the file system may want */
	long	index;		/* this+dev uniquely identifies a file */
} FCOOKIE, *FCOOKIEPTR;

/* structure for opendir/readdir/closedir */
typedef struct dirstruct {
	FCOOKIE fc;		/* cookie for this directory */
	ushort	index;		/* index of the current entry */
	ushort	flags;		/* flags (e.g. tos or not) */
#define TOS_SEARCH	0x01
	char	fsstuff[60];	/* anything else the file system wants */
				/* NOTE: this must be at least 45 bytes */
	struct dirstruct *next;	/* linked together so we can close them
				   on process termination */
} DIR, *DIRPTR;
       

typedef struct fileptr {
	short	links;	    /* number of copies of this descriptor */
	ushort	flags;	    /* file open mode and other file flags */
	long	pos;	    /* position in file */
	long	devinfo;    /* device driver specific info */
	FCOOKIE	fc;	    /* file system cookie for this file */
	struct devdrv *dev; /* device driver that knows how to deal with this */
	struct fileptr *next; /* link to next fileptr for this file */
} *FILEPTR;

typedef struct flock {
	short l_type;			/* type of lock */
#define F_RDLCK		0
#define F_WRLCK		1
#define F_UNLCK		3
	short l_whence;			/* SEEK_SET, SEEK_CUR, SEEK_END */
	long l_start;			/* start of locked region */
	long l_len;			/* length of locked region */
	short l_pid;			/* pid of locking process
						(F_GETLK only) */
} FLOCK;

/* LOCK structure used by the kernel internally */

typedef struct ilock {
	struct flock l;		/* the actual lock */
	struct ilock *next;	/* next lock in the list */
	long	reserved[4];	/* reserved for future expansion */
} LOCK, *iLOCK;

typedef struct devdrv {
	long ARGS_ON_STACK (*open)	(FILEPTR f);
	long ARGS_ON_STACK (*write)	(FILEPTR f, const char *buf, long bytes);
	long ARGS_ON_STACK (*read)	(FILEPTR f, char *buf, long bytes);
	long ARGS_ON_STACK (*lseek)	(FILEPTR f, long where, int whence);
	long ARGS_ON_STACK (*ioctl)	(FILEPTR f, int mode, _VOID *buf);
	long ARGS_ON_STACK (*datime)	(FILEPTR *f, short *timeptr, int rwflag);
	long ARGS_ON_STACK (*close)	(FILEPTR f, int pid);
	long ARGS_ON_STACK (*select)	(FILEPTR f, long proc, int mode);
	_VOID ARGS_ON_STACK (*unselect)	(FILEPTR f, long proc, int mode);
/* extensions, check dev_descr.drvsize (size of DEVDRV struct) before calling:
 * fast RAW tty byte io  */
	long ARGS_ON_STACK (*writeb)	(FILEPTR f, const char *buf, long bytes);
	long ARGS_ON_STACK (*readb)	(FILEPTR f, char *buf, long bytes);
/* what about: scatter/gather io for DMA devices...
 *	long ARGS_ON_STACK (*writev)	(FILEPTR f, const struct iovec *iov, long cnt);
 *	long ARGS_ON_STACK (*readv)	(FILEPTR f, const struct iovec *iov, long cnt);
 */
} DEVDRV;

typedef struct filesys {
	struct	filesys	*next;	/* link to next file system on chain */
	long	fsflags;
#define FS_KNOPARSE	0x01	/* kernel shouldn't do parsing */
#define FS_CASESENSITIVE	0x02	/* file names are case sensitive */
#define FS_NOXBIT	0x04	/* if a file can be read, it can be executed */
#define	FS_LONGPATH	0x08	/* file system understands "size" argument to
				   "getname" */
#define FS_NO_C_CACHE	0x10	/* don't cache cookies for this filesystem */
#define FS_DO_SYNC	0x20	/* file system has a sync function */
	long	ARGS_ON_STACK (*root) (int drv, FCOOKIE *fc);
	long	ARGS_ON_STACK (*lookup) (FCOOKIE *dir, const char *name, FCOOKIE *fc);
	long	ARGS_ON_STACK (*creat) (FCOOKIE *dir, const char *name, unsigned mode,
				int attrib, FCOOKIE *fc);
	DEVDRV * ARGS_ON_STACK (*getdev) (FCOOKIE *fc, long *devspecial);
	long	ARGS_ON_STACK (*getxattr) (FCOOKIE *file, XATTR *xattr);
	long	ARGS_ON_STACK (*chattr) (FCOOKIE *file, int attr);
	long	ARGS_ON_STACK (*chown) (FCOOKIE *file, int uid, int gid);
	long	ARGS_ON_STACK (*chmode) (FCOOKIE *file, unsigned mode);
	long	ARGS_ON_STACK (*mkdir) (FCOOKIE *dir, const char *name, unsigned mode);
	long	ARGS_ON_STACK (*rmdir) (FCOOKIE *dir, const char *name);
	long	ARGS_ON_STACK (*remove) (FCOOKIE *dir, const char *name);
	long	ARGS_ON_STACK (*getname) (FCOOKIE *relto, FCOOKIE *dir,
			    char *pathname, int size);
	long	ARGS_ON_STACK (*rename) (FCOOKIE *olddir, char *oldname,
			    FCOOKIE *newdir, const char *newname);
	long	ARGS_ON_STACK (*opendir) (DIR *dirh, int tosflag);
	long	ARGS_ON_STACK (*readdir) (DIR *dirh, char *name, int namelen, FCOOKIE *fc);
	long	ARGS_ON_STACK (*rewinddir) (DIR *dirh);
	long	ARGS_ON_STACK (*closedir) (DIR *dirh);
	long	ARGS_ON_STACK (*pathconf) (FCOOKIE *dir, int which);
	long	ARGS_ON_STACK (*dfree) (FCOOKIE *dir, long *buf);
	long	ARGS_ON_STACK (*writelabel) (FCOOKIE *dir, const char *name);
	long	ARGS_ON_STACK (*readlabel) (FCOOKIE *dir, char *name, int namelen);
	long	ARGS_ON_STACK (*symlink) (FCOOKIE *dir, const char *name, const char *to);
	long	ARGS_ON_STACK (*readlink) (FCOOKIE *dir, char *buf, int len);
	long	ARGS_ON_STACK (*hardlink) (FCOOKIE *fromdir, const char *fromname,
				FCOOKIE *todir, const char *toname);
	long	ARGS_ON_STACK (*fscntl) (FCOOKIE *dir, const char *name, int cmd, long arg);
	long	ARGS_ON_STACK (*dskchng) (int drv);
	long	ARGS_ON_STACK (*release) (FCOOKIE *);
	long	ARGS_ON_STACK (*dupcookie) (FCOOKIE *new, FCOOKIE *old);
	long	ARGS_ON_STACK (*sync) (_VOID);
} FILESYS;


/* flags for open() modes */
#define O_RWMODE  	0x03	/* isolates file read/write mode */
#	define O_RDONLY	0x00
#	define O_WRONLY	0x01
#	define O_RDWR	0x02
#	define O_EXEC	0x03	/* execute file; used by kernel only */

/* 0x04 is for future expansion */
#define O_APPEND	0x08	/* all writes go to end of file */

#define O_SHMODE	0x70	/* isolates file sharing mode */
#	define O_COMPAT	0x00	/* compatibility mode */
#	define O_DENYRW	0x10	/* deny both read and write access */
#	define O_DENYW	0x20	/* deny write access to others */
#	define O_DENYR	0x30	/* deny read access to others */
#	define O_DENYNONE 0x40	/* don't deny any access to others */

#define O_NOINHERIT	0x80	/* private file (not passed to child) */

#undef O_CREAT
#undef O_TRUNC
#undef O_EXCL
#define O_NDELAY	0x100	/* don't block for i/o on this file */
#define O_CREAT		0x200	/* create file if it doesn't exist */
#define O_TRUNC		0x400	/* truncate file to 0 bytes if it does exist */
#define O_EXCL		0x800	/* fail open if file exists */

#define O_USER		0x0fff	/* isolates user-settable flag bits */

#define O_GLOBAL	0x1000	/* for opening a global file */

/* kernel mode bits -- the user can't set these! */
#define O_TTY		0x2000
#define O_HEAD		0x4000
#define O_LOCK		0x8000

/* GEMDOS file attributes */

/* macros to be applied to FILEPTRS to determine their type */
#define is_terminal(f) (f->flags & O_TTY)

/* lseek() origins */
#ifndef SEEK_SET
#define	SEEK_SET	0		/* from beginning of file */
#define	SEEK_CUR	1		/* from current location */
#define	SEEK_END	2		/* from end of file */
#endif

/* Dpathconf and Sysconf return this when a value is not limited
   (or is limited only by available memory) */

#define UNLIMITED	0x7fffffffL

/* various character constants and defines for TTY's */
#define MiNTEOF 0x0000ff1a	/* 1a == ^Z */

/* defines for tty_read */
#define RAW	0
#define COOKED	0x1
#define NOECHO	0
#define ECHO	0x2
#define ESCSEQ	0x04		/* cursor keys, etc. get escape sequences */

/* constants for Fcntl calls */
#define F_DUPFD		0		/* handled by kernel */
#define F_GETFD		1		/* handled by kernel */
#define F_SETFD		2		/* handled by kernel */
#	define FD_CLOEXEC	1	/* close on exec flag */

#define F_GETFL		3		/* handled by kernel */
#define F_SETFL		4		/* handled by kernel */
#define F_GETLK		5
#define F_SETLK		6
#define F_SETLKW	7
#define F_GETOPENS	8		/* handled by kernel */

/* more constants for various Fcntl's */
#define FSTAT		(('F'<< 8) | 0)		/* handled by kernel */
#define FIONREAD	(('F'<< 8) | 1)
#define FIONWRITE	(('F'<< 8) | 2)
#define FUTIME		(('F'<< 8) | 3)
#define FTRUNCATE	(('F'<< 8) | 4)
#define FIOEXCEPT	(('F'<< 8) | 5)
#define TIOCGETP	(('T'<< 8) | 0)
#define TIOCSETN	(('T'<< 8) | 1)
#define TIOCGETC	(('T'<< 8) | 2)
#define TIOCSETC	(('T'<< 8) | 3)
#define TIOCGLTC	(('T'<< 8) | 4)
#define TIOCSLTC	(('T'<< 8) | 5)
#define TIOCGPGRP	(('T'<< 8) | 6)
#define TIOCSPGRP	(('T'<< 8) | 7)
#define TIOCFLUSH	(('T'<< 8) | 8)
#define TIOCSTOP	(('T'<< 8) | 9)
#define TIOCSTART	(('T'<< 8) | 10)
#define TIOCGWINSZ	(('T'<< 8) | 11)
#define TIOCSWINSZ	(('T'<< 8) | 12)
#define TIOCGXKEY	(('T'<< 8) | 13)
#define TIOCSXKEY	(('T'<< 8) | 14)
#define TIOCIBAUD	(('T'<< 8) | 18)
#define TIOCOBAUD	(('T'<< 8) | 19)
#define TIOCCBRK	(('T'<< 8) | 20)
#define TIOCSBRK	(('T'<< 8) | 21)
#define TIOCGFLAGS	(('T'<< 8) | 22)
#define TIOCSFLAGS	(('T'<< 8) | 23)
#define TIOCOUTQ	(('T'<< 8) | 24)
#define TIOCSETP	(('T'<< 8) | 25)
#define TIOCHPCL	(('T'<< 8) | 26)
#define TIOCCAR		(('T'<< 8) | 27)
#define TIOCNCAR	(('T'<< 8) | 28)
#define TIOCWONLINE	(('T'<< 8) | 29)
#define TIOCSFLAGSB	(('T'<< 8) | 30)
#define TIOCGSTATE	(('T'<< 8) | 31)
#define TIOCSSTATEB	(('T'<< 8) | 32)
#define TIOCGVMIN	(('T'<< 8) | 33)
#define TIOCSVMIN	(('T'<< 8) | 34)

/* cursor control Fcntls:
 * NOTE THAT THESE MUST BE TOGETHER
 */
#define TCURSOFF	(('c'<< 8) | 0)
#define TCURSON		(('c'<< 8) | 1)
#define TCURSBLINK	(('c'<< 8) | 2)
#define TCURSSTEADY	(('c'<< 8) | 3)
#define TCURSSRATE	(('c'<< 8) | 4)
#define TCURSGRATE	(('c'<< 8) | 5)

/* process stuff */
#define PPROCADDR	(('P'<< 8) | 1)
#define PBASEADDR	(('P'<< 8) | 2)
#define PCTXTSIZE	(('P'<< 8) | 3)
#define PSETFLAGS	(('P'<< 8) | 4)
#define PGETFLAGS	(('P'<< 8) | 5)
#define PTRACESFLAGS	(('P'<< 8) | 6)
#define PTRACEGFLAGS	(('P'<< 8) | 7)
#	define	P_ENABLE	(1 << 0)	/* enable tracing */
#ifdef NOTYETDEFINED
#	define	P_DOS		(1 << 1)	/* trace DOS calls - unimplemented */
#	define	P_BIOS		(1 << 2)	/* trace BIOS calls - unimplemented */
#	define	P_XBIOS		(1 << 3)	/* trace XBIOS calls - unimplemented */
#endif

#define PTRACEGO	(('P'<< 8) | 8)	/* these 4 must be together */
#define PTRACEFLOW	(('P'<< 8) | 9)
#define PTRACESTEP	(('P'<< 8) | 10)
#define PTRACE11	(('P'<< 8) | 11)
#define PLOADINFO	(('P'<< 8) | 12)
#define	PFSTAT		(('P'<< 8) | 13)

#define SHMGETBLK	(('M'<< 8) | 0)
#define SHMSETBLK	(('M'<< 8) | 1)

/* terminal control constants (tty.sg_flags) */
#define T_CRMOD		0x0001
#define T_CBREAK	0x0002
#define T_ECHO		0x0004
/* #define T_XTABS	0x0008  unimplemented*/
#define T_RAW		0x0010
/* #define T_LCASE	0x0020  unimplemented */

#define T_NOFLSH	0x0040		/* don't flush buffer when signals
					   are received */
#define T_TOS		0x0080
#define T_TOSTOP	0x0100
#define T_XKEY		0x0200		/* Fread returns escape sequences for
					   cursor keys, etc. */
#define T_ECHOCTL	0x0400		/* echo ctl chars as ^x */
/* 0x0800 still available */

#define T_TANDEM	0x1000
#define T_RTSCTS	0x2000
#define T_EVENP		0x4000		/* EVENP and ODDP are mutually exclusive */
#define T_ODDP		0x8000

#define TF_FLAGS	0xF000

/* some flags for TIOC[GS]FLAGS */
#define TF_CAR		0x800		/* nonlocal mode, require carrier */
#define TF_NLOCAL	TF_CAR

#define TF_BRKINT	0x80		/* allow breaks interrupt (like ^C) */

#define TF_STOPBITS	0x0003
#define TF_1STOP	0x0001
#define TF_15STOP	0x0002
#define	TF_2STOP	0x0003

#define TF_CHARBITS	0x000C
#define TF_8BIT		0
#define TF_7BIT		0x4
#define TF_6BIT		0x8
#define TF_5BIT		0xC

/* the following are terminal status flags (tty.state) */
/* (the low byte of tty.state indicates a part of an escape sequence still
 * hasn't been read by Fread, and is an index into that escape sequence)
 */
#define TS_ESC		0x00ff
#define TS_BLIND	0x800		/* tty is `blind' i.e. has no carrier
					   (cleared in local mode) */
#define TS_HOLD		0x1000		/* hold (e.g. ^S/^Q) */
#define TS_HPCL		0x4000		/* hang up on close */
#define TS_COOKED	0x8000		/* interpret control chars */

/* Dcntl constants and types */
#define DEV_NEWTTY	0xde00
#define DEV_NEWBIOS	0xde01
#define DEV_INSTALL	0xde02

/* structures for terminals */
typedef struct tchars {
	char t_intrc;
	char t_quitc;
	char t_startc;
	char t_stopc;
	char t_eofc;
	char t_brkc;
} tchars;

typedef struct ltchars {
	char t_suspc;
	char t_dsuspc;
	char t_rprntc;
	char t_flushc;
	char t_werasc;
	char t_lnextc;
} ltchars;

typedef struct sgttyb {
	char sg_ispeed;
	char sg_ospeed;
	char sg_erase;
	char sg_kill;
	ushort sg_flags;
} sgttyb;

typedef struct winsize {
	short	ws_row;
	short	ws_col;
	short	ws_xpixel;
	short	ws_ypixel;
} winsize;

typedef struct xkey {
	short	xk_num;
	char	xk_def[8];
} xKey, *xKeyPtr;

typedef struct tty {
	short		pgrp;		/* process group of terminal */
	short		state;		/* terminal status, e.g. stopped */
	short		use_cnt;	/* number of times terminal is open */
	short		aux_cnt;	/* number of times terminal is open as
					   /dev/aux */
	struct sgttyb 	sg;
	struct tchars 	tc;
	struct ltchars 	ltc;
	struct winsize	wsiz;
	long		rsel;		/* selecting process for read */
	long		wsel;		/* selecting process for write */
	xKeyPtr		xkey;		/* extended keyboard table */
	long		hup_ospeed;	/* saved ospeed while hanging up */
	unsigned short	vmin, vtime;	/* min chars, timeout for RAW reads */
	long		resrvd[1];	/* for future expansion */
} TTY, *ttyPtr;

typedef struct dev_descr {
	DEVDRV	*driver;
	short	dinfo;
	short	flags;
	struct tty *tty;
	long	drvsize;		/* size of DEVDRV struct */
	long	fmode;
	long	reserved[2];
} devDescr;

#endif /* __MAGIC_FILESYS_H__ */
