/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __CAT_FONTSELECT_H__
#define __CAT_FONTSELECT_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif
#ifndef __PORTVDI_H__
#include <portvdi.h>
#endif
#include <wdwmanag.h>


#define FontSelectErrNoFonts       0    /* Keine Fonts geladen  */
#define FontSelectErrNoGdos       -1    /* Kein GDOS            */
#define FontSelectErrLessMem      -2    /* Kein Speicher        */
#define FontSelectErrWorkstation  -3    /* Workstation nicht ueber mtAppl geoeffnet */

#define FontsFFIRST 0
#define FontsFNEXT  1
#define FontsFPREV  2
#define FontsFLAST  3

#define FONT_NAME_LEN 33

typedef struct _tfontinfo {
	_UBYTE name[FONT_NAME_LEN]; /* Name des Fonts */
	_WORD id;                   /* Font-ID */
	_WORD color;                /* Farbindex */
	_WORD rot;                  /* Rotationsrichtung */
	_WORD halign;               /* Horizontale Ausrichtung */
	_WORD valign;               /* Vertikale Ausrichtung */
	_WORD chw;                  /* Zeichenbreite */
	_WORD chh;                  /* Zeichenhoehe */
	_WORD boxw;                 /* Zellenbreite */
	_WORD boxh;                 /* Zellenhoehe */
	_UWORD effect;              /* Texteffekte */
	_WORD minsize;              /* Minimale Groesse des Fonts */
	_WORD maxsize;              /* Maximale Groesse des Fonts */
	_WORD point;                /* aktuelle Groesse des Fonts */
	fix31 font_width;           /* Breite bei FSM-Fonts */
	_WORD skew;                 /* Neigung bei FSM-Fonts */
	_BOOL mono;                 /* TRUE, wenn Monospaced */
	_BOOL fsm;                  /* TRUE, wenn FSM-Font */
} tFontinfo;


#define FS_APPLY            0x00000001L
#define FS_TTONLY           0x00000002L
#define FS_EFFECTS          0x00000004L
#define FS_COLOR            0x00000008L
#define FS_MONOSPACED_ONLY  0x00000010L
#define FS_ENABLE_HELP      0x00000020L

_BOOL FontSelect_DoSelectFont(_WORD handle, _BOOL monoFonts, _WORD fontId, _WORD fontSize, CONST _UBYTE *title, FONT_ACCEPT_FUNC func, HWND win);
 /* Fuehrt kompletten Dialog durch. 
  * handle : VDI-Handle der Workstation;
  * monoFonts: Nur monospaced-Fonts werden angezeigt.
  */

_VOID W_Font_Select(HVDI vdi_handle, _WORD fontId, _WORD fontSize, CONST _UBYTE *title, _ULONG flags, FONT_ACCEPT_FUNC func, HWND win);
/* externen Font-Selector aufrufen */

_WORD FontSelect_FontNumber(_WORD handle);
/* Anzahl der zur Verfuegung stehenden Fonts */

_WORD FontSelect_LoadFonts(_WORD handle);
/* Laedt GEM-Fonts, wenn moeglich. Dazu muss natuerlich ein GDOS praesent 
 * sein. Returniert einen Errorcode bzw. die Anzahl der geladenen Fonts.
 */

_VOID FontSelect_UnloadFonts(_WORD handle);
/* Gibt die geladenen Fonts wieder frei. */

#define FontSelectFFIRST 0
#define FontSelectFNEXT  1
#define FontSelectFPREV  2
#define FontSelectFLAST  3

_WORD FontSelect_FontList(_WORD handle, _WORD flag);
/* Geht entsprechend flag durch die Liste der Fonts. Returniert
 * den Font-ID oder 0, wenn Ende der Liste erreicht.
 *
 * FontSelectFFIRST == Liefert den ersten Font
 * FontSelectFNEXT  == Naechster Font in der Liste
 * FontSelectFPREV  == Vorhergehender Font in der Liste
 */

_VOID FontSelect_FontInfo(_WORD handle, _WORD font, tFontinfo *info);
/* Info ueber den Font */

_WORD FontSelect_FontActive(_WORD handle);
/* Liefert den momentan aktiven Font */

_VOID FontSelect_FontSelect(_WORD handle, _WORD font);
/* Selektion eines Fonts. Der Parameter 'actual' bestimmt, ob der Zeichen-
 * satz mit den aktuellen Werten des Vorgaengers uebernommen wird (Groesse,
 * Farbe, Effekte etc.) - actual == TRUE, oder ob die Werte eingestellt werden
 * sollen, die als letztes fuer den neuen Font gueltig waren (actual == FALSE). 
 */

_WORD FontSelect_FontSize(_WORD handle, _WORD size, _WORD *cw, _WORD *ch, _WORD *bw, _WORD *bh);
/* Bestimmung der Groesse des aktuellen Fonts in POINTS.
 * cw == Zeichenbreite
 * ch == Zeichenhoehe
 * bw == Breite der Zeichenbox
 * bh == Hoehe der Zeichenbox
 */

_WORD FontSelect_FontDefsize(_WORD handle);
/* Stellt die Max-Groesse DIV 2 des aktuellen Fonts ein. Dies ist in der
 * Regel der Default-Size des Fonts.  
 * Beim Systemfont wird die Groesse entsprechend der Aufloesung gesetzt. 
 */

_VOID FontSelect_SetFont(
	_WORD handle,
	_WORD *font,
	_WORD *fontSize,
	_BOOL getBoldWidth,
	_BOOL *mono,
	_BOOL *fsm,
	_WORD *boxw,
	_WORD *boxh);
/* Diese Prozedur hierhin verlagert. Setzt den Font und die Groesse und
 * gibt die Ausmasse zurueck und die Attribute mono und fsm.
 * getBoldWidth: Bei einigen (auch unproportionalen) Zeichensaetzen
 * ist die Breite eines fetten Zeichens anders als die eines normalen
 * Zeichens. In einigen Faellen ist es deshalb sinnvoll, die Breite
 * des fetten Zeichens zu nehmen (z.B. formatierte Ausgabe in einer Liste)
 */

_WORD FontSelect_NextBigger(_WORD handle);
_WORD FontSelect_NextSmaller(_WORD handle);
/* Liefern die naechstmoegliche Fontgroesse des aktuellen Fonts */

_VOID FontSelect_FontColor(_WORD handle, _WORD color);
/* Farbe des Fonts einstellen */

_VOID FontSelect_FontEffekt(_WORD handle, _UWORD effekt);
/* Effekte des Fonts einstellen */

_WORD FontSelect_FontRotate(_WORD handle, _WORD angle);
/* Winkel einstellen, in dem der Font ausgegeben wird. Bei Bitmap-
 * Fonts sind hier nur 0, 90, 180 und 270 Grad moeglich.
 * angle ist in Zehntel Grad anzugeben (1800 fuer 180 Grad)
 */

_VOID FontSelect_FontWidth(_WORD handle, fix31 width);
/* Bestimmung des Breiten-Verhaeltnisses der Zeichen.  NUR MIT FSM-FONTS!!! */

_VOID FontSelect_FontSkew(_WORD handle, _WORD skew);
/* Bestimmung der Neigung der Zeichen.  NUR MIT FSM-FONTS!!! */

_VOID FontSelect_FontExtent(_WORD handle, CONST _UBYTE *str, _WORD *rect);
/* Berechnet die Ausdehnung eines Strings.
 * ACHTUNG: In rect wird Platz fuer 4 Koordinaten-Paare erwartet, die
 *          das Rechteck beschreiben!
 */

_WORD FontSelect_FontError(_VOID);
/* Liefert nach einer Fontaktion eine Fehlermeldung.  Dabei werden als
 * Grundlage die Fehlercodes von FSM-GDOS benutzt:
 *
 *  NoError ==        0
 *  CharNotFound ==   1
 *  ReadError ==      8
 *  OpenError ==      9
 *  BadFileformat == 10
 *  OutOfMemory ==   11
 *  MiscError ==     -1
 */

#ifdef __MAGICTOOL_FONTS_INTERN__
struct _font;
typedef struct _font Font, *FONT;
struct _font {
	tFontinfo info;
	FONT next;
};
typedef struct _fontlist {
	FONT fonts;
	FONT dummy;
	_WORD number;
} Fontlist, *FONTLIST;

#define cFonts         0x01
#define cPhysical      0x02
       /* cOpen Wird nicht mehr benoetigt, da der Pointer nun beim Schliessen
        * der Workstation geloescht wird.
        */

typedef struct _wsinfo WsInfo, *WsPtr;

struct _wsinfo {
	HVDI handle;
	_UWORD flags;
	FONTLIST list;
	WsPtr next;
	WsPtr last;
};

extern WsPtr Intern(_WORD handle);
#endif /* __MAGICTOOL_FONTS_INTERN__ */


#endif /* __CAT_FONTSELECT_H__ */
