/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __CAT_LISTMAN_H__
#define __CAT_LISTMAN_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif

struct _listInfoType;
typedef struct _listInfoType listInfoType, *LISTINFO;
#ifndef __LISTMAN_IMPLEMENTATION__
struct _listInfoType { int dummy; };
#endif

/* Handelt scrollbare Listen in Fenstern und Dialogboxen */


/* Wird aufgerufen, wenn ein Eintrag selektiert wurde.
 * Gibt zurueck, ob der Dialog verlassen werden soll,
 * falls es in einem Dialog ist.
 * TRUE = verlassen
 */
typedef _BOOL (*selectEntryProc)(_VOID *entry, _VOID *env, _WORD line);


/* Wird aufgerufen, um festzustellen, ob ein Eintrag selektiert
 * ist oder nicht
 */
typedef _BOOL (*isSelectedProc)(_VOID *entry, _VOID *env);

/* Wird aufgerufen, um festzustellen, ob ein Eintrag disabled ist oder nicht */
typedef _BOOL (*entryIsSelectableProc)(_VOID *entry, _VOID *env);


/* Wird aufgerufen, um einen Eintrag zu zeichen.
 * x und y enthalten die Position, offset ist ein "Offset" vom linken
 * Rand aus, cliprect muss beruecksichtigt werden!
 * Wenn der entry NIL ist, dann muss ein leeres Rechteck gezeichnet
 * werden
 */
typedef _VOID (*drawEntryProc)(_VOID *entry, _VOID *env, _WORD x, _WORD y, _WORD offset, CONST GRECT *clip);

/* Wird aufgerufen, wenn ein Eintrag aus der Liste gezogen und fallengelassen
 * wurde. x und y enthalten die absolute Position beim Droppen, window das Fensterhandle
 * des Fensters, auf welches der Eintrag gezogen wurde. (MH)
 */
typedef _VOID (*dragDropProc)(_VOID *entry, _VOID *env, _WORD x, _WORD y, HWND window);


typedef enum {
	smNone,
	smOne,
	smMulti
} selectMode;

/* Listenverwaltung auch ueber Prozedurvariablen */


/* Setzt Liste zurueck auf Anfang */
typedef _VOID (*ResetProc)(_VOID *list);


/* Liefert naechstes oder vorheriges Element,
 * NIL ansonsten
 */
typedef _VOID *(*MoveProc)(_VOID *list);


/* Gibt die Anzahl der Eintraege und die Breite der
 * Liste zurueck
 */
typedef _VOID (*CountProc)(_VOID *list, _LONG *length, _WORD *width);

_BOOL ListMan_InitList(
	_VOID *list,
	_VOID *env,
	selectEntryProc selectEntry,
	isSelectedProc isSelectedEntry,
	entryIsSelectableProc isSelectableEntry,
	drawEntryProc drawEntry,
	dragDropProc dragDrop,
	ResetProc resetList,
	MoveProc nextList,
	MoveProc prevList,
	CountProc countList,
	selectMode select,
	LISTINFO *listInfo);

/* Von den beiden folgenden Prozeduren muss mindestens eine nach
 * der Initialisierung der Liste aufgerufen werden
 */

_BOOL ListMan_SetListWdwSpecs(
	LISTINFO listInfo,
	HWND wdwHandle,
	CONST GRECT *area,
	_WORD voffset,
	_WORD vsize,
	_WORD hstep,
	_WORD maxwidth);
/* Setzt weitere Informationen fuer die Liste.
 * wdwHandle: Das Fenster, in dem die Liste dargestellt wird
 * area: Arbeitsbereich, in dem die Liste dargestellt wird
 * voffset: Abstand zwischen zwei Eintraegen
 * vsize  : Hoehe eines Eintrages
 * hstep  : Wie breit ist eine Spalte
 * maxwidth: maximale Breite eines Eintrags
 */

_BOOL ListMan_SetListDialSpecs(
	LISTINFO listInfo,
	OBJECT *tree,
	_WORD entryBox,
	_WORD boxBack,
	_WORD voffset,
	_WORD vsize,
	_WORD hstep,
	_WORD maxwidth);
/* Setzt weitere Informationen fuer die Liste.
 * tree: Der Baum, in dem die Liste dargestellt wird
 * entryBox: Objektindex fuer das Objekt, in dem alle Objekte
 *           dargestellt werden
 * boxBack: Objektindex des Hintergrundes der Box (fuer Slider)
 * voffset: Abstand zwischen zwei Eintraegen
 * vsize  : Hoehe eines Eintrages
 * hstep  : Wie breit ist eine Spalte
 * maxwidth: maximale Breite eines Eintrags (wenn = 0, dann wird kein
 * horizontaler Slider eingesetzt)
 */

_BOOL ListMan_FreeList(LISTINFO listInfo);

_VOID ListMan_DrawList(LISTINFO listInfo, CONST GRECT *clip);
/* Zeichnet die Liste neu und auch die Slider, Clipping wird
 * beachtet
 */

_VOID ListMan_SetDragDropProc(OBJECT *tree, dragDropProc dragDrop);
/* Setzt die D&D-Prozedur aufgrund des Trees (MH) 
 */

_LONG ListMan_ListClick(LISTINFO listInfo, _WORD x, _WORD y, _UWORD kstate, _WORD clicks);

/* Wird aufgerufen, wenn ein Mausklick stattfand. Gibt zurueck,
 * ob ein Eintrag angeklickt wurde (>= 0) oder ob
 * der Click ausserhalb des Bereiches war (-1)
 * oder ob der Click behandelt wurde (-2)
 * Wenn ein Eintrag angeklickt wurde, dann wird dieser auch selektiert,
 * falls notwendig.
 */

_VOID ListMan_ListUpdateEntry(LISTINFO listInfo, _LONG entry);
/* Einen bestimmten Eintrag neu zeichnen lassen, falls noetig
 */
_VOID ListMan_ListInvertEntry(LISTINFO listInfo, _LONG entry);
/* Der Eintrag entry wurde (extern) selektiert, den Rest handhaben
 * Nuetzlich fuer Autolocator
 */

_VOID ListMan_ListDeselEntry(LISTINFO listInfo, _LONG entry);
/* Deselektiert den Eintrag entry. Ebenfalls fuer Autolocator
 */

_VOID ListMan_ListScroll2Selection(LISTINFO listInfo, _BOOL draw);
/* Scrollt die Liste so, dass der erste selektierte Eintrag
 * sichtbar ist.  draw: auch wirklich zeichnen!
 */

_VOID ListMan_ListPgDown(LISTINFO listInfo);
_VOID ListMan_ListLnDown(LISTINFO listInfo);
_VOID ListMan_ListPgUp(LISTINFO listInfo);
_VOID ListMan_ListLnUp(LISTINFO listInfo);
_VOID ListMan_ListLnRight(LISTINFO listInfo);
_VOID ListMan_ListPgRight(LISTINFO listInfo);
_VOID ListMan_ListLnLeft(LISTINFO listInfo);
_VOID ListMan_ListPgLeft(LISTINFO listInfo);
_VOID ListMan_ListFirst(LISTINFO listInfo);
_VOID ListMan_ListLast(LISTINFO listInfo);
_VOID ListMan_ListLnMv(LISTINFO listInfo,const _WORD lines);

/* Selektion innerhalb der Liste verschieben */
_LONG ListMan_ListSelUp(LISTINFO listInfo);
_LONG ListMan_ListSelPgUp(LISTINFO listInfo);
_LONG ListMan_ListSelDown(LISTINFO listInfo);
_LONG ListMan_ListSelPgDown(LISTINFO listInfo);
_LONG ListMan_ListSelFirst(LISTINFO listInfo);
_LONG ListMan_ListSelLast(LISTINFO listInfo);

_LONG ListMan_ListListElems(LISTINFO listInfo);
/* Gibt die Anzahl der Listenelemente zurueck
 */

_VOID *ListMan_ListFindEntry(LISTINFO listInfo, _LONG index);
/* Gibt die Adresse des Eintrags index zurueck
 */

_LONG ListMan_ListFirstSelected(LISTINFO listInfo);
/* Gibt den ersten selektieren Eintrag zurueck, -1 im Fehlerfall
 */

_LONG ListMan_ListFirstEntry(LISTINFO listInfo);
/* Gibt den ersten angezeigten Eintrag zurueck, -1 im Fehlerfall
 */

/* Die Funktionen brauche ich gar nicht: */
/* Neue Position in der Liste setzen. Pos: 0..1000 */
_VOID ListMan_ListVSlide(LISTINFO listInfo, _WORD pos);
_VOID ListMan_ListHSlide(LISTINFO listInfo, _WORD pos);

_VOID ListMan_SetNewList(LISTINFO listInfo, _VOID *list, _BOOL redraw);

#endif /* __CAT_LISTMAN_H__ */
