/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __MAGIC_COOKIE_H__
#define __MAGIC_COOKIE_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif

/*----------------------------------------------------------------------*
 * MagicCookie  Zugriff auf den Cookiejar                               *
 *                                                                      *
 * Cookies sind, aehnlich XBRA im System global zugaengliche Variablen.   *
 * Sie bestehen jeweils aus einem Namen (4 Byte, moeglichst ein lesbarer *
 * Text) und einem 32Bit-Wert. Sie werden vom Betriebssystem (ab TOS-   *
 * Version 1.06 oder von residenten Programmen eingetragen.             *
 *                                                                      *
 * Sie dienen dazu, eine statische Schnittstelle zu anderen Programmen  *
 * herzustellen. Wenn beispielsweise ein residentes Programm (im AUTO-  *
 * Ordner) einen Cookie anlegt, dessen Wert ein Zeiger auf einen eige-  *
 * nen Datenbereich ist, koennen andere Programme spaeter diesen Cookie   *
 * finden und auf die Daten des residenten Programms zugreifen. Atari   *
 * hat ausserdem einige Cookies definiert, die den Rechner und die      *
 * Hardware beschreiben.                                                *
 *                                                                      *
 * VORSICHT: Wenn Sie selbst eigene Cookies eintragen wollen, muessen    *
 * Sie einiges mehr an Wissen darueber haben (z.B. Atari STE TOS Release *
 * Notes vom 12.1.90). Vor allem sollten Sie Cookies nur in residenten  *
 * Programmen erzeugen, jedoch _NIEMALS_ in Accessories oder Applika-   *
 * tionen.  Die hier vorgestellten Routinen implementieren deshalb auch *
 * nur Abfragemethoden fuer Cookies!  Fuer diejenigen, die diese Thema    *
 * weiter interessiert sind dem Magic-Paket die Cookiejar-Routinen (C-  *
 * Quellcode) von Arnd Beissner beigefuegt.                              * 
 *----------------------------------------------------------------------*/

/* Von Atari vordefinierte Cookies, die den Rechner beschreiben. */

#define COOKIE_CPU            "_CPU" /* Hauptprozessor: 0, 10, 20, 30 usw. */
#define COOKIE_FPU            "_FPU" /* Mathe-Koprozessor: 0, 1, 2 im 1. Word */
#define COOKIE_Machine        "_MCH" /* Rechner; 1. Word: 0ST, 1STE, 2TT */
#define COOKIE_Video          "_VDO" /* Video-Hardware; 1. Word: 0ST, 1STE, 2TT */
#define COOKIE_Sound          "_SND" /* Bits im Longword: 0: GI/Yamaha(ST), 1: DMA(TT) */
#define COOKIE_Switches       "_SWI" /* BIOS-interne Konfigurationswerte */
#define COOKIE_FastReadBuff   "_FRB" /* Adresse des 64KB-Puffers f. Disk-Treiber im TT */
#define COOKIE_LaserPrinter   "_SLM" /* Wird vom Diablo-Treiber (ab Version 1.4) gesetzt. */
#define COOKIE_STeFix         "_INF" /* Fix fuer den Desktop.Inf-Bug im Desktop des STE. */
#define COOKIE_DosFlock       "_FLK" /* GEMDOS verfuegt ueber Record-Locking */
#define COOKIE_Network        "_NET" /* Typ des verwendeten Netzwerks */
#define COOKIE_Poolfix        "_OOL" /* Poolfix 3 von Atari Corp */
#define COOKIE_HdDisk         "_FDC" /* Laufwerk-Konfiguration ab TOS 2.06
                                      * im 1. Word  0   DD-Laufwerke (720/360kB)
                                      *             1   HD-Laufwerke (1.44MB)
                                      *             2   ED-Laufwerke (2.88MB)
                                      */

/* Andere bekannte Cookies */

#define COOKIE_Overscan       "OVER" /* Autoswitch-Overscan von K.Isakovic */
#define COOKIE_TOS14Fix       "TFIX" /* Tos14Fix */
#define COOKIE_VDIFix         "VFIX" /* VDIFix-Progamm von K.Isakovic */
#define COOKIE_ReflexCard     "RFLX" /* Treiber der Reflex-Karte von OverScan GbR */
#define COOKIE_NVDI           "NVDI" /* NVDI von Bela */
#define COOKIE_BigScreen2     "BIGS" /* Grossbildschirmsimulator von Julian Reschke */
#define COOKIE_VirtualScreen  "VSCR" /* Zeiger auf Infostruktur von Bigscreen2 */
#define COOKIE_Gemini         "GMNI" /* GEMINI, der Shareware-Desktop */
#define COOKIE_Mupfel         "MUPF" /* MUPFEL von Gereon Steffens */
#define COOKIE_MakeJar        "MkJr" /* MakeCookiejar von Thomas Schallar */
#define COOKIE_TTTF           "TTTF" /* ThirtyTwoToTwentyFour von Marcel Waldvogel */
#define COOKIE_FileSelector   "FSEL" /* Extended Fileselector z.B. Patzelbox */
#define COOKIE_LetemFly       "LTMF" /* Let'em Fly von Oliver Scheel */
#define COOKIE_FSEL           "FSEL"
#define COOKIE_MagiX          "MagX" /* Magi!X */
#define COOKIE_MagicMac       "MgMc" /* Magic Mac */
#define COOKIE_MagicPC        "MgPC" /* Magic PC */
#define COOKIE_WIN_LNK        "DYWL" /* WIN_LNK installiert */		/*DYJ 5.30b */
#define COOKIE_Magic          "MagC" /* Magic Dials */
#define COOKIE_MultiGEM       "MGEM" /* Multi GEM */
#define COOKIE_TOS2WIN        "_T2W"
#define COOKIE_MINT           "MiNT"
#define COOKIE_Geneva         "Gnva"
#define COOKIE_NAES           "nAES"
#define COOKIE_XFSL           "xFSL"
#define COOKIE_UFSL           "UFSL"


_BOOL FindCookie(const _UBYTE *cookie, _LONG *value);
/* Liefert TRUE, wenn der uebergebene Cookie gefunden wird. Liefert
 * dann in value den dazugehoerigen Longword-Wert, bei FALSE ist
 * value immer Null.
 */

_BOOL ModifyCookie(const _UBYTE *cookie, _LONG value);
/* Sucht den Cookie und liefert FALSE, wenn er nicht vorhanden ist.
 * Wird er gefunden, wird value als neuer Cookie-Wert eingesetzt und
 * die Funktion liefert TRUE.
 */

#ifndef _DOSVARS
#define _DOSVARS
typedef struct
{
	_BYTE		*in_dos;		 	  		/* Adresse der DOS- Semaphore */
	_WORD		*dos_time; 					/* Adresse der DOS- Zeit */
	_WORD		*dos_date; 					/* Adresse des DOS- Datums */
	_LONG		res1;
	_LONG		res2;
	_LONG		res3;
	_VOID		*act_pd;		 	  		/* Laufendes Programm */
	_LONG		res4;
	_WORD		res5;
	_VOID		*res6;
	_VOID		*res7;						/* interne DOS- Speicherliste */
	_VOID		(*resv_intmem)(void);	 	/* DOS- Speicher erweitern */
	_LONG		(*etv_critic)(void);		/* etv_critic des GEMDOS */
	_BYTE *		((*err_to_str)(_BYTE e));	/* Umrechnung Code->Klartext */
	_LONG		res8;
	_LONG		res9;
	_LONG		res10;
} DOSVARS;
#endif

#ifndef _AESVARS
#define _AESVARS
typedef struct
{
	_LONG 	magic;							/* muss $87654321 sein */
	_VOID 	*membot;		   				/* Ende der AES- Variablen	*/
	_VOID 	*aes_start;		   				/* Startadresse */
	_LONG 	magic2;			 				/* ist 'MAGX' */
	_LONG 	date; 							/* Erstelldatum ttmmjjjj */
	_VOID 	(*chgres)(_WORD res, _WORD txt);	/* Aufloesung aendern */
	_LONG 	(**shel_vector)(_VOID);  		/* residentes Desktop */
	_BYTE 	*aes_bootdrv; 		 			/* von hieraus wurde gebootet	*/
	_WORD	*vdi_device;					/* vom AES benutzter VDI-Treiber */
	_VOID 	**nvdi_workstation;
	_VOID 	*shelw_doex;
	_VOID 	*shelw_isgr;
	_WORD 	version;		   				/* z.B. $0201 ist V2.1 */
	_WORD	release;		   				/* 0=alpha..3=release */
	_VOID	*_basepage;
	_WORD	*moff_cnt;
	_ULONG	shel_buf_len;
	_VOID	*shel_buf;
	_VOID	*notready_list;
	_VOID	*menu_up;
	_VOID	*menutree;
	_VOID	*desktree;
	_VOID	*desk_1stob;
	_ULONG	dos_magic;
	_ULONG	max_windn;
	_WORD	(**fsel)(_UBYTE *path, _UBYTE *name, _WORD *button, CONST _UBYTE *title);
	_LONG	dummy;
} AESVARS;
#endif

#ifndef _MAGX_COOKIE
#define _MAGX_COOKIE
typedef struct
{
	_LONG	config_status;
	DOSVARS	*dosvars;
	AESVARS	*aesvars;
} MAGX_COOKIE;
#endif

struct MgMcPrintDesc {
    _VOID       **printHdl;
    _BOOL _CDECL (*doPrintSetup)(_BOOL alwaysInteractively);
    _LONG       reserved[8];
};

typedef struct MgMcCookie {
    _WORD       vers;
    _WORD       size;
    _LONG       flags1;
    _VOID       *scrnPMPtr;
    _UBYTE      *updatePalette;
    _VOID       (*modeMac)(_VOID);
    _VOID       (*modeAtari)(_VOID);
    _VOID       (*getBaseMode)(_VOID);
    _VOID       (*getIntrCount)(_VOID);
    _VOID       (*intrLock)(_VOID);
    _VOID       (*intrUnlock)(_VOID);
    _VOID       (*callMacContext)(_VOID);
    _VOID       *atariZeroPage;
    _VOID       *macA5;
    _VOID       (*macAppSwitch)(_VOID);
    _VOID       (*controlSwitch)(_VOID);
    _LONG       hardwareAttr1;
    _LONG       hardwareAttr2;
    _VOID       *magiC_BP;
    _UBYTE      *auxOutName;
    _UBYTE      *auxInName;
    _VOID       (*auxControl)(_VOID);
    struct MgMcPrintDesc *printDescPtr;
    _LONG _CDECL (*configKernel)(_WORD function, _VOID *data);
    _LONG       reserved[14];
} MAGICMAC_COOKIE;

#endif /* __MAGIC_COOKIE_H__ */
