/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __MAGICTOOL_ALERTS_H__
#define __MAGICTOOL_ALERTS_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif


_WORD mtAlerts_Alert(_WORD def, CONST _UBYTE *str);
/* Alertbox, wie FormAlert, jedoch mit den Spezialitaeten von MagicDials 
 * Icon 1 - 3 sind die gewohnten Alert-Icons, andere Icons koennen mittels
 * SetIconslot gesetzt werden (siehe auch Modul AlertIcons). Besonderheiten:
 *
 * - Als Erweiterung gegenueber den normalen Alerts koennen bei mtAlerts
 *   16 Zeilen Text mit bis zu 60 Zeichen und bis zu 4 Buttons angegeben
 *   werden.
 *
 * - Die Icons koennen auch ueber Buchstaben addressiert werden, wie dies 
 *   bei den RCS INTERFACE gemacht wird. Die Groesse des Images ist nur 
 *   durch die Bildschirmgroesse begrenzt.  Die Ausloesetaste eines Buttons 
 *   kann wie gewohnt mit eine vorangehenden '[' angegeben werden.
 *   Beispiele:
 *
 *   '[3][Das ist ein normales|Alerticon][[Aha!]'
 *   '[C][Sollte mit SetIconslot|installiert sein...][[Genau!|[Jawoll!]'
 *   '[128][Auch Zahlen funktionieren![[Pfanntastisch!]'
 *
 * - In den Alerttexten (nicht den Buttons!!!) koennen folgende Steuer-
 *   zeichen enthalten sein:
 * 
 *   %D == Der komplette Text dieser Zeile wird doppelt unterstrichen
 *   %U == Der komplette Text dieser Zeile wird einfach unterstrichen
 *   %F == Der komplette Text dieser Zeile wird fett geschrieben.
 *
 *   Andere Steuerzeichen werden ignoriert und in den Text geschrieben.
 *   Beachten Sie bitte, dass der einfache und doppelte Unterstrich schon
 *   im Bereich des nachfolgenden Textes liegen. Hier sollte man eine
 *   Leerzeile '||' lassen.  Beispiele:
 *
 *   '[3][%DProgramm-Abbruch||Moechten Sie das Programm wirklich beenden?][[Ja|[Nein]';
 */

_WORD mtAlerts_PosAlert(_WORD x, _WORD y, _WORD def, CONST _UBYTE *str);
/* Wie Alert, die Alertbox wird jedoch an Position x,y dargestellt
 * (linke obere Ecke)
 */

_VOID mtAlerts_SetIconColor(_WORD color);
/* Setzt die Farbe des Icons. Damit kann man im Farbmodus wichtige Zustaende
 * besonders hervorheben.
 */

_VOID /* BITBLK */ *mtAlerts_Iconslot(_WORD slot);
/* Liefert den Inhalt des Iconslots mit Nummer slot; NIL wenn nicht belegt,
 * oder slot nicht existent.
 */

_VOID mtAlerts_SetIconslot(_WORD slot, _VOID /* BITBLK */ *icon);
/* Setzt ein Icon der Nummer 4..256. icon zeigt auf einen Bitblk, wie er
 * z.B. vom RCS erzeugt wird. 
 */

typedef enum {
	alert_left,
	alert_center,
	alert_right,
	alert_gemicon,
	alert_alticon
} AlertMode;

_VOID mtAlerts_ConfigAlert(AlertMode mode);
/* Konfiguriert, wie sich Alerts praesentieren.
 * left ==    Buttons linksbuendig
 * center ==  Buttons Zentriert (wie AES, default!)
 * right ==   Buttons rechtsbuendig
 * gemicon == Die 'normalen' GEM-Icons benutzen (default!)
 * alticon == Die alternativen Icons (etwas Mac-Like) benutzen
 */

#endif /* __MAGICTOOL_ALERTS_H__ */
