/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __MAGICTOOL_APPL_H__
#define __MAGICTOOL_APPL_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif

/*----------------------------------------------------------------------*
 * mtAppl       Basisfunktionen einer Applikation                       *
 *----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*
 *          Auf die folgenden Variablen NUR LESEND ZUGREIFEN!!!         *
 *----------------------------------------------------------------------*/

extern _WORD VDIHandle;         /* Handle der geoeffneten Workstation */
extern _WORD MaxWidth;          /* Maximale Anzahl Pixel waagerecht   */
extern _WORD MaxHeight;         /* Maximale Anzahl Pixel senkrecht    */
extern _WORD MaxColors;         /* Maximale Anzahl Farben             */
extern _WORD Bitplanes;         /* Anzahl der Bitplanes               */
extern _WORD CharWidth;         /* Werte von MagicAES.GrafHandle      */
extern _WORD CharHeight;        /* -"- */
extern _WORD BoxWidth;          /* -"- */
extern _WORD BoxHeight;         /* -"- */
extern _WORD AESFontid;         /* Font-ID des AES-Fonts              */
extern _WORD AESFontsize;       /* Hoehe des AES-Fonts                */
extern _BOOL AESFontMonospaced; /* Flag monospaced/proportional       */
extern GRECT deskSize;          /* Die Ausmasse des Desktops          */

extern _WORD PrivateWS;
    /* Fuer die Module mtDials, mtMenus und mtPopups: Hier handelt
     * es sich um eine private Workstation, die nur von diesen Modulen
     * benutzt werden sollte. Fuer allgemeine Anwendungen sollte die
     * Workstation VDIHandle (siehe oben) benutzt werden. Fuer merkwuerdige
     * Effekte bei Benutzung von PrivateWS keine Gewaehr!!!
     *
     * Die Workstation wird defaultmaessig so eingestellt:
     *
     * Fuellfarbe == weiss; Kein Rand beim Fuellen zeichnen; Zeichen-
     * alignment == Oberkante Zeichenzelle
     */

/* Nur zum Lesen! */
extern _BOOL multiGEM;					/* ist MultiGEM (uaeaeh) installiert? */
extern _BOOL multiTOS;					/* oder MultiTos */
extern _BOOL multiTask;					/* Allgemein Multitasking (Mag!X oder MultiTOS) */
extern _BOOL magIx;						/* ist Mag!X installiert? */
extern _UWORD magIxVer;					/* Welche Mag!X-Version */
extern _BOOL magicMac;					/* ist das hier ein Mac? */
extern _BOOL magicPC;                   /* oder vielleicht eher ein PC? */
extern _BOOL UseCatDelink;          /* Sollen unter MagicPC Links von Cat verfolgt werden? */
extern _BOOL IsMagiCScroll;				/* MagiC handelt lange Editfelder */

_VOID ApplInit(_VOID);
/* Meldet die Applikation beim AES an, oeffnet VDI-Workstation mit Handle
 * VDIHandle (siehe oben).  Wird im Modulkoerper ausgefuehrt und braucht
 * nicht weiter beachtet zu werden!
 */

_VOID ApplTerm(_WORD ret);
/* gibt alle benutzten Workstations frei und meldet die Applikation beim
 * AES wieder ab.  Gibt den Wert return an das aufrufende Programm zurueck.
 */

_VOID ApplPath(_UBYTE *path);
/* liefert den Pfad, von dem das Programm gestartet wurde */

_VOID ApplName(_UBYTE *name);
/* liefert den Namen, unter dem das Programm gestartet wurde. Dazu muss
 * allerdings das Programm mittels shel_write gestartet worden sein.
 * Der Atari-Desktop tut dies immer, Gemini setzt zumindest die Parameter
 * richtig.
 */

typedef enum _device {
	Screen,
	Plotter,
	Printer,
	Metafile,
	Camera,
	Tablett
} Device;

_WORD OpenWorkstation(Device device, _WORD w, _WORD h, _BOOL rc);
/* Oeffnet eine Workstation, mit allem Pipapo, returniert Handle.
 * Jede ueber diese Funktion geoeffnete Workstation muss mit CloseWorkstation
 * wieder geschlossen werden.  Ein Aufruf von ApplTerm erledigt dies auch
 * sehr elegant.
 *
 * Bei device == Screen wird eine virtuelle Workstation geoeffnet, alle
 * anderen oeffnen eine physikalische Workstation auf dem jeweiligen Geraet.
 * Dazu muss natuerlich ein GDOS geladen sowie der entsprechende Treiber und
 * die dazugehoerigen Fonts im ASSIGN.SYS angegeben sein. Der Treiber muss
 * auch fuer das GDOS ladbar sein, sonst gibts bloss Murks...
 * Bitte unbedingt den Returnwert auf > 0 vergleichen, sonst gibts bei
 * der Ausgabe Crash!
 *
 * Die Parameter w und h spielen nur eine Rolle, wenn ein Matrixdrucker
 * angesprochen werden soll. Hier kann die maximale Breite und Hoehe in
 * Pixel angegeben werden.
 *
 * Wenn rc == TRUE werden Rasterkoordinaten verwendet, ansonsten werden
 * NDC-Koordinaten benutzt.
 *
 * Alle hier geoeffneten Workstations werden beim Aufruf von ApplTerm wieder
 * geschlossen
 */

_VOID CloseWorkstation(_WORD *handle);
/* Schliesst eine Workstation */

_ULONG VqGdos(_VOID);
/* Retuniert den GDOS-Wert (siehe MagicVDI). Prueft auch, ob es sich um ein
 * ABC-GEM handelt. In diesem Falle retuniert die Prozedur _LONG (21) fuer
 * GEM 2.1
 */

/*----------------------------------------------------------------------*
 * Maus-Handling: GEM hat im allgemeinen etwas Probleme mit der Maus.   *
 *                Deshalb ist es sinnvoll, die Maus zentral ueber ein    *
 *                Modul zu steuern.                                     *
 *----------------------------------------------------------------------*/

_VOID MouseOn(_VOID);
_VOID MouseArrow(_VOID);
_VOID MouseCursor(_VOID);
_VOID MouseBusy(_VOID);
_VOID MouseFinger(_VOID);
_VOID MouseHand(_VOID);
_VOID MouseThincross(_VOID);
_VOID MouseThickcross(_VOID);
_VOID MouseOutline(_VOID);
_VOID MouseUser(_VOID);
/* Diese Routinen schalten die Maus mit dem jeweilige Bild EIN */

_VOID MouseOff(_VOID);
/* Schaltet die Maus AUS */

_VOID UserMouse(_WORD *form);
/* Setzt eine User-Definierte Maus. Sichtbar erst mit MouseUser! */

_VOID MouseState(_WORD *form, _BOOL *sichtbar);
/* Liefert den Modulinternen Status der Maus. Dieser Status muss nicht der
 * Wirklichkeit entsprechen, da andere Applikationen (ACC oder unter
 * MultiGEM) die Maus veraendert haben koennten. Leider gibt es seitens des
 * GEM keine Moeglichkeit festzustellen, welche Mausform aktiv ist, bzw.
 * ob die Maus auch sichtbar ist.
 */

_VOID StoreMouse(_VOID);
/* Merkt sich die augenblickliche Mausform und Sichtbarkeit. Die Maus
 * kann nun veraendert werden, eingeschaltet, ausgeschaltet wie auch immer.
 * Mit RestoreMouse wird die gemerkte Maus wieder hergestellt. Das gilt
 * auch fuer eine Userdef-Maus! Die Prozedur ist bis zu einer Tiefe von
 * 64 schachtelbar. Die MagicLib- und MagicTools-Module benutzen dies, 
 * wenn sie (die Module) eine bestimmte Mausform haben wollen. So merkt 
 * sich z.B. der Fontselector aus mtFonts, wie die Maus vorher war und 
 * restauriert sie wieder; ebenso verfahren die mtAlerts. 
 */

_VOID RestoreMouse(_VOID);
/* Restauriert eine mit StoreMouse gemerkte Mausform und Sichtbarkeit. */

_VOID SetMouse(_WORD x, _WORD y);
/* Ein Schmankerl, und dazu noch ganz legal: Mausposition festlegen!*/

_WORD appl_locate(CONST _UBYTE *name, _BOOL startIt);
/* Applikation suchen und gegebenenfalls nachstarten */

_WORD appl_locate_args(CONST _UBYTE *name, CONST _UBYTE *args);


_VOID Objc_draw(OBJECT *tree, _WORD start, _WORD depth, CONST GRECT *clip);
_VOID Form_dial(_WORD flag, CONST GRECT *small, CONST GRECT *big);
_VOID Wind_calc(_WORD mode, _UWORD comp, CONST GRECT *in, GRECT *out);


#endif /* __MAGICTOOL_APPL_H__ */
