/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __MAGICTOOL_AREA_H__
#define __MAGICTOOL_AREA_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif
#ifndef __PORTVDI_H__
#include <portvdi.h>
#endif

struct _Area;
typedef struct _Area Area, *AREA;

#ifdef __MAGICTOOL_AREA_IMPLEMENTATION__
struct _Area {
	_WORD x;
	_WORD y;
	_WORD w;
	_WORD h;
	_BOOL init;
	_ULONG size;
	MFDB mfdb;
};
#else
struct _Area { int dummy; };
#endif


_BOOL mtArea_NewArea(AREA *a);
/* Legt neue AREA-Variable an, FALSE wenn nicht gelungen  */

_VOID mtArea_DisposeArea(AREA *a);
/* Gibt die AREA-Variable frei, falls noch Bildspeicher alloziert ist,
 * wird dieser ebenfalls freigegeben
 */

_VOID mtArea_FreeArea(AREA a);
/* Gibt den allozierten Bildspeicher frei */


/* hndl == Handle der Workstation, die beim Blitten benutzt werden soll.
 *        Wichtig fuer Clipping!  Bisher wurde mtAppl.VDIHandle benutzt,
 *        was bei eingeschaltetem Clipping fuer mtAppl.VDIHandle natuerlich
 *        voll in die Hose geht!  Hp 23.10.91
 *
 */

_BOOL mtArea_SaveArea(_WORD hndl, AREA a, CONST GRECT *rect);
/* Sichert einen Bildschirmbereich ins RAM. FALSE, wenn nicht genuegend
 * RAM zur Verfuegung stand. Ist bereits ein Bildspeicher alloziert und
 * dieser fuer das zu sichernde Teilstueck zu klein, wird dieser freigegeben
 * und ein neues Teilsstueck alloziert.
 */

_VOID mtArea_CopyArea(_WORD hndl, AREA a, _WORD x, _WORD y);
/* Kopiert einen mit SaveArea gespeicherten Bereich an Position x, y */

_VOID mtArea_RestoreArea(_WORD hndl, AREA a);
/* Blittet einen mit SaveArea gespeicherten Bereich wieder auf die
 * Originalposition am Schirm
 */

_VOID mtArea_MoveArea(_WORD hndl, AREA a, _WORD xmove, _WORD ymove, _WORD *x, _WORD *y);
/* "Bewegt" einen AREA-Bereich um eine Anzahl Pixel. Es wird dabei entlang 
 * der X- und Y-Achse verschoben.  'xmove' ist die Verschiebung entlang der
 * X-Achse (negative Werte verschieben nach LINKS, positive nach RECHTS);
 * 'ymove' ist die Verschiebung entlang der Y-Achse (negativ verschiebt
 * nach OBEN, positiv nach UNTEN).  Diese Aenderung ermoeglicht ein viel 
 * einfacheres Handling durch die Klientmodule mtDials und mtPopups.
 * Es wird wie folgend vorgegangen:
 *
 *  MoveArea (handle, area, 0, -5, x, y);
 *
 *  +----------------------------+
 *  |XXXXXXXXXXXXXXXXXXXXXXXXXXXX|
 *  +----------------------------+
 *  |                            |
 *  |                            |
 *  |                            |
 *  |                            |
 *  |                            |
 *  +----------------------------+
 *  |############################|
 *  +----------------------------+
 *
 * Zuerst wird die alte Bildschirmflaeche (!=) im in Frage kommenden Bereich
 * wieder hergestellt.  Die gesamte Flaeche wird intern um 5 Pixel nach unten
 * geschoben, die unteren 5 Pixel der Flaeche (!=) gehen dabei verloren. 
 * Anschliessend wird vom Bildschirm an der aktuellen Position 5 Pixel-Reihen
 * in die oberen Pixel der Flaeche (X) kopiert.
 *
 * In x,y wird die neue Position des Rechtecks geliefert.
 */

_VOID *mtArea_Data(AREA a);

#endif /* __MAGICTOOL_AREA_H__ */
