/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __MAGICTOOL_DIALS_H__
#define __MAGICTOOL_DIALS_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif

/* Procedure-Type for DialDo for overloading */

typedef _WORD (*DialDoProc)(OBJECT *tree, _WORD start);


#define MoveBox          17 /* Neue Objekttypen, kompatibel zu */
#define SpecButton       18 /* Flydials von Julian Reschke     */
#define SpecText         19
#define FrameBox         20
#define HelpButton       21 /* DERZEIT NICHT VON MAGIC UNTERSTUETZT!!! */
#define CircleButton     22
#define ThreeState       23 /* 3-Status-Button! State 3 == Selected & Checked! */
#define LongEdit         24 /* Erweiterter Objekttyp fuer lange Editfelder */
#define UndoButton       31

#define UndoButtonFlag   0x4000  /* Flag 14 kennzeichnet einen Undobutton */
#define HelpButtonFlag   0x8000  /* Flag 15 kennzeichnet einen Helpbutton */

/*-----------------------------------------------------------------------*/

_BOOL mtDials_NewDial(OBJECT *tree);
/* Initialisiert die internen Dialogvariablen und muss vor Gebrauch der
 * Dialogfunktionen fuer jeden Objektbaum einmal aufgerufen worden sein.
 * Bei FALSE stehen die Moeglichkeiten der MagicDials fuer den Objektbaum
 * nicht zur Verfuegung. Der Aufruf einer Routine aus diesem Modul kann
 * dann einen Systemabsturz zur Folge haben.  Ein mehrfaches Aufrufen
 * fuer einen Objektbaum wird von der Prozedur ignoriert.
 */

_VOID mtDials_DisposeDial(OBJECT *tree);
/* Loescht die interne Dialogvariable */

/*-----------------------------------------------------------------------*/

_VOID mtDials_ObjcExtype(OBJECT *tree, _WORD entry, _WORD extyp);
/* Setzt erweiterten Typ */

_VOID mtDials_GetObjcExtype(OBJECT *tree, _WORD entry, _WORD *extyp, _WORD *typ);
/* Liefert den erweiterten Typ, auch von G_PROGDEF-Objekten, sofern diese
 * mit mtXobjects installiert wurden
 */

_UWORD mtDials_GetKbdState(_VOID);
/* Diese Prozedur liefert den Status der Sondertasten (Shift, Alternate,
 * Control etc. der bei _Rueckkehr_ aus DialDo gueltig war. Kann dazu benutzt
 * werden, um im Zusammenhang mit SetUserkey bestimmte Aktionen auszuloesen.
 * Idee: Dirk Steins (wer sonst?)
 */

#define CSCREEN        0
#define CMOUSE         1
#define CPOS           2

_VOID mtDials_DialCenter(OBJECT *tree, _WORD flag, _WORD x, _WORD y, GRECT *rect);
/* Zentriert den Dialog.  Folgende Aenderung ist ab 3.30 wirksam: Der
 * Dialog wird immer auf die im Eselsohr-Popup definierten Werte abgestimmt,
 * ES SEI DENN, 'flag' == CPOS!!!  CSCREEN und CMOUSE haben somit keine
 * Wirkung mehr, und sind nur noch aus Kompatibilitaetsgruenden vorhanden. 
 * Default-Modus ist 'Zentrieren'.  
 *
 * Schon laenger drin, aber nicht ausdruecklich erwaehnt: Bei Modus 'Zentrieren'
 * wird der virtuelle Screendriver BIGSCREEN2 von Julian Reschke (SciLab)
 * unterstuetzt, d.H, ist ein Bildschirmtreiber ueber den VirtualScreen-Cookie
 * installiert, wird natuerlich nicht in den virtuellen Screen zentriert,
 * sondern in dem gerade Sichtbaren Teil.
 * 
 * CPOS  Es wird auf die in x, y uebergebene Pos gesetzt. Ueberbuegelt
 *       auch ein mit dem Eselsohr-Popup eingestellte Werte.
 * CSCREEN, CMOUSE werden nicht mehr beachtet, sondern die Einstellungen
 * des globalen Konfigs uebernommen.
 *
 * rect enthaelt nach Aufruf die entsprechende Flaeche des Dialogs
 */


#define DSTART         0
#define DGROW          1
#define DSHRINK        2
#define DFINISH        3
#define DDISABLE       4
#define DENABLE        5


_VOID mtDials_DialForm(OBJECT *tree, _WORD flag, CONST GRECT *small, CONST GRECT *big);
/* wie bei FormDial, nur dass bei DSTART tatsaechlich der Hintergrund gebuffert
 * wird und bei DFINISH wieder freigegeben wird.  small und big kommen aus-
 * schliesslich bei DGROW und DSHRINK zum Einsatz.
 *
 * Zusaetzliche Flags DDISABLE und DENABLE:
 * DDISABLE: Der Dialog wird im "Desktop-Grau" dargestellt, um zu verdeut-
 * lichen, dass seine Funktionen derzeit nicht zu erreichen sind. Sinnvoll,
 * wenn man ueber einen Dialog einen anderen legt - der untere wird grau (nur
 * die gesetzen Pixel).
 * DENABLE: Stellt nach DDISABLE den Dialog wieder her.
 */

_VOID mtDials_DialDraw(OBJECT *tree, _WORD entry, _WORD depth, CONST GRECT *clip, _BOOL clipping);
/* Zeichnet den uebergebenen Dialog, handling wie ObjcDraw.
 * clip ist ein optionales Clipping-Rechteck, das beachtet wird, wenn
 * clipping TRUE uebergeben wird.
 */

_VOID mtDials_DialChange(OBJECT *tree, _WORD entry, _WORD state, CONST GRECT *clip, _BOOL clipping, _BOOL set, _BOOL draw);
/* MagicDial-Version von ObjcChange:
 * tree ==       Adresse des Objektbaums
 * entry ==      Nummer des Objekts
 * state ==      Status-Flag, das geaendert werden soll
 * clip ==       Clipping-Rechteck
 * clipping ==   TRUE, wenn geclipt werden soll
 * set ==        TRUE, wenn Status gesetzt werden soll
 * draw ==       TRUE, wenn das Objekt neugezeichnet werden soll
 */

_WORD mtDials_DialDo(OBJECT *tree, _WORD edit);
/* form_do fuer MagicDials, edit ist die Nummer des ersten EDITABLE-
 * Objekts. Hat der Dialog kein EDITABLE, sollte edit aus Geschwindig-
 * keitsgruenden auf -1 gesetzt werden.
 */

/*------------------------------------------------------------------------*
 * Definitionen fuer Userhandler                                           *
 *------------------------------------------------------------------------*/

typedef _UWORD (*mtDials_UserHandler)(OBJECT *tree, _WORD mx, _WORD my, _UBYTE scan, _UWORD kbshift, _UWORD event, _WORD edobj, _VOID *special);

/* Eine Prozedur dieses Typs wird - sofern sie fuer einen Dialog installiert
 * wird - von DialDo bei Abarbeitung eines Dialogs aufgerufen. Die Prozedur
 * wird entsprechend dem Callmode bei der Installation aufgerufen.
 *
 *  tree ==       Adresse des Objektbaumes
 *  mx, my ==     Mausposition
 *  scan ==       Scancode
 *  kbshift ==    Status der Sondertasten
 *  event ==      welcher Event wars (Mausklick, Tastatur, keins von beidem)
 *  edobj ==      aktuelles Edit-Objekt
 *
 * Als Returnwert ist ein BITSET vorgesehen. Derzeit gibt es aber noch keine
 * Flags, die von DialDo ausgewertet werden. Ist das BITSET leer, macht
 * DialDo weiter wie gewohnt, ist es nicht leer, bricht DialDo mit -1 ab.
 */
 
#define CallByEvent    0x0001
        /* UserHandler sofort nach einem Event rufen */
#define CallByHandling 0x0002
        /* UserHandler nach der Objektbehandlung durch DialDo aufrufen */ 
#define CallByTimer    0x0004
        /* DialDo verwendet normalerweise keinen Timer-Event bei seiner
         * Eventabfrage. Ist jedoch ein UserHandler mit CallByTimer 
         * installiert, wird ein MultiEvent mit Timer bei 10 Millisekunden
         * ausgefuehrt. Es wird wie bei CallByEvent verfahren, also der User-
         * Handler vor dem Behandeln des Objekts durch DialDo aufgerufen.
         */
#define CallByRect     0x0008
        /* Hiermit wird ein Userhandler als MouseRect-Event installiert.
         * Es wird wie bei CallByEvent verfahren, also der UserHandler vor
         * dem Behandeln des Objekts durch DialDo aufgerufen.
         */
#define CallByMessage  0x0010
        /* Hier werden auf Message-Ereignisse reagiert.
         * Es wird wie bei CallByEvent verfahren, also der UserHandler vor
         * dem Behandeln des Objekts durch DialDo aufgerufen.
         */

_VOID mtDials_InstallHandler(OBJECT *tree, mtDials_UserHandler proc, _WORD callmode, _WORD timer, _WORD rmode, GRECT *rect, _VOID *mesag, _VOID *special);
/* Installiert eine Prozedur vom Typ UserHandler. 'callmode' bestimmt, in 
 * welcher Weise die Prozedur aufgerufen wird (CallByEvent, CallByHandling,
 * CallByTimer bzw. CallByRect).
 *
 * tree ==       Adresse des Objektbaums
 * proc ==       Die UserHandler-Prozedur
 * callmode ==   Die Art der Behandlung
 * timer ==      Wert fuer Timer-Event
 * rmode ==      Rechteck-Event-Modus (0 == Betreten; 1 == Verlassen)
 * rect ==       Rechteck fuer den Event
 */

_VOID mtDials_RemoveHandler(OBJECT *tree);
/* Deinstalliert die Handleprozedur. */

_BOOL mtDials_AbortOnClick(OBJECT *tree);
/* Legt fest, dass der Dialog bei Click ausserhalb abgebrochen wird */

#define UseSolid       0x0001
#define UseEdit        0x0002
#define UseALT         0x0004
#define UseGrowbox     0x0008
#define UseCenter      0x0010
#define UseMouse       0x0020
#define UsePos         0x0040
#define Res7           0x0080
#define Res8           0x0100
#define Res9           0x0200
#define Res10          0x0400
#define Res11          0x0800
#define Res12          0x1000
#define Res13          0x2000
#define Res14          0x4000
#define UseConfig      0x8000

_VOID mtDials_DialConfig(_UWORD flag, _BOOL set);
/* Setzt die Konfiguration der MagicDials.
 *
 * UseSolid ==    TRUE == Solides Verschieben (Default), sonst mit Rahmen 
 * UseEdit ==     TRUE == Integriertes ObjcEdit (Default), sonst AES benutzten
 * UseALT ==      TRUE == ALTERNATE loest Button aus (Default), sonst CONTROL
 * UseGrowbox ==  TRUE == Grow/ und Shrink-Boxen benutzen (Default)
 * 
 * Die folgenden Flags definieren, wohin die Dialogbox am Bildschirm 
 * gezeichnet wird, egal was in DialCenter angegeben wurde. Alle Flags 
 * sind nicht Default, damit eine grundsaetzliche Belegung ueber DialCenter 
 * gegeben ist.
 *
 * UseCenter ==   TRUE == Dialogbox wird auf dem Schirm zentriert
 * UseMouse ==    TRUE == Dialogbox wird um die Mausposition zentriert
 * UsePos ==      TRUE == Dialogbox wird an letzter Position gezeichnet
 *
 * Res7..Res14 == Reserviert fuer zukuenftige Anwendungen
 *
 * UseConfig ==   TRUE == Konfig-Popup zulassen (Default) 
 */

_UWORD mtDials_GetDialConfig(_VOID);
/* Liefert das augenblickliche Konfigurations-Set */


_VOID mtDials_SetUserkey(OBJECT *tree, _WORD object, _UBYTE scan, _UWORD kbstate, _BOOL action, _BOOL set);
/* Setzt die vom Benutzer konfigurierbaren Tasten. Sinn und Zweck der Uebung
 * ist, dass der Dialog bei bestimmten Tastendruecken ohne direkte Wirkung auf
 * den Dialog verlassen wird. Die Liste der Tasten wird fuer jeden Dialog
 * extra aufgebaut. Einmal aufgebaut, bleibt sie bis DisposeDial bestehen.
 *
 * tree ==       Adresse des Objektbaumes
 *
 * object ==     Die Nummer des Objekts, welches beim korrespondierenden
 *              Tastendruck von DialDo returniert wird. ACHTUNG! Es wird
 *              nicht kontrolliert, ob das Objekt auch existiert!
 *
 * scan ==       Scancode der Taste die den Ruecksprung aus DialDo veran-
 *              lassen soll.
 *
 * kbstate ==    Status der Sondertasten, der zusaetzlich zum Scancode
 *              vorhanden sein muss.
 *
 * action ==     Bei TRUE wird eine Bildschirmaktion ausgefuehrt, so als
 *              ob das Objekt angeklickt wurde.
 *
 * set ==        Bei TRUE wird der Tastencode in die Liste eingetragen,
 *              ansonsten geloescht.
 */

_VOID mtDials_ResetUserkeys(OBJECT *tree);
/* Loescht saemtliche Userkey-Eintraege fuer einen Objektbaum */


/* Fuer die UserObjects werden einige objektunabhaengige Routinen exportiert.
 * Der geneigte Benutzer sollte eine genaue Vorstellung von dem haben,
 * was er tun will; insbesondere sollte er den Sourcecode von mtDials
 * aufmerksam studieren...
 */

_BOOL mtDials_DialCharTable(_UBYTE *ch);
/* Ruft einen Standarddialog mit einer Zeichenauswahl auf. 
 * TRUE: Zeichen wurde ausgewaehlt, steht dann in ch
 */

/* Erweiterungen fuer Fensterdialoge */

_BOOL mtDials_HandleEvent(_UWORD event,
	_WORD mx,
	_WORD my,
	_UWORD *button,
	_WORD *taste,
	_UWORD *kbshift,
	_UBYTE *scan,
	_WORD *clicks,
	OBJECT *tree,
	_BOOL editable,
	_WORD *edit,
	_WORD *pos,
	_WORD *entry);
/* Behandelt den Event fuer einen Baum
 * Wird fuer WinDials benoetigt
 */

_VOID mtDials_PrepDial(OBJECT *tree, _WORD start, _BOOL *editable, _WORD *pos, _WORD *edit);
/* Bereitet den Dialog vor, setzt die Userkeys und gibt die
 * Editwerte zurueck. Wird fuer WinDials benoetigt
 */

_VOID mtDials_ObjcEdit(OBJECT *tree, _WORD object, _WORD *pos, _WORD ch, _WORD kind);
/* Das ObjcEdit der MagicDials. Wird fuer WinDials benoetigt
 */

_VOID mtDials_DisableMenu(OBJECT *menu, _BOOL disable);
/* Disablet ein Menu bei einem modalen Fensterdialog
 * Wird fuer WinDials benoetigt
 */

DialDoProc mtDials_OverloadDialDo(DialDoProc newDialDo);

_BOOL mtDials_IsOverloadedDialDo(_VOID);

_BOOL mtDials_Init(_VOID);

#endif /* __MAGICTOOL_DIALS_H__ */
