/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __MAGICTOOL_TEXTFILES_H__
#define __MAGICTOOL_TEXTFILES_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __MAGICTOOL_TEXTFILES_H__
#include <mtttextfi.h>
#endif
#ifndef __FILE_IO_H__
#include <file_io.h>
#endif

typedef enum { READ, WRITE, APPEND } Textmode;

struct _textfile;
typedef struct _textfile Textfile, *TEXTFILENO;

#ifndef __MAGICTOOL_TEXTFILES_IMPLEMENTATION__
struct _textfile { int dummy; };
#endif


F_ERROR mtTextfiles_OpenTextfile(CONST _UBYTE *name, Textmode mode, _ULONG puffer, TEXTFILENO *text);
/* Oeffnet eine Textdatei. puffer gibt die Groesse des zu benutzenden Puffer-
 * speichers an. 
 */ 

_VOID mtTextfiles_CloseTextfile(TEXTFILENO *text);
/* Schliesst eine Textdatei */

_VOID mtTextfiles_FlushBuffer(TEXTFILENO text);
/* Schreibt den Textpuffer auf Diskette */

_VOID mtTextfiles_Reset(TEXTFILENO text);
/* Setzt den Dateizeiger an den Dateianfang */

_ULONG mtTextfiles_Textpos(TEXTFILENO text);
/* Ermittelt die Position des Dateizeigers in Bytes vom Anfang der Datei */

_VOID mtTextfiles_SetTextpos(TEXTFILENO tex, _ULONG pos);
/* Setzt die Position des Dateizeigers in Bytes vom Anfang der Datei */

_BOOL mtTextfiles_EndofText(TEXTFILENO text);
/* TRUE, wenn bei einem zum Lesen geoeffneten Text das Dateiende ereicht
 * worden ist.
 */

_VOID mtTextfiles_WriteChar(TEXTFILENO text, _UBYTE c);
/* Schreibt Zeichen */

_VOID mtTextfiles_WriteLine(TEXTFILENO text, CONST _UBYTE *s); 
/* schnelle Stringausgabe, kein LF */

_VOID mtTextfiles_WriteLn(TEXTFILENO text);
/* CR + LF */

_VOID mtTextfiles_WriteCard(TEXTFILENO text, _UWORD wert, _UWORD len);
_VOID mtTextfiles_WriteInt(TEXTFILENO text, _WORD wert, _UWORD len);
_VOID mtTextfiles_WriteLongCard(TEXTFILENO text, _ULONG wert, _UWORD len);
_VOID mtTextfiles_WriteLongInt(TEXTFILENO text, _LONG wert, _UWORD len);
_VOID mtTextfiles_WriteReal(TEXTFILENO text, double wert, _UWORD len);
/* Schreibt die entspechenden Werte mit Laenge len */


_VOID mtTextfiles_ReadSpec(TEXTFILENO text, CONST _UBYTE *check, _UBYTE *str, size_t strsize);
/* Liest solange Zeichen bis das gelesene Zeichen nicht in check ist.
 * Fuehrende Zeichen die nicht in check sind werden uebersprungen.
 */

_UBYTE mtTextfiles_ReadChar(TEXTFILENO text);
/* Liest Zeichen */

_VOID mtTextfiles_ReadLine(TEXTFILENO text, _UBYTE *s, size_t strsize);
/* Liest Zeichen ein bis ein CR oder LF auftritt.  Achtung! Der 
 * Dateizeiger steht auf dem Zeichen, das den Abbruch ausgeloest hat.
 */

_VOID mtTextfiles_ReadLn(TEXTFILENO text);
/* Liest Zeichen, bis ein CR, LF oder CR/LF auftritt.
 * Achtung! Der Dateizeiger steht auf dem Zeichen nach dem LF!
 */

_VOID mtTextfiles_Skip(TEXTFILENO text, _UBYTE cSkip);

_UWORD mtTextfiles_ReadCard(TEXTFILENO text);
_WORD mtTextfiles_ReadInt(TEXTFILENO text);
_ULONG mtTextfiles_ReadLongCard(TEXTFILENO text);
_LONG mtTextfiles_ReadLongInt(TEXTFILENO text);
double mtTextfiles_ReadReal(TEXTFILENO text);
/* Liest die entspechenden Werte ein. Fuehrende Blanks werden
 * ueberlesen, ein Blank oder ein Zeichen kleiner als Blank
 * bricht die Eingabe ab.
 */

#endif /* __MAGICTOOL_TEXTFILES_H__ */
