/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __MAGICTOOL_UTILS_H__
#define __MAGICTOOL_UTILS_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif


typedef OBJECT *tObjcTree;
/* Wird beim direkten manipulieren des Objektbaumes verwendet. 
 * Beispiel:
 * 
 * VAR tree:== tObjcTree;
 *
 * IF SELECTED IN tree^[object].obState THEN ...
 */

/* Modifizieren der obState und obFlags */

EXTERN _VOID SetObjFlags(OBJECT *tree, _WORD obj, _UWORD flags);
/* Setzt bit in tree^[entry].obFlags */

EXTERN _VOID DelObjFlags(OBJECT *tree, _WORD obj, _UWORD flags);
/* Loescht bit in tree^[entry].obFlags */

EXTERN _VOID EnableObjFlags(OBJECT *tree, _WORD obj, _UWORD flags, _BOOL setit);
/* Setzt bzw. loescht bit in tree^[entry].obFlags, in Abhaengigkeit
 * von set: Bei TRUE wird gesetzt, sonst geloescht
 */

EXTERN _BOOL GetObjFlags(OBJECT *tree, _WORD obj, _UWORD flags);
/* TRUE, wenn bit in tree^[entry].obFlags */

EXTERN _VOID SetObjState(OBJECT *tree, _WORD obj, _UWORD state);
/* Setzt bit in tree^[entry].obState */

EXTERN _VOID DelObjState(OBJECT *tree, _WORD obj, _UWORD state);
/* Loescht bit in tree^[entry].obState */

EXTERN _BOOL GetObjState(OBJECT *tree, _WORD obj, _UWORD state);
/* TRUE, wenn bit in tree^[entry].obState */

EXTERN _VOID EnableObjState(OBJECT *tree, _WORD obj, _UWORD state, _BOOL setit);
/* Setzt bzw. loescht bit in tree^[entry].obState, in Abhaengigkeit
 * von set: Bei TRUE wird gesetzt, sonst geloescht
 */

#ifndef NO_MACROS

#define DelObjState(tree, obj, state) ((tree)[obj].ob_state &= ~(state))
#define SetObjState(tree, obj, state) ((tree)[obj].ob_state |= (state))


#define GetObjState(tree, obj, state) (((tree)[obj].ob_state & (state)) ? TRUE : FALSE)
#define EnableObjState(tree, obj, state, cond) \
	((cond) ? SetObjState(tree, obj, state) : DelObjState(tree, obj, state))
#define GetObjFlags(tree, obj, flags) (((tree)[obj].ob_flags & (flags)) ? TRUE : FALSE)
#define EnableObjFlags(tree, obj, flags, cond) \
	((cond) ? SetObjFlags(tree, obj, flags) : DelObjFlags(tree, obj, flags))

#endif /* NO_MACROS */


/* Hilfsfunktionen fuer Three-State-Buttons */

#define NOCHANGE 0       /* gepunktet  */
#define CLEAR    1       /* geloescht   */
#define SETNEW   2       /* angekreuzt */

_WORD GetThreeState(tObjcTree tree, _WORD entry);
/* Holt Status eines 3-State-Buttons */

_VOID SetThreeState(tObjcTree tree, _WORD entry, _WORD val);
/* Setzt Status eines 3-State-Buttons */

/* Manipulieren von String-Objekten */

_VOID ObjcString(tObjcTree tree, _WORD entry, _UBYTE *str, size_t strsize);
/* Liefert den String der folgenden Objekte: 
 * GBOXCHAR, GTEXT, GBOXTEXT, GFTEXT, GFBOXTEXT, GBUTTON,
 * GSTRING,  GTITLE
 */

_UBYTE *ObjcStringAdr(tObjcTree tree, _WORD entry);
/* Liefert einen Zeiger auf den String des Objekts */

_VOID SetObjcString(tObjcTree tree, _WORD entry, CONST _UBYTE *str);
/* Setzt den String der folgenden Objekte indem str direkt in die Resource
 * kopiert wird:  GBOXCHAR, GTEXT, GBOXTEXT, GFTEXT, GFBOXTEXT, GBUTTON,
 * GSTRING,  GTITLE
 *
 * Ist der String kuerzer als die maximale Laenge wird mit Blanks bis zur
 * maximalen Laenge aufgefuellt. Dies gilt nicht fuer TEDINFO-Objekte!
 */

_VOID SetObjcStringAdr(tObjcTree tree, _WORD entry, CONST _UBYTE *str, size_t strsize);
/* Setzt den String der folgenden Objekte durch Austausch des entspr.
 * Zeigers:  GBOXCHAR, GTEXT, GBOXTEXT, GFTEXT, GFBOXTEXT, GBUTTON,
 * GSTRING,  GTITLE
 */

_VOID ObjcStrLen(tObjcTree tree, _WORD entry, _WORD *textLen, _WORD *tmpltLen);
/* Liefert die Laenge eine String-Objekts, bei TEDINFO-Objekten auch die
 * Laenge des Templates; sonst ist tmpltLen -1.  Idee: Dirk Steins
 */

_UBYTE *GetObjcStringAdr(OBJECT *tree, _WORD entry);


/* Objektpositionen, Flaechen, Rahmen etc. 
 * 
 * SEHR WICHTIG!!!
 * ---------------
 * ALLE Rechtecke sind jeweils Koordinate (x,y) sowie Breite und Hoehe!!!
 */

_VOID ObjcPos(tObjcTree tree, _WORD entry, _WORD *x, _WORD *y);
/* berechnet die Position eines Objekts. Ist zwar das gleiche wie 
 * ObjcOffset, aber wesentlich schneller, da es direkt im Objektbaum
 * operiert, und nicht erst ueber einen Trap2 gehen muss.
 */

_WORD ObjcParent(tObjcTree objc, _WORD entry);
/* Liefert das Parent-Objekt zu entry */

_VOID ObjcRect(tObjcTree tree, _WORD entry, GRECT *rect);
/* Liefert Objektkoordinaten des Objektes relativ zum Parent-Objekt.
 * Idee: Dirk Steins 
 */

_VOID SetObjcRect(tObjcTree tree, _WORD entry, CONST GRECT *rect);
/* Setzt Koordinaten (obX, obY, obWidth und obHeight des Objekts auf
 * die in rect uebergebenen Werte. (Relativ zum Parent-Objekt)
 * Idee: Dirk Steins
 */ 

_VOID ObjcArea(tObjcTree tree, _WORD entry, GRECT *rect);
/* Liefert Objektkoordinaten des Objekts relativ zum Bildschirm Ursprung */

_WORD ObjcFrame(tObjcTree tree, _WORD obj);
/* Liefert die Rahmendicke eines Objekts. Dabei werden Stati wie OUTLINED und
 * Shadowed beruecksichtigt. Ist das Ergebnis kleiner als 0, liegt der Rahmen
 * AUSSERHALB des Objekts, ansonsten INNERHALB!
 */

_VOID CalcArea(tObjcTree tree, _WORD entry, GRECT *rect);
/* Berechnet Umgebungsrechteck des Objektes. Die Objektstati werden 
 * dabei beruecksichtigt.
 */

#define ObjSearchType     0
#define ObjSearchState    1
#define ObjSearchFlags    2

_WORD ScanFlags(tObjcTree tree, _WORD set, _WORD entry, _UWORD flag);
/* Sucht einen bestimmtes Flag im Objektbaum, abhaengig von set. 
 * Wenn set == SearchType wird ein obType gesucht, bei set == SearchState wird 
 * ein Flag im obState gesucht, bei set == SearchFlags in obFlags
 */

_BOOL ScanMenu(tObjcTree tree, _WORD scan, _UWORD kbshift, _WORD *title, _WORD *item);
/* Scannt einen Menuebaum nach bestimmten Eintraegen. So kann z.B. der User
 * die Tastenbelegungen aendern, ohne das das Programm geaendert werden muss...
 * Die Tasteneintraege muessen allerdings in einer besonderen Form vorliegen!
 * Es koennen normale Tasten, Tasten mit Control und Tasten mit Alternate
 * vorkommen.  Ein normaler Tastencode muss - wie beim STE/TT-Desktop - in 
 * eckige Klammern eingefasst sein. Fuer Control wird das Zeichen ^ verwendet,
 * fuer Alternate das Zeichen, welches im Fuller-Feld eines Fensters steht.
 * Beispiel:
 *
 *   Eintrag Normal    [M]
 *   Eintrag Control   ^M
 *   Eintrag Alternate #M
 *
 * Die Routine liefert TRUE, wenn ein Eintrag gefunden wurde.
 *
 * tree =       Adresse des Menuebaums
 * first =      Erster Eintrag ab dem gesucht werden soll
 * last =       Letzter Eintrag in dem gesucht werden soll
 * scan =       Scancode der gedrueckten Taste
 * kbshift =    Status der Sondertasten
 * titel =      der gefundene Menuetitel oder -1
 * item =       der gefundene Menueeintrag oder -1
 */


/* Vermischtes */

_UBYTE CharCode(_WORD scan, _UWORD kbshift);
/* Liest die Tastaturtabelle aus und liefert anhand Scancode und Sonder-
 * tastenstatus den ASCII-Code.
 */

_WORD ScanCode(_UBYTE ch);
/* Umkehrfunktion zu CharCode. Sucht anhand eines Chars nach dem Scancode,
 * 0 wenn nicht gefunden. Achtung: Es existieren nicht zu allen Zeichen
 * Scancodes, sondern nur fuer jede Taste!!!
 */

_BOOL DoubleClick(_WORD *value);
/* Maskiert Bit15 aus value aus; liefert TRUE, wenn es gesetzt war.
 * Sinn der Uebung: DialDo bzw. FormDo liefern bei einem Doppelklick
 * das angeklickte Objekt mit gesetzem Bit15.
 */

_VOID Bounce(_VOID);
/* "Entprellt" die Maustaste. d.h. wartet, bis keine Maustaste mehr
 * gedrueckt wird.
 */

_WORD Min(_WORD i1, _WORD i2);
_LONG LMin(_LONG i1, _LONG i2);
/* Liefert den kleineren der beiden Werte */

_WORD Max(_WORD i1, _WORD i2);
_LONG LMax(_LONG i1, _LONG i2);
/* Liefert den groesseren der beiden Werte */

_WORD Abs(_WORD i1);


/* Bei Magic sind alle Rechtecke ueblicherweise RELATIVE Koordinaten, will 
 * sagen, sie bestehen aus Koordinate (x, y) sowie BREITE und HOEHE. Dies
 * entspricht den Rechtecken, wie sie das AES erwartet.
 * Leider gilt dies nicht fuer VDI-Rechtecke. Das VDI erwartet Rechtecke
 * in ABSOLUTEN KOORDINATEN, also Koordinate "Links oben" und Koordinate
 * (Rechts unten).
 * 
 * Die folgende Porzeduren dienen zum Manipulieren von Reckteck-
 * KOORDINATEN.
 */

_VOID AbsRect(GRECT *rect);
 /* Diese Prozedur wandelt ein relatives Rechteck in ein absolutes */

_VOID RelRect(GRECT *rect);
 /* Diese Prozedur wandelt ein absolutes Rechteck in ein relatives */

_VOID RectToVars(CONST GRECT *rect, _BOOL abs, _WORD *x, _WORD *y, _WORD *w, _WORD *h);
/* Macht aus einem Rechteck vier einzelne Variable.  Bei abs == TRUE werden
 * absolute Werte geliefert (w:== x + w;).  Geht davon aus, dass die Daten
 * in rect ein Rechteckt mit Koordinate und Breite/Hoehe darstellen -
 * entsprechend dem Typ tRect;
 */

_VOID VarsToRect(_WORD x, _WORD y, _WORD w, _WORD h, _BOOL abs, GRECT *rect);
/* Umkehrfunktion zu RectToVars */

_VOID AbsRectToVars(CONST GRECT *rect, _BOOL abs, _WORD *x, _WORD *y, _WORD *w, _WORD *h);
/* Wie RectToVars, geht aber von absoluten Koordinaten in rect aus */

_VOID AbsVarsToRect(_WORD x, _WORD y, _WORD w, _WORD h, _BOOL abs, GRECT *rect);
/* Umkehrfunktion zu AbsRectToVars */

#endif /* __MAGICTOOL_UTILS_H__ */
