/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __MAGICTOOL_XOBJECTS_H__
#define __MAGICTOOL_XOBJECTS_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif


typedef _UWORD _CDECL (*DrawProc)(PARMBLK *pb);

#ifdef DOCUMENTATION

 * Eine Prozedur dieses Typs ist fuer die Darstellung eines Objekts voll 
 * eigenverantwortlich. Die Prozedur kann tun was sie will, ausser das 
 * AES aufrufen. Das ist STRENG VERBOTEN, da das AES nicht reentrant ist.
 * (Zuwiderhandlung wird mit Bomben nicht unter 4 Stueck bestraft!), 
 *
 * TYPE PtrPARMBLK == POINTER TO PARMBLK;
 *      PARMBLK ==    RECORD
 *                    pbTree:      ADDRESS;  /* Adresse des Objektbaums */
 *                    pbObj:       _WORD; /* Nummer des Objekts */
 *                    prPrevstate: _UWORD;  /* Status vor dem Aufruf */
 *                    prCurrstate: _UWORD;  /* Neuer Status */
 *                    pbX:         _WORD; /* X-Position des Objekts */
 *                    pbY:         _WORD; /* Y-Position des Objekts */
 *                    pbW:         _WORD; /* Breite des Objekts */
 *                    pbH:         _WORD; /* Hoehe des Objekts */
 *                    pbXc:        _WORD; /* X-Pos des Clip-Rechtecks */
 *                    pbYc:        _WORD; /* Y-Pos des Clip-Rechtecks */
 *                    pbWc:        _WORD; /* Breite des Clip-Rechtecks */
 *                    pbHc:        _WORD; /* Hoehe des Clip-Rechtecks */
 *                    pbParm:      ADDRESS;  /* Objcspec des Objekts */
 *                   END;
 *
 * Fuer Megamax-Modula-2:
 * Die Prozedur wird nicht direkt vom AES angesprungen, sondern ueber einen
 * Assembler-Handler, der intern im Modul arbeitet.  Der Handler wechselt
 * vor dem Aufruf der Zeichenroutine in den Usermodus (Das AES laeuft zu 
 * diesem Zeitpunkt im Supervisor), so dass die Routine keinerlei Probleme
 * haben sollte, und nach der Rueckkehr aus DrawProc automatisch wieder in 
 * den Supervisormodus. In der Routine koennen alle Register mit den bekannten
 * Einschraenkungen des MM2 verwendet werden.
 *
 * Fuer TDI, SPC und LPR:
 * Bis auf weiteres ist es leider nicht moeglich, richtige PROGDEF-Objekte zu
 * benutzen.  Ein Dialog, der PROGDEFs benutzt, muss mit mtDials.DialDraw
 * gezeichnet werden, um in den Genuss der PROGDEFs zu kommen!
 *
#endif

_BOOL InstUserdef(OBJECT *tree, _WORD objc, DrawProc proc, _VOID *env);
#ifdef DOCUMENTATION
 * Installiert ein Userdef-Objekt. FALSE wenn fehlgeschlagen (bereits instal-
 * liert, oder kein Speicher mehr fuer die Struktur).
 *
 * tree ==       Objektbaum
 * objc ==       Nummer des Objekts, fuer das proc installiert wird
 * proc ==       Zeichenprozedur.
 * private ==    ein optionaler Parameter, z.B. ein Zeiger auf eigene Daten
 *
 * Der obSpec eines solchen Objekts wird ein Zeiger auf eine USERBLK-
 * Struktur. Modulintern ist diese AES-Struktur erweitert:
 *
 * TYPE PtrUSERBLK == POINTER TO USERBLK;
 *      USERBLK ==    RECORD
 *                    ubCode: PROC;     /* Handle-Prozedur */
 *                    ubPara: ADDRESS;  /* der originale obSpec */
 *                    obType: _WORD; /* der originale obType */
 *                    draw:   DrawProc; /* die eigentliche Zeichenprozedur */
 *                   END; 
 *
 * In ubPara wird der urspruengliche obSpec (siehe MagicAES.Objcspec) abgelegt,
 * damit diese Daten nicht verlorengehen.
 *
#endif

_VOID FreeUserdef(OBJECT *tree, _WORD objc);
/* Gibt ein Userdef-Objekt wieder frei, dabei werden obSpec und obType
 * restauriert.
 */

_WORD GetObtype(OBJECT *tree, _WORD objc);
/* Liefert den obType eines Objekts. Erkennt, ob es sich um ein Userdef 
 * handelt und liefert - falls es mit diesem Modul installiert wurde - 
 * den originalen obType (der obType aus MagicAES.OBJECT wird ja mit 
 * MagicAES.GPROGDEF ueberschrieben!)
 */

OBSPEC *GetObSpec(OBJECT *tree, _WORD objc);
/* Liefert den obSpec eines Objekts. Erkennt, ob es sich um ein Userdef
 * handelt und liefert - falls es mit diesem Modul installiert wurde - 
 * den originalen obSpec (der obSpec aus MagicAES.OBJECT wird ja mit 
 * MagicAES.USERBLK ueberschrieben!)
 */

_VOID *GetPrivate(OBJECT *tree, _WORD objc);
/* Liefert den bei USERDEFs, die ueber dieses Modul installiert wurden den
 * 'private'-Parameter, oder NIL, falls kein USERDEF.
 */

#endif /* __MAGICTOOL_XOBJECTS_H__ */
