#ifndef __PROTOKOLL_H__
#define __PROTOKOLL_H__
/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __PORTAB_H__
#include <portab.h>
#endif

/*

XAcc und AV/VA-Protokoll in Modula-2

Autor : Johannes Goettker-Schnetmann

Last Change : 26.02.1991

*/


#define memDescrSize  2048 /* 2 kB globaler Speicher */

extern _VOID *memPtr;

#define PROT_NAMEPTR(name) \
	(_UWORD)((_ULONG)(name) >> 16), \
	(_UWORD)(_ULONG)(name)


typedef _BOOL (*HandleProtoMessage)(CONST _WORD *message);

typedef struct gl_mem
{
	_WORD			id;			/* Die AES-ID		*/
	struct gl_mem	*nxt;		/* einfach verkettet	*/
	char			mem;		/*das 1. Zeichen*/
}GL_MEM;


_VOID Protokoll_Send(_WORD dest, _WORD msg, _WORD d3, _WORD d4, _WORD d5, _WORD d6, _WORD d7);
_VOID Protokoll_SendMess(_WORD msg, _WORD catType, _ULONG l, _WORD i2, _WORD i3, _WORD dest);
_VOID Protokoll_SendMessInt(_WORD msg, _WORD catType, _WORD i1, _WORD i2, _WORD i3, _WORD i4, _WORD dest);
_VOID Protokoll_SendAdr(_WORD msg, _WORD catType, _WORD i1, _VOID *l, _WORD i2, _WORD dest);
_VOID Protokoll_SendPtr(_WORD msg, _VOID *l, _WORD dest);


_BOOL Protokoll_Send2Filter(_WORD to, _UWORD length, _VOID *what);
/* Sendet eine Nachricht mit Filterdaten an ein anderes Programm
 * oder ein Accessory
 */

_VOID Protokoll_SendHelp(CONST _UBYTE *str);
/* Unterstuetzung fuer PC-Help Protokoll. Sendet einmal Help aus. */

_BOOL Protokoll_SendExtendedHelp(CONST _UBYTE *str, CONST _UBYTE *helpText,_WORD avModus);
/* Unterstuetzung fuer PC-Help Protokoll. Sendet einmal Help aus,
 * mit Erweiterung um Vorzugsnamen des Hilfetextes.
 */

_BOOL Protokoll_SendPrintFile(CONST _UBYTE *path, CONST _UBYTE *name, _BOOL withEffects);
/* Druckt die angegebene Datei ueber Idealist bzw. das CALCLOCK-Protokoll
 */

_VOID Protokoll_SendPathUpdate(CONST _UBYTE *path);
/* Schickt ein AV_PATH_UPDATE an den AV_SERVER */

_VOID Protokoll_SendTerminate(_BOOL terminate);

_VOID Protokoll_ProtoInit(_VOID);
/* XACC-Initialisierung unter MTOS */

_BOOL Protokoll_Init(_VOID);

_VOID Protokoll_Broadcast(_VOID *message);

_VOID Protokoll_Handle_Message(HandleProtoMessage func);

_BOOL Protokoll_Check_Message(CONST _VOID *message);

_BOOL Protokoll_Waitfor(_WORD msg, _WORD timeout, _WORD *message);

_BOOL VaProto_Quote(_UBYTE *ziel,size_t maxlen,CONST _UBYTE *quell,_WORD id);
/* kopiert quell nach ziel, wobei ggf gequoted wird.*/

size_t VaProto_UnQuote(_UBYTE *ziel,size_t maxlen,CONST _UBYTE *quell);
/* kopiert quell nach ziel, wobei ggf entquoted wird.*/

GL_MEM *Protokoll_GetGlMem(size_t len);
/* besorgt globalen Speicher	*/

GLOBAL _VOID Protokoll_FreeGlMem(_WORD id,void *mem);
/*gibt ihn wieder frei */


#endif /* __PROTOKOLL_H__ */
