#ifndef __TERMINAL_H__
#define __TERMINAL_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif

/* Das folgende Set entspricht dem Attributset von 1stWord */

typedef enum {
	fat = 0x0100,                       /* Fettschrift                  */
	light = 0x0200,                     /* helle Schrift                */
	italic = 0x0400,                    /* kursive Schrift              */
	underline = 0x0800,                 /* unterstrichen                */
	superscript = 0x1000,               /* hochgestellte Schrift        */
	subscript = 0x2000,                 /* tiefgestellte Schrift        */
	invers = 0x4000                     /* inverse Schrift              */
} TerminalAttribute;
typedef _UWORD TerminalAttribut;


struct _TerminalTyp;
typedef struct _TerminalTyp TerminalTyp, *TERMINAL;

#ifndef __WIN_TERMINAL_IMPLEMENTATION__
struct _TerminalTyp { int dummy; };
#endif


/*--------------------------------------------------------------------------* 
 * Die Ausgabe der folgenden Prozeduren erfolgt immer ins aktuelle Terminal * 
 *--------------------------------------------------------------------------*/

_VOID TerminalWriteChar(TERMINAL t, _UBYTE c);
/* Schreibt Zeichen */

_VOID TerminalWriteString(TERMINAL t, CONST _UBYTE *s);
/* schnelle Stringausgabe, kein LF */

_VOID TerminalWriteLine(TERMINAL t, CONST _UBYTE *s);
/* schnelle Stringausgabe, mit LF */

_VOID TerminalWriteLn(TERMINAL t);
/* CR + LF */

_VOID TerminalWriteCard(TERMINAL t, _UWORD wert, _UWORD len);
_VOID TerminalWriteInt(TERMINAL t, _WORD wert, _UWORD len);
_VOID TerminalWriteLongCard(TERMINAL t, _ULONG wert, _UWORD len);
_VOID TerminalWriteLongInt(TERMINAL t, _LONG wert, _UWORD len);
/* Schreibt die entspechenden Werte mit Laenge len */

_VOID TerminalInsertLine(TERMINAL t);
/* Fuegt Leerzeile bei aktueller Cursorposition ein. Alle Zeilen unterhalb
 * werden nach unten verschoben. Die unterste Zeile geht verloren.
 */

_VOID TerminalDeleteLine(TERMINAL t);
/* Loescht Zeile an aktueller Cursorposition. Alle Zeilen unterhalb
 * werden nach oben verschoben. Die unterste Zeile wird leer.
 */

_VOID TerminalWhereXY(TERMINAL t, _UWORD *x, _UWORD *y);
/* Ermittelt Cursorposition */

_VOID TerminalGotoXY(TERMINAL t, _UWORD x, _UWORD y);
/* Setzt Cursor auf Spalte/Zeile.  Die HOME-Position ist 0,0 */

/* Hinweis fuer WhereXY und GotoXY: 
 * ------------------------------- 
 * Diese Prozeduren arbeiten nur korrekt, wenn im Terminal ausschliesslich
 * EIN Font verwendet wurde, der zudem noch MONOSPACED sein muss. Fuer den
 * Systemfont ist dies gegeben.
 *
 * Werden FSM-Fonts oder proportionale Bitmap-Fonts verwendet, ist eine
 * Spalten/Zeilen-Weise Positionsberechnung nicht moeglich. Bei Einsatz
 * solcher Fonts sollten die Prozeduren WhereCursor und SetCursor benutzt
 * werden.
 */

_VOID TerminalWhereCursor(TERMINAL t, _WORD *x, _WORD *y);
/* Ermittelt Cursorposition in Rasterkoordinaten */

_VOID TerminalSetCursor(TERMINAL t, _WORD x, _WORD y);
/* Setzt Cursorposition in Rasterkoordinaten */

_VOID TerminalCursorOn(TERMINAL t);
_VOID TerminalCursorOff(TERMINAL t);
/* Cursorhandling */

_VOID TerminalCursorStop(TERMINAL t);
/* Stoppt blinkenden Cursor. Wird ggw. nicht unterstuetzt! */

_VOID TerminalCursorBlink(TERMINAL t);
/* Laesst Cursor blinken.  Wird ggw. nicht unterstuetzt! */

/* Hinweis:
 * --------
 * Cursorhandling wird gegenwaertig nur bei monospaced Fonts unterstuetzt.
 */

_VOID TerminalWrapOff(TERMINAL t);
_VOID TerminalWrapOn(TERMINAL t);
/* Umbruch am Zeilenende. WrapOff ist Default */

_VOID TerminalClearLine(TERMINAL t);
/* Loescht aktuelle Zeile */

_VOID TerminalClearStartOfLine(TERMINAL t);
/* Loescht zum Zeilenanfang */

_VOID TerminalClearEndOfLine(TERMINAL t);
/* Loescht zum Zeilenende */

_VOID TerminalClearScreen(TERMINAL t);
/* Loescht Bildschirm */

_VOID TerminalClearStartOfScreen(TERMINAL t);
/* Loescht zum Anfang des Bildschirms */

_VOID TerminalClearEndOfScreen(TERMINAL t);
/* Loescht zum Ende des Bildschirms */

_VOID TerminalForegroundColor(TERMINAL t, _WORD color);
_VOID TerminalBackgroundColor(TERMINAL t, _WORD color);
/* Setzt die Farben des Terminals */

_VOID TerminalInversOn(TERMINAL t);
_VOID TerminalInversOff(TERMINAL t);
/* Invers ein und wieder aus */

_VOID TerminalSetAttribut(TERMINAL t, TerminalAttribut attrib);
/* Setzt die Textattribute des Terminals */

_VOID TerminalSetTabspace(TERMINAL t, _WORD tab);
/* Setzt die Breite des Tab-Sprungs, Default == 8 */

/*--------------------------------------------------------------------------*
 *                      Support fuer mehrere Terminals                       *
 *--------------------------------------------------------------------------*/

_BOOL TerminalOpen(TERMINAL *term, CONST _UBYTE *title);
/* Oeffnet ein Terminal an Pos rect.x, rect.y und Breite/Hoehe rect.w, rect.h.
 */

_VOID TerminalClose(TERMINAL *term);
/* Schliesst das Terminal. */

_VOID TerminalShow(TERMINAL term);
/* Bringt das Terminal-Fenster nach oben. */

_VOID TerminalGetMaximum(GRECT *rect);
/* Liefert groesstmoegliche Flaeche des Terminals */

_VOID TerminalClipRect(TERMINAL t, _BOOL doit, CONST GRECT *rect);
/* Clipping. */

_BOOL TerminalWasOpen(_VOID);

#endif /* __MAGICTOOL_TERMINAL_H__ */

