/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __CAT_WDWMANAGER_H__
#define __CAT_WDWMANAGER_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif
#ifndef __PORTAES_H__
#include <portaes.h>
#endif

#define UP      0
#define DOWN    1
#define RIGHT   0
#define LEFT    1
        
typedef struct {
	_LONG x;
	_LONG y;
	_LONG w;
	_LONG h;
} LGRECT;
extern _BOOL WinDialExit;

typedef _BOOL (*handleMessProc)(CONST _WORD *pBuff, _UWORD kstate);
typedef _BOOL (*handleGlobalKeyProc)(_WORD taste, _UBYTE key, _UBYTE scan, _UWORD kstate);
typedef _BOOL (*handleButtonProc)(HWND wdw, _VOID *special, _WORD mx, _WORD my, _UWORD kstate, _UWORD button, _WORD clicks);
typedef _BOOL (*handleMessageProc)(HWND wdw, _VOID *special, const _WORD *msg);
typedef _BOOL (*handleKeyProc)(HWND wdw, _VOID *special, _WORD taste, _UBYTE key, _UBYTE scan, _UWORD kstate);
typedef _BOOL (*handleTimerProc)(HWND wdw, _VOID *special);
typedef _BOOL (*handleRectProc)(HWND wdw, _VOID *special, _WORD which, _WORD mox, _WORD moy);
typedef _BOOL (*getRectEventProc)(HWND wdw, _VOID *special, GRECT *rectangle, _WORD *type);
typedef _VOID (*snapProc)(HWND wdw, _VOID *special, GRECT *rect);
typedef _BOOL (*closeProc)(HWND wdw, HVDI vdiH, _VOID *special, _BOOL force);
typedef _BOOL (*topProc)(HWND wdw, _VOID *special);
typedef _VOID (*untopProc)(HWND wdw, _VOID *special);
typedef _VOID (*drawProc)(HWND wdw, HVDI vdiH, _VOID *special, CONST GRECT *rect);
typedef _VOID (*wdwupdateProc)(HWND wdw, _VOID *special, _BOOL begin);
typedef _VOID (*setWorkProc)(HWND wdw, _VOID *special, CONST LGRECT *document, _BOOL slideJN);
typedef _VOID (*getRectProc)(HWND wdw, HVDI vdiH, _VOID *special, GRECT *rect);
typedef _VOID (*hideProc)(HWND wdw, _VOID *special, _BOOL hide);
typedef _VOID (*handleNewTopProc)(HWND wdw);
typedef _VOID (*handleEventProc)(HWND wdw);
typedef _VOID (*getExtsProc)(HWND wdw, _VOID *special, _UBYTE *extdata, size_t datasize);
typedef _BOOL (*acceptDataProc)(HWND wdw, _VOID *special, _VOID *header);
typedef _VOID (*writeDataProc)(HWND wdw, _VOID *special, _VOID *data, _ULONG datalen);
typedef _VOID (*readDataProc)(HWND wdw, _VOID *special, _VOID **data, _WORD ixx);
typedef _BOOL (*getHeaderProc)(HWND wdw, _VOID *special, _WORD idx, _UBYTE ext[4], _UBYTE *name, size_t namesize, _ULONG *size);
typedef _BOOL (*queueTestProc)(HWND wdw);

typedef _BOOL (*FONT_ACCEPT_FUNC)(HWND win, _WORD id, _WORD size, _BOOL isMono);
typedef _BOOL (*COLOR_ACCEPT_FUNC)(HWND win, _WORD color);




/* ---------------------------------------------------------------------*/
/* Variablen                                                            */

extern HWND theTopWindow;         /* Enthaelt immer die Nummer des Topwindows
                                   * Wird in HandleEvent upgedatet
                                   */
extern HWND ownTopWindow;         /* Enthaelt immer die Nummer des Topwindows
                                   * von CAT, also des eigenen, hoechsten 
                                   * Fensters. Muss nicht mit theTopWindow 
                                   * identisch sein!
                                   */

/* ---------------------------------------------------------------------*/


/* Ein paar allgemeine Rechteck- und Clip-Funktionen */

_VOID WdwManager_SetClip(HVDI hdl, CONST GRECT *clip, _BOOL relative);
/* Setzt das Clipping-Rechteck fuer die workstation. 
 * Wenn relative == TRUE ist, wird das Rechteck noch umgewandelt in
 * ein absolutes Rechteck ( (x, y, w, h) ==> (x, y, x2, y2))
 */

_BOOL WdwManager_RectList(HWND wdw, _WORD flag, GRECT *work);
/* Liefert die Rechteckliste fuer das Fenster.
 * Flag == 0: Erstes Rechteck liefern.
 * Flag == 1: Naechstes Rechteck liefern.
 * In work wird das Rechteck zurueckgegeben.
 * TRUE == Es ist noch ein Rechteck vorhanden
 */

_BOOL WdwManager_RectListActive(_VOID); 
/* Gibt zurueck, ob gerade eine Rechteckliste abgearbeitet wird
 */

/* ---------------------------------------------------------------------*/

_BOOL WdwManager_CreateWindow(
	CONST GRECT *fullSize,
	GRECT *currSize, 
	_UWORD wdwParts,
	CONST _UBYTE *info,
	CONST _UBYTE *titel,
    _VOID *env,
	_BOOL supportIconify,
	_BOOL openWorkstation,
	HWND *wdwHandle,
	HVDI *vdiHandle);
/* Erzeugt ein Fenster, das mit Default-Prozeduren behandelt wird. 
 * Weitere Einstellungen muss man danach vornehmen 
 */

_BOOL WdwManager_SetWindowDraw(HWND wdw, drawProc draw);
/* Setzt die Zeichenprozedur fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowSnap(HWND wdw, snapProc snap);
/* Setzt die Snapprozedur fuer ein Fenster. Diese wird dann
 * bei jedem Sizen und einmal direkt mit den aktuellen Werten
 * aufgerufen. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowButton(HWND wdw, handleButtonProc button);
/* Setzt den Buttonhandler fuer ein Fenster. False: Fenster nicht gefunden
 */
_BOOL WdwManager_SetWindowMessage(HWND wdw, handleMessageProc message);
/* Setzt den Buttonhandler fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowKey(HWND wdw, handleKeyProc key);
/* Setzt den Keyhandler fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowTimer(HWND wdw, handleTimerProc timer);
/* Setzt den Timerhandler fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowRectProc(HWND wdw, handleRectProc rect);
/* Setzt den Rechteckhandler fuer ein Fenster. 
 * False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowClose(HWND wdw, closeProc close);
/* Setzt den Closehandler fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowTop(HWND wdw, topProc top);
/* Setzt den Tophandler fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowUntop(HWND wdw, untopProc untop);
/* Setzt den Untophandler fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowUpdate(HWND wdw, wdwupdateProc update);
/* Setzt den Updatehandler fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowSwork(HWND wdw, setWorkProc setWdw);
/* Setzt den Handler fuer Dokumentaenderungen im Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowGetScroll(HWND wdw, getRectProc getScroll);
/* Setzt den GetScrollRect-Handler fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowHide(HWND wdw, hideProc hide);
/* Setzt den Hidehandler fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowPixOff(HWND wdw, _WORD pixelOffset);
/* Setzt den linken Offset fuer ein Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowMode(HWND wdw, _BOOL onlyThisTop);
/* Setzt den Modus fuer ein Fenster. False: Fenster nicht gefunden oder
 * es ist schon ein anderes Fenster als einziges das Topwindow. 
 */

_BOOL WdwManager_SetWindowDial(HWND wdw, OBJECT *dial);
/* Setzt die Dialogadresse fuer das Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_SetWindowMenu(HWND wdw, OBJECT *menu);
/* Setzt die Menuadresse fuer das Fenster. False: Fenster nicht gefunden
 */

_BOOL WdwManager_OpenWindow(
	handleButtonProc button,
	handleKeyProc key,
	handleTimerProc timer,
	CONST GRECT *fullSize,
	GRECT *currSize,
	_UWORD wdwParts,
	_BOOL wdwSnap,
	CONST _UBYTE *info,
	CONST _UBYTE *titel,
	snapProc snapP,
	closeProc close,
	drawProc draw,
	topProc top,
	untopProc untop,
	wdwupdateProc update,
	setWorkProc setWdw,
	getRectProc	getScroll,
	hideProc hide,
	_WORD pixelOffset,
	_VOID *special,
	_BOOL onlyThisTop,
	_BOOL showIt,
	_BOOL supportIconify,
	_BOOL openWorkstation,
	_BOOL hasIdem,
	HWND *wdwHandle,
	HVDI *vdiHandle);
/* Erlaeuterung der Parameter:
 * button: Diese Prozedur wird dann aufgerufen, wenn ein Button-Event aufgetreten ist 
 *         und dieser im Workbereich des Fensters lag. Es ist auch moeglich,
 *         handleButtonProc (NIL) zu uebergeben.
 * message: Diese Prozedur wird dann aufgerufen, wenn eine Message aufgetreten ist 
 *         und dieser im Workbereich des Fensters lag. Es ist auch moeglich,
 *         handleMessageProc (NIL) zu uebergeben.
 * key:    Diese Prozedur wird dann aufgerufen, wenn das Fenster das TopWindow ist
 *         und ein Keyboard-Event aufgetreten ist. Es ist auch moeglich, handleKeyProc (NIL)
 *         zu uebergeben, dann wird diese nicht aufgerufen.
 * timer:  Diese Prozedur wird aufgerufen, wenn ein Timer-Event aufgetreten ist. 
 *         Man kann auch handleTimerProc (NIL) uebergeben, dann wird nichts ausgefuehrt.
 * fullSize,
 * currSize : Die Aussenrechtecke. fullSize ist die maximale Groesse des Fensters, currSize
 *            die Groesse, mit der das Fenster geoeffnet werden soll.
 * wdwParts : Ein Set der Window-Elemente (MagicAES.D)
 * wdwSnap  : Wenn TRUE uebergeben wird, wird beim Veraendern der Groesse und der Position des 
 *            Fensters die Prozedur snapP aufgerufen. Diese muss dann installiert sein!
 * info,
 * titel    : Die Strings, die als Titelzeile bzw. als Infozeile angezeigt werden sollen.
 * snapP: Diese Prozedur wird aufgerufen, wenn wdwSnap == TRUE ist und das Fenster in der Groesse
 *        veraendert wurde oder verschoben wurde. Nur, wenn wdwSnap == FALSE ist, darf man auch 
 *        einen NIL-Pointer uebergeben.
 * close: Diese Prozedur wird aufgerufen, wenn ein CloseEvent aufgetreten ist oder CloseWindow
 *        aufgerufen wurde. Nur, wenn close TRUE zurueckgibt, wird das Fenster auch geschlossen.
 *        CloseWindow darf von close NICHT aufgerufen werden. Diese Prozedur muss vorhanden sein,
 *        ein NIL-Pointer ist nicht zulaessig!
 * top: Diese Prozedur wird aufgerufen, wenn das Fenster getoppt werden soll. Nur, wenn TRUE zurueck-
 *      gegeben wird, wird das Fenster auch getoppt. Diese Prozedur muss vorhanden sein,
 *      ein NIL-Pointer ist nicht zulaessig!
 * untop:  Diese Prozedur wird aufgerufen, wenn das Fenster enttopped wurde. Es ist auch moeglich,
 *         NIL zu uebergeben.
 * update: Diese Prozedur wird aufgerufen, wenn Ausgaben in das Fenster gemacht werden sollen. Vor 
 *         den Ausgaben wird update (..., MagicAES.BEG_UPDATE) aufgerufen, am Ende der Ausgaben
 *         update (..., MagicAES.END_UPDATE). Diese Prozedur muss vorhanden sein,
 *         ein NIL-Pointer ist nicht zulaessig!
 * setWdw: Diese Prozedur wird aufgerufen, wenn sich intern die Position des virtuellen Fensters
 *         auf dem Dokument veraendert hat. Wenn das durch Scrollen geschah, dann ist slided TRUE.
 *         Diese Prozedur muss vorhanden sein, ein NIL-Pointer ist nicht zulaessig!
 * getScroll: Diese Prozedur muss das Rechteck zurueckliefern, in dem das Fenster gescrollt werden darf.
 *            Es wird die aktuelle Arbeitsflaeche des Fensters (WorkArea) uebergeben und es muss
 *            eine Flaeche zurueckgegeben werden, in der das Scrollen erlaubt ist. 
 *            Es ist auch moeglich, die Arbeitsflaeche ohne Einschraenkungen zurueckzugeben.
 *            Es wird keine Ueberpruefung gemacht, ob das Rechteck vernuenftige Werte enthaelt,
 *            man sollte also darauf achten, dass man keine unsinnigen Werte zurueckgibt.
 *            Diese Prozedur muss vorhanden sein, ein NIL-Pointer ist nicht zulaessig!
 * hide:   Diese Prozedur wird aufgerufen, wenn das Fenster entweder mittels HideWindow
 *         versteckt wird oder wenn es iconifiziert wird. Die Prozedur sollte gesetzt
 *         sein, muss es aber nicht. 
 * pixelOffset: Um diesen Offset wird der Fensterinhalt beim Zeichnen versetzt gezeichnet.
 *              Dieser muss den internen Scrollroutinen bekannt sein, da es sont Probleme 
 *              gibt.
 * env: Ein beliebiger Parameter, der an die Serviceroutinen zurueck uebergeben wird. Kann auch
 *      ein Pointer auf eine Struktur sein.
 * onlyThisTop: Wenn dieser Parameter TRUE ist, dann wird nur dieses Fenster getoppt 
 *              und kein anderes. Nuetzlich fuer modale Fensterdialoge.
 * showIt:      Wenn dieser Parameter TRUE ist, dann wird das Fenster auch direkt geoeffnet.
 *              Ansonsten muss extra noch ShowWindow aufgerufen werden.
 * hasIdem:		Gleiche Inhalte werden durch " o.. ersetzt, mu beim Scrollen bercksichtig werden
 *				neu in 4.18
 * Rueckgabewerte:
 * wdwHandle: Das Handle des Fensters. Nur gueltig, falls TRUE zurueckgegeben wird.
 * vdiHandle: Das Handle der zugeordneten Workstation. Nur gueltig, falls TRUE zurueckgegeben wird.
 */

_BOOL WdwManager_ShowWindow(HWND wdw);
/* Ein vorhandenes, aber nicht geoeffnetes Fenster wird geoeffnet.
 * Geht nicht mit iconifizierten Windows! Die hideProc fuer das Fenster
 * wird aufgerufen.
 * FALSE: Window nicht vorhanden oder ist schon offen.
 */

_BOOL WdwManager_HideWindow(HWND wdw);
/* Ein offenes Fenster wird geschlossen, aber nicht geloescht. 
 * Die hideProc fuer das Fenster wird aufgerufen. 
 * Das Window wird dadurch nicht iconifiziert!
 * FALSE: Window nicht vorhanden oder ist schon geschlossen oder ikonifiziert.
 */

_VOID WdwManager_SendCloseWindow(HWND wdw);
/* Sendet eine Schliessnachricht an das Fenster 
 * Geht sowohl fuer eigene als auch fuer fremde Fenster, wenn
 * diese in der Fensterliste vorhanden sind oder ein TOS
 * laeuft, das die Abfrage der ID zum Fenster erlaubt.
 */

_BOOL WdwManager_CloseWindow(HWND wdw, _BOOL force);
/* Schliesst Fenster und entfernt die Handler aus der Liste 
 * Bei force == TRUE wird das Fenster dann garantiert geschlossen!
 * Falls das Fenster nicht vom Modul verwaltet wird, aber trotzdem 
 * in der WindowQueue enthalten ist, dann wird an das Fenster
 * eine Close-Nachricht geschickt.
 */

_BOOL WdwManager_CloseAllWindows(_BOOL force);
/* Schliesst alle Fenster, die dem Modul bekannt sind. 
 * Die Fenster werden alle ueber CloseWindow geschlossen,
 * und falls ein CloseWindow FALSE zurueckliefert, dann
 * wird der Returnwert zurueckgegeben und abgebrochen.
 */

_VOID WdwManager_FullWindow(HWND wdw);
/* Tut so, als ob das Fenster eine Fulled-Nachricht erhalten haette.
 * Falls das Fenster nicht vom Modul verwaltet wird, aber trotzdem 
 * in der WindowQueue enthalten ist, dann wird an das Fenster
 * eine Fulled-Nachricht geschickt.
 */

_VOID WdwManager_TopWindow(HWND wdw);
/* Tut so, als ob das Fenster eine Topped-Nachricht erhalten haette.
 * Falls das Fenster nicht vom Modul verwaltet wird, aber trotzdem 
 * in der WindowQueue enthalten ist, dann wird an das Fenster
 * eine Topped-Nachricht geschickt.
 */

_BOOL WdwManager_HandleEvent(
	_UWORD *event, CONST _WORD *pbuff, _UWORD buts,
	_UWORD kstate, _UBYTE key, _UBYTE scan, _WORD taste, _WORD mx, _WORD my, _WORD clicks);
/* Verteilt die events, falls noetig. Wenn behandelt, wird TRUE zurueckgegeben,
 * andernfalls FALSE
 */

_VOID WdwManager_HandlePendingEvents(_VOID);
/* Arbeitet aufgelaufene Message-Events ab
 */

_VOID WdwManager_GetTopWindow(HWND *wdw, HVDI *handle);
/* Liefert das Handle des Top-Fensters (direkt ueber AES)
 * und zusaetzlich noch das VDI-Handle der Workstation des Fensters.
 */

_BOOL WdwManager_IsOwnTop(_VOID);
/* Liefert zurueck, ob ein eigenes Fenster, das vom Windowmanager verwaltet
 * wird, das Topwindow ist
 */

_BOOL WdwManager_WindowIsIcon(HWND wdw);
/* Liefert TRUE, wenn das Window vorhanden und iconifiziert ist 
 */

_BOOL WdwManager_ModalWindowTop(_VOID);
/* Liefert TRUE, wenn das Topwindow ein eigenes und modal ist 
 */

_BOOL WdwManager_WindowIsShaded(HWND wdw);
/* Liefert TRUE, wenn das Window vorhanden und mittels WindowShade 
 * geshadet ist (oder ausserhalb des Bildschirms liegt)
 */

_BOOL WdwManager_WindowIsFullable(HWND wdw);
/* Liefert TRUE, wenn das Window vorhanden und fullbar ist,
 * d.h. es hat einen Sizer oder es ist nur in der Queue drin.
 */

_BOOL WdwManager_WindowIsCloseable(HWND wdw);
/* Liefert TRUE, wenn das Window vorhanden und schliessbar ist.
 * Schliessbar heisst, es hat einen Closer oder es ist in der Windowqueue
 */

_VOID WdwManager_SetWindowIcon(OBJECT *icon);
/* Setzt das Icon, das in iconifizierten Fenstern gezeichnet wird. 
 * icon ist die Adresse eines Objektbaumes;
 * Bei icon == NIL wird nichts gezeichnet.
 */

HWND WdwManager_ClickInWindow(_WORD x, _WORD y);
/* Liefert das Windowhandle zurueck, falls sich an der Position 
 * x,y ein Fenster befindet (muss nicht dem Modul bekannt sein!)
 */

_VOID WdwManager_ScrollInWdw(HWND wdw, GRECT *source, _WORD dir, _WORD amount);
/* Diese Funktion ermoeglicht das Scrollen innerhalb eines Dokumentes
 * unter Beruecksichtigung der Rechteckliste und mit automatischem Redraw 
 * der neu zu zeichnenden Stellen nach dem Scrollen
 */

/* Das Konzept des virtuellen Dokuments:


Intern innerhalb des Moduls wird fuer jedes Fenster ein virtuelles
Dokument (VD) verwaltet. Der aktuelle Fensterausschnitt ist immer
nur ein Ausschnitt aus diesem virtuellen Dokument. Intern wird das 
ungefaehr wie folgt behandelt:


            x-Position des Dokumentes
                |
                v
       <---------------- Breite des Dokuments ---------->
       +------------------------------------------------+ ^
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
 ----> |        +---------------------+                 | | 
y-Pos. |        |                     |                 | |
       |        | aktueller           |                 | |
       |        | Ausschnitt          |                 | |
       |        |                     |                 | |
       |        |                     |                 | |
       |        |                     |                 | |
       |        |                     |                 | |
       |        +---------------------+                 | Hoehe 
       |                                                | des 
       |                                                | Dokuments
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       |                                                | |
       +------------------------------------------------+ V

Damit die Verwaltung dieses VD auch funktioniert, muessen einige 
Parameter gesetzt sein. Die einzelnen Werte fuer das Dokument werden
in sogenannten Einheiten verwaltet. Dabei kann man fuer x und y Aufloesung
unterschiedliche Einheiten angeben. Diese Einheiten sind der xFactor und
der yFactor. So ist es zum Beispiel moeglich, die Hoehe in Textzeilen anzugeben,
aber die Breite in Pixeln. In dem Fall waere der yFactor (Hoehe) == die Hoehe einer
Textzeile, und der xFactor (Breite) == 1 (Default). 
Ebenso muss man die Hoehe und Breite einer Seite in Einheiten fuer das Blaettern
angeben, und man muss auch noch angeben, um wieviele Einheiten beim
einzelnen Scrollen gescrollt werden soll.
In obigem Beispiel waere dann bei einem normalen Textbildschirm die
Hoehe 20 Zeilen und die Breite 600 Pixel. Und beim Scrollen gibt man dann
fuer minScrollH eine 1 an (eine Zeile), aber da man nicht Pixelweise seitwaerts
scrollen will, dort z.B. eine 8 (fuer einen 8 Pixel breiten Zeichensatz). 
*/


/* Prozeduren zur Verwaltung und zum Handling des 
 * virtuellen Dokuments.
 */

_VOID WdwManager_SetDocumentParms(HWND wdw, _LONG xFac, _LONG yFac);
/* Setzt die Parameter fuer die Ausschnittsberechnung 
 */

_VOID WdwManager_SetScrollParms(HWND wdw, _LONG pageW, _LONG pageH, _LONG minScrollW, _LONG minScrollH, _BOOL lstPgF);
/* Setzt die Parameter fuer das Blaettern im Dokument 
 */

_VOID WdwManager_GetScrollParms(HWND wdw, _LONG *pageW, _LONG *pageH, _LONG *minScrollW, _LONG *minScrollH);
/* Erfragt die Parameter fuer das Blaettern im Dokument 
 */

_VOID WdwManager_SetNewDocument(HWND wdw, CONST LGRECT *doc, _BOOL redraw);
/* Setzt die aktuellen Dokument-Parameter ohne automatsiches Scrollen
 * Wenn redraw == TRUE, wird ein Redraw ueber das komplette Fenster ausgeloest.
 */

_VOID WdwManager_SetWdwDocument(HWND wdw, CONST LGRECT *doc);
/* Setzt die aktuellen Dokument-Parameter.
 * Es wird automatisch so gescrollt, das der aktuelle Ausschnitt 
 * sichtbar ist.
 */

_VOID WdwManager_GetWdwDocument(HWND wdw, LGRECT *doc);
/* Erfragt die aktuellen Dokument-Parameter
 */

/* Blaettern im dargestellten Dokument
 */ 
_BOOL WdwManager_PageUp(HWND wdw);
/* Es wird um pageH Einheiten hochgeblaettert.
 */

_BOOL WdwManager_PageDown(HWND wdw);
/* Es wird um pageH Einheiten nach unten geblaettert.
 */

_VOID WdwManager_PageLeft(HWND wdw);
/* Es wird um pageW Einheiten nach links geblaettert.
 */

_VOID WdwManager_PageRight(HWND wdw);
/* Es wird um pageW Einheiten nach rechts geblaettert.
 */

_VOID WdwManager_ScrollUp(HWND wdw, _WORD lines);
/* Es wird um lines * minScrollH Einheiten nach oben geblaettert.
*/

_BOOL WdwManager_WouldScrollUp(HWND wdw, _WORD lines);

_VOID WdwManager_ScrollDown(HWND wdw, _WORD lines);
/* Es wird um lines * minScrollH Einheiten nach unten geblaettert.
*/

_BOOL WdwManager_WouldScrollDown(HWND wdw, _WORD lines);

_VOID WdwManager_ScrollLeft(HWND wdw, _WORD amount);
/* Es wird um amount * minScrollW Einheiten nach links geblaettert.
*/

_VOID WdwManager_ScrollRight(HWND wdw, _WORD amount);
/* Es wird um amount * minScrollW Einheiten nach links geblaettert.
*/

_UWORD WdwManager_WdwComps(HWND wdw);
/* Liefert die Window-Elemente zurueck 
 */

_VOID WdwManager_SnapWdw(HWND wdw);
/* Ruft die Snap-Funktion fuer das Fenster auf
 */

_VOID WdwManager_SetWdwSize(HWND wdw, CONST GRECT *rect);
/* Setzt neue aktuelle Aussengroesse fuer das Fenster
 */

_VOID WdwManager_GetWdwSize(HWND wdw, GRECT *rect);
/* Erfragt aktuelle Aussengroesse fuer das Fenster
 */

_VOID WdwManager_SetWdwWork(HWND wdw, CONST GRECT *rect);
/* Setzt neue aktuelle Arbeitsflaechengroesse fuer das Fenster
 */

_VOID WdwManager_GetWdwWork(HWND wdw, GRECT *rect);
/* Erfragt aktuelle Arbeitsflaechengroesse fuer das Fenster
 */

_VOID WdwManager_RedrawWdw(HWND wdw, CONST GRECT *frame);
/* Zeichnet das Fenster im Bereich Frame neu, beachtet
 * Rechteckliste und ruft indirekt die drawProc auf
 */

_VOID WdwManager_FullRedrawWdw(HWND wdw);
/* Zeichnet das Fenster komplett neu, beachtet
 * Rechteckliste und ruft indirekt die drawProc auf
 */
_VOID WdwManager_ClearWdw(HWND wdw,_WORD color);
/* Lscht das Fenster komplett, beachtet Rechteckliste
 */

_VOID WdwManager_SetWdwInfoline(HWND wdw, CONST _UBYTE *info);
/* Setzt die Infozeile fuer das Fenster
 */

_VOID WdwManager_SetWdwTitle(HWND wdw, CONST _UBYTE *title);
/* Setzt den Fenstertitel 
 */

/*------- Menu & Keyboard -----------------------------------------------------*/

_VOID WdwManager_InstallGlobalMenu(OBJECT *tree, handleMessProc menuP);
/* Gibt dem Modul die Adresse des Menubaums bekannt. Damit kann dieser dann disablet
 * werden, wenn ein modaler Dialog aufgerufen wird.
 * Ausserdem wird eine Callback-Funktion installiert, die bei Menu-Events aufgerufen
 * wird.
 * Es wird nur ein Menu und eine Callbackfunktion verwendet, ein Deinstall
 * ist daher nicht notwendig, ein erneuter Aufruf ueberschreibt die
 * eingestellten Werte
 */


_VOID WdwManager_DisableMenu(OBJECT *menu, _BOOL disable);
/* Schaltet einen Menubaum auf enabled oder disabled 
 * menu == NIL: ein installiertes globales Menu wird disabled oder enabled
 */

_VOID WdwManager_InstallGlobalKeyProc(handleGlobalKeyProc key);
/* Installiert einen globalen Keyhandler, der dann aufgerufen wird, wenn
 * die Taste von keinem Fenster behandelt wurde.
 * Wenn Shift und eine andere Sondertaste gedrueckt wird, dann wird 
 * diese Callback-Funktion vor den Fenstern aufgerufen
 */

/*------- QueueModul ----------------------------------------------------------*/

_VOID WdwManager_NewWindowIsTop(HWND handle, _WORD apId);
/* Dem Modul wird mitgeteilt, das ein neues Fenster getopt 
 * wurde. Nur noetig bei nicht vom Modul verwalteten Fenstern
 */

_VOID WdwManager_WindowIsClosed(HWND handle);
/* Dem Modul wird mitgeteilt, das ein Fenster geschlossen 
 * wurde. Nur noetig bei nicht vom Modul verwalteten Fenstern.
 */

_VOID WdwManager_TopNextWindow(_VOID);
/* Das naechste Window wird getoppt.
 */

_BOOL WdwManager_TopNextPossible(_VOID);
/* Es wird zurueckgeliefert, ob es moeglich ist,
 * ein anderes Fenster zu toppen.
 */

_BOOL WdwManager_GetSecondWdw(HWND *wdw);
/* Liefert zweitoberstes Fenster zurueck.
 * Geht nur ueber Queue, benutzt nicht die Liste.
 * Gibt TRUE zurueck, falls erfolgreich.
 */

HWND WdwManager_GetOwnTopWindow(_VOID);
/* Liefert das Topwindow der Applikation zurueck */

_BOOL WdwManager_IsInQueue(HWND wdw);
/* Liefert TRUE zurueck, wenn das Fenster wdw in 
 * der verwalteten Queue ist.
 */

_BOOL WdwManager_GetApId(HWND wdw, _WORD *apId);
/* Liefert TRUE zurueck, wenn das Fenster bekannt ist, und die 
 * Applikation-ID der Applikation zum Fenster.
 * Funktioniert nur fuer fremde Fenster
 */

HWND WdwManager_FindSpecialTop(queueTestProc hdlTest);
/* Sucht nach einem bestimmten Fenster von Anfang der Queue
 * bis zum Ende. Bricht ab, wenn proc TRUE liefert.
 * Gibt das Fensterhandle zurueck oder -1
 */

#define STACKED         0
#define BYSIDE          1
#define OVERLAP         2
#define TILED           3

_VOID WdwManager_ArrangeWindows(_WORD mode);
/* Ordnet die Fenster an. Es ist einer der 
 * vorher definierten Anordnungsmodi moeglich.
 */

/* Temporaere Funktion, um Support von Iconify von MTOS 1.07 Beta einzuschalten
 */
_VOID WdwManager_EnableIconify(_BOOL enable);

/* Drag & Drop Support fuer MTOS */

_VOID WdwManager_WdwInstallDDClient(HWND wdw, getExtsProc getExts, acceptDataProc acceptData,
										  writeDataProc writeData);
/* Ein Fenster installiert fuer sich Drag & Drop. Damit unterstuetzt
 * es automatisch Drag & Drop. 
 */

_VOID WdwManager_WdwInstallDDServer(HWND wdw, getHeaderProc getHeader, readDataProc readData);
/* Ein Fenster installiert sich als Server fuer Drag & Drop. 
 * Danach kann es dann die Prozedur WdwDDServe benutzen, 
 * die Drag & Drop durchfuehrt.
 */

_BOOL WdwManager_WdwDDServe(HWND wdw, _WORD mx, _WORD my, _UWORD kstate);
/* Initialisiert Drag & Drop als Server. wdw ist das Source-Fenster(!!),
 * vorher muss sich das Fenster als Server angemeldet haben. Dann wird
 * ueber die angemeldeten Prozeduren versucht, Drag & Drop durchzufuehren
 */

handleNewTopProc WdwManager_SetHandleNewTop(handleNewTopProc proc);
/* Setzt den Handler fuer Fensterwechsel */

handleEventProc WdwManager_SetHandlePostEvent(handleEventProc proc);
/* Setzt den Handler fuer Ereignisse */

_BOOL WdwManager_AcceptFont(HWND win, FONT_ACCEPT_FUNC func);
_BOOL WdwManager_AcceptsFonts(HWND win, FONT_ACCEPT_FUNC *func);
_BOOL WdwManager_AcceptColor(HWND wdw, COLOR_ACCEPT_FUNC func);
_BOOL WdwManager_AcceptsColor(HWND wdw, COLOR_ACCEPT_FUNC *func);

_VOID WdwManager_Snapshot(HWND win);

_VOID WdwManager_Init(_VOID);
_VOID allRedrawWdw(_VOID);

#endif /* __CAT_WDWMANAGER_H__ */
