/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#define __LISTDL_IMPLEMENTATION__

#include <listdl.h>
#include <portaes.h>
#include <rectfunc.h>
#include <mtutils.h>
#include <mtappl.h>
#include <mtalerts.h>
#include <mtdials.h>
#include <lists.h>
#include <listutil.h>
#include <windials.h>
#include <wdwmanag.h>
#include <routinec.h>
#include <routines.h>
#include <ro_mem.h>
#include <portvdi.h>
#include <mtxobjec.h>	



struct _dial {
	/* externe Parameter */
	OBJECT *tree;
	LISTINFO listInfo;
	_VOID *env;
	_WORD boxIndex;
	_WORD backIndex;
	_WORD okBut;
	_WORD cancelBut;
	_WORD doubleClickBut;
	ldElemSet flags;
	_WORD autoEdField;
	_LONG lastSel;
	ldHandler hdl;
	List currentList;
	
	/* Fuer Fensterdialoge: */
	wdSetProc listSetValues;
	wdSetProc listGetValues;
	lsCheckExitProc listCheckExit;
	_VOID **lastElemPtr;

	specialButHdler *listSpecials;
	_WORD listSpecialCount;
	_WORD autoEdit;

	inLoopProc inLoop;
	_WORD exitBut;
	_BOOL initial;

	DIAL next;
};


#define is_num(c) ((c) >= '0' && (c) <= '9')
#define is_space_num(c) (is_num(c) || (c) == ' ')
#define is_special(c) ((c) == '\b' || (c) == '|')


LOCAL DIAL dialList;
LOCAL DIAL autoDial;

#define LISTDL_UNDO        (-1)
#define LISTDL_CURSOR_UP   (-3)
#define LISTDL_CURSOR_DOWN (-4)
#define LISTDL_HOME        (-5)
#define LISTDL_END         (-6)
#define LISTDL_CANCEL      (-7)
 
/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID ListDl_SetUserkeys(OBJECT *tree, _BOOL set)
{
	_WORD i;
	
	for (i = 0; i < 5; i++)
	{
		mtDials_SetUserkey(tree, LISTDL_CURSOR_UP,   0x48, i, FALSE, set); /* Cursor Up */
		mtDials_SetUserkey(tree, LISTDL_CURSOR_DOWN, 0x50, i, FALSE, set); /* Cursor Down */
	}
	mtDials_SetUserkey(tree, LISTDL_HOME,  0x47, 0,        FALSE, set); /* Home */
	mtDials_SetUserkey(tree, LISTDL_END,   0x47, K_LSHIFT, FALSE, set); /* Shift-Home */
}

/*** ---------------------------------------------------------------------- ***/

/* Baut aus den uebergebenen Daten einen ldHandler zusammen
*/
GLOBAL _VOID ListDl_BuildLdHandler(
	_VOID *listStart,
	ResetProc resetList,
	MoveProc nextEntry,
	MoveProc prevEntry,
	CountProc listElems,
	entryIsSelectableProc isEnabled,
	selectEntryProc selEntry,
	entryToStringProc entryToStr,
	isSelectedProc isSelected,
	drawEntryProc drawEntry,
	dragDropProc dragDrop,
	_WORD voffset, _WORD vsize, _WORD hStep, _WORD maxWidth,
	ldHandler *handler)
{
	handler->list = listStart;
	handler->resetList = resetList;
	handler->nextEntry = nextEntry;
	handler->prevEntry = prevEntry;
	handler->listElems = listElems;
	handler->isEnabled = isEnabled;
	handler->selEntry = selEntry;
	handler->entryToStr = entryToStr;
	handler->entry4SToStr = entryToStr;/*aus Kompatibilittsgrnden*/
	handler->isSelected = isSelected;
	handler->drawEntry = drawEntry;
	handler->dragDrop = dragDrop;
	handler->voffset = voffset;
	handler->vsize = vsize;
	handler->hStep = hStep;
	handler->maxWidth = maxWidth;
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID ListDl_BuildLdHandlerX(entryToStringProc entry4SToStr,ldHandler *handler)*/
/*	Setzt neue Parameter, ideal wre hier ein Defaultparameter gewesen,			*/
/*	aber den gibt es in C nicht													*/
/*	Parameter:	entry4SToStr:	Routine um Eintrag zu extrahieren zur Suche		*/
/*				handler:		der Handler fr den es gilt						*/
/*	Ergebnis:	keines															*/		
/*	20.3.00 von Dimitri Junker		DYJ 4.14a									*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID ListDl_BuildLdHandlerX(
	entryToStringProc entry4SToStr,
	ldHandler *handler)
{
	handler->entry4SToStr = entry4SToStr;
}

/*** ---------------------------------------------------------------------- ***/

/* Bereitet eine scrollbare Liste vor.
 * Dabei muss ein DialInit schon erfolgt sein.
 */
GLOBAL _BOOL ListDl_PrepareListDial(
	OBJECT *tree,
	ldElemSet elems,
	ldHandler *handler,
	_WORD boxIndex,
	_WORD boxBack,
	_WORD okBut,
	_WORD cancelBut,
	_VOID *env,
	_WORD doubleClickButton,
	DIAL *pd)
{
	DIAL d, d2;
	selectMode selMode;

	*pd = d = NEW(DialInfo, "ListDl_PrepareDial");
	if (d == NULL)
		return FALSE;
	MemSetZeroStruct(*d);

	d->next = NULL;
	d->tree       = tree;
	d->flags      = elems;
	d->hdl        = *handler;
	d->inLoop     = FUNK_NULL;

	d->boxIndex   = boxIndex;
	d->backIndex  = boxBack;
	d->okBut      = okBut;
	d->cancelBut  = cancelBut;
	d->env        = env;
	d->doubleClickBut = doubleClickButton;
	/* Jetzt noch ein paar Sachen initialisieren */

	/* selMode festlegen */
	if (!(elems & ldSelect))
	{
		selMode = smNone;
	} else if (elems & ldMultiple)
	{
		selMode = smMulti;
	} else
	{
		selMode = smOne;
	}
	/* ListInfo erzeugen */
	if (!ListMan_InitList(
		handler->list,
		env,
		handler->selEntry,
		handler->isSelected,
		handler->isEnabled,
		handler->drawEntry,
		handler->dragDrop,
		handler->resetList,
		handler->nextEntry,
		handler->prevEntry,
		handler->listElems,
		selMode,
		&d->listInfo))
	{
		OFREE(d);
		return FALSE;
	}
	Lists_InitList(&d->currentList);
	
	if (!ListMan_SetListDialSpecs(
		d->listInfo,
		tree, boxIndex, boxBack,
		handler->voffset, handler->vsize, handler->hStep, handler->maxWidth))
	{
		ListMan_FreeList(d->listInfo);
		OFREE(d);
		return FALSE;
	}

	/* Jetzt in Liste haengen */
	if (dialList == NULL)
	{
		dialList = d;
	} else
	{
		d2 = dialList;
		while (d2->next != NULL)
			d2 = d2->next;
		d2->next = d;
	}

	/* Selektierten Eintrag suchen! */
	ListMan_ListScroll2Selection(d->listInfo, FALSE);
	d->lastSel = ListMan_ListFirstSelected(d->listInfo);

	if (elems & ldArrows)
	{
		ListDl_SetUserkeys(tree, TRUE);
	}

	/*
	mtDials_DialDraw(tree, ROOT, MAX_DEPTH, &sFrame, FALSE);
	*/
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* gibt einen Listenhandler wieder frei. */
GLOBAL _BOOL ListDl_ReleaseListDial(DIAL *pd)
{
	DIAL d, d2;

	if ((d = *pd) == NULL)
		return FALSE;
	d2 = dialList;
	if (d2 == d)
	{
		dialList = d->next;
	} else
	{
		while (d2 != NULL && d2->next != d)
			d2 = d2->next;
		if (d2 == NULL)
			return FALSE;
		/* d aushaengen */
		d2->next = d->next;
	}
	/* ListMan freigeben */
	ListMan_FreeList(d->listInfo);
	if (d->flags & ldArrows)
	{
		/* Userkeys wieder austragen */
		ListDl_SetUserkeys(d->tree, FALSE);
	}
	Lists_DeleteSimpleList(&d->currentList, FUNK_NULL, TRUE);
	OFREE(d);
	*pd = NULL;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt den zu dem Tree und dem Button gehoerenden DIAL zurueck,
 * ansonsten NULL.
 * Wenn der Button < 0 ist, wird nur nach dem Tree gesucht.
 */
GLOBAL DIAL ListDl_FindDial(OBJECT *tree, _WORD button, _BOOL treeOnly)
{
	DIAL d;
	_WORD but;
	_BOOL useArrows;

	d = dialList;
	if(button<0)
		treeOnly=TRUE;
	but = button & 0x7fff;
	while (d != NULL)
	{
		useArrows = d->flags & ldArrows;
		if (d->tree == tree)
		{
			if (treeOnly)
				return d;
			if ((but == d->boxIndex) ||
				(but == d->backIndex ) ||
				(button == LISTDL_UNDO) ||
				((button == LISTDL_CURSOR_UP) && useArrows) ||
				((button == LISTDL_CURSOR_DOWN) && useArrows) ||
				((button == LISTDL_HOME) && useArrows) ||
				((button == LISTDL_END) && useArrows) ||
				(but == d->okBut) ||
				(but == d->cancelBut) ||
				(button == LISTDL_CANCEL))
			{
				return d;
			}
		}
		d = d->next;
	}
	return d;
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt den DoubleClickButton */
GLOBAL _VOID ListDl_SetDcButton(OBJECT *tree, _WORD button)
{
	DIAL dial;
	
	dial = ListDl_FindDial(tree, -1, TRUE);
	if (dial != NULL)
		dial->doubleClickBut = button;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListDl_InitAutoLocator(DIAL dial)
{
	autoDial = dial;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListDl_FreeAutoLocator(_VOID)
{
	autoDial = NULL;
}

/*** ------------------------------------------------------------------------ ***/
GLOBAL _UWORD ListDl_AutoLocate(OBJECT *tree, _WORD mx, _WORD my, _UBYTE scan, _UWORD kstate, _UWORD event, _WORD edObj, _VOID *special)
{
	_WORD i,step=1;
	_LONG idx;
	_VOID *ptx;
	DIAL d;
	_UBYTE search[256];
	_UBYTE lastStr[256];
	_UBYTE str[256];
	_BOOL cmpstr;
	size_t len;

	UNUSED(mx);
	UNUSED(my);
	UNUSED(special);
	if (!(event & MU_KEYBD))
		return 0;			/* Ist uns egal */
	/* Cursortasten, Funktionstasten und noch ein paar Sondertasten abfangen */
	if ((scan >= 59 && scan <= 72) ||
		scan == 75 ||
		scan == 77 ||
		(scan >= 80  && scan <= 98) ||
		scan == 42 ||
		scan == 54 ||
		scan == 71)
	{
		return 0;
	}

	/* So, jetzt ist das also fuer uns interessant! */
	if (autoDial == NULL)
	{
		/* Dial suchen */
		d = ListDl_FindDial(tree, LISTDL_UNDO, TRUE);
		if (d == NULL)
			return 0;
	} else
	{
		d = autoDial;
	}
	/* Jetzt suchen */
	/* Test auf Namecompletion */
	d->autoEdField = edObj;
	if (d->autoEdField >= 0)
	{
		/* String aus Editfeld holen */
		ObjcString(tree, edObj, search, sizeof(search));
		strUpr(search);
		strleft(search);
	} else
	{
		/* Nur Taste auswerten */
		search[0] = ucase(CharCode(scan, kstate));
		search[1] = '\0';
	}
	/* ListBox an aktuelle Koordinaten anpassen */
	/* DYJ: ist d->hdl.compTyp!=0 soll nicht ergnzt werden sondern zum nchsten passenden gewchselt werden*/
	if (scan == 15 /* TAB */ && d->autoEdField >= 0 && ((d->flags&ldTabEqNxt)==0))
	{
		lastStr[0] = '\0';
		d->hdl.resetList(d->hdl.list);
		/* Namen komplettieren */
		do
		{
			ptx = d->hdl.nextEntry(d->hdl.list);
			if (ptx != NULL)
			{
				d->hdl.entryToStr(ptx, d->env, str, sizeof(str));
				strleft(str);
				if (d->hdl.isEnabled(ptx, d->env) && strnCaseCmp(str, search, strLen(search)) == 0)
				{
					/* Ist enthalten, also merken */
					if (lastStr[0] != '\0')
					{
						/* vergleichen mit letztem, bis der erste Unterschied auftritt */
						i = 0;
						while (lastStr[i] != '\0' && ucase(str[i]) == ucase(lastStr[i]))
							++i;
						strCpy(lastStr, str);
						lastStr[i] = '\0';
					} else
					{
						strCpy(lastStr, str);
					}
				}
			}
		} while (ptx != NULL);
		if (lastStr[0] != '\0')
		{
			strdelRightSpace(lastStr);
			SetObjcString(tree, edObj, lastStr);
			WinDials_WinEditChanged(tree, edObj);
			return 1;
		} else
		{
			return 0;
		}
	} else
	{
		if (d->flags & ldShrinkList)
			ListMan_SetNewList(d->listInfo, d->hdl.list, FALSE);
		len = strLen(search);
		if (len != 0)
		{
			Lists_DeleteSimpleList(&d->currentList, FUNK_NULL, TRUE);
			if (d->flags & ldShrinkList)
			{
				if (!Lists_CreateList(&d->currentList, "ListDl_AutoLocate: currentList"))
					return 0;
				d->currentList.user = ((List *)d->hdl.list)->user;
			}
			switch(scan)
			{
				case 15:		/* TAB */
					if (d->flags&ldTabEqNxt)
					{
						if((kstate&3)==0)	/* keine Shift Taste	*/
						{
							step=1;
							if(d->lastSel==ListMan_ListListElems(d->listInfo))
							{
								idx=-1;
								ptx=NULL;
							}
							else
							{
								idx=d->lastSel+1;	/*DYJ ab dem nchsten Element suchen*/
								if((ptx = d->hdl.nextEntry(d->hdl.list))==NULL)
									step=0;
							}
						}
						else
						{
							step=-1;
							if(d->lastSel<0 )
							{
								idx=ListMan_ListListElems(d->listInfo);
								ptx=NULL;
							}
							else
							{
								idx=d->lastSel-1;	/*DYJ ab dem vorigen Element suchen*/
								if((ptx = d->hdl.prevEntry(d->hdl.list))==NULL)
									step=0;
							}
						}
						break;
					}
				case 0:			/* keine nderung*/
					if (d->flags&ldTabEqNxt && d->lastSel>-1)
					{
						idx=d->lastSel;	/*DYJ ab dem nchsten(TAB) bzw gleichen(0) Element suchen*/
						ptx = d->hdl.prevEntry(d->hdl.list);
						ptx=*d->lastElemPtr;
						break;
					}
				default:
					idx = 0;
					d->hdl.resetList(d->hdl.list);	/* sonst ab Start suchen*/
					ptx = d->hdl.nextEntry(d->hdl.list);
			}
			if(step)
			{
				do
				{
					if (ptx != NULL)
					{
						d->hdl.entry4SToStr(ptx, d->env, str, sizeof(str));
						strUpr(str);
						strleft(str);
						if(d->flags&ldMatchAny)
							cmpstr=strStr(str, search) !=NULL;
						else
							cmpstr=strnCmp(str, search, len) == 0;
						if (d->hdl.isEnabled(ptx, d->env) && cmpstr)
						{
							/* Jetzt auch darstellen */
							*d->lastElemPtr = ptx;
							ListMan_ListInvertEntry(d->listInfo, idx);
							d->lastSel = idx;
							if (d->inLoop != FUNK_NULL)
								d->inLoop(ptx, d->env);
							return 0;
						}
					}
					idx+=step;
					if(step==1)
						ptx = d->hdl.nextEntry(d->hdl.list);
					else
						ptx = d->hdl.prevEntry(d->hdl.list);
				} while (ptx != NULL);
			}
		} else
		{
			ptx = NULL;
		}
		/* Nichts gefunden, alten Eintrag deselektieren */
		if (d->lastSel >= 0)
			ListMan_ListDeselEntry(d->listInfo, d->lastSel);
		if(step==1)
			d->lastSel = -1;
		else
			d->lastSel = ListMan_ListListElems(d->listInfo);
		if (d->inLoop != FUNK_NULL)
			d->inLoop(ptx, d->env);
		if (!Lists_ListEmpty(&d->currentList))
		{
			Lists_DeleteSimpleList(&d->currentList, FUNK_NULL, TRUE);
			ListMan_SetNewList(d->listInfo, d->hdl.list, TRUE);
		}
	}
	return 0;
}
/*** ---------------------------------------------------------------------- ***/

/* Verwaltet die Events, wenn etwas in diesem Dial angeklickt wurde
 */
GLOBAL _BOOL ListDl_HandleListDial(DIAL d, _WORD *but, _WORD mx, _WORD my, _UWORD kState, _VOID **lastElem)
{
	GRECT sFrame;
	_BOOL doubleClick;
	_WORD clicks;
	_LONG selLast, oldLast;
	_BOOL noSelect;
	_BOOL multiple;

	noSelect = !(d->flags & ldSelect);
	multiple = (d->flags & ldMultiple) != 0;
	oldLast = d->lastSel;
	if (*but == -1) /* Userproc */
	{
		/* Redraw von edObj */
		if (d->autoEdField >= 0)
			WinDials_WinDialDraw(d->tree, d->autoEdField, 0, &sFrame, FALSE);
		return FALSE;
	}
	if (d->initial && *but < 0)
		return FALSE;

	if (*but == LISTDL_CURSOR_UP)      /* Cursor Up */
	{
		if ((kState & K_CTRL) || noSelect || multiple)
		{
			/* Ctrl */
			if (kState & (K_LSHIFT | K_RSHIFT))
			{
				/* Ctrl-Shift */
				ListMan_ListPgUp(d->listInfo);
			} else
			{
				ListMan_ListLnUp(d->listInfo);
			}
		} else
		{
			if (kState & (K_LSHIFT | K_RSHIFT))
			{
				/* Shift + Control */
				d->lastSel = ListMan_ListSelPgUp(d->listInfo);
			} else
			{
				d->lastSel = ListMan_ListSelUp(d->listInfo);
			}
		}
	} else if (*but == LISTDL_CURSOR_DOWN) /* Cursor Down */
	{
		if ((kState & K_CTRL) || noSelect || multiple)
		{
			/* ctrl: Scrollen */
			if (kState & (K_LSHIFT | K_RSHIFT))
			{
				/* Shift */
				ListMan_ListPgDown(d->listInfo);
			} else
			{
				ListMan_ListLnDown(d->listInfo);
			}
		} else
		{
			/* Selected Entry Down   */
			if (kState & (K_LSHIFT | K_RSHIFT))
			{
				d->lastSel = ListMan_ListSelPgDown(d->listInfo);
			} else
			{
				d->lastSel = ListMan_ListSelDown(d->listInfo);
			}
		}
	} else if (*but == LISTDL_HOME)
	{
		/* Home: Nach oben gehen */
		if (noSelect || multiple)
		{
			ListMan_ListFirst(d->listInfo);
		} else
		{
			d->lastSel = ListMan_ListSelFirst(d->listInfo);
		}
	} else if (*but == LISTDL_END)
	{
		/* Shift-Home: Ans Ende gehen */
		if (noSelect || multiple)
		{
			ListMan_ListLast(d->listInfo);
		} else
		{
			d->lastSel = ListMan_ListSelLast(d->listInfo);
		}
	} else if (*but == LISTDL_CANCEL)
	{
		return TRUE;
	}
	
	*lastElem = ListMan_ListFindEntry(d->listInfo, d->lastSel);
	if (d->lastSel != oldLast)
		WinDials_WinEditChanged(d->tree, -1);
	doubleClick = (*but & 0x8000) != 0;
	*but &= 0x7fff;
	if (*but == d->okBut || *but == d->cancelBut)
	{
		return TRUE;
	} else if (*but == d->boxIndex || *but == d->backIndex)
	{
		clicks = 1;
		if (doubleClick)
			++clicks;
		selLast = ListMan_ListClick(d->listInfo, mx, my, kState, clicks);
		if (selLast >= -1)
		{
			d->lastSel = selLast;
			WinDials_WinEditChanged(d->tree, -1);
		}
		*lastElem = ListMan_ListFindEntry(d->listInfo, d->lastSel);
		if (*lastElem != NULL && *but == d->boxIndex &&
			((doubleClick && (d->flags & ldDoubleExit) && d->doubleClickBut >= 0) ||
			 (d->flags & ldAutoSelect)))
		{
			if (d->inLoop != FUNK_NULL)
				d->inLoop(*d->lastElemPtr, d->env);
			if ((!GetObjState(d->tree, d->doubleClickBut, DISABLED) &&
				 GetObjFlags(d->tree, d->doubleClickBut, SELECTABLE)) ||
				 (d->flags & ldAutoSelect))
			{
				*but = d->doubleClickBut;
				if (*but > 0)
				{
					SetObjState(d->tree, *but, SELECTED);
					WinDials_WinDialDraw(d->tree, *but, 0, &sFrame, FALSE);
				}
				return *but == d->okBut || *but == d->cancelBut;
			}
		}
	}
	return FALSE;
}
/*** ---------------------------------------------------------------------- ***/

/* Dummy-Prozedur als InLoopProc */
GLOBAL _VOID ListDl_dummyInLoop(_VOID *entry, _VOID *env)
{
	UNUSED(entry);
	UNUSED(env);
}

/*** ---------------------------------------------------------------------- ***/

/* Dummy-Prozedur fuer Exitcheck, gibt immer TRUE zurueck */
GLOBAL _BOOL ListDl_dummyCheckExit(OBJECT *tree, _VOID *env, _WORD button, _VOID *lastSel, _UWORD kstate)
{
	UNUSED(tree);
	UNUSED(env);
	UNUSED(button);
	UNUSED(lastSel);
	UNUSED(kstate);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet die Liste komplett inklusive aller Elemente */
GLOBAL _VOID ListDl_DrawListDial(DIAL d, CONST GRECT *clip)
{
	GRECT upperFrame;

	d->initial = FALSE;
	CalcArea(d->tree, d->backIndex, &upperFrame);
	if (rc_intersect(clip, &upperFrame))
		ListMan_DrawList(d->listInfo, &upperFrame);
}

/*** ---------------------------------------------------------------------- ***/

/* Handelt eine komplette Scrollbare Liste.
 * Gibt den Index des Exit-Buttons zurueck.
 * In lastElem wird die Adresse des zuletzt selektierten Elements zurueckgegeben.
 * Bei multiple == TRUE koennen mehrere Eintraege selektiert werden.
 */

GLOBAL _WORD ListDl_doListDial(
	OBJECT *tree,
	ldElemSet elems,
	ldHandler *handler,
	_WORD boxIndex,
	_WORD boxBack,
	_WORD okBut,
	_WORD cancelBut,
	_VOID *env,
	_WORD doubleClickButton,
	_WORD autoEdit,
	inLoopProc inLoop,
	specialButHdler *specials,
	_WORD numSpecials,
	wdSetProc setValues,
	wdSetProc getValues,
	_VOID **lastElem)
{
	_WORD but;
	_UWORD buts;
	_UWORD kstate;
	_BOOL draw, exit;
	DIAL d, d2;
	GRECT lowerFrame, upperFrame;
	_WORD mx, my;
	_WORD i;
	_UBYTE str[256];
	_UBYTE str2[256];

	UNUSED(setValues);
	UNUSED(getValues);

	if (elems & ldAutolocate)
		mtDials_InstallHandler(tree, ListDl_AutoLocate, CallByHandling, 0, 0, &upperFrame, NULL, NULL);

	mtDials_DialCenter(tree, CSCREEN, 0, 0, &upperFrame);
	mtDials_DialForm(tree, DSTART, &upperFrame, &lowerFrame);

	if (!ListDl_PrepareListDial(tree, elems,
	    handler,
	    boxIndex, boxBack,
	    okBut, cancelBut,
	    env, doubleClickButton,
	    &d))
	{
		*lastElem = NULL;
		mtDials_DialForm(tree, DFINISH, &upperFrame, &lowerFrame);
		return cancelBut;
	}
	d->lastElemPtr = lastElem;
	Wind_Update(BEG_UPDATE);
	Wind_Update(BEG_MCTRL);
	mtDials_DialDraw(tree, ROOT, MAX_DEPTH, &upperFrame, FALSE);
	CalcArea(tree, boxBack, &upperFrame);
	ListMan_DrawList(d->listInfo, &upperFrame);
	if (elems & ldAutolocate)
	{
		buts = ListDl_AutoLocate(tree, 0, 0, 0, 0, MU_KEYBD, autoEdit, d);
	} else
	{
	}
	MouseArrow();
	for (;;)
	{
		if (d->inLoop != FUNK_NULL)
			inLoop(*lastElem, env);      /* inLoopProc aufrufen */

		but = mtDials_DialDo(tree, 0);
		graf_mkstate(&mx, &my, &buts, &kstate);
		kstate = mtDials_GetKbdState();

		d2 = ListDl_FindDial(tree, but, FALSE);
		if (d2 != NULL)
		{
			if (ListDl_HandleListDial(d2, &but, mx, my, kstate, lastElem))
				break;
		}
		if (d->inLoop != FUNK_NULL)
			inLoop(*lastElem, env);      /* inLoopProc aufrufen */

		/* specials */
		but &= 0x7fff;
		exit = FALSE;
		for (i = 0; i < numSpecials; i++)
		{
			if (but == specials[i].objc &&
				!GetObjState(tree, but, DISABLED) &&
				GetObjFlags(tree, but, SELECTABLE))
			{
/*DYJ 5.30f Start*/
				specials[i].proc(but, env, d->listInfo, &draw, &exit,&d->lastSel);
/*DYJ  Ende; alt:	
				specials[i].proc(but, env, d->listInfo, &draw, &exit);
*/
				if (draw)
				{
					CalcArea(tree, boxBack, &upperFrame);
					ListMan_DrawList(d->listInfo, &upperFrame);
					if (elems & ldAutolocate)
						buts = ListDl_AutoLocate(tree, 0, 0, 0, 0, MU_KEYBD, autoEdit, d);
					d->lastSel = ListMan_ListFirstSelected(d->listInfo);
					*d->lastElemPtr = ListMan_ListFindEntry(d->listInfo, d->lastSel);
				}
				if (exit)
					break;
			}
		}
		if (exit)
			break;
	}
	kstate = mtDials_GetKbdState();
	if ((elems & ldAutolocate) && d->lastSel >= 0 && autoEdit >= 0)
	{
		/* Ein Eintrag ist selektiert und der Dialog wurde nicht mit Shift-xxx verlassen */
		*lastElem = ListMan_ListFindEntry(d->listInfo, d->lastSel);
		/* String noch setzen */
		if (*lastElem != NULL)
		{
			d->hdl.entryToStr(*lastElem, env, str, sizeof(str));
			if (!(kstate & (K_LSHIFT | K_RSHIFT)))
			{
				SetObjcString(d->tree, autoEdit, str);
			} else
			{
				/* Vergleichen, ob der String im Editfeld und der von dem
				 * selektierten Eintrag gleich sind
				 */
				ObjcString(d->tree, autoEdit, str2, sizeof(str2));
				if (strCmp(str2, str) != 0)
					*lastElem = NULL;
			}
		}
	}
	DelObjState(tree, but, SELECTED);
	ListDl_ReleaseListDial(&d);
	mtDials_DialForm(tree, DFINISH, &upperFrame, &lowerFrame);
	mtDials_RemoveHandler(tree);
	Wind_Update(END_MCTRL);
	Wind_Update(END_UPDATE);
	return but;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

typedef struct _listbox {
	OBJECT *tree;
	_WORD current;
	_WORD selected;
	CONST _UBYTE **strings;
	_WORD numStrings;
	_WORD globalLength;
} LISTBOX;

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID listboxReset(_VOID *list)
{
	LISTBOX *box;
	
	box = list;
	box->current = 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *listboxNext(_VOID *list)
{
	LISTBOX *box;
	
	box = list;
	if (box->current < box->numStrings)
	{
		++box->current;
		return &box->strings[box->current - 1];
	}
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID *listboxPrev(_VOID *list)
{
	LISTBOX *box;
	
	box = list;
	if (box->current > 0)
	{
		--box->current;
		return &box->strings[box->current];
	}
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL listboxEnabled(_VOID *entry, _VOID *list)
{
	CONST _UBYTE **str;
	
	UNUSED(list);
	str = entry;
	return **str != '-';
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID listboxCount(_VOID *list, _LONG *length, _WORD *width)
{
	LISTBOX *box;
	
	box = list;
	*length = box->numStrings;
	*width = box->globalLength;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL listboxSelect(_VOID *entry, _VOID *list, _WORD line)
{
	LISTBOX *box;
	CONST _UBYTE **str;
	_WORD selected;
	
	box = list;
	str = entry;
	selected = (_WORD)(str - box->strings);
	if (selected == box->selected)
		box->selected = -1;
	else
		box->selected = selected;
	UNUSED(line);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL listboxIsSelected(_VOID *entry, _VOID *list)
{
	LISTBOX *box;
	
	box = list;
	return box->selected >= 0 && entry == &box->strings[box->selected];
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID listboxToString(_VOID *entry, _VOID *list, _UBYTE *str, size_t strsize)
{
	CONST _UBYTE **string;
	
	UNUSED(list);
	string = entry;
	strMcpy(str, strsize, *string);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID listboxDrawEntry(_VOID *entry, _VOID *env, _WORD x, _WORD y, _WORD offset, CONST GRECT *pclip)
{
	CONST _UBYTE **str;
	LISTBOX *box;
	_WORD fillcol;
	GRECT fill;
	GRECT clip;
	_BOOL isSel;
	
	UNUSED(offset);
	
	str = entry;
	box = env;
	
	fill.g_x = x;
	fill.g_y = y;
	fill.g_w = fill.g_x + box->globalLength - 1;
	fill.g_h = fill.g_y + CharHeight - 1;
	
	clip = *pclip;
	clip.g_w = clip.g_x + clip.g_w - 1;
	clip.g_h = clip.g_y + clip.g_h - 1;
	vs_clip(PrivateWS, TRUE, (_WORD *)&clip);
	
	vswr_mode(PrivateWS, MD_REPLACE);
	fillcol = WHITE;
	isSel = FALSE;
	if (entry != NULL && listboxIsSelected(entry, env))
	{
		fillcol = BLACK;
		isSel = TRUE;
	}
	MouseOff();
	vsf_color(PrivateWS, fillcol);
	vr_recfl(PrivateWS, (_WORD *)&fill);
	if (entry != NULL)
	{
		if (!listboxEnabled(entry, env))
		{
			_WORD pxy[4];
			
			vsf_color(PrivateWS, BLACK);
			vsf_interior(PrivateWS, FIS_PATTERN);
			vsf_style(PrivateWS, IP_4PATT);
			pxy[0] = x;
			pxy[1] = y + CharHeight / 2 - 1;
			pxy[2] = x + box->globalLength - 1;
			pxy[3] = pxy[1] + 1;
			v_bar(PrivateWS, pxy);
			vsf_interior(PrivateWS, FIS_SOLID);
			vsf_style(PrivateWS, IP_HOLLOW);
			vsf_color(PrivateWS, WHITE);
		} else
		{
			if (isSel)
			{
				vswr_mode(PrivateWS, MD_XOR);
			}
			
			vst_effects(PrivateWS, TXT_NORMAL);
			v_gtext(PrivateWS, x, y, *str);
			vswr_mode(PrivateWS, MD_REPLACE);
		}
	}
	vs_clip(PrivateWS, FALSE, (_WORD *)&clip);
	MouseOn();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD listboxRect(OBJECT *tree, _WORD mx, _WORD my, _UBYTE scan, _UWORD kbshift, _UWORD event, _WORD edObj, _VOID *special)
{
	DIAL d;
	GRECT mouseRect;
	_WORD obj;
	_WORD index;
	LISTBOX *box;
	
	UNUSED(scan);
	UNUSED(kbshift);
	UNUSED(edObj);
	d = special;
	box = d->env;
	if (event & MU_M1)
	{
		mouseRect.g_x = mx;
		mouseRect.g_y = my;
		mouseRect.g_w = 1;
		mouseRect.g_h = 1;
		mtDials_InstallHandler(tree, listboxRect, CallByRect, 0, 1, &mouseRect, NULL, d);
		obj = objc_find(tree, ROOT, MAX_DEPTH, mx, my);
		if (obj == d->boxIndex)
		{
			ObjcArea(tree, d->boxIndex, &mouseRect);
			index = (_WORD)(ListMan_ListFirstEntry(d->listInfo) + (my - mouseRect.g_y) / d->hdl.vsize);
		} else
		{
			index = -1;
		}
		if (index != box->selected)
		{
			if (box->selected >= 0)
				ListMan_ListDeselEntry(d->listInfo, box->selected);
			if (index >= 0)
				ListMan_ListInvertEntry(d->listInfo, index);
		}
	}
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD ListDl_ListboxPopup(_WORD x, _WORD y, _WORD w, _WORD h, CONST _UBYTE **strings, _WORD numStrings, _WORD presel)
{
	_WORD but;
	DIAL d, d2;
	GRECT lowerFrame, upperFrame;
	_UWORD kstate;
	_UWORD buts;
	_WORD mx, my;
	ldElemSet elems;
	_VOID *lastElem;
	ldHandler handler;
	_WORD okButton;
	OBJECT tree[3] = {
		{ -1, 1, 1, G_BOX, NONE, SHADOWED, { 0xFF1100L }, 0,0, 29,10 },
		{ 0, 2, 2, G_BOXCHAR, TOUCHEXIT, NORMAL, { 0xFF1100L }, 0,0, 29,10 },
		{ 1, -1, -1, G_BOXCHAR, TOUCHEXIT|LASTOB, NORMAL, { 0xFF1100L }, 0,0, 27,10 },
	};
	_WORD boxIndex, boxBack;
	LISTBOX listbox;
	_WORD i;
	GRECT mouseRect;
	_UWORD udummy;
	
	elems = ldSelect | ldArrows | ldModal | ldDoubleExit | ldAutoSelect;
	okButton = ROOT;
	boxBack = 1;
	boxIndex = 2;
	
	tree[ROOT].ob_x = x;
/* das +-10 ist ein Sicherheitsabstand wegen Rand u..*/
	if (y < (deskSize.g_y+10))
		y = deskSize.g_y+10;
	if ((y+h) > (deskSize.g_y+deskSize.g_h-10))
		y = deskSize.g_y+deskSize.g_h-h-10;

	tree[ROOT].ob_y = y;
	tree[ROOT].ob_width = w + 2 * CharWidth;
	if (h > numStrings * CharHeight)
		h = numStrings * CharHeight;
	tree[ROOT].ob_height = h;
	tree[boxBack].ob_width = tree[ROOT].ob_width;
	tree[boxBack].ob_height = h;
	tree[boxIndex].ob_width = w;
	tree[boxIndex].ob_height = h;
	
	if (!mtDials_NewDial(tree))
		return -1;
	
	CalcArea(tree, ROOT, &upperFrame);
	lowerFrame = upperFrame;
	mtDials_DialForm(tree, DSTART, &upperFrame, &lowerFrame);
	
	listbox.tree = tree;
	listbox.selected = presel;
	listbox.strings = strings;
	listbox.numStrings = numStrings;
	listbox.globalLength = 0;
	if (AESFontMonospaced)
	{
		for (i = 0; i < numStrings; i++)
			listbox.globalLength = Max(listbox.globalLength, (_WORD)strLen(strings[i]));
		listbox.globalLength *= CharWidth;
	} else
	{
		_WORD extent[8];
		
		for (i = 0; i < numStrings; i++)
		{
			vqt_extent(PrivateWS, strings[i], extent);
			listbox.globalLength = Max(listbox.globalLength, extent[2] - extent[0]);
		}
	}
	listbox.globalLength = tree[boxIndex].ob_width;
	
	ListDl_BuildLdHandler(&listbox,
		listboxReset,
		listboxNext,
		listboxPrev,
		listboxCount,
		listboxEnabled,
		listboxSelect,
		listboxToString,
		listboxIsSelected,
		listboxDrawEntry,
		FUNK_NULL,
		0, CharHeight,
		8, 0,
		&handler);
	
	if (!ListDl_PrepareListDial(tree, elems,
		&handler,
		boxIndex, boxBack,
		okButton, -1,
		&listbox, okButton,
	    &d))
	{
		mtDials_DialForm(tree, DFINISH, &upperFrame, &lowerFrame);
		mtDials_DisposeDial(tree);
		return -1;
	}
	lastElem = NULL;
	d->lastElemPtr = &lastElem;
	Wind_Update(BEG_UPDATE);
	Wind_Update(BEG_MCTRL);
	graf_mkstate(&mouseRect.g_x, &mouseRect.g_y, &udummy, &udummy);
	mouseRect.g_w = 1;
	mouseRect.g_h = 1;
	mtDials_InstallHandler(tree, listboxRect, CallByRect, 0, 1, &mouseRect, NULL, d);
	mtDials_SetUserkey(tree, LISTDL_CANCEL, 0x61, 0, FALSE, TRUE);
	mtDials_SetUserkey(tree, LISTDL_CANCEL, 0x01, 0, FALSE, TRUE);
	mtDials_SetUserkey(tree, okButton, 0x1c, 0, FALSE, TRUE);
	mtDials_SetUserkey(tree, okButton, 0x72, 0, FALSE, TRUE);
	mtDials_AbortOnClick(tree);
	mtDials_DialDraw(tree, ROOT, MAX_DEPTH, &upperFrame, FALSE);
	CalcArea(tree, boxBack, &upperFrame);
	ListMan_DrawList(d->listInfo, &upperFrame);
	MouseArrow();
	for (;;)
	{
		if (mtDials_IsOverloadedDialDo())
		{
			WinDials_AlertsInWindow(FALSE);
			but = mtDials_DialDo(tree, 0);
			WinDials_AlertsInWindow(TRUE);
		} else
		{
			but = mtDials_DialDo(tree, 0);
		}
		graf_mkstate(&mx, &my, &buts, &kstate);
		kstate = mtDials_GetKbdState();

		d2 = ListDl_FindDial(tree, but, FALSE);
		if (d2 != NULL)
		{
			if (ListDl_HandleListDial(d2, &but, mx, my, kstate, &lastElem))
				break;
		}

		but &= 0x7fff;
	}
	mtDials_RemoveHandler(tree);
	ListDl_ReleaseListDial(&d);
	mtDials_DialForm(tree, DFINISH, &upperFrame, &lowerFrame);
	mtDials_RemoveHandler(tree);
	Wind_Update(END_MCTRL);
	Wind_Update(END_UPDATE);
	mtDials_DisposeDial(tree);
	if (but != okButton || lastElem == NULL)
		return -1;
	but = 0;
	handler.resetList(handler.list);
	while (handler.nextEntry(handler.list) != lastElem)
		but++;
	return but;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID listSetGetValues(OBJECT *tree, _VOID *env, _BOOL set, _WORD exitBut)
{
	DIAL d;
	_UBYTE str[256];
	_UBYTE str2[256];
	_UWORD kstate;

	d = env;
	if (set)
	{
		/* Userkeys setzen */
		if (d->flags & ldArrows)
		{
			ListDl_SetUserkeys(tree, TRUE);
		}

		/* Wird am Anfang aufgerufen */
		d->listSetValues(tree, d->env, set, exitBut);
		ListMan_SetListWdwSpecs(
			d->listInfo,
			WinDials_IsWinDial(tree),
			NULL,
			d->hdl.voffset, d->hdl.vsize, d->hdl.hStep, d->hdl.maxWidth);
	} else
	{
		/* Jetzt unseren Kram freigeben */

		kstate = mtDials_GetKbdState();
		if ((d->flags & ldAutolocate) && d->lastSel >= 0 && d->autoEdit >= 0)
		{
			/* Ein Eintrag ist selektiert und der Dialog wurde nicht mit Shift-xxx verlassen */
			*d->lastElemPtr = ListMan_ListFindEntry(d->listInfo, d->lastSel);
			/* String noch setzen */
			if (*d->lastElemPtr != NULL)
			{
				d->hdl.entryToStr(*d->lastElemPtr, d->env, str, sizeof(str));
				if (!(kstate & (K_LSHIFT | K_RSHIFT)))
				{
					SetObjcString(tree, d->autoEdit, str);
				} else
				{
					/* Vergleichen, ob der String im Editfeld und der von dem
					 * selektierten Eintrag gleich sind
					 */
					ObjcString(tree, d->autoEdit, str2, sizeof(str2));
					if (strCmp(str2, str) != 0)
						*d->lastElemPtr = NULL;
				}
			}
		}
		/* Get wird nur am Ende aufgerufen, wir koennen jetzt den DIAL freigeben */
		d->listGetValues(tree, d->env, set, d->exitBut);
		DelObjState(tree, d->exitBut, SELECTED);
		ListDl_ReleaseListDial(&d);
		mtDials_RemoveHandler(tree);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID listDraw(OBJECT *tree, _VOID *env, CONST GRECT *clip)
{
	DIAL d;
	
	d = env;
	ListMan_DrawList(d->listInfo, clip);
	if (d->initial)
	{
		d->initial = FALSE;
		/* Autolocator ggf. aufrufen */
		if (d->flags & ldAutolocate)
			ListDl_AutoLocate(tree, 0, 0, 0, 0, MU_KEYBD, d->autoEdit, d);
	}
}

/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _BOOL listMessage(OBJECT *tree, _VOID *env,const _WORD *msg)			*/
/*	Fhrt Listspezifische Messages aus, bisher nur WM_MOUSEWHEEL				*/
/*	Parameter:	tree:	das Formular ber dem das Rad bedient wurde				*/
/*				env:	das zugehrige dial										*/
/*				msg:	die komplette Message wie sie von evnt_multi kommt		*/
/*	Ergebnis:	TRUE falls Message abgehandelt wurde							*/		
/*	24.10.00 von Dimitri Junker	DYJ 4.18zk										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _BOOL listMessage(OBJECT *tree, _VOID *env,const _WORD *msg)
{
	DIAL d, d2;
	_WORD obj;
	_BOOL ret=FALSE;
	d = env;
	if (msg[0] == WM_MOUSEWHEEL)      /* Wheel */
	{
		obj=objc_find(tree,0,10,msg[5],msg[6]);
		d2 = ListDl_FindDial(tree, obj, FALSE);
		if (d2 != NULL && GetObtype(tree, obj)==G_BOXCHAR)
		{
			ListMan_ListLnMv(d->listInfo,-msg[4]);
			ret= TRUE;
		}
	}
	return ret;
}
/*** ---------------------------------------------------------------------- ***/
LOCAL _BOOL listButton(OBJECT *tree, _VOID *env, _WORD button, _WORD mx, _WORD my, _UWORD kstate, _WORD clicks)
{
	DIAL d, d2;
	_BOOL exit, draw;
	_WORD i;
	GRECT upperFrame;

	UNUSED(clicks);
	d = env;
	if (d->inLoop != FUNK_NULL)
		d->inLoop(*d->lastElemPtr, d->env);

	d2 = ListDl_FindDial(tree, button, FALSE);
	if (d2 != NULL)
	{
		if (ListDl_HandleListDial(d2, &button, mx, my, kstate, d->lastElemPtr))
		{
			/* Exit gewuenscht */
			if (d->inLoop != FUNK_NULL)
				d->inLoop(*d->lastElemPtr, d->env);
			d->exitBut = button;
			return d->listCheckExit(tree, d->env, button, *d->lastElemPtr, kstate);
		}
		if (d->inLoop != FUNK_NULL)
			d->inLoop(*d->lastElemPtr, d->env);
	}
	/* specials */
	if(button!=-1)
	{
		button &= 0x7fff;
		for (i = 0; i < d->listSpecialCount; i++)
		{
			if (button == d->listSpecials[i].objc &&
				!GetObjState(tree, button, DISABLED)
				/* && GetObjFlags(tree, button, SELECTABLE) */ )
			{
/*DYJ 5.30f Start*/
				d->listSpecials[i].proc(button, d->env, d->listInfo, &draw, &exit,&d->lastSel);
				if (draw)
				{
/*DYJ  Ende; alt:	
				d->listSpecials[i].proc(button, d->env, d->listInfo, &draw, &exit);
				if (draw)
				{
					d->lastSel = -1;	
*/
					if ((d->flags & ldAutolocate) && (d->flags & ldShrinkList))
						ListDl_AutoLocate(tree, 0, 0, 0, 0, MU_KEYBD, d->autoEdit, d);
					CalcArea(tree, d->backIndex, &upperFrame);
					ListMan_DrawList(d->listInfo, &upperFrame);
					if ((d->flags & ldAutolocate) && !(d->flags & ldShrinkList))
						ListDl_AutoLocate(tree, 0, 0, 0, 0, MU_KEYBD, d->autoEdit, d);
					d->lastSel = ListMan_ListFirstSelected(d->listInfo);
					*d->lastElemPtr = ListMan_ListFindEntry(d->listInfo, d->lastSel);
				}
				if (exit)
				{
					/* Eventuell rausgehen, wenn Validierung erfolgreich */
					d->exitBut = button;
					if (d->inLoop != FUNK_NULL)
						d->inLoop(*d->lastElemPtr, d->env);
					return d->listCheckExit(tree, d->env, button, *d->lastElemPtr, kstate);
				}
			} else if (button == d->listSpecials[i].objc)
			{
				/* Kann hier nur ankommen, wenn der Button disabled ist */
				DelObjState(tree, button, SELECTED);
				WinDials_WinDialDraw(tree, button, 0, &upperFrame, FALSE);
			}
		}
	}
	/*
	 * Muss wieder gesucht werden!!!
	 */
	d = ListDl_FindDial(tree, 0, TRUE);
	if (d != NULL && d->inLoop != FUNK_NULL)
		d->inLoop(*d->lastElemPtr, d->env);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Handelt eine komplette Scrollbare Liste.
 * Gibt den Index des Exit-Buttons zurueck.
 * In lastElem wird die Adresse des zuletzt selektierten Elements zurueckgegeben.
 * Bei multiple == TRUE koennen mehrere Eintraege selektiert werden.
 */
GLOBAL _BOOL ListDl_WinListDial(
	OBJECT *tree,
	ldElemSet elems,
	ldHandler *handler,
	_WORD boxIndex,
	_WORD boxBack,
	_WORD okBut,
	_WORD cancelBut,
	_VOID *env,
	_WORD doubleClickButton,
	_WORD autoEdit,
	inLoopProc inLoop,
	specialButHdler *specials,
	_WORD specialCount,
	lsCheckExitProc checkExit,
	wdSetProc setValues,
	wdSetProc getValues,
	_VOID **lastElem)
{
	DIAL d;
	GRECT upperFrame;

	if (!ListDl_PrepareListDial(tree,
		elems,
		handler,
		boxIndex,
		boxBack,
		okBut,
		cancelBut,
		env,
		doubleClickButton,
		&d))
	{
		*lastElem = NULL;
		return FALSE;
	}
	d->listSetValues = setValues;
	d->listGetValues = getValues;
	d->listCheckExit = checkExit;
	d->lastElemPtr = lastElem;
	d->listSpecials = specials;
	d->listSpecialCount = specialCount;
	d->autoEdit = autoEdit;
	d->inLoop   = inLoop;
	d->initial  = TRUE;

	if ((d->flags & ldClearEdit) && d->autoEdit >= 0)
	{
		_UBYTE str[256];
		_UBYTE search[256];
		_VOID *ptx;

		/* String aus Editfeld holen */
		ObjcString(tree, d->autoEdit, search, sizeof(search));
		strUpr(search);
		strleft(search);
		ptx = NULL;
		if (*search != '\0')
		{
			d->hdl.resetList(d->hdl.list);
			while ((ptx = d->hdl.nextEntry(d->hdl.list)) != NULL)
			{
				d->hdl.entryToStr(ptx, d->env, str, sizeof(str));
				strUpr(str);
				strleft(str);
				if (d->hdl.isEnabled(ptx, d->env) && strCmp(str, search) == 0)
					break;
			}
		}
		if (ptx == NULL)
			SetObjcString(tree, d->autoEdit, "");
	}
	
	if (!WinDials_OpenWinDial(tree,
		(elems & ldModal) != 0,
		listSetGetValues,
		listSetGetValues,
		listButton,
		listDraw,
		NULL,
		d))
	{
		ListDl_ReleaseListDial(&d);
		*lastElem = NULL;
		return FALSE;
	}
	WinDials_SetMessage(tree,listMessage);
	if (elems & ldAutolocate)
		mtDials_InstallHandler(tree, ListDl_AutoLocate, CallByHandling, 0, 0, &upperFrame, NULL, NULL);

	if (elems & ldModal)
		WinDials_WinDialHandleEvents();

	return TRUE;
}
