/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#define __LISTMAN_IMPLEMENTATION__

#include <listman.h>
#include <portvdi.h>
#include <mtutils.h>
#include <mtappl.h>
#include <mtdials.h>
#include <mtxobjec.h>
#include <dialslid.h>
#include <listutil.h>
#include <wdwmanag.h>
#include <rectfunc.h>
#include <ro_mem.h>
#include <w_graf.h>

/* Handelt scrollbare Listen in Fenstern und Dialogboxen */

struct _listInfoType {
	OBJECT *tree;  /* Objektbaum */
	_WORD boxindex;
	_WORD boxbgindex;
	/* Fensterhandle. Ohne Fenster muss es < 0 sein */
	HWND wdw;
	/* Rechteck der Box, in dem alle Eintraege liegen.
	 * Nur noetig, wenn ein wdw benutzt wird,
	 * ansonsten werden die Objektrechtecke benutzt
	 */
	GRECT area;

	/* Abstand der einzelnen Eintraege */
	_WORD hoffset;
	_WORD voffset;
	/* Groesse der einzelnen Eintraege */
	_WORD hsize;
	_WORD vsize;

	/* maximale Breite eines Eintrags. Ist dieser Wert 0,
	 * so wird kein horizontaler Slider benutzt
	 */
	_WORD maxWidth;

	/* Schrittweite beim horizontalen Scrollen */
	_WORD hstep;

	/* horizontale Position, falls hor. Slider */
	_WORD hpos;

	/* erster anzuzeigender Eintrag */
	_LONG startIndex;

	/* letzter selektierter Eintrag */
	_LONG lastIndex;

	/* Die Liste ... */
	_VOID *list;

	/* ... und die Verwaltungsprozeduren dafuer */
	ResetProc resetList;
	MoveProc nextEntry;
	MoveProc prevEntry;
	CountProc listElems;

	/* weitere Funktionen */
	entryIsSelectableProc isEnabled;
	selectEntryProc selEntry;
	isSelectedProc isSelected;
	drawEntryProc draw;
	dragDropProc dragDrop;

	/* Selektionsmodus */
	selectMode selMode;

	/* Das freie Environment */
	_VOID *env;

	/* Die Sachen hier koennen nicht extern gesetzt werden */
	_LONG listLength;  /* Anzahl der Eintraege in der Liste */
	_WORD lines;  /* Anzahl der darstellbaren Zeilen */

	/* Die Sliderspecs */
	SLIDER vSlid;
	SLIDER hSlid;

	/* Und Zeiger auf den naechsten fuer Liste */
	LISTINFO next;
};



LOCAL LISTINFO infoList;


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Interne Prozeduren */

LOCAL _VOID *findListEntry(LISTINFO d, _LONG index)
{
	_VOID *ptx;
	_LONG count;
	_WORD w;

	ptx = NULL;
	if (index < 0)
		return NULL;
	d->listElems(d->list, &count, &w);
	if (count > 0)
	{
		d->resetList(d->list);
		while ((ptx = d->nextEntry(d->list)) != NULL && index > 0)
			--index;
	}
	return ptx;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _LONG findUp(LISTINFO d, _LONG index)
{
	_VOID *pt;
	_LONG idx;
	_BOOL useLI;

	idx = index;
	if (index >= d->listLength)
	{
		index = LMin(d->startIndex + d->lines - 1, d->listLength - 1);
	}
	if (index < 0)
	{
		index = 0;
	}
	useLI = d->lastIndex >= d->startIndex && d->lastIndex < d->startIndex + d->lines;
	/* Hier noch Eintrag suchen, der auch angewaehlt werden kann! */
	for (;;)
	{
		pt = findListEntry(d, index);
		if (pt != NULL && d->isEnabled(pt, d->env))
			break;
		--index;
		if (pt == NULL)
		{
			index = idx;
			for (;;)
			{
				pt = findListEntry(d, index);
				if (pt != NULL && d->isEnabled(pt, d->env))
					break;
				++index;
				if (pt == NULL)
				{
					index = -1;
					break;
				}
			}
			break;
		}
	}
	if (useLI && d->lastIndex >= 0 && index > d->lastIndex)
		return d->lastIndex;
	return index;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _LONG findDown(LISTINFO d, _LONG index)
{
	_VOID *pt;
	_LONG idx;
	_BOOL useLI;

	idx = index;
	if (index < 0 || index >= d->listLength)
		index = d->startIndex;
	useLI = d->lastIndex >= d->startIndex && d->lastIndex < d->startIndex + d->lines;
	/* Hier noch Eintrag suchen, der auch angewaehlt werden kann! */
	for (;;)
	{
		pt = findListEntry(d, index);
		if (pt != NULL && d->isEnabled(pt, d->env))
			break;
		++index;
		if (pt == NULL)
		{
			index = idx;
			for (;;)
			{
				pt = findListEntry(d, index);
				if (pt != NULL && d->isEnabled(pt, d->env))
					break;
				--index;
				if (pt == NULL)
				{
					index = -1;
					break;
				}
			}
			break;
		}
	}
	if (useLI && d->lastIndex >= 0 && index < d->lastIndex)
		return d->lastIndex;
	return index;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID positionList(LISTINFO d)
{
	/* Koordinaten und Groesse des vertikalen Sliders */
	_WORD vslidx;
	_WORD vslidy;
	_WORD vslidlen;
	/* Koordinaten und Groesse des horizontalen Sliders */
	_WORD hslidx;
	_WORD hslidy;
	_WORD hslidlen;

	if (d->tree != NULL)
	{
		CalcArea(d->tree, d->boxindex, &d->area);
	}
	/* Position berechnen */
	vslidx = d->area.g_x + d->area.g_w - 1;
	vslidy = d->area.g_y;
	vslidlen = d->area.g_h;
	DialSlider_SliderExtents(d->vSlid, vslidx, vslidy, vslidlen, FALSE);
	hslidx = d->area.g_x;
	hslidy = d->area.g_y + d->area.g_h - 1;
	hslidlen = d->area.g_w;
	DialSlider_SliderExtents(d->hSlid, hslidx, hslidy, hslidlen, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/* Berechnet den Platz eines Eintrages
 */
LOCAL _VOID calcEntryPos(LISTINFO d, _WORD pos, GRECT *r)
{
	if (d->tree != NULL)
		ObjcArea(d->tree, d->boxindex, r);
	else
		*r = d->area;
	/* Jetzt die Position berechnen */
	r->g_x = r->g_x - d->hpos;
	r->g_y = r->g_y + pos * (d->vsize + d->voffset);
	r->g_h = d->vsize + d->voffset;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID showEntry(LISTINFO d, _LONG index, _WORD pos, CONST GRECT *clip)
{
	GRECT r, r2;
	_VOID *ptx;

	ObjcArea(d->tree, d->boxindex, &r2);
	if (rc_intersect(clip, &r2))
	{
		ptx = findListEntry(d, index);
		calcEntryPos(d, pos, &r);
		if (d->tree != NULL)
			d->draw(ptx, d->env, r.g_x, r.g_y, d->hpos, &r2);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL showAll(LISTINFO d, CONST GRECT *clip)
{
	_WORD j;
	_VOID *ptx;
	GRECT r, r2;

	if (clip->g_w <= 0 || clip->g_h <= 0)
		return FALSE;
	if (d->wdw >= 0)
	{
		if (WdwManager_WindowIsIcon(d->wdw))
			return FALSE;
	}
	ObjcArea(d->tree, d->boxindex, &r2);
	if (rc_intersect(clip, &r2))
	{
		if (d->startIndex < 0)
			d->startIndex = 0;
		ptx = findListEntry(d, d->startIndex);
		for (j = 0; j < d->lines; j++)
		{
			calcEntryPos(d, j, &r);
			d->draw(ptx, d->env, r.g_x, r.g_y, d->hpos, &r2);
			if (ptx != NULL)
				ptx = d->nextEntry(d->list);
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID scrollV(LISTINFO d, _LONG newStart)
{
	_LONG j;
	GRECT redraw;
	_LONG diff;
	_LONG oldStart;
	GRECT rect, r1, r2;
	GRECT upperFrame;
	GRECT lowerFrame;
	_WORD pxy[8];
	MFDB screen;

	if (newStart < 0)
		newStart = 0;
	oldStart = d->startIndex;
	if (newStart == d->startIndex)
		return;
	d->startIndex = newStart;

	ObjcArea(d->tree, d->boxindex, &redraw);
	diff = oldStart - newStart;
	if (diff < 0)
		diff = -diff;
	if (diff < d->lines)
	{
		/* calculate upper & lower frame for BitBlk */
		ObjcArea(d->tree, d->boxindex, &rect);
		r1 = rect;
		r1.g_h = d->vsize + d->voffset;
		r2 = r1;
		r2.g_y = r2.g_y + (d->lines - 1 - (_WORD)diff) * (d->vsize + d->voffset);
		upperFrame = ListUtils_FrameRects(&r1, &r2);

		r1 = rect;
		r1.g_y = r1.g_y + (_WORD)diff * (d->vsize + d->voffset);
		r1.g_h = d->vsize + d->voffset;
		r2 = rect;
		r2.g_h = d->vsize + d->voffset;
		r2.g_y = r2.g_y + (d->lines - 1) * (d->vsize + d->voffset);

		lowerFrame = ListUtils_FrameRects(&r1, &r2);
		lowerFrame = rc_clip(&lowerFrame, &deskSize);
		upperFrame = rc_clip(&upperFrame, &deskSize);

		if (newStart < oldStart)
			ListUtils_WordArray(&upperFrame, &lowerFrame, pxy);
		else
			ListUtils_WordArray(&lowerFrame, &upperFrame, pxy);
		MouseOff();

		screen.fd_addr = 0;
		vro_cpyfm(PrivateWS, S_ONLY, pxy, &screen, &screen);

		if (newStart < oldStart)
		{
			/* von oben neu Zeichnen */
			for (j = 0; j < diff; j++)
				showEntry(d, d->startIndex + j, (_WORD)j, &redraw);
		} else
		{
			/* von unten neu zeichen */
			for (j = diff; j > 0; j--)
				showEntry(d, d->startIndex + d->lines - j, d->lines - (_WORD)j, &redraw);
		}
		MouseOn();
	} else
	{
		showAll(d, &redraw);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID scrollH(LISTINFO d, _WORD newPos)
{
	GRECT redraw;
	_WORD diff;
	_WORD oldStart;
	GRECT r1;
	GRECT leftFrame;
	GRECT rightFrame;
	_WORD pxy[8];
	MFDB screen;

	if (newPos < 0)
		newPos = 0;
	oldStart = d->hpos;
	if (newPos == d->hpos)
		return;
	d->hpos = newPos;

	ObjcArea(d->tree, d->boxindex, &redraw);
	redraw = rc_clip(&redraw, &deskSize);
	diff = oldStart - d->hpos;
	if (diff < 0)
		diff = -diff;
	if (diff < redraw.g_w)
	{
		/* calculate left & right frame for BitBlk */
		leftFrame = redraw;
		leftFrame.g_w -= diff;
		rightFrame = leftFrame;
		rightFrame.g_x += diff;

		CalcArea(d->tree, d->boxindex, &r1);
		r1 =  rc_clip(&r1, &deskSize);
		if (newPos < oldStart)
		{
			ListUtils_WordArray(&leftFrame, &rightFrame, pxy);
		} else
		{
			ListUtils_WordArray(&rightFrame, &leftFrame, pxy);
			r1.g_x = r1.g_x + r1.g_w - 1 - diff;
		}
		r1.g_w = diff + 1;
		MouseOff();

		screen.fd_addr = 0;
		vro_cpyfm(PrivateWS, S_ONLY, pxy, &screen, &screen);

		/* Jetzt Teilbereich neu zeichnen */
		showAll(d, &r1);
		MouseOn();
	} else
	{
		showAll(d, &redraw);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CenterEntry(LISTINFO d, _LONG entry, _BOOL drawIt)
{
	_LONG newStart, halfSize;

	positionList(d);
	if (entry < 0)
		return;
	if (entry < d->startIndex || entry >= d->startIndex + d->lines)
	{
		halfSize = d->lines / 2;
		if (entry >= halfSize)
			newStart = entry - halfSize;
		else
			newStart = entry;
		if (newStart >= d->listLength - d->lines)
			newStart = d->listLength - d->lines;
		if (newStart < 0)
			newStart = 0;
		if (drawIt)
			scrollV(d, newStart);
		d->startIndex = newStart;
		if (drawIt)
			DialSlider_SetSliderPos(d->vSlid, d->startIndex, TRUE);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL ListMan_InitList(
	_VOID *list,
	_VOID *env,
	selectEntryProc selectEntry,
	isSelectedProc isSelectedEntry,
	entryIsSelectableProc isSelectableEntry,
	drawEntryProc drawEntry,
	dragDropProc dragDrop,
	ResetProc resetList,
	MoveProc nextList,
	MoveProc prevList,
	CountProc countList,
	selectMode select,
	LISTINFO *listInfo)
{
	LISTINFO d;

	*listInfo = NEW(listInfoType, "ListMan_InitList");
	if (*listInfo == NULL)
		return FALSE;
	d = *listInfo;
	MemSetZeroStruct(*d);

	d->list       = list;
	d->env        = env;
	d->selEntry   = selectEntry;
	d->isEnabled  = isSelectableEntry;
	d->isSelected = isSelectedEntry;
	d->draw       = drawEntry;
	d->dragDrop   = dragDrop;
	
	d->resetList  = resetList;
	d->nextEntry  = nextList;
	d->prevEntry  = prevList;
	d->listElems  = countList;
	d->selMode    = select;

	d->tree       = NULL;
	d->wdw        = -1;

	/* Jetzt noch ein paar Sachen initialisieren */
	d->vSlid      = NULL;
	d->hSlid      = NULL;

	d->lastIndex = -1;
	d->startIndex = 0;
	d->hoffset    = 0;
	d->hpos       = 0;

	/* Jetzt in Liste haengen */
	d->next = infoList;
	infoList = d;

	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet die Liste neu und auch die Slider, Clipping wird
 * beachtet
 */
LOCAL _VOID internalDrawList(LISTINFO d, CONST GRECT *clip, _BOOL showList, _BOOL posSlider, _BOOL showSlider)
{
	GRECT cr;
	_LONG ll;
	_WORD ww;

	if (d->tree != NULL)
	{
		CalcArea(d->tree, d->boxindex, &d->area);
		ObjcArea(d->tree, d->boxindex, &cr);
	} else
	{
		cr = d->area;
	}
	if (posSlider)
	{
		d->listElems(d->list, &ll, &ww);
		if (d->listLength != ll)
		{
			d->listLength = ll;
			DialSlider_SetSliderScale(d->vSlid, d->listLength);
			/* Jetzt noch evtl. lastIndex updaten */
			if (d->startIndex + d->lines > d->listLength)
				d->startIndex = d->listLength - d->lines;
			d->lastIndex = ListMan_ListFirstSelected(d);
			/*
			if (d->selMode == smOne)
				ListMan_ListScroll2Selection(d, TRUE);
			*/
		}
		if (d->maxWidth > 0 && ww > 0 && d->maxWidth != ww)
		{
			d->maxWidth = ww;
			DialSlider_SetSliderScale(d->hSlid, d->maxWidth);
			d->hpos = 0;
		}
	}
	if (showList)
	{
		if (!rc_intersect(clip, &cr) ||
			!showAll(d, &cr))
			return;
	}
	positionList(d);
	if (showSlider)
	{
		DialSlider_SetSliderPos(d->vSlid, d->startIndex, FALSE);
		DialSlider_DrawSlider(d->vSlid, clip, TRUE);
		DialSlider_DrawSlider(d->hSlid, clip, TRUE);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Von den beiden folgenden Prozeduren muss mindestens eine nach
 * der Initialisierung der Liste aufgerufen werden
 */


/* Setzt weitere Informationen fuer die Liste.
 * wdwHandle: Das Fenster, in dem die Liste dargestellt wird
 * area: Arbeitsbereich, in dem die Liste dargestellt wird
 * voffset: Abstand zwischen zwei Eintraegen
 * vsize  : Hoehe eines Eintrages
 * maxwidth: maximale Breite eines Eintrags
 */
GLOBAL _BOOL ListMan_SetListWdwSpecs(
	LISTINFO d,
	HWND wdwHandle,
	CONST GRECT *area,
	_WORD voffset,
	_WORD vsize,
	_WORD hstep,
	_WORD maxwidth)
{
	UNUSED(area);
	d->wdw         = wdwHandle;
	d->voffset     = voffset;
	d->vsize       = vsize;
	d->hstep       = hstep;
	d->maxWidth    = maxwidth;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt weitere Informationen fuer die Liste.
 * tree: Der Baum, in dem die Liste dargestellt wird
 * entryBox: Objektindex fuer das Objekt, in dem alle Objekte
 *           dargestellt werden
 * boxBack: Objektindex des Hintergrundes der Box (fuer Slider)
 * voffset: Abstand zwischen zwei Eintraegen
 * vsize  : Hoehe eines Eintrages
 * maxwidth: maximale Breite eines Eintrags (wenn == 0, dann wird kein
 * horizontaler Slider eingesetzt)
 */

GLOBAL _BOOL ListMan_SetListDialSpecs(
	LISTINFO d,
	OBJECT *tree,
	_WORD entryBox,
	_WORD boxBack,
	_WORD voffset,
	_WORD vsize,
	_WORD hstep,
	_WORD maxwidth)
{
	GRECT backrect;
	GRECT cliparea;
	_WORD ww;

	d->tree = tree;
	d->boxindex    = entryBox;
	d->boxbgindex  = boxBack;
	d->voffset     = voffset;
	d->vsize       = vsize;
	d->hstep       = hstep;
	d->maxWidth    = maxwidth;
	/* Ok, und  jetzt koennen wir die Slider initialisieren */
	/* Das ist der Workbereich */
	CalcArea(d->tree, d->boxindex, &d->area);	
	/* Das brauchen wir fuer die Slider */
	ObjcRect(d->tree, d->boxbgindex, &backrect);
	/* Anpassen an Rand der Slider */
	++backrect.g_w;
	if (maxwidth > 0)
		++backrect.g_h;
	SetObjcRect(d->tree, d->boxbgindex, &backrect);
	/* Anzahl Zeilen berechnen */
	d->lines = d->area.g_h / (d->vsize + d->voffset);
	/* area clippen auf Vielfaches der Hoehe */
	ObjcRect(d->tree, d->boxindex, &cliparea);
	cliparea.g_h = d->lines * (d->vsize + d->voffset);
	SetObjcRect(d->tree, d->boxindex, &cliparea);
	/* Jetzt den vertikalen Slider plazieren */
	d->listElems(d->list, &d->listLength, &ww);
	if (!DialSlider_CreateSlider(&d->vSlid, FALSE, 1, d->lines, d->listLength, d->startIndex))
		return FALSE;
	if (d->maxWidth > 0)
	{
		/* auch horizontalen Slider erzeugen */
		if (!DialSlider_CreateSlider(&d->hSlid, TRUE, d->hstep, d->area.g_w, d->maxWidth, d->hpos))
			return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* gibt einen Listenhandler wieder frei. */
GLOBAL _BOOL ListMan_FreeList(LISTINFO listInfo)
{
	LISTINFO d2;
	GRECT backrect;

	if (listInfo == NULL)
		return FALSE;
	d2 = infoList;
	if (d2 == listInfo)
	{
		infoList = listInfo->next;
	} else
	{
		while (d2 != NULL && d2->next != listInfo)
			d2 = d2->next;
		if (d2 == NULL)
			return FALSE;
		/* listInfo aushaengen */
		d2->next = listInfo->next;
	}
	/* Jetzt Slider freigeben */
	DialSlider_FreeSlider(&listInfo->vSlid);
	DialSlider_FreeSlider(&listInfo->hSlid);
	if (listInfo->tree != NULL)
	{
		/* Userdefs freigeben */
		FreeUserdef(listInfo->tree, listInfo->boxbgindex);
		FreeUserdef(listInfo->tree, listInfo->boxindex);
		ObjcRect(listInfo->tree, listInfo->boxbgindex, &backrect);
		/* Anpassen an Rand der Slider */
		--backrect.g_w;
		if (listInfo->maxWidth > 0)
			--backrect.g_h;
		SetObjcRect(listInfo->tree, listInfo->boxbgindex, &backrect);
	}
	OFREE(listInfo);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet die Liste neu und auch die Slider, Clipping wird
 * beachtet
 */
GLOBAL _VOID ListMan_DrawList(LISTINFO listInfo, CONST GRECT *clip)
{
	internalDrawList(listInfo, clip, TRUE, TRUE, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Setzt die D&D-Prozedur (MH) 
 */
GLOBAL _VOID ListMan_SetDragDropProc(OBJECT *tree, dragDropProc dragDrop)
{
	LISTINFO d;

	d = infoList;
	while (d != NULL && d->tree != tree)
		d = d->next;
	if (d != NULL)
		d->dragDrop = dragDrop;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID deselectEntry(LISTINFO d, _LONG index, _BOOL show)
{
	_VOID *entry;
	_WORD line;

	if (d->selMode == smOne)
	{
		/* einfach nur deselektieren */
		entry = findListEntry(d, index);
		/* Eintrag ist selektierbar */
		if (entry != NULL && d->isEnabled(entry, d->env) && d->isSelected(entry, d->env))
		{
			/* Jetzt Eintrag (de-)selektieren */
			if (index >= d->startIndex && index < d->startIndex + d->lines && show)
				line = (_WORD)(index - d->startIndex);
			else
				line = -1;
			d->selEntry(entry, d->env, line);
			if (index >= d->startIndex && index < d->startIndex + d->lines)
				showEntry(d, index, (_WORD)(index - d->startIndex), &d->area);
			d->lastIndex = -1;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID selectEntry(LISTINFO d, _LONG index, _BOOL *isSel, _BOOL show, _BOOL byClick, _WORD clicks)
{
	_VOID *entry, *lastEntry;
	_WORD line;

	*isSel = FALSE;
	if (d->selMode == smNone)
	{
		return;
	} else if (d->selMode == smOne)
	{
		if (index != d->lastIndex || byClick)
		{
			/* Ok, wirklich neuer Eintrag */
			/* eventuell selektierten Eintrag deselektieren */
			if (index != d->lastIndex)
			{
				lastEntry = findListEntry(d, d->lastIndex);
				if (lastEntry != NULL && d->isSelected(lastEntry, d->env))
				{
					d->selEntry(lastEntry, d->env, -1);
					/* Jetzt eventuell neu zeichnen */
					if (d->lastIndex >= d->startIndex && d->lastIndex < d->startIndex + d->lines)
						showEntry(d, d->lastIndex, (_WORD)(d->lastIndex - d->startIndex), &d->area);
				}
			}
			entry = findListEntry(d, index);
			/* Eintrag ist selektierbar */
			*isSel = (entry != NULL) && d->isEnabled(entry, d->env);
			if (entry != NULL && *isSel)
			{
				/* Jetzt neuen Eintrag selektieren */
				if (index >= d->startIndex && index < d->startIndex + d->lines && show)
					line = (_WORD)(index - d->startIndex);
				else
					line = -1;
				if (entry != NULL && d->isSelected(entry, d->env) && byClick && clicks > 1)
				{
				} else
				{
					d->selEntry(entry, d->env, line);
				}
				if (index >= d->startIndex && index < d->startIndex + d->lines)
					showEntry(d, index, (_WORD)(index - d->startIndex), &d->area);
				d->lastIndex = index;
			}
		}
	} else if (d->selMode == smMulti)
	{
		entry = findListEntry(d, index);
		*isSel = d->isEnabled(entry, d->env);
		if (entry != NULL && *isSel)
		{
			/* Neuen Eintrag selektieren */
			if (index >= d->startIndex && index < d->startIndex + d->lines && show)
				line = (_WORD)(index - d->startIndex);
			else
				line = -1;
			if (clicks < 2)
				d->selEntry(entry, d->env, line);
			if (index >= d->startIndex && index < d->startIndex + d->lines)
				showEntry(d, index, (_WORD)(index - d->startIndex), &d->area);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID slidVertical(SLIDER slid, _LONG newPos, _VOID *env)
{
	UNUSED(slid);
	scrollV(env, newPos);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID slidHorizontal(SLIDER slid, _LONG newPos, _VOID *env)
{
	UNUSED(slid);
	scrollH(env, (_WORD)newPos);
}

/*** ---------------------------------------------------------------------- ***/

/* Wird aufgerufen, wenn ein Mausklick stattfand. Gibt zurueck,
 * ob ein Eintrag angeklickt wurde (>= 0) oder ob
 * der Click ausserhalb des Bereiches war (-1)
 * oder ob der Click behandelt wurde (-2)
 * Wenn ein Eintrag angeklickt wurde, dann wird dieser auch selektiert,
 * falls notwendig.
 */
GLOBAL _LONG ListMan_ListClick(LISTINFO d, _WORD mx, _WORD my, _UWORD kstate, _WORD clicks)
{
	_LONG saveIndex, oldIndex;
	_LONG i;
	_UWORD button;
	_LONG elem;
	_LONG selIndex = 0, index;
	_BOOL isSel;
	GRECT br;
	GRECT DDFrame;
	_WORD x, y;
	_UWORD dummy;
	HWND hdl;
	_VOID *entry;

	positionList(d);
	/* Erstmal nachsehen, ob auf einen Slider geclickt wurde */
	elem = DialSlider_SliderClick(d->vSlid, mx, my, clicks, slidVertical, d);
	if (elem >= 0)
		return -2;
	elem = DialSlider_SliderClick(d->hSlid, mx, my, clicks, slidHorizontal, d);
	if (elem >= 0)
		return -2;
	/* Nicht auf einen der Slider, nachsehen, ob es im Workbereich ist */
	if (rc_inside(mx, my, &d->area))
	{
		/* Ist im Workbereich, behandeln */
		index = d->startIndex + (my - d->area.g_y) / (d->vsize + d->voffset);
		if (clicks > 0 && index < d->listLength)
		{
			/* Element getroffen, selektieren. Falls Maustaste gedrueckt bleibt,
			 * dann Bereich selektieren (je nach SelectMode)
			 */
			selectEntry(d, index, &isSel, TRUE, TRUE, clicks);
			if (!isSel)
				index = -1;
			saveIndex = index;
			
			/* Soll eine D&D-Aktion durchgefuehrt werden? (MH) */
			if (isSel && d->dragDrop != FUNK_NULL)
			{
				calcEntryPos(d, (_WORD)(d->lastIndex - d->startIndex), &DDFrame);
				MouseHand();
				Graf_dragbox(&DDFrame, &deskSize, &x, &y);
				MouseArrow();
				graf_mkstate(&x, &y, &dummy, &dummy);
				hdl = wind_find(x, y);
				entry = findListEntry(d, index);
				d->dragDrop(entry, d->env, x, y, hdl);
			}

			/* Jetzt feststellen, ob Maustaste noch gedrueckt ist und
			 * dann in ScrollModus uebergehen, aber nur wenn multiple erlaubt
			 * ist.
			 */
			Wind_Update(BEG_MCTRL);
			graf_mkstate(&mx, &my, &button, &kstate);
			/* Jetzt Baserechteck berechnen und Object unter Mauskoordinaten
			 * suchen
			 */
			oldIndex = index;
			while (button & MOB_LEFT)
			{
				index = d->startIndex + (my - d->area.g_y) / (d->vsize + d->voffset);
				if (index < d->startIndex || my < d->area.g_y)
				{
					/* scrollen */
					ListMan_ListLnUp(d);
				} else if (index >= d->startIndex + d->lines)
				{
					ListMan_ListLnDown(d);
				}
				if (d->selMode == smMulti)
				{
					if ((index <= saveIndex && index > oldIndex) ||
						(index >= saveIndex  && index < oldIndex))
					{
						/* vorherige togglen */
						if (index > oldIndex)
						{
							for (i = oldIndex; i < index; i++)
								selectEntry(d, i, &isSel, TRUE, FALSE, 0);
						} else
						{
							for (i = oldIndex; i > index; i--)
								selectEntry(d, i, &isSel, TRUE, FALSE, 0);
						}
						oldIndex = index;
					} else if ((index < saveIndex && index < oldIndex) ||
							(index > saveIndex && index > oldIndex))
					{
						if (index > oldIndex)
						{
							for (i = index; i > oldIndex; i--)
								selectEntry(d, i, &isSel, TRUE, FALSE, 0);
						} else
						{
							for (i = index; i < oldIndex; i++)
								selectEntry(d, i, &isSel, TRUE, FALSE, 0);
						}
						oldIndex = index;
					}
				} else if (d->selMode == smOne)
				{
					if (index != oldIndex)
					{
						selectEntry(d, index, &isSel, TRUE, FALSE, 0);
						if (isSel)
							selIndex = index;
						else
							index = selIndex;
						oldIndex = index;
					}
				}
				/* Und jetzt die Tasten wieder abfragen */
				graf_mkstate(&mx, &my, &button, &kstate);
			}
			Wind_Update(END_MCTRL);
			return index;
		} else if (clicks == 0)
		{
			return index;
		}
	} else
	{
		/* Jetzt noch nachsehen, ob es im Hintergrundrechteck liegt und
		 * dann ggf. warten, bis die Maustaste losgelassen wird.
		 */
		if (d->tree != NULL)
		{
			CalcArea(d->tree, d->boxbgindex, &br);
			if (rc_inside(mx, my, &br))
			{
				/* Warten, bis die Maustaste losgelassen wird
				 */
				Bounce();
			}
		}
	}
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

/* Einen bestimmten Eintrag neu zeichnen lassen, falls noetig
 */
GLOBAL _VOID ListMan_ListUpdateEntry(LISTINFO listInfo, _LONG entry)
{
	_LONG line;

	/* Wir lassen den einfach neuzeichnen */
	line = entry - listInfo->startIndex;
	if (line >= 0 && line < listInfo->lines)
		showEntry(listInfo, entry, (_WORD)line, &listInfo->area);
}

/*** ---------------------------------------------------------------------- ***/

/* Der Eintrag entry soll (extern) selektiert werden, den Rest handhaben
 * Nuetzlich fuer Autolocator
 */
GLOBAL _VOID ListMan_ListInvertEntry(LISTINFO listInfo, _LONG entry)
{
	_BOOL isSel;
	/* Wir lassen den einfach neuzeichnen */
	positionList(listInfo);
	selectEntry(listInfo, entry, &isSel, FALSE, FALSE, 0);
	ListMan_ListScroll2Selection(listInfo, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

/* Deselektiert den Eintrag entry, falls dieser selektiert ist */
GLOBAL _VOID ListMan_ListDeselEntry(LISTINFO listInfo, _LONG entry)
{
	positionList(listInfo);
	deselectEntry(listInfo, entry, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/* Scrollt die Liste so, dass der erste selektierte Eintrag
 * sichtbar ist.
 */
GLOBAL _VOID ListMan_ListScroll2Selection(LISTINFO listInfo, _BOOL draw)
{
	listInfo->lastIndex = ListMan_ListFirstSelected(listInfo);
	/* Und jetzt zentrieren. Abfrage ist nicht noetig, da
	 * schon in CenterEntry drin
	 */
	CenterEntry(listInfo, listInfo->lastIndex, draw);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListPgDown(LISTINFO listInfo)
{
	_LONG oldStart;

	positionList(listInfo);
	oldStart = listInfo->startIndex;
	listInfo->startIndex += listInfo->lines;
	if (listInfo->startIndex + listInfo->lines >= listInfo->listLength)
		listInfo->startIndex = listInfo->listLength - listInfo->lines;
	if (listInfo->startIndex < 0)
		listInfo->startIndex = 0;
	if (oldStart != listInfo->startIndex)
	{
		DialSlider_SetSliderPos(listInfo->vSlid, listInfo->startIndex, TRUE);
		showAll(listInfo, &listInfo->area);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListPgUp(LISTINFO listInfo)
{
	_LONG oldStart;

	positionList(listInfo);
	oldStart = listInfo->startIndex;
	listInfo->startIndex -= listInfo->lines;
	if (listInfo->startIndex < 0)
		listInfo->startIndex = 0;
	if (oldStart != listInfo->startIndex)
	{
		DialSlider_SetSliderPos(listInfo->vSlid, listInfo->startIndex, TRUE);
		showAll(listInfo, &listInfo->area);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListLnDown(LISTINFO listInfo)
{
	positionList(listInfo);
	if (listInfo->startIndex + listInfo->lines < listInfo->listLength)
	{
		scrollV(listInfo, listInfo->startIndex + 1);
		DialSlider_SetSliderPos(listInfo->vSlid, listInfo->startIndex, TRUE);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListLnUp(LISTINFO listInfo)
{
	positionList(listInfo);
	if (listInfo->startIndex > 0)
	{
		scrollV(listInfo, listInfo->startIndex - 1);
		DialSlider_SetSliderPos(listInfo->vSlid, listInfo->startIndex, TRUE);
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID ListMan_ListLnMv(LISTINFO listInfo,const _WORD lines)			*/
/*	Scrollt die Liste um eine Anzahl Zeilen										*/
/*	Parameter:	listInfo:	die zu scrollende Liste								*/
/*				lines:	die Anzahl Zeilen mit Vorzeichen						*/
/*	Ergebnis:	keines															*/		
/*	24.10.00 von Dimitri Junker	DYJ 4.18zk										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID ListMan_ListLnMv(LISTINFO listInfo,const _WORD lines)
{
	_LONG oldStart;

	positionList(listInfo);
	oldStart = listInfo->startIndex;
	listInfo->startIndex += lines;
	if (listInfo->startIndex + listInfo->lines >= listInfo->listLength)
		listInfo->startIndex = listInfo->listLength - listInfo->lines;
	if (listInfo->startIndex < 0)
		listInfo->startIndex = 0;
	if (oldStart != listInfo->startIndex)
	{
		DialSlider_SetSliderPos(listInfo->vSlid, listInfo->startIndex, TRUE);
		showAll(listInfo, &listInfo->area);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListFirst(LISTINFO listInfo)
{
	_LONG oldStart;

	positionList(listInfo);
	oldStart = listInfo->startIndex;
	listInfo->startIndex = 0;
	if (oldStart != listInfo->startIndex)
	{
		DialSlider_SetSliderPos(listInfo->vSlid, listInfo->startIndex, TRUE);
		showAll(listInfo, &listInfo->area);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListLast(LISTINFO listInfo)
{
	_LONG oldStart;

	positionList(listInfo);
	oldStart = listInfo->startIndex;
	listInfo->startIndex = listInfo->listLength - listInfo->lines;
	if (listInfo->startIndex < 0)
		listInfo->startIndex = 0;
	if (oldStart != listInfo->startIndex)
	{
		DialSlider_SetSliderPos(listInfo->vSlid, listInfo->startIndex, TRUE);
		showAll(listInfo, &listInfo->area);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListPgLeft(LISTINFO listInfo)
{
	_WORD newOffset;

	positionList(listInfo);
	if (listInfo->hpos >= listInfo->area.g_w)
		newOffset = listInfo->hpos - listInfo->area.g_w;
	else
		newOffset = 0;
	scrollH(listInfo, newOffset);
	DialSlider_SetSliderPos(listInfo->hSlid, listInfo->hpos, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListPgRight(LISTINFO listInfo)
{
	_WORD newOffset;

	positionList(listInfo);
	if (listInfo->hpos < listInfo->maxWidth - listInfo->area.g_w)
		newOffset = listInfo->hpos + listInfo->area.g_w;
	else
		newOffset = listInfo->maxWidth - listInfo->area.g_w;
	scrollH(listInfo, newOffset);
	DialSlider_SetSliderPos(listInfo->hSlid, listInfo->hpos, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListLnRight(LISTINFO listInfo)
{
	positionList(listInfo);
	scrollH(listInfo, listInfo->hpos - listInfo->hstep);
	DialSlider_SetSliderPos(listInfo->hSlid, listInfo->hpos, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_ListLnLeft(LISTINFO listInfo)
{
	positionList(listInfo);
	scrollH(listInfo, listInfo->hpos + listInfo->hstep);
	DialSlider_SetSliderPos(listInfo->hSlid, listInfo->hpos, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _LONG doSelect(LISTINFO listInfo, _LONG newIdx)
{
	_BOOL isSel;

	positionList(listInfo);
	DialSlider_SetSliderPos(listInfo->vSlid, listInfo->startIndex, TRUE);
	CenterEntry(listInfo, newIdx, TRUE);
	if (/* newIdx >= 0 && */ newIdx != listInfo->lastIndex)
	{
		selectEntry(listInfo, newIdx, &isSel, TRUE, FALSE, 0);
	}
	ListMan_ListScroll2Selection(listInfo, TRUE);	/*MH 4.14l */
	listInfo->lastIndex = newIdx;
	return newIdx;
}

/*** ---------------------------------------------------------------------- ***/

/* Selektion innerhalb der Liste verschieben */
GLOBAL _LONG ListMan_ListSelUp(LISTINFO listInfo)
{
	_LONG newIdx;

	positionList(listInfo);
	if (listInfo->lastIndex < listInfo->startIndex ||
		listInfo->lastIndex >= listInfo->startIndex + listInfo->lines)
	{
		if (listInfo->lines > 0)                /* MH 4.14l */
			newIdx = listInfo->listLength - 1;
		else
			newIdx = -1;
	} else if (listInfo->lastIndex > 0)
	{
		newIdx = listInfo->lastIndex - 1;
	} else
	{
		newIdx = listInfo->startIndex + listInfo->lines - 1;
	}
	newIdx = findUp(listInfo, newIdx);
	return doSelect(listInfo, newIdx);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG ListMan_ListSelPgUp(LISTINFO listInfo)
{
	_LONG newIdx;

	positionList(listInfo);
	if (listInfo->lastIndex < listInfo->startIndex ||
		listInfo->lastIndex >= listInfo->startIndex + listInfo->lines)
	{
		ListMan_ListPgUp(listInfo);
		newIdx = -1;
	} else if (listInfo->lastIndex >= listInfo->lines)
	{
		newIdx = listInfo->lastIndex - listInfo->lines;
	} else if (listInfo->lastIndex >= 0)
	{
		newIdx = 0;
	} else
	{
		ListMan_ListPgUp(listInfo);
		newIdx = -1;
	}
	newIdx = findUp(listInfo, newIdx);
	CenterEntry(listInfo, newIdx, TRUE);
	return doSelect(listInfo, newIdx);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG ListMan_ListSelDown(LISTINFO listInfo)
{
	_LONG newIdx;

	if ((listInfo->lastIndex + 1) < listInfo->listLength)
	{
		if (listInfo->lastIndex < listInfo->startIndex ||
			listInfo->lastIndex >= listInfo->startIndex + listInfo->lines)
		{
			newIdx = -1;
		} else if (listInfo->lastIndex >= 0)
		{
			newIdx = listInfo->lastIndex + 1;
		} else
		{
			newIdx = listInfo->startIndex;
		}
		newIdx = findDown(listInfo, newIdx);
		return doSelect(listInfo, newIdx);
	}
	return listInfo->lastIndex;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG ListMan_ListSelPgDown(LISTINFO listInfo)
{
	_LONG newIdx;

	positionList(listInfo);
	if ((listInfo->lastIndex + 1) < listInfo->listLength)
	{
		if (listInfo->lastIndex < listInfo->startIndex ||
			listInfo->lastIndex >= listInfo->startIndex + listInfo->lines)
		{
			ListMan_ListPgDown(listInfo);
			newIdx = listInfo->startIndex;
		} else if (listInfo->lastIndex >= 0 &&
			listInfo->lastIndex + listInfo->lines < listInfo->listLength)
		{
			newIdx = listInfo->lastIndex + listInfo->lines;
		} else if (listInfo->lastIndex >= 0)
		{
			newIdx = listInfo->listLength - 1;
		} else
		{
			ListMan_ListPgDown(listInfo);
			newIdx = listInfo->startIndex;
		}
		newIdx = findDown(listInfo, newIdx);
		CenterEntry(listInfo, newIdx, TRUE);
		return doSelect(listInfo, newIdx);
	}
	return listInfo->lastIndex;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG ListMan_ListSelFirst(LISTINFO listInfo)
{
	positionList(listInfo);
	scrollV(listInfo, 0);
	return doSelect(listInfo, findUp(listInfo, 0));
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG ListMan_ListSelLast(LISTINFO listInfo)
{
	positionList(listInfo);
	scrollV(listInfo, listInfo->listLength - listInfo->lines);
	return doSelect(listInfo, findDown(listInfo, listInfo->listLength - 1));
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG ListMan_ListListElems(LISTINFO listInfo)
{
	_LONG ll;
	_WORD ww;

	listInfo->listElems(listInfo->list, &ll, &ww);
	listInfo->listLength = ll;
	return ll;
}

/*** ---------------------------------------------------------------------- ***/

/* Gbt den ersten selektieren Eintrag zurueck, -1 im Fehlerfall
 */
GLOBAL _LONG ListMan_ListFirstSelected(LISTINFO listInfo)
{
	_VOID *pt;
	_LONG ll;
	_WORD ww;
	_LONG i;

	listInfo->lastIndex = -1;
	listInfo->listElems(listInfo->list, &ll, &ww);
	listInfo->listLength = ll;
	listInfo->resetList(listInfo->list);
	pt = listInfo->nextEntry(listInfo->list);
	i = 0;
	while (pt != NULL)
	{
		if (listInfo->isSelected(pt, listInfo->env))
		{
			listInfo->lastIndex = i;
			break;
		}
		pt = listInfo->nextEntry(listInfo->list);
		i++;
	}
	return listInfo->lastIndex;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID *ListMan_ListFindEntry(LISTINFO listInfo, _LONG index)
{
	return findListEntry(listInfo, index);
}

/*** ---------------------------------------------------------------------- ***/

/* Gibt den ersten angezeigten Eintrag zurueck, -1 im Fehlerfall
 */
GLOBAL _LONG ListMan_ListFirstEntry(LISTINFO listInfo)
{
	return listInfo->startIndex;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListMan_SetNewList(LISTINFO listInfo, _VOID *list, _BOOL draw)
{
	GRECT redraw;

	listInfo->list = list;
	if (draw)
	{
		listInfo->lastIndex = ListMan_ListFirstSelected(listInfo);
		listInfo->startIndex = 0;
		positionList(listInfo);
		ObjcArea(listInfo->tree, listInfo->boxbgindex, &redraw);
		listInfo->listLength = 0;
		ListMan_DrawList(listInfo, &redraw);
		ListMan_ListScroll2Selection(listInfo, TRUE);
	}
}
