/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <listutil.h>
#include <mtappl.h>

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListUtils_WordArray(CONST GRECT *r1, CONST GRECT *r2, _WORD *arr)
{
	arr[0] = r1->g_x;
	arr[1] = r1->g_y;
	arr[2] = r1->g_x + r1->g_w - 1;
	arr[3] = r1->g_y + r1->g_h - 1;
	arr[4] = r2->g_x;
	arr[5] = r2->g_y;
	arr[6] = r2->g_x + r2->g_w - 1;
	arr[7] = r2->g_y + r2->g_h - 1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL GRECT ListUtils_FrameRects(CONST GRECT *r1, CONST GRECT *r2)
{
	GRECT r;
	_WORD x1, x2;

	if (r1->g_x > r2->g_x)
		r.g_x = r2->g_x;
	else
		r.g_x = r1->g_x;
	if (r1->g_y > r2->g_y)
		r.g_y = r2->g_y;
	else
		r.g_y = r1->g_y;
	x1 = r1->g_x + r1->g_w;
	x2 = r2->g_x + r2->g_w;
	if (x1 < x2)
		r.g_w = x2 - r.g_x;
	else
		r.g_w = x1 - r.g_x;
	x1 = r1->g_y + r1->g_h;
	x2 = r2->g_y + r2->g_h;
	if (x1 < x2)
		r.g_h = x2 - r.g_y;
	else
		r.g_h = x1 - r.g_y;
	return r;
}

/*** ---------------------------------------------------------------------- ***/

/* liefert das Rechteck aus r1, das nicht von r2 ueberdeckt wird
 */
GLOBAL GRECT ListUtils_DiffVertRect(CONST GRECT *r1, CONST GRECT *r2)
{
	GRECT r;

	r = *r1;
	/* Hoehe auf absolute Koordinate */
	r.g_h = r1->g_y + r1->g_h - 1;
	if (r2->g_y < r1->g_y && r2->g_y + r2->g_h >= r1->g_y)
		r.g_y = r2->g_y + r2->g_h - 1;
	if (r1->g_y < r2->g_y && r1->g_y + r1->g_h > r2->g_y)
		r.g_h = r2->g_y;
	/* Hoehe wieder auf relative Koordinate */
	r.g_h = r.g_h - r.g_y + 1;
	/* Jetzt das gleiche fuer die X-Richtung */
	r.g_w = r1->g_x + r1->g_w - 1;
	/* Breite auf absolute Koordinate */
	if (r2->g_x < r1->g_x && r2->g_x + r2->g_w >= r1->g_x)
		r.g_x = r2->g_x + r2->g_w - 1;
	if (r1->g_x < r2->g_x && r1->g_x + r1->g_w > r2->g_x)
		r.g_w = r2->g_x;
	/* Breite wieder auf relative Koordinate */
	r.g_w = r.g_w - r.g_x + 1;
	return r;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListUtils_CalcAbsArea(OBJECT *tree, _WORD obj, GRECT *r)
{
	_WORD border;

	border = ObjcFrame(tree, obj);
	ObjcArea(tree, obj, r);
	if (border < 0) /* Rahmen ausserhalb des Objekts! */
	{
		border = -border;
		r->g_x -= border;
		r->g_y -= border;
		r->g_w += border * 2;
		r->g_h += border * 2;
	}
	if (GetObjState(tree, obj, DISABLED))
	{
		r->g_w += border * 2;
		r->g_h += border * 2;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ListUtils_formSetSlider(OBJECT *dial, _WORD parent, _WORD slider, _BOOL vertical, _LONG maxVal, _LONG currentVal, _BOOL draw)
{
	GRECT pFrame;
	GRECT sFrame;
	GRECT dummy;
	_LONG Epp;

	ObjcRect(dial, parent, &pFrame);
	ObjcRect(dial, slider, &sFrame);
	ListUtils_CalcAbsArea(dial, slider, &dummy);

	if (vertical)
	{
		if (pFrame.g_h == sFrame.g_h)
			return;					/* volle Hoehe */
		/* Wert 1000*(Eintraege/Pixel) */
		Epp = (maxVal * 1000) / (_LONG)(pFrame.g_h - sFrame.g_h);
		sFrame.g_y = (_WORD)((currentVal * 1000L) / Epp);
		if (sFrame.g_y + sFrame.g_h >= pFrame.g_h)
			sFrame.g_y = pFrame.g_h - sFrame.g_h;
		if (sFrame.g_y < 0)
			sFrame.g_y = 0;
	} else
	{
		if (pFrame.g_w == sFrame.g_w)
			return;					/* volle Breite */
		Epp = ((maxVal * 1000) / (_LONG)(pFrame.g_w - sFrame.g_w));
		sFrame.g_x = (_WORD)((currentVal * 1000L) / Epp);
		if (sFrame.g_x + sFrame.g_w >= pFrame.g_w)
			sFrame.g_x = pFrame.g_w - sFrame.g_w;
		if (sFrame.g_x < 0)
			sFrame.g_x = 0;
	}

	SetObjcRect(dial, slider, &sFrame);
	ListUtils_CalcAbsArea(dial, slider, &sFrame);
	if (draw)
	{
		dummy = ListUtils_DiffVertRect(&dummy, &sFrame);
		Objc_draw(dial, parent, 0, &dummy);
		Objc_draw(dial, slider, 0, &sFrame);
	}
}

