#include <portaes.h>
#include <tos.h>
#include <mtdir.h>
#include <mtappl.h>
#include <mtdials.h>
#include <mttextfi.h>
#include <mttime.h>
#include <mtutils.h>
#include <strconv.h>
#include <mtrsc.h>

#include <ro_mem.h>
#include <routine.h>
#include <file_io.h>


#define COD_EXTENSION ".cod"

typedef enum { mm2, tdi, array, other } tCompiler;

/* Wandelt ein RSC-File in formatierten Text.
 * Fuer MM2 als initialisiertes Konstanten-Feld, fuer andere Compiler als
 * CODE- bzw. INLINE-Prozedur.  Ein solches RSC-File kann dann mittels
 * mtRsc.RelocRsc wie ein normales RSC-File geladen werden. So ist es
 * moeglich, Resourcen im Programmcode zu halten, ohne eigenes RSC-File
 * benutzen zu muessen.
 */

LOCAL _UWORD resource[0x8000];

LOCAL _UBYTE inFile[PATH_MAX];
LOCAL _UBYTE inPath[PATH_MAX];
LOCAL _UBYTE inName[PATH_MAX];

LOCAL _UBYTE outFile[PATH_MAX];
LOCAL _UBYTE outPath[PATH_MAX];
LOCAL _UBYTE outName[PATH_MAX];

LOCAL FILENO in;
LOCAL _ULONG inBytes;
LOCAL TEXTFILENO out;

LOCAL tCompiler compiler;

LOCAL GRECT rect, clip;
LOCAL OBJECT *maindial;
LOCAL RESOURCE rsc;
LOCAL _UBYTE str[256];


LOCAL _UBYTE RscData[] = {
#include <rsccoder.inc>
};
#include <rsccoder.h>

GLOBAL _UBYTE ProgramName[] = "RscCoder";

/*----------------------------------------------------------------------*/

LOCAL _VOID WriteHex(_UWORD num)
{
	HexToStr(num, 4, str, sizeof(str));
	str[0] = '0';
	mtTextfiles_WriteLine(out, str);
	mtTextfiles_WriteLine(out, "H");
}


LOCAL _VOID WriteLine(CONST _UBYTE *str, _BOOL newline)
{
	mtTextfiles_WriteLine(out, str);
	if (newline)
		mtTextfiles_WriteLn(out);
}


#define rsh_size resource[17]

int main(_VOID)
{
	_WORD objc;
	_UWORD i, j, n;
	_UBYTE *p;
	
	ApplInit();
	mtDials_Init();
	
	compiler = mm2;
	MouseBusy();
	rect.g_x = (MaxWidth / 2) - BoxWidth;
	rect.g_y = (MaxHeight / 2) - BoxHeight;
	rect.g_w = BoxWidth * 2;
	rect.g_h = BoxHeight * 2;
	if ((rsc = RscReloc(RscData, sizeof(RscData))) == NULL)
		ApplTerm(-1);
	MouseArrow();
	MouseOn();
	Wind_Update(BEG_UPDATE);
	maindial = RscGaddr(rsc, R_TREE, Main);
	if (!mtDials_NewDial(maindial))
	{
		RscFree(rsc);
		ApplTerm(-1);
	}
	mtDials_DialCenter(maindial, CSCREEN, 0, 0, &clip);
	mtDials_DialForm(maindial, DGROW, &rect, &clip);
	mtDials_DialForm(maindial, DSTART, &clip, &clip);
	mtDials_DialDraw(maindial, ROOT, MAX_DEPTH, &clip, FALSE);
	for (;;)
	{
		MouseArrow();
		objc = mtDials_DialDo(maindial, -1);
		DoubleClick(&objc);
		if (objc == Formm2)
			compiler = mm2;
		else if (objc == Fortdi)
			compiler = tdi;
		else if (objc == Forother)
			compiler = other;
		else if (objc == Forarray)
			compiler = array;
		else if (objc == Cancel)
			break;
		else if (objc == Ok)
		{
			MouseOn();
			if (mtDir_GetDir(inPath, inName, "Resource w\204hlen"))
			{
				strBcpy(inFile, inPath);
				F_Path_Dirname(inFile);
				F_Path_Append(inFile, inName);
				strBcpy(outPath, inPath);
				F_Path_Dirname(outPath);
				F_Path_Append(outPath, "*" COD_EXTENSION);
				strBcpy(outName, inName);
				if ((p = strrchr(outName, '.')) != NULL)
					strcpy(p, COD_EXTENSION);
				else
					strBcat(outName, COD_EXTENSION);
				MouseOn();
				if (Dpathconf(inFile, DP_CASE) != DP_CASESENS)
				{
					strUpr(outName);
					strUpr(outPath);
				}
				if (mtDir_GetDir(outPath, outName, "Codedatei w\204hlen"))
				{
					MouseBusy();
					strBcpy(outFile, outPath);
					F_Path_Dirname(outFile);
					F_Path_Append(outFile, outName);
					
					/* Resource-Array loeschen */
					MemSetZeroArray(resource);
					
					/* Resource lesen */
					in = (FILENO)Fopen(inFile, FO_READ);
					if (in > 0)
					{
						inBytes = Fread(in, sizeof(resource), resource);
						Fclose(in);
					}
					
					/* Eventuell vorhandenes File loeschen */
					Fdelete(outFile);
					
					/* Codefile erzeugen */
					if (mtTextfiles_OpenTextfile(outFile, WRITE, 0, &out) == GERR_OK)
					{
						WriteLine("(*----------------------------------------------------------------------*", TRUE);
						WriteLine(" *        Resource-Coder 1.03  (C)92 by Peter Hellinger Software        *", TRUE);
						WriteLine(" *----------------------------------------------------------------------*", TRUE);
						WriteLine(" *           Inline-Resource erzeugt am ", FALSE);
						DateString(Tgetdate(), str, sizeof(str));  WriteLine(str, FALSE);
						WriteLine(" ", FALSE);
						TimeString(Tgettime(), str, sizeof(str));  WriteLine(str, FALSE);
						WriteLine("               *", TRUE);
						WriteLine(" *----------------------------------------------------------------------*)", TRUE);
						WriteLine("", TRUE);
						
						if (compiler == mm2)
						{
							/* Typ schreiben */
							WriteLine("TYPE tRscData = ARRAY [0..", FALSE);
							mtTextfiles_WriteCard(out, rsh_size / 2 - 1, 0);
							WriteLine("] OF CARDINAL;", TRUE); 
							WriteLine("", TRUE);
							
							/* Daten schreiben */
							WriteLine("CONST RscData = tRscData {", TRUE);
							
							j = 0;
							n = 0;
							for (;;)
							{
								if (j >= rsh_size)
									break;
								WriteLine("        ", FALSE);
								for (i = 0; i < 8 && j < rsh_size; i++)
								{
									WriteHex(resource[n]);
									j += 2;
									if (j != rsh_size)
										WriteLine(", ", FALSE);
									n += 1;
								}
								mtTextfiles_WriteLn(out);
							}
							mtTextfiles_WriteLn(out);
							WriteLine("        }; (* Ende RscData *)", TRUE);
							
						} else if (compiler == array)
						{
							/* Typ schreiben */
							WriteLine("VAR  RscData = ARRAY [0..", FALSE);
							mtTextfiles_WriteCard(out, rsh_size / 2 - 1, 0);
							WriteLine("] OF CARDINAL;", TRUE); 
							WriteLine("", TRUE);
							
							/* Daten schreiben */
							WriteLine("PROCEDURE InitRscData;", TRUE);
							WriteLine("BEGIN", TRUE);
							
							n = 0;
							for (j = 0; j < rsh_size; j += 2)
							{
								WriteLine(" RscData[", FALSE);
								mtTextfiles_WriteCard(out, n, 0);
								WriteLine("]:= ", FALSE);
								WriteHex(resource[n]);
								WriteLine("; ", TRUE);
								n += 1;
							}
							WriteLine("END InitRscData;", TRUE);
						} else
						{
							if (compiler == tdi)
							{
								/* Prozedur schreiben */
								WriteLine("(*$S-, $T-, $P-*)", TRUE);
							}
							
							WriteLine("PROCEDURE RscData;", TRUE);
							WriteLine("BEGIN", TRUE);
							for (i = j = n = 0; j < rsh_size; n++)
							{
								if (i == 0)
								{
									if (compiler == tdi)
										WriteLine(" CODE (", FALSE);
									else
										WriteLine(" INLINE (", FALSE);
								}
								WriteHex(resource[n]);
								++i;
								j += 2;
								if (j == rsh_size || i == 8)
								{
									WriteLine(");", TRUE);
									i = 0;
								} else
								{
									WriteLine(", ", FALSE);
								}
							}
							if (i != 0)
								WriteLine(");", TRUE);
							WriteLine("END RscData;", TRUE);
							if (compiler == tdi)
								WriteLine("(*$S=, $T=, $P=*)", TRUE);
						}
						WriteLine("(*----------------------------------------------------------------------*)", TRUE);
						mtTextfiles_CloseTextfile(&out);
					}
				}
			}
			DelObjState(maindial, objc, SELECTED);
			mtDials_DialDraw(maindial, objc, 0, &clip, FALSE);
		}
	}
	mtDials_DialForm(maindial, DSHRINK, &rect, &clip);
	mtDials_DialForm(maindial, DFINISH, &clip, &clip);
	Wind_Update(END_UPDATE);
	RscFree(rsc);
	ApplTerm(0);
	return 0;
}

