/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <mintutil.h>
#include <mintbind.h>
#include <magiccoo.h>
#include <strconv.h>
#include <routinec.h>
#include <routines.h>


typedef union {
	_BYTE b;
	_WORD w;
	_LONG l;
	unsigned int card;
	int i;
	unsigned long longcard;
	long int lint;
	_UBYTE c;
	_UWORD bset;
	_ULONG lbset;
	_BOOL bool;
	int r[4];
	_VOID *a;
} voidType;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*
 * Startet einen parallelen Prozess der nach /dev/null ausgibt.
 * Liefert den PID des neuen Prozesses.
 */

GLOBAL _WORD ExecDevNull(CONST _UBYTE *name, CONST _UBYTE *cmdline, CONST _UBYTE *env)
{
	_WORD null, i, pid;
	_WORD old[4];
	
	/* Kanaele -1 .. 2 duplizieren */
	for (i = 0; i < 4; i++)
		old[i] =  (_WORD)Fdup(i - 1);
	
	/* Kanaele -1 .. 2auf /dev/null */
	null = (_WORD)Fopen("u:\\dev\\null", 2);
	for (i = 0; i < 4; i++)
		Fforce(i - 1, null);
	
	/* Prozess verdoppeln          */
	pid = (_WORD)Pfork();
	if (pid == 0)
	{
		/* Der neue Kindprozess...     */

		/* Kopien schliessen            */
		for (i = 0; i < 4; i++)
			Fclose(old[i]);
		
		/* Ueberlagern       */
		Pexec(200, name, cmdline, env);
		Pterm(-1);
		/* Kommt hier nur an, wenn Pexec nicht klappte */
	}

	/* Kanaele -1 .. 2 zuruecksetzen */
	for (i = 0; i < 4; i++)
	{
		Fforce(i - 1, old[i]);
		Fclose(old[i]);
	}
	
	Fclose(null);
	return pid;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Startet einen parallelen Prozess der in eine neue Pipe ausgibt.
 * Liefert den PID des neuen Prozesses und den Filehandle
 * der Pipe in 'pipe'.
 */

GLOBAL _WORD ExecPipe(CONST _UBYTE *name, CONST _UBYTE *cmdline, CONST _UBYTE *env, _WORD *pipe)
{
	_UBYTE pipename[40];
	_WORD i, pid;
	_WORD old[4];
	_WORD masterfd, slavefd;
	size_t len;
	
	*pipe = -1;
	
	i = 0;
	for (;;)
	{
		if (i > 999)
			break;
		strBcpy(pipename, "u:\\pipe\\pty.");
		len = strLen(pipename);
		NumToStr(i, 10, 3, '0', pipename + len, sizeof(pipename) - len);
		masterfd = (_WORD)Fcreate(pipename, 0x0004);
		if (masterfd > 0)
			break;
		++i;
	}
	
	if (masterfd < 0)
		return -1;
	
	slavefd = (_WORD)Fopen(pipename, 2);
	if (slavefd < 0)
	{
		Fclose(masterfd);
		return -1;
	}
	
	for (i = 0; i < 4; i++)
		old[i] = (_WORD)Fdup(i -1);
	for (i = 0; i < 4; i++)
		Fforce(i - 1, slavefd);
	
	pid = (_WORD)Pfork();
	if (pid == 0)
	{
		for (i = 0; i < 4; i++)
			Fclose(old[i]);
		Pexec(200, name, cmdline, env);
		Pterm(-1);
	}
	
	for (i = 0; i < 4; i++)
	{
		Fforce(i - 1, old[i]);
		Fclose(old[i]);
	}
	
	Fclose(slavefd);
	*pipe = masterfd;
	return pid;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Liest von einer Pipe, sofern dort Daten vorhanden sind.
 * Liefert Null, wenn keine Daten da waren, -1 wenn der
 * Pipe-Prozess bendet wurde und einen positiven Wert, wenn
 * Zeichen gelesen wurden.
 */

GLOBAL _LONG ReadPipe(_WORD pid, _WORD pipe, _VOID *buf, _LONG bufsize, _WORD *retCode)
{
	_LONG size;
	_ULONG cSize;
	_LONG lc;
	_LONG rus[8];
	_LONG result;
	
	size = Finstat(pipe);
	if (size > 0)
	{
		if (size > bufsize)
			size = bufsize;
		cSize = Fread(pipe, size, buf);
		size = cSize;
	}

	/* Testen ob Prozess gestorben ist */
	if (size <= 0 && Pkill(pid, 0) < 0)
	{
		/* Meist kommt trotzdem noch etwas  */
		/* in der Pipe an, also kurz warten */
		lc = 0;
		(_VOID)Fselect(150, &lc, &lc, NULL);

		/* Pipe schliessen, wenn nichts kam  */
		if (Finstat(pipe) == 0)
			Fclose(pipe);

		/* Zombie entfernen */
		result = Pwait3(1, rus);
		if (result != -33)/*!ERROR*/
			*retCode = (_WORD)result;
		else
			*retCode = 0;
	}
	return size;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL IsMiNT(_VOID)
{
	_LONG val;
	
	return FindCookie("MiNT", &val);
}
