/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <mtalerts.h>
#include <mtdials.h>
#include <mtappl.h>
#include <magiccoo.h>
#include <strconv.h>
#include <mtrsc.h>
#include <mtutils.h>
#include <ro_mem.h>
#include <termproc.h>



LOCAL _UBYTE RscData[] = {
#include <alertrsc.inc>
};
 

#define cMaxChars       60    /* Maximale Zeichenzahl pro Zeile */
#define cMaxText        16    /* Maximale Anzahl Zeilen */
#define cMaxButt         5    /* Maximale Anzahl Buttons */
#define cMaxIcon       256    /* Maximale Anzahl Iconslots */
        
#define Box            0
#define Mover          1

#define Left           0
#define Center         1
#define Right          2

LOCAL BITBLK *Slot[cMaxIcon + 1];
LOCAL OBJECT TheAlert[24];
LOCAL _UBYTE Text[cMaxText + 1][cMaxChars + 1];
LOCAL struct {
	_UBYTE text[cMaxChars + 1];
	_WORD objc;
} Button[cMaxButt + 1];
LOCAL _WORD buttmode;
LOCAL _WORD IconColor = BLACK;
LOCAL OBJECT *Icontree;
LOCAL RESOURCE rsc;
LOCAL _BOOL inAlert;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _VOID ExitMtAlerts(_VOID)
{
	if (rsc != NULL)
	{
		RscFree(rsc);
		rsc = NULL;
		Icontree = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL InitMtAlerts(_VOID)
{
#define MacBut   0x0100
#define alRight  0x0200
#define alCenter 0x0400
#define alLeft   0x0800
#define Magic    "MagC"
	union {
		_LONG l;
		struct {
			_WORD v;
			_UWORD s;
		} x;
	} a;
	
	if (rsc == NULL)
	{
		if ((rsc = RscReloc(RscData, sizeof(RscData))) == NULL)
			return FALSE;
		InstallTermproc(ExitMtAlerts);
		Icontree = RscGaddr(rsc, R_TREE, 0);
		if (FindCookie(Magic, &a.l) && a.x.v == 0)
		{
			mtAlerts_ConfigAlert(a.x.s & MacBut ? alert_alticon : alert_gemicon);
			mtAlerts_ConfigAlert(a.x.s & alLeft ? alert_left : a.x.s & alRight ? alert_right : alert_center);
		} else
		{
			mtAlerts_ConfigAlert(alert_center);
			mtAlerts_ConfigAlert(alert_gemicon);
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

#define SetObjc(ob, typ, x, y, w, h, f, s, spec) \
	(ob)->ob_next = -1; \
	(ob)->ob_head = -1; \
	(ob)->ob_tail = -1; \
	(ob)->ob_type = typ; \
	(ob)->ob_x = x; \
	(ob)->ob_y = y; \
	(ob)->ob_width = w; \
	(ob)->ob_height = h; \
	(ob)->ob_flags = f; \
	(ob)->ob_state = s; \
	(ob)->ob_spec.index = (_LONG)(spec)

LOCAL _VOID mtAlerts_PrepareAlert(_VOID)
{
	SetObjc(&TheAlert[ROOT], G_BOX, 0, 0, 0, 0, FL3DBAK, OUTLINED, OBSPEC_MAKE('\0', 2, BLACK, LWHITE, FALSE, 0, WHITE));
	SetObjc(&TheAlert[Mover], (MoveBox << 8)|G_IBOX, 0, 0, 16, 16, NONE, OUTLINED, OBSPEC_MAKE('\0', 1, BLACK, LWHITE, FALSE, 0, WHITE));
	objc_add(TheAlert, ROOT, Mover);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID /* BITBLK */ *mtAlerts_Iconslot(_WORD slot)
{
	if (slot > 0 && slot <= cMaxIcon)
		return Slot[slot];
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtAlerts_SetIconslot(_WORD slot, _VOID /* BITBLK */ *icon)
{
	/* Achtung, Standard-Icons nicht ueberschreiben! */
	if (slot > 3 && slot <= cMaxIcon)
		Slot[slot] = icon;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD mtAlerts_IconNum(CONST _UBYTE *num)
{
	if (num[0] >= 'A')
		return num[0] - 'A' + 10;
	return StrToInt(num, NULL, NULL);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD idoAlert(_WORD xp, _WORD yp, _WORD mode, _WORD def, CONST _UBYTE *str)
{
	_WORD i, j, cw2, c, text, butt, ioff, tw, bw, w2, objc, ypos, typ, idx;
	_UWORD set;
	GRECT clip, s, x;
	BITBLK *iblk;
	_BOOL DefaultButton;
	
	/* Initialisieren */
	mtAlerts_PrepareAlert();
	for (i = 1; i <= cMaxText; i++)
		Text[i][0] = '\0';
	for (i = 1; i <= cMaxButt; i++)
		Button[i].text[0] = '\0', Button[i].objc = -1;
	objc = 2;
	cw2 = CharWidth * 2;
	ioff = cw2;
	ypos = CharHeight;
	if (str[0] != '[')
		return -1;
	
	/* Image-Nummer auslesen */
	i = 1;
	while (str[i] != ']' && str[i] != '\0')
		i++;
	
	/* Image aus der Image-Liste holen */
	iblk = mtAlerts_Iconslot(mtAlerts_IconNum(&str[1]));
	if (iblk != NULL)
	{
		ioff += iblk->bi_wb * 8;
		ypos += iblk->bi_hl + CharHeight * 2;
		iblk->bi_color = IconColor;
		SetObjc(&TheAlert[objc], G_IMAGE, CharWidth, CharHeight, iblk->bi_wb * 8, iblk->bi_hl, NONE, NORMAL, iblk);
		objc_add(TheAlert, ROOT, objc);
		objc++;
	} else
	{
		ypos += CharHeight * 2;
	}

	/* Alles bis zum naechsten '[' ueberspringen */
	while (str[i] != '[' && str[i] != '\0')
		i++;

	/* Textzeilen auslesen. Die einzelnen Zeichen sind durch '|' getrennt */
	text = 1;
	tw = 0;
	for (;;)
	{
		typ = 0;
		set = 0;
		j = 0;
		if (str[i] != '\0')
			++i;
		while (str[i] != '|' && str[i] != ']' && str[i] != '\0' && j < cMaxChars)
		{
			if (str[i] == '%')
			{
				switch (str[i+1])
				{
				case 'D':
					set |= OUTLINED;
					typ = SpecText << 8;
					i += 2;
					break;
				case 'U':
					set |= SHADOWED;
					typ = SpecText << 8;
					i += 2;
					break;
				case 'F':
					set |= WHITEBAK;
					typ = SpecText << 8;
					i += 2;
					break;
				case 'P':
					{
						CONST _UBYTE *p = ProgramName;
						
						while (*p != '\0' && j < cMaxChars)
							Text[text][j++] = *p++;
						i += 2;
					}
					break;
				default:
					Text[text][j++] = str[i++];
					if (str[i] != '\0' && j < cMaxChars)
						Text[text][j++] = str[i++];
					break;
				}
			} else
			{
				Text[text][j++] = str[i++];
			}
		}
		Text[text][j] = '\0';
		c = j * CharWidth;
		SetObjc(&TheAlert[objc], G_STRING|typ, ioff, text * CharHeight, c, CharHeight, NONE, set, Text[text]);
		objc_add(TheAlert, ROOT, objc);
		objc++;
		if (tw < (c + ioff))
			tw = c + ioff;
		text++;
		if (text > cMaxText)
			return -1;
		if (str[i] == ']' || str[i] == '\0')
			break;
	}

	/* Y-Position der Buttons */
	if (((text + 1) * CharHeight) > ypos)
		c = (text + 1) * CharHeight;
	else
		c = ypos;
	
	/* Bis zum Beginn der Buttontexte scannen */
	while (str[i] != '[' && str[i] != '\0')
		++i;
	

	/* Buttons auslesen. Die Buttons sind durch | getrennt. Ein '[' kennzeichnet
	 * die Ausloesetaste des Buttons.
	 */
	butt = 1;
	DefaultButton = FALSE;
	for (;;)
	{
		Button[butt].text[0] = ' ';
		j = 1;
		if (str[i] != '\0')
			i++;
		/* Undo-Button kann mit ':', Defaultbutton mit '.' erzwungen werden */
		set = EXIT | SELECTABLE;
		if (str[i] == ':')
		{
			set |= UndoButtonFlag;
			i++;
		} else if (str[i] == '.' && !DefaultButton)
		{
			set |= DEFAULT;
			DefaultButton = TRUE;
			i++;
		}
		while (str[i] != '|' && str[i] != ']' && str[i] != '\0' && j < cMaxChars)
			Button[butt].text[j++] = str[i++];
		Button[butt].text[j++] = ' ';
		Button[butt].text[j] = '\0';

		/* Merken, welche Objektnummer wir gekriegt haben */
		Button[butt].objc = objc;
		
		/* Button in den Baum eintragen */
		SetObjc(&TheAlert[objc], (SpecButton << 8)|G_BUTTON, 0, c, CharWidth * j, CharHeight, set, NORMAL, Button[butt].text);
		objc_add(TheAlert, ROOT, objc);
		
		/* Buttons auf gleiche Groesse bringen */
		if (butt > 1)
		{
			if (TheAlert[objc].ob_width < TheAlert[objc - 1].ob_width)
				TheAlert[objc].ob_width = TheAlert[objc - 1].ob_width;
			else if (TheAlert[objc].ob_width > TheAlert[objc - 1].ob_width)
				for (idx = 1; idx <= butt; idx++)
					TheAlert[Button[idx].objc].ob_width = TheAlert[objc].ob_width;
		}
		++objc;
		++butt;
		if (str[i] == ']' || str[i] == '\0' || butt > cMaxButt)
			break;
	}
	SetObjFlags(TheAlert, objc-1, LASTOB);
	
	--butt;
	bw = 0;
	for (i = 1; i <= butt; i++)
	{
		bw += TheAlert[Button[i].objc].ob_width;
		if (i > 1)
			bw += cw2;
		if (!DefaultButton && i == def)
			TheAlert[Button[i].objc].ob_flags |= DEFAULT;
	}
	
	/* Grundobjekt korrigieren */
	if (tw > bw)
		w2 = tw + CharWidth;
	else
		w2 = bw;
	w2 += cw2;
	w2 += cw2;
	TheAlert[ROOT].ob_width = w2;
	TheAlert[ROOT].ob_height = TheAlert[Button[butt].objc].ob_y + 2 * CharHeight;
	TheAlert[Mover].ob_x = TheAlert[ROOT].ob_width - TheAlert[Mover].ob_width;
	
	/* Buttons zentrieren */
	switch (buttmode)
	{
	case Left:
		w2 = cw2;
		break;
	case Center:
		w2 = (TheAlert[ROOT].ob_width - bw) / 2;
		break;
	default:
		w2 = (TheAlert[ROOT].ob_width - bw) - cw2;
		break;
	}
	
	for (i = 1; i <= butt; i++)
	{
		TheAlert[Button[i].objc].ob_x = w2;
		w2 += TheAlert[Button[i].objc].ob_width + cw2;
	}
	
	if (!mtDials_NewDial(TheAlert))
	{
		form_alert(1, "[3][Nicht genug Speicher|f\201r Alert!][Abbruch]");
		return -1;
	}
	
	if (mode == 2)
		mtDials_DialCenter(TheAlert, CPOS, xp, yp, &clip);
	else
		mtDials_DialCenter(TheAlert, CSCREEN, 0, 0, &clip);
	
	if (mtDials_IsOverloadedDialDo())
	{
		StoreMouse();
		MouseArrow();
		c = mtDials_DialDo(TheAlert, -1);
		RestoreMouse();
	} else
	{
		Wind_Update(BEG_UPDATE);
		mtDials_DialForm(TheAlert, DSTART, &s, &x);
		mtDials_DialDraw(TheAlert, ROOT, MAX_DEPTH, &clip, FALSE);
		StoreMouse();
		MouseArrow();
		c = mtDials_DialDo(TheAlert, -1);
		RestoreMouse();
		mtDials_DialForm(TheAlert, DFINISH, &s, &x);
		Wind_Update(END_UPDATE);
	}
	mtDials_DisposeDial(TheAlert);
	DoubleClick(&c);
	
	for (j = 1; j <= butt; j++)
		if (c == Button[j].objc)
			return j;
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD mtAlerts_doAlert(_WORD xp, _WORD yp, _WORD mode, _WORD def, CONST _UBYTE *str)
{
	_WORD res;
	
	if (!InitMtAlerts())
		return -1;
	if (inAlert)
		return -1;
	inAlert = TRUE;
	res = idoAlert(xp, yp, mode, def, str);
	inAlert = FALSE;
	return res;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD mtAlerts_Alert(_WORD def, CONST _UBYTE *str)
{
	return mtAlerts_doAlert(0, 0, 0, def, str);
}
/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD mtAlerts_PosAlert(_WORD x, _WORD y, _WORD def, CONST _UBYTE *str)
{
	return mtAlerts_doAlert(x, y, 2, def, str);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtAlerts_SetIconColor(_WORD color)
{
	IconColor = color;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtAlerts_ConfigAlert(AlertMode mode)
{
	_WORD i;

	if (!InitMtAlerts())
		return;
	switch (mode)
	{
	case alert_left:
		buttmode = Left;
		break;
	case alert_center:
		buttmode = Center;
		break;
	case alert_right:
		buttmode = Right;
		break;
	case alert_gemicon:
		for (i = 1; i <= 3; i++)
			Slot[i] = Icontree[i].ob_spec.bitblk;
		break;
	case alert_alticon:
		for (i = 1; i <= 3; i++)
			Slot[i] = Icontree[i + 3].ob_spec.bitblk;
		break;
	}
}
