/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#define __MAGICTOOL_AREA_IMPLEMENTATION__
#include <mtarea.h>
#include <mtutils.h>
#include <mtappl.h>
#include <ro_mem.h>

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _BOOL mtArea_NewArea(AREA *pa)
{
	AREA a;
	
	a = NEW(Area, "mtArea_NewArea");
	if (a != NULL)
	{
		a->x = -1;
		a->y = -1;
		a->w = -1;
		a->h = -1;
		a->init = FALSE;
		a->size = (_ULONG)-1;
		a->mfdb.fd_addr = NULL;
		*pa = a;
		return TRUE;
	}
	*pa = NULL;
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtArea_DisposeArea(AREA *pa)
{
	AREA a;
	
	a = *pa;
	if (a != NULL)
	{
		if (a->mfdb.fd_addr != NULL)
			SFREE(a->mfdb.fd_addr);
		OFREE(a);
		*pa = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtArea_FreeArea(AREA a)
{
	if (a != NULL)
	{
		if (a->mfdb.fd_addr != NULL)
		{
			SFREE(a->mfdb.fd_addr);
			a->mfdb.fd_addr = NULL;
		}
		a->x = -1;
		a->y = -1;
		a->w = -1;
		a->h = -1;
		a->init = FALSE;
		a->size = (_ULONG)-1;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID *mtArea_Data(AREA a)
{
	if (a != NULL)
		return a->mfdb.fd_addr;
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL mtArea_SaveArea(_WORD hndl, AREA a, CONST GRECT *r)
{
	_WORD pxy[8];
	MFDB ScreenMFDB;
	
	if (a == NULL)
		return FALSE;
	if (a->init && (r->g_w > a->w || r->g_h > a->h))
	{
		SFREE(a->mfdb.fd_addr);
		a->mfdb.fd_addr = NULL;
		a->init = FALSE;
	}
	if (!a->init)
	{
		a->mfdb.fd_wdwidth = (r->g_w + 15) >> 4;
		a->mfdb.fd_w = r->g_w;
		a->mfdb.fd_h = r->g_h;
		a->mfdb.fd_stand = 0;
		a->mfdb.fd_nplanes = Bitplanes;
		a->size = (_ULONG)a->mfdb.fd_wdwidth * 2L * a->mfdb.fd_h * Bitplanes;
		a->mfdb.fd_addr = MALLOC(a->size, "mtArea_SaveArea");
		if (a->mfdb.fd_addr == NULL)
			return FALSE;
		a->init = TRUE;
	}
	a->x = r->g_x;
	a->y = r->g_y;
	a->w = r->g_w;
	a->h = r->g_h;
	MouseOff();
	pxy[0] = r->g_x;
	pxy[1] = r->g_y;
	pxy[2] = r->g_x + r->g_w - 1;
	pxy[3] = r->g_y + r->g_h - 1;
	pxy[4] = 0;
	pxy[5] = 0;
	pxy[6] = r->g_w - 1;
	pxy[7] = r->g_h - 1;
	ScreenMFDB.fd_addr = NULL;
	vro_cpyfm(hndl, S_ONLY, pxy, &ScreenMFDB, &a->mfdb);
	MouseOn();
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtArea_CopyArea(_WORD hndl, AREA a, _WORD x, _WORD y)
{
	_WORD pxy[8];
	MFDB ScreenMFDB;
	
	if (a != NULL && a->init)
	{
		MouseOff();
		pxy[0] = 0;
		pxy[1] = 0;
		pxy[2] = a->w - 1;
		pxy[3] = a->h - 1;
		pxy[4] = x;
		pxy[5] = y;
		pxy[6] = x + pxy[2];
		pxy[7] = y + pxy[3];
		ScreenMFDB.fd_addr = NULL;
		vro_cpyfm(hndl, S_ONLY, pxy, &a->mfdb, &ScreenMFDB);
		MouseOn();
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtArea_RestoreArea(_WORD hndl, AREA a)
{
	if (a != NULL)
		mtArea_CopyArea(hndl, a, a->x, a->y);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtArea_MoveArea(_WORD hndl, AREA a, _WORD xm, _WORD ym, _WORD *xx, _WORD *yy)
{
#define fly 3
	_WORD p, mx, my, w1, h1;
	_WORD pxy[8];
	MFDB ScreenMFDB;
	
	if (a != NULL && a->init)
	{
		MouseOff();
		ScreenMFDB.fd_addr = NULL;
		
		pxy[0] = 0; pxy[1] = 0;
		pxy[2] = MaxWidth - 1; pxy[3] = MaxHeight - 1;
		vs_clip(hndl, FALSE, pxy);
		
		mx = a->x + a->w;
		my = a->y + a->h;
		w1 = a->w - 1;
		h1 = a->h - 1;
		if (ym < 0)
		{
			/* nach oben */
			p = -ym;
			if (p >= (a->h / fly))
				p = a->h / fly;
			if ((a->y - p) < deskSize.g_y)
				p = a->y - deskSize.g_y;
			if (p > 0)
			{
				/* Bildschirm teilrestaurieren */
				pxy[0] = 0; pxy[1] = a->h - p;
				pxy[2] = w1; pxy[3] = h1;
				pxy[4] = a->x; pxy[5] = my - p;
				pxy[6] = mx - 1; pxy[7] = my - 1;
				vro_cpyfm(hndl, S_ONLY, pxy, &a->mfdb, &ScreenMFDB);
				
				/* Rest intern verschieben */
				pxy[0] = 0; pxy[1] = 0;
				pxy[2] = w1; pxy[3] = a->h - p - 1;
				pxy[4] = 0; pxy[5] = p;
				pxy[6] = w1; pxy[7] = a->h - 1;
				vro_cpyfm(hndl, S_ONLY, pxy, &a->mfdb, &a->mfdb);
				
				/* Bildschirm neuen Teil sichern */
				pxy[0] = a->x; pxy[1] = a->y - p;
				pxy[2] = mx - 1; pxy[3] = a->y - 1;
				pxy[4] = 0; pxy[5] = 0;
				pxy[6] = w1; pxy[7] = p - 1;
				vro_cpyfm(hndl, S_ONLY, pxy, &ScreenMFDB, &a->mfdb);
				
				a->y -= p;
				my = a->y + a->h;
			}
		} else if (ym > 0)
		{
			/* nach unten */
			p = ym;
			if (p >= (a->h / fly))
				p = a->h / fly;
			if ((my + p) > MaxHeight)
				p = MaxHeight - my;
			if (p > 0)
			{
				/* Bildschirm teilrestaurieren */
				pxy[0] = 0; pxy[1] = 0;
				pxy[2] = w1; pxy[3] = p - 1;
				pxy[4] = a->x; pxy[5] = a->y;
				pxy[6] = mx - 1; pxy[7] = a->y + p - 1;
				vro_cpyfm(hndl, S_ONLY, pxy, &a->mfdb, &ScreenMFDB);
				
				/* Rest intern verschieben */
				pxy[0] = 0; pxy[1] = p;
				pxy[2] = w1; pxy[3] = a->h - 1;
				pxy[4] = 0; pxy[5] = 0;
				pxy[6] = w1; pxy[7] = a->h - p - 1;
				vro_cpyfm(hndl, S_ONLY, pxy, &a->mfdb, &a->mfdb);
				
				/* Bildschirm neuen Teil sichern */
				pxy[0] = a->x; pxy[1] = my;
				pxy[2] = mx - 1; pxy[3] = my + p - 1;
				pxy[4] = 0; pxy[5] = a->h - p;
				pxy[6] = w1; pxy[7] = h1;
				vro_cpyfm(hndl, S_ONLY, pxy, &ScreenMFDB, &a->mfdb);

				a->y += p;
				my = a->y + a->h;
			}
		}
		
		if (xm < 0)
		{
			/* nach links */
			p = -xm;
			if (p >= (a->w / fly))
				p = a->w / fly;
			if ((a->x - p) < deskSize.g_x)
				p = a->x - deskSize.g_x;
			if (p > 0)
			{
				/* Bildschirm teilrestaurieren */
				pxy[0] = a->w - p; pxy[1] = 0;
				pxy[2] = w1; pxy[3] = h1;
				pxy[4] = mx - p; pxy[5] = a->y;
				pxy[6] = mx - 1; pxy[7] = my - 1;
				vro_cpyfm(hndl, S_ONLY, pxy, &a->mfdb, &ScreenMFDB);
				
				/* Rest intern verschieben */
				pxy[0] = 0; pxy[1] = 0;
				pxy[2] = a->w - p - 1; pxy[3] = h1;
				pxy[4] = p; pxy[5] = 0;
				pxy[6] = a->w - 1; pxy[7] = h1;
				vro_cpyfm(hndl, S_ONLY, pxy, &a->mfdb, &a->mfdb);
				
				/* Bildschirm neuen Teil sichern */
				pxy[0] = a->x - p; pxy[1] = a->y;
				pxy[2] = a->x - 1; pxy[3] = my - 1;
				pxy[4] = 0; pxy[5] = 0;
				pxy[6] = p - 1; pxy[7] = h1;
				vro_cpyfm(hndl, S_ONLY, pxy, &ScreenMFDB, &a->mfdb);
				
				a->x -= p;
			}
		} else if (xm > 0)
		{
			/* nach rechts */
			p = xm;
			if (p >= (a->w / fly))
				p = a->w / fly;
			if ((mx + p) > MaxWidth)
				p = MaxWidth - mx;
			if (p > 0)
			{
				/* Bildschirm teilrestaurieren */
				pxy[0] = 0; pxy[1] = 0;
				pxy[2] = p - 1; pxy[3] = h1;
				pxy[4] = a->x; pxy[5] = a->y;
				pxy[6] = a->x + p - 1; pxy[7] = my - 1;
				vro_cpyfm(hndl, S_ONLY, pxy, &a->mfdb, &ScreenMFDB);
				
				/* Rest intern verschieben */
				pxy[0] = p; pxy[1] = 0;
				pxy[2] = a->w - 1; pxy[3] = h1;
				pxy[4] = 0; pxy[5] = 0;
				pxy[6] = a->w - p - 1; pxy[7] = h1;
				vro_cpyfm(hndl, S_ONLY, pxy, &a->mfdb, &a->mfdb);
				
				/* Bildschirm neuen Teil sichern */
				pxy[0] = mx; pxy[1] = a->y;
				pxy[2] = mx + p - 1; pxy[3] = my - 1;
				pxy[4] = a->w - p; pxy[5] = 0;
				pxy[6] = w1; pxy[7] = h1;
				vro_cpyfm(hndl, S_ONLY, pxy, &ScreenMFDB, &a->mfdb);

				a->x += p;
			}
		}
		
		MouseOn();
		*xx = a->x;
		*yy = a->y;
	}
}
