/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <mtdials.h>
#include <portvdi.h>
#include <mtutils.h>
#include <mtappl.h>
#include <magiccoo.h>
#include <mtpopups.h>
#include <mtrsc.h>
#include <mtxobjec.h>
#include <mtarea.h>
#include <tos.h>
#include <w_graf.h>
#include <termproc.h>
#include <keyboard.h>

#include <routinec.h>
#include <routines.h>
#include <routine.h>
#include <ro_mem.h>
#include <debug.h>
#include <file_io.h>


static _UBYTE RscData[] = {
#include <mtdialrs.inc>
};
#include <mtdialrs.h>


/*** ---------------------------------------------------------------------- ***/

#define ShortCut       '['    /* Zeichen das dem Shortcut vorausgeht */
#define MaxKeys        255

#define cCoords        0x2000    /* Flag 13 kennzeichnet Objekte mit gefixten Koordinaten */

#define cMove          0x0001 /* Dialog ist verschiebbar */
#define cRestore       0x0002 /* DDISABLE wurde aufgerufen */
#define cUser          0x0004 /* Userhandler ist installiert */
#define cAbort         0x0008 /* Dialog wird bei Click ausserhalb abgebrochen */

typedef struct {
	_WORD object;
	_UBYTE scan;
	_UWORD kbstate;
	_BOOL action;
} Userkey;

typedef Userkey Keylist[MaxKeys + 1]; /* +1 fuer EndOfList */
                        /* Liste der Shortcuts und Userkeys, die der Dialog
                         * kennt und verwaltet
                         */

typedef struct _Dialog Dialog, *DIALOG;
struct _Dialog {
	OBJECT *tree;      /* Objektbaum des Dialogs */
	AREA   back;       /* Hintergrund */
	AREA   front;      /* Vordergrund */
	Keylist keys;      /* Liste der Tastencodes */
	_UWORD flags;      /* Zustandsflags */
	mtDials_UserHandler proc; /* UserHandler */
	_WORD pmode;       /* Modus des Handlers */
	_WORD ptime;       /* Timerwert */
	_WORD prmod;       /* Rechteckmodus */
	GRECT prect;       /* Rechteck */
	_WORD *pmess;      /* Adresse des Messagepuffers */
	_VOID *procspecial;
	DIALOG next;       /* Naechster Dialog in der Kette */
};

LOCAL KEYTAB *Tastatur;  /* Zeiger auf Tastaturtabelle */

LOCAL _WORD scancodes[91];
                         /* Tabelle der Scancodes anhand des ASCII-Zeichens */

LOCAL _UWORD Config;     /* Globales Config-Flagset            */
LOCAL _WORD ROffset;     /* Offset fuer Radio- und Checkbutton */
LOCAL _WORD ChSize;      /* Default-Fontgroesse in PIXEL       */
LOCAL _WORD ChWidth;     /* Default-Zeichenbreite in PIXEL     */
LOCAL _UWORD kbshift;    /* Globaler Tastaturstatus            */
LOCAL GRECT bound;       /* Flaeche des Desktops               */
LOCAL GRECT small;       /* Default-Rechteck mit Groesse Null  */
LOCAL _UWORD ShortKey;   /* Aktuelle Ausloesetaste (ALT/CTRL)  */
LOCAL DIALOG Dials;      /* Liste der Dialoge                  */
LOCAL OBJECT *asciitab;  /* Dialog fuer Nicht-Tastatur Zeichen */
LOCAL OBJECT *confdial;  /* Konfigurations-Popupmenue          */
LOCAL _BOOL mode3D;      /* 3D-Modus An/Aus                    */
LOCAL RESOURCE rsc;

LOCAL _VOID *mKnopf[2];  /* Runder Knopf */
LOCAL _VOID *mSelect[2]; /* Ankreuzbox */
LOCAL _VOID *mCircle[2]; /* Circlebutton */
        /* Wie oben jedoch fuer Aufloesungen */
LOCAL _VOID *fKnopf[2];
LOCAL _VOID *fSelect[2];
LOCAL _VOID *fCircle[2];
LOCAL _VOID *mPunktEin;
LOCAL _VOID *fPunktEin;

/* Allgemeine Variable */

/* Fuer lange Editfelder */
LOCAL _UBYTE theText[1024];
LOCAL _WORD lastPos;

LOCAL DialDoProc dialDo;     /* Functionpointer to DialDo */

LOCAL _WORD oldPos;
LOCAL _BOOL insMode;


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _VOID mtDials_DialConfig(_UWORD flag, _BOOL set)
{
	if (set)
		Config |= flag;
	else
		Config &= ~flag;
	ShortKey = Config & UseALT ? K_ALT : K_CTRL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UWORD mtDials_GetDialConfig(_VOID)
{
	return Config;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UWORD mtDials_GetKbdState(_VOID)
{
	return kbshift;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL DIALOG GetDialog(OBJECT *tree)
{
	DIALOG p;

	for (p = Dials; p != NULL; p = p->next)
		if (p->tree == tree)
			return p;
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD FindUserKey(Keylist keys, _WORD object, _UBYTE scan, _UWORD kbstate)
{
	_WORD i;

	for (i = 0; i <= MaxKeys; i++)
	{
		if (keys[i].scan == 0xff)
			return -1;
		if (keys[i].object == object &&
			keys[i].scan == scan &&
			keys[i].kbstate == kbstate)
			return i;
	}
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_SetUserkey(OBJECT *tree, _WORD object, _UBYTE scan, _UWORD kbstate, _BOOL action, _BOOL set)
{
	DIALOG dial;
	_WORD i;
	_BOOL eol;

	dial = GetDialog(tree);
	if (dial == NULL || scan == 0)
		return;
	i = 0;
/*DYJ 5.38d Start*/
	/* Bei Shift-Tasten nicht unterscheiden */
	if (kbstate & (K_LSHIFT | K_RSHIFT))
		kbstate |= (K_LSHIFT | K_RSHIFT);
/*DYJ  Ende; */
	if (set)
	{
		/* Testen, ob nicht schon eingetragen */
		if (FindUserKey(dial->keys, object, scan, kbstate) >= 0)
			return;
		while (dial->keys[i].scan != 0 && dial->keys[i].scan != 0xff)
			++i;
		if (i < MaxKeys)
		{
/*DYJ 5.38d Start*/
/*DYJ  Ende; alt:	
			/* Bei Shift-Tasten nicht unterscheiden */
			if (kbstate & (K_LSHIFT | K_RSHIFT))
				kbstate |= (K_LSHIFT | K_RSHIFT);
*/
			eol = dial->keys[i].scan == 0xff;
			dial->keys[i].object = object;
			dial->keys[i].scan = scan;
			dial->keys[i].kbstate = kbstate;
			dial->keys[i].action = action;
			if (eol)
				dial->keys[i + 1].scan = 0xff;
		}
	} else
	{
		i = FindUserKey(dial->keys, object, scan, kbstate);
		if (i >= 0)
		{
			dial->keys[i].object = NIL;
			dial->keys[i].scan = 0;
			dial->keys[i].kbstate = 0;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_ResetUserkeys(OBJECT *tree)
{
	DIALOG q;
	_WORD i;

	q = GetDialog(tree);
	if (q == NULL)
		return;
	for (i = 0; i < MaxKeys; i++)
	{
		q->keys[i].object = NIL;
		q->keys[i].scan = 0xff;
		q->keys[i].kbstate = 0;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_ObjcExtype(OBJECT *tree, _WORD entry, _WORD extyp)
{
	if (tree[entry].ob_type != G_USERDEF)
	{
		tree[entry].ob_type = (tree[entry].ob_type & 0x00ff) | ((extyp << 8) & 0xff00);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_GetObjcExtype(OBJECT *tree, _WORD entry, _WORD *extyp, _WORD *typ)
{
	_WORD type;

	type = GetObtype(tree, entry);
	*extyp = (type >> 8) & 0xff;
	*typ = type & 0xff;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD GetLowbyte(_WORD value)
{
	return value & 0xff;
}

/*----------------------------------------------------------------------*
 *                   Zeichenroutinen unabhaengig vom Objekt              *
 *----------------------------------------------------------------------*/

/* Zeichnet eine Rechteckflaeche in Farbe color */

LOCAL _VOID mtDials_Rect(_WORD x, _WORD y, _WORD w, _WORD h, _WORD color)
{
	_WORD pxy[4];

	vsf_color(PrivateWS, color);
	pxy[0] = x;
	pxy[1] = y;
	pxy[2] = x + w;
	pxy[3] = y + h;
	v_bar(PrivateWS, pxy);
	if (color != WHITE)
		vsf_color(PrivateWS, WHITE);
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet eine Linie */
LOCAL _VOID mtDials_Line(_WORD x, _WORD y, _WORD w, _WORD h)
{
	_WORD pxy[4];

	pxy[0] = x;
	pxy[1] = y;
	pxy[2] = x + w;
	pxy[3] = y + h;
	v_pline(PrivateWS, 2, pxy);
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet einen Rahmen, times bestimmt die Dicke */
LOCAL _VOID mtDials_Frame(_WORD x, _WORD y, _WORD w, _WORD h, _WORD times)
{
	_WORD pxy[10 * 12];
	_WORD i;
	_WORD a, b;

	if (times > 10)
		times = 12;
	a = b = 0;
	for (i = 0; i < times; i++)
	{
		--x;
		--y;
		w += 2;
		h += 2;
		pxy[a + 0] = x;
		pxy[a + 1] = y;
		pxy[a + 2] = x + w;
		pxy[a + 3] = y;
		pxy[a + 4] = pxy[a + 2];
		pxy[a + 5] = y + h;
		pxy[a + 6] = x;
		pxy[a + 7] = pxy[a + 5];
		pxy[a + 8] = x;
		pxy[a + 9] = y;
		a += 10;
		b += 5;
	}
	v_pline(PrivateWS, b, pxy);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID mtDials_Frame3D(_WORD x, _WORD y, _WORD w, _WORD h, _WORD times, _WORD hCol, _WORD dCol)
{
	_WORD pxy[10];
	_WORD i;

	for (i = 0; i < times; i++)
	{
		--x;
		--y;
		w += 2;
		h += 2;
		vsl_color(PrivateWS, hCol);
		pxy[0] = x;
		pxy[1] = y + h;
		pxy[2] = x;
		pxy[3] = y;
		pxy[4] = x + w;
		pxy[5] = y;
		v_pline(PrivateWS, 3, pxy);

		vsl_color(PrivateWS, dCol);
		pxy[0] = x + w;
		pxy[1] = y;
		pxy[2] = pxy[0];
		pxy[3] = y + h;
		pxy[4] = x;
		pxy[5] = pxy[3];
		v_pline(PrivateWS, 3, pxy);
	}
	vsl_color(PrivateWS, BLACK);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID mtDials_Circle(_WORD x, _WORD y, _WORD w, _WORD h, _WORD fillColor, _BOOL fill)
{
	vsf_perimeter(PrivateWS, TRUE);
	vsf_color(PrivateWS, fillColor);
	vsf_interior(PrivateWS, fill ? FIS_SOLID : FIS_HOLLOW);
	v_ellipse(PrivateWS, x + w / 2, y + h / 2, w / 2, h / 2);
	vsf_color(PrivateWS, WHITE);
	vsf_perimeter(PrivateWS, FALSE);
	vsf_interior(PrivateWS, FIS_SOLID);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID mtDials_Circle3D(_WORD x, _WORD y, _WORD w, _WORD h, _BOOL sel)
{
#define upperRightRadius 450
#define lowerLeftRadius 2200
	_WORD midX, midY, radX, radY;
	_WORD col1, col2, col3, col4;

	midX = x + w / 2;
	midY = y + h / 2;
	radX = w / 2;
	if (w & 1)
		++radX;
	radY = h / 2;
	if (h & 1)
		++radY;
	vsf_perimeter(PrivateWS, TRUE);
	vsf_interior(PrivateWS, FIS_HOLLOW);
	if (!sel)
	{
		col1 = LBLACK;
		col2 = WHITE;
		col3 = LBLACK;
		col4 = BLACK;
	} else
	{
		col1 = BLACK;
		col2 = LBLACK;
		col3 = WHITE;
		col4 = LBLACK;
	}
	vsl_color(PrivateWS, col1);
	v_ellarc(PrivateWS, midX, midY, radX + 1, radY + 1, upperRightRadius, lowerLeftRadius);
	vsl_color(PrivateWS, col2);
	v_ellarc(PrivateWS, midX, midY, radX, radY, upperRightRadius, lowerLeftRadius);
	vsl_color(PrivateWS, col3);
	v_ellarc(PrivateWS, midX, midY, radX, radY, lowerLeftRadius, upperRightRadius);
	vsl_color(PrivateWS, col4);
	v_ellarc(PrivateWS, midX, midY, radX + 1, radY + 1, lowerLeftRadius, upperRightRadius);

	if (sel)
	{
		vsf_interior(PrivateWS, FIS_SOLID);
		vsf_color(PrivateWS, BLACK);
		v_ellipse(PrivateWS, midX, midY, 2, 2);
	}
	vsf_color(PrivateWS, WHITE);
	vsf_perimeter(PrivateWS, FALSE);
	vsf_interior(PrivateWS, FIS_SOLID);
	vsl_color(PrivateWS, BLACK);
}

/*** ---------------------------------------------------------------------- ***/

typedef enum { circle, radio, checkbutton } ButtonType;

LOCAL _VOID mtDials_Image(_WORD x, _WORD y, ButtonType typ, _BOOL sel, _BOOL check)
{
	MFDB m;
	MFDB ScreenMFDB;
	_WORD h;
	_WORD pxy[8];
	_WORD colors[2];

	ScreenMFDB.fd_addr = 0;
	if (CharHeight == 8 || CharHeight == 16)
	{
		switch (typ)
		{
		case circle:
			if (CharHeight < 16)
				h = 7, m.fd_addr = fCircle[sel];
			else
				h = 15, m.fd_addr = mCircle[sel];
			break;
		case radio:
			if (CharHeight < 16)
				h = 7, m.fd_addr = fKnopf[sel];
			else
				h = 15, m.fd_addr = mKnopf[sel];
			break;
		default:
		case checkbutton:
			if (CharHeight < 16)
				h = 7, m.fd_addr = (check && sel) ? fPunktEin : fSelect[sel];
			else
				h = 15, m.fd_addr = (check && sel) ? mPunktEin : mSelect[sel];
			break;
		}
		m.fd_w = 16;
		m.fd_h = h;
		m.fd_wdwidth = 1;
		m.fd_stand = 1;
		m.fd_nplanes = 1;
		mtDials_Rect(x, y, 16, h - 1, WHITE);
		pxy[0] = 0; pxy[1] = 0;
		pxy[2] = 15; pxy[3] = h;
		pxy[4] = x; pxy[5] = y;
		pxy[6] = x + 15; pxy[7] = y + h - 1;
		colors[0] = BLACK;
		colors[1] = WHITE;
		vrt_cpyfm(PrivateWS, MD_REPLACE, pxy, &m, &ScreenMFDB, colors);
	} else
	{
		mtDials_Rect(x, y, CharWidth * 2, CharHeight - 1, WHITE);
		switch (typ)
		{
		case circle:
			mtDials_Frame(x, y, CharWidth * 2, CharHeight - 1, 1);
			v_gtext(PrivateWS, x + CharWidth / 2, y + 1, "\002"); /* down arrow */
			if (sel)
			{
				vswr_mode(PrivateWS, MD_XOR);
				mtDials_Rect(x + 1, y + 1, CharWidth * 2 - 2, Max(CharHeight - 3, 1), BLACK);
				vswr_mode(PrivateWS, MD_REPLACE);
			}
			break;
		case radio:
			/* Radiobutton:
			 * Bei Groessen < 12 Punkt Rechteck Outlined und Innenflaeche
			 * gefuellt oder nicht, ansonsten Kreise zeichnen 
			 */
			if (CharHeight < 12)
			{
				mtDials_Frame(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), 1);
				mtDials_Frame(x + 4, y + 4, Max(CharWidth * 2 - 8, 1), Max(CharHeight - 8, 1), 1);
				if (sel)
					mtDials_Rect(x + 4, y + 4, Max(CharWidth * 2 - 8, 1), Max(CharHeight - 8, 1), BLACK);
			} else
			{
				mtDials_Circle(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), BLACK, FALSE);
				if (sel)
					mtDials_Circle(x + 5, y + 5, Max(CharWidth * 2 - 10, 1), Max(CharHeight - 10, 1), BLACK, TRUE);
			}
			break;
		case checkbutton:
			/* Checkboxen: Rechteck, ggf. gefuellt oder Linien durch */
			mtDials_Frame(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), 1);
			if (check && sel)
			{
				vsf_color(PrivateWS, BLACK);
				vsf_interior(PrivateWS, FIS_PATTERN);
				vsf_style(PrivateWS, IP_1PATT);
				mtDials_Rect(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), BLACK);
				vsf_color(PrivateWS, WHITE);
				vsf_interior(PrivateWS, FIS_SOLID);
				vsf_style(PrivateWS, IP_HOLLOW);
			} else if (sel)
			{
				mtDials_Line(x + 2, y + 2, CharWidth * 2 - 4, CharHeight - 4);
				mtDials_Line(x + CharWidth * 2 - 2, y + 2, -CharWidth * 2 + 4, CharHeight - 4);
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID mtDials_Image3D(_WORD x, _WORD y, ButtonType typ, _BOOL sel, _BOOL check)
{
	MFDB m;
	MFDB ScreenMFDB;
	_WORD h;
	_WORD pxy[8];
	_WORD colors[2];

	ScreenMFDB.fd_addr = 0;
	if ((CharHeight == 8 || CharHeight == 16) && typ == circle)
	{
		switch (typ)
		{
		case circle:
			if (CharHeight < 16)
				h = 7, m.fd_addr = fCircle[sel];
			else
				h = 15, m.fd_addr = mCircle[sel];
			break;
		case radio:
			if (CharHeight < 16)
				h = 7, m.fd_addr = fKnopf[sel];
			else
				h = 15, m.fd_addr = mKnopf[sel];
			break;
		default:
		case checkbutton:
			if (CharHeight < 16)
				h = 7, m.fd_addr = (check && sel) ? fPunktEin : fSelect[sel];
			else
				h = 15, m.fd_addr = (check && sel) ? mPunktEin : mSelect[sel];
			break;
		}
		m.fd_w = 16;
		m.fd_h = h;
		m.fd_wdwidth = 1;
		m.fd_stand = 1;
		m.fd_nplanes = 1;
		mtDials_Rect(x, y, 16, h - 1, LWHITE);
		pxy[0] = 0; pxy[1] = 0;
		pxy[2] = 15; pxy[3] = h;
		pxy[4] = x; pxy[5] = y;
		pxy[6] = x + 15; pxy[7] = y + h - 1;
		colors[0] = BLACK;
		colors[1] = LWHITE;
		vrt_cpyfm(PrivateWS, MD_TRANS, pxy, &m, &ScreenMFDB, colors);
	} else
	{
		mtDials_Rect(x, y, CharWidth * 2, CharHeight - 1, LWHITE);
		switch (typ)
		{
		case circle:
			mtDials_Frame(x, y, CharWidth * 2, CharHeight - 1, 1);
			vswr_mode(PrivateWS, MD_TRANS);
			v_gtext(PrivateWS, x + CharWidth / 2, y + 1, "\002"); /* down arrow */
			if (sel)
			{
				vswr_mode(PrivateWS, MD_XOR);
				mtDials_Rect(x + 1, y + 1, CharWidth * 2 - 2, Max(CharHeight - 3, 1), BLACK);
			}
			vswr_mode(PrivateWS, MD_REPLACE);
			break;
		case radio:
			if (CharHeight < 12)
			{
				if (sel)
				{
					mtDials_Frame3D(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), 1, LBLACK, WHITE);
					mtDials_Rect(x + 4, y + 4, Max(CharWidth * 2 - 8, 1), Max(CharHeight - 8, 1), BLACK);
				} else
				{
 					mtDials_Frame3D(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), 1, WHITE, LBLACK);
 				}
			} else
			{
				mtDials_Circle3D(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), sel);
			}
			break;
		case checkbutton:
			if (sel)
			{
				mtDials_Frame3D(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), 1, LBLACK, WHITE);
				if (check)
				{
					vswr_mode(PrivateWS, MD_TRANS);
					vsf_color(PrivateWS, BLACK);
					vsf_interior(PrivateWS, FIS_PATTERN);
					vsf_style(PrivateWS, IP_1PATT);
					mtDials_Rect(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), BLACK);
					vsf_color(PrivateWS, WHITE);
					vsf_interior(PrivateWS, FIS_SOLID);
					vsf_style(PrivateWS, IP_HOLLOW);
					vswr_mode(PrivateWS, MD_REPLACE);
					/* Innen mit dunkel fuellen */
/*					mtDials_Rect(x + 3, y + 3, Max(CharWidth * 2 - 6, 1), Max(CharHeight - 6, 1), LBLACK); */
				} else
				{
					/* Linien ueber Kreuz malen */
					mtDials_Line(x + 2, y + 2, CharWidth * 2 - 4, CharHeight - 4);
					mtDials_Line(x + 2, y + 3, CharWidth * 2 - 5, CharHeight - 5);
					mtDials_Line(x + CharWidth * 2 - 2, y + 2, -CharWidth * 2 + 4, CharHeight - 4);
					mtDials_Line(x + CharWidth * 2 - 2, y + 3, -CharWidth * 2 + 5, CharHeight - 5);
				}
			} else
			{
				mtDials_Frame3D(x + 2, y + 2, Max(CharWidth * 2 - 4, 1), Max(CharHeight - 4, 1), 1, WHITE, LBLACK);
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet einen String, scannt dabei auch nach den '['-Shortcuts
 * Dadurch werden nur Shortcuts erkannt, die auch wirklich gezeichnet 
 * wurden!  Returniert einen eventuellen ShortCut als _Scancode_.
 * Dieser muss dann nur noch mittels SetUserkey eingetragen werden.
 *
 * x ==      X-Position in absoluten Koordinaten
 * y ==      Y-Position in absoluten Koordinaten
 * width ==  Wirkt im Zusammenhang mit center: Auf diese Breite wird
 *          der String zentriert
 * string == Der zu schreibende String
 * flags ==  Entspricht den Flags, die auch bei Buttons verwendet werden
            In der Regel kann man hier den obState uebergeben.
 * center == String wird zentriert, wenn center == TRUE. Siehe width
 */
LOCAL _WORD mtDials_String(_WORD x, _WORD y, _WORD width, _UBYTE *string, _UWORD flags, _BOOL center)
{
	_WORD pos, i, uwidth;
	_WORD len, c;
	_UWORD eff;
	_UBYTE ch;
	_UBYTE str[256];
	_WORD extent[8];

	ch = '\0';
	if ((flags & SELECTED))
		vswr_mode(PrivateWS, MD_XOR);
	else
		vswr_mode(PrivateWS, MD_TRANS);
	len = 0;
	c = 0;
	pos = -1;
	uwidth = 0;
	for (;;)
	{
		if (string[c] == '\0')
			break;
		if (string[c] == ShortCut)
		{
			pos = c * ChWidth;
			uwidth = ChWidth;
			++c;
			ch = ucase(string[c]);
			if (!AESFontMonospaced && len < sizeof(str)-1)
			{
				str[len] = '\0';
				vqt_nextent(PrivateWS, str, len, extent);
				pos = extent[2] - extent[0];
				str[len] = string[c];
				str[len + 1] = '\0';
				vqt_nextent(PrivateWS, str, len + 1, extent);
				uwidth = (extent[2] - extent[0]) - pos;
			}
		}
		if (len >= sizeof(str)-1)
			break;
		str[len] = string[c];
		++c;
		++len;

	}
	str[len] = '\0';
	if (center)
	{
		if (AESFontMonospaced)
		{
			i = (width - (len * ChWidth)) / 2;
		} else
		{
			vqt_nextent(PrivateWS, str, len, extent);
			i = (width - (extent[2] - extent[0])) / 2;
		}
	} else
	{
		i = 0;
	}
	eff = 0;
	if (flags & DISABLED)
		eff |= TXT_LIGHT;
	if (flags & WHITEBAK)
		eff |= TXT_THICKENED;
	vst_effects(PrivateWS, eff);
	v_gntext(PrivateWS, x + i, y, str, len);
	if (pos >= 0)
	{
		if (flags & DISABLED)
		{
			vsl_type(PrivateWS, USERLINE);
			vsl_udsty(PrivateWS, 0x5555);
		}
		mtDials_Line(x + i + pos - 1, y + CharHeight - 1, uwidth, 0);
		if (flags & DISABLED)
			vsl_type(PrivateWS, SOLID);
	}
	vswr_mode(PrivateWS, MD_REPLACE);
	vst_effects(PrivateWS, 0);
	if ((ch >= '0' && ch <= '9') || (ch >= 'A' && ch <= 'Z'))
		return scancodes[ch];
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet einen String, scannt dabei auch nach den '['-Shortcuts.
 * Dadurch werden nur Shortcuts erkannt, die auch gezeichnet wurden!
 */
LOCAL _WORD mtDials_String3D(_WORD x, _WORD y, _WORD width, _UBYTE *string, _UWORD flags, _BOOL center, _BOOL textmove)
{
	_WORD pos, i, uwidth;
	_WORD len, c;
	_UWORD eff;
	_UBYTE ch;
	_UBYTE str[256];
	_WORD extent[8];

	ch = '\0';
	if (textmove && (flags & SELECTED))
		++x, ++y;
	vswr_mode(PrivateWS, MD_TRANS);
	len = 0;
	c = 0;
	pos = -1;
	uwidth = 0;
	for (;;)
	{
		if (string[c] == '\0')
			break;
		if (string[c] == ShortCut)
		{
			pos = c * ChWidth;
			uwidth = ChWidth;
			++c;
			ch = ucase(string[c]);
			if (!AESFontMonospaced && len < sizeof(str)-1)
			{
				str[len] = '\0';
				vqt_nextent(PrivateWS, str, len, extent);
				pos = extent[2] - extent[0];
				str[len] = string[c];
				str[len + 1] = '\0';
				vqt_nextent(PrivateWS, str, len + 1, extent);
				uwidth = (extent[2] - extent[0]) - pos;
			}
		}
		if (len >= sizeof(str)-1)
			break;
		str[len] = string[c];
		++c;
		++len;

	}
	str[len] = '\0';
	if (center)
	{
		if (AESFontMonospaced)
		{
			i = (width - (len * ChWidth)) / 2;
		} else
		{
			vqt_nextent(PrivateWS, str, len, extent);
			i = (width - (extent[2] - extent[0])) / 2;
		}
	} else
	{
		i = 0;
	}
	eff = 0;
	if (flags & DISABLED)
		eff |= TXT_LIGHT;
	if (flags & WHITEBAK)
		eff |= TXT_THICKENED;
	vst_effects(PrivateWS, eff);
	v_gntext(PrivateWS, x + i, y, str, len);
	if (pos >= 0)
	{
		if (flags & DISABLED)
		{
			vsl_type(PrivateWS, USERLINE);
			vsl_udsty(PrivateWS, 0x5555);
		}
		mtDials_Line(x + i + pos - 1, y + CharHeight - 1, uwidth, 0);
		if (flags & DISABLED)
			vsl_type(PrivateWS, SOLID);
	}
	vswr_mode(PrivateWS, MD_REPLACE);
	vst_effects(PrivateWS, 0);
	if ((ch >= '0' && ch <= '9') || (ch >= 'A' && ch <= 'Z'))
		return scancodes[ch];
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet einen Schatten , thick bestimmt die Dicke */
LOCAL _VOID mtDials_Shadow(_WORD x, _WORD y, _WORD w, _WORD h, _WORD thick)
{
	_WORD pxy[6];
	_WORD i;

	for (i = 0; i < thick; i++)
	{
		pxy[0] = x;
		pxy[1] = y + h;
		pxy[2] = x + w;
		pxy[3] = pxy[1];
		pxy[4] = pxy[2];
		pxy[5] = y;
		v_pline(PrivateWS, 3, pxy);
		++w;
		++h;
	}
}

/*** ---------------------------------------------------------------------- ***/

/* zeichnet die Movebox */
LOCAL _UWORD _CDECL mtDials_DrawMover(PARMBLK *p)
{
	_WORD i;
	OBJECT *t;
	DIALOG d;
	_WORD pxy[14];

	t = p->pb_tree;
	d = GetPrivate(t, p->pb_obj);
	if (d != NULL)
	{
		--p->pb_x;
		mtDials_Rect(p->pb_x - 2, p->pb_y - 2, 2, p->pb_h, WHITE);
		mtDials_Rect(p->pb_x - 2, p->pb_y + p->pb_h, p->pb_w + 2, 2, WHITE);
		pxy[0] = p->pb_x - 3;
		pxy[1] = p->pb_y - 3;
		pxy[2] = p->pb_x + p->pb_w + 3;
		pxy[3] = p->pb_y + p->pb_h + 3;
		pxy[4] = pxy[0];
		pxy[5] = pxy[3];
		pxy[6] = pxy[0];
		pxy[7] = pxy[1];
		{
			pxy[8] = p->pb_x;
			pxy[9] = p->pb_y;
			pxy[10] = p->pb_x;
			pxy[11] = p->pb_y + p->pb_h;
			pxy[12] = p->pb_x + p->pb_w;
			pxy[13] = pxy[11];
			i = 7;
		}
		v_pline(PrivateWS, i, pxy);
	}
	return NORMAL;
}

/*** ---------------------------------------------------------------------- ***/

/* Zeichnet einen Knopf, eine Crossbox oder einen Button */
LOCAL _UWORD _CDECL mtDials_DrawButton(PARMBLK *p)
{
	_WORD col;
	_WORD thick, off, roff, ch, bh, i;
	_BOOL center, selected;
	OBJECT *t;
	OBSPEC obspec;
	ButtonType butt;
	GRECT r;

	t = p->pb_tree;
	obspec.index = p->pb_parm;
	selected = p->pb_currstate & SELECTED;
	if (GetObjFlags(t, p->pb_obj, cCoords))
	{
		r.g_x = p->pb_x + 4;
		r.g_y = p->pb_y + 4;
		r.g_w = p->pb_w - 8;
		r.g_h = p->pb_h - 8;
	} else
	{
		r.g_x = p->pb_x;
		r.g_y = p->pb_y;
		r.g_w = p->pb_w;
		r.g_h = p->pb_h;
	}
	roff = 0;
	center = TRUE;
	if (GetObjFlags(t, p->pb_obj, RBUTTON) || !GetObjFlags(t, p->pb_obj, EXIT))
	{
		/* Knopf bzw. Crossbox zeichnen */
		mtDials_Rect(r.g_x, r.g_y, r.g_w, r.g_h - 1, WHITE);
		butt = GetObjFlags(t, p->pb_obj, RBUTTON) ? radio : checkbutton;
		mtDials_Image(r.g_x, r.g_y, butt, selected, FALSE);
		roff = ROffset;
		center = FALSE;
	} else if (GetObjFlags(t, p->pb_obj, EXIT))
	{
		col = selected ? BLACK : WHITE;
		mtDials_Rect(r.g_x, r.g_y, r.g_w, r.g_h, col);
		if (GetObjFlags(t, p->pb_obj, DEFAULT))
			thick = 3;
		else if (GetObjFlags(t, p->pb_obj, EXIT))
			thick = 2;
		else
			thick = 1;
		mtDials_Frame(r.g_x, r.g_y, r.g_w, r.g_h, thick);
		if (GetObjState(t, p->pb_obj, SHADOWED))
			mtDials_Shadow(r.g_x, r.g_y, r.g_w, r.g_h, thick * 2);
	}
	if (r.g_h > CharHeight)
		off = (r.g_h - CharHeight) / 2;
	else
		off = 0;
	if (p->pb_currstate & CHECKED)
	{
		vst_point(PrivateWS, 1, &ChWidth, &i, &i, &bh);
		off = (r.g_h / 2) - 2;
		center = TRUE;
	}
	ch = mtDials_String(r.g_x + roff, r.g_y + off, r.g_w, obspec.free_string, p->pb_currstate, center);
	mtDials_SetUserkey(t, p->pb_obj, ch, ShortKey, TRUE, TRUE);
	if (p->pb_currstate & CHECKED)
	{
		vst_height(PrivateWS, ChSize, &i, &i, &i, &i);
		ChWidth = CharWidth;
	}
	return NORMAL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD _CDECL mtDials_Draw3DButton(PARMBLK *p)
{
	_WORD thick, off, roff, col, ch, bh, i;
	_BOOL selected, center;
	OBJECT *t;
	OBSPEC obspec;
	ButtonType butt;
	GRECT r;
	_BOOL realButt;

	t = p->pb_tree;
	obspec.index = p->pb_parm;
	selected = p->pb_currstate & SELECTED;
	realButt = FALSE;
	if (GetObjFlags(t, p->pb_obj, cCoords))
	{
		r.g_x = p->pb_x + 4;
		r.g_y = p->pb_y + 4;
		r.g_w = p->pb_w - 8;
		r.g_h = p->pb_h - 8;
	} else
	{
		r.g_x = p->pb_x;
		r.g_y = p->pb_y;
		r.g_w = p->pb_w;
		r.g_h = p->pb_h;
	}
	roff = 0;
	center = TRUE;
	if (GetObjFlags(t, p->pb_obj, RBUTTON) ||
		!GetObjFlags(t, p->pb_obj, EXIT))
	{
		/* Knopf bzw. Crossbox zeichnen */
		mtDials_Rect(r.g_x, r.g_y, r.g_w, r.g_h - 1, LWHITE);
		butt = GetObjFlags(t, p->pb_obj, RBUTTON) ? radio : checkbutton;
		mtDials_Image3D(r.g_x, r.g_y, butt, selected, FALSE);
		roff = ROffset;
		center = FALSE;
	} else if (GetObjFlags(t, p->pb_obj, EXIT))
	{
		realButt = TRUE;
		col = LWHITE;
		mtDials_Rect(r.g_x, r.g_y, r.g_w, r.g_h, col);
		if (GetObjFlags(t, p->pb_obj, DEFAULT))
			thick = 2;
		else
			thick = 1;
		if (selected)
			mtDials_Frame3D(r.g_x, r.g_y, r.g_w, r.g_h, thick, LBLACK, WHITE);
		else
			mtDials_Frame3D(r.g_x, r.g_y, r.g_w, r.g_h, thick, WHITE, LBLACK);
		mtDials_Frame(r.g_x - thick, r.g_y - thick, r.g_w + thick * 2, r.g_h + thick * 2, 1);
		if (GetObjState(t, p->pb_obj, SHADOWED))
			mtDials_Shadow(r.g_x, r.g_y, r.g_w, r.g_h + 1, thick * 2);
	}
	if (r.g_h > CharHeight)
		off = (r.g_h - CharHeight) / 2;
	else
		off = 0;
	if (p->pb_currstate & CHECKED)
	{
		vst_point(PrivateWS, 1, &ChWidth, &i, &i, &bh);
		off = (r.g_h / 2) - 2;
		center = TRUE;
	}
	ch = mtDials_String3D(r.g_x + roff, r.g_y + off, r.g_w, obspec.free_string, p->pb_currstate, center, realButt);
	mtDials_SetUserkey(t, p->pb_obj, ch, ShortKey, TRUE, TRUE);
	if (p->pb_currstate & CHECKED)
	{
		vst_height(PrivateWS, ChSize, &i, &i, &i, &i);
		ChWidth = CharWidth;
	}
	return NORMAL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD _CDECL mtDials_DrawText(PARMBLK *p)
{
	_WORD i, l, bh, ch;
	OBJECT *t;
	OBSPEC obspec;
	_UBYTE *str;

	t = p->pb_tree;
	obspec.index = p->pb_parm;
	i = GetLowbyte(GetObtype(t, p->pb_obj));
	switch (i)
	{
	case G_TEXT:
	case G_FTEXT:
	case G_BOXTEXT:
	case G_FBOXTEXT:
		str = obspec.tedinfo->te_ptext;
		break;
	case G_STRING:
		str = obspec.free_string;
		break;
	default:
		return NORMAL;
	}
	if (p->pb_currstate & CHECKED)
		vst_point(PrivateWS, 1, &ChWidth, &i, &i, &bh);
	else
		bh = p->pb_h;
	if (p->pb_currstate & DRAW3D)
	{
		/* Zeichnet Shadowtext */
		vswr_mode(PrivateWS, MD_TRANS);
		vst_color(PrivateWS, BLACK);
		v_gtext(PrivateWS, p->pb_x + 2, p->pb_y + 2, str);
		vst_color(PrivateWS, WHITE);
		v_gtext(PrivateWS, p->pb_x + 1, p->pb_y + 1, str);
		vst_color(PrivateWS, BLACK);
		v_gtext(PrivateWS, p->pb_x, p->pb_y, str);
		vswr_mode(PrivateWS, MD_REPLACE);
	} else
	{
		ch = mtDials_String(p->pb_x, p->pb_y, p->pb_w, str, p->pb_currstate, FALSE);
		if (GetObjFlags(t, p->pb_obj, SELECTABLE|TOUCHEXIT))
			mtDials_SetUserkey(t, p->pb_obj, ch, ShortKey, TRUE, TRUE);
		if (p->pb_currstate & (SHADOWED|OUTLINED))
		{
			l = (_WORD)strLen(str);
			while (l > 0 && str[l-1] == ' ')
				--l;
			l *= CharWidth;
			mtDials_Line(p->pb_x, p->pb_y + bh, l, 0);
			if (p->pb_currstate & OUTLINED)
				mtDials_Line(p->pb_x, p->pb_y + bh + 2, l, 0);
		}
	}
	vst_height(PrivateWS, ChSize, &i, &i, &i, &i);
	vst_effects(PrivateWS, 0);
	ChWidth = CharWidth;
	return NORMAL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID mtDials_BuildStr(CONST _UBYTE *text, CONST _UBYTE *tmplt, _WORD offs, _WORD maxLen, _UBYTE *str)
{
	_WORD i;		/* Index fuer neuen String */
	_WORD j;		/* Index fuer Text */
	_WORD k;		/* Index fuer Template */

	i = 0;
	k = 0;
	j = 0;
	while (text[j] != '\0' && j < offs)
		++j;
	while (i < maxLen && tmplt[k] != '\0')
	{
		if (tmplt[k] == '_')
		{
			if (text[j] == '\0')
				str[i] = tmplt[k];
			else
				str[i] = text[j++];
		} else
		{
			str[i] = tmplt[k];
		}
		++i;
		++k;
	}
	if (text[j] != '\0')
		str[i++] = '\003';
	else
		str[i++] = ' ';
	str[i] = '\0';
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD _CDECL mtDials_DrawEdText(PARMBLK *p)
{
	_WORD i;
	_WORD col;
	OBJECT *t;
	OBSPEC obspec;
	TEDINFO *ted;
	_WORD x, y;
	_WORD maxLen, obW;

	t = p->pb_tree;
	obspec.index = p->pb_parm;
	obW = t[p->pb_obj].ob_width;
	if (GetObjFlags(t, p->pb_obj, cCoords))
		obW -= ChWidth;
	i = GetLowbyte(GetObtype(t, p->pb_obj));
	switch (i)
	{
	case G_TEXT:
	case G_FTEXT:
	case G_BOXTEXT:
	case G_FBOXTEXT:
		ted = obspec.tedinfo;
		break;
	default:
		return NORMAL;
	}
	if ((p->pb_currstate & SELECTED) && !(p->pb_currstate & DRAW3D))
		col = BLACK;
	else
		col = WHITE;
	if (p->pb_currstate & OUTLINED)
	{
		_WORD x, y, w, h;
		
		x = p->pb_x - 2;
		y = p->pb_y - 2;
		w = p->pb_w + 4;
		h = p->pb_h + 3;
		if (mode3D)
			mtDials_Frame3D(x, y, w, h, 1, WHITE, LBLACK);
		else
			mtDials_Frame(x, y, w, h, 1);
	}
	if ((ted->te_font & TE_FONT_MASK) == SMALL)
		vst_point(PrivateWS, 1, &i, &i, &i, &i);
	/*
	 * Pfeil rechts hinter Eingabefeld loeschen
	 */
	mtDials_Rect(p->pb_x + obW - 1, p->pb_y, ChWidth, p->pb_h - 1, col);
	maxLen = Min(obW / ChWidth, ted->te_tmplen - 1);
	mtDials_BuildStr(ted->te_ptext, ted->te_ptmplt, ted->te_fontid, maxLen, theText);
	y = p->pb_y + (p->pb_h - CharHeight) / 2;
	x = p->pb_x;
	v_gtext(PrivateWS, x, y, theText);
	vst_height(PrivateWS, ChSize, &i, &i, &i, &i);
	return NORMAL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD _CDECL mtDials_DrawFrame(PARMBLK *p)
{
	_WORD j, off, off2;
	_WORD col;
	OBSPEC obspec;
	_UWORD eff;

	obspec.index = p->pb_parm;
	eff = 0;
	if (p->pb_currstate & SELECTED)
		col = BLACK;
	else
		col = WHITE;
	mtDials_Rect(p->pb_x, p->pb_y, p->pb_w, p->pb_h, col);
	mtDials_Frame(p->pb_x, p->pb_y, p->pb_w, p->pb_h, 1);
	if (p->pb_currstate & SHADOWED)
		mtDials_Shadow(p->pb_x, p->pb_y, p->pb_w, p->pb_h, 3);
	if (p->pb_currstate & OUTLINED)
		mtDials_Frame(p->pb_x + 2, p->pb_y + 2, p->pb_w - 4, p->pb_h - 4, 1);
	if (p->pb_currstate & SELECTED)
		vswr_mode(PrivateWS, MD_XOR);
	if (p->pb_currstate & DISABLED)
		eff |= TXT_LIGHT;
	if (p->pb_currstate & WHITEBAK)
		eff |= TXT_THICKENED;
	vst_effects(PrivateWS, eff);
	if (p->pb_currstate & CHECKED)
		vst_point(PrivateWS, 1, &ChWidth, &j, &j, &j), off2 = ChWidth / 2;
	else
		off2 = 0;
	if (p->pb_currstate & CROSSED)
	{
		_WORD extent[8];
		
		vqt_extent(PrivateWS, obspec.free_string, extent);
		off = p->pb_x + (p->pb_w - (extent[2] - extent[0])) / 2;
	} else
	{
		off = p->pb_x + ChWidth / 2;
	}
	v_gtext(PrivateWS, off, p->pb_y - (CharHeight / 2) + off2, obspec.free_string);
	vst_height(PrivateWS, ChSize, &j, &j, &j, &j);
	ChWidth = CharWidth;
	vst_effects(PrivateWS, 0);
	vswr_mode(PrivateWS, MD_REPLACE);
	return NORMAL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD _CDECL mtDials_Draw3DFrame(PARMBLK *p)
{
	_WORD j, off, off2, width;
	OBSPEC obspec;
	_UWORD eff;
	_WORD extent[8];
	
	obspec.index = p->pb_parm;
	eff = TXT_NORMAL;
	if (p->pb_currstate & SELECTED)
		mtDials_Frame3D(p->pb_x, p->pb_y, p->pb_w, p->pb_h, 1, LBLACK, WHITE);
	else
		mtDials_Frame3D(p->pb_x, p->pb_y, p->pb_w, p->pb_h, 1, WHITE, LBLACK);
	if (p->pb_currstate & DISABLED)
		eff |= TXT_LIGHT;
	if (p->pb_currstate & WHITEBAK)
		eff |= TXT_THICKENED;
	vst_effects(PrivateWS, eff);
	if (p->pb_currstate & CHECKED)
	{
		vst_point(PrivateWS, 1, &ChWidth, &j, &j, &j);
		off2 = ChWidth / 2;
	} else
	{
		off2 = 0;
	}
	vqt_extent(PrivateWS, obspec.free_string, extent);
	width = extent[2] - extent[0];
	if (p->pb_currstate & CROSSED)
		off = p->pb_x + (p->pb_w - width) / 2;
	else
		off = p->pb_x + ChWidth / 2;
	mtDials_Rect(off, p->pb_y - (CharHeight / 2) + off2, width, (CharHeight / 2), LWHITE);
	vswr_mode(PrivateWS, MD_TRANS);
	v_gtext(PrivateWS, off, p->pb_y - (CharHeight / 2) + off2, obspec.free_string);
	vst_height(PrivateWS, ChSize, &j, &j, &j, &j);
	ChWidth = CharWidth;
	vst_effects(PrivateWS, 0);
	vswr_mode(PrivateWS, MD_REPLACE);
	return NORMAL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD _CDECL mtDials_DrawCircle(PARMBLK *p)
{
	_WORD ex, ob;
	OBJECT *t;
	_UWORD ret;

	t = p->pb_tree;
	mtDials_GetObjcExtype(t, p->pb_obj, &ex, &ob);
	ret = NORMAL;
	if (ob == G_BOXCHAR)
	{
		if(mode3D)
		{
			mtDials_Image3D(p->pb_x + 1, p->pb_y, circle, p->pb_currstate & SELECTED, FALSE);
			mtDials_Frame3D(p->pb_x + 1, p->pb_y, p->pb_w - 1, p->pb_h - 1, 1, WHITE, LBLACK);
		}
		else
		{
			mtDials_Image(p->pb_x + 1, p->pb_y, circle, p->pb_currstate & SELECTED, FALSE);
			mtDials_Frame(p->pb_x + 1, p->pb_y, p->pb_w - 1, p->pb_h - 1, 1);
		}

		if (p->pb_currstate & DISABLED)
			ret |= DISABLED;
	}
	return ret;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD _CDECL mtDials_DrawThreeState(PARMBLK *p)
{
	_WORD off, ch;
	OBJECT *t;
	OBSPEC obspec;

	t = p->pb_tree;
	obspec.index = p->pb_parm;
	if (p->pb_h > CharHeight)
		off = (p->pb_h - CharHeight) / 2;
	else
		off = 0;
	if (mode3D)
	{
		mtDials_Rect(p->pb_x, p->pb_y, p->pb_w + ROffset, p->pb_h - 1, LWHITE);
		ch = mtDials_String3D(p->pb_x + ROffset, p->pb_y + off, p->pb_w, obspec.free_string, NONE, FALSE, FALSE);
	} else
	{
		mtDials_Rect(p->pb_x, p->pb_y, p->pb_w + ROffset, p->pb_h - 1, WHITE);
		ch = mtDials_String(p->pb_x + ROffset, p->pb_y + off, p->pb_w, obspec.free_string, NONE, FALSE);
	}
	mtDials_SetUserkey(t, p->pb_obj, ch, ShortKey, TRUE, TRUE);
	if (p->pb_currstate != p->pb_prevstate)
	{
		if ((p->pb_prevstate & SELECTED) && (p->pb_prevstate & CHECKED))
			p->pb_prevstate ^= CHECKED;
		else if ((p->pb_prevstate & SELECTED) && !(p->pb_prevstate & CHECKED))
			p->pb_prevstate ^= SELECTED;
		else if (!(p->pb_prevstate & SELECTED))
		{
			p->pb_prevstate ^= SELECTED;
			if (!(p->pb_prevstate & CHECKED))
				p->pb_prevstate ^= CHECKED;
		}
		p->pb_currstate = p->pb_prevstate;
		t[p->pb_obj].ob_state = p->pb_currstate;
	}
	if (p->pb_currstate & CHECKED)
	{
		if (mode3D)
			mtDials_Image3D(p->pb_x, p->pb_y, checkbutton, TRUE, FALSE);
		else
			mtDials_Image(p->pb_x, p->pb_y, checkbutton, TRUE, FALSE);
	} else
	{
		if (mode3D)
			mtDials_Image3D(p->pb_x, p->pb_y, checkbutton, !(p->pb_currstate & SELECTED), TRUE);
		else
			mtDials_Image(p->pb_x, p->pb_y, checkbutton, !(p->pb_currstate & SELECTED), TRUE);
	}
	return NORMAL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UWORD _CDECL DrawLine(PARMBLK *p)
{
	_WORD pxy[4];
	
	vsf_color(PrivateWS, BLACK);
	vsf_interior(PrivateWS, FIS_PATTERN);
	vsf_style(PrivateWS, IP_4PATT);
	pxy[0] = p->pb_x;
	pxy[1] = p->pb_y + p->pb_h / 2 - 1;
	pxy[2] = p->pb_x + p->pb_w - 1;
	pxy[3] = pxy[1] + 1;
	v_bar(PrivateWS, pxy);
	vsf_interior(PrivateWS, FIS_SOLID);
	vsf_style(PrivateWS, IP_HOLLOW);
	vsf_color(PrivateWS, WHITE);
	return NORMAL;
}

/*----------------------------------------------------------------------*
 * Folgende Prozeduren manipulieren in der Dialogliste                  *
 *----------------------------------------------------------------------*/

GLOBAL _BOOL mtDials_NewDial(OBJECT *tree)
{
	DIALOG q;
	_WORD i;
	DIALOG dial;
	_WORD ex, typ;
	_BOOL b;

	q = GetDialog(tree);
	if (q == NULL)
	{
		/* Nur Neueintraege vornehmen */
		dial = NEW(Dialog, "mtDials_NewDialog");
		if (dial == NULL)
			return FALSE;
		MemSetZeroStruct(*dial);
		if (!mtArea_NewArea(&dial->back))
		{
			OFREE(dial);
			return FALSE;
		}
		mtArea_NewArea(&dial->front);
		dial->tree = tree;
		dial->flags = 0;
		dial->next = NULL;
		dial->pmode = 0;
		if (Dials == NULL)
		{
			Dials = dial;
		} else
		{
			q = Dials;
			while (q->next != NULL)
				q = q->next;
			q->next = dial;
		}
		mtDials_ResetUserkeys(tree);
		/* Baum scannen und Extended Objects installieren */
		i = 0;
		for (;;)
		{
			mtDials_GetObjcExtype(tree, i, &ex, &typ);
			b = TRUE;
			switch (ex)
			{
			case MoveBox:
				if (typ == G_IBOX)
					b = InstUserdef(tree, i, mtDials_DrawMover, dial);
				break;
			case SpecButton:
			case UndoButton:
				b = InstUserdef(tree, i, mode3D ? mtDials_Draw3DButton : mtDials_DrawButton, dial);
				if (!GetObjFlags(tree, i, cCoords))
				{
					tree[i].ob_x -= 4;
					tree[i].ob_y -= 4;
					tree[i].ob_width += 8;
					tree[i].ob_height += 8;
					SetObjFlags(tree, i, cCoords);
				}
				break;
			case SpecText:
				b = InstUserdef(tree, i, mtDials_DrawText, dial);
				break;
			case FrameBox:
				b = InstUserdef(tree, i, mode3D ? mtDials_Draw3DFrame : mtDials_DrawFrame, dial);
				break;
			case HelpButton:
				if (typ == G_STRING && GetObjState(tree, i, DISABLED))
					if (GetObSpec(tree, i)->free_string[0] == '-')
						b = InstUserdef(tree, i, DrawLine, dial);
				break;
			case CircleButton:
				if (typ == G_BOXCHAR)
					b = InstUserdef(tree, i, mtDials_DrawCircle, dial);
				break;
			case ThreeState:
				b = InstUserdef(tree, i, mtDials_DrawThreeState, dial);
				break;
			case LongEdit:
				if (IsMagiCScroll)
				{
					/* Wir machen hier gar nichts, User muss die manuell
					 * initialisieren
					 */
				} else
				{
					b = InstUserdef(tree, i, mtDials_DrawEdText, dial);
					if (!GetObjFlags(tree, i, cCoords))
					{
						tree[i].ob_width += CharWidth;
						SetObjFlags(tree, i, cCoords);
					}
				}
				break;
			}
			if (!b)
			{
				mtDials_DisposeDial(tree);
				return FALSE;
			}
			if (GetObjFlags(tree, i, LASTOB))
				break;
			++i;
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_DisposeDial(OBJECT *tree)
{
	DIALOG p;
	_WORD i;
	DIALOG dial;
	_WORD ex, typ;

	dial = GetDialog(tree);
	if (dial != NULL)
	{
		i = 0;
		for (;;)
		{
			mtDials_GetObjcExtype(tree, i, &ex, &typ);
			switch (ex)
			{
			case MoveBox:
			case SpecButton:
			case SpecText:
			case FrameBox:
			case HelpButton:
			case CircleButton:
			case ThreeState:
			case UndoButton:
				FreeUserdef(tree, i);
				if (ex == SpecButton || ex == UndoButton)
				{
					if (GetObjFlags(tree, i, cCoords))
					{
						tree[i].ob_x += 4;
						tree[i].ob_y += 4;
						tree[i].ob_width -= 8;
						tree[i].ob_height -= 8;
						DelObjFlags(tree, i, cCoords);
					}
				}
				break;
			case LongEdit:
				FreeUserdef(tree, i);
				if (GetObjFlags(tree, i, cCoords))
				{
					tree[i].ob_width -= CharWidth;
					DelObjFlags(tree, i, cCoords);
				}
				break;
			}
			if (GetObjFlags(tree, i, LASTOB))
				break;
			++i;
		}
		mtArea_DisposeArea(&dial->back);
		mtArea_DisposeArea(&dial->front);
		if (dial != Dials)
		{
			for (p = Dials; p->next != dial; p = p->next)
				;
			p->next = dial->next;
		} else
		{
			Dials = dial->next;
		}
		OFREE(dial);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_InstallHandler(OBJECT *tree, mtDials_UserHandler proc, _WORD callmode, _WORD timer, _WORD rmode, GRECT *rect, _VOID *mesag, _VOID *special)
{
	DIALOG dial;

	dial = GetDialog(tree);
	if (dial != NULL)
	{
		if (callmode > 0)
		{
			dial->proc = proc;
			dial->pmode = callmode;
			dial->ptime = timer;
			dial->prmod = rmode;
			if ((callmode & CallByRect) && rect != NULL)
			{
				dial->prect = *rect;
			} else
			{
				dial->prect.g_x = 0;
				dial->prect.g_y = 0;
				dial->prect.g_w = 0;
				dial->prect.g_h = 0;
			}
			dial->pmess = mesag;
			dial->procspecial = special;
			dial->flags |= cUser;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_RemoveHandler(OBJECT *tree)
{
	DIALOG dial;

	dial = GetDialog(tree);
	if (dial != NULL)
	{
		dial->flags &= ~cUser;
		dial->pmode = 0;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL mtDials_AbortOnClick(OBJECT *tree)
{
	DIALOG dial;

	dial = GetDialog(tree);
	if (dial != NULL)
	{
		dial->flags |= cAbort;
		return TRUE;
	}
	return FALSE;
}


/*----------------------------------------------------------------------*
 * Prozeduren unabhaengig von der Dialogliste                            *
 *----------------------------------------------------------------------*/

GLOBAL _VOID mtDials_DialDraw(OBJECT *tree, _WORD entry, _WORD depth, CONST GRECT *clip, _BOOL clipping)
{
	GRECT r;

	MouseOff();
	if (clipping)
	{
		r = *clip;
		AbsRect(&r);
		vs_clip(PrivateWS, TRUE, (_WORD *)&r);
		Objc_draw(tree, entry, depth, clip);
		vs_clip(PrivateWS, FALSE, (_WORD *)&r);
	} else
	{
		CalcArea(tree, ROOT, &r);
		Objc_draw(tree, entry, depth, &r);
	}
	MouseOn();
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_DialChange(OBJECT *tree, _WORD entry, _WORD state, CONST GRECT *clip, _BOOL clipping, _BOOL set, _BOOL draw)
{
	EnableObjState(tree, entry, state, set);
	if (draw)
		mtDials_DialDraw(tree, entry, 0, clip, clipping);
}

/*** ---------------------------------------------------------------------- ***/

/*----------------------------------------------------------------------*
 *                      Initalisierung des Dialogs                      *
 *----------------------------------------------------------------------*/

GLOBAL _VOID mtDials_DialCenter(OBJECT *tree, _WORD flag, _WORD x, _WORD y, GRECT *rect)
{
	typedef struct {
		char   cookie[4];
		_UBYTE product[4];
		_UWORD version;
		_WORD  vx;
		_WORD  vy;
		_WORD  vw;
		_WORD  vh;
	} *PTRVscr;

	_WORD f;
	PTRVscr Vscr;
	_UWORD bs;

	if (flag == CPOS)
	{
		tree[ROOT].ob_x = x;
		tree[ROOT].ob_y = y;
	} else
	{
		if (!(Config & UsePos))
		{
			/* Zentrieren */
			form_center(tree, &rect->g_x, &rect->g_y, &rect->g_w, &rect->g_h);
			if (FindCookie(COOKIE_VirtualScreen, (_LONG *)(&Vscr)))
			{
				if (COOKIE_VirtualScreen[0] == Vscr->cookie[0] &&
					COOKIE_VirtualScreen[1] == Vscr->cookie[1] &&
					COOKIE_VirtualScreen[2] == Vscr->cookie[2] &&
					COOKIE_VirtualScreen[3] == Vscr->cookie[3])
				{
					tree[ROOT].ob_x = Vscr->vx + (Vscr->vw / 2) - (tree[ROOT].ob_width / 2);
					tree[ROOT].ob_y = Vscr->vy + (Vscr->vh / 2) - (tree[ROOT].ob_height / 2);
				}
			}
			if (Config & UseMouse)
			{
				graf_mkstate(&x, &y, &bs, &bs);
				tree[ROOT].ob_x = x - (tree[ROOT].ob_width / 2);
				tree[ROOT].ob_y = y - (tree[ROOT].ob_height / 2);
			}
		}
	}
	/* Screengrenzen pruefen */
	f = ObjcFrame(tree, 0);
	if (f < 0)
		f = -f;
	else
		f = 0;
	if ((tree[ROOT].ob_x + tree[ROOT].ob_width + f) > (deskSize.g_x + deskSize.g_w))
		tree[ROOT].ob_x = deskSize.g_x + deskSize.g_w - tree[ROOT].ob_width - f;
	if ((tree[ROOT].ob_y + tree[ROOT].ob_height + f) > (deskSize.g_y + deskSize.g_h))
		tree[ROOT].ob_y = deskSize.g_y + deskSize.g_h - tree[ROOT].ob_height - f;
	if (tree[ROOT].ob_x < deskSize.g_x + f)
		tree[ROOT].ob_x = deskSize.g_x + f;
	if (tree[ROOT].ob_y < deskSize.g_y + f)
		tree[ROOT].ob_y = deskSize.g_y + f;
	CalcArea(tree, ROOT, rect);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_DialForm(OBJECT *tree, _WORD flag, CONST GRECT *smll, CONST GRECT *big)
{
	GRECT clp;
	DIALOG dial;

	dial = GetDialog(tree);
	MouseOff();
	if (dial != NULL)
	{
		CalcArea(dial->tree, ROOT, &clp);
		switch (flag)
		{
		case DSTART:
			if (mtArea_SaveArea(PrivateWS, dial->back, &clp))
				dial->flags |= cMove;
			else
				Form_dial(FMD_START, &small, &clp);
			break;
		case DGROW:
			if (Config & UseGrowbox)
				Form_dial(FMD_GROW, smll, big);
			break;
		case DSHRINK:
			if (Config & UseGrowbox)
				Form_dial(FMD_SHRINK, smll, big);
			break;
		case DFINISH:
			if (dial->flags & cMove)
			{
				mtArea_RestoreArea(PrivateWS, dial->back);
				mtArea_FreeArea(dial->back);
				dial->flags &= ~cMove;
			} else
			{
				Form_dial(FMD_FINISH, smll, &clp);
			}
			break;
		case DDISABLE:
			if (mtArea_SaveArea(PrivateWS, dial->front, &clp))
				dial->flags |= cRestore;
			MouseOff();
			vswr_mode(PrivateWS, MD_ERASE);
			vsf_interior(PrivateWS, FIS_PATTERN);
			vsf_style(PrivateWS, IP_4PATT);
			mtDials_Rect(clp.g_x, clp.g_y, clp.g_w - 1, clp.g_h - 1, WHITE);
			vsf_style(PrivateWS, IP_HOLLOW);
			vsf_interior(PrivateWS, FIS_SOLID);
			vswr_mode(PrivateWS, MD_REPLACE);
			MouseOn();
			break;
		case DENABLE:
			if (dial->flags & cRestore)
			{
				mtArea_RestoreArea(PrivateWS, dial->front);
				mtArea_FreeArea(dial->front);
				dial->flags &= ~cRestore;
			} else
			{
				Objc_draw(tree, ROOT, MAX_DEPTH, &clp);
			}
			break;
		}
	}
	MouseOn();
}

/*** ---------------------------------------------------------------------- ***/

/*----------------------------------------------------------------------*
 *                              Dialoghandling                          *
 *----------------------------------------------------------------------*/

LOCAL _VOID mtDials_Redraw(OBJECT *tree, AREA area, _BOOL blitten, CONST GRECT *clp)
{
	if (blitten)
		mtArea_CopyArea(PrivateWS, area, clp->g_x, clp->g_y);
	else
		Objc_draw(tree, ROOT, MAX_DEPTH, clp);
}

/*** ---------------------------------------------------------------------- ***/

/* Bewegt den Dialog */
LOCAL _VOID mtDials_moveDial(OBJECT *t, _WORD x, _WORD y)
{
	_WORD xx, yy, f, ox, oy;
	_UWORD butt;
	_BOOL blit, moved;
	GRECT clp;
	DIALOG dial;
	_UWORD bs;
	AREA area;
	
	dial = GetDialog(t);
	StoreMouse();
	if (dial->flags & cMove)
	{
		f = ObjcFrame(t, 0);
		f = f < 0 ? -f : 0;
		CalcArea(t, ROOT, &clp);
		blit = mtArea_NewArea(&area) && mtArea_SaveArea(PrivateWS, area, &clp);
		ox = x;
		oy = y;
		graf_mkstate(&xx, &yy, &butt, &bs);
		MouseHand();
		if (!(Config & UseSolid))
		{
			Graf_dragbox(&clp, &bound, &xx, &yy);
			moved = (clp.g_x != xx) || (clp.g_y != yy);
			if (moved)
			{
				mtArea_RestoreArea(PrivateWS, dial->back);
				dial->tree[0].ob_x = xx + f;
				dial->tree[0].ob_y = yy + f;
				CalcArea(t, ROOT, &clp);
				mtArea_SaveArea(PrivateWS, dial->back, &clp);
				mtDials_Redraw(t, area, blit, &clp);
			}
		} else
		{
			for (;;)
			{
				graf_mkstate(&x, &y, &butt, &bs);
				if (!(butt & MOB_LEFT))
					break;
				if ((y != oy) || (x != ox))
				{
					mtArea_MoveArea(PrivateWS, dial->back, x - ox, y - oy, &xx, &yy);
					t[ROOT].ob_x = xx + f;
					t[ROOT].ob_y = yy + f;
					ox = x;
					oy = y;
					CalcArea(t, ROOT, &clp);
					mtDials_Redraw(t, area, blit, &clp);
				}
			}
		}
		mtArea_DisposeArea(&area);
	}
	RestoreMouse();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID mtDials_hideDial(OBJECT *t)
{
	_BOOL blit;
	GRECT clp;
	DIALOG dial;
	AREA area;
	
	dial = GetDialog(t);
	StoreMouse();
	if (dial->flags & cMove)
	{
		CalcArea(t, ROOT, &clp);
		blit = mtArea_NewArea(&area) && mtArea_SaveArea(PrivateWS, area, &clp);
		mtArea_RestoreArea(PrivateWS, dial->back);
		MouseOff();
		vswr_mode(PrivateWS, MD_TRANS);
		if (MaxColors > 2)
		{
			vsl_type(PrivateWS, USERLINE);
			vsl_udsty(PrivateWS, 0x5555); /* Puentjes! */
		}
		mtDials_Frame(clp.g_x + 1, clp.g_y + 1, clp.g_w - 3, clp.g_h - 3, 1);
		vsl_type(PrivateWS, SOLID);
		vswr_mode(PrivateWS, MD_REPLACE);
		MouseBusy();
		Bounce();
		mtDials_Redraw(t, area, blit, &clp);
		mtArea_DisposeArea(&area);
	}
	RestoreMouse();
}


/*----------------------------------------------------------------------*
 *          Eventroutine zum Abfragen der Tasten und Mausknoepfe         *
 *----------------------------------------------------------------------*/

LOCAL _UWORD mtDials_DoEvent(
	_WORD *x, _WORD *y,
	_UWORD *button,
	_WORD *taste,
	_UWORD *kbshift,
	_UBYTE *scan,
	_UBYTE *ascii,
	_WORD *clicks,
	OBJECT *tree)
{
	_UWORD event;
	DIALOG d;
	_WORD message[8];
	
	event = MU_KEYBD | MU_BUTTON;
	d = GetDialog(tree);
	if (d != NULL)
	{
		if (d->pmode & CallByTimer)
			event |= MU_TIMER;
		if (d->pmode & CallByRect)
			event |= MU_M1;
		if (d->pmode & CallByMessage)
			event |= MU_MESAG;
		event = evnt_multi(event, 258, 3, 0,
			d->prmod, d->prect.g_x, d->prect.g_y, d->prect.g_w, d->prect.g_h,
			0, 0, 0, 0, 0,
			d->pmess ? d->pmess : message,
			0, 0,
			x, y, button, kbshift, taste, clicks);
	} else
	{
		event = evnt_multi(event, 258, 3, 0,
			0, 0, 0, 0, 0,
			0, 0, 0, 0, 0,
			message,
			0, 0,
			x, y, button, kbshift, taste, clicks);
	}
	*scan = (*taste >> 8) & 0xff;
	*ascii = (_UBYTE)(*taste & 0xff);
	return event;
}


/*----------------------------------------------------------------------*
 *                      Neue Objekt-Edit Funktionen                     *
 *                                                                      *
 *  Idee und Ausfuehrung: Dirk Steins                                    *
 *                                                                      *
 *  Umgeschrieben und auf Lauffaehigkeit mit anderen Compilern angepasst *
 *  (SPC und LPR kennen kein SET OF _UBYTE): Peter Hellinger              *
 *----------------------------------------------------------------------*/

typedef _UBYTE *PtrMaxStr;

/* Einige Variable wurden global deklariert, damit sie nicht in jeder
 * Subprozedur zu ObjcEdit neu ermittelt werden muessen. Das spart Zeit
 * und Code. Die Variablen werden bei jedem ObjcEdit-Aufruf neu gesetzt,
 * um immer dem aktuellen Objekt zu entsprechen.
 */

LOCAL TEDINFO *spec; /* Zeiger auf die TeD-Struktur */
LOCAL _UBYTE storestr[256];
LOCAL PtrMaxStr te_ptmplt;
LOCAL PtrMaxStr te_ptext;
LOCAL PtrMaxStr te_pvalid; /* die Strings im TeD-Objekt */
LOCAL GRECT rect;     /* Umgebungsrechteck des Objekts */
LOCAL _WORD wbox;  /* Fontgroesse */
LOCAL _WORD hbox;
LOCAL _WORD viewlen;
LOCAL _WORD leftOffs;


LOCAL _BOOL is_space(_UBYTE ch)
{
	return ch == 0x09 || ch == 0x0a || ch == 0x0b || ch == ' ';
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL isextchr(_UBYTE ch, CONST _UBYTE *extchr)
{
	while (*extchr != '\0')
	{
		if (ch == *extchr)
			return TRUE;
		++extchr;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL valid(_WORD idx, _UBYTE *pedchar)
{
	_WORD len;
	_UBYTE edchar = *pedchar;
	
	len = (_WORD)strLen(te_pvalid);
	if (idx >= len)
		idx = len - 1;
	switch (te_pvalid[idx])
	{
	case 'X':
		return TRUE;
	case 'x':
		*pedchar = ucase(edchar);
		return TRUE;
	case '9':
		return is_digit(edchar);
	case 'A':
	case 'N':
		if (is_alpha(edchar) || (edchar == ' ') ||
			(edchar >= 0xc2 && edchar <= 0xdc) ||
			isextchr(edchar, "\200\216\217\220\222\231\232\236\245\265\266\267\270\341"))
		{
			*pedchar = ucase(edchar);
			return TRUE;
		}
		break;
	case 'P':
	case 'p':
	case 'F':
		if (is_alnum(edchar) || (edchar >= 0x80) ||
			isextchr(edchar, "\\?*:._"))
		{
			*pedchar = ucase(edchar);
			return TRUE;
		}
		break;
	case 'f':
		if (is_alnum(edchar) || (edchar >= 0x80) || (edchar == '_'))
		{
			*pedchar = ucase(edchar);
			return TRUE;
		}
		break;
	case 'a':
	case 'n':
		if (is_alpha(edchar) || is_space (edchar) || ((edchar) > 0x80))
			return TRUE;
		break;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD GetCursor(_WORD idx, _WORD *newidx, _BOOL index)
{
	_WORD i, j, cpos;
	
	i = 0;
	cpos = 0;
	/* Ende der Maske suchen */
	while ((te_ptmplt[cpos] != '_') && (te_ptmplt[cpos] != '\0'))
		++cpos;
	/* Position bestimmen */
	if (index)
		j = cpos;
	else
		j = i;
	while ((j < idx) && (te_ptmplt[cpos] != '\0') && (te_ptext[i] != '\0'))
	{
		++cpos;
		++i;
		if (i < (spec->te_txtlen-1))
			while ((te_ptmplt[cpos] != '_') && (te_ptmplt[cpos] != '\0'))
				++cpos;
		if (index)
			j = cpos;
		else
			j = i;
	}
	if (index)
		return i;
	if (i < idx)
		*newidx = i;
	return cpos;
}

/*** ---------------------------------------------------------------------- ***/

#define TE_LEFT           0
#define TE_RIGHT          1
#define TE_CENTER         2

LOCAL _WORD JustPos(_VOID)
{
	_WORD extent[8];
	_WORD width;
	
	if (spec->te_just == TE_RIGHT)
		return rect.g_x + rect.g_w;
	if (spec->te_just == TE_CENTER)
	{
		if (AESFontMonospaced)
		{
			width = wbox * (spec->te_tmplen - 1);
		} else
		{
			vqt_extent(PrivateWS, te_ptmplt, extent);
			width = extent[2] - extent[0];
		}
		return rect.g_x + (rect.g_w - width) / 2;
	}
	return rect.g_x;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID drawCursor(_WORD cpos)
{
	_WORD cx, cy, xadd;
	_WORD extent[8];
	_WORD width;
	
	cy = rect.g_y + (rect.g_h - hbox) / 2;
	cx = JustPos();
	vswr_mode(PrivateWS, MD_XOR);
	MouseOff();
	if (AESFontMonospaced)
	{
		if (spec->te_just == TE_RIGHT)
			xadd = -(wbox * ((_WORD)strLen(te_ptext) - cpos));
		else
			xadd = wbox * cpos;
		xadd -= leftOffs * wbox;
		if (insMode)
			mtDials_Line(cx + xadd, cy - 1, 0, hbox + 1);
		else
			mtDials_Rect(cx + xadd, cy - 1, wbox - 1, hbox + 1, BLACK);
	} else
	{
		if (spec->te_just == TE_RIGHT)
		{
			vqt_extent(PrivateWS, te_ptext + cpos, extent);
			width = extent[2] - extent[0];
			xadd = -width;
		} else
		{
			vqt_nextent(PrivateWS, te_ptext, cpos, extent);
			width = extent[2] - extent[0];
			xadd = width;
		}
		vqt_nextent(PrivateWS, te_ptext, leftOffs, extent);
		xadd -= extent[2] - extent[0];
		if (insMode)
		{
			mtDials_Line(cx + xadd, cy - 1, 0, hbox + 1);
		} else
		{
			if (cpos >= strLen(te_ptext))
			{
				width = ChWidth;
			} else
			{
				vqt_nextent(PrivateWS, te_ptext, cpos + 1, extent);
				width = (extent[2] - extent[0]) - width;
			}
			mtDials_Rect(cx + xadd, cy - 1, width - 1, hbox + 1, BLACK);
		}
	}
	MouseOn();
	vswr_mode(PrivateWS, MD_REPLACE);
}

/*** ---------------------------------------------------------------------- ***/

typedef enum {  CurRight, CurLeft, CurUp, CurDown,
                ShCurLeft, ShCurRight, ShCurUp, ShCurDown,
                CtrlCurRight, CtrlCurLeft, CtrlCurUp, CtrlCurDown,
                Ins, ShiftIns, ShHome, CtrlHome, CtrlC, CtrlX, CtrlV, Home, BackSpace,
                Del, Escape, null, noKey } NormKey;

LOCAL NormKey normkey(_UWORD kstate, _WORD ch)
{
	_UBYTE scan;

	scan = (_UBYTE)((ch >> 8) & 0xff);
	if (kstate & (K_LSHIFT | K_RSHIFT))
	{
		switch (scan)
		{
			case 0x36:
			case 0x4d: return ShCurRight;
			case 0x2a:
			case 0x4b: return ShCurLeft;
			case 0x48: return ShCurUp;
			case 0x50: return ShCurDown;
			case 0x52: return ShiftIns;
			case 0x47: return ShHome;
		}
	} else if (kstate & K_CTRL)
	{
		switch (scan)
		{
			case 0x36:
			case 0x4d:
			case 0x74: return CtrlCurRight;
			case 0x2a:
			case 0x4b:
			case 0x73: return CtrlCurLeft;
			case 0x48: return CtrlCurUp;
			case 0x50: return CtrlCurDown;
			case 0x47: return CtrlHome;
			case 0x2d: return CtrlX;
			case 0x2e: return CtrlC;
			case 0x2f: return CtrlV;
		}
	} else if (kstate & K_ALT)
	{
		/* nichts */
	} else
	{
		switch (scan)
		{
			case 0x01: return Escape;
			case 0x0e: return BackSpace;
			case 0x47: return Home;
			case 0x48: return CurUp;
			case 0x49: return ShCurUp;
			case 0x4b: return CurLeft;
			case 0x4d: return CurRight;
			case 0x50: return CurDown;
			case 0x51: return ShCurDown;
			case 0x52: return Ins;
			case 0x53: return Del;
		}
	}
	return noKey;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL doAsciiTab(_BOOL careForRect, _UBYTE *ch)
{
	GRECT work;
	_BOOL res;
	_WORD voidI, i, mx, my, wx, wy;
	GRECT clp;
	_UBYTE ascii;
	_WORD taste;
	_WORD clicks;
	_UBYTE scan;
	_UWORD button;
	_UWORD event;
	
	res = FALSE;
	if (mtDials_NewDial(asciitab))
	{
		mtDials_DialCenter(asciitab, CSCREEN, 0, 0, &clp);
		
		if (careForRect)
		{
			ObjcArea(asciitab, ROOT, &work);
			work.g_y = rect.g_y - work.g_h - CharHeight * 2;
			if (work.g_y < deskSize.g_y)
			{
				work.g_y = rect.g_y + rect.g_h + CharHeight * 2;
				if (work.g_y + work.g_h > deskSize.g_y + deskSize.g_h)
					mtDials_DialCenter(asciitab, CSCREEN, 0, 0, &clp);
			}
			SetObjcRect(asciitab, ROOT, &work);
			CalcArea(asciitab, ROOT, &clp);
		}
		
		mtDials_DialForm(asciitab, DSTART, &clp, &clp);
		mtDials_DialDraw(asciitab, ROOT, MAX_DEPTH, &clp, FALSE);
		
		for (;;)
		{
			event = mtDials_DoEvent(&mx, &my, &button, &taste, &kbshift, &scan, &ascii, &clicks, NULL);
			if (event & MU_BUTTON)
			{
				voidI = objc_find(asciitab, ROOT, MAX_DEPTH, mx, my);
				if (voidI == 1)
				{
					mtDials_moveDial(asciitab, mx, my);
				} else if (voidI < 0 && (button & MOB_LEFT))
				{
					Bounce();
					break;
				} else if (button & MOB_LEFT)
				{
					Bounce();
					ObjcPos(asciitab, voidI, &wx, &wy);
					i = ((mx - wx) / ChWidth); /* !!!TODO */
					if (i < 0)
						i = 0;
					if ((asciitab[voidI].ob_type & 0xff) == G_STRING)
					{
						*ch = asciitab[voidI].ob_spec.free_string[i];
						if (*ch != ' ')
						{
							res = TRUE;
							break;
						}
					}
				}
			}
			
			if ((event & MU_KEYBD) && (scan == 0x61 || scan == 0x01))
				break;
		}
		mtDials_DialForm(asciitab, DFINISH, &clp, &clp);
		mtDials_DisposeDial(asciitab);
	}
	return res;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID editObject(OBJECT *tree, _WORD obj, _WORD kstate, _UWORD edchar, _WORD *idx, _WORD kind, _BOOL mode);

LOCAL _VOID InsertNormalChar(OBJECT *tree, _WORD obj, _UBYTE edchar, _WORD *idx, _WORD kind, _BOOL mode, _BOOL *draw)
{
	_WORD cpos;
	_BOOL tmpltFound;
	size_t l, j;
	_UBYTE *adr;
	
	cpos = GetCursor(*idx, idx, FALSE);
	tmpltFound = FALSE;
	j = cpos;
	if (edchar != '_' && edchar != '\0')
	{
		/* Suche nach Template-Zeichen ab cPos in M2 */
		for (;;)
		{
			adr = strChr(&te_ptmplt[j], edchar);
			if (adr != NULL)
			{
				j = adr - te_ptmplt;
				if ((te_ptmplt[j+1] == '_') && (te_ptmplt[j-1] == '_'))
				{
					tmpltFound = TRUE;
					break;
				}
				++j;
			} else
			{
				break;
			}
		}
	}
	
	if (tmpltFound)
	{
		te_ptext[*idx] = '\0'; /* String terminieren */
		while (cpos != j)
		{
			if (te_ptmplt[cpos] == '_')
			{
				/* Leerzeichen einfuegen */
				editObject(tree, obj, 0, ' ', idx, kind, TRUE);
			}
			++cpos;
		}
		*draw = TRUE;
	} else
	{
		/* Kein Template-Zeichen eingegeben */
		if ((edchar != '\0') && (mode || valid(*idx, &edchar)))
		{
			if ((*idx) < (spec->te_txtlen - 1))
			{
				if (insMode)
				{
					adr = &te_ptext[*idx];
					l = strLen(te_ptext);
					memmove(adr + 1, adr, l - *idx + 1);
				} else
				{
					/* Auf String-Terminierung achten */
					if (te_ptext[*idx] == '\0')
						te_ptext[*idx + 1] = '\0';
				}
				te_ptext[*idx] = edchar;
			} else
			{
				te_ptext[*idx - 1] = edchar;
			}
			if ((*idx) < spec->te_txtlen)
				++(*idx);
			if (!mode)
				*draw = TRUE;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID CharInsert(OBJECT *tree, _WORD obj, _WORD *idx, _WORD kind, _BOOL mode, _BOOL *draw)
{
	_UBYTE ch;
	_BOOL res;
	_WORD i = *idx;
	_WORD len;
	
	len = (_WORD)strLen(te_pvalid);
	if (i >= len)
		i = len - 1;
	if (te_pvalid[i] == 'X')
	{
		Wind_Update(BEG_MCTRL);
		res = doAsciiTab(TRUE, &ch);
		Wind_Update(END_MCTRL);
		if (res)
			InsertNormalChar(tree, obj, ch, idx, kind, mode, draw);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD EdInit(_WORD *idx, _BOOL mode, _BOOL isExt)
{
	size_t len;
	_WORD cpos;
	
	leftOffs = 0;
	len = strLen(te_ptext);
	if ((*idx < 0) || (*idx > len))
		*idx = (_WORD)len;
	if (isExt)
	{
		leftOffs = spec->te_fontid;
		if (*idx > leftOffs + viewlen)
			*idx = leftOffs;
		else if (*idx < leftOffs)
			leftOffs = Max(*idx - viewlen, 0);
		if (len < viewlen)
			leftOffs = 0;
		spec->te_fontid = leftOffs;
	}
	cpos = GetCursor(*idx, idx, FALSE);
	if (!mode)
		drawCursor(cpos);
	return cpos;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL getScrap(_UBYTE *scrap)
{
	if (scrp_read(scrap) == 0)
		return FALSE;
	if (scrap[0] == '\0')
		return FALSE;
	F_Path_Append(scrap, "scrap.txt");
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL writeToClip(CONST _UBYTE *str)
{
	_UBYTE name[PATH_MAX];
	_WORD hdl;
	
	if (!getScrap(name))
		return FALSE;
	hdl = (_WORD)Fcreate(name, 0);
	if (hdl < 0)
		return FALSE;
	Fwrite(hdl, strLen(str), str);
	Fclose(hdl);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID readFromClip(OBJECT *tree, _WORD obj, _WORD *idx, _WORD kind)
{
	_UBYTE name[PATH_MAX];
	_WORD hdl;
	_UBYTE ch;
	
	if (!getScrap(name))
		return;
	hdl = (_WORD)Fopen(name, 0);
	if (hdl < 0)
		return;
	for (;;)
	{
		if (Fread(hdl, 1, &ch) != 1)
			break;
		if (ch != '\r' && ch != '\n')
			editObject(tree, obj, 0, ch, idx, kind, TRUE);
	}
	Fclose(hdl);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD EdChar(OBJECT *tree, _WORD obj, _WORD kstate, _UWORD edchar, _WORD *idx, _WORD kind, _BOOL mode, _BOOL isExt)
{
	_BOOL draw;
	NormKey knorm;
	_UBYTE *adr;
	size_t len;
	_WORD cpos;
	
	draw = FALSE;
	knorm = normkey(kstate, edchar);
	if ((oldPos != -1) && !mode) /* alten Cursor loeschen */
		drawCursor(oldPos);
	switch (knorm)
	{
	case CtrlX:
	case CtrlC:
		if (writeToClip(te_ptext) && (knorm == CtrlX))
		/* Editfeld loeschen */
			knorm = Escape;
		break;
	case CtrlV:
		readFromClip(tree, obj, idx, kind);
		draw = TRUE;
		break;
	default:
		break;
	}
	switch (knorm)
	{
	case ShHome:
		strBcpy(storestr, te_ptext);
		break;
	case Home:
		strMcpy(te_ptext, spec->te_txtlen, storestr);
		*idx = 0;
		draw = TRUE;
		break;
	case CurLeft:
		if (*idx > 0)
			--(*idx);
		break;
	case ShCurLeft:
		*idx = 0;
		break;
	case CurRight:
		if (te_ptext[*idx] != '\0')
			++(*idx);
		break;
	case ShCurRight:
		*idx = (_WORD)strLen(te_ptext);
		break;
	case Ins:
		insMode = !insMode;
		break;
	case ShiftIns:
		CharInsert(tree, obj, idx, kind, mode, &draw);
		break;
	case Del:
		if (te_ptext[*idx] != '\0')
		{
			adr = &te_ptext[*idx];
			len = strLen(te_ptext);
			memcpy(adr, adr + 1, len - *idx);
			draw = TRUE;
		}
		break;
	case BackSpace:
		if (*idx > 0 && te_ptext[*idx - 1] != '\0')
		{
			adr = te_ptext + *idx;
			len = strLen(te_ptext);
			memcpy(adr - 1, adr, len - *idx + 1);
			--(*idx);
			draw = TRUE;
		}
		break;
	case Escape:
		memset(te_ptext, '\0', spec->te_txtlen);
		*idx = 0;
		draw = TRUE;
		break;
	case CtrlCurRight:
#define in_word(off) (is_alnum(te_ptext[*idx - off]) || te_ptext[*idx - off] >= 0x80)
		while (te_ptext[*idx] != '\0' && in_word(0))
			++(*idx);
		while (te_ptext[*idx] != '\0' && !in_word(0))
			++(*idx);
		break;
	case CtrlCurLeft:
		while (*idx > 0 && !in_word(1))
			--(*idx);
		while (*idx > 0 && in_word(1))
			--(*idx);
		break;
	case CtrlX:
	case CtrlC:
	case CtrlV:
		break;
	default:
		InsertNormalChar(tree, obj, (_UBYTE)edchar, idx, kind, mode, &draw);
		break;
	}
	cpos = GetCursor(*idx, idx, FALSE);
	if (isExt)
	{
		if (*idx <= leftOffs)
		{
			leftOffs = Max(0, *idx - 1);
			draw = TRUE;
		} else if (*idx >= leftOffs + viewlen)
		{
			leftOffs = *idx - viewlen;
			draw = TRUE;
		}
		spec->te_fontid = leftOffs;
	}
	if (draw && !mode)
		Objc_draw(tree, obj, 0, &deskSize);
	if (!mode)
		drawCursor(cpos);
	return cpos;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID editObject(OBJECT *tree, _WORD obj, _WORD kstate, _UWORD edchar, _WORD *idx, _WORD kind, _BOOL mode)
{
	_WORD extyp, ob;
	_BOOL isExt;
	
	mtDials_GetObjcExtype(tree, obj, &extyp, &ob);
	isExt = extyp == LongEdit;
	if (GetObjFlags(tree, obj, EDITABLE) &&
		!GetObjFlags(tree, obj, HIDETREE) &&
		(ob == G_FTEXT || ob == G_FBOXTEXT))
	{
		if (kind == ED_INIT)
		{
			oldPos = EdInit(idx, mode, isExt);
		} else if (kind == ED_CHAR)
		{
			oldPos = EdChar(tree, obj, kstate, edchar, idx, kind, mode, isExt);
		} else if (kind == ED_END)
		{
			if (oldPos != -1 && !mode)
				drawCursor(oldPos);
			if (isExt)
			{
				leftOffs = 0;
			}
			oldPos = -1;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID UpdateEdobj(OBJECT *tree, _WORD object)
{
	spec = GetObSpec(tree, object)->tedinfo;
	if ((spec->te_font & TE_FONT_MASK) == SMALL)
	{
		wbox = 6;
		hbox = 6;
	} else
	{
		wbox = ChWidth;
		hbox = CharHeight;
	}
	te_ptmplt = spec->te_ptmplt;
	te_ptext = spec->te_ptext;
	te_pvalid = spec->te_pvalid;
	ObjcArea(tree, object, &rect);
	viewlen = tree[object].ob_width / ChWidth;
	if (GetObjFlags(tree, object, cCoords))
	{
		--viewlen;
		rect.g_w -= ChWidth;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_ObjcEdit(OBJECT *tree, _WORD object, _WORD *pos, _WORD ch, _WORD kind)
{
	if (!IsMagiCScroll)
	{
		UpdateEdobj(tree, object);
		editObject(tree, object, kbshift, ch, pos, kind, FALSE);
	} else
	{
		/* Workaround fuer WindowDials */
		if (*pos < 0)
			*pos = lastPos;
		objc_edit(tree, object, ch, pos, kind);
		lastPos = *pos;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID SwitchKey(OBJECT *tree, _UWORD von, _UWORD nach)
{
	DIALOG d;
	_WORD j;
	
	d = GetDialog(tree);
	for (j = 0; j < MaxKeys; j++)
	{
		if (d->keys[j].scan == 0xff)
			return;
		if (d->keys[j].kbstate == von)
			d->keys[j].kbstate = nach;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID mtDials_DoConfig(OBJECT *tree, _WORD mx, _WORD my)
{
	_WORD i;
	
	if ((Config & UseConfig) && confdial != NULL)
	{
		EnableObjState(confdial, MTD_CONF_ALT, CHECKED, Config & UseALT);
		EnableObjState(confdial, MTD_CONF_CTRL, CHECKED, !(Config & UseALT));
		EnableObjState(confdial, MTD_CONF_SOLID, CHECKED, Config & UseSolid);
		EnableObjState(confdial, MTD_CONF_EDIT, CHECKED, Config & UseEdit);
#ifdef MTD_CONF_CENTER
		EnableObjState(confdial, MTD_CONF_CENTER, CHECKED, Config & UseCenter);
#endif
		EnableObjState(confdial, MTD_CONF_MOUSE, CHECKED, Config & UseMouse);
		EnableObjState(confdial, MTD_CONF_POS, CHECKED, Config & UsePos);
		EnableObjState(confdial, MTD_CONF_GROWBOX, CHECKED, Config & UseGrowbox);
		
		i = mtPopups_TreePopup(confdial, mx - 10 * CharWidth, my, MTD_CONF_CTRL);
		switch (i)
		{
		case MTD_CONF_ALT:
			mtDials_DialConfig(UseALT, TRUE);
			SwitchKey(tree, K_CTRL, K_ALT);
			break;
		case MTD_CONF_CTRL:
			mtDials_DialConfig(UseALT, FALSE);
			SwitchKey(tree, K_ALT, K_CTRL);
			break;
		case MTD_CONF_SOLID:
			Config ^= UseSolid;
			break;
		case MTD_CONF_EDIT:
			Config ^= UseEdit;
			break;
#ifdef MTD_CONF_CENTER
		case MTD_CONF_CENTER:
			Config ^= UseCenter;
			if (Config & UseCenter)
				Config &= ~(UseMouse | UsePos);
			break;
#endif
		case MTD_CONF_MOUSE:
			Config ^= UseMouse;
			if (Config & UseMouse)
				Config &= ~(UseCenter | UsePos);
			break;
		case MTD_CONF_POS:
			Config ^= UsePos;
			if (Config & UsePos)
				Config &= ~(UseCenter | UseMouse);
			break;
		case MTD_CONF_GROWBOX:
			Config ^= UseGrowbox;
			break;
		}
	}
}


/*----------------------------------------------------------------------*
 *                      Das "etwas andere" form_do                      *
 *----------------------------------------------------------------------*/

#define NoObject 32765

LOCAL _VOID Scankey(OBJECT *tree, _UBYTE scan, _UWORD kbshift, _WORD *ob, _BOOL *act)
{
	DIALOG d;
	_WORD i;
	
	d = GetDialog(tree);
	*ob = NoObject;
	*act = FALSE;
	switch (scan)
	{
		case 103:
		case 104:
		case 105: scan = scan - 95; break;
		case 106:
		case 107:
		case 108: scan = scan - 101; break;
		case 109:
		case 110:
		case 111: scan = scan - 107; break;
		case 112: scan = 11; break;
		case 120:
		case 121:
		case 122:
		case 123:
		case 124:
		case 125:
		case 126:
		case 127:
		case 128:
		case 129:
		case 130:
		case 131:
		case 132: scan = scan - 118; break;
	}
	
	/* Bei Shift-Tasten nicht unterscheiden */
	if (kbshift & (K_LSHIFT | K_RSHIFT))
		kbshift |= K_LSHIFT | K_RSHIFT;
	i = 0;
	for (;;)
	{
		if (d->keys[i].scan == 0xff)
			break;
		/* Testen, ob dieses Objekt gemeint war */
		if ((d->keys[i].scan == scan) && (d->keys[i].kbstate == kbshift))
		{
			/* Koennte es sein */
			if ((d->keys[i].object < 0) ||
				(!GetObjState(tree, d->keys[i].object, DISABLED) && !GetObjFlags(tree, d->keys[i].object, HIDETREE)))
			{
				*ob = d->keys[i].object;
				*act = d->keys[i].action;
				break;
			}
		}
		++i;
	}
}


/*-------------------------------------------------------------------------*/
/*-                                                                       -*/
/*- Event abarbeiten -> TRUE: Objekt erwischt                             -*/
/*-                                                                       -*/
/*-------------------------------------------------------------------------*/

LOCAL _VOID ChangeEdit(OBJECT *tree, _WORD eintrag, _WORD *edit, _WORD *pos, _BOOL editable)
{
	if (editable)
	{
		mtDials_ObjcEdit(tree, *edit, pos, 0, ED_END);
		if (eintrag != *edit)
		{
			*edit = eintrag;
			insMode = TRUE;
		}
		mtDials_ObjcEdit(tree, *edit, pos, 0, ED_INIT);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL mtDials_HandleEvent(_UWORD event,
	_WORD mx,
	_WORD my,
	_UWORD *button,
	_WORD *taste,
	_UWORD *kstate,
	_UBYTE *scan,
	_WORD *clicks,
	OBJECT *tree,
	_BOOL editable,
	_WORD *edit,
	_WORD *pos,
	_WORD *entry)
{
	DIALOG dial;
	_WORD kret, key, nxt, i, typ, cx, ex;
	_UWORD user;
	_BOOL action;
	_UBYTE ascii;
	
	nxt = *edit;
	kbshift = *kstate;
	dial = GetDialog(tree);
	if (dial == NULL)
	{
		/* darf eigentlich nicht passieren */
		return FALSE;
	}
	
	/* Userhandler aufrufen */
	if (dial->flags & cUser)
	{
		if ((dial->pmode & CallByEvent) ||
			((dial->pmode & CallByTimer) && (event & MU_TIMER)) ||
			((dial->pmode & CallByRect) && (event & MU_M1)) ||
			((dial->pmode & CallByMessage) && (event & MU_MESAG)))
		{
			user = (*dial->proc)(tree, mx, my, *scan, kbshift, event, *edit, dial->procspecial);
			if (user != 0)
			{
				*entry = -1;
				return TRUE;
			}
		}
	}
	
	/*--- Objekt mit Maustaste gewaehlt? ---*/
	if (event & MU_BUTTON)
	{
		if ((*button & MOB_RIGHT) /* || ((*button & MOB_LEFT) && (kbshift != 0)) */)
		{
			/* Dialog durchsichtig machen */
			if (*button & MOB_RIGHT)
			{
				*entry = objc_find(tree, ROOT, MAX_DEPTH, mx, my);
				if (*entry >= 0)
					mtDials_GetObjcExtype(tree, *entry, &ex, &typ);
				else
					ex = -1;
			} else
			{
				ex = MoveBox;
			}
			if (ex == MoveBox)
			{
				if (editable)
					mtDials_ObjcEdit(tree, *edit, pos, 0, ED_END);
				mtDials_hideDial(tree);
				*entry = -1;
				if (editable)
					mtDials_ObjcEdit(tree, *edit, pos, 0, ED_INIT);
			}
		} else if (*button & MOB_LEFT)
		{
		
			/* Mit gedrueckter Maustaste in einen Button fahren zulassen */
			for (;;)
			{
				graf_mkstate(&mx, &my, button, &kbshift);
				*entry = objc_find(tree, ROOT, MAX_DEPTH, mx, my);
				if (!(*button & MOB_LEFT))
				{
					/* Corrected by Steffen Engel */
					if ((*entry > 0) && GetObjState(tree, *entry, DISABLED))
						*entry = 0;
					break;
				}
				if (*entry < 0)
				{
					if (dial->flags & cAbort)
					{
						*entry = -7; /* LISTDL_CANCEL */
						return TRUE;
					}
					Pling();
				} else
				{
					mtDials_GetObjcExtype(tree, *entry, &ex, &typ);
					if (!GetObjState(tree, *entry, DISABLED))
					{
						if (ex == MoveBox || GetObjFlags(tree, *entry, SELECTABLE|EXIT|TOUCHEXIT))
							break;
					}
				}
			}
			
			if (*entry < 0) /* Ausserhalb */
			{
				if (dial->flags & cAbort)
				{
					*entry = -7; /* LISTDL_CANCEL */
					return TRUE;
				}
				Pling();
			} else /* Moveobjekt angeklickt? */
			{
				mtDials_GetObjcExtype(tree, *entry, &ex, &typ);
				if (ex == MoveBox)
				{
					if (editable)
						mtDials_ObjcEdit(tree, *edit, pos, 0, ED_END);
					if (*clicks > 1)
						mtDials_DoConfig(tree, mx, my);
					else 
						mtDials_moveDial(tree, mx, my);
					if (editable)
						 mtDials_ObjcEdit(tree, *edit, pos, 0, ED_INIT);
					*entry = -1;
				} else /* Anderes Objekt erwischt */
				{
					if (GetObjFlags(tree, *entry, EDITABLE) &&
						!GetObjFlags(tree, *entry, HIDETREE))
					{
						/* Edit-Cursor plazieren */
						/* nur wenn es kein Fensterdialog oder oberstes Fenster ist */
						if (IsMagiCScroll)
						{
							ChangeEdit(tree, *entry, edit, pos, editable);
						} else
						{
							UpdateEdobj(tree, *entry);
							cx = JustPos();
							if (insMode)
								*pos = (mx - cx + wbox / 2) / wbox; /* !!!TODO */
							else
								*pos = (mx - cx) / wbox;
							/* Geaendert, arbeitete direkt auf dem Baum und rechnete nicht
							 * mit Userdef-Editable
							 */
							if (*pos > spec->te_tmplen - 1)
								*pos = spec->te_tmplen - 2;
							if (*pos < 0)
								*pos = 0;
							*pos = GetCursor(*pos, &i, TRUE);
						}
						ChangeEdit(tree, *entry, edit, pos, editable);
					}
					
					if (form_button(tree, *entry, *clicks, &nxt) == 0)
						return TRUE;
				}
			}
		}
	}
	
	/*--- Objekt mit Tastatur gewaehlt ---*/
	if (event & MU_KEYBD)
	{
		TranslateKey(&ascii, scan, taste, kstate);
		kbshift = *kstate;
		/* Erst Scankey abfragen. Haben Vorrang!!! */
		Scankey(tree, *scan, kbshift, entry, &action);
		if (!editable && (*entry == NoObject))
		{
			/* Buttons in Dialogen ohne Editfelder sind auch
			 * ohne Sondertaste bedienbar
			 */
			kbshift = kbshift + ShortKey;
			Scankey(tree, *scan, kbshift, entry, &action);
			kbshift = *kstate;
			/* kbshift wieder restaurieren, wird moeglicherweise noch
			 * gebraucht
			 */
		}
		
		if (*entry < 0)
		{
			return TRUE;
		} else if (*entry == NoObject)
		{
			/* vielleicht Editeingabe? */
			if (*edit < 0)  /* Workaround fuer Mag!X */
				kret = form_keybd(tree, ROOT, ROOT, *taste, &nxt, &key);
			else
				kret = form_keybd(tree, *edit, ROOT, *taste, &nxt, &key);
			if (kret > 0)
			{
				if (nxt > 0)
				{
					if (Config & UseEdit)
						*pos = -1;
					ChangeEdit(tree, nxt, edit, pos, editable);
					if (editable)
						*entry = *edit;
					else
						*entry = -1;
				} else if (key > 0)
				{
					if (editable)
					{
						mtDials_ObjcEdit(tree, *edit, pos, key, ED_CHAR);
						*entry = *edit;
					} else
					{
						*entry = -1;
					}
				}
			} else
			{
				*entry = nxt;
				return TRUE;
			}
		} else
		{
			if (action)
			{
				if (form_button(tree, *entry, 1, &nxt) == 0)
					return TRUE;
			} else
			{
				return TRUE;
			}
		}
	}
	
	/* Userhandler aufrufen */
	if ((dial->flags & cUser) && (dial->pmode & CallByHandling))
	{
		user = (*dial->proc)(tree, mx, my, *scan, kbshift, event, *edit, dial->procspecial);
		if (user != 0)
		{
			*entry = -1;
			return TRUE;
		}
	}
	
	return FALSE; /* nix gefunden */
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_PrepDial(OBJECT *tree, _WORD start, _BOOL *editable, _WORD *pos, _WORD *edit)
{
#define Undo 0x61
#define Help 0x62
#define Enter 0x72
#define Return 0x1c
	_WORD i;
	
 	i = ScanFlags(tree, ObjSearchFlags, ROOT, UndoButtonFlag);
 	if (i < 0)
 		i = ScanFlags(tree, ObjSearchType, ROOT, (UndoButton << 8) | G_BUTTON);
	if (i >= 0)
		mtDials_SetUserkey(tree, i, Undo, 0, TRUE, TRUE);
	i = ScanFlags(tree, ObjSearchFlags, ROOT, HelpButtonFlag);
	if (i < 0)
		i = ScanFlags(tree, ObjSearchType, ROOT, (HelpButton << 8) | G_BOXTEXT);
	if (i >= 0)
		mtDials_SetUserkey(tree, i, Help, 0, TRUE, TRUE);
	
	if (start < 0 || GetObjFlags(tree, start, HIDETREE))
	{
		/* Sicher kein Editfeld */
		*edit = -1;
		*editable = FALSE;
	} else if (!GetObjFlags(tree, start, EDITABLE))
	{
		/* Startobjekt ist kein Editfeld; Suchen! */
		*edit = ScanFlags(tree, ObjSearchFlags, ROOT, EDITABLE);
		*editable = *edit >= 0;
	} else
	{
		/* Uebergebenes Feld ist Editfeld */
		*edit = start;
		*editable = TRUE;
	}
	*pos = -1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD InternalDialDo(OBJECT *tree, _WORD start)
{
	_WORD pos, entry, edit;
	_BOOL editable;
	DIALOG dial;
	_UBYTE ascii;
	_WORD taste;
	_WORD clicks;
	_UBYTE scan;
	_UWORD button;
	_WORD mx, my;
	_UWORD event;
	
	dial = GetDialog(tree);
	if (dial != NULL)
	{
		mtDials_PrepDial(tree, start, &editable, &pos, &edit);
		if (editable)
			mtDials_ObjcEdit(tree, edit, &pos, 0, ED_INIT);
		
		Wind_Update(BEG_MCTRL);  /* Finger von der Maus, AES! */
		
		MouseOn();
		
		for (;;)
		{
			entry = -1;
			event = mtDials_DoEvent(&mx, &my, &button, &taste, &kbshift, &scan, &ascii, &clicks, tree);
			
			if (mtDials_HandleEvent(event, mx, my, &button, &taste, &kbshift, &scan, &clicks,
				tree, editable, &edit, &pos, &entry))
				break;
		}
		
		if (editable)
			mtDials_ObjcEdit(tree, edit, &pos, 0, ED_END);
		if ((event & MU_BUTTON) && clicks > 1)
		{
			/* Doppelklick: Bit15 im Return-Wert setzen */
			entry |= 0x8000;
		}
		Wind_Update(END_MCTRL);
		return entry;
	}
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD mtDials_DialDo(OBJECT *tree, _WORD edit)
{
	return dialDo(tree, edit);
}

/*** ---------------------------------------------------------------------- ***/

DialDoProc mtDials_OverloadDialDo(DialDoProc newDialDo)
{
	DialDoProc oldDialDo;
	
	if (newDialDo == FUNK_NULL)
	{
		oldDialDo = InternalDialDo;
		dialDo = InternalDialDo;
	} else
	{
		oldDialDo = dialDo;
		dialDo = newDialDo;
	}
	return oldDialDo;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL mtDials_IsOverloadedDialDo(_VOID)
{
	return dialDo != InternalDialDo;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtDials_DisableMenu(OBJECT *menu, _BOOL disable)
{
	_WORD o, title;
	
	title = menu[menu[menu[ROOT].ob_head].ob_head].ob_head;
	o = menu[menu[menu[ROOT].ob_head].ob_next].ob_head;
	EnableObjState(menu, menu[o].ob_head, DISABLED, disable);
	for (;;)
	{
		if (title > menu[title].ob_next)
			break;
		title = menu[title].ob_next;
		EnableObjState(menu, title, DISABLED, disable);
		DelObjState(menu, title, SELECTED);
	}
	menu_bar(menu, 1);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL mtDials_DialCharTable(_UBYTE *ch)
{
	_UBYTE tmp;
	_BOOL res;
	
	Wind_Update(BEG_MCTRL);
	res = doAsciiTab(FALSE, &tmp);
	Wind_Update(END_MCTRL);
	if (res)
		*ch = tmp;
	return res;
}

/*----------------------------------------------------------------------*
 *                      Initialisierung des Moduls                      *
 *----------------------------------------------------------------------*/

LOCAL _VOID mtDials_DisposeDials(_VOID)
{
	while (Dials != NULL)
		mtDials_DisposeDial(Dials->tree);
	if (rsc != NULL)
	{
		RscFree(rsc);
		rsc = NULL;
		asciitab = NULL;
		confdial = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL KEYTAB *GetKeyTable(_VOID)
{
	return Keytbl((_VOID *)-1, (_VOID *)-1, (_VOID *)-1);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL mtDials_Init(_VOID)
{
	OBJECT *adr;
	MAGX_COOKIE *mgxPtr;
	_WORD f, j, i;
	union {
		_LONG l;
		struct {
			_WORD v;
			_UWORD s;
		} x;
	} a;
	
	if (rsc == NULL)
	{
		ChSize = AESFontsize;
		ChWidth = CharWidth;
		ROffset = CharWidth * 3;
		small.g_x = 0; small.g_y = 0; small.g_w = 0; small.g_h = 0;
		bound = deskSize;
		
		/* Testen, ob 3D-Mode an ist */
		i = appl_xgetinfo(APG_OBJECTS, &f, &j, &j, &j);
		
		mode3D = (i == 1) && (f == 1) && (Bitplanes >= 4);
		
		mgxPtr = NULL;
		FindCookie(COOKIE_MagiX, (_LONG *)&mgxPtr);
		
		if (i != 1)
		{
			/* ApplGetinfo ist fehlgeschlagen */
			if (gl_ap_version >= 0x340 && mgxPtr == NULL)
			/* Falcon-TOS und kein MagiC */
				mode3D = Bitplanes >= 4;
		}
		
		/* Testen, ob Systemfont proportional ist */
		appl_xgetinfo(APG_FONT, &j, &j, &f, &j);
		
		Dials = NULL;
		InstallTermproc(mtDials_DisposeDials);
		Config = UseSolid | UseEdit | UseALT | UseGrowbox | UseConfig | UseCenter;
		
		/* ScanCode-Tabelle fuer die zulaessigen Shortcuts ermitteln */
		Tastatur = GetKeyTable();
		for (i = 1; i <= 99; i++)
		{
			f = ((_UBYTE *)(Tastatur->capslock))[i];
			switch (f)
			{
			case '0': case '1': case '2': case '3': case '4':
			case '5': case '6': case '7': case '8': case '9':
			case 'A': case 'B': case 'C': case 'D': case 'E':
			case 'F': case 'G': case 'H': case 'I': case 'J':
			case 'K': case 'L': case 'M': case 'N': case 'O':
			case 'P': case 'Q': case 'R': case 'S': case 'T':
			case 'U': case 'V': case 'W': case 'X': case 'Y': case 'Z':
				scancodes[f] = i;
				break;
			case ':': case ';': case '<': case '=': case '>': case '?': case '@':
				scancodes[f] = -1;
				break;
			}
		}
		
		/* Modulinterne Resource laden */
		if ((rsc = RscReloc(RscData, sizeof(RscData))) == NULL)
			return FALSE;
		
		asciitab = RscGaddr(rsc, R_TREE, MTDIALS_ASCIITAB);
		oldPos = -1;
		insMode = TRUE;
		confdial = RscGaddr(rsc, R_TREE, MTDIALS_CONFDIAL);
		adr = RscGaddr(rsc, R_TREE, MTDIALS_BUTTONS);
		
		mSelect[FALSE] = adr[MSELECT_OFF].ob_spec.bitblk->bi_pdata;
		mSelect[TRUE] =  adr[MSELECT_ON ].ob_spec.bitblk->bi_pdata;
		mKnopf[FALSE] =  adr[MKNOPF_OFF ].ob_spec.bitblk->bi_pdata;
		mKnopf[TRUE] =   adr[MKNOPF_ON  ].ob_spec.bitblk->bi_pdata;
		mCircle[FALSE] = adr[MCIRCLE_OFF].ob_spec.bitblk->bi_pdata;
		mCircle[TRUE] =  adr[MCIRCLE_ON ].ob_spec.bitblk->bi_pdata;
		mPunktEin =      adr[MPUNKT_EIN ].ob_spec.bitblk->bi_pdata;
		fSelect[FALSE] = adr[FSELECT_OFF].ob_spec.bitblk->bi_pdata;
		fSelect[TRUE] =  adr[FSELECT_ON ].ob_spec.bitblk->bi_pdata;
		fKnopf[FALSE] =  adr[FKNOPF_OFF ].ob_spec.bitblk->bi_pdata;
		fKnopf[TRUE] =   adr[FKNOPF_ON  ].ob_spec.bitblk->bi_pdata;
		fCircle[FALSE] = adr[FCIRCLE_OFF].ob_spec.bitblk->bi_pdata;
		fCircle[TRUE] =  adr[FCIRCLE_ON ].ob_spec.bitblk->bi_pdata;
		fPunktEin =      adr[FPUNKT_EIN ].ob_spec.bitblk->bi_pdata;
		
		/* Jetzt nach Cookie suchen und ggf. Einstellungen daraus uebernehmen */
		if (FindCookie(COOKIE_Magic, &a.l))
		{
			/* Cookie gefunden */
			if (a.x.v == 0)        /* Versionsnummer im ersten Wort */
			{
				Config = a.x.s | UseConfig;
				/* Nun auf Gueltigkeit pruefen */
				if ((Config & (UseCenter|UseMouse|UsePos)) == 0)
					Config |= UseCenter;
			}
		}
		mtDials_DialConfig(UseALT, Config & UseALT);
		
		dialDo = InternalDialDo;
	}
	return TRUE;
}
