/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <mtdir.h>
#include <magiccoo.h>
#include <magicsys.h>
#include <xbra.h>
#include <portaes.h>
#include <magicstr.h>

#include <routinec.h>
#include <routines.h>
#include <file_io.h>
#include <atari.h>		

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/


GLOBAL _BOOL mtDir_ExSelector(_VOID)
{
	static _BOOL exselector,first=TRUE;
	
	if (first)
	{
		first=FALSE;
		exselector = FindCookie(COOKIE_FSEL, NULL);
		if (!exselector && CanDoSuper())
			exselector = xbra_test(0x46536d70L, 0x88L);	/* FSmp */
		if (!exselector)
			exselector = gl_ap_version >= Tos104;
	}
	return exselector;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL mtDir_GetDir(_UBYTE *pfad, _UBYTE *name, CONST _UBYTE *msg)
{
	_BOOL b;
	_WORD button;
	
	if (*pfad == '\0')
	{
		F_Path_Get(pfad);
		F_Path_Append(pfad, ALL_FILE_MASK);
	}
	if (msg != NULL && *msg != '\0' && mtDir_ExSelector())
	{
		fsel_exinput(pfad, name, &button, msg);
	} else
	{
		fsel_input(pfad, name, &button);
	}
	b = button == 1;
	if (!b)
		*name = '\0';
	return b;
}

/*** ---------------------------------------------------------------------- ***/

/*** ------------------------------------------------------------------------ ***/
/*	_BOOL mtDir_FselGet(_UBYTE *InPath, _UBYTE *InSel, CONST _UBYTE *ext, CONST _UBYTE *msg, _BOOL pathOnly)*/
/*	Stellt eine Fileselect-Box dar												*/
/*	Parameter:	InPath:	Ordner (bergabe und Rckgabe)							*/
/*				InSel:	Filename (bergabe und Rckgabe)						*/
/*				ext:	Wildcard												*/
/*				msg:	Titel													*/
/*				pathOnly:	Ordner erwnscht									*/
/*	Ergebnis:	keines															*/		
/*** ------------------------------------------------------------------------ ***/
GLOBAL _BOOL mtDir_FselGet(_UBYTE *InPath, _UBYTE *InSel, CONST _UBYTE *ext, CONST _UBYTE *msg, _BOOL pathOnly)
{
	struct {
		_ULONG id;
		_UWORD version;
	} *fsel;
	_UBYTE scrapP[PATH_MAX];
	_UBYTE scrapN[PATH_MAX];
	_BOOL noMCtrl;

	noMCtrl = FindCookie(COOKIE_FSEL, (_LONG *)&fsel) && fsel && fsel->version >= 0x200;
	strBcpy(scrapP, InPath);
	strBcpy(scrapN, InSel);
	F_Path_Append(InPath, ext);
	if (!noMCtrl)
		Wind_Update(BEG_MCTRL);
	if (mtDir_GetDir(InPath, InSel, msg))
	{
		if (!noMCtrl)
			Wind_Update(END_MCTRL);
		F_Path_Dirname(InPath);
		if (pathOnly)
			return TRUE;
		if (*InSel != '\0')
			return TRUE;
	} else
	{
		if (!noMCtrl)
			Wind_Update(END_MCTRL);
	}
	strMcpy(InPath, PATH_MAX, scrapP);
	strMcpy(InSel, PATH_MAX, scrapN);
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL mtDir_GetFile(CONST _UBYTE *wild, CONST _UBYTE *message, _UBYTE *file)
{
	_UBYTE p[PATH_MAX];
	_UBYTE n[PATH_MAX];
	
	strBcpy(p, wild);
	if (mtDir_GetDir(p, n, message))
	{
		strMcpy(file, PATH_MAX, p);
		F_Path_Dirname(file);
		F_Path_Append(file, n);
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/*** ------------------------------------------------------------------------ ***/
/*	_VOID mtDir_SplitPath(CONST _UBYTE *path, _UBYTE *pfad, _UBYTE *name, _UBYTE *suff)	*/
/*								*/
/*	Parameter:	path:	der aufzuteilende Pfad									*/
/*				pfad:	Rckgabe: Pfadanteil									*/
/*				name:	Rckgabe: Filename incl Endung							*/
/*				suff:	Rckgabe: nur die Endung								*/
/*	Ergebnis:	keines															*/		
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID mtDir_SplitPath(CONST _UBYTE *path, _UBYTE *pfad, _UBYTE *name, _UBYTE *suff)
{
	size_t c, d, len, pLen;
	
	*pfad = '\0';
	*name = '\0';
	*suff = '\0';
	len = strLen(path);
	if (len == 0)
		return;
	c = len;
	if (c > 0)
	{
		--c;
		while (c > 0 && path[c] != '.')
			--c;
		if (path[c] == '.')
		{
			++c;
			d = 0;
			while (path[c] != '\0' && d < (PATH_MAX-1))
			{
				suff[d] = path[c];
				++c;
				++d;
			}
			suff[d] = '\0';
		}
	}
	
	c = len;
	if (c > 0)
		--c;
	while (c > 0 && path[c] != '\\' && path[c] != ':')
		--c;
	if (path[c] == '\\' || path[c] == ':')
		++c;
	pLen = c;
	d = 0;
	for (c = c; c < len; c++)
		name[d++] = path[c];
	name[d] = '\0';
	
	if (pLen > 0)
	{
		for (d = 0; d < pLen; d++)
			pfad[d] = path[d];
		pfad[d] = '\0';
	}
}
