/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <mtpopups.h>
#include <mtappl.h>
#include <mtarea.h>
#include <mtmenuba.h>
#include <mtutils.h>
#ifndef __PORTVDI_H__
#include <portvdi.h>
#endif
#include <routines.h>

typedef enum { mouse, pos } Posmode;


#define cStrMax       50
#define cMaxObjects   50

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _WORD DoMenu(OBJECT *t, _WORD start, _BOOL moveable, AREA area)
{
	_WORD x, y, ox, oy, f, o, xx, yy;
	_WORD ob, oldob, scan, minobj;
	_UWORD button, kbshift, event, b;
	AREA fa;
	GRECT fr;
	
	vswr_mode(PrivateWS, MD_XOR);
	vsf_color(PrivateWS, BLACK);
	vsf_perimeter(PrivateWS, FALSE);
	oldob = -1;
	ob = -1;
	ox = -1;
	oy = -1;
	minobj = moveable ? 2 : 1;
	StoreMouse();
	Wind_Update(BEG_MCTRL);
	for (;;)
	{
		event = mtMenubase_DoEvent(&x, &y, &button, &scan);
		if (x != ox || y != oy)
		{
			ob = objc_find(t, start, MAX_DEPTH, x, y);
			ox = x;
			oy = y;
			if (ob != oldob)
			{
				MouseOff();
				mtMenubase_DrawBar(t, oldob);
				oldob = -1;
				if (ob >= minobj && !GetObjState(t, ob, DISABLED) && GetObjFlags(t, ob, SELECTABLE))
				{
					mtMenubase_DrawBar(t, ob);
					oldob = ob;
				}
				MouseOn();
			}
		}
		if (event & MU_KEYBD)
		{
			if (mtMenubase_MenuKeyboard(t, scan, minobj, oldob, &ob))
			{
				MouseOff();
				mtMenubase_DrawBar(t, oldob);
				MouseOn();
				break;
			}
			if (oldob != ob)
			{
				MouseOff();
				mtMenubase_DrawBar(t, oldob);
				mtMenubase_DrawBar(t, ob);
				oldob = ob;
				MouseOn();
			}
		} else if (event & MU_BUTTON)
		{
			if (ob == 1 && moveable)
			{
				MouseHand();
				mtArea_NewArea(&fa);
				f = ObjcFrame(t, start);
				if (f < 0)
					f = -f;
				else
					f = 0;
				for (;;)
				{
					graf_mkstate(&x, &y, &button, &b);
					if (!(button & MOB_LEFT))
						break;
					if (x != ox || y != oy)
					{
						CalcArea(t, start, &fr);
						if (mtArea_SaveArea(PrivateWS, fa, &fr))
						{
							mtArea_MoveArea(PrivateWS, area, x - ox, y - oy, &xx, &yy);
							t[start].ob_x = xx + f;
							t[start].ob_y = yy + f;
							CalcArea(t, start, &fr);
							mtArea_CopyArea(PrivateWS, fa, fr.g_x, fr.g_y);
							ox = x;
							oy = y;
						}
					}
				}
				CalcArea(t, start, &fr);
				mtArea_CopyArea(PrivateWS, fa, fr.g_x, fr.g_y);
				mtArea_DisposeArea(&fa);
				MouseArrow();
			} else
			{
				for (;;)
				{
					graf_mkstate(&x, &y, &button, &kbshift);
					if (!(button & MOB_LEFT))
						break;
					o = objc_find(t, start, MAX_DEPTH, x, y);
					if (o != ob)
						break;
				}
				if (!(button & MOB_LEFT))
				{
					if (ob >= minobj)
					{
						MouseOff();
						mtMenubase_DrawBar(t, ob);
						MouseOn();
					}
					break;
				}
			}
		}
	}
	vswr_mode(PrivateWS, MD_REPLACE);
	vsf_color(PrivateWS, WHITE);
	RestoreMouse();
	Wind_Update(END_MCTRL);
	if (ob >= 0 && GetObjState(t, ob, DISABLED))
		ob = -1;
	return ob - start;
}

/*** ---------------------------------------------------------------------- ***/

#define SetObjc(ob, typ, x, y, w, h, f, s, spec) \
	(ob)->ob_next = -1; \
	(ob)->ob_head = -1; \
	(ob)->ob_tail = -1; \
	(ob)->ob_type = typ; \
	(ob)->ob_x = x; \
	(ob)->ob_y = y; \
	(ob)->ob_width = w; \
	(ob)->ob_height = h; \
	(ob)->ob_flags = f; \
	(ob)->ob_state = s; \
	(ob)->ob_spec.index = (_LONG)(spec)

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID PosMenu(OBJECT *menu, OBJECT *Main, _WORD ob, _WORD maxW, _WORD maxH)
{
	_WORD x, y;
	_UWORD b;
	
	if (ob > 0)
	{
		x = Main[ROOT].ob_x + Main[ob].ob_x + (Main[ob].ob_width / 2);
		y = Main[ROOT].ob_y + Main[ob].ob_y - (CharWidth / 2);
	} else
	{
		graf_mkstate(&x, &y, &b, &b);
	}
	menu[ROOT].ob_x = x;
	menu[ROOT].ob_y = y;
	menu[ROOT].ob_width = maxW;
	menu[ROOT].ob_height = maxH;
	mtMenubase_PlaceOnScreen(menu);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD MakeMenu(OBJECT *tree, OBJECT *menu, OBJECT *Main, TEDINFO *title, _WORD subnum, _WORD type)
{
	_WORD maxW, maxH, num, i, j, ob, len, offset;
	
	len = title->te_txtlen;
	maxW = (len + 4) * CharWidth;
	maxH = CharHeight + 1;
	/*-- Basisobjekt --*/
	SetObjc(menu, G_BOX, 0, 0, 0, 0, NONE, NORMAL, OBSPEC_MAKE('\0', -1, BLACK, BLACK, FALSE, IP_HOLLOW, WHITE));
	num = 1;
	/*-- Titelzeile --*/
	SetObjc(&menu[num], G_BOXTEXT, 0, 0, 0, CharHeight, NONE, NORMAL, title);
	objc_add(menu, ROOT, num);
	num++;
	/*-- Suchposition im Baum festlegen --*/
	if (subnum > 0)
	{
		ob = subnum + 1;
		for (j = 1; j <= (subnum - 3); j++)
			ob = tree[ob].ob_next;
		if (ob < subnum)
			return -1;
		offset = ob - 1;
		j = tree[ob].ob_head;
	} else
	{
		ob = 2;
		offset = 1;
		j = 0;
	}
	/*-- Objekte addieren --*/
	for (;;)
	{
		i = ScanFlags(tree, ObjSearchType, j, type);
		if (i < 0)
			break;
		if (num >= cMaxObjects)
			break;
		if (tree[i].ob_width > maxW)
			maxW = tree[i].ob_width;
		SetObjFlags(tree, i, SELECTABLE);
		SetObjc(&menu[num], tree[i].ob_type, 0, maxH, tree[i].ob_width, CharHeight, tree[i].ob_flags, tree[i].ob_state, tree[i].ob_spec.index);
		objc_add(menu, ROOT, num);
		num++;
		maxH += CharHeight;
		j = i + 1;
		if (i == tree[ob].ob_tail)
			break;
	}
	SetObjFlags(menu, num-1, LASTOB);
	
	mtMenubase_SameLength(menu, num, maxW);
	PosMenu(menu, Main, subnum, maxW, maxH);
	return offset;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD mtPopups_PopupMenu(OBJECT *menu, CONST _UBYTE *title)
{
	_WORD m, s, ret, len, off1, off2;
	GRECT r;
	TEDINFO MainTitle, SubTitle;
	AREA mainArea, subArea;
	OBJECT Main[cMaxObjects];
	OBJECT Sub[cMaxObjects];
	
	if (menu == NULL)
		return -1;
	len = (_WORD)strLen(title);
	MainTitle.te_ptext = NO_CONST(title);
	MainTitle.te_ptmplt = NO_CONST("");
	MainTitle.te_pvalid = NO_CONST("");
	MainTitle.te_font = IBM;
	MainTitle.te_fontid = 0;
	MainTitle.te_just = 0;
	MainTitle.te_color = COLSPEC_MAKE(BLACK, BLACK, TRUE, IP_2PATT, BLACK);
	MainTitle.te_fontsize = 0;
	MainTitle.te_thickness = -1;
	MainTitle.te_txtlen = len + 1;
	MainTitle.te_tmplen = 1;
	off1 = MakeMenu(menu, Main, Main, &MainTitle, 0, G_TITLE);
	CalcArea(Main, ROOT, &r);
	mtArea_NewArea(&mainArea);
	mtArea_SaveArea(PrivateWS, mainArea, &r);
	Objc_draw(Main, ROOT, MAX_DEPTH, &r);
	MouseOn(); /* ??? */
	for (;;)
	{
		m = DoMenu(Main, ROOT, TRUE, mainArea) + off1;
		if (m < 1)
		{
			ret = -1;
			break;
		}
		if (GetObjFlags(menu, m, EXIT))
		{
			ret = m;
			break;
		}
		if (m > 2)
		{
			len = (_WORD)strLen(Main[m - off1].ob_spec.free_string);
			SubTitle = MainTitle;
			SubTitle.te_ptext = Main[m - off1].ob_spec.free_string;
			SubTitle.te_txtlen = len + 1;
			SubTitle.te_just = 2;
			off2 = MakeMenu(menu, Sub, Sub, &SubTitle, m, G_STRING);
			CalcArea(Sub, ROOT, &r);
			mtArea_NewArea(&subArea);
			mtArea_SaveArea(PrivateWS, subArea, &r);
			Objc_draw(Sub, ROOT, MAX_DEPTH, &r);
			s = DoMenu(Sub, ROOT, TRUE, subArea);
			mtArea_RestoreArea(PrivateWS, subArea);
			mtArea_DisposeArea(&subArea);
			if (s > 0)
			{
				ret = s + off2;
				break;
			}
		}
	}
	mtArea_RestoreArea(PrivateWS, mainArea);
	mtArea_DisposeArea(&mainArea);
	return ret;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD DoStringPopup(CONST _UBYTE *string, CONST _UBYTE *title, Posmode posmode, _WORD posX, _WORD posY)
{
	_WORD i, j, m, s, maxW, maxH, num;
	GRECT mr;
	_UBYTE StrArray[cMaxObjects][cStrMax+1];
	OBJECT Main[cMaxObjects];
	TEDINFO MainTitle;
	_BOOL moveable;
	AREA mainArea;
	
	/*-- Basisobjekt --*/
	SetObjc(Main, G_BOX, 0, 0, 0, 0, NONE, SHADOWED, OBSPEC_MAKE('\0', -1, BLACK, BLACK, FALSE, IP_HOLLOW, WHITE));
	num = 1;
	if (title != NULL && *title != '\0')
	{
		/* Titelzeile */
		MainTitle.te_ptext = NO_CONST(title);
		MainTitle.te_ptmplt = NO_CONST("");
		MainTitle.te_pvalid = NO_CONST("");
		MainTitle.te_font = IBM;
		MainTitle.te_fontid = 0;
		MainTitle.te_just = 2;
		MainTitle.te_color = COLSPEC_MAKE(BLACK, BLACK, TRUE, IP_2PATT, BLACK);
		MainTitle.te_fontsize = 0;
		MainTitle.te_thickness = -1;
		MainTitle.te_txtlen = (_WORD)strLen(title) + 1;
		MainTitle.te_tmplen = 1;
		SetObjc(&Main[num], G_BOXTEXT, 0, 0, 0, CharHeight, NONE, NORMAL, &MainTitle);
		objc_add(Main, ROOT, num);
		num++;
		moveable = TRUE;
		maxH = CharHeight + 1;
		maxW = (MainTitle.te_txtlen + 1) * CharWidth;
	} else
	{
		moveable = FALSE;
		maxH = 0;
		maxW = 0;
	}
	i = j = 0;
	for (;;)
	{
		if (num >= cMaxObjects)
			break;
		s = 0;
		while (string[i] != '\0' && string[i] != '|' && s < cStrMax)
		{
			StrArray[j][s] = string[i];
			++i;
			++s;
		}
		StrArray[j][s] = '\0';
		SetObjc(&Main[num], G_STRING, 0, maxH, (s + 2) * CharWidth, CharHeight, SELECTABLE, NORMAL, StrArray[j]);
		objc_add(Main, ROOT, num);
		++num;
		maxW = Max(maxW, (s + 1) * CharWidth);
		++j;
		maxH += CharHeight;
		if (string[i] == '\0')
			break;
		++i;
	}
	mtMenubase_SameLength(Main, num, maxW);
	Main[ROOT].ob_width = maxW;
	Main[ROOT].ob_height = maxH;
	if (posmode == mouse)
	{
		PosMenu(Main, Main, ROOT, maxW, maxH);
	} else
	{
		Main[ROOT].ob_x = posX;
		Main[ROOT].ob_y = posY;
		mtMenubase_PlaceOnScreen(Main);
	}
	CalcArea(Main, ROOT, &mr),
	mtArea_NewArea(&mainArea);
	mtArea_SaveArea(PrivateWS, mainArea, &mr);
	Objc_draw(Main, ROOT, MAX_DEPTH, &mr);
	MouseOn(); /* ??? */
	m = DoMenu(Main, ROOT, moveable, mainArea);
	mtArea_RestoreArea(PrivateWS, mainArea);
	mtArea_DisposeArea(&mainArea);
	if (m > 0)
		return m - 1;
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD mtPopups_StringPopup(CONST _UBYTE *string, CONST _UBYTE *title)
{
	return DoStringPopup(string, title, mouse, 0, 0);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD mtPopups_PosPopup(_WORD x, _WORD y, CONST _UBYTE *string, CONST _UBYTE *title)
{
	return DoStringPopup(string, title, pos, x, y);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD mtPopups_TreeStartPopup(OBJECT *tree, _WORD start, _WORD x, _WORD y, _WORD idx)
{
	_WORD m;
	GRECT mr;
	AREA mainArea;
	
	tree[start].ob_x = 0;
	tree[start].ob_y = 0;
	tree[ROOT].ob_x = x;
	tree[ROOT].ob_y = y - idx * CharHeight;
	mtMenubase_PlaceOnScreen(tree);
	CalcArea(tree, start, &mr);
	mtArea_NewArea(&mainArea);
	mtArea_SaveArea(PrivateWS, mainArea, &mr);
	Objc_draw(tree, start, MAX_DEPTH, &mr);
	MouseOn(); /* ??? */
	m = DoMenu(tree, start, FALSE, mainArea);
	mtArea_RestoreArea(PrivateWS, mainArea);
	mtArea_DisposeArea(&mainArea);
	if (m > 0)
		return m;
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD mtPopups_TreePopup(OBJECT *tree, _WORD x, _WORD y, _WORD idx)
{
	return mtPopups_TreeStartPopup(tree, ROOT, x, y, idx);
}
