/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#define __MAGICTOOL_TEXTFILES_IMPLEMENTATION__
#include <mttextfi.h>
#include <tos.h>
#include <ro_mem.h>
#include <strconv.h>
#include <convert.h>
#include <routinec.h>
#include <file_io.h>
#include <routines.h>

#define NEEDS_MATH 0

                                
struct _textfile {
	_UBYTE name[PATH_MAX];
	Textmode mode;  /* Modus */
	FILENO file;  /* Filehandle */
	_ULONG size; /* Groesse des Puffers (0..xxx) */
	_ULONG count; /* Anzahl der gelesenen Bytes */
	_ULONG fptr; /* Schreib/Lesepos im File */
	_ULONG bptr; /* Schreib/Lesepos im Puffer */
	_UBYTE *buff;   /* Der Puffer persoenlich */
	_BOOL eof;   /* TRUE, wenn Ende der Datei erreicht */
};

#define cMax  0x8000L

#define cr 0x0d
#define lf 0x0a
#define ctrlZ 0x1a
#define Eof 0

typedef _UBYTE *tBuffer;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL F_ERROR mtTextfiles_OpenTextfile(CONST _UBYTE *fname, Textmode modus, _ULONG puffer, TEXTFILENO *text)
{
	TEXTFILENO t;
	FILENO fd;
	
	*text = NULL;
	t = NEW(Textfile, "OpenTextfile");
	if (t == NULL)
		return -39 /* GERR_NOMEM */;
	MemSetZeroStruct(*t);
	strBcpy(t->name, fname);
	t->mode = modus;
	t->bptr = 0;
	t->eof = FALSE;
	t->count = 0;
	t->fptr = 0;
	if (puffer == 0 || puffer > cMax)
		t->size = cMax;
	else
		t->size = puffer;
	t->buff = SYSMALLOC(t->size, MEM_NOERR, "OpenTextFile: buffer");
	if (t->buff == NULL)
	{
		t->size = 512;
		t->buff = MALLOC(t->size, "OpenTextFile: buffer");
		if (t->buff == NULL)
		{
			OFREE(t);
			return -39 /* GERR_NOMEM */;
		}
	}
	switch (modus)
	{
	case READ:
		fd = (FILENO)Fopen(t->name, 0);
		if (!FILENO_OK(fd))
		{
			FREE(t->buff, t->size);
			OFREE(t);
			return TO_HANDLE(fd);
		}
		t->file = fd;
		t->count = Fread(t->file, t->size, t->buff);
		t->eof = t->count == 0;
		break;
	
	case WRITE:
		fd = (FILENO)Fcreate(t->name, 0);
		if (!FILENO_OK(fd))
		{
			FREE(t->buff, t->size);
			OFREE(t);
			return TO_HANDLE(fd);
		}
		t->file = fd;
		break;
	
	case APPEND:
		fd = (FILENO)Fopen(t->name, 1);
		if (!FILENO_OK(fd))
		{
			FREE(t->buff, t->size);
			OFREE(t);
			return TO_HANDLE(fd);
		}
		t->file = fd;
		t->fptr = Fseek(0L, t->file, 2);
		break;
	
	default:
		FREE(t->buff, t->size);
		OFREE(t);
		return -3 /* GERR_INVAL */;
	}
	*text = t;
	return GERR_OK;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_FlushBuffer(TEXTFILENO text)
{
	if (text != NULL)
	{
		if (text->mode != READ && text->bptr > 0)
		{
			text->count = text->bptr;
			text->bptr = 0;
			text->count = Fwrite(text->file, text->count, text->buff);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ReadBuffer(TEXTFILENO text)
{
	if (text != NULL)
	{
		if (text->mode == READ)
		{
			text->fptr = Fseek(0L, text->file, 1);
			text->count = Fread(text->file, text->size, text->buff);
			text->bptr = 0;
			text->eof = text->count == 0;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_CloseTextfile(TEXTFILENO *text)
{
	TEXTFILENO t;
	
	if ((t = *text) != NULL)
	{
		mtTextfiles_FlushBuffer(t);
		Fclose(t->file);
		FREE(t->buff, t->size);
		OFREE(t);
		*text = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_Reset(TEXTFILENO text)
{
	if (text != NULL)
	{
		switch (text->mode)
		{
		case READ:
			text->fptr = Fseek(0L, text->file, 0);
			text->count = Fread(text->file, text->size, text->buff);
			text->bptr = 0;
			text->eof = text->count == 0;
			break;
		
		case WRITE:
			mtTextfiles_FlushBuffer(text);
			text->fptr = Fseek(0L, text->file, 0);
			text->bptr = 0;
			text->count = 0;
			text->eof = FALSE;
			break;
		
		case APPEND:
			mtTextfiles_FlushBuffer(text);
			text->fptr = Fseek(0L, text->file, 2);
			text->bptr = 0;
			text->count = 0;
			text->eof = FALSE;
			break;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _ULONG mtTextfiles_Textpos(TEXTFILENO text)
{
	if (text != NULL)
		return text->fptr + text->bptr;
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_SetTextpos(TEXTFILENO text, _ULONG pos)
{
	if (text != NULL)
	{
		switch (text->mode)
		{
		case READ:
			text->fptr = Fseek(pos, text->file, 0);
			text->count = Fread(text->file, text->size, text->buff);
			text->bptr = 0;
			text->eof = text->count == 0;
			break;
		
		case WRITE:
		case APPEND:
			mtTextfiles_FlushBuffer(text);
			text->fptr = Fseek(pos, text->file, 0);
			text->bptr = 0;
			text->count = 0;
			text->eof = FALSE;
			break;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL mtTextfiles_EndofText(TEXTFILENO text)
{
	if (text != NULL)
		return text->eof;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_WriteChar(TEXTFILENO text, _UBYTE c)
{
	if (text != NULL && text->mode != READ)
	{
		text->buff[text->bptr++] = c;
		if (text->bptr >= text->size)
			mtTextfiles_FlushBuffer(text);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_WriteLine(TEXTFILENO text, CONST _UBYTE *s)
{
	if (text != NULL && text->mode != READ)
	{
		while (*s != '\0')
		{
			text->buff[text->bptr++] = *s++;
			if (text->bptr >= text->size)
				mtTextfiles_FlushBuffer(text);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_WriteLn(TEXTFILENO text)
{
	mtTextfiles_WriteChar(text, cr);
	mtTextfiles_WriteChar(text, lf);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_WriteCard(TEXTFILENO text, _UWORD wert, _UWORD len)
{
	_UBYTE string[256];
	
	mtTextfiles_WriteLine(text, CardToStr(wert, len, string, sizeof(string)));
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_WriteInt(TEXTFILENO text, _WORD wert, _UWORD len)
{
	_UBYTE string[256];
	
	mtTextfiles_WriteLine(text, IntToStr(wert, len, string, sizeof(string)));
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_WriteLongCard(TEXTFILENO text, _ULONG wert, _UWORD len)
{
	_UBYTE string[256];
	
	mtTextfiles_WriteLine(text, CardToStr(wert, len, string, sizeof(string)));
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_WriteLongInt(TEXTFILENO text, _LONG wert, _UWORD len)
{
	_UBYTE string[256];
	
	mtTextfiles_WriteLine(text, IntToStr(wert, len, string, sizeof(string)));
}

/*** ---------------------------------------------------------------------- ***/

#if NEEDS_MATH

GLOBAL _VOID mtTextfiles_WriteReal(TEXTFILENO text, double wert, _UWORD len)
{
	_UBYTE string[256];
	
	ConvFix(wert, len, len / 2, string, sizeof(string));
	mtTextfiles_WriteLine(text, string);
}
#endif

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UBYTE mtTextfiles_ReadChar(TEXTFILENO text)
{
	_UBYTE c;
	
	c = Eof;
	if (text != NULL && text->mode == READ && !text->eof)
	{
		c = text->buff[text->bptr++];
		if (text->bptr == text->count)
			ReadBuffer(text);
	}
	return c;
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID mtTextfiles_Skip(TEXTFILENO text, _UBYTE cSkip)						*/
/*	berspringt das nchste Zeichen falls es gleich zu dem bergebenen ist		*/
/*	Z.B. zum berspringen des LF nach CR										*/
/*	Parameter:	text:	das Textfile											*/
/*				cSkip:	das zu berspringende Zeichen 							*/
/*	Ergebnis:	keines															*/		
/*	15.2.03 von Dimitri Junker	DYJ 5.20zg										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID mtTextfiles_Skip(TEXTFILENO text, _UBYTE cSkip)
{
	if (text != NULL && text->mode == READ && !text->eof)
	{
		if(text->buff[text->bptr]==cSkip)
		{
			text->bptr++;
			if (text->bptr == text->count)
				ReadBuffer(text);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_ReadLine(TEXTFILENO text, _UBYTE *s, size_t strsize)
{
	_UBYTE c;
	
	if (text != NULL && text->mode == READ && strsize > 0)
	{
		--strsize;
		while (strsize && !text->eof)
		{
			c = text->buff[text->bptr];
			if (c == cr || c == lf)
				break;
			*s++ = c;
			strsize--;
			++text->bptr;
			if (text->bptr == text->count)
				ReadBuffer(text);
		}
		*s = '\0';
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_ReadLn(TEXTFILENO text)
{
	_UBYTE ch;
	
	if (text != NULL && text->mode == READ)
	{
		for (;;)
		{
			ch = mtTextfiles_ReadChar(text);
			switch (ch)
			{
			case Eof:
				return;
			case cr:
				if (text->eof)
					return;
				ch = text->buff[text->bptr];
				if (ch == lf)
				{
					++text->bptr;
					if (text->bptr == text->count)
						ReadBuffer(text);
				}
				return;
			case lf:
				return;
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID mtTextfiles_ReadSpec(TEXTFILENO text, CONST _UBYTE *check, _UBYTE *str, size_t strsize)
{
	_UBYTE ch;
	
	if (text != NULL && text->mode == READ && strsize > 0)
	{
		--strsize;
		ch = '\0';
		while (!text->eof)
		{
			ch = mtTextfiles_ReadChar(text);
			if (strChr(check, ch) != NULL)
				break;
		}
		while (strsize)
		{
			*str++ = ch;
			strsize--;
			if (text->eof)
				break;
			ch = mtTextfiles_ReadChar(text);
			if (strChr(check, ch) == NULL)
			{
				if (text->bptr == 0)
				{
					mtTextfiles_SetTextpos(text, mtTextfiles_Textpos(text) - 1);
				} else
				{
					--text->bptr;
				}
			}
		}
		*str = '\0';
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UWORD mtTextfiles_ReadCard(TEXTFILENO text)
{
	_UBYTE string[256];
	
	mtTextfiles_ReadSpec(text, "0123456789", string, sizeof(string));
	return StrToNum(string, 10, NULL, NULL);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD mtTextfiles_ReadInt(TEXTFILENO text)
{
	_UBYTE string[256];
	
	mtTextfiles_ReadSpec(text, "+-0123456789", string, sizeof(string));
	return StrToInt(string, NULL, NULL);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _ULONG mtTextfiles_ReadLongCard(TEXTFILENO text)
{
	_UBYTE string[256];
	
	mtTextfiles_ReadSpec(text, "0123456789", string, sizeof(string));
	return StrToLNum(string, 10, NULL, NULL);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG mtTextfiles_ReadLongInt(TEXTFILENO text)
{
	_UBYTE string[256];
	
	mtTextfiles_ReadSpec(text, "+-0123456789", string, sizeof(string));
	return StrToLInt(string, NULL, NULL);
}

/*** ---------------------------------------------------------------------- ***/

#if NEEDS_MATH
GLOBAL double mtTextfiles_ReadReal(TEXTFILENO text)
{
	_UBYTE string[256];
	
	mtTextfiles_ReadSpec(text, "+-0123456789.Ee", string, sizeof(string));
	return StrToReal(string, NULL, NULL);
}
#endif /* NEEDS_MATH */
