/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 */


#include <mtutils.h>
#include <magicsys.h>
#include <mtxobjec.h>
#include <tos.h>
#include <routinec.h>
#include <routines.h>
#define WAS_DEFAULT        0x1000    /* Flag 12 kennzeichnet Objekte deren DEFAULT Status vorbergehend gelscht werden mute */


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/*----------------------------------------------------------------------*
 *                      Einfache Rechenfunktioen                        *
 *----------------------------------------------------------------------*/

GLOBAL _WORD Min(_WORD c1, _WORD c2)
{
	if (c1 < c2)
		return c1;
	return c2;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG LMin(_LONG c1, _LONG c2)
{
	if (c1 < c2)
		return c1;
	return c2;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD Max(_WORD c1, _WORD c2)
{
	if (c1 > c2)
		return c1;
	return c2;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _LONG LMax(_LONG c1, _LONG c2)
{
	if (c1 > c2)
		return c1;
	return c2;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD Abs(_WORD c1)
{
	if (c1 >= 0)
		return c1;
	return -c1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD GetLowbyte(_WORD value)
{
	return value & 0xff;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD GetThreeState(OBJECT *tree, _WORD entry)
{
	switch (tree[entry].ob_state & (CHECKED | SELECTED))
	{
	case CHECKED | SELECTED:
		return SETNEW;
	case SELECTED:
		return CLEAR;
	case 0:
		return NOCHANGE;
	}
	return -1; /* FEHLER */
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SetThreeState(OBJECT *tree, _WORD entry, _WORD val)
{
	switch (val)
	{
	case NOCHANGE:
		DelObjState(tree, entry, SELECTED|CHECKED);
		break;
	case CLEAR:
		SetObjState(tree, entry, SELECTED);
		DelObjState(tree, entry, CHECKED);
		break;
	case SETNEW:
		SetObjState(tree, entry, SELECTED|CHECKED);
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ObjcString(OBJECT *tree, _WORD entry, _UBYTE *str, size_t strsize)
{
	_WORD ob;
	OBSPEC *spec;
	
	ob = GetLowbyte(GetObtype(tree, entry));
	spec = GetObSpec(tree, entry);
	switch (ob)
	{
	case G_BOXCHAR:
		str[0] = OBSPEC_GET_CHARACTER(*spec);
		if (strsize > 1)
			str[1] = '\0';
		break;
	case G_TEXT:
	case G_BOXTEXT:
	case G_FTEXT:
	case G_FBOXTEXT:
		strMcpy(str, strsize, spec->tedinfo->te_ptext);
		break;
	case G_BUTTON:
	case G_STRING:
	case G_TITLE:
		strMcpy(str, strsize, spec->free_string);
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UBYTE *ObjcStringAdr(OBJECT *tree, _WORD entry)
{
	_WORD ob;
	OBSPEC *spec;
	
	ob = GetLowbyte(GetObtype(tree, entry));
	spec = GetObSpec(tree, entry);
	switch (ob)
	{
	case G_BOXCHAR:
		/* return &spec->obspec.character; */
		return (_UBYTE *)(spec);
	case G_TEXT:
	case G_BOXTEXT:
	case G_FTEXT:
	case G_FBOXTEXT:
		return spec->tedinfo->te_ptext;
	case G_BUTTON:
	case G_STRING:
	case G_TITLE:
		return spec->free_string;
	}
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SetObjcString(OBJECT *tree, _WORD entry, CONST _UBYTE *str)
{
	_WORD ob;
	OBSPEC *spec;
	_WORD i;
	_WORD tl;
	_UBYTE *cp;
	_UBYTE leer[]="";
	if(!str)
		str=leer;

	ob = GetLowbyte(GetObtype(tree, entry));
	spec = GetObSpec(tree, entry);
	switch (ob)
	{
	case G_BOXCHAR:
		OBSPEC_SET_CHARACTER(*spec, str[0]);
		break;
	
	case G_ICON:
	case G_CICON:
		cp = ob == G_CICON ? spec->ciconblk->monoblk.ib_ptext : spec->iconblk->ib_ptext;
		for (i = 0; cp[i] != '\0'; i++)
			cp[i] = ' ';
		for (i = 0; cp[i] != '\0' && str[i] != '\0'; i++)
			cp[i] = str[i];
		break;
	
	case G_TEXT:
	case G_BOXTEXT:
	case G_FTEXT:
	case G_FBOXTEXT:
		tl = spec->tedinfo->te_txtlen - 1;
		cp = spec->tedinfo->te_ptext;
		for (i = 0; i < tl && str[i] != '\0'; i++)
			cp[i] = str[i];
		cp[i] = '\0';
		break;
	
	case G_BUTTON:
	case G_TITLE:
	case G_STRING:
		cp = spec->free_string;
		for (i = 0; cp[i] != '\0'; i++)
			cp[i] = ' ';
		for (i = 0; cp[i] != '\0' && str[i] != '\0'; i++)
			cp[i] = str[i];
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UBYTE *GetObjcStringAdr(OBJECT *tree, _WORD entry)
{
	_WORD ob;
	OBSPEC *spec;
	_UBYTE *cp;
	
	ob = GetLowbyte(GetObtype(tree, entry));
	spec = GetObSpec(tree, entry);
	switch (ob)
	{
	case G_BOXCHAR:
		/* cp = &spec->obspec.character; */
		cp = NULL;
		break;
	
	case G_ICON:
	case G_CICON:
		cp = ob == G_CICON ? spec->ciconblk->monoblk.ib_ptext : spec->iconblk->ib_ptext;
		break;
	
	case G_TEXT:
	case G_BOXTEXT:
	case G_FTEXT:
	case G_FBOXTEXT:
		cp = spec->tedinfo->te_ptext;
		break;
	
	case G_BUTTON:
	case G_TITLE:
	case G_STRING:
		cp = spec->free_string;
		break;
	
	default:
		cp = NULL;
		break;
	}
	return cp;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SetObjcStringAdr(OBJECT *tree, _WORD entry, CONST _UBYTE *str, size_t strsize)
{
	_WORD ob;
	OBSPEC *spec;
	
	ob = GetLowbyte(GetObtype(tree, entry));
	spec = GetObSpec(tree, entry);
	switch (ob)
	{
	case G_TEXT:
	case G_FTEXT:
	case G_BOXTEXT:
	case G_FBOXTEXT:
		spec->tedinfo->te_ptext = NO_CONST(str);
		spec->tedinfo->te_txtlen = (_WORD)strsize;
		break;
	case G_BUTTON:
	case G_TITLE:
	case G_STRING:
		spec->free_string = NO_CONST(str);
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ObjcStrLen(OBJECT *tree, _WORD entry, _WORD *textLen, _WORD *tmpltLen)
{
	_WORD ob;
	OBSPEC *spec;
	
	*textLen = -1;
	*tmpltLen = -1;
	ob = GetLowbyte(GetObtype(tree, entry));
	spec = GetObSpec(tree, entry);
	switch (ob)
	{
	case G_TEXT:
	case G_FTEXT:
	case G_BOXTEXT:
	case G_FBOXTEXT:
		*textLen = spec->tedinfo->te_txtlen - 1;
		*tmpltLen = spec->tedinfo->te_tmplen - 1;
		break;
	case G_BUTTON:
	case G_TITLE:
	case G_STRING:
		*textLen = (_WORD)strLen(spec->free_string);
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD ObjcParent(OBJECT *tree, _WORD entry)
{
	if (entry <= 0)
		return ROOT;
	for (;;)
	{
		if(tree[tree[entry].ob_next].ob_tail==entry)	/*7e*/
			return tree[entry].ob_next;
		entry = tree[entry].ob_next;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ObjcPos(OBJECT *tree, _WORD entry, _WORD *x, _WORD *y)
{
	_WORD i;
	
	i = entry;
	*x = tree[i].ob_x;
	*y = tree[i].ob_y;
	while (i > 0)
	{
		i = ObjcParent(tree, i);
		*x += tree[i].ob_x;
		*y += tree[i].ob_y;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ObjcArea(OBJECT *tree, _WORD entry, GRECT *rect)
{
	ObjcRect(tree, entry, rect);
	if (entry > 0)
		ObjcPos(tree, entry, &rect->g_x, &rect->g_y);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID ObjcRect(OBJECT *tree, _WORD entry, GRECT *rect)
{
	rect->g_x = tree[entry].ob_x;
	rect->g_y = tree[entry].ob_y;
	rect->g_w = tree[entry].ob_width;
	rect->g_h = tree[entry].ob_height;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID SetObjcRect(OBJECT *tree, _WORD entry, CONST GRECT *r)
{
	tree[entry].ob_x = r->g_x;
	tree[entry].ob_y = r->g_y;
	tree[entry].ob_width = r->g_w;
	tree[entry].ob_height = r->g_h;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD ObjcFrame(OBJECT *tree, _WORD entry)
{
	_WORD border;
	_WORD ob;
	OBSPEC *spec;
	
	ob = GetLowbyte(GetObtype(tree, entry));
	border = 0;
	switch (ob)
	{
	case G_BUTTON:
		border = -1;
		if (GetObjFlags(tree, entry, DEFAULT))
			--border;
		if (GetObjFlags(tree, entry, EXIT))
			--border;
		break;
	case G_TEXT:
	case G_FTEXT:
	case G_BOXTEXT:
	case G_FBOXTEXT:
		spec = GetObSpec(tree, entry);
		border = spec->tedinfo->te_thickness;
		if (border > 127)
			border = border - 256;
		break;
	case G_BOX:
	case G_IBOX:
	case G_BOXCHAR:
		spec = GetObSpec(tree, entry);
		border = OBSPEC_GET_FRAMESIZE(*spec);
		if (border > 127)
			border = border - 256;
		break;
	}
	if (GetObjState(tree, entry, OUTLINED) && border > -3)
		border = -3;
	return border;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID CalcArea(OBJECT *tree, _WORD obj, GRECT *r)
{
	_WORD hAdd, vAdd;
	_WORD border;
	_UWORD version;
	
	border = ObjcFrame(tree, obj);
	ObjcArea(tree, obj, r);
	if (border < 0)
	{
		border = -border;
		r->g_x -= border;
		r->g_y -= border;
		r->g_w += border * 2;
		r->g_h += border * 2;
	}
	if (GetObjState(tree, obj, SHADOWED))
	{
		r->g_w += border * 2;
		r->g_h += border * 2;
	}
	/* Jetzt Anpassung an 3D-Buttons von MTOS */
	version = gl_ap_version;
	if (version >= 0x0400 ||
		(version >= 0x0340 && version != 0x0399))
	{
		if (GetObjFlags(tree, obj, FL3DIND))
		{
			if (objc_sysvar(0, AD3DVALUE, 0, 0, &hAdd, &vAdd) > 0)
			{
				r->g_x -= hAdd;
				r->g_y -= vAdd;
				r->g_w += hAdd * 2;
				r->g_h += vAdd * 2;
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

/*7e */
GLOBAL _WORD ScanFlags(OBJECT *tree, _WORD set, _WORD entry, _UWORD flag)
{
	_WORD o, r,lst_o;
	
	if (tree != NULL)
	{
		o = entry;
		do
		{
			if ((set == ObjSearchFlags && flag == HIDETREE) || !GetObjFlags(tree, o, HIDETREE))
			{
				switch (set)
				{
				case ObjSearchType:
					if ((_WORD)flag == GetObtype(tree, o))
						return o;
					break;
				case ObjSearchState:
					if (GetObjState(tree, o, flag))
						return o;
					break;
				case ObjSearchFlags:
					if (GetObjFlags(tree, o, flag))
						return o;
					break;
				}
			}
			if (tree[o].ob_head >= 0)
			{
				r = ScanFlags(tree, set, tree[o].ob_head, flag);
				if (r >= 0)
					return r;
			}
			lst_o=o;
			o = tree[o].ob_next;
		} while (o>-1 && tree[o].ob_tail !=lst_o);
	}
	return -1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL ScanMenu(OBJECT *tree, _WORD scan, _UWORD kbshift, _WORD *title, _WORD *item)
{
	_WORD o, x;
	_UBYTE s[4];
	
	*title = -1;
	*item = -1;
	x = 0;
	if (kbshift & K_CTRL)
		s[x] = '^';
	else if (kbshift & K_ALT)
		s[x] = 0x07;
	else
		s[x] = '[';
	++x;
	s[x] = ucase(CharCode(scan, kbshift));
	if (s[x] == '\0')
		return FALSE;
	++x;
	s[x] = '\0';
	*title = tree[tree[tree[ROOT].ob_head].ob_head].ob_head;
	o = tree[tree[tree[ROOT].ob_head].ob_next].ob_head;
	for (;;)
	{
		*item = tree[o].ob_head;
		for (;;)
		{
			if (tree[*item].ob_type == G_STRING && !GetObjState(tree, *item, DISABLED))
			{
				if (strStr(tree[*item].ob_spec.free_string, s) != NULL)
				{
					menu_tnormal(tree, *title, 0);
					return TRUE;
				}
			}
			if(tree[tree[*item].ob_next].ob_tail==*item)	/*7e*/
				break;
			*item = tree[*item].ob_next;
		}
		if (*title > tree[*title].ob_next)
			break;
		*title = tree[*title].ob_next;
		o = tree[o].ob_next;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _UBYTE CharCode(_WORD scan, _UWORD kbshift)
{
	KEYTAB *tab;
	_UBYTE *ptr;
	
	tab = Keytbl((_VOID *)-1, (_VOID *)-1, (_VOID *)-1);
	if (kbshift & (K_LSHIFT|K_RSHIFT))
		ptr = tab->shift;
	else if (kbshift & K_CAPSLOCK)
		ptr = tab->capslock;
	else
		ptr = tab->unshift;
	return ptr[scan];
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD ScanCode(_UBYTE ch)
{
	KEYTAB *tab;
	_UBYTE *ptr;
	_WORD i;
	
	tab = Keytbl((_VOID *)-1, (_VOID *)-1, (_VOID *)-1);
	ptr = tab->capslock;
	ch = ucase(ch);
	for (i = 1; i <= 53; i++)
		if (ptr[i] == ch)
			return i;
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL DoubleClick(_WORD *value)
{
	_BOOL b;
	
	if (*value & 0x8000)
	{
		b = TRUE;
		*value &= 0x7fff;
	} else
	{
		b = FALSE;
	}
	return b;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Bounce(_VOID)
{
	_WORD x, y;
	_UWORD button, b;
	
	evnt_button(1, 1, 0, &x, &y, &button, &b);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID AbsRect(GRECT *r)
{
	r->g_w = r->g_x + r->g_w - 1;
	r->g_h = r->g_y + r->g_h - 1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID RelRect(GRECT *r)
{
	r->g_w = r->g_w - r->g_x + 1;
	r->g_h = r->g_h - r->g_y + 1;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID RectToVars(CONST GRECT *r, _BOOL abs, _WORD *x, _WORD *y, _WORD *w, _WORD *h)
{
	*x = r->g_x;
	*y = r->g_y;
	*w = r->g_w;
	*h = r->g_h;
	if (abs)
	{
		*w += r->g_x - 1;
		*h += r->g_y - 1;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID VarsToRect(_WORD x, _WORD y, _WORD w, _WORD h, _BOOL abs, GRECT *r)
{
	r->g_x = x;
	r->g_y = y;
	r->g_w = w;
	r->g_h = h;
	if (abs)
	{
		r->g_w += x - 1;
		r->g_h += y - 1;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID AbsRectToVars(CONST GRECT *r, _BOOL abs, _WORD *x, _WORD *y, _WORD *w, _WORD *h)
{
	*x = r->g_x;
	*y = r->g_y;
	*w = r->g_w;
	*h = r->g_h;
	if (!abs)
	{
		*w -= r->g_x - 1;
		*h -= r->g_y - 1;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID AbsVarsToRect(_WORD x, _WORD y, _WORD w, _WORD h, _BOOL abs, GRECT *r)
{
	r->g_x = x;
	r->g_y = y;
	r->g_w = w;
	r->g_h = h;
	if (!abs)
	{
		r->g_w -= x - 1;
		r->g_h -= y - 1;
	}
}

/*** ---------------------------------------------------------------------- ***/

#undef DelObjFlags

GLOBAL _VOID DelObjFlags(OBJECT *tree, _WORD obj, _UWORD flags)
{
	if((flags&HIDETREE)==HIDETREE && (tree[obj].ob_flags &WAS_DEFAULT)==WAS_DEFAULT)	
	{
		tree[obj].ob_flags |= DEFAULT;
		flags|=WAS_DEFAULT;
	}
	tree[obj].ob_flags &= ~flags;
}

/*** ---------------------------------------------------------------------- ***/

#undef DelObjState
GLOBAL _VOID DelObjState(OBJECT *tree, _WORD obj, _UWORD state)
{
	tree[obj].ob_state &= ~state;
}

/*** ---------------------------------------------------------------------- ***/

#undef GetObjFlags
GLOBAL _BOOL GetObjFlags(OBJECT *tree, _WORD obj, _UWORD flags)
{
	return (tree[obj].ob_flags & flags) ? TRUE : FALSE;
}

/*** ---------------------------------------------------------------------- ***/

#undef GetObjState
GLOBAL _BOOL GetObjState(OBJECT *tree, _WORD obj, _UWORD state)
{
	return (tree[obj].ob_state & state) ? TRUE : FALSE;
}

/*** ---------------------------------------------------------------------- ***/

#undef SetObjFlags
GLOBAL _VOID SetObjFlags(OBJECT *tree, _WORD obj, _UWORD flags)
{
	if((flags&HIDETREE)==HIDETREE && (tree[obj].ob_flags &DEFAULT)==DEFAULT)	
	{
		tree[obj].ob_flags &= ~DEFAULT;
		flags|=WAS_DEFAULT;
	}
	tree[obj].ob_flags |= flags;

}

/*** ---------------------------------------------------------------------- ***/

#undef SetObjState
GLOBAL _VOID SetObjState(OBJECT *tree, _WORD obj, _UWORD state)
{
	tree[obj].ob_state |= state;
}

/*** ---------------------------------------------------------------------- ***/

#undef EnableObjFlags
GLOBAL _VOID EnableObjFlags(OBJECT *tree, _WORD obj, _UWORD flags, _BOOL setit)
{
	if (setit)
		SetObjFlags(tree, obj, flags);
	else
		DelObjFlags(tree, obj, flags);
}

/*** ---------------------------------------------------------------------- ***/

#undef EnableObjState
GLOBAL _VOID EnableObjState(OBJECT *tree, _WORD obj, _UWORD state, _BOOL setit)
{
	if (setit)
		SetObjState(tree, obj, state);
	else
		DelObjState(tree, obj, state);
}
