/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <protokol.h>
#include <xacc.h>
#include <vaproto.h>
#include <termproc.h>
#include <wdwmanag.h>

#include <portaes.h>
#include <mtappl.h>
#include <ro_mem.h>

typedef struct _msgHandler MSGHANDLER;
struct _msgHandler {
	HandleProtoMessage func;
	MSGHANDLER *next;
};

GLOBAL _VOID *memPtr;			/* global buffer */

LOCAL MSGHANDLER *handlers;

GL_MEM *GlMem=NULL;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

/* Anpassen, damit man damit demnaechst laengere Msgs verschicken kann */
GLOBAL _VOID Protokoll_Send(_WORD dest, _WORD msg, _WORD d3, _WORD d4, _WORD d5, _WORD d6, _WORD d7)
{
	_WORD message[8];

	message[0] = msg;
	message[1] = gl_apid;
	message[2] = 0;
	message[3] = d3;
	message[4] = d4;
	message[5] = d5;
	message[6] = d6;
	message[7] = d7;
	appl_write(dest, 16, message);
	/* Um es auch mal abzuschicken.. */
	Wind_Update(END_UPDATE);
	Wind_Update(BEG_UPDATE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Protokoll_SendMess(_WORD msg, _WORD catType, _ULONG l, _WORD i2, _WORD i3, _WORD dest)
{
	Protokoll_Send(dest, msg, catType, PROT_NAMEPTR(l), i2, i3);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Protokoll_SendMessInt(_WORD msg, _WORD catType, _WORD i1, _WORD i2, _WORD i3, _WORD i4, _WORD dest)
{
	Protokoll_Send(dest, msg, catType, i1, i2, i3, i4);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Protokoll_SendAdr(_WORD msg, _WORD catType, _WORD i1, _VOID *l, _WORD i2, _WORD dest)
{
	Protokoll_Send(dest, msg, catType, i1, PROT_NAMEPTR(l), i2);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Protokoll_SendPtr(_WORD msg, _VOID *p, _WORD dest)
{
	Protokoll_Send(dest, msg, PROT_NAMEPTR(p), 0, 0, 0);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Protokoll_Broadcast(_VOID *message)
{
	_WORD apMode, apType, apId;
	_UBYTE apName[80];
	_WORD i;

	if (multiTOS || (magIx && magIxVer >= 0x200))
	{
		apMode = APS_FIRST;
		while (appl_search(apMode, apName, &apType, &apId))
		{
			apMode = APS_NEXT;
			if (apType != APK_SYS && apId != gl_apid)
				appl_write(apId, 16, message);
		}
	} else if (magIx && magIxVer < 0x200 && gl_ap_count > 1)
	{
		for (i = 0; i < gl_ap_count; i++)
		{
			apName[0] = '?';
			apName[1] = '\0';
			apName[2] = (_UBYTE) i;
			apName[3] = '\0';
			apId = appl_find(apName);
			if (apId != gl_apid)
				appl_write(apId, 16, message);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID Protokoll_Exit(_VOID)
{
	MSGHANDLER *handler;
	
	if (memPtr != NULL)
	{
		FREE(memPtr, memDescrSize);
		memPtr = NULL;
	}
	while (handlers != NULL)
	{
		handler = handlers->next;
		OFREE(handlers);
		handlers = handler;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Protokoll_Init(_VOID)
{
	if (memPtr == NULL)
	{
		memPtr = SYSMALLOC(memDescrSize, MEM_NOERR | MEM_SHARABLE, "Protokoll_Init");
		if (memPtr == NULL)
			return FALSE;
		InstallTermproc(Protokoll_Exit);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Protokoll_Handle_Message(HandleProtoMessage func)
{
	MSGHANDLER *handler;
	
	if (!Protokoll_Init())
		return;
	handler = NEW(MSGHANDLER, "Protokoll_Handle_Message");
	if (handler != NULL)
	{
		handler->func = func;
		handler->next = handlers;
		handlers = handler;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Protokoll_Check_Message(CONST _VOID *message)
{
	_BOOL done;
	MSGHANDLER *handler;
	
	done = FALSE;
	for (handler = handlers; handler != NULL; handler = handler->next)
		done |= handler->func(message);
	return done;
}

/*** ---------------------------------------------------------------------- ***/

OS_GLOBAL _BOOL Protokoll_Waitfor(_WORD msg, _WORD timeout, _WORD *message)
{
	_UWORD event;
	_WORD dummy;
	_UWORD button, kstate;
	_WORD mox, moy;
	
	for (;;)
	{
		Wind_Update(END_UPDATE);
		event = evnt_multi(MU_MESAG|MU_TIMER,
			0, 0, 0,
			0, 0, 0, 0, 0,
			0, 0, 0, 0, 0,
			message,
			timeout, 0,
			&mox, &moy, &button, &kstate, &dummy, &dummy);
		Wind_Update(BEG_UPDATE);
		if (event & MU_MESAG)
		{
			if (message[0] == msg)
				return TRUE;
			WdwManager_HandleEvent(&event, message, button, kstate, 0, 0, 0, mox, moy, 0);
		}
		if (event & MU_TIMER)
			return FALSE;
	}
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL GL_MEM *findGlMem(_WORD id)											*/
/*	Sucht den Globalen Speicherblock nach ID									*/
/*	Parameter:	id																*/
/*	Ergebnis:	Pointer auf Block oder NULL										*/		
/*	1.6.00 von Dimitri Junker	DYJ 4.17f										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL GL_MEM *findGlMem(_WORD id)
{
	GL_MEM *mem=GlMem;
	while(mem)
	{
		if( mem->id==id)
			break;
		mem=mem->nxt;
	}
	return(mem);
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL GL_MEM *Protokoll_GetGlMem(size_t len)								*/
/*	Holt neuen globalen Speicher												*/
/*	Parameter:	len: Lnge des Speichers										*/
/*	Ergebnis:	Block															*/		
/*	1.6.00 von Dimitri Junker	DYJ 4.17f										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL GL_MEM *Protokoll_GetGlMem(size_t len)
{
	GL_MEM *mem;
	static _WORD id=1;
	mem = SYSMALLOC(len+sizeof(GL_MEM), MEM_NOERR | MEM_SHARABLE, "Protokoll_GetGlMem");
	if (mem)
	{
		while(!id || findGlMem(id))
			id++;
		mem->nxt=GlMem;
		GlMem=mem;
		mem->id=id;
	}
	return(mem);
}
/*** ------------------------------------------------------------------------ ***/
/*	GLOBAL _VOID Protokoll_FreeGlMem(_WORD id,void *mem)						*/
/*	lscht einen Eintrag, identifiziert entweder ber die ID 					*/
/*	oder ber den Satrt des Speichers											*/
/*	Parameter:	id:	ID oder 0(dann identifiziert ber mem)						*/
/*				mem: Beginn des Speichers										*/
/*	Ergebnis:	keines															*/		
/*	1.6.00 von Dimitri Junker	DYJ 4.17f										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID Protokoll_FreeGlMem(_WORD id,void *mem)
{
	GL_MEM *del=NULL,*ptr;
	if(id)
		del=findGlMem(id);
	if(!del && mem)
		del=(GL_MEM*)((_ULONG)mem-OffsetOf(GL_MEM,mem));
	if(GlMem==del)
		GlMem=del->nxt;
	else
	{
		ptr=GlMem;
		while(ptr)
		{
			if( ptr->nxt==del)
				break;
			ptr=ptr->nxt;
		}
		ptr->nxt=del->nxt;
	}
	SFREE(del);
}

