/*
 * Diese Datei ist Teil der Magic-Library,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Magic ist Copyright (c) by Peter Hellinger.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <rectfunc.h>
#include <mtutils.h>

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL rc_intersect(CONST GRECT *r1, GRECT *r2)
{
	_WORD xl, yu, xr, yd;

	xl = Max(r1->g_x, r2->g_x);
	yu = Max(r1->g_y, r2->g_y);
	xr = Min(r1->g_x + r1->g_w, r2->g_x + r2->g_w);
	yd = Min(r1->g_y + r1->g_h, r2->g_y + r2->g_h);

	r2->g_x = xl;
	r2->g_y = yu;
	r2->g_w = xr - xl;
	r2->g_h = yd - yu;

	return (xr > xl) && (yd > yu);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL GRECT rc_clip(CONST GRECT *frame, CONST GRECT *clip)
{
	GRECT r;

	r = *frame;
	if (r.g_x < clip->g_x)
	{
		r.g_w = r.g_w - clip->g_x + r.g_x;
		r.g_x = clip->g_x;
	}
	if (r.g_y < clip->g_y)
	{
		r.g_h = r.g_h - clip->g_y + r.g_y;
		r.g_y = clip->g_y;
	}
	if (r.g_w < 0 || r.g_h < 0)
		r.g_w = r.g_h = 0;

	if (r.g_x + r.g_w > clip->g_x + clip->g_w)
		r.g_w = clip->g_x + clip->g_w - r.g_x;
	if (r.g_y + r.g_h > clip->g_y + clip->g_h)
		r.g_h = clip->g_y + clip->g_h - r.g_y;
	if (r.g_w < 0 || r.g_h < 0)
		r.g_w = r.g_h = 0;
	return r;
}

/*** ---------------------------------------------------------------------- ***/

/* Stellt fest, ob der Punkt mit den Koordinaten x,y im Rechteck r liegt */
GLOBAL _BOOL rc_inside(_WORD x, _WORD y, CONST GRECT *r)
{
	return x >= r->g_x && y >= r->g_y && x < (r->g_x + r->g_w) && y < (r->g_y + r->g_h);
}

/*** ---------------------------------------------------------------------- ***/

/* Stellt fest, ob die Rechtecke r1 und r2 identisch sind */
GLOBAL _BOOL rc_equal(CONST GRECT *r1, CONST GRECT *r2)
{
	return r1->g_x == r2->g_x &&
		   r1->g_y == r2->g_y &&
		   r1->g_w == r2->g_w &&
		   r1->g_h == r2->g_h;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID rc_union(CONST GRECT *p1, GRECT *p2)
{
	GRECT r;

	if (p2->g_w == 0 || p2->g_h == 0)
	{
		*p2 = *p1;
	} else
	{
		r.g_x = Min(p1->g_x, p2->g_x);
		r.g_y = Min(p1->g_y, p2->g_y);
		r.g_w = Max(p1->g_x + p1->g_w, p2->g_x + p2->g_w) - r.g_x;
		r.g_h = Max(p1->g_y + p1->g_h, p2->g_y + p2->g_h) - r.g_y;
		*p2 = r;
	}
}
