#define __WIN_TERMINAL_IMPLEMENTATION__
#include <terminal.h>
#include <tos.h>
#include <portvdi.h>
#include <mtappl.h>
#include <fontsele.h>
#include <convert.h>
#include <ro_mem.h>
#include <wdwmanag.h>
#include <rectfunc.h>
#include <confvars.h>

typedef _UWORD TCHAR;
#define MAXCOLS 200
#define MAXROWS 100

struct _TerminalTyp {
	GRECT work;                         /* Position                     */
	_WORD px;                           /* X-Pixelpos des Cursor        */
	_WORD py;                           /* Y-Pixelpos des Cursors       */
	_WORD pw;                           /* Cursor-Breite                */
	_WORD ph;                           /* Cursor-Hoehe                 */
	_WORD numCols;                      /* Anzahl der Spalten           */
	_WORD numRows;                      /* Anzahl der Zeilen            */
	_WORD zBreite;                      /* Zeichenbreite                */
	_WORD zHoehe;                       /* Zeichenhoehe                 */
	_WORD sx;                           /*  */
	_WORD sy;                           /*  */
	_WORD curRow;
	_WORD curCol;
	_WORD scurRow;
	_WORD scurCol;
	_WORD xoff;                         /* Offset fuer kursive Schrift  */
	_WORD yoff;                         /* fuer tiefgestellte Schrift   */
	_WORD wert1;                        /* Zwischenspeicher             */
	_WORD wert2;                        /* Zwischenspeicher             */
	_WORD vColor;                       /* Vordergrundfarbe (Text)      */
	_WORD hColor;                       /* Hintergrundfarbe             */
	_WORD tab;                          /* Tabweite                     */
	_WORD size;                         /* Fontgroesse                  */
	_WORD full;                         /* Volle Fontgroesse            */
	_WORD half;                         /* Halbe Fontgroesse            */
	_WORD font;                         /* Fontnummer                   */
	TerminalAttribut effect;            /* Texteffekte                  */
	_UWORD zustand;                     /* Zustand des Terminals        */
	
	HVDI vdi;
	HWND wdw;
	TERMINAL *origPtr;
	_WORD blinkTime;
	_WORD blinkCount;
	TCHAR text[MAXROWS][MAXCOLS];
};


/*--------------------------------------------------------------------------*
 *                      Zustandsflags des Terminals                         *
 *--------------------------------------------------------------------------*/

#define cActive       0x0001   /* Terminal aktiv (Vordergrund)             */
#define cErase        0x0002   /* Zeichen vorloeschen?                     */
#define cCursor       0x0004   /* Cursor ein?                              */
#define cBlink        0x0008   /* Cursor blinkend?                         */
#define cWrap         0x0010   /* Word-Wrapping                            */
#define cEscape       0x0020   /* Escape wurde ausgegeben                  */
#define cEscYs        0x0040   /* Escape Y: Spaltenwert wird erwartet      */
#define cEscYz        0x0080   /* Escape Y: Zeilenwert wird erwartet       */
#define cEscb         0x0100   /* Escape b: Vordergrundfarbe wird erwartet */
#define cEscc         0x0200   /* Escape c: Hintergrundfarbe wird erwartet */
#define cBColor       0x0400   /* Hintergrundfarbe beim Schreiben          */
#define cRedraw       0x0800   /* Redrawfeature eingeschaltet              */
#define cFSM          0x1000   /* Gesetzt wenn der Font ein FSM-Font ist   */
#define cCursOn       0x2000   /* Cursor gerade sichtbar?                  */
#define cDirty        0x4000   /* Irgendwas wurde geaendert                */
#define cCurDirty     0x8000   /* Cursor muss neu gezeichnet werden        */

/*** ---------------------------------------------------------------------- ***/

#define cConsolePos  "ConsolePos"
#define cConsoleOpen "ConsoleOpen"

/*** ---------------------------------------------------------------------- ***/

LOCAL HVDI InversHandle;
LOCAL HVDI NormalHandle;

/*** ---------------------------------------------------------------------- ***/

/* Schaltet Cursor ein und aus, XOR-Modus */
LOCAL _VOID TerminalCursor(TERMINAL t, _BOOL clip)
{
	_WORD pxy[4];
	GRECT gr;
	GRECT curs;
	_BOOL doit;
	
	if (t->zustand & cCursor)
	{
		curs.g_x = t->px + t->work.g_x + t->xoff;
		curs.g_y = t->py + t->work.g_y + t->yoff;
		curs.g_w = t->pw;
		curs.g_h = t->ph;
		if (clip)
		{
			MouseOff();
			doit = WdwManager_RectList(t->wdw, TRUE, &gr);
			while (doit)
			{
				if (rc_intersect(&curs, &gr) &&
					rc_intersect(&deskSize, &gr))
				{
					pxy[0] = gr.g_x;
					pxy[1] = gr.g_y;
					pxy[2] = pxy[0] + gr.g_w - 1;
					pxy[3] = pxy[1] + gr.g_h - 1;
					vr_recfl(InversHandle, pxy);
				}
				doit = WdwManager_RectList(t->wdw, FALSE, &gr);
			}
			MouseOn();
		} else
		{
			pxy[0] = curs.g_x;
			pxy[1] = curs.g_y;
			pxy[2] = pxy[0] + t->pw - 1;
			pxy[3] = pxy[1] + t->ph - 1;
			vr_recfl(InversHandle, pxy);
		}
		t->zustand ^= cCursOn;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ClearLine(TERMINAL t, _WORD y)
{
	_WORD x;
	
	for (x = 0; x < t->numCols; x++)
		t->text[y][x] = ' ' | cDirty;
	t->zustand |= cDirty;
}

/*** ---------------------------------------------------------------------- ***/

/* Geht davon aus, dass Maus und Cursor aus sind. Diese Routine wird nur
 * intern aufgerufen, so dass dieses Vorgehen keine Probleme bereitet
 */
LOCAL _VOID TerminalScrollup(TERMINAL t)
{
	_WORD x, y;
	
	for (y = 1; y < t->numRows; y++)
		for (x = 0; x < t->numCols; x++)
			t->text[y-1][x] = t->text[y][x];
	ClearLine(t, t->numRows - 1);
}

/*** ---------------------------------------------------------------------- ***/

/* Handled alles, was mit Escape eingeleitet wurde */

LOCAL _VOID TerminalEmulator(TERMINAL t, _UBYTE ch)
{
	if (t->zustand & cEscYs)
	{
		t->wert1 = ch - 32;
		t->zustand &= ~cEscYs;
		t->zustand |= cEscYz;
		return;
	}
	if (t->zustand & cEscYz)
	{
		t->wert2 = ch - 32;
		if (t->wert2 < 0)
			t->wert2 = 0;
		TerminalGotoXY(t, t->wert2, t->wert1);
		t->zustand &= ~(cEscYz | cEscape);
		return;
	}
	if (t->zustand & cEscb)
	{
		TerminalForegroundColor(t, ch);
		t->zustand &= ~(cEscb | cEscape);
		return;
	}
	if (t->zustand & cEscc)
	{
		TerminalBackgroundColor(t, ch);
		t->zustand &= ~(cEscc | cEscape);
		return;
	}
	t->zustand &= ~cEscape;
	switch (ch)
	{
	case 'A':
		if (t->curRow > 0)
		{
			t->py -= t->zHoehe;
			--t->curRow;
			t->zustand |= cCurDirty;
		}
		break;
	case 'B':
		if (t->curRow < (t->numRows - 1))
		{
			t->py += t->zHoehe;
			++t->curRow;
			t->zustand |= cCurDirty;
		}
		break;
	case 'C':
		if (t->curCol < t->numCols)
		{
			t->px += t->zBreite;
			++t->curCol;
			t->zustand |= cCurDirty;
		}
		break;
	case 'D':
		if (t->curCol > 0)
		{
			t->px -= t->zBreite;
			--t->curCol;
			t->zustand |= cCurDirty;
		}
		break;
	case 'E':
		TerminalClearScreen(t);
		TerminalGotoXY(t, 0, 0);
		break;
	case 'H':
		TerminalGotoXY(t, 0, 0);
		break;
	case 'I':
		if (t->curRow > 0)
		{
			t->py -= t->zHoehe;
			--t->curRow;
		} else
		{
			TerminalInsertLine(t);
		}
		t->zustand |= cCurDirty;
		break;
	case 'J':
		TerminalClearEndOfScreen(t);
		break;
	case 'K':
		TerminalClearEndOfLine(t);
		break;
	case 'L':
		TerminalInsertLine(t);
		break;
	case 'M':
		TerminalDeleteLine(t);
		break;
	case 'Y':
		t->zustand |= cEscYs | cEscape;
		break;
	case 'b':
		t->zustand |= cEscb | cEscape;
		break;
	case 'c':
		t->zustand |= cEscc | cEscape;
		break;
	case 'e':
		TerminalCursorOn(t);
		break;
	case 'f':
		TerminalCursorOff(t);
		break;
	case 'j':
		t->sx = t->px;
		t->sy = t->py;
		t->scurRow = t->curRow;
		t->scurCol = t->curCol;
		break;
	case 'k':
		t->px = t->sx;
		t->py = t->sy;
		t->curRow = t->scurRow;
		t->curCol = t->scurCol;
		t->zustand |= cCurDirty;
		break;
	case 'l':
		TerminalClearLine(t);
		TerminalGotoXY(t, 0, t->curRow);
		break;
	case 'o':
		TerminalClearStartOfLine(t);
		break;
	case 'p':
		TerminalSetAttribut(t, t->effect | invers);
		break;
	case 'q':
		TerminalSetAttribut(t, t->effect & ~invers);
		break;
	case 'v':
		TerminalWrapOn(t);
		break;
	case 'w':
		TerminalWrapOff(t);
		break;
	default:
		if (ch >= 0x80)
			TerminalSetAttribut(t, (TerminalAttribut) (ch & 0x7f));
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID WriteChar(TERMINAL t, _UBYTE c)
{
	if (t->px >= t->work.g_w || t->curCol >= t->numCols)
	{
		if (t->zustand & cWrap)
		{
			t->px = 0;
			t->curCol = 0;
			t->curRow++;
			if (t->curRow >= t->numRows)
			{
				t->curRow = t->numRows - 1;
				TerminalScrollup(t);
			}
			t->py = t->curRow * t->zHoehe;
		} else
		{
			t->curCol = t->numCols - 1;
			t->px = t->work.g_w - t->zBreite;
		}
	}
	t->px += t->zBreite;
	t->text[t->curRow][t->curCol] = c | cDirty;
	if (t->curCol < MAXCOLS)
		++t->curCol;
	t->zustand |= cDirty | cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

/* Bearbeitet alles was kleiner als Blank ist */
LOCAL _VOID TerminalControlChar(TERMINAL t, _UBYTE ch)
{
	_WORD s;

	switch (ch)
	{
	case 0:
		break;
	case 0x07:
		Pling();
		break;
	case 0x08: /* Backspace */
		if (t->curCol > 0)
		{
			t->px -= t->zBreite;
			--t->curCol;
			t->zustand |= cCurDirty;
		}
		break;
	case 0x09: /* Tab */
		s = ((t->curCol + t->tab) / t->tab) * t->tab;
		t->curCol = s;
		t->px = s * t->zBreite;
		t->zustand |= cCurDirty;
		break;
	case 0x0a: /* Linefeed */
		t->curRow++;
		if (t->curRow >= t->numRows)
		{
			t->curRow--;
			TerminalScrollup(t);
		}
		t->py = t->curRow * t->zHoehe;
		t->zustand |= cCurDirty;
		break;
	case 0x0c: /* Home */
		TerminalClearScreen(t);
		break;
	case 0x0d: /* Carriage-Return */
		t->px = 0;
		t->curCol = 0;
		t->zustand |= cCurDirty;
		break;
	case 0x1b: /* Escape */
		t->zustand |= cEscape;
		break;
	default:
		WriteChar(t, ch | 0x40 | (t->effect ^ invers));
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWriteChar(TERMINAL t, _UBYTE c)
{
	if (t->zustand & cEscape)
	{
		TerminalEmulator(t, c);
	} else if (c < 0x20)
	{
		TerminalControlChar(t, c);
	} else
	{
		WriteChar(t, c | t->effect);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWriteString(TERMINAL t, CONST _UBYTE *string)
{
	for (;;)
	{
		if (*string == '\0')
			break;
		switch (*string)
		{
		case 0x1b:
			++string;
			t->zustand |= cEscape;
			while (*string != '\0' && (t->zustand & cEscape))
			{
				TerminalEmulator(t, *string);
				string++;
			}
			break;
		case 0x00: case 0x01: case 0x02: case 0x03: case 0x04: case 0x05: case 0x06: case 0x07:
		case 0x08: case 0x09: case 0x0a: case 0x0b: case 0x0c: case 0x0d: case 0x0e: case 0x0f:
		case 0x10: case 0x11: case 0x12: case 0x13: case 0x14: case 0x15: case 0x16: case 0x17:
		case 0x18: case 0x19: case 0x1a:            case 0x1c: case 0x1d: case 0x1e: case 0x1f:
			TerminalControlChar(t, *string);
			string++;
			break;
		default:
			WriteChar(t, *string | t->effect);
			string++;
			break;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWriteLn(TERMINAL t)
{
	t->px = 0;
	t->py += t->zHoehe;
	t->curCol = 0;
	t->curRow++;
	if (t->curRow >= t->numRows)
	{
		t->curRow--;
		TerminalScrollup(t);
	}
	t->py = t->curRow * t->zHoehe;
	t->zustand |= cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWriteLine(TERMINAL t, CONST _UBYTE *string)
{
	TerminalWriteString(t, string);
	TerminalWriteLn(t);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWriteCard(TERMINAL t, _UWORD wert, _UWORD len)
{
	_UBYTE string[256];

	ConvNum(wert, 10, len, ' ', string, sizeof(string));
	TerminalWriteString(t, string);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWriteInt(TERMINAL t, _WORD wert, _UWORD len)
{
	_UBYTE string[256];

	ConvInt(wert, len, string, sizeof(string));
	TerminalWriteString(t, string);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWriteLongCard(TERMINAL t, _ULONG wert, _UWORD len)
{
	_UBYTE string[256];

	ConvLNum(wert, 10, len, ' ', string, sizeof(string));
	TerminalWriteString(t, string);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWriteLongInt(TERMINAL t, _LONG wert, _UWORD len)
{
	_UBYTE string[256];

	ConvInt(wert, len, string, sizeof(string));
	TerminalWriteString(t, string);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalInsertLine(TERMINAL t)
{
	UNUSED(t); /* TODO */
	t->zustand |= cDirty | cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalDeleteLine(TERMINAL t)
{
	UNUSED(t); /* TODO */
	t->zustand |= cDirty | cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWhereXY(TERMINAL t, _UWORD *x, _UWORD *y)
{
	*x = t->px / t->zBreite;
	*y = t->py / t->zHoehe;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalGotoXY(TERMINAL t, _UWORD x, _UWORD y)
{
	t->px = x * t->zBreite;
	t->py = y * t->zHoehe;
	if (t->px < 0)
		t->px = 0;
	if (t->py < 0)
		t->py = 0;
	if (t->px >= t->work.g_w)
		t->px = t->work.g_w - t->zBreite;
	if (t->py >= t->work.g_h)
		t->py = t->work.g_h - t->zHoehe;
	t->curRow = y;
	t->curCol = x;
	t->zustand |= cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWhereCursor(TERMINAL t, _WORD *x, _WORD *y)
{
	*x = t->px;
	*y = t->py;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalSetCursor(TERMINAL t, _WORD x, _WORD y)
{
	t->px = x;
	t->py = y;
	if (t->px < 0)
		t->px = 0;
	if (t->py < 0)
		t->py = 0;
	if (t->px >= t->work.g_w)
		t->px = t->work.g_w - t->zBreite;
	if (t->py >= t->work.g_h)
		t->py = t->work.g_h - t->zHoehe;
	t->zustand |= cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalCursorOn(TERMINAL t)
{
	t->zustand |= cCursor | cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalCursorOff(TERMINAL t)
{
	if (t->zustand & cCursOn)
	{
		TerminalCursor(t, TRUE);
	}
	t->zustand &= ~cCursor;
}

/*** ---------------------------------------------------------------------- ***/

/* Wird ggw. nicht unterstuetzt */
GLOBAL _VOID TerminalCursorStop(TERMINAL t)
{
	t->zustand &= ~cBlink;
}

/*** ---------------------------------------------------------------------- ***/

/* Wird ggw. nicht unterstuetzt */
GLOBAL _VOID TerminalCursorBlink(TERMINAL t)
{
	t->zustand |= cBlink;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWrapOff(TERMINAL t)
{
	t->zustand &= ~cWrap;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalWrapOn(TERMINAL t)
{
	t->zustand |= cWrap;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalClearLine(TERMINAL t)
{
	/* TODO */
	t->zustand |= cDirty | cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalClearStartOfLine(TERMINAL t)
{
	/* TODO */
	t->zustand |= cDirty | cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalClearEndOfLine(TERMINAL t)
{
	/* TODO */
	t->zustand |= cDirty | cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalClearScreen(TERMINAL t)
{
	_WORD x, y;
	
	for (y = 0; y < MAXROWS; y++)
		for (x = 0; x < MAXCOLS; x++)
			t->text[y][x] = cDirty | ' ';
	t->zustand |= cDirty | cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalClearStartOfScreen(TERMINAL t)
{
	_WORD y, x;
	
	y = t->curRow;
	for (x = 0; x < t->curCol; x++)
		t->text[y][x] = cDirty | ' ';
	for (--y; y >= 0; y--)
		for (x = 0; x < t->numCols; x++)
			t->text[y][x] = cDirty | ' ';
	t->zustand |= cDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalClearEndOfScreen(TERMINAL t)
{
	_WORD y, x;
	
	y = t->curRow;
	for (x = t->curCol; x < t->numCols; x++)
		t->text[y][x] = cDirty | ' ';
	for (++y; y < t->numRows; y++)
		for (x = 0; x < t->numCols; x++)
			t->text[y][x] = cDirty | ' ';
	t->zustand |= cDirty | cCurDirty;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalForegroundColor(TERMINAL t, _WORD color)
{
	t->vColor = color;
	vst_color(t->vdi, color);
	vsf_color(InversHandle, color);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalBackgroundColor(TERMINAL t, _WORD color)
{
	t->hColor = color;
	vsf_color(NormalHandle, color);
	if (color > WHITE)
	{
		t->zustand |= cBColor;
		vswr_mode(t->vdi, MD_XOR);
	} else
	{
		t->zustand &= ~cBColor;
		vswr_mode(t->vdi, MD_REPLACE);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalInversOn(TERMINAL t)
{
	t->effect |= invers;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalInversOff(TERMINAL t)
{
	t->effect &= ~invers;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalSetAttribut(TERMINAL t, TerminalAttribut attrib)
{
	_WORD dummy;
	_UWORD bs;

	if (attrib & (subscript | superscript | italic))
		t->zustand |= cErase;
	if (attrib & italic)
	{
		t->xoff = t->zBreite / 2;
	} else
	{
		t->xoff = 0;
	}
	switch (attrib & (subscript | superscript))
	{
	case subscript | superscript:
		t->yoff = t->zHoehe / 4;
		t->size = vst_point(t->vdi, t->half, &dummy, &dummy, &dummy, &dummy);
		break;
	case subscript:
		t->yoff = t->zHoehe / 2;
		t->size = vst_point(t->vdi, t->half, &dummy, &dummy, &dummy, &dummy);
		break;
	case superscript:
		t->yoff = 0;
		t->size = vst_point(t->vdi, t->half, &dummy, &dummy, &dummy, &dummy);
		break;
	default:
		t->yoff = 0;
		t->size = vst_point(t->vdi, t->full, &dummy, &dummy, &dummy, &dummy);
		break;
	}
	bs = 0;
	if (attrib & fat) bs |= TXT_THICKENED;
	if (attrib & light) bs |= TXT_LIGHT;
	if (attrib & italic) bs |= TXT_SKEWED;
	if (attrib & underline) bs |= TXT_UNDERLINED;
	vst_effects(t->vdi, bs);
	t->effect = attrib;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalSetTabspace(TERMINAL t, _WORD tab)
{
	t->tab = tab;
}

/*--------------------------------------------------------------------------*
 *                              GDOS-Fonthandling                           *
 *--------------------------------------------------------------------------*/

LOCAL _VOID UpdateTerminal(TERMINAL t)
{
	tFontinfo info;

	t->font = FontSelect_FontActive(t->vdi);
	FontSelect_FontInfo(t->vdi, t->font, &info);
	t->size = info.point;
	t->full = info.point;
	if (info.fsm)
		t->zustand |= cFSM, t->half = t->full / 2;
	else
		t->zustand &= ~cFSM, t->half = t->full - 1;
	t->zBreite = info.boxw;
	t->zHoehe = info.boxh;
	t->pw = t->zBreite;
	t->ph = t->zHoehe;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL Terminal_Init(_VOID)
{
	if (InversHandle <= 0)
	{
		InversHandle = OpenWorkstation(Screen, 0, 0, TRUE);
		if (InversHandle < 0)
			return FALSE;
		vswr_mode(InversHandle, MD_XOR);
		vsf_interior(InversHandle, FIS_SOLID);
		vsf_color(InversHandle, BLACK);

		NormalHandle = OpenWorkstation(Screen, 0, 0, TRUE);
		if (NormalHandle < 0)
		{
			CloseWorkstation(&InversHandle);
			return FALSE;
		}
		vswr_mode(NormalHandle, MD_REPLACE);
		vsf_interior(NormalHandle, FIS_SOLID);
		vsf_color(NormalHandle, WHITE);
	}
	return TRUE;
}

/*--------------------------------------------------------------------------*
 *                      Support fuer mehrere Terminals                       *
 *--------------------------------------------------------------------------*/

LOCAL _BOOL termClose(HWND wdw, HVDI vdi, _VOID *special, _BOOL force)
{
	TERMINAL t;
	
	UNUSED(wdw);
	UNUSED(vdi);
	UNUSED(force);
	t = special;
	*(t->origPtr) = NULL;
	OFREE(t);
	ConfVars_SetConfigBool(cConsoleOpen, FALSE);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID termDraw(HWND wdw, HVDI vdiH, _VOID *special, CONST GRECT *rect)
{
	TERMINAL t;
	_WORD x, y;
	_UBYTE text[MAXCOLS + 1];
	
	UNUSED(wdw);
	t = special;
	TerminalClipRect(t, TRUE, rect);
	if (t->zustand & cCursOn)
		TerminalCursor(t, FALSE);
	for (y = 0; y < t->numRows; y++)
	{
		for (x = 0; x < t->numCols; x++)
		{
			text[x] = t->text[y][x] & 0xff;
		}
		text[x] = '\0';
		v_gntext(vdiH, t->work.g_x + t->xoff, t->work.g_y + y * t->zHoehe + t->yoff, text, x);
	}
	TerminalCursor(t, FALSE);
	TerminalClipRect(t, FALSE, rect);
	t->zustand &= ~(cDirty | cCurDirty);
	t->blinkCount = t->blinkTime;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID SendRedraw(HWND win, CONST GRECT *r)
{
	_WORD message[8];

	message[0] = WM_REDRAW;
	message[1] = gl_apid;
	message[2] = 0;
	message[3] = win;
	message[4] = r->g_x;
	message[5] = r->g_y;
	message[6] = r->g_w;
	message[7] = r->g_h;
	appl_write(gl_apid, 16, message);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL termTimer(HWND wdw, HVDI vdiH, _VOID *special)
{
	TERMINAL t;
	GRECT r;
	
	UNUSED(vdiH);
	t = special;
	if (t->zustand & cDirty)
	{
		t->zustand &= ~cDirty;
		WdwManager_GetWdwWork(wdw, &r);
		SendRedraw(wdw, &r);
		WdwManager_HandlePendingEvents();
	}
	if (t->zustand & cBlink)
	{
		if (t->blinkCount < 100)
		{
			t->blinkCount = t->blinkTime;
			t->zustand |= cCurDirty;
		} else
		{
			t->blinkCount -= 100;
		}
	}
	if (t->zustand & cCurDirty)
	{
		TerminalCursor(t, TRUE);
		t->zustand &= ~cCurDirty;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID termGetScroll(HWND wdw, HVDI vdiH, _VOID *special, GRECT *rect)
{
	TERMINAL t;
	GRECT full;
	
	UNUSED(vdiH);
	t = special;
	t->work = *rect;
	WdwManager_GetWdwSize(wdw, &full);	/* Groesse des Fensters abfragen */
	ConfVars_SetConfigRect(cConsolePos, &full);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID termSnap(HWND wdw, HVDI vdiH, _VOID *special, GRECT *rect)
{
	TERMINAL t;
	_WORD cols, rows;
	
	UNUSED(wdw);
	UNUSED(vdiH);
	t = special;
	cols = (rect->g_w + t->zBreite / 2) / t->zBreite;
	rows = (rect->g_h + t->zHoehe / 2) / t->zHoehe;
	rect->g_w = cols * t->zBreite;
	rect->g_h = rows * t->zHoehe;
	t->numRows = rows;
	t->numCols = cols;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL TerminalOpen(TERMINAL *term, CONST _UBYTE *title)
{
	TERMINAL t;
	GRECT r;
	GRECT full;
	_WORD dummy;
	_UWORD comps;
	
	if (!Terminal_Init())
		return FALSE;
	t = NEW(TerminalTyp, "TerminalOpen");
	if (t != NULL)
	{
		MemSetZeroStruct(*t);
		t->curRow = 0;
		t->curCol = 0;
		t->numRows = 25;
		t->numCols = 80;
		t->blinkTime = 600;
		t->vColor = BLACK;
		t->hColor = WHITE;
		t->tab = 8;
		t->zustand = 0;
		t->effect = 0;
		t->xoff = 0;
		t->yoff = 0;
		t->origPtr = term;
		comps = CLOSER | MOVER | SMALLER | SIZER;
		if (title != NULL && *title != '\0')
			comps |= NAME;
		r.g_x = deskSize.g_x + 20;
		r.g_y = deskSize.g_y + 20;
		r.g_w = t->numCols * CharWidth;
		r.g_h = t->numRows * CharHeight;
		Wind_calc(WC_BORDER, comps, &r, &r);
		ConfVars_GetConfDefRect(cConsolePos, &r, &r);
		TerminalGetMaximum(&full);
		if (WdwManager_CreateWindow(&full, &r, comps,
			"", title,
			t, TRUE, TRUE, &t->wdw, &t->vdi))
		{
			t->work = r;
			vst_font(t->vdi, AESFontid);
			FontSelect_LoadFonts(t->vdi);
			FontSelect_FontSize(t->vdi, AESFontsize, &dummy, &dummy, &dummy, &dummy);
			vswr_mode(t->vdi, MD_REPLACE);
			vst_color(t->vdi, BLACK);
			vst_alignment(t->vdi, 0, 5, &dummy, &dummy);
			t->font = FontSelect_FontActive(t->vdi);
			UpdateTerminal(t);
			TerminalClearScreen(t);
			t->zustand |= cActive;
			WdwManager_SetWindowClose(t->wdw, termClose);
			WdwManager_SetWindowDraw(t->wdw, termDraw);
			WdwManager_SetWindowTimer(t->wdw, termTimer);
			WdwManager_SetWindowGetScroll(t->wdw, termGetScroll);
			WdwManager_SetWindowSnap(t->wdw, termSnap);
			WdwManager_ShowWindow(t->wdw);
			TerminalGotoXY(t, 0, 0);
			TerminalWrapOn(t);
			ConfVars_SetConfigBool(cConsoleOpen, TRUE);
		} else
		{
			OFREE(t);
			t = NULL;
		}
	}
	*term = t;
	return t != NULL;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalClose(TERMINAL *term)
{
	if (*term != NULL)
	{
		WdwManager_CloseWindow((*term)->wdw, TRUE);
		*term = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalShow(TERMINAL term)
{
	WdwManager_ShowWindow(term->wdw);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalGetMaximum(GRECT *rect)
{
	*rect = deskSize;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TerminalClipRect(TERMINAL t, _BOOL doit, CONST GRECT *rect)
{
	_WORD pxy[4];

	if (doit)
	{
		pxy[0] = rect->g_x;
		pxy[1] = rect->g_y;
		pxy[2] = rect->g_x + rect->g_w - 1;
		pxy[3] = rect->g_y + rect->g_h - 1;
		vs_clip(t->vdi, TRUE, pxy);
		vs_clip(NormalHandle, TRUE, pxy);
		vs_clip(InversHandle, TRUE, pxy);
	} else if (t->effect & italic)
	{
		pxy[0] = t->work.g_x;
		pxy[1] = t->work.g_y;
		pxy[2] = t->work.g_x + t->work.g_w - 1;
		pxy[3] = t->work.g_y + t->work.g_h - 1;
		vs_clip(t->vdi, TRUE, pxy);
		vs_clip(NormalHandle, TRUE, pxy);
		vs_clip(InversHandle, TRUE, pxy);
	} else
	{
		vs_clip(t->vdi, FALSE, pxy);
		vs_clip(NormalHandle, FALSE, pxy);
		vs_clip(InversHandle, FALSE, pxy);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL TerminalWasOpen(_VOID)
{
	_BOOL bool;
	
	ConfVars_GetConfDefBool(cConsoleOpen, &bool, FALSE);
	return bool;
}
