/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <vaproto.h>
#include <protokol.h>

#include <portaes.h>
#include <mtappl.h>
#include <wdwmanag.h>

#include <routinec.h>
#include <routines.h>
#include <file_io.h>
#include <ro_mem.h>
#include <debug.h>
#include <confvars.h> 		

/* PC-Help Protokoll */
#define AC_HELP    1025         /* message to DA */
#define AC_REPLY   1026         /* message from DA */
#define AC_VERSION 1027         /* Ask DA for version */
#define AC_COPY    1028         /* Ask DA to copy screen to clipboard */

/* AV/VA-Protokoll */

typedef struct str_protost	
{
	_WORD				id;
	_UWORD				bits3;	/* message[3] von VA_PROTOSTATUS oder AV_PROTOKOLL 	*/
	_UWORD				bits4;	/* message[4] von VA_PROTOSTATUS	*/
	struct str_protost	*nxt;
}PROTOST;
PROTOST *AvServer=NULL,*AvClients=NULL;

/* CalcClock */
#define CALCLOCK    100		/* == WM_M_BDROPPED!!! */
#define CALCLOCK_OK 101		/* == SM_M_SPECIAL!!! */

LOCAL CONST _UBYTE *vaprotoName;
LOCAL size_t vaprotoNameSize;
LOCAL VaStartProg startProg;
LOCAL VaOpenWind openWind;
LOCAL VaStart vaStart;
LOCAL VaSendkey vaSendkey;
LOCAL _WORD my_av_status;


#define ADD_AV_STATUS(status) \
	if (func != FUNK_NULL) \
		my_av_status |= status; \
	else \
		my_av_status &= ~(status)
	

#define TRACE_PROT 1

#if TRACE_PROT
#  define TRACE(x) SPEC_DEBUG_OUT(x)
#else
#  define TRACE(x) 
#endif


/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID SendVaProtostatus(_WORD id)
{
	_WORD message[8];
	_VOID *mem;

	if (!Protokoll_Init())
		return;
	mem = memPtr;
	message[0] = VA_PROTOSTATUS;
	message[1] = gl_apid;
	message[2] = 0;
	message[3] = VA_PROT_SENDKEY |VA_PROT_ACCWINDOPEN  |VA_PROT_STARTPROG |
				 VA_PROT_OPENWIND|VA_PROT_ACCWINDCLOSED|VA_PROT_EXIT|
				 VA_PROT_QUOTING |VA_PROT_STARTED;	
	message[4] = 0;
	message[5] = 0;
	*((_UBYTE **)(&message[6])) = mem;
	MemCpy(mem, vaprotoName, vaprotoNameSize);
	appl_write(id, 16, message);
	Wind_Update(END_UPDATE); /* Abschicken */
	Wind_Update(BEG_UPDATE);
}

/*** ---------------------------------------------------------------------- ***/

/* dem Acc mitteilen, ob der Programmstart geklappt hat */
LOCAL _VOID SendProgstart(CONST _WORD *message, _BOOL ok)
{
	Protokoll_SendMess(VA_PROGSTART, ok ? 1 : 0, 0L, 0, 0, message[1]);
}

/*** ---------------------------------------------------------------------- ***/

/* Das ist unbenutzt, ist wohl auch nur in Gemini sinnvoll */
LOCAL _VOID SendVaWindopen(_WORD apId, _BOOL ok)
{
	Protokoll_SendMess(VA_WINDOPEN, ok ? 1 : 0, 0L, 0, 0, apId);
}

/*** ---------------------------------------------------------------------- ***/
LOCAL _WORD FindAVServer(_VOID)
{
	_UBYTE *env;
	_UBYTE avServer[80];
	_WORD apType;
	_WORD id;

	id = -1;
	if ((env = getenv("AVSERVER")) != NULL)
	{
		id = appl_locate(env, FALSE);
		TRACE((EO_DEBUG, "AVServer (env): %s %d", env, id));
	}
	if (id < 0)
	{
		/* Nach Applikation AVSERVER suchen */
		id = appl_locate("AVSERVER", FALSE);
		TRACE((EO_DEBUG, "AVServer (app): %d", id));
	}
	if (id < 0)
	{
		if (multiTOS || (magIx && magIxVer >= 0x200))
		{
			if (appl_search(APS_SHEL, avServer, &apType, &id))
			{
				if (apType != APK_APP)
					id = -1;
			}
		}
		TRACE((EO_DEBUG, "AVServer (shell): %d", id));
	}
	if (id < 0)
	{
		if (gl_apid != 0)
		{
			id = 0;
		}
		TRACE((EO_DEBUG, "AVServer: %d", id));
	}
	if (gl_apid == id)
		id = -1;
	return id;
}

/*** ---------------------------------------------------------------------- ***/
GLOBAL _VOID Protokoll_SendPath(_WORD msg, _WORD apid, CONST _UBYTE *path, CONST _UBYTE *name)
{
	_WORD message[8];
	_UBYTE *mem;
	GL_MEM *gl_mem;
	size_t len=0l;

	if (!Protokoll_Init())
		return;
	message[7] = 0;
	if(path)
		len=PATH_MAX;
	if(name)
		len+=strLen(name)+1;
	switch(msg)
	{	case AV_STARTPROG:	/* identifizierbare Antwort ber id	in 7*/
			message[7] = 1;
		case VA_START:	/* identifizierbare Antwort ber Pointer	*/
			gl_mem=Protokoll_GetGlMem(len);
			if(gl_mem)
			{
				mem=&gl_mem->mem;
				if(message[7])
					message[7]=gl_mem->id;
				break;
			}
			else
				message[7] = 0;
		case AV_PATH_UPDATE:
		case AV_OPENWIND:
		case AC_HELP:
		case AV_COPYFILE:
		default:
			mem = memPtr;
			break;
	}
	message[0] = msg;
	message[1] = gl_apid;
	message[2] = 0;
	*((_UBYTE **)(&message[3])) = mem;
	if (name != NULL)
	{
		strMcpy(mem, PATH_MAX, path);
		strMcpy(mem + PATH_MAX, PATH_MAX, name);
		*((_UBYTE **)(&message[5])) = mem + PATH_MAX;
	} else
	{
		strMcpy(mem, memDescrSize, path);
		message[5] = 0;
		message[6] = 0;
	}
	appl_write(apid, 16, message);
	Wind_Update(END_UPDATE); /* Um es auch mal abzuschicken.. */
	Wind_Update(BEG_UPDATE);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Protokoll_SendPrintFile(CONST _UBYTE *path, CONST _UBYTE *name, _BOOL withEffects)
{
	_WORD message[8];
	_WORD accId;
	_UWORD retry, noMess;
	_WORD dummy;
	_UWORD udummy;
	_UWORD event;

	if (!Protokoll_Init())
		return FALSE;
	/* Erstmal nach Idealist suchen */
	accId = appl_locate("IDEALIST", FALSE);
	if (accId < 0)
	{
		/* Idealist nicht gefunden, IList suchen */
		accId = appl_locate("ILIST", FALSE);
		if (accId < 0)
			return FALSE;
	}

	/* Idealist gefunden, Calclock-Message aufbauen */
	strCpy(memPtr, path);
	F_Path_Append(memPtr, name);

	message[0] = CALCLOCK;
	message[1] = gl_apid;
	message[2] = 0;
	message[3] = (_WORD)strLen(memPtr);
	*((_UBYTE **)(&message[4])) = memPtr;
	message[6] = 1; /* copies */
	message[7] = 0x0001; /* Bit 0 setzen, tenmporaere Datei loeschen */
	if (withEffects)
		message[7] |= 0x0100;
	else
		message[7] |= 0x0200;

	appl_write(accId, 16, message);

	Wind_Update(END_UPDATE); /* Um es auch mal abzuschicken.. */

	/* Wait for Event */

	retry = 0;
	noMess = 0;
	do
	{	/* Eine kleine Schleife mit TimeOut und Retry-Zaehler */
		event = evnt_multi(MU_MESAG | MU_TIMER,
			0, 0, 0,
			0, 0, 0, 0, 0,
			0, 0, 0, 0, 0,
			message,
			250, 0,
			&dummy, &dummy, &udummy, &udummy, &dummy, &dummy);

		if ((event & MU_MESAG) && message[0] != CALCLOCK_OK)
		{
			Wind_Update(BEG_UPDATE);
			/* Standardmessages direkt verarbeiten */
			if (!WdwManager_HandleEvent(&event, message, 0, 0, '\0', '\0', 0, 0, 0, 0))
			{
				/* Nicht verarbeitet? Aufbewahren, wird vielleicht spaeter
				 * mal verarbeitet
				 */
				appl_write(gl_apid, 16, message);
			}
			Wind_Update(END_UPDATE);

			++noMess;
			if (noMess > 20)
				event = MU_TIMER;
		}

		if (event & MU_TIMER)
			++retry;
	} while (message[0] != CALCLOCK_OK && retry <= 3);
	Wind_Update(BEG_UPDATE);

	return message[0] == CALCLOCK_OK;
}

/*** ---------------------------------------------------------------------- ***/

/* Schickt ein AV_PATH_UPDATE an den AV_SERVER */

GLOBAL _VOID Protokoll_SendPathUpdate(CONST _UBYTE *path)
{
	_WORD avId;
	_UBYTE pathQ[PATH_MAX];
	if ((avId = FindAVServerFor(AV_PATH_UPDATE)) >= 0)
	{
		VaProto_Quote(pathQ,PATH_MAX,path,avId);
		Protokoll_SendPath(AV_PATH_UPDATE, avId, pathQ, NULL);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* einer anderen Applikation sagen, das sie ein Fenster oeffnen soll
 * ist fuer den Fall, das CAT oder Eddix schon geoeffnet sind, dann wird
 * diese Message verschickt.
 */

GLOBAL _VOID VaProto_SendOpenwind(_WORD id, CONST _UBYTE *path, CONST _UBYTE *name)
{
	_UBYTE pathQ[PATH_MAX];
	VaProto_Quote(pathQ,PATH_MAX,path,id);
	Protokoll_SendPath(AV_OPENWIND, id, pathQ, name);
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Request Help via PC_HELP Protokoll
 * wird bei markiertem Block und Shift-Help fuer externe Hilfe verwendet
 */
GLOBAL _VOID VaProto_SendHelp(CONST _UBYTE *str, CONST _UBYTE *helpServer)
{
	_WORD accId;

	accId = appl_locate(helpServer, FALSE);
	if (accId >= 0)
	{
		Protokoll_SendPath(AC_HELP, accId, str, NULL);	
	}
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Request Help via VA_START from ST-GUIDE
 * wird fuer die eigene Hilfe-Datei verwendet
 */
/* der Kram mit memPtr wird schon in Protokoll_SendPath gemacht*/
GLOBAL _BOOL Protokoll_SendExtendedHelp(CONST _UBYTE *str, CONST _UBYTE *helpText,_WORD avModus)
{
	_WORD apid;
	_UBYTE mess[memDescrSize - 4 - PATH_MAX];
	_UBYTE *mem;
	_WORD out3, dummy;
	_BOOL ret=FALSE;
	_UBYTE filenameQ[PATH_MAX];
	_WORD quote_id;

	if (!Protokoll_Init())
		return FALSE;
	*mess = '\0';
	if (!F_Path_Absolute(helpText) &&
		strCaseStr(helpText, ".hyp") != NULL)
		strBcat(mess, "*:\\");
	strBcat(mess, helpText);

	if(avModus && (apid = FindAVServerFor(AV_STARTPROG))>= 0) 
	{
		if(avModus==2)
			quote_id=apid;
		else
			quote_id=DONT_QUOTE;
		VaProto_Quote(filenameQ,PATH_MAX,mess,quote_id);
		Protokoll_SendPath(AV_STARTPROG,apid, filenameQ, str);
		ret=TRUE;
	}
	else
	{
		mem = memPtr;
		strMcpy(mem, PATH_MAX, mess);
		strMcpy(mem + PATH_MAX, memDescrSize - PATH_MAX, str);
		if (!appl_xgetinfo(65, &dummy, &dummy, &out3, &dummy) ||
			!(out3 & 0x01) ||
			!shel_help(SHP_HELP, mem, mem + PATH_MAX))
		{
			apid = appl_locate("ST-GUIDE", FALSE);
			if (apid >= 0)	/* sonst: ST-Guide nicht gefunden, interne Hilfe aufrufen */
			{
				VaProto_Quote(filenameQ,PATH_MAX,mess,apid);
				strMcpy(mess, PATH_MAX,filenameQ );
				strBcat(mess, " ");
				strBcat(mess, str);
				Protokoll_SendPath(VA_START, apid, mess, NULL);
				ret=TRUE;
			}
		}
	}
	return ret;
}


/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD parseArgs(_UBYTE *str)
{
	_WORD cnt = 1;
	_UBYTE *c = str;
	_BOOL in_quote = FALSE;

	while (*c != '\0')
	{
		switch (*c)
		{
		case ' ':
			if (!in_quote)
			{
				*c = '\0';
				cnt++;
				while (c[1] == ' ')
					strCpy(c+1, c+2);
			}
			break;
		case '\'':
			strCpy(c, c + 1);
			if (!in_quote)
			{
				in_quote = TRUE;
			} else
			{
				if (*c != '\'')
				{
					in_quote = FALSE;
					*c = '\0';
					if (c[1] != '\0')
						cnt++;
				}
			}
			break;
		}
		c++;
	}
	return cnt;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID startAll(CONST _WORD *message, _UBYTE *cmdLine, VaStart func)
{
	_WORD numArgs;
	_UBYTE *c;
	_UBYTE *p;
	_UBYTE *args;
	
	if (func != FUNK_NULL)
	{
		if (cmdLine != NULL && *cmdLine != '\0')
		{
			args = STRDUP(cmdLine, "startAll");
			if (args != NULL)
			{
				numArgs = parseArgs(args);
				c = args;
				while (numArgs != 0)
				{
					p = c;
					while (*c++ != '\0')
						;
					func(message, p);
					numArgs--;
				}
				SFREE(args);
			}
		} else
		{
			func(message, cmdLine);
		}
	}
}


/*** ---------------------------------------------------------------------- ***/
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL PROTOST **VaProto_GetClient(_WORD id)									*/
/*	gibt einen doppelten Pointer auf PROTOST eines AV-Client zurck.			*/
/*	Parameter:	id des gesuchten Client											*/
/*	Ergebnis:	gesuchter Pointer. Falls nicht vorhanden ist *ptr=NULL			*/		
/*	1.9.00 von Dimitri Junker	DYJ 4.18p										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL PROTOST **VaProto_GetClient(_WORD id)
{
	PROTOST **protost=&AvClients;
	while(*protost)
	{
		if((*protost)->id==id)
			break;
		protost=&((*protost)->nxt);
	}
	return protost;
}

LOCAL _VOID VaProto_SaveClient(_WORD id,_UWORD bitmap)
{
	PROTOST *protost;
	protost=*VaProto_GetClient(id);
	if(!protost)
	{
		protost=MALLOC(sizeof(PROTOST),"AV-Client");
		if(protost)
		{
			protost->nxt=AvClients;
			AvClients=protost;
		}
	}
	if(protost)
	{
		protost->bits3=bitmap;
		protost->id=id;
	}
}


LOCAL _VOID VaProto_SaveServer(_WORD id,_UWORD bitmap1,_UWORD bitmap2)
{
	if(!AvServer)
	{
		AvServer=MALLOC(sizeof(PROTOST),"AV-Server");
		if(AvServer)
			AvServer->nxt=NULL;
	}
	if(AvServer)
	{
		AvServer->bits3=bitmap1;
		AvServer->bits4=bitmap2;
		AvServer->id=id;
	}
}

LOCAL _VOID VaProto_DelClient(_WORD id)
{
	PROTOST **ptr,*del;
	ptr=VaProto_GetClient(id);
	if(*ptr)	
	{
		del=*ptr;
		*ptr=del->nxt;
		FREE(del,sizeof(PROTOST));
	}
}

GLOBAL _BOOL VaProto_Quote(_UBYTE *ziel,size_t maxlen,CONST _UBYTE *quell,_WORD id)
{
	PROTOST *client;
	_BOOL quote=TRUE;
	CONST _UBYTE *ptrq;
	_UBYTE *ptrz;
	size_t max_ap;

	switch(id)
	{
		case DONT_QUOTE:
			quote=FALSE;
		case DO_QUOTE:
			break;
		default:
			if(AvServer && AvServer->id==id)
				quote=VA_SERVER_QUOTING(AvServer->bits3);
			else
			{
				client=*VaProto_GetClient(id);
				if(client)
					quote=VA_ACC_QUOTING(AvServer->bits3);
			}
			break;
	}
	max_ap=maxlen-strLen(quell)-1;
	if(quote && strChr(quell,' '))
	{
		strCpy(ziel,"'");
		ptrq=quell;
		ptrz=ziel+1;
		max_ap-=2;				/* die 2 ' */
		while(*ptrq && max_ap>=0)
		{
			*ptrz=*ptrq;
			if(*ptrq=='\'')
			{
				*(++ptrz)='\'';
				max_ap--;
			}
			ptrz++;
			ptrq++;
		}
		if(max_ap>=0)
			strCpy(ptrz,"'");
		else
			*ptrz='\0';
	}
	else
		strMcpy(ziel,maxlen,quell);
	return(max_ap>=0);
	
}

/*** ------------------------------------------------------------------------ ***/
/*	size_t VaProto_UnQuote(_UBYTE *ziel,size_t maxlen,CONST _UBYTE *quell)		*/
/*	Umkehrfunktion zur vorigen. Ende ist entweder beim ' oder wenn nicht		*/
/*	beim 1. Leerzeichen		 													*/
/*	Parameter:	ziel:	da soll das Ergebnis hin								*/
/*				maxlen:	so lang darf es hchstens werden						*/
/*				quell:	und da ist der ggf. gequotete String					*/
/*	Ergebnis:	keines															*/		
/*	18.11.01 von Dimitri Junker	DYJ 4.30zb										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL size_t VaProto_UnQuote(_UBYTE *ziel,size_t maxlen,CONST _UBYTE *quell)
{
	size_t ret=0l;
	
	if(quell && *quell)
	{
		if(quell[0]=='\'')
		{
			ret++;
			while(quell[ret])
			{
				if(quell[ret]=='\'')
				{
					ret++;
					if(quell[ret]!='\'')
						break;
				}
				*(ziel++)=quell[ret++];
			}
			*(ziel++)='\0';
		}
		else
		{
			_UBYTE *ptr;
			strMcpy(ziel,maxlen,quell);
			ptr=strChr(ziel,' ');
			if(ptr)
				*ptr='\0';
			ret=strLen(ziel);
		}
	}
	else
		*ziel='\0';
	return ret;
}
/*** ---------------------------------------------------------------------- ***/


LOCAL _BOOL VaProto_HandleMessage(CONST _WORD *message)
{
	_UBYTE *pName;
	_UBYTE *pCmd;
	
	switch (message[0])
	{
	case AV_PROTOKOLL:			/* hier behandelt, WatchDog suchen */
		TRACE((EO_TRACE, "AV_PROTOKOLL: $%04x", message[3]));
		SendVaProtostatus(message[1]);                  /* .. und dem Acc sagen, was wir koennen */
		VaProto_SaveClient(message[1],message[3]);
		break;

	case VA_PROTOSTATUS:
		TRACE((EO_TRACE, "VA_PROTOSTATUS: $%04x $%04x $%04x", message[3], message[4], message[5]));
		VaProto_SaveServer(message[1],message[3],message[4]);
		break;
	case VA_PROGSTART:
		Protokoll_FreeGlMem(message[7],NULL);
		break;
	case AV_STARTED:
		Protokoll_FreeGlMem(0,*((_VOID **)(&message[3])));
		break;
	case AC_CLOSE:
		VaProto_Init(vaprotoName,vaprotoNameSize);
		FREE(AvServer,sizeof(PROTOST));
		AvServer=NULL;
		break;
	case AV_EXIT:
		VaProto_DelClient(message[1]);
		break;

	/* AV-Msgs, die direkt hier behandelt werden */
	case AV_SENDKEY:               /* Tastendruck durchreichen */
		TRACE((EO_TRACE, "AV_SENDKEY: $%04x, $%04x", message[4], message[3]));
		if (vaSendkey != FUNK_NULL)
			vaSendkey(message, message[4], message[3]);
		break;

	case AV_ACCWINDOPEN:	/* AccFenster auf -> rein in die Queue */
		TRACE((EO_TRACE, "AV_ACCWINDOPEN: %d, %d", message[1], message[3]));
		WdwManager_NewWindowIsTop(message[3], message[1]);
		break;

	case AV_ACCWINDCLOSED:	/* AccFenster zu -> raus aus der Queue */
		TRACE((EO_TRACE, "AV_ACCWINDCLOSED: %d, %d", message[1], message[3]));
		WdwManager_WindowIsClosed(message[3]);
		break;

	case AV_STARTPROG:		/* Acc sagt: Folgendes Programm starten! */
		pName = *((_UBYTE **)(&message[3]));
		pCmd = *((_UBYTE **)(&message[5]));
		TRACE((EO_TRACE, "AV_STARTPROG: %s, %s", pName ? (const char *)pName : "(null)", pCmd ? (const char *)pCmd : "(null)"));
		if (startProg != FUNK_NULL &&
			startProg(message, pName, pCmd))
		{
			SendProgstart(message, TRUE);
		} else
		{
			SendProgstart(message, FALSE);
		}
		break;

	case AV_OPENWIND:                              /* Woasn doas? */
		pName = *((_UBYTE **)(&message[3]));
		pCmd = *((_UBYTE **)(&message[5]));
		TRACE((EO_TRACE, "AV_OPENWIND: %s, %s", pName ? (const char *)pName : "(null)", pCmd ? (const char *)pCmd : "(null)"));
		if (openWind != FUNK_NULL &&
			openWind(message, pName, pCmd))
		{
			SendVaWindopen(message[1], TRUE);
		} else
		{
			SendVaWindopen(message[1], FALSE);
		}
		break;

	case VA_START:
		/* Unter Mag!X: Neue Dokumente auf CAT gezogen oder so */
		pCmd = *((_UBYTE **)(&message[3]));
		TRACE((EO_TRACE, "VA_START: %d: %s", message[1], pCmd ? (const char *)pCmd : "(null)"));
		startAll(message, pCmd, vaStart);
		Protokoll_SendPtr(AV_STARTED, pCmd, message[1]);
		break;
	
	case VA_DRAGACCWIND:
		pCmd = *((_UBYTE **)(&message[6]));
		TRACE((EO_TRACE, "VA_DRAGACCWIND: %d: %d: %s", message[1], message[3], pCmd ? (const char *)pCmd : "(null)"));
		startAll(message, pCmd, vaStart);
		Protokoll_SendPtr(AV_STARTED, pCmd, message[1]);
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID VaProto_Register_Startprog(VaStartProg func)
{
	startProg = func;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID VaProto_Register_Openwind(VaStartProg func)
{
	openWind = func;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID VaProto_Register_Start(VaStart func)
{
	vaStart = func;
	ADD_AV_STATUS(AV_PROTOKOLL_START|AV_PROTOKOLL_STARTED|AV_PROTOKOLL_QUOTING);	
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID VaProto_Register_Sendkey(VaSendkey func)
{
	vaSendkey = func;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD FindAVServerFor(_WORD msg)
{
	_WORD id=-1;
	if(AvServer)
	{
		id=AvServer->id;
		switch( msg)
		{
			case AV_SENDKEY:
				if((AvServer->bits3&VA_PROT_SENDKEY)==0)
					id=-1;
				break;
			case AV_ASKFILEFONT:
				if((AvServer->bits3&VA_PROT_ASKFILEFONT)==0)
					id=-1;
				break;
			case AV_ASKCONFONT:
				if((AvServer->bits3&VA_PROT_ASKCONFONT)==0)
					id=-1;
				break;
			case AV_OPENCONSOLE:
				if((AvServer->bits3&VA_PROT_OPENCONSOLE)==0)
					id=-1;
				break;
			case AV_ASKOBJECT:
				if((AvServer->bits3&VA_PROT_ASKOBJECT)==0)
					id=-1;
				break;
			case AV_OPENWIND:
				if((AvServer->bits3&VA_PROT_OPENWIND)==0)
					id=-1;
				break;
			case AV_STARTPROG:
				if((AvServer->bits3&VA_PROT_STARTPROG)==0)
					id=-1;
				break;
			case AV_ACCWINDOPEN:
				if((AvServer->bits3&VA_PROT_ACCWINDOPEN)==0)
					id=-1;
				break;
			case AV_ACCWINDCLOSED:
				if((AvServer->bits3&VA_PROT_ACCWINDCLOSED)==0)
					id=-1;
				break;
			case AV_STATUS:
				if((AvServer->bits3&VA_PROT_STATUS)==0)
					id=-1;
				break;
			case AV_GETSTATUS:
				if((AvServer->bits3&VA_PROT_GETSTATUS)==0)
					id=-1;
				break;
			case AV_COPY_DRAGGED:
				if((AvServer->bits3&VA_PROT_COPY_DRAGGED)==0)
					id=-1;
				break;
			case AV_PATH_UPDATE:
				if((AvServer->bits3&VA_PROT_PATH_UPDATE)==0)
					id=-1;
				break;
			case AV_WHAT_IZIT:
				if((AvServer->bits3&VA_PROT_WHAT_IZIT)==0)
					id=-1;
				break;
			case AV_DRAG_ON_WINDOW:
				if((AvServer->bits3&VA_PROT_DRAG_ON_WINDOW)==0)
					id=-1;
				break;
			case AV_EXIT:
				if((AvServer->bits3&VA_PROT_EXIT)==0)
					id=-1;
				break;
			case AV_XWIND:
				if((AvServer->bits3&VA_PROT_XWIND)==0)
					id=-1;
				break;
			case VA_FONTCHANGED:
				if((AvServer->bits3&VA_PROT_FONTCHANGED)==0)
					id=-1;
				break;
			case AV_STARTED:
				if((AvServer->bits3&VA_PROT_STARTED)==0)
					id=-1;
				break;
			case AV_FILEINFO:
				if((AvServer->bits3&VA_PROT_FILEINFO)==0)
					id=-1;
				break;
			case VA_FILECHANGED:
				if((AvServer->bits3&VA_PROT_FILECHANgED)==0)
					id=-1;
				break;
			case AV_COPYFILE:
				if((AvServer->bits4&VA_PROT_COPYFILE)==0)
					id=-1;
				break;
			case VA_FILECOPIED:
				if((AvServer->bits4&VA_PROT_FILECOPIED)==0)
					id=-1;
				break;
			case AV_DELFILE:
				if((AvServer->bits4&VA_PROT_DELFILE)==0)
					id=-1;
				break;
			case VA_FILEDELETED:
				if((AvServer->bits4&VA_PROT_FILEDELETED)==0)
					id=-1;
				break;
			case AV_VIEW:
				if((AvServer->bits4&VA_PROT_VIEW)==0)
					id=-1;
				break;
			case VA_VIEWED:
				if((AvServer->bits4&VA_PROT_VIEWED)==0)
					id=-1;
				break;
			case AV_SETWINDPOS:
				if((AvServer->bits4&VA_PROT_SETWINDPOS)==0)
					id=-1;
				break;
		}
	}
	return id;
}

GLOBAL _VOID VaProto_Init(CONST _UBYTE *appName, size_t len)
{
	_WORD avId;
	if ((avId = FindAVServer()) >= 0 && Protokoll_Init())
	{
		Protokoll_Handle_Message(VaProto_HandleMessage);
		TRACE((EO_TRACE, "sending AV_PROTOKOLL: $%02x", my_av_status));
		MemCpy(memPtr, appName, len);
		Protokoll_SendMessInt(AV_PROTOKOLL, my_av_status, 0, 0, PROT_NAMEPTR(memPtr), avId);
		vaprotoName = appName;
		vaprotoNameSize = len;
	}
}



/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID VaProto_SendTerminate(_BOOL terminate)
{
PROTOST *del;	
	struct {
		_UWORD messId;
		_WORD apId;
		_WORD over;
		_WORD r[5];
	} m;

	if (terminate)
	{
		m.messId = AV_EXIT;
		m.apId   = gl_apid;
		m.over   = 0;
		m.r[0] = gl_apid;
		m.r[1] = 0;
		m.r[2] = 0;
		m.r[3] = 0;
		m.r[4] = 0;
		Protokoll_Broadcast((_WORD *)&m);
		if(AvServer)
			FREE(AvServer,sizeof(PROTOST));
		while(AvClients)
		{
			del=AvClients;
			AvClients=AvClients->nxt;
			FREE(del,sizeof(PROTOST));
		}
		
	}
}
