/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


/*****************************************************************************
 *		XFSL.C
 *****************************************************************************/

#include <xfsl.h>
#include <fslx.h>
#include <xufsl.h>
#include <fontsele.h>
#include <mtappl.h>
#include <magiccoo.h>
#include <ro_mem.h>
#include <routinec.h>
#include <debug.h>
#include <termproc.h>
#include <routines.h>

#define FONTSEL_MAGIX 1
#define FONTSEL_XFSL  2
#define FONTSEL_UFSL  3

typedef struct _fontsel {
	CONST _UBYTE *title;
	FONT_ACCEPT_FUNC func;
	_WORD fontId;
	_WORD fontSize;
	tFontinfo info;
	HWND for_window;
	
	_VOID *data;
	_ULONG flags;
	_WORD num_fonts;
	_WORD type;
	_BOOL ok;
	_BOOL in_window;
	HVDI vdi_handle;
	HWND win_handle;

	FNT_DIALOG *fnt_dialog;

	xFSL *xfsl;
	xFSL_PAR xpar;
	PFONTINFO pfont;

	xUFSL *ufsl;
	FW_INFO fw_info;
	_WORD skew;
	
	struct _fontsel *next;
} FONTSEL;

LOCAL FONTSEL *fontsel_list;

#define FS_SAMPLE_TEXT "The quick brown fox jumps over the lazy dog"

#define FONT_SELECT   0x7a19

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID fontsel_apply(FONTSEL *fs)
{
	FontSelect_FontInfo(fs->vdi_handle, fs->fontId, &fs->info);
	fs->func(fs->for_window, fs->fontId, fs->fontSize, fs->info.mono);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID fontsel_end(FONTSEL *fs)
{
	FONTSEL **l;
	
	switch (fs->type)
	{
	case FONTSEL_MAGIX:
		if (fs->fnt_dialog != NULL)
		{
			if (fs->win_handle > 0)
				fnts_close(fs->fnt_dialog);
			fnts_delete(fs->fnt_dialog, fs->vdi_handle);
		}
		break;
	case FONTSEL_XFSL:
		fs->xfsl->xfsl_exit(fs->win_handle);
		break;
	}
	if (fs->ok)
	{
		fontsel_apply(fs);
	}
	if (fs->vdi_handle > 0)
	{
		CloseWorkstation(&fs->vdi_handle);
	}
	if (fs->in_window)
		WdwManager_DisableMenu(NULL, FALSE);
	l = &fontsel_list;
	while (*l)
	{
		if (*l == fs)
		{
			*l = fs->next;
			OFREE(fs);
			break;
		}
		l = &(*l)->next;
	}
	if (fontsel_list == NULL)
		fnts_func = FUNK_NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID fontwinsel_exit(_VOID)
{
	while (fontsel_list != NULL)
	{
		fontsel_list->ok = FALSE;
		fontsel_end(fontsel_list);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID xfsl_error(_WORD e)
{
	switch (e)
	{
	case xFS_PARERROR:
		form_alert(1, "[2][Fehler in den bergebenen|Parametern!][Abbruch]");
		break;
	case xFS_LOADERROR:
		form_alert(1, "[2][Fehler beim Nachladen|von XFSL.OVL!][Abbruch]");
		break;
	case xFS_RES_ERROR:
		form_alert(1, "[2][Auflsung zu gering!][Abbruch]");
		break;
	case xFS_NO_HANDLE:
		form_alert(1, "[2][Kein Handle frei!][Abbruch]");
		break;
	case xFS_NO_WINDOW:
		form_alert(1, "[2][Kein Fenster frei!][Abbruch]");
		break;
	case xFS_NO_FONTS:
		form_alert(1, "[2][Keine Fonts geladen!][Abbruch]");
		break;
	case xFS_NO_FONTSIZE:
		form_alert(1, "[2][Fontgre konnte nicht|identifiziert werden!][Abbruch]");
		break;
	case xFS_ACTIVE:
		form_alert(1, "[2][Fontselektor ist bereits aktiv!][Abbruch]");
		break;
	case xFS_ERROR:
		form_alert(1, "[2][Speichermangel!][Abbruch]");
		break;
	default:
		if (e < 0)
			form_error(-31-e); /* Gemdos-Fehlermeldung */
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL hndl_fnts_event(_BOOL get_event, _VOID *e)
{
	FONTSEL *fs;
	_LONG id, pt, ratio;
	_WORD exit, check_boxes;
	
	for (fs = fontsel_list; fs != NULL; fs = fs->next)
	{
		if (fs->type == FONTSEL_MAGIX && fs->fnt_dialog != NULL && !get_event)
		{
			if (!fnts_evnt(fs->fnt_dialog, e, &exit, &check_boxes, &id, &pt, &ratio))
			{
				((EVNT *)e)->mwhich &= ~MU_KEYBD;
				if (exit == FNTS_OPT && (fs->flags & FS_ENABLE_HELP))
				{
					/* help_show("Fontselector"); TODO */
					return FALSE; /* weiter machen */
				}
				fs->fontId = (_WORD)id;
				fs->fontSize = fix31_to_point(pt);
				fs->ok = exit == FNTS_OK || exit == FNTS_SET;
				fontsel_end(fs);
				return TRUE;
			}
			if (fs->in_window)
			{
				HWND top;
				_WORD dummy;
				
				wind_get(0, WF_TOP, &top, &dummy, &dummy, &dummy);
				if (fs->win_handle == top)
					((EVNT *)e)->mwhich &= ~MU_KEYBD;
			}
		} else if (fs->type == FONTSEL_XFSL)
		{
			if (fs->in_window)
			{
				if (get_event)
					exit = fs->xfsl->xfsl_event(fs->win_handle, e);
				else
					exit = xFS_EVENT;
			} else
			{
				exit = fs->xfsl->xfsl_event(0, e);
			}
			if (exit == xFS_OK)
			{
				fs->ok = TRUE;
				fs->fontId = fs->xpar.font->fontid;
				fs->fontSize = fs->xpar.font->fontsize.size;
			}
			if (exit == xFS_HELP && (fs->flags & FS_ENABLE_HELP))
			{
				_UBYTE *p;
				
				p = strChr(fs->xpar.helpinfo, ',');
				if (p != NULL)
				{
					*p = '\0';
					/* help_show_with(fs->xpar.helpinfo, p + 1); TODO */
					*p = ',';
				} else
				{
					/* help_show_with(fs->xpar.helpinfo, NULL); TODO */
				}
				return FALSE; /* weiter machen */
			}
			if (exit != xFS_EVENT)
			{
				fontsel_end(fs);
				return TRUE;
			}
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID _CDECL ufsl_message(_UWORD event, _WORD message[])
{
	if (event & MU_MESAG)
	{
		WdwManager_HandleEvent(&event, message, 0, 0, 0, 0, 0, 0, 0, 0);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _WORD _CDECL ufsl_help(_VOID)
{
	/* help_show("Fontselector"); TODO */
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID call_ufsl(FONTSEL *sel, _ULONG ufsl_flags)
{
	if (sel->ufsl->id == xUFS_ID)
	{
		_WORD color;
		_WORD style;
		
		color = BLACK;
		style = 0;
		sel->win_handle = sel->ufsl->font_selexinput(sel->vdi_handle, ufsl_flags,
			NULL, sel->title, 
			AESFontid, AESFontid,
			&sel->fontId, &sel->fontSize, &sel->fw_info,
			&style, &color, &sel->skew);
	} else if (sel->ufsl->version >= 0x97)
	{
		sel->win_handle = sel->ufsl->fontsel_exinput(sel->vdi_handle,
			ufsl_flags & UFS_PROP_FONTS ? 0 : 1,
			sel->title, 
			&sel->fontId, &sel->fontSize);
	} else
	{
		sel->win_handle = sel->ufsl->fontsel_input(sel->vdi_handle,
			sel->num_fonts,
			ufsl_flags & UFS_PROP_FONTS ? 0 : 1,
			sel->title,
			&sel->fontId, &sel->fontSize);
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL CONST _UBYTE *fsel_help_text(_VOID)
{
#if TODO
	switch (rsc_objc_language)
	{
		case EXTOB_LANG_GERMAN: return "Hilfe";
	}
#endif
	return "Help";
}

/******************************************************************************/
/* W_Font_Select()                                                            */
/* -------------------------------------------------------------------------- */
/* Entwickler......: Th.Otto                                                  */
/* 1.Version.......:                                                          */
/* letzte Aenderung:                                                          */
/* -------------------------------------------------------------------------- */
/* Dialog zur Auswahl eines Fonts durch den Benutzer                          */
/*                                                                            */
/* In der Atari-Version suchen wir zuerst nach einer XFSL Schnittstelle, dann */
/* nach UFSL, dann nach der Environment-Variablen FONTSELECT, und schliesslich*/
/* nach den fnts_xx-Routinen von MagC                                         */
/* -------------------------------------------------------------------------- */
/* Parameter:                                                                 */
/* -> desc         = Beschreibung des Fonts                                   */
/* -> title        = Ueberschrift fuer Dialog                                 */
/* -> func         = Callback-Funktion die bei Auswahl aufgerufen wird        */
/* -> data         = Daten fuer Callback-Funktion                             */
/* -------------------------------------------------------------------------- */
/* Rueckgabe:                                                                 */
/*					 keine													  */
/******************************************************************************/

GLOBAL _VOID W_Font_Select(HVDI vdi_handle, _WORD fontId, _WORD fontSize, CONST _UBYTE *title, _ULONG flags, FONT_ACCEPT_FUNC func, HWND win)
{
	FONTSEL *sel;
	_WORD handle;
	_WORD dummy, out1;
	_UBYTE *path;
	_BOOL found;
	
	found = FALSE;
	if ((sel = NEW(FONTSEL, "W_Font_Select")) != NULL)
	{
		InstallTermproc(fontwinsel_exit);
		MemSetZeroStruct(*sel);
		sel->fontId = fontId;
		sel->fontSize = fontSize;
		/*
		 * Wir machen eine neue Workstation auf,
		 * weil wir nicht wissen was der Fontselector
		 * damit so alles anstellt
		 */
		handle = OpenWorkstation(Screen, 0, 0, TRUE);
		if (handle > 0)
		{
			sel->title = title;
			sel->func = func;
			sel->for_window = win;
			sel->flags = flags;
			sel->vdi_handle = handle;
			sel->next = fontsel_list;
			fontsel_list = sel;

			sel->num_fonts = FontSelect_LoadFonts(handle);
			FontSelect_FontSelect(sel->vdi_handle, sel->fontId);
			FontSelect_FontSize(sel->vdi_handle, sel->fontSize, &dummy, &dummy, &dummy, &dummy);
			FontSelect_FontInfo(sel->vdi_handle, fontId, &sel->info);
			/* xFSL-Schnittstelle vorhanden? */
			if (FindCookie(COOKIE_XFSL, (_LONG *)&sel->xfsl) && sel->xfsl && sel->xfsl->xfsl_init)
			{
				HWND top_window, new_top_window;

				SPEC_DEBUG_OUT((EO_DEBUG, "using XFSL"));
				found = TRUE;
				sel->type = FONTSEL_XFSL;
				sel->xpar.par_size = (_WORD)sizeof(sel->xpar);
				sel->xpar.pfi_size = (_WORD)sizeof(sel->pfont);
				sel->xpar.control = CC_WINDOW | CC_NOWIDTH | CC_NOALIGN;
				sel->xpar.headline = sel->title;
				sel->xpar.example = FS_SAMPLE_TEXT;
				sel->xpar.font = &sel->pfont;
				sel->xpar.fontflags = 0;
				if (!(flags & FS_EFFECTS))
					sel->xpar.control |= CC_NOATTR | CC_NOROTATION;
				if (!(flags & FS_COLOR))
					sel->xpar.control |= CC_NOCOLOR;
				if (flags & FS_TTONLY)
					sel->xpar.fontflags |= FF_TTF;
				else
					sel->xpar.fontflags |= FF_VECTOR | FF_BITMAP;
				if (flags & FS_MONOSPACED_ONLY)
					sel->xpar.fontflags |= FF_MONOSPACED;
				else
					sel->xpar.fontflags |= FF_MONOSPACED | FF_PROPORTIONAL;
				if (flags & FS_ENABLE_HELP)
				{
					sel->xpar.helptext = fsel_help_text();
				}
				sel->pfont.fontid = sel->fontId;
				sel->pfont.fontname = sel->info.name;
				sel->pfont.fontsize.size = sel->fontSize;
				sel->pfont.fontcol = BLACK;
				sel->pfont.validtype = V_CHAR_IND;
				wind_get(0, WF_TOP, &top_window, &dummy, &dummy, &dummy);
				SPEC_DEBUG_OUT((EO_DEBUG, "top_window: %d", top_window));
				sel->win_handle = sel->xfsl->xfsl_init(sel->vdi_handle, &sel->xpar);
				if (sel->win_handle == xFS_NO_WINDOW)
				{
					sel->xpar.control &= ~CC_WINDOW;
					sel->win_handle = sel->xfsl->xfsl_init(sel->vdi_handle, &sel->xpar);
				}
				wind_get(0, WF_TOP, &new_top_window, &dummy, &dummy, &dummy);
				SPEC_DEBUG_OUT((EO_DEBUG, "new_top_window: %d", new_top_window));
				if (sel->win_handle >= 0)
				{
					/*
					 * HuGo gibt 0 zurueck, auch wenn der Dialog im Fenster liegt
					 */
					if (sel->win_handle == 0 && top_window != new_top_window)
						sel->in_window = TRUE;
					else if (sel->win_handle > 0)
						sel->in_window = TRUE;
					if (sel->in_window)
					{
						WdwManager_DisableMenu(NULL, TRUE);
						fnts_func = hndl_fnts_event;
					} else
					{
						while (!hndl_fnts_event(TRUE, NULL))
							;
					}
					return;
				} else
				{
					xfsl_error(sel->win_handle);
				}
			} else

			/* UFSL-Schnittstelle vorhanden? */
			if (FindCookie(COOKIE_UFSL, (_LONG *)&sel->ufsl) &&
				sel->ufsl != NULL &&
				sel->ufsl->fontsel_init != FUNK_NULL &&
				sel->ufsl->fontsel_init() == 0)
			{
				_ULONG ufsl_flags;
				
				SPEC_DEBUG_OUT((EO_DEBUG, "using UFSL"));
				found = TRUE;
				sel->type = FONTSEL_UFSL;
				ufsl_flags = UFS_SIZE_CHANGE | UFS_GDOS_FONTS | UFS_WIND_DIALOG;
				if (flags & FS_COLOR)
					ufsl_flags |= UFS_COLOR_CHANGE;
				if (flags & FS_EFFECTS)
					ufsl_flags |= UFS_ATTR_CHANGE;
				if (!(flags & FS_MONOSPACED_ONLY))
					ufsl_flags |= UFS_PROP_FONTS;
				if (flags & FS_TTONLY)
					ufsl_flags |= UFS_VECTOR_FONTS;
				if (flags & FS_ENABLE_HELP)
				{
					TEDINFO *ted = sel->ufsl->helpbutton->ob_spec.tedinfo;
					
					strMcpy(ted->te_ptext, ted->te_txtlen, fsel_help_text());
					sel->ufsl->helpfunc = ufsl_help;
				} else
				{
					sel->ufsl->helpfunc = FUNK_NULL;
				}
				sel->fw_info = 0;
				if (sel->ufsl->version >= 0x91)
					sel->ufsl->examplestr = FS_SAMPLE_TEXT;
				sel->ufsl->dialtyp = UFS_WINDOW;
				if (sel->ufsl->version >= 0x96)
					sel->ufsl->msgfunc = ufsl_message;
				call_ufsl(sel, ufsl_flags);
				if (sel->win_handle == UFS_NO_WINDOW)
				{
					sel->ufsl->dialtyp = UFS_DIALOG;
					ufsl_flags &= ~UFS_WIND_DIALOG;
					call_ufsl(sel, ufsl_flags);
				}
				if (sel->win_handle >= 0)
				{
					sel->ok = sel->win_handle == UFS_OK;
				} else
				{
					xfsl_error(sel->win_handle);
				}
			} else
			
			/* Environment FONTSELECT gesetzt? */
			if ((path = getenv("FONTSELECT")) != NULL)
			{
				_WORD apid;
				
				SPEC_DEBUG_OUT((EO_DEBUG, "using FONTSELECT"));
				found = TRUE;
				apid = appl_locate(path, TRUE);
				if (apid >= 0)
				{
					_WORD message[8];
					
					message[0] = FONT_SELECT;
					message[1] = gl_apid;
					message[2] = 0;
					message[3] = win;
					message[4] = sel->fontId;
					message[5] = sel->fontSize;
					message[6] = BLACK;
					message[7] = 0;
					appl_write(apid, 16, message);
				}
			} else
			
			/* MagiC fnts_xx vorhanden? */
			if (appl_xgetinfo(APG_RESVD, &out1, &dummy, &dummy, &dummy) && (out1 & 0x07) == 0x07)
			{
				_WORD button_flags;
				_WORD font_flags;
				CONST _UBYTE *opt_button;
				
				SPEC_DEBUG_OUT((EO_DEBUG, "using MagC Font selector"));
				found = TRUE;
				sel->type = FONTSEL_MAGIX;
				font_flags = 0;
				button_flags = 0;
				if (flags & FS_APPLY)
					button_flags |= FNTS_BSET;
				if (flags & FS_TTONLY)
					font_flags |= FNTS_OUTL;
				else
					font_flags |= FNTS_OUTL | FNTS_BTMP;
				if (flags & FS_MONOSPACED_ONLY)
					font_flags |= FNTS_MONO;
				else
					font_flags |= FNTS_MONO | FNTS_PROP;
				if (flags & FS_ENABLE_HELP)
					opt_button = fsel_help_text();
				else
					opt_button = NULL;
				sel->fnt_dialog = fnts_create(sel->vdi_handle, sel->num_fonts, font_flags, FNTS_3D, FS_SAMPLE_TEXT, opt_button);
				if (sel->fnt_dialog != NULL)
				{
					if ((sel->win_handle = fnts_open(sel->fnt_dialog, button_flags,
						-1, -1, sel->fontId, point_to_fix31(sel->fontSize), point_to_fix31(1))) > 0)
					{
						sel->in_window = TRUE;
						fnts_func = hndl_fnts_event;
						WdwManager_DisableMenu(NULL, TRUE);
						return;
					} else
					{
						xfsl_error(xFS_NO_WINDOW);
					}
				} else
				{
					xfsl_error(xFS_NO_HANDLE);
				}
			} else
			
			{
				SPEC_DEBUG_OUT((EO_DEBUG, "No Fontselector available"));
			}
		} else
		{
			xfsl_error(xFS_NO_HANDLE);
		}
		fontsel_end(sel);
	}
	if (!found)
		FontSelect_DoSelectFont(vdi_handle, (flags & FS_MONOSPACED_ONLY) != 0, fontId, fontSize, title, func, win);
}
