/*
 * Diese Datei ist Teil des Megamax Modula-2 Entwicklungssytems,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Megamax Modula-2 ist Copyright (c) by Thomas Tempelmann.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <convert.h>
#include <magicstr.h>
#include <routinec.h>
#include <routines.h>
#include <math.h>



static _VOID getch(GetProc get, GetInfo *info)
{
	get(info);
	info->ch = ucase(info->ch);
}


static _ULONG StrToLC(GetProc get, GetInfo *info, _BOOL *valid)
{
	_ULONG v = 0;
	_UWORD c;
	
	*valid = FALSE;
	if (info->ch == '+')
		getch(get, info);
	switch (info->ch)
	{
	case '%':
		getch(get, info);
		c = info->ch;
		if (c == '0' || c == '1')
		{
			*valid = TRUE;
			v = c - '0';
			for (;;)
			{
				getch(get, info);
				c = info->ch;
				if (c != '0' && c != '1')
					break;
				if (v >= 0x80000000L)
					*valid = FALSE; /* overflow */
				v <<= 1;
				v |= c - '0';
			}
		}
		break;
		
	case '$':
		getch(get, info);
		c = info->ch;
		if ((c >= '0' && c <= '9') || (c >= 'A' && c <= 'F'))
		{
			*valid = TRUE;
			v = c >= 'A' ? (c - 'A' + 10) : (c - '0');
			for (;;)
			{
				getch(get, info);
				c = info->ch;
				if (!((c >= '0' && c <= '9') || (c >= 'A' && c <= 'F')))
					break;
				if (v >= 0x1000000L)
					*valid = FALSE; /* overflow */
				v <<= 4;
				v |= c >= 'A' ? (c - 'A' + 10) : (c - '0');
			}
		}
		break;
	
	default:
		c = info->ch;
		if (c >= '0' && c <= '9')
		{
			_ULONG v2;
			
			*valid = TRUE;
			v = c - '0';
			for (;;)
			{
				getch(get, info);
				c = info->ch;
				if (c < '0' || c > '9')
					break;
				v2 = v * 10 + (c - '0');
				if ((v2 / 10) != v)
					*valid = FALSE; /* overflow */
				v = v2;
			}
		}
		break;
	}
	return v;
}


static _VOID skip(GetProc get, GetInfo *info)
{
	do
	{
		getch(get, info);
	} while (info->ch == ' ' || info->ch == '\t');
}


_ULONG ConvToLCard(GetProc get, GetInfo *info, _BOOL *valid)
{
	skip(get, info);
	return StrToLC(get, info, valid);
}


_UWORD ConvToCard(GetProc get, GetInfo *info, _BOOL *valid)
{
	_ULONG v = ConvToLCard(get, info, valid);
	if (v >= 0x10000L)
		*valid = FALSE;
	return (_UWORD)v;
}


_LONG ConvToLInt(GetProc get, GetInfo *info, _BOOL *valid)
{
	_LONG v;
	_BOOL sign = FALSE;
	
	skip(get, info);
	if (info->ch == '-')
	{
		getch(get, info);
		sign = TRUE;
	}
	v = StrToLC(get, info, valid);
	if (sign)
		v = -v;
	return v;
}


_WORD ConvToInt(GetProc get, GetInfo *info, _BOOL *valid)
{
	_LONG v = ConvToLInt(get, info, valid);
	
	if (((_LONG)(_WORD)v) != v)
		*valid = FALSE;
	return (_WORD)v;
}


LOCAL _ULONG ConvLN(GetProc get, GetInfo *info, _ULONG base, _BOOL *valid)
{
	_ULONG v = 0, v2;
	_UWORD c;
	
	c = info->ch;
	if ((c >= '0' && c <= '9') || c >= 'A')
	{
		c = c >= 'A' ? (c - 'A' + 10) : (c - '0');
		if (c < base)
		{
			*valid = TRUE;
			v = c;
			for (;;)
			{
				getch(get, info);
				c = info->ch;
				if (!((c >= '0' && c <= '9') || c >= 'A'))
					break;
				c = c >= 'A' ? (c - 'A' + 10) : (c - '0');
				if (c >= base)
					break;
				v2 = v * base + c;
				if ((v2 / base) != v)
					*valid = FALSE; /* overflow */
				v = v2;
			}
		}
	}
	return v;
}

_ULONG ConvToLNum(GetProc get, GetInfo *info, _UWORD base, _BOOL *valid)
{
	_ULONG v = 0;
	
	*valid = FALSE;
	skip(get, info);
	if (base > 1 && base <= 36)
		v = ConvLN(get, info, base, valid);
	return v;
}

_UWORD ConvToNum(GetProc get, GetInfo *info, _UWORD base, _BOOL *valid)
{
	_ULONG v = ConvToLNum(get, info, base, valid);
	if (v >= 0x10000l)
		*valid = FALSE;
	return (_UWORD)v;
}


static _VOID reverse(_UBYTE *str, _UWORD space, _UWORD len, _UBYTE fill, size_t strsize)
{
	_UBYTE *end;
	_UBYTE c;
	
	--strsize;
	if (space <= strsize)
		while (len < space)
			str[len++] = fill;
	end = str + len;
	*end = '\0';
	while (str < end)
	{
		c = *str;
		*str++ = *--end;
		*end = c;
	}
}


static _UWORD LCToStr(_ULONG v, _UBYTE *str, size_t strsize)
{
	_UWORD len = 0;
	
	--strsize;
	while (strsize)
	{
		*str++ = (_UBYTE)((v % 10) + '0');
		v /= 10;
		len++;
		strsize--;
		if (v == 0)
			break;
	}
	return len;
}


_VOID ConvCard(_ULONG v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	_UWORD len;
	
	len = LCToStr(v, str, strsize);
	reverse(str, spc, len, ' ', strsize);
}


static _UWORD LIToStr(_LONG v, _UBYTE *str, size_t strsize)
{
	_UWORD len;
	
	if (v < 0)
	{
		len = LCToStr(-v, str, strsize - 1);
		str[len++] = '-';
	} else
	{
		len = LCToStr(v, str, strsize);
	}
	return len;
}


_VOID ConvInt(_LONG v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	_UWORD len;
	
	len = LIToStr(v, str, strsize);
	reverse(str, spc, len, ' ', strsize);
}


static _UWORD LHToStr(_ULONG v, _UBYTE *str, size_t strsize)
{
	_UWORD len = 0;
	
	--strsize;
	while (strsize)
	{
		*str++ = "01234567890ABCDEF"[v & 0x0f];
		v >>= 4;
		len++;
		strsize--;
		if (v == 0)
			break;
	}
	return len;
}


_VOID ConvLHex(_ULONG v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	_UWORD len;
	
	len = LHToStr(v, str + 1, strsize - 1);
	reverse(str + 1, spc, len, '0', strsize - 1);
	str[0] = '$';
}


_VOID ConvHex(_UWORD v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	ConvLHex(v, spc, str, strsize);
}


static _UWORD LBToStr(_ULONG v, _UBYTE *str, size_t strsize)
{
	_UWORD len = 0;
	
	--strsize;
	while (strsize)
	{
		*str++ = "01"[v & 0x01];
		v >>= 1;
		len++;
		strsize--;
		if (v == 0)
			break;
	}
	return len;
}


_VOID ConvLBin(_ULONG v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	_UWORD len;
	
	len = LBToStr(v, str + 1, strsize - 1);
	reverse(str + 1, spc, len, '0', strsize - 1);
	str[0] = '%';
}


_VOID ConvBin(_UWORD v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	ConvLBin(v, spc, str, strsize);
}


_VOID ConvLNum(_ULONG v, _UWORD base, _UWORD spc, _UBYTE fill, _UBYTE *str, size_t strsize)
{
	_UWORD len = 0;
	
	--strsize;
	while (strsize)
	{
		*str++ = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"[v % base];
		v /= base;
		len++;
		strsize--;
		if (v == 0)
			break;
	}
	reverse(str - len, spc, len, fill, strsize + len + 1);
}


_VOID ConvNum(_UWORD v, _UWORD base, _UWORD spc, _UBYTE fill, _UBYTE *str, size_t strsize)
{
	ConvLNum(v, base, spc, fill, str, strsize);
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID ConvFix(double v, _UWORD spc, _UWORD n, _UBYTE *str, size_t strsize)	*/
/*	Double->String																*/
/*	Parameter:	wie bei den anderen															*/
/*	Ergebnis:	keines															*/		
/*	24.4.00 von Dimitri Junker	DYJ 4.15d										*/
/*** ------------------------------------------------------------------------ ***/
_VOID ConvFix(double v, _UWORD spc, _UWORD n, _UBYTE *str, size_t strsize)
{
	_WORD nmax;
	nmax=(_WORD)strsize-(int)log10(v)-2;
	if(nmax<-1 || spc>strsize)
	{
		*str='\0';
		return;
	}
	if(nmax==-1)
		n=0;
	else
	{
		if(n>nmax)
			n=nmax;
	}
	sprintf(str,"%*.*f",spc,n,v);
}

