/*
 * Diese Datei ist Teil des Megamax Modula-2 Entwicklungssytems,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Megamax Modula-2 ist Copyright (c) by Thomas Tempelmann.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#ifndef __CONVERT_H__
#define __CONVERT_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif


/*
 * Grundmodul zur Umwandlung von CARDINAL/INTEGER/REAL-Werten in
 * die Textdarstellung und umgekehrt.
 */


typedef struct {
	_UBYTE ch;
	_VOID *userPtr;
} GetInfo;

typedef _VOID (*GetProc)(GetInfo *info);


_ULONG ConvToLCard(GetProc get, GetInfo *info, _BOOL *valid);
_UWORD ConvToCard(GetProc get, GetInfo *info, _BOOL *valid);
_LONG ConvToLInt(GetProc get, GetInfo *info, _BOOL *valid);
_WORD ConvToInt(GetProc get, GetInfo *info, _BOOL *valid);

  /*
     'get' bestimmt eine Zeichenhol-Funktion, die jeweils ein Zeichen
     in 'info.ch' ablegt. Wenn keine Zeichen mehr vorhanden sind, sollte
     das Zeichen 0C darin abgelegt werden. 'info.userPtr' wird von den
     'ConvTo...'-Funktionen nicht benoetigt, sie kann zum Datenaustausch
     zw. dem Aufrufer und der Funktion 'get' dienen. Das 'info'-Record
     wird nicht veraendert.
     
     Wenn 'valid' nach dem Aufruf FALSE ist, bedeutet das, dass der
     erwartete Wert nicht ermittelt werden konnte.
     
     Fuer die vier ConvTo...-Funktionen gilt:
     
     - fuehrende Blanks und TABs werden ignoriert, innerhalb der Zahl
       duerfen dann keine Leerzeichen mehr vorkommen.
     - ein eventuell vorangestelltes '+' wird ignoriert.
     - Praefixe steuern die Interpretation der folgenden Zeichen:
       '$' fuer Hexadezimal, '%' fuer Binaer, kein Praefix bedeutet
       dezimal.
     - Bei ConvToInt bzw. ConvToLInt muss das '-' zuerst stehen, erst
       dann darf der Praefix folgen
       
     Beispiele: ConvToCard ('  123 12') = 123
                ConvToLInt ('-$80') = -128L
  */


_ULONG ConvToLNum(GetProc get, GetInfo *info, _UWORD base, _BOOL *valid);
_UWORD ConvToNum(GetProc get, GetInfo *info, _UWORD base, _BOOL *valid);
  /*
     Diese beiden Funktionen werten eine Zahl zur Basis 'base' aus.
     Dabei muss 'base' zwischen einschliesslich 2 und 36 liegen (sonst
     liefert valid FALSE).
     Die Ziffern 10 bis 36 muessen dabei durch die Buchstaben 'A' bis 'Z'
     dargestellt sein. Die Buchstaben duerfen gross und klein sein.
     Die Zahlen duerfen kein Vorzeichen und keinen Praefix ('$' oder '%')
     haben.
     Fuehrende Blanks und TABs werden ignoriert.
  */

double ConvToReal(GetProc get, GetInfo *info, _BOOL *valid);

  /*
    ConvToReal liefert den Wert einer reellen Zahl. Dabei darf entweder der
    Nachkommateil (einschliesslich '.') oder der Vorkommateil fehlen. Ebenfalls
    braucht kein Exponent angegeben werden. Ein eventueller Skalierungsfaktor
    wird durch ein vorangestelltes 'E' erkannt und darf im Bereich von -1232
    bis +1232 liegen.
    
    Es gilt:
    
    - fuehrende Blanks und TABs werden ignoriert, innerhalb der Zahl duerfen
      dann keine Leerzeichen mehr vorkommen.
    - ein eventuell vorangestelltes '+' wird ignoriert.
    
    z.B. : ConvToReal ('123E12') = 1.23 E14
           ConvToReal ('.012E-23') = 1.2 E-25
  */



/* ======================================================================= *
 * Es folgen Funktionen zur Umwandlung von Zahlen in Strings. Dabei gilt   *
 * allgemein:                                                              *
 * - 'spc' bestimmt die Mindestlaenge des resultierenden Strings 'str'.     *
 * - Wenn das Ergebnis (mit Beruecksichtigung von 'spc' !) nicht in den     *
 *   String passt, tritt ein Laufzeitfehler ein ('string overflow').        *
 * ======================================================================= */


_VOID ConvCard(_ULONG v, _UWORD spc, _UBYTE *str, size_t strsize);
_VOID ConvInt(_LONG v, _UWORD spc, _UBYTE *str, size_t strsize);

  /*
    Die Conv...-Funktionen wandeln eine Zahl in einen String um. Dabei steht
    ein eventuelles Minuszeichen direkt vor der ersten Ziffer. Falls die Zahl
    weniger Stellen (inklusive Vorzeichen) als in 'spc' angegeben haben sollte,
    so werden entsprechend viele Leerzeichen vorangestellt.
    Die Hoechstlaengen der Ergebnisse sind, sofern 'spc' nicht groesser angegeben
    wird: 5 bei 'CardToStr', 6 bei 'IntToStr', 10 bei 'LCardToStr', 11 bei
    'LIntToStr'.
    
    z.B. : ConvCard (123,10) = '       123'
           ConvInt (-2345,2) = '-2345'
  */

_VOID ConvLHex(_ULONG v, _UWORD spc, _UBYTE *str, size_t strsize);
_VOID ConvHex(_UWORD v, _UWORD spc, _UBYTE *str, size_t strsize);
_VOID ConvLBin(_ULONG v, _UWORD spc, _UBYTE *str, size_t strsize);
_VOID ConvBin(_UWORD v, _UWORD spc, _UBYTE *str, size_t strsize);

  /*
    Die Hex- und Bin-Funktionen erzeugen, wie der Name sagt, Strings in
    hexadezimaler bzw. binaerer Repraesentation. Es werden nur positive Zahlen
    erzeugt. Zu Beachten ist, dass den Zahlen dabei ein '$' bzw. '%' vorange-
    stellt wird und dieses Zeichen auch mit zur Mindestlaengenangabe 'spc' zaehlt.
    Ggf. ist das vorangestellte Zeichen mit der Funktion 'Strings.Delete' zu
    entfernen. Fuehrende Nullen werden nicht unterdrueckt. Die Hoechstlaengen der
    Ergebnisse sind, sofern 'spc' nicht groesser angegeben wird: 5 bei 'HexToStr',
    9 bei 'LHexToStr', 17 bei 'BinToStr', 33 bei 'LBinToStr'.
    
    z.B.: ConvHex (128,4)  = '$080'
          ConvHex (128,0)  = '$80'
          ConvLHex (-1L,0) = '$FFFFFFFF' (8 Ziffern)
          ConvBin (100,9)  = '%01100100'
          ConvLBin (-1L,1) = '%11111111111111111111111111111111' (32 Ziffern)
  */

_VOID ConvLNum(_ULONG v, _UWORD base, _UWORD spc, _UBYTE fill, _UBYTE *str, size_t strsize);
_VOID ConvNum(_UWORD v, _UWORD base, _UWORD spc, _UBYTE fill, _UBYTE *str, size_t strsize);
  /*
     Diese beiden Funktionen erzeugen eine Zahl zur Basis 'base'.
     Wenn 'base' < 2 oder > 36, dann tritt ein Laufzeitfehler auf
     (Range Error).
     'fill' bestimmt das Zeichen, mit dem vorn aufgefuellt wird, wenn
     'space' groesser als die eigentliche Zahl ist.
  */


_VOID ConvFloat(double v, _UWORD spc, _UWORD n, _UBYTE *str, size_t strsize);

  /*
    Durch diese Funktion wird die Zahl 'v' in wissenschaftlicher Notation dar-
    gestellt. Es wird eine Vorkommastelle und 'n' Nachkommastellen, gefolgt von
    'E', dem Vorzeichen des Skalierungsfaktors und dem vierstelligen Wert.
    Ggf. wird der String links mit Leerzeichen bis zur Laenge 'spc' aufgefuellt.
    
    z.B. : RealFloatToStr (0.0123,12,1) = '   1.2E-0002'
           RealFloatToStr (9.9,0,10) = '9.9000000000E+0000'
  */

_VOID ConvFix(double v, _UWORD spc, _UWORD n, _UBYTE *str, size_t strsize);

  /*
    Diese Funktion stellt 'v' in Festpunktnotation dar, wobei entsprechend viele
    Vorkommastellen, jedoch nur genau 'n' Nachkommastellen gebracht werden.
    Wenn der String kuerzer als 'spc' sein sollte, so werden entsprechend viele
    Leerzeichen vorangestellt.
    
    z.B. : RealFixToStr (0.0123,12,1) = '         0.0'
           RealFixToStr (9.9,0,10) = '9.9000000000'
  */

_VOID ConvReal(double v, _UWORD spc, _UWORD n, _UBYTE *str, size_t strsize);

  /*
    Diese Prozedur verwendet je nach Lage des Wertes 'v' innerhalb der Werte von
    FixToFloatMin und FixToFloatMax das Festkomma- oder Fliesskomma-Format zur
    Darstellung von 'v'.
  */
  
_VOID ConvEng(double v, _UWORD spc, _UWORD n, _UBYTE *str, size_t strsize);

  /*
    Durch diese Funktion wird die Zahl 'v' in Ingenieur-Notation dargestellt.
    Es werden bis zu drei Vorkommastellen und 'n' Nachkommastellen, gefolgt
    von 'E', dem Vorzeichen des Skalierungsfaktors und dem vierstelligen Wert,
    der immer ein Vielfaches von Drei ist. Ggf. wird der String links mit Leer-
    zeichen bis zur Laenge 'spc' aufgefuellt.
    
    z.B. : RealEngToStr (0.0123,12,1) = '  12.3E-0003'
           RealEngToStr (9.9,0,10) = '9.9000000000E+0000'
  */

#endif /* __CONVERT_H__ */
