/*
 * Diese Datei ist Teil des Megamax Modula-2 Entwicklungssytems,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Megamax Modula-2 ist Copyright (c) by Thomas Tempelmann.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */



#ifndef __STRCONV_H__
#define __STRCONV_H__

#ifndef __PORTAB_H__
#include <portab.h>
#endif

/*
 * Umwandlungsfunktionen zw. Zahlen und Strings
 */

_ULONG StrToLCard(CONST _UBYTE *str, size_t *pos, _BOOL *valid);
_UWORD StrToCard(CONST _UBYTE *str, size_t *pos, _BOOL *valid);
_LONG StrToLInt(CONST _UBYTE *str, size_t *pos, _BOOL *valid);
_WORD StrToInt(CONST _UBYTE *str, size_t *pos, _BOOL *valid);

  /*
     Wenn 'valid' nach dem Aufruf FALSE ist, bedeutet das, dass der
     geforderte Wert nicht ermittelt werden konnte.

     'pos' ist vor dem Aufruf auf die Stringposition zu setzen, ab der
     die Zahl gelesen werden soll. Nach dem Aufruf liefert 'pos' die
     Position im String, hinter der die umgewandelte Zahl endete.
     Endet der String nach der Zahl, ist 'pos' = Length(str).
     In der Regel ist 'pos' also vor dem Aufruf auf Null zu setzen!

     Fuer die vier StrTo...-Funktionen gilt:

     - fuehrende Blanks werden ignoriert, innerhalb der Zahl duerfen
       dann keine Leerzeichen mehr vorkommen.
     - ein eventuell vorangestelltes '+' wird ignoriert.
     - Praefixe steuern die Interpretation der folgenden Zeichen:
       '$' fuer Hexadezimal, '%' fuer Binaer, kein Praefix bedeutet
       dezimal.
     - Bei StrToInt bzw. StrToLInt muss das '-' zuerst stehen, erst
       dann darf der Praefix folgen
       
     Beispiele: StrToCard ('  123 12') = 123
                StrToLInt ('-$80') = -128L
  */

_ULONG StrToLNum(CONST _UBYTE *str, _WORD base, size_t *pos, _BOOL *valid);
_UWORD StrToNum(CONST _UBYTE *str, _WORD base, size_t *pos, _BOOL *valid);
double StrToReal(CONST _UBYTE *str, size_t *pos, _BOOL *valid);

  /*
    StrToReal liefert den Wert einer reellen Zahl. Dabei darf entweder der
    Nachkommateil (einschliesslich '.') oder der Vorkommateil fehlen. Ebenfalls
    braucht kein Exponent angegeben werden. Ein eventueller Skalierungsfaktor
    wird durch ein vorangestelltes 'E' erkannt und darf im Bereich von -1232
    bis +1232 liegen.

    Es gilt:
    
    - fuehrende Blanks werden ignoriert, innerhalb der Zahl duerfen
      dann keine Leerzeichen mehr vorkommen.
    - ein eventuell vorangestelltes '+' wird ignoriert.
    - 'ThousandsChar' werden ueberlesen.
    
    z.B. : StrToReal ('123E12') = 1.23 E14
           StrToReal ('.012E-23') = 1.2 E-25
  */



/* ======================================================================= *
 * Es folgen Funktionen zur Umwandlung von Zahlen in Strings. Dabei gilt   *
 * allgemein:                                                              *
 * - 'spc' bestimmt die Mindestlaenge des resultierenden Strings 'str'.     *
 * ======================================================================= */

_UBYTE *CardToStr(_ULONG v, _UWORD spc, _UBYTE *str, size_t strsize);
_UBYTE *IntToStr(_LONG v, _UWORD spc, _UBYTE *str, size_t strsize);

  /*
    Die ...ToStr-Funktionen wandeln eine Zahl in einen String um. Dabei steht
    ein eventuelles Minuszeichen direkt vor der ersten Ziffer. Falls die Zahl
    weniger Stellen (inklusive Vorzeichen) als in 'spc' angegeben haben sollte,
    so werden entsprechend viele Leerzeichen vorangestellt.
    
    z.B. : CardToStr (123,10) = '       123'
           IntToStr (-2345,2) = '-2345'
  */

_UBYTE *LHexToStr(_LONG v, _UWORD spc, _UBYTE *str, size_t strsize);
_UBYTE *HexToStr(_WORD v, _UWORD spc, _UBYTE *str, size_t strsize);
_UBYTE *LBinToStr(_LONG v, _UWORD spc, _UBYTE *str, size_t strsize);
_UBYTE *BinToStr(_WORD v, _UWORD spc, _UBYTE *str, size_t strsize);

  /*
    Die Hex- und Bin-Funktionen erzeugen, wie der Name sagt, Strings in
    hexadezimaler bzw. binaerer Repraesentation. Es werden nur positive Zahlen
    erzeugt. Zu Beachten ist, dass den Zahlen dabei ein '$' bzw. '%' vorange-
    stellt wird und dieses Zeichen auch mit zur Mindestlaengenangabe 'spc' zaehlt.
    Ggf. ist das vorangestellte Zeichen mit der Funktion 'Strings.Delete' zu
    entfernen. Fuehrende Nullen werden nicht unterdrueckt.
    
    z.B.: HexToStr (128,4)  = '$080'
          HexToStr (128,0)  = '$80'
          LHexToStr (-1L,0) = '$FFFFFFFF' (8 Ziffern)
          BinToStr (100,9)  = '%01100100'
          LBinToStr (-1L,1) = '%11111111111111111111111111111111' (32 Ziffern)
  */

_UBYTE *LNumToStr(_LONG lnum, _WORD base, _UWORD space, _UBYTE fill, _UBYTE *str, size_t strsize);
_UBYTE *NumToStr(_WORD lnum, _WORD base, _UWORD space, _UBYTE fill, _UBYTE *str, size_t strsize);
  /*
   * 'fill' bestimmt das Zeichen, mit dem ggf. vorn aufgefuellt wird.
   */

_UBYTE *FloatToStr(double v, _UWORD space, _UWORD n, _UBYTE *str, size_t strsize);

  /*
    Durch diese Funktion wird die Zahl 'v' in wissenschaftlicher Notation dar-
    gestellt. Es wird eine Vorkommastelle und 'n' Nachkommastellen, gefolgt von
    'E', dem Vorzeichen des Skalierungsfaktors und dem vierstelligen Wert.
    Ggf. wird der String links mit Leerzeichen bis zur Laenge 'spc' aufgefuellt.
    
    z.B. : FloatToStr (0.0123,12,1) = '   1.2E-0002'
           FloatToStr (9.9,0,10) = '9.9000000000E+0000'
  */

_UBYTE *FixToStr(double v, _UWORD space, _UWORD n, _UBYTE *str, size_t strsize);

  /*
    Diese Funktion stellt 'v' in Festpunktnotation dar, wobei entsprechend viele
    Vorkommastellen, jedoch nur genau 'n' Nachkommastellen gebracht werden.
    Wenn der String kuerzer als 'spc' sein sollte, so werden entsprechend viele
    Leerzeichen vorangestellt.
    
    z.B. : FixToStr (0.0123,12,1) = '         0.0'
           FixToStr (9.9,0,10) = '9.9000000000'
  */

_UBYTE *RealToStr(double v, _UWORD space, _UWORD n, _UBYTE *str, size_t strsize);

  /*
    Diese Prozedur verwendet je nach Lage des Wertes 'v' innerhalb der Werte
    von FixToFloatMin und FixToFloatMax (aus 'MOSConfig') das Festkomma- oder
    Fliesskomma-Format zur Darstellung von 'v'.
  */
  
_UBYTE *EngToStr(double v, _UWORD space, _UWORD n, _UBYTE *str, size_t strsize);

  /*
    Durch diese Funktion wird die Zahl 'v' in Ingenieur-Notation dargestellt.
    Es werden bis zu drei Vorkommastellen und 'n' Nachkommastellen, gefolgt
    von 'E', dem Vorzeichen des Skalierungsfaktors und dem vierstelligen Wert,
    der immer ein Vielfaches von Drei ist. Ggf. wird der String links mit Leer-
    zeichen bis zur Laenge 'spc' aufgefuellt.
    
    z.B. : EngToStr (0.0123,12,1) = '  12.3E-0003'
           EngToStr (9.9,0,10) = '9.9000000000E+0000'
  */

#endif /* __STRCONV_H__ */
