/*
 * Diese Datei ist Teil des Megamax Modula-2 Entwicklungssytems,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Megamax Modula-2 ist Copyright (c) by Thomas Tempelmann.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <keyboard.h>
#include <tos.h>
#include <portaes.h>

 /*
  *                                       Thomas Tempelmann 28.08.88
  *                                            Fuer Megamax Modula-2
  *
  *   Dieses Modul bietet Funktionen zum Auswerten der erweiterten
  *   Tastencodes, wie sie z.B. von den Funktionstasten und den
  *   Cursortasten erzeugt werden.
  *
  *   Es sind auch Funktionen vorhanden, die zwischen verschiedenen
  *   internen Darstellungsformaten von Tasten mit Scan-Codes konvertieren.
  *   (z.B. von 'GEMGlobals.GemChar/.SpecialKeySet' nach dem hier verwendeten
  *   'Key'-Type).
  *
  * ---------------------
  *  30.08.89  TT  Die Tastenabfragefunktionen wurden entfernt;
  *                intern wurden Belegung von wdLeft/Right und soln/eoln
  *                vertauscht.
  *
  */


LOCAL _BOOL MagicPCKeyTrans;


_VOID GemCharToKey(_WORD gemChar, _UBYTE kstate, Key *k)
{
	k->ch = (_UBYTE)(gemChar & 0xff);
	k->scan = (gemChar >> 8) & 0xff;
	if (kstate & K_RSHIFT)
		kstate |= K_LSHIFT;
	kstate >>= 1;
	k->ctrl = kstate;
}


Cap KeyCap(CONST Key *k)
{
	static Cap CapTbl[] = {
          unspecCap,escCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,backspaceCap,
          tabCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,returnCap,
          unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,  /* Control..'~' */
          unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,  /* Shift..Shift */
          unspecCap, /* print screen */
          unspecCap,unspecCap,unspecCap, /* Alternate; Space; Caps */
          f1Cap,f2Cap,f3Cap,f4Cap,f5Cap,f6Cap,f7Cap,f8Cap,f9Cap,f10Cap,
          unspecCap,unspecCap,clrHomeCap,upCap,unspecCap,unspecCap,leftCap,unspecCap,rightCap,unspecCap,unspecCap,downCap,
          unspecCap,insertCap,deleteCap,
          f1Cap,f2Cap,f3Cap,f4Cap,f5Cap,f6Cap,f7Cap,f8Cap,f9Cap,f10Cap,
          unspecCap,unspecCap,unspecCap,undoCap,helpCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,unspecCap,
          enterCap,leftCap,rightCap,unspecCap,unspecCap,clrHomeCap
	};
	
	if (k->scan >= sizeof(CapTbl))
		return unspecCap;
	return CapTbl[k->scan];
}


SpecialCode SpecialKey(CONST Key *k)
{
	KEYTAB *KeyTable;
	_UBYTE *unshift;
	
	static SpecialCode KeyTbl[] = {
		unspecKey,esc,      unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,
		unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,backspace,tab,
		unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,
		unspecKey,unspecKey,unspecKey,unspecKey,return_,  unspecKey,unspecKey,unspecKey,
		unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,
		unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,
		unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey, /* print screen */
		unspecKey,unspecKey,unspecKey,f1,       f2,       f3,       f4,       f5,
		f6,       f7,       f8,       f9,       f10,      unspecKey,unspecKey,unspecKey,
		unspecKey,pgUp,     unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,
		unspecKey,pgDown,   unspecKey,unspecKey,f11,      f12,      f13,      f14,
		f15,      f16,      f17,      f18,      f19,      f20,		unspecKey,unspecKey,
		unspecKey,undo,     help,     unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,
		unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,unspecKey,
		unspecKey,unspecKey,enter,    wdLeft,   wdRight,  unspecKey,pgDown,   home,
		alt1,     alt2,     alt3,     alt4,     alt5,     alt6,     alt7,     alt8,
		alt9,     alt0,     unspecKey,unspecKey,pgUp
	};
	_UWORD scan;
	
	scan = k->scan;
	if (scan >= sizeof(KeyTbl))
		return unspecKey;
	/* Sonderbehandlung */
	switch (scan)
	{
		case 0x47: return k->ch == 0 ? home : clr;
		case 0x48: return k->ch == 0 ? up : pgUp;
		case 0x4b:
		case 0x73: return (k->ctrl & 2) ? wdLeft : k->ch == 0 ? left : soln;
		case 0x4d:
		case 0x74: return (k->ctrl & 2) ? wdRight : k->ch == 0 ? right : eoln;
		case 0x50: return k->ch == 0 ? down : pgDown;
		case 0x52: return k->ch == 0 ? insert : insLine;
		case 0x53: return k->ch == 0x7f ? delete : delLine;
	default:
		if (k->ch == 0 && (k->ctrl & 4))
		{
			KeyTable = Keytbl((_VOID *)-1, (_VOID *)-1, (_VOID *)-1);
			unshift = KeyTable->unshift;
			if (unshift[scan] >= 'a' && unshift[scan] <= 'z')
				return unshift[scan] - 'a' + altA;
		}
		if (k->ch <= 0x1a && k->ch != 0 && (k->ctrl & 2) && KeyTbl[scan] == unspecKey)
			return k->ch - 1 + ctrlA;
		break;
	}
	return KeyTbl[scan];
}


_BOOL IsSpecial(CONST Key *k)
{
	return SpecialKey(k) != unspecKey;
}


_BOOL IsCharacter(CONST Key *k)
{
	if (k->ch == 0)
	{
		switch (k->scan)
		{
			case 0x03: /* Ctrl-2 */
			case 0x6e: /* Ctrl-2 auf Ziffernblock */
			case 0x39: /* Space */
				return TRUE;
		}
	} else
	{
		switch (k->scan)
		{
			case 0x4a: /* '+' auf Ziffernblock */
			case 0x4e: /* '-' auf Ziffernblock */
			case 0x53: /* Delete */
				return TRUE;
		}
		if (k->scan < 0x45 || k->scan >= 0x54)
			return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

/* Schaltet die Tastenuebersetzung fuer MagicPC ein */
GLOBAL _VOID SetMagicPCKeyTranslation(_BOOL setIt)
{
	MagicPCKeyTrans = setIt;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID TranslateKey(_UBYTE *key, _UBYTE *scan, _WORD *taste, _UWORD *kstate)
{
	if (MagicPCKeyTrans)
	{
		switch (*taste)
		{
		case 0x6328:	/* PageUp auf PC, umgesetzt auf Num-( */
			*taste = 0x4838;
			*scan = 0x48;
			*key = 0x38;
			*kstate = K_LSHIFT;
			break;
		case 0x6429:	/* PageDown auf PC, umgesetzt auf Num-) */
			*taste = 0x5032;
			*scan = 0x50;
			*key = 0x32;
			*kstate = K_LSHIFT;
			break;
		}
	}
	switch (*taste)
	{
	case 0x4900:	/* PageUp auf Mac/PC */
		*taste = 0x4838;
		*scan = 0x48;
		*key = 0x38;
		*kstate = K_LSHIFT;
		break;
	case 0x5100:	/* PageDown auf Mac/PC */
		*taste = 0x5032;
		*scan = 0x50;
		*key = 0x32;
		*kstate = K_LSHIFT;
		break;
	case 0x4f00:	/* End auf Mac/PC */
		*taste = 0x4737;
		*scan = 0x47;
		*key = 0x37;
		*kstate = K_LSHIFT;
		break;
	}
}
