/*
 * Diese Datei ist Teil des Megamax Modula-2 Entwicklungssytems,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Megamax Modula-2 ist Copyright (c) by Thomas Tempelmann.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 * 1.9.03 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.30c'
 *	Sortieren groer Listen beschleunigt
 */


#define __MOS_LISTS_IMPLEMENTATION__
#include <lists.h>
#include <ro_mem.h>

typedef struct _lcarrier Carrier;

struct _lcarrier {
	LCarrier next;
	LCarrier prev;
	_VOID *data;
};

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _VOID Lists_InitList(List *list)
{
	if (list != NULL)
	{
		list->current = NULL;
		list->root = NULL;
		list->user = NULL;
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Lists_ResetList(List *list)
{
	if (list != NULL)
		list->current = list->root;
}

/*** ---------------------------------------------------------------------- ***/

#if DEBUG_ALLOC
GLOBAL _BOOL Lists_CreateList(List *list, CONST _UBYTE *from)
#else
#undef Lists_CreateList
GLOBAL _BOOL Lists_CreateList(List *list)
#endif
{
	if (list == NULL)
		return FALSE;
	list->root = NEW(Carrier, from);
	if (list->root == NULL)
		return FALSE;
	list->root->prev = list->root;
	list->root->next = list->root;
	list->root->data = NULL;
	list->current = list->root;
	list->user = NULL;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_ListEmpty(CONST List *list)
{
	return list == NULL || list->root == NULL || list->root->next == list->root;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_DeleteList(List *list)
{
	if (!Lists_ListEmpty(list))
		return FALSE;
	if (list != NULL && list->root != NULL)
	{
		OFREE(list->root);
		list->root = NULL;
		list->current = NULL;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Lists_DeleteSimpleList(List *l, lists_delete_proc delete, _BOOL killCarrier)
{
	_VOID *entry;

	Lists_ResetList(l);
	entry = Lists_PrevEntry(l);
	while (entry != NULL)
	{
		Lists_RemoveEntry(l, delete);
		entry = Lists_CurrentEntry(l);
	}
	if (killCarrier)
		Lists_DeleteList(l);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_InsertEntry(List *list, CONST _VOID *entry)
{
	LCarrier p;
	
	if (list == NULL || list->root == NULL)
		return FALSE;
	if ((p = NEW(Carrier, "Lists_InsertEntry")) == NULL)
		return FALSE;
	p->next = list->current->next;
	p->prev = list->current;
	p->data = NO_CONST(entry);
	p->next->prev = p;
	p->prev->next = p;
	list->current = p;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_AppendEntry(List *list, CONST _VOID *entry)
{
	LCarrier p;
	
	if (list == NULL || list->root == NULL)
		return FALSE;
	if ((p = NEW(Carrier, "Lists_AppendEntry")) == NULL)
		return FALSE;
	p->prev = list->root->prev;
	p->next = list->root;
	p->data = NO_CONST(entry);
	p->prev->next = p;
	p->next->prev = p;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_RemoveEntry(List *list, lists_delete_proc delete)
{
	LCarrier p;
	_VOID *entry;
	
	if (list == NULL ||
		list->root == NULL ||
		list->root->next == list->root ||
		list->current == list->root)
		return FALSE;
	p = list->current;
	list->current = list->current->prev;
	p->next->prev = p->prev;
	p->prev->next = p->next;
	entry = p->data;
	if (delete != FUNK_NULL)
		delete(entry);
	OFREE(p);
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/
GLOBAL _VOID *Lists_NextEntry(List *list)
{
	if (list == NULL || list->current == NULL)
		return NULL;
	list->current = list->current->next;
	if (list->current == list->root)
		return NULL;
	return list->current->data;
}
/*** ------------------------------------------------------------------------ ***/
/*	_VOID Lists_SetNextEntry(List *list,void* data)								*/
/*	Setzt data des nchsten Eintrag in List auf den bergebenen Wert			*/
/*	Parameter:	list:	die Liste												*/
/*				data:	der neue Wert											*/
/*	Ergebnis:	keines															*/		
/*	1.9.03 von Dimitri Junker	DYJ 5.30c										*/
/*** ------------------------------------------------------------------------ ***/
GLOBAL _VOID Lists_SetNextEntry(List *list,void* data)
{
	if (list == NULL || list->current == NULL)
		return;
	list->current = list->current->next;
	if (list->current == list->root)
		return;
	list->current->data=data;
}


/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID *Lists_PrevEntry(List *list)
{
	if (list == NULL || list->current == NULL)
		return NULL;
	list->current = list->current->prev;
	if (list->current == list->root)
		return NULL;
	return list->current->data;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID *Lists_CurrentEntry(List *list)
{
	if (list == NULL || list->current == NULL || list->current == list->root)
		return NULL;
	return list->current->data;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL LCarrier Lists_GetCurrent(List *list)
{
	if (list == NULL)
		return NULL;
	return list->current;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID Lists_SetCurrent(List *list, LCarrier current)
{
	if (list != NULL)
		list->current = current;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_FindEntry(List *list, CONST _VOID *entry)
{
	LCarrier scan;
	
	if (list == NULL || list->root == NULL || list->current == NULL)
		return FALSE;
	for (scan = list->root->next; scan != list->root; scan = scan->next)
		if (scan->data == entry)
		{
			list->current = scan;
			return TRUE;
		}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_ScanEntries(List *list, LDir dir, LCondProc cond, CONST _VOID *info)
{
	if (list == NULL || list->current == NULL)
		return FALSE;
	if (dir == Lists_forward)
	{
		for (;;)
		{
			list->current = list->current->next;
			if (list->current == list->root)
				return FALSE;
			if (cond(list->current->data, NO_CONST(info)))
				return TRUE;
		}
	} else
	{
		for (;;)
		{
			list->current = list->current->prev;
			if (list->current == list->root)
				return FALSE;
			if (cond(list->current->data, NO_CONST(info)))
				return TRUE;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_EndOfList(List *list)
{
	if (list == NULL || list->current == NULL || list->root == NULL)
		return TRUE;
	return list->current == list->root;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_FirstEntry(List *list)
{
	if (list == NULL || list->current == NULL || list->root == NULL)
		return TRUE;
	return list->current->prev == list->root;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL Lists_LastEntry(List *list)
{
	if (list == NULL || list->current == NULL || list->root == NULL)
		return TRUE;
	return list->current->next == list->root;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _ULONG Lists_NoOfEntries(List *list)
{
	_ULONG n;
	LCarrier scan;
	
	if (list == NULL || list->root == NULL)
		return 0;
	n = 0;
	for (scan = list->root->next; scan != list->root; scan = scan->next)
		n++;
	return n;
}
