/*
 * Diese Datei ist Teil des Megamax Modula-2 Entwicklungssytems,
 * und darf in dieser Form nur als Teil des CAT-Projekts
 * verwendet werden.
 * 
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Megamax Modula-2 ist Copyright (c) by Thomas Tempelmann.
 *
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <strconv.h>
#include <convert.h>

#define NEEDS_MATH 1


typedef struct {
	CONST _UBYTE *pstr;
	size_t len;
	size_t *pidx;
	size_t pos;
	_BOOL valid;
} MyInf;


static _VOID chrGet(GetInfo *info)
{
	_UBYTE ch = '\0';
	MyInf *inf = info->userPtr;
	
	if ((*(inf->pidx)) < inf->len)
	{
		ch = *(inf->pstr);
		if (ch != '\0')
		{
			++(*(inf->pidx));
			++(inf->pstr);
		}
	}
	info->ch = ch;
}


_ULONG StrToLCard(CONST _UBYTE *str, size_t *pos, _BOOL *valid)
{
	GetInfo info;
	MyInf inf;
	_ULONG v;
	
	info.userPtr = &inf;
	inf.len = 0x7fffffffL;
	inf.pos = 0;
	inf.valid = FALSE;
	if (pos == NULL)
		pos = &inf.pos;
	if (valid == NULL)
		valid = &inf.valid;
	inf.pstr = str + *pos;
	inf.pidx = pos;
	v = ConvToLCard(chrGet, &info, valid);
	if (info.ch != '\0')
		--(*pos);
	return v;
}


_UWORD StrToCard(CONST _UBYTE *str, size_t *pos, _BOOL *valid)
{
	GetInfo info;
	MyInf inf;
	_UWORD v;
	
	info.userPtr = &inf;
	inf.len = 0x7fffffffL;
	inf.pos = 0;
	inf.valid = FALSE;
	if (pos == NULL)
		pos = &inf.pos;
	if (valid == NULL)
		valid = &inf.valid;
	inf.pstr = str + *pos;
	inf.pidx = pos;
	v = ConvToCard(chrGet, &info, valid);
	if (info.ch != '\0')
		--(*pos);
	return v;
}


_LONG StrToLInt(CONST _UBYTE *str, size_t *pos, _BOOL *valid)
{
	GetInfo info;
	MyInf inf;
	_LONG v;
	
	info.userPtr = &inf;
	inf.len = 0x7fffffffL;
	inf.pos = 0;
	inf.valid = FALSE;
	if (pos == NULL)
		pos = &inf.pos;
	if (valid == NULL)
		valid = &inf.valid;
	inf.pstr = str + *pos;
	inf.pidx = pos;
	v = ConvToLInt(chrGet, &info, valid);
	if (info.ch != '\0')
		--(*pos);
	return v;
}


_WORD StrToInt(CONST _UBYTE *str, size_t *pos, _BOOL *valid)
{
	GetInfo info;
	MyInf inf;
	_WORD v;
	
	info.userPtr = &inf;
	inf.len = 0x7fffffffL;
	inf.pos = 0;
	inf.valid = FALSE;
	if (pos == NULL)
		pos = &inf.pos;
	if (valid == NULL)
		valid = &inf.valid;
	inf.pstr = str + *pos;
	inf.pidx = pos;
	v = ConvToInt(chrGet, &info, valid);
	if (info.ch != '\0')
		--(*pos);
	return v;
}


_ULONG StrToLNum(CONST _UBYTE *str, _WORD base, size_t *pos, _BOOL *valid)
{
	GetInfo info;
	MyInf inf;
	_ULONG v;
	
	info.userPtr = &inf;
	inf.len = 0x7fffffffL;
	inf.pos = 0;
	inf.valid = FALSE;
	if (pos == NULL)
		pos = &inf.pos;
	if (valid == NULL)
		valid = &inf.valid;
	inf.pidx = pos;
	inf.pstr = str + *pos;
	v = ConvToLNum(chrGet, &info, base, valid);
	if (info.ch != '\0')
		--(*pos);
	return v;
}


_UWORD StrToNum(CONST _UBYTE *str, _WORD base, size_t *pos, _BOOL *valid)
{
	GetInfo info;
	MyInf inf;
	_UWORD v;
	
	info.userPtr = &inf;
	inf.len = 0x7fffffffL;
	inf.pos = 0;
	inf.valid = FALSE;
	if (pos == NULL)
		pos = &inf.pos;
	if (valid == NULL)
		valid = &inf.valid;
	inf.pidx = pos;
	inf.pstr = str + *pos;
	v = ConvToNum(chrGet, &info, base, valid);
	if (info.ch != '\0')
		--(*pos);
	return v;
}


#if NEEDS_MATH
double StrToReal(CONST _UBYTE *str, size_t *pos, _BOOL *valid)
{
	GetInfo info;
	MyInf inf;
	double v;
	
	info.userPtr = &inf;
	inf.len = 0x7fffffffL;
	inf.pos = 0;
	inf.valid = FALSE;
	if (pos == NULL)
		pos = &inf.pos;
	if (valid == NULL)
		valid = &inf.valid;
	inf.pidx = pos;
	inf.pstr = str + *pos;
	v = ConvToReal(chrGet, &info, valid);
	if (info.ch != '\0')
		--(*pos);
	return v;
}
#endif /* NEEDS_MATH */


_UBYTE *CardToStr(_ULONG v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	ConvCard(v, spc, str, strsize);
	return str;
}


_UBYTE *IntToStr(_LONG v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	ConvInt(v, spc, str, strsize);
	return str;
}


_UBYTE *LHexToStr(_LONG v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	ConvLHex(v, spc, str, strsize);
	return str;
}


_UBYTE *HexToStr(_WORD v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	ConvHex(v, spc, str, strsize);
	return str;
}


_UBYTE *LBinToStr(_LONG v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	ConvLBin(v, spc, str, strsize);
	return str;
}


_UBYTE *BinToStr(_WORD v, _UWORD spc, _UBYTE *str, size_t strsize)
{
	ConvBin(v, spc, str, strsize);
	return str;
}


_UBYTE *LNumToStr(_LONG v, _WORD base, _UWORD space, _UBYTE fill, _UBYTE *str, size_t strsize)
{
	ConvLNum(v, base, space, fill, str, strsize);
	return str;
}


_UBYTE *NumToStr(_WORD v, _WORD base, _UWORD space, _UBYTE fill, _UBYTE *str, size_t strsize)
{
	ConvNum(v, base, space, fill, str, strsize);
	return str;
}


#if NEEDS_MATH
_UBYTE *FloatToStr(double v, _UWORD space, _UWORD n, _UBYTE *str, size_t strsize)
{
	ConvFloat(v, space, n, str, strsize);
	return str;
}


_UBYTE *FixToStr(double v, _UWORD space, _UWORD n, _UBYTE *str, size_t strsize)
{
	ConvFix(v, space, n, str, strsize);
	return str;
}


_UBYTE *RealToStr(double v, _UWORD space, _UWORD n, _UBYTE *str, size_t strsize)
{
	ConvReal(v, space, n, str, strsize);
	return str;
}


_UBYTE *EngToStr(double v, _UWORD space, _UWORD n, _UBYTE *str, size_t strsize)
{
	ConvEng(v, space, n, str, strsize);
	return str;
}
#endif /* NEEDS_MATH */
