/*      STDLIB.H

        Standard Library Includes

        Copyright (c) Borland International 1990
        All Rights Reserved.
*/


#if !defined( __STDLIB_H__ )
#define __STDLIB_H__


#define EXIT_FAILURE    1
#define EXIT_SUCCESS    0
#define RAND_MAX        32767


#ifndef _SIZE_T
#define _SIZE_T
typedef unsigned long   size_t;
#endif

#ifdef EMUL_UNIX
typedef long wchar_t;
#endif


typedef struct
{
    int    quot;
    int    rem;
} div_t;

typedef struct
{
    long   quot;
    long   rem;
} ldiv_t;
div_t   div( int n, int d );
ldiv_t  ldiv( long n, long d );


double  atof( const char *str );
int     atoi( const char *str );
long    atol( const char *str );

char    *ecvt( double d, int ndig, int *dec, int *sign );
char    *fcvt( double d, int ndig, int *dec, int *sign );
char    *gcvt( double value, int dec, char * buf );

char    *itoa( int value, char *string, int radix );
char    *ltoa( long value, char *string, int radix );
char    *ultoa( unsigned long value, char *string, int radix );

void    *malloc( size_t size );
void    *calloc( size_t elt_count, size_t elt_size );
void    free( void *ptr );
void    *realloc( void *ptr, size_t size );

int     abs( int x );
long    labs( long x );

int     rand( void );
void    srand( unsigned int seed );
#define random( x ) (rand() % (x))
double  strtod( const char *s, char **endptr );

int     system( const char *command );

void    exit( int status );
void    _exit( int status );
void    abort( void );
int     atexit( void (*func)( void ) );

char    *getenv( const char *name );
int putenv(const char *string);

void    *bsearch( const void *key, const void *base,
          size_t nmemb, size_t size,
          int (*compar)(const void *, const void *) );
void    qsort( void *base, size_t nmemb, size_t size,
          int (*compar)(const void *, const void *) );

#ifdef EMUL_UNIX
extern long int strtol(const char *, char **, int);
extern unsigned long int strtoul(const char *, char **, int );
extern char *getcwd(char *, int);
extern char *getlogin(void);
extern char *getpass(const char *);
extern int getpw(int, char *);
extern int isatty(int);
extern char *mktemp(char *);
extern int putenv(const char *);
extern char *ttyname(int);
extern int ttyslot(void);
extern void *valloc(size_t);
#endif /* EMUL_UNIX */

#endif /* __STDLIB_H__ */

/************************************************************************/
