/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 */


#include <putzvers.h>
#include <portaes.h>
#include <mtappl.h>
#include <mtdials.h>
#include <mtalerts.h>
#include <mtaesmen.h>
#include <mtutils.h>
#ifndef __MINTBIND_H__
#include <mintbind.h>
#endif
#include <signal_.h>
#include <ro_mem.h>
#include <strconv.h>
#include <routines.h>
#include <routinec.h>
#include <file_io.h>

#include <putzrsc.h>
#include <putzwind.h>
#include <putzlist.h>
#include <putzdo.h>
#include <putzgr.h>
#include <putzlog.h>
#include <putzacti.h>
#include <putzgrou.h>
#include <putzhelp.h>

#include <convertd.h>
#include <wdwmanag.h>
#include <menu_key.h>
#include <datasys.h>
#include <confvars.h>
#include <mtpaths.h>
#include <catgloba.h>
#include <alias.h>

#include <protokol.h>
#include <xacc.h>
#include <vaproto.h>
#include <gscript.h>
#include <wildcard.h>

#include <routine.h>
#include <fslx.h>

#include <debug.h>

#include <catputz.h>
#include <wiederv2.h>


/* Konzept fuer CatPutz:
 *
 * Loeschen in einer Gruppe geht wie folgt vor:
 *
 * Es wird ein Array angelegt von 0..65535, in dem pro Message der neue Index
 * eingetragen wird. Als Feldindex dient dabei die Messagenummer.
 * Das Feld wird zuerst mit 65535 fuer alle Eintraege initialisiert.
 * Bei einer Teilloeschung wird der Messagetext durch den String
 * <Message wurde geloescht> ersetzt.
 * Bei einer Totalloeschung wird die Nachricht komplett entfernt.
 * Die Kommentarverkettung wird am Ende (wegen der rechts-Verkettung)
 * ueber das Array wiederhergestellt.
 * Das ganze funktioniert so, dass alle nachrichten, die nicht geloescht werden,
 * in ein neues *.DAT File kopiert werden. Bei einer Teilloeschung wird der
 * Messagetext ersetzt und die Messagelaenge angepasst.
 *
 * 2.5.04 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.36f'
 *	Teile von Catputz waren noch nicht ins Englische bersetzt.
 *	Jetzt sollte nur noch die Logausgabe fehlen.
 *
 * 4.5.04 Dimitri Junker (DYJ) gekennzeichnet mit 'DYJ 5.36m'
 *	Kleine Umbenennung
 */
 
#define cWdwOpen "StatWindowOpen"
#define cNoNiceLines					"NoNiceLines"

/*
 * Name unter dem sich CatPutz ueber das XACC-Protokoll zu erkennen gibt
 */
#define appName    "CatPutz \0" "XDSC\0" "1MausTausch-Programm\0" "2DC\0" "XFontAck\0" "\0"

/*
 * So meldet sich CatPutz per Default bei AppGetLongName beim GEMScript-Interpreter
 */
/*DYJ 5.36m Start*/
#define GS_APPLONGNAME "CatPutz 5"
/*DYJ  Ende; alt:	
#define GS_APPLONGNAME "CatPutz 4"
*/


GLOBAL _UBYTE ProgramName[] = "CATPUTZ";
GLOBAL _BOOL PrgHasDataBase=TRUE;		/*DYJ 5.26a */

GLOBAL _UBYTE HomePath[PATH_MAX];

LOCAL HWND wdwHandle;
LOCAL _BOOL quitProg;
LOCAL _BOOL restartCat=FALSE;	/*DYJ 5.36c */
LOCAL _UBYTE catPath[PATH_MAX]="";	/*DYJ 5.36c */
LOCAL _BOOL groupDeleted;
LOCAL _BOOL logOpen;

/*DYJ 5.36f Start*/
/*DYJ  Ende; alt:	
#define AL_IS_DA     "[3][%P:|CAT oder CatPutz|ist schon gestartet, deshalb|kann CatPutz nicht laufen!][[Abbruch]"
#define AL_CMD_ERROR "[3][%%P:|Fehler in Kommandozeile:|%s|%s][[Abbruch]"
*/


      
/******************************************************************************
 *** ---------------------------------------------------------------------- ***
 * Window handling
 *** ---------------------------------------------------------------------- ***
 ******************************************************************************/

LOCAL _VOID EnableItem(OBJECT *tree, _WORD item, _BOOL enable)
{
	if (enable)
	{
		if (GetObjState(tree, item, DISABLED))
		{
			menu_ienable(tree, item, 1);
			DelObjState(tree, item, DISABLED);
		}
	} else
	{
		if (!GetObjState(tree, item, DISABLED))
		{
			menu_ienable(tree, item, 0);
			SetObjState(tree, item, DISABLED);
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID EnableMenuItems(_VOID)
{
	_BOOL wdwOpen;
	
	ConfVars_GetConfDefBool(cWdwOpen, &wdwOpen, FALSE);
	if (wdwOpen)
		wdwOpen = PutzWindows_IsPutzWindow(wdwHandle);
	if (!wdwOpen)
		wdwHandle = -1;
	EnableItem(menu, FMOPEN, !wdwOpen);
	EnableItem(menu, FMCLOSE, wdwOpen);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID openWind(_VOID)
{
	PutzWindows_OpenWindow(&displayList, &wdwHandle);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID closeWind(_VOID)
{
	if (wdwHandle >= 0)
	{
		WdwManager_CloseWindow(wdwHandle, FALSE);
		wdwHandle = -1;
	}
}

/******************************************************************************
 *** ---------------------------------------------------------------------- ***
 * Menu handling
 *** ---------------------------------------------------------------------- ***
 ******************************************************************************/

LOCAL _VOID careForWindow(_VOID)
{
	_BOOL isOpen;

	ConfVars_GetConfDefBool(cWdwOpen, &isOpen, FALSE);
	if (isOpen)
		openWind();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID menuManager(_WORD item, _WORD menuIndex, _UWORD kstate)
{
	UNUSED(kstate);
	switch (item)
	{
	case FMOPEN:
		if (!PutzWindows_IsPutzWindow(wdwHandle))
			openWind();
		break;
	case FMCLOSE:
		if (PutzWindows_IsPutzWindow(wdwHandle))
			closeWind();
		break;
	case FMSAVE:
		PutzRsc_SaveParameter(DataPath);
		break;
	case FMQUIT:
		quitProg = TRUE;
		break;
	case ABOUTOBJ:
		PutzRsc_doInfo();
		break;
	}
	menu_tnormal(menu, menuIndex, 1);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL msgHandler(CONST _WORD *message, _UWORD kstate)
{
	_BOOL done;

	done = Protokoll_Check_Message(message);
	switch (message[0])
	{
	case MN_SELECTED:
		menuManager(message[4], message[3], kstate);
		break;
	case AP_TERM:
		SPEC_DEBUG_OUT((EO_TRACE, "AP_TERM"));
		quitProg = TRUE;
		break;
	default:
		done = FALSE;
		break;
	}
	return done;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.36c Start*/
LOCAL _BOOL performCmdDelete(grListEntry *group, delMode mode, MSGNR number, CATDATE date, _BOOL deselect,_BOOL noConfirm)
/*DYJ  Ende; alt:	
LOCAL _BOOL performCmdDelete(grListEntry *group, delMode mode, MSGNR number, CATDATE date, _BOOL deselect)
*/
{
	putzOptsRec putzOpts;
	_BOOL res;

	res = PutzDo_InitGroupFlags(group->info->catNumber, FALSE, &putzOpts); 
	if (!res && mode == PutzTypes_dFlags)
	{
		PutzLog_putTime();
		PutzLog_WriteString("L\224schen nach Flags nicht m\224glich, da Datumsfehler in");
		PutzLog_WriteLn();
		PutzLog_WriteString("der Gruppenkonfiguration von ");
		PutzLog_WriteString(group->info->name);
		PutzLog_WriteLn();
		return res;
	}
	putzOpts.dMode = mode;
	switch (mode)
	{
	case PutzTypes_dNum:
		PutzLog_putTime();
		PutzLog_WriteString("L\224sche nach Messages");
		PutzLog_WriteLn();
		PutzLog_putTime();
		PutzLog_WriteString("vorgesehene Restmessages: ");
		PutzLog_WriteCard(number);
		PutzLog_WriteLn();
		putzOpts.number = number;
		break;
		
	case PutzTypes_dDate:
		PutzLog_putTime();
		PutzLog_WriteString("L\224sche nach Datum");
		PutzLog_WriteLn();
		PutzLog_putTime();
		PutzLog_WriteString("L\224schdatum: ");
		PutzLog_WriteCard(date);
		PutzLog_WriteLn();
		putzOpts.delDate = date;
		break;
	
	case PutzTypes_dDateAndNum:
		PutzLog_putTime();
		PutzLog_WriteString("L\224sche nach Datum und Messages");
		PutzLog_WriteLn();
		PutzLog_putTime();
		PutzLog_WriteString("vorgesehene Restmessages: ");
		PutzLog_WriteCard(number);
		PutzLog_WriteLn();
		PutzLog_putTime();
		PutzLog_WriteString("L\224schdatum: ");
		PutzLog_WriteCard(date);
		PutzLog_WriteLn();
		putzOpts.number = number; 
		putzOpts.delDate = date;
		break;
	
	case PutzTypes_dFlags:
		PutzLog_putTime();
		PutzLog_WriteString("L\224sche nach Flags");
		PutzLog_WriteLn();
		break;
	}
	if (mode == PutzTypes_dNum && number == 0)
	{
/*DYJ 5.36l Start*/
		res = PutzWindows_DeleteGroup(wdwHandle, group, noConfirm);
/*DYJ  Ende; alt:	
		res = PutzWindows_DeleteGroup(wdwHandle, group, TRUE);
*/
		WdwManager_FullRedrawWdw(wdwHandle);
		if (res)
			groupDeleted = TRUE;
	} else
	{
		PutzAction_InitActionBox();
		res = PutzGroup_deleteInGroup(group, &putzOpts, mode);
		if(!res)
			group->status=GRP_ERR;
		PutzAction_ReleaseActionBox();
		if (PutzWindows_IsPutzWindow(wdwHandle))
		{
			PutzDo_CalcTotals();
			if (deselect && putzOpts.deselAfterDel)
				PutzWindows_SelectEntry(wdwHandle, group, FALSE);
			PutzWindows_RedrawEntry(wdwHandle, group);
			PutzWindows_RedrawEntry(wdwHandle, &totalEntry);
		}
	}
	return res;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID open_putzlog(_VOID)
{
	if (!logOpen)
	{
		PutzLog_OpenLog(DataPath);
		PutzLog_putTime();
		PutzLog_WriteString(CATPUTZ_VERSION);
		PutzLog_WriteString(" f\201r CAT ab 3.0 gestartet");
		PutzLog_WriteLn();
		logOpen = TRUE;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID cmdError(CONST _UBYTE *cmd, CONST _UBYTE *str)
{
	_UBYTE alt[1024];
	
	open_putzlog();
	PutzLog_WriteLn();
	PutzLog_WriteString("Fehler in Kommandozeile:");
	PutzLog_WriteLn();
	PutzLog_WriteString(cmd);
	PutzLog_WriteString(" : ");
	PutzLog_WriteString(str);
	PutzLog_WriteLn();
	PutzLog_WriteString("L\224schen \201ber Kommandozeile abgebrochen.");
	PutzLog_WriteLn();
	PutzLog_CloseLog();
/*DYJ 5.36f Start*/
	sprintf(alt, PutzRsc_WrkString(AL_CMD_ERROR), cmd, str);
/*DYJ  Ende; alt:	
	sprintf(alt, AL_CMD_ERROR, cmd, str);
*/
	mtAlerts_Alert(1, alt);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findName(_VOID *entry, _VOID *info)
{
	grListEntry *gr = entry;
	CONST _UBYTE *name = info;
	
	return strCaseCmp(gr->info->name, name) == 0;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findGroup(_UBYTE *name, grListEntry **group)
{
	_UBYTE *p;
	
	for (;;)
	{
		Lists_ResetList(&putzList);
		if (Lists_ScanEntries(&putzList, Lists_forward, findName, name))
		{
			*group = Lists_CurrentEntry(&putzList);
			return *group != NULL;
		} else
		{
			/* Suchen nach erstem '_' und durch ' ' ersetzen */
			p = strChr(name, '_');	/*DYJ 5.20w */
			if (p == NULL)
				break;
			*p = ' ';
		}
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID close_putzlog(_VOID)
{
/* :language: */
	if (logOpen)
	{
		/* Ok, das war's, weiter im normalen Programm */
		PutzDo_CalcTotals();
		PutzLog_WriteLn();
		PutzLog_WriteLn();
		PutzLog_WriteString("--------------------------------------------------");
		PutzLog_WriteLn();
		PutzLog_putTime();
		PutzLog_WriteString(" L\224schen \201ber Kommandozeile beendet.");
		PutzLog_WriteLn();
		PutzLog_WriteLine("STATISTIK: ");
		PutzLog_WriteString("           Gesamtanzahl Messages vorher: ");
		PutzLog_WriteCard(totalEntry.preMsgs);
		PutzLog_WriteLn();
		PutzLog_WriteString("           Gesamtanzahl Messages nachher: ");
		PutzLog_WriteCard(totalEntry.postMsgs);
		PutzLog_WriteLn();
		PutzLog_WriteString("           Gesamtanzahl totalgel\224schter Messages: ");
		PutzLog_WriteCard(totalEntry.totalDel);
		PutzLog_WriteLn();
		PutzLog_WriteString("           Gesamtanzahl teilgel\224schter Messages: ");
		PutzLog_WriteCard(totalEntry.partDel);
		PutzLog_WriteLn();
		PutzLog_WriteString("           Gesamtanzahl besch\204digter Messages: ");
		PutzLog_WriteCard(totalEntry.badDel);
		PutzLog_WriteLn();
		PutzLog_WriteString("           Gesamtanzahl Bytes vorher: ");
		PutzLog_WriteCard(totalEntry.preBytes);
		PutzLog_WriteLn();
		PutzLog_WriteString("           Gesamtanzahl Bytes nachher: ");
		PutzLog_WriteCard(totalEntry.postBytes);
		PutzLog_WriteLn();
		PutzLog_WriteLn();
		PutzLog_CloseLog();
		logOpen = FALSE;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL performGroupDelete(_UBYTE *groupName, delMode mode, MSGNR number, CATDATE date, _BOOL deselect)
{
	grListEntry *group;
	
	if (groupName[0] != '\0')
	{
		open_putzlog();
		if (!findGroup(groupName, &group))
		{
			/* Gruppe existiert nicht, Fehlermeldung */
/*DYJ 5.36f Start*/
			cmdError(groupName, PutzRsc_WrkString(ERR_GRU));
/*DYJ  Ende; alt:	
			cmdError(groupName, "Gruppe nicht vorhanden.");
*/
			return FALSE;
		}
/*DYJ 5.36c Start*/
		if (!performCmdDelete(group, mode, number, date, deselect,TRUE))
/*DYJ  Ende; alt:	
		if (!performCmdDelete(group, mode, number, date, deselect))
*/
		{
			/* Fehler beim Loeschen! */
/*DYJ 5.36f Start*/
				cmdError(group->info->name, PutzRsc_WrkString(ERR_DEL));
/*DYJ  Ende; alt:	
			cmdError(group->info->name, "Fehler beim L\224schen!");
*/
			return FALSE;
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/*DYJ 5.36c Start*/
LOCAL _BOOL deleteInAllGroups(delMode mode, MSGNR number, CATDATE date, _BOOL selectedOnly,_BOOL deletePers,_BOOL noConfirm)
/*DYJ  Ende; alt:	
LOCAL _BOOL deleteInAllGroups(delMode mode, MSGNR number, CATDATE date, _BOOL selectedOnly)
*/
{
	LCarrier hlpPtr;
	grListEntry *group;
	
	open_putzlog();
	Lists_ResetList(&putzList);
	while ((group = Lists_NextEntry(&putzList)) != NULL)
	{
		hlpPtr = Lists_GetCurrent(&putzList);
/*DYJ 5.36c Start*/
		if (group->info->catNumber != dataSys_private || deletePers)
/*DYJ  Ende; alt:	
		if (group->info->catNumber != dataSys_private)
*/
		{
			if (!selectedOnly || group->selected)
			{
/*DYJ 5.36c Start*/
				if (!performCmdDelete(group, mode, number, date, selectedOnly,noConfirm))
/*DYJ  Ende; alt:	
				if (!performCmdDelete(group, mode, number, date, selectedOnly))
*/
				{
					/* Fehler beim Loeschen! */
/*DYJ 5.36f Start*/
					cmdError(group->info->name, PutzRsc_WrkString(ERR_DEL));
/*DYJ  Ende; alt:	
					cmdError(group->info->name, "Fehler beim L\224schen!");
*/
					return FALSE;
				}
			}
		}
		Lists_SetCurrent(&putzList, hlpPtr);
	}
	return TRUE;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

LOCAL _WORD loglevel;
LOCAL _WORD tracelevel;
LOCAL FILE *logfile;
LOCAL _UBYTE logfile_name[PATH_MAX];
LOCAL ERROROUT_FUNC old_error;

/*** ---------------------------------------------------------------------- ***/

/*GS 4.37a */
LOCAL _VOID LogOpen(_VOID)
{
	_UBYTE backup[PATH_MAX];
	
	if (getenv("CAT_LOGLEVEL") != NULL)
		loglevel = StrToInt(getenv("CAT_LOGLEVEL"), NULL, NULL);
	else
		loglevel = 3;
	tracelevel = -1;
#if SPEC_DEBUG
	is_DEBUGGING=(loglevel==4);
#endif
	if (loglevel >= 0)
	{
		F_Path_Get_Program(logfile_name, "catdebug.log");
		F_Path_Get_Program(backup, "catdebug.bak");
		F_File_Delete(backup);
		F_Path_Rename(logfile_name, backup);
		SPEC_DEBUG_OUT((EO_DEBUG, "*** starting"));
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID LogClose(_VOID)
{
	SPEC_DEBUG_OUT((EO_DEBUG, "*** exiting"));
	ErrorOut_SetFunk(old_error);
	loglevel = -1;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL error_func(CONST _UBYTE *prg, EO_MODUS modus, _WORD nr, CONST _UBYTE *modestr, CONST _UBYTE *str)
{
	_WORD level;
	_UBYTE date[100];
	_WORD count, i;
	CONST _UBYTE *p;

	UNUSED(prg);
	UNUSED(nr);

	switch (modus)
	{
	case EO_INFO:
		modestr = "INFO  ";
		level = 1;
		break;
	case EO_DEBUG:
		modestr = "DEBUG ";
		level = 2;
		break;
	case EO_TRACE:
		modestr = "TRACE ";
		level = 3;
		break;
	case EO_ERROR:
		modestr = "*** Error: ";
		level = 0;
		break;
	case EO_FATAL:
	default:
		modestr = "*** Fatal: ";
		level = 0;
		break;
	case EO_EXIT:
		LogClose();
		return FALSE;
	case EO_INIT:
		LogOpen();
		return FALSE;
	}

	T_Strlftime(PtrSize(date), "%d.%m.%y %H:%M:%S", T_Systime());

	if (level <= tracelevel)
	{
		count = fprintf(stderr, "%s: %s", prg, modestr);
		for (p = str; *p != '\0'; p++)
		{
			putc(*p, stderr);
			if (*p == '\n')
			{
				for (i = 0; i < count; i++)
					putc(' ', stderr);
			}
		}
		putc('\n', stderr);
	}

	if (loglevel >= 0 && level <= loglevel)
	{
		if (logfile == NULL)
		{
			if ((logfile = fopen(logfile_name, "a")) == NULL)
			{
				ErrorOut_SetFunk(FUNK_NULL);
				loglevel = -1;
				ErrorOut(EO_FATAL, "can't create log file `%s', further logging disabled", logfile_name);
			}
		}
		if (logfile != NULL)
		{
			count = fprintf(logfile, "%-8.8s: %s %s", prg, date, modestr);
			for (; *str; str++)
			{
				putc(*str, logfile);
				if (*str == '\n')
				{
					for (i = 0; i < count; i++)
						putc(' ', logfile);
				}
			}
			putc('\n', logfile);
			fclose(logfile);
			logfile = NULL;
		}
	}
	return modus == EO_ERROR || modus == EO_FATAL;
}

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

typedef struct {
	CATDATE delDate;
	MSGNR delNumber;
/*DYJ 5.36c Start*/
/*DYJ  Ende; alt:	
	_BOOL allGroups;
*/
	_BOOL gotArgs;
	CONST _UBYTE *inf_from_argv;
	delMode dMode;
	_BOOL modeSet;
	_BOOL dateSet;
	_BOOL numberSet;
} COMMAND_ARGS;

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID setMode(COMMAND_ARGS *args)
{
	if (!args->modeSet)
	{
		if (args->dateSet)
		{
			if (args->numberSet)
				args->dMode = PutzTypes_dDateAndNum;
			else
				args->dMode = PutzTypes_dDate;
		} else if (args->numberSet)
		{
			args->dMode = PutzTypes_dNum;
		} else
		{
			args->dMode = PutzTypes_dFlags;
		}
		args->modeSet = TRUE;
		args->numberSet = FALSE;
		args->dateSet = FALSE;
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL ParseCommandLine(_WORD argc, CONST _UBYTE **argv, COMMAND_ARGS *args, _BOOL doit)
{
	_UBYTE cmd[256];
	_WORD i;
	_BOOL valid;
	
	args->delDate = 0;
	args->delNumber = 0;
/*DYJ 5.36c Start*/
/*DYJ  Ende; alt:	
	args->allGroups = FALSE;
*/
	args->inf_from_argv = NULL;
	args->dMode = PutzTypes_dFlags;
	args->gotArgs = FALSE;
	args->modeSet = FALSE;
	args->numberSet = FALSE;
	args->dateSet = FALSE;
	
	for (i = 1; i < argc; i++)
	{
		if (argv[i][0] == '-' && argv[i][1] != '\0')
		{
			strBcpy(cmd, argv[i] + 2);
			switch (argv[i][1])
			{
			case 'D':	/* Datum in Kommandozeile */
			case 'd':
				if (ConvertDate_DateOk(cmd))
				{
					args->delDate = ConvertDate_StrToDate(cmd);
					if (args->delDate > ConvertDate_CurrentDate())
					{
						/* Fehlermeldung! */
/*DYJ 5.36f Start*/
						cmdError(cmd, PutzRsc_WrkString(ERR_FUT));
/*DYJ  Ende; alt:	
						cmdError(cmd, "Datum gr\224\236er als Tagesdatum.");
*/
						return FALSE;
					}
					args->dateSet = TRUE;
					args->modeSet = FALSE;
				} else
				{
					/* Fehlerhaftes Datum, Fehlermeldung! */
/*DYJ 5.36f Start*/
					cmdError(cmd, PutzRsc_WrkString(ERR_DATE));
/*DYJ  Ende; alt:	
					cmdError(cmd, "Fehlerhaftes Datum.");
*/
					return FALSE;
				}
				break;
				
			case 'M':	/* Anzahl fuer Messages in Kommandozeile */
			case 'm':
				args->delNumber = StrToCard(cmd, NULL, &valid);
				if (valid)
				{
					args->numberSet = TRUE;
					args->modeSet = FALSE;
				} else
				{
					/* Fehlermeldung */
/*DYJ 5.36f Start*/
					cmdError(cmd, PutzRsc_WrkString(ERR_NONUM));
/*DYJ  Ende; alt:	
					cmdError(cmd, "Keine Nummer.");
*/
					return FALSE;
				}
				break;
			
			case 'F':
			case 'f':
				args->dMode = PutzTypes_dFlags;
				args->modeSet = TRUE;
				break;
			
			case 'G':	/* Gruppenname in Kommandozeile */
			case 'g':
				args->gotArgs = TRUE;
				if (doit)
				{
					setMode(args);
					if (performGroupDelete(cmd, args->dMode, args->delNumber, args->delDate, FALSE) == FALSE)
						return FALSE;
				}
				break;
			
			case 'A':	/* In allen Gruppen loeschen bis auf Persoenliche */
			case 'a':
/*DYJ 5.36c Start*/
/*DYJ  Ende; alt:	
				args->allGroups = TRUE;
*/
				args->gotArgs = TRUE;
				setMode(args);
				if (doit &&
					(args->dMode != PutzTypes_dNum || args->delNumber != 0))
					/* wird ignoriert, wuerde sonst alle Gruppen loeschen! */
				{
/*DYJ 5.36c Start*/ 
					if (deleteInAllGroups(args->dMode, args->delNumber, args->delDate, FALSE,strPbrk(cmd,"pP")!=NULL,strPbrk(cmd,"cC")==NULL) == FALSE)
/*DYJ  Ende; alt:	
					if (deleteInAllGroups(args->dMode, args->delNumber, args->delDate, FALSE) == FALSE)
*/
						return FALSE;
				}
				break;
				
			case 'Q':
			case 'q':
				quitProg = TRUE;
				break;
/*DYJ 5.36c Start*/
			case 'R':
			case 'r':
				restartCat = TRUE;
				strBcpy(catPath,cmd);
				break;
			case 'l':
			case 'L':
				args->gotArgs = TRUE;
				setMode(args);
				if (doit)
				{
					grListEntry *group;
					if (!findGroup(cmd, &group))
					{
						/* Gruppe existiert nicht, Fehlermeldung */
						cmdError(cmd, PutzRsc_WrkString(ERR_GRU));
						return FALSE;
					}
					if(!PutzGroup_deleteGroup(group, TRUE))
						return FALSE;
				}
				break;
/*DYJ  Ende; */
			
			default:
				/* Fehlerhafte Kommandozeile */
/*DYJ 5.36f Start*/
				cmdError(argv[i], PutzRsc_WrkString(ERR_OPT));
/*DYJ  Ende; alt:	
				cmdError(argv[i], "Unbekannte Option.");
*/
				return FALSE;
			}
		} else
		{
			if (args->inf_from_argv != NULL)
			{
				/* zwei sind einer zu viel! arg[1] kann Filename fuer CAT.INF sein */
/*DYJ 5.36f Start*/
				cmdError(argv[i], PutzRsc_WrkString(ERR_SYN));
/*DYJ  Ende; alt:
				cmdError(argv[i], "Falsche Syntax.");
*/
				return FALSE;
			}
			args->inf_from_argv = argv[i];
		}
	}
	
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

/* Lesen von CAT.INF
 * Jetzt wird die entsprechende CAT-Funktion benutzt,
 * und auch die Variablen aus CAT.
 * Zeile 11:    Database-Pfad
 */
LOCAL _BOOL InitCatPutz(CONST _UBYTE *inf_from_argv)
{
/*DYJ 5.22a Start*/
	if (!PutzRsc_InitResource())
/*DYJ  Ende; alt:	
	if (!PutzRsc_InitResource(RscName))
*/
		return FALSE;
	
	CatGlobal_busyMouse();
	PutzAction_InitModule(actBox);
	CatGlobal_SetAppl(ProgramName);
	CatGlobal_GetApplPath(infPath);
	/* Jetzt CAT.INF lesen */
	*infName = '\0';
	*infPath = '\0';
	return PutzGr_ReadCatInf(FALSE, inf_from_argv);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL KeyMsg(CONST _WORD *pBuff, _WORD kReturn, _UWORD moKState)
{
	_UBYTE ch, scan;
	_UWORD evt;
	_BOOL done;
	
	done = FALSE;
	if (MenuKey_Test(menu, kReturn, moKState, menuManager, FALSE))
	{
		done = TRUE;
	} else
	{
		/* Event-Handler aufrufen mit MU_KEYBD */
		evt = MU_KEYBD;
		scan = (kReturn >> 8) & 0xff;
		ch = kReturn & 0xff;
		done = WdwManager_HandleEvent(&evt, pBuff, 0, moKState, ch, scan, kReturn, 0, 0, 0);
	}
	/* Anderer Tastendruck wird evtl. uebergemangelt.. */
	return done;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL vaStart(CONST _WORD *message, CONST _UBYTE *pCmd)
{
	UNUSED(message);
	/* VA_START: pCmd enthaelt Kommandozeile */
	if (pCmd != NULL && *pCmd != '\0')
	{
		/* Jetzt Kommandozeile auswerten:
		 * entweder ist ein Dateiname enthalten oder sie faengt mit
		 * - an und ist eine Kommandoliste
		 */
		if (pCmd[0] != '-')
		{
			/* Ist wohl ein Dateiname, also INF laden */
			PutzWindows_SwitchInf(wdwHandle, FALSE, pCmd);
		}
	} else
	{
		/* Leere Kommandozeile, Fenster toppen */
		if (PutzWindows_IsPutzWindow(wdwHandle))
			WdwManager_TopWindow(wdwHandle);
		else
			openWind();
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *gs_erg(CONST _UBYTE *res)
{
	_UBYTE *erg;
	size_t len;
	
	/*
	 * 1 Byte mehr als eigentlich noetig, weil das Ergebnis
	 * mit 2 Null-Byte abgeschlossen werden muss
	 */
	len = strLen(res);	/*DYJ 5.20w */
	erg = SYSMALLOC(len + 2, MEM_NOERR | MEM_SHARABLE, "gs_erg");
	if (erg != NULL)
	{
		strCpy(erg, res);	/*DYJ 5.20w */
		erg[len + 1] = '\0';
	}
	return erg;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *gs_ok(_VOID)
{
	return gs_erg("1");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *gs_error(_VOID)
{
	return gs_erg("0");
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *gs_res(_BOOL res)
{
	return res ? gs_ok() : gs_error();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL HWND nameToWdw(CONST _UBYTE *arg)
{
	HWND wdw;
	_BOOL valid;
	
	wdw = StrToInt(arg, NULL, &valid);
	if (!valid)
		return -1;
	return wdw;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _UBYTE *wdwToName(HWND wdw)
{
	_UBYTE str[20];

	IntToStr(wdw, 0, str, sizeof(str));
	return gs_erg(str);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSaveAs(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	UNUSED(argc);
	UNUSED(argv);
	if (argc > 2)
		wdw = nameToWdw(argv[2]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0 && argc > 1 && argv[1] != NULL && argv[1][0] != '\0')
	{
		res = PutzWindows_Save(wdw, argv[1], FALSE);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsClose(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	if (argc <= 1)
	{
		wdw = WdwManager_GetOwnTopWindow();
		WdwManager_CloseWindow(wdw, FALSE);
	} else
	{
		while (--argc)
		{
			wdw = nameToWdw(*++argv);
			WdwManager_CloseWindow(wdw, FALSE);
		}
	}
	*erg = gs_ok();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsGetFront(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	UNUSED(argc);
	UNUSED(argv);
	wdw = WdwManager_GetOwnTopWindow();
	*erg = wdwToName(wdw);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsToFront(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	
	if (argc > 1)
	{
		wdw = nameToWdw(argv[1]);
	} else
	{
		wdw = WdwManager_GetOwnTopWindow();
	}
	if (wdw >= 0)
	{
		WdwManager_TopWindow(wdw);
		*erg = gs_ok();
	} else
	{
		*erg = gs_error();
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsCopy(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0)
	{
		res = PutzWindows_Save(wdw, NULL, TRUE);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSelectAll(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0)
	{
		res = PutzWindows_SelectAll(wdw, TRUE);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSelectNone(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	HWND wdw;
	_BOOL res;
	
	if (argc > 1)
		wdw = nameToWdw(argv[1]);
	else
		wdw = WdwManager_GetOwnTopWindow();
	res = FALSE;
	if (wdw >= 0)
	{
		res = PutzWindows_SelectAll(wdw, FALSE);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL SelectEntries(_UBYTE *groupName, _BOOL select)
{
	grListEntry *group;

	if (groupName == NULL || groupName[0] == '\0')
		return FALSE;
	Lists_ResetList(&putzList);
	while ((group = Lists_NextEntry(&putzList)) != NULL)
	{
		if (WildCards_MatchName(group->info->name, groupName, TRUE))
		{
			PutzWindows_SelectEntry(wdwHandle, group, select);
			PutzWindows_RedrawEntry(wdwHandle, group);
		}
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsSelect(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_BOOL res;
	
	res = FALSE;
	if (argc > 1)
		res = SelectEntries(argv[1], TRUE);
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsDeSelect(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_BOOL res;
	
	res = FALSE;
	if (argc > 1)
		res = SelectEntries(argv[1], FALSE);
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsOpen(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	UNUSED(argc);
	UNUSED(argv);
	if (PutzWindows_IsPutzWindow(wdwHandle))
		WdwManager_TopWindow(wdwHandle);
	else
		openWind();
	*erg = gs_ok();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsReadInf(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_BOOL res;
	
	if (argc > 1 && argv[1] != NULL && argv[1][0] != '\0')
	{
		/* Ist wohl ein Dateiname, also INF laden */
		res = PutzWindows_SwitchInf(wdwHandle, FALSE, argv[1]);
	} else
	{
		res = PutzWindows_SwitchInf(wdwHandle, TRUE, NULL);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsDeleteByFlags(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_BOOL res;
	
	if (argc > 1 && argv[1] != NULL && argv[1][0] != '\0')
	{
		res = performGroupDelete(argv[1], PutzTypes_dFlags, 0, 0L, TRUE);
	} else
	{
/*DYJ 5.36c Start*/
		res = deleteInAllGroups(PutzTypes_dFlags, 0, 0L, TRUE,FALSE,TRUE);
/*DYJ  Ende; alt:	
		res = deleteInAllGroups(PutzTypes_dFlags, 0, 0L, TRUE);
*/
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsDeleteByNumber(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_BOOL res;
	MSGNR delNumber;
	
	delNumber = 0;
	if (argc <= 1 || argv[1] == NULL)
		res = FALSE;
	else
		delNumber = StrToCard(argv[1], NULL, &res);
	if (res)
	{
		if (argc > 2 && argv[2] != NULL && argv[2][0] != '\0')
		{
			res = performGroupDelete(argv[2], PutzTypes_dNum, delNumber, 0L, TRUE);
		} else
		{
/*DYJ 5.36c Start*/
			res = deleteInAllGroups(PutzTypes_dNum, delNumber, 0L, TRUE,FALSE,TRUE);
/*DYJ  Ende; alt:	
			res = deleteInAllGroups(PutzTypes_dNum, delNumber, 0L, TRUE);
*/
		}
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsDeleteByDate(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_BOOL res;
	CATDATE delDate;
	
	res = FALSE;
	delDate = 0;
	if (argc > 1 && argv[1] != NULL)
	{
		if (ConvertDate_DateOk(argv[1]))
		{
			delDate = ConvertDate_StrToDate(argv[1]);
			if (delDate <= ConvertDate_CurrentDate())
				res = TRUE;
		}
	}
	if (res)
	{
		if (argc > 2 && argv[2] != NULL && argv[2][0] != '\0')
		{
			res = performGroupDelete(argv[2], PutzTypes_dDate, 0, delDate, TRUE);
		} else
		{
/*DYJ 5.36c Start*/
			res = deleteInAllGroups(PutzTypes_dDate, 0, delDate, TRUE,FALSE,TRUE);
/*DYJ  Ende; alt:	
			res = deleteInAllGroups(PutzTypes_dDate, 0, delDate, TRUE);
*/
		}
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsDeleteByDateAndNumber(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_BOOL res;
	CATDATE delDate;
	MSGNR delNumber;
	
	res = FALSE;
	delNumber = 0;
	delDate = 0;
	if (argc > 2)
	{
		if (argv[1] != NULL && ConvertDate_DateOk(argv[1]))
		{
			delDate = ConvertDate_StrToDate(argv[1]);
			if (delDate <= ConvertDate_CurrentDate())
				res = TRUE;
		}
		delNumber = StrToCard(argv[2], NULL, &res);
	}
	if (res)
	{
		if (argc > 3 && argv[3] != NULL && argv[3][0] != '\0')
		{
			res = performGroupDelete(argv[3], PutzTypes_dDateAndNum, delNumber, delDate, TRUE);
		} else
		{
/*DYJ 5.36c Start*/
			res = deleteInAllGroups(PutzTypes_dDateAndNum, delNumber, delDate, TRUE,FALSE,TRUE);
/*DYJ  Ende; alt:	
			res = deleteInAllGroups(PutzTypes_dDateAndNum, delNumber, delDate, TRUE);
*/
		}
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsDeleteGroup(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_BOOL res;
	
	res = FALSE;
	if (argc > 1 && argv[1] != NULL && argv[1][0] != '\0')
	{
		res = performGroupDelete(argv[1], PutzTypes_dNum, 0, 0L, TRUE);
	}
	*erg = gs_res(res);
}

/*** ---------------------------------------------------------------------- ***/

typedef struct _gs_command {
	CONST _UBYTE *name;
	_VOID (*func)(_WORD argc, _UBYTE *argv[], _UBYTE **erg);
} GS_COMMAND;

LOCAL _VOID gsGetAllCommands(_WORD argc, _UBYTE *argv[], _UBYTE **erg);
LOCAL _VOID gsCheckCommand(_WORD argc, _UBYTE *argv[], _UBYTE **erg);

LOCAL GS_COMMAND gs_command[] = {
	{ "CheckCommand", gsCheckCommand },
	{ "Close", gsClose },
	{ "Copy", gsCopy },
	{ "GetAllCommands", gsGetAllCommands },
	{ "GetFront", gsGetFront },
	{ "Open", gsOpen },
	{ "SaveAs", gsSaveAs },
	{ "SelectAll", gsSelectAll },
	{ "SelectNone", gsSelectNone },
	{ "DeSelectAll", gsSelectNone },
	{ "ToFront", gsToFront },

	{ "ReadInf", gsReadInf },
	{ "Select", gsSelect },
	{ "DeSelect", gsDeSelect },
	{ "DeleteByFlags", gsDeleteByFlags },
	{ "DeleteByNumber", gsDeleteByNumber },
	{ "DeleteByDate", gsDeleteByDate },
	{ "DeleteByDateAndNumber", gsDeleteByDateAndNumber },
	{ "DeleteGroup", gsDeleteGroup },
	
	{ NULL, FUNK_NULL }
};

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsGetAllCommands(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_WORD i;
	size_t len;
	_UBYTE *p;
	
	UNUSED(argc);
	UNUSED(argv);
	len = 0;
	for (i = 0; gs_command[i].name != NULL; i++)
		len += strLen(gs_command[i].name) + 1;	/*DYJ 5.20w */
	len += 14 + 1; /* AppGetLongName */
	len += 4 + 1;  /* Quit */
	
	*erg = SYSMALLOC(len + 1, MEM_NOERR | MEM_SHARABLE, "gsGetAllCommands");
	if (*erg != NULL)
	{
		p = *erg;
		for (i = 0; gs_command[i].name != NULL; i++)
		{
			strCpy(p, gs_command[i].name);	/*DYJ 5.20w */
			p += strLen(gs_command[i].name) + 1;	/*DYJ 5.20w */
		}
		strCpy(p, "AppGetLongName");	/*DYJ 5.20w */
		p += 14 + 1;
		strCpy(p, "Quit");	/*DYJ 5.20w */
		p += 4 + 1;
		*p = '\0';
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID gsCheckCommand(_WORD argc, _UBYTE *argv[], _UBYTE **erg)
{
	_WORD i;
	
	if (argc > 1)
	{
		for (i = 0; gs_command[i].name != NULL; i++)
			if (strCaseCmp(argv[1], gs_command[i].name) == 0)
			{
				*erg = gs_ok();
				return;
			}
		if (strCaseCmp(argv[1], "Quit") == 0 ||
			strCaseCmp(argv[1], "AppGetLongName") == 0)
		{
			*erg = gs_ok();
			return;
		}
	}
	*erg = gs_error();
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID PutzGScript_Init(_VOID)
{
	_WORD i;
	
	GScript_Init(GS_APPLONGNAME);
	for (i = 0; gs_command[i].name != NULL; i++)
		GEMScript_Command(gs_command[i].name, gs_command[i].func);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID cleanup(_VOID)
{	
	closeWind();
	ResetMenuline();
	ConfVars_CloseConfig();
#if DEBUG_ALLOC
	PutzGr_Exit();
	Alias_Exit();
	ConfVars_DeleteAllVars();
	/* Infofiles_Exit(); */
	GroupSelect_Exit();
	WiederVorlage_Exit();
	PutzRsc_ClrRsrc();
#else
	mem_test_ignore();
#endif
	close_putzlog();
	Xacc_SendTerminate(TRUE);
	/* und auch an die VA-Partner */
	VaProto_SendTerminate(TRUE);
	GScript_Exit();
	WdwManager_HandlePendingEvents();
	Wind_Update(END_UPDATE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID __CDECL sigHandler(_LONG sig)
{
	UNUSED(sig);
	/* Sofort raus hier wenn wir schon mal hier waren */
	Psignal(SIGQUIT, SIG_IGN);
	Psignal(SIGTERM, SIG_IGN);
	Psignal(SIGPIPE, SIG_IGN);
	Psignal(SIGXCPU, SIG_IGN);
	Psignal(SIGHUP, SIG_IGN);
	Psignal(SIGSEGV, SIG_IGN);
	Psignal(SIGBUS, SIG_IGN);
	SPEC_DEBUG_OUT((EO_FATAL, "terminated by signal %ld", sig));
	quitProg = TRUE;
	cleanup();
	Psigreturn();
	ApplTerm(3);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID installSignalHandler(_VOID)
{
	/* Folgende Signale werden abgefangen und ignoriert:
	 * SIGHUP, SIGINT, SIGSYS,
	 */
	Psigblock((1L << SIGHUP) | (1L << SIGINT) | (1L << SIGSYS));
	/* Folgende Signale werden mit Programmterminierung befolgt:
	 * SIGQUIT, SIGPIPE, SIGTERM, SIGXCPU
	 * Dabei ist es mir egal, ob die Installation geklappt hat oder nicht.
	 */
#ifdef SIGQUIT
	Psignal(SIGQUIT, (_LONG)sigHandler);
#endif
#ifdef SIGTERM
	Psignal(SIGTERM, (_LONG)sigHandler);
#endif
#ifdef SIGPIPE
	Psignal(SIGPIPE, (_LONG)sigHandler);
#endif
#ifdef SIGXCPU
	Psignal(SIGXCPU, (_LONG)sigHandler);
#endif
#ifdef SIGHUP
	Psignal(SIGHUP, (_LONG)sigHandler);
#endif
#ifdef SIGSEGV
	Psignal(SIGSEGV, (_LONG)sigHandler);
#endif
#ifdef SIGBUS
	Psignal(SIGBUS, (_LONG)sigHandler);
#endif
}

/*** ---------------------------------------------------------------------- ***/

int main(int argc, char *const *argv)
{
	GRECT mrect;
	_WORD moX, moY;
	_WORD kReturn;
	_UWORD kstate;
	_UWORD moButton;
	_WORD moClicks;
	_UBYTE key, scan;
	_UWORD event;
	_WORD message[8];
	COMMAND_ARGS args;
	_BOOL args_ok;
	_WORD exit_code;
	EVENT e;
	EVNT ev;
	_BOOL noNiceLines;
	
#define helpScan 0x62
	
	mrect.g_x = 0;
	mrect.g_y = 0;
	mrect.g_w = 1;
	mrect.g_h = 1;
	groupDeleted = FALSE;
	wdwHandle = -1;
	
	old_error = ErrorOut_SetFunk(error_func);
	ApplInit();
	installSignalHandler();
	CatGlobal_Init();
	mtDials_Init();
	
	mtAlerts_ConfigAlert(alert_alticon);       /* mtAlerts konfigurieren */
	mtDials_DialConfig(UseConfig, FALSE); /* mtDials konfigurieren */
	MouseArrow();
	
	Wind_Update(BEG_UPDATE);
	ConfVars_Init();
	WdwManager_Init();
	
	quitProg = FALSE;
	args_ok = ParseCommandLine(argc, (CONST _UBYTE **)argv, &args, FALSE);
	
	if (!InitCatPutz(args.inf_from_argv))
	{
		exit_code = 1;
	} else
	{
		WdwManager_EnableIconify(TRUE);
		MouseArrow();
		if (multiTOS || (magIx && magIxVer >= 0x300))
		{
			/* MultiTOS */
			menu_register(gl_apid, CATPUTZ_MENUNAME);
		}
		menu_register(-1, ProgramName);
		
		{
			_WORD level, dummy;
			
			/* Und auch Bescheid sagen, dass wir AP_TERM verstehen */
			if ((appl_xgetinfo(APG_SHELLW, &level, &dummy, &dummy, &dummy) && ((level & 0xff) >= 9)) ||
				(magIxVer >= 0x300))
			{
				shel_write(9, 1, 1, NULL, NULL);
			}
		}
		
		ConfVars_GetConfDefBool(cNoNiceLines, &noNiceLines, FALSE);
		if(!noNiceLines)
			InitMenuline(menu);
		DrawMenuline(menu);
		
		/* Menubar bei Windowmanager anmelden */
		WdwManager_InstallGlobalMenu(menu, msgHandler);
		
		PutzList_InitFormat();
		if (!args.gotArgs)
			PutzGr_ReadSelectedState(&putzList);
		
		Wind_Update(END_UPDATE);
		WdwManager_HandlePendingEvents();
		Wind_Update(BEG_UPDATE);
		
		careForWindow();
		
		Xacc_Register_Key(KeyMsg);
		Xacc_Init(appName, sizeof(appName));
		VaProto_Register_Sendkey(KeyMsg);
		VaProto_Register_Start(vaStart);
		VaProto_Init(appName, sizeof(appName));
		PutzGScript_Init();

		if (args_ok)
		{
			Wind_Update(END_UPDATE);
			ParseCommandLine(argc, (CONST _UBYTE **)(argv), &args, TRUE);
			close_putzlog();
			Wind_Update(BEG_UPDATE);
		}
		
		while (!quitProg)
		{
			Wind_Update(END_UPDATE);

			e.ev_mflags = MU_KEYBD | MU_BUTTON | MU_MESAG | MU_M1;
			e.ev_mwich = 0;
			e.ev_mbclicks = 256 | 3;
			e.ev_bmask = 3;
			e.ev_mbstate = 0;
			e.ev_mm1flags = 1;
			e.ev_mm1x = mrect.g_x;
			e.ev_mm1y = mrect.g_y;
			e.ev_mm1width = mrect.g_w;
			e.ev_mm1height = mrect.g_h;
			e.ev_mm2flags = 0;
			e.ev_mm2x = 0;
			e.ev_mm2y = 0;
			e.ev_mm2width = 0;
			e.ev_mm2height = 0;
			e.ev_mtlocount = 350;
			e.ev_mthicount = 0;

			if (fnts_func != NULL)
				fnts_func(TRUE, &e);
			if (e.ev_mwich == 0)
			{
				e.ev_mwich = evnt_multi(e.ev_mflags, e.ev_mbclicks, e.ev_bmask, e.ev_mbstate,
					e.ev_mm1flags, e.ev_mm1x, e.ev_mm1y, e.ev_mm1width, e.ev_mm1height,
					e.ev_mm2flags, e.ev_mm2x, e.ev_mm2y, e.ev_mm2width, e.ev_mm2height,
					e.ev_mmgpbuf,
					e.ev_mtlocount, e.ev_mthicount,
					&e.ev_mmox, &e.ev_mmoy, &e.ev_mmobutton, &e.ev_mmokstate, &e.ev_mkreturn, &e.ev_mbreturn);
			}

			event = e.ev_mwich;
			moX = e.ev_mmox;
			moY = e.ev_mmoy;
			moButton = e.ev_mmobutton;
			kstate = e.ev_mmokstate;
			kReturn = e.ev_mkreturn;
			moClicks = e.ev_mbreturn;
			message[0] = e.ev_mmgpbuf[0];
			message[1] = e.ev_mmgpbuf[1];
			message[2] = e.ev_mmgpbuf[2];
			message[3] = e.ev_mmgpbuf[3];
			message[4] = e.ev_mmgpbuf[4];
			message[5] = e.ev_mmgpbuf[5];
			message[6] = e.ev_mmgpbuf[6];
			message[7] = e.ev_mmgpbuf[7];
			
			if (fnts_func != FUNK_NULL)
			{
				ev.mwhich = event;
				ev.mox = moX;
				ev.moy = moY;
				ev.button = moButton;
				ev.state = kstate;
				ev.key = kReturn;
				ev.clicks = moClicks;
				ev.message[0] = message[0];
				ev.message[1] = message[1];
				ev.message[2] = message[2];
				ev.message[3] = message[3];
				ev.message[4] = message[4];
				ev.message[5] = message[5];
				ev.message[6] = message[6];
				ev.message[7] = message[7];
				fnts_func(FALSE, &ev);
				event = (event & ~MU_KEYBD) | (ev.mwhich & MU_KEYBD);
			}

			key = kReturn & 0xff;
			scan = (kReturn >> 8) & 0xff;
			Wind_Update(BEG_UPDATE);
			WdwManager_HandleEvent(&event, message, moButton, kstate, key, scan, kReturn, moX, moY, moClicks);
			if (event & MU_KEYBD)
			{
				if (scan == 1)	/* ESC */
					WdwManager_FullRedrawWdw(wdwHandle);
				if (MenuKey_Test(menu, kReturn, kstate, menuManager, FALSE))
				{
				} else if (scan == helpScan)
				{
					PutzHelp_DoHelp(PutzHelp_general);
				}
			}
			mrect.g_x = moX;
			mrect.g_y = moY;
			EnableMenuItems();
		}
		
		exit_code = groupDeleted ? 0x4713: 0;
		if (AutoSaveOptions)
			PutzRsc_SaveParameter(DataPath);
	}
	cleanup();
/*DYJ 5.36c Start*/
	if(restartCat)
	{
	
		/* Putzprogramm mittels ShelWrite starten */
		if(*catPath)
			shel_write(1, 1, 0, catPath, "");
		else
		{
			_UBYTE catName[PATH_MAX];
			CatGlobal_ApplPath(catName);
			F_Path_Append(catName, "CAT.PRG");
			shel_write(1, 1, 0, catName, "");
		}
	}
/*DYJ  Ende; */
	ApplTerm(exit_code);
	return exit_code;
}
