/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 */


#include <putzgr.h>
#include <putztype.h>
#include <putzlog.h>
#include <putzgrou.h>
#include <putzrsc.h>
#include <putzinit.h>
#include <putzdo.h>
#include <convertd.h>
#include <groupsel.h>
#include <wildcard.h>
#include <catgloba.h>
#include <mtpaths.h>
#ifndef __MINTBIND_H__
#include <mintbind.h>
#endif
#include <filename.h>
#include <magicsys.h>
#include <mtappl.h>
#include <mtalerts.h>
#include <mtdir.h>
#include <strconv.h>
#include <confvars.h>

#include <file_io.h>
#include <ro_mem.h>
#include <routinec.h>
#include <routines.h>
#include <dir_io.h>
#include <data.h>


/*DYJ 5.36f Start*/
#include <catputz.h>
/*DYJ  Ende; alt:	
#define noInfFile "[3][%P:|GRUPPEN.INF nicht|gefunden][[Abbruch]"
#define noMemAlrt "[3][%P:|Konnte Gruppenliste|nicht anlegen!|(Zuwenig Speicher)][[Abbruch]"

#define AL_READ_GROUPLIST "[3][%P:|Kann Gruppenliste|nicht lesen!][[Abbruch]"
*/

#define allWildPat  "*.*"
#define datPat      "*" datTag
#define parPat      "*" parTag
#define tabPat      "*" tabTag

LOCAL _UWORD maxNameLength = 10;

/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _UWORD PutzGr_MaxNameLength(_VOID)
{
	return maxNameLength;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzGr_FreeGroupList(_VOID)
{
	Lists_DeleteSimpleList(&putzList, mem_free, TRUE);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL ptrDispEntry appEntry(List *l, putzlistEntryType typ, _VOID *adr)
{
	ptrDispEntry entry;
	
	entry = NEW(displayListEntry, "PutzGr_AppEntry");
	if (entry != NULL)
	{
		entry->type = typ;
		entry->data = adr;
		Lists_AppendEntry(l, entry);
	}
	return entry;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzGr_BuildDisplayList(List *l)
{
	grListEntry *data;

	Lists_CreateList(l, "PutzGr_BuildDisplayList");
	appEntry(l, listHead1, NULL);
	appEntry(l, listHead2, NULL);
	appEntry(l, separatorLine, NULL);
	Lists_ResetList(&putzList);
	while ((data = Lists_NextEntry(&putzList)) != NULL)
		appEntry(l, dataEntry, data);
	appEntry(l, separatorLine, NULL);
	appEntry(l, dataEntry, &totalEntry);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzGr_ClearDisplayList(_BOOL killCarrier)
{
	Lists_DeleteSimpleList(&displayList, mem_free, killCarrier);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL findNumber(_VOID *entry, _VOID *info)
{
	CONST CAT_GROUP_NUMBER *intPtr;
	CONST grListEntry *gr;
	
	gr = entry;
	intPtr = info;
	return gr->info->catNumber == *intPtr;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL grListEntry *PutzGr_FindGroupByNumber(CAT_GROUP_NUMBER number)
{
	Lists_ResetList(&putzList);
	if (Lists_ScanEntries(&putzList, Lists_forward, findNumber, &number))
		return Lists_CurrentEntry(&putzList);
	return NULL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL grListEntry *FindGroup(CONST _UBYTE *name, _BOOL ignoreCase)
{
	CAT_GROUP_NUMBER groupNumber;
	_BOOL valid;
	
	if (WildCards_MatchName(name, privatePat, ignoreCase))
	{
		/* Private */
		groupNumber = dataSys_private;
	} else
	{
		while (*name != '\0' && (*name < '0' || *name > '9'))
			name++;
		/* Jetzt die Zahl bilden */
		groupNumber = StrToInt(name, NULL, &valid);
		if (!valid)
			return NULL;
		++groupNumber;
	}
	/* Gruppe zu Nummer suchen */
	return PutzGr_FindGroupByNumber(groupNumber);
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID PutzGr_SearchGroups(_VOID)
{
	_UBYTE tmp[PATH_MAX];
	grListEntry *gr;
	_LONG groupCount;
	_BOOL oneFound;
	Directory *dir;
	DirectoryEntry *entry;
	
	groupCount = 0;
	dir = D_Opendir(DataPath);
	if (dir != NULL)
	{
		while ((entry = D_Readdir(dir)) != NULL)
		{
			strBcpy(tmp, entry->d_name);
			
			oneFound = FALSE;
			if (WildCards_MatchName(tmp, groupPat, TRUE) ||
				WildCards_MatchName(tmp, privatePat, TRUE))
			{
				/* Gruppe zu Nummer suchen */
				if ((gr = FindGroup(tmp, TRUE)) != NULL)
				{
					oneFound = TRUE;
					/* Typ feststellen */
					if (WildCards_MatchName(tmp, datPat, TRUE))
					{
						/* Ist Datendatei */
						gr->datDa = TRUE;
						gr->preBytes = entry->d_st.st_size;
					} else if (WildCards_MatchName (tmp, parPat, TRUE))
					{
						/* Ist Parameterdatei */
						gr->parDa = TRUE;
						if(gr->tabDa)
						{
							if((gr->preMsgs*sizeof(pBlock)+dbHeaderLength)!=entry->d_st.st_size)
								gr->preMsgs = (entry->d_st.st_size - dbHeaderLength) / sizeof(pBlock_X);
						}
						else
							gr->preMsgs = (entry->d_st.st_size - dbHeaderLength) / sizeof(pBlock);
					} else if (WildCards_MatchName(tmp, tabPat, TRUE))
					{
						/* Ist TAB-Datei */
						if(gr->parDa)
						{
							if((gr->preMsgs*sizeof(CRC16)*2)!=entry->d_st.st_size)
								gr->preMsgs = entry->d_st.st_size  / sizeof(CRC16);
						}
						else
							gr->preMsgs = entry->d_st.st_size  / sizeof(CRC16)/2;
						gr->tabDa = TRUE;
					} else
					{
						oneFound = FALSE;
					}
				}
			}
			if (oneFound)
			{
				++groupCount;
				PutzInit_SetCurrentValue(groupCount);
			}
			/* Naechsten Eintrag nehmen */
			CatGlobal_busyMouse();
		}
		
		D_Closedir(dir);
	}
}

/*** ---------------------------------------------------------------------- ***/

/* Diese Prozedur liefert eine Liste der Gruppen, die in CAT vorhanden
 * und damit lesbar sind. Die Gruppe 'PERSOENLICHE' ist mit enthalten.
 * Die Liste ist nach der Lesereihenfolge sortiert, die Gruppe 'PERSOENLICHE'
 * ist immer als erste Gruppe vorhanden (die Lesenummer davon kann nicht
 * geaendert werden).
 * FALSE: Liste konnte nicht erzeugt werden
 */
LOCAL _BOOL GetReadList(List *list)
{
#if 1
	return GroupSelect_GetReadList(list, FALSE, FALSE, FALSE);
#else
	group *new;

	MouseBusy();
	if (!GroupSelect_CopyList(list, GroupSelect_copyByNum))
	{
		MouseArrow();
		return FALSE;
	}
	/* Jetzt haben wir eine neue Liste, die wir nur noch nach der
	 * Lesereihenfolge sortieren muessen
	 */
	{
#if 0	/* Ist per Default nach Namen sortiert */
		SortList(list, sByName);
#endif
	}
	/* So, jetzt haben wir eine sortierte Liste. Jetzt muessen wir nur
	 * noch die persoenliche Gruppe evtl. dazu eintragen und vorne einfuegen.
	 */
	new = create_private_group();
	if (new == NULL)
	{
		GroupSelect_DeleteList(list);
		MouseArrow();
		return FALSE;
	}
	Lists_ResetList(list);
	if (!Lists_InsertEntry(list, new))
	{
		FREE(new->data, new->dataLength);
		OFREE(new);
		GroupSelect_DeleteList(list);
		MouseArrow();
		return FALSE;
	}
	MouseArrow();
	return TRUE;
#endif
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzGr_SaveSelectedState(List *gr)
{
	grListEntry *entry;
	_UBYTE gName[256];
	
	Lists_ResetList(gr);
	while ((entry = Lists_NextEntry(gr)) != NULL)
	{
		/* Gruppennamen von Leerzeichen befreien */
		strBcpy(gName, entry->info->name);
		strUpr(gName);
		CatGlobal_ReplaceChar(gName, ' ', '_');    /* Spaces durch Underscores ersetzen */
		strBcat(gName, ".selected");
		ConfVars_SetConfigBool(gName, entry->selected);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzGr_ReadSelectedState(List *gr)
{
	grListEntry *entry;
	_UBYTE gName[256];
	
	Lists_ResetList(gr);
	while ((entry = Lists_NextEntry(gr)) != NULL)
	{
		/* Gruppennamen von Leerzeichen befreien */
		strBcpy(gName, entry->info->name);
		strUpr(gName);
		CatGlobal_ReplaceChar(gName, ' ', '_');    /* Spaces durch Underscores ersetzen */
		strBcat(gName, ".selected");
		ConfVars_GetConfDefBool(gName, &entry->selected, FALSE);
	}
}


/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL PutzGr_InitGroups(_VOID)
{
	groupPtr entry;
	grListEntry *grEntry;
	_UWORD len;
	_UBYTE varName[256];
	
	PutzInit_InitInitBox();
	/* Jetzt erstmal die Gruppenliste einlesen */
	if (!GroupSelect_ReadGroupList())
	{
/*DYJ 5.36f Start*/
		mtAlerts_Alert(1, PutzRsc_WrkString(AL_READ_GROUPLIST));
/*DYJ  Ende; alt:	
		mtAlerts_Alert(1, AL_READ_GROUPLIST);
*/
		PutzInit_CloseInitBox();
		return FALSE;
	}
	CatGlobal_busyMouse();
	
	totalEntry.info = NULL;
	totalEntry.preMsgs = 0;
	totalEntry.preBytes = 0;
	totalEntry.badDel = 0;
	totalEntry.totalDel = 0;
	totalEntry.partDel = 0;
	totalEntry.postMsgs = 0;
	totalEntry.postBytes = 0;
	
	/* Gruppenliste holen */
	if (!GetReadList(&groupList))
	{
/*DYJ 5.36f Start*/
		mtAlerts_Alert(1,PutzRsc_WrkString(noMemAlrt));
/*DYJ  Ende; alt:	
		mtAlerts_Alert(1, noMemAlrt);
*/
		PutzInit_CloseInitBox();
		return FALSE;
	}
	
	/* Anzahl Gruppen * 3 als Maximalwert */
	PutzInit_SetMaxValue(Lists_NoOfEntries(&groupList) * 3);
	
	CatGlobal_busyMouse();
	
	/* Jetzt alle Eintraege durchscannen, und die neue Liste anlegen mit
	 * den Infopointern 
	 */
	if (!Lists_CreateList(&putzList, "PutzGr_InitGroups: putzlist"))
	{
		GroupSelect_DeleteList(&groupList);
/*DYJ 5.36f Start*/
		mtAlerts_Alert(1,PutzRsc_WrkString(noMemAlrt));
/*DYJ  Ende; alt:	
		mtAlerts_Alert(1, noMemAlrt);
*/
		PutzInit_CloseInitBox();
		return FALSE;
	}
	
	Lists_ResetList(&groupList);
	maxNameLength = 0;
	while ((entry = Lists_NextEntry(&groupList)) != NULL)
	{
		grEntry = NEW(grListEntry, "PutzGr_InitGroups");
		if (grEntry == NULL)
		{
			/* Listen loeschen */
			PutzGr_FreeGroupList();
			GroupSelect_DeleteList(&groupList);
/*DYJ 5.36f Start*/
			mtAlerts_Alert(1,PutzRsc_WrkString(noMemAlrt));
/*DYJ  Ende; alt:	
			mtAlerts_Alert(1, noMemAlrt);
*/
			PutzInit_CloseInitBox();
			return FALSE;
		}
		len = (_UWORD)strLen(entry->display_name);	/*DYJ 5.20w */
		if (len > maxNameLength)
			maxNameLength = len;

		/* Gruppennamen von Leerzeichen befreien */
		strBcpy(varName, entry->name);
		strUpr(varName);
		CatGlobal_ReplaceChar(varName, ' ', '_');    /* Spaces durch Underscores ersetzen */
		strBcat(varName, ".selected");
		
		grEntry->info = entry;
		grEntry->selected  = FALSE;
		grEntry->preMsgs   = 0;
		grEntry->preBytes  = 0;
		grEntry->badDel    = 0;
		grEntry->totalDel  = 0;
		grEntry->partDel   = 0;
		grEntry->postMsgs  = 0;
		grEntry->postBytes = 0;
		grEntry->tabDa     = FALSE;
		grEntry->parDa     = FALSE;
		grEntry->datDa     = FALSE;
		grEntry->status=GRP_NOTH;
		CatGlobal_busyMouse();
		if (!Lists_AppendEntry(&putzList, grEntry))
		{
			/* Listen loeschen */
			PutzGr_FreeGroupList();
			GroupSelect_DeleteList(&groupList);
/*DYJ 5.36f Start*/
			mtAlerts_Alert(1,PutzRsc_WrkString(noMemAlrt));
/*DYJ  Ende; alt:	
			mtAlerts_Alert(1, noMemAlrt);
*/
			PutzInit_CloseInitBox();
			return FALSE;
		}
	}
	
	/* Jetzt haben wir die Liste, jetzt gehen wir durch das Datenbank-
	 * Verzeichnis und holen uns die Daten der ganzen Gruppen
	 */
	PutzGr_SearchGroups();
	
	/* Und jetzt noch die Liste fuer die Anzeige aufbauen */

	PutzGr_ClearDisplayList(TRUE);
	PutzGr_BuildDisplayList(&displayList);
	
	PutzDo_CalcTotals();
	
	PutzInit_CloseInitBox();
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL PutzGr_SelectFile(CONST _UBYTE *title, CONST _UBYTE *mask, _UBYTE *path, _UBYTE *fName, _BOOL makeFullName)
{
	_UBYTE name[PATH_MAX];
	_UBYTE targetPath[PATH_MAX];
	
	if (*path == '\0')
	{
		F_Path_Get(targetPath);
	} else
	{
		strBcpy(targetPath, path);
	}
	if (mask != NULL && *mask != '\0')
	{
		F_Path_Append(targetPath, mask);
	} else
	{
		F_Path_Append(targetPath, ALL_FILE_MASK);
	}
	strBcpy(name, fName);
	if (!mtDir_GetDir(targetPath, name, title))
		return FALSE;
	F_Path_Dirname(targetPath);
	strMcpy(path, PATH_MAX, targetPath);
	if (makeFullName)
	{
		strMcpy(fName, PATH_MAX, targetPath);
		F_Path_Append(fName, name);
	} else
	{
		strMcpy(fName, PATH_MAX, name);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzGr_GetTempPath(_VOID)
{
	_BOOL tmpAvail;
	CONST _UBYTE *env;
	
	tmpAvail = FALSE;
	env = getenv("TMP");
	if (env != NULL)
	{
		strBcpy(tmpPath, env);
		tmpAvail = F_Path_Set(tmpPath);
	}
	if (!tmpAvail)
	{
		env = getenv("TMPDIR");
		if (env != NULL)
		{
			strBcpy(tmpPath, env);
			tmpAvail = F_Path_Set(tmpPath);
		}
	}
	if (!tmpAvail)
	{
		/* kein tmp-Pfad vorhanden, Db-Path nehmen */
		strBcpy(tmpPath, DataPath);
	}
	if (!defaultOpts.useTmp)
		strBcpy(tmpPath, DataPath);
	if (defaultOpts.tmpLock)
		strBcpy(tmpPath, defaultOpts.tmpDir);
	if (!F_Path_Set(tmpPath))
		strBcpy(tmpPath, DataPath);
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL PutzGr_ReadCatInf(_BOOL noCommand, CONST _UBYTE *inf_from_argv)
{
	if (!MTPaths_ReadInf(infPath, infName, noCommand, TRUE, inf_from_argv))
		return FALSE;
	CatGlobal_busyMouse();
	F_Path_Set(DataPath);
	PutzRsc_LoadParameter(DataPath);

	initWiederv();
	
	/* Infofiles_LoadInfoInf(TRUE, FALSE); */
	if (!PutzGr_InitGroups())
	{
		MouseArrow(); 
		return FALSE;
	}
	
	/* suchen nach Environment-Variable TMP oder TEMP */
	PutzGr_GetTempPath();
	
	MouseArrow();
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL PutzGr_ReadNewInf(_BOOL doFsel, CONST _UBYTE *newinf)
{
	_UBYTE oldDir[PATH_MAX];
	_UBYTE oldInfName[PATH_MAX];
	_UBYTE suff[PATH_MAX];
	
	strCpy(oldDir, infPath);	/*DYJ 5.20w */
	strCpy(oldInfName, infName);	/*DYJ 5.20w */
	if (doFsel)
	{
/*DYJ 5.36f Start*/
		if (!PutzGr_SelectFile(	PutzRsc_WrkString(loadInf), "*.inf", infPath, infName, FALSE))
/*DYJ  Ende; alt:	
		if (!PutzGr_SelectFile("CAT.INF laden", "*.inf", infPath, infName, FALSE))
*/
			return FALSE;
	} else
	{
		mtDir_SplitPath(newinf, infPath, infName, suff);
	}
	/* Gruppenliste loeschen */
	PutzGr_FreeGroupList();
	GroupSelect_DeleteList(&groupList);
	if (!PutzGr_ReadCatInf(doFsel, NULL))
	{
		/* Altes CAT.INF wieder lesen */
		strCpy(infPath, oldDir);	/*DYJ 5.20w */
		strCpy(infName, oldInfName);	/*DYJ 5.20w */
		return PutzGr_ReadCatInf(doFsel, NULL);
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzGr_Exit(_VOID)
{
	PutzGr_ClearDisplayList(TRUE);
	PutzGr_FreeGroupList();
	GroupSelect_DeleteList(&groupList);
}
