/*
 * Diese Datei ist Teil des CAT-Projekts.
 *
 * CAT ist Freeware; es darf unter den Bedingungen der
 * CAT Public License weitergegeben und/oder modifiziert
 * werden.
 *
 * Eine Kopie der CAT Public License liegt diesem Paket bei
 * (siehe CATPL.TXT).
 *
 * Copyright (c) 1997, Dirk Steins
 * Jegliche Veraenderung dieses Hinweises ist NICHT ERLAUBT.
 */

/*
 * Die vorliegende Version wurde nach C portiert von
 * Thorsten Otto (Thorsten_Otto@do.maus.ruhr.de)
 *
 */


#include <putzlist.h>
#include <strconv.h>
#include <confvars.h>

#include <routinec.h>
#include <routines.h>
#include <magicstr.h>
#include <catgloba.h>
#include <catputz.h>		/*DYJ 5.22a */
#include <PUTZRSC.H>		/*DYJ 5.22a */


#define maxParts 20                           /* Anzahl der maximalen verschiedenen Zeilenteile */
#define partFormatLength 32                   /* maximale Laenge eines Formatstrings */

#define cKaputt2    "#  (%)"
#define cTotalDel2  "#  (%)"
#define cPartDel2   "#  (%)"
#define cPreMsg1    "Messages"
#define cPostMsg1   "Messages"
#define cDelBytes1  "Bytes"

    
typedef enum {
	pGroup, pBadDel, pTotalDel, pPartDel, 
	pPreMsgs, pPostMsgs, pPreBytes, pPostBytes,
	pDelBytes,pGruFla
} partNameType;
 
typedef _UBYTE partFormat[partFormatLength];

typedef struct {
	partNameType type;
	_WORD w;
	partFormat formatStr;
} partType;

typedef partType partArrayType[maxParts];

typedef struct {
	_WORD lines;
	_WORD width;
	_WORD maxNameWidth;
	partArrayType parts;
} partArrayDesc;

LOCAL partArrayDesc listParts;


/******************************************************************************/
/*** ---------------------------------------------------------------------- ***/
/******************************************************************************/

GLOBAL _WORD PutzList_GetPartCount(_VOID)
{
	return listParts.lines;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD PutzList_GetMaxLineWidth(_VOID)
{
	_WORD width, i;
	
	width = 0;
	for (i = 0; i < listParts.lines; i++)
		width += PutzList_GetPartWidth(i);
	return width;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _WORD PutzList_GetPartWidth(_WORD part)
{
	if ((listParts.parts[part].type == pGroup) &&
		(listParts.parts[part].w == -1))
	{
		/* Maximale Laenge eines Gruppennamens zurueckgeben */
		return listParts.maxNameWidth;
	}
	return listParts.parts[part].w;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL partNameType PutzList_GetPartType(_WORD part)
{
	return listParts.parts[part].type;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID ExtendStr(_UBYTE *str, size_t l, TextJustification just)
{
	size_t c, f1, f2, pos;
	
	c = strLen(str);	/*DYJ 5.20w */
	if (c < l)
	{
		if (just == justLeft)
		{
			while (c < l)
				str[c++] = ' ';
			str[c] = '\0';
		} else if (just == justCenter)
		{
			f1 = (l - c) / 2;
			f2 = (l - c) - f1;
			pos = c;
			while (pos)
			{
				pos--;
				str[pos + f1] = str[pos];
			}
			pos = f1;
			while (pos)
				str[--pos] = ' ';
			pos = c + f1;
			while (f2)
			{
				str[pos++] = ' ';
				f2--;
			}
			str[pos] = '\0';
		} else
		{
			f1 = l - c;
			pos = c;
			str[l] = '\0';
			while (pos)
			{
				--pos;
				str[pos + f1] = str[pos];
			}
			pos = f1;
			while (pos)
			{
				--pos;
				str[pos] = ' ';
			}
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _ULONG CalcPercent(_ULONG org, _ULONG remain)
{
	if (org == 0 || remain == 0)
		return 0;
	return ((1000UL * remain) / org + 5UL) / 10UL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _ULONG CalcKiloBytes(_ULONG theNumber)
{
	return (theNumber + 1023UL) / 1024UL;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _VOID FormatNumber(_ULONG remain, _ULONG org, _UWORD width, _UBYTE *str, size_t strsize)
{
	_UBYTE perStr[20];
	
	CardToStr(remain, 0, str, strsize);
	strCpy(perStr, " (");	/*DYJ 5.20w */
	CardToStr(CalcPercent(org, remain), 2, perStr + 2, sizeof(perStr) - 2);
	strBcat(perStr, "%)");
	if (strLen(str) + strLen(perStr) <= width)	/*DYJ 5.20w */
	{
		strMcat(str, strsize, perStr);
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL TextJustification PutzList_GetPartJustification(displayListEntry *ptr, _WORD part)
{
	if (PutzList_GetPartType(part) == pGroup)
		return justLeft; 
	if (ptr->type == dataEntry)
		return justRight;
	return justCenter;
}
/*** ------------------------------------------------------------------------ ***/
/*	LOCAL _WORD PutzDo_HasGroupFlags(CONST _UBYTE *name)						*/
/*	Gibt zurck wieviele Gruppenspeziefische Optionen es gibt.					*/
/*	Parameter:	name: Gruppenname												*/
/*	Ergebnis:	Anzahl Optionen													*/		
/*	3.12.00 von Dimitri Junker	DYJ 4.21z										*/
/*** ------------------------------------------------------------------------ ***/
LOCAL _WORD PutzDo_HasGroupFlags(CONST _UBYTE *name)
{
	groupName gName;
	groupName varName;
	_BOOL boolVal;
/*DYJ 5.23e Start*/
	groupName deleteStr;
/*DYJ  Ende; alt:	
	groupName delStr;
*/
	_UBYTE *end;
	_WORD val,ret=0;
	
	/* Gruppennamen von Leerzeichen befreien */
	strBcpy(gName, name);
	if (strCmp(gName, "Default") != 0)
	{
		strUpr(gName);
		CatGlobal_ReplaceChar(gName, ' ', '_');    /* Spaces durch Underscores ersetzen */
	}
	
	/* Variablennamen zusammenbauen */
	/* Fuer Interessant-Flag */
	strBcpy(varName, gName);
	end=strChr(varName,'\0');	/*DYJ 5.20w */
	strBcat(varName, ".delInteresting");
	if(ConfVars_GetConfigInt(varName, &val))
		ret++;
	/* Fuer User-Flag 1 */
	*end='\0';
	strBcat(varName, ".delUserflag1");
	if(ConfVars_GetConfigInt(varName, &val))
		ret++;
	/* Fuer User-Flag 2 */
	*end='\0';
	strBcat(varName, ".delUserflag2");
	if(ConfVars_GetConfigInt(varName, &val))
		ret++;
	/* Fuer Gelesen-Flag */
	*end='\0';
	strBcat(varName, ".delRead");
	if(ConfVars_GetConfigInt(varName, &val))
		ret++;
	/* Fuer Gefiltert-Flag */
	*end='\0';
	strBcat(varName, ".delFiltered");
	if(ConfVars_GetConfigInt(varName, &val))
		ret++;
	/* Fuer Ungelesen-Flag */
	*end='\0';
	strBcat(varName, ".delUnread");
	if(ConfVars_GetConfigInt(varName, &val))
		ret++;
	
	/* Loeschdatum zusammensetzen */
	*end='\0';
	strBcat(varName, ".deleteByDate");
	if(ConfVars_GetConfigBool(varName, &boolVal))
		ret++;
	if (boolVal)
	{
		*end='\0';
		strBcat(varName, ".deleteDate");
/*DYJ 5.23e Start*/
		if(ConfVars_GetConfigString(varName, deleteStr, sizeof(delStr)))
/*DYJ  Ende; alt:	
		if(ConfVars_GetConfigString(varName, delStr, sizeof(delStr)))
*/
			ret++;
	}
	return ret;
}
/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzList_GetPart(displayListEntry *ptr, _WORD part, _UBYTE *str, size_t strsize, _BOOL fillUp)
{
	_WORD maxWidth;
	grListEntry *dataPtr;
	_BOOL weiter=TRUE;
	
	*str = '\0';
	maxWidth = PutzList_GetPartWidth(part);
	if (maxWidth >= strsize)
		maxWidth = (_UWORD)strsize - 1;
	dataPtr = ptr->data;
	if(ptr->type == dataEntry && dataPtr->status==GRP_NOTH)
	{
		switch (PutzList_GetPartType(part))
		{
		case pGroup:
		case pPreMsgs:
		case pPreBytes:
		case pGruFla:
			break;
		
		case pBadDel:
		case pTotalDel:
		case pPartDel:
			Copy(" ( %)", 0, maxWidth, str, strsize);
			weiter=FALSE;
			break;
		case pPostMsgs:
			Copy(" ", 0, maxWidth, str, strsize);
			weiter=FALSE;
			break;
		case pPostBytes:
		case pDelBytes:
			Copy(" KB", 0, maxWidth, str, strsize);
			weiter=FALSE;
			break;
		}
	}else if(ptr->type == dataEntry && dataPtr->status==GRP_ERR)
	{
		switch (PutzList_GetPartType(part))
		{
		case pBadDel:
			Copy(" ERR!", 0, maxWidth, str, strsize);
		case pPostBytes:
		case pTotalDel:
		case pPartDel:
		case pPostMsgs:
		case pDelBytes:
			weiter=FALSE;
		case pGroup:
		case pPreMsgs:
		case pPreBytes:
		case pGruFla:
			break;
		}
	}
	if(weiter)
	{
		switch (PutzList_GetPartType(part))
		{
		case pGroup:
			if (ptr->type == listHead1)
			{
				Copy(PutzRsc_WrkString(cName), 0, maxWidth, str, strsize);		/*DYJ 5.22a */
			} else if (ptr->type == dataEntry)
			{
				if (dataPtr->info == NULL)
/*DYJ 5.36f Start*/
					Copy(PutzRsc_WrkString(cTotal), 0, maxWidth, str, strsize);
/*DYJ  Ende; alt:	
					Copy(cTotal, 0, maxWidth, str, strsize);
*/
				else
					Copy(dataPtr->info->display_name, 0, maxWidth, str, strsize);
			}
			break;
		
		case pBadDel:
			if (ptr->type == listHead1)
			{
				Copy(PutzRsc_WrkString(cKaputt1), 0, maxWidth, str, strsize);	/*DYJ 5.22a */
			} else if (ptr->type == listHead2)
			{
				Copy(cKaputt2, 0, maxWidth, str, strsize);
			} else if (ptr->type == dataEntry)
			{
				FormatNumber(dataPtr->badDel, dataPtr->preMsgs, maxWidth, str, strsize);
			}
			break;
		
		case pTotalDel:
			if (ptr->type == listHead1)
			{
				Copy(PutzRsc_WrkString(cTotalDel1), 0, maxWidth, str, strsize);	/*DYJ 5.22a */
			} else if (ptr->type == listHead2)
			{
				Copy(cTotalDel2, 0, maxWidth, str, strsize);
			} else if (ptr->type == dataEntry)
			{
				FormatNumber(dataPtr->totalDel, dataPtr->preMsgs, maxWidth, str, strsize);
			}
			break;
		
		case pPartDel:
			if (ptr->type == listHead1)
			{
				Copy(PutzRsc_WrkString(cPartDel1), 0, maxWidth, str, strsize);	/*DYJ 5.22a */
			} else if (ptr->type == listHead2)
			{
				Copy(cPartDel2, 0, maxWidth, str, strsize);
			} else if (ptr->type == dataEntry)
			{
				FormatNumber(dataPtr->partDel, dataPtr->preMsgs, maxWidth, str, strsize);
			}
			break;
			
		case pPreMsgs:
			if (ptr->type == listHead1)
			{
				Copy(cPreMsg1, 0, maxWidth, str, strsize);
			} else if (ptr->type == listHead2)
			{
				Copy(PutzRsc_WrkString(cPreMsg2), 0, maxWidth, str, strsize);	/*DYJ 5.22a */
			} else if (ptr->type == dataEntry)
			{
				CardToStr(dataPtr->preMsgs, 0, str, strsize);
			}
			break;
			
		case pPostMsgs:
			if (ptr->type == listHead1)
			{
				Copy(cPostMsg1, 0, maxWidth, str, strsize);
			} else if (ptr->type == listHead2)
			{
				Copy(PutzRsc_WrkString(cPostMsg2), 0, maxWidth, str, strsize);	/*DYJ 5.22a */
			} else if (ptr->type == dataEntry)
			{
				CardToStr(dataPtr->postMsgs, 0, str, strsize);
			}
			break;
			
		case pPreBytes:
			if (ptr->type == listHead1)
			{
				Copy(PutzRsc_WrkString(cPreBytes1), 0, maxWidth, str, strsize);	/*DYJ 5.22a */
			} else if (ptr->type == listHead2)
			{
/*DYJ 5.22a Start*/
				Copy(PutzRsc_WrkString(cPreMsg2), 0, maxWidth, str, strsize);
/*DYJ  Ende; alt:	
				Copy(cPreBytes2, 0, maxWidth, str, strsize);
*/
			} else if (ptr->type == dataEntry)
			{
				CardToStr(CalcKiloBytes(dataPtr->preBytes), 0, str, strsize);
				strMcat(str, strsize, " KB");
			}
			break;
		
		case pPostBytes:
			if (ptr->type == listHead1)
			{
/*DYJ 5.22a Start*/
				Copy(PutzRsc_WrkString(cPreBytes1), 0, maxWidth, str, strsize);
			} else if (ptr->type == listHead2)
			{
				Copy(PutzRsc_WrkString(cPostMsg2), 0, maxWidth, str, strsize);
/*DYJ  Ende; alt:	
				Copy(cPostBytes1, 0, maxWidth, str, strsize);
			} else if (ptr->type == listHead2)
			{
				Copy(cPostBytes2, 0, maxWidth, str, strsize);
*/
			} else if (ptr->type == dataEntry)
			{
				CardToStr(CalcKiloBytes(dataPtr->postBytes), 0, str, strsize);
				strMcat(str, strsize, " KB");
			}
			break;
		
		case pDelBytes:
			if (ptr->type == listHead1)
			{
				Copy(cDelBytes1, 0, maxWidth, str, strsize);
			} else if (ptr->type == listHead2)
			{
				Copy(PutzRsc_WrkString(cDelBytes2), 0, maxWidth, str, strsize);	/*DYJ 5.22a */
			} else if (ptr->type == dataEntry)
			{
				if (dataPtr->postBytes > 0)
				{
					if(dataPtr->preBytes >= dataPtr->postBytes)
						CardToStr(CalcKiloBytes(dataPtr->preBytes - dataPtr->postBytes), 0, str, strsize);
					else
					{
						*str='-';
						CardToStr(CalcKiloBytes(dataPtr->postBytes - dataPtr->preBytes), 0, str+1, strsize-1);
					}
				}
				else
					strMcpy(str, strsize, "0");
				strMcat(str, strsize, " KB");
			}
			break;
		case pGruFla:
			if (ptr->type == listHead1)
			{
				Copy(PutzRsc_WrkString(cGruFla1), 0, maxWidth, str, strsize);	/*DYJ 5.22a */
			} else if (ptr->type == listHead2)
			{
				Copy(PutzRsc_WrkString(cGruFla2), 0, maxWidth, str, strsize);	/*DYJ 5.22a */
			} else if (ptr->type == dataEntry)
			{
				if(dataPtr->info && PutzDo_HasGroupFlags(dataPtr->info->name))
					Copy("\b", 0, maxWidth, str, strsize);
				else
					Copy(" ", 0, maxWidth, str, strsize);
			}
			break;
		}
	}
	if (fillUp)
	{
		ExtendStr(str, maxWidth, PutzList_GetPartJustification(ptr, part));
	}
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzList_SetMaxNameLength(_WORD maxLen)
{
	listParts.maxNameWidth = maxLen;
}

/*** ---------------------------------------------------------------------- ***/

LOCAL _BOOL parseFormatLine(CONST _UBYTE *listline, _BOOL onlyTest)
{
	partArrayDesc p;
	_LONG num;
	_UBYTE ch;
	partNameType pName;
	partFormat fStr;
	_WORD i;
	_BOOL negative;
	CONST _UBYTE *origline;
#define is_num(c) ((c) == '-' || ((c) >= '0' && (c) <= '9'))
#define is_skip(c) ((c) == ' ' || (c) == ',')
#define is_format(c) FALSE

	if (listline == NULL)
		return FALSE;
	origline = listline;
	p.lines = 0;
	p.width = 0;
	while (*listline != '\0')
	{
		/* Ersten Eintrag suchen */
		while (is_skip(*listline))
			++listline;
		if (*listline != '\0')
		{
			ch = *listline;
			for (i = 0; i < sizeof(fStr); i++)
				fStr[i] = '\0';
			if (listline[1] != '(')
				return FALSE;
			listline += 2;        /* z+1 == '(' */
			num = 0;
			negative = FALSE;
			if (*listline == '-')
			{
				negative = TRUE;
				listline++;
			}
			while (is_num(*listline))
			{
				num = num * 10 + (*listline - '0');
				listline++;
			}
			if (negative)
				num = -num;
			if (num == 0 || num > 1024)
				return FALSE;
			if (*listline == '\0' ||
				(*listline != ')' &&
				 *listline != ',' && !is_format(ch)))
			{ 
				return FALSE;
			}
			if (*listline == ',' && is_format(ch))
			{
				/* Formatstring lesen */
				++listline;    /* ',' ueberspringen */
				i = 0;
				while (*listline != ')' && *listline != '\0' && i < sizeof(fStr))
				{
					fStr[i] = *listline;
					++listline;
					++i;
				}
				if (i >= sizeof(fStr) || *listline == '\0')
					return FALSE; 
			}
			++listline;   /* ')' ueberspringen */
			switch (ch)
			{
				case 'B': pName = pBadDel; break;
				case 'D': pName = pTotalDel; break;
				case 'G': pName = pGroup; break;
				case 'M': pName = pPreMsgs; break;
				case 'N': pName = pPostMsgs; break;
				case 'T': pName = pPartDel; break;
				case 'V': pName = pPreBytes; break;
				case 'W': pName = pPostBytes; break;
				case 'X': pName = pDelBytes; break;
				case 'F': pName = pGruFla; break;
				default: return FALSE;
			}
			p.parts[p.lines].type = pName;
			p.parts[p.lines].w = (_WORD)num;
			strCpy(p.parts[p.lines].formatStr, fStr);	/*DYJ 5.20w */
			++p.lines;
			p.width += (_WORD)num;
		}
	}
	if (p.lines == 0)
		return FALSE;
	if (onlyTest)
		return TRUE;
	ConfVars_SetConfigString(PutzRsc_WrkString(cFormatLine), origline);	/*DYJ 5.22a */
	listParts = p;
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

GLOBAL _BOOL PutzList_ParseFormat(CONST _UBYTE *format)
{
	return parseFormatLine(format, FALSE);
}

/*** ---------------------------------------------------------------------- ***/

/* Das Format einer Zeile besteht aus einzelnen Kuerzeln 
 * fuer einen Eintrag und darauffolgend in Klammern die Weite dieses Eintrages.
 * Beispiel: D(12),I(10),U(30),W(30),F(2)
 * Erlaubte Zeilenelemente:
 * G: Gruppe. Bei einer Weite von -1 wird die Laenge des laengsten Namens 
 *    genommen
 * B: Anzahl beschaedigte Nachrichten
 * D: Anzahl totalgeloeschter Nachrichten 
 * T: Anzahl teilgeloeschter Nachrichten
 * M: Anzahl Nachrichten vorher
 * N: Anzahl Nachrichten nachher
 * V: Kilobytes vorher
 * W: Kilobytes nachher
 * X: Kilobytes geloescht
 * F: Eigene Gruppenflags
 */
#if 0
LOCAL _BOOL PutzList_CheckFormatLine(CONST _UBYTE *listline)
{
	return parseFormatLine(listline, TRUE);
}
#endif

/*** ---------------------------------------------------------------------- ***/

GLOBAL _VOID PutzList_InitFormat(_VOID)
{
	_UBYTE str[256];
	
	ConfVars_GetConfDefStr(PutzRsc_WrkString(cFormatLine), str, sizeof(str), "F(2),G(-1),M(12),V(10),D(12),T(12),B(12),N(12),W(10),X(10)");	/*DYJ 5.22a */
	PutzList_ParseFormat(str);
}
